/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var object;
var old;
var current;
var cluster;
var myDialog;
var dock;

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  object = myDialog.arguments.object;
  current = myDialog.arguments.subobject;
  cluster = current.cluster;
  old = current.old(false, true);
  refresh(true);
}

function refresh(onload) {
  stringlist('gresTypes', '');
  check('externalServer');
  text('prefix');
  text('etc');
  text('prolog');
  text('epilog');
  text('prologSlurmctld');
  number('fastSchedule', 0, 2);

  var placeholders = current.get('placeholders');
  displayPlaceholders(placeholders);

  refreshPowerSaving(onload);
  refreshCgroups(onload);
}

function refreshPowerSaving(onload) {
  check('powerSavingEnabled');
  text('suspendTime', '^[0-9]+$');
  text('suspendTimeout', '^[0-9]+$');
  text('suspendProgram');
  text('resumeTimeout', '^[0-9]+$');
  text('resumeProgram');

  var enabled = 'true';
  if(onload) {
    enabled=!current.get('powerSavingEnabled');
  } else {
    var obj = document.getElementById('powerSavingEnabled');
    enabled = obj.checked;
  }
  for(var i=1; i<5; i++) {
    document.getElementById('ps_hbox' + i).setAttribute('hidden', eval(enabled));
  }
}

function refreshCgroups(onload) {
  var cgroups = current.get('cgroups');
  check('cgroups', false, 'enabled');
  check('cgroups', false, 'constrainCores');
  check('cgroups', false, 'constrainDevices');
  check('cgroups', false, 'constrainRamSpace');
  check('cgroups', false, 'constrainSwapSpace');
  check('cgroups', false, 'taskAffinity');
  check('cgroups', false, 'autoMount');

  text('cgroups', '^', 'mountPoint');
  text('cgroups', '^', 'releaseAgentDir');
  text('cgroups', '^', 'allowedDevicesFile');

  text('cgroups', '^[0-9]+$', 'allowedRamSpace');
  text('cgroups', '^[0-9]+$', 'allowedSwapSpace');
  text('cgroups', '^[0-9]+$', 'maxRAMPercent');
  text('cgroups', '^[0-9]+$', 'maxSwapPercent');
  text('cgroups', '^[0-9]+$', 'minRAMSpace');

  var enabled = 'true';
  if(onload) {
    enabled=!cgroups.enabled;
  } else {
    var obj = document.getElementById('cgroupsenabled');
    enabled = obj.checked;
  }
  for(var i=1; i<10; i++) {
    document.getElementById('cg_hbox' + i).setAttribute('hidden', eval(enabled));
  }
}

function displayPlaceholders(placeholders) {
  var rows = document.getElementById('placeholders');
  if(rows) {
    while(rows.firstChild) rows.removeChild(rows.firstChild);
    var last = placeholders.length - 1;
    while(last >= 0) {
      if(!placeholders[last].toBeRemoved) break;
      last--;
    }
    for(var i = 0; i < placeholders.length; i++) {
      if(placeholders[i].toBeRemoved) continue;
      var row = document.createElement('row');
      row.setAttribute('align', 'center');
      var textbox = document.createElement('textbox');
      textbox.setAttribute('id', placeholders[i].childType + '_' + i + '_name');
      textbox.setAttribute('type', 'text');
      textbox.setAttribute('width', '165px');
      textbox.setAttribute('readonly', 'true');
      textbox.setAttribute('value', placeholders[i].queue);
      row.appendChild(textbox);

      var hbox = document.createElement('vbox');
      hbox.setAttribute('flex', 1);
      var button = document.createElement('button');
      button.setAttribute('class', 'graybutton');
      button.setAttribute('label', 'Modify');
      button.setAttribute('oncommand', 'editPlaceholder(' + i + ');');
      hbox.appendChild(button);
      row.appendChild(hbox);

      var button = document.createElement('button');
      button.setAttribute('class', 'minus-button');
      button.setAttribute('oncommand', 'removePlaceholder(' + i + ');');
      row.appendChild(button);

      if(i == last) {
        var button = document.createElement('button');
        button.setAttribute('class', 'plus-button');
        button.setAttribute('oncommand', 'addPlaceholder();');
        row.appendChild(button);
      } else {
        var spacer = document.createElement('spacer');
        row.appendChild(spacer);
      }
      rows.appendChild(row);
    }
    if(last < 0) {
      var row = document.createElement('row');
      var button = document.createElement('button');
      button.setAttribute('class', 'plus-button');
      button.setAttribute('oncommand', 'addPlaceholder();');
      row.appendChild(button);
      rows.appendChild(row);
    }
  } else {
    println('No rows for ' + placeholders.childType);
  }
}

function addPlaceholder() {
  var placeholders = current.get('placeholders');
  var placeholder = new dock.cvJobQueuePlaceholder();
  placeholder.queue = ''
  placeholder.baseNodeName = ''
  placeholder.maxNodes = '0'
  placeholders.push(placeholder);
  placeholders.modified = true;
  current.set('modified', true);
  refresh(false);
  setfocus('queue');
}

function removePlaceholder(i) {
  var placeholders = current.get('placeholders');
  current.set('modified', true);
  if((i >= 0) && (i < placeholders.length)) {
    placeholders .splice(i, 1);
    current.set("placeholders", placeholders);
  }
  refresh(false);
}

function editPlaceholder(i) {
  var placeholders = current.get('placeholders');
  var editfile = 'advancedjobqueueplaceholder.xul';
  var args = new Object();
  args['object'] = object;
  args['subobject'] = current;
  args['cluster'] = cluster;
  args['placeholder'] = placeholders[i];
  args['parent'] = this;
  var d = new dock.cvDialog(editfile, args);
  d.onOk = function() {
    refresh(false);
  }
  dock.dialogs.showDialog(d);
}

function ok() {
  if(current.get('modified')) object.set('modified', true);
  close();
  myDialog.onOk();
}

function cancel() {
  if(!window.closed) {
    current.old(old);
    close();
  }
}





