/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh certificates" + ' - ' + document.documentURI);

  dock.workerpool.getWorker(current.actualurl).call('cmcert', 'getCertificates', null, function(code, data) {
    if(code == 0) {
      setCallbackResult(data);
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);
}

function setCallbackResult(value) {
  var list = value;
  tree = document.getElementById('tree');
  data = new Array();
  var showrevoked = document.getElementById('showrevoked').checked;
  for(var i = 0; i < list.length; i++) {
    if(showrevoked || !list[i].revoked) {
      data.push(createTreeItem(list[i]));
    }
  }
  for(var i = list.length; i < MINTREEITEMS; i++) data.push(blankTreeItem());
  var select = selected('list');
  loadTable();
  setSelected('list', select);
}

function createTreeItem(one) {
  var item = new Object();
  item['key'] = one.serialNumber;
  item['revoked'] = one.revoked;
  item['name'] = one.commonName;
  item['serial'] = one.serialNumber;
  item['profile'] = one.profile;
  item['daysLeft'] = one.remainingDays;
  item['country'] = one.country;
  return item;
}

function showCertificate() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a certificate");
  } else if(select.length > 1) {
    dock.displayError("Please select only one certificate");
  } else {
    dock.workerpool.getWorker(current.actualurl).call('cmcert', 'getCertificate', parseInt(select[0]), function(code, data) {
      if(code == 0) {
        var args = new Object();
        args['cluster'] = current;
        args['old'] = data;
        if(args['old']) {
          var d = new dock.cvDialog("showcertificate.xul", args);
          d.onOk = function() {
            refresh();
          }
          dock.dialogs.showDialog(d);
        } else {
          dock.displayInfo("Certificate no longer found");
          refresh();
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
  }
}

function revokeCertificate() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a certificate");
  } else {
    dock.waitIcon();
    var work = {'cluster': current, 'todo': select.length};
    for(var i = 0; i < select.length; i++) {
      dock.workerpool.getWorker(current.actualurl).call('cmcert', 'revokeCertificate', parseInt(select[i]), function(code, data) {
        if(code != 0) {
          dock.jsonCallError(this, code, data);
        }
        this.current.todo--;
        if(this.current.todo <= 0) {              
          refresh();
          dock.unwaitIcon();
        }
      }, work);
    }
  }
}

function unrevokeCertificate() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a certificate");
  } else {
    dock.waitIcon();
    var work = {'cluster': current, 'todo': select.length};
    for(var i = 0; i < select.length; i++) {
      dock.workerpool.getWorker(current.actualurl).call('cmcert', 'unrevokeCertificate', parseInt(select[i]), function(code, data) {
        if(code != 0) {
          dock.jsonCallError(this, code, data);
        }
        this.current.todo--;
        if(this.current.todo <= 0) {              
          refresh();
          dock.unwaitIcon();
        }
      }, work);
    }
  }
}

function addCertificate() {
  var args = new Object();
  args['cluster'] = current;
  args['cert'] = 0;
  var d = new dock.cvDialog("addcertificate.xul", args);
  d.onOk = function() {
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function showRevoked() {
  if(document.getElementById('showrevoked').checked) {
    document.getElementById('revoked').removeAttribute('hidden');
  } else {
    document.getElementById('revoked').setAttribute('hidden', true);
  }
  refresh();
}
