/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var cluster;
var waiting = 0;
var worknodes = 0;
var workresult = 0;
var force = false;

var perPage = 25;
var page = 1;
var pages = 1;
var rows = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function stop() {
  force = false;
}

function refresh(use) {
  if(typeof(use) == 'boolean') {
    force = use;
  } else if(!!use) {
    current = use;
    if(current.isCluster) {
      cluster = use;
    } else {
      cluster = use.cluster;
    }
    var x = new Array();
    setCallbackResult(x, x);
  }
  println("Refresh burn overview" + ' - ' + document.documentURI);

  starSort(dock.generalSettings.burnsort);
  
  var nodes;
  if(cluster == current) {
    nodes = cluster.allKeys('SlaveNode', false);
  } else {
    nodes = cluster.findNodesIn(current.getType(), current.get('uniqueKey'));
  }
  if(force) {
    dock.waitIcon();
    dock.workerpool.getWorker(cluster.actualurl).call('cmdevice', 'getBurnStatus', [nodes], function(code, data) {
      if(code == 0) {
        setCallbackResult(nodes, data);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
    waiting += 1;
  }
}

function starSort(index) {
  var i = 1;
  while(true) {
    var obj = document.getElementById('sort' + i);
    if(!obj) break;
    var value = obj.getAttribute('value');
    var star = value.indexOf('*');
    if((index == i) && (star < 0)) {
      obj.setAttribute('value', value + ' *');
    } else if((index != i) && (star >= 0)) {
      star = value.lastIndexOf(' ', star);
      obj.setAttribute('value', value.substr(0, star));
    }
    i++;
  }
}

function sortBy(event) {
  var index = 1;
  if(typeof(event) == 'number') {
    index = event;
  } else {
    var item = event.target;
    index = parseInt(item.getAttribute('sort'));
  }
  starSort(index);
  dock.generalSettings.burnsort = index;
  waiting += 1;
  dock.waitIcon();
  if(workresult) setCallbackResult(worknodes, workresult);
}

function setCallbackResult(nodes, result) {
  worknodes = nodes;
  workresult = result;
  
  var timecol = document.getElementById('sort7');

  var sortedlist = new Array();
  for(var i = 0; i < nodes.length; i++) {
    var node = cluster.find(nodes[i]);
    if(node) {
      var status = '';
      var warnings = '';
      var phase = '';
      var time = '';
      var text = '';
      var name = '';
      var pxe = 'no';
      if(i < result.length) {
        status = result[i].error;
        if(node.get('startNewBurn')) pxe = 'yes';
        warnings = result[i].warnings;
        phase = result[i].phaseName;
        time = result[i].phaseTime;
        if(timecol && (time.indexOf('/') > 0)) {
          timecol.setAttribute('value', 'Total/Phase time');
          timecol = 0;
        }
        var tests = result[i].testStatusList;
        for(var j = 0; j < tests.length; j++) {
          text += tests[j].name + '(';
          if(tests[j].started) text += 'S';
          if(tests[j].failed) text += 'F';
          if(tests[j].warning) text += 'W';
          if(tests[j].passed) text += 'P';
          if(tests[j].status.length) {
            text += trim(tests[j].status);
          }
          text += ') ';
        }
        if(node.get('burning') || node.get('startNewBurn')) {
          var burn = node.get('burnConfig');
          if(burn) name = burn.name;
        }
      }
      var one = [0, node.resolveName(), name, status, pxe, warnings, phase, time, text, i, nodes[i]];
      one[0] = one[dock.generalSettings.burnsort];
      if(typeof(one[0]) == 'string') one[0] = one[0].toLowerCase();
      sortedlist.push(one);
    }
  }
  sortedlist.sort();
  
  rows = new Array();
  for(var i0 = 0; i0 < sortedlist.length; i0++) {
    var hostname = sortedlist[i0][1];
    var name = sortedlist[i0][2];
    var status = sortedlist[i0][3];
    var pxe = sortedlist[i0][4];
    var warnings = sortedlist[i0][5];
    var phaseName = sortedlist[i0][6];
    var phaseTime = sortedlist[i0][7];
    var text = sortedlist[i0][8];
    var i = sortedlist[i0][9];
    var key = sortedlist[i0][10];
   
    var row = document.createElement("row");
    row.setAttribute('align', 'center');
    row.setAttribute("height", "19px");
    if(i0 % 2 == 0) row.setAttribute("style", "background-color: #dcdcdc;");

    var label = document.createElement("label");
    label.setAttribute('value', hostname);
    row.appendChild(label);
    var label = document.createElement("label");
    label.setAttribute('value', name);
    row.appendChild(label);
    var label = document.createElement("label");    
    label.setAttribute('value', status);      
    row.appendChild(label);    
    var label = document.createElement("label");    
    label.setAttribute('value', pxe);      
    row.appendChild(label);    
    var label = document.createElement("label");
    label.setAttribute('value', warnings);
    row.appendChild(label);    
    var label = document.createElement("label");
    label.setAttribute('value', phaseName);
    row.appendChild(label);    
    var label = document.createElement("label");
    label.setAttribute('value', phaseTime);
    row.appendChild(label);    
    var label = document.createElement("label");
    label.setAttribute('value', text);
    row.appendChild(label);    

    var label = document.createElement('button');
    label.setAttribute("label", 'Show');
    label.setAttribute("class", 'graybutton');
    label.setAttribute("oncommand", 'burnLog(' + nodes[i] + ');');
    row.appendChild(label);      
    rows.push(row);
  }
  if(waiting > 0) {
    dock.unwaitIcon();
    waiting -= 1;
  }
  var n = Math.max(1, Math.ceil(rows.length / perPage));
  createPageMenu(n);
  displayPage();
}

function allNodes(needCommit) {
  if(!!current.cluster) {
    return current.cluster.findNodesIn(current.getType(), current.get('uniqueKey'));
  } else {
    return cluster.allKeys(parent.frames[0].basetype, false, needCommit);
  }
}

function burnLog(key) {
  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;
  
  key = isundefined(key, 0);
  var args = new Object();
  args['nodes'] = allNodes(false, false);
  args['cluster'] = cluster;
  args['select'] = key;
  var d = new dock.cvDialog("burnlog.xul", args);
  dock.dialogs.showDialog(d);
}

function displayPage(index) {
  page = isundefined(index, page);
  var list = document.getElementById('list');
  while(list.childNodes.length > 2) list.removeChild(list.lastChild);
  
  var b = Math.max(perPage * (page - 1), 0);
  var e = Math.min(perPage * page - 1, rows.length);
//   println('displayPage ' + page + ' [' + b + ',' + e + ')');
  for(var i = b; i < e; i++) {
    list.appendChild(rows[i]);
  }
  updatePageMenu(page);
}

function createPageMenu(n) {
  var pagespopup = document.getElementById("pages");
  var m = pagespopup.childNodes.length;
  for(var i = n; i < m; i++) pagespopup.removeChild(pagespopup.lastChild);
  for(var i = m + 1; i <= n; i++) {
    var item = document.createElement('menuitem');
    item.setAttribute('page', i);
    item.setAttribute('label', i);
    pagespopup.appendChild(item);
  }
  pages = n;
}

function updatePageMenu(page) {
  var pagespopup = document.getElementById("pages");
  pagespopup.parentNode.selectedIndex = page - 1;
}

function setPage(index) {
//   println("setPage " + index + " [1," + pages + "]");
  if((index >= 1) && (index <= pages)) {
    displayPage(index);
  }
}

function perpagemenu(event) {
  var perPageX = parseInt(event.target.getAttribute('page'));
  if(perPageX != perPage) {
    var old = perPage * (page - 1);
    perPage = perPageX
    var n = Math.max(1, Math.ceil(rows.length / perPageX));    
    createPageMenu(n);
    displayPage(Math.min(1 + Math.floor(old / perPageX), n));
  }
}

function pagemenu(event) {
  setPage(parseInt(event.target.getAttribute('page')));
}

function pagetop() {
  setPage(1);
}

function pagebottom() {
  setPage(pages);
}

function pageup() {
  setPage(page - 1)
}

function pagedown() {
  setPage(page + 1);
}
