/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'OSServiceConfig';
var subfield = 'services';
var singular = 'OS service config';
var plural = 'OS service config';
var editfile = 'editosserviceconfig.xul';

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function allNodes(needCommit) {
  needCommit = isundefined(needCommit, false);
  var catkey = current.get('uniqueKey');
  var list = new Array();
  var found = current.cluster.find("SlaveNode");
  for(var i = 0; i < found.length; i++) {
    if(catkey == found[i].get('category')) {
      var key = found[i].get('uniqueKey');
      if(key > dock.MAX_LOCAL_KEY) {
        if((!needCommit || !found[i].get('modified')) && !found[i].get('toBeRemoved')) list.push(key);
      }
    }
  }
  return list;
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh category services" + ' - ' + document.documentURI);

  tree = document.getElementById('tree');
  data = new Array();
  var config = current.get('services');

  for(var i = 0; i < config.length; i++) {
    if(!config[i].toBeRemoved || alsoRemoved) {
      data.push(oneTreeItem(config[i]));
    }
  }

  for(var i = data.length; i < MINTREEITEMS; i++) data.push(blankTreeItem());
  var select = selected('list');
  loadTable();
  setSelected('list', select);
  dock.addHistory('Category services refreshed');
}

function oneTreeItem(one) {
  var item = new Object();
  item['key'] = one.uniqueKey + '|' + one.name;
  item['name'] = one.name;
  item['monitored'] = one.monitored;
  item['autostart'] = one.autostart;
  item['modified'] = one.modified;
  item['toBeRemoved'] = one.toBeRemoved;
  return item;
}

function newService() {
  var args = new Object();
  args['object'] = current;
  args['osserviceconfig'] = 0;
  current.cluster.currentSubObjectType = basetype;
  current.cluster.currentSubObjectList = current.get(subfield);  
  var d = new dock.cvDialog(editfile, args);
  d.onOk = function() {
    refresh();
  }
  d.onClose = function() {
    current.cluster.currentSubObjectType = 0;
    current.cluster.currentSubObjectList = 0;  
  }  
  dock.dialogs.showDialog(d);
}
function editService() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("No service selected");
  } else if(select.length == 1) {
    var name = select[0].split('|');
    var args = new Object();
    args['object'] = current;
    args['osserviceconfig'] = name[0];
    current.cluster.currentSubObjectType = basetype;
    current.cluster.currentSubObjectList = current.get(subfield);  
    var d = new dock.cvDialog(editfile, args);
    d.onOk = function() {
      refresh();
    }
    d.onClose = function() {
      current.cluster.currentSubObjectType = 0;
      current.cluster.currentSubObjectList = 0;  
    }  
    dock.dialogs.showDialog(d);
  } else {
    dock.displayError("Please select only one service");
  }
}
function removeService() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("No service selected");
  } else {
    printall(select);
    var args = new Object();
    args['question'] = "Are you sure you want to remove these services?";
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      var text = '';
      for(var i = 0; i < select.length; i++) {
        var name = select[i].split('|');
        current.deleteSubObjectFromKey('services', name[0]);
      }
      refresh();
      if(text.length > 0) dock.displayError(text);
    }
    dock.dialogs.showDialog(d);
  }
}

function statusService() {
//   var list = current.get('services');
//   var select = selected('list');
//   if(select.length == 0) {
//   } else {
//     var index = indexes(list, select);
//     if(index.length < list.length) {
//       for(var i = 0; i < index.length; i++) {
//
//       }
//     }
//   }
  var args = new Object();
  args['category'] = current;
  var d = new dock.cvDialog("categoryservicestatus.xul", args);
  //d.onOk = function() { }
  dock.dialogs.showDialog(d);
}
