/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var myDialog;
var dock
var current;

function onLoad() {
  dock = getDock();
  println("Error initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  current = myDialog.arguments.category;
  refresh();

}


function allNodes(needCommit) {
  needCommit = isundefined(needCommit, false);
  var catkey = current.get('uniqueKey');
  var list = new Array();
  var found = current.cluster.find("SlaveNode");
  for(var i = 0; i < found.length; i++) {
    if(catkey == found[i].get('category')) {
      var key = found[i].get('uniqueKey');
      if(key > dock.MAX_LOCAL_KEY) {
        if((!needCommit || !found[i].get('modified')) && !found[i].get('toBeRemoved')) list.push(key);
      }
    }
  }
  return list;
}

function refresh() {
  var nodes = allNodes();
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmserv', 'pgetOSServices', [nodes], function(code, data) {
      if(code == 0) {
        setCallbackResult(nodes, data);
      } else {
        setCallbackResult(new Array(), new Array());
        document.getElementById('model').setAttribute('value', 'N/A');
        if(code) dock.jsonCallError(this, code, data);
      }
    }, current);
}

function setCallbackResult(nodes, services) {
  var cols = document.getElementById('cols');
  while(cols.childNodes.length > 1) cols.removeChild(cols.lastChild);
  var list = document.getElementById('list');
  while(list.firstChild) cols.removeChild(list.firstChild);

  if(services.length > 0) {
    tree = document.getElementById('tree');
    data = new Array();
    var header = new Array();
    for(var i = 0; i < services.length; i++) {
      var osserv = services[i].osservices;
      for(var j = 0; j < osserv.length; j++) {
        var name = osserv[j].name;
        if(header.find(name) < 0) {
          header.push(name);
          var splitter = document.createElement('splitter');
          splitter.setAttribute('class', 'tree-splitter');
          cols.appendChild(splitter);
          var col = document.createElement('treecol');
          col.setAttribute('id', name);
          col.setAttribute('flex', '1');
          col.setAttribute('label', name);
          col.setAttribute('class', 'generictreecol');
          col.setAttribute('onclick', 'sort(this);');
          col.setAttribute('sortActive', true);
          col.setAttribute('sortDirection', 'ascending');
          cols.appendChild(col);
        }
      }
    }
    for(var i = 0; i < services.length; i++) {
      data.push(oneTreeItem(nodes[i], services[i].osservices, header));
    }
    loadTable();
  }
}

function oneTreeItem(node, one, header) {
  var item = new Object();
  item['hostname'] = current.cluster.resolveName(node);
  for(var i = 0; i < one.length; i++) {
    var status = "Unknown";
    switch(one[i].status) {
    case('DOWN') :
      status = "Down";
      break;
    case('UP') :
      status = "Up";
      break;
    case('STOPPED') :
      status = "Stopped";
      break;
    case('SICK') :
      status = "Sick (" + one[i].sicknessMessage + ")";
      break;
    case('FAILING') :
      status = "Failing";
      break;
    case('NOTNEEDED') :
      status = "Not used";
      break;
    case('UNKNOWN') :
      status = "Unknown (/etc/init.d script timeout)";
      break;
    case('NOTFOUND') :
      status = "Unknown (/etc/init.d script not found/executable)";
      break;
    }
    item[one[i].name] = status;
  }
  return item;
}
