/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var led = 0;
var leds = 0;
var ledTimer = 0;
var ledCounter = 0;
var categorynodetree;
var categorynodedata;

var goodColors = ['green', 'blue', 'red', 'amber'];
var kvmnodesfile = 'ucskvmnodes.xul';

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh UCS" + ' - ' + document.documentURI);
  
  refreshLiveNodeSummary();
  var configUCS = current.getUCS(false);
  var liveUCS = current.getLiveUCS();
  if(liveUCS) {
    refreshLive();
    refreshAll();
  } else {
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'getLiveUCS', [[current.get('uniqueKey')], false], function(code, data) {
        if(code == 0) {
          setCallbackResult(data);
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
    if(configUCS) refreshAll();
  }
}

function setCallbackResult(data) {
  if(isArray(data) && data.length) {
    current.ucsInfo = data[0];
    refreshLive();
    var configUCS = current.getUCS(false);
    if(!configUCS) refreshAll();
  }
}

function stop() {
  println("Stop ucs settings");
  if(ledTimer) window.clearTimeout(ledTimer);
  ledCounter = 0;
}

function refreshLiveNodeSummary() {
  println("++ UCS refreshLive category nodes...");
  categorynodetree = document.getElementById('categorynodetree');
  categorynodedata = new Array();
  var nodelist = current.cluster.findNodesIn(current.get('baseType'), current.get('uniqueKey'));
  for ( var i = 0; i < nodelist.length; i++) {
    var nodeobj = current.cluster.find(nodelist[i]);
    if (nodeobj) {
      var foundcimc = false;
      var interfaces = nodeobj.get('networks');
      for(var j = 0; j < interfaces.length; j++) {
        if(startswith(interfaces[j].name, 'cimc')) {
          foundcimc = true;
          break;
        }
      }
      if (foundcimc) { 
        categorynodedata.push(createTreeItemNodeSummary(nodeobj, i));
      }
    }
  }
  for(var i = nodelist.length; i < MINTREEITEMS; i++) categorynodedata.push(blankTreeItem());
  var select = selected('nodelistcategory');
  loadTable(categorynodetree, categorynodedata);
  setSelected('nodelistcategory', select);
}

function blinking() {
  var needTimer = false;
  if(led) {
    var obj = document.getElementById('locator-led');
    if(obj) {
      var color = led.color;
      obj.setAttribute("tooltiptext", color + ' ' + led.operState);
      if(goodColors.find(color) >= 0) {
        if(led.operState == 'blinking') {
          needTimer = true;
          if(ledCounter % 2 == 1) color = 'blank';
        }
        if(ledCounter == 0 || led.operState == 'blinking') obj.setAttribute("src", "./skin/image/LED_" + color + ".png");
      }
    }
  }
  if(leds) {
    for(var i = 0; i < leds.length; i++) {
      var name = leds[i].name;
      var obj = document.getElementById(name);
      if(obj) {
        var color = leds[i].color;
        obj.setAttribute("tooltiptext", color + ' ' + leds[i].operState);
        if(goodColors.find(color) >= 0) {
          if(leds[i].operState == 'blinking') {
            needTimer = true;
            if(ledCounter % 2 == 1) color = 'blank';
          } else if(ledCounter) {
            continue;
          }
          obj.setAttribute("src", "./skin/image/LED_" + color + ".png");
        }
      }
    }
  }
  ledCounter++;
  if(needTimer) ledTimer = window.setTimeout('blinking();', 1000);
}

function refreshAll() {
  println("++ UCS refreshAll");
  for(var i = 0; i < window.frames.length; i++) {
    if(!!window.frames[i].refresh) {
      window.frames[i].refresh(current);
    }
  }
}

function openKVMNodesDialog() {
  var args = new Object();
  args['current'] = current;
  var d = new dock.cvDialog(kvmnodesfile, args);
  dock.dialogs.showDialog(d);
}

function createTreeItemNodeSummary(node, i) {
  var item = new Object();
  if(node.get('uniqueKey')) {
    item['key'] = node.get('uniqueKey');
  } else {
    item['key'] = -1000 -i;
  }
  item['nodename'] = node.resolveName();
  if (node.ucsInfo) {
    var ucsstatus = node.ucsInfo.ucsStatus;
    var firmware = node.ucsInfo.firmwareRunning;
    var numvnics = (node.ucsInfo.vicAdapterSettings) ? node.ucsInfo.vicAdapterSettings.length:0;
    var numvhbas = (node.ucsInfo.fcAdapterSettings) ? node.ucsInfo.fcAdapterSettings.length:0;
    if (ucsstatus) {
      item['nodemodel'] = ucsstatus['model'];
      item['nodeserial'] = ucsstatus['serial'];
      item['nodeuuid'] = ucsstatus['uuid'];
      item['nodevendor'] = ucsstatus['vendor'];
      item['nodenumvnics'] = numvnics;
      item['nodenumvhbas'] = numvhbas;
    }
    var firmware = node.ucsInfo.firmwareRunning;
    if(firmware){
      for(var i = 0; i < firmware.length; i++) {
        item['nodefirmware' + firmware[i].type] = firmware[i].version;
      }
    }
    var bootorder = node.ucsInfo.biosBootOrder;
    var bootOrderCache = new Object();
    var bootorderstring = "";
    for (var i = 0; i < bootorder.length; i++) {
      bootOrderCache[bootorder[i].order] = getBootOrderShortName(bootorder[i]);
    }
    for ( var k in bootOrderCache ) {
      bootorderstring += (k == bootorder.length) ? k + ':' + bootOrderCache[k]:k + ':' + bootOrderCache[k] + ','; 
    }
    item['nodebootorder'] = bootorderstring;
  }
  return item;
}

function selectNodeObject() {
  var selecteditem = selected('nodelistcategory');
  if(selecteditem.length == 0) {
    dock.displayError("Please select a device");
  } else if(selecteditem.length > 1) {
    dock.displayError("Please select only one device");
  } else {
    current.cluster.select(selecteditem[0]);
  }
}

function getBootOrderShortName (obj) {
  if (obj.type) {
    if (obj.type.indexOf("hdd-order") == 0) {
      return "HDD";
    } else if (obj.type.indexOf("internal-efi-shell") == 0) {
      return "EFI";
    } else if (obj.type.indexOf("network-device-order") == 0) {
      return "PXE";
    } else {
      return obj.type;
    }
  }
}
