/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var IS_CEPH_TAB = 1;

var dock;
var current = 0;
var overview;

var oldcluster = 0;
var allCephOSDNodeKeys;

var graph;

var timer = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
  addEventListener("resize", onResize, false);
}

function onResize() {
  var svg = document.getElementById("svg");
  if(svg) {
    svg.setAttribute("height", "25px");
    svg.setAttribute("width", svg.parentNode.boxObject.width);
    svg.setAttribute("height", svg.parentNode.boxObject.height);
    redraw();
  }
}

function getCurrentSelection() {
  return [[current], 0, ""];
}

function stop() {
  println("Stop Ceph overview");
  if(graph) graph.stop();
  if(timer) window.clearTimeout(timer);
}

function redraw() {
  if(graph) {
    graph.update();
  }
}

function hideCeph() {  
  if(parent.document.getElementById('noceph').hasAttribute('hidden')) {
    parent.document.getElementById('tabbox').setAttribute('hidden', true);
    parent.document.getElementById('noceph').removeAttribute('hidden');
  }
}

function showCeph() {
  if(parent.document.getElementById('tabbox').hasAttribute('hidden')) {
    parent.document.getElementById('tabbox').removeAttribute('hidden');
    parent.document.getElementById('noceph').setAttribute('hidden', true);
  }
}

function refresh(use) {
  println("Refresh ceph overview" + ' - ' + document.documentURI);
  if(!!use) {
    current = use.find('Ceph');
    if(current.length == 1) {
      current = current[0];
      showCeph();
    } else {
      current = 0;
      hideCeph();
      return;
    }
  } else if(!current) {
    return;
  }
  
  var list = document.getElementById('nodes');
  while(list.childNodes.length > 2) list.removeChild(list.lastChild);

  allCephOSDNodeKeys = new Array();
  var roles = new Object();
  var cephKey = current.get('uniqueKey');
  var nodes = current.cluster.find('Node');
  for(var i = 0; i < nodes.length; i++) {
    var cephRoles = nodes[i].getCephRoles(cephKey);
    if(cephRoles.length) {
      for(var j = 0; j < cephRoles.length; j++) {
        cephRoles[j] = cephRoles[j].childType.replace("Ceph", "").replace("Role", "");
      }
      var role = cephRoles.join(', ');
      if(roles.hasOwnProperty(role)) {
        roles[role].push(nodes[i].get('hostname'));
      } else {
        roles[role] = [nodes[i].get('hostname')];
      }
      if (role.search('OSD') >= 0) {
        allCephOSDNodeKeys.push(nodes[i].get('uniqueKey'));
      }
    }
  }
  for(var role in roles) {
    list.appendChild(cephNodeRow(role, dotFormat(roles[role])));
  }
  update();
  
  var cluster = current.cluster;
  if(cluster != oldcluster) {
    setCallbackResult(new dock.cvGuiClusterOverview());
    if(graph) {
      graph.cluster = cluster;
      graph.deviceMetric[0].nodes = allCephOSDNodeKeys;
      graph.clearScreen();
    } else {
      graph = new cvGraph(cluster, document.getElementById("svg"), document.getElementById("box"));
      graph.WM = 32;
      graph.HM = 61;
      graph.legend = -1;
    }
    oldcluster = cluster;
  }  
  
  var list = document.getElementById("metriclist");
  var oldkey = parseInt(list.getAttribute('metricKey'));
  var oldparam = list.getAttribute('param');
  var oldlabel = list.getAttribute('label');
  
  var name = cluster.find(oldkey);
  if(name) {
    name = name.resolveName();
    if(!startswith(oldlabel, name)) {
      oldkey = 0;
    }
  }
  
  if(kisNaN(oldkey) || (oldkey == 0)) {
    if(isArray(cluster.cephMetric)) {
      oldkey = cluster.cephMetric[0];
      oldparam = cluster.cephMetric[1];
      oldlabel = '';
      var metric = cluster.find(cluster.cephMetric[0]);
      if(metric) {
        var text = metric.resolveName();
        if(cluster.cephMetric[1].length > 0) text += "[" + cluster.cephMetric[1].metricParam + "]";
        if(metric.get('unit')) text += " (" + metric.get('unit') + ")";
        oldlabel = text;
      }
    }
  }

  var mmc = cluster.allMetrics();
  createMetricPopup(cluster, mmc, 'metriclist', false, true, ['OPENSTACK','DELLNSS','AMAZONEC2','HADOOP_JVM','HADOOP_DFS','HADOOP_UGI','HADOOP_MAPRED','HADOOP_RPC','HADOOP_RPCDETAILED','HADOOP_METRICSYSTEM', 'LUSTRE']);

  var found = false;
  for(var i = 0; (i < mmc.length) && !found; i++) {
    found = ((mmc[i].metric == oldkey) && (mmc[i].metricParam == oldparam));
  }

  if(!found) {
    if(mmc.length == 0) {
      var item = document.createElement("menuitem");
      item.setAttribute('metricKey', 0);
      item.setAttribute('param', '');
      item.setAttribute('label', '');
      list.firstChild.appendChild(item);
      oldkey = 0;
      oldparam = '';
      oldlabel = '';
    } else {
      oldkey = mmc[0].metric;
      oldparam = mmc[0].metricParam;
      var def = cluster.find('Metric', current.resolveName() + '_osd.stat_bytes_used_sum');
      if(def) {
        def = def.get('uniqueKey');
        for(var i = 0; i < mmc.length; i++) {
          if(mmc[i].metric == def) {
            oldkey = def
            oldparam = mmc[i].metricParam;
            break;
          }
        }
      }
      var metric = cluster.find(oldkey);
      if(metric) {
        oldlabel = metric.resolveName();
        if(oldparam.length > 0) oldlabel += "[" + oldparam + "]";
        if(metric.get('unit')) oldlabel += " (" + metric.get('unit') + ")";
      }
    }
  }
  list.setAttribute('metricKey', oldkey);
  list.setAttribute('param', oldparam);
  list.setAttribute('label', oldlabel);
  
  if(graph.deviceMetric.length > 0) {
    graph.deviceMetric[0].nodes = allCephOSDNodeKeys;
    graph.deviceMetric[0].metric = oldkey;
    graph.deviceMetric[0].param = oldparam;
  } else {
    var dmp = new cvGraphDeviceMetric(0, oldkey, oldparam, allCephOSDNodeKeys);
    dmp.groupType = 1; // heatmap
    graph.allowRedraw(true);      
    var r = graph.add(dmp);
  }
  redraw();
}

function update() {
   dock.workerpool.getWorker(current.cluster.actualurl).call('cmgui', 'getCephOverview', [current.cluster.sessionid, current.get('uniqueKey')], function(code, data) {
       if(code == 0) {
         setCallbackResult(data);
       } else {
         dock.jsonCallError(this, code, data);
       }
     }, current);
}

function setCallbackResult(value) {
  overview = value;
  //printall(overview);
  //printall(current.object);
  overviewValue('status');
  overviewValue('numMons');
  overviewValue('numPgs');
  overviewValue('numOsds');
  overviewValue('numUpOsds');
  overviewValue('numInOsds');
  overviewBar('pgmap', overview.pgsBytesUsed, overview.pgsBytesTotal, 'B', true);
 
  overviewOsdPool(overview.osdpools);
 
  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 1000);  
}

function overviewOsdPool(pools) {
  if (typeof pools === "undefined" || pools.length == 0) {
    var obj = document.getElementById('osdpoolinfo');
    obj.setAttribute('hidden', true);
    return;
  } else {
    var obj = document.getElementById('osdpoolinfo');
    obj.setAttribute('hidden', false);
  }
//  printall(pools);
  obj = document.getElementById('osdpool');
  while (obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
  for (var i = 0; i < pools.length; i++) {
    obj.appendChild(OsdPoolInfo(pools[i].name, pools[i].bytes, pools[i].objects, pools[i].clones, pools[i].degraded, pools[i].unfound, pools[i].rd, pools[i].rdBytes, pools[i].wr, pools[i].wrBytes));

  }

  function OsdPoolInfo(name, kb, objects, clones, degraded, unfound, rd, rdKB, wr, wrKB) {
    var row = document.createElement('row');
    row.setAttribute("flex",1);
    row.setAttribute("align", "center");
    row.setAttribute("height", "17px");
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", name);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", formatKMGselect(kb, 'B'));
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", objects);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", clones);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", degraded);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", unfound);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", rd);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", formatKMGselect(rdKB, 'B'));
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", wr);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", formatKMGselect(wrKB, 'B'));
    row.appendChild(label);
    return row
  }
}

function cephNodeRow(roles, hosts) {
  var row = document.createElement('row');
  row.setAttribute("flex", "1");
  row.setAttribute("align", "center");
  row.setAttribute("height", "17px");
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", roles);
  row.appendChild(label);
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", hosts);
  row.appendChild(label);
  return row;
}

function selectMetric(event) {
  var list = document.getElementById("metriclist");
  var metric = event.target;
  if(!!metric) {
    graph.last = 0;
    var old = graph.deviceMetric[0].device;
    graph.deviceMetric[0].metric = parseInt(metric.getAttribute('metricKey'));
    graph.deviceMetric[0].param = metric.getAttribute('param');
    graph.deviceMetric[0].unit = current.cluster.find(graph.deviceMetric[0].metric).get('unit');
    var name = metric.getAttribute('label');
    list.setAttribute('metricKey', graph.deviceMetric[0].metric);
    list.setAttribute('param', graph.deviceMetric[0].param);
    list.setAttribute('label', name);
    if(old != graph.deviceMetric[0].device) graph.handleDeviceChange(graph.deviceMetric[0]);
    redraw();
    current.hadoopMetric = [graph.deviceMetric[0].metric, graph.deviceMetric[0].param];
  }
}
