/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var myDialog;
var current;

var currentDeck;
var DECK_GENERAL = 'general';
var DECK_MONITORS = 'monitors';
var DECK_OSDS = 'osds';
var DECK_SUMMARY = 'summary';
var DECK_DEPLOY = 'deploy';

var timer = 0;
var count = 0;

var userWantsToFixOSDPoolDefaultSize = true;
var fakeModalDialogActive = false;

var userChoices = {
  globalJournalSize: 5120,
  globalOSDPoolDefaultSize: 3,
  publicNetworkKey: -1,
  clusterNetworkKey: -1,

  pickCtor: {
    'Monitor': CephMonitorContainer,
    'OSD': CephOSDContainer
  },

  containerMap: {
    'Monitor': {},
    'OSD': {}
  },
  updateContainer: function(updatedContainer) {
    this.containerMap[updatedContainer.serviceType][updatedContainer.uniqueKey] = updatedContainer;
  },
  addContainers: function(serviceType, baseType, keys) {
    var ctorF = this.pickCtor[serviceType];
    for (var i = 0; i < keys.length; i++) {
      var key = keys[i];
      if (!(key in this.containerMap[serviceType])) {
        this.containerMap[serviceType][key] = new ctorF(baseType, key);
      }
    }
  },
  getContainer: function(serviceType, uniqueKey) {
    return this.containerMap[serviceType][uniqueKey];
  },
  getContainers: function(serviceType, baseType) {
    var that = this;
    var containers = Object.keys(that.containerMap[serviceType]).map(function(key) {
      return that.containerMap[serviceType][key];
    });
    if (typeof(baseType) !== 'undefined') {
      containers = containers.filter(function(item) { return item.baseType === baseType });
    }
    return containers;
  },
  removeContainers: function(serviceType, keys) {
    var that = this;
    keys.forEach(function(key) {
      delete that.containerMap[serviceType][key];
    });
  },

  getNumberOfNodesInCategory: function(categoryUniqueKey) {
    var nodeKeys = current.findNodesIn('category', categoryUniqueKey);
    return nodeKeys.length;
  },

  getNodeObjects: function(serviceType) {
    var nodes = this.getContainers(serviceType, 'Node').map(function(nodeContainer) {
      return current.find(nodeContainer.uniqueKey);
    });
    this.getContainers(serviceType, 'Category').forEach(function(catContainer) {
      var nodeKeys = current.findNodesIn('category', catContainer.uniqueKey);
      nodes = nodes.concat(nodeKeys.map(function(nodeKey) {
        return current.find(nodeKey);
      }));
    });
    return nodes;
  },

  getNodeObjectsFiltered: function(serviceType, filterF) {
    var nodes = this.getContainers(serviceType, 'Node').filter(filterF).map(function(nodeContainer) {
      return current.find(nodeContainer.uniqueKey);
    });
    this.getContainers(serviceType, 'Category').filter(filterF).forEach(function(catContainer) {
      var nodeKeys = current.findNodesIn('category', catContainer.uniqueKey);
      nodes = nodes.concat(nodeKeys.map(function(nodeKey) {
        return current.find(nodeKey);
      }));
    });
    return nodes;
  },

  getAllOSDHostnames: function () {
    var osdNodeNames = this.getNodeObjects('OSD').map(function(node) {
      return node.get('hostname');
    });
    
    return new Set(osdNodeNames);
  },

  countOSDs: function () {
    var configuredOSDs = this.containerMap['OSD'];
    var keys = Object.keys(configuredOSDs);

    var self = this;
    var count = keys.reduce(function(previousValue, currentValue, currentIndex, array) {
      var currentNode = configuredOSDs[currentValue];
      
      if (currentNode.baseType === 'Node') {
        return previousValue + currentNode.osdAssociations.length;
      }
      else if (currentNode.baseType === 'Category') {
        return previousValue + self.getNumberOfNodesInCategory(currentNode.uniqueKey) * currentNode.osdAssociations.length;
      }
    }, 0);
    
    return count;
  },

  toXML: function() {
    var xml = toXML(this);
    return vkbeautify.xml(xml, 2);
  }
};

var defaultNetwork = {
    name: '<default>',
    uniqueKey: -1,
  };

function _get_name() {
  var fieldName = (this.baseType === 'Node') ? 'hostname' : 'name';
  return current.find(this.uniqueKey).get(fieldName);
}

function CephMonitorContainer(baseType, uniqueKey) {
  this.uniqueKey = uniqueKey;
  this.baseType = baseType; // 'Node' or 'Category'
  this.serviceType = 'Monitor';
  this.bootstrap = 'auto';
  this.dataPath = '/var/lib/ceph/mon/$cluster-$hostname';
};
CephMonitorContainer.prototype.bootstrap_options = ['auto', 'true', 'false'];
CephMonitorContainer.prototype.getName = function() {
  return _get_name.call(this);
}
CephMonitorContainer.prototype.createTreeItem = function() {
  return {
    key: this.uniqueKey,
    baseType: this.baseType,
    name: this.getName(),
    bootstrap: this.bootstrap,
    dataPath: this.dataPath
  };
}
CephMonitorContainer.prototype.validate = function() {
  var results = [];
  if (this.dataPath.length === 0) {
    var one = "The 'Data path' field cannot be empty. The default value is '/var/lib/ceph/mon/$cluster-$hostname'.";
    results.push(one);
  }
  return results;
}
CephMonitorContainer.prototype.clone = function() {
  var newContainer = new CephMonitorContainer(this.baseType, this.uniqueKey);
  newContainer.bootstrap = this.bootstrap;
  newContainer.dataPath = this.dataPath;
  return newContainer;
}

function CephOSDContainer(baseType, uniqueKey) {
  this.uniqueKey = uniqueKey;
  this.baseType = baseType; // 'Node' or 'Category'
  this.serviceType = 'OSD';
  this.sharedJournalDevice = '';
  this.osdAssociations = [ new OSDAssociation('osd0') ];
};
CephOSDContainer.prototype.getName = function() {
  return _get_name.call(this);
}
CephOSDContainer.prototype.createTreeItem = function() {
  return {
    key: this.uniqueKey,
    baseType: this.baseType,
    name: this.getName(),
    sharedJournalDevice: this.sharedJournalDevice,
    osdAssociations: this.osdAssociations.map(function(assoc) {
      return assoc.name;
    }).join(',')
  };
}
CephOSDContainer.prototype.validate = function() {
  var results = [];
  return results;
}
CephOSDContainer.prototype.clone = function() {
  var newContainer = new CephOSDContainer(this.baseType, this.uniqueKey);
  newContainer.sharedJournalDevice = this.sharedJournalDevice;
  newContainer.osdAssociations = this.osdAssociations.map(function(assoc) {
    return assoc.clone();
  });
  return newContainer;
}

function getNumericIndexFromDeckString(deckString) {
  // get index for nextDeck
  var deck = document.getElementById('deck');
  for (var i = 0; i < deck.childNodes.length; i++) {
    var cephindex = deck.childNodes[i].getAttribute('cephindex');
    if (deckString == cephindex) {
      return i;
    }
  }
}

function setStage(num) {
  for (var i = 1; i <= 5; i++) {
    var stageid = 'stage' + i;
    if (i == num) {
      setAttr(stageid,'style', 'margin: 0px; padding: 1px; font-size: 12px; color: black; background-color: #7cc200;');
    } else {
      setAttr(stageid, 'style', 'margin: 0px; padding: 1px; font-size: 12px;');
    }
  }
}

function fillListDropdown(popupId, list, excludeFunction) {
  excludeFunction = isundefined(excludeFunction, false);
  var drop = document.getElementById(popupId);
  var popup = drop.firstChild;
  if (!popup) {
    println("!!! ERROR !!!: has no menupopup");
    return;
  }
  while (popup.firstChild) popup.removeChild(popup.firstChild);
  if (excludeFunction && (typeof(excludeFunction) == 'string')) eval('excludeFunction = ' + excludeFunction);
  for (var i = 0; i < list.length; i++) {
    var key = (typeof(list[i].uniqueKey) != 'undefined' ? list[i].uniqueKey : list[i].get('uniqueKey'));
    var name = (typeof(list[i].resolveName) != 'undefined' ? list[i].resolveName() : list[i].name);
    if (excludeFunction && excludeFunction(name)) continue;
    var item = document.createElement('menuitem');
    item.setAttribute('key', key);
    item.setAttribute('label', name);
    popup.appendChild(item);
  }
}

function setDefaultIfPresent(dropDownID, defaultVal) {
  var dropDown = document.getElementById(dropDownID);
  for (var i = 0; i < dropDown.itemCount; i++) {
    if (dropDown.getItemAtIndex(i).getAttribute('label') == defaultVal) {
      dropDown.selectedIndex = i;
      break;
    }
  }
}

function onLoad() {
  dock = getDock();
  println("Add Ceph Wizard initialized: " + this.name);
  var DIALOG_NUM = /.*\[(\d+)\]/g;
  var index = parseInt(DIALOG_NUM.exec(this.name)[1]);
  myDialog = dock.dialogs.list[index];
  current = myDialog.arguments.cluster;
  currentDeck = DECK_GENERAL;
  generalSettingsScreen();
}

function next() {
  println('next from ' + currentDeck);

  switch (currentDeck) {
    case DECK_GENERAL:
      if (!checkAndSaveGeneralSettings()) return;
      currentDeck = DECK_MONITORS;
      monitorSelectScreen();
      break;
    case DECK_MONITORS:
      if (!checkMonitors()) return;
      currentDeck = DECK_OSDS;
      osdSelectScreen();
      break;
    case DECK_OSDS:
      if (!checkOSDs()) return;
      currentDeck = DECK_SUMMARY;
      summaryScreen();
      break;
    case DECK_SUMMARY:
      currentDeck = DECK_DEPLOY;
      deploy();
      break;

  }

  configureButtons();
  refresh();
}

function prev() {
  println('prev from ' + currentDeck);

  switch (currentDeck) {
    case DECK_MONITORS:
      currentDeck = DECK_GENERAL;
      generalSettingsScreen();
      break;
    case DECK_OSDS:
      currentDeck = DECK_MONITORS;
      monitorSelectScreen();
      break;
    case DECK_SUMMARY:
      currentDeck = DECK_OSDS;
      osdSelectScreen();
      break;
    case DECK_DEPLOY:
      currentDeck = DECK_SUMMARY;
      summaryScreen();
      break;
  }

  configureButtons();
  refresh();
}

function cancel() {
  var args = { question: 'Are you sure you want to quit the wizard?' };
  var d = new cvDialog("chrome://cmgui/content/question.xul", args);
  d.onOk = function() {
    close();
  };
  dock.dialogs.showDialog(d);
}

function validNetworks() {
  var networks = current.find('Network');
  return networks.filter(function(network) {
    return network.get('type') != 'GLOBAL';
  });
}

function getNetworkNameByUniqueKey(uniqueKey) {
  if (uniqueKey === -1) {
    return defaultNetwork.name;
  }
  
  return current.find(uniqueKey).get('name')
}

function checkAndSaveGeneralSettings() {
  var globalJournalSize = getAttr('globalJournalSize', 'value');
  if ((globalJournalSize.length === 0) || (Number(globalJournalSize) < 0)) {
    dock.displayError("The 'Journal size' field must contain a number equal to or greater than 0.");
    return false;
  }
  userChoices.globalJournalSize = Number(globalJournalSize);

  var globalOSDPoolDefaultSize = getAttr('globalOSDPoolDefaultSize', 'value');
  if ((globalOSDPoolDefaultSize.length === 0) || (Number(globalOSDPoolDefaultSize) < 1)) {
    dock.displayError("The 'OSD Pool Default Size' field must contain a number equal to or greater than 1.");
    return false;
  }
  userChoices.globalOSDPoolDefaultSize = Number(globalOSDPoolDefaultSize);

  var pnName = getAttr('publicNetwork', 'label');

  var cmdPublicNetwork = current.find('Network', pnName);
  if (cmdPublicNetwork)
    userChoices.publicNetworkKey = cmdPublicNetwork.get('uniqueKey');
  else
    userChoices.publicNetworkKey = -1;
  
  var cnName = getAttr('clusterNetwork', 'label');
  var cmdClusterNetwork = current.find('Network', cnName);

  if (cmdClusterNetwork)
    userChoices.clusterNetworkKey = cmdClusterNetwork.get('uniqueKey');
  else
    userChoices.clusterNetworkKey = -1;

  return true;
}

function checkMonitors() {
  if (userChoices.getContainers('Monitor').length === 0) {
    dock.displayError('At least one Monitor must be present.');
    return false;
  }
  return true;
}

// modal dialogs have been disabled in r4502, but we need something
// that will disable the buttons while the dialog is active
function setFakeModalDialogActive(value) {
  fakeModalDialogActive = value;
  configureButtons();
}

function fixCommonPoolSizeError() {
  var nOsdNodes = userChoices.getAllOSDHostnames().size;

  if (nOsdNodes == 2 && userChoices.globalOSDPoolDefaultSize == 3) {

    // the dialog is shown asynchronously, we need extra variables to track state
    if (!userWantsToFixOSDPoolDefaultSize) {
      return true;
    }

    var args = {
      yeslabel: "OK",
      yesaccesskey: "O",
      nolabel: "Skip",
      noaccesskey: "S",
      question: "You have configured 2 OSD nodes, but the OSD Default Pool Size " +
                "(number of copies of the data) is currently still set at its default of 3. " +
                "You will not be able to reach a clean Ceph state with these settings.\n\n" +
                "Press OK to lower the OSD Default Pool Size to 2, to match the number of OSD nodes."
    }

    var d = new cvDialog("chrome://cmgui/content/question.xul", args);
    d.onOk = function() {
      userChoices.globalOSDPoolDefaultSize = 2;
      setAttr('globalJournalSize', 'value', userChoices.globalJournalSize);
      next();
    };
    d.onNo = function() {
      userWantsToFixOSDPoolDefaultSize = false;
      next();
    };
    d.onClose = function () {
      setFakeModalDialogActive(false);
    }

    setFakeModalDialogActive(true);
    dock.dialogs.showDialog(d);
    return false;
  }
  return true;
}

function checkOSDs() {
  if (!fixCommonPoolSizeError()) {
    return false;
  }
  return true;
}

function generalSettingsScreen() {
  setStage(1);
  setTextMessages({
    networksMessage: "Select public and cluster networks for Ceph. Public network is a network " +
                     "for communicating between Ceph Monitors, OSDs and clients. Cluster network " +
                     "is for OSDs to do rebalancing of data. If unsure use defaults.",
    journalMessage: "Specify default journal size for OSDs. It's possible to override this value for " +
                    "individual OSDs in the 'Configure Ceph OSDs' section of this wizard."
  });
  var osdPoolDefaultSizeMessage = "Sets the number of replicas for objects in the pool.\nDo not set this number to more than the number of OSDs deployed in the cluster.";

  var pnName = defaultNetwork.name;
  if (userChoices.publicNetworkKey !== -1) {
    pnName = current.find(userChoices.publicNetworkKey).get('name');
  }
  var cnName = defaultNetwork.name;
  if (userChoices.clusterNetworkKey !== -1) {
    cnName = current.find(userChoices.clusterNetworkKey).get('name');
  }
  var networks = validNetworks();
  networks.unshift(defaultNetwork);
  
  fillListDropdown('publicNetwork', networks);
  setDefaultIfPresent('publicNetwork', pnName);
  fillListDropdown('clusterNetwork', networks);
  setDefaultIfPresent('clusterNetwork', cnName);
  setAttr('globalJournalSize', 'value', userChoices.globalJournalSize);

  setAttr('globalOSDPoolDefaultSizeTip', 'tooltiptext', osdPoolDefaultSizeMessage);
  setAttr('globalOSDPoolDefaultSize', 'value', userChoices.globalOSDPoolDefaultSize);
}

function getCurrentServiceType() {
  var serviceType = '';
  if (currentDeck === DECK_MONITORS) {
    serviceType = 'Monitor';
  } else if (currentDeck === DECK_OSDS) {
    serviceType = 'OSD';
  }
  return serviceType;
}

function configureButtons() {
  var pages = document.getElementById('deck').childNodes.length - 1;
  var index = parseInt(document.getElementById('deck').selectedIndex);
  println("current deck: " + currentDeck + ", pages: " + pages + ", index: " + index);
  index = getNumericIndexFromDeckString(currentDeck);
  println('going to ' + currentDeck + ' (' + index + ')');
  document.getElementById('deck').selectedIndex = index;
  document.getElementById('prev').setAttribute('hidden', ((index === 0)
      || (index === pages)));
  document.getElementById('next').setAttribute('hidden', index >= pages-1);
  document.getElementById('deploy').setAttribute('hidden', index !== pages-1);
  document.getElementById('finish').setAttribute('hidden', index < pages);
  document.getElementById('finish').setAttribute('disabled', true);
  document.getElementById('retry').setAttribute('hidden', true);

  // disable buttons to fake modal behaviour
  setAttr('prev', 'disabled', fakeModalDialogActive);
  setAttr('next', 'disabled', fakeModalDialogActive);
  setAttr('canc', 'disabled', fakeModalDialogActive);
  setAttr('selectCategoriesButton', 'disabled', fakeModalDialogActive);
  setAttr('selectNodesButton', 'disabled', fakeModalDialogActive);
  if (fakeModalDialogActive) {
    disableRowModifications();
  }

  var serviceType = getCurrentServiceType();
  if (serviceType) {
    setAttr('selectEditButton', 'oncommand', "editItem('" + serviceType + "')");
    setAttr('selectCategoriesButton', 'oncommand', "selectItems('" + serviceType + "', 'Category');");
    setAttr('selectNodesButton', 'oncommand', "selectItems('" + serviceType + "', 'Node');");
    setAttr('selectRemoveButton', 'oncommand', "removeItems('" + serviceType + "');");
  }
  setAttr('selectEditButton', 'hidden', serviceType === '');
  setAttr('selectCategoriesButton', 'hidden', serviceType === '');
  setAttr('selectNodesButton', 'hidden', serviceType === '');
  setAttr('selectRemoveButton', 'hidden', serviceType === '');
}

function monitorSelectScreen() {
  setStage(2);
  setTextMessages({
    monitorsMessage: "Add nodes and categories to be configured as Ceph Monitors using the " +
                     "'Add Nodes' and 'Add Categories' buttons at the bottom. Properties of a " +
                     "Monitor can be edited using the 'Edit' button.\n\nFor production deployments " +
                     "it is recommended to have an odd number of Monitors, at least 3."
  });
  disableRowModifications();
}

function osdSelectScreen() {
  userWantsToFixOSDPoolDefaultSize = true;
  setStage(3);
  setTextMessages({
    osdsMessage: "Add nodes and categories to be configured as Ceph OSDs. For each node and category " +
                 "a number of OSD Associations, each corresponding to a particular OSD service, can " +
                 "be specified and configured using the 'Edit' button.\n\nFor production deployments " +
                 "it is recommended to have at least 3 OSD nodes."
  });
  disableRowModifications();
}

function WarningWithConfirmation(messagesElt, confirmElt) {
  this.messagesElt = messagesElt;
  this.confirmElt = confirmElt;
}

WarningWithConfirmation.prototype.set = function(messages) {
  this.clear();
  if (messages && messages.length) {
    this.setMessages(messages);
    this.addConfirmation();
  }
}

WarningWithConfirmation.prototype.clear = function() {
  while (this.messagesElt.hasChildNodes()) {
    this.messagesElt.removeChild(this.messagesElt.firstChild);
  }
  while (this.confirmElt.hasChildNodes()) {
    this.confirmElt.removeChild(this.confirmElt.firstChild);
  }
  setAttr("deploy", "disabled", "false");
}

WarningWithConfirmation.prototype.setMessages = function(messages) {
  var header = document.createElement("shadowlabel");
  header.setAttribute("class", "plain");
  header.setAttribute("value", "Detected issues:");
  header.setAttribute("style", "font-size: 14px;");

  this.messagesElt.appendChild(header);
  var messagesElt = this.messagesElt;
  
  messages.forEach(function(message) {
    var hbox = document.createElement('hbox');
    hbox.setAttribute("style", "width: 150px;");
    hbox.setAttribute("class", "warning");

    var image = document.createElement('image');
    image.setAttribute("style", "margin-left: 5px;");
    image.setAttribute("src", "chrome://cmgui/content/icons/warning.png");

    var label = document.createElement("label");
    label.setAttribute("value", message.title);

    var tooltip = document.createElement('image');
    tooltip.setAttribute("src", "chrome://cmgui/content/icons/notice.png");
    tooltip.setAttribute("tooltiptext", message.message);
    
    hbox.appendChild(image);
    hbox.appendChild(label);
    hbox.appendChild(tooltip);

    messagesElt.appendChild(hbox);
  });
}

function acceptWarnings(event) {
  var checkbox = event.target;

  if (checkbox.checked)
    setAttr("deploy", "disabled", "false");
  else
    setAttr("deploy", "disabled", "true");
}

WarningWithConfirmation.prototype.addConfirmation = function() {
  setAttr("deploy", "disabled", "true");

  var confirmCheckbox = document.createElement('checkbox');
  confirmCheckbox.setAttribute("label", "Potential issues have been detected in the current configuration. Please resolve the problems, or select the checkbox to proceed anyway.");
  confirmCheckbox.setAttribute("checked", "false");
  confirmCheckbox.setAttribute("oncommand", "return acceptWarnings(event);");
  
  this.confirmElt.appendChild(confirmCheckbox);
}

var warningWithConfirmation = null;

function summaryScreen() {
  setStage(4);
  setTextMessages({
    summaryChoicesMessage: "The selected choices are shown below. They will be used to create an XML configuration file.",
    summaryMessage: "By pressing 'Deploy' a temporary XML file will be created with the selected configuration and Ceph " +
                    "will be deployed using this XML file.\n\n" +
                    "Before proceeding please ensure the nodes configured to be Ceph Monitors or OSDs are up. " +
                    "The nodes might get rebooted afterwards to reprovision and/or have their disk layout adjusted.\n\n" +
                    "Alternatively, clicking the 'Save Configuration' " +
                    "button will produce an XML configuration file which can be further customized, if needed, and then " +
                    "used as the input configuration file for the cm-ceph-setup command line utility.\n" +
                    "Press 'Show Configuration' to get the Ceph deployment configuration as XML."
  });

  var box = document.getElementById('summaryChoices');
  while (box.firstChild) box.removeChild(box.firstChild);

  println("cluster network uniqueKey = " + userChoices.clusterNetworkKey);
  appendChoice(box, 'Public network: ', getNetworkNameByUniqueKey(userChoices.publicNetworkKey));
  appendChoice(box, 'Cluster network: ', getNetworkNameByUniqueKey(userChoices.clusterNetworkKey));
  appendChoice(box, 'Journal size: ', userChoices.globalJournalSize + ' MiB');
  appendChoice(box, 'OSD Pool Default Size: ', userChoices.globalOSDPoolDefaultSize);
  
  var monitorNodeNames = userChoices.getNodeObjects('Monitor').map(function(node) {
    return node.get('hostname');
  }).join(', ');
  appendChoice(box, 'Monitor nodes: ', monitorNodeNames);

  var osdNamesSet = userChoices.getAllOSDHostnames();

  var warningMessages = []

  if (osdNamesSet.size === 0) {
    var osdWarning = "No OSDs are defined. While it is possible to deploy a Ceph cluster with Monitors " +
                     "only, it won't be possible to store any data on it. Please add OSDs either by going " +
                     "back to the 'Configure Ceph OSDs' section, or manually after running the wizard by assigning Ceph OSD " +
                     "role to the appropriate categories or nodes.\n\n";

    warningMessages.push({title: "No OSDs defined.",
                          message: osdWarning});
  } else {
    appendChoice(box, 'OSD nodes: ', [...osdNamesSet].join(', '));

    if (userChoices.globalOSDPoolDefaultSize > osdNamesSet.size) {
      var poolSizeWarning = "osd pool default size is higher than the number of configured OSDs nodes.\n" +
          "Total of " + osdNamesSet.size + " OSDs nodes, minimum of " + userChoices.globalOSDPoolDefaultSize +
          " needed for the pool size.\nIn the default Ceph configuration, the amount of OSD nodes should" +
          " be at least the default size of the pool.\n\n";
      warningMessages.push({title: "Default osd pool size is not correct.",
                            message: poolSizeWarning});
    }
  }

  if (!warningWithConfirmation) {
    warningWithConfirmation = new WarningWithConfirmation(
        document.getElementById("warningSection"),
        document.getElementById("confirm"));
  }
  warningWithConfirmation.set(warningMessages);
}

function appendChoice(box, fieldName, fieldValue) {
  var hbox = document.createElement('hbox');
  box.appendChild(hbox);
  var field = document.createElement('shadowlabel');
  field.setAttribute('class', 'staticdataheader');
  field.setAttribute('value', fieldName);
  hbox.appendChild(field);
  var label = document.createElement('label');
  label.setAttribute('value', fieldValue);
  hbox.appendChild(label);
}

function refresh() {
  var serviceType = getCurrentServiceType();
  if (serviceType) {
    tree = document.getElementById(serviceType + 'Tree');
    data = userChoices.getContainers(serviceType).map(function(container) {
      return container.createTreeItem();
    });
    loadTable();
  }
  return true;
}

function selectItems(serviceType, baseType) {
  var selectedItemKeys = userChoices.getContainers(serviceType, baseType).map(function(item) {
    return item.uniqueKey;
  });

  var excludeFunction = function(node) { return false; }

  if ((serviceType === 'OSD') && (baseType === 'Node')) {
    excludeFunction = function(node) {
      return node.get('childType') === 'MasterNode';
    }
  }
  
  var args = {
    cluster: current,
    baseType: baseType,
    currentnodes: selectedItemKeys,
    excludeFunction: excludeFunction
  };
  var d = new dock.cvDialog('chrome://cmgui/content/cephwizard/selectitems.xul', args);
  d.onOk = function(keys) {
    userChoices.addContainers(serviceType, baseType, keys);
    refresh();
  };
  dock.dialogs.showDialog(d);
}

function removeItems(serviceType) {
  var keys = selected(serviceType + 'List');
  userChoices.removeContainers(serviceType, keys);
  refresh();
  //After removal next row is not selected
  disableRowModifications();
}

function disableRowModifications() {
  document.getElementById('selectEditButton').setAttribute('disabled', true);
  document.getElementById('selectRemoveButton').setAttribute('disabled', true);
}

function enableRowModifications(serviceType) {
  if (fakeModalDialogActive)
    return;
  var keys = selected(serviceType + 'List');
  if (keys.length === 0) {
    return;
  }
  document.getElementById('selectEditButton').setAttribute('disabled', false);
  document.getElementById('selectRemoveButton').setAttribute('disabled', false);
}

function editItem(serviceType) {
  if (fakeModalDialogActive)
    return;
  var keys = selected(serviceType + 'List');
  if (keys.length === 0) {
    return;
  }
  var args = {
    cluster: current,
    container: userChoices.getContainer(serviceType, keys[0]).clone()
  };
  var xul_file;
  if (serviceType === 'Monitor') {
    xul_file = 'chrome://cmgui/content/cephwizard/editmonitor.xul';
  } else if (serviceType === 'OSD') {
    xul_file = 'chrome://cmgui/content/cephwizard/editosd.xul';
  } else {
    xul_file = '';
  }
  var d = new dock.cvDialog(xul_file, args);
  d.onOk = function(editedContainer) {
    userChoices.updateContainer(editedContainer);
    refresh();
  };
  dock.dialogs.showDialog(d);
}

function showConfiguration() {
  var xmlConfig = "";
  try {
    xmlConfig = userChoices.toXML();
  } catch (err) {
    println('exception ' + err + ' while generation xml file');
    dock.displayError('Error while generating XML file');
    return false;
  }

  var args = {
    title: 'Ceph configuration',
    style: 'background-color: white !important;',
    text: xmlConfig
  };
  var d = new cvDialog(dock.myVersion + "showtext.xul", args);
  dock.dialogs.showDialog(d);
}

function saveConfiguration() {
  var xmlConfig = "";
  try {
    xmlConfig = userChoices.toXML();
  } catch (err) {
    println('exception ' + err + ' while generation xml file');
    dock.displayError('Error while generating XML file');
    return false;
  }

  var file = '';
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
  fp.init(window, "Save Ceph setup as", nsIFilePicker.modeSave);
  var path = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
  path.initWithPath(dock.BASE_PATH);
  fp.displayDirectory = path;
  fp.appendFilter("Ceph setups", "*.xml");
  var res = fp.show();
  if ((res == nsIFilePicker.returnOK) || (res == nsIFilePicker.returnReplace)) {
    file = fp.file.path;
    if (!endswith(file, '.xml')) file += '.xml';
  } else {
    return;
  }
  savefile(file, xmlConfig);
}

function enableRetry() {
  document.getElementById('retry').setAttribute('hidden', false);
}

function retry() {
  deploy();
  configureButtons();
  refresh();
}

function deploy() {
  setStage(5);
  var xmlConfig = "";
  try {
    xmlConfig = userChoices.toXML();
  } catch (err) {
    println('exception ' + err + ' while generation xml file');
    dock.displayError('Error while generating XML file');
    return false;
  }
  setAttr('deployMessage', 'value', 'Deploying Ceph');

  var id = 0;
  overviewBar('progressBar', 0, 100);
  dock.workerpool.getWorker(current.actualurl).call('cmceph', 'executeCephSetup', [xmlConfig], function(code, data) {
    if(code == 0) {
      id = data.setupExecutionID;
      timer = window.setTimeout('checkStatus(' + id +');', 1000);
    } else {
      dock.jsonCallError(this, code, data);
      enableRetry();
    }
  }, current);
}

function finish() {
  close();
}

function checkStatus(id) {
  timer = 0;
  count++;
  if (count > 2000) {
    return;
  }

  dock.workerpool.getWorker(current.actualurl).call('cmceph', 'getCephSetupExecution', [id], function(code, data) {
    if(code !== 0) {
      dock.jsonCallError(this, code, data);
      enableRetry();
      return;
    }
    if (!data) {
      dock.displayError('A Ceph instance is already being installed.');
      return;
    }

    var splitResult = splitArrayWithPattern(data.datacout, /-------/);
    var steps = splitResult.before;
    var finalMessage = splitResult.after.join('\n');

    var progressPerc = steps.length * 10;
    overviewBar('progressBar', progressPerc, 100);

    if (steps.length > 0) {
      var deployStepsDisplay = new ProgressStepDisplayer('deploySteps');
      deployStepsDisplay.clearAll();
      for (var i = 0; i < steps.length - 1; i++) {
        deployStepsDisplay.add('FINISHED', steps[i]);
      }
      var lastLine = steps[steps.length - 1];
      var lastLineStatus;
      if (data.finishedAt != 0 && !data.successful) {
        if (lastLine.indexOf('ERROR') === 0) {
          finalMessage = 'The deployment has failed with the following error. Please correct the error and try again.\n\n' +
                         lastLine + '\n\n' ;
        } else {
          finalMessage = 'The deployment has failed with an unknown error. ';
        }
        finalMessage += 'See /var/log/cm-ceph-setup.log on the headnode for details.';
        lastLine = 'Failed';
        lastLineStatus = 'FAILED';
        enableRetry();
      } else if (finalMessage.length > 0) {
        lastLineStatus = 'FINISHED';
      } else {
        lastLineStatus = 'STARTED';
      }
      deployStepsDisplay.add(lastLineStatus, lastLine);
    }

    // Warn a user if not enough bootstrap Monitor nodes are up by basically
    // grepping the stript's output to figure out at what stage the process is.
    if ((steps.length > 0) && (steps[steps.length-1].indexOf('Wait for the majority of bootstrap monitors to be up') === 0)) {
      var msg = checkEnoughMonitorsUP();
      if (msg !== '') finalMessage = msg;
    }

    setAttr('deployOutput', 'value', finalMessage);
    if (data.finishedAt == 0) {
      document.getElementById('canc').setAttribute('disabled', true);
      document.getElementById('prev').setAttribute('disabled', true);
      timer = window.setTimeout('checkStatus(' + id +');', 3000);
    } else if (data.successful) {
      overviewBar('progressBar', 100, 100);
      document.getElementById('finish').setAttribute('disabled', false);
    } else { // failed
      document.getElementById('prev').setAttribute('disabled', false);
      document.getElementById('finish').setAttribute('disabled', false);
    }
  }, current);
}
