/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var current;
var myDialog;
var dock;
var container;

var help = {
  sharedJournalDevice: "A block device to be split into a number of partitions equal to the number of OSD Associations, " +
                       "with each partition being used as an OSD journal.",
  name: "Name of the OSD Association. It's included in the name of ceph-osd processes running on Ceph OSD nodes.",
  blockdev: "If present, specifies a block device to be partitioned, formatted and mounted under the Data path " +
            "specified below, to store OSD data; must be a full block device name, e.g. sdb or /dev/sdb, not a partition. " +
            "If empty, OSD data will be stored on a node's system disk.",
  dataPath: "A path under which the OSD data is stored on a node. $cluster and $id will be substituted with the " +
            "Ceph cluster name, usually 'ceph', and the id of Ceph OSD service, respectively. It is recommended " +
            "to use the default value.",
  journalPath: "A path under which the OSD journal is stored on a node. $cluster and $id will be substituted with the " +
               "Ceph cluster name, usually 'ceph', and the id of Ceph OSD service, respectively. It is recommended " +
               "to use the default value.",
  journalSize: "Size of the OSD journal, which can either be a file or a partition if 'Journal on partition' is set to 'yes'.",
  journalOnPartition: "Allocate first partition on the specified bock device to be used as the OSD journal."
};

function refresh(use) {
  screen();
}

function adjustInputChoices() {
  var isSharedJournal = (getAttr('sharedJournalDevice', 'value') !== '');
  for (var i = 0; i < container.osdAssociations.length; i++) {
    var journalSizeElt = document.getElementById('Journal size' + i);
    var journalOnPartitionElt = document.getElementById('Journal on partition' + i);

    if (isSharedJournal) {
      journalSizeElt['disabled'] = true;
      journalOnPartitionElt['disabled'] = true;
      continue;
    }
    journalSizeElt['disabled'] = false;

    var blockDevice = getAttr('Block device' + i, 'value');

    if (blockDevice == '') {
      journalOnPartitionElt['disabled'] = true;
    } else {
      journalOnPartitionElt['disabled'] = false;
    }
  }
}

function onLoad() {
  dock = getDock();
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  current = myDialog.arguments.cluster;
  container = myDialog.arguments.container;

  screen();
}

function screen() {
  setAttr('sharedJournalDeviceTip', 'tooltiptext', help.sharedJournalDevice);

  setAttr('type', 'value', container.baseType + ':');
  setAttr('name', 'value', container.getName());
  setAttr('sharedJournalDevice', 'value', container.sharedJournalDevice);

  clearOSDAssociations();
  if (container.osdAssociations.length > 0) {
    setAttr('emptyAssocBox', 'hidden', 'true');
    createOSDAssociations();
  } else {
    setAttr('emptyAssocBox', 'hidden', 'false');
  }
  adjustInputChoices();
}

function assignOSDAssociations() {
  for (var i = 0; i < container.osdAssociations.length; i++) {
    var assoc = container.osdAssociations[i];
    assoc.name = getAttr('OSD Association' + i, 'value');
    assoc.blockdev = joinPath('/dev', getAttr('Block device' + i, 'value'));
    assoc.dataPath = getAttr('Data path' + i, 'value');
    assoc.journalPath = getAttr('Journal path' + i, 'value');
    assoc.journalSize = Number(getAttr('Journal size' + i, 'value'));
    assoc.journalOnPartition = (getAttr('Journal on partition' + i, 'selectedIndex', 'notrim') === 0);
  }
}

function ok() {
  container.sharedJournalDevice = joinPath('/dev', getAttr('sharedJournalDevice', 'value'));

  assignOSDAssociations();

  var validateMessages = container.validate();
  if (validateMessages.length > 0) {
    dock.displayError(validateMessages.join('\n\n'));
    return;
  }
  close();
  if(myDialog.onOk) {
    myDialog.onOk(container);
  }
}

function cancel() {
  close();
  if(myDialog.onCancel) myDialog.onCancel();
  if(myDialog.onClose) myDialog.onClose();
}

function clearOSDAssociations() {
  var rows = document.getElementById('osdAssociations');
  while (rows.firstChild) {
    rows.removeChild(rows.firstChild);
  }
}

function createToolTip(row, tooltip) {
  var hbox = document.createElement('hbox');
  hbox.setAttribute('width', '3px');
  hbox.setAttribute('pack', 'end');
  row.appendChild(hbox);
  var img = document.createElement('image');
  img.setAttribute('src', 'chrome://cmgui/content/icons/notice.png');
  img.setAttribute('tooltiptext', tooltip);
  row.appendChild(img);
}

function createTextboxRow(rows, assocIdx, description, value, tooltip, placeholder='') {
  var row = document.createElement("row");
  row.setAttribute('align', 'center');
  rows.appendChild(row);
  var hbox = document.createElement("hbox");
  hbox.setAttribute('pack', 'end');
  var label = document.createElement("shadowlabel");
  label.setAttribute('value', description + ':');
  label.setAttribute('class', 'staticdataheader');
  hbox.appendChild(label);
  row.appendChild(hbox);
  hbox = document.createElement("hbox");
  hbox.setAttribute('pack', 'end');
  var label = document.createElement("textbox");
  label.setAttribute('id', description + assocIdx);
  label.setAttribute('oninput', "adjustInputChoices();");
  label.setAttribute('value', value);
  label.setAttribute('width', '210px');
  label.setAttribute('placeholder', placeholder);
  hbox.appendChild(label);
  row.appendChild(hbox);
  createToolTip(row, tooltip);
  return row;
}

function createYesNoRow(rows, assocIdx, description, value, tooltip) {
  var row = document.createElement("row");
  row.setAttribute('align', 'center');
  rows.appendChild(row);
  var hbox = document.createElement("hbox");
  hbox.setAttribute('pack', 'end');
  var label = document.createElement("shadowlabel");
  label.setAttribute('value', description + ':');
  label.setAttribute('class', 'staticdataheader');
  hbox.appendChild(label);
  row.appendChild(hbox);
  hbox = document.createElement("hbox");
  var menulist = document.createElement("menulist");
  hbox.appendChild(menulist);
  menulist.setAttribute('oncommand', "adjustInputChoices();");
  menulist.setAttribute('id', description + assocIdx);
  var popup = document.createElement("menupopup");
  menulist.appendChild(popup);
  ["yes", "no"].forEach(function(option) {
    var item = document.createElement('menuitem');
    item.setAttribute('label', option);
    popup.appendChild(item);
  });
  row.appendChild(hbox);
  menulist.selectedIndex = (value === true) ? 0 : 1;
  createToolTip(row, tooltip);
  return row;
}

function createButtonsRow(row, index) {
  var minus = document.createElement("button");
  minus.setAttribute('class', 'minus-button');
  minus.setAttribute('oncommand', 'removeOSDAssociation(' + index + ');');
  row.appendChild(minus);
  var plus = document.createElement("button");
  plus.setAttribute('class', 'plus-button');
  plus.setAttribute('oncommand', 'addOSDAssociation();');
  row.appendChild(plus);
}

function createOSDAssociations() {
  var rows = document.getElementById('osdAssociations');
  for (var i = 0; i < container.osdAssociations.length; i++) {
    var assoc = container.osdAssociations[i];
    var firstRow = createTextboxRow(rows, i, 'OSD Association', assoc.name, help.name);
    createButtonsRow(firstRow, i);
    createTextboxRow(rows, i, 'Block device', assoc.blockdev, help.blockdev, 'e.g. sdb or /dev/sdb');
    createTextboxRow(rows, i, 'Data path', assoc.dataPath, help.dataPath);
    createTextboxRow(rows, i, 'Journal path', assoc.journalPath, help.journalPath);
    createTextboxRow(rows, i, 'Journal size', assoc.journalSize || '', help.journalSize, '<default>');
    createYesNoRow(rows, i, 'Journal on partition', assoc.journalOnPartition, help.journalOnPartition);

    var spacer1 = document.createElement('spacer');
    spacer1.setAttribute('height', '15px');
    rows.appendChild(spacer1);
    if (i < container.osdAssociations.length - 1) {
      var hbox = document.createElement('hbox');
      hbox.setAttribute('class', 'staticdottedline');
      rows.appendChild(hbox);
      var spacer2 = document.createElement('spacer');
      spacer2.setAttribute('height', '5px');
      rows.appendChild(spacer2);
    }
  }
}

function addOSDAssociation() {
  assignOSDAssociations();
  
  var assoc = new OSDAssociation('osd' + container.osdAssociations.length);
  container.osdAssociations.push(assoc);
  refresh();
}

function removeOSDAssociation(i) {
  container.osdAssociations.splice(i, 1);
  refresh();
}
