/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

// Neeeds to be here because of using in both addcephwizard.js and editosd.js
function OSDAssociation(name) {
  this.name = name;
  this.blockdev = '';
  this.dataPath = '/var/lib/ceph/osd/$cluster-$id';
  this.journalPath = '/var/lib/ceph/osd/$cluster-$id/journal';
  this.journalSize = 0;
  this.journalOnPartition = false;
  this.sharedJournalDevice = '';
}
OSDAssociation.prototype.clone = function() {
  var newAssoc = new OSDAssociation(this.name);
  newAssoc.blockdev = this.blockdev;
  newAssoc.dataPath = this.dataPath;
  newAssoc.journalPath = this.journalPath;
  newAssoc.journalSize = this.journalSize;
  newAssoc.journalOnPartition = this.journalOnPartition;
  newAssoc.sharedJournalDevice = this.sharedJournalDevice;
  return newAssoc;
}

function addCephWizard() {
  var win = dock.windowMediator.getMostRecentWindow('addcephwizard');
  if ( win ) {
    dock.windowMediator.getMostRecentWindow('addcephwizard').focus();
    return false;
  }
  var args = new Object();
  args['cluster'] = current;
  var d = new dock.cvDialog("chrome://cmgui/content/cephwizard/addcephwizard.xul", args);
  d.onOk = function() {
    //refresh();
  }
  dock.dialogs.showDialog(d);
}

function setAttr(id, attr, value) {
  document.getElementById(id).setAttribute(attr, value);
}

function getAttr(id, attr, notrim) {
  if (notrim === undefined) {
    return document.getElementById(id)[attr].trim();
  } else {
    return document.getElementById(id)[attr];
  }
}

function setTextMessages(messages) {
  for (var fieldId in messages) {
    if (messages.hasOwnProperty(fieldId)) {
      setAttr(fieldId, 'value', messages[fieldId]);
    }
  }
}

function joinPath(prefix, sourcePath) {
  if (sourcePath === '' || sourcePath[0] === '/')
    return sourcePath;
  if (prefix[prefix.length - 1] === '/')
    prefix = prefix.substr(0, prefix.length - 1);
  return prefix + '/' + sourcePath;
}

function addTextNode(doc, node, name, value) {
  var elem = doc.createElement(name);
  var text = doc.createTextNode(value);
  elem.appendChild(text);
  node.appendChild(elem);
}

function toXML(c) {
  var doc = document.implementation.createDocument('', 'cephConfig', null);

  var networksElem = doc.createElement("networks");
  if (c.publicNetworkKey !== -1 || c.clusterNetworkKey !== -1)
    doc.documentElement.appendChild(networksElem);
  
  if (c.publicNetworkKey !== -1) {
    var publicNetworkName = current.find(c.publicNetworkKey).get('name');
    addTextNode(doc, networksElem, 'public', publicNetworkName);
  }

  if (c.clusterNetworkKey !== -1) {
    var clusterNetworkName = current.find(c.clusterNetworkKey).get('name');
    addTextNode(doc, networksElem, 'cluster', clusterNetworkName);
  }

  if (c.globalJournalSize !== 0) {
    addTextNode(doc, doc.documentElement, 'journalsize', c.globalJournalSize);
  }

  addTextNode(doc, doc.documentElement, 'osdpooldefaultsize', c.globalOSDPoolDefaultSize)

  var monitorsElem = doc.createElement("monitors");
  c.getContainers('Monitor').forEach(function(mon) {
    var monElem = doc.createElement("monitor");
    monitorsElem.appendChild(monElem);
    var type = (mon.baseType === 'Node') ? 'hostname' : 'category';
    addTextNode(doc, monElem, type, mon.getName());
    addTextNode(doc, monElem, 'monitordata', mon.dataPath);
    var bootstrapAttribute = document.createAttribute('bootstrap');
    bootstrapAttribute.textContent = mon.bootstrap;
    monElem.attributes.setNamedItem(bootstrapAttribute);
  });
  if (monitorsElem.children.length > 0) {
    doc.documentElement.appendChild(monitorsElem);
  }

  var osdsElem = doc.createElement("osds");
  c.getContainers('OSD').forEach(function(osd) {
    var osdElem = doc.createElement("osd");
    osdsElem.appendChild(osdElem);
    var type = (osd.baseType === 'Node') ? 'hostname' : 'category';
    addTextNode(doc, osdElem, type, osd.getName());

    var isSharedJournal = (osd.sharedJournalDevice !== '');
    if (isSharedJournal) {
      var sjElem = doc.createElement("sharedjournal");
      osdElem.appendChild(sjElem);
      addTextNode(doc, sjElem, 'blockdev', osd.sharedJournalDevice);
    }

    var osdAssociationsElem = doc.createElement("osdassociations");
    osd.osdAssociations.forEach(function(assoc) {
      var assocElem = doc.createElement("osdassociation");
      osdAssociationsElem.appendChild(assocElem);
      addTextNode(doc, assocElem, 'name', assoc.name);
      if (assoc.blockdev !== '') {
        addTextNode(doc, assocElem, 'blockdev', assoc.blockdev);
      }
      if (assoc.dataPath !== '') {
        addTextNode(doc, assocElem, 'osddata', assoc.dataPath);
      }
      if (assoc.journalPath !== '') {
        addTextNode(doc, assocElem, 'journaldata', assoc.journalPath);
      }
      if (assoc.journalSize > 0) {
        addTextNode(doc, assocElem, 'journalsize', assoc.journalSize);
      }
      if (assoc.journalOnPartition) {
        addTextNode(doc, assocElem, 'journalonpartition', 'yes');
      }
    });
    osdElem.appendChild(osdAssociationsElem);
  });
  if (osdsElem.children.length > 0) {
    doc.documentElement.appendChild(osdsElem);
  }

  return new XMLSerializer().serializeToString(doc);
}

function splitArrayWithPattern(a, pattern) {
  var i = 0;
  for (; i < a.length; i++) {
    if (a[i].match(pattern)) {
      break;
    }
  }
  return {
    before: a.slice(0, i),
    after:  a.slice(i+1, a.length)
  };
}

function checkEnoughMonitorsUP() {
  var monsBSTrue = userChoices.getNodeObjectsFiltered('Monitor', function(container) {
    return container.bootstrap === 'true';
  });
  var allBSTrueUP = monsBSTrue.every(function(node) {
    return node.status && node.status.status === 'UP';
  });

  var monsBSAuto = userChoices.getNodeObjectsFiltered('Monitor', function(container) {
    return container.bootstrap === 'auto';
  });
  var someBSAutoUP = monsBSAuto.some(function(node) {
    return node.status && node.status.status === 'UP';
  });

  if ((monsBSTrue.length > 0 && allBSTrueUP) || (monsBSTrue.length === 0 && someBSAutoUP)) {
    return '';
  } else {
    var msg = 'Not enough Ceph Monitor nodes are up to proceed with the setup.\n\n';
    if (monsBSTrue.length > 0) {
      msg += 'All of the following nodes needs to be up: ' +
             monsBSTrue.map(function(node) { return node.get('hostname')}).join(', ') + '\n';
    }
    if (monsBSAuto.length > 0) {
      msg += 'At least one of the following nodes needs to be up: ' +
             monsBSAuto.map(function(node) { return node.get('hostname')}).join(', ') + '\n';
    }
    return msg;
  }
}
