/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var myDialog;
var dock;
var cluster;

function onLoad() {
  dock = getDock();
  println("Certificates initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  cluster = myDialog.arguments.cluster;

  if(myDialog.arguments.certificate) cluster.requestid = -1;

  var prof = document.getElementById('profile');
  if(prof) {
    var list = cluster.find('Profile');
    for(var i = 0; i < list.length; i++) {
      var name = list[i].get('name');
      var nonuser = list[i].get('nonuser');
      if (!nonuser) {
        var item = document.createElement('menuitem');
        item.setAttribute('label', name);
        if(i == 0) item.setAttribute('selected', 'true');
        prof.firstChild.appendChild(item);
      }
    }
    prof.selectedIndex = 0;
  }

  refresh();
}

function refresh() {
  var enddate = document.getElementById('enddate');
  if(enddate) {
    var d = new Date();
    d = new Date(d.getYear() + 1901, d.getMonth(), d.getDay());
    datetext('enddate', DATE_FORMAT, d);
  }

  if(myDialog.arguments.request) {
    document.getElementById('name').value = myDialog.arguments.request.commonName;
    document.getElementById('organization').value = myDialog.arguments.request.organization;
    document.getElementById('unit').value = myDialog.arguments.request.organizationalUnit;
    document.getElementById('locality').value = myDialog.arguments.request.locality;
    document.getElementById('state').value = myDialog.arguments.request.state;
    document.getElementById('country').value = myDialog.arguments.request.country;
  } else if(myDialog.arguments.old) {
    document.getElementById('name').value = myDialog.arguments.old.commonName;
    document.getElementById('organization').value = myDialog.arguments.old.organization;
    document.getElementById('unit').value = myDialog.arguments.old.organizationalUnit;
    document.getElementById('locality').value = myDialog.arguments.old.locality;
    document.getElementById('state').value = myDialog.arguments.old.state;
    document.getElementById('country').value = myDialog.arguments.old.country;

    var d = new Date();
    d.setTime(1000 * (d.getTime() / 1000 + myDialog.arguments.old.remainingDays * 86400));
    document.getElementById('enddate').setAttribute('value', d.format(DATE_FORMAT));
    document.getElementById('enddate').setAttribute('time', Math.floor(d.getTime() / 1000));

    document.getElementById('profile').value = myDialog.arguments.old.profile;
    document.getElementById('syslogin').value = myDialog.arguments.old.sysLogin;
    document.getElementById('profile').setAttribute('label', myDialog.arguments.old.profile);

    document.getElementById('name').setAttribute('disabled', true);
    document.getElementById('organization').setAttribute('disabled', true);
    document.getElementById('unit').setAttribute('disabled', true);
    document.getElementById('locality').setAttribute('disabled', true);
    document.getElementById('state').setAttribute('disabled', true);
    document.getElementById('enddate').setAttribute('disabled', true);
    document.getElementById('enddatepicker').setAttribute('disabled', true);
    document.getElementById('country').setAttribute('disabled', true);
    document.getElementById('profile').setAttribute('disabled', true);
    document.getElementById('syslogin').setAttribute('disabled', true);
  }
}

function addCertificate(button) {
  button = isundefined(button, false);
  if((cluster.requestid > 0) || requestCertificate()) {
    if(issueCertificate()) close();
    myDialog.onOk();
  }
}

function issueCertificate(button) {
  button = isundefined(button, false);
  if(!document.getElementById('profile').selectedItem) {
    dock.displayInfo("Specify profile");
  } else {
    var d = new Date();
    var days = parseInt(document.getElementById('enddate').getAttribute('time'));
    days = Math.ceil((days - d.getTime() / 1000) / 86400);
    var profile = document.getElementById('profile').selectedItem.getAttribute('label');
    var syslog = document.getElementById('syslogin').value;
    var id = -1;
    var name = document.getElementById('name').value;
    var organ = document.getElementById('organization').value;
    var unit = document.getElementById('unit').value;
    var loca = document.getElementById('locality').value;
    var state = document.getElementById('state').value;
    var country = document.getElementById('country').value;
    if(country.length != 2) {
      dock.displayError("Country should contain the 2 letter country code");
      return false;
    }

    if(button) {
      id = myDialog.arguments.request.requestId;
    } else {
      id = cluster.requestid;
    }
    if(id > 0) {
      dock.workerpool.getWorker(cluster.actualurl).call('cmcert', 'issueCertificateRequest', [id, country, state, loca, organ, unit, name, days, profile, syslog, false], function(code, data) {
          if(code == 0) {
            if(button && data) {
              dock.displayInfo("Certificate issued");
              myDialog.onOk();
              close();
            } else if(!data) {
              dock.displayError("Certificate issue failed");
            }            
          } else {
            dock.jsonCallError(this, code, data);
          }
        }, cluster);
    } else {
      dock.displayError("No certificate request id");
    }
  }
  return false;
}

function requestCertificate(button) {
  button = isundefined(button, false);
  if(cluster.requestid > 0) {
    dock.displayInfo("Request already pending");
    return false;
  }
  var keylen = parseInt(document.getElementById('keylen').selectedItem.getAttribute('label'));
  cluster.certkey = cluster.services['cmcert'].generatePrivateKey(keylen);
  if(cluster.certkey) {
    var name = document.getElementById('name').value;
    if(blank(name)) {
      dock.displayError("Name should be set");
      return false;
    }
    var organ = document.getElementById('organization').value;
    var unit = document.getElementById('name').value;
    var loca = document.getElementById('locality').value;
    var state = document.getElementById('state').value;
    var country = document.getElementById('country').value;
    cluster.csr = cluster.services['cmcert'].generateCertificateRequest(cluster.certkey, country, state, loca, organ, unit, name);
    cluster.requestid = cluster.services['cmcert'].requestCertificate(cluster.csr, cluster.sessionid);
    if(cluster.requestid == 0) {
      dock.displayError("Unable to request certificate");
      cluster.requestid = -1;
      return false;
    }
    return true;
  } else {
    dock.displayError("Unable to create private key");
    return false;
  }
}

function saveCertificate(button) {
  button = isundefined(button, false);
  if(document.getElementById("password").value == document.getElementById("retype").value) {
    var password = document.getElementById("password").value;
    var file = document.getElementById("filename").value;
    if(dock.getDirectorySeparator() == "\\") {
      file = file.replace(/([^\\])\\([^\\])/g, "$1\\\\$2");  
    }
    if(cluster.services['cmcert'].createPFX(cluster.certkey, myDialog.arguments.certificate, password, file)) {
      dock.displayInfo("Certificate saved");
      close();
    } else {
      dock.displayInfo("Unable to save certificate");
    }
  } else {
    dock.displayInfo("Password mismatch");
  }
}

function denyCertificate(button) {
  button = isundefined(button, false);
  var args = new Object();
  args['title'] = 'Deny Certificate Request';
  args['question'] = 'Reason';
  args['default'] = 'access denied';
  args['known'] = new Array();
  var d = new dock.cvDialog("newobject.xul", args);
  d.onOk = function(reason) {
    if(current.services['cmcert'].removeCertificateRequest(myDialog.arguments.request, reason)) {
      close();
    } else {
      dock.displayInfo("Deny certificate failed");
    }
  }
  dock.dialogs.showDialog(d);
}

/* Open file browser to choose pfx file */
function browse() {
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
  fp.init(window, "Select a File", nsIFilePicker.modeSave);
  fp.appendFilters(nsIFilePicker.filterAll);
  fp.appendFilter("PFX Files","*.pfx");

  var res = fp.show();
  if((res == nsIFilePicker.returnOK) || (res == nsIFilePicker.returnReplace)) {
    var thefile = fp.fileURL;
    var fn = thefile.path;
    if(!endswith(fn, '.pfx') && (fn.indexOf('.') <= fn.indexOf(dock.getDirectorySeparator()))) fn += '.pfx';
    document.getElementById("filename").value = fn;
  }
}

function setDate(name) {
  var args = new Object();
  var t = document.getElementById(name).getAttribute('time');
  var d = new Date();
  d.setTime(t * 1000);
  args['date'] = d;
  var d = new dock.cvDialog("datepickerwindow.xul", args);
  d.onOk = function(date) {
    document.getElementById(name).setAttribute('value', date.format(DATE_FORMAT));
    document.getElementById(name).setAttribute('time', Math.floor(date.getTime() / 1000));
  }
  dock.dialogs.showDialog(d);
}
