/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  var myDialog = dock.dialogs.list[index];

  document.getElementById('openauto').checked = dock.generalSettings.autoOpenChanges;

  refresh();
}

function refresh() {
  tree = document.getElementById('tree');
  data = new Array();
  for(var j = 0; j < dock.clusterlist.length;  j++) {
    var list = dock.clusterlist[j].modified();
    for(var i = 0; i < list.length; i++) {
      data.push(createTreeItem(dock.clusterlist[j], list[i]));
    }
  }

  loadTable();
  if(dock.connectedClustersCount() > 1) {
    document.getElementById('cluster').removeAttribute('hidden');
  }
}

function createTreeItem(cluster, object) {
  var item = new Object();
  item['key'] = cluster.key + '|' + object.get('uniqueKey');
  item['cluster'] = cluster.name;
  item['type'] = object.getType(true);
  item['name'] = object.resolveName();
  return item;
}

function openauto() {
  dock.generalSettings.autoOpenChanges = document.getElementById('openauto').checked;
}

function getLists(alsoLocal) {
  var select = selected('list');
  if(select.length == 0) {
    for(var i = 0; i < data.length; i++) {
      select.push(data[i].key);
    }
  }

  var names = new Array();
  var list = new Array();
  for(var i = 0; i < select.length; i++) {
    var keys = select[i]. split('|');
    var cluster = dock.matchCluster(keys[0]);
    if(!!cluster) {
      if(alsoLocal || (keys[1] > dock.MAX_LOCAL_KEY)) {
        var obj = cluster.find(keys[1]);
        if(!!obj) {
          list.push(obj);
          names.push(obj.resolveName());
        } else {
          // TODO
        }
      }
    } else {
      // TODO
    }
  }
  return [list, names];
}

function saveall() {
  var [list, names] = getLists(true);
  if(list.length == 0) {
    dock.displayInfo('Nothing to save', window);
  } else {
    var args = new Object();
    args['question'] = 'Are you sure you want to save: ' + names.join(', ');
    var d = new dock.cvDialog("question.xul", args);
    d.parentWindow = window;
    d.onOk = function() {
      for(var i = 0; i < list.length; i++) {
        list[i].commit();
      }
    }
    dock.dialogs.showDialog(d);
  }
}

function revertall() {
  var [list, names] = getLists(false);
  if(list.length == 0) {
    dock.displayInfo('Nothing to revert', window);
  } else {
    var args = new Object();
    args['question'] = 'Are you sure you want to revert: ' + names.join(', ');
    var d = new dock.cvDialog("question.xul", args);
    d.parentWindow = window;
    d.onOk = function() {
      for(var i = 0; i < list.length; i++) {
        list[i].refresh();
      }
    }
    dock.dialogs.showDialog(d);
  }
}
