/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var ALL = 'All';

function cvCloneSettings(action, parent, subfield, work, filename) {
  this.actions = 0;
  this.attributes = 0;
  this.propkeys = 0;
  this.operations = 0;
  this.overwrite = 0;
  this.notes = '';
  this.warnings = 0;

  this.propall = 0;
  this.propsel = 0;

  this.action = action;
  this.parent = parent;
  this.subfield = subfield;
  this.work = work;
  this.destination = 0;
  this.filename = filename;
}

cvCloneSettings.prototype.save = function() {
  var data = "++++BEGIN CLONE SETTINGS++++\n";
  data += "actions = " + this.actions.toSource() + ";\n";
  data += "attributes = " + this.attributes.toSource() + ";\n";
  data += "propkeys = " + this.propkeys.toSource() + ";\n";
  data += "operations = " + this.operations.toSource() + ";\n";
  data += "overwrite = " + this.overwrite.toSource() + ";\n";
  data += "notes = '" + this.notes + "';\n";
  data += "++++END CLONE SETTINGS++++\n";
  return data;
}

cvCloneSettings.prototype.load = function(data) {
  if(!isArray(data)) data = data.split("\n");
  if(data[0] != "++++BEGIN CLONE SETTINGS++++") return false;
  var i = 1;
  while(i < data.length) {
    if(data[i] == "++++END CLONE SETTINGS++++") {
      return true;
    }
    eval("this." + data[i]);
    i++;
  }
  return false;
}

cvCloneSettings.prototype.getSubfield = function(index) {
  if(isArray(this.subfield)) {
    if(index < 0) {
      return this.subfield[this.subfield.length + index];
    } else {
      return this.subfield[index];
    }
  } else {
    return this.subfield;
  }
}

cvCloneSettings.prototype.getParent = function(index) {
  if(isArray(this.parent)) {
    if(index < 0) {
      return this.parent[this.parent.length + index];
    } else {
      return this.parent[index];
    }
  } else {
    return this.parent;
  }
}

cvCloneSettings.prototype.parentCount = function() {
  if(isArray(this.parent)) {
    return this.parent.length;
  } else if(this.parent) {
    return 1;
  }
  return 0;
}

cvCloneSettings.prototype.merge = function(add) {
  if(isArray(this.actions)) {
    this.actions = this.actions.concat(add.actions);
    this.attributes = this.attributes.concat(add.attributes);
    this.propkeys = this.propkeys.concat(add.propkeys);
    this.operations = this.operations.concat(add.operations);
    this.overwrite = this.overwrite.concat(add.overwrite);
  } else {
    this.actions = add.actions;
    this.attributes = add.attributes;
    this.propkeys = add.propkeys;
    this.operations = add.operations;
    this.overwrite = add.overwrite;
  }
}

cvCloneSettings.prototype.addAction = function(action, attributes, propkey, operations, overwrite) {
  if(!isArray(this.actions)) {
    this.actions = new Array();
    this.attributes = new Array();
    this.operations = new Array();
    this.overwrite = new Array();
    this.propkeys = new Array();
  }
  this.actions.push(action);
  this.attributes.push(attributes);
  this.propkeys.push(propkey);
  this.operations.push(operations);
  this.overwrite.push(overwrite);
  return this.actions.length;
}

cvCloneSettings.prototype.removeLastAction = function() {
  if(isArray(this.actions)) {
    this.actions.splice(0, 1);
    this.attributes.splice(0, 1);
    this.operations.splice(0, 1);
  }
}

cvCloneSettings.prototype.build = function() {
  var list = new Array();
  this.warnings = new Array();

  if(this.action == dock.CLONE_SYNC_ONE) {
    for(var i = 0; i < this.destination.length; i++) {
      var index = this.destination[i].split('|');
      var cluster = dock.getCluster(index[0]);
      var one = [cluster.find(index[1])];
      for(var j = 2; j < index.length; j++) {
        var subone = one[j -  2].subObjectFromKey(this.getSubfield(j -  2), index[j]);
        if(subone) {
          one.push(subone);
        } else {
          dock.displayError('Somebody else changed ' + one[0].resolveName() + ', restart the wizard');
          return false;
        }
      }
      var dest = one[one.length -  1];
      list.push([this.work[0].url, dest.url]);
    }
  } else if(this.action == dock.CLONE_SYNC_N) {
    for(var i = 0; i < this.destination.length; i++) {
      var index = this.destination[i].split('|');
      var cluster = dock.getCluster(index[0]);
      if(this.parentCount() == 0) {
        for(var j = 0; j < this.work.length; j++) {
          var dest = cluster.find(this.work[j].get('baseType'), this.work[j].resolveName());
          if(dest) {
            list.push([this.work[j].url, dest.url]);
          } else {
            var msg = "Unable to find " + this.work[j].getType(true).toLowerCase() + " " + this.work[j].resolveName() + " in cluster " + cluster.name;
            if(this.warnings.find(msg) < 0) this.warnings.push(msg);
          }
        }
      } else {
        var one = [cluster.find(index[1])];
        for(var j = 2; j < index.length; j++) {
          var subone = one[j -  2].subObjectFromKey(this.getSubfield(j -  2), index[j]);
          if(subone) {
            one.push(subone);
          } else {
            dock.displayError('Somebody else changed ' + one[0].resolveName() + ', restart the wizard');
            return false;
          }
        }
        var dest = one[one.length -  1];
        var sublist = dest.getFromSubfield(settings.getSubfield(- 1));
        for(var j = 0; j < this.work.length; j++) {
          var found = false;
          for(var k = 0; !found && (k < sublist.length); k++) {
            if(string_case_compare(this.work[j].resolveName(), sublist[k].resolveName())) {
              list.push([this.work[j].url, sublist[k].url]);
              found = true;
            }
          }
          if(!found) {
            var msg = "Unable to find " + this.work[j].getType(true).toLowerCase() + " " + this.work[j].resolveName() + " belonging to " + dest.resolveName() + " in cluster " + cluster.name;
            if(this.warnings.find(msg) < 0) this.warnings.push(msg);
          }
        }
      }
    }
  } else if(this.action == dock.CLONE_CREATE) {
    var overwrite = false; // TODO: boolean
    for(var i = 0; i < this.destination.length; i++) {
      var index = this.destination[i].split('|');
      var cluster = dock.getCluster(index[0]);
      if(this.parentCount() == 0) {
        for(var j = 0; j < this.work.length; j++) {
          list.push([this.work[j].url, "key://" + cluster.key]);
        }
      } else {
        var one = cluster.find(index[1]);
        for(var j = 0; j < this.work.length; j++) {
          list.push([this.work[j].url, one.url]);
        }
      }
    }
  }
  return (this.addAction(this.action, this.propsel, this.propkeys, list, false) > 0); // TODO set correct overwrite
}

cvCloneSettings.prototype.execute = function() {
  var error = new Array();
  for(var i = 0; i < this.actions.length; i++) {
    var action = this.actions[i];    
    var props;
    if((action == dock.CLONE_SYNC_ONE) || (action == dock.CLONE_SYNC_N)) {
      if(this.attributes[i].find(ALL) >= 0) {
        props = 0;
      } else {
        props = this.attributes[i];
      }
    }
    var propkeys = this.propkeys[i];
    for(var j = 0; j < this.operations[i].length; j++) {
      var src = dock.findObject(this.operations[i][j][0]);
      var dest = dock.findObject(this.operations[i][j][1]);
      if(!src) {
        var msg = 'Source object no longer defined';
        if(error.find(msg) < 0) error.push(msg);
        continue;
      }
      if(!dest) {
        if(this.actions[i] != dock.CLONE_CREATE) {
          var msg = 'Destination object no longer defined';
          if(error.find(msg) < 0) error.push(msg);
          continue;
        }
        println("Destination object not found: " + this.operations[i][j][1]);
        println("Source object:                " + this.operations[i][j][0]);
      }
      var resolveParams = new Object();
      var rp = src.getResolveParameters();
      for(var k = 0; k < rp.length; k++) {
        var name = rp[k].splice(0, 1);
        resolveParams[name] = rp[k];
      }
      var oldrack = -1;
      if(!dest.isCluster && dest.get('baseType') == 'Device') oldrack = dest.get('rack');
      if((action == dock.CLONE_SYNC_ONE) || (action == dock.CLONE_SYNC_N)) {
        if(props) {
          src.copyProps(dest, props, resolveParams, propkeys, error);
        } else {
          var allprops = new Array();
          var params = src.getClonableParameters();
          params.sort();
          for(var k = 0; k < params.length; k++) {
            allprops.push(p[0]);
          }
          src.copyProps(dest, allprops, resolveParams, propkeys, error);
        }
      } else if(action == dock.CLONE_CREATE) {
        if(src.owner) { // sub
          var old = dest.subObjectFromName(src.field, src.resolveName());
          if(old) {
            if(this.overwrite[i] || old.get('toBeRemoved')) {
              var allprops = new Array();
              var params = src.getClonableParameters();
              params.sort();
              for(var k = 0; k < params.length; k++) {
                allprops.push(p[0]);
              }
              src.copyProps(old, allprops, resolveParams, propkeys, error);
              old.set('toBeRemoved', false);
            } else {
              var msg = 'Not overwriting ' + old.resolveNameFull(true);
              if(error.find(msg) < 0) error.push(msg);
            }
          } else if(action == dock.CLONE_CREATE) {
            var clone = src.clone(dest.cluster, error);
            var tmp = new dock.cvObject(dest.cluster, clone, dest, src.field);
            if((tmp.getType() == 'FSExport') && (src.cluster != dest.cluster)) {
              var hosts = tmp.get('hosts');
              if(startswith(hosts, '!') && endswith(hosts, '!') && (hosts.length > 2)) {              
                var cco = src.cluster.find(hosts.substr(1, hosts.length - 2));
                var rco = dest.cluster.find(cco.getType(), cco.resolveName());
                if(rco) {
                  tmp.set('hosts', '!' + rco.get('uniqueKey') + '!');
                } else {
                  var msg = 'Unable to find ' + cco.getType(true).toLowerCase() + ' ' + cco.resolveName() + ' in cluster ' + dest.cluster.name;
                  if(error.find(msg) < 0) error.push(msg);
                  tmp.set('hosts', 'Not found: ' + cco.resolveName());
                }
              }
            }
            dest.addSubObject(tmp);
          } else {
            // no such target
          }
        } else { // dest == target cluster
          var old = dest.find(src.resolveName(), src.get('baseType'));
          if(old) {
            if(this.overwrite[i] || old.get('toBeRemoved')) {
              var allprops = new Array();
              var params = src.getClonableParameters();
              params.sort();
              for(var k = 0; k < params.length; k++) {
                var p = params[k].split('\n')
                allprops.push(p[0]);
              }
              src.copyProps(old, allprops, resolveParams, propkeys, error);
              old.set('toBeRemoved', false);
            } else {
              var msg = 'Not overwriting ' + old.resolveNameFull(true);
              if(error.find(msg) < 0) error.push(msg);
            }
          } else {
            var clone = src.clone(dest, error);
            dest.addObject(clone, false);
          }
        }
      }
      if(oldrack > 0 && dock.SUBTREE) updateRackSubTree(dest, oldrack);
    }
  }
  if(error.length > 0) {
    var error = error.join("\n");
    dock.displayInfo(error, window);
  }
  return true;
}

cvCloneSettings.prototype.getOverview = function() {
  // TODO:   remove true for debug only
  var clustername = true || (dock.connectedClustersCount() > 1);
  var text = new Array();
  if(this.actions.length > 1) {
    text.push('Combination of ' + this.actions.length + ' actions');
  }
  for(var i = 0; i < this.actions.length; i++) {
    if(this.actions.length > 1) {
      text.push('= = = = = = = = = = = = = = = = ' + (i + 1) + ' = = = = = = = = = = = = = = = =');
    }
    var op = 'Unknown action';
    switch(this.actions[i]) {
      case(dock.CLONE_SYNC_ONE):
      case(dock.CLONE_SYNC_N): op = 'Synchronize'; break;
      case(dock.CLONE_CREATE): op = 'Export';
    }
    if((this.attributes[i][0] == ALL) || (this.actions[i] == dock.CLONE_CREATE)) {
      text.push('Synchronize all parameters');
    } else {
      var human = new Array();
      var done = new Array();
      for(var j = 0; j < this.operations[i].length; j++) {
        var src = dock.findObject(this.operations[i][j][0]);
        if(src && (done.find(src.getType()) < 0)) {
          done.push(src.getType());
          var params = src.getClonableParameters();
          for(var k = 0; k < params.length; k++) {
            if(typeof(params[k]) == 'string') {
              var p = params[k].split('\n');
              if(this.attributes[i].find(p[0]) >= 0) {
                p[1] = p[1].toLowerCase();
                if(human.find(p[1]) < 0) {
                  human.push(p[1]);
                }
              }
            }
          }
        }
      }
      human = human.sort();
      text.push('Synchronize attributes: ' + human.join(', '));
    }
    text.push('Total operations: ' + this.operations[i].length);
    text.push('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -');
    for(var j = 0; j < this.operations[i].length; j++) {
      var src = dock.findObject(this.operations[i][j][0]);
      var dest = dock.findObject(this.operations[i][j][1]);
      if(src) {
        src = src.resolveNameFull(clustername);
      } else {
        src = 'Unresolved: ' + this.operations[i][j][0];
      }
      if(dest) {
        if(dest.isCluster) {
          dest = dest.name;
        } else {
          dest = dest.resolveNameFull(clustername);
        }
      } else {
        dest = 'Unresolved: ' + this.operations[i][j][1];
      }
      text.push(op + ' ' + src + ' to ' + dest);
    }
  }
  return text.join("\n");
}
