/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */
var wizard = new NodeCreationWizard(); 
var cloudprovider = null;
var cluster;

function onLoad() {
  dock = getDock();
  println("Cloud Provider Setup Wizard initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  current = myDialog.arguments.cluster;
  cluster = myDialog.arguments.cluster;
  wizard.common.basetype = myDialog.arguments.basetype;
  wizard.common.masterInCloud = cluster.masterOnCloud;
  cloudprovider = myDialog.arguments.cloudprovider;
  window.addEventListener('close', function (event) {
      event.preventDefault();
      wizard.common.cancel();
    },
    true
  );
  wizard.common.initDefaultConfig();
  wizard.common.go(wizard.common.getNextStage.bind(undefined, null));
}

function NodeRangeScreen(wizard) {
  WizardScreen.call(this, wizard);

  this.initDefaultConfig = function() {
    this.initNodeNames();
    var format = 'c' + cluster.base.get('slaveName') + '%0' + cluster.base.get('slaveDigits') + 'd';
    wizard.config.node_range = [ sprintf(format, 1),sprintf(format, 16) ];
    // category
    var categories = cluster.find('NodeCategory');
    wizard.config.node_category = null;
    for(var i = 0; i < categories.length; i++) {
      if(categories[i].get('name') == 'cloud-node') {
        wizard.config.node_category = categories[i].resolveName();
      }
    }
    if(!wizard.config.node_category) {
      for(var i = 0; i < categories.length; i++) {
        if(categories[i].get('uniqueKey') == cluster.base.get('defaultCategory')) {
          wizard.config.node_category = categories[i].resolveName();
        }
      }
    }
    var request = {};
    var self = this;
    //see: http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html
    //parseKMG(size)

    this.volume_types = {
      "SC1" : [ parseKMG('500 GiB'), parseKMG('16 TiB') ], 
      "ST1" : [ parseKMG('500 GiB'), parseKMG('16 TiB') ],
      "STANDARD" : [ parseKMG('500 GiB'), parseKMG('16 TiB') ], 
      "IO1" : [ parseKMG('4 GiB'), parseKMG('16 TiB') ],
      "GP2" : [ parseKMG('1 GiB'), parseKMG('16 TiB') ] };
    wizard.common.remoteValidation("cluster_extension_get_region2types", request, function(response) {
      if(response.error) {
        self.error = true;
        self.displayError(response.error.split('\n'));
        wizard.common.hide('main-wait-for-config');
      } else {
        self.initRegions(response.result.lookup, response.result.default);
        self.initCategorySelect();
        self.initRegionSelect();
        self.adjustNodeRange(wizard.config.region + '-c' + cluster.base.get('slaveName'));
        self.initCloudTypeSelect(wizard.config.region);
        self.initStorageDriverSelect(wizard.config.storage, wizard.config.volume_type);
        document.getElementById('spotPrice').value = wizard.config.spotprice;
        document.getElementById('spotPersistent').checked = wizard.config.spotpersistent;
        wizard.common.hide('main-wait-for-config');
        wizard.common.unhide('node-range-form');
      }
    });
  }
  
  this.preEnter = function() {
    //wizard.common.unhide('loadConfigButton');
  };

  this.initNodeNames = function() {
    this.node_names = {};
    var self = this;
    cluster.find("Node").filter(function(obj) {
      self.node_names[obj.resolveName()] = true;
    });
  };

  this.initCategorySelect = function() {
    var cat = document.getElementById('category');
    var categories = cluster.find('NodeCategory');
    for(var i = 0; i < categories.length; i++) {
      var item = document.createElement('menuitem');
      item.setAttribute('key', categories[i].get('uniqueKey'));
      item.setAttribute('label', categories[i].resolveName());
      cat.firstChild.appendChild(item);
      if(wizard.config.node_category == categories[i].resolveName()) {
        cat.selectedIndex = i;
      }
    }
  };


  /////////////////////////////////////////////////
  // Regions 
  /////////////////////////////////////////////////
  this.initRegions = function(regions2types, defaults) { 
    var private_clouds = cloudprovider.get('privateClouds');    
    this.regions = {};
    this.regions2types = {};
    for(var i = 0; i < private_clouds.length; i++) {
      var region_object = cluster.find(private_clouds[i].region);
      if(region_object) {
        var name = region_object.resolveName();
        this.regions[region_object.object.uniqueKey] = region_object;
        if(regions2types.hasOwnProperty(name)) {
          this.regions2types[name] = regions2types[name];
        } else {
          this.regions2types[name] = [];
        }
      }
    };
    this.regions_default_types = defaults;
    // region
    var region = cloudprovider.get('defaultRegion');
    wizard.config.region = null;
    if(region) {
      region = cluster.find(region);
      if(region) {
        if(this.regions.hasOwnProperty(region.get('uniqueKey'))) {
          wizard.config.region = region.resolveName();
        }
      }
    }
    if(!wizard.config.region) {
      for(var k in this.regions) {
        wizard.config.region = this.regions[k].resolveName();
        break;
      }
    }
    this.fallbackInstanceType(wizard.config.region);
    this.adjustNodeNames(wizard.config.region + "-cnode");
  };

  this.fallbackInstanceType = function(region) {
    var type = cloudprovider.get('defaultType');
    wizard.config.instance_type = null;
    if(type) {
      type = cluster.find(type);
      if(type) {
        if(this.regions2types[region].indexOf(type.resolveName()) != -1) {
          wizard.config.instance_type = type.resolveName();
        }
      }
    }
    if(!wizard.config.instance_type) {
      this.fallbackInstanceType(wizard.config.region);
      for(var i = 0; i < this.regions_default_types.length; i++) {
        if(this.regions2types[region].indexOf(this.regions_default_types[i]) != -1) {
          wizard.config.instance_type = this.regions_default_types[i];
          break;
        }
      }
    }
  };

  this.initRegionSelect = function() {    
    var box = document.getElementById('region');
    var popup = box.firstChild;
    while(popup.firstChild) popup.removeChild(popup.firstChild);
    function addRegion(key, name) {
      var item = document.createElement('menuitem');
      item.setAttribute('key', key);
      item.setAttribute('label', name);
      popup.appendChild(item);
    }
    for(var k in this.regions) {
      var name = this.regions[k].resolveName();
      if(wizard.config.region == name) {
        addRegion(k, name);
      }
    }
    for(var k in this.regions) {
      var name = this.regions[k].resolveName();
      if(wizard.config.region != name) {
        addRegion(k, name);
      }
    }
    box.selectedIndex = 0;
  };

  this.changeRegion = function(object) {
    this.initCloudTypeSelect(object.selectedItem.getAttribute('label'));
    wizard.config.region = object.selectedItem.getAttribute('label');
    this.adjustNodeRange(wizard.config.region + "-cnode");
    this.adjustNodeNames(wizard.config.region + "-cnode");
  }

  this.changeType = function(object) {
    wizard.config.instance_type = object.selectedItem.getAttribute('label');
  }

  ////////////////////////////////////////////////////
  // type 
  ////////////////////////////////////////////////////
  this.initCloudTypeSelect = function(region) {
    var box = document.getElementById('type');
    var popup = box.firstChild;
    var self = this;
    while(popup.firstChild) popup.removeChild(popup.firstChild);
    var lu = {};
    for(var i = 0; i < self.regions2types[region].length; i++) {
      lu[self.regions2types[region][i]] = true;
    };
    if(!lu.hasOwnProperty(wizard.config.instance_type)) {
      this.fallbackInstanceType(region);
    }
    var list = cluster.find('CloudType');
    
    function addCloudType(name, type) {
      var item = document.createElement('menuitem');
      item.setAttribute('key', type.get('uniqueKey'));
      item.setAttribute('label', name);
      box.firstChild.appendChild(item);
    }
    for(var i = 0; i < list.length; i++) {
      var name = list[i].resolveName();
      if(lu.hasOwnProperty(name) && wizard.config.instance_type == name) {
        addCloudType(name, list[i]);
      }
    }
    for(var i = 0; i < list.length; i++) {
      var name = list[i].resolveName();
      if(lu.hasOwnProperty(name)  && wizard.config.instance_type != name) {
        addCloudType(name, list[i]);
      }
    }
    box.selectedIndex = 0;
  };

  ////////////////////////////////////////////////////
  // node names 
  ////////////////////////////////////////////////////
  this.adjustNodeRange = function(prefix) {
    var max = 0;
    cluster.find("Node").filter(function(obj) {
      var name = obj.resolveName();
      if(startswith(name, prefix)) {
        var nn = nameNumber(name);
        if(nn[1] != "") {
          var number = myParseInt(nn[1]);
          if(number > max) {
            max = number;
          }
        }
      }
    });
    max++;
    var format = '%0' + cluster.base.get('slaveDigits') + 'd';
    wizard.config.node_range[0] = prefix + sprintf(format, max);
    wizard.config.node_range[1] = prefix + sprintf(format, max + 16);
    document.getElementById('firstnode').value = wizard.config.node_range[0];
    document.getElementById('lastnode').value = wizard.config.node_range[1];
  };

  this.adjustNodeNames = function(prefix) {
    var n0 = nameNumber(document.getElementById('firstnode').value);
    wizard.config.node_range[0] = prefix + n0[1];
    document.getElementById('firstnode').value = wizard.config.node_range[0]
    var n1 = nameNumber(document.getElementById('lastnode').value);
    wizard.config.node_range[1] = prefix + n1[1];
    document.getElementById('lastnode').value = wizard.config.node_range[1];
  };

  this.changeFirst = function(obj) {
    var n0 = nameNumber(document.getElementById('firstnode').value);
    if(n0[1] == "") {
      document.getElementById('firstnode').value = wizard.config.node_range[0];
    } else {
      wizard.config.node_range[0] = n0[0] + n0[1];
      var n1 = nameNumber(document.getElementById('lastnode').value);
      wizard.config.node_range[1] = n0[0] + n1[1];
      document.getElementById('lastnode').value = wizard.config.node_range[1];
    }
  };

  this.changeSecond = function(obj) {
    var n1 = nameNumber(document.getElementById('lastnode').value);
    if(n1[1] == "") {
      document.getElementById('lastnode').value = wizard.config.node_range[1];
    } else {
      wizard.config.node_range[1] = n1[0] + n1[1];
      var n0 = nameNumber(document.getElementById('firstnode').value);
      wizard.config.node_range[0] = n1[0] + n0[1];
      document.getElementById('firstnode').value = wizard.config.node_range[0];
    }
  };

  this.getNodeRange = function() {
    var ret = [];
    var n0 = nameNumber(document.getElementById('firstnode').value);
    var n1 = nameNumber(document.getElementById('lastnode').value);
    var errors = [];
    if(n0[0] != n1[0]) {
      errors.push("The name part node range should be equal");
    } else if(n0[1].length != n1[1].length) {
      errors.push("The number of digits of the node range should be equal");
    }
    var prefix = n0[0];
    var s = myParseInt(n0[1]);
    var e = myParseInt(n1[1]);
    var err = [];
    if(s > e) {
      errors.push("The last node should have a larger number than the first");
    }
    if(errors.length == 0) {
      var format = '%0' + cluster.base.get('slaveDigits') + 'd';
      var node_name_errors = [];
      for(var i =s; i <= e; i++) {
        var name = prefix + sprintf(format, i);
        if(this.node_names.hasOwnProperty(name)) {
          node_name_errors.push(name);
        }
        ret.push(name);
      } 
      if(node_name_errors.length > 0) {
        errors.push("The following nodes already exist: " + dotFormat(node_name_errors));
      }
    }
    if(errors.length > 0) {
      this.displayError(errors);
      return null;
    }
    return ret;
  };

  ////////////////////////////////////////////////////
  // StorageType
  ////////////////////////////////////////////////////
  this.setStorageType = function(object) {
    var value = object.selectedItem.getAttribute('label');
    wizard.config.storage = value;
    this.initStorageDriverSelect(wizard.config.storage, wizard.config.volume_type);
    this.checkStorageSize();
  };

  this.setVolumeType = function(object) {
    var value = object.selectedItem.getAttribute('label');
    wizard.config.volume_type = value;
    this.checkStorageSize();
  };

  this.checkStorageSize = function() {
    if(wizard.config.storage != 'EBS') {
      wizard.common.hide('storagesize-error');
      return true;
    } else {
      var value = document.getElementById('storagesize').value;
      if(regexMatch(value, '^\\s*[0-9]+(\.[0-9]*)?\s*[kmgtKMGT]?i?[Bb]?\\s*$')) {
        var mb = parseKMG(value);
        if(this.volume_types.hasOwnProperty(wizard.config.volume_type)) {
          var r = this.volume_types[wizard.config.volume_type];
          if(mb < r[0] || mb > r[1]) {
            wizard.common.unhide('storagesize-error');
            document.getElementById('storagesize-error-message').value = 'size is out of range ' + formatKMG(r[0]) + " .. " + formatKMG(r[1]);
            wizard.common.unhide('storagesize-error');
            return false;
          } else {
            wizard.common.hide('storagesize-error');
            return true;
          }
        } else {
          wizard.common.hide('storagesize-error');
          return true;
        }
        return true;
      } else {
        wizard.common.unhide('storagesize-error');
        document.getElementById('storagesize-error-message').value = 'invalid storage size format';
        return false;
      }
    }
  };
  
  this.initStorageDriverSelect = function(value, volume_type) {
    if(value == 'EBS') {
      wizard.common.unhide('volumeType');
      wizard.common.unhide('storagesize');
      var elem = document.getElementById("volumeType");
      var popup = elem.firstChild;
      while(popup.firstChild) {
        popup.removeChild(popup.firstChild);
      }
      var selected = -1;
      var i = 0;
      for(var t in this.volume_types) {
        var item = document.createElement('menuitem');
        item.setAttribute('label', t);
        item.setAttribute('key', t);
        if(volume_type == t) {
          item.setAttribute('selected', true);
          selected = i;
        }
        i++;
        popup.appendChild(item);
      }
      elem.selectedIndex = selected;
    } else {
      wizard.common.hide('volumeType');
      wizard.common.hide('storagesize');
    }
  };

  ////////////////////////////////////////////////////
  // error
  ////////////////////////////////////////////////////
  this.displayError = function(msg) {
    wizard.common.unhide("node-range-error");
    var elem = document.getElementById('node-range-error');
    while(elem.firstChild) {
      elem.removeChild(elem.firstChild);
    }
    if(!Array.isArray(msg)) {
      msg = [ msg ];
    }
    for(var i = 0; i < msg.length; i++) {
      var hbox = document.createElement('hbox');
      var image = document.createElement('image');
      image.setAttribute('src', "chrome://cmgui/content/icons/alert.png");
      hbox.appendChild(image);
      var label = document.createElement('label');
      label.setAttribute('pack', 'end');
      label.setAttribute('value', msg[i]);
      label.setAttribute('flex', '1');
      label.setAttribute('class', 'error');
      hbox.appendChild(label);
      elem.appendChild(hbox);
    }
  };

  /////////////////////////////////////////////////////
  // navigation
  /////////////////////////////////////////////////////
  this.showSpotPrice = function(object) {
    wizard.common.unhide('spotprice-box');
  };

  this.hideSpotPrice = function(object) {
    wizard.common.hide('spotprice-box');
  };

  this.enter = function() {
    println('enter wizard.main.enter');
    var self = this;
    wizard.common.hide("node-range-error");
    wizard.config.current_stage = wizard.currentStage.name;
    document.getElementById('firstnode').value = wizard.config.node_range[0];
    document.getElementById('lastnode').value =  wizard.config.node_range[1];
  };

  this.leave = function(automatic) {
    //wizard.common.hide('loadConfigButton');
  };

  this.forward = function(automatic, followup) {
    var node_names = this.getNodeRange();
    var err = [];
    var spotPrice = document.getElementById('spotPrice').value;
    if(spotPrice == 'N/A') {
      spotPrice = 0;
    } else if(!isNumeric(spotPrice)) {
      err.push("Spot price needs to be a positive number or N/A for on demand instances");
      this.error = true;
    }
    if(this.error) {
      this.displayError(err);
      return false;
    }
    wizard.config.spotprice = document.getElementById('spotPrice').value;
    wizard.config.spotpersistent = document.getElementById('spotPersistent').checked;
    wizard.config.node_range[0] = document.getElementById('firstnode').value;
    wizard.config.node_range[1] = document.getElementById('lastnode').value;
    var format = '%0' + cluster.base.get('slaveDigits') + 'd';
    wizard.config.storage = document.getElementById('storage').selectedItem.getAttribute('label');
    if(wizard.config.storage == 'EBS') {
      if(this.checkStorageSize()) {
        wizard.config.storagesize = document.getElementById('storagesize').value;
      } else {
        this.error = true;
      }
    }
    wizard.config.region = document.getElementById('region').selectedItem.getAttribute('label');
    wizard.config.nodes = [];
    if(node_names) {
      for(var i = 0; i < node_names.length; i++) {
        var name = node_names[i];
        wizard.config.nodes.push({
          hostname : name,
          category : wizard.config.node_category,
          type : wizard.config.instance_type,
          storage : wizard.config.storage,
          storagesize : wizard.config.storagesize,
          region : wizard.config.region,
          spotprice : wizard.config.spotprice,
          spotpersistent : wizard.config.spotpersistent,
          volume_type : wizard.config.volume_type,
          interfaces : {}
        });
      } 
    } else {
      this.error = true;
    }
    return true;
  };
}

////////////////////////////////////////////////////////////////////////////
// network 
////////////////////////////////////////////////////////////////////////////
function NetworkInterfacesScreen(wizard) {
  WizardScreen.call(this, wizard);

  this.initDefaultConfig = function() {
  };

  this.getNetworkTypes = function() {
    if(wizard.common.masterInCloud) {
      return ['eth0' ];
    } else {
      return ['eth0', 'tun0'];
    }
  };

  this.generateIp = function(network, index, offset, dhcp) {
    dhcp = isundefined(dhcp, false);
    if(dhcp || (network.get('type') == 'CLOUD')) return '0.0.0.0';
    if(wizard.common.masterInCloud) return '0.0.0.0';
    var ip = network.get('baseAddress');
    if(ip != '0.0.0.0') {
      ip = ip.split('.');
      ip[0] = myParseInt(ip[0]);
      ip[1] = myParseInt(ip[1]);
      ip[2] = myParseInt(ip[2]);
      ip[3] = myParseInt(ip[3]) + index;
      if(ip[3] > 254) {
        ip[2] = ip[2] + Math.floor(ip[3] / 254);
        ip[3] = (ip[3] - 1) % 254 + 1;
      }
      if(isArray(offset)) {
        var carry = 0;
        for(var i = 3; i >= 0; i--) {
          var sum = ip[i] + offset[i] + carry;
          if(sum >= 256) {
            ip[i] = sum - 256;
            carry = 1;
          } else {
            ip[i] = sum;
            carry = 0;
          }
        }
      }
      return ip.join('.');
    }
    return ip;
  }

  this.preEnter = function() {
    //wizard.common.unhide('loadConfigButton');
  };

  this.displayError = function(msg) {
    wizard.common.unhide("network-interfaces-error");
    document.getElementById('network-interfaces-error-msg').value = msg;
  };

  this.initNetworkList = function(element, selected, networks) {
    var popup = document.getElementById(element).firstChild;
    while(popup.firstChild) {
      popup.removeChild(popup.firstChild);
    }
    for(var i = 0; i < networks.length; i++) {
      var item = document.createElement('menuitem');
      var name = networks[i].get('name');
      item.setAttribute('key', networks[i].get('uniqueKey'));
      item.setAttribute('label', networks[i].get('name'));
      popup.appendChild(item);
      if(name == selected) {
        popup.parentElement.selectedIndex = i;
      }
    }
  };

  this.getCloudNodesForRegion = function(region_key) {
    return cluster.find("Node").filter(function(obj) {
        return ( obj.get('childType') == 'CloudNode' && 
                 obj.object.hasOwnProperty('cloudSettings') && 
                 obj.object.cloudSettings.region== region_key );
    });
  }

  this.getCloudDirectorsForRegion = function(region_key) {
    var cloud_directors = cluster.find('Node').filter(function(obj) {
      var roles = obj.getRolesByFilter(function(role) { 
        return role.childType == "CloudDirectorRole";
      });
      return ( 
        obj.get('childType') == 'CloudNode' && 
        roles.length > 0 &&
        obj.object.hasOwnProperty('cloudSettings') && 
        obj.object.cloudSettings.region== region_key );
    });
    return cloud_directors;
  };

  this.getTunnelNetworksForNodes = function(nodes) {
    var tunnel_networks = [];
    for(var i = 0; i < nodes.length; i++) {
      var interfaces = nodes[i].get('networks');
      for(var j = 0; j < interfaces.length; j++) {
        if(interfaces[j].network) {
          var network = cluster.find(interfaces[j].network);
          if(network && network.get('type') == 'TUNNEL') {
            tunnel_networks.push(network);
          }
        }
      }
    }
    return tunnel_networks;
  };


  this.generateIpAddresses = function(region_key=null) {
    if(!region_key) {
      var region = cluster.find('CloudRegion', wizard.config.region);
      region_key = region.get('uniqueKey');
    }
    var types = this.getNetworkTypes();
    for(var i = 0; i < types.length; i++) {
      var net = document.getElementById('network' + types[i]).selectedItem;
      if(net && (net.getAttribute('key') != '0')) {
        net = parseInt(net.getAttribute('key'));
        var network = cluster.find(net);
        wizard.config.network_interfaces[types[i]] = {
          network : network.resolveName()
        };
        for(var j = 0; j < wizard.config.nodes.length; j++) {
          var nn = nameNumber(wizard.config.nodes[j].hostname);
          var index;
          if(nn[1] == '') index = j+1;
          else index = myParseInt(nn[1]);
          var ip = this.generateIp(network, index, null);
          if(!wizard.config.nodes[j].interfaces.hasOwnProperty(types[i])) {
            wizard.config.nodes[j].interfaces[types[i]] = {
              ip : ip,
              network : network.resolveName(),
            };
          }
        }
      }
    }
  };

  this.customizeIpAddresses = function() {
    for(var j = 0; j < wizard.config.nodes.length; j++) {
      var node = wizard.config.nodes[j];
      for(var iface in node.interfaces) {
        var elem = document.getElementById(node.hostname + "#" + iface + "#0");
        if(elem) {
          var ip = [];
          for(var i = 0; i < 4; i++) {
            var elem = document.getElementById(node.hostname + "#" + iface + "#" + i);
            ip.push(elem.value.trim());
          }
          wizard.config.nodes[j].interfaces[iface].ip = ip.join(".");
        }
      }
    };
  };

  this.checkIpConflicts = function(region_key=null) {
    if(!region_key) {
      var region = cluster.find('CloudRegion', wizard.config.region);
      region_key = region.get('uniqueKey');
    }
    var ip2node_interface = {};
    var nodes = this.getCloudNodesForRegion(region_key);
    for(var i = 0; i < nodes.length; i++) {
      for(var j = 0; j < nodes[i].object.networks.length; j++) {
        if(nodes[i].object.networks[j].childType == 'NetworkTunnelInterface') {
          var ip = nodes[i].object.networks[j].ip;
          ip2node_interface[ip] = { 
            ip : ip,
            hostname: nodes[i].resolveName(),
            interf : nodes[i].object.networks[j].name,
          };
        }
      }
    }
    var new_ips = {};
    for(var j = 0; j < wizard.config.nodes.length; j++) {
      var node = wizard.config.nodes[j];
      for(var iface in node.interfaces) {
        var ip = node.interfaces[iface].ip;
        if(ip != '0.0.0.0') {
          if(!new_ips.hasOwnProperty(ip)) {
            new_ips[ip] = [];
          }
          new_ips[ip].push( {
            ip : ip,
            hostname : node.hostname, 
            interf : iface,
          });
        }
      }
    }
    var conflicts = {};
    for(var j = 0; j < wizard.config.nodes.length; j++) {
      var node = wizard.config.nodes[j];
      for(var iface in node.interfaces) {
        var ip = node.interfaces[iface].ip;
        var ifconf = [];
        if(ip != '0.0.0.0') {
          if(ip2node_interface.hasOwnProperty(ip)) {
            ifconf.push(ip2node_interface[ip]);
          }
          for(var i = 0; i < new_ips[ip].length; i++) {
            if(new_ips[ip][i].hostname != node.hostname) {
              ifconf.push(new_ips[ip][i]);
            }
          }
          if(ifconf.length > 0) {
            conflicts[node.hostname + '#' + iface] = ifconf;
          }
        }
      }
    }
    return conflicts;
  };


  this.displayIpTextboxes = function(conflicts) {
    var self = this;
    var region = cluster.find('CloudRegion', wizard.config.region);
    var region_key = region.get('uniqueKey');

    function onCommandFunction(event) {
      self.customizeIpAddresses();
      self.displayConflicts(self.checkIpConflicts(region_key));
    };

    function createIpInput(ip, id) {
      ip = ip.split('.');
      var box = document.createElement('hbox');
      for(var i = 0; i < 4; i++) {
        var textbox = document.createElement('textbox');
        textbox.setAttribute('value', ip[i]);
        textbox.setAttribute('width', '30px');
        textbox.setAttribute('id', id + '#' + i);
        textbox.addEventListener("change", onCommandFunction, false);
        box.appendChild(textbox);
      }
      return box;
    }
    var rows = document.getElementById('network-interfaces-ip-conflicts-rows');
    while (rows.childNodes.length > 1) {
      rows.removeChild(rows.lastChild);
    }
    for(var j = 0; j < wizard.config.nodes.length; j++) {
      var node = wizard.config.nodes[j];
      for(var iface in node.interfaces) {
        var ip = node.interfaces[iface].ip;
        if(ip != '0.0.0.0') {
          var row = document.createElement('row');
          row.setAttribute('align', 'center');
          rows.appendChild(row);
          var label = document.createElement('label');
          label.setAttribute('value', node.hostname);
          row.appendChild(label);
          var label = document.createElement('label');
          label.setAttribute('value', iface);
          row.appendChild(label);
          var box = createIpInput(ip, node.hostname + "#" + iface);
          row.appendChild(box);
          var box = document.createElement('vbox');
          box.setAttribute('id', 'conflict-' + node.hostname + '#' + iface);
          row.appendChild(box);
          var row = document.createElement('row');
          var spacer = document.createElement('spacer');
          spacer.setAttribute('height', '5px');
          row.appendChild(spacer);
          rows.appendChild(row);
        }
      }
    }
    wizard.common.unhide('network-interfaces-ips');
  };

  this.displayConflicts = function(conflicts) {
    if(Object.keys(conflicts).length > 0) {
      wizard.common.unhide('network-interfaces-ips-conflicts');
    } else {
      wizard.common.hide('network-interfaces-ips-conflicts');
    }
    for(var j = 0; j < wizard.config.nodes.length; j++) {
      var node = wizard.config.nodes[j];
      for(var iface in node.interfaces) {
        var ip = node.interfaces[iface].ip;
        var key = node.hostname + '#' + iface;
        var elem = document.getElementById('conflict-' + key);
        if(elem) {        
          while (elem.firstChild) {
            elem.removeChild(elem.firstChild);
          }
          if(conflicts.hasOwnProperty(key)) {
            for(var k = 0; k < conflicts[key].length; k++) {
              var conf = conflicts[key][k];
              var label = document.createElement('label');
              label.setAttribute('value', 'conflict with ' + conf.hostname + ": " + conf.interf);
              label.setAttribute('class', 'error');
              elem.appendChild(label);
            }
          }
        }
      }
    }
  };

  this.enter = function() {
    println('enter wizard.network_interface.enter');
    var region = cluster.find('CloudRegion', wizard.config.region);
    var region_key = region.get('uniqueKey');
    var privateClouds = cloudprovider.get("privateClouds").filter(function(obj) {
      return (obj.region == region_key);
    });
    var networks = [];
    for(var i = 0; i < privateClouds.length; i++) {
      var default_network_name = "";
      for(var j = 0; j < privateClouds[i].subnets.length; j++) {
        var network = cluster.find(privateClouds[i].subnets[j]);
        networks.push(network);
        var name = network.get('name');
        if(name.endsWith('private')) {
          default_network_name = name;
        }
      }
    }
    this.initNetworkList('networketh0', default_network_name, networks);
    var tunnel_networks = this.getTunnelNetworksForNodes(this.getCloudDirectorsForRegion(region_key));
    if(tunnel_networks.length > 0) {
      wizard.common.unhide("box-tun0");
      this.initNetworkList('networktun0', 
                           tunnel_networks[0].resolveName(), 
                           tunnel_networks);
    } else {
      wizard.common.hide("box-tun0");
    }
    this.generateIpAddresses(region_key);
    var conflicts = this.checkIpConflicts(region_key);
    if(Object.keys(conflicts).length > 0) {
      this.displayIpTextboxes(conflicts);
      this.displayConflicts(conflicts);
    }
  };

  this.leave = function(automatic) {
    //wizard.common.hide('loadConfigButton');
  };
  
  this.forward = function(automatic, followup) {
    var types = this.getNetworkTypes();
    var n = 0;
    for(var j = 0; j < types.length; j++) {
      var netw = document.getElementById('network' + types[j]).selectedItem;
      if(netw && (netw.getAttribute('key') != '0')) {
        n++;
      }
    }
    if(n == 0) {
      this.displayError("Specify at least one interface");
      this.error = true;
      return;
    }
    this.customizeIpAddresses();
    var conflicts = this.checkIpConflicts();
    if(Object.keys(conflicts).length > 0) {
      this.error = true;
    }
  };
}

function SummaryWizardScreen(wizard) {
  WizardScreen.call(this, wizard);

  this.forward = function() {
  };

  this.leave = function() {
    println('enter wizard.summary.leave');
    wizard.common.unhide('nextButton');
    wizard.common.hide('okButton');

  };

  this.initTable = function() {
    var data = wizard.config.nodes;
    var tree = document.getElementById('tree');
    var current_col = document.getElementById('hostname');
    var tree_columns = document.getElementById('tree-columns');
    for(var j = 0; j < data.length; j++) {
      for(var nif in data[j].interfaces) {
        if(data[j].interfaces[nif].ip == '0.0.0.0') {
          data[j][nif] = 'DHCP';
        } else {
          data[j][nif] = data[j].interfaces[nif].ip;
        }
      }
    }
    while (tree_columns.firstChild) {
      tree_columns.removeChild(tree_columns.firstChild);
    }
    
    function addCol(id, label) {
      var new_node = document.createElement('treecol');
      new_node.setAttribute('id', id);
      new_node.setAttribute('flex', '1');
      new_node.setAttribute('label', label);
      new_node.setAttribute('class', 'generictreecol');
      new_node.setAttribute('sortActive', false);
      tree_columns.appendChild(new_node);
      var new_node = document.createElement('splitter');
      new_node.setAttribute('class', 'tree-splitter');
      tree_columns.appendChild(new_node);
    }
    addCol('hostname', 'hostname');
    for(var ni in wizard.config.network_interfaces) {
      var ni_obj = wizard.config.network_interfaces[ni];
      addCol(ni, ni + "(" + ni_obj.network + ")");
    }
    var summary_elems = [
      "region",
      "instance_type",
      "storage",
    ];
    for(var i = 0; i < summary_elems.length; i++) {
      var name = 'summary-' + summary_elems[i];
      var value = wizard.config[summary_elems[i]];
      document.getElementById(name).textContent = value;
    }
    loadTable(tree, data);
  };

  this.enter = function() {
    println('enter wizard.summary.enter');
    wizard.config.current_stage = wizard.currentStage.name;
    this.initTable();
    wizard.common.hide('nextButton');
    wizard.common.unhide('okButton');
  };

  this.createNodes = function(nodes) {
    function addInterface(node, value, iface, network, additionalName) {
      var basetype = node.get('childType');
      if(regexMatch(value, dock.REGEX_IPv4)) {
        var intf;
        var provisioning = -1;
        if(regexMatch(iface, '^(ipmi|ilo)[0-9]+$')) {
          intf = new dock.cvNetworkBmcInterface();
          node.set('powerControl', iface, true, -1);
        } else if(regexMatch(iface, dock.REGEX_IF_ALIAS)) {
          intf = new dock.cvNetworkAliasInterface();
        } else if(regexMatch(iface, dock.REGEX_IF_VLAN)) {
          intf = new dock.cvNetworkVLANInterface();
        } else if(regexMatch(iface, '^map0+$')) {
          intf = new dock.cvNetworkNetMapInterface();
        } else if(regexMatch(iface, '^br[0-9]+$')) {
          intf = new dock.cvNetworkBridgeInterface();
        } else if(regexMatch(iface, '^tun[0-9]+$')) {
          intf = new dock.cvNetworkTunnelInterface();
        } else if(regexMatch(iface, dock.REGEX_IF_PHYSICAL)) {
          intf = new dock.cvNetworkPhysicalInterface();
          if(iface == 'BOOTIF') {
            provisioning = 1;
          } else {
            provisioning = 0;
          }
        } else {
          println("If you see this, the primary regex is wrong!");
          return false;
        }
        intf = new dock.cvObject(cluster, intf);
        intf.set('name', iface);
        if(value == '0.0.0.0') intf.set('dhcp', true);
        intf.set('ip', value);
        intf.set('network', network);
        if(additionalName.length > 0) {
          var names = new Array();
          names.push(node.get('hostname') + additionalName);
          intf.set('additionalHostnames', names);
        }
        var list = node.get('networks');
        list.push(intf.object);
        node.set('networks', list, true, -1);

        var oldp = node.get('provisioningNetwork');
        if(((provisioning == 0) && (oldp == 0)) || (provisioning > 0)) {
          node.set('provisioningNetwork', intf.get('uniqueKey'), true, -1);
        }
        if(basetype == 'CloudNode') {
          if(intf.object.childType == 'NetworkTunnelInterface') {
            node.set('managementNetwork', network);
          }
        }
        return true;
      } else {
        dock.displayError(value + ' is not a valid ip');
      }
      return false;
    }

    function setCloudType(node, value, node_obj) {
      var type = 0;
      if(value.indexOf('(') <= 0) {
        type = cluster.find('EC2Type', value);
        if(!type) {
          dock.displayError('Cloud type not found ' + value);
          return false;
        }
        if(isObject(type)) {
          type = type.get('uniqueKey');
        } else {
          dock.displayError('Cloud type not found ' + value);
          return false;
        }
      }
      var cloudsettings = node.get('cloudSettings');
      if(cloudsettings) {
        if(cloudsettings.childType == 'EC2Settings') {
          cloudsettings.provider = cloudprovider.get('uniqueKey');
          cloudsettings.type = type;
          // KDR: do here ?
          cloudsettings.spotPrice = node_obj.spotprice;
          cloudsettings.spotPersistent = node_obj.spotpersistent;
          return true;
        }
      }
      return false;
    }

    function setCloudRegion(node, value) {
      var region = 0;
      if(value.indexOf('(') <= 0) {
        region = cluster.find('CloudRegion', value);
        if(!region) {
          dock.displayError('Cloud region not found ' + value);
          return false;
        }
        region = region.get('uniqueKey');
      }
      var cloudsettings = node.get('cloudSettings');
      if(cloudsettings) {
        if(cloudsettings.childType == 'EC2Settings') {
          cloudsettings.provider = cloudprovider.get('uniqueKey');
          cloudsettings.region = region;
          return true;
        }
      }
      return false;
    }

    function setCloudStorage(node, value, size="", volumeType="") {
      var s;
      if(value == "EBS") {
        if(regexMatch(size, '^\\s*[0-9]+(\.[0-9]*)?\s*[kmgtKMGT]?i?[Bb]?\\s*$')) {
          s = new dock.cvEC2EBSStorage();
          s.name = 'ebs';
          s.drive = 'sdb';
          s.size = parseKMG(size);
          s.volumeType = volumeType;
        } else {
          dock.displayError('Invalid cloud storage size: "' + size + '"');
          return false;
        }
      } else if(value.toLowerCase() == 'ephemeral') {
        s = new dock.cvEC2EphemeralStorage();
        s.name = 'Ephemeral0';
        s.drive = 'sdb';
      } else {
        dock.displayError('Invalid cloud storage: "' + value + '"');
        return false;
      }
      var cloudsettings = node.get('cloudSettings');
      if(cloudsettings) {
        if(cloudsettings.childType == 'EC2Settings') {
          s = new dock.cvObject(cluster, s);
          s.set('toBeRemoved', false);
          s.set('modified', true);
          var storage = cloudsettings.storage;
          storage.push(s.object);
          cloudsettings.storage = storage;
          cloudsettings.modified = true;
          return true;
        }
      }
      return false;
    }

    function checkManagementNetwork(node) {
      var setman = 0;
      var mannet = node.getManagementNetwork();
      var list = node.get('networks');
      for(var i = 0; i < list.length; i++) {
        var netw = list[i].network;
        if(netw) {
          if(netw == mannet) {
            setman = 0;
            break;
          } else {
            var network = node.cluster.find(netw);
            if(network) {
              var ntype = network.get('type');
              if((ntype == 'INTERNAL') && (list[i].childType != "NetworkAliasInterface") && (list[i].childType != "NetworkBmcInterface")) {
                setman = netw;
              } else if((ntype == 'EXTERNAL') && (setman == 0)) {
                setman = netw;
              }
            }
          }
        }
      }
      if(setman && (setman != mannet)) {
        node.set('managementNetwork', setman);
      }
    }

    for(var i = 0; i < nodes.length; i++) {
      var obj = nodes[i];
      var unboxed_node = new dock.cvCloudNode();
      var node = cluster.addObject(unboxed_node, false, false);
      var category = cluster.find('Category', obj.category); 
      node.set('partition', cluster.base.get('uniqueKey'), true, -1);
      node.set('category', parseInt(category.get('uniqueKey')), true, -1);
      node.set('powerControl', 'cloud');
      node.set('hostname', obj.hostname, true, 1);
      var good = true;
      for(var ni in  obj.interfaces) {
        var network = cluster.find('Network', obj.interfaces[ni].network);
        if(!network) {
            dock.displayError('Network ' + obj.interfaces[ni].network + ' not defined');
            return false;
          }
        var key = network.get('uniqueKey');
        good = addInterface(node, obj.interfaces[ni].ip, ni, key, '');
        if(!good) break;
      }
      if(good) good = setCloudType(node, obj.type, obj);
      if(good) good = setCloudRegion(node, obj.region);
      if(good) good = setCloudStorage(node, obj.storage, obj.storagesize, obj.volume_type);
      checkManagementNetwork(node);
      if(good) {
        cluster.addOneObject(node);
      } else {
        node.set('toBeRemoved', true);
        cluster.removeObject(node);
        return false;
      }
    }
    return true;
  };

  this.ok = function() {
    var create = [];
    var create_nodes = [];
    var old = [];
    var do_add = true;
    var self = this;
    for(var i = 0; i < wizard.config.nodes.length; i++) {
      var hostname = wizard.config.nodes[i].hostname;
      var node = cluster.find('Device', hostname);
      if(node) {
        old.push(hostname);
      } else {
        create.push(hostname);
        create_nodes.push(wizard.config.nodes[i]);
      }
    }
    if(create.length == 0) {
      dock.displayError('Nothing to do, all selected nodes already exist');
      close();
    } else {
      self.createNodes(create_nodes);
      close();
      myDialog.onOk();
    }
  };
}

function NodeCreationWizard() {
  Wizard.call(this);
  this.main = new NodeRangeScreen(this);
  this.network_interfaces = new NetworkInterfacesScreen(this);
  this.summary = new SummaryWizardScreen(this);
  this.deck = this.createDeck( [ { 'name' : 'main'}, 
                                 { 'name' : 'network_interfaces'}, 
                                 { 'name' : 'summary'} ] );
  this.currentStage = null;
  this.previousStage = null;
  this.common.loadConfiguration = function() {
    var win = dock.windowMediator.getMostRecentWindow('detachedopenstackloadconfig');
    if ( win ) {
      dock.windowMediator.getMostRecentWindow('detachedopenstackloadconfig').focus();
      return false;
    }
    var args = new Object();
    args['title'] = 'Load Node Configuration';
    args['style'] = 'background-color: white !important;';
    var d = new dock.cvDialog("chrome://cmgui/content/openstack/install/loadconfig.xul", args);
    d.onOk = function (config, fileName){
      wizard.config = config;
      wizard.common.resetErrors();
      wizard.common.currentDeck().leave(false);
      wizard.common.currentDeck().forward(false);
      wizard.common.go(wizard.common.getStage.bind(undefined, wizard.config.current_stage));
    };
    dock.dialogs.showDialog(d);
  }
}



