/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */
var wizard = new ClusterExtensionWizard(); 

function onLoad() {
  dock = getDock();
  println("Cloud Provider Setup Wizard initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  current = myDialog.arguments.provider;
  window.addEventListener('close', function (event) {
      event.preventDefault();
      wizard.common.cancel();
    },
    true
  );
  wizard.common.initDefaultConfig();
  wizard.common.go(wizard.common.getNextStage.bind(undefined, null));
}

function MainWizardScreen(wizard) {
  WizardScreen.call(this, wizard);

  this.configIsLoaded = false;

  this.preEnter = function() {
    wizard.common.unhide('loadConfigButton');
  };

  this.enter = function() {
    println('enter wizard.main.enter');
  };

  this.leave = function(automatic) {
    wizard.common.hide('loadConfigButton');
  };

  this.initImageType = function(type, value) {
    if(value) {
      wizard['init_' + type] = [ value ];
    } else {
      wizard['init_' + type] = [];
    }
    for(var i = 0; i <  wizard.config.modules.clusterextension[type].length; i++) {
      wizard['init_' + type].push(wizard.config.modules.clusterextension[type][i]);
    }
    wizard.config.modules.clusterextension[type] = wizard['init_' + type];
  };

  this.getConfig = function(response, followup) {
    var self = this;

    var keyMapping = {
      'accessKeyId' :       [ "aws_access_key", "access_key" ],
      'accessKeySecret' :   [ "aws_secret_access_key", "secret_key"],
      'accountId' :         [ "aws_account_id", "account_id"],
      'username' :          [ "username", "username"],
      'name':               [ "provider_name","provider_name"] 
    }
    wizard.common.hide('main-wait-for-config');
    wizard.common.unhide('credential-wait-for-validation');
    if(response.error) {
      self.error = true;
    } else {
      wizard.config.modules = response.result.modules; 
      var request = { 
        "region" : wizard.config.modules.clusterextension.advanced.verify_creds_region
      };
      for(var i in keyMapping) {
        request[keyMapping[i][0]] = current.get(i);
        wizard.config.modules.clusterextension[keyMapping[i][1]] = request[keyMapping[i][0]];
      }
      var type = current.cluster.find(current.object.defaultDirectorType);
      if(type) {
        self.initImageType('default_cloud_director_type',  type.get('name'));
      } else {
        self.initImageType('default_cloud_director_type',  null);
      }
      var type = current.cluster.find(current.object.defaultType);
      if(type) {
        self.initImageType('default_cloud_node_type',  type.get('name'));
      } else {
        self.initImageType('default_cloud_node_type',  null);
      }
      var region = current.cluster.find(current.object.defaultRegion);
      if(region) {
        var default_region = region.get('name');
        printall("default_region = " + default_region);
        wizard.config.modules.clusterextension.default_region = default_region;
      }
      wizard.common.remoteValidation("cluster_extension_verify_credentials", request, function(response2) {
        var error_message = document.getElementById('credentials-message');
        if(response2.error) {
          self.error = true;
          wizard.common.unhide('credentials-validation-error');
          error_message.value = response2.error;
        } else {
          if(!response2.result.verified) {
            self.error = true;
            wizard.common.unhide('credentials-validation-error');
            error_message.value = response2.result.msg;
          } else {
            wizard.common.hide('credentials-validation-error');
          }
          wizard.common.enable("nextButton");
          wizard.common.hide('credential-wait-for-validation');
          self.configIsLoaded = true;
          followup();
        }
      });
    }
  };

  this.asyncForward = function(automatic, followup) {
    if(this.configIsLoaded) {
      followup();
    } else {
      var self = this;
      var request = { "tool" : "cm-cluster-extension" };
      //var existing_providers = current.find('CloudProvider');
      wizard.common.disable("nextButton");
      wizard.common.unhide('main-wait-for-config');
      wizard.common.hide('credential-wait-for-validation');
      wizard.common.remoteValidation("cm_setup_get_tool_config", request, function(response) {
        if(response.error) {
          document.getElementById('config-error-message').value = response.error;
          wizard.common.hide('main-wait-for-config');
          wizard.common.unhide('config-error');
          self.error = true;
        } else {
          var request2 = {};
          wizard.common.remoteValidation("cluster_extension_get_region2types", request2, function(response2) {
            if(response2.error) {
              document.getElementById('config-error-message').value = response2.error;
              wizard.common.hide('main-wait-for-config');
              wizard.common.unhide('config-error');
              self.error = true;
            } else {
              wizard._region2instance_types = response2.result.lookup;
              self.getConfig(response, followup);
            }
          });
        }
      });
    } 
  };
}

function ClusterExtensionWizard() {
  Wizard.call(this);
  this.dryMode = false;
  this.forceDryMode = false;
  this.advancedMode = false;
  this.configuration_name = "VPC Configuration"; 
  this.main = new MainWizardScreen(this);
  this.regions = new RegionsWizardScreen(this);
  this.summary = new SummaryWizardScreen(this);
  this.deploy = new DeployWizardScreen(this);
  this.deploy.cm_setup_tool_script = 'cm-cluster-extension';
  this.deploy.cm_setup_tool_name   = 'Cloud Extension';
  this.deck = this.createDeck( [ { 'name' : 'main'}, 
                                 { 'name' : 'regions'}, 
                                 { 'name' : 'summary'}, 
                                 { 'name' : 'deploy'} ] );
  this.currentStage = null;
  this.previousStage = null;
}
