/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */
var wizard = new ClusterExtensionWizard(); 

function onLoad() {
  dock = getDock();
  println("Cloud Provider Setup Wizard initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  current = myDialog.arguments.cluster;

  window.addEventListener('close', function (event) {
      event.preventDefault();
      wizard.common.cancel();
    },
    true
  );
  wizard.common.initDefaultConfig();
  wizard.common.go(wizard.common.getNextStage.bind(undefined, null));
}

function MainWizardScreen(wizard) {
  WizardScreen.call(this, wizard);

  this.configIsLoaded = false;

  this.preEnter = function() {
    wizard.common.unhide('loadConfigButton');
  };

  this.enter = function() {
    println('enter wizard.main.enter');
  };

  this.leave = function(automatic) {
    wizard.common.hide('loadConfigButton');
  };
  
  this.adjustName = function(name, existing_providers) {
    var lookup = {};
    for(var i = 0; i < existing_providers.length; i++) {
      lookup[existing_providers[i].object.name] = true;
    }
    var prefix="amazon";
    var i = 2;
    while(lookup.hasOwnProperty(name)) {
      name = prefix + i;
      i++;
    }
    return name;
  };

  this.asyncForward = function(automatic, followup) {
    if(this.configIsLoaded) {
      followup();
    } else {
      var self = this;
      var request = { "tool" : "cm-cluster-extension" };
      var existing_providers = current.find('CloudProvider');
      wizard.common.disable("nextButton");
      wizard.common.unhide('main-wait-for-config');
      wizard.common.remoteValidation("cm_setup_get_tool_config", request, function(response) {
        if(response.error) {
          document.getElementById('config-error-message').value = response.error;
          wizard.common.hide('main-wait-for-config');
          wizard.common.unhide('config-error');
          self.error = true;
        } else {
          var request2 = {};
          wizard.common.remoteValidation("cluster_extension_get_region2types", request2, function(response2) {
            if(response2.error) {
              document.getElementById('config-error-message').value = response2.error;
              wizard.common.hide('main-wait-for-config');
              wizard.common.unhide('config-error');
              self.error = true;
            } else {
              wizard._region2instance_types = response2.result.lookup;
              wizard.config.modules = response.result.modules; 
              wizard.config.modules.clusterextension.provider_name = self.adjustName(wizard.config.modules.clusterextension.provider_name,
                                                                                     existing_providers);
              self.configIsLoaded = true;
              wizard.init_default_cloud_director_type = wizard.config.modules.clusterextension.default_cloud_director_type;
              wizard.init_default_cloud_node_type = wizard.config.modules.clusterextension.default_cloud_node_type;
            }
            wizard.common.enable("nextButton");
            wizard.common.hide('main-wait-for-config');
            followup();
          });
        }
      });
    }
  };
}

function CredentialsWizardScreen(wizard) {
  WizardScreen.call(this, wizard);

  this.elemKeyMapping = {
    "credentials-provider_name" : "provider_name",
    "credentials-username":       "username",
    "credentials-account_id":     "account_id",
    "credentials-access_key":     "access_key",
    "credentials-secret_key":     "secret_key"};

  this.asyncForward = function(automatic, followup) {
    println('asyncForward');
    if(document.getElementById("credentials-validate").checked) {
      var self = this;
      // prepare remote validation request
      var request_mapping = {
        "credentials-access_key": "aws_access_key",
        "credentials-secret_key": "aws_secret_access_key" };
      var request = {
        "region" : wizard.config.modules.clusterextension.advanced.verify_creds_region
      };
      wizard.common.getFieldValues(request_mapping, request);
      for(var k in this.elemKeyMapping) {
        wizard.common.disable(k);
      }
      wizard.common.disable("nextButton");
      wizard.common.unhide('credential-wait-for-validation');
      wizard.common.getFieldValues(this.elemKeyMapping, 
                                   wizard.config.modules.clusterextension);
      wizard.common.remoteValidation("cluster_extension_verify_credentials", request, function(response) {

        var error_message = document.getElementById('credentials-message');
        if(response.state == 3) {
          self.error = true;
          wizard.common.unhide('credentials-validation-error');
          error_message.value = response.error;
        } else {
          if(response.result.hasOwnProperty('aws_account_id')) {
            wizard.config.modules.clusterextension.account_id = response.result.aws_account_id;
          }
          if(response.result.hasOwnProperty('username')) {
            wizard.config.modules.clusterextension.username = response.result.username;
          }
          if(!response.result.verified) {
            self.error = true;
            wizard.common.setFieldValues(self.elemKeyMapping, 
                                         wizard.config.modules.clusterextension);
            wizard.common.unhide('credentials-validation-error');
            wizard.common.unhide('credentials-username-box');
            wizard.common.unhide('credentials-account_id-box');
            error_message.value = response.result.msg;
          } else {
            wizard.common.setFieldValues(self.elemKeyMapping, 
                                         wizard.config.modules.clusterextension);
            wizard.common.hide('credentials-validation-error');
          }
        }
        for(var k in self.elemKeyMapping) {
          wizard.common.enable(k);
        }
        wizard.common.enable("nextButton");
        wizard.common.hide('credential-wait-for-validation');
        wizard.deck['credentials'].enable_click = true;
        followup();
      });
    } else {
      //this.forward(automatic);
      followup();
    }
  };

  this.forward = function(automatic) {
    //wizard.common.getFieldValues(this.elemKeyMapping, 
    //                             wizard.config.modules.clusterextension);
  };

  this.enter = function() {
    println('enter wizard.credentials.enter');
    wizard.common.hide('credentials-validation-error');
    wizard.common.setFieldValues(this.elemKeyMapping, 
                                 wizard.config.modules.clusterextension);
  }
}

function ClusterExtensionWizard() {
  Wizard.call(this);
  this.dryMode = false;
  this.forceDryMode = false;
  this.advancedMode = false;
  this.configuration_name = "VPC Configuration"; 
  this.main = new MainWizardScreen(this);
  this.credentials = new CredentialsWizardScreen(this);
  this.regions = new RegionsWizardScreen(this);
  this.summary = new SummaryWizardScreen(this);
  this.deploy = new DeployWizardScreen(this);
  this.deploy.cm_setup_tool_script = 'cm-cluster-extension';
  //this.deploy.cm_setup_tool_script = 'cm-setup-mock';
  this.deploy.cm_setup_tool_name   = 'Cloud Extension';

  this.deck = this.createDeck( [ { 'name' : 'main'}, 
                                 { 'name' : 'credentials'}, 
                                 { 'name' : 'regions'}, 
                                 { 'name' : 'summary'}, 
                                 { 'name' : 'deploy'} ] );
  this.currentStage = null;
  this.previousStage = null;
}
