'user strict';
global.__base = __dirname + '/../../';
var test = require('unit.js');
var expect    = require("chai").expect;
var Context = require('cm-context');

var context;
var wizard_module_ctx;
var wizard;

function Element(parent) {
  this.children = [];
  this.parentElement = parent;
  this.appendChild = function(elem) {
    this.children.push(elem);
  };
}

function testSetup() {
  context = new Context(global.__base);
  wizard_module_ctx = context.load('cm-setup/wizard.js');
  wizard_module_ctx = context.load('cm-setup/deploy.js');
  wizard_module_ctx = context.load('cloud/setup/regionsWizardScreen.js');
  wizard_module_ctx = context.load('cloud/setup/summaryWizardScreen.js');
  wizard_module_ctx = context.load('cloud/setup/clusterExtensionWizard.js');
  wizard = new context.context.ClusterExtensionWizard(); 
  // setup configuration
  wizard.config.modules = {};
  wizard.config.modules.clusterextension = {
    _region2instance_types: { 'region-1': [ 't1', 't2' ], 
                              'region-2': [ 't1', 't3' ],
                              'region-3': [ 't1', 't2', 't3'] },

    default_cloud_director_type: [ 't2' ],
    default_cloud_node_type: ['t1']
  };

  context.context.menulist_default_cloud_director_type = new Element();
  context.context.menulist_default_cloud_director_type.selectedIndex = -1;
  context.context.menupopup_default_cloud_director_type = new Element(context.context.menulist_default_cloud_director_type);

  context.context.menulist_default_cloud_node_type = new Element();
  context.context.menulist_default_cloud_node_type.selectedIndex = -1;
  context.context.menupopup_default_cloud_node_type = new Element(context.context.menulist_default_cloud_node_type);
  
  context.context.document = { 
    _elements : {
      "summary-default_cloud_director_type": context.context.menupopup_default_cloud_director_type,
      "summary-default_cloud_node_type": context.context.menupopup_default_cloud_node_type,
    },

    getElementById : function(name) {
      return this._elements[name];
    },
    createElement : function(name) {
      return {
        setAttribute : function(k,v) {}
      }
    }
  };

  context.context.printall = function(x) {
  };
}

describe('Cluster Extension Wizard', function(){
  it('summary.getIntersectionOfInstanceTypes determines overlap of types for set of regions', function() {
    testSetup();
    test.assert.deepEqual(wizard.summary.getIntersectionOfInstanceTypes([ "region-1"]),
                          ['t1', 't2']);
    test.assert.deepEqual(wizard.summary.getIntersectionOfInstanceTypes([ "region-1", "region-2", "region-3" ]),
                          ['t1']);
    test.assert.deepEqual(wizard.summary.getIntersectionOfInstanceTypes([ "region-1", "region-3"]),
                          ['t1', 't2']);
  });

  it('summary.createDefaultTypeItems for set of regions', function() {
    testSetup();
    wizard.summary.createDefaultTypeItems([ "region-1", "region-3"]);
    test.assert.equal(context.context.menulist_default_cloud_node_type.selectedIndex, 0);
    test.assert.equal(context.context.menulist_default_cloud_director_type.selectedIndex, 1);
  });
});

