/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'CloudNode';
var singular = 'cloud node';
var plural = 'cloud nodes';

function allKeys() {
  if(current && current.cluster) return current.cluster.findNodesIn(current.get('baseType'), current.get('uniqueKey'));
  return [];
}

function specialrefresh() {
  tree = document.getElementById('tree');
  var list = allKeys();
  data = new Array();
  var modified = false;
  for(var i = 0; i < list.length; i++) {
    list[i] = current.cluster.find(list[i]);
    if(!list[i].get('toBeRemoved') || alsoRemoved) {
      data.push(createTreeItem(list[i]));
      modified = modified || list[i].get('modified') || list[i].get('toBeRemoved');
    } else {
      modified = true;
    }
  }
  for(var i = list.length; i < MINTREEITEMS; i++) data.push(blankTreeItem());
  var select = selected('list');
  loadTable();
  setSelected('list', select);

  if(modified) {
    document.getElementById('save').removeAttribute('disabled');
  } else {
    document.getElementById('save').setAttribute('disabled', true);
  }
  return true;
}

function addObject() {
  var args = new Object();
  args['title'] = 'Add new ' + singular;
  args['question'] = 'Hostname';
  args['known'] = current.cluster.known('Device', 'hostname');
  args['default'] = 'cnode';
  var d = new dock.cvDialog("newobject.xul", args);
  d.onOk = function(name) {
    var object = new dock.cvCloudNode();
    object.hostname = name;
    var base = current.cluster.getBasePartition();
    if(isObject(base)) {
      object.partition = base.object.uniqueKey;
      object.category = base.object.defaultCategory;
    }
    var cloudsettings = object.cloudSettings;
    if(!cloudsettings || (cloudsettings.childType != 'EC2Settings')) {
      object.cloudSettings = new dock.cvEC2Settings();
      cloudsettings = object.cloudSettings;
    }
    if(cloudsettings.uniqueKey == 0) {
      cloudsettings = new dock.cvObject(current.cluster, cloudsettings);
      cloudsettings = cloudsettings.object;
    }
    if(cloudsettings.childType == 'EC2Settings') {
      var providers = current.cluster.find('EC2Provider');
      if(providers.length > 0) cloudsettings.provider = providers[0].get('uniqueKey');
      var s = new dock.cvEC2EBSStorage();
      s.name = 'ebs';
      s.drive = 'sdb';
      s.size = 42 * 1024 * 1024 * 1024;
      s = new dock.cvObject(current.cluster, s);
      s.set('toBeRemoved', false);
      s.set('modified', true);
      var storage = new Array();
      storage.push(s.object);
      cloudsettings.storage = storage;
      cloudsettings.modified = true;      
    }
    current.cluster.addObject(object, dock.openAfterCreate);
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function createWizard() {
  var args = new Object();
  args['cluster'] = current.cluster;
  args['basetype'] = basetype;
  args['cloudprovider'] = current;
  var d = new dock.cvDialog("chrome://cmgui/content/cloud/addnode/nodeCreationWizard.xul", args);
  d.onOk = function() {
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function createDirector() {
  var args = new Object();
  args['cluster'] = current.cluster;
  args['object'] = 0;
  args['clouddirector'] = true;
  args['provider'] = current;
  var d = new dock.cvDialog("chrome://cmgui/content/cloud/setup/cloudDirectorWizard.xul", args);
  d.onOk = function() {
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function moveDirector() {
  var cluster = current;
  if (!!current.cluster) cluster = current.cluster;
  var select = getSelected();
  if (select.length == 0) {
    dock.displayError('Please select a ' + singular);
  } else if (select.length > 1) {
    dock.displayError('Please select only one ' + singular);
  } else {
    var node = cluster.find(select[0]);
    if (node) {
      if (!node.hasCloudDirectorRole()) {
        dock.displayError('The selected ' + singular + ' is not a cloud director');
      } else if (!isVPCavail(node)) {
        dock.displayError('No Private clouds available to move the selected ' + singular + '.\nTo move a Cloud director into a Private cloud, the Private cloud should have at least two subnets configured.');
      } else {
        var instanceId = node.get('cloudSettings').instanceId;
        var spotId = node.get('cloudSettings').spotId;
        if (!!instanceId || !!spotId) {
          dock.displayError('The selected ' + singular + ' cannot be moved');
        } else {
          var args = new Object();
          args['cluster'] = cluster;    
          args['clouddirector'] = select[0];
          var d = new dock.cvDialog("moveclouddirector.xul", args);
          d.onOk = function() {
            refresh();
          }
          dock.dialogs.showDialog(d);
        }
      }
    }
  }
}

function isVPCavail(node) {
  var numAvail = 0;

  var cluster = current;
  if (!!current.cluster) cluster = current.cluster;

  var alreadyUsedNets = new Array();
  var cloudNodes = cluster.find('CloudNode');
  for (var i = 0; i < cloudNodes.length; i++) {
    if (cloudNodes[i].hasCloudDirectorRole()) {
      var networks = cloudNodes[i].get('networks');
      for (var j = 0; j < networks.length; j++) {
        if (networks[j].childType == 'NetworkPhysicalInterface') {
          alreadyUsedNets.push(networks[j].network);
        }
      }
    }
  }

  var region = node.get('cloudSettings').region;
  var provider = cluster.find(node.get('cloudSettings').provider);
  var vpcs = provider.get('privateClouds');
  for (var i = 0; i < vpcs.length; i++) {
    if ((vpcs[i].subnets.length > 1) && (vpcs[i].region == region)) {
      var availSub = true;
      for (var j = 0; j < vpcs[i].subnets.length; j++) {
        if (alreadyUsedNets.find(vpcs[i].subnets[j]) != -1) {
          availSub = false;
        }
      }
      if (availSub) {
        numAvail++;
      }
    }
  }

  if (numAvail > 0) {
    return true;
  }

  return false;
}
