/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var serv;
var timer = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function stop() {
  println("Stop cloudprovider prices");
  if(timer) window.clearTimeout(timer);
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh cloudprovider prices" + ' - ' + document.documentURI);

  dock.workerpool.getWorker(current.cluster.actualurl).call('cmmon', 'getEC2SpotPriceData', ['', ''], function(code, data) {
    if(code == 0) {
      setCallbackResult(data);
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);

  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('refresh();', dock.generalSettings.statusRefresh * 10000);  
}

function findIn(list, name) {
  for(var i = 0; i < list.length; i++) {
    if(list[i].resolveName() == name) return list[i];
  }
  return "Not found: " + name;
}

function digit2(val) {
  if(val < 0.001) {
    val = sprintf('%.4f', val);
  } else if(val < 0.01) {
    val = sprintf('%.3f', val);
  } else {
    val = sprintf('%.2f', val);
    var idx = val.indexOf('.');
    if(idx < 0) {
      val += '.00';
    } else if(idx + 2 == val.length) {
      val += '0';
    }
  }
  return val;
}

var typeMap = {'micro': 0, 'small': 1, 'medium': 2, 'large': 3, 'xlarge': 4};
function typeSize(t) {
  var ti = 1;
  var tt;
  if(t[0] >= '0' && t[0] <= '9') {
    ti = parseInt(t[0]);
    tt = t.substr(1);
  } else {
    tt = t;
  }
  if(typeMap.hasOwnProperty(tt)) {
    tt = typeMap[tt];
  }
  return [ti, tt];
}

function typeSort(a, b) {
  var va = a.object.name.split('.');
  var vb = b.object.name.split('.');
  if(va[0] > vb[0]) {
    return 1;
  } else if(va[0] < vb[0]) {
    return -1;
  } else {
    var ta = typeSize(va[1]);
    var tb = typeSize(vb[1]);
    if(ta[1] > tb[1]) {
      return 1;
    } else if(ta[1] < tb[1]) {
      return -1;
    } else if(ta[0] > tb[0]) {
      return 1;
    } else if(ta[0] < tb[0]) {
      return -1;
    }
  }
  return 0;
}

function setCallbackResult(spotPrices) {
  var ss = new Object();
  for(var i = 0; i < spotPrices.length; i++) {
    ss[spotPrices[i].metricParam] = digit2(spotPrices[i].rate) + ' $/h';
  }

  var regions = current.cluster.find('CloudRegion');
  var types = current.cluster.find('CloudType');
  types.sort(typeSort);

  var rows = document.getElementById('prices');
  while(rows.firstChild) rows.removeChild(rows.firstChild);
  var N = rows.parentNode.firstChild.childNodes.length;
  var row;
  
  var i0 = 0;
  for(var i = 0; i < types.length; i++) {
    if(typeof(types[i]) == 'string') {
      println(types[i]);
      continue;
    }
    var type = types[i].resolveName();
    var prices = types[i].get('ondemandPrices');
    var pp = new Object();
    for(var j = 0; j < prices.length; j++) {
      var idx = prices[j].indexOf(':');
      if(idx > 0) pp[prices[j].substr(0, idx)] = digit2(parseFloat(prices[j].substr(idx + 1))) + ' $/h';
    }
    var matrix = new Array();
    for(var j = 0; j < regions.length; j++) {    
      var region = regions[j].resolveName();      
      var ondemandPrice = pp[region];
      if(ondemandPrice) {
        var zones = regions[j].get('availabilityZones');
        for(var k = 0; k < zones.length; k++) {
          var one = [zones[k].name, ondemandPrice];
          var spotPrice = ss[zones[k].name + ',' + type];
          if(spotPrice) {
            one.push(spotPrice);
          } else {
            one.push('N/A');
          }
          matrix.push(one);
        }
      }
    }
    var matrix = matrix.sort();
    var vbox = overviewGenericPage(type, ['Zone', 'Ondemand', 'Spot'], matrix);
    if((i0 % N) == 0) {
      row = document.createElement('row');
      rows.appendChild(row);
    }
    row.appendChild(vbox);
    i0++;
  }
}
