/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var ipDirectorMap; // ipDirectorMap[ipkey] gives uniqueKey for the director associated with Static IP

var basetype = 'CloudStaticIP';
var singular = 'Cloud static IP';
var plural = 'Cloud static IPs';
var allocatefile = "allocatestaticip.xul";

function refresh(use) {
  println("Refresh " + plural + ' - ' + document.documentURI);

  if(!!use) current = use;

  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;

  ipDirectorMap = new Object();
  var cloudNodes = cluster.find("CloudNode");
  for (var i = 0; i < cloudNodes.length; i++) {
    if (cloudNodes[i].object.cloudSettings.cloudStaticIP) {
      ipDirectorMap[cloudNodes[i].object.cloudSettings.cloudStaticIP] = cloudNodes[i].resolveName();
    }
  }

  var cloudProviderKey = current.object.uniqueKey;
  var list = cluster.find(basetype);
  tree = document.getElementById('tree');
  data = new Array();
  for (var i = 0; i < list.length; i++) {
    if (list[i].get('cloudProvider') == cloudProviderKey) {
      data.push(createTreeItem(list[i]));
    }
  }
  for(var i = list.length; i < MINTREEITEMS; i++) data.push(blankTreeItem());
  var select = selected('list');
  loadTable();
  setSelected('list', select);
}

function createTreeItem(one) {
  var item = new Object();
  item['key'] = one.get('uniqueKey');
  var provider = current.cluster.find(one.get('cloudProvider'));
  var providerName = provider.resolveName();
  item['provider'] = providerName;
  var region = current.cluster.find(one.get('cloudRegion'));
  var regionName = '';
  if (region) {
    regionName = region.resolveName();
  }
  item['region'] = regionName;
  item['staticip'] = one.get('ip');
  var directorName = ipDirectorMap[one.get('uniqueKey')];
  if (!!directorName) {
    item['assignedto'] = directorName;
  } else {
    item['assignedto'] = "<not assigned>";
  }

  return item;
}

function allocate() {
  var args = new Object();
  args['cluster'] = current.cluster;
  args['cloudProvider'] = current;
  var d = new dock.cvDialog(allocatefile, args);
  d.onOk = function() {
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function release() {
  var select = getSelected();
  if (select.length == 0) {
    dock.displayError('Please select a ' + singular);
  } else if (select.length > 1) {
    dock.displayError('Please select only one ' + singular);
  } else {
    var cluster = current;
    if (!!current.cluster) cluster = current.cluster;
    
    var staticip = cluster.find(select[0]);
    var ip = staticip.get('ip');
    var allocationId = staticip.get('allocationId');
    var directorName = ipDirectorMap[staticip.get('uniqueKey')];
    if (!!directorName) {
      dock.displayInfo("The selected " + singular + " is assigned to '" + directorName + "' and cannot be released.\nFirst unassign it in the 'Cloud Settings' tab.");
    } else {
      var args = new Object();
      args['question'] = 'Are you sure you want to release this ' + singular + '?';
      var d = new dock.cvDialog('question.xul', args);
      d.onOk = function() {
        dock.workerpool.getWorker(cluster.actualurl).call('cmcloud', 'unallocateStaticIP', [ip, allocationId], function(code, data) {
          if(code == 0) {
            releaseCallback(data);
          } else {
            dock.jsonCallError(this, code, data);
          }
        }, cluster);
      }
      dock.dialogs.showDialog(d);
    }
  }
}

function releaseCallback(data) {
  if (data.success) {
    dock.currentTab.contentWindow.refresh();
  } else {
    dock.displayError('Error while releasing the selected ' + singular);
  }
}
