/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function updateTypeDescription(current) {
  var settings = current.get('cloudSettings');
  if(settings.childType == 'EC2Settings') {
    var type = settings.type;
    if(!type) {
      var p = settings.provider;
      if(p) {
        p = current.cluster.find(p);
        if(p) {
          if(current.hasCloudDirectorRole()) {
            type = p.get('defaultDirectorType');
          } else {
            type = p.get('defaultType');
          }
        }
      }
    }
    type = current.cluster.find(type);
    if(type) {
      document.getElementById('cloudtypedescription').value = strip_beginning_trailing_newlines(type.get('description'));
    } else {
      document.getElementById('cloudtypedescription').value = "";
    }
  }
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh node cloud settings" + ' - ' + document.documentURI);
  document.getElementById('revert').setAttribute('disabled', current.object.uniqueKey < dock.MAX_LOCAL_KEY);

  var settings = current.get('cloudSettings');
  if(settings.childType == 'EC2Settings') {
    text('cloudSettings', '', 'instanceId');
    text('cloudSettings', '', 'spotId');
//     text('cloudSettings', '', 'sshConnectString');
    text('cloudSettings', '^[0-9]+(\.[0-9]+)?$', 'spotPrice');
    check('cloudSettings', false, 'spotPersistent');    
//     text('cloudSettings', '', 'options');
//     text('cloudSettings', '', 'kernel');
//     text('cloudSettings', '', 'initrd');
    var p = settings.provider;
    var dt = '', dr = '', da = '', di = '';
    if(p) {
      p = current.cluster.find(p);
      if(p) {
        var n = p.resolveName();
        if(current.hasCloudDirectorRole()) {
          var d = current.cluster.find(p.get('defaultDirectorType'));
          if(d) dt = d.resolveName() + ' (' + n + ')';
        } else {
          var d = current.cluster.find(p.get('defaultType'));
          if(d) dt = d.resolveName() + ' (' + n + ')';
        }
        var d = current.cluster.find(p.get('defaultRegion'));
        if(d) dr = d.resolveName() + ' (' + n + ')';
        // TODO: AZ
//         var d = current.cluster.find(p.get('defaultAvailabilityZone'));
//         if(d) da = d.resolveName() + ' (' + n + ')';
        var d = current.cluster.find(p.get('defaultAMI'));
        if(d) {
          di = d.resolveName() + ' (' + n + ')';
        } else {
          d = current.cluster.getLatestEC2AMI();
          if(d) {
            di = 'latest:' + d.resolveName() + ' (' + n + ')';
          } else {
            di = 'No default (' + n + ')';
          }
        }
      }
    }
    resolve('cloudSettings', 'EC2Type', 'type', dt);
    resolve('cloudSettings', 'EC2Region', 'region', dr);
    resolve('cloudSettings', 'EC2AvailabilityZone', 'availabilityZone', da);
    resolve('cloudSettings', 'EC2AMI', 'ami', di);

    // dropdown for cloud static IPs
    // list to be filled with all available (not used by other directors) IPs in the same region
    var ipList = new Array();
    var currentRegion = settings.region;
    if (currentRegion == 0) { // default region
      currentRegion = p.get('defaultRegion');
    }
    // IPs already used
    var cloudNodes = current.cluster.find('CloudNode');
    var notAvailIPs = new Array();
    for(var i = 0; i < cloudNodes.length; i++) {
      if (current.get('uniqueKey') != cloudNodes[i].get('uniqueKey')) {
        notAvailIPs.push(cloudNodes[i].get('cloudSettings').cloudStaticIP);
      }
    }
    var ips = current.cluster.find('CloudStaticIP');
    for(var i = 0; i < ips.length; i++) {
      if ((ips[i].get('cloudRegion') == currentRegion) && (notAvailIPs.indexOf(ips[i].get('uniqueKey')) == -1)) {
        ipList.push([ips[i].get('uniqueKey'), ips[i].get('ip')]);
      }
    }
    ipList.push(['0', 'none']);
    kenum('cloudSettings', ipList, 'cloudStaticIP');
    check('cloudSettings', false, 'releaseStaticIPOnTermination');
    
    var started = (settings.instanceId != '') || (settings.spotId != '');
    setDisabled('cloudSettingsoptions', started);
    setDisabled('cloudSettingskernel', started);
    setDisabled('cloudSettingsinitrd', started);
    setDisabled('cloudSettingstype', started);
    setDisabled('cloudSettingsregion', started);
    setDisabled('cloudSettingsavailabilityZone', started);
    setDisabled('cloudSettingsami', started);
    setDisabled('cloudSettingscloudStaticIP', started);
    setDisabled('cloudSettingsreleaseStaticIPOnTermination', started);
    setDisabled('cloudSettingsspotPrice', started);
    setDisabled('cloudSettingsspotPersistent', started);

    setDisabled('cloudSettingsregion', true);
    if(!started) extraeval('cloudSettingstype', 'updateTypeDescription(current);');
   
    var storage = settings.storage;
    addEC2Storage(storage, started);
    updateTypeDescription(current);
  } else {
    println('--------------- NOT EC2 SETTINGS -----------------');
    printall(settings);
  }
}

function addEC2Storage(storage, started) {
  var rows = document.getElementById('storage');
  while(rows.firstChild) rows.removeChild(rows.firstChild);
  var header = [ "", "Type", "Name", "Drive", "Size", "Snapshot ID" ];
  var count = 0;
  var row = document.createElement("row");
  row.setAttribute('align', 'center');
  rows.appendChild(row);
  for(var i = 0; i < header.length; i++) {
    var label = document.createElement("shadowlabel");
    label.setAttribute("value", header[i]);
    label.setAttribute("class", "staticdataheader");
    row.appendChild(label);
  }
  for(var i = 0; i < storage.length; i++) {
    if(storage[i].toBeRemoved) continue;
    count++;
    var isEBS = storage[i].childType == 'EC2EBSStorage';
    var row = document.createElement("row");
    row.setAttribute('align', 'center');
    rows.appendChild(row);

    var hbox = document.createElement("hbox");
    hbox.setAttribute('pack', 'end');
    var label = document.createElement("shadowlabel");
    label.setAttribute('value', 'Storage:');
    label.setAttribute('class', 'staticdataheader');
    hbox.appendChild(label);
    row.appendChild(hbox);

    var menu = document.createElement("menulist");
    if(started) menu.setAttribute('disabled', true);
    menu.setAttribute('class', 'tablemenulist');
    var popup = document.createElement("menupopup");
    menu.appendChild(popup);
    var item = document.createElement("menuitem");
    item.setAttribute('label', 'EBS');
    popup.appendChild(item);
    var item = document.createElement("menuitem");
    item.setAttribute('label', 'Ephemeral');
    popup.appendChild(item);    
    menu.setAttribute('id', 'cloudSettingsstorage_' + i + 'childType');
    menu.setAttribute('what', i);
    menu.setAttribute('class', 'stringlist');
    row.appendChild(menu);
    popup.addEventListener("command", storagetype_command, true);
    if(isEBS) {
      menu.selectedIndex = 0;
    } else {
      menu.selectedIndex = 1;
    }

    var textbox = document.createElement("textbox");
    if(started) textbox.setAttribute('disabled', true);
    textbox.setAttribute('id', 'cloudSettingsstorage_' + i + 'name');
    row.appendChild(textbox);

    var textbox = document.createElement("textbox");
    if(started) textbox.setAttribute('disabled', true);
    textbox.setAttribute('id', 'cloudSettingsstorage_' + i + 'drive');
    row.appendChild(textbox);

    var textbox = document.createElement("textbox");
    if(started || !isEBS) textbox.setAttribute('disabled', true);
    if(isEBS) {
      textbox.setAttribute('id', 'cloudSettingsstorage_' + i + 'size');
    } else {
      textbox.setAttribute('value', 'auto');
    }      
    row.appendChild(textbox);
    
    if(isEBS) {
      var textbox = document.createElement("textbox");
      textbox.setAttribute('id', 'cloudSettingsstorage_' + i + 'snapshotId');
      if(started) textbox.setAttribute('disabled', true);
      row.appendChild(textbox);
    } else {
      var textbox = document.createElement("hbox");
      row.appendChild(textbox);
    }
    if(!started) {
      if(storage.length > 1) {
        var minus = document.createElement("button");
        minus.setAttribute('class', 'minus-button');
        minus.setAttribute('oncommand', "removeStorageItem(" + i + ");");
        row.appendChild(minus);
      }
      if(i == storage.length - 1) {
        var plus = document.createElement("button");
        plus.setAttribute('class', 'plus-button');
        plus.setAttribute('oncommand', "addStorageItem();");
        row.appendChild(plus);
      } else {
        var hbox = document.createElement("hbox");
        hbox.setAttribute('height', '4px');
        rows.appendChild(hbox);
      }
    }
  }

  for(var i = 0; i < storage.length; i++) {
    if(storage[i].toBeRemoved) continue;
    var isEBS = storage[i].childType == 'EC2EBSStorage';
    text('cloudSettingsstorage_' + i, '^', 'storage|name');
    text('cloudSettingsstorage_' + i, '^(/dev/)?(s|h|xv)d[a-z]$', 'storage|drive');
    if(isEBS) {
      displayfunction('cloudSettingsstorage_' + i + 'size', 'formatKMGselect(value, "B");');
      parsefunction('cloudSettingsstorage_' + i + 'size', 'parseKMG(value);');
      text('cloudSettingsstorage_' + i, '^[0-9]+(\.[0-9]*)?\s*[kmgtKMGT]?i?[Bb]?$', 'storage|size');
      text('cloudSettingsstorage_' + i, '^.*$', 'storage|snapshotId');
    }
  }

  if(!started && (count == 0)) {
    var row = document.createElement("row");
    row.setAttribute('align', 'center');
    rows.appendChild(row);

    var hbox = document.createElement("hbox");
    hbox.setAttribute('pack', 'end');
    var label = document.createElement("shadowlabel");
    label.setAttribute('value', 'Storage:');
    label.setAttribute('class', 'staticdataheader');
    hbox.appendChild(label);
    row.appendChild(hbox);

    var plus = document.createElement("button");
    plus.setAttribute('class', 'plus-button');
    plus.setAttribute('oncommand', "addStorageItem();");
    row.appendChild(plus);
  }
}

function addStorageItem() {
  var settings = current.get('cloudSettings');
  if(settings.childType == 'EC2Settings') {
    var storage = settings.storage;
    var one = new dock.cvEC2EBSStorage();
    one = new dock.cvObject(current.cluster, one);
    one.set('toBeRemoved', false);
    one.set('modified', true);
    storage.push(one.object);
    addEC2Storage(storage, false);
  }
}

function removeStorageItem(index) {
  var settings = current.get('cloudSettings');
  if(settings.childType == 'EC2Settings') {
    var storage = settings.storage;
    if(storage.length > index) {
      storage[index].toBeRemoved = true;
      addEC2Storage(storage, false);
    }
  }
}

function storagetype_command(event) {
  var item = event.target;
  var value = item.getAttribute('label');
  var index = parseInt(item.parentNode.parentNode.getAttribute('what'));
  var settings = current.get('cloudSettings');
  if(settings.childType == 'EC2Settings') {
    var storage = settings.storage;
    if(storage.length > index) {
      if((value == 'EBS') && (storage[index].childType == 'EC2EphemeralStorage')) {
        var one = new dock.cvEC2EBSStorage();
        one = new dock.cvObject(current.cluster, one);
        one.set('toBeRemoved', false);
        one.set('modified', true);
        storage[index] = one.object
        settings.storage = storage;
        settings.modified = true;
        addEC2Storage(storage, false);
      } else if((value == 'Ephemeral') && (storage[index].childType == 'EC2EBSStorage')) {
        var one = new dock.cvEC2EphemeralStorage();
        one = new dock.cvObject(current.cluster, one);
        one.set('toBeRemoved', false);
        one.set('modified', true);
        storage[index] = one.object;
        settings.storeage = storage;
        settings.modified = true;
        addEC2Storage(storage, false);
      }
    }
  }
}
