/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var USE_EVENT_THREAD = 2; // 0 -> no thread, 1 -> remotecm, 2 -> xul-thread
var MAX_GRAPH_REMOTECM = 8;

function cvCluster() {
  this.protocol = 0;
  this.host = 0;
  this.port = 0;
  this.url = 0;
  this.actualurl = 0;
  this.autoconnect = false;
  this.key = -1;

  this.uri = "http://www.brightcomputing.com";
  this.connected = false;
  this.connecting = false;
  this.reconnecting = false;
  this.serviceMap = 0;
  this.objects = 0;
  this.name = '';
  this.sessionid = 0;
  this.eventworker = 0;
  this.failedEventCalls = 0;
  this.statusworker = 0;
  this.eventindex = -1;
  this.certificate = '';
  this.username = '';
  this.password = '';
  this.temppassword = '';
  this.temptoken = '';
  this.logincanceled = false;
  this.requestid = -1;
  this.certkey = 0;
  this.csr = 0;
  this.treeitem = 0;
  this.menuitem = 0;
  this.menuitemf = 0;
  this.branch = 0;
  this.nextLocalKey = 0;
  this.setup = 0;
  this.base = 0;
  this.currentSubObjectType = 0;
  this.currentSubObjectList = 0;
  this.treeClosed = 0;
  this.activePassiveKeys = 0;
  this.syslogin = '';
  this.brightVersion = '';
  this.masterOnCloud = -1;

  this.reconnectTimeout = 0;
  this.connectcount = 0;
  this.rackMetrics = 0;
  this.mainMetric = 0;
  this.hadoopMetric = 0;
  this.cephMetric = 0;
  this.complete = 0;
  this.completed = 0;

  this.isKickStart = false;

  this.updateRack = true;
  this.updateHealth = true;

  this.followRedirect = true;
  this.timeOffset = 0;

  this.infoMessage = new Object();
  this.graphUse = 0;
  this.eventRetries = 0;
  this.isCluster = true;

  this.treeFillList = 0;
  this.treeFillIndex = 0;
  this.treeFillTimer = 0;

  this.forcedDisconnect = false;

  this.extraComplete = 0;
  this.userDefinedMetricTypes = 0;

  this.licenseInfo = 0;
  this.versionInfo = 0;

  this.healthDeviceSelection = 0;
  this.rackDeviceSelection = 0;

  this.nodeJobCache = 0;
  this.jobNodeCache = 0;
  this.lastJobCacheUpdate = 0;
  this.jobCache = 0;
  this.jobCacheTimer = 0;
  this.jobCacheIndex = 0;
  this.jobCacheStart = 0;
  
  this.lustreErrors = new Array();

  this.monitoringExtraGroups = 0;

  this.rack = 0;
  this.health = 0;

  this.chassisMap = 0;
  this.stackMap = 0;

  this.treeFillBusy = false;
  this.treeFillFollowUp = 0;

  this.stopping = false;

  this.LDAPstatus = new Object();

  this.genericDeviceInfo = new Object();

  this.hasUCS = false;
  this.hasDellSettings = false;
  this.hasIEEL = false;

  this.hasCeph = false;
  this.hasHadoop = false;
  this.hasOpenStack = false;
  this.hasPuppet = false;
  this.hasKube = false;

  this.openStackloggedin = false;
  this.fakedOpenStackApiSettings = false;

  // grep "<child " openstackapientity.xml | sed 's#.*"\(.*\)" simp.*#"\1",#g' | tr '\n' ' '
  this.openStackApiEntities = ["OpenStackApiNetwork", "OpenStackApiPort", "OpenStackApiSubnet", "OpenStackApiRouter",
                               "OpenStackApiFloatingIP", "OpenStackApiImage", "OpenStackApiVolume", "OpenStackApiVolumeSnapshot",
                               "OpenStackApiVolumeType", "OpenStackApiProject", "OpenStackApiDomain", "OpenStackApiUser",
                               "OpenStackApiRole", "OpenStackApiGroup", "OpenStackApiRoleAssignment", "OpenStackApiService",
                               "OpenStackApiEndpoint", "OpenStackApiFlavor", "OpenStackApiAgent", "OpenStackApiHostAggregate",
                               "OpenStackApiServer", "OpenStackApiSecurityGroup", "OpenStackApiHypervisor"];

  this.cephState = 0;
  this.cephStateLookup = 0;
  this._listeners = {};
  this._listener_id = {};
  
  this.globalConfig = new Object();
};

cvCluster.prototype.addListener = function(type, listener){
  var ret = 0;
  if(typeof this._listener_id[type] == "undefined") {
     this._listener_id[type] = 0;
  }
  this._listener_id[type]++;
  ret = this._listener_id[type];
  if (typeof this._listeners[type] == "undefined"){
      this._listeners[type] = {};
  } 
  this._listeners[type][ret] = listener;
  return ret;
};

cvCluster.prototype.dispatchEvent = function(event) {
  var evtype = "";
  if (typeof event == "string"){
    evtype = event;
  }
  else if (!event.type) {
    evtype = event.childType
  }
  else {
    evtype = event.type;
  }
  if (evtype == ""){  //falsy
    throw new Error("Event object missing 'type' property.");
  }
  if (typeof this._listeners[evtype] != "undefined" && (this._listeners[evtype] instanceof Object) ){
    var listeners = this._listeners[evtype];
    var keys = Object.keys(this._listeners[evtype]).sort(function(a, b){return a-b});
    for(var i =0; i < keys.length; i++) {
      this._listeners[evtype][keys[i]](event, keys[i]);
    }
  }
};

cvCluster.prototype.removeListener = function(type, listener_id) {
	if (this._listeners[type] instanceof Object){
    if(typeof this._listeners[type][listener_id] != "undefined") {
      delete this._listeners[type][listener_id];
    }
  }
};


cvCluster.prototype.save = function() {
  var data = "++++BEGIN CLUSTER++++\n";
  data += "url = '" + this.url + "';\n";
  data += "name = '" + this.name.replace(/'/, "\\'") + "';\n";
  var file = this.certificate;
  if(dock.getDirectorySeparator() == "\\") {
    file = file.replace(/([^\\])\\([^\\])/g, "$1\\\\$2");
  }
  data += "certificate = '" + file + "';\n";
  data += "username = '" + this.username + "';\n";
  data += "password = '" + this.password.replace(/'/, "\\'") + "';\n";
  data += "autoconnect = " + this.autoconnect + ";\n";
  data += "key = " + this.key + ";\n";
  data += "followRedirect = " + this.followRedirect + ";\n";
  data += "brightVersion = '" + this.brightVersion + "';\n";
  if(isArray(this.treeClosed)) {
    data += "treeClosed = " + this.treeClosed.toSource() + ";\n";
  } else {
    data += "treeClosed = " + this.treeClosed + ";\n";
  }
  if(isArray(this.rackMetrics)) {
    data += "rackMetrics = " + this.rackMetrics.toSource() + ";\n";
  } else {
    data += "rackMetrics = " + this.rackMetrics + ";\n";
  }
  if(isArray(this.mainMetric)) {
    data += "mainMetric = " + this.mainMetric.toSource() + ";\n";
  } else {
    data += "mainMetric = " + this.mainMetric + ";\n";
  }
  if(isArray(this.cephMetric)) {
    data += "cephMetric = " + this.cephMetric.toSource() + ";\n";
  } else {
    data += "cephMetric = " + this.cephMetric + ";\n";
  }
  if(isArray(this.hadoopMetric)) {
    data += "hadoopMetric = " + this.hadoopMetric.toSource() + ";\n";
  } else {
    data += "hadoopMetric = " + this.hadoopMetric + ";\n";
  }
  data += "++++END CLUSTER++++\n";
  return data;
}

cvCluster.prototype.load = function(data) {
  if(data[0] != "++++BEGIN CLUSTER++++") return false;
  var i = 1;
  while(i < data.length) {
    if(data[i] == "++++END CLUSTER++++") {
      data.splice(0, i + 1);
      this.setUrl(this.url);
      return true;
    }
    try {
      eval("this." + data[i]);
    } catch(err) {
      println("Error reading cluster.dat: " + err);
    }
    i++;
  }
  return false;
}

cvCluster.prototype.setUrl = function(url) {
  url = parseUrl(url);
  this.protocol = url[0].toLowerCase();
  this.host = url[1];
  this.port = url[2];
  this.url = this.protocol + "://" + this.host + ":" + this.port;
}

cvCluster.prototype.usernameDialog = function(question) {
  var args = new Object();
  args['title'] = 'Login: ';
  if(this.name.length > 0) {
    args['title'] += this.name;
  } else {
    args['title'] += this.host + ":" + this.port;
  }
  args['cluster'] = this;
  args['question'] = question;
  var d = new cvDialog("password.xul", args);
  d.onOk = function(cluster, username) {
    cluster.username = username;
  }
  d.dialogOptions = 'chrome,extrachrome,resizable,modal,centerscreen';
  dialogs.showDialog(d);
}

cvCluster.prototype.passwordDialog = function(question) {
  var args = new Object();
  args['title'] = 'Login: ';
  if(this.name.length > 0) {
    args['title'] += this.name;
  } else {
    args['title'] += this.host + ":" + this.port;
  }
  args['cluster'] = this;
  args['question'] = question;
  args['askpwd'] = true;
  var d = new cvDialog("password.xul", args);
  d.onOk = function(cluster, password) {
    cluster.temppassword = password;
  }
  d.onCancel = function(cluster) {
    cluster.logincanceled = true;
  }
  d.dialogOptions = 'chrome,extrachrome,resizable,modal,centerscreen';
  dialogs.showDialog(d);
}

cvCluster.prototype.connect = function() {

  // we pressed "Cancel" in the password popup
  if (this.logincanceled) {
    this.logincanceled = false;
    return false;
  }

  if ((this.username.length == 0) && (this.certificate.length > 0)) {
    dock.displayError('Certificate authentication is not supported in this version of CMGui');
    return false;
  }

  if (this.username.length == 0) {
    this.usernameDialog('Username: ');
  }

  fixSSLCertificateException(this.url);

  if ((this.temppassword.length == 0) && (this.password.length > 0)) {
    this.temppassword = this.password;
  }

  var login = json_login(this.url, this.username, this.temppassword, this.temptoken, this.followRedirect);
  if (login[0]) {
    println('connecting');
    if (login[3]) {
      println("was redirected to " + login[3]);
      this.actualurl = login[3];
      println("following from " + this.url + " to " + this.actualurl);
    } else {
      this.actualurl = this.url;
    }
    this.temppassword = ''; // should be cleaned every time
    this.connect2();
  } else if (login[2] && (login[2].ret == 3)) { // challenge sent
    println('login[2]' + login[2]);
    var challenge = login[2].challenge;
    var token = login[2].token;
    this.temptoken = token;
    this.passwordDialog(challenge);
    this.connect();
    this.temptoken = '';
  } else {
    if (login[1] || login[2]) { // cmd answered
      var message = "Unable to login to host '" + this.host + ":" + this.port + "'.";
      if (login[1] && (login[1].length > 0)) {
        message += "\n\nMessage returned by cmdaemon:\n" + login[1];
      } else if (login[2]) {
        message += "\n\nAuthentication timed out.";
      }
    } else { // no answer from cmd
      var message = "Unable to connect to host '" + this.host + ":" + this.port + "'. Is cmdaemon running?";
    }
    displayError(message);
    dock.unwaitIcon();
    this.temppassword = '';
    this.connecting = false;
    return false;
  }
  return false;
}

cvCluster.prototype.connect2 = function() {
  if(this.connecting || this.connected) return false;

  this.connecting = true;
  dock.waitIcon();
  window.setTimeout(createObjectCallback(this, this.connect3), 1);
  return true;
}

cvCluster.prototype.reconnect = function() {
  if(this.connecting || !this.connected) return false;
  this.disconnect();
  this.connect2();
  return true;
}

cvCluster.prototype.connect3 = function() {
  this.reconnectTimeout = 0;
  this.forcedDisconnect = false;
  this.updateHealth = true;
  this.updateRack = true;
  this.licenseInfo = 0;

  // this.url is the url saved in preferences, so we connect ALWAYS to this.actualurl
  header(this.actualurl);

  this.syslogin = this.username;
  if(this.syslogin.length == 0) this.syslogin = 'root';

  if(this.connectcount == 0) dock.MAX_THREADS += dock.MAX_THREADS_PER_CLUSTER;
  this.connectcount++;

  var jsw = dock.workerpool.getWorker(this.actualurl);
  var va = jsw.call('cmmain', 'verifyAPI', ["cmgui", CMDAEMON_API_HASH], 0);

  var apicheck = true;
  if((va[0] != 0) || !va[1] || isObject(va[1])) {
    if(va[0] != 0) {
      displayError("Unable to connect to cluster " + this.host + ":" + this.port);
    } else if(isObject(va[1])) {
      displayError("Error from cluster: " + va[1].errormessage);
    } else {
      if(apicheck) {
        displayError("This cluster is running a different version of the cmdaemon");
      } else {
        displayError("This cluster is running a different version of the cmdaemon\nCMGui will continue, be careful committing objects which might have had extra parameters added to them.");
      }
    }
    if(apicheck) {
      this.services = new Object();
      header();
      dock.unwaitIcon();
      this.connecting = false;
      return false;
    }
  }

  this.brightVersion = dock.brightVersion;
  this.extraComplete = 0;

  dock.workerpool.getWorker(this.actualurl).call('cmmain', 'getActivePassiveKeys', null, function(code, data) {
      if(code == 0) {
        this.current.activePassiveKeys = data;
      } else {
        dock.jsonCallError(this, code, data);
      }
      this.current.extraComplete++;
    }, this);

  dock.workerpool.getWorker(this.actualurl).call('cmmain', 'getServerStatus', null, function(code, data) {
      if(code == 0) {
        var tmine = new Date();
        tmine = Math.floor(tmine.getTime() / 1000);
        var tmaster = data.myTime;
        this.current.timeOffset = tmaster - tmine;
        println("Time offset: " + this.current.timeOffset);
      } else {
        dock.jsonCallError(this, code, data);
      }
      this.current.extraComplete++;
    }, this);

  dock.workerpool.getWorker(this.actualurl).call('cmmain', 'getClusterSetup', null, function(code, data) {
      if(code == 0) {
        this.current.setup = data;
      } else {
        dock.jsonCallError(this, code, data);
      }
      this.current.extraComplete++;
    }, this);

  dock.workerpool.getWorker(this.actualurl).call('cmmon', 'getUserDefinedMetricTypes', null, function(code, data) {
      if(code == 0) {
        this.current.userDefinedMetricTypes = data;
      } else {
        dock.jsonCallError(this, code, data);
      }
      this.current.extraComplete++;
    }, this);

  dock.workerpool.getWorker(this.actualurl).call('cmmain', 'masterOnCloud', null, function(code, data) {
      if(code == 0) {
        this.current.masterOnCloud = data;
      } else {
        dock.jsonCallError(this, code, data);
      }
      this.current.extraComplete++;
    }, this);
  dock.workerpool.getWorker(this.actualurl).call('cmlustre', 'ieelInstalled', [true], function(code, data) {
      if(code == 0) {
        this.current.hasIEEL = data.isInstalled;
      } else {
        dock.jsonCallError(this, code, data);
      }
      this.current.extraComplete++;
    }, this);
  dock.workerpool.getWorker(this.actualurl).call('cmmain', 'getManagedFilePaths', [true], function(code, data) {
    if(code == 0) {
      var managedFilePaths = [];
      for (var i=0; i<data['FilePathExpression'].length; i++) {
        managedFilePaths.push({'regex': data['FilePathExpression'][i], 'manager': data['Manager'][i]});
      }
      this.current.managedFilePaths = managedFilePaths;
    } else {
      dock.jsonCallError(this, code, data);
    }
    this.current.extraComplete++;
  }, this);
  dock.workerpool.getWorker(this.actualurl).call('cmmain', 'getGlobalConfig', [true], function(code, data) {
      if(code == 0) {
        for(var i = 0; i < data.length; i++) {
          var idx = data[i].indexOf('=');
          if(idx > 0) {
            this.current.globalConfig[data[i].substr(0, idx)] = data[i].substr(idx + 1);
            println("Global config: " + data[i]);
          }
        }
      } else {
        println("Unable to get global config");
      }
      this.current.extraComplete++;
    }, this);

  header(undefined, undefined, '-');
  this.connected = true;
  println("Connected, filling local storage... ");
  addHistory("Connected, filling local storage...");
  var n = this.refresh();
  println("Local storage done. Got " + n + " objects.");

  var nodes = this.find('Node');
  var nodeKeys = new Array();
  for(var i = 0; i < nodes.length; i++) {
    nodeKeys.push(nodes[i].get('uniqueKey'));
  }
  dock.workerpool.getWorker(this.actualurl).call('cmdevice', 'getSysInfoCollectors', [nodeKeys], function(code, data) {
      if(code == 0) {
        this.current.setAllSysInfo(nodes, data);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, this);
  dock.workerpool.getWorker(this.actualurl).call('cmdevice', 'getLiveUCS', [nodeKeys, false], function(code, data) {
      if(code == 0) {
        this.current.setAllLiveUCSInfo(nodes, data);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, this);
  dock.workerpool.getWorker(this.actualurl).call('cmdevice', 'getLiveDellSettings', [nodeKeys, false], function(code, data) {
      if(code == 0) {
        this.current.setAllLiveDellSettingsInfo(nodes, data);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, this);

  if(this.hasCeph) {
    dock.workerpool.getWorker(this.actualurl).call('cmceph', 'requestState', null, function(code, data) {
        if(code == 0) {
          this.current.setCephState(data);
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, this);
  }

  if((this.name == '') && (this.treeClosed.length == 0)) {
    var racks = this.find('Rack');
    for(var i = 0; i < racks.length; i++) {
      this.treeClosed.push('Rack_' + racks[i].get('uniqueKey'));
    }
    var groups = this.find('NodeGroup');
    for(var i = 0; i < groups.length; i++) {
      this.treeClosed.push('NodeGroup_' + groups[i].get('uniqueKey'));
    }
    var chassis = this.find('Chassis');
    for(var i = 0; i < chassis.length; i++) {
      this.treeClosed.push('Chassis_' + chassis[i].get('uniqueKey'));
    }
  }

  while(this.extraComplete != 8) {
    mainthread.processNextEvent(true);
  }

  this.createTree();

  this.eventworker = dock.workerpool.getWorker(this.actualurl, 'eventworker.js');
  this.eventworker.current = this;
  this.eventworker.createWorker();
  this.eventworker.callback = function(code, data) {
    if(isObject(data) && data.hasOwnProperty('errormessage')) code = 100;
    if((code == 0) && typeof(data) == 'number') {
      this.current.sessionid = data;
      println("Registering session ok. (" + this.current.sessionid + ")");
      this.postMessage(['-']);
    } else if((code == 0) && isArray(data)) {
      var good = true;
      this.current.failedEventCalls = 0;
      for(var i = 0; i < data.length; i++) {
        if(isObject(data[i])) {
          var latest = 0;
          if(i == data.length - 1) {
            latest = data.length - 1;
          } else {
            latest = -1;
          }
//           printall(data[i]);
//           println(" ++ latest: " + latest);
          good = this.current.processEvent(data[i], latest) && good;
        } else {
          println('None event:');
          printall(data[i]);
          good = false;
        }
      }
//       println("Got " + data.length + " events, good = " + good);
      refreshEventViewers();
      if(good) {
        if(this.current.sessionid == 0) {
          this.postMessage(['SESSION', 4]); // Type 4: cmgui
        } else {
          window.setTimeout(createObjectCallback(this, this.postMessage, ['-']), 1); // Otherwise remains in the same timeloop
        }
      } else {
        println('Event thread ended (1)' + this.current.stopping);
        if(!this.current.stopping) {
          this.current.missedEvent();
        }
      }
    } else {
      if(code == 0) {
        println('Event thread didn\'t receive event list but: ' + typeof(data));
      } else if(code == 635) {
        this.current.missedEvent(true);
      } else {
        println('Event thread, error code: ' + code);
      }
      printall(data);
      // We get in this branch in case an XmlHttpRequest fails or times out due to a slow network.
      // We have to call postMessage() with '-' because this will have the worker issue a jsoncall to:
      //   cmsession::waitForEvents(sessionid, eventWaitTime)
      // This makes sure it doesn't stop completely after this failure.
      this.postMessage(['-']);

      this.current.failedEventCalls++;
      println('Event thread failed: ' + this.current.failedEventCalls + ' / ' + dock.MAX_FAILED_EVENTS);
      if(this.current.failedEventCalls > dock.MAX_FAILED_EVENTS) {
        println('Event thread ended (2) ' + this.current.stopping);
        if(!this.current.stopping) {
          this.current.missedEvent();
        }
      }
    }
  }
  this.eventworker.postMessage(['LAST', dock.generalSettings.eventCache]);

  this.getStatusThread();
  header();
  this.connected = true;
  this.addMonitoringMenu();
  this.select(0);
  this.connecting = false;
  dock.unwaitIcon();

  if(dock.generalSettings.newVersionNag) {
    this.checkNewVersion();
  }
  
  return true;
}

cvCluster.prototype.checkNewVersion = function() {
  dock.workerpool.getWorker(this.actualurl).call('cmmain', 'cmguiVersion', null, function(code, data) {
    if(code == 0) {
      println("Versions : " + CMGUI_VERSION);
      printall(data);
      if(CMGUI_VERSION < data.distVersion) {
        if(CMGUI_VERSION != data.clusterVersion) {
          dock.displayInfo("A newer distribution version of cmgui is available: r" + data.distVersion + ", current version: r" + CMGUI_VERSION);
        }
      } else if(CMGUI_VERSION < data.clusterVersion) {
        dock.displayInfo("A newer version of cmgui is available: r" + data.clusterVersion + ", current version: r" + CMGUI_VERSION);
      }
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, this);
}

cvCluster.prototype.logout = function(forced, disconnect) {
  if(this.connecting) return false;

  forced = isundefined(forced, false);
  disconnect = isundefined(disconnect, false);
  if(this.sessionid) {
    var n = 0;
    for(var key in this.objects) {
      if(this.objects[key].get('modified') || this.objects[key].get('toBeRemoved')) {
        n += 1;
      }
    }
    if((n > 0) && !forced) {
      return false;
    } else {
      var done = false;
      println("Unregistering session ... ");
      this.stopping = true;
      this.extraComplete = 0;
      dock.workerpool.getWorker(this.actualurl).call('cmsession', 'unregisterSession', this.sessionid, function(code, data) {
          this.current.sessionid = 0;
          if(code == 0) {
            println("unregister: ok");
            done = data;
          } else {
            dock.jsonCallError(this, code, data);
          }
          this.current.stopping = false;
          if((code != 0) || !data && force) data = this.current.disconnect(true);
          this.current.extraComplete++;
        }, this);
      var t0 = Date.now();
      while(this.extraComplete < 1) {
        mainthread.processNextEvent(true);
        if(Date.now() - t0 > 2000) {
          println("unregister: timeout");
          this.stopping = false;
          this.disconnect(true);
          break;
        }
      }
      // KDR: timing constrained, event kills all workers
//       if(done) {
//         this.extraComplete = 0;
//         dock.workerpool.getWorker(this.actualurl).call('logout', '', null, function(code, data) {
//             if(code == 0) {
//               println("logout: ok");
//             } else {
//               dock.jsonCallError(this, code, data);
//             }
//             this.current.extraComplete++;
//           }, this, true);
//         while(this.extraComplete < 1) {
//           mainthread.processNextEvent(true);
//         }
//       }
      return done;
    }
  } else if(disconnect) {
    return true;
  } else {
    return this.disconnect(true);
  }
}

cvCluster.prototype.connectionLost = function() {
  this.disconnect(true, true);
  if(dock.generalSettings.autoReconnect) {
    if(!this.reconnecting) {
      this.reconnecting = true;
      var args = new Object();
      args['cluster'] = this;
      var d = new cvDialog("autoreconnect.xul", args);
      dialogs.showDialog(d);
    }
    return true;
  }
  return false;
}

cvCluster.prototype.callFailed = function() {
  if(dock.generalSettings.badConnection) {
    println("+++ Warning: a called failed. Bad connection on, continue. +++");
    return false;
  } else {
    this.disconnect(true, true);
    return true;
  }
}

cvCluster.prototype.missedEvent = function(forced) {
  forced = isundefined(forced, false);
  if(dock.generalSettings.badConnection && !forced) {
    println("+++ Warning: a called failed. Bad connection on, continue. +++");
  } else {
    this.disconnect(true, true);
  }
}

cvCluster.prototype.disconnect = function(forced, event) {
  this.forcedDisconnect = forced;
  if(forced) this.select(-1);
  event = isundefined(event, false);
  if(!this.connecting && this.connected) {
    header();
    if(this.url == this.actualurl) {
      println("Disconnecting from cluster " + this.actualurl + " ...");
    } else {
      println("Disconnecting from cluster " + this.actualurl + " / " + this.url + " ...");
    }
    if(!event) this.logout(true, true);
    this.connected = false;
    this.clearTree();
    this.objects = 0;
    this.sessionid = 0;
    this.LDAPstatus = new Object();
    saveClusters();
    println("done.");

    if((typeof(currentObject) == 'undefined') || (currentObject == this) || (typeof(currentObject.cluster) == 'undefined') || (currentObject.cluster == this)) this.select(-1);
    this.removeMonitoringMenu();
    if(!forced) this.select(-1);

    header();
  }
  if(!this.stopping) dock.workerpool.stop(this.actualurl);
  this.eventworker = 0;

  return true;
}

/* Fill the objectname -> service map */
cvCluster.prototype.findService = function(call) {
  if(!this.serviceMap) {
    this.serviceMap = {"getDevices" : "cmdevice", "getDevice" : "cmdevice", "addDevice" : "cmdevice", "updateDevice" : "cmdevice", "validateDevice" : "cmdevice", "removeDevice" : "cmdevice", "queryRemoveDevice" : "cmdevice",
                       "getBackgroundTasks" : "cmmain", "getBackgroundTask" : "cmmain", "addBackgroundTask" : "cmmain", "updateBackgroundTask" : "cmmain", "validateBackgroundTask" : "cmmain", "removeBackgroundTask" : "cmmain", "queryRemoveBackgroundTask" : "cmmain",
                       "getConfigurationOverlays" : "cmdevice", "getConfigurationOverlay" : "cmdevice", "addConfigurationOverlay" : "cmdevice", "updateConfigurationOverlay" : "cmdevice", "validateConfigurationOverlay" : "cmdevice", "removeConfigurationOverlay" : "cmdevice", "queryRemoveConfigurationOverlay" : "cmdevice",
                       "getNodeGroups" : "cmdevice", "getNodeGroup" : "cmdevice", "addNodeGroup" : "cmdevice", "updateNodeGroup" : "cmdevice", "validateNodeGroup" : "cmdevice", "removeNodeGroup" : "cmdevice", "queryRemoveNodeGroup" : "cmdevice",
                       "getCategories" : "cmdevice", "getCategory" : "cmdevice", "addCategory" : "cmdevice", "updateCategory" : "cmdevice", "validateCategory" : "cmdevice", "removeCategory" : "cmdevice", "queryRemoveCategory" : "cmdevice",
                       "getNetworks" : "cmnet", "getNetwork" : "cmnet", "addNetwork" : "cmnet", "updateNetwork" : "cmnet", "validateNetwork" : "cmnet", "removeNetwork" : "cmnet", "queryRemoveNetwork" : "cmnet",
                       "getPartitions" : "cmpart", "getPartition" : "cmpart", "addPartition" : "cmpart", "updatePartition" : "cmpart", "validatePartition" : "cmpart", "removePartition" : "cmpart", "queryRemovePartition" : "cmpart",
                       "getFSParts" : "cmprov", "getFSPart" : "cmprov", "addFSPart" : "cmprov", "updateFSPart" : "cmprov", "validateFSPart" : "cmprov", "removeFSPart" : "cmprov", "queryRemoveFSPart" : "cmprov",
                       "getRacks" : "cmpart", "getRack" : "cmpart", "addRack" : "cmpart", "updateRack" : "cmpart", "validateRack" : "cmpart", "removeRack" : "cmpart", "queryRemoveRack" : "cmpart",
                       "getSoftwareImages" : "cmpart", "getSoftwareImage" : "cmpart", "addSoftwareImage" : "cmpart", "updateSoftwareImage" : "cmpart", "validateSoftwareImage" : "cmpart", "removeSoftwareImage" : "cmpart", "queryRemoveSoftwareImage" : "cmpart",
                       "getUsers" : "cmuser", "getUser" : "cmuser", "addUser" : "cmuser", "updateUser" : "cmuser", "validateUser" : "cmuser", "removeUser" : "cmuser", "queryRemoveUser" : "cmuser",
                       "getGroups" : "cmuser", "getGroup" : "cmuser", "addGroup" : "cmuser", "updateGroup" : "cmuser", "validateGroup" : "cmuser", "removeGroup" : "cmuser", "queryRemoveGroup" : "cmuser",
                       "getCMServices" : "cmauth", "getCMService" : "cmauth", "addCMService" : "cmauth", "updateCMService" : "cmauth", "validateCMService" : "cmauth", "removeCMService" : "cmauth", "queryRemoveCMService" : "cmauth",
                       "getProfiles" : "cmauth", "getProfile" : "cmauth", "addProfile" : "cmauth", "updateProfile" : "cmauth", "validateProfile" : "cmauth", "removeProfile" : "cmauth", "queryRemoveProfile" : "cmauth",
                       "getJobQueues" : "cmjob", "getJobQueue" : "cmjob", "addJobQueue" : "cmjob", "updateJobQueue" : "cmjob", "validateJobQueue" : "cmjob", "removeJobQueue" : "cmjob", "queryRemoveJobQueue" : "cmjob",
                       "getMetrics" : "cmmon", "getMetric" : "cmmon", "addMetric" : "cmmon", "updateMetric" : "cmmon", "validateMetric" : "cmmon", "removeMetric" : "cmmon", "queryRemoveMetric" : "cmmon",
                       "getHealthChecks" : "cmmon", "getHealthCheck" : "cmmon", "addHealthCheck" : "cmmon", "updateHealthCheck" : "cmmon", "validateHealthCheck" : "cmmon", "removeHealthCheck" : "cmmon", "queryRemoveHealthCheck" : "cmmon",
                       "getThreshActions" : "cmmon", "getThreshAction" : "cmmon", "addThreshAction" : "cmmon", "updateThreshAction" : "cmmon", "validateThreshAction" : "cmmon", "removeThreshAction" : "cmmon", "queryRemoveThreshAction" : "cmmon",
                       "getHadoopHDFSs" : "cmhadoop", "getHadoopHDFS" : "cmhadoop", "addHadoopHDFS" : "cmhadoop", "updateHadoopHDFS" : "cmhadoop", "validateHadoopHDFS" : "cmhadoop", "removeHadoopHDFS" : "cmhadoop", "queryRemoveHadoopHDFS" : "cmhadoop",
                       "getCephs" : "cmceph", "getCeph" : "cmceph", "addCeph" : "cmceph", "updateCeph" : "cmceph", "validateCeph" : "cmceph", "removeCeph" : "cmceph", "queryRemoveCeph" : "cmceph",
                       "getCephOSDPools" : "cmceph", "getCephOSDPool" : "cmceph", "addCephOSDPool" : "cmceph", "updateCephOSDPool" : "cmceph", "validateCephOSDPool" : "cmceph", "removeCephOSDPool" : "cmceph", "queryRemoveCephOSDPool" : "cmceph",
                       "getPuppets" : "cmpuppet", "getPuppet" : "cmpuppet", "addPuppet" : "cmpuppet", "updatePuppet" : "cmpuppet", "validatePuppet" : "cmpuppet", "removePuppet" : "cmpuppet", "queryRemovePuppet" : "cmpuppet",
                       "getKubeClusters" : "cmkube", "getKubeCluster" : "cmkube", "addKubeCluster" : "cmkube", "updateKubeCluster" : "cmkube", "validateKubeCluster" : "cmkube", "removeKubeCluster" : "cmkube", "queryRemoveKubeCluster" : "cmkube",                       
                       "getOpenStacks" : "cmopenstack", "getOpenStack" : "cmopenstack", "addOpenStack" : "cmopenstack", "updateOpenStack" : "cmopenstack", "validateOpenStack" : "cmopenstack", "removeOpenStack" : "cmopenstack", "queryRemoveOpenStack" : "cmopenstack", "getOpenStackTenants" : "cmopenstack",
                       "getMonConfs" : "cmmon", "getMonConfByKeys": "cmmon", "getMonConf" : "cmmon", "addMonConf" : "cmmon", "updateMonConf" : "cmmon", "validateMonConf" : "cmmon", "removeMonConf" : "cmmon", "queryRemoveMonConf" : "cmmon",
                       "getCloudJobDescriptions" : "cmcloud", "getCloudJobDescription" : "cmcloud", "addCloudJobDescription" : "cmcloud", "updateCloudJobDescription" : "cmcloud", "validateCloudJobDescription" : "cmcloud", "removeCloudJobDescription" : "cmcloud", "queryRemoveCloudJobDescription" : "cmcloud",
                       "getCloudProviders" : "cmcloud", "getCloudProvider" : "cmcloud", "addCloudProvider" : "cmcloud", "updateCloudProvider" : "cmcloud", "validateCloudProvider" : "cmcloud", "removeCloudProvider" : "cmcloud", "queryRemoveCloudProvider" : "cmcloud",
                       "getCloudTypes" : "cmcloud", "getCloudType" : "cmcloud", "addCloudType" : "cmcloud", "updateCloudType" : "cmcloud", "validateCloudType" : "cmcloud", "removeCloudType" : "cmcloud", "queryRemoveCloudType" : "cmcloud",
                       "getCloudImages" : "cmcloud", "getCloudImage" : "cmcloud", "addCloudImage" : "cmcloud", "updateCloudImage" : "cmcloud", "validateCloudImage" : "cmcloud", "removeCloudImage" : "cmcloud", "queryRemoveCloudImage" : "cmcloud",
                       "getCloudRegions" : "cmcloud", "getCloudRegion" : "cmcloud", "addCloudRegion" : "cmcloud", "updateCloudRegion" : "cmcloud", "validateCloudRegion" : "cmcloud", "removeCloudRegion" : "cmcloud", "queryRemoveCloudRegion" : "cmcloud",
                       "getCloudStaticIPs" : "cmcloud", "getCloudStaticIP" : "cmcloud" };
    for(var i = 0; i < this.openStackApiEntities.length; i++) {
      this.serviceMap['get' + this.openStackApiEntities[i]] = "cmopenstack";
      this.serviceMap['get' + this.openStackApiEntities[i] + 's'] = "cmopenstack";
      this.serviceMap['add' + this.openStackApiEntities[i]] = "cmopenstack";
      this.serviceMap['remove' + this.openStackApiEntities[i]] = "cmopenstack";
      this.serviceMap['update' + this.openStackApiEntities[i]] = "cmopenstack";
      this.serviceMap['queryRemove' + this.openStackApiEntities[i]] = "cmopenstack";
      this.serviceMap['validate' + this.openStackApiEntities[i]] = "cmopenstack";
    }
  }
  if(this.serviceMap.hasOwnProperty(call)) return this.serviceMap[call];
  println("Unknown call: " + call);
  return '';
}

/* Refresh objects
 * ()             -> refresh all
 * (key)          -> refresh key (existing object only)
 * (key, type)    -> refresh key (new object)
 * (getDevices)   -> refresh all devices
 * (name, type)   -> get name from service
 */
cvCluster.prototype.refresh = function(what, type, arg, arg2) {
  if(!!what) {
    if(isNumeric(what)) {
      var key = parseInt(what);
      if(!!this.objects[key]) {
        var baseType = this.objects[key].get('baseType');
        this.hasCeph = this.hasCeph || (baseType == 'Ceph');
        this.hasHadoop = this.hasHadoop || (baseType == 'HadoopHDFS');
        this.hasOpenStack = this.hasOpenStack || (baseType == 'OpenStack');
        this.hasPuppet = this.hasPuppet || (baseType == 'Puppet');
        this.hasKube = this.hasKube || (baseType == 'KubeCluster');
        return this.objects[key].refresh();
      } else {
        var call = 'get' + firstUpper(type);
        var service = this.findService(call);
        if(service) {
          var args;
          var plus;
          if(type == 'BackgroundTask' || startswith(type, 'OpenStackApi')) {
            plus = 0;
          } else {
            plus = '';
          }
          if(typeof(arg) == 'undefined') {
            args = plus + key;
          } else if(typeof(arg2) == 'undefined') {
            args = [arg, plus + key];
          } else {
            args = [arg, arg2, plus + key];
          }
          dock.workerpool.getWorker(this.actualurl).call(service, call, args, function(code, data) {
              if(code == 0) {
                if(data == null) {
                  println('The RPC ' + call + ' returned null for: ' + args);
                } else {
                  var old = this.current.hasOwnProperty(key);
                  this.current.hasCeph = this.current.hasCeph || (data.baseType == 'Ceph');
                  this.current.hasHadoop = this.current.hasHadoop || (data.baseType == 'HadoopHDFS');
                  this.current.hasOpenStack = this.current.hasOpenStack || (data.baseType == 'OpenStack');
                  this.current.hasPuppet = this.current.hasPuppet || (data.baseType == 'Puppet');
                  this.current.hasKube = this.current.hasKube || (data.baseType == 'KubeCluster');
                  this.current.addObject(data, false);
                  if(type == 'Device' && !old) {
                    this.current.objects[key].updateStatus();
                  }
                }
              } else {
                dock.jsonCallError(this, code, data);
              }
            }, this);
          return 1;
        } else {
          displayError('Service not found: ' + call);
        }
        return 0;
      }
    } else if(!!type) {
      println(" +++++++++++++++ TODO(1) ++++++++++++");
//       var call = 'get' + firstUpper(type);
//       var service = this.findService(call);
//       if(service) {
//         var obj;
//         if(typeof(arg) == "undefined") {
//           obj = service[call](what);
//         } else {
//           obj = service[call](arg, what);
//         }
//         if(!!obj) {
//           this.objects[objs[i].uniqueKey] = new cvObject(this, obj);
//           return 1;
//         }
//       } else {
//         displayError('Service not found: ' + call);
//       }
//       return 0;
    } else {
      var call = what;
      var service = this.findService(call);
      if(service) {
        var args = null
        if(typeof(arg2) != "undefined") {
          args = [arg, arg2];
        } else if(typeof(arg) != "undefined") {
          if(isArray(arg)) {
            args = [arg];
          } else {
            args = arg;
          }
        }
        dock.workerpool.getWorker(this.actualurl).call(service, call, args, function(code, data) {
            if(code == 0) {
              for(var i = 0; i < data.length; i++) {
                this.current.hasCeph = this.current.hasCeph || (data[i].baseType == 'Ceph');
                this.current.hasHadoop = this.current.hasHadoop || (data[i].baseType == 'HadoopHDFS');
                this.current.hasOpenStack = this.current.hasOpenStack || (data[i].baseType == 'OpenStack');
                this.current.objects[data[i].uniqueKey] = new cvObject(this.current, data[i]);
              }
            } else {
              dock.jsonCallError(this, code, data);
            }
            this.current.complete++;
            this.current.completed.push(what);
          }, this);
        return 1;
      } else {
        displayError('Service not found: ' + call);
      }
      return 0;
    }
  } else {
    this.objects = new Object();
    this.completed = new Array();
    this.complete = 0;
    var types = ['getPartitions', 'getDevices', 'getCategories', 'getNetworks', 'getNodeGroups', 'getGroups', 'getUsers', 'getSoftwareImages',
                 'getMetrics', 'getHealthChecks', 'getThreshActions', 'getCMServices', 'getProfiles', 'getRacks', 'getJobQueues',
                 'getHadoopHDFSs', 'getCephs', 'getOpenStacks', 'getBackgroundTasks', 'getConfigurationOverlays', 'getPuppets', 'getKubeClusters',
                 'getCloudProviders', 'getCloudTypes', 'getCloudImages', 'getCloudRegions', ['getCloudStaticIPs', 0, 0]];
    if(!dock.generalSettings.delayedMonConfLoad) {
      types.push('getMonConfs');
    }
    for(var i = 0; i < this.openStackApiEntities.length; i++) {
      types.push(['get' + this.openStackApiEntities[i] + 's', 0]);
    }
    println("Fetching data for " + types.length + " different types of entities");
    var simpleTypes = new Array();
    for(var i = 0; i < types.length; i++) {
      if(isArray(types[i])) {
        if(types[i].length >= 3) {
          this.refresh(types[i][0], false, types[i][1], types[i][2]);
        } else if(types[i].length >= 2) {
          this.refresh(types[i][0], false, types[i][1]);
        } else {
          this.refresh(types[i][0]);
        }
        simpleTypes.push(types[i][0]);
      } else {
        this.refresh(types[i]);
        simpleTypes.push(types[i]);
      }
    }
    var start = new Date();
    start = start.getTime();
    var llt = 0;
    while(this.complete != types.length) {
      mainthread.processNextEvent(true);
      var now = new Date();
      now = now.getTime();
      var c = Math.floor((now - start) / 1000);
      if(c > llt && (c % 10) == 0) {
        println("Get all taking a long time: " + c + ", complete = " + this.complete + "/" + types.length + ", left:");
        var todo = simpleTypes.diff(this.completed);
        printall(todo);
        llt = c;
        if(dock.generalSettings.maxWaitForAll && (c > dock.generalSettings.maxWaitForAll)) {
          println("++ Warning, waited long enough, some entities will not be loaded!! ++");
          break;
        }
      }
    }
    var count = 0;
    for (var k in this.objects) {
      if (this.objects.hasOwnProperty(k)) {
        ++count;
      }
    }
    return count;
  }
  return 0;
}

cvCluster.prototype.getStatusThread = function() {
  var devices = this.allKeys('Device');
  dock.workerpool.getWorker(this.actualurl).call('cmdevice', 'getStatusForDeviceArray', [devices], function(code, data) {
      if(code == 0) {
        for(var i = 0; i < data.length; i++) {
          this.current.objects[devices[i]].stateString(data[i]);
        }
        println("Retrieve status for all devices [" + devices.length + "]:[" + data.length + "] (threaded) ... done");
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, this);
}

cvCluster.prototype.modified = function(what) {
  var list = new Array();
  if(!!what) {
    for(var obj in this.objects) {
      if((this.objects[obj].object.baseType == what) || (this.objects[obj].object['is' + what])) {
        if(this.objects[obj].get('modified') || this.objects[obj].get('toBeRemoved')) {
          list.push(this.objects[obj]);
        }
      }
    }
  } else {
    for(var obj in this.objects) {
      if(this.objects[obj].get('modified') || this.objects[obj].get('toBeRemoved')) {
        list.push(this.objects[obj]);
      }
    }
  }
  return list;
}

cvCluster.prototype.findNotToBeRemoved = function(what) {
  var list = this.find(what);
  var i = 0;
  while(i < list.length) {
    if(list[i].get('toBeRemoved')) {
      list.splice(i, 1);
    } else {
      i++;
    }
  }
  return list;
}

cvCluster.prototype.typeMap = function(what) {
  var r = new Object();
  for(var obj in this.objects) {
    var work = this.objects[obj];
    if(work.object == null) {
      println("Skipping object which is null (3)"); // TODO: remove here?
    } else {
      if(baseChildTypeCheck(work.object, what)) {
        r[obj] = work;
      }
    }
  }
  return r;
}

cvCluster.prototype.resolveTypeMap = function(what) {
  var r = new Object();
  for(var obj in this.objects) {
    var work = this.objects[obj];
    if(work.object == null) {
      println("Skipping object which is null (3)"); // TODO: remove here?
    } else {
      if((work.get('baseType') == what) || (work.object['is' + what])) {
        r[work.resolveName()] = work;
      }
    }
  }
  return r;
}

cvCluster.prototype.subFind = function(what, name) {
  var idx = what.indexOf('@');
  var r = new Array();
  if(idx > 0) {
    var s = what.split('@');
    var top;
    idx = s[0].indexOf(':');
    if(idx >= 0) {
      var f = this.find(s[0].substr(0, idx), s[0].substr(idx + 1));
      if(f) top = [f];
    } else {
      top = this.find(s[0]);
    }
    for(var i = 0; i < top.length; i++) {
      var work = top[i];
      for(var j = 1; j < s.length; j++) { // TODO: add real recursion
        var list = work.get(s[j]);
        if(!isArray(list)) {
          if(!list) continue;
          list = [list];
        }
        for(var k = 0; k < list.length; k++) {
          var one = new dock.cvObject(work.cluster, list[k], work, s[j]);
          r.push(one);
        }
      }
    }
  }
  if(!!name) {
    if(name && isNumeric(name)) name = parseInt(name);
    var match = false;
    if(typeof(what) == 'number') {
      for(var i = 0; i < r.length; i++) {
        var work = r[i];
        if(work.get('uniqueKey') == name) {
          match = work;
          break;
        }
      }
    } else {
      var min = (1 << 16);
      for(var i = 0; i < r.length; i++) {
        var work = r[i];
        if(work.object == null) {
          println("Skipping object which is null (1s)");
        } else {
          var found = inverseResolveName(work, this, name);
          if((found >= 0) && (min > found)) {
            min = found;
            match = work;
          }
        }
      }
    }
    return match;
  }
  return r;
}

cvCluster.prototype.findAttribute = function(keys, attribute) {
  var objects = this.find(keys);
  var result = [];
  for(var i = 0; i < objects.length; i++) {
    if(objects[i].get(attribute)) {
      result.push(objects[i].get(attribute));
    }
  }
  return result;
}

cvCluster.prototype.find = function(what, name, sorted) {
  sorted = isundefined(sorted, true);
  if(!!what) {
    if(!!name) {
      for(var obj in this.objects) {
        var work = this.objects[obj];
        if(work.object == null) {
          println("Skipping object which is null (1)"); // TODO: remove here?
        } else {
          var min = (1 << 16);
          var match;
          if(baseChildTypeCheck(work.object, what)) {
            var found = inverseResolveName(work, this, name);
            if((found >= 0) && (min > found)) {
              min = found;
              match = obj;
            }
          }
        }
      }
      if(match) return this.objects[match];
      return false;
    } else {
      if(isArray(what)) {
        var all = new Array();
        for(var i = 0; i < what.length; i++) {
          all = all.concat(this.find(what[i]));
        }
        return all;
      } else if(typeof(what) == 'number') {
        if(!!this.objects[what]) {
          return this.objects[what];
        } else {
          return false;
        }
      } else if(isNumeric(what)) {
        var key = parseInt(what);
        if((key > 0) && this.objects[key]) {
          return this.objects[key];
        } else {
          return false;
        }
      } else if((typeof(what) == 'string') && (what.indexOf('@') > 0)) {
        return this.subFind(what, name);
      } else {
        var objs = new Array();
        what = firstUpper(what);
        for(var obj in this.objects) {
          if(this.objects[obj].object == null) {
            println("Skipping object which is null (2)"); // TODO: remove here?
          } else {
            if(baseChildTypeCheck(this.objects[obj].object, what)) {
              objs.push(this.objects[obj]);
            }
          }
        }
        if(sorted) objs.sort(function(a,b){return b.resolveName() < a.resolveName()});
        return objs;
      }
    }
  }
  return false;
}

cvCluster.prototype.findBy = function(args) {
  if(typeof(args.baseType)==='string' && typeof(args.filter)==='function') {
    var objects = this.find(args.baseType);
    return objects.filter(args.filter)
  } else if(Array.isArray(args.baseType) && typeof(args.filter)==='function') {
    var keys = {};
    var objects = this.find(args.baseType);
    objects = objects.filter(function(item) {
      if(!keys.hasOwnProperty(item.object.uniqueKey)) {
        keys[item.object.uniqueKey] = true;
        return true;
      } else {
        return false;
      }
    });
    return objects.filter(args.filter)
  } else {
    throw 'ERROR: findBy with passed args not implemented';
  }
}

cvCluster.prototype.getKeyFor = function(relevantBaseType, find, parameter, newKey) {
  if(relevantBaseType == this.currentSubObjectType) {
    for(var i = 0; i < this.currentSubObjectList.length; i++) {
      if(this.currentSubObjectList[i][parameter] == find) {
        return this.currentSubObjectList[i].uniqueKey;
      }
    }
    return newKey;
  } else {
    for(var obj in this.objects) {
      if(this.objects[obj].relevantBaseType() == relevantBaseType) {
        if(this.objects[obj].get(parameter) == find) {
          return obj;
        }
      }
    }
  }
  return 0;
}

cvCluster.prototype.update = function(what) {
  var n = 0;
  if(!!what) {
    if(isNumeric(what)) {
      var key = parseInt(what);
      if(this.objects[key]) {
        this.objects[key].update();
      }
    }
  } else {
    for(var obj in this.objects) {
      if(obj.modified) {
        obj.update();
      }
    }
  }
  return n;
}

cvCluster.prototype.addObject = function(object, show, add) {
  show = isundefined(show, true);
  add = isundefined(add, true);
  var uniqueKey;
  if(object.hasOwnProperty('object')) {
    uniqueKey = object.get('uniqueKey');
    if(!uniqueKey) {
      uniqueKey = (++this.nextLocalKey);
      object.set('uniqueKey', uniqueKey);
      setUniqueKey(object.object, this);
      this.objects[uniqueKey] = object;
    }
  } else {
    uniqueKey = object.uniqueKey;
    if(uniqueKey && this.objects.hasOwnProperty(uniqueKey)) {
      this.objects[uniqueKey].object = object;
//       this.objects[uniqueKey].modified = object.modified;
//       this.objects[uniqueKey].toBeRemoved = object.toBeRemoved;
      object = this.objects[uniqueKey];
    } else if(uniqueKey > dock.MAX_LOCAL_KEY) {
      object = new cvObject(this, object);
      this.objects[uniqueKey] = object;
    } else {
      uniqueKey = (++this.nextLocalKey);
      object.uniqueKey = uniqueKey;
      object.modified = true;
      object.toBeRemoved = false;
      setUniqueKey(object, this);
      object = new cvObject(this, object);
      this.objects[uniqueKey] = object;
    }
  }
  if(add && object.displayInTree()) {
    this.addOneObject(object);
    if(object.get('baseType') == 'Device') {
      var rack = object.get('rack');
      if(rack) {
        var tmp = this.find(rack);
        if(tmp && tmp.branch) this.addOneObject(object, tmp.branch);
      }
    }
  }
  if(show) object.show();
  return object;
}

cvCluster.prototype.removeObject = function(key, arg1, arg2) {
  var current = this.objects[key];
  if(!!current) {
    current.remove(arg1, arg2);
    this.updateRack = this.updateRack || (current.get('baseType') == 'MonConf');
    return true;
  }
  return false;
}

cvCluster.prototype.replaceKey = function(oldKey, newKey) {
  for(var i in this.objects) {
    replaceKey(this.objects[i].object, oldKey, newKey, this);
  }
}

cvCluster.prototype.locate = function(what) {
  var type = what.get('childType');
  if((type == 'Chassis') || (type == 'EthernetSwitch') || (type == 'IBSwitch') || (type == 'MyrinetSwitch')) {
    var members = what.get('members');
    members.push(what.get('uniqueKey'));
    dock.rackBlinkKey = members;
    if(this.rack) this.rack.updateDeviceSelection(members);
    var tab = dock.document.getElementById('clustertab');
    tab.contentWindow.setTab('rackview');
    this.select(0);
    return;
  } else if(what.get('baseType') == 'NodeGroup') {
    var members = what.get('nodes');
    if(members.length > 0) {
      dock.rackBlinkKey = members;
      if(this.rack) this.rack.updateDeviceSelection(members);
      var tab = dock.document.getElementById('clustertab');
      tab.contentWindow.setTab('rackview');
      this.select(0);
      return;
    } else {
      dock.displayInfo('This node group contains no nodes');
    }
  }
  dock.rackBlinkKey = [what.get('uniqueKey')];
  if(this.rack) this.rack.updateDeviceSelection([what.get('uniqueKey')]);
  var tab = dock.document.getElementById('clustertab');
  tab.contentWindow.setTab('rackview');
  this.select(0);
}

function openTreeElement(clusters, parent) {
  if(!!parent) {
    var index = clusters.contentView.getIndexOfItem(parent);
    if((index < 0) || !clusters.contentView.isContainerOpen(index)) {
      openTreeElement(clusters, parent.parentNode.parentNode);
      if(index < 0) index = clusters.contentView.getIndexOfItem(parent);
      if((index >= 0) && !clusters.contentView.isContainerOpen(index)) clusters.contentView.toggleOpenState(index);
    }
  }
}

cvCluster.prototype.select = function(key, page) {
  dock.waitIcon();
  page = isundefined(page, '');
  var clusters = document.getElementById("treecluster");
  if(key < 0) {
    clusters.contentView.selection.select(0);
    currentTab.contentWindow.refresh();
  } else {
    var work = 0;
    if((typeof(key) != "undefined") && key) work = this.objects[key];
    if(isObject(work)) {
      if(work.treeitem) {
        println(" select " + key + " : " + work.resolveName());
        currentObject = work;
        if(!clusters.contentView.isContainerOpen(0)) clusters.contentView.toggleOpenState(0);
        work = work.treeitem.parentNode.parentNode;
        openTreeElement(clusters, work.parentNode.parentNode);
        var index = clusters.contentView.getIndexOfItem(work);
        if(index >= 0) {
          clusters.contentView.selection.clearSelection();
          clusters.contentView.selection.select(index);
        }
      } else {
        currentObject = this;
      }
    } else if(typeof(key) == 'string') {
      if(!clusters.contentView.isContainerOpen(0)) clusters.contentView.toggleOpenState(0);
      var branch = this.branch[key];
      if(!branch) branch = this.branch[key + 'Folder'];
      if(branch) {
        var index = clusters.contentView.getIndexOfItem(branch.parentNode);
        if(index >= 0) {
          clusters.contentView.selection.select(index);
          currentObject = 0;
        } else {
          var index = clusters.contentView.getIndexOfItem(branch);
          if(index >= 0) {
            clusters.contentView.selection.select(index);
            currentObject = 0;
          }
        }
      }
    } else {
      if(!clusters.contentView.isContainerOpen(0)) clusters.contentView.toggleOpenState(0);
      if(this.treeitem && this.treeitem.parentNode) {
        var index = clusters.contentView.getIndexOfItem(this.treeitem.parentNode.parentNode);
        if(index >= 0) {
          clusters.contentView.selection.select(index);
          currentObject = this;
        } else {
          clusters.contentView.selection.select(0);
          currentObject = 0;
        }
      } else {
        clusters.contentView.selection.select(0);
        currentObject = 0;
      }
    }
  }
  if(page.length) window.setTimeout('currentTab.contentWindow.setTab("' + page + '")', 50);
  dock.unwaitIcon();
}

cvCluster.prototype.clearTree = function() {
  if(this.treeitem) {
    var clusters = document.getElementById("clusters");
    this.treeClosed = new Array();
    for(var x in this.branch) {
      if(typeof(this.branch[x]) != 'undefined') {
        if(this.branch[x].parentNode.getAttribute('open') == 'false') {
          this.treeClosed.push(x);
        }
      }
    }
    for(var key in this.objects) {
      if(!!this.objects[key].branch) {
        if(this.objects[key].branch.parentNode.getAttribute('open') == 'false') {
          var name;
          if(!!this.objects[key].get('childType')) {
            name = this.objects[key].get('childType') + '_' + key
          } else {
            name = this.objects[key].get('baseType') + '_' + key
          }
          this.treeClosed.push(name);
        }
      }
    }
    var item = this.treeitem.parentNode.parentNode;
    clusters.removeChild(item);
    this.branch = 0;
    this.treeitem = 0;
  }
}

/* Call this to completly refresh the tree */
/* SINGLE PARTITION ONLY */
cvCluster.prototype.createTree = function() {
  var partitions = this.find('Partition');
  var networks = this.find('Network');
  var switches = this.find('Switch');
  var powerdistributionunits = this.find('PowerDistributionUnit');
  var nodes = this.find('Node');
  var chassis = this.find('Chassis');
  var gpuunits = this.find('GpuUnit');
  var categories = this.find('Category');
  var softwareimages = this.find('SoftwareImage');
  var nodegroups = this.find('NodeGroup');
  var racksensors = this.find('RackSensor');
  var genericdevices = this.find('GenericDevice');
  var racks = this.find('Rack');
  var cloudproviders = this.find('CloudProvider');
  var hadoophdfs = this.find('HadoopHDFS');
  var ieel = this.find('Ieel');

  this.base = this.getBasePartition();
  if(this.base) {
    var oldName = this.name;
    this.name =  this.base.get('clusterName');
    if(oldName != this.name) {
      saveClusters();
    }
    var id = "key://" + this.key;
    var tree = document.getElementById('clusterTree');
    var clusters = document.getElementById('clusters');
    if(!!this.branch) {
      println("Branch already set: " + this.branch);
      return true;
    } else {
      this.branch = new Object();
      [this.branch['cluster'], this.treeitem] = this.addClusterTreeElement(clusters, 'Cluster', 'Cluster_16.png', this.base.get('clusterName'), id, 1, '', 0, false);
    }
    // REBRANDING
    var treeswitches = rebranding(dock.TreeSwitches, 'Switches');
    var treenetworks = rebranding(dock.TreeNetworks, 'Networks');
    var treepdus = rebranding(dock.TreePowerDistributionUnits, 'Power Distribution Units');
    var treesi = rebranding(dock.TreeSoftwareImages, 'Software Images');
    var treecat = rebranding(dock.TreeCategories, 'Node Categories');
    var treehn = rebranding(dock.TreeHeadNodes, 'Head Nodes');
    var treerack = rebranding(dock.TreeRacks, 'Racks');
    var treechas = rebranding(dock.TreeChassis, 'Chassis');
    var treesmp = rebranding(dock.TreeVirtualSMPNodes, 'Virtual SMP Nodes');
    var treenode = rebranding(dock.TreeNodes, 'Nodes');
    var treecloud = rebranding(dock.TreeCloudNodes, 'Cloud Nodes');
    var treemic = rebranding(dock.TreeMICNodes, 'MIC Nodes');
    var treegpu = rebranding(dock.TreeGPUUnits, 'GPU Units');
    var treeotherdevices = rebranding(dock.TreeOtherDevices, 'Other Devices');
    var treeng = rebranding(dock.TreeNodeGroups, 'Node Groups');
    var treeuser = rebranding(dock.TreeUsersGroups, 'Users & Groups');
    var treework = rebranding(dock.TreeWorkload, 'Workload Management');
    var treemon = rebranding(dock.TreeMonitoring, 'Monitoring Configuration');
    var treeautho = rebranding(dock.TreeAuthorisation, 'Authorization');
    var treeauthe = rebranding(dock.TreeAuthentication, 'Authentication');
    var treehadoophdfs = rebranding(dock.TreeHadoopHDFS, 'Big Data');
    var treeopenstack = rebranding(dock.TreeOpenStack, 'OpenStack');
    var treeceph = rebranding(dock.TreeCeph, 'Ceph');
    var treeieel = rebranding(dock.TreeIeel, 'IEEL');
    var treepuppet = rebranding(dock.TreePuppet, 'Puppet');
    var treekube = rebranding(dock.TreePuppet, 'Kubernetes');
    var treedockerregistry = rebranding(dock.TreePuppet, 'Docker Registry')
    // END: REBRANDING

    if(treeswitches.length) this.branch['SwitchFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'SwitchFolder', 'FolderSwitch_16.png', treeswitches, id + '/Switch', 1, '', 0, false)[0];
    if(treenetworks.length) this.branch['NetworkFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'NetworkFolder', 'FolderNetwork_16.png', treenetworks, id + '/Network', 1, '', 0, false)[0];
    if(treepdus.length) this.branch['PowerDistributionUnitFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'PowerDistributionUnitFolder', 'FolderPowerSwitch_16.png', treepdus, id + '/PowerDistributionUnit', 1, '', 0, false)[0];
//   this.branch['partition'] = this.addClusterTreeElement(this.branch['cluster'], 'partitionFolder', 'Folder_16.png', 'Partitions', id + '/Partition', 1, '', 0, false)[0];
    if(treesi.length) this.branch['SoftwareImageFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'SoftwareImageFolder', 'FolderSoftware_16.png', treesi, id + '/SoftwareImage', 1, '', 0, false)[0];
    if(treecat.length) this.branch['CategoryFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'CategoryFolder', 'FolderCategory_16.png', treecat, id + '/Category', 1, '', 0, false)[0];
    if(treehn.length) this.branch['HeadNodeFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'HeadNodeFolder', 'FolderNode_16.png', treehn, id + '/MasterNode', 1, '', 0, false)[0];
    if(treerack.length) this.branch['RackFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'RackFolder', 'FolderRack_16.png', treerack, id + '/Rack', 1, '', 0, false)[0];
    if(treechas.length) this.branch['ChassisFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'ChassisFolder', 'FolderNode_16.png', treechas, id + '/Chassis', 1, '', 0, false)[0];
    if(treesmp.length) this.branch['VirtualSMPNodeFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'VirtualSMPNodeFolder', 'FolderNode_16.png', treesmp, id + '/VirtualSMPNode', 1, '', 0, false)[0];
    if(treenode.length) this.branch['NodeFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'NodeFolder', 'FolderNode_16.png', treenode, id + '/Node', 1, '', 0, false)[0];
//     this.branch['VirtualNodeFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'VirtualNodeFolder', 'FolderNode_16.png', 'Virtual Nodes', id + '/VirtualNode', 1, '', 0, false)[0];
    if(treecloud.length) this.branch['CloudNodeFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'CloudNodeFolder', 'FolderNode_16.png', treecloud, id + '/CloudNode', 1, '', 0, false)[0];
    if(treemic.length) this.branch['MICNodeFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'MICNodeFolder', 'FolderNode_16.png', treemic, id + '/MICNode', 1, '', 0, false)[0];
    if(treegpu.length) this.branch['GpuUnitFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'GpuUnitFolder', 'FolderNode_16.png', treegpu, id + '/GpuUnit', 1, '', 0, false)[0];
    if(treeotherdevices.length) this.branch['OtherDeviceFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'OtherDeviceFolder', 'FolderNode_16.png', treeotherdevices, id + '/GenericDevice', 1, '', 0, false)[0];
    if(treeng.length) this.branch['NodeGroupFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'NodeGroupFolder', 'FolderNodeGroup_16.png', treeng, id + '/NodeGroup', 1, '', 0, false)[0];
    if(treehadoophdfs.length) this.branch['HadoopHDFSFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'HadoopHDFSFolder', 'Folder_16.png', treehadoophdfs, id + '/HadoopHDFS', 1, '', 0, false)[0];
    if(treeceph.length) this.branch['Ceph'] = this.addClusterTreeElement(this.branch['cluster'], 'Ceph', 'Ceph_16.png', treeceph, id + '/Ceph', 2, '', 0, false)[0];
    if(treepuppet.length) this.branch['Puppet'] = this.addClusterTreeElement(this.branch['cluster'], 'Puppet', 'Puppet_16.png', treepuppet, id + '/Puppet', 2, '', 0, false)[0];
    if(treekube.length) this.branch['Kubernetes'] = this.addClusterTreeElement(this.branch['cluster'], 'Kubernetes', 'Kubernetes_16.png', treekube, id + '/Kubernetes', 2, '', 0, false)[0];
    if(treedockerregistry.length) this.branch['DockerRegistry'] = this.addClusterTreeElement(this.branch['cluster'], 'DockerRegistry', 'docker-logo_16.png', treedockerregistry, id + '/DockerRegistry', 2, '', 0, false)[0];
    // Disabled vnodes temporarily
    if(treeopenstack.length) this.branch['OpenStackFolder'] = this.addClusterTreeElement(this.branch['cluster'], 'OpenStackFolder', 'OpenStack_16.png', treeopenstack, id + '/OpenStack', 1, '', 0, false)[0];
    if(treeuser.length) this.branch['UserGroup'] = this.addClusterTreeElement(this.branch['cluster'], 'UserGroupSetup', 'UserGroup_16.png', treeuser, id + '/User', 2, '', 0, false)[0];
    if(treework.length) this.branch['Workload'] = this.addClusterTreeElement(this.branch['cluster'], 'WorkloadSetup', 'Workload_16.png', treework, id + '/Workload', 2, '', 0, false)[0];
    if(treemon.length) this.branch['Monitoring'] = this.addClusterTreeElement(this.branch['cluster'], 'MonitoringSetup', 'Monitoring_16.png', treemon, id + '/Monitoring', 2, '', 0, false)[0];
    if(treeautho.length) this.branch['Authorisation'] = this.addClusterTreeElement(this.branch['cluster'], 'AuthorisationSetup', 'Authorization_16.png', treeautho, id + '/Authorisation', 2, '', 0, false)[0];
    if(treeauthe.length) this.branch['Authentication'] = this.addClusterTreeElement(this.branch['cluster'], 'AuthenticationSetup', 'Authentication_16.png', treeauthe, id + '/Authentication', 2, '', 0, false)[0];
    if ( this.hasIEEL ) {
      if(treeieel.length) this.branch['IEEL'] = this.addClusterTreeElement(this.branch['cluster'], 'IEEL', 'StoragePair_16.png', treeieel, id + '/IEEL', 2, '', 0, false)[0];
    }

    dock.waitIcon();
    this.treeFillList = switches.concat(powerdistributionunits, networks, categories, softwareimages, racksensors, genericdevices, cloudproviders, nodes, nodegroups, chassis, gpuunits, racks, hadoophdfs);
    this.treeFillIndex = 0;
    this.treeFillBusy = true;
    this.treeFillTimer = window.setInterval(createObjectCallback(this, this.addObjectsThread), dock.FILL_TIMEOUT);

    return true;
  } else {
    displayError("This node is not running the active head node");
    this.connecting = false;
    this.disconnect(true);
    return false;
  }
}

cvCluster.prototype.findBranch = function(name) {
  for(var i in this.branch) {
    var b = this.branch[i];
    if(b) {
      if((b.getAttribute('id') == name)) {
        return b;
      } else if((b.parentNode.getAttribute('id') == name)) {
        return b.parentNode;
      }
    } else {
      println("No branch for: " + i);
    }
  }
  return false;
}

cvCluster.prototype.addObjectsThread = function() {
  println("addObjectsThread: " + this.treeFillIndex + " " + this.treeFillList.length);
  var n = 0;
  while((this.treeFillIndex < this.treeFillList.length) && (n < dock.FILL_COUNT)) {
    this.addOneObject(this.treeFillList[this.treeFillIndex]);
    this.treeFillIndex++;
    n++;
  }
  if(this.treeFillIndex >= this.treeFillList.length) {
    dock.unwaitIcon();
    window.clearInterval(this.treeFillTimer);
    this.treeFillTimer = 0;
    println("addObjectsThread: DONE: " + this.treeFillIndex);
    this.treeFillList = new Array(); // for reuse
    this.treeFillIndex = 0;
    this.treeFillBusy = false;
    if(this.treeFillFollowUp) {
      println("Calling tree fill follow up: " + this.treeFillFollowUp.length);
      for(var i = 0; i < this.treeFillFollowUp.length; i++) {
        this.treeFillFollowUp[i]();
      }
      this.treeFillFollowUp = 0;
    }
  }
}

cvCluster.prototype.addOneObject = function(object, parent, extra) {
  if(!parent && isArray(object) && (object.length == 2)) {
    parent = object[1];
    object = object[0];
  }
  if(!object.displayInTree()) return;

  extra = isundefined(extra, '');
  var type = object.get('baseType');
  var name = object.resolveName();
  
  var isChild = false;
  if(type == 'Device') {
    type = object.get('childType');
    var state = object.stateString(true);
    if(type == 'PhysicalNode') {
      if(dock.SCALEMPDEMO) state = 'Used';
      if(typeof(parent) == 'undefined') {
        parent = this.branch['NodeFolder'];
      } else {
        isChild = true;
      }
      var branch, treeitem;
      [branch, treeitem] = this.addClusterTreeElement(parent, "Node", "PhysicalNode-" + state + "_16.png", name, object.url, 0, extra);
      object.setTreeitem(treeitem, isChild);
      if(!isChild) object.branch = branch;
    } else if(type == 'VirtualSMPNode') {
      if(typeof(parent) == 'undefined') {
        parent = this.branch['VirtualSMPNodeFolder'];
        if(typeof(parent) == 'undefined') return;
      } else {
        isChild = true;
      }
      var children = 0;
      var primaryKey = 0;
      if(dock.SUBTREE && !isChild) {
        children = object.get('members');
        primaryKey = object.get('primaryNode');
      }
      if(isArray(children) && (object.branch == 0)) {
        [object.branch, object.treeitem] = this.addClusterTreeElement(parent, type, "VirtualSMPNode-" + state + "_16.png", name, object.url, 1, '', object.get('uniqueKey'));
        for(var i = 0; i < children.length; i++) {
          var child = object.cluster.find(children[i]);
          if(!!child) {
            var primary = '';
            if(children[i] == primaryKey) primary = ' (primary)';
            this.addOneObject(child, object.branch, primary);
          }
        }
      } else {
        var treeitem = this.addClusterTreeElement(parent, type, "VirtualSMPNode-" + state + "_16.png", name, object.url, 0)[1];
        object.setTreeitem(treeitem, isChild);
      }
    } else if(type == 'VirtualNode') {
      if(typeof(parent) == 'undefined') {
        parent = this.branch['OpenStackFolder'];
      } else {
        isChild = true;
      }
      // Disabled vnodes temporarily
      var treeitem = this.addClusterTreeElement(parent, type, "VirtualNode-" + state + "_16.png", name, object.url, 0)[1];
      object.setTreeitem(treeitem, isChild);
    } else if(type == 'CloudNode') {
      if(typeof(parent) == 'undefined') {
        var settings = object.get('cloudSettings');
        if(settings.childType == 'EC2Settings') {
          var p = settings.provider;
          p = object.cluster.find(p);
          if(!p) {
            println("Cloud provider for node " + settings.provider + " not found");
            return;
          }
          parent = p.branch;
          isChild = false;
        } else {
          println("Non EC2 nodes node supported yet");
          printall(settings);
        }
        if(!parent) return;
      } else {
        isChild = true;
      }
      var treeitem = this.addClusterTreeElement(parent, type, "CloudNode-" + state + "_16.png", name, object.url, 0)[1];
      object.setTreeitem(treeitem, isChild);
    } else if(type == 'MasterNode') {
      if(this.activePassiveKeys.length <= 1) {
        if(typeof(parent) == 'undefined') {
          parent = this.branch['HeadNodeFolder'];
          if(typeof(parent) == 'undefined') return;
        } else {
          isChild = true;
        }
        var treeitem = this.addClusterTreeElement(parent, "HeadNode", "MasterNode-" + state + "_16.png", name, object.url, 0)[1];
        object.setTreeitem(treeitem, isChild);
      } else {
        var activePassive = '';
        var i;
        for(i = 0; i < this.activePassiveKeys.length; i++) {
          if(this.activePassiveKeys[i] == object.get('uniqueKey')) {
            break;
          }
        }
        if(i == 0) {
          activePassive = ' (active)';
        } else if(i < this.activePassiveKeys.length) {
          activePassive = ' (passive)';
        }
        if(typeof(parent) == 'undefined') {
          parent = this.branch['HeadNodeFolder'];
          if(typeof(parent) == 'undefined') return;
        } else {
          isChild = true;
        }
        var treeitem = this.addClusterTreeElement(parent, "HeadNode", "MasterNode-" + state + "_16.png", name, object.url, 0, activePassive)[1];
        object.setTreeitem(treeitem, isChild);
      }
    } else if(type == 'MICNode') {
      if(typeof(parent) == 'undefined') {
//         var settings = object.get('micSettings');
//         var p = settings.host;
//         p = object.cluster.find(p);
//         if(!p) {
//           println("Host node for micnode " + settings.host + " not found");
//           return;
//         }
//         parent = p.branch;
        isChild = false;
        parent = this.branch['MICNodeFolder'];
      } else {
        isChild = true;
      }
      var treeitem = this.addClusterTreeElement(parent, type, "MICNode-" + state + "_16.png", name, object.url, 0)[1];
      object.setTreeitem(treeitem, isChild);
    } else if(type == 'PowerDistributionUnit') {
      if(typeof(parent) == 'undefined') {
        parent = this.branch['PowerDistributionUnitFolder'];
        if(typeof(parent) == 'undefined') return;
      } else {
        isChild = true;
      }
      var treeitem = this.addClusterTreeElement(parent, type, type + "-" + state + "_16.png", name, object.url, 0)[1];
      object.setTreeitem(treeitem, isChild);
    } else if((type == 'RackSensor') || (type == 'GenericDevice')) {
      if(typeof(parent) == 'undefined') {
        parent = this.branch['OtherDeviceFolder'];
        if(typeof(parent) == 'undefined') return;
      } else {
        isChild = true;
      }
      var treeitem = this.addClusterTreeElement(parent, type, type + "-" + state + "_16.png", name, object.url, 0)[1];
      object.setTreeitem(treeitem, isChild);
    } else if(type == 'GpuUnit') {
      if(typeof(parent) == 'undefined') {
        parent = this.branch['GpuUnitFolder'];
        if(typeof(parent) == 'undefined') return;
      } else {
        isChild = true;
      }
      var treeitem = this.addClusterTreeElement(parent, type, type + "-" + state + "_16.png", name, object.url, 0)[1];
      object.setTreeitem(treeitem, isChild);
    } else if(type == 'Chassis') {
      if(typeof(parent) == 'undefined') {
        parent = this.branch['ChassisFolder'];
        if(typeof(parent) == 'undefined') return;
      } else {
        isChild = true;
      }
      var children = 0;
      if(dock.SUBTREE && !isChild) {
        children = object.get('members');
      }
      if(isArray(children) && (object.branch == 0)) {
        [object.branch, object.treeitem] = this.addClusterTreeElement(parent, type, type + "-" + state + "_16.png", name, object.url, 1, '', object.get('uniqueKey'));
        for(var i = 0; i < children.length; i++) {
          var child = object.cluster.find(children[i]);
          if(!!child) {
            if(this.treeFillTimer && dock.SUBTREE_IN_THREAD) {
              this.treeFillList.push([child, object.branch]);
            } else {
              this.addOneObject(child, object.branch);
            }
          }
        }
      } else {
        var treeitem = this.addClusterTreeElement(parent, type, type + "-" + state + "_16.png", name, object.url, 0)[1];
        object.setTreeitem(treeitem, isChild);
      }
    } else {
      if(typeof(parent) == 'undefined') {
        parent = this.branch['SwitchFolder'];
        if(typeof(parent) == 'undefined') return;
      } else {
        isChild = true;
      }
      var children = 0;
      if(dock.SUBTREE && !isChild) {
        children = object.get('members');
      }
      if(isArray(children) && (object.branch == 0)) {
        [object.branch, object.treeitem] = this.addClusterTreeElement(parent, type, "Switch-" + state + "_16.png", name, object.url, 1, '', object.get('uniqueKey'));
        for(var i = 0; i < children.length; i++) {
          var child = object.cluster.find(children[i]);
          if(!!child) {
            if(this.treeFillTimer && dock.SUBTREE_IN_THREAD) {
              this.treeFillList.push([child, object.branch]);
            } else {
              this.addOneObject(child, object.branch);
            }
          }
        }
        if(children.length == 0) object.hideBranch();
      } else {
        var treeitem = this.addClusterTreeElement(parent, type, "Switch-" + state + "_16.png", name, object.url, 0)[1];
        object.setTreeitem(treeitem, isChild);
      }
    }
  } else if(type == 'CloudProvider') { // TODO-CLOUD: CloudProvider
    parent = this.branch['CloudNodeFolder'];
    isChild = false;
    if(!!parent) {
      var children = 0;
//       if(dock.SUBTREE && !isChild) {
//         children = object.cluster.findNodesIn(type, object.get('uniqueKey'));
//       }
      [object.branch, object.treeitem] = this.addClusterTreeElement(parent, 'CloudProvider', "Cloud_16.png", name, object.url, 1, '', object.get('uniqueKey'));
//       for(var i = 0; i < children.length; i++) {
//         var child = object.cluster.find(children[i]);
//         if(!!child) this.addOneObject(child, object.branch);
//       }
    } else {
      println("No such folder: " + type);
    }
  } else {
    if(typeof(parent) == 'undefined') {
      if(typeof(this.branch[type + 'Folder']) != 'undefined') {
        parent = this.branch[type + 'Folder'];
      }
    } else {
      isChild = true;
    }
    if(!!parent) {
      var itype = type;
      var children = 0;
      if(type == 'NodeGroup') {
        if(object.get('type') == 'STORAGEPAIR') {
          itype = 'StoragePair';
        }
      }
      if(dock.SUBTREE && !isChild) {
        if(type == 'NodeGroup') {
          children = object.get('nodes');
        } else if(type == 'Rack') {
          children = object.cluster.findNodesIn(type, object.get('uniqueKey'));
        }
      }
      var ctype = type;
      if((ctype == 'Category') && (object.get('childType') == 'MICNodeCategory')) ctype = 'MICCategory';
      if(isArray(children) && (object.branch == 0)) {
        [object.branch, object.treeitem] = this.addClusterTreeElement(parent, ctype, itype + "_16.png", name, object.url, 1, '', object.get('uniqueKey'));
        for(var i = 0; i < children.length; i++) {
          var child = object.cluster.find(children[i]);
          if(!!child) {
            if(this.treeFillTimer && dock.SUBTREE_IN_THREAD) {
              this.treeFillList.push([child, object.branch]);
            } else {
              this.addOneObject(child, object.branch);
            }
          }
        }
      } else {
        if(ctype == 'HadoopHDFS') {
          if(object.get('isSparkInstance')) {
            // ML: custom image for Spark (http://publicdomainvectors.org/en/free-clipart/Image-of-electricity-spark-orange-icon/31132.html)
	    itype = "HadoopSpark";
          } else if(object.get('isCassandraInstance')) {
	    // ML: custom image for Cassandra (http://publicdomainvectors.org/en/free-clipart/Blue-female-eye-vector-drawing/15344.html)
	    itype = "HadoopCassandra";
	  }
        }
        var treeitem = this.addClusterTreeElement(parent, ctype, itype + "_16.png", name, object.url, 0)[1];
        object.setTreeitem(treeitem, isChild);
      }
//     } else {
//       println("No such folder: " + type);
    }
  }
}

cvCluster.prototype.addClusterTreeElement = function(parent, property, image, text, globalID, container, state, key, sort) {
  state = isundefined(state, '');
  key = isundefined(key, 0);
  sort = isundefined(sort, true);
  var item = document.createElement('treeitem');
  item.setAttribute('id', globalID);
  item.setAttribute('property', property);
  if(key > 0) property += '_' + key;
  item.setAttribute('title', text);
  if(container) {
    item.setAttribute('container', 'true');
    if(isArray(this.treeClosed) && (this.treeClosed.find(property) >= 0)) {
      item.setAttribute('open', 'false');
    } else {
      item.setAttribute('open', 'true');
    }
  }
  var row = document.createElement('treerow');
  var cell = document.createElement('treecell');
  cell.setAttribute('src', 'chrome://cmgui/content/icons/' + image);
  cell.setAttribute('label', ' ' + text + state); // space between image and label, css doesn't work
  row.appendChild(cell);
  item.appendChild(row);

  var child;
  if(container == 1) {
    child = document.createElement('treechildren');
    child.setAttribute('id', text);
    item.appendChild(child);
  } else if(container == 2) {
    child = item;
  }
  var lt = parent.lastChild;
  if(lt && sort) {
    if(lt.getAttribute('title') > text) { // add during startup, will most likely sorted
      var i0 = 0;
      var i2 = parent.childNodes.length - 1;
      while(i0 < i2) {
        var i1 = Math.floor((i0 + i2) / 2);
        var e = parent.childNodes[i1].getAttribute('title');
        if(e > text) {
          i2 = i1;
        } else if(e < text) {
          i0 = i1 + 1;
        } else {
          i2 = i1;
          break;
        }
      }
      if(i2 < parent.childNodes.length) {
        parent.insertBefore(item, parent.childNodes[i2]);
        item = 0;
      }
      // binary sort, normal sort below
//         var it = parent.firstChild;
//         while(it) {
//           if(it.getAttribute('title') > text) {
//             parent.insertBefore(item, it);
//             item = 0;
//             break;
//           }
//           it = it.nextSibling;
//         }
    }
  }
  if(item) parent.appendChild(item);
  return [child, cell];
}

cvCluster.prototype.processEvent = function(event, latest) {
  if(event == null) {
    println("*** Empty event, disconnecting ***");
    this.disconnect(true);
    return false;
  }
  return eventconvert(this, event, latest);
}

cvCluster.prototype.refreshMerge = function(update, type) {
  if(type == "Category") {
    if(this.objects[update].get('monConfId') == 0) {
      dock.workerpool.getWorker(current.actualurl).call('cmdevice', 'getCategory', update, function(code, data) {
        if(code == 0) {
          this.objects[update].set('monConfId', data.monConfId);
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
    }
  }
}

cvCluster.prototype.cloudObjectsChanged = function(update, remove) {
  var cu = new Array();
  var cr = new Array();
  for(var i = 0; i < 5; i++) {
    cu.push(new Array());
    cr.push(new Array());
  }
  for(var i = 0; i < update.length; i++) {
    if(isCloudProviderKey(update[i])) {
      cu[0].push(update[i]);
    } else if(isCloudRegionKey(update[i])) {
      cu[1].push(update[i]);
    } else if(isCloudImageKey(update[i])) {
      cu[2].push(update[i]);
    } else if(isCloudTypeKey(update[i])) {
      cu[3].push(update[i]);
    } else if(isCloudStaticIPKey(update[i])) {
      cu[4].push(update[i]);
    }
  }
  for(var i = 0; i < remove.length; i++) {
    if(isCloudProviderKey(remove[i])) {
      cr[0].push(remove[i]);
    } else if(isCloudRegionKey(remove[i])) {
      cr[1].push(remove[i]);
    } else if(isCloudImageKey(remove[i])) {
      cr[2].push(remove[i]);
    } else if(isCloudTypeKey(remove[i])) {
      cr[3].push(remove[i]);
    } else if(isCloudStaticIPKey(remove[i])) {
      cr[4].push(remove[i]);
    }
  }
  if(cu[0].length || cr[0].length) this.objectsChanged(cu[0], cr[0], 'CloudProvider');
  if(cu[1].length || cr[1].length) this.objectsChanged(cu[1], cr[1], 'CloudRegion');
  if(cu[2].length || cr[2].length) this.objectsChanged(cu[2], cr[2], 'CloudImage');
  if(cu[3].length || cr[3].length) this.objectsChanged(cu[3], cr[3], 'CloudType');
  if(cu[4].length || cr[4].length) this.objectsChanged(cu[4], cr[4], 'CloudStaticIP');
}

cvCluster.prototype.hadoopObjectsChanged = function(update, remove) {
  var cu = new Array();
  var cr = new Array();
  for(var i = 0; i < 1; i++) {
    cu.push(new Array());
    cr.push(new Array());
  }
  for(var i = 0; i < update.length; i++) {
    if(isHadoopHDFSKey(update[i])) {
      cu[0].push(update[i]);
    }
  }
  for(var i = 0; i < remove.length; i++) {
    if(isHadoopHDFSKey(remove[i])) {
      cr[0].push(remove[i]);
    }
  }
  if(cu[0].length || cr[0].length) this.objectsChanged(cu[0], cr[0], 'HadoopHDFS');
}

cvCluster.prototype.sysInfoChanged = function(nodes) {
  dock.workerpool.getWorker(this.actualurl).call('cmdevice', 'getSysInfoCollectors', [nodes], function(code, data) {
      if(code == 0) {
        this.current.setAllSysInfo(nodes, data);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, this);  
}

cvCluster.prototype.tasksChanged = function(update, remove) {
  for(var i = 0; i < remove.length; i++) {
    if(this.objects.hasOwnProperty(remove[i])) {
      delete this.objects[remove[i]];
    }
  }
  if(update.length) {
    dock.workerpool.getWorker(this.actualurl).call('cmmain', 'getBackgroundTasksByKeys', [update], function(code, data) {
      if(code == 0) {
        for(var i = 0; i < data.length; i++) {
          this.current.objects[data[i].uniqueKey] = new dock.cvObject(this.current, data[i]);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      var taskmanager = dock.dialogs.find('taskmanager.xul');
      if(taskmanager && !taskmanager.closed) taskmanager.refresh();
    }, this);
  } else if(remove.length) {
    var taskmanager = dock.dialogs.find('taskmanager.xul');
    if(taskmanager && !taskmanager.closed) taskmanager.refresh();
  }
}

cvCluster.prototype.objectsChanged = function(update, remove, type, parentKey) {
  parentKey = isundefined(parentKey, 0);
  var showChanges = false;
  for(var i = 0; i < update.length; i++) {
    if(this.objects.hasOwnProperty(update[i])) {
      var object = this.objects[update[i]];
      if(!!object && object.localChanges() && object.get('modified')) {
        this.refreshMerge(update[i], type);
        var name;
        if((object.get('baseType') == 'MonConf')) {
          name = "the monitoring setup for " + this.getMonConfName(object.get('uniqueKey'));
        } else {
          name = object.resolveName();
        }
        dock.displayInfo("Both you and someone else made changes to " + name + ". Save " + name + " to override the other person's changes. Revert " + name + " to undo your changes and get the new changes of the other person.");
        continue;
      }
    }
    if(parentKey) {
      this.refresh(update[i], type, parentKey);
    } else {
      this.refresh(update[i], type);
    }
  }
  var removedCurrent = false;
  for(var i = 0; i < remove.length; i++) {
    if(this.objects.hasOwnProperty(remove[i])) {
      var object = this.objects[remove[i]];
      if(!!object) {
        if(object.localChanges() && !object.get('toBeRemoved') && (object.get('modified') || (currentObject == object))) {
          object.object.uniqueKey = (++this.nextLocalKey);
          object.set('modified', true);
          this.objects[object.object.uniqueKey] = object;
          object.url = this.key + "/" + object.object.baseType + "/" + object.object.uniqueKey;
          object.updateTreeItemUrl();
          object.treeitem.parentNode.parentNode.setAttribute('id', object.url);
          delete this.objects[remove[i]];
          dock.displayInfo(object.resolveName() + " was removed by someone else. Because you made changes to this " + object.get('baseType').toLowerCase() + " it will remain as a new ");
        } else {
          println(" entity removed: " + object.resolveName());
          removedCurrent = removedCurrent || (currentObject == object);
          showChanges = showChanges || object.get('modified');
          object.removeAllTreeItems();
          delete this.objects[remove[i]];
        }
      }
    }
  }
  if(type == 'Rack' || type == 'Device') this.updateRack = true;
  if((currentObject == this) || (startswith(currentTab.id, type)) || (refreshTypesCheck(currentTab.id, type))) {
    timedOutRefresh();
  }
//   var refreshDelay = 2000;
//   if(currentObject == this) {
//     window.setTimeout('currentTab.contentWindow.refresh();', refreshDelay);
//   } else if(startswith(currentTab.id, type)) {
//     window.setTimeout('currentTab.contentWindow.refresh();', refreshDelay);
//   } else if(refreshTypesCheck(currentTab.id, type)) {
//     window.setTimeout('currentTab.contentWindow.refresh();', refreshDelay);
//   }
  if(showChanges) {
    dock.updateListOfChanges();
  }
  if(removedCurrent) this.select(0);
}

cvCluster.prototype.getCGroupMetrics = function() {
  var cgroups = new Object();
  var metrics = this.find("Metric");
  for(var i = 0; i < metrics.length; i++) {
    if(startswith(metrics[i].get('metricClass'), 'CGROUPS_')) {
      cgroups[metrics[i].get('uniqueKey')] = metrics[i];
    }
  }
  return cgroups;
}

cvCluster.prototype.isCGroupMetric = function(key) {
  var metric = this.find(key);
  var mclass = '';
  if(metric) {
    if(dock.baseChildTypeCheck(metric.object, 'Metric')) {
      mclass = metric.get('metricClass');
    } else if(dock.baseChildTypeCheck(metric.object, 'HealthCheck')) {
      mclass = metric.get('HealthCheckClass');
    }
  }
  println("++++ isCGroupMetric " + metric.resolveName() + " : " + mclass);
  return startswith(mclass, 'CGROUPS_');
}

cvCluster.prototype.allMetricsAndChecks = function(cgroup) {
  var list = this.allMetrics(cgroup);
  return list.concat(this.allChecks(cgroup));
}

cvCluster.prototype.allMetrics = function(cgroup) {
  cgroup = isundefined(cgroup, 0);

  var list = new Array();
  var names = new Object();
  var monconfs = this.find("MonConf");
  for(var i = 0; i < monconfs.length; i++) {
    var mmc = monconfs[i].get('metrics');
    for(var j = 0; j < mmc.length; j++) {
      var metric = this.find(mmc[j].metric);
      if(metric) {
        var isCgroup = startswith(metric.get('metricClass'), 'CGROUPS_');
        if((cgroup == 0 && isCgroup) || (cgroup == -1 && !isCgroup)) continue;
        var text = metric.resolveName();
        if(mmc[j].metricParam.length > 0) text += " (" + mmc[j].metricParam + ")";
        if(!names.hasOwnProperty(text)) {
          list.push(mmc[j]);
          names[text] = 1;
        }
      }
    }
  }
  return list;
}

cvCluster.prototype.allChecks = function(cgroup) {
  cgroup = isundefined(cgroup, 0);

  var list = new Array();
  var names = new Object();
  var monconfs = this.find("MonConf");
  for(var i = 0; i < monconfs.length; i++) {
    var mmc = monconfs[i].get('checks');
    for(var j = 0; j < mmc.length; j++) {
      var healthcheck = this.find(mmc[j].healthcheck);
      if(healthcheck) {
        var isCgroup = startswith(healthcheck.get('healthCheckClass'), 'CGROUPS_');
        if((cgroup == 0 && isCgroup) || (cgroup == -1 && !isCgroup)) continue;
        var text = healthcheck.resolveName();
        if(mmc[j].healthcheckParam.length > 0) text += " (" + mmc[j].healthcheckParam + ")";
        if(!names.hasOwnProperty(text)) {
          list.push(mmc[j]);
          names[text] = 1;
        }
      }
    }
  }
  return list;
}

cvCluster.prototype.allChecksMetricsAsMetricPrmId = function(key, metrics, checks, classname) {
  key = isundefined(key, 0);
  metrics = isundefined(metrics, true);
  checks = isundefined(checks, true);
  classname = isundefined(classname, '');
  var monconfs;
  if(key) {
    monconfs = [this.getMonConfForDevice(key)];
  } else {
    monconfs = this.find("MonConf");
  }
  var list = new Array();
  for(var i = 0; i < monconfs.length; i++) {
    if(monconfs[i]) {
      if(metrics) {
        var mmc = monconfs[i].get('metrics');
        for(var j = 0; j < mmc.length; j++) {
          if(classname.length > 0) {
            var one = this.find(mmc[j].metric);
            if(!one || one.get('metricClass') != classname) continue;
          }
          var one = new dock.cvMetricPrmId();
          one.metricId = mmc[j].metric;
          one.metricParam = mmc[j].metricParam;
          list.push(one);
        }
      }
      if(checks) {
        var mmc = monconfs[i].get('checks');
        for(var j = 0; j < mmc.length; j++) {
          if(classname.length > 0) {
            var one = this.find(mmc[j].healthcheck);
            if(!one || one.get('healthCheckClass') != classname) continue;
          }
          var one = new dock.cvMetricPrmId();
          one.metricId = mmc[j].healthcheck;
          one.metricParam = mmc[j].healthcheckParam;
          list.push(one);
        }
      }
    }
  }
  return list;
}

cvCluster.prototype.getMonHealthConf = function(key, id) {
  var monconf = this.getMonConfForDevice(key);
  var mmc = monconf.get('checks');
  for(var j = 0; j < mmc.length; j++) {
    if(mmc[j].uniqueKey == id) return mmc[j];
  }
  return 0;
}

cvCluster.prototype.getThreshold = function(key, id) {
  var monconf = this.getMonConfForDevice(key);
  var mmc = monconf.get('metrics');
  for(var j = 0; j < mmc.length; j++) {
    var thresholds = mmc[j].thresholds;
    for(var i = 0; i < thresholds.length; i++) {
      if(thresholds[i].uniqueKey == id) return thresholds[i];
    }
  }
  return 0;
}

cvCluster.prototype.findNodesIn = function(type, key) {
  type = type.toLowerCase();
  var nodes = new Array();
  if(type == "master") {
    var list = this.find("MasterNode");
    for(var i = 0; i < list.length; i++) {
      nodes.push(list[i].object.uniqueKey);
    }
  } else if(type == "nodecategory" || type == "category") {
    var list = this.find("SlaveNode");
    for(var i = 0; i < list.length; i++) {
      if(list[i].get("category") == key) {
        nodes.push(list[i].object.uniqueKey);
      }
    }
  } else if(type == "miccategory") {
    var list = this.find("MICNode");
    for(var i = 0; i < list.length; i++) {
      if(list[i].get("category") == key) {
        nodes.push(list[i].object.uniqueKey);
      }
    }
  } else if(type == "rack") {
    var list = this.find("Device");
    for(var i = 0; i < list.length; i++) {
      if(list[i].get("rack").rack == key) {
        nodes.push(list[i].object.uniqueKey);
      }
    }
  } else if(type == "nodegroup") {
    var group = this.find(key);
    if(group) nodes = group.get("nodes");
  } else if(type == "chassis") {
    var group = this.find(key);
    if(group) nodes = group.get("members");
  } else if(type == "cmdaemonfailovergroup") {
    var base = this.getBasePartition();
    var groups = base.get('failoverGroups');
    for(var i = 0; i < groups.length; i++) {
      if(groups[i].uniqueKey == key) {
        nodes = groups[i].nodes;
        break;
      }
    }
  } else if(type == "monconf") {
    switch(key) {
    case(ALL_POWERDISTRIBUTIONUNITS) : return this.allKeys('PowerDistributionUnit');
    case(ALL_ETHERNETSWITCHES) : return this.allKeys('EthernetSwitch');
    case(ALL_MYRINETSWITCHES) : return this.allKeys('MyrinetSwitch');
    case(ALL_IBSWITCHES) : return this.allKeys('IBSwitch');
    case(ALL_MASTERNODES) : return this.allKeys('MasterNode');
    case(ALL_RACKSENSORS) : return this.allKeys('RackSensor');
    case(ALL_GENERICS) : return this.allKeys('GenericDevice'); // TODO: model
    case(ALL_CHASSIS) : return this.allKeys('Chassis');
    case(ALL_GPUUNITS) : return this.allKeys('GpuUnit');
    case(ALL_MICNODES) : return this.allKeys('MIC');
    default:
      var monconf = this.find(key);
      if(monconf) return this.findNodesIn('category', monconf.get('catId'));
    }
  } else if(type == "cloudprovider") {
    var list = this.find("CloudNode");
    for(var i = 0; i < list.length; i++) {
      var cloudsettings = list[i].get('cloudSettings');
      if(!!cloudsettings && cloudsettings.childType == 'EC2Settings') {
        if(cloudsettings.provider == key) nodes.push(list[i].object.uniqueKey);
      }
    }
  } else {
    println("Unknown type for findNodesIn: '" + type + "'");
  }
  return nodes;
}

cvCluster.prototype.belongsToChassis = function(key) {
  if(this.chassisMap) {
    if(this.chassisMap.hasOwnProperty(key)) {
      return this.chassisMap[key];
    }
  } else {
    println("cvCluster.prototype.belongsToChassis: NO MAP");
    var chassis = this.find('Chassis');
    for(var i = 0; i < chassis.length; i++) {
      var members = chassis[i].get("members");
      if(members.find(key) >= 0) {
        return chassis[i].get('uniqueKey');
      }
    }
  }
  return 0;
}

cvCluster.prototype.belongsToChassisMap = function() {
  this.chassisMap = new Object();
  var chassis = this.find('Chassis');
  for(var i = 0; i < chassis.length; i++) {
    var key = chassis[i].get('uniqueKey');
    var members = chassis[i].get("members");
    for(var j = 0; j < members.length; j++) {
      this.chassisMap[members[j]] = key;
    }
  }
}

cvCluster.prototype.belongsToStack = function(key) {
  if(this.stackMap) {
    if(this.stackMap.hasOwnProperty(key)) {
      return this.stackMap[key];
    }
  } else {
    println("cvCluster.prototype.belongsToStack: NO MAP");
    var chassis = this.find('Switch');
    for(var i = 0; i < chassis.length; i++) {
      var members = chassis[i].get("members");
      if(members.find(key) >= 0) {
        return chassis[i].get('uniqueKey');
      }
    }
  }
  return 0;
}

cvCluster.prototype.belongsToStackMap = function(subtype) {
  this.stackMap = new Object();
  var chassis = this.find(subtype);
  for(var i = 0; i < chassis.length; i++) {
    var key = chassis[i].get('uniqueKey');
    var members = chassis[i].get("members");
    for(var j = 0; j < members.length; j++) {
      this.stackMap[members[j]] = key;
    }
  }
}

cvCluster.prototype.known = function(type, field) {
  var list = this.find(type);
  for(var i = 0; i < list.length; i++) {
    list[i] = list[i].get(field);
  }
  return list;
}

cvCluster.prototype.resolveName = function(key) {
  key = isundefined(key, 0);
  if(key == 0) return this.name;
  var object = this.objects[key];
  if(object) return object.resolveName();
  return "";
}

cvCluster.prototype.resolveKey = function(name) {
  for(var key in this.objects) {
    if(this.objects[key].resolveName() == name) {
      return key;
    }
  }
  return 0;
}

cvCluster.prototype.resolveList = function(keys) {
  var resolve = new Array();
  keys.sort();
  for(var i = 0; i < keys.length; i++) {
    if((i > 0) && (keys[i - 1] == keys[i])) continue; // remove duplicates
    var one = this.find(keys[i]);
    if(!!one) resolve.push(one.resolveName());
  }
  return dotFormat(resolve);
}

cvCluster.prototype.resolveAll = function(keys) {
  var resolve = new Array();
  for(var i = 0; i < keys.length; i++) {
    var one = this.find(keys[i]);
    if(!!one) {
      resolve.push(one.resolveName());
    } else {
      resolve.push('');
    }
  }
  return resolve;
}

cvCluster.prototype.resolveText = function(text, type) {
  var result = undotFormat(text);
  if(result[0]) {
    var list = result[1];
    var keys = new Array();
    var unresolved = new Array();
    var badtype = new Array();
    for(var i = 0; i < list.length; i++) {
      var key = this.resolveKey(list[i]);
      if(key) {
        if(typeof(type) != "undefined") {
          if((this.objects[key].object.baseType == type) || (this.objects[key].object['is' + type])) {
            if(keys.find(key) < 0) keys.push(key);
          } else {
            badtype.push(list[i]);
          }
        } else {
          if(keys.find(key) < 0) keys.push(key);
        }
      } else {
        unresolved.push(list[i]);
      }
    }
    result[1] = keys.sort();
    var i = 1;
    while(i < result[1].length) {
      if(result[1][i - 1] == result[1][i]) {
        result[1].splice(i, 1);
      } else {
        i++;
      }
    }
    var out = '';
    if(unresolved.length > 0) out = "Unresolved: " + dotFormat(unresolved) + "\n";
    if(badtype.length > 0) out += "Bad type: " + dotFormat(badtype) + "\n";
    if(out.length > 0) {
      dock.displayInfo(out);
    }
  }
  return result;
}

cvCluster.prototype.allKeys = function(type, local, needCommit) {
  local = isundefined(local, true);
  needCommit = isundefined(needCommit, false);
  var found = this.find(type);
  var list = new Array();
  if(!found) {
    println("cvCluster.prototype.allKeys: " + type);
    return list;
  }
  for(var i = 0; i < found.length; i++) {
    var key = found[i].get('uniqueKey');
    if(!local || (key > MAX_LOCAL_KEY)) {
      if((!needCommit || !found[i].get('modified')) && !found[i].get('toBeRemoved')) list.push(key);
    }
  }
  return list;
}

cvCluster.prototype.stripKeys = function(found, local, needCommit) {
  local = isundefined(local, true);
  needCommit = isundefined(needCommit, false);
  var list = new Array();
  for(var i = 0; i < found.length; i++) {
    var object = this.find(found[i]);
    if(isObject(object)) {
      var key = object.get('uniqueKey');
      if(!local || (key > MAX_LOCAL_KEY)) {
        if((!needCommit || !object.get('modified')) && !object.get('toBeRemoved')) list.push(key);
      }
    }
  }
  return list;
}

cvCluster.prototype.getMonConfForDevice = function(key) {
  var device = this.find(key);
  if(device) return device.getMonitoringConfiguration();
  println("Device not found: " + key);
  return 0;
}

cvCluster.prototype.getMonConfName = function(key) {
  var treeotherdevices = rebranding(dock.TreeOtherDevices, 'Generic Devices');
  var treepdus = rebranding(dock.TreePowerDistributionUnits, 'Power Distribution Units');
  var treesensors = rebranding(dock.TreeSensors, 'Rack Sensors');
  var treechas = rebranding(dock.TreeChassis, 'Chassis');

  switch(key) {
  case(ALL_POWERDISTRIBUTIONUNITS) : if(treepdus.length) return "All " + treepdus; return "";
  case(ALL_ETHERNETSWITCHES) : return "All Ethernet Switches";
  case(ALL_MYRINETSWITCHES) : return "All Myrinet Switches";
  case(ALL_IBSWITCHES) : return "All IB Switches";
  case(ALL_MASTERNODES) : return "All Head Nodes";
  case(ALL_RACKSENSORS) : if(treesensors.length) return "All " + treesensors; return "";
  case(ALL_GENERICS) : if(treeotherdevices.length) return "All " + treeotherdevices; return "";
  case(ALL_CHASSIS) : if(treechas.length) return "All " + treechas; return "";
  case(ALL_GPUUNITS) : return "All GPU units";
  case(ALL_MICNODES) : return "All MICs";
  default:
    var monconf = this.find(key);
    if(monconf) {
      var cat = this.find(monconf.get('catId'));
      if(!!cat) return cat.resolveName();
    }
  }
  return "Unknown";
}

cvCluster.prototype.getMonConfImage = function(key) {
  switch(key) {
  case(ALL_POWERDISTRIBUTIONUNITS) : return 'chrome://cmgui/content/icons/PowerDistributionUnit_16.png';
  case(ALL_ETHERNETSWITCHES) : return 'chrome://cmgui/content/icons/EthernetSwitch_16.png';
  case(ALL_MYRINETSWITCHES) : return 'chrome://cmgui/content/icons/MyrinetSwitch_16.png';
  case(ALL_IBSWITCHES) : return 'chrome://cmgui/content/icons/IBSwitch_16.png';
  case(ALL_MASTERNODES) : return 'chrome://cmgui/content/icons/MasterNode_16.png';
  case(ALL_RACKSENSORS) : //return 'chrome://cmgui/content/icons/RackSensor_16.png';
  case(ALL_GENERICS) : return 'chrome://cmgui/content/icons/GenericDevice_16.png';
  case(ALL_CHASSIS) : return 'chrome://cmgui/content/icons/Chassis_16.png';
  case(ALL_GPUUNITS) : return 'chrome://cmgui/content/icons/GpuUnit_16.png';
  case(ALL_MICNODES) : return 'chrome://cmgui/content/icons/MICNode_16.png';
  default: return 'chrome://cmgui/content/icons/SlaveNode_16.png';
  }
  return '';
}

cvCluster.prototype.addMonitoringMenu = function() {
  var menu = document.getElementById("monmenu");
  this.menuitem = document.createElement("menuitem");
  this.menuitem.setAttribute('label', this.name);
  this.menuitem.setAttribute('oncommand', "showMonitoring('" + this.key + "');");
  menu.appendChild(this.menuitem);
  document.getElementById("notconnected").setAttribute('hidden', true);;

  var menu = document.getElementById("filtermenu");
  this.menuitemf = document.createElement("menuitem");
  this.menuitemf.setAttribute('label', this.name);
  this.menuitemf.setAttribute('oncommand', "showFilter('" + this.key + "');");
  menu.appendChild(this.menuitemf);
  document.getElementById("notconnectedf").setAttribute('hidden', true);
}

cvCluster.prototype.removeMonitoringMenu = function() {
  if(this.menuitem) {
    var menu = document.getElementById("monmenu");
    menu.removeChild(this.menuitem);
    this.menuitem = 0;
  }
  if(this.menuitemf) {
    var menu = document.getElementById("filtermenu");
    menu.removeChild(this.menuitemf);
    this.menuitemf = 0;
  }
  for(var i = 0; i < clusterlist.length; i++) {
    if(clusterlist[i].connected) return;
  }
  document.getElementById("notconnected").setAttribute('hidden', false);
  document.getElementById("notconnectedf").setAttribute('hidden', false);
}

cvCluster.prototype.getBasePartition = function(cmd) {
  var partitions = this.find("Partition");
  if(partitions.length == 1) return partitions[0];
  for(var i = 0; i < partitions.length; i++) {
    if(partitions[i].resolveName.toLowerCase() == "base") return partitions[i];
  }
  return null;
}

cvCluster.prototype.runCommandOnMaster = function(cmd) {
  var base = this.getBasePartition();
  if(base) {
    var nodes = [base.get('primaryMaster')];
    dock.workerpool.getWorker(current.actualurl).call('cmproc', 'startAsynchronousCommand', [this.syslogin, cmd,, this.sessionid, nodes], function(code, data) {
        if(code == 0) {
          // good ?
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
    return true;
  }
  return false;
}

cvCluster.prototype.ensureInfoMessage = function(keys) {
  var list = new Array();
  for(var i = 0; i < keys.length; i++) {
    if(keys[i] > 0) {
      if(!this.infoMessage.hasOwnProperty(keys[i])) {
        this.infoMessage[keys[i]] = '';
        list.push(keys[i]);
      }
    }
  }
  if(list.length > 0) {
    // KDR: wait ??
    dock.workerpool.getWorker(current.actualurl).call('cmmon', 'getInfoMessages', [list], function(code, data) {
        if(code == 0) {
          for(var i = 0; i < this.input[0].length; i++) {
            this.current.infoMessage[this.input[0][i]] = data[i];
          }
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
  }
}

cvCluster.prototype.composeInfoMessage = function(keys) {
  var list = new Array();
  for(var i = 0; i < keys.length; i++) {
    if(keys[i]) {
      if(!!this.infoMessage[keys[i]]) {
        list.push(this.infoMessage[keys[i]]);
      }
    }
  }
  return list.join("\n");
}

cvCluster.prototype.getInfoMessage = function(key) {
  var info = this.infoMessage[key];
  if(typeof(info) == 'undefined') return "TODO";
  return info;
}

cvCluster.prototype.getGraphCM = function() {
  var m = 1e+308;
  var j = -1;
//   for(var i = 0; i < this.graphUse.length; i++) {
//     if(this.graphUse[i] == 0) {
//       j = i;
//       break;
//     } else if(this.graphUse[i] < m) {
//       m = this.graphUse[i];
//       j = i;
//     }
//   }
  this.graphUse[j]++;
  return j;
}

cvCluster.prototype.freeGraphCM = function(index) {
  this.graphUse[index]--;
  if(this.graphUse[index] < 0) {
    println('Free graph ' + index + ' less than 0!!');
  }
}

cvCluster.prototype.setUniqueKey = function(copy, originalCluster) {
  if(isArray(copy)) {
    if(copy.length > 0) {
      if(typeof(copy[0]) == 'object') {
        var list = new Array();
        for(var k = 0; k < copy.length; k++) {
          var tmp = new dock.cvObject(originalCluster, copy[k]);
          var clone = tmp.clone(this);
          this.nextLocalKey++;
          clone.uniqueKey = this.nextLocalKey;
          list.push(clone);
        }
        return list;
      }
    }
  } else if(typeof(copy) == 'object') {
    var tmp = new dock.cvObject(originalCluster, copy);
    var clone = tmp.clone(this);
    this.nextLocalKey++;
    clone.uniqueKey = this.nextLocalKey;
    return clone;
  }
  return copy;
}

cvCluster.prototype.findGlobalNetwork = function() {
  var networks = this.find('Network');
  for(var i = 0; i < networks.length; i++) {
    if(networks[i].get('type') == 'GLOBAL') {
      return networks[i];
    }
  }
  return false;
}

cvCluster.prototype.findNetMapNetwork = function() {
  var networks = this.find('Network');
  for(var i = 0; i < networks.length; i++) {
    if(networks[i].get('type') == 'NETMAP') {
      return networks[i];
    }
  }
  return false;
}

cvCluster.prototype.findCloudNetworks = function(baseAddress, netmaskBits) {
  baseAddress = isundefined(baseAddress, 0);
  netmaskBits = isundefined(netmaskBits, 0);
  var r = new Array()
  var networks = this.find('Network');
  for(var i = 0; i < networks.length; i++) {
    if(networks[i].get('type') == 'CLOUD') {
      if (baseAddress && netmaskBits) { // additional filter
        if ((networks[i].get('baseAddress') == baseAddress) && (networks[i].get('netmaskBits') == netmaskBits)) {
          r.push(networks[i]);
        }
      } else {
        r.push(networks[i]);
      }
    }
  }
  return r;
}

cvCluster.prototype.findTunnelNetworks = function() {
  var r = new Array()
  var networks = this.find('Network');
  for(var i = 0; i < networks.length; i++) {
    if(networks[i].get('type') == 'TUNNEL') {
      r.push(networks[i]);
    }
  }
  return r;
}

cvCluster.prototype.findInternalNetworks = function() {
  var r = new Array()
  var networks = this.find('Network');
  for(var i = 0; i < networks.length; i++) {
    if(networks[i].get('type') == 'INTERNAL') {
      r.push(networks[i]);
    }
  }
  return r;
}

cvCluster.prototype.findExternalNetworks = function() {
  var r = new Array();
  var networks = this.find('Network');
  for(var i = 0; i < networks.length; i++) {
    if(networks[i].get('type') == 'EXTERNAL') {
      r.push(networks[i]);
    }
  }
  return r;
}

cvCluster.prototype.findManagementNetworks = function() {
  var r = new Array()
  var base = this.getBasePartition();
  if(base) {
    var network = this.find(base.get('managementNetwork'));
    if(network) {
      r.push(network);
    } else {
      println('findManagementNetworks, no base management network');
    }
  } else {
    println('findManagementNetworks, no base partition');
  }
  var networks = this.find('Network');
  for(var i = 0; i < networks.length; i++) {
    if(networks[i].get('management') && r.find(networks[i]) < 0) {
      r.push(networks[i]);
    }
  }
  return r;
}

cvCluster.prototype.findOpenStackNetwork = function() {
  var networks = this.find('Network');
  for(var i = 0; i < networks.length; i++) {
    if((networks[i].get('type') == 'INTERNAL') && (networks[i].get('openStackType') == 'FLAT')) {
      return networks[i];
    }
  }
  return 0;
}

cvCluster.prototype.metricEnumName = function(metric, proto, cgroup) {
  metric = isundefined(metric, true);
  proto = isundefined(proto, true);
  cgroup = isundefined(cgroup, 0);
  var list;
  if(cgroup >= 0) {
      list = [['CPU',                         'CPU'],
              ['GPU',                         'GPU'],
              ['DISK',                        'Disk'],
              ['MEMORY',                      'Memory'],
              ['NETWORK',                     'Network'],
              ['ENVIRONMENTAL',               'Environmental'],
              ['OPERATINGSYSTEM',             'Operating System'],
              ['INTERNAL',                    'Internal'],
              ['WORKLOAD',                    'Workload'],
              ['CLUSTER',                     'Cluster'],
              ['AMAZONEC2',                   'Amazon EC2'],
              ['CLOUD',                       'Cloud'],
              ['MISC',                        'Misc'],
              ['HADOOP_JVM',                  'Hadoop/JVM'],
              ['HADOOP_DFS',                  'Hadoop/DFS'],
              ['HADOOP_UGI',                  'Hadoop/UGI'],
              ['HADOOP_MAPRED',               'Hadoop/map reduce'],
              ['HADOOP_RPC',                  'Hadoop/RPC'],
              ['HADOOP_RPCDETAILED',          'Hadoop/RPC/detailed'],
              ['HADOOP_METRICSYSTEM',         'Hadoop/metric system'],
              ['OPENSTACK',                   'OpenStack'],
              ['OPENSTACK_API_IMAGE',         'OpenStack/API/Image'],
              ['OPENSTACK_API_IDENTITY',      'OpenStack/API/Identity'],
              ['OPENSTACK_API_COMPUTE',       'OpenStack/API/Compute'],
              ['OPENSTACK_API_NETWORK',       'OpenStack/API/Network'],
              ['OPENSTACK_API_VOLUME',        'OpenStack/API/Volume'],
              ['OPENSTACK_API_ALLSERVICES',   'OpenStack/API/AllServices'],
              ['OPENSTACK_API_ORCHESTRATION', 'OpenStack/API/Orchestration'],
              ['OPENSTACK_VM',                'OpenStack/VM'], // KDR: this is a fake entry to make OpenStack VM instance magic work
              ['OPENSTACK_VM_MEMORY',         'OpenStack/VM/Memory'],
              ['OPENSTACK_VM_BLOCKDEVICE',    'OpenStack/VM/BlockDevice'],
              ['OPENSTACK_VM_NETWORK',        'OpenStack/VM/Network'],
              ['OPENSTACK_VM_CPU',            'OpenStack/VM/CPU'],
              ['OPENSTACK_VM_OTHER',          'OpenStack/VM/Other'],
              ['OPENSTACK_HYPERVISORS',       'OpenStack/Hypervisors'],
              ['OPENSTACK_OTHER',             'OpenStack/Other'],
              ['OPENSTACK_PROJECTS',          'OpenStack/Projects'],
              ['OPENSTACK_OBJECTS',           'OpenStack/Objects'],
              ['DELLNSS',                     'Dell NSS'],
              ['CEPH',                        'Ceph'],
              ['LUSTRE',                      'Lustre']
    ];
    if(metric && proto) {
      list.push(['PROTOTYPE', 'Prototype']);
    }
    for(var i = 0; i < this.userDefinedMetricTypes.length; i++) {
      if(this.userDefinedMetricTypes[i].length > 0) {
        list.push(['USERDEFINED' + (i + 1), this.userDefinedMetricTypes[i]]);
      }
    }
  } else {
    list = new Array();
  }
  if(cgroup != 0) {
    list.push(['CGROUPS_BLKIO', 'CGroup block IO']);
    list.push(['CGROUPS_CPU', 'CGroup cpu']);
    list.push(['CGROUPS_MEMORY', 'CGroup memory']);
    list.push(['CGROUPS_NETWORK', 'CGroup network']);
  }
  return list;
}

cvCluster.prototype.metricTypeToName = function(metric, kclass) {
  var list = this.metricEnumName(metric);
  for(var i = 0; i < list.length; i++) {
    if(list[i][0] == kclass) return list[i][1];
  }
  return 'Undefined';
}

cvCluster.prototype.getLatestEC2AMI = function() {
  var list = this.find('EC2AMI');
  var index = 0;
  var r;
  for(var i = 0; i < list.length; i++) {
    var name = list[i].resolveName();
    if(startswith(name, 'brightinstaller-')) {
      name = name.substr(16);
      if(isNumeric(name)) {
        var v = myParseInt(name);
        if(v > index) {
          v = index;
          r = list[i];
        }
      }
    }
  }
  return r;
}

cvCluster.prototype.getJobCache = function(key) {
  if(!this.nodeJobCache.hasOwnProperty(key)) return new Array();
  return this.nodeJobCache[key];
}

cvCluster.prototype.getJobNodeCache = function(job) {
  if(!this.jobNodeCache.hasOwnProperty(job)) return {nodes: [], startTime: 0, endTime: 0};
  return this.jobNodeCache[job];
}

cvCluster.prototype.fakeJobCache = function(jobs) {
  if(dock.JOBGROUPDEMO && jobs.length == 0) {
    var job = new dock.cvSGEJob();
    job.jobID = 'job.0001';
    job.jobname = 'test1';
    job.username = 'koen';
    job.queue = 'short';
    job.nodes = ['node001.cm.cluster'];
    job.starttime = 'Wed Oct  3 11:03:02';
    jobs.push(job);
    var job = new dock.cvSGEJob();
    job.jobID = 'job.0002';
    job.jobname = 'test2';
    job.username = 'koen';
    job.queue = 'short';
    job.nodes = ['node001.cm.cluster', 'node003.cm.cluster'];
    job.starttime = 'Wed Oct  3 11:03:02';
    jobs.push(job);
    var job = new dock.cvSlurmJob();
    job.jobID = 'job.0003';
    job.jobname = 'test3';
    job.username = 'koen';
    job.queue = 'short';
    job.nodes = ['node002', 'node004', 'node005', 'node006', 'node007', 'node008'];
    job.starttime = 'Sat Dec 31 11:03:02';
    jobs.push(job);
  }
}

cvCluster.prototype.resolveName2KeyMap = function(type) {
  type = isundefined(type, false);
  var name2key = new Object();
  for(var obj in this.objects) {
    var work = this.objects[obj];
    if(work.object != null) {
      if(!type || baseChildTypeCheck(work.object, type)) {
        name2key[work.resolveName()] = obj;
      }
    }
  }
  return name2key;
}

cvCluster.prototype.processJobCache = function(jobs) {
  if(this.jobCacheTimer > 0) {
    println('Updating job cache, [CANCEL]');
    window.clearTimeout(this.jobCacheTimer);
    this.jobCacheTimer = 0;
  }
  this.jobCacheStart = Date.now() / 1000;
  println('Updating job cache, [START] ' + jobs.length);
  this.fakeJobCache(jobs);
  this.nodeJobCache = new Object();
  this.jobNodeCache = new Object();
  this.jobCache = jobs;
  this.jobCacheIndex = 0;
  this.jobCacheTimer = window.setTimeout(createObjectCallback(this, this.processJobCacheThread), 0);
}

cvCluster.prototype.processJobCacheThread = function() {
  if((10 * (this.jobCacheIndex + dock.FILL_COUNT) / this.jobCache.length) > (10 * this.jobCacheIndex / this.jobCache.length)) {  
    println('Updating job cache, [RUN] ' + this.jobCacheIndex + "/" + this.jobCache.length);
  }
  var hostname2key = this.resolveName2KeyMap('Device');
  for(var i = 0; (i < dock.FILL_COUNT) && (this.jobCacheIndex < this.jobCache.length); i++, this.jobCacheIndex++) {
    var job = this.jobCache[this.jobCacheIndex];
    var nodes = job.nodes;
    var jobID = job.jobID;
    var t0 = parseJobTime(job.starttime);
    var t1 = 0;
    var scheduler = job.childType.replace('Job','').toLowerCase();
    var nodeKeys = new Array();
    for(var j = 0; j < nodes.length; j++) {
      var key = 0;
      var hostname;
      var idx = nodes[j].indexOf('.');
      if(idx < 0) {
        hostname = nodes[j];
      } else {
        hostname = nodes[j].substr(0, idx);
      }
      if(hostname2key.hasOwnProperty(hostname)) {
        key = hostname2key[hostname];
      } else {
        continue;
      }
      nodeKeys.push(key);
      if(!this.nodeJobCache.hasOwnProperty(key)) this.nodeJobCache[key] = new Array();
      this.nodeJobCache[key].push(jobID); // TODO: jobname?
    }
    this.jobNodeCache[scheduler + '|' + jobID] = {nodes: nodeKeys, startTime: t0, endTime: t1};
  }
  if(this.jobCacheIndex < this.jobCache.length) {
    this.jobCacheTimer = window.setTimeout(createObjectCallback(this, this.processJobCacheThread), dock.FILL_TIMEOUT);
  } else {
    this.jobCacheTimer = 0;
    println('Updating job cache, [DONE]. duration: ' + (Date.now() / 1000 - this.jobCacheStart));
  }
}

cvCluster.prototype.updateNodeJobCache = function(force, followUp) {
  force = isundefined(force, false);
  followUp = isundefined(followUp, 0);
  var t0 = Date.now() / 1000;
  if(force || (t0 - this.lastJobCacheUpdate) > 60) {
    println('Retrieve data for job cache: ' + (t0 - this.lastJobCacheUpdate));
    this.lastJobCacheUpdate = t0;
    dock.workerpool.getWorker(this.actualurl).call('cmjob', 'getJobs', '', function(code, data) {
        if(code == 0) {
          this.current.processJobCache(data);
          if(followUp) followUp();
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, this);
    return false;
  }
  return true;
}

cvCluster.prototype.checkLDAP = function() {
  dock.workerpool.getWorker(this.actualurl).call('cmuser', 'checkLDAP', null, function(code, data) {
    if(code == 0) {
      this.current.LDAPstatus.checked = true;
      if (data.length > 0) {
        this.current.LDAPstatus.errorstring = data;
        alert('LDAP server error: ' + this.current.LDAPstatus.errorstring);
      }
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, this);
}

cvCluster.prototype.setGenericInfoWait = function(key, func) {
  if(this.genericDeviceInfo.hasOwnProperty(key)) {
    var values = this.genericDeviceInfo[key];
    delete this.genericDeviceInfo[key];
    return values;
  } else {
    this.genericDeviceInfo[key] = func;
    return false;
  }
}

cvCluster.prototype.clearGenericInfoWait = function(key) {
  return delete this.genericDeviceInfo[key];
}

cvCluster.prototype.updateGenericInfoWait = function(key, data) {
  if(this.genericDeviceInfo.hasOwnProperty(key)) {
    if(typeof(this.genericDeviceInfo[key]) == 'function') {
      this.genericDeviceInfo[key](data);
    } else {
      println("updateGenericInfoWait: not a function");
      printall(this.genericDeviceInfo[key]);
    }
    delete this.genericDeviceInfo[key];
  } else {
    this.genericDeviceInfo[key] = data;
  }
}

cvCluster.prototype.isUCS = function() {
  return this.hasUCS;
}

cvCluster.prototype.isDell13G = function() {
  return this.hasDellSettings;
}

cvCluster.prototype.isCeph = function() {
  return this.hasCeph;
}

cvCluster.prototype.isHadoop = function() {
  return this.hasHadoop;
}

cvCluster.prototype.isOpenStack = function() {
  return this.hasOpenStack;
}

cvCluster.prototype.isKubernetes = function() {
  return this.hasKube;
}

cvCluster.prototype.isPuppet = function() {
  return this.isPuppet;
}

cvCluster.prototype.isDockerRegistry = function() {
  return this.isDockerRegistry;
}

cvCluster.prototype.findActivePassiveMetric = function(nodegroup) {
  var nodes = nodegroup.get('nodes');
  for(var i = 0; i < nodes.length; i++) {
    var node = this.find(nodes[i]);
    if(!node) continue;
    var monconf = this.getMonConfForDevice(nodes[i]);
    if(!monconf) continue;
    var metrics = monconf.get('metrics');
    for(var j = 0; j < metrics.length; j++) {
      var metric = this.find(metrics[j].metric);
      if(!metric) continue;
      if(metric.get('name') == 'StoragePairState') {
        println('findActivePassiveMetric ' + node.get('hostname') + ' : ' + metric.get('name'));
        return [metrics[j].metric, metrics[j].metricParam];
      } else if(metric.get('name') == 'CMDFailoverGroupState') {
        println('findActivePassiveMetric ' + node.get('hostname') + ' : ' + metric.get('name'));
        return [metrics[j].metric, metrics[j].metricParam];
      }
    }
  }
  return false;
}

// TODO: cache?
cvCluster.prototype.getDellNSSNodes = function() {
//   var nodes = this.find('Node');
//   for(var i = 0; i < nodes.length; i++) {
//
//   }
}

cvCluster.prototype.setAllSysInfo = function(nodes, data) {
  println('SetAllSysInfo: ' + nodes.length + ' ' + data.length);
  var j = 0;
  for(var i = 0; i < nodes.length && j < data.length; i++) {
    var node = nodes[i];
    var key = node;
    if(isObject(key)) {
      key = key.get('uniqueKey');
    } else {
      node = this.find(node);
    }
    if((key == data[j].uniqueKey) || (key == data[j].refNodeUniqueKey)) {
      node.sysInfo = data[j];
      j++;
    } else {
      println('Unknown SetAllSysInfo: ' + data[j].uniqueKey);
    }
  }
}

cvCluster.prototype.setAllLiveUCSInfo = function(nodes, data) {
  println('SetAllLiveUCSInfo: ' + nodes.length + ' ' + data.length);
  var j = 0;
  for(var i = 0; i < nodes.length && j < data.length; i++) {
    if(nodes[i].get('uniqueKey') == data[j].uniqueKey) {
//     if((nodes[i].get('uniqueKey') == data[j].uniqueKey) || (nodes[i].get('uniqueKey') == data[j].refNodeUniqueKey)) {
      nodes[i].ucsInfo = data[j];
      j++;
      this.hasUCS = true;
    }
  }
}

cvCluster.prototype.setAllLiveDellSettingsInfo = function(nodes, data) {
  println('SetAllLiveDellSettingsInfo: ' + nodes.length + ' ' + data.length);
  var j = 0;
  for(var i = 0; i < nodes.length && j < data.length; i++) {
    if(nodes[i].get('uniqueKey') == data[j].uniqueKey) {
//     if((nodes[i].get('uniqueKey') == data[j].uniqueKey) || (nodes[i].get('uniqueKey') == data[j].refNodeUniqueKey)) {
      nodes[i].dellSettingsInfo = data[j];
      j++;
      this.hasDellSettings = true;
    }
  }
}

cvCluster.prototype.setCephState = function(data) {
  println('Set Ceph state');
  this.cephState = data;
  if(data) {
    this.cephStateLookup = new Object();
    var n = Math.min(data.OSDMapNodes.length, data.OSDMapAssocNames.length, data.OSDMapIds.length);
    for(var i = 0; i < n; i++) {
      this.cephStateLookup[[data.OSDMapNodes[i], data.OSDMapAssocNames[i]]] = data.OSDMapIds[i];
    }
  } else {
    this.cephStateLookup = 0;
  }
}

cvCluster.prototype.getCephOSDid = function(node, osd) {
  if(this.cephStateLookup && this.cephStateLookup.hasOwnProperty([node, osd])) return this.cephStateLookup[[node, osd]];
  return -1;
}

cvCluster.prototype.setLicenseInfo = function(info) {
  if(this.licenseInfo == 0) {
    this.licenseInfo = info;
    if(info.licensedHadoopNodes < 0) {
      println('Hide all Hadoop');
      if(this.branch.hasOwnProperty('HadoopHDFSFolder')) {
        this.branch['HadoopHDFSFolder'].parentNode.setAttribute('hidden', true);
      } else {
        println('No HadoopHDFSFolder branch');
      }
    }
    if(info.licensedOpenStackNodes < 0) {
      println('Hide all OpenStack');
      if(this.branch.hasOwnProperty('OpenStackFolder')) {
        this.branch['OpenStackFolder'].parentNode.setAttribute('hidden', true);
      } else {
        println('No OpenStackFolder branch');
      }
    }
  }
}

cvCluster.prototype.lustreErrorCache = function(errorMessage) {
  if (errorMessage) {
    this.lustreErrors.push(errorMessage);
  }
}

cvCluster.prototype.fakeOpenStackApiSettings = function(current) {
  if(this.fakedOpenStackApiSettings===false) {
    return true;
  }

  var parentKey = 0;
  if(current) parentKey = current.get('uniqueKey');

  var domains = new Array();
  var users = new Array();
  var flavors = new Array();
  var groups = new Array();
  var projects = new Array();
  var servers = new Array();
  var roleAssignments = new Array();

  for(var i = 0; i < 5; i++) {
    var domain = new dock.cvOpenStackApiDomain();
    domain.name = 'test.domain.' + sprintf("%04d", i);
    domain.UUID = 'TEST-UUID-DOMAIN-' + sprintf("%04d", i);
    domain.ownerUniqueKey = parentKey;
    this.addObject(domain, false);
    domain.modified = false;
    domains.push(domain.uniqueKey);
  }

  for(var i = 0; i < 10; i++) {
    var group = new dock.cvOpenStackApiGroup();
    group.name = 'test.group.' + sprintf("%04d", i);
    group.UUID = 'TEST-UUID-GROUP-' + sprintf("%04d", i);
    group.ownerUniqueKey = parentKey;
    var d = (i % domains.length);
    group.domainUUID = 'TEST-UUID-DOMAIN-' + sprintf("%04d", d);
    group.domainKey = domains[d];

    this.addObject(group, false);
    group.modified = false;
    groups.push(group.uniqueKey);
  }

  for(var i = 0; i < 10; i++) {
    var user = new dock.cvOpenStackApiUser();
    user.name = 'test.user.' + sprintf("%04d", i);
    user.UUID = 'TEST-UUID-USER-' + sprintf("%04d", i);
    user.ownerUniqueKey = parentKey;
    user.description = 'DESCR TEST-UUID-USER-' + sprintf("%04d", i);
    user.email = 'TEST-UUID-USER-' + sprintf("%04d", i) + '@brightcomputing.com';
    user.password = 'PASSWORD-' + sprintf("%04d", i);
    var d = (i % domains.length);
    user.domainUUID = 'TEST-UUID-DOMAIN-' + sprintf("%04d", d);
    user.domainKey = domains[d];
    var u = 0;
    for(var j = 0; j < i; j++) {
      if (j<=groups.length) {
        u = j;
      }
      user.groupUUIDs.push('TEST-UUID-GROUP-' + sprintf("%04d", u));
      user.groupKeys.push(groups[u]);
    }
    this.addObject(user, false);
    user.modified = false;
    users.push(user.uniqueKey);
  }

  for(var i = 0; i < 20; i++) {
    var project = new dock.cvOpenStackApiProject();
    project.name = 'test.project.' + sprintf("%04d", i);
    project.UUID = 'TEST-UUID-PROJECT-' + sprintf("%04d", i);
    project.ownerUniqueKey = parentKey;
    var d = (i % domains.length);
    project.domainUUID = 'TEST-UUID-DOMAIN-' + sprintf("%04d", d);
    project.domainKey = domains[d];

    this.addObject(project, false);
    project.modified = false;
    projects.push(project.uniqueKey);
  }

  for(var i = 0; i < 10; i++) {
    var flavor = new dock.cvOpenStackApiFlavor();
    flavor.name = 'test.flavor.' + sprintf("%04d", i);
    flavor.UUID = 'TEST-UUID-FLAVOR-' + sprintf("%04d", i);
    flavor.ownerUniqueKey = parentKey;
    flavor.disk = 100000;
    flavor.ephemeral = 20;
    flavor.swap = 30;
    flavor.vcpus = 2;
    flavor.ram = 40;
    flavor.rxTxFactor = 5;
    flavor.extraSpecs = 'aa';

//    var d = (i % domains.length);
//    flavor.domainUUID = 'TEST-UUID-DOMAIN-' + sprintf("%04d", d);
//    flavor.domainKey = domains[d];

    this.addObject(flavor, false);
    flavor.modified = false;
    flavors.push(flavor.uniqueKey);
  }

  for(var i = 0; i < 50; i++) {
    var server = new dock.cvOpenStackApiServer();
    server.name = 'test.server.' + sprintf("%04d", i);
    server.UUID = 'TEST-UUID-SERVER-' + sprintf("%04d", i);
    server.ownerUniqueKey = parentKey;
    var p = (i % projects.length);
    server.projectUUID = 'TEST-UUID-PROJECT-' + sprintf("%04d", p);
    server.projectKey = projects[p];
    var u = (i % users.length);
    server.userUUID = 'TEST-UUID-USER-' + sprintf("%04d", u);
    server.userKey = users[u];
    var f = (i % flavors.length);
    server.flavorUUID = 'TEST-UUID-FLAVOR-' + sprintf("%04d", u);
    server.flavorKey = flavors[u];

    this.addObject(server, false);
    server.modified = false;
    servers.push(server.uniqueKey);
  }

  for(var i = 0; i < 100; i++) {
    var roleAssignment = new dock.cvOpenStackApiRoleAssignment();
    roleAssignment.name = 'test.role.assignment.' + sprintf("%04d", i);
    roleAssignment.UUID = 'TEST-UUID-ROLE-ASSIGNMENT-' + sprintf("%04d", i);
    roleAssignment.ownerUniqueKey = parentKey;
    var d = (i % projects.length);
    roleAssignment.domainUUID = 'TEST-UUID-DOMAIN-' + sprintf("%04d", d);
    roleAssignment.domainKey = domains[d];
    var p = (i % projects.length);
    roleAssignment.projectUUID = 'TEST-UUID-PROJECT-' + sprintf("%04d", p);
    roleAssignment.projectKey = projects[p];
    var u = (i % users.length);
    roleAssignment.userUUID = 'TEST-UUID-USER-' + sprintf("%04d", u);
    roleAssignment.userKey = users[u];

    this.addObject(roleAssignment, false);
    roleAssignment.modified = false;
    roleAssignments.push(roleAssignment.uniqueKey);
  }

  return true;
}

cvCluster.prototype.activeHeadNode = function() {
  if(this.activePassiveKeys.length <= 0) return null;
  return this.find(this.activePassiveKeys[0]);  
}

cvCluster.prototype.passiveHeadNode = function() {
  if(this.activePassiveKeys.length <= 1) return null;
  return this.find(this.activePassiveKeys[1]);
}
