/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var basetype = 'ConfigurationOverlay';
var singular = 'Configuration Overlay';
var plural = 'Configuration Overlays';

var dock;
var cluster;
var current;
var pending_use = null;
var timer = 0;

//var listRoles;
//var nonmovableRoles;
var hdfskey;
var is_puppet_view = false;
var hadoopSecondFilter;

var overlay_controller = null;

var refreshLevel = 1;
var loaded = false;

var data = [];
var tree; // need global variable for tree filter

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
  var total_items = document.getElementById('tableTotalItems');
  tree = document.getElementById('tree');
  if(total_items) {
      tree.addEventListener('row-count', function (event) {
          var count = 0;
          var t = document.getElementById('tree');
          if(t.view && t.view.rowCount) {
              count = t.view.rowCount - 1;
          }
          if (count===1) {
              var msg = 'Displaying ' + count + ' item';
          } else {
              var msg = 'Displaying ' + count + ' items';
          }
          document.getElementById('tableTotalItems').setAttribute('value', msg);
      }, false);
  }
  loaded = true;
  overlay_controller = createOverlayController(location.hash, location.pathname);
  overlay_controller.showButtons();
  if (pending_use) {
    //println('Refresh called with use while loading. Refreshing now. ' + document.title + ' - ' + document.documentURI);
    refresh(pending_use);
    pending_use = null;
    //this.refresh(window.parent.window.current);
  }
}

function refresh(use, select=null) {
  if(!!use && !loaded) {
    //println('Refresh called with use but page is not loaded yet. ' + 
    //        document.title + ' - ' + document.documentURI);
    pending_use = use;
    return;
  }
  println('.Refresh: ' + document.title + ' - ' + document.documentURI);
  if(!!use) current = use;
  
  if ( current.isCluster )    cluster = current;
  else                        cluster = current.cluster; 
  tree = document.getElementById('tree');

  if ( location.hash.indexOf('#HADOOP') === 0) {
    initHadoopView();
  } 
  else if(location.hash.indexOf('#PUPPET') === 0) {
    is_puppet_view = true;
  }
  else if(location.hash.indexOf('#OPENSTACK') === 0) {
  }
  else {
    var treeActionMenu = document.getElementById("treeActionMenu");
    if(treeActionMenu) {
      treeActionMenu.setAttribute("onpopupshowing", "return false;");
    }
    overlay_controller.overlay_filter   = function(overlay)  { return true; };
    overlay_controller.node_filter      = function(node)     { return true; };
    overlay_controller.category_filter  = function(category) { return true; };
  }

  var ungrouped_data = overlay_controller.createClusterRoleData(cluster,hdfskey);
  if(select) {
    overlay_controller.open_items[select] = true;
  }
  data = [];
  if(endswith(location.pathname, 'clusterconfoverlays.xul')) {
      data = overlay_controller.groupRoleDataByContainer(ungrouped_data);
  }
  else if(endswith(location.pathname, 'clusternoderoles.xul')) {
      data = overlay_controller.groupRoleDataByNode(ungrouped_data, cluster);
  }
  else if(endswith(location.pathname, 'clustergroupedroles.xul')) {
      data = overlay_controller.groupRoleDataByRole(ungrouped_data, cluster);
  }

  for(var i = data.length; i < MINTREEITEMS; i++)  {
      data.push(blankTreeItem());
  }

  loadTable(tree, data);
  if(select) {
    setSelected('list', [ select ] );
  }
  
  var modified = false;
  for(var i = 0; i < data.length; i++) {
    if(data[i].modified) {
      modified = true;
      break;
    }
  }
  document.getElementById('saveCO').setAttribute('disabled', !modified);
  document.getElementById('revert').setAttribute('disabled', !modified);
}


function hadoopRoleFilter(obj) {
  var roles = obj.getHadoopRoles(hdfskey, false);
  if(roles.length == 0) {
    return false;
  } else if(refreshLevel == 1) {
    return true;
  } else {
    for(var j = 0; j < roles.length; j++) {
      if(hadoopTypesToRoles[hadoopSecondFilter][roles[j].childType]) {
        return true;
      }
    }
    return false;
  }
};

function puppetRoleFilter(obj) {
  var puppet_roles = obj.getRolesByFilter(function(role) {
      if(startswith(role.childType, 'Puppet')) {
         return true;
      }
  });
  return puppet_roles.length > 0;
}

function openStackRoleFilter(obj) {
  // to be replaced with popup view 
  var os_roles = obj.getRolesByFilter(function(role) {
      if(startswith(role.childType, 'OpenStack') || 
         role.childType == 'HAProxyRole' || 
         role.childType == 'MemcachedRole' || 
         role.childType == 'GaleraRole' || 
         role.childType == 'OpenvSwitchRole') {
          return true;
      }
      return false;
  });
  return os_roles.length > 0;
}

function getSelected() {
  return selected('list');
}

/////////////////////////////////////////////////
// init functions 
/////////////////////////////////////////////////
function initHadoopView() {
  hadoopSecondFilter = location.hash.split('-')[1];
  if ( !(hadoopSecondFilter === undefined) )   refreshLevel = 2;
  hdfskey = getHadoopKey();
  overlay_controller.overlay_filter  = hadoopRoleFilter;
  overlay_controller.node_filter     = hadoopRoleFilter;
  overlay_controller.category_filter = hadoopRoleFilter;
  
  //confoverlays = cluster.find('ConfigurationOverlay').filter( filterHadoopOverlays );
    
  // depending on installed Hadoop Services (e.g., HDFS, YARN, HBase, etc.) build 
  // a list of roles available for manipulation and a list of 'nonmovable' roles  
  overlay_controller.listRoles = new Array();
  overlay_controller.nonmovableRoles = new Array();
  
  document.getElementById('saveCO').setAttribute('oncommand', "checkHadoopRoleAssignment();saveList('ConfigurationOverlay');");

  // Todo: rename it if required 
  //document.getElementById('configoverlayname').setAttribute('label', "Hadoop Configuration Group");

  if (current.object.enableFailover) {
    overlay_controller.listRoles =   overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['HDFS-HA-movable']));
    overlay_controller.nonmovableRoles =   overlay_controller.nonmovableRoles.concat(Object.keys(hadoopTypesToRoles['HDFS-HA-non-movable']));
  } else {
    overlay_controller.listRoles =   overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['HDFS-movable']));
    overlay_controller.nonmovableRoles =   overlay_controller.nonmovableRoles.concat(Object.keys(hadoopTypesToRoles['HDFS-non-movable']));
  } 
  if ( current.checkHadoopService('MRv1') ) { 
    overlay_controller.listRoles =   overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['MRv1-movable']));
    overlay_controller.nonmovableRoles =   overlay_controller.nonmovableRoles.concat(Object.keys(hadoopTypesToRoles['MRv1-non-movable']));
  }  
  if ( current.checkHadoopService('YARN') ) {
    overlay_controller.listRoles =   overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['YARN-movable']));  
    overlay_controller.nonmovableRoles =   overlay_controller.nonmovableRoles.concat(Object.keys(hadoopTypesToRoles['YARN-non-movable']));
  }  
  if ( current.checkHadoopService('HBase') ) { 
    overlay_controller.listRoles =   overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['HBase-movable']));  
    overlay_controller.nonmovableRoles =   overlay_controller.nonmovableRoles.concat(Object.keys(hadoopTypesToRoles['HBase-non-movable']));
  }  
  if ( current.checkHadoopService('Spark') ) {
    overlay_controller.listRoles =   overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['Spark-movable']));  
    overlay_controller.nonmovableRoles =   overlay_controller.nonmovableRoles.concat(Object.keys(hadoopTypesToRoles['Spark-non-movable']));
  }  
  if ( current.checkHadoopService('AccumuloTablet') )  overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['AccumuloTablet']));
  if ( current.checkHadoopService('AccumuloMaster') )  overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['AccumuloMaster']));  
  if ( current.checkHadoopService('Drill') ) overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['Drill']));
  if ( current.checkHadoopService('Pig') ) overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['Pig']));
  if ( current.checkHadoopService('FlinkJobManager') ) overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['FlinkJobManager']));
  if ( current.checkHadoopService('FlinkTaskManager') ) overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['FlinkTaskManager']));
  if ( current.checkHadoopService('StormNimbus') ) overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['StormNimbus']));
  if ( current.checkHadoopService('StormSupervisor') ) overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['StormSupervisor']));
  if ( current.checkHadoopService('KafkaServer') ) overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['KafkaServer']));
  if ( current.checkHadoopService('Hive') )  overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['Hive']));  
  if ( current.checkHadoopService('Sqoop') )  overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['Sqoop']));  
  if ( current.checkHadoopService('ZooKeeper') )  overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['ZooKeeper']));  
  if ( current.checkHadoopService('KMS') )  overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['KMS']));  
  if ( current.checkHadoopService('Zeppelin') )  overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['Zeppelin']));  
  if ( current.checkHadoopService('Cassandra') )  overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['Cassandra']));  
  if ( current.checkHadoopService('AlluxioMaster') )  overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['AlluxioMaster']));  
  if ( current.checkHadoopService('AlluxioWorker') )  overlay_controller.listRoles = overlay_controller.listRoles.concat(Object.keys(hadoopTypesToRoles['AlluxioWorker']));  
  
  var object_type_col = document.getElementById("objecttype");
  if(object_type_col) {
    object_type_col.setAttribute("hidden", true);
  }
}

///////////////////////////////////////////////////////
// edit configuration overlays
///////////////////////////////////////////////////////
function getConfigurationOverlayByKey(key) {
  var k = 0;
  for(; k < data.length; k++) {
      var is_overlay = (data[k].container && 
                        data[k].container.getType() == 'ConfigurationOverlay');
      
      if(is_overlay && data[k].key == key) 
      {
        return data[k].container;
      }
      if(data[k].treeChildren) {
          for(var i =0; i < data[k].treeChildren.length; i++) {
              if(data[k].treeChildren[i].container && 
                 data[k].treeChildren[i].container.getType() == 'ConfigurationOverlay' &&
                 data[k].treeChildren[i].key                 == key) 
              {
                  return data[k].treeChildren[i].container;
              }
              else  if(is_overlay && data[k].treeChildren[i].key == key) {
                  return data[k].container;
              }
          }
      }
  }
  return null;
}

function getObjectByKey(key) {
  var k = 0;
  for(; k < data.length; k++) {
      var is_object = (data[k].container ? true : false);
      if(is_object && data[k].key == key) 
      {
        return data[k].container;
      }
      if(data[k].treeChildren) {
          for(var i =0; i < data[k].treeChildren.length; i++) {
              if(data[k].treeChildren[i].container && 
                 data[k].treeChildren[i].key                 == key) 
              {
                  return data[k].treeChildren[i].container;
              }
              else  if(is_object && data[k].treeChildren[i].key == key) {
                  return data[k].container;
              }
          }
      }
  }
  return null;
}

function goToOverlay() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a configuration overlay");
  } else if(select.length > 1) {
    dock.displayError("Please select only one configuration overlay");
  } else {
    var k = 0;
    var key = null;
    for(; k < data.length; k++) {
      if(data[k].container && data[k].key == select[0]) {
        key = data[k].container.get('uniqueKey');
        break;
      }
      if(data[k].treeChildren) {
        for(var i =0; i < data[k].treeChildren.length; i++) {
          if(data[k].treeChildren[i].container && data[k].treeChildren[i].key == select[0]) {
            key = data[k].treeChildren[i].container.get('uniqueKey');
            break;
          }
        }
      }
      if(key) break;
    }
    if(key) {
      var sub_tabbox = window.parent.document.getElementById('sub-tabbox');
      var config = window.parent.document.getElementById("config_iframe");
      if(sub_tabbox) {
        sub_tabbox.selectedIndex = 0;
        config.contentWindow.refresh(null, key); 
      }
    }
  }
}

function getInfoMessageForOtherObjects(object) {
  var msg = "";
  if(object.object.childType == "PhysicalNode") {
     msg = 'Select ' + object.resolveName() + ' in the Nodes resource menu and edit the roles in the roles tab;'
  }
  else if(object.object.childType == "MasterNode") {
    msg = 'Select ' + object.resolveName() + ' in the Head Nodes resource menu and edit the role in the roles tab';
  }
  else if(object.object.childType == "VirtualNode") {
    msg = 'Select ' + object.resolveName() + ' in the OpenStack resource menu and edit the role in the roles tab';
  }
  else if(object.object.childType == "NodeCategory") {
    msg = 'Select ' + object.resolveName() + ' in the Node Category resource menu and edit the role in the roles tab';
  }
  return msg;
}

function showInfoClickedOnClusterConfig(selected) {
    var overlay = getConfigurationOverlayByKey(selected);
    if(!overlay) {
      var object = getObjectByKey(selected);
      var msg = getInfoMessageForOtherObjects(object);
      if(msg) {
          dock.displayInfo(msg);
      }
      else {
          dock.displayInfo("Please select a configuration overlay.");
      }
      return;
    }
    if(hadoopRoleFilter(overlay)) {
        dock.displayInfo("Select a Big Data instance in the resource menu and edit the roles in the Configuration tab");
    }
    else if(puppetRoleFilter(overlay)) {
        dock.displayInfo("Select Puppet in the resource menu and edit the roles in the Configuration tab");
    }
    else if(openStackRoleFilter(overlay)) {
        dock.displayInfo("Select OpenStack in the resource menu and edit the roles in the Configuration tab");
    }
    else {
        dock.displayInfo("This overlay has been edited in cmsh. Edit the roles with cmsh.");
    }
}

function editObject() {
  if(!current) {
    return;
  }
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a configuration overlay");
  } else if(select.length > 1) {
    dock.displayError("Please select only one configuration overlay");
  } else {
    if(overlay_controller.xulEditOverlayFilename == null) {
        showInfoClickedOnClusterConfig(select[0]);
        return;
    }
    var overlay = getConfigurationOverlayByKey(select[0]);
    if(!overlay) {
      var object = getObjectByKey(select[0]);
      var msg = getInfoMessageForOtherObjects(object);
      dock.displayInfo("Please select a configuration overlay.\n" + msg);
    }
    else {
      var args = overlay_controller.editOverlay(hdfskey, cluster.find(overlay.get('uniqueKey')), current);
      overlay_controller.open_items[overlay.get('uniqueKey')] = 1;
      var tmp  = select[0].split("::");
      if(tmp.length > 1) {
        args['select_tab'] = tmp[tmp.length-1];
      }
      var d = new dock.cvDialog(overlay_controller.xulEditOverlayFilename,args);
      d.onOk = function() {
        // if location hash defined (e.g., set to "#HDFS") then parent of parent window 
        // has to be refreshed
        // similar in other places for New, Clone, Revert operations
        // Note: refreshing for Revert operation is defined in generictreecell.js
	      //overlay_filter  = function(obj) { return true; };
        if ( refreshLevel == 2 )    parent.parent.refresh();
        else    parent.refresh();   
      }
      dock.dialogs.showDialog(d);
      refresh();
    }
  }
}


function newObject() {
  if(overlay_controller.xulEditOverlayFilename == null) {
    return;
  }
  var args = overlay_controller.createNewOverlay(hdfskey, current);
  if(!args) {
    dock.displayError("Cannot create Overlay");
  }
  var d = new dock.cvDialog(overlay_controller.xulEditOverlayFilename + '#new',args);
  d.onOk = function() {
    var overlays = cluster.find('ConfigurationOverlay');
    if ( refreshLevel == 2 )  parent.parent.refresh();
    else    parent.refresh();
  }
  dock.dialogs.showDialog(d);
}



function cloneObject() {
  if(overlay_controller.xulEditOverlayFilename == null) {
    return;
  }
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a configuration overlay");
  } else if(select.length > 1) {
    dock.displayError("Please select only one configuration overlay");
  } else {
    var overlay = getConfigurationOverlayByKey(select[0]);
    if(!overlay) {
      dock.displayError("Please select a configuration overlay");
    }
    else {
      var opener = function(args) {
        if(args) {
          var d = new dock.cvDialog(overlay_controller.xulEditOverlayFilename + '#clone',args);
          d.onOk = function() {
            if ( refreshLevel == 2 )    parent.parent.refresh();
            else    parent.refresh();   
          }
          dock.dialogs.showDialog(d);
        }
      };
      overlay_controller.cloneOverlay(hdfskey, overlay, current, opener);
    }
  }
}

function removeObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a configuration overlay");
  } else if(select.length > 1) {
    dock.displayError("Please select only one configuration overlay");
  } else {
    var overlay = getConfigurationOverlayByKey(select[0]);
    if(!overlay) {
      dock.displayError("Please select a configuration overlay");
    }
    else {
      var args = {};
      var alien_roles =  overlay_controller.getAlienRoles(hdfskey, overlay);
      args['question'] = '';
      if(alien_roles.length > 0) {
        args['question'] = "This overlay contains additional roles:\n" + alien_roles.join("\n") + "\n\n";
      }
      args['question']+= 'Are you sure you want to remove this configuration overlay?';
      var d = new dock.cvDialog('question.xul', args);
      d.onOk = function() {
        for(var i = 0; i < select.length; i++)   cluster.removeObject(overlay.get('uniqueKey'));
        refresh();
      }
      dock.dialogs.showDialog(d);
    }
  }
}

// puppet related functions
function puppetPreview() {
  var puppet_preview_xul = 'puppet/puppetpreview.xul';
  var preselected_nodes = {};
  var overlays = cluster.find('ConfigurationOverlay');
  for(var i = 0; i < overlays.length; i++) {
    if(overlays[i].get('modified')) {
      var nodes = overlays[i].get('nodes');
      for(var j =0; j < nodes.length; j++) {
        preselected_nodes[nodes[j]] = true;
      }
    }
  }
  var args = {
    cluster : cluster,
    preselected_nodes : Object.keys(preselected_nodes)
  }

  var d = new dock.cvDialog(puppet_preview_xul, args);
  d.onOk = function() {
  }
  dock.dialogs.showDialog(d);
}

// hadoop related functions
function getHadoopKey() {
  if ( current.getType() == 'HadoopHDFS' )   return current.get('uniqueKey');
  else   return;
}

function checkHadoopRoleAssignment() {
  var overlays = cluster.find('ConfigurationOverlay');
  var hadoopRoleWarnings = new Array();
  var text = 'Warning: Hadoop role(-s) below are not assigned/configured via Hadoop configuration groups\n\n';

  if ( overlays.length > 0) {
    var map = overlays[0].buildConfigurationOverlayNodeRoleMapping(hdfskey);

    for (var key in map) {
      for(var m = 0; m < map[key].length; m++) {
        if ( map[key][m].name == '___NODE' ) {
          var dat = key.split('/');
          hadoopRoleWarnings.push('Role ' + convertRoleType2ServiceName(dat[1]) + ' explicitly assigned to ' + cluster.find(dat[0]).resolveName());
        }
        if ( map[key][m].name == '___CATEGORY' ) {
          var dat = key.split('/');
          hadoopRoleWarnings.push('Role ' + convertRoleType2ServiceName(dat[1]) + ' implicitly assigned (via category) to ' + cluster.find(dat[0]).resolveName());
        }
      }
    }
  }
 
  if ( hadoopRoleWarnings.length > 0) {
    for(var k = 0; k < hadoopRoleWarnings.length; k++)    text += '\t' + hadoopRoleWarnings[k] + '\n';      
    dock.displayInfo(text + '\nUse cmsh to assign/unassign/configure these roles.');
  }
  
  return;
}


// not in use; left for reference
/*
function getConfigsForRole(rolename) {

  var configs = new Array();

  var overlays = current.cluster.find('ConfigurationOverlay');
  for(var k = 0; k < overlays.length; k++) {
    var roles = overlays[k].get('roles');
    for(var j = 0; j < roles.length; j++) {
      if(roles[j].childType == rolename) {
        for(var i = 0; i < roles[j].config.length; i++) {
          // println ("######################" + roles[j].config[i].hdfs + " ---- " + current.get('uniqueKey'));
          if(roles[j].config[i].hdfs == current.get('uniqueKey')) {
            var role = new dock.cvObject(current.cluster, roles[j], overlays[k], 'roles');
            var config = new dock.cvObject(current.cluster, roles[j].config[i], role, 'config');
            configs.push(config);
            printall(roles[j].config[i]);
          } 
          // else {
          //   printall(roles[j]);
          //   printall(roles[j].config[i]);
          // }
        }
      }
    }
  }
  return configs;
}
*/
