/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function OverlayController(xul_file) {
  this.role_filter            = null;
  this.overlay_filter         = function(overlay)       { return true; };
  this.node_filter            = function(node)          { return true; };
  this.category_filter        = function(category)      { return true; };
  this.customization_filter   = function(customization) { return customization.enabled; };
  this.customization_entry_filter = function(entry)     { return entry.enabled; };
  this.open_items             = {};
  this.xulEditOverlayFilename = null;

  this.role_label_lookup      = { 'HAProxyRole'                  : 'HAProxy',
                                  'MemcachedRole'                : 'Memcached',
                                  'GaleraRole'                   : 'Galera',
                                  'OpenvSwitchRole'              : 'Open vSwitch',
                                  'OpenStackIdentityApiRole'     : 'Identity',
                                  'OpenStackVolumeApiRole'       : 'Volume API',
                                  'OpenStackVolumeRole'          : 'Volume',
                                  'OpenStackVolumeBackupRole'    : 'Volume Backup',
                                  'OpenStackVolumeSchedulerRole' : 'Volume Scheduler',
                                  'OpenStackComputeApiRole'      : 'Compute API',
                                  'OpenStackComputeApiEC2Role'   : 'Compute API EC2',
                                  'OpenStackComputeApiMetadataRole' : 'Compute API Metadata',
                                  'OpenStackComputeConductorRole'   : 'Compute Conductor',
                                  'OpenStackComputeRole'            : 'Compute Role', 
                                  'OpenStackComputeSchedulerRole'   : 'Compute Scheduler',
                                  'OpenStackComputeVNCProxyRole'    : 'Compute VNC Proxy', 
                                  'OpenStackDashboardRole'          : 'Dashboard',
                                  'OpenStackObjectApiRole'          : 'Object API', 
                                  'OpenStackObjectAccountRole'      : 'Object Account API', 
                                  'OpenStackObjectContainerRole'    : 'Object Container API', 
                                  'OpenStackObjectStoreRole'        : 'Object Store API', 
                                  'OpenStackImageApiRole'           : 'Image API', 
                                  'OpenStackImageRegistryRole'      : 'Image Registry', 
                                  'OpenStackMessageQueueServerRole' : 'Message Queue Server (AMQP)',
                                  'OpenStackNetworkApiRole'         : 'Network API', 
                                  'OpenStackNetworkDHCPAgentRole'   : 'Network DHCP Agent',
                                  'OpenStackNetworkL3AgentRole'     : 'Network L3 Agent',
                                  'OpenStackNetworkLBAgentRole'     : 'Network LB Agent Role',
                                  'OpenStackNetworkMetadataAgentRole' : 'Network Metadata Agent',
                                  'OpenStackNetworkOVSAgentRole'      : 'Network OVS Agent',
                                  'OpenStackOrchestrationRole'      : 'Orchestration',
                                  'OpenStackOrchestrationApiRole'   : 'Orchestration API',

                                  // ELK roles:
                                  'KibanaRole' : 'Kibana', 
                                  'ElasticSearchRole' : 'Elasticsearch',
                                  'LogstashServerRole' : 'Logstash Server',
                                  'LogstashForwarderRole': 'Logstash Forwarder',

                                  // Puppet
                                  'PuppetRole'                      : 'Puppet'
                                };
  this.role_list              = [];
  this.shownButtons           = [ "refresh" ];

  if(xul_file && ( endswith(xul_file, 'clusternoderoles.xul') || 
                   endswith(xul_file, 'clustergroupedroles.xul') )) {
    this.shownButtons.push("goToOverlay");
  }

  this.showButtons = function() {
    for(var i = 0; i < this.shownButtons.length; i++) {
      var button = document.getElementById(this.shownButtons[i]);
      if(button) {
        button.removeAttribute('collapsed');
      }
    }
  }

  this.createNewConfigOverlayName = function (cluster, prefix) {
    var overlays = cluster.find('ConfigurationOverlay');
    var names = {};
    for(var k = 0; k < overlays.length; k++) {
      names[overlays[k].resolveName()]=true;
    }
    var counter = 1;
    var name = prefix + "_" + counter;
    while(names.hasOwnProperty(name)) {
      counter++;
      name = prefix + "_" + counter;
    }
    return name;
  }

  this.createNewConfigOverlayPriority = function (cluster) {
    var overlays = cluster.find('ConfigurationOverlay');
    var max = 490;
    for(var k = 0; k < overlays.length; k++) {
      if( overlays[k].object.toBeRemoved) continue;
      var p = parseInt(overlays[k].get('priority'));
      if( p > max) max = p;
    }
    if( max == 240 ) return 260;
    else if(max == 740) return 760;
    else if(max >= 990)  return 1000;
    else return max + 10;
  }

  this.createNewOverlay = function(hdfskey, current) {
    return null;
  }

  this.editOverlay = function(hdfskey, overlay, current) {
    return null;
  }

  this.getAlienRoles = function(hdfskey, overlay) {
    var filtered_roles = {};
    var roles = [];
    if(hdfskey != undefined) {
      if(typeof(this.role_filter) === 'function') roles = overlay.getHadoopRoles(hdfskey, false).filter(this.role_filter);
      else roles = overlay.getHadoopRoles(hdfskey, false);
    }
    else if(typeof(this.role_filter=='funtion')) {
      roles = overlay.get('roles').filter(this.role_filter);
    }
    else {
      roles = overlay.get('roles');
    }
    for(var i = 0; i < roles.length; i++) {
      if(roles[i].toBeRemoved) continue;
      filtered_roles[roles[i].name] = 1;
    }
    roles = overlay.get('roles');
    var alien_roles = [];
    for(var i = 0; i < roles.length; i++) {
      if(roles[i].toBeRemoved) continue;
      if(!filtered_roles.hasOwnProperty(roles[i].name))  {
        alien_roles.push(roles[i].childType);
      }
    }
    return alien_roles;
  }

  this.cloneOverlay = function(hdfskey, overlay, current, opener) {
    var alien_roles =  this.getAlienRoles(hdfskey, overlay);
    var self = this;
    var do_open = function() {
      var cluster = current.isCluster ? current : current.cluster;
      // use the same priority for a new cloned overlay
      // newcloned.priority = parseInt(overlay.get('priority'));
      var newcloned = overlay.clone();
      newcloned.name = overlay.get('name') + "-cloned";
      var intPriority = parseInt(overlay.get('priority'));
      // Note: new-cloned-overlay priority set to priority + 10 
      if ( intPriority == -1  )         newcloned.priority = -1;
      else if( intPriority == 240)      newcloned.priority = 260;
      else if( intPriority == 740)      newcloned.priority = 760;
      else if ( intPriority >= 990 )    newcloned.priority = 1000;
      else    newcloned.priority = intPriority + 10;
      // setting false to second and third arguments is important as it avoids 
      // switching to main clustergotl overview window when a configuration group 
      // window is closed or cancelled
      var args = {
        object : cluster.addObject(newcloned,false, false),
        controller : self,
        enable_elements : [],
        is_puppet_role :  false 
      };
      self.configureCloneArgs(args,hdfskey);
      opener(args);
    }
    if(alien_roles.length > 0) {
      var args = new Object();
      args['title'] = "Clone Configuration";
      args['question'] = "This overlay contains additional roles:\n" + alien_roles.join("\n") + "\n\nDo you really want to continue?";
      var d = new dock.cvDialog("question.xul", args);
      d.onOk = function() {
        do_open();
      };
      dock.dialogs.showDialog(d);
    }
    else {
      do_open();
    }
  }

  this.configureCloneArgs = function(args){
  }

  this.addSelectableRolesToAddButton = function(overlay, view) {
  }

  this.initTabs = function(current, view) {
  }

  this.attachRole = function(current, roletype) {
  }
  

  /////////////////////////////////////////////////
  /// aggregate functions 
  /////////////////////////////////////////////////
  this.puppetRoleNamePrefix = function(role) {
    return "Puppet::";
  }

  this.createPuppetRoleData = function(ret, role) {
    var role_name = this.puppetRoleNamePrefix(role);
    if(role.preambleDeclarations) {
      var write_preamble = false;
      for(var i = 0; i < role.preambleDeclarations.length; i++) {
        if(role.preambleDeclarations[i] != "") {
          write_preamble = true;
          break;
        }
      }
      if(write_preamble) {
        ret.push({
          key      : role.uniqueKey + "::Preamble",
          modified : role.modified,
          name     : role_name + "Preamble",
          short_name : "Puppet"
        });
      }
    }
    if(role.variableDeclarations.length) {
      ret.push({
        key      : role.uniqueKey + "::Variables",
        modified : role.modified,
        name     : role_name + "Variables",
        short_name : "Puppet"
      });
    }
    for(var i = 0; i < role.classDeclarations.length; i++) {
      ret.push({
        key      : role.uniqueKey + "::" + role.classDeclarations[i].uniqueKey,
        modified : role.modified,
        name     : role_name + role.classDeclarations[i].className,
        short_name : "Puppet"
      });
    }
    for(var i = 0; i < role.resourceDeclarations.length; i++) {
      var titles = role.resourceDeclarations[i].titles;
      for(var k = 0; k < titles.length; k++) {
        var myname = role.resourceDeclarations[i].resourceType + "::" + titles[k];
          ret.push({
            key      : role.uniqueKey + "::" + role.resourceDeclarations[i].uniqueKey,
            modified : role.modified,
            name     : role_name + myname,
            short_name : "Puppet"
          });
      }
    }
    if(ret.length == 0) {
      // no declarations
      role_name = "Puppet";
      ret.push({
        key      : role.uniqueKey,
        modified : role.modified,
        name     : role_name,
        short_name : "Puppet"
      });
    }
  }
  
  this.createRoleData = function(one, hdfskey) {

    var roles = [];
    var tmp_roles = [];
    if(typeof(this.role_filter) === 'function') roles = one.getHadoopRoles(hdfskey, false).filter(this.role_filter);
    else roles = one.getHadoopRoles(hdfskey, false);
    // add none hadoop roles
    if(typeof(hdfskey) === 'undefined') {
      if(typeof(this.role_filter) === 'function') 
        tmp_roles = one.get('roles').filter(this.role_filter);
      else
        tmp_roles = one.get('roles');
    }
    var ret = [];
    for(var j = 0; j < tmp_roles.length; j++) 
    {
      if(!startswith(tmp_roles[j].childType, 'Hadoop'))
      {
        roles.push(tmp_roles[j]);
      }
    } 
    for (var j = 0; j < roles.length; j++) {
      if (!roles[j].toBeRemoved ) { 
        if(startswith(roles[j].childType, 'Puppet')) {
          this.createPuppetRoleData(ret, roles[j]);
        }
        else {
          var role_name = convertRoleType2ServiceName(roles[j].childType);
          if(this.role_label_lookup.hasOwnProperty(role_name)) {
            role_name = this.role_label_lookup[role_name];
          }
          ret.push({
            key      : roles[j].uniqueKey,
            modified : roles[j].modified,
            name     : role_name,
            short_name : role_name
          });
        }
      }
    }
    return ret;
  }



  this.createClusterRoleData = function(cluster, hdfskey) {
    function getAllNodesOfOverlay(overlay, nodes_in_cat) {
      var nodes = overlay.get('nodes');
      var tmp_nodes = {};
      for(var i = 0; i < nodes.length; i++) {
        tmp_nodes[nodes[i]] = 1;
      }
      var overlay_cats = overlay.get('categories');
      for(var i = 0; i < overlay_cats.length; i++) {
        var catkey = overlay_cats[i];
        for(var j =0; j < nodes_in_cat[catkey].length; j++) {
          tmp_nodes[nodes_in_cat[catkey][j]]=1;
        }
      }
      nodes = Object.keys(tmp_nodes);
      return nodes;
    }

    var data = new Array();
    var confoverlays = cluster.find('ConfigurationOverlay').filter(this.overlay_filter);
    var categories  = cluster.find('NodeCategory');
    var nodes = cluster.find('SlaveNode');
    var nodes_in_cat = {};
    for(var k = 0; k < nodes.length; k++) {
      var catkey = nodes[k].get('category');
      if(!nodes_in_cat[catkey]) nodes_in_cat[catkey] = [];
      nodes_in_cat[catkey].push(nodes[k].get('uniqueKey'));
    }
    for(var k = 0; k < confoverlays.length; k++) {
      var one   = confoverlays[k];
      var nodes = getAllNodesOfOverlay(one, nodes_in_cat);
      var roles = this.createRoleData(one,hdfskey);
      var name  = one.resolveName();
      for(var j = 0; j < roles.length; j++) {
        data.push({ 
          clusterref        : cluster,
          key               : one.get('uniqueKey'),
          modified          : one.get('modified'),
          objecttype        : 'Overlay',
          container         : one,
          name              : name,
          priority          : one.get('priority'),
          nodes             : nodes,
          role              : roles[j],
        });
      }
      var customizations = one.get('customizationFiles').filter(this.customization_filter);
      for(var j = 0; j < customizations.length; j++) {
        var entries = customizations[j].entries.filter(this.customization_entry_filter);
        var pseudo_role = { 
          key      : customizations[j].uniqueKey,
          modified : customizations[j].modified,
          name     : "Customization::" + customizations[j].name,
          short_name       : "Customization",
          is_customization : true,
          entries          : entries,
          
        }
        data.push({
          clusterref        : cluster,
          key               : one.get('uniqueKey'),
          modified          : one.get('modified'),
          objecttype        : 'Overlay',
          name              : one.resolveName(),
          priority          : one.get('priority'),
          nodes             : nodes,
          role              : pseudo_role,
          container         : one,
        });
      }
    }
    var categories  = cluster.find('NodeCategory').filter(this.category_filter);
    for(var k = 0; k < categories.length; k++) {
      var one = categories[k];
      var catkey = one.get('uniqueKey');
      var roles = this.createRoleData(one, hdfskey);

      for(var j = 0; j < roles.length; j++) {
        data.push({ 
          clusterref        : cluster,
          key               : one.get('uniqueKey'),
          modified          : one.get('modified'),
          objecttype        : 'Category',
          name              : one.resolveName(),
          priority          : 250,
          nodes              : nodes_in_cat[catkey],
          role              : roles[j],
          container         : one,
        });
      }
    }


    var nodes = cluster.find('Node').filter(this.node_filter);
    for(var k = 0; k < nodes.length; k++)  {
      var one = nodes[k];
      var roles = this.createRoleData(one, hdfskey);
      var key = one.get('uniqueKey');
      for(var j = 0; j < roles.length; j++) {
        data.push({ 
          clusterref        : cluster,
          key               : key,
          modified          : one.get('modified'),
          objecttype        : 'Node',
          name              : one.resolveName(),
          priority          : 750,
          nodes             : [ key ],
          role              : roles[j],
          container         : one,
        });
      }
    }
    return data;
  }


  /////////////////////////////////////////////////
  /// group functions 
  /////////////////////////////////////////////////
  this.groupRoleDataByContainer = function (data) {
    var tmp = {};
    for(var i = 0; i < data.length; i++) {
      var one = data[i];
      if(!tmp[one.key]) {
        tmp[one.key] = { 
          key          : one.key,
          type         : one.type,
          clusterref   : one.clusterref,
          modified     : one.modified,
          objecttype   : one.objecttype,
          name         : one.name,
          container    : one.container,
          priority     : one.priority,
          nodes        : one.nodes,
          roles        : [],
        }; 
      }
      tmp[one.key].roles.push(one.role);
    }
    var keys = Object.keys(tmp);
    var ret  = [];
    for (var i = 0; i < keys.length; i++) {
      var one = tmp[keys[i]];
      var self = this;
      var isOpen = this.open_items.hasOwnProperty(one.key) && this.open_items[one.key];
      one.nodesconfigured = dotFormatKeys(cluster, one.nodes);
      one.treeIsContainer = true;
      one.treeIsOpen      = isOpen; 
      one.treeOnOpenClose = function(open, data) { self.open_items[data.key]=open;  };
      one.treeChildren    = [];
      var str_roles = {};
      for(var j = 0; j < one.roles.length; j++) {
        one.treeChildren.push({
          parent_type : one.objecttype,
          parent_key  : one.key,
          key         : one.key + "::" + one.roles[j].key, 
          roles       : one.roles[j].name,
          modified    : one.roles[j].modified,
          container    : one.container,
        });
        str_roles[one.roles[j].short_name] = 1;
      }
      one.roles = Object.keys(str_roles);
      one.roles.sort();

      if(one.roles.length == 1 && one.treeChildren.length == 1 && 
         one.roles[0] == one.treeChildren[0].roles) {
        // If there is only one role there is no point
        // in adding sub elements
        one.treeIsContainer = true;
        one.treeIsOpen      = false; 
        one.treeChildren    = [];
      }
      ret.push(one);
    }
    return ret;
  }

  this.groupRoleDataByNode = function(data, cluster) {
    var nodes = cluster.find('Node');
    var ret   = [];
    var tmp   = {};
    for(var i = 0; i < data.length; i++) {
      var one = data[i];
      var len = (one && one.nodes) ? one.nodes.length : 0;
      for(var k = 0; k < len; k++)  {
        var node_key = one.nodes[k];
        if(!tmp[node_key]) tmp[node_key] = [];
        tmp[node_key].push(one);
      }
    }
    for(var k = 0; k < nodes.length; k++)  {
      var one = nodes[k];
      var node_key = one.get('uniqueKey');
      var isOpen = this.open_items.hasOwnProperty(node_key) && this.open_items[node_key];
      var self = this;
      if(tmp[node_key]) {
        var item = { 
          key          : node_key,
          type         : one.type,
          clusterref   : one.clusterref,
          modified     : one.modified,
          name         : one.get('hostname'),
          type         : [],
          treeIsContainer : true,
          treeIsOpen      : isOpen,
          treeOnOpenClose : function(open, data) { self.open_items[data.key]=open;  },
          treeChildren    : []
        };
        var roles = this.processRolesOfNode(tmp[node_key]);
        var str_roles = {};
        var one_key = one.get('uniqueKey');
        for(var i = 0; i < roles.length; i++) {
          var role = roles[i];
          str_roles[role.role.short_name] = 1;
          item.treeChildren.push({
            type              : role.role.type,
            key               : one_key + "::" + role.key,
            container         : role.container,
            modified          : role.container.modified,
            roles             : role.role.name,
            objecttype        : role.objecttype,
            objectname        : role.name,
            priority          : role.priority

          });
        }
        item.roles = Object.keys(str_roles);
        item.roles.sort();
        ret.push(item);
      }
    }
    return ret;
  };

  this.groupRoleDataByRole = function(data, cluster) {
    var ret = [];
    var roles_lookup = {};
    var nodes_lookup = {};
    for(var i = 0; i < data.length; i++) {
      var one = data[i];
      if(!roles_lookup[one.role.name]) roles_lookup[one.role.name] = [];
      roles_lookup[one.role.name].push(one);
    }
    var nodes = cluster.find('Node');
    for(var i = 0; i < nodes.length; i++) {
      nodes_lookup[nodes[i].get('uniqueKey')] = nodes[i];
    }
    for(var role_name in roles_lookup) {
      var isOpen = this.open_items.hasOwnProperty(role_name) && this.open_items[role_name];
      var self = this;

      var item = {
        key             : role_name,
        name            : role_name,
        treeIsContainer : true,
        treeIsOpen      : isOpen,
        treeOnOpenClose : function(open, data) { self.open_items[data.key]=open;  },
        treeChildren    : []
      };
      var tmp_roles = roles_lookup[role_name]; 
      var node_gr   = {};
      for(var j = 0; j < tmp_roles.length; j++) {
        var one = tmp_roles[j];
        var len = (one && one.nodes) ? one.nodes.length : 0;
        for(var k = 0; k < len; k++) {
          var node_key = one.nodes[k];
          if(nodes_lookup[node_key]) {
            var node = nodes_lookup[node_key];
            if(typeof(node_gr[node_key]) == "undefined") node_gr[node_key] = [];
            node_gr[node_key].push(one);
          }
        }
      }
      var node_keys = [];
      for(var node_key in node_gr) {
        if(nodes_lookup[node_key]) {
          var node = nodes_lookup[node_key];
          node_keys.push(node_key);
          node_gr[node_key] =  this.processRolesOfNode(node_gr[node_key]);
          for(var i =0; i < node_gr[node_key].length; i++) {
            var one = node_gr[node_key][i];
            item.treeChildren.push({
              key        : "" + node.get('uniqueKey') + "::" + one.key,
              node       : node.get('hostname'),
              objecttype : one.objecttype,
              objectname : one.name,
              priority   : one.priority,
              container  : one.container,
              modified   : one.container.modified,

            });
          }
        }
      }
      item.nodes = dotFormatKeys(cluster, node_keys);
      item.treeChildren.sort(function(a,b) { return a.node > b.node; });
      ret.push(item);
    }
    return ret;
  }

  ///////////////////////////////////////////////////////
  // helpers for grouping 
  ///////////////////////////////////////////////////////
  this.processRolesOfNode = function(roles_list) {
    roles_list.sort(function(a,b) { return a.priority - b.priority; } );
    var customizations = {};
    var ret = [];
    var processed = {};
    for(var i = 0; i < roles_list.length; i++) {
      var role = roles_list[i];
      var container_key = role.container.get('uniqueKey');
      if(role.role && role.role.is_customization) {
        var custom_key = role.role.name;
        if(typeof(customizations[custom_key]) == 'undefined') customizations[custom_key] = {};
        for(var j = 0; j < role.role.entries.length; j++) {
          var entry = role.role.entries[j];
          if(typeof(processed[custom_key]) == 'undefined') processed[custom_key] = {};
          if(!processed[custom_key][container_key]) {
            processed[custom_key][container_key] = true; // avoid duplicates from the same container
            customizations[custom_key][entry.key] = role;
          }
        }
      }
      else {
        ret.push(role);
      }
    }
    for(var i in customizations) {
      for(var j in customizations[i]) {
        ret.push(customizations[i][j]);
      }
    }
    return ret;
  }
}



///////////////////////////////////////////////////////////////
//
// Puppet overlays
//
///////////////////////////////////////////////////////////////
function PuppetOverlayController(xul_file) {
  this.xulEditOverlayFilename = 'configoverlayedit.xul';

  function puppetRoleFilter(obj) {
    var puppet_roles = obj.getRolesByFilter(function(role) {
      if(startswith(role.childType, 'Puppet')) {
         return true;
      }
    });
    return puppet_roles.length > 0;
  }
  if(xul_file && ( endswith(xul_file, 'clusternoderoles.xul') || 
                   endswith(xul_file, 'clustergroupedroles.xul') )) {
    this.shownButtons = [ "refresh", "goToOverlay" ];
  }
  else {
    this.shownButtons = [ "refresh", 
                          "saveCO", 
                          "revert", 
                          "removeConfOverlay", 
                          "cloneConfOverlay", 
                          "newConfOverlay", 
                          "openConfOverlay", 
                          "puppetPreviewButton" ];
  }

  this.overlay_filter       = puppetRoleFilter;
  this.node_filter          = puppetRoleFilter;
  this.category_filter      = puppetRoleFilter;
  this.customization_filter = function(obj)  { return false; };
  this.role_filter          = function(role) { return startswith(role.childType, 'Puppet'); }
  this.is_puppet_view       = true;

  this.puppetRoleNamePrefix = function(role) {
    return "";
  }

  this.getPuppetFromOverlay = function(overlay) {
    var puppet = null;
    var puppets = overlay.cluster.find('Puppet');
    if(puppets && puppets.length > 0) {
      puppet = puppets[0];
    }
    return puppet;
  }

  this.getPuppetRole = function(overlay) {
    var roles = overlay.getRolesByFilter(function(role) {
      return (role.childType == "PuppetRole" && !role.toBeRemoved);
    });
    if(roles.length > 0) {
      return new dock.cvObject(overlay.cluster, roles[0]);
    }
    return null;
  }

  this.initTabs = function(overlay, view) {
    var puppet_role = this.getPuppetRole(overlay);
    if(!puppet_role) return;
    view.addPuppetSettingsTab();
    var found_key = false;
    var iframesrc  = 'chrome://cmgui/content/puppet/configoverlaypuppetclass.xul';
    for(var k = 0; k < puppet_role.object.classDeclarations.length; k++) {
      if(!puppet_role.object.classDeclarations[k].toBeRemoved) {
        var role_conf = {
          label : puppet_role.object.classDeclarations[k].className,
          name : puppet_role.object.classDeclarations[k].uniqueKey,
          src : iframesrc + '#' + encodeURIComponent(puppet_role.object.classDeclarations[k].uniqueKey)
        };
        view.createTab(role_conf, puppet_role.object.classDeclarations[k]);
        //view.createPuppetFrame(puppet_role.object.classDeclarations[k].uniqueKey, puppet_role.object.classDeclarations[k].className);
      }
    }

    for(var k = 0; k < puppet_role.object.resourceDeclarations.length; k++) {
      if(!puppet_role.object.resourceDeclarations[k].toBeRemoved) {
        var title = puppet_role.object.resourceDeclarations[k].titles.length > 0 ? puppet_role.object.resourceDeclarations[k].titles[0] : "";
        var role_conf = {
          label : title,
          name : puppet_role.object.resourceDeclarations[k].uniqueKey,
          src : iframesrc + '#' + encodeURIComponent(puppet_role.object.resourceDeclarations[k].uniqueKey)
        };
        view.createTab(role_conf, puppet_role.object.resourceDeclarations[k],
                       puppet_role.object.resourceDeclarations[k].resourceType,
                       true);
      }
    }
  };


  this.addSelectableRolesToAddButton = function(overlay, view) {
    var puppet = this.getPuppetFromOverlay(overlay);
    if(puppet) {
      var scannedModules = puppet.get('scannedModules');
      var scannedClasses = puppet.get('scannedClasses');
      var modules2classes = {};
      var nagios_group    = {};
      var attachable_roles    = {};
      scannedClasses.sort(function(a,b){
          return a.name > b.name;
      });
      for(var i = 0; i < scannedClasses.length; i++) {
        var module_key = scannedClasses[i].puppetModuleKey;
        var puppetType = scannedClasses[i].puppetType;
        if(puppetType == 'BuiltinTypes' && scannedClasses[i].name.indexOf('nagios') == 0) {
          puppetType = 'BuiltinNagios';
        }
        if(typeof(modules2classes[module_key]) == 'undefined') {
          modules2classes[module_key] = {};
        }
        if(typeof(modules2classes[module_key][puppetType]) == 'undefined') {
          modules2classes[module_key][puppetType] = [];
        }
        modules2classes[module_key][puppetType].push(scannedClasses[i]);
        attachable_roles[scannedClasses[i].uniqueKey] = scannedClasses[i];
      }
      var create_items = function(module_name,cls_list, t) {
        for(var j = 0; j < cls_list.length; j++) {
          view.addMenuItem({
            name   : cls_list[j].uniqueKey,
            label  : cls_list[j].name,
            group  : module_name,
          });
        }
      };

      for(var i = 0; i < scannedModules.length; i++) {
        var module_key = scannedModules[i].uniqueKey;
        if(typeof(modules2classes[module_key])!='undefined') {
          for(var t in modules2classes[module_key]) {
            if(t == 'BuiltinNagios') {
              continue;
            }
            view.addMenuLabel(scannedModules[i].name, t);
            var cls_list  = modules2classes[module_key][t];
            create_items(scannedModules[i].name, cls_list, t);
          }
          if(typeof(modules2classes[module_key]['BuiltinNagios']) != 'undefined') {
            view.addSubmenu(scannedModules[i].name,"Nagios");
            var cls_list  = modules2classes[module_key]['BuiltinNagios'];
            create_items("Nagios", cls_list, 'BuiltinNagios');
          }
        }
      }
    }
  }


  this.createNewOverlay = function(hdfskey, current) {
    var cluster = current.isCluster ? current : current.cluster;
    var newconfgroup = new dock.cvConfigurationOverlay();
    var puppet_role = new dock.cvPuppetRole();
    var args = new Object();
    var puppet = null;
    var puppets = cluster.find('Puppet');
    if(puppets && puppets.length > 0) {
      puppet = puppets[0];
    }
    if(!puppet) {
      return null;
    }
    puppet_role.puppetObject = puppet.object.uniqueKey;
    newconfgroup.priority = this.createNewConfigOverlayPriority(cluster);
    newconfgroup.name = this.createNewConfigOverlayName(cluster, 'PuppetConfiguration');
    newconfgroup.roles = [ puppet_role ];
    cluster.addObject(puppet_role,false,false);
    // setting false to second and third arguments is important as it avoids 
    // switching to main cluster overview window when a configuration group 
    // window is closed or cancelled
    return { 
      is_puppet_role  : true,
      enable_elements : [ "button_preview" ],
      object          : cluster.addObject(newconfgroup,false,false),
      controller      : this };
  }

  this.editOverlay = function(hdfskey, overlay, current) {
    var cluster = current.isCluster ? current : current.cluster;
    var args = { 
      object : overlay,
      controller : this,
      is_puppet_role : true,
      enable_elements : [ "button_preview" ]
    }
    return args;
  }

  this.configureCloneArgs = function(args){
    args.is_puppet_role = true;
    args.enable_elements = [ "button_preview" ];
  }

  this.attachRole = function(overlay, class_key) {
    var puppet_role = this.getPuppetRole(overlay);
    var puppet = this.getPuppetFromOverlay(overlay);
    var puppet_class = null;
    var cluster = current.isCluster ? current : current.cluster;
    if(puppet) {
      var scannedClasses = puppet.get('scannedClasses');
      for(var i = 0; i < scannedClasses.length; i++) {
        if(scannedClasses[i].uniqueKey == class_key) {
          puppet_class = scannedClasses[i];
          break;
        }
      }
      //puppet_class = scannedClasses.find(function(obj) { return obj.uniqueKey == class_key; });
    }
    if(puppet_role && puppet_class) {
      if(puppet_class.puppetType == 'Class') {
        for(var k = 0; k < puppet_role.object.classDeclarations.length; k++) {
          if(!puppet_role.object.classDeclarations[k].toBeRemoved && 
             puppet_role.object.classDeclarations[k].className == puppet_class.name) {
            return puppet_role.object.classDeclarations[k].uniqueKey;
          }
        }
        var declaration = new dock.cvPuppetClassDeclaration();
        declaration.className             = puppet_class.name; 
        declaration.classConfiguration    = [];
        puppet_role.object.classDeclarations.push(declaration);
        declaration.puppetClassFactoryKey = puppet_class.uniqueKey;
        declaration.isDefault             = false;
        declaration.sourceKeys            = [];
        overlay.setModified(true);
        puppet_role.setModified(true);
        var decl = cluster.addObject(declaration,puppet_role,"classDeclarations"); 
        return decl;
      }
      else { 
        var titles = {};
        for(var k = 0; k < puppet_role.object.resourceDeclarations.length; k++) {
          if(!puppet_role.object.resourceDeclarations[k].toBeRemoved) {
            titles[(puppet_role.object.resourceDeclarations[k].titles.length > 0 ? puppet_role.object.resourceDeclarations[k].titles[0] : "")] = 1;
          }
        }
        var counter = 1;
        var new_title = puppet_class.name + "_" + counter;
        while(titles.hasOwnProperty(new_title)) {
          counter++;
          new_title = puppet_class.name + "_" + counter;
        }
        var declaration = new dock.cvPuppetResourceDeclaration();
        declaration.resourceType             = puppet_class.name; 
        declaration.resourceConfiguration    = [];
        declaration.titles                   = [ new_title ];
        puppet_role.object.resourceDeclarations.push(declaration);
        declaration.puppetClassFactoryKey = puppet_class.uniqueKey;
        declaration.isDefault             = false;
        declaration.sourceKeys            = [];
        overlay.setModified(true);
        puppet_role.setModified(true);
        var decl = cluster.addObject(declaration,puppet_role,"resourceDeclarations"); 
        return decl;

      }
    }
  }

  this.detachRole = function(overlay, key) {
    var puppet_role = this.getPuppetRole(overlay);
    for(var k = 0; k < puppet_role.object.classDeclarations.length; k++) {
      if(puppet_role.object.classDeclarations[k].uniqueKey == key) {
        puppet_role.object.classDeclarations[k].toBeRemoved = true;
        overlay.setModified(true);
        puppet_role.setModified(true);
        return key;
      }
    }
    for(var k = 0; k < puppet_role.object.resourceDeclarations.length; k++) {
      if(puppet_role.object.resourceDeclarations[k].uniqueKey == key) {
        puppet_role.object.resourceDeclarations[k].toBeRemoved = true;
        overlay.setModified(true);
        puppet_role.setModified(true);
        return key;
      }
    }
    return "";
  }
}


/////////////////////////////////////////////////////////
//
// Openstack
//
/////////////////////////////////////////////////////////
function OpenStackOverlayController(xul_file) {
  this.xulEditOverlayFilename = 'configoverlayedit.xul';
  if(xul_file && ( endswith(xul_file, 'clusternoderoles.xul') || 
                   endswith(xul_file, 'clustergroupedroles.xul') )) {
    this.shownButtons = [ "refresh", "goToOverlay" ];
  }
  else {
    this.shownButtons = [ "refresh", 
                          "saveCO", 
                          "revert", 
                          "removeConfOverlay", 
                          "cloneConfOverlay", 
                          "newConfOverlay", 
                          "openConfOverlay"
                        ];
  }

  this.role_list = [
      { 'name'       : 'HAProxyRole',
        'long_label' : 'HAProxy Role',
        'group'      : 'Other',
        'group2'     : 'Api',
        'src'        : 'chrome://cmgui/content/openstack/configrole/other/haproxyrole.xul' },
     
      { 'name'       : 'MemcachedRole',
        'long_label' : 'Memcached Role',
        'group'      : 'Other',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/other/memcachedrole.xul' },

      { 'name'       : 'GaleraRole',
        'long_label' : 'Galera Role',
        'group'      : 'Other',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/other/galerarole.xul' },

      { 'name'       : 'OpenvSwitchRole',
        'long_label' : 'Open vSwitch Role',
        'group'      : 'Other',
        'group2'     : '',
        'src'        : 'chrome://cmgui/content/openstack/configrole/other/openvswitchrole.xul' },

      { 'name'       : 'OpenStackIdentityApiRole',
        'long_label' : 'Identity Role (keystone)',
        'group'      : 'Keystone',
        'group2'     : 'Api',
        'src'        : 'chrome://cmgui/content/openstack/configrole/identity/apirole.xul' },


      { 'name'       : 'OpenStackVolumeApiRole',
        'long_label' : 'Volume API Role (cinder-api)',
        'group'      : 'Cinder',
        'group2'     : 'Api',
        'src'        : 'chrome://cmgui/content/openstack/configrole/volume/apirole.xul' },

      { 'name'       : 'OpenStackVolumeRole',
        'long_label' : 'Volume Role (cinder-volume)',
        'group'      : 'Cinder',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/volume/role.xul' },

      { 'name'       : 'OpenStackVolumeBackupRole',
        'long_label' : 'Volume Backup Role (cinder-backup)',
        'group'      : 'Cinder',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/volume/backuprole.xul' },

      { 'name'       : 'OpenStackVolumeSchedulerRole',
        'long_label' : 'Volume Scheduler Role (cinder-scheduler)',
        'group'      : 'Cinder',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/volume/schedulerrole.xul' },

      { 'name'       : 'OpenStackComputeApiRole',
        'long_label' : 'Compute API Role (nova-api)',
        'group'      : 'Nova',
        'group2'     : 'Api',
        'src'        : 'chrome://cmgui/content/openstack/configrole/compute/apirole.xul' },

      { 'name'       : 'OpenStackComputeApiEC2Role',
        'long_label' : 'Compute API EC2 Role (nova-api-ec2)',
        'group'      : 'Nova',
        'group2'     : 'Api',
        'src'        : 'chrome://cmgui/content/openstack/configrole/compute/apiec2role.xul' },
      
      { 'name'       : 'OpenStackComputeApiMetadataRole', 
        'long_label' : 'Compute API Metadata Role (nova-api-metadata)',
        'group'      : 'Nova',
        'group2'     : 'Api',
        'src'        : 'chrome://cmgui/content/openstack/configrole/compute/apimetadatarole.xul' },

      { 'name'       :  'OpenStackComputeConductorRole',
        'long_label' : 'Compute Conductor Role (nova-conductor)',
        'group'      : 'Nova',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/compute/conductorrole.xul' },
      
      { 'name'       : 'OpenStackComputeRole',
        'long_label' : 'Compute Role (nova-compute and libvirt)',
        'group'      : 'Nova',
        'group2'     : 'Compute Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/compute/role.xul' },

      { 'name'       : 'OpenStackComputeSchedulerRole',
        'long_label' : 'Compute Scheduler Role (nova-scheduler)',
        'group'      : 'Nova',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/compute/schedulerrole.xul' },
      
      { 'name'       : 'OpenStackComputeVNCProxyRole',
        'long_label' : 'Compute VNC Proxy Role (nova-novncproxy)',
        'group'      : 'Nova',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/compute/vncproxyrole.xul' },
      
      { 'name'       : 'OpenStackDashboardRole',
        'long_label' : 'Dashboard Role (horizon)',
        'group'      : 'Horizon',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/dashboard/role.xul' },
      
      { 'name'       : 'OpenStackObjectApiRole',
        'long_label' : 'Object API Role (swift-api)',
        'group'      : 'Swift',
        'group2'     : 'Api',
        'src'        : 'chrome://cmgui/content/openstack/configrole/object/apirole.xul' },

      { 'name'       : 'OpenStackObjectAccountRole',
        'long_label' : 'Object API Role (swift-account)',
        'group'      : 'Swift',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/object/accountrole.xul' },
      
      { 'name'       : 'OpenStackObjectContainerRole',
        'long_label' : 'Object API Role (swift-container)',
        'group'      : 'Swift',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/object/containerrole.xul' },
      
      { 'name'       : 'OpenStackObjectStoreRole',
        'long_label' : 'Object API Role (swift-store)',
        'group'      : 'Swift',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/object/storerole.xul' },
      
      { 'name'       : 'OpenStackImageApiRole',
        'long_label' : 'Image API Role (glance-api)',
        'group'      : 'Glance',
        'group2'     : 'Api',
        'src'        : 'chrome://cmgui/content/openstack/configrole/image/apirole.xul' },
      
      { 'name'       : 'OpenStackImageRegistryRole',
        'long_label' : 'Image Registry Role (glance-registry)',
        'group'      : 'Glance',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/image/registryrole.xul' },
      
      { 'name'       : 'OpenStackMessageQueueServerRole',
        'long_label' : 'Message Queue Server Role (AMQP)',
        'group'      : 'Other',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/other/messagequeueserverrole.xul' },
      
      { 'name'       : 'OpenStackNetworkApiRole',
        'long_label' : 'Network API Role (neutron-server)',
        'group'      : 'Neutron',
        'group2'     : 'Api',
        'src'        : 'chrome://cmgui/content/openstack/configrole/network/apirole.xul' },
      
      
      { 'name'       : 'OpenStackOrchestrationRole',
        'long_label' : 'Orchestration Role (heat-engine)',
        'group'      : 'Heat',
        'group2'     : 'Controller Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/orchestration/role.xul' },
      
      { 'name'       : 'OpenStackOrchestrationApiRole',
        'long_label' : 'Orchestration API Role (heat-api)',
        'group'      : 'Heat',
        'group2'     : 'Api',
        'src'        : 'chrome://cmgui/content/openstack/configrole/orchestration/apirole.xul' },

      { 'name'       : 'OpenStackNetworkDHCPAgentRole',
        'long_label' : 'Network DHCP Agent',
        'group'      : 'Neutron',
        'group2'     : 'Network Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/network/dhcpagentrole.xul' },

      { 'name'       : 'OpenStackNetworkL3AgentRole',
        'long_label' : 'Network L3 Agent',
        'group'      : 'Neutron',
        'group2'     : 'Network Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/network/l3agentrole.xul' },

      { 'name'       : 'OpenStackNetworkLBAgentRole',
        'long_label' : 'Network LB Agent Role',
        'group'      : 'Neutron',
        'group2'     : 'Network Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/network/lbagentrole.xul' },

      { 'name'       : 'OpenStackNetworkMetadataAgentRole',
        'long_label' : 'Network Metadata Agent',
        'group'      : 'Neutron',
        'group2'     : 'Network Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/network/metadataagentrole.xul' },

      { 'name'       : 'OpenStackNetworkOVSAgentRole',
        'long_label' : 'Network OVS Agent',
        'group'      : 'Neutron',
        'group2'     : 'Network Node',
        'src'        : 'chrome://cmgui/content/openstack/configrole/network/ovsagentrole.xul' },

      /// ELK Stack ///
      { 'name'       : 'KibanaRole', 
        'long_label' : 'ELK Kibana',
        'group'      : 'ELK Stack', 
        'group2'     : 'ELK Stack', 
        'src'        : 'chrome://cmgui/content/openstack/configrole/elk/kibanarole.xul' }, 

      { 'name'       : 'ElasticSearchRole', 
        'long_label' : 'ELK ElasticSearch',
        'group'      : 'ELK Stack', 
        'group2'     : 'ELK Stack', 
        'src'        : 'chrome://cmgui/content/openstack/configrole/elk/elasticsearchrole.xul' }, 

      { 'name'       : 'LogstashServerRole', 
        'long_label' : 'ELK Logstash Server',
        'group'      : 'ELK Stack', 
        'group2'     : 'ELK Stack', 
        'src'        : 'chrome://cmgui/content/openstack/configrole/elk/logstashserverrole.xul' }, 

      { 'name'       : 'LogstashForwarderRole', 
        'long_label' : 'ELK Logstash Forwarder',
        'group'      : 'ELK Stack', 
        'group2'     : 'ELK Stack', 
        'src'        : 'chrome://cmgui/content/openstack/configrole/elk/logstashforwarderrole.xul' }, 
    ];
  this.groups = [];
  this.roleLookup = {};
  this.xulsrc2roles = {};

  for(var i = 0; i < this.role_list.length; i++) {
    if(this.role_label_lookup.hasOwnProperty(this.role_list[i].name)) {
      this.role_list[i].label = this.role_label_lookup[this.role_list[i].name];
    }
    else {
      this.role_list[i].label = this.role_list[i].name;
    }
    if(!this.role_list[i].hasOwnProperty('long_label')) {
      this.role_list[i].long_label = this.role_list[i].label;
    }

    this.roleLookup[this.role_list[i].name] = this.role_list[i];
    this.xulsrc2roles[this.role_list[i].src] = this.role_list[i];
  }
  var roleLookup = this.roleLookup;
  this.role_filter = function(role) {
    if(role.toBeRemoved) return false;
    return (roleLookup.hasOwnProperty(role.childType));
    //startswith(role.childType, 'OpenStack') || role.childType==="HAProxyRole" || role.childType==="MemcachedRole");
  };

  this.addSelectableRolesToAddButton = function(overlay, view) {
    var roles = overlay.get("roles");
    var assigned_roles = {};
    for(var i = 0; i < roles.length; i++) {
      if(roles[i].toBeRemoved) continue;
      assigned_roles[roles[i].childType] = roles[i];
    };
    for(var i = 0; i < this.role_list.length; i++) {
      view.addMenuItem(this.role_list[i]);
      //if(assigned_roles.hasOwnProperty(this.role_list[i].name)) { 
      //view.disableAddMenuItem(this.role_list[i].name);
     // }
    }
  }

  this.initTabs = function(overlay, view) {
    var roles = overlay.get('roles');
    var role_count = {};
    var group_name = "group";
    if(view.hasOwnProperty("current_view_suffix")) {
      group_name+= view.current_view_suffix;
    }
    for(var i = 0; i < roles.length; i++) {
      if(roles[i].toBeRemoved) continue;
      if(this.roleLookup.hasOwnProperty(roles[i].childType)) {
        var role_conf = this.roleLookup[roles[i].childType];
        var gr = role_conf[group_name];
        if(role_count.hasOwnProperty(gr)) {
          role_count[gr]++;
        }
        else {
          role_count[gr]=1;
        }
      }
    }

    for(var i = 0; i < roles.length; i++) {
      if(roles[i].toBeRemoved) continue;
      if(this.roleLookup.hasOwnProperty(roles[i].childType)) {
        var role_conf = this.roleLookup[roles[i].childType];
        var one = new dock.cvObject(overlay.cluster, roles[i], overlay, 'roles');
        var gr = role_conf[group_name];
        if(role_count[gr] == 1) {
          // only one role in group -> no nested tabs
          view.createTab(role_conf, one, gr);
        }
        else {
          view.createTab(role_conf, one, gr, true);
        } 
      }
    }
  }

  this.attachRole = function(current, roletype) {
    //var declaration = new dock.cvPuppetClassDeclaration();
    var roleConstructor = dock["cv" + roletype];
    var role = new roleConstructor();
    var one  = new dock.cvObject(current.cluster, role, current, "roles");
    var already_assigned = null;
    for(var i = 0; i < current.object.roles.length; i++) {
      if(!current.object.roles[i].toBeRemoved && current.object.roles[i].childType == roletype) {
        already_assigned = current.object.roles[i].uniqueKey;
        break;
      }
    }
    if(already_assigned !== null) {
      return already_assigned;
    }
    current.object.roles.push(role);
    current.setModified(true);
    return one;
  }

  this.detachRole = function(current, key) {
    var ret = "";
    current.setModified(true);
    for(var i =0; i < current.object.roles.length; i++) {
      if(current.object.roles[i].uniqueKey == key) {
        current.object.roles[i].toBeRemoved = true;
        ret = current.object.roles[i].childType;
        break;
      }
    }
    return ret;
  }

  this.createNewOverlay = function(hdfskey, current) {
    var cluster = current.isCluster ? current : current.cluster;
    var newconfgroup = new dock.cvConfigurationOverlay();
    var args = new Object();
    newconfgroup.priority = this.createNewConfigOverlayPriority(cluster);
    newconfgroup.name = this.createNewConfigOverlayName(cluster, 'OpenStackConfiguration');
    newconfgroup.roles = [ ];
    // setting false to second and third arguments is important as it avoids 
    // switching to main cluster overview window when a configuration group 
    // window is closed or cancelled
    return { 
      is_puppet_view : false,
      enable_elements : [ "openstack_grouping" ],
      object         : cluster.addObject(newconfgroup,false,false),
      controller     : this };
  }

  this.configureCloneArgs = function(args){
    args.enable_elements = [ "openstack_grouping" ];
  }


  this.editOverlay = function(hdfskey, overlay, current) {
    var args = { 
      object : overlay,
      controller : this,
      is_puppet_role : false,
      enable_elements : [ "openstack_grouping" ],
    }
    return args;
  }
}

//////////////////////////////////////////////////////////////////////////
//
// HadoopOverlayController 
//
//////////////////////////////////////////////////////////////////////////
function HadoopOverlayController(xul_file) {
  this.xulEditOverlayFilename = 'hadoop/edithadoopconfgroupsettings.xul';
  if(xul_file && ( endswith(xul_file, 'clusternoderoles.xul') || 
                   endswith(xul_file, 'clustergroupedroles.xul') )) {
    this.shownButtons = [ "refresh", "goToOverlay" ];
  }
  else {
    this.shownButtons = [ "refresh", 
                          "saveCO", 
                          "revert", 
                          "removeConfOverlay", 
                          "cloneConfOverlay", 
                          "newConfOverlay", 
                          "openConfOverlay" ];
  }

  this.listRoles = undefined;
  this.nonmovableRoles = undefined;

  this.createNewOverlay = function(hdfskey, current) {
    var cluster = current.isCluster ? current : current.cluster;
    var newconfgroup = new dock.cvConfigurationOverlay();
    var args = new Object();
    newconfgroup.priority = 500;
    args.is_puppet_view = false;
    if ( !(hdfskey === undefined) ) {
      args['hdfs'] = hdfskey;
      args['availroles'] = this.listRoles;
      args['nonmovroles'] = this.nonmovableRoles;
      newconfgroup.name = current.get('name') + "-";
    }
    args['object'] = cluster.addObject(newconfgroup,false,false);
    args['controller'] = this;
    return args;
  }

  this.editOverlay = function(hdfskey, overlay, current) {
    var cluster = current.isCluster ? current : current.cluster;
    var args = { 
      object : overlay,
      controller : this,
      is_puppet_role : false
    }
    if ( !(hdfskey === undefined) ) {
      args['hdfs'] = hdfskey;
      args['availroles'] = this.listRoles;
      args['nonmovroles'] = this.nonmovableRoles;
    }
    return args;
  }

  this.configureCloneArgs = function(args, hdfskey){
    args.enable_elements = [ "openstack_grouping" ];
      if (!(hdfskey === undefined) ) {
        args['hdfs'] = hdfskey;
        args['availroles'] = overlay_controller.listRoles;
        args['nonmovroles'] = overlay_controller.nonmovableRoles;
      }
  }
}

///////////////////////////////////////////////////////////
//
// Factory
//
///////////////////////////////////////////////////////////
function createOverlayController(location_hash, xul_file) {
  if ( location_hash.indexOf('#HADOOP') === 0) {
    return new HadoopOverlayController(xul_file);
  } 
  else if(location_hash.indexOf('#PUPPET') === 0) {
    return new PuppetOverlayController(xul_file);
  }
  else if(location_hash.indexOf('#OPENSTACK') === 0) {
    return new OpenStackOverlayController(xul_file);
  }
  else {
    return new OverlayController(xul_file);
  }
}

PuppetOverlayController.prototype = new OverlayController;
PuppetOverlayController.prototype.constructor = PuppetOverlayController;

OpenStackOverlayController.prototype = new OverlayController;
OpenStackOverlayController.prototype.constructor = OpenStackOverlayController;

HadoopOverlayController.prototype = new OverlayController;
HadoopOverlayController.prototype.constructor = HadoopOverlayController;

