/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var activeMaster = -1;
var timer = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) current = use.getBasePartition(); // FUTURE: Only one partition
  println("Refresh cluster settings" + ' - ' + document.documentURI);

  resolve('primaryMaster', 'MasterNode');
  resolve('failover', 'MasterNode', 'secondaryMaster', true);
  autorefresh('failover', 'secondaryMaster');
  check('failover', false, 'disableAutomaticFailover');

  resolve('failover', 'Network', 'failoverNetwork', true);

  text('failover', '^[0-9]{1,4}$', 'keepalive');
  text('failover', '^[0-9]{1,4}$', 'warntime');
  text('failover', '^[0-9]{1,4}$', 'deadtime');
  text('failover', '^[0-9]{1,4}$', 'initdead');
  text('failover', '^[0-9]{1,4}$', 'quorumTime');

  text('failover', '', 'mountScript');
  text('failover', '', 'unmountScript');

  text('failover', '', 'preFailoverScript');
  text('failover', '', 'postFailoverScript');

  update();
}

function update() {
  var bad = true;
  var fo = current.get('failover');
  if(fo) {
    var secondary = fo.secondaryMaster;
    if(secondary) {
      dock.workerpool.getWorker(current.cluster.actualurl).call('cmpart', 'cmdaemonFailoverFullStatus', null, function(code, data) {
          var bad = true;
          if(data && code == 0) {
            updateFailoverStatus(data);
            bad = false;
          }
          document.getElementById('status').setAttribute('disabled', bad);
          if(bad) {
            document.getElementById('hasfailover').setAttribute('hidden', true);
          } else {
            document.getElementById('hasfailover').removeAttribute('hidden');
          }
          if(code != 0) {
            dock.jsonCallError(this, code, data);
          }
        }, current);
    } else {
      document.getElementById('status').setAttribute('disabled', bad);
      if(bad) {
        document.getElementById('hasfailover').setAttribute('hidden', true);
      } else {
        document.getElementById('hasfailover').removeAttribute('hidden');
      }
    }
  }
}

function stop() {
  if(timer) window.clearTimeout(timer);
}

function updateFailoverStatus(status) {
  var primary = current.cluster.find(current.get('primaryMaster'));
  var secondary = current.cluster.find(current.get('failover').secondaryMaster);

  if(primary && secondary) {
    document.getElementById('primary_hostname').setAttribute('value', primary.get('hostname'));
    document.getElementById('secondary_hostname').setAttribute('value', secondary.get('hostname'));

    if(status.length == 2) {
      activeMaster = 0;
      // primary vs. secondary doesn't mean active and passive, there is a different property for that
      //  actually available in both the primary and secondary status objects(!)
      if(secondary.get('uniqueKey') == status[0].masterKey) activeMaster = 1;

//       if(!startswith(getOS(), "WIN")) { // TODO: enable again, tested with 1.9.2 (not working)
        drawImages(status[activeMaster].error, status[activeMaster].peers, 'primary');
        drawImages(status[1 - activeMaster].error, status[1 - activeMaster].peers, 'secondary');
//       }

      if(activeMaster == 0) {
        document.getElementById('primary_state').setAttribute('value', '(active)');
        document.getElementById('secondary_state').setAttribute('value', '(passive)');
      } else {
        document.getElementById('primary_state').setAttribute('value', '(passive)');
        document.getElementById('secondary_state').setAttribute('value', '(active)');
      }
    }
  }

  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 1000);
}

function drawImages(error, peers, name) {
  if(error) {
    document.getElementById(name + '_icon').setAttribute('src', 'chrome://cmgui/content/icons/Node-Down_32.png');
  } else {
    document.getElementById(name + '_icon').setAttribute('src', 'chrome://cmgui/content/icons/Node-Up_32.png');
  }
  if(peers.length == 1) {
    var leds = document.getElementById(name + '_leds');
    var names = peers[0].status;
    var count = peers[0].failCount;
    while(leds.firstChild) leds.removeChild(leds.firstChild);
    for(var i = 0; i < Math.min(names.length, count.length); i++) {
      var image = document.createElement('image');
      if(error) {
        image.setAttribute('src', './skin/image/LED_blank.png');
      } else if(count[i]) {
        image.setAttribute('src', './skin/image/LED_red.png');
      } else {
        image.setAttribute('src', './skin/image/LED_green.png');
      }
      image.setAttribute('tooltiptext', names[i]);
      image.setAttribute('id', name + '_' + names[i]);
      leds.appendChild(image);
    }
  } else {
    println("TODO: > 2 node failover setup");
  }
}

function failover() {
  if(current.get('modified')) {
    dock.displayError("Partition " + current.resolveName() + " needs to be saved first");
  } else {
    var fo = current.get('failover');
    if(fo) {
      var secondary = fo.secondaryMaster;
      if(secondary) {
        var dest = 0;
        if(activeMaster == 0) {
          dest = fo.secondaryMaster;
        } else if(activeMaster == 1) {
          dest = current.get('primaryMaster');
        } else {
          dock.displayError("Active head node not set");
          return;
        }
        var name = current.cluster.resolveName(dest);
        if(name) {
          var args = new Object();
          args['question'] = "Are you sure you want to make " + name + " the active head node?";
          var d = new dock.cvDialog("question.xul", args);
          d.onOk = function() {
            dock.workerpool.getWorker(current.cluster.actualurl).call('cmpart', 'cmdaemonFailoverMakeMaster', [ dest ], function(code, done) {
              if(code != 0) {
                dock.displayError("Call to cmdaemon for initiating manual failover failed.\n" + code + ": " + done);
                return;
              }
              switch(done) {
                case(2) : dock.displayError(name + " could not be reached (cmdaemon down)"); break;
                case(-2): dock.displayError(name + " is already active head node"); break;
                case(-1): dock.displayError(name + " could not become active head node"); break;
                case(-5): dock.displayError(name + " shutdown in progress"); break;
                case(-6): dock.displayError(name + " could not become master, provisioning subsystem is busy. Try again later"); break;
                case(-7): dock.displayError(name + " is not part of a failover setup"); break;
                case(-9): dock.displayError(name + " failed to start monitoring, not becoming active head node"); break;
                case(-10): dock.displayError(name + " failed to start monitoring, not becoming active head node"); break;
                case(-11): dock.displayError(name + " was unable to find itself in the database, was hostname changed manually?"); break;
                case(-12): dock.displayError(name + " does not have a master role"); break;
                case(-20): dock.displayError("Warning: shared storage failed to start\n" + name + " became active head node, reconnecting your cmgui"); break;
                case(0) : dock.displayInfo( name + " became active head node, reconnecting your cmgui");
                          this.current.reconnectTimeout = window.setTimeout(createObjectCallback(this.current, this.current.reconnnect), 1000);
                          break;
                default:  dock.displayError("Internal error: " + done); break;
              }
            }, current.cluster);
          }
          dock.dialogs.showDialog(d);
        } else {
          dock.displayError("Master not found");
        }
      } else {
        dock.displayError("Failover is not set up");
      }
    }
  }
}
