/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var activeMaster = -1;
var timer = 0;
var hastatus = 0;

var editfile = 'editfailovergroup.xul';

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) current = use.getBasePartition(); // FUTURE: Only one partition
  println("Refresh cluster failover groups" + ' - ' + document.documentURI);

  stop();
  dock.waitIcon();
  update();
}

function update() {
  println("Update cluster failover groups " + Math.floor(Date.now() / 1000));
  dock.waitIcon();
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmpart', 'failoverGroupStatus', null, function(code, data) {
      if(code == 0) {
        println("------------- data -----------");
        setCallbackResult(data);
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
}

function setCallbackResult(value) {
  hastatus = value;

  var frag = document.createDocumentFragment();
  var groups = current.get('failoverGroups');  
  for(var i = 0; i < groups.length; i++) {
    if(!groups[i].toBeRemoved) {
      frag.appendChild(failoverGroupItem(groups[i]));
    }
  }
  frag.appendChild(addfailoverGroupItem());
  
  var box = document.getElementById('failoverGroups');
  while(box.firstChild) box.removeChild(box.firstChild);
  box.appendChild(frag);
  dock.unwaitIcon();
  
  if(groups.length) timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 1000);
}

function stop() {
  if(timer) window.clearTimeout(timer);
}

function addFailoverGroup() {
  var args = new Object();
  args['object'] = current;
  args['subobject'] = 0;
  var d = new dock.cvDialog(editfile, args);
  d.onOk = function() {
    stop();
    update();
  }
  dock.dialogs.showDialog(d);
}

function editFailoverGroup(key) {
  var args = new Object();
  args['object'] = current;
  args['subobject'] = key;
  var d = new dock.cvDialog(editfile, args);
  d.onOk = function() {
    stop();
    update();
  }
  dock.dialogs.showDialog(d);
}

function removeFailoverGroup(key) {
  var groups = current.get('failoverGroups');
  for(var i = 0; i < groups.length; i++) {
    if(groups[i].uniqueKey == key) {
      groups[i].name += "@TO-BE-REMOVED@"; // to prevent duplicate names
      groups[i].toBeRemoved = true;
      current.set('modified', true);
      stop();
      update();
      break;
    }
  }
}

function makeActive(key) {
  dock.waitIcon();
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmpart', 'cmdaemonGroupSwitchActive', [key], function(code, data) {
      dock.unwaitIcon();
      if(code == 0) {
        stop();
        timer = window.setTimeout('update();', 2500);
        makeActiveResult(key, data);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
}

function makeActiveResult(key, result) {
  var msg;
  var node = current.cluster.find(key);
  var hostname = node.resolveName();
  switch(result) {
    case(-9) : msg = "Head not no longer active"; break;
    case(-8) : msg = hostname + " not committed yet?"; break;
    case(-7) : msg = hostname + " not defined server side"; break;
    case(-2) : msg = "Failover group not found"; break;
    case(-1) : msg = "Failover group configuration not found"; break;
    case(1)  : msg = hostname + " already active"; break;
    case(2)  : msg = hostname + " not UP"; break;
    case(0)  : msg = hostname + " becoming active ..."; break;
    default  : msg = "Unknown error: " + result; break;
  }
  dock.displayInfo(msg);
}

function failoverGroupItem(failoverGroup) {
  var active = new Object();
  if(hastatus) {
    for(var i = 0; i < hastatus.length; i++) {
      active[hastatus[i].activeNodeKey] = hastatus[i];
    }
  }
  var box = document.createElement('vbox');
  var box3 = document.createElement('vbox');
  box3.setAttribute('flex', '1');
  box3.setAttribute('class', 'clustertop');
  box3.setAttribute('style', 'padding: 12px;');
  var label = document.createElement('shadowlabel');
  label.setAttribute('class', 'clustercaption');
  label.setAttribute('value', failoverGroup.name);
  box3.appendChild(label);
  var line = document.createElement('hbox');
  line.setAttribute('class', 'staticdottedline');
  box3.appendChild(line);
  
  var nbox = document.createElement('hbox');
  nbox.setAttribute('align', 'top');
  for(var i = 0; i < failoverGroup.nodes.length; i++) {
    var key = failoverGroup.nodes[i];
    var node = current.cluster.find(key);
    if(!node || !node.stateString) continue;
    var state = node.stateString(true);
    var box4 = document.createElement('vbox');
    box4.setAttribute('align', 'center');
    box4.setAttribute('width', '128px');
    box4.setAttribute('style', 'margin-left: 12px; margin-right: 12px;'); //  margin-top: 4px; margin-bottom: 4px;
    var label = document.createElement('shadowlabel');
    label.setAttribute('class', 'clusterproperty');
    if(active.hasOwnProperty(key)) {
      label.setAttribute('value', node.resolveName() + ' (Active)');
    } else {
      label.setAttribute('value', node.resolveName());
    }
    box4.appendChild(label);
    var box5 = document.createElement('box');
    var image = document.createElement('image');
    image.setAttribute('src', "chrome://cmgui/content/icons/Node-" + state + "_32.png");
    box5.appendChild(image);
    box4.appendChild(box5);
    if((failoverGroup.uniqueKey > dock.MAX_LOCAL_KEY) && ((state == 'Up') || (state == 'Up-Health'))  && !active.hasOwnProperty(key)) {
      var button = document.createElement('button');
      button.setAttribute('class', 'graybutton');
      button.setAttribute('label', 'Make active');
      button.setAttribute('oncommand', 'makeActive(' + key + ');');
      box4.appendChild(button);  
    }
    nbox.appendChild(box4);
  }
  box3.appendChild(nbox);
  
  var box2 = document.createElement('hbox');
  box2.setAttribute('align', 'center');
  box2.setAttribute('class', 'clusterbottom');
  var spacer = document.createElement('spacer');
  spacer.setAttribute('width', '3px');
  box2.appendChild(spacer);

  var button = document.createElement('button');
  button.setAttribute('tooltiptext', 'Edit');
  button.setAttribute('class', 'editcluster-button');
  button.setAttribute('oncommand', 'editFailoverGroup(' + failoverGroup.uniqueKey + ');');
  box2.appendChild(button);

  var button = document.createElement('button');
  button.setAttribute('tooltiptext', 'Delete');
  button.setAttribute('class', 'removecluster-button');
  button.setAttribute('oncommand', 'removeFailoverGroup(' + failoverGroup.uniqueKey + ');');
  box2.appendChild(button);

  var spacer = document.createElement('spacer');
  spacer.setAttribute('flex', '1');
  box2.appendChild(spacer);

  var spacer = document.createElement('spacer');
  spacer.setAttribute('width', '5px');
  box2.appendChild(spacer);

  box.appendChild(box3);
  box.appendChild(box2);
  return box;
}

function addfailoverGroupItem() {
  var box = document.createElement('vbox');
  box.setAttribute('class', 'staticdatasmall');
  box.setAttribute('pack', 'center');
  var box2 = document.createElement('hbox');
  box2.setAttribute('align', 'center');
  var button = document.createElement('button');
  button.setAttribute('class', 'addcluster-button');
  button.setAttribute('oncommand', 'addFailoverGroup();');
  box2.appendChild(button);
  var label = document.createElement('shadowlabel');
  label.setAttribute('class', 'clusterproperty');
  label.setAttribute('value', 'Add a new failover group');
  box2.appendChild(label);

  box.appendChild(box2);
  return box;
}
