/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function cvObject(cluster, object, owner, field, setkey) {
  setkey = isundefined(setkey, true);
  this.treeitem = 0;
  this.branch = 0;
  this.treeitemChild = 0;
  this.cluster = cluster;
  this.parent = 0;
  this.object = object;
  if(!this.object.uniqueKey) {
    if(setkey) this.object.uniqueKey = (++this.cluster.nextLocalKey);
    this.object.modified = true;
    this.object.toBeRemoved = false;
  }
  if(typeof(owner) == 'undefined' || owner == 0) {
    this.owner = 0;
    this.field = 0;
    this.url = "key://" + cluster.key + "/" + this.get('baseType') + "/" + this.object.uniqueKey;
  } else {
    this.owner = owner;
    this.field = field;
    this.url = owner.url + "/" + field + "/" + this.object.uniqueKey;
  }
  this.status = 0;
  this.stateFlappingCount = 0
  this.isCluster = false;
  this.sysInfo = 0;
  this.ucsInfo = 0;
  this.dellSettingsInfo = 0;
  this.ucsLog = 0;
  this.identified = -1; // IDENTIFY_DONE 0, UNIDENTIFY_DONE 1, IDENTIFY_UNKNOWN 2, IDENTIFY_FAILED 3, IDENTIFY_UNAVAILABLE 4
  this.isIdentifiedCallbacks = 0;
  this.softwareImageRevisionCache = 0;
}

cvObject.prototype.setTreeitem = function(treeitem, child) {
  child = isundefined(child, false);
  if(child) {
    if(!this.treeitem) {
      this.treeitem = treeitem;
    } else {
      if(!isArray(this.treeitemChild)) this.treeitemChild = new Array();
      this.treeitemChild.push(treeitem);
    }
  } else {
    this.treeitem = treeitem;
  }
}

cvObject.prototype.removeTreeitemFromParent = function(branch) {
  if(isArray(this.treeitemChild)) {
    for(var i = 0; i < this.treeitemChild.length; i++) {
      var item = this.treeitemChild[i].parentNode.parentNode;
      if(item.parentNode == branch) {
        branch.removeChild(item);
        this.treeitemChild.splice(i, 1);
        return true;
      }
    }
  }
  return false;
}

cvObject.prototype.hideBranch = function() {
  if(!!this.treeitem) {
    this.treeitem.parentNode.parentNode.removeAttribute('container');
  }
}

cvObject.prototype.showBranch = function() {
  if(!!this.treeitem) {
    this.treeitem.parentNode.parentNode.setAttribute('container', 'true');
  }
}

cvObject.prototype.updateBranch = function(param, list, hide, data) {
  if(dock.SUBTREE) {
    hide = isundefined(hide, false);
    var old;
    if(typeof(data) == 'undefined') {
      old = this.get(param);
    } else {
      list = this.get(param);
      old = data;
    }
    old.sort();
    list.sort();
    var i = 0;
    var j = 0;
    while((i < old.length) && (j < list.length)) {
      if(old[i] < list[j]) {
        var child = this.cluster.find(old[i]);
        if(!!child) {
          child.removeTreeitemFromParent(this.branch);
        } else {
          println("Unable to find child (1)");
        }
        i++;
      } else if(old[i] > list[j]) {
        var child = this.cluster.find(list[j]);
        if(!!child) {
          this.cluster.addOneObject(child, this.branch);
        } else {
          println("Unable to find child (2)");
        }
        j++;
      } else {
        i++;
        j++;
      }
    }
    while(i < old.length) {
      var child = this.cluster.find(old[i]);
      if(!!child) {
        child.removeTreeitemFromParent(this.branch);
      } else {
        println("Unable to find child (3)");
      }
      i++;
    }
    while(j < list.length) {
      var child = this.cluster.find(list[j]);
      if(!!child) {
        this.cluster.addOneObject(child, this.branch);
      } else {
        println("Unable to find child (4)");
      }
      j++;
    }
    if(hide) {
      if((list.length == 0) && (old.length > 0)) {
        this.hideBranch();
      } else if((list.length > 0) && (old.length == 0)) {
        this.showBranch();
      }
    }
  }
  if(typeof(data) == 'undefined') this.set(param, list);
}

cvObject.prototype.updateStatus = function(key) {
  if(this.get('baseType') == 'Device') {
    dock.workerpool.getWorker(this.cluster.actualurl).call('cmdevice', 'getStatus', this.get('uniqueKey'), function(code, data) {
        if(code == 0) {
          this.current.stateString(data);
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, this);
  }
}

cvObject.prototype.relevantBaseType = function() {
  var r = this.get('baseType');
  if(r === 'OpenStackApiEntity') {
    return this.get('childType');
  }
  return r;
};

cvObject.prototype.rpcType = function() {
  var r = this.get('baseType');
  if(r == 'CMDaemonBackgroundTask') return 'BackgroundTask';
  if(r == 'OpenStackApiEntity') return this.get('childType');
  return r;
}

cvObject.prototype.rpcPlus = function() {
  if((this.get('baseType') == 'CMDaemonBackgroundTask') || (this.get('baseType') == 'OpenStackApiEntity')) return 0;
  return '';
}

cvObject.prototype.rpcGetArgs = function() {
  var args = this.rpcPlus() + this.get('uniqueKey')
  if(this.get('baseType') == 'OpenStackApiEntity') return [this.getParentKey(), args];
  return args;
}

cvObject.prototype.localChanges = function() {
  var r = this.get('baseType');
  if(r == 'CMDaemonBackgroundTask') return false;
  return true;
}

cvObject.prototype.refresh = function(followup) {
  if(this.get('uniqueKey') < MAX_LOCAL_KEY) {
    return this.remove();
  }
  var call = 'get' + this.rpcType();
  var args = this.rpcGetArgs();
  var service = this.cluster.findService(call);
  dock.workerpool.getWorker(this.cluster.actualurl).call(service, call, args, function(code, data) {
      if(code == 0) {
        var oldModified = false;
        if(!!this.current.object) oldModified = this.current.object.modified;
        var children = 0;
        var param = 0;
        var hide = false;
        var oldRack = -1;
        if(dock.SUBTREE) {
          if(this.current.object.baseType == 'NodeGroup') {
            children = this.current.get('nodes');
            param = 'nodes';
          } else if(this.current.object.baseType == 'Device') {
            oldRack = this.current.object.rack;
            if(this.current.object.childType == 'Chassis') {
              children = this.current.get('members');
              param = 'members';
            } else if((this.current.object.childType == 'EthernetSwitch') || (this.current.object.childType == 'IBSwitch') || (this.current.object.childType == 'MyrinetSwitch')) {
              children = this.current.get('members');
              param = 'members';
              hide = true;
            }
          }
        }
        this.current.object = data;
        this.current.updateLabels();
        addHistory(this.current.get('baseType') + " '" + this.current.resolveName() + "' refreshed");
        hasBeenRefreshed(this.current);
        if(oldModified != this.current.get('modified')) {
          modifiedFlagChanged(this.current);
        }
        if(dock.SUBTREE) {
          if(!!children) this.current.updateBranch(param, 0, hide, children);
          if(oldRack >= 0) updateRackSubTree(this.current, oldRack);
        }
        if(typeof(followup)=='function') {
          followup();
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, this);
}


cvObject.prototype.updateLabels = function(property, value) {
  if(this.treeitem) {
    var activePassive = '';
    if(this.cluster.activePassiveKeys.length > 1) {
      var i;
      for(i = 0; i < this.cluster.activePassiveKeys.length; i++) {
        if(this.cluster.activePassiveKeys[i] == this.object.uniqueKey) {
          break;
        }
      }
      if(i == 0) {
        activePassive = ' (active)';
      } else if(i < this.cluster.activePassiveKeys.length) {
        activePassive = ' (passive)';
      }
    }
    // add space because padding does not work
    var text = ' ' + this.resolveName() + activePassive;
    this.treeitem.setAttribute('label', text);
    if(this.treeitemChild && isArray(this.treeitemChild)) {
      for(var i = 0; i < this.treeitemChild.length; i++) {
        this.treeitemChild[i].setAttribute('label', text);
      }
    }
  }
  if(property == "clusterName") {
    this.cluster.treeitem.setAttribute('label', ' ' + value);
    this.cluster.name = value;
    this.cluster.menuitem.setAttribute('label', value);
  }
}

cvObject.prototype.set = function(property, value, change, update) {
  change = isundefined(change, true);
  update = isundefined(update, 0);
  if(property.indexOf('|') < 0) {
    if(this.object.hasOwnProperty(property)) {
      this.object[property] = value;

      var base = this;
      if(change && ((property != 'modified') || value)) base = this.setModified();
      var type = base.get('baseType');
      if((type == 'Device') || (type == 'Category') || (type == 'Partition') || (type == 'Network') ||
         (type == 'SoftwareImage') || (type == 'User') || (type == 'Group') || (type == 'NodeGroup') ||
         (type == 'JobQueue') || (type == 'Profile') || (type == 'Rack') || (type == 'HadoopHDFS') ||
         (type == 'MonConf') || (type == 'Metric') || (type == 'HealthCheck') || (type == 'ThreshAction')) {
        if((update >= 0) && !this.owner) base.updateLabels(property, value);
        if(change && ((update == 0) || (property == 'modified') || (update > 1))) modifiedFlagChanged(base);
      }
    } else {
      println("cvObject.prototype.set: No such property: " + property);
      this.print();
      stackTrace();
      return;
    }
  } else {
    var plist = property.split('|');
    var part = this;
    var owner = this.owner;
    for(var i = 0; i < plist.length; i++) {
      var idx = plist[i].indexOf('(');
      if(idx > 0) {
        var jdx = plist[i].indexOf(')');
        if(!isObject(part)) return;
        var work = part;
        if(!work.hasOwnProperty('isCluster')) work = new cvObject(this.cluster, work, owner);
        if(i + 1 == plist.length) {
          work.set(plist[i].substr(0, idx), value);
        } else {
          part = work.get(plist[i].substr(0, idx));
          if(!isObject(part)) return;
          owner = work;
        }
      } else {
        if(!isObject(part)) return;
        var work = new cvObject(this.cluster, part, owner);
        if(i + 1 == plist.length) {
          work.set(plist[i], value);
        } else {
          owner = work;
          part = work.get(plist[i]);
        }
      }
    }
  }
}

cvObject.prototype.setModified = function(value) {
  value = isundefined(value, true);
  this.object['modified'] = true;
  var owner = this.owner;
  var base = this;
  while(owner) {
    base = owner;
    owner.object['modified'] = true;
    owner = owner.owner;
  }
  return base;
}

cvObject.prototype.has = function(property) {
  return this.object.hasOwnProperty(property);
}

cvObject.prototype.get = function(property) {
  if(property.indexOf('|') < 0) {
    try {
      if(this.object.hasOwnProperty(property)) {
        return this.object[property];
      } else {
        println("cvObject.prototype.get: No such property: " + property);
        this.print();
        stackTrace();
      }
    } catch (e) {
      stackTrace();
    }
  } else { // micSettings(XeonPhiSettings)|overlay
    var plist = property.split('|');
    value = this;
    for(var i = 0; i < plist.length; i++) {
      var idx = plist[i].indexOf('(');
      if(idx > 0) {
        var jdx = plist[i].indexOf(')');
        if(!isObject(value)) return null;
        var work = value;
        if(!work.hasOwnProperty('isCluster')) work = new cvObject(this.cluster, work);
        value = work.get(plist[i].substr(0, idx));
        if(!isObject(value)) return null;
        var type = plist[i].substr(idx + 1, jdx - idx - 1);
      } else {
        if(!isObject(value)) return null;
        var work = new cvObject(this.cluster, value);
        value = work.get(plist[i]);
      }
    }
    return value;
  }
  return null;
}

cvObject.prototype.commit = function(reboot, followup, wait, force) {
  reboot = isundefined(reboot, 0);
  wait = isundefined(wait, false);
  force = isundefined(force, 0);
  var modified = this.object['modified'];
  var add = this.object.uniqueKey < MAX_LOCAL_KEY;
  this.commitWait = 0;
  if(add) {
    print("Add ... ");
    var call = 'add' + this.rpcType();
    var service = this.cluster.findService(call);
    dock.workerpool.getWorker(this.cluster.actualurl).call(service, call, [this.object, force], function(code, data) {
      if(code == 0) {
        var key = 0;
        if(data.hasOwnProperty('newKey')) {
          key = data.newKey;
        } else if(data.hasOwnProperty('key')) {
          key = data.key;
        } else {
          println("!!! no key during add !!!");
          printall(data);
        }
        var taskId = data.taskId;
        var validate;
        if(data.hasOwnProperty('validate')) {
          validate = data.validate;
        } else if(data.hasOwnProperty('validationArray')) {
          validate = data.validationArray;
        } else if(data.hasOwnProperty('result')) {
          validate = data.result;
        } else if(data.hasOwnProperty('err')) {
          validate = data.err;
        }
        if(key) {
          var oldKey = this.current.get('uniqueKey');
          delete this.current.cluster.objects[oldKey];
          this.current.set('uniqueKey', key);
          this.current.set('modified', false);
          this.current.cluster.replaceKey(oldKey, key);
          this.current.cluster.objects[key] = this.current;
          this.current.url = "key://" + this.current.cluster.key + "/" + this.current.get('baseType') + "/" + key;
          if(this.current.treeitem) this.current.treeitem.parentNode.parentNode.setAttribute('id', this.current.url);
          if(this.current.treeitemChild && isArray(this.current.treeitemChild)) {
            for(var i = 0; i < this.current.treeitemChild.length; i++) {
              this.current.treeitemChild[i].parentNode.parentNode.setAttribute('id', this.current.url);
            }
          }
          println("ok, key = " + key + ", modified = " + this.current.get('modified'));
          addHistory(this.current.get('baseType') + " '" + this.current.resolveName() + "' added");
          modifiedFlagChanged(this.current);
          if(isArray(validate) && (validate.length > 0)) {
            var args = new Object();
            args['object'] = this.current;
            args['validate'] = validate;
            var d = new cvDialog("validate.xul", args);
            dialogs.show(d);
          }
          this.current.updateStatus();
          if(typeof(followup) == 'function') {
            println("Calling follow up function");
            followup();
          }
        } else if(isArray(validate)) {
          println("failed.");
          var args = new Object();
          args['object'] = this.current;
          args['validate'] = validate;
          var d = new cvDialog("validate.xul", args);
          dialogs.show(d);
        } else {
          println("failed.");
          dock.displayError("Unable to save " + this.current.get('baseType') + ".");
          printall(data);
          printall(validate);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      this.current.commitWait++;
    }, this);
  } else {
    print("Update ...");
    var call = 'update' + this.rpcType();
    var service = this.cluster.findService(call);
    dock.workerpool.getWorker(this.cluster.actualurl).call(service, call, [this.object, force], function(code, data) {
      if(code == 0) {
        var success;
        if(data.hasOwnProperty('success')) {
          success = data.success;
        } else if(data.hasOwnProperty('result')) {
          success = data.result;
        } else {
          println("!!! no success during update !!!");
          printall(data);
        }
        var taskId = data.taskId;
        var validate;
        if(data.hasOwnProperty('validate')) {
          validate = data.validate;
        } else if(data.hasOwnProperty('validationArray')) {
          validate = data.validationArray;
        } else if(data.hasOwnProperty('result')) {
          validate = data.result;
        } else if(data.hasOwnProperty('err')) {
          validate = data.err;
        }
        if(success) {
          this.current.set('modified', false);
          println("done.");
          addHistory(this.current.get('baseType') + " '" + this.current.resolveName() + "' updated");
          modifiedFlagChanged(this.current);
          if((reboot <= 0) && isArray(validate) && (validate.length > 0)) {
            var args = new Object();
            args['object'] = this.current;
            args['validate'] = validate;
            var d = new cvDialog("validate.xul", args);
            dialogs.show(d);
          } else if(reboot == 1) {
            var keys = [this.current.get('uniqueKey')];
            dock.workerpool.getWorker(this.current.cluster.actualurl).call('cmdevice', 'preboot', [keys], function(code, data) {
              if(code == 0) {
                if(!data[0]) dock.displayInfo("Reboot for " + this.current.resolveName() + " failed, try power reset");
              } else {
                dock.jsonCallError(this, code, data);
              }
            }, this.current);
          } else if(reboot == 2) {
            var keys = [this.current.get('uniqueKey')];
            dock.workerpool.getWorker(this.current.cluster.actualurl).call('cmdevice', 'powerCycle', [keys, 0.0, this.current.cluster.sessionid, true, true], function(code, data) {
              if(code == 0) {
                if(!data) dock.displayInfo("Power reset for " + this.current.resolveName() + " failed");
              } else {
                dock.jsonCallError(this, code, data);
              }
            }, this.current);
          }
          if(typeof(followup) == 'function') {
            println("Calling follow up function");
            followup();
          }
        } else if(isArray(validate)) {
          println("failed.");
          var args = new Object();
          args['object'] = this.current;
          args['validate'] = validate;
          var d = new cvDialog("validate.xul", args);
          dialogs.show(d);
        } else {
          println("failed.");
          dock.displayError("Unable to save " + this.current.get('baseType') + ".");
          printall(data);
          printall(validate);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      this.current.commitWait++;
    }, this);
  }
  this.cluster.updateRack = this.cluster.updateRack || add || (this.get('baseType') == 'MonConf');
  this.cluster.updateHealth = this.cluster.updateHealth || add || (this.get('baseType') == 'MonConf');
  while(wait && (this.commitWait < 1)) {
    dock.mainthread.processNextEvent(true);
  }
  return true;
}

cvObject.prototype.validate = function() {
  if((this.get('baseType') == "User") || (this.get('baseType') == "Group")) {
    dock.displayError("Unable to validate " + this.get('baseType') + ".");
  } else {
    var call = 'validate' + this.get('baseType');
    var service = this.cluster.findService(call);
    dock.workerpool.getWorker(this.cluster.actualurl).call(service, call, this.object, function(code, data) {
      if(code == 0) {
        var args = new Object();
        args['object'] = this.current;
        args['validate'] = data;
        var d = new cvDialog("validate.xul", args);
        dialogs.show(d);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, this);
  }
}

cvObject.prototype.commitNonThreaded = function() {
  this.commit(false, null, true);
  return !this.get('modified');
}

cvObject.prototype.removeAllTreeItems = function() {
  var item = this.treeitem;
  if(item) {
    item = item.parentNode.parentNode;
    if(item) {
      item.parentNode.removeChild(item);
    } else {
      println("Unable to remove " + this.resolveName() + " from parent item");
    }
    if(this.treeitemChild && isArray(this.treeitemChild)) {
      for(var i = 0; i < this.treeitemChild.length; i++) {
        item = this.treeitemChild[i].parentNode.parentNode;
        if(item) {
          item.parentNode.removeChild(item);
        } else {
          println("Unable to remove " + this.resolveName() + " from parent item (2)");
        }
      }
    }
  }
}

cvObject.prototype.updateTreeItemUrl = function() {
  var item = this.treeitem;
  if(item) {
    item = item.parentNode.parentNode;
    if(item) item.setAttribute('id', this.url);
    if(this.treeitemChild && isArray(this.treeitemChild)) {
      for(var i = 0; i < this.treeitemChild.length; i++) {
        item = this.treeitemChild[i].parentNode.parentNode;
        if(item) item.setAttribute('id', this.url);
      }
    }
  }
}

cvObject.prototype.remove = function(arg1, arg2) {
  var item = this.treeitem;
  this.set('toBeRemoved', true, false);
  if(this.get('uniqueKey') < MAX_LOCAL_KEY) {
    addHistory("'" + this.resolveName() + "' removed");
    this.removeAllTreeItems();
    if(dock.currentObject == this) {
      this.cluster.select(0);
    }
    delete this.cluster.objects[this.get('uniqueKey')];
  } else {
    var call = 'remove' + this.rpcType();
    var service = this.cluster.findService(call);
    var args;
    var force = 0; // TODO
    if(this.get('baseType') == 'Job') {
      args = [this.get('childType').replace("Job", "").toLowerCase(), this.object.jobID];
    } else if(typeof(arg1) == 'undefined') {
      args = [this.get('uniqueKey'), force];
    } else if(typeof(arg2) == 'undefined') {
      args = [this.get('uniqueKey'), arg1, force];
    } else {
      args = [this.get('uniqueKey'), arg1, arg2, force];
    }
    if(this.getParentKey() > 0) {
      args.unshift(this.getParentKey());
    }
    dock.workerpool.getWorker(this.cluster.actualurl).call(service, call, args, function(code, data) {
      if(code == 0) {
        var success = false;
        if(data.hasOwnProperty('success')) {
          success = data.success;
        } else if(data.hasOwnProperty('result')) {
          success = data.result;
        } else {
          printall(data);
        }
        var taskId = data.taskId;
        var changes = data.changes;
        if(success) {
          addHistory("'" + this.current.resolveName() + "' removed");
          this.current.removeAllTreeItems();
          if(dock.currentObject == this.current) {
            this.current.cluster.select(0);
          }
          delete this.current.cluster.objects[this.current.get('uniqueKey')];
        } else if(isArray(changes)) {
          this.current.set('toBeRemoved', false, false);
          var args = new Object();
          args['object'] = this.current;
          args['usedby'] = changes;
          var d = new cvDialog("usedby.xul", args);
          dialogs.show(d);
          printall(data);
          printall(changes);
        } else {
          println("failed.");
          this.current.set('toBeRemoved', false, false);
          dock.displayError("Unable to save " + this.current.get('baseType') + ".");
        }
        if(typeof(dock.currentTab.contentWindow.refresh) == 'function') {
          dock.currentTab.contentWindow.refresh();
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, this);
  }
}

cvObject.prototype.print = function() {
  printall(this.object);
}

cvObject.prototype.resolveNameFull = function(clustername) {
  var name = this.resolveName();
  if(this.owner) {
    name = this.owner.resolveNameFull(clustername) + ' : ' + name;
  } else if(clustername) {
    name = this.cluster.name + ' : ' + name;
  }
  return name;
}

cvObject.prototype.resolveName = function() {
  return resolveName(this);
}

cvObject.prototype.show = function(page) {
  // KDR: make sure all non base types are here!
  if((this.get('baseType') != "Metric") && (this.get('baseType') != "ThreshAction") && (this.get('baseType') != "JobQueue") &&
     (this.get('baseType') != "HealthCheck") && (this.get('baseType') != "User") && (this.get('baseType') != "Group")) {
    this.cluster.select(this.object.uniqueKey, page);
    setCurrentObject(this);
  }
}

cvObject.prototype.old = function(old, full) {
  full = isundefined(full, false);
  if(typeof(old) == "undefined" || (old == false)) {
    return dock.cloneEntity(this.object, 0, '', full);
  } else {
    for(var i in old) {
      this.set(i, old[i], false);
    }
  }
  return old;
}

cvObject.prototype.stateString = function(state, health) {
  var set = (typeof(state) != "undefined") && state;
  health = isundefined(health, false);
  if(set) {
    if(typeof(state) == 'boolean') {
      set = false;
      health = state;
    } else {
      this.status = state;
    }
  }
  var s;
  if(!this.status || this.status.forcedClosed) {
    s = "Closed";
  } else {
    switch(this.status.status) {
      case 'PENDING':
      case 'DOWN': s = "Down"; break;
      case 'GOING_DOWN':
      case 'UP': s = "Up"; break;
      case 'VIRTUAL_SMPNODE_MEMBER': s = "Used"; break;
      default: s = "Closed";
    }
  }
  if(DEMO) {
    if(s == "Closed") s = "Up";
  } else if(CLOUDDEMO) {
    var name = this.resolveName();
    if(startswith(name, 'pnode')) {
      var i = myParseInt(this.resolveName().substr(5));
      if(i <= 5) s = "Up";
    } else if(startswith(name, 'vnode')) {
      var i = myParseInt(this.resolveName().substr(5));
//       if((((i + 5) % 7 == 0) || ((i + 1) % 5 == 0) || ((i + 7) % 8 == 0))) s = "Up";
//       s = "Up";
    }
  }

  if(set) {
    try {
      var image = this.treeitem.getAttribute("src");
      var i = image.indexOf('-');
      if(i > 0) {
        if(dock.SCALEMPDEMO && dock.baseChildTypeCheck(this.object, 'PhysicalNode')) s = 'Used';
        if((this.status.healthCheckFailed || this.status.restartRequired > 0) && (s == "Up")) {
          s += "-Health";
          health = false; // see below
        }
        image = image.substr(0, i + 1) + s + "_16.png";
        if(this.treeitem) this.treeitem.setAttribute("src", image);
        if(isArray(this.treeitemChild)) {
          for(var i = 0; i < this.treeitemChild.length; i++) {
            this.treeitemChild[i].setAttribute("src", image);
          }
        }
        if(this == currentObject) {
          image = image.replace("16", "32");
          document.getElementById("titleimage").setAttribute('src', image);
          if(!!currentObject.cluster) {
            document.getElementById("titlecluster").setAttribute('value', currentObject.cluster.name);
            document.getElementById("titlename").setAttribute('value', currentObject.object.resolveName());
          } else {
            document.getElementById("titlename").setAttribute('value', currentObject.name);
            document.getElementById("titlecluster").setAttribute('value', '');
          }
          currentTab.contentWindow.refresh();
        }
      } else {
        println("Status set for a not state containing icon");
      }
    } catch(e) {
//       println("No item for: " + this.resolveName() + " (too soon?)");
    }
  }
  if(health && (s == "Up")) {
    if(this.status.healthCheckFailed || this.status.restartRequired > 0) {
      s += "-Health";
    }
  }
  return s;
}

cvObject.prototype.isUp = function(state) {
  return (this.status.status == "UP");
}

cvObject.prototype.fullStateString = function(state) {
  if(typeof(state) != "undefined") this.stateString(state);
  var s = "Unknown";
  if(DEMO) {
    if(this.status) {
      switch(this.status.status) {
        case 'DOWN': s = "Down"; break;
        case 'GOING_DOWN':
        case 'UP': s = "Installing"; break;
        case 'OPENING': s = "Opening"; break;
        case 'CLOSED': s = "Up"; break;
        case 'INSTALLING': s = "Installing"; break;
        case 'INSTALLER_FAILED': s = "Installer Failed"; break;
        case 'INSTALLER_REBOOTING': s = "Installer Rebooting"; break;
        case 'INSTALLER_CALLINGINIT': s = "Installer Calling Init"; break;
        case 'INSTALLER_UNREACHABLE': s = "Installer Unreachable"; break;
        case 'INSTALLER_BURNING':
        case 'BURNING': s = "Burning"; break;
        case 'PENDING': s = "Pending"; break;
        case 'VIRTUAL_SMPNODE_MEMBER': s = "Aggregated"; break;
        default:;
      }
    }
  } else {
    if(CLOUDDEMO) {
      var name = this.resolveName();
      if(startswith(name, 'pnode')) {
        var i = myParseInt(this.resolveName().substr(5));
        if(i <= 5) this.status.status = Components.interfaces.cvIDevStatus.UP;
      } else if(startswith(name, 'vnode')) {
        var i = myParseInt(this.resolveName().substr(5));
        if(((i + 5) % 7 == 0) || ((i + 1) % 5 == 0) || ((i + 7) % 8 == 0)) this.status.status = Components.interfaces.cvIDevStatus.UP;
      }
    }
    if(this.status) {
      if(this.status.forcedClosed) {
        s  = "Closed";
      } else {
        switch(this.status.status) {
          case 'DOWN': s = "Down"; break;
          case 'GOING_DOWN':
          case 'UP': s = "Up"; break;
          case 'OPENING': s = "Opening"; break;
          case 'CLOSED': s = "Closed"; break;
          case 'INSTALLING': s = "Installing"; break;
          case 'INSTALLER_FAILED': s = "Installer Failed"; break;
          case 'INSTALLER_REBOOTING': s = "Installer Rebooting"; break;
          case 'INSTALLER_CALLINGINIT': s = "Installer Calling Init"; break;
          case 'INSTALLER_UNREACHABLE': s = "Installer Unreachable"; break;
          case 'INSTALLER_BURNING':
          case 'BURNING': s = "Burning"; break;
          case 'PENDING': s = "Pending"; break;
          case 'VIRTUAL_SMPNODE_MEMBER': s = "Aggregated"; break;
          default:;
        }
      }
      var b = false;
      if(this.status.userMessage.length > 0) {
        s += " (" + this.status.userMessage;
        b = true;
      }
      if(this.status.infoMessage.length > 0) {
        if(b) {
          s += ", ";
        } else {
          s += " (";
        }
        b = true;
        s += this.status.infoMessage;
      }
      if(this.status.pingable > 0) {
        if(b) {
          s += ", ";
        } else {
          s += " (";
        }
        b = true;
        s += "pingable";
      }
      if(this.status.sshable > 0) {
        if(b) {
          s += ", ";
        } else {
          s += " (";
        }
        b = true;
        s += "ssh-able";
      }
      if(this.status.restartRequired > 0) {
        if(b) {
          s += ", ";
        } else {
          s += " (";
        }
        b = true;
        s += "restart required";
      }
      if(this.status.provisioningFailed) {
        if(b) {
          s += ", ";
        } else {
          s += " (";
        }
        b = true;
        s += "provisioning failed";
      }
      if(this.status.stateFlapping) {
        if(b) {
          s += ", ";
        } else {
          s += " (";
        }
        b = true;
        s += "state flapping";
        this.stateFlappingCount++;
      } else {
        this.stateFlappingCount = 0;
      }
      if(b) s += ")";
    }
  }
  return s;
}

cvObject.prototype.topViewState = function() {
  if(this.status && !this.status.forcedClosed) {
    switch(this.status.status) {
      case 'GOING_DOWN':
      case 'UP': {
        if(this.status.healthCheckFailed) {
          return "up_health";
        } else {
          return "up";
        }
      }
      case 'DOWN': return "down";
//       case 'OPENING':
//       case 'INSTALLER_UNREACHABLE':
//       case 'PENDING':
//       case 'VIRTUAL_SMPNODE_MEMBER':
//       case 'CLOSED': return "closed";
//       case 'INSTALLING':
//       case 'INSTALLER_FAILED':
//       case 'INSTALLER_REBOOTING':
//       case 'INSTALLER_CALLINGINIT': return "installing";
//       case 'INSTALLER_BURNING':
//       case 'BURNING': return "burning";
      default:;
    }
  }
  return "closed";
}

cvObject.prototype.known = function(type, field) {
  var list = this.cluster.find(type);
  if(isArray(list)) {
    for(var i = 0; i < list.length; i++) {
      list[i] = list[i][field];
    }
    return list;
  } else {
    println("ERROR: cvObject.prototype.known");
    printall(type);
    printall(field);
    return [];
  }
}

cvObject.prototype.getType = function(human, plural) {
  human = isundefined(human, false);
  plural = isundefined(plural, false);
  var type = '';
  if(this.object.hasOwnProperty('childType')) type = this.get('childType');
  if(type.length == 0) type = this.get('baseType');
  if(human) {
    if(type == 'MonConf') {
      type = 'Monitoring Configuration';
    } else if(type == 'MonMetricConf') {
      type = 'Monitoring Metric Configuration';
    } else if(type == 'MonHealthConf') {
      type = 'Monitoring Health Configuration';
    } else if(type == 'ThreshAction') {
      type = 'Action';
    } else if(type == 'ThreshActionConf') {
      type = 'Threshold';
    } else if(type == 'FSMount') {
      type = 'FS Mount';
    } else if(type == 'FSExport') {
      type = 'FS Export';
    } else if(regexMatch(type, '^Network.*Interface$')) {
      type = 'Network Interface';
    } else if(regexMatch(type, '^.*Role$')) {
      type = 'Role';
    }
    if(plural) {
      type = singularToPlural(type);
    }
  } else if(plural) {
    type = singularToPlural(type);
  }
  return type;
}

cvObject.prototype.getClonableParameters = function() {
  return dock.cloneParameters(this.object);
}

cvObject.prototype.getResolveParameters = function() {
  return dock.resolveParameters(this.object);
}

cvObject.prototype.getFromSubfield = function(subfield, alsoremoved) {
  alsoremoved = isundefined(alsoremoved, false);
  var list = new Array();
  var all = this.get(subfield);
  for(var i = 0; i < all.length; i++) {
    if(alsoremoved || !all[i].toBeRemoved) {
      list.push(new dock.cvObject(this.cluster, all[i], this, subfield));
    }
  }
  return list;
}

cvObject.prototype.setFromSubfield = function(subfield, all) {
  var list = new Array();
  for(var i = 0; i < all.length; i++) {
    list.push(all[i].object);
  }
  this.set(subfield, list);
}

cvObject.prototype.subObjectsFromKeys = function(field, keys) {
  var list = this.get(field);
  var objs = new Array();
  for(var i = 0; i < list.length; i++) {
    if(keys.find(list[i].uniqueKey) >= 0) {
      objs.push(new dock.cvObject(this.cluster, list[i], this, field));
    }
  }
  return objs;
}

cvObject.prototype.subObjectFromKey = function(field, key) {
  var list = this.get(field);
  for(var i = 0; i < list.length; i++) {
    if((list[i].uniqueKey == key) || ((key <= -1000) && (-1000 - i == key))) {
      return (new dock.cvObject(this.cluster, list[i], this, field));
    }
  }
  return 0;
}

cvObject.prototype.subObjectFromName = function(field, name) {
  var list = this.get(field);
  for(var i = 0; i < list.length; i++) {
    var one = new dock.cvObject(this.cluster, list[i], this, field);
    if(one.resolveName() == name) {
      return one;
    }
  }
  return 0;
}

cvObject.prototype.deleteSubObjectFromKey = function(field, key) {
  if(typeof(key) == 'string') key = myParseInt(key);
  var list = this.get(field);
  for(var i = 0; i < list.length; i++) {
    if(list[i].uniqueKey == key) {
      //list.splice(i, 1);
      list[i].toBeRemoved = true;
      this.set(field, list);
      return true;
    }
  }
  return false;
}

cvObject.prototype.addSubObject = function(add) {
  var list = this.get(add.field);
  if(isArray(list)) {
    list.push(add.object);
    this.set(add.field, list);
  } else {
    this.set(add.field, add.object);
  }
}

cvObject.prototype.findClusterKeys = function(oldcluster, error) {
  var resolveParams = new Object();
  var rp = this.getResolveParameters();
  for(var i = 0; i < rp.length; i++) {
    var orp = rp[i];
    var name = orp.splice(0, 1);
    resolveParams[name] = orp;
  }
  var params = this.getClonableParameters();
  for(var i = 0; i < params.length; i++) { // elementof
    var prop = params[i];
    if(typeof(resolveParams[prop[0]]) != 'undefined') {
      var eo = resolveParams[prop[0]][1];
      if(eo.length > 0) {
        var data = this.get(prop[0]);
        var name = '';
        var list = this.get(eo);
        for(var j = 0; j < list.length; j++) {
          if(list[j].uniqueKey == data) {
            var tmp = new dock.cvObject(this.cluster, list[j], this, prop[0]);
            name = tmp.resolveName();
            break;
          }
        }
        resolveParams[prop[0]].push(name);
      }
    }
  }
  for(var i = 0; i < params.length; i++) {
    var prop = params[i];
    var data = this.get(prop[0]);
    if(typeof(resolveParams[prop[0]]) == 'undefined') {
      if(isArray(data)) {
        if((data.length > 0) && (typeof(data[0]) == 'object')) {
          for(var j = 0; j < data.length; j++) {
            var tmp = new dock.cvObject(this.cluster, data[j], this, prop[0]);
            tmp.findClusterKeys(oldcluster);
          }
        }
      } else if(typeof(data) == 'object') {
        var tmp = new dock.cvObject(this.cluster, data, this, prop[0]);
        tmp.findClusterKeys(oldcluster);
      }
    } else {
      var eo = resolveParams[prop[0]][1];
      if(eo.length > 0) {
        this.set(prop[0], 0);
        var name = resolveParams[prop[0]][2];
        for(var j = 0; j < list.length; j++) {
          var tmp = new dock.cvObject(this.cluster, list[j], this, prop[0]);
          if(name == tmp.resolveName()) {
            this.set(prop[0], tmp.get('uniqueKey'));
            break;
          }
        }
      } else if(isArray(data)) {
        var list = new Array();
        for(var i = 0; i < data.length; i++) {
          var cco = oldcluster.find(data[i]);
          if(!!cco) {
            var rco = this.cluster.find(cco.getType(), cco.resolveName());
            if(rco) {
              list.push(rco.get('uniqueKey'));
            } else if(isArray(error)) {
              var msg = 'Unable to find ' + cco.getType(true).toLowerCase() + ' ' + cco.resolveName() + ' in cluster ' + this.cluster.name;
              if(error.find(msg) < 0) error.push(msg);
            }
          }
        }
        this.set(prop[0], list);
      } else {
        var cco = oldcluster.find(data);
        if(!!cco) {
          var rco = this.cluster.find(cco.getType(), cco.resolveName());
          if(rco) {
            this.set(prop[0], rco.get('uniqueKey'));
          } else if(isArray(error)) {
            var msg = 'Unable to find ' + cco.getType(true).toLowerCase() + ' ' + cco.resolveName() + ' in cluster ' + this.cluster.name;
            if(error.find(msg) < 0) error.push(msg);
          }
        }
      }
    }
  }
}

cvObject.prototype.copyProps = function(dest, props, resolveParams, propkeys, error) {
  for(var j = 0; j < props.length; j++) {
    var copy = this.get(props[j]);
    if(!resolveParams.hasOwnProperty(props[j])) {
      if((props[j] == 'hosts') && startswith(copy, '!') && endswith(copy, '!') && (copy.length > 2)) {
        if(dest.cluster != this.cluster) {
          var cco = this.cluster.find(copy.substr(1, copy.length - 2));
          var rco = dest.cluster.find(cco.getType(), cco.resolveName());
          if(rco) {
            copy = '!' + rco.get('uniqueKey') + '!';
          } else {
            var msg = 'Unable to find ' + cco.getType(true).toLowerCase() + ' ' + cco.resolveName() + ' in cluster ' + dest.cluster.name;
            if(error.find(msg) < 0) error.push(msg);
            copy = 'Not found: ' + cco.resolveName();
          }
        } else {
          copy = dest.cluster.setUniqueKey(copy, this.cluster);
        }
      } else {
        copy = dest.cluster.setUniqueKey(copy, this.cluster);
      }
    } else if(dest.cluster != this.cluster) {
      if(isArray(copy)) {
        for(var k = 0; k < copy.length; k++) {
          var cco = this.cluster.find(copy[k]);
          if(cco) {
            var rco = dest.cluster.find(cco.getType(), cco.resolveName());
            if(rco) {
              copy[k] = rco.get('uniqueKey');
            } else {
              var msg = 'Unable to find ' + cco.getType(true).toLowerCase() + ' ' + cco.resolveName() + ' in cluster ' + dest.cluster.name;
              if(error.find(msg) < 0) error.push(msg);
            }
          }
        }
      } else {
        var cco = this.cluster.find(copy);
        if(cco) {
          var rco = dest.cluster.find(cco.getType(), cco.resolveName());
          if(rco) {
            copy = rco.get('uniqueKey');
          } else {
            var msg = 'Unable to find ' + cco.getType(true).toLowerCase() + ' ' + cco.resolveName() + ' in cluster ' + dest.cluster.name;
            if(error.find(msg) < 0) error.push(msg);
          }
        }
      }
    }
    if((dest.cluster != this.cluster) || (propkeys.find(props[j]) < 0)) {
      dest.set(props[j], copy);
    }
  }
}

cvObject.prototype.clone = function(cluster, error, full) {
  error = isundefined(full, new Array());
  full = isundefined(full, false);
  var clone = dock.cloneEntity(this.object, 0, '', full);
  clone.modified = true;
  clone.toBeRemoved = false;
  if(!!cluster && (this.cluster != cluster)) {
    var tmp = new dock.cvObject(cluster, clone);
    tmp.findClusterKeys(this.cluster, error);
  }
  return clone;
}

cvObject.prototype.getManagementNetwork = function() {
 if(this.get('baseType') == 'Device') {
    if(dock.baseChildTypeCheck(this.object, 'Node')) {
      var man = 0;
      if(baseChildTypeCheck(this.object, 'SlaveNode')) {
        man = this.get('managementNetwork');
        if(!man) {
          var x = this.get('category');
          if(x) {
            x = this.cluster.find(x);
            if(x) man = x.get('managementNetwork');
          }
        }
      }
      if(!man) {
        x = this.get('partition');
        if(x) {
          x = this.cluster.find(x);
          if(x) man = x.get('managementNetwork');
        }
      }
      return man;
    } else {
      return this.get('network');
    }
  } else {
    println('getIPs for ' + this.resolveName());
  }
  return 0;
}

cvObject.prototype.getIPs = function(management) {
  management = isundefined(management, true);
  if(this.get('baseType') == 'Device') {
    if(dock.baseChildTypeCheck(this.object, 'Node')) {
      var man = 0;
      if(management) {
        if(baseChildTypeCheck(this.object, 'SlaveNode')) {
          man = this.get('managementNetwork');
          if(!man) {
            var x = this.get('category');
            if(x) {
              x = this.cluster.find(x);
              if(x) man = x.get('managementNetwork');
            }
          }
        }
        if(!man) {
          x = this.get('partition');
          if(x) {
            x = this.cluster.find(x);
            if(x) man = x.get('managementNetwork');
          }
        }
      }
      var list = [];
      var n = this.get('networks');
      for(var i = 0; i < n.length; i++) {
        if(!management || (n[i].network == man)) {
          list.push([n[i].network, n[i].ip, n[i].name, i]);
        }
      }
      return list;
    } else {
      return [[this.get('network'), this.get('ip')]];
    }
  } else {
    println('getIPs for ' + this.resolveName());
  }
  return [];
}

cvObject.prototype.hasCloudDirectorRole = function() {
  if(this.get('childType') != 'CloudNode') return false;
  var roles = this.get('roles');
  for(var i = 0; i < roles.length; i++) {
    if(roles[i].childType == 'CloudDirectorRole') return true;
  }
  return false;
}

cvObject.prototype.getCloudRegionKey = function() {
  if(this.get('childType') != 'CloudNode') return 0;
  var cloudsettings = this.get('cloudSettings');
  if(cloudsettings) {
    if(cloudsettings.childType == 'EC2Settings') {
      var region = cloudsettings.region;
      if(!region) {
        var provider = this.cluster.find(cloudsettings.provider);
        if(provider) region = provider.get('defaultRegion');
      }
      return region;
    }
  }
  return 0;
}

cvObject.prototype.getCloudRegion = function() {
  var region = this.getCloudRegionKey();
  if(region) return this.cluster.find(region);
  return 0;
}

cvObject.prototype.getRack = function() {
  if(this.get('childType') == 'CloudNode') return this.getCloudRegionKey();  
  return this.get('rack').rack;
}

cvObject.prototype.getRackRoomName = function() {
  var room = trim(this.get('room'));
  if(room == '') room = 'Untitled Room';
  return room;
}

cvObject.prototype.shortHostname = function(prefix) {
  var hostname = this.get('hostname');
  if(this.get('childType') == 'MICNode') {
    prefix = isundefined(prefix, false);
    for(var i = hostname.length; i > 0; i--) {
      if((hostname[i - 1] < '0') || (hostname[i - 1] > '9')) {
        if(prefix) return 'mic' + hostname.substr(i);
        return hostname.substr(i);
      }
    }
  }
  return hostname;
}

cvObject.prototype.containerIndex = function() {
  var index = this.get('indexInsideContainer');
  if(this.get('childType') == 'MICNode') {
    return index + 2; // force at 2, host will go at 1
  }
  return index;
}

cvObject.prototype.displayInTree = function() {
  var baseType = this.get('baseType');
  if(baseType == 'SoftwareImage') {
    return (this.get('parentSoftwareImage') == 0);
//     if(this.get('revisionID') == 0) return true;
//     if(this.get('revisionsDescriptionList').length) return true;
//     return false;
  } else if(baseType == 'OpenStack') {
    return false;
  }
  return true;
}

cvObject.prototype.isUCS = function() {
  if((this.get('childType') == 'PhysicalNode') || (this.get('childType') == 'MasterNode')) {
    if(startswith(this.get('powerControl'), 'cimc')) {
      this.cluster.hasUCS = true;
      return true;
    }
    var interfaces = this.get('networks');
    for(var i = 0; i < interfaces.length; i++) {
      if(startswith(interfaces[i].name, 'cimc')) {
        this.cluster.hasUCS = true;
        return true;
      }
    }
  } else if ( (this.get('childType') == 'NodeCategory') ) {
    var nodelist = this.cluster.findNodesIn(this.get('baseType'), this.get('uniqueKey'));
    for ( var i = 0; i < nodelist.length; i++ ) {
      var nodeobj = this.cluster.find(nodelist[i]);
      if (nodeobj) {
        var interfaces = nodeobj.get('networks');
        for(var j = 0; j < interfaces.length; j++) {
          if(startswith(interfaces[j].name, 'cimc')) {
            return true;
          }
        }
      }
    }
  }
  return false;
}

cvObject.prototype.getLiveUCS = function() {
  return this.ucsInfo;
}

cvObject.prototype.getUCS = function(alsoLive) {
  var isPhysicalNode = (this.get('childType') == 'PhysicalNode');
  if(isPhysicalNode || (this.get('childType') == 'MasterNode')) {
    var ucsInfo = this.get('ucsInfoConfigured');
    if(ucsInfo) return ucsInfo;
    if(isPhysicalNode) {
      var cat = this.cluster.find(this.get('category'));
      var ucsInfo = cat.get('ucsInfoConfigured');
      if(ucsInfo) return ucsInfo;
    }
  }
  if(isundefined(alsoLive, true)) return this.getLiveUCS();
  return 0;
}

cvObject.prototype.applyDellSettings = function() {
  if((this.get('childType') == 'PhysicalNode')) {
    dock.workerpool.getWorker(this.cluster.actualurl).call('cmdevice', 'applyDellSettings', [[this.get('uniqueKey')]], function(code, data) {
        if(code != 0) {
          dock.jsonCallError(this, code, data);
        }
      }
    );
  } else if ( this.get('childType') == 'NodeCategory' ) {
    var nodelist = this.cluster.findNodesIn(this.get('baseType'), this.get('uniqueKey'));
    dock.workerpool.getWorker(this.cluster.actualurl).call('cmdevice', 'applyDellSettings', [nodelist], function(code, data) {
        if(code != 0) {
          dock.jsonCallError(this, code, data);
        }
      }
    );
  }
}

cvObject.prototype.isDell13G = function() {
  if((this.get('childType') == 'PhysicalNode') || (this.get('childType') == 'MasterNode')) {
    if(this.sysInfo) {
      return regexMatch(this.sysInfo.systemName, 'R730|R730XD|R630|R430|R930|FC630|FC430|FC830|M630|M830|C6320|C4130');
    }
    return false;
  } else if ( (this.get('childType') == 'NodeCategory') ) {
    if (this.cluster.hasDellSettings) {
      return true;
    }
    return false;
  }
  return false;
}

cvObject.prototype.getLiveDellSettings = function() {
  return this.dellSettingsInfo;
}

cvObject.prototype.getDellSettings = function(alsoLive) {
  var isPhysicalNode = (this.get('childType') == 'PhysicalNode');
  if(isPhysicalNode || (this.get('childType') == 'MasterNode')) {
    var dellSettingsInfo = this.get('dellSettings');
    if(dellSettingsInfo) return dellSettingsInfo;
    if(isPhysicalNode) {
      var cat = this.cluster.find(this.get('category'));
      var dellSettingsInfo = cat.get('dellSettings');
      if(dellSettingsInfo) return dellSettingsInfo;
    }
  }
  if(isundefined(alsoLive, true)) return this.getLiveDellSettings();
  return 0;
}

cvObject.prototype.getSoftwareImage = function(alsoCategory) {
  alsoCategory = isundefined(alsoCategory, true);
  var proxy = this.get('softwareImageProxy');
  if(!proxy) {
    if(alsoCategory && dock.baseChildTypeCheck(this.object, 'SlaveNode')) {
      var category = this.cluster.find(this.get('category'));
      if(category) return category.getSoftwareImage();
    }    
    return 0;
  } else if(proxy.revisionID < 0) {
    return proxy.parentSoftwareImage;
  } else {
    var parent = this.cluster.find(proxy.parentSoftwareImage);
    if(!parent) return 0;
    var revisions = parent.get('revisionsList');
    if(proxy.revisionID == 0) {
      if(revisions.length) {
        return revisions[revisions.length - 1];
      } else {
        return proxy.parentSoftwareImage;
      }
    } else {
      if(!this.softwareImageRevisionCache) this.softwareImageRevisionCache = new Object();
      if(this.softwareImageRevisionCache.hasOwnProperty(proxy.revisionID)) {
        return this.softwareImageRevisionCache[proxy.revisionID];
      } else {
        var copy = false;
        var i1 = revisions.length;
        if(i1 == 0) return proxy.parentSoftwareImage;
        var i0 = 0;
        while(i0 < i1) {
          var i2 = Math.floor((i0 + i1) / 2);
          var image = this.cluster.find(revisions[i2]);
          if(image) {
            var revisionID = image.get('revisionID');
            if(revisionID == proxy.revisionID) {
              this.softwareImageRevisionCache[revisionID] = revisions[i2];
              return revisions[i2];
            } else if(revisionID < proxy.revisionID) {
              i0 = i2 + 1;
            } else {
              i1 = i2;
            }
          } else {
            println('revisionsList of ' + parent.resolveName() + ' contains bad image at ' + i2);
            if(!copy) { // make a copy: because of remove below
              copy = true;
              revisions = revisions.slice();
            }
            revisions.splice(i2, 1);
          }
        }
        println('revisionsList of ' + parent.resolveName() + ' revision not found ' + proxy.revisionID);
        return 0;
      }
    }
  }
}

cvObject.prototype.isHadoop = function() {
  return (this.getHadoopRoles().length > 0);
}


// checkHadoopService() returns true if there is at least one role of
// a given type (types are: 'HDFS','MRv1','YARN','HBase','ZooKeeper','KMS','Spark','Hive','Sqoop') exists,
// returns false otherwise
//
// Checking is done as follows: loop through all configuration overlays, for each overlay only roles
// associated with a current hadoop instance are checked against corresponsding roles of a given type
cvObject.prototype.checkHadoopService = function(hadoopServiceType,allFlag) {
  // mapping between Hadoop service type and Bright role's baseType; 9 (Hadoop-)types as of 25.03.15
  /*
  var hadoopTypesToRoles = { "HDFS": { "HadoopDataNodeRole":true, "HadoopNameNodeRole":true, "HadoopSecondaryNameNodeRole":true, "HadoopJournalRole":true, "HadoopNFSGatewayRole":true },
                             "MRv1": { "HadoopTaskTrackerRole":true, "HadoopJobTrackerRole":true },
                             "YARN": { "HadoopYARNClientRole":true, "HadoopYARNServerRole":true },
                             "HBase": { "HadoopHBaseClientRole":true, "HadoopHBaseServerRole":true },
                             "Spark": { "HadoopSparkWorkerRole":true, "HadoopSparkMasterRole":true, "HadoopSparkYARNRole":true },
                             "ZooKeeper": { "HadoopZooKeeperRole":true },
                             "Hive": { "HadoopHiveRole":true },
                             "Sqoop": { "HadoopSqoopRole":true },
                             "KMS": { "HadoopKMServerRole":true }
                           };
  */
  if ( typeof hadoopServiceType !== 'undefined' && typeof allFlag !== 'undefined' )    return hadoopTypesToRoles[hadoopServiceType];
   
  var overlays = this.cluster.find('ConfigurationOverlay');
  for(var k = 0; k < overlays.length; k++) {
    var roles = overlays[k].getHadoopRoles(this.get('uniqueKey'),false);
    for(var j = 0; j < roles.length; j++) {
      if ( roles[j].childType in hadoopTypesToRoles[hadoopServiceType] ) {
        return true;
      } 
    }
  }

  return false;    
}


cvObject.prototype.getHadoopRoles = function(hdfsKey, alsoCategory) {
  hdfsKey = isundefined(hdfsKey, 0);
  alsoCategory = isundefined(alsoCategory, true);
  if(dock.baseChildTypeCheck(this.object, 'Node') || dock.baseChildTypeCheck(this.object, 'Category') || dock.baseChildTypeCheck(this.object, 'ConfigurationOverlay') ) {
    var hadoopRoles = new Array();
    var roles = this.get('roles');
    var types = new Object();
    for(var i = 0; i < roles.length; i++) {
      if(roles[i].toBeRemoved) continue;
      if(startswith(roles[i].childType, 'Hadoop')) {
        if(hdfsKey == 0) {
          hadoopRoles.push(roles[i]);
          types[roles[i].childType] = roles[i];
        } else {
          for(var j = 0; j < roles[i].config.length; j++) {
            if(roles[i].config[j].hdfs == hdfsKey) {
              hadoopRoles.push(roles[i]);
              types[roles[i].childType] = roles[i];
              break;
            }
          }
        }
      }
    }
    if(alsoCategory && dock.baseChildTypeCheck(this.object, 'SlaveNode')) {
      var category = this.cluster.find(this.get('category'));
      if(category) {
        var croles = category.getHadoopRoles(hdfsKey);
        for(var i = 0; i < croles.length; i++) {
          if(!types.hasOwnProperty(croles[i].childType)) {
            hadoopRoles.push(croles[i]);
          }
        }
      }
    }
    return hadoopRoles;
  }
  return [];
}

cvObject.prototype.buildConfigurationOverlayNodeRoleMapping = function(hdfsKey) {
  hdfsKey = isundefined(hdfsKey, 0);
  var mapCONR = new Object();
  var unodes = {};    // keys of this object are the keys of all the nodes in Configuration Overlays for a given Hadoop instance (defined by hdfsKey)

  // function to populate mapCONR object with priority values for given nodeID and list of roles
  var populateMapCONR = function(nodeID,roles,priorityObj)  {
      for(var m = 0; m < roles.length; m++) {
          var cKey = nodeID + '/' + roles[m].childType;
          if ( !(cKey in mapCONR) ) mapCONR[cKey] = new Array();
          mapCONR[cKey].push(priorityObj);
      }
  };


  if( dock.baseChildTypeCheck(this.object, 'ConfigurationOverlay') ) {
      var clusterobj = this.cluster;
      var uniquenodes = new Object();
      // get list of Configuration Overlays
      var overlays = clusterobj.find('ConfigurationOverlay');

      // loop through all the overlays
      for(var k = 0; k < overlays.length; k++) {
          // get name, priority, nodes and Hadoop-related(!) roles for configuration overlay
          var confname = overlays[k].resolveName();
          var prvalue = overlays[k].get('priority');

          // to ignore disabled conf-overlays, i.e. with priority=-1  
          if ( parseInt(prvalue) == -1 )    continue;
          
          var nodes = overlays[k].get('nodes');
          var overroles = overlays[k].getHadoopRoles(hdfsKey, false);

          // object to store priority for role confugurations in Configuration Overlay
          var cop = { 'name': confname, 'priority': prvalue };

          for(var n = 0; n < nodes.length; n++) {
              unodes[nodes[n]] = 1;
              populateMapCONR(nodes[n],overroles,cop);
          }
      }

      // loop through all the nodes in Configuration Overlays for a given Hadoop instance
      for (var key in unodes) {
          var nodeobj = clusterobj.find(key);

          // get Hadoop-related roles for node object (use 'false' as a second arg to avoid merging with roles due to node's category)
          var noderoles = nodeobj.getHadoopRoles(hdfsKey, false);
          // object to store node's priority for role configurations ('name' set to '___NODE', 'priority' hardcodedly set to 750)
          var cop = { 'name': '___NODE', 'priority': 750 };
          populateMapCONR(key,noderoles,cop);

          // head node is not in any of the categories, thus don't check its categories' roles
          if(nodeobj.getType() != 'MasterNode') {
              var nodecategory = nodeobj.get('category');
              var catobj = clusterobj.find(nodecategory);

              //printall(catobj);

              // get Hadoop-related roles for node's category (use 'false' as a second arg in 'getHadoopRoles')
              var catroles = catobj.getHadoopRoles(hdfsKey, false);
              // object to store category's priority for role configurations ('name' set to '___CATEGORY', 'priority' hardcodedly set to 250)
              cop = { 'name': '___CATEGORY', 'priority': 250 };
              populateMapCONR(key,catroles,cop);
          }
      }
    }

  return mapCONR;
}



cvObject.prototype.isCeph = function() {
  return (this.getCephRoles().length > 0);
}

cvObject.prototype.getCephRoles = function(cephKey) {
  cephKey = isundefined(cephKey, 0);
  if(dock.baseChildTypeCheck(this.object, 'Node') || dock.baseChildTypeCheck(this.object, 'Category')) {
    var cephRoles = new Array();
    var roles = this.get('roles');
    var types = new Object();
    for(var i = 0; i < roles.length; i++) {
      if(roles[i].toBeRemoved) continue;
      if(startswith(roles[i].childType, 'Ceph')) {
        cephRoles.push(roles[i]);
        types[roles[i].childType] = roles[i];
      }
    }
    if(dock.baseChildTypeCheck(this.object, 'SlaveNode')) {
      var category = this.cluster.find(this.get('category'));
      if(category) {
        var croles = category.getCephRoles(cephKey);
        for(var i = 0; i < croles.length; i++) {
          if(!types.hasOwnProperty(croles[i].childType)) {
            cephRoles.push(croles[i]);
          }
        }
      }
    }
    return cephRoles;
  }
  return [];
}

cvObject.prototype.isIdentified = function(callback) {
  if(!this.isIdentifiedCallbacks) this.isIdentifiedCallbacks = new Array();
  this.isIdentifiedCallbacks.push(callback);

  if(this.isIdentifiedCallbacks.length <= 1) {
    dock.workerpool.getWorker(this.cluster.actualurl).call('cmdevice', 'isidentified', this.get('uniqueKey'), function(code, data) {
      if(code == 0) {
        this.current.identified = data;
        if(this.current.isIdentifiedCallbacks) {
          for(var i = 0; i < this.current.isIdentifiedCallbacks.length; i++) {
            this.current.isIdentifiedCallbacks[i]();
          }
        }
        this.current.isIdentifiedCallbacks = 0;
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, this);
    if(this.identified >= 0) { // old value
      this.isIdentifiedCallbacks[this.isIdentifiedCallbacks.length - 1]();
    }
  }
}

cvObject.prototype.getParentKey = function() {
  if(this.object.baseType == 'OpenStackApiEntity') return this.get('ownerUniqueKey');
  return 0;
}

cvObject.prototype.resolveNameForUUID = function(uuidAttribute) {
  if(!this.object.baseType == 'OpenStackApiEntity') {
    println("Not an OpenStackApiEntity!");
    return '';
  }

  var keyAttribute = uuidAttribute.substr(0, uuidAttribute.length - 4) + 'Key';
  if(this.object.hasOwnProperty(keyAttribute)) {
    var foreignObject = this.cluster.find(this.object[keyAttribute]);
    if(!!foreignObject) {
      return foreignObject.resolveName();
    }
  }
  return '';
}

cvObject.prototype.resolveNameForUUIDs = function(uuidsAttribute) {
  if(!this.object.baseType == 'OpenStackApiEntity') {
    println("Not an OpenStackApiEntity!");
    return '';
  }

  var keyAttribute = uuidsAttribute.substr(0, uuidsAttribute.length - 5) + 'Keys';
  if(this.object.hasOwnProperty(keyAttribute)) {
    var foreignObjects = this.cluster.find(this.object[keyAttribute]);
    var names = new Array();
    if(!!foreignObjects) {
      for(var i = 0; i < foreignObjects.length; i++) {
        names.push(foreignObjects[i].resolveName());
      }
      return names.join(', ');
    }
  }
  return '';
}

cvObject.prototype.correctAllUUID = function() {
  //println("correctAllUUID");
  for(var attribute in this.object) {
    if(attribute.length>4) {
      if(endswith(attribute, 'UUID')) {
        var keyAttribute = attribute.substr(0, attribute.length - 4) + 'Key';
        var foreignObject = this.cluster.find(this.object[keyAttribute]);
        if(!!foreignObject) {
          this.object[attribute] = foreignObject.object.UUID;
        } else {
          this.object[attribute] = '';
        }
        //printall(this.object);
      } else if(endswith(attribute, 'UUIDs')) {
        var keyAttribute = attribute.substr(0, attribute.length - 5) + 'Keys';
        var foreignObjects = this.cluster.find(this.object[keyAttribute]);
        var UUIDs = new Array();
        for(var i = 0; i < foreignObjects.length; i++) {
          UUIDs.push(foreignObjects[i].object.UUID);
        }
        this.object[attribute] = UUIDs;
        //printall(this.object);
      }
    }
  }
  //printall(this.object);
  return 0;
}

cvObject.prototype.getRolesByFilter = function(filter, alsoCategory) {
  alsoCategory = isundefined(alsoCategory, true);
  if(dock.baseChildTypeCheck(this.object, 'Node') || dock.baseChildTypeCheck(this.object, 'Category') || dock.baseChildTypeCheck(this.object, 'ConfigurationOverlay') ) {
    var filteredRoles = new Array();
    var allRoles = this.get('roles');
    var types = new Object();
    for(var i = 0; i < allRoles.length; i++) {
      if(filter(allRoles[i])) {
        filteredRoles.push(allRoles[i]);
        types[allRoles[i].childType] = allRoles[i];
      }
    }
    if(alsoCategory && dock.baseChildTypeCheck(this.object, 'SlaveNode')) {
      var category = this.cluster.find(this.get('category'));
      if(category) {
        var categoryRoles = category.getRolesByFilter(filter);
        for(var i = 0; i < categoryRoles.length; i++) {
          if(!types.hasOwnProperty(categoryRoles[i].childType)) {
            filteredRoles.push(categoryRoles[i]);
          }
        }
      }
    }
    return filteredRoles;
  }
  return [];
}

cvObject.prototype.getMonitoringConfiguration = function() {
  if(this.object.baseType == 'Category') {
    return this.cluster.find(this.get('monConfId'));
  } else if(this.object.baseType == 'Device') {
    var catMC = 0;
    switch(this.object.childType) {
      case('MasterNode') :            catMC = ALL_MASTERNODES; break;
      case('GenericDevice') :         catMC = ALL_GENERICS; break;
      case('MICNode') :               catMC = ALL_MICNODES; break;
      case('PowerDistributionUnit') : catMC = ALL_POWERDISTRIBUTIONUNITS; break;
      case('RackSensor') :            catMC = ALL_RACKSENSORS; break;
      case('Chassis') :               catMC = ALL_CHASSIS; break;
      case('GpuUnit') :               catMC = ALL_GPUUNITS; break;
      case('EthernetSwitch') :        catMC = ALL_ETHERNETSWITCHES; break;
      case('IBSwitch') :              catMC = ALL_IBSWITCHES; break;
      case('MyrinetSwitch') :         catMC = ALL_MYRINETSWITCHES; break;
      default: 
        var category = this.cluster.find(this.object.category);
        if(category) catMC = category.get('monConfId');
    }
    if(catMC == 0) {
      println("No category MonConf key found for " + this.resolveName());
      return;
    }
    catMC = this.cluster.find(catMC);
    var devMC = this.cluster.find(this.get('derivedMonConfId'));
    if(!devMC) {
      println("No device MonConf found for " + this.resolveName());
      return catMC;
    }
    if(!catMC) {
      println("No category MonConf found for " + this.resolveName());
      return devMC;
    }
    var sumMC = new dock.cvMonConf();
    sumMC.normalPickupInterval = Math.min(catMC.get('normalPickupInterval'), devMC.get('normalPickupInterval'));
    sumMC.normalPickupInterval = Math.min(catMC.get('scrutinyPickupInterval'), devMC.get('scrutinyPickupInterval'));
    sumMC.metrics = [].concat(devMC.get('metrics'), catMC.get('metrics'));
    sumMC.checks = [].concat(devMC.get('checks'), catMC.get('checks'));
//     println("MonConf for " + this.resolveName() + ", metrics: " + sumMC.metrics.length + " (" + devMC.get('metrics').length + "," + catMC.get('metrics').length + ")");
    return new dock.cvObject(this.cluster, sumMC, 0, '', false);
  }
  return null;
}


cvObject.prototype.getAllMonitoringConfigurations = function() {
  var r = []
  var metricsOnly = true;
  if(this.object.baseType == 'Device') {
    metricsOnly = false;
    r.push(this.getMonitoringConfiguration());
  } else if(this.object.baseType == 'Category') {
    r.push(this.getMonitoringConfiguration());
  } else if(this.object.baseType == 'NodeGroup') {
    if((this.get('type') == 'STORAGEPAIR')) metricsOnly = false;
    var nodes = this.get('nodes');
    for(var i = 0; i < nodes.length; i++) {
      var node = this.cluster.find(nodes[i]);
      if(node) r.push(node.getMonitoringConfiguration());
    }    
  } else if(this.object.baseType == 'MonConf') {
    r.push(this); // ??
  } else if(this.object.baseType == 'CMDaemonFailoverGroup') {
    metricsOnly = false;
    var nodes = this.get('nodes');
    for(var i = 0; i < nodes.length; i++) {
      var node = this.cluster.find(nodes[i]);
      if(node) r.push(node.getMonitoringConfiguration());
    }    
  }
  for(var i = r.length - 1; i >= 0; i--) {
    if(r[i] === null) {
      r.splice(i, 1);
    } else if(metricsOnly) {
      r[i].object.checks = [];
    }
  }
  return r;
}
