/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var rack = 0;
var oldcluster = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  println("Refresh cluster rackview" + ' - ' + document.documentURI);
  if(!!use) {
    var monconf = use.find(dock.ALL_MASTERNODES);
    if(!monconf) {
      var position = document.getElementById('rackposition');
      while(position.firstChild) position.removeChild(position.firstChild);
      var caption = document.getElementById('rackcaption');
      while(caption.firstChild) caption.removeChild(caption.firstChild);
      var data = document.getElementById('rackdata');
      while(data.firstChild) data.removeChild(data.firstChild);
      var legend = document.getElementById('legend');
      while(legend.firstChild) legend.removeChild(legend.firstChild);

      var label = document.createElement('shadowlabel');
      label.setAttribute('class', 'staticdataheader');
      label.setAttribute('value', "No monitoring configuration, most likely because cmdaemon is running with the -M option");
      position.appendChild(label);
      
      document.getElementById('refresh').setAttribute('disabled', true);
      document.getElementById('setup').setAttribute('disabled', true);
      document.getElementById('rackbar').setAttribute('hidden', true);
      
      current = use;
      oldcluster = current;
      return;
    } else {
      document.getElementById('refresh').removeAttribute('disabled');
      document.getElementById('setup').removeAttribute('disabled');
      document.getElementById('rackbar').removeAttribute('hidden');
    }

    current = use;
    if(!rack || (oldcluster != current) || current.updateRack) {
      current.updateRack = false;
      dock.rackBlinkKey = new Array();
      var oldZoom = 0;
      var oldMetric = 0
      var oldFill = true;
      if(rack) {
        oldZoom = rack.zoom;
        oldMetric = rack.metrics;
        oldFill = rack.fillNoData
      }
      rack = new cvRack(current);
      rack.zoom = oldZoom;
      rack.fillNoData = oldFill;
      if(oldMetric) rack.metrics = oldMetric;
    }
    oldcluster = current;
  }
  var devices = current.allKeys('Device');
  if(devices.length < dock.generalSettings.maxRackNodes) {  
    document.getElementById('rackdeck').selectedIndex = 0;
    rack.init();
    rack.refresh();
  } else {
    println('Too many nodes for rackview');
    document.getElementById('toomany').setAttribute('value', 'Too many devices defined, ' + devices.length + ' > ' + dock.generalSettings.maxRackNodes + '. Increasing this number ($HOME:/.cm/cmgui/cmgui.settings) may result in an unusable cmgui.');
    document.getElementById('rackdeck').selectedIndex = 1;
  }
}

function update() {
  if(rack) rack.refresh();
}

function stop() {
  if(rack) rack.stop();
}

function setZoom(value) { // BIG == 0
  document.getElementById('rackzoomout').setAttribute('disabled', value != 0);
  document.getElementById('rackzoomin').setAttribute('disabled', value == 0);
  rack.redraw(value);
}

function setup() {
  var file = "racksetup.xul";
  var old = dock.dialogs.find(file);
  if(old) {
    if(!old.closed) {
      dock.displayError("A rack setup window is already open", window);
      return;
    }
  }
  var args = new Object();
  args['rack'] = rack;
  var d = new dock.cvDialog(file, args);
  d.parentWindow = window;
  dock.dialogs.show(d);
}

function nodetooltipshow(event) {
  var device = document.tooltipNode;
  var key = 0;
  while(!key && device) {
    key = device.getAttribute('key');
    if(kisNaN(key) || !key) {
      key = 0;
      device = device.parentNode;
    }
  }
  if(!key) return false;
  device = current.find(key);
  var state = device.stateString();
  var type = device.get('childType');
  state = type + "-" + state + "_16.png";

  document.getElementById('devicetipimage').setAttribute('src', 'chrome://cmgui/content/icons/' + state);
  document.getElementById('devicetipname').setAttribute('value', device.get('hostname'));
  var realrack = device.get('rack');
  if(realrack) {
    realrack = current.find(realrack);
    if(!!realrack) {
      realrack = realrack.resolveName();
    } else {
      realrack = 'None';
    }
  }
  document.getElementById('devicetipposition').setAttribute('value', 'Rack: ' + realrack + ', position: ' + device.get('rack').position));
  var data = document.getElementById('devicetipdata')
  var values = rack.deviceValues(key);
  var metrics = rack.metricInfo();
  for(var l = 0; l < metrics.length; l++) {
    var nodata = (l >= values.length) || (typeof(values[l]) == "undefined") || kisNaN(values[l]);
    var label;
    if(l + 2 < data.childNodes.length) {
      label = data.childNodes[l + 2];
    } else {
      label = document.createElement('label');
      label.setAttribute('style', 'color: #333333;');
      data.appendChild(label);
    }
    if(nodata) {
      label.setAttribute('value', metrics[l][0] + ': no data');
    } else if(metrics[l][2]) {
      if(values[l] < 0.5) {
        label.setAttribute('value', metrics[l][0] + ': PASS');
      } else if(values[l] > 1.5) {
        label.setAttribute('value', metrics[l][0] + ': FAIL');
      } else {
        label.setAttribute('value', metrics[l][0] + ': UNKNOWN');
      }
    } else {
      label.setAttribute('value', metrics[l][0] + ': ' + formatKMG(values[l]) + metrics[l][1]);
    }
  }
  for(var i = data.childNodes.length - 1; i >= 2 + metrics.length; i--) {
    data.removeChild(data.lastChild);
  }
  return true;
}

function opendevice(key) {
  current.select(key);
}
