/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var tracker = 0;
var notdone = -1;
var timer = 0;
var result;
var badfiles;
var badtexts;
var cm;
var md5sums;
var joinoutput = 0;
var hidedown = false;

var openStreams;
var localWaitCount = 0;

var shellData = 0;

var nodesHostname = 0;
var buildFieldNodes = 0;
var buildFieldNodesIndex = 0;
var buildFieldNodesState = 0

var MAX_OPEN_STREAMS = 512;
var LOAD_FLAGS_BYPASS_CACHE = 256;

var oct0600 = parseInt('0600', 8); // octal literals and octal escape sequences are deprecated

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function badfile(name, text) {
  var path = dock.getCacheDirectory() + name;
  savefile(path, text);
  return 'file://' + path;
}

function createbadfiles() {
  badtexts = new Object();
  badtexts[dock.PRE_NODE_DOWN] = "Node down\n";
  badtexts[dock.PRE_SOAP_ERROR] = "Failed to start (soap error communicating with node)\n";
  badtexts[dock.PRE_NO_SUCH_DEVICE] = "Failed to start (device was deleted on server)\n";
  badtexts[dock.PRE_NO_SUCH_NODE] = "Failed to start (node was deleted on server)\n";
  badtexts[dock.PRE_UNKNOWN] = "Failed to start (unknown error)\n";
  badtexts[dock.PRE_BAD_USER] = "User unknown\n";
  
  badfiles = new Object();
  badfiles[dock.PRE_NODE_DOWN] = badfile('nodedown.txt', badtexts[dock.PRE_NODE_DOWN]);
  badfiles[dock.PRE_SOAP_ERROR] = badfile('soaperror.txt', badtexts[dock.PRE_SOAP_ERROR]);
  badfiles[dock.PRE_NO_SUCH_DEVICE] = badfile('nosuchdevice.txt', badtexts[dock.PRE_NO_SUCH_DEVICE]);
  badfiles[dock.PRE_NO_SUCH_NODE] = badfile('nosuchnode.txt', badtexts[dock.PRE_NO_SUCH_NODE]);
  badfiles[dock.PRE_UNKNOWN] = badfile('unknown.txt', badtexts[dock.PRE_UNKNOWN]);
  badfiles[dock.PRE_BAD_USER] = badfile('baduser.txt', badtexts[dock.PRE_BAD_USER]);
  for(var i in badfiles) {
    var md5 = md5sum(badfiles[i].substr(7));
    badfiles[i] = [badfiles[i], md5];
  }
}

function refresh(use, oldTracker, oldNotdone) {
  if(!!use) {
    createbadfiles();
    if(current != use) {
      document.getElementById("list").value = '';
//       printall(window.frames[0].document);
      var tree = window.frames[0].document.getElementById("tree");
      while(tree.firstChild) tree.removeChild(tree.firstChild);
    }
    current = use;
  }
  println("Refresh cluster shell" + ' - ' + document.documentURI);
  
  if(dock.generalSettings.shellSingle) {
    setView(1);
    document.getElementById('normalview').setAttribute('hidden', true);
    document.getElementById('treeview').setAttribute('hidden', true);
  } else {
    document.getElementById('normalview').removeAttribute('hidden');    
    document.getElementById('treeview').removeAttribute('hidden');
  }

  var group = document.getElementById("group");
  var old = -1;
  var item = group.selectedItem;
  if(item) old = item.getAttribute("key");
  var index = -1;
  var popup = group.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);

  var item = document.createElement("menuitem");
  item.setAttribute("class", "menuitem-iconic");
  item.setAttribute("image", "chrome://cmgui/content/icons/Category_16.png");
  item.setAttribute("type", "master");
  item.setAttribute("key", 0);
  item.setAttribute("label", "Head Nodes");
  popup.appendChild(item);
  item.doCommand();
  if(old <= 0) index = 0;

  var defaultCategory = current.getBasePartition().get('defaultCategory');

  var list = current.find("NodeCategory");
  for(var i = 0; i < list.length; i++) {
    var item = document.createElement("menuitem");
    item.setAttribute("class", "menuitem-iconic");
    item.setAttribute("image", "chrome://cmgui/content/icons/Category_16.png");
    item.setAttribute("type", "category");
    item.setAttribute("key", list[i].get('uniqueKey'));
    item.setAttribute("label", list[i].resolveName());
    popup.appendChild(item);
    if(old == list[i].get('uniqueKey')) {
      index = popup.childNodes.length - 1;
    } else if((old <= 0) && (list[i].get('uniqueKey') == defaultCategory)) {
      index = popup.childNodes.length - 1;
    }
  }
  var list = current.find("NodeGroup");
  for(var i = 0; i < list.length; i++) {
    var item = document.createElement("menuitem");
    item.setAttribute("class", "menuitem-iconic");
    item.setAttribute("image", "chrome://cmgui/content/icons/NodeGroup_16.png");
    item.setAttribute("type", "nodegroup");
    item.setAttribute("key", list[i].get('uniqueKey'));
    item.setAttribute("label", list[i].resolveName());
    popup.appendChild(item);
    if(old == list[i].get('uniqueKey')) index = popup.childNodes.length - 1;
  }
  var list = current.find("Chassis");
  for(var i = 0; i < list.length; i++) {
    var item = document.createElement("menuitem");
    item.setAttribute("class", "menuitem-iconic");
    item.setAttribute("image", "chrome://cmgui/content/icons/Chassis_16.png");
    item.setAttribute("type", "chassis");
    item.setAttribute("key", list[i].get('uniqueKey'));
    item.setAttribute("label", list[i].resolveName());
    popup.appendChild(item);
    if(old == list[i].get('uniqueKey')) index = popup.childNodes.length - 1;
  }
  if(index > 0) group.selectedIndex = index;
}

function execute() {
  document.getElementById("first").setAttribute('hidden', true);
  if(tracker) {
    delete dock.shellTracker[tracker];
    tracker = 0;
  }
  md5sums = new Object();
  var cmd = document.getElementById("command");
  var group = document.getElementById("group");
  var item = group.selectedItem;
  var nodes = current.findNodesIn(item.getAttribute("type"), item.getAttribute("key"));
  if(!!nodes && nodes.length > 0) {
    if(timer) {
      timer = window.clearTimeout(timer);
    }
    if(isObject(result)) {
      for(var i = 0; i < openStreams.length; i++) {
        var first = openStreams[i];
        result[first][2].close();
        result[first][2] = 0;
      }
      for(var i in result) {
        if(result[i][0]) {
          try {
            result[i][0].remove(false);
          } catch(e) {
            println(e);
          }
        }
      }
    }

    if(dock.generalSettings.shellPrepend > 0) {    
      nodesHostname = new Object();
      for(var i = 0; i < nodes.length; i++) {
        nodesHostname[nodes[i]] = current.resolveName(nodes[i]);
      }
    }
    
    var pri = new dock.cvProgramRunnerInput();
    pri.cmd = cmd.value;
    pri.user = current.syslogin;
    pri.startInShell = true;
    pri.mergeCoutCerr = true;
    pri.updateinterval = 1;
    dock.workerpool.getWorker(current.actualurl).call('cmproc', 'runCommand', [nodes, pri, current.sessionid], function(code, data) {
        if(code == 0) {
          tracker = data.input.tracker;
          notdone = nodes.length;
          dock.shellTracker[tracker] = dock.getAlreadyArrivedShellTracker(tracker);
          println('pexec: data recieved');
          buildCommandFields(data);
//           document.getElementById("list").value = '';
          var cmds = cmd.getAttribute('autocompletesearchparam');
          var add = "value: '" + cmd.value.replace(/'/g, "\\\'") + "'";
          if(cmds.indexOf(add) <= 0) {
            cmds = cmds.substr(0, cmds.length - 1);
            if(cmds.length > 1) cmds += ",";
            cmds += "{" + add + "}]";
            cmd.setAttribute('autocompletesearchparam', cmds);
          }
          //cmd.value = '';
        } else {
          println('JSON call ' + this.rpc + ' (' + code + ')');
          printall(data);
          var message = 'Unable to start command, error: ' + code;
          if(isObject(data)) {
            if(data.hasOwnProperty('errormessage')) {
              message = data.errormessage;
            } else if(data.hasOwnProperty('title')) {
              message = data.errormessage;
            }
          } else if(typeof(data) == 'string' && data.length) {
            message = data;
          }
          dock.displayError(message, window);
        }
      }, current);
  } else {
    dock.displayError('No nodes selected', window);
  }
}

function buildCommandFields(status) {
  dock.waitIcon();
  result = new Object();
  openStreams = new Array();

  var file = dock.cacheFile('cmguipexec.txt');
  var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
  outputStream.init(file, 0x02 | 0x08 | 0x20, oct0600, 0);
  var browser = document.getElementById("list");
  result[0] = [file, browser, outputStream, true];

  buildFieldNodesState = status.state;
  buildFieldNodes = status.nodes;
  buildFieldNodesIndex = 0;
  buildCommandNodeFields();
}

function buildCommandNodeFields() {
  var parent = window.frames[0].document;
  var tree = parent.getElementById("tree");
  if(buildFieldNodesIndex == 0) while(tree.firstChild) tree.removeChild(tree.firstChild);
  
  println('buildCommandNodeFields: ' + buildFieldNodesIndex + ' / ' + buildFieldNodes.length);
  var n = 0;
  for(; buildFieldNodesIndex < buildFieldNodes.length; buildFieldNodesIndex++) {
    n++;
    if(dock.MAX_PARALLEL_SHELL_UDPATE && (n > dock.MAX_PARALLEL_SHELL_UDPATE)) {
      timer = window.setTimeout('buildCommandNodeFields();', 50);      
      return;
    }
    var nodekey = buildFieldNodes[buildFieldNodesIndex];
    var node = current.find(nodekey);
    if(!!node) {
      var name = node.resolveName();
      var state = buildFieldNodesState[buildFieldNodesIndex];
      var browser = false;
      if(!dock.generalSettings.shellSingle) {
        if(openStreams.length > MAX_OPEN_STREAMS) {
          var first = openStreams.shift();
          result[first][2].close();
          result[first][2] = 0;
        }
        browser = addFieldElement(tree, nodekey, name, state, parent);
      }
      if(badfiles.hasOwnProperty(state)) {
        if(!hidedown) updateOne(0, badtexts[state], false, name);
        if(browser) {
          browser.loadURI(badfiles[state][0], null, null);
          result[nodekey] = [0, browser, 0, true, badfiles[state][1]];
          
          if(md5sums.hasOwnProperty(badfiles[state][1])) {
            md5sums[badfiles[state][1]].push(nodekey);
            joinNode(nodekey, parent);
          } else {
            md5sums[badfiles[state][1]] = [nodekey];
            joinNode(nodekey, parent);
          }
        }
        notdone--;
      } else if(browser) {
        var file = dock.cacheFile('cmguipexec.txt');
        var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
        outputStream.init(file, 0x02 | 0x08 | 0x20, oct0600, 0);
        result[nodekey] = [file, browser, outputStream, true];
        openStreams.push(nodekey);
      }
    }
  }
  println('buildCommandNodeFields: DONE');
  dock.unwaitIcon();
  timer = window.setTimeout('updateField();', 50);
}

function addFieldElement(tree, key, name, state, parent) {
  var vbox = parent.createElement("vbox");
  vbox.setAttribute('id', 'box' + key);
  vbox.setAttribute('class', 'staticdata');
  var hbox = parent.createElement("hbox");
  hbox.setAttribute('align', 'center');
  var spacer = parent.createElement("spacer");
  spacer.setAttribute('width', '4px');
  hbox.appendChild(spacer);
  var image = parent.createElement("image");
  image.setAttribute('src', 'chrome://cmgui/content/icons/Node_16.png');
  image.setAttribute('id', 'image' + key);
  hbox.appendChild(image);
  var label = parent.createElement("label");
  label.setAttribute('id', 'name' + key);
  label.setAttribute('value', name);
  hbox.appendChild(label);
  if((state == dock.PRE_RUNNING) || (state == dock.PRE_QUEUED)) {
    var image = parent.createElement("image");
    image.setAttribute('src', 'chrome://cmgui/content/icons/throbber.gif');
    image.setAttribute('id', 'throbber' + key);
    hbox.appendChild(image);
  }
  vbox.appendChild(hbox);
  var hbox = parent.createElement("hbox");
  hbox.setAttribute('id', 'line' + key);
  hbox.setAttribute('class', 'staticdottedline');
  hbox.setAttribute('style', 'margin-top: 2px;');
  vbox.appendChild(hbox);
  var textbox = parent.createElement("browser");
  textbox.setAttribute('id', 'browser' + key);
  textbox.setAttribute('height', '100px');
  textbox.setAttribute('class','browser');
  textbox.setAttribute('type','content');
  textbox.setAttribute('encoding', 'utf-8');
  vbox.appendChild(textbox);
  tree.appendChild(vbox);

  return textbox;
}

function updateOne(index, data, close, name) {
  name = isundefined(name, false);
  if((index > 0) && dock.generalSettings.shellSingle) {
    return 0;    
  } else if(!result.hasOwnProperty(index)) {
    println("Work not found for: " + index);
    return 0;
  }
  var work = result[index];
  if(work[2] == 0) {
    if(openStreams.length > MAX_OPEN_STREAMS) {
      var first = openStreams.shift();
      result[first][2].close();
      result[first][2] = 0;
    }
    work[2] = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
    work[2].init(work[0], 0x02 | 0x10, oct0600, 0);
    openStreams.push(index);
  }
  if(name) {
    name += ': ';
    var lines = data.split('\n');
    if(lines.length > 0) {
      for(var i = 0; i < lines.length - 1; i++) {
        work[2].write(name, name.length);
        work[2].write(lines[i], lines[i].length);
        work[2].write('\n', 1);
      }
      i = lines.length - 1;
      if(lines[i].length > 0) {
        work[2].write(name, name.length);
        work[2].write(lines[i], lines[i].length);
        work[2].write('\n', 1);
      }
    }
  } else {
    work[2].write(data, data.length);
  }
  if(close) {
    work[2].close();
    work[2] = 0;
    var pos = openStreams.find(index);
    if(pos >= 0) openStreams.splice(pos, 1);
  } else {
    work[2].flush();
  }
//     if(work[3]) {
  work[1].loadURI('file://' + work[0].path, null, dock.ENCODING);
//       work[3] = false;
//     } else { // does NOT by pass cache :(
//       work[1].reloadWithFlags(LOAD_FLAGS_BYPASS_CACHE);
//     }
  return work;
}

function updateField() {
  dock.waitIcon();
  var parent = window.frames[0].document;
  var index = document.getElementById("deck").selectedIndex;
  var n = 0;
  var workShell = dock.shellTracker[tracker];
  println('updateField: ' + workShell.length);
  while(n < workShell.length) {
    var one = workShell[n];
    var name = false;
    if(dock.generalSettings.shellPrepend > 0) {    
      if(nodesHostname.hasOwnProperty(one[1])) {
        name = nodesHostname[one[1]];
      }
    }
    
    updateOne(0, one[2], one[0], name);
    var work = updateOne(one[1], one[2], one[0]);
    if(one[0]) {
      notdone--;
      if(work) {
        var obj = parent.getElementById("throbber" + one[1]);
        if(!!obj) {
          obj.setAttribute("hidden", true);
        } else {
          println("Not found: throbber" + one[1]);
        }
        var md5 = md5sum(work[0].path);
        work.push(md5);
        if(md5sums.hasOwnProperty(md5)) {
          md5sums[md5].push(one[1]);
        } else {
          md5sums[md5] = [one[1]];
        }
        joinNode(one[1], parent);
      }
    }
    n++;
    if((notdone > 0) && dock.MAX_PARALLEL_SHELL_UDPATE && (n > dock.MAX_PARALLEL_SHELL_UDPATE)) {
      workShell.splice(0, n);
      dock.unwaitIcon();
      println('updateField: thread ' + workShell.length);
      timer = window.setTimeout('updateField();', 50);
      return;
    }
  }
  dock.shellTracker[tracker] = new Array();
  dock.unwaitIcon();
  if(notdone > 0) {
    println('updateField: wait');
    timer = window.setTimeout('updateField();', 1000);
  } else {
    timer = 0;
    if(result[0][2]) result[0][2].close();
    println('updateField: ALL DONE');
  }
}

function joinNode(node, parent) {
  var md5 = result[node][4];
  var first = md5sums[md5][0];
  if(first != node) {
    var objf = parent.getElementById("name" + first);
    var name = objf.getAttribute('join');
    if(name.length == 0) name = objf.getAttribute('value');
    name += ' ' + parent.getElementById("name" + node).getAttribute('value');
    objf.setAttribute('join', name);
    if(joinoutput) {
      objf.setAttribute('value', dotFormat(name.split(' ')));
      parent.getElementById("box" + node).setAttribute('hidden', true);
    }
  } else {
    var objf = parent.getElementById("name" + first);
    var name = objf.getAttribute('join');
    if(name.length == 0) name = objf.getAttribute('value');
    objf.setAttribute('join', name);
  }
}

function joinOutput() {
  var parent = window.frames[0].document;
  if(document.getElementById('joinoutput').checked) {
    joinoutput = 1;
    for(var i in result) {
      if((i > 0) && (result[i].length == 5)) {
        var obj = parent.getElementById("name" + i);
        var name = obj.getAttribute('join');
        if(name.length > 0) {
          obj.setAttribute('value', dotFormat(name.split(' ')));
        } else {
          obj.parentNode.parentNode.setAttribute('hidden', true);
        }
      }
    }
  } else {
    joinoutput = 0;
    for(var i in result) {
      if((i > 0) && (result[i].length == 5)) {
        var obj = parent.getElementById("name" + i);
        var name = obj.getAttribute('join');
        if(name.length > 0) {
          obj.setAttribute('value', name.split(' ')[0]);
        } else {
          obj.parentNode.parentNode.setAttribute('hidden', false);
        }
      }
    }
  }
}

function hideDown() {
  hidedown = document.getElementById('hidedown').checked;
}

function setView(index) {
  document.getElementById("deck").selectedIndex = index;
  document.getElementById("normalview").setAttribute("disabled", index == 1);
  document.getElementById("treeview").setAttribute("disabled", index == 0);
  
  document.getElementById('hidedown').setAttribute('hidden', index == 0);
  document.getElementById('joinoutput').setAttribute('hidden', index == 1);
}

function detach() {
  var args = new Object();
  args['cluster'] = current;
  var d = new dock.cvDialog("detachedshell.xul", args);
  d.parentWindow = window;
  dock.dialogs.show(d);
}

function showHistory() {
  var args = new Object();
  args['cluster'] = current;
  var d = new dock.cvDialog("shellhistory.xul", args);
  d.parentWindow = window;
  dock.dialogs.show(d);
}

// function waitIcon() {
//   println('waitIcon');
//   dock.waitIcon();
// //   if(window == dock) {
// //     dock.waitIcon();
// //   } else {
// //     if(localWaitCount == 0) {
// //       window.setCursor("wait");
// //       for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("wait");
// //     }
// //     localWaitCount++;
// //   }
// }
// 
// function unwaitIcon() {
//   println('unwaitIcon');
//   dock.unwaitIcon();
// //   if(window == dock) {
// //     dock.unwaitIcon();
// //   } else {
// //     localWaitCount--;
// //     if(localWaitCount <= 0) {
// //       window.setCursor("auto");
// //       for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("auto");
// //       if(localWaitCount < 0) println("Mismatch waitCursor");
// //       localWaitCount = 0;
// //     }
// //   }
// }
