/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var monconf;
var overview;
var oldcluster = 0;
var graph;
var timer = 0;

var first = true;

function onLoad() {
  dock = getDock();
  addEventListener("resize", onResize, false);
}

function onResize() {
  var svg = document.getElementById("svg");
  svg.setAttribute("height", "25px");
  svg.setAttribute("width", svg.parentNode.boxObject.width);
  svg.setAttribute("height", svg.parentNode.boxObject.height - 128); // 128 ?? KDR
  redraw();
}

function getCurrentSelection() {
  return [[current.getBasePartition()], 0, ""];
}

function stop() {
  if(graph) graph.stop();
  if(timer) window.clearTimeout(timer);
}

function refresh(use) {
  if(!!use) {
    var active = use.activeHeadNode();
    if(!active) return;
    monconf = active.getMonitoringConfiguration();
    if(!monconf) return;
    current = use;
  }
  
  if(!!current.cluster) current = current.cluster;
  if(current != oldcluster) {
    setCallbackResult(new dock.cvGuiClusterOverview());
    if(graph) {
      graph.cluster = current;
      graph.clearScreen();
    } else {
      graph = new cvGraph(current, document.getElementById("svg"), document.getElementById("box"));
      graph.colors = [[['#7cc300', '#7dc200', '#4d7900']]];
      graph.WM = 32;
      graph.HM = 61;
      graph.legend = -1;
    }
    oldcluster = current;
  }

  var list = document.getElementById("metriclist");
  var oldkey = parseInt(list.getAttribute('metricKey'));
  
  var name = current.find(oldkey);
  if(name) {
    name = name.resolveName();
    if(!startswith(oldlabel, name)) {
      oldkey = 0;
    }
  }
  var oldobject = parseInt(list.getAttribute('objectKey'));
  if(oldobject && !current.find(oldobject)) oldobject = 0;
  
  var oldparam = list.getAttribute('param');
  var oldlabel = list.getAttribute('label');
  if(kisNaN(oldkey) || (oldkey == 0)) {
    if(isArray(current.mainMetric)) {
      oldkey = current.mainMetric[0];
      oldparam = current.mainMetric[1];
      oldlabel = '';
      var metric = current.find(oldkey);
      if(metric) {
        var text = metric.resolveName();
        if(oldparam.length > 0) text += "[" + oldparam + "]";
        if(metric.get('unit')) text += " (" + metric.get('unit') + ")";
        oldlabel = text;
      }
    }
  }

  var mmc = monconf.get("metrics");
  createMetricPopup(current, mmc, 'metriclist', false, true, ['OPENSTACK','CEPH','AMAZONEC2','HADOOP_JVM','HADOOP_DFS','HADOOP_UGI','HADOOP_MAPRED','HADOOP_RPC','HADOOP_RPCDETAILED','HADOOP_METRICSYSTEM','DELLNSS', 'LUSTRE']);
  createDellNSSMetricPopup(current, 'metriclist');

  var found = false;
  for(var i = 0; (i < mmc.length) && !found; i++) {
    found = ((mmc[i].metric == oldkey) && (mmc[i].metricParam == oldparam));
  }

  if(!found) {
    if(mmc.length == 0) {
      var item = document.createElement("menuitem");
      item.setAttribute('metricKey', 0);
      item.setAttribute('objectKey', 0);
      item.setAttribute('param', '');
      item.setAttribute('label', '');
      list.firstChild.appendChild(item);
      oldkey = 0;
      oldparam = '';
      oldlabel = '';
    } else {
      oldkey = mmc[0].metric;
      oldparam = mmc[0].metricParam;
      var def = current.find('Metric', 'NodesUp');
      if(def) {
        def = def.get('uniqueKey');
        for(var i = 0; i < mmc.length; i++) {
          if(mmc[i].metric == def) {
            oldkey = def
            oldparam = mmc[i].metricParam;
            break;
          }
        }
      }
      var metric = current.find(oldkey);
      if(metric) {
        oldlabel = metric.resolveName();
        if(oldparam.length > 0) oldlabel += "[" + oldparam + "]";
        if(metric.get('unit')) oldlabel += " (" + metric.get('unit') + ")";
      }
    }
  }
  list.setAttribute('metricKey', oldkey);
  list.setAttribute('objectKey', oldobject);
  list.setAttribute('param', oldparam);
  list.setAttribute('label', oldlabel);

  graph.clear();
  if(oldobject == 0 || isNaN(oldobject)) oldobject = current.activePassiveKeys[0];
  println('oldobject: "' + oldobject + '" ' + typeof(oldobject));
  if(oldobject) {
    if(oldkey) {
      var dmp = new cvGraphDeviceMetric(oldobject, oldkey, oldparam);
      var r = graph.add(dmp);
      if(r < 0) {
        dock.displayError("Too many metrics, only " + -r + " are supported");
      } else if(r != 0) {
        dock.displayError("Incompatible metrics: " + r);
      }
      //if(dock.DEMO) graph.zoom[0] *= 24 * 7;
      redraw();
    } else {
      dock.displayError("No metrics defined");
    }
  } else {
    dock.displayError("Primary head node not defined");
  }
  update();
}

function update() {
  if(!current.connected) return;
  dock.workerpool.getWorker(current.actualurl).call('cmgui', 'getClusterOverview', null, function(code, data) {
      if(code == 0) {
        setCallbackResult(data);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
  
//   if(first && fileExists('/home/koen/.bashrc')) {  // TODO: KDR remove test code
//     var url = 'http://dev3/koen/helloworld.xul';
//     dock.workerpool.getWorker(current.actualurl).wget(url, function(code, data) {
//         println('---------------------------------------');
//         println(code);
//         println(data);
//         if(code == 0) {
//           var tab = document.createElement("shadowtab");
//           tab.setAttribute('label', 'Hello World');    
//           parent.document.getElementById('tabs').appendChild(tab);
//           var iframe = document.createElement("iframe");
// //           iframe.setAttribute('src', 'notes.xul');    
//           parent.document.getElementById('tabpanels').appendChild(iframe);
//           iframe.setAttribute('src', "data:text/html;charset=utf-8," + encodeURIComponent(data));
// //           var wnd = iframe.contentWindow;
// //           wnd.location.src = "data:text/html;charset=utf-8," + encodeURIComponent(data);
// //           var iframedoc = iframe.document;
// //           if(iframe.contentDocument) {
// //             iframedoc = iframe.contentDocument;
// //           } else if (iframe.contentWindow) {
// //             iframedoc = iframe.contentWindow.document;
// //           }
// //           if(iframedoc){
// //             iframedoc.open();
// //             iframedoc.writeIn(data);
// //             iframedoc.close();
// //           } else {
// //             println('No iframedoc');
// //           }
//         } else {
//           dock.jsonCallError(this, code, data);
//         }
//       }, current);
//   }
  first = false;
}

function setCallbackResult(value) {
  overview = value;
  if(overview == null) return;
  if(dock.DEMO) {
    overview.nodesUp = 503;
    overview.nodesDown = 7;
    overview.nodesClosed = 2;
    overview.devicesUp = 64;
    overview.devicesDown = 0;
    overview.devicesClosed = 0;
    overview.phaseLoad = 783;
    overview.coresUp = 503 * 8;
    overview.coresTotal = 512 * 8;
    overview.memoryUsed = 16e+9 * 503;
    overview.memoryTotal = 16e+9 * 512;
    overview.users = 13;
    overview.usersTotal = 38;
    overview.occupationRate = 83.2;
    overview.usageUser = 0.48;
    overview.usageSystem = 0.29;
    overview.usageIdle = 0.10;

    var workload = new Array();
    var wl = new dock.cvGuiWorkload();
    wl.name = "short.q";
    wl.running = 32;
    wl.queued = 43;
    wl.completed = 482;
    wl.averageDuration = 26871;
    wl.estimatedDelay = 32742;
    workload.push(wl);
    var wl = new dock.cvGuiWorkload();
    wl.name = "medium.q";
    wl.running = 5;
    wl.queued = 11;
    wl.completed = 41;
    wl.averageDuration = 226871;
    wl.estimatedDelay = 405232;
    workload.push(wl);
    var wl = new dock.cvGuiWorkload();
    wl.name = "long.q";
    wl.running = 8;
    wl.queued = 13;
    wl.completed = 91;
    wl.averageDuration = 726871;
    wl.estimatedDelay = 1343232;
    workload.push(wl);
    overview.setWorkload(workload.length, workload);

    var disks = new Array();
    var dk = new dock.cvGuiDiskUsage();
    dk.mountpoint = '/';
    dk.free = 23e+9;
    dk.used = 17e+9;
    disks.push(dk);
    var dk = new dock.cvGuiDiskUsage();
    dk.mountpoint = '/boot';
    dk.free = 89e+6;
    dk.used = 15e+6;
    disks.push(dk);
    var dk = new dock.cvGuiDiskUsage();
    dk.mountpoint = '/home';
    dk.free = 10000e+9;
    dk.used = 894e+9;
    disks.push(dk);

    overview.setDisks(disks.length, disks);
  }

  overviewValue('nodesUp');
  overviewValue('nodesDown');
  overviewValue('nodesClosed');
  overviewValue('micsUp');
  overviewValue('micsDown');
  overviewValue('micsClosed');
  overviewValue('gpuUnitsUp');
  overviewValue('gpuUnitsDown');
  overviewValue('gpuUnitsClosed');
  overviewValue('devicesUp');
  overviewValue('devicesDown');
  overviewValue('devicesClosed');
  var phaseLoadUnit = document.getElementById("phaseLoadUnit");
  if(kisNaN(overview.phaseLoad) || (overview.phaseLoad <= 0)) {
    overviewValue('phaseLoad', 'N/A');
    phaseLoadUnit.setAttribute('hidden', true);
  } else {
    overviewValue('phaseLoad', sprintf('%.1f', overview.phaseLoad));
    phaseLoadUnit.setAttribute('hidden', false);
  }
  overviewDisks();
  overviewWorkloads();
  overviewUptime('uptime');
  overviewBar('cores', overview.coresUp, overview.coresTotal);
  overviewBar('mics', overview.micsUp, overview.micsTotal);
  overviewBar('gpus', overview.gpusUp, overview.gpusTotal);
  overviewBar('memory', overview.memoryUsed, overview.memoryTotal, 'B', true);
  overviewBar('usersbar', overview.users, overview.usersTotal);
  overviewBar('rate', overview.occupationRate, -100, '%');
  overviewUsage('usage', overview.usageUser, overview.usageSystem, overview.usageIdle);

  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 1000);
}

function selectMetric(event) {
  var list = document.getElementById("metriclist");
  var metric = event.target;
  if(!!metric) {
    graph.last = 0;
    var old = graph.deviceMetric[0].device;
    var newDevice = parseInt(metric.getAttribute('objectKey'));
    if(newDevice) graph.deviceMetric[0].device = newDevice;
    graph.deviceMetric[0].metric = parseInt(metric.getAttribute('metricKey'));
    graph.deviceMetric[0].param = metric.getAttribute('param');
    graph.deviceMetric[0].unit = current.find(graph.deviceMetric[0].metric).get('unit');
    var name = metric.getAttribute('label');
    list.setAttribute('objectKey', graph.deviceMetric[0].device);
    list.setAttribute('metricKey', graph.deviceMetric[0].metric);
    list.setAttribute('param', graph.deviceMetric[0].param);
    list.setAttribute('label', name);
    if(newDevice && (old != graph.deviceMetric[0].device)) graph.handleDeviceChange(graph.deviceMetric[0]);
    redraw();
    current.mainMetric = [graph.deviceMetric[0].metric, graph.deviceMetric[0].param];
  }
}

function redraw() {
  if(graph) {
    graph.update();
  }
}
