/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */
function DeployWizardScreen() {
  WizardScreen.call(this);
  this.timer = 0;
  this.timer_info = 0;
  this.count = 0;
  this.status = 'STARTED';
  this.current_question = null;
  this.questions = {};
  this.execution_id = 0;
  this.cm_setup_tool_script = 'cm-setup-mock';
  this.cm_setup_tool_name   = 'Mock';

  // some hooks that can be overriden:
  this.beforeCallSetupExecute = function() { 
  };

  this.onCallSetupExecuteSuccess = function() {
  };

  this.onCallSetupExecuteFailed = function() {
  };

  this.onShowDetailedLog = function() {
  };

  this.onHideDetailedLog = function() {
  };

  this.onShowSuccess = function() {
  };

  this.onShowFailed = function() {
  };

  this.onEnter = function() {
  };
  
  this.forward = function() {
    if (this.status === 'FAILED') {
      //wizard.deploy.error = true;
    }
    return this.error;
  };

  this.preEnter = function() {
    wizard.common.disable('cancelButton');
    wizard.common.disable('previousButton');
    wizard.common.hide('nextButton');

    wizard.common.unhide('finishButton');
    wizard.common.disable('finishButton');
    wizard.common.unhide('deploy-detailedLogButton');
  };

  this.leave = function() {
    if (wizard.deploy.status === 'FAILED') {
      wizard.common.enable('cancelButton');
      wizard.common.enable('previousButton');
      wizard.common.unhide('nextButton');
      wizard.common.hide('finishButton');
    } else {
      wizard.common.enable('finishButton');
    }
  };

  

  this.enter = function() {
    println('enter wizard.deploy.enter');
    this.onEnter();
    if (!wizard.deploy.started) {
      wizard.deploy.started = true;
    }
    wizard.common.unhide('deploy-detailedLogButton');
    document.getElementById('deploy-detailedLogButton').label = 'Show Log';
    wizard.common.hide('deploy-output');
    wizard.common.hide('deploy-copyLogContent');
    var elem_icon = document.getElementById('current-stage-icon');
    if(elem_icon) {
      elem_icon.setAttribute('src', "chrome://cmgui/content/icons/throbber.gif");
    }
    var elem_msg = document.getElementById('current-stage-message');
    if(elem_msg) {
      elem_msg.setAttribute('value', 'Starting deployment');
    }
    wizard.deploy.timer = 0;
    wizard.deploy.timer_info = 0;
    wizard.deploy.count = 0;
    wizard.deploy.status = 'STARTED';
    wizard.deploy.current_question = null;
    wizard.deploy.questions = {};
    wizard.common.hide('deploy-question-element');
    overviewBar('deploy-progressBar', 0, 100);
    wizard.deploy.initSuccess();
    wizard.deploy.initFailed();
    this.beforeCallSetupExecute();
    this.startWorking();
  };

  this.startWorking = function() {
    var self = this;
    var extraOptions = new Array();
    var id = 0;
    var cluster = current.hasOwnProperty('cluster') ? current.cluster : current;
    dock.workerpool.getWorker(cluster.actualurl).call('cmmain', 'cmSetupExecute', [ this.cm_setup_tool_script,
                                                                                    wizard.common.configToYaml(), 
                                                                                    extraOptions], 
     function(code, data) {
       if (code == 0) {
          id = data.setupExecutionID;
          self.execution_id = id;
          self.timer = window.setTimeout('wizard.deploy.checkStatus(' + id +');', 1000);
          self.onCallSetupExecuteSuccess();
       } else {
          wizard.common.disable('cancelButton');
          wizard.common.enable('finishButton');
          wizard.common.hide('deploy-failed');
          wizard.common.hide('deploy-detailedLogButton');
          var elem_msg = document.getElementById('current-stage-message');
          if(elem_msg) {
            elem_msg.setAttribute('value', 'Deployment has failed (cmSetupExecute):' + data.title);
          }
          var elem_icon = document.getElementById('current-stage-icon');
          if(elem_icon) {
            elem_icon.setAttribute('src', "chrome://cmgui/content/icons/alert.png");
          }
          wizard.common.hide('deploy-output');
          wizard.common.hide('deploy-copyLogContent');
          self.onCallSetupExecuteFailed();
          dock.jsonCallError(this, code, data);
       }
      }, cluster);
  };

  this.toggleDetailedLog = function() {
    if (document.getElementById('deploy-output').getAttribute('hidden')==='true'){
      document.getElementById('deploy-detailedLogButton').label = 'Hide Log';
      wizard.common.unhide('deploy-output');
      wizard.common.unhide('deploy-copyLogContent');
      this.onShowDetailedLog();
      //document.getElementById('deploy-info-headline').label = 'Detailed deployment log';
    } else {
      document.getElementById('deploy-detailedLogButton').label = 'Show Log';
      wizard.common.hide('deploy-output');
      wizard.common.hide('deploy-copyLogContent');
      this.onHideDetailedLog();
      //document.getElementById('deploy-info-headline').label = '';
      //document.getElementById('deploy-info').innerHTML = '';
    }
  };

  this.onCancel = function() {
    var args = new Object();
    args['question'] = 'Are you sure you cancel ongoing deployment?';
    var d = new cvDialog("chrome://cmgui/content/question.xul", args);
    d.onOk = function() {
      var cluster = current.hasOwnProperty('cluster') ? current.cluster : current;
      dock.workerpool.getWorker(cluster.actualurl).call('cmmain', 
                                                        'cmSetupRemoveExecution', [wizard.deploy.execution_id ], 
      function(code, data) {
        if (code == 0) {
          var elem_msg = document.getElementById('current-stage-message');
          if(elem_msg) {
            elem_msg.setAttribute('value', 'The deployment has been canceled. Please wait...');
          }
          var elem_msg = document.getElementById('current-stage-icon');
          if(elem_msg) {
            elem_msg.setAttribute('src', "chrome://cmgui/content/icons/throbber.gif");
          }
        } 
        else {
          dock.jsonCallError(this, code, data);
        }
      });
    };
    dock.dialogs.showDialog(d);
  };

  this.checkStatus = function(id) { 
    function parseSetupExecutionResponse(response, id, successful) {
      var filteredLog = [];
      var steps = {};
      var warnings = {};
      var pre_warnings = [];
      var previousStep = null;
      var progressPerc = 0;
      var input_questions = [];
      var status = "";
      for (var i=0; i < response.length; i++) {
        var matched = false;
        var matched_pid = /^ERROR: Found pid file .*$/g.exec(response[i]);
        if(matched_pid) {
          matched = true;
          filteredLog.push(response[i]);
          status = "aborted";
        }
        if (response[i].match(/^Connecting to CMDaemon$/g)) {
          matched = true;
        }
        var match1 = /^#### stage: (.+)$/g.exec(response[i]);
        if (match1) {
          matched = true;
          if(successful) {
            steps[match1[1]] = 'SUCCESS';
          }
          else {
            steps[match1[1]] = 'INPROGRESS';
          }
          if (previousStep === null) {
            if(pre_warnings.length > 0) {
              if(!warnings.hasOwnProperty(match1[1])) {
                warnings[match1[1]] = [];
              }
              warnings[match1[1]] = warnings[match1[1]].concat(pre_warnings);
            }
          }
          if (previousStep !== null) {
            steps[previousStep] = 'SUCCESS';
          }
          previousStep = match1[1];
          filteredLog.push(response[i]);
        }

        var match2 = /^### ERROR FOUND ###$/g.exec(response[i]);
        if (match2) {
          matched = true;
          if (previousStep !== null) {
            steps[previousStep] = 'ERROR';
          }
        }

        var match3 = /^## Progress: (\d{1,3})$/g.exec(response[i]);
        if (match3) {
          matched = true;
          progressPerc = parseInt(match3[1]);
          if (progressPerc === 100) {
            steps[previousStep] = 'SUCCESS';
          }
        }
        var match4 = /^###### WARNING: (.*)$/g.exec(response[i]);
        if (match4) {
          matched = true;
          filteredLog.push(match4[1]);
          if(previousStep !== null) {
            if(!warnings.hasOwnProperty(previousStep)) {
              warnings[previousStep] = []
            }
            warnings[previousStep].push(match4[1]);
          }
          else {
            pre_warnings.push(match4[1]);
          }
        }
        var match5 = /^### INPUT (.+)###$/g.exec(response[i]);
        if(match5) {
          var possible_answers = match5[1].split('/'); 
          matched = true;
          input_questions.push( { step:     previousStep,
                                  answer:   "",
                                  id:       i, 
                                  setupExecutionID : id,
                                  possible_answers : possible_answers } );
        }
        var match6 = /^#### status: (.+)$/g.exec(response[i]);
        if(match6) {
          matched = true;
          status = match6[1];
        }
        if (!matched) {
          filteredLog.push(response[i]);
        }
      }
      return {steps: steps,
              progress: progressPerc,
              log: filteredLog,
              warning : warnings,
              questions : input_questions,
              status : status,
              currentStep : previousStep };
    }

    function printDeploySteps(steps, warning={}, questions={}, current_question=null) {
      var deploySteps = document.getElementById('deploy-steps');
      var deploySuccess = document.getElementById('deploy-success');
      var questions_lookup = {};
      var n_retries = {};
      var question_keys = Object.keys(questions);
      question_keys.sort(function(a,b) { return a - b; });
      question_keys.forEach(function(q_id) {
        if(!questions_lookup.hasOwnProperty(questions[q_id].step)) {
          questions_lookup[questions[q_id].step] = [];
        }
        questions_lookup[questions[q_id].step].push(questions[q_id]);
      });
      while (deploySteps.childNodes.length > 4) {
        deploySteps.removeChild(deploySteps.childNodes[1]);
      }
      Object.keys(steps).forEach(function(step) {
        var status  = steps[step];
        if(warning.hasOwnProperty(step)) {
          status = "WARNING";
        }
        if(questions_lookup.hasOwnProperty(step)) {
          status = "WARNING";
          var last_answer = '';
          questions_lookup[step].forEach(function(q) {
            if(q.answer !== "") {
              last_answer = q.answer;
            }
          });
          if(last_answer == 'u') {
            status = 'ERROR';
            if(!warning.hasOwnProperty(step)) {
              warning[step] = [];
            }
            warning[step].push("Deployment has been rolled back at this stage");
          }
          else if(last_answer == 'a') {
            status = 'ERROR';
            if(!warning.hasOwnProperty(step)) {
              warning[step] = [];
            }
            warning[step].push("Deployment has been aborted at this stage");
          }
          else if(last_answer == 'r') {
            status = 'WARNING';
            if(!warning.hasOwnProperty(step)) {
              warning[step] = [];
            }
            warning[step].push("Stage has been retried");
          }
          else if(last_answer == 's') {
            status = 'ERROR';
            if(!warning.hasOwnProperty(step)) {
              warning[step] = [];
            }
            warning[step].push("Stage has been skipped");
          }
        }
        wizard.deploy.addRow(deploySteps, deploySuccess, step, status, 
                             ( warning.hasOwnProperty(step) && Array.isArray(warning[step]) ? warning[step] : [])); 
      });
      if(document.getElementById('deploy-autoScroll').checked) {
        var deployStepsScroll = document.getElementById('deploy-steps-scroll-region');
        deployStepsScroll.scrollTop = deployStepsScroll.scrollHeight;
      }
    }

    function printCancel() {
      var deploySteps = document.getElementById('deploy-steps');
      var deploySuccess = document.getElementById('deploy-success');
      while (deploySteps.childNodes.length > 4) {
        deploySteps.removeChild(deploySteps.childNodes[1]);
      }
      wizard.common.hide('deploy-question-element');
      wizard.common.unhide('deploy-canceled');
    }

    function printDeployLogs(logs) {
      var deployOutput = document.getElementById('deploy-output');
      while (deployOutput.childNodes.length > 0) {
        deployOutput.removeChild(deployOutput.lastChild);
      }
      logs.forEach(function(log) {
        var vbox = document.createElement('vbox');
        vbox.textContent = log.replace(/\x1b[^m]*m/g, '');
        deployOutput.appendChild(vbox);
      });
      if(document.getElementById('deploy-autoScroll').checked) {
        deployOutput.scrollTop = deployOutput.scrollHeight; 
      }
    }

    function showQuestion(questions, current_question, currentStep) {
      var elem = document.getElementById('deploy-question-element');
      if(current_question !== null && questions.hasOwnProperty(current_question))  {
        if(elem) {
          elem.removeAttribute('hidden');
        }
        var buttons = elem.getElementsByTagName("button");
        var label2key = {
          'Undo' : 'u', 
          'Retry' : 'r', 
          'Abort' : 'a', 
          'Skip' : 's',
          'Debug' : 'D'
        };
        if(!questions[current_question].hasOwnProperty('possible_answers')) {
          questions[current_question].possible_answers = [ 'u', 'r', 'a', 's', 'D' ];
        }
        for(var i = 0; i < buttons.length; i++) {
          var label = buttons[i].getAttribute('label');
          if(label2key.hasOwnProperty(label)) {
            var k = label2key[label];
            var disable = (questions[current_question].possible_answers.indexOf(k) == -1);
            buttons[i].setAttribute('disabled',  disable);

          }
        }
        var elem_msg = document.getElementById('current-stage-message');
        if(elem_msg) {
          elem_msg.setAttribute('value', 'Stage ' + questions[current_question].step + ' has failed');
        }
        var elem_icon = document.getElementById('current-stage-icon');
        if(elem_icon) {
          elem_icon.setAttribute('src', "chrome://cmgui/content/icons/alert.png");
        }
        // no cancel possible if answer is expected
        wizard.common.disable('cancelButton');
      }
      else {
        if(elem) {
          elem.setAttribute('hidden',true);
        }
        var elem_msg = document.getElementById('current-stage-message');
        if(currentStep != null && elem_msg) {
          
          elem_msg.setAttribute('value', currentStep);
        }
        var elem_icon = document.getElementById('current-stage-icon');
        if(elem_icon) {
          elem_icon.setAttribute('src', "chrome://cmgui/content/icons/throbber.gif");
        }
        wizard.common.enable('cancelButton');
      }
    }

    if(wizard.deploy.current_question == null || 
       !wizard.deploy.questions.hasOwnProperty(wizard.deploy.current_question))  {
      wizard.common.enable('cancelButton');
    }
    println('checkStatus with count: ' + wizard.deploy.count);
    wizard.deploy.count++;
    if (wizard.deploy.count > 5000) {
      println('checkStatus stopped');
      wizard.deploy.status = 'FAILED';
      wizard.deploy.prepareButtonsAfterInstall();
      window.focus();
      wizard.deploy.showFailed(['The wizard has timed out.', 
			                          'Please check /var/log/' + this.cm_setup_tool_script + '.log on the headnode'], [],"aborted");
      if(wizard.deploy.timer_info) {
        clearTimeout(wizard.deploy.timer_info);
      }
      return;
    }
    if(wizard.deploy.status == 'CANCELED') {
      printall("check status of canceled job");
      printCancel();
      wizard.common.unhide('finishButton');
      wizard.common.enable('finishButton');
      wizard.common.disable('cancelButton');
      return;
    }
    var self = this;
    var cluster = current.hasOwnProperty('cluster') ? current.cluster : current;
    dock.workerpool.getWorker(cluster.actualurl).call('cmmain', 'cmSetupGetExecution', [id], function(code, data) {
      if(code == 0) {
        if (!data) {
          dock.displayError('An ' + this.cm_setup_tool_script + ' instance is already being installed.');
          return;
        }

        var parsedResponse = parseSetupExecutionResponse(data.datacout, id, data.successful);
        var rolled_back = false;
        for(var i = 0; i < parsedResponse.questions.length; i++) {
          var q = parsedResponse.questions[i];
          if(!wizard.deploy.questions.hasOwnProperty(q.id)) {
            wizard.deploy.questions[q.id] = q;
            wizard.deploy.current_question = q.id;
          }
          if(wizard.deploy.questions[q.id].answer == 'u') {
            parsedResponse.status = 'undone';
          }
        }
        overviewBar('deploy-progressBar', parsedResponse.progress, 100);
        printDeploySteps(parsedResponse.steps, parsedResponse.warning, wizard.deploy.questions, wizard.deploy.current_question);
        printDeployLogs(parsedResponse.log);
        showQuestion(wizard.deploy.questions, wizard.deploy.current_question, parsedResponse.currentStep);
        if (data.finishedAt == 0) {
          wizard.deploy.timer = window.setTimeout('wizard.deploy.checkStatus(' + id +');', 5000);
        } else if (data.successful && parsedResponse.status == "aborted") {
          wizard.deploy.status = 'FAILED';
          wizard.deploy.prepareButtonsAfterInstall();
          window.focus();
          wizard.deploy.showFailed(data.errors, data.warnings, "aborted");
          if(wizard.deploy.timer_info) {
            clearTimeout(wizard.deploy.timer_info);
          }
        } else if (data.successful && parsedResponse.status == "undone") {
          wizard.deploy.status = 'FAILED';
          wizard.deploy.prepareButtonsAfterInstall();
          window.focus();
          wizard.deploy.showFailed(data.errors, data.warnings, "aborted");
          if(wizard.deploy.timer_info) {
            clearTimeout(wizard.deploy.timer_info);
          }
        } else if (data.successful) {
          overviewBar('deploy-progressBar', 100, 100);
          wizard.deploy.status = 'SUCCESS';
          wizard.deploy.prepareButtonsAfterInstall();
          window.focus();
          wizard.deploy.showSuccess(data.errors, data.warnings);
          if(wizard.deploy.timer_info) {
            clearTimeout(wizard.deploy.timer_info);
          }
        } else {
          wizard.deploy.status = 'FAILED';
          wizard.deploy.prepareButtonsAfterInstall();
          window.focus();
          wizard.deploy.showFailed(data.errors, data.warnings);
          if(wizard.deploy.timer_info) {
            clearTimeout(wizard.deploy.timer_info);
          }
        }
      } else {
        wizard.common.disable('cancelButton');
        wizard.common.enable('finishButton');
        wizard.common.hide('deploy-failed');
        wizard.common.hide('deploy-detailedLogButton');
        var elem_msg = document.getElementById('current-stage-message');
        if(elem_msg) {
          elem_msg.setAttribute('value', 'Deployment has failed (cmSetupGetExecution):' + data.title);
        }
        var elem_icon = document.getElementById('current-stage-icon');
        if(elem_icon) {
          elem_icon.setAttribute('src', "chrome://cmgui/content/icons/alert.png");
        }
        wizard.common.hide('deploy-output');
        wizard.common.hide('deploy-copyLogContent');
        self.onCallSetupExecuteFailed();
        dock.jsonCallError(this, code, data);
      }
    }, cluster);
  };


  this.addRow = function(parentElementNode, ElementNodeSibling, value, status, warnings=[]) {
    var row = document.createElement('row');
    row.setAttribute('align', 'center');
    var image = document.createElement('image');
    if (status === 'INPROGRESS') {
      image.setAttribute('src', 'chrome://cmgui/content/icons/throbber.gif');
    } else if (status === 'SUCCESS') {
      image.setAttribute('src', 'chrome://cmgui/content/icons/ok.png');
    } else if (status === 'ERROR') {
      image.setAttribute('src', 'chrome://cmgui/content/icons/alert.png');
    } else if (status === 'WARNING') {
      image.setAttribute('src', 'chrome://cmgui/content/icons/warning.png');
    }
    row.appendChild(image);
    var stepDesc = document.createElement('shadowlabel');
    if(value.length > 80) {
      stepDesc.setAttribute('value', value.substring(0,80) + "...");
    }
    else {
      stepDesc.setAttribute('value', value);
    }
    row.appendChild(stepDesc);
    parentElementNode.insertBefore(row, ElementNodeSibling);
    if(Array.isArray(warnings) && warnings.length > 0) {
      warnings.forEach(function(warn){ 
        var row = document.createElement('row');
        row.setAttribute('align', 'center');
        var vbox = document.createElement('vbox');
        vbox.setAttribute('flex', '0');
        row.appendChild(vbox);
        var stepDesc = document.createElement('shadowlabel');
        stepDesc.setAttribute('value', warn);
        row.appendChild(stepDesc);
        parentElementNode.insertBefore(row, ElementNodeSibling);
      });
    }
  };

  this.giveAnswer = function(answer) {
    if(wizard.deploy.current_question !== null && wizard.deploy.questions.hasOwnProperty(wizard.deploy.current_question)) {
      var q_id = wizard.deploy.current_question;
      var setupExecutionID = wizard.deploy.questions[wizard.deploy.current_question].setupExecutionID;
      var elem_icon = document.getElementById('current-stage-icon');
      if(elem_icon) {
        elem_icon.setAttribute('src', "chrome://cmgui/content/icons/throbber.gif");
      }
      var elem_msg = document.getElementById('current-stage-message');
      if(elem_msg) {
        if(answer == 'r') {
          elem_msg.setAttribute('value', 'retrying stage');
        } else if(answer == 'u') {
          elem_msg.setAttribute('value', 'rolling back');
        } else if(answer == 's') {
          elem_msg.setAttribute('value', 'skipping');
        } else if(answer == 'a') {
          elem_msg.setAttribute('value', 'aborting');
        }
      }
      var cluster = current.hasOwnProperty('cluster') ? current.cluster : current;
      dock.workerpool.getWorker(cluster.actualurl).call('cmmain', 'cmSetupExecuteAnswer',
                                                        [ setupExecutionID, q_id, answer, []], function(code, data) {
        if (code == 0) {
          if(data.success) {
            wizard.deploy.questions[wizard.deploy.current_question].answer = answer;
            wizard.deploy.current_question = null;
            var elem = document.getElementById('deploy-question-element');
            if(elem) {
              elem.setAttribute('hidden',true);
            }
          }
          else {
            dock.displayError('Answer was not comitted: ' + data.message);
          }
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, cluster);
    }
  };

  this.toggleAdvancedAnswers = function(show) {
    var element_names = [ "wizard-deploy-answer-abort", "wizard-deploy-answer-skip", "wizard-deploy-answer-debug",
                          "deploy-question-skip-hint", "deploy-question-abort-hint", "deploy-question-debug-hint" ];
    for(var i = 0; i < element_names.length; i++) {
      var elem = document.getElementById(element_names[i]);
      if(elem) {
        if(show) elem.removeAttribute('hidden');
        else elem.setAttribute('hidden', true);
      }
    }
    var elem = document.getElementById("deploy-question-unhide-hint");
    if(elem) {
      if(show) elem.setAttribute('hidden', true);
      else elem.removeAttribute('hidden');
    }
  };

  this.initSuccess = function() {
    wizard.common.hide('deploy-success');
    wizard.common.hide('deploy-nodesToReboot');
    wizard.common.clearChildren('deploy-nodesToRebootList');
    wizard.common.hide('deploy-successSummary');
    wizard.common.clearChildren('deploy-successSummaryErrorWarnings');
  };

  this.showSuccess = function(errors, warnings) {
    wizard.common.unhide('deploy-success');
    wizard.common.disable('cancelButton');
    wizard.common.enable('finishButton');
    wizard.common.hide('deploy-detailedLogButton');
    wizard.common.hide('deploy-output');
    wizard.common.hide('deploy-copyLogContent');
    if (errors.length>0 || warnings.length>0) {
      wizard.common.unhide('deploy-successSummary');
      var successSummaryErrorWarnings = document.getElementById('deploy-successSummaryErrorWarnings');
      errors.forEach(function(error) {
        wizard.deploy.addRow(successSummaryErrorWarnings, null, error, 'ERROR');
      });
      warnings.forEach(function(warning) {
        wizard.deploy.addRow(successSummaryErrorWarnings, null, warning, 'WARNING');
      });
    }
    var elem_msg = document.getElementById('current-stage-message');
    if(elem_msg) {
      elem_msg.setAttribute('value', this.cm_setup_tool_name + " has been deployed successfully.");
    }
    var elem_icon = document.getElementById('current-stage-icon');
    if(elem_icon) {
      elem_icon.setAttribute('src', "chrome://cmgui/content/icons/ok.png");
    } else {
      printall("cannot find element current-stage-icon");
    }
    wizard.common.unhide('deploy-success-info');
    this.onShowSuccess();
  };

  this.initFailed = function() {
    wizard.common.hide('deploy-failed');
    wizard.common.hide('deploy-failedSummary');
    wizard.common.clearChildren('deploy-failedSummaryErrorWarnings');
  };

  this.showFailed = function(errors, warnings, aborted="") {
    println('show failed');
    wizard.common.disable('cancelButton');
    wizard.common.enable('finishButton');
    wizard.common.unhide('deploy-failed');
    wizard.common.hide('deploy-detailedLogButton');
    wizard.common.unhide('deploy-output');
    wizard.common.unhide('deploy-copyLogContent');
    //wizard.common.hide('deploy-info');
    if (errors.length>0 || warnings.length>0) {
      wizard.common.unhide('deploy-failedSummary');
      var successSummaryErrorWarnings = document.getElementById('deploy-failedSummaryErrorWarnings');
      errors.forEach(function(error) {
        wizard.deploy.addRow(successSummaryErrorWarnings, null, error, 'ERROR');
      });
      warnings.forEach(function(warning) {
        wizard.deploy.addRow(successSummaryErrorWarnings, null, warning, 'WARNING');
      });
    }
    var elem_msg = document.getElementById('current-stage-message');
    if(elem_msg) {
      elem_msg.setAttribute('value', this.cm_setup_tool_name + " deployment has failed.");
    }
    var elem_icon = document.getElementById('current-stage-icon');
    if(elem_icon) {
      elem_icon.setAttribute('src', "chrome://cmgui/content/icons/alert.png");
    } else {
      printall("element not found current-stage-icon");
    }
    this.onShowFailed();
  };

  this.prepareButtonsAfterInstall = function() {
    if (wizard.deploy.status === 'FAILED') {
      wizard.common.enable('cancelButton');
      wizard.common.enable('previousButton');
    } else {
      wizard.common.enable('finishButton');
    }
  };

  this.copyLogToClipboard = function() {
    var deployOutput = document.getElementById('deploy-output');
    if(deployOutput) {
      var str = "";
      for(var i = 0; i < deployOutput.childNodes.length; i++) {
        str+= deployOutput.childNodes[i].textContent + "\n";
      };
      copyToClipboard(str);
    }
  }
}
