'user strict';
global.__base = __dirname + '/../../';
var test = require('unit.js');
var expect    = require("chai").expect;
var Context = require('cm-context');

var context;
var wizard;

var wizard_module_ctx;

function testSetup() {
  context = new Context(global.__base);
  wizard_module_ctx = context.load('cm-setup/wizard.js');
  wizard = new TestWizard();
  wizard.common.setStage = function() {
    // mockup
  };
  // don't be verbose
  context.context.println = function() {
  }
}

function MainWizardScreen() {
  wizard_module_ctx.WizardScreen.call(this);

  this.screen_name = "main";

  this.enter = function() {
    context.context.println('enter wizard.main.enter');
  }
  this.forward = function() {
    return true;
  }
}

function WizardScreen2() {
  wizard_module_ctx.WizardScreen.call(this);

  this.screen_name = "screen2";

  this.enter = function() {
  //  println('enter wizard.main.enter');
  }

  this.forward = function() {
    return true;
  }
}

function WizardScreen3() {
  wizard_module_ctx.WizardScreen.call(this);

  this.screen_name = "screen3";

  this.enter = function() {
//    println('enter wizard.main.enter');
  }

  this.forward = function() {
    return true;
  }
}

function DeployScreen() {
  wizard_module_ctx.WizardScreen.call(this);

  this.screen_name = "deploy";

  this.enter = function() {
//    println('enter wizard.main.enter');
  }

  this.forward = function() {
    return true;
  }
}


function TestWizard() {
  wizard_module_ctx.Wizard.call(this);
  this.main    = new MainWizardScreen();
  this.screen2 = new WizardScreen2();
  this.screen3 = new WizardScreen3();
  this.deploy  = new DeployScreen();
  this.deck = {
    main: {
      name: 'main',
      next: 'screen2',
      previous: null,
      automatic: false,
      index: 0
    },
    screen2: {
      name: 'screen2',
      next: 'screen3',
      previous: 'main',
      automatic: true,
      index: 1
    },
    screen3: {
      name: 'screen3',
      next: 'deploy',
      previous: 'screen2',
      automatic: true, 
      index: 2
    },
    deploy: {
      name: 'deploy',
      next: null,
      previous: 'screen3',
      automatic: false,
      index: 3
    }
  };
}

describe('Wizard Navigation', function(){
  it('current stage is null before init', function() {
    testSetup();
    expect(wizard.currentStage).to.be.null;
    expect(wizard.previousStage).to.be.null;
  });
  it('current stage is first main screen after go', function() {
    testSetup();
    wizard.common.go(wizard.common.getNextStage.bind(undefined, null));
    expect(wizard.previousStage).to.be.null;
    expect(wizard.currentStage).to.be.a('object');
    expect(wizard.currentStage.name).to.be.equal('main');
    expect(wizard.common.currentDeck()).to.be.equal(wizard.main);
  });

  it('navigate next, step by step (wizard.advancedMode == true)', function() {
    testSetup();
    wizard.common.go(wizard.common.getNextStage.bind(undefined, null));
    wizard.advancedMode = true;
    wizard.common.next(false);
    expect(wizard.currentStage.name).to.be.equal('screen2');
    expect(wizard.common.currentDeck()).to.be.equal(wizard.screen2);
    expect(wizard.previousStage.name).to.be.equal('main');

    wizard.common.next(false);
    expect(wizard.currentStage.name).to.be.equal('screen3');
    expect(wizard.common.currentDeck()).to.be.equal(wizard.screen3);
    expect(wizard.previousStage.name).to.be.equal('screen2');

    wizard.common.next(false);
    expect(wizard.currentStage.name).to.be.equal('deploy');
    expect(wizard.common.currentDeck()).to.be.equal(wizard.deploy);
    expect(wizard.previousStage.name).to.be.equal('screen3');
  });

  it('navigate next, express (wizard.advancedMode == false)', function() {
    testSetup();
    wizard.common.go(wizard.common.getNextStage.bind(undefined, null));
    wizard.advancedMode = false;
    wizard.common.next(false);
    expect(wizard.currentStage.name).to.be.equal('deploy');
    expect(wizard.common.currentDeck()).to.be.equal(wizard.deploy);
    expect(wizard.previousStage.name).to.be.equal('screen3');
  });


  it('navigate next, step by step (wizard.advancedMode == true) with failures', function() {
    testSetup();
    wizard.common.go(wizard.common.getNextStage.bind(undefined, null));
    wizard.advancedMode = true;


    wizard.common.next(true);
    expect(wizard.currentStage.name).to.be.equal('screen2');
    expect(wizard.common.currentDeck()).to.be.equal(wizard.screen2);
    expect(wizard.previousStage.name).to.be.equal('main');

    wizard.common.next(true);
    expect(wizard.currentStage.name).to.be.equal('screen3');
    expect(wizard.common.currentDeck()).to.be.equal(wizard.screen3);
    expect(wizard.previousStage.name).to.be.equal('screen2');

    // failing screen3 -> don't navigate to deploy
    wizard.screen3.forward = function() {
      wizard.screen3.error = true;
    };
    wizard.common.next(true);
    expect(wizard.currentStage.name).to.be.equal('screen3');
    expect(wizard.common.currentDeck()).to.be.equal(wizard.screen3);
    expect(wizard.previousStage.name).to.be.equal('screen2');
    expect(wizard.common.hasError(true)).to.be.true;


    // correct errors
    wizard.screen3.forward = function() {
      wizard.screen3.error = false;
    };

    wizard.common.next(true);
    expect(wizard.currentStage.name).to.be.equal('deploy');
    expect(wizard.common.currentDeck()).to.be.equal(wizard.deploy);
    expect(wizard.previousStage.name).to.be.equal('screen3');
  });


  it('navigate next, express (wizard.advancedMode == false) with failures ', function() {
    testSetup();
    wizard.common.go(wizard.common.getNextStage.bind(undefined, null));
    wizard.advancedMode = false;

    // failing screen3 -> don't navigate to deploy
    wizard.screen3.forward = function() {
      wizard.screen3.error = true;
    };
    wizard.common.next(true);
    expect(wizard.currentStage.name).to.be.equal('screen3');
    expect(wizard.common.currentDeck()).to.be.equal(wizard.screen3);
    expect(wizard.previousStage.name).to.be.equal('screen2');

    // correct errors
    wizard.screen3.forward = function() {
      wizard.screen3.error = false;
    };

    wizard.common.next(true);
    expect(wizard.currentStage.name).to.be.equal('deploy');
    expect(wizard.common.currentDeck()).to.be.equal(wizard.deploy);
    expect(wizard.previousStage.name).to.be.equal('screen3');
  });
});
