/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var myDialog;

function setSimpleRadio(choice, elementIds) {
  elementIds.forEach(function(elementId, index) {
    var obj = document.getElementById(elementId);
    if (obj.getAttribute('disabled')!=='true') {
      if (choice===index) {
        obj.setAttribute('on', true);
      } else {
        obj.setAttribute('on', false);
      }
    }
  });
}


function fillListDropdown(args) {
  var popupId = args.popupId;
  var list = (typeof args.list !== 'undefined' ) ? args.list : [];
  var selectedElement = args.selectedElement;
  var clear = (typeof args.clear !== 'undefined' ) ? args.clear : true;

  var noneSelected = true;
  var drop = document.getElementById(popupId);
  var popup = drop.firstChild;
  if (!popup) {
    println("!!! ERROR !!!: has no menupopup");
    return;
  }

  if (clear) {
    var current = popup.firstChild;
    var next = null;
    while (current) {
      next = current.nextSibling;
      if (current.getAttribute('clear') !== 'false') {
        popup.removeChild(current);
      }
      current = next;
    }
  }

  for (var i = 0; i < list.length; i++) {
    if (Array.isArray(list[i])) {
      for (var j = 0; j < list[i].length; j++) {
        var index = popup.childNodes.length - list.length + i + 1;
        addItem(popup.childNodes[index], list[i][j]);
      }
    } else {
      addItem(null, list[i]);
    }
  }
  if (selectedElement!==undefined && selectedElement.defaultIndex!==undefined && noneSelected) {
    if (typeof drop.editable !== 'undefined' && drop.editable === true) {
      drop.value = selectedElement.value;
    } else {
      drop.selectedIndex = selectedElement.defaultIndex;
    }
  }

  function addItem(referenceNode, element) {
    if (typeof element.uniqueKey !== 'undefined') {
      var key = element.uniqueKey;
    } else if (typeof element.get === 'function') {
      var key = element.get('uniqueKey');
    } else {
      var key = i;
    }
    if (typeof element.resolveName !== 'undefined') {
      var name = element.resolveName();
    } else if (typeof element.name !== 'undefined') {
      var name = typeof element.name;
    } else {
      var name = element;
    }

    var item = document.createElement('menuitem');
    item.setAttribute('key', key);
    item.setAttribute('label', name);
    item.setAttribute('value', name);
    //popup.appendChild(item);
    popup.insertBefore(item, referenceNode);
    if (selectedElement!==undefined) {
      if (selectedElement.key!==undefined && selectedElement.key===key || selectedElement.label!==undefined && selectedElement.label===name || selectedElement.value!==undefined && selectedElement.value===name) {
        drop.selectedIndex = i;
        noneSelected = false;
      }
    }
  }
}

function fillListBox(args) {
  var popupId = args.elementNodeId;
  var list = (typeof args.list !== 'undefined' ) ? args.list : [];
  var selectedElement = args.selectedElement;
  var clear = (typeof args.clear !== 'undefined' ) ? args.clear : true;

  var noneSelected = true;
  var listbox = document.getElementById(popupId);
  if (!listbox) {
    println("!!! ERROR !!!: " + popupId + " not found.");
    return;
  }
  if (clear) {
    while (listbox.firstChild) {
      listbox.removeChild(listbox.firstChild);
    }
  }

  for (var i = 0; i < list.length; i++) {
    if (typeof list[i].uniqueKey !== 'undefined') {
      var key = list[i].uniqueKey;
    } else if (typeof list[i].get === 'function') {
      var key = list[i].get('uniqueKey');
    } else {
      var key = i;
    }
    if (typeof list[i].resolveName !== 'undefined') {
      var name = list[i].resolveName();
    } else if (typeof list[i].name !== 'undefined') {
      var name = typeof list[i].name;
    } else {
      var name = list[i];
    }

    var cell = document.createElement('listcell');
    cell.setAttribute('key', key);
    cell.setAttribute('label', name);
    cell.setAttribute('value', name);
    var item = document.createElement('listitem');
    item.appendChild(cell);
    listbox.appendChild(item);

    if (selectedElement!==undefined) {
      if (selectedElement.key   !== undefined && selectedElement.key.indexOf(key)     > -1 ||
          selectedElement.label !== undefined && selectedElement.label.indexOf(name) > -1 ||
          selectedElement.value !== undefined && selectedElement.value.indexOf(name) > -1) {
        listbox.addItemToSelection(item);
        noneSelected = false;
      }
    }
  }
  if (selectedElement!==undefined && selectedElement.defaultIndex!==undefined && noneSelected) {
    listbox.selectedIndex = selectedElement.defaultIndex;
  }
}

function toggleViewPassword(tag) {
  if(Array.isArray(tag)) {
    tag.forEach(toggleViewPassword);
  }
  else {
    if (document.getElementById(tag).getAttribute('type')==='password') {
      document.getElementById(tag).removeAttribute('type');
      var elem = document.getElementById(tag + '2');
      if(elem) {
        elem.removeAttribute('type');
      }
    } else {
      document.getElementById(tag).setAttribute('type', 'password');
      var elem = document.getElementById(tag + '2');
      if(elem) {
        elem.setAttribute('type', 'password');
      }
    }
  }
}

function setBaseNetmask(elements, ipaddr, netmaskBits, unset) {
  for (var i = 0; i < 4; i++) {
    var octet = ipaddr.octets[i];
    if (netmaskBits >= (8 * (i+1))) {
      document.getElementById(elements[i]).setAttribute('disabled', true);
    } else {
      document.getElementById(elements[i]).removeAttribute('disabled');
      if (unset) {
        octet = '';
      }
    }
    document.getElementById(elements[i]).value = octet;
  }
}

function copyToClipboard(copytext) { 
  //An XPCOM wrapper for the data which you want to put on the clipboard.
  var str = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
  if (!str) return false;
  str.data = copytext;
  //This object is the component @mozilla.org/widget/transferable;1 which implements the interface nsITransferable.
  var trans = Components.classes["@mozilla.org/widget/transferable;1"].
    createInstance(Components.interfaces.nsITransferable);
  if (!trans) return false;
  trans.addDataFlavor("text/unicode");
  trans.setTransferData("text/unicode", str, copytext.length * 2);
  var clipid = Components.interfaces.nsIClipboard;
  var clip = Components.classes["@mozilla.org/widget/clipboard;1"].getService(clipid);
  if (!clip) return false;
  clip.setData(trans, null, clipid.kGlobalClipboard);
} 

function WizardScreen(arg_wizard) {
  this.error = false;
  this.started = false;
  if(!!arg_wizard) {
    this.wizard = wizard;
  }
  else {
    this.wizard = arg_wizard;
  }
  this.initDefaultConfig = function() {
  };

  this.hasHelp = function() {
    return (document.getElementById(wizard.currentStage.name + '-help') !== null);
  };

  this.getHelp = function() {
    return document.getElementById(wizard.currentStage.name + '-help').innerHTML;
  };

  this.reset = function() {
    throw Error('Must be implemented in child.');
  };

  this.preEnter = function() {
  };

  this.enter = function() {
    throw Error('Must be implemented in child.');
  };

  this.leave = function() {
  };

  this.forward = function(automatic) {
    // validate form data and store it to configuration 
    // if validation is not successful the error flag of the WizardScreen should be set
    //  wizard.the_screen.error = true;
    // in this case wizard.next() will not proceed to the next screen 
    // even in expressMode.
    throw Error('Must be implemented in child.');
  };

  this.asyncForward = function(automatic, followup) {
    // override this function if async. validation is required
    this.forward(automatic);
    followup();
  };

}

function Wizard() {
  this.dryMode = false;
  this.forceDryMode = false;
  // if advancedMode == true, navigate through all stages
  // if advancedMode == false, jump over stages with automatic == true
  //                           if stage forward function of automatic stage 
  //                           returns false, go to that stage
  this.advancedMode = false;
  this.lastConfigSaved = false;
  this.currentStage = null;
  this.previousStage = null;
  this.configuration_name = "Configuration"; // can be overwritten like "OpenStack Configuration"
  this.deck = {
  };
  this.config = {};

  this.createDeck = function(data) {
    var ret = {};
    for(var i = 0; i < data.length; i++) {
      var rec = {
        name:      data[i].name,
        previous:  (i > 0 ? data[i-1].name : null),
        next:      ( (i < data.length-1) ? data[i+1].name : null),
        automatic: (data[i].hasOwnProperty('automatic') && data[i].automatic),
        index:     i,
        enable_click : (data[i].hasOwnProperty('enable_click') && data[i].enable_click),
      }
      ret[rec.name] = rec;
    }
    return ret;
  };
  var wizard = this;
  this.common = {
    beforeInitDefaultConfig: function(followup){
      followup();
    },
    initDefaultConfig: function() {
      this.beforeInitDefaultConfig(function() {
        printall("initDefaultConfig");
        Object.keys(wizard.deck).forEach(function(deckName){
          if(wizard.hasOwnProperty(deckName)) {
            wizard[deckName].initDefaultConfig();
          } else {
            throw Error("wizard." + deckName + " undefined");
          }
        });
      });
    },
    setConfig: function(obj, variable, value) {
      if (obj[variable] !== value) {
        // println('setConfig: ' + variable + ', value: ' + value);
        obj[variable] = value;
        wizard.common.unsetLastConfigSaved();
      }
    },

    getStage: function(name) {
      return wizard.deck[name];
    },

    getNextStage: function(currentStage) {
      if (currentStage===null) {
        return wizard.deck.main;
      } else if (currentStage.next === null) {
        return currentStage;
      } else if (typeof currentStage.next === 'function') {
        return wizard.deck[currentStage.next()];
      } else {
        if(wizard.deck.hasOwnProperty(currentStage.next)) {
          return wizard.deck[currentStage.next];
        } else {
          throw Error('invalid transition ' + currentStage.name + ' -> ' + currentStage.next);
        }
      }
    },

    getPreviousStage: function(currentStage) {
      if (currentStage===null) {
        return wizard.deck.main;
      } else if (currentStage.previous === null) {
        return currentStage;
      } else if (typeof currentStage.previous === 'function') {
        return wizard.deck[currentStage.previous()];
      } else {
        return wizard.deck[currentStage.previous];
      }
    },

    currentDeck: function() {
      return wizard[wizard.currentStage.name];
    },

    go: function(getStage) {
      var currentStage = getStage();
      var async = false;
      if (currentStage!==wizard.currentStage) {
        if(wizard.currentStage && currentStage.index > wizard.currentStage.index) {
          wizard.common.resetErrors();
          wizard.common.currentDeck().leave(false);
          async = true;
          wizard.common.currentDeck().asyncForward(false,
                                                   function() {
	                                             if (!wizard.common.hasError(false)) {
                                                       wizard.previousStage = wizard.currentStage;
                                                       wizard.currentStage = currentStage;
                                                       println('go to ' + wizard.currentStage.name);
                                                       wizard.common.setStage();
                                                       wizard.common.currentDeck().preEnter();
                                                       wizard.common.currentDeck().enter();
                                                     }
                                                   });
        }
        else {
          wizard.previousStage = wizard.currentStage;
          wizard.currentStage = currentStage;
        }
      };
      if(!async) {
        println('go to ' + wizard.currentStage.name);
        wizard.common.setStage();
        wizard.common.currentDeck().preEnter();
        wizard.common.currentDeck().enter();
      }
    },

    goto: function(deckName) {
      //    wizard.common.resetErrors();
      wizard.common.currentDeck().leave(false);
      //    if (!wizard.common.hasError()) {
      wizard.common.go(wizard.common.getStage.bind(undefined, deckName));
      //    }
    },

    next: function(automatic) {
      println('next from: ' + wizard.currentStage.name + ', automatic: ' + automatic);

      wizard.common.resetErrors();
      wizard.common.currentDeck().leave(automatic);
      wizard.common.currentDeck().asyncForward(automatic,
         function() {
	   if (!wizard.common.hasError(automatic)) {
             if (wizard.currentStage.name==='deploy') {
               wizard.common.finish();
             } else {
               var getStage = wizard.common.getNextStage.bind(undefined, wizard.currentStage);
               var currentStage = getStage();
               wizard.common.resetErrors();
               wizard.common.currentDeck().leave(false);
               wizard.previousStage = wizard.currentStage;
               wizard.currentStage = currentStage;
               println('go to ' + wizard.currentStage.name);
               wizard.common.setStage();
               wizard.common.currentDeck().preEnter();
               wizard.common.currentDeck().enter();
             }
             if (!wizard.advancedMode && wizard.currentStage.automatic) {
               wizard.common.next(true);
             }
	   }
	 });
    },

    previous: function(automatic) {
      println('previous from: ' + wizard.currentStage.name + ', automatic: ' + automatic);

      //    wizard.common.resetErrors();
      wizard.common.currentDeck().leave(false);
      //    if (!wizard.common.hasError()) {
      wizard.common.go(wizard.common.getPreviousStage.bind(undefined, wizard.currentStage));
      //    }
    },

    resetErrors: function() {
      wizard.common.currentDeck().error = false;
    },

    hasError: function(automatic) {
      if (!automatic && wizard.common.currentDeck().error) {
        dock.displayError("The form has errors. Please correct it.");
      }
      return wizard.common.currentDeck().error;
    },

    setStage: function() {
      function highlightCurrentStageInMenu() {
        if (wizard.currentStage===null) {
          return;
        }
        var elementNode = document.getElementById('stage-' + wizard.currentStage.name);
        if (elementNode) {
          elementNode.classList.add('current');
        }
      }

      function unhighlightPreviousStageInMenu() {
        if (wizard.previousStage===null) {
          return;
        }
        var elementNode = document.getElementById('stage-' + wizard.previousStage.name);
        if (elementNode) {
          elementNode.classList.remove('current');
        }
      }

      function setHelpButtonStatus() {
        if (wizard.common.currentDeck().hasHelp()) {
          wizard.common.enable('helpButton');
        } else {
          wizard.common.disable('helpButton');
        }
      }

      function setNextButtonStatus() {
        if (wizard.currentStage.next===null) {
          wizard.common.disable('nextButton');
        } else {
          wizard.common.enable('nextButton');
        }
      }

      function setPreviousButtonStatus() {
        if (wizard.currentStage.previous===null) {
          wizard.common.disable('previousButton');
        } else {
          wizard.common.enable('previousButton');
        }
      }

      function setAdvancedMode() {
        if (wizard.advancedMode) {
          wizard.common.setUnvisible('common-basicCheck');
          wizard.common.setVisible('common-advancedCheck');
        } else {
          wizard.common.setVisible('common-basicCheck');
          wizard.common.setUnvisible('common-advancedCheck');
        }
      }

      function setDeck() {
        document.getElementById('deck').selectedIndex = wizard.currentStage.index;
      }

      function refreshMenu() {
        function enable(elementNodeId) {
          wizard.common.clearEvents(elementNodeId);
          wizard.common.enable(elementNodeId);
          document.getElementById(elementNodeId).addEventListener('click', function(event) {
            var deckName = event.target.id.substr(6);
            wizard.common.goto(deckName);
          }, false);
          document.getElementById(elementNodeId).classList.add('activated');
        }

        function disable(elementNodeId) {
          wizard.common.clearEvents(elementNodeId);
          wizard.common.disable(elementNodeId);
          document.getElementById(elementNodeId).classList.remove('activated');
        }

        var stage = null;
        var stageDone = {};
        if (wizard.currentStage.name !== 'deploy') {
          do {
            stage = wizard.common.getNextStage(stage);
            if (stage.index !== null) {
              enable('stage-' + stage.name);
            }
            stageDone[stage.name] = stage;
          } while (stage.next !== null && stage.name !== wizard.currentStage.name);
        }

        Object.keys(wizard.deck).forEach(function (stageName) {
          if (stageDone[stageName] === undefined && wizard.deck[stageName].index !== null) {
            if(!wizard.deck[stageName].hasOwnProperty('enable_click') || !wizard.deck[stageName].enable_click) {
              disable('stage-' + stageName);
            }
          }
        });
      }

      highlightCurrentStageInMenu();
      unhighlightPreviousStageInMenu();
      setHelpButtonStatus();
      setNextButtonStatus();
      setPreviousButtonStatus();
      setAdvancedMode();
      setDeck();
      refreshMenu();
    },

    cancel: function() {
      var currentDeck = wizard.common.currentDeck();
      if(typeof(currentDeck.onCancel) === 'function') {
        currentDeck.onCancel();
      }
      else {
        var args = new Object();
        args['question'] = 'Are you sure you want to quit the wizard?';
        var d = new cvDialog("chrome://cmgui/content/question.xul", args);
        d.onOk = function() {
          close();
        };
        dock.dialogs.showDialog(d);
      }
    },

    finish: function () {
      if (!wizard.lastConfigSaved) {
        var args = new Object();
        args['question'] = 'Are you sure you want to exit the wizard without saving the current configuration?';
        var d = new cvDialog("chrome://cmgui/content/question.xul", args);
        d.onOk = function() {
          close();
        };
        dock.dialogs.showDialog(d);
      } else {
        close();
      }
    },

    hide: function(elementNodeId) {
      var elem = document.getElementById(elementNodeId);
      if(!elem) {
        printall("undefined element " + elementNodeId);
      } else {
        elem.setAttribute('hidden', true);
      }
    },

    unhide: function(elementNodeId) {
      var elem = document.getElementById(elementNodeId);
      if(!elem) {
        printall("undefined element " + elementNodeId);
      } else {
        elem.removeAttribute('hidden');
      }
    },

    toggleHide: function(elementNodeId) {
      if (document.getElementById(elementNodeId).getAttribute('hidden') === 'true') {
        wizard.common.unhide(elementNodeId);
      } else {
        wizard.common.hide(elementNodeId);
      }
    },

    setVisible: function(elementNodeId) {
      var elem = document.getElementById(elementNodeId);
      if(elem) {
        elem.classList.remove('invisible');
      } else {
        println("setVisible(): elementNodeId " + elementNodeId);
      }
    },

    setUnvisible: function(elementNodeId) {
      var elem = document.getElementById(elementNodeId);
      if(elem) {
        elem.classList.add('invisible');
      } else {
        println("setUnVisible(): elementNodeId " + elementNodeId);
      }
    },

    toggleVisibility: function(elementNodeId) {
      if (document.getElementById(elementNodeId).classList.contains('invisible')) {
        document.getElementById(elementNodeId).classList.remove('invisible');
      } else {
        document.getElementById(elementNodeId).classList.add('invisible');
      }
    },

    disable: function(elementNodeId) {
      document.getElementById(elementNodeId).setAttribute('disabled', true);
    },

    enable: function(elementNodeId) {
      document.getElementById(elementNodeId).removeAttribute('disabled');
    },

    clearEvents: function (elementNodeId) {
      var elementNode = document.getElementById(elementNodeId);
      if(elementNode) {
        var elementNodeClone = elementNode.cloneNode(true);
        elementNode.parentNode.replaceChild(elementNodeClone, elementNode);
      } else {
        throw Error('Invalid node ' + elementNodeId);
      }
    },

    clearChildren: function(elementNodeId) {
      var elementNode = document.getElementById(elementNodeId);
      if(!elementNode) {
        printall("element not defined " + elementNodeId);
      } else {
        while (elementNode.childNodes.length > 0) {
          elementNode.removeChild(elementNode.lastChild);
        }
      }
    },

    displayError: function(errorElementId, automatic) {
      if (!automatic) {
        wizard.common.unhide(errorElementId);
      }
    },

    /////////////////////////////////////////////////
    // Managing fields 
    /////////////////////////////////////////////////
    getFieldValue : function(elementId) {
      var elem = document.getElementById(elementId);
      if(elem) {
        return elem.value;
      }
      else {
        throw Error('Field ' + elementId + ' not found.');
      }
    },

    getFieldValues : function(elementKeyMapping, obj) {
      for(var elementId in elementKeyMapping) {
        var elem = document.getElementById(elementId);
        var key = elementKeyMapping[elementId];
        if(elem) {
          obj[key] = elem.value;
        }
        else {
          throw Error('Field ' + elementId + ' not found.');
        }
      }
    },

    setFieldValue : function(elementId, value) {
      var elem = document.getElementById(elementId);
      if(elem) {
        elem.value = value;
      }
      else {
        throw Error('Field ' + elementId + ' not found.');
      }
    },
    
    setFieldValues : function(elementKeyMapping, obj) {
      for(var elementId in elementKeyMapping) {
        var elem = document.getElementById(elementId);
        var key = elementKeyMapping[elementId];
        println(elementId + " " + key);
        if(elem) {
          if(obj.hasOwnProperty(key)) {
            elem.value = obj[key];
          }
          else {
            throw Error('Object has no key ' + key  + '.');
          }
        }
        else {
          throw Error('Field ' + elementId + ' not found.');
        }
      }
    },
    
    /////////////////////////////////////////////////
    // Loading, Showing, Saving Configurations
    /////////////////////////////////////////////////
    configToYaml : function() {
      function doExpand() {
        var config = JSON.parse(JSON.stringify(wizard.config));
        return config;
      }
      return jsyaml.dump(doExpand(), {
        flowLevel: -1
      });
    },

    setLastConfigSaved : function() {
      wizard.lastConfigSaved = true;
      //wizard.common.disable('saveConfigButton');
    },

    unsetLastConfigSaved : function() {
      wizard.lastConfigSaved = false;
      //wizard.common.enable('saveConfigButton');
    },

    showConfiguration : function() {
      wizard.common.resetErrors();
      wizard.common.currentDeck().leave(false);
      wizard.common.currentDeck().asyncForward(false, function() {
	if (!wizard.common.hasError(false)) {
          var args = new Object();
          args['title'] = wizard.configuration_name;
          args['style'] = 'background-color: white !important;';
          args['text'] = wizard.common.configToYaml();
          var d = new dock.cvDialog(dock.myVersion + "showtext.xul", args);
          dock.dialogs.showDialog(d);
	}
	else {
          return;
	}
      });
    },  // showConfig

    saveConfiguration : function() {
      wizard.common.resetErrors();
      function saveMe() {
	if (!wizard.common.hasError(false)) {
          var file ='';
          var nsIFilePicker = Components.interfaces.nsIFilePicker;
          var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
          fp.init(window, "Save " + wizard.configuration_name + " setup as", nsIFilePicker.modeSave);
          var path = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
          path.initWithPath(dock.BASE_PATH);
          fp.displayDirectory = path;
          fp.appendFilter(wizard.configuration_name, "*.yaml");
          var res = fp.show();
          if ((res == nsIFilePicker.returnOK) || (res == nsIFilePicker.returnReplace)) {
            file = fp.file.path;
            if (!endswith(file, '.yaml')) {
              file += '.yaml';
            }
          } else {
            return;
          }
          savefile(file, wizard.common.configToYaml());
          wizard.common.setLastConfigSaved();
	}
	else {
          return ;
	}
      };

      if (wizard.currentStage.name !== 'deploy' &&
          wizard.currentStage.name !== 'summary') {
        wizard.common.currentDeck().leave(false);
        wizard.common.currentDeck().asyncForward(false, saveMe);
      } else {
	saveMe();
      }
    }, // saveConfiguration

    loadConfiguration : function() { 
      var win = dock.windowMediator.getMostRecentWindow('detachedopenstackloadconfig');
      if ( win ) {
        dock.windowMediator.getMostRecentWindow('detachedopenstackloadconfig').focus();
        return false;
      }
      var args = new Object();
      args['title'] = wizard.configuration_name;
      args['style'] = 'background-color: white !important;';
      args['validation'] = function(yamlContent) {
        if (typeof yamlContent.modules === 'undefined' && typeof yamlContent.modules.core === 'undefined') {
          throw 'Invalid config file';
        }
      };
      var d = new dock.cvDialog("chrome://cmgui/content/openstack/install/loadconfig.xul", args);
      d.onOk = function (config, fileName){
        if(!config.hasOwnProperty('setup')) {
          config['setup'] = {
            "only_reboot_controllers" : false,
            "max_reboot_wait_sec" : 1200
          };
        }
        wizard.config = config;
        wizard.common.resetErrors();
        wizard.common.currentDeck().leave(false);
        wizard.common.currentDeck().forward(false);
        wizard.common.go(wizard.common.getStage.bind(undefined, 'main'));
      };
      dock.dialogs.showDialog(d);
    }, // load configuration


    ///////////////////////////////////////////////////
    // remote validation 
    ///////////////////////////////////////////////////
    remoteValidation : function(operation, obj, handler_function) {
      var self = this;
      var obj_json = JSON.stringify(obj);
      var cluster = current.hasOwnProperty('cluster') ? current.cluster : current;
      function checkResult(id) {
                println('checking validation result');
                dock.workerpool.getWorker(cluster.actualurl).call('cmmain', 'cmSetupGetRemoteRequest', [ id, false],
                function(code, data) {
                  if (code == 0) {
                    if(data.state == 2 || data.state == 3 || data.state == 4) {
                      if(data.state == 3) {
                        data.error = data.result;
                        data.result = {};
                      } else if(data.state == 4) {
                        data.error = "undefined request " + id;
                        data.result = {};
                      } else {
                        data.result = JSON.parse(data.result);
                        data.error = "";
                      }
                      // workaround to remove the request (needs to be done on cmdaemon side when returing a non pending job 
                      dock.workerpool.getWorker(cluster.actualurl).call('cmmain', 'cmSetupGetRemoteRequest', [ id, true],
                                                                        function(code, data) {});
                      handler_function(data);
                    }
                    else {
                      window.setTimeout(function() { checkResult(id); }, 1000);
                    }
                  } else {
                    dock.jsonCallError(this, code, data);
                  }
                }, cluster);
      }
     printall("cmSetupRemoteRequest " + operation);
     dock.workerpool.getWorker(cluster.actualurl).call('cmmain', 'cmSetupRemoteRequest', [ operation, obj_json ],
       function(code, data) {
                 if (code == 0) {
                   printall("cmSetupRemoteRequest ID=" + data);
                   checkResult(parseInt(data));
                 } else {
                   dock.jsonCallError(this, code, data);
                 }
       }, cluster);
    }
  } // common
}


