/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var old;
var cluster;
var nodes;
var myDialog;
var myDialogIndex;
var okPressed = false;
var is_puppet_role = false;
var puppet_role = null;

var attachable_roles = {};
var selected_role_key = null;
var overlay_controller = null;
var overlay_view = null;
var loaded = false;

function OverlayEditView() {
  this.tabs = document.getElementById('tabs');
  this.tabpanels = document.getElementById('tabpanels');

  this.add_role_popup = document.getElementById('selectrole');
  this.add_role_popup2 = document.getElementById('selectrole2');
  this.role_groups_menu = {};
  this.role_groups_menu2 = {};
  this.current_view_suffix = "";

  this.group_tabs   = {};
  this.group_panels = {};

  this.reset = function() {
    this.group_tabs   = {};
    this.group_panels = {};
    if(this.tabs) {
      while (this.tabs.firstChild) this.tabs.removeChild(this.tabs.firstChild);
    }
    if(this.tabpanels) {
      while (this.tabpanels.firstChild) this.tabpanels.removeChild(this.tabpanels.firstChild);
    }
  }

  this.addPuppetSettingsTab = function() {
    if(this.tabs.children.length == 0) {
      var shadowtab = document.createElement('shadowtab');
      setAttributes(shadowtab, { 
        'subtab'  : 'true', 
        'id'      : 'tab_settings', 
        'label'   : 'Settings / Variables / Preamble ', 
      });
      var iframeelem = document.createElement('iframe');
      var iframesrc  = 'chrome://cmgui/content/puppet/configoverlaypuppetsettings.xul';
      setAttributes(iframeelem, {
        'id': 'iframe_settings', 
        'src': iframesrc });
      this.tabs.appendChild(shadowtab);
      this.tabpanels.appendChild(iframeelem);
    }
  }
    
  this.addGroupTab = function(group_name) {
    var shadowtab = document.createElement('shadowtab');
    shadowtab.setAttribute('id',    'tab_groups_' + group_name);
    shadowtab.setAttribute('label',  group_name);
    this.tabs.appendChild(shadowtab);
    var tabpanel = document.createElement('tabpanel');
    tabpanel.setAttribute("id", 'tabpanel_groups_' + group_name);
    var tabbox  = document.createElement('tabbox');
    tabbox.setAttribute("flex", 1);
    tabbox.setAttribute("id", 'tabbox_groups_' + group_name);
    var scrollbox = document.createElement('arrowscrollbox');
    var sub_tabs  = document.createElement('tabs');
    sub_tabs.setAttribute("id", "tabs_groups_" + group_name);
    var sub_tabpanels = document.createElement('tabpanels');
    sub_tabpanels.setAttribute("id", "tabpanels_groups_" + group_name);
    sub_tabpanels.setAttribute("flex", 1);
    scrollbox.appendChild(sub_tabs);
    tabbox.appendChild(scrollbox);
    tabbox.appendChild(sub_tabpanels);
    tabpanel.appendChild(tabbox);
    this.tabpanels.appendChild(tabpanel);
    this.group_tabs[group_name]  = sub_tabs;
    this.group_panels[group_name] = sub_tabpanels;
  }

  this.createTab = function(role_conf, role, group_name = "", create_subtab=false) {
    var shadowtab = document.createElement('shadowtab');
    var uniqueKey = role.hasOwnProperty('object') ? role.object.uniqueKey : role.uniqueKey;
    shadowtab.setAttribute('id',    'tab_' + uniqueKey);
    var label = role_conf.label;
    if(!create_subtab && group_name != "") {
      label = group_name + ": " + label;
    }
    shadowtab.setAttribute('label', label);
    shadowtab.setAttribute('parent_group', group_name);
    var iframeelem = document.createElement('iframe');
    iframeelem.setAttribute('id',  'iframe_' + uniqueKey);
    iframeelem.current = role;
    iframeelem.current_overlay = current;
    iframeelem.setAttribute('src', role_conf.src);
    if(!create_subtab || group_name == "") {
      this.tabs.appendChild(shadowtab);
      this.tabpanels.appendChild(iframeelem);
    }
    else { 
      if(!this.group_tabs.hasOwnProperty(group_name)) {
        this.addGroupTab(group_name);
      }
      this.group_tabs[group_name].appendChild(shadowtab);
      this.group_panels[group_name].appendChild(iframeelem);
    }
  }


  this.addMenuItem = function(role_conf) {
    function addMenuGroup(add_role_popup, role_groups_menu, group_name, role_conf, group_suffix="") {
      if(!role_groups_menu.hasOwnProperty(group_name)) {
        var menu = document.createElement("menu");
        menu.setAttribute("label", group_name);
        menu.setAttribute("value", group_name);
        menu.setAttribute("id",    "menu_group" + group_suffix + "_" + group_name);
        var menupopup = document.createElement("menupopup");
        menu.appendChild(menupopup);
        add_role_popup.appendChild(menu);
        role_groups_menu[group_name] = menupopup;
      }
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute("value",  role_conf.name);
      menuitem.setAttribute("label",  role_conf.label);
      menuitem.setAttribute("id",     "menu_role" + group_suffix + "_" + role_conf.name);
      //menuitem.setAttribute("class", "staticdataheader");
      role_groups_menu[group_name].appendChild(menuitem);
    }
    if(this.add_role_popup) {
      addMenuGroup(this.add_role_popup,  this.role_groups_menu, role_conf.group, role_conf);
      addMenuGroup(this.add_role_popup2, this.role_groups_menu2, role_conf.group2, role_conf,"2");
    }
  }

  this.addMenuLabel = function(group_name, label) {
    if(!this.role_groups_menu.hasOwnProperty(group_name)) {
      var menu = document.createElement("menu");
      menu.setAttribute("label", group_name);
      menu.setAttribute("value", group_name);
      menu.setAttribute("id",    "menu_group_" + group_name);
      var menupopup = document.createElement("menupopup");
      menu.appendChild(menupopup);
      this.add_role_popup.appendChild(menu);
      this.role_groups_menu[group_name] = menupopup;
    }
    var menuitem = document.createElement("menuitem");
    menuitem.setAttribute("value", "");
    menuitem.setAttribute("label", label);
    menuitem.setAttribute("disabled", "true");
    menuitem.setAttribute("class", "staticdataheader");
    var sep1 = document.createElement('menuseparator');
    var sep2 = document.createElement('menuseparator');
    this.role_groups_menu[group_name].appendChild(sep1);
    this.role_groups_menu[group_name].appendChild(menuitem);
    this.role_groups_menu[group_name].appendChild(sep2);
  }

  this.addSubmenu = function(group_name, label) {
    if(!this.role_groups_menu.hasOwnProperty(group_name)) {
      var menu = document.createElement("menu");
      menu.setAttribute("label", group_name);
      menu.setAttribute("value", group_name);
      menu.setAttribute("id",    "menu_group_" + group_name);
      var menupopup = document.createElement("menupopup");
      menu.appendChild(menupopup);
      this.add_role_popup.appendChild(menu);
      this.role_groups_menu[group_name] = menupopup;
    }
    var submenu = document.createElement("menu");
    submenu.setAttribute("label", label);
    var submenupopup = document.createElement("menupopup");
    //create_items(submenupopup, module_key, 'BuiltinNagios');
    submenu.appendChild(submenupopup);
    this.role_groups_menu[group_name].appendChild(submenu);
    this.role_groups_menu[label] = submenupopup;
  }


  this.enableAddMenuItem = function(name) {
    var sufficies = [ "" , "2" ];
    for(var c = 0; c < 2; c++) {
      var menu_item = document.getElementById('menu_role'+sufficies[c] + '_' + name);
      if(menu_item) {
        menu_item.removeAttribute("disabled");
        //var menu = menu_item.parentNode.parentNode;
        //menu.removeAttribute("hidden");
      }
    }
  }

  this.disableAddMenuItem = function(val) {
    var sufficies = [ "" , "2" ];
    for(var c = 0; c < 2; c++) {
      var menu_item = document.getElementById('menu_role' + sufficies[c] + '_' + val);
      if(menu_item) {
        //var menupopup = menu_item.parentNode;
        menu_item.setAttribute("disabled", "true");
        //var only_hidden = true;
        //for(var i = 0; i < menupopup.children.length; i++) {
        //  if(!menupopup.children[i].getAttribute("hidden")) {
        //    only_hidden = false;
        //    break;
        //  }
        //}
        //if(only_hidden) {
        //  var menu = menupopup.parentNode;
        //  if(menu) {
        //    //menu.setAttribute("hidden", "true");
        //  }
        //}
      }
    }
  }
  
  this.getSelectedRoleKey = function() {
    var index = document.getElementById("tabbox").selectedIndex;
    if(index < this.tabs.children.length && index < this.tabpanels.children.length) {
      var key = this.tabpanels.children[index].id;
      var res = key.match(/^tabpanel_groups_(.+)$/);
      if(res) {
        var tabbox = document.getElementById("tabbox_groups_" + res[1]);
        var tabs = document.getElementById("tabs_groups_" + res[1]);
        var tabpanels = document.getElementById("tabpanels_groups_" + res[1]);
        if(tabbox && tabs) {
          var index = tabbox.selectedIndex;
          if(index < tabs.children.length && index < tabpanels.children.length) {
            key = tabpanels.children[index].id;
          }
        }
      }
      var res = key.match(/^iframe_(\d+)$/);
      if(res) {
        return parseInt(res[1]);
      }
    }
    return 0;
  }
}


function setAttributes(el, attrs) {
  for(var key in attrs)   el.setAttribute(key, attrs[key]);
}

function resetTabFocus() {
  var tabboxes = document.getElementsByTagName("tabbox");
  for(var i = 0; i < tabboxes.length; i++) {
    tabboxes[i].selectedIndex = 0;
  }
}

function changeTabFocus(ind) {
  var tabbox = document.getElementById("tabbox");
  tabbox.selectedIndex = ind;
  var tabboxes = tabbox.getElementsByTagName("tabbox");
  if(ind < tabboxes.length) {
    tabboxes[ind].selectedIndex = 0;
  }
}

function changeTabFocusByRole(role_name) {
  var tab = document.getElementById('tab_' + role_name);
  if(tab) {
    var parent_group = tab.getAttribute('parent_group');
    var tabbox = document.getElementById("tabbox");
    if(parent_group) {
      var tab_group = document.getElementById('tab_groups_' + parent_group);  
      if(tab_group) {
        var j = 0;
        while( (tab_group = tab_group.previousSibling) != null ) j++;
        tabbox.selectedIndex = j;
        tabbox = document.getElementById('tabbox_groups_' + parent_group);
      }
    }
    var i = 0;
    while( (tab = tab.previousSibling) != null ) i++;
    tabbox.selectedIndex = i;
  }
}

function getTabFocus() {
  return document.getElementById("tabbox").selectedIndex;
}

function onLoad() {
  dock = getDock();
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  myDialogIndex = index;
  if(myDialog.arguments.is_puppet_role) {
    is_puppet_role = true;
    var customizations = document.getElementById('managecustomizations');
    if(customizations) {
      customizations.setAttribute('hidden', true);
    }
    // rename buttons 
    var addrolebutton = document.getElementById('addrolebutton');   
    addrolebutton.setAttribute('label', 'Add Resource / Class');
    addrolebutton.setAttribute('width', '150');
    var removerolebutton = document.getElementById('removerolebutton');   
    removerolebutton.setAttribute('label', 'Remove');
    puppet_role = null;
    for(var i = 0; i < myDialog.arguments.object.object.roles.length; i++) {
      if(myDialog.arguments.object.object.roles[i].childType == "PuppetRole") {
        puppet_role = new dock.cvObject(myDialog.arguments.object.cluster, myDialog.arguments.object.object.roles[i], myDialog.arguments.object, "roles");
        break;
      }
    }
  }

  if(myDialog.arguments.enable_elements) {
    for(var i = 0; i < myDialog.arguments.enable_elements.length; i++) {
      var element = document.getElementById(myDialog.arguments.enable_elements[i]);
      if(element) {
        element.removeAttribute('hidden');
      }
    }
  }


  var select_tab = null;
  if(myDialog.arguments.hasOwnProperty('select_tab')) {
    select_tab = myDialog.arguments.select_tab;
  }
  // makes 'name' field changeable for non-committed yet overlays
  if ( myDialog.arguments.object.get('uniqueKey') < dock.MAX_LOCAL_KEY || is_puppet_role) {
    document.getElementById('name').removeAttribute("readonly");
  }
  println('.Load: ' + document.title + ' - ' + document.documentURI);
  overlay_controller = myDialog.arguments.controller;
  overlay_view       = new OverlayEditView();
  overlay_controller.addSelectableRolesToAddButton(myDialog.arguments.object, overlay_view);
  loaded = true;
  var tooltip = document.getElementById('priority-tooltip');
  if(tooltip) {
      tooltip.tooltipText=
      "Values allowed: [0..249],[251..749],[751..1000] or -1\n\n" + 
      "Multiple configuration groups can be set for a node.\n" + 
      "A configuration overlay with a higher priority is applied to its associated node instead of a configuration overlay with a lower priority.\n\n" + 
      "The configuration group with the highest priority then determines the actual assigned role.\n" + 
      "Configuration groups an take on priorities in the range 0-1000, except for 250 and 750, which are forbidden.\n" + 
      "Setting a priority of -1 means that the configuration group is ignored.";
  }

  refresh(myDialog.arguments.object, select_tab);
}


function refresh(use, __selected_role_key=null) {
  if(!loaded) return;
  if(!!use) {
     current = use;
     old = current.old(false, true);
  }
  if(!!__selected_role_key) {
    selected_role_key = __selected_role_key;
  }
  println("Refresh Configuration Overlay " + ' - ' + document.documentURI);
  cluster = current.cluster;
  nodes = current.get('nodes');
  // text('priority', '^[0-9]+$');
  text('priority', dock.REGEX_COPriority);
  text('name', '');

  var elem = document.getElementById('nodesconfigured'); 
  elem.setAttribute("value", dotFormatKeys(cluster, nodes) );

  var puppet = null;
  if(is_puppet_role) {
    var puppets = cluster.find('Puppet');
    if(puppets && puppets.length > 0) {
      puppet = puppets[0];
    }
  }
  overlay_controller.initTabs(current, overlay_view, overlay_view.current_view_suffix);
  resetTabFocus();
  if(selected_role_key) changeTabFocusByRole(selected_role_key);
  else changeTabFocus(0);
}

function puppetPreview() {
  var puppet_preview_xul = 'puppet/puppetpreview.xul';
  var args = {
    cluster : cluster,
    preselected_nodes : nodes
  }
  var d = new dock.cvDialog(puppet_preview_xul, args);
  d.onOk = function() {
  }
  dock.dialogs.showDialog(d);
}

////////////////////////////////////////////////////////////////////////////////
//
// add / remove roles
//
////////////////////////////////////////////////////////////////////////////////
function attachrole(val) {
  var role = overlay_controller.attachRole(current, val);
  if(typeof(role) != "object") {
    changeTabFocusByRole(role);
  }
  else {
    overlay_view.reset();
    overlay_controller.initTabs(current, overlay_view);
    resetTabFocus();
    changeTabFocusByRole(role.object.uniqueKey);
    if(role.object.baseType != "PuppetResourceDeclaration") {
      //overlay_view.disableAddMenuItem(val);
    }
  }
}

function detachrole() {
  var tabs = document.getElementById('tabs');   
  var tabpanels = document.getElementById('tabpanels');   
  var key = overlay_view.getSelectedRoleKey();
  var type = overlay_controller.detachRole(current, key);
  overlay_view.reset();
  overlay_controller.initTabs(current, overlay_view);
  changeTabFocus(0);
  overlay_view.enableAddMenuItem(type);
}

////////////////////////////////////////////////////////////////
function manageNodes() {
    var xulfilename = dock.myVersion + 'configoverlaynodeselection.xul';
    var args = new Object();
    args['configoverlay'] = current;
    var d = new dock.cvDialog(xulfilename,args);

    d.onOk = function() {
      overlay_view.reset();
      refresh();
    }
    dock.dialogs.showDialog(d);
}

function buttonCancel() {
  // if 'OK' button (that keeps changes done on config-group) 
  // pressed then window should simply be closed; otherwise original 
  // object (config-group at the time of window pop-up) restored
  if (!okPressed) {
    if ( location.hash == "#clone" || location.hash == "#new"   ) { 
      cluster.removeObject(current.get('uniqueKey'));
    }
    current.old(old);
    close();
  }
  else {
    close();
  }
}

function buttonOk() {
  if(puppet_role) {
    // Todo: check why this is needed when granularity is set
    if(puppet_role.get('modified')) {
      current.setModified(true);
    }
  }
  var roles = current.get('roles');
  var has_roles = false;
  for(var i = 0; i < roles.length; i++) {
    if(!roles[i].toBeRemoved) {
      has_roles = true;
      break;
    }
  }
  if(!has_roles) {
    dock.displayError('No roles assigned: Add at least one role.');
    return;
  }
  if(current.resolveName() == '') {
      dock.displayError('The name of the overlay is required.');
      return;
  }
  if ( checkConfOverlayName() ) {
    okPressed = true;
    myDialog.onOk();
    close();
    // warnings section
    if ( current.get('nodes').length == 0 ) {
      dock.displayInfo("Warning: No nodes have been defined for configuration group.");
    }
  }
  else {
    dock.displayError('This name is used: Change configuration group name.\nNote: see all configuration groups (overlays) in the Cluster Tab or\n run \'configurationoverlay list\' in cmsh.');
    return;
  }
}

function groupRoles() {
  var group = document.getElementById("openstack_grouping_classes");
  if(group) {
    overlay_view.current_view_suffix = group.value;
    var vis_button = document.getElementById("addrolebutton" + group.value);
    var invis_button = (group.value == "")  ? document.getElementById("addrolebutton2") : document.getElementById("addrolebutton");
    if(vis_button) vis_button.removeAttribute("hidden");
    if(invis_button) invis_button.setAttribute("hidden", true);
    overlay_view.reset();
    refresh();
  }
}


// Todo: make common function with hadoop overlays
function checkConfOverlayName() {
  var overlays = cluster.find('ConfigurationOverlay');
  var uniqueKey = current.get('uniqueKey');
  var coNames = [];   
  var current_name = current.resolveName();
  for(var k = 0; k < overlays.length; k++) {
    if ( overlays[k].get('uniqueKey') != uniqueKey  && 
	 overlays[k].resolveName() == current_name) {
      return false;
    }
  }
  return true;
}

/**
 * Call this to redraw all the tabs inside the (OpenStack) Overlay.
 *
 * This is used for example by the "Manage Customizations" button, where all tabs are hidden
 *  during editing of customizations, and later restored to keep the UI simple.
 */
function refreshTabs()
{
    overlay_view.reset();
    overlay_controller.initTabs(current, overlay_view);
    resetTabFocus();
}
