/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var listing = (function () {

function showListingButtons() {
    var boxes = document.querySelectorAll('hbox#listing_buttons button');
    for (var i=0; i<boxes.length; i++) {
        boxes[i].setAttribute('disabled', false);
        boxes[i].style.display = '';
    }
}

function hideListingButtons() {
    var boxes = document.querySelectorAll('hbox#listing_buttons button');
    for (var i=0; i<boxes.length; i++) {
        boxes[i].setAttribute('disabled', true);
        boxes[i].style.display = 'none';
    }
}

function managerForFilePath(filename, managedFilePaths) {
    for (var i = 0; i < managedFilePaths.length; i++) {
        var re = new RegExp(managedFilePaths[i]['regex'].replace(/\//g, '\\/'));
        if (re.exec(filename)) {
            return managedFilePaths[i]['manager'];
        }
    }
    return 'system';
}

/**
 * This is basically a NodeRoleCustomizationListingController from 7.1 with small modifications.
 * @param chromeWindow
 * @constructor
 */
function CustomizationListingController(chromeWindow) {
    this.chromeWindow = chromeWindow;

    this.getField = function() {
        return 'customizationFiles';
    };

    this.getNewDataObject = function() {
        return new this.dock.cvCustomizationFile();
    };

    this.getEditorRule = function(key) {
        return new EditorRuleCustomizationEntry(key);
    };

    this.getListing = function(object) {
        return object.get('customizationFiles');
    };

    this.createTreeItem = function(one) {
        var item = new Object();
        for(var attribute in one) {
            if(attribute === 'uniqueKey') {
                item['key'] = one['uniqueKey'];
            } else if(attribute === 'entries') {
                item['l_numentries'] = one[attribute].length;
            } else {
                item['l_' + attribute] = one[attribute];
            }
        }
        item['l_manager'] = managerForFilePath(one['name'], this.object.cluster.managedFilePaths);
        return item;
    };

    this.removeObject = function() {
        var instance = this;
        if(this.select.length === 0) {
            dock.displayError('Please select an entry.');
            return;
        }
        var args = new Object();
        if(this.select.length === 1) {
            args['question'] = 'Are you sure you want to remove this entry?';
        } else {
            args['question'] = 'Are you sure you want to remove these entries?';
        }
        var d = new dock.cvDialog('chrome://cmgui/content/question.xul', args);
        d.onOk = function() {
            showListingButtons();
            var files = instance.object.get('customizationFiles');
            for(var i=instance.select.length - 1; i >= 0; i--) {
                for(var j=0; j<files.length; j++) {
                    if(files[j].uniqueKey === parseInt(instance.select[i])) {
                        files[j].toBeRemoved = true;
                        instance.object.object.modified = true;
                    }
                }
            }
            instance.clearSelected();
            instance.refresh();
        };
        dock.dialogs.showDialog(d);
    };

    this.revertFinalize = function() {
        this.refresh();
    };
    this.onOk = function () {
        window.parent.closeManageCustomizations();
    }
    this.onCancel = function () {
        window.parent.closeManageCustomizations();
    }
};

/*
 * *******************************************************************************************************************************************************
 */

function EditorRuleCustomizationEntry(key) {
    this.addTitle = "Add Customization";
    this.editTitle = "Edit Customization";
    this.api = 'Domain';
    this.key = key;

    function splitSectionAndKey(key) {
        // "[SECTION] key"
        var split = key.match(/\[([^\]]+)\] {0,1}(.*)/);
        if (split && split.length === 3) {
            split = split.splice(1);
        }
        // "SECTION/key"
        else {
            split = key.split('/');
        }
        // "key"
        if (split.length !== 2) {
            split = ['', key];
        }
        return split;
    }

    this.onInitObject = function () {
        hideListingButtons();
    };

    this.getCurrentObject = function(editorController) {
        var cluster = editorController.object.cluster;
        for(var i = 0; i < editorController.object.object.customizationFiles.length; i++) {
            if(editorController.object.object.customizationFiles[i].uniqueKey==key) {
                var dataobject = editorController.object.object.customizationFiles[i];
            }
        }
        return new editorController.dock.cvObject(cluster, dataobject, editorController.object, 'configFileCustomizations');
    };

    this.dataBindingAndCheck = function(editorController) {
        this.resetEvents('formContainer');
        function initFileTypeEnum(types) {
            kenum({'what': 'type', 'list' : types, 'onCommandFunction' : onChangeRefreshEditorRule});
        }
        var managedFilePaths = editorController.object.cluster.managedFilePaths,
            manager = document.querySelector('textbox#manager'),
            onChangeRefreshManagedBy = function () {
                manager.value = managerForFilePath(document.querySelector('textbox#name').value, managedFilePaths);
                if (manager.value === 'system') {
                    // System does not support XML file types
                    initFileTypeEnum(['ENV', 'INI', 'Generic']);
                } else {
                    initFileTypeEnum(['XML', 'ENV', 'INI', 'Generic']);
                }
            },
            onChangeRefreshEditorRule = function () {
                editorController.refresh();
                return true;
            }
        ;

        text({'what' : 'name', 'format' : '^', 'onCommandFunction' : onChangeRefreshManagedBy, 'keyUpIsChange' : true});
        check('enabled', '^');
        manager.value = managerForFilePath(document.querySelector('textbox#name').value, managedFilePaths);
        manager.setAttribute('disabled', 'true');
        initFileTypeEnum(['XML', 'ENV', 'INI', 'Generic']);

        var isIni = document.querySelector('#type').value === 'INI',
            isGeneric = document.querySelector('#type').value === 'Generic',
            arrayObjectAttributeListable = new ArrayObjectAttributeListable(editorController.currentObject, 'entries'),
            parent_inputToOutput = arrayObjectAttributeListable.inputToOutput,
            parent_outputToInput = arrayObjectAttributeListable.outputToInput
        ;

        // @override
        arrayObjectAttributeListable.inputToOutput = function() {
            // Fool our parent function, or it won't initialize the array with these fields..
            for (var i=0; i<this.inputValue.length; i++) {
                this.inputValue[i].section_part = this.inputValue[i].section_part || '';
                this.inputValue[i].key_part = this.inputValue[i].key_part || '';
            }
            parent_inputToOutput.bind(arrayObjectAttributeListable)();
            for (var i=0; i<this.outputValue.length; i++) {
                var convert = [];
                var key = this.outputValue[i].shift();
                this.outputValue[i].shift();               // we recalculate this
                this.outputValue[i].shift();               // we recalculate this
                convert.push(key);                         // key (containing both section and key)
                var split = splitSectionAndKey(key);
                convert.push(split[0]);                    // section_part extracted from key
                convert.push(split[1]);                    // key_part extracted from key
                convert.push(this.outputValue[i].shift()); // value
                convert.push(this.outputValue[i].shift()); // action
                convert.push(this.outputValue[i].shift()); // enabled
                this.outputValue[i] = convert;
            }
        }
        // @override
        arrayObjectAttributeListable.outputToInput = function() {
            for (var i=0; i<this.outputValue.length; i++) {
                var convert = [];
                this.outputValue[i].shift();                            // we recalculate this
                var section_part = this.outputValue[i].shift();
                var key_part = this.outputValue[i].shift();
                if (isIni) {
                    convert.push('[' + section_part + '] ' + key_part); // recalculated key (section + key parts)
                } else {
                    convert.push(key_part);                             // use key (section part does not apply)
                }
                convert.push(section_part);                             // unused
                convert.push(key_part);                                 // unused
                convert.push(this.outputValue[i].shift());              // value
                convert.push(this.outputValue[i].shift());              // action
                convert.push(this.outputValue[i].shift());              // enabled
                this.outputValue[i] = convert;
            }
            return parent_outputToInput.bind(arrayObjectAttributeListable)();
        }
        arrayObjectAttributeListable.setObject(editorController.dock.cvCustomizationEntry);
        arrayObjectAttributeListable.setColumnOrder({'key': 0, 'section_part' : 1, 'key_part' : 2, 'value': 3, 'action': 4, 'enabled': 5});
        arrayObjectAttributeListable.setColumnHeaderLabels(['Key', 'Section', 'Key', 'Value', 'Action', 'Enabled']);
        arrayObjectAttributeListable.initInputValue();

        var customListDataViewMapper = new CustomListDataViewMapper(arrayObjectAttributeListable);
        customListDataViewMapper.setElementsConfig([
            [
                {'type': 'hidden', 'attributes': [], 'events': []},
                {'type': isIni ? 'textbox' : 'hidden', 'attributes': [], 'events': []},
                {'type': isGeneric ? 'hidden' : 'textbox', 'attributes': [], 'events': []},
                {'type': 'textbox', 'attributes': [], 'events': []},
                {'type': isIni ? 'menulist' : 'hidden', 'attributes': [], 'events': [], 'menuitems': [
                    { 'value': 'DEFAULT', 'label': '' },
                    { 'value': 'SMART_ADD', 'label': 'Smart Add' },
                    { 'value': 'APPEND', 'label': 'Append' },
                    { 'value': 'PREPEND', 'label': 'Prepend' },
                    { 'value': 'REMOVE', 'label': 'Remove' },
                    { 'value': 'PRESERVE', 'label': 'Preserve' }
                ]},
                {'type': 'checkimage', 'attributes': [], 'events': []}
            ]
        ]);
        customListDataViewMapper.setNewItem(['key', 'section', 'key', 'value', 'SMART_ADD', true]);
        var addDelListWidget = new AddDelListWidget({
            'rootNodeId': 'entries',
            'dataViewMapper': customListDataViewMapper
        });
        addDelListWidget.populate();
    };

    this.inputCheck = function(editorController) {
        if(editorController.currentObject.get('toBeRemoved')===true) {
            return true;
        }
        if(blank(editorController.currentObject.get('name'))) {
            editorController.dock.displayError("Name cannot be empty");
            return false;
        }
        // TODO: do duplicate key checking here?
        //for(var i=0; i<editorController.currentObject.get('entries').length; i++) {}
        return true;
    };

    this.onEditOk = function () {
        showListingButtons();
    };

    this.onEditCancel = function () {
        showListingButtons();
    };
}


CustomizationListingController.prototype = new ListingController();
CustomizationListingController.prototype.constructor = CustomizationListingController;

EditorRuleCustomizationEntry.prototype = new EditorRuleBase();
EditorRuleCustomizationEntry.prototype.constructor = EditorRuleCustomizationEntry;

return new CustomizationListingController(this);

})();

