/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var KEY_ENTER = 13;
var KEY_SPACE = 32;
var KEY_ESCAPE = 27;

var daysInMonth = new Array(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
var displayDate;
var selectDate;

function ok() {
  if(parent) parent.ok();
}

function refresh(date) {
  displayDate = date;
  selectDate = date;
  showMonth();
}

function select(event, day, month, year) {
  var text = document.getElementById('time');
  var value = text.value.split(':');
  var hour = parseInt(value[0]);
  var minute = parseInt(value[1]);
  refresh(new Date(year, month, day, hour, minute, 0));
  if(event.detail > 1) ok();
}

function getDays(month, year) {
  var days = daysInMonth[month];
  if((1 == month) && (0 == year % 4) && (0 != (year % 100)) || (0 == year % 400)) days++;
  return days;
}

function displayDays(date) {
  var days = new Array();
  var s = date.getDay();
  var year;
  var month;
  if(s > 0) {
    if(date.getMonth() == 0) {
      year = 1900 + date.getYear() - 1;
      month = 11;
    } else {
      year = 1900 + date.getYear();
      month = date.getMonth() - 1;
    }
    var n = getDays(month, year);
    for(var i = n - s + 1; i <= n; i++) {
      var x = new Date(year, month, i);
      days.push(x);
    }
  }
  year = 1900 + date.getYear();
  month = date.getMonth();
  var n = getDays(month, year);
  for(var i = 1; i <= n; i++) {
    var x = new Date(year, month, i);
    days.push(x);
  }
  var m = 7 - ((s + n) % 7);
  if(m < 7) {
    if(date.getMonth() == 11) {
      year = 1900 + date.getYear() + 1;
      month = 1;
    } else {
      year = 1900 + date.getYear();
      month = date.getMonth() + 1;
    }
    for(var i = 1; i <= m; i++) {
      var x = new Date(year, month, i);
      days.push(x);
    }
  }
  return days;
}

function showMonth(next) {
  var year = displayDate.getYear() + 1900;
  var month = displayDate.getMonth();
  if((typeof(next) != 'undefined')) {
    month += next;
    if(month < 0) {
      month += 12;
      year--;
    } else if(month > 11) {
      month -= 12;
      year++;
    }
  }
  displayDate = new Date(year, month, 1);
  var today = new Date();
  today = today.format('dd/mm/yyyy');
  var sel = selectDate.format('dd/mm/yyyy');

  var rows = document.getElementById('rows');
  while(rows.childNodes.length > 1) rows.removeChild(rows.lastChild);

  var days = displayDays(displayDate);
  var row;
  for(var i = 0; i < days.length; i++) {
    if((i % 7) == 0) {
      row = document.createElement('row');
      row.setAttribute('class', 'daterow');
    }
    var label = document.createElement('label');
    label.setAttribute('value', days[i].getDate());
    if(days[i].getMonth() == displayDate.getMonth()) {
      label.setAttribute('class', 'normalday');
    } else {
      label.setAttribute('class', 'beforeday');
    }
    label.setAttribute('onclick', 'select(event, ' + days[i].getDate() + ', ' + days[i].getMonth() + ', ' + (1900 + days[i].getYear()) + ');');
    label.setAttribute('today', days[i].format('dd/mm/yyyy') == today);
    label.setAttribute('selected', days[i].format('dd/mm/yyyy') == sel);
    row.appendChild(label);
    if((i % 7) == 6) rows.appendChild(row);
  }
  document.getElementById('month').setAttribute('value', displayDate.format("mmm yyyy"));
  document.getElementById('time').setAttribute('value', selectDate.format("HH:MM"));
}


function timeChange() {
  var text = document.getElementById('time');
  var value = text.value;
  var format = '^[0-9]{1,2}\:[0-9]{2}$';
  if(regexMatch(value, format)) {
    value = value.split(':');
    var hour = parseInt(value[0]);
    var minute = parseInt(value[1]);
    refresh(new Date(selectDate.getYear() + 1900, selectDate.getMonth(), selectDate.getDate(), hour, minute, 0));
  } else {
    text.value = selectDate.format("HH:MM");
  }
  text.setAttribute("style", "");
}

function timeCommand() {
  var text = document.getElementById('time');
  var value = text.value;
  var format = '^[0-9]{1,2}:[0-9]{2}$';
  if(regexMatch(value, format)) {
    text.setAttribute("style", "");
  } else {
    text.setAttribute("style", "color: red;");
  }
}

function timeKeypress(event) {
  var text = document.getElementById('time');
  var value = text.value;
  var format = '^[0-9]{1,2}\:[0-9]{2}$';
  switch(event.keyCode) {
    case KEY_ENTER:
      if(regexMatch(value, format)) {
        value = value.split(':');
        var hour = parseInt(value[0]);
        var minute = parseInt(value[1]);
        refresh(new Date(selectDate.getYear() + 1900, selectDate.getMonth(), selectDate.getDate(), hour, minute, 0));
        ok();
      } else {
        text.value = selectDate.format("HH:MM");
      }
      break;
    case KEY_ESCAPE:
      text.setAttribute("style", "");
      text.value = selectDate.format("HH:MM");
   }
}
