/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var led = 0;
var leds = 0;
var ledTimer = 0;
var ledCounter = 0;
var categorynodetree;
var categorynodedata;

var firmwareupdatefile = "dell13gfirmwareupdate.xul";
var applydellsettingsfile = "applydell13gsettings.xul";

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) current = use;
  println("Category refresh Dell Settings");
  
  refreshNodeSummary();
  var configuredDellSettings = current.get('dellSettings');
  if(configuredDellSettings) {
    refreshAll();
    showSubTabs(true);
  } else {
    showSubTabs(false); 
  }
}

function showSubTabs(show) {    
  if (show) {
    document.getElementById('show_category_dell_settings').selectedIndex = 0;
    document.getElementById('category_dell_settings_buttons').hidden = false;
  } else {
    document.getElementById('show_category_dell_settings').selectedIndex = 1;
    document.getElementById('category_dell_settings_buttons').hidden = true;
  }
}

function stop() {
  println("Category stop dell settings");
  if(ledTimer) window.clearTimeout(ledTimer);
  ledCounter = 0;
}

function refreshNodeSummary() {
  println("++ Category refresh dell nodes...");
  categorynodetree = document.getElementById('categorynodetree');
  categorynodedata = new Array();
  var nodelist = current.cluster.findNodesIn(current.get('baseType'), current.get('uniqueKey'));
  for ( var i = 0; i < nodelist.length; i++) {
    var nodeobj = current.cluster.find(nodelist[i]);
    if (nodeobj && nodeobj.isDell13G()) {
      categorynodedata.push(createTreeItemNodeSummary(nodeobj, i));
    }
  }
  for(var i = nodelist.length; i < MINTREEITEMS; i++) categorynodedata.push(blankTreeItem());
  var select = selected('nodelistcategory');
  loadTable(categorynodetree, categorynodedata);
  setSelected('nodelistcategory', select);
}

function blinking() {
  var needTimer = false;
  if(led) {
    var obj = document.getElementById('locator-led');
    if(obj) {
      var color = led.color;
      obj.setAttribute("tooltiptext", color + ' ' + led.operState);
      if(goodColors.find(color) >= 0) {
        if(led.operState == 'blinking') {
          needTimer = true;
          if(ledCounter % 2 == 1) color = 'blank';
        }
        if(ledCounter == 0 || led.operState == 'blinking') obj.setAttribute("src", "./skin/image/LED_" + color + ".png");
      }
    }
  }
  if(leds) {
    for(var i = 0; i < leds.length; i++) {
      var name = leds[i].name;
      var obj = document.getElementById(name);
      if(obj) {
        var color = leds[i].color;
        obj.setAttribute("tooltiptext", color + ' ' + leds[i].operState);
        if(goodColors.find(color) >= 0) {
          if(leds[i].operState == 'blinking') {
            needTimer = true;
            if(ledCounter % 2 == 1) color = 'blank';
          } else if(ledCounter) {
            continue;
          }
          obj.setAttribute("src", "./skin/image/LED_" + color + ".png");
        }
      }
    }
  }
  ledCounter++;
  if(needTimer) ledTimer = window.setTimeout('blinking();', 1000);
}

function refreshAll() {
  println("++ Category dell settings refreshAll");
  for(var i = 0; i < window.frames.length; i++) {
    if(!!window.frames[i].refresh) {
      window.frames[i].refresh(current);
    }
  }
}

function createTreeItemNodeSummary(node, i) {
  var item = new Object();
  if(node.get('uniqueKey')) {
    item['key'] = node.get('uniqueKey');
  } else {
    item['key'] = -1000 -i;
  }
  item['nodename'] = node.resolveName();
  var node_sysinfo = node.sysInfo;
  if (node_sysinfo) {
    item['system_name'] = node_sysinfo['systemName'];
    item['system_manufacturer'] = node_sysinfo['systemManufacturer'];
    item['vendor_tag'] = node_sysinfo['vendorTag'];
    item['bios_version'] = node_sysinfo['biosVersion'];
    item['bios_date'] = node_sysinfo['biosDate'];
    item['bios_vendor'] = node_sysinfo['biosVendor'];
    item['motherboard_manufacturer'] = node_sysinfo['motherboardManufacturer'];
    item['motherboard_name'] = node_sysinfo['motherboardName'];
    item['network_interfaces'] = node_sysinfo['nics'];
    item['dell_storage'] = (node_sysinfo['dellStorage'] == 0) ? 'None':node_sysinfo['dellStorage'];

    var node_dellsettings = node.getDellSettings();
    if (node_dellsettings) {
      var bootorderstring = node_dellsettings.biosBootOrder;
      item['node_bootorder'] = bootorderstring;
    }
  }
  return item;
}

function selectNodeObject() {
  var selecteditem = selected('nodelistcategory');
  if(selecteditem.length == 0) {
    dock.displayError("Please select a device");
  } else if(selecteditem.length > 1) {
    dock.displayError("Please select only one device");
  } else {
    current.cluster.select(selecteditem[0]);
  }
}

function importFromNode() {
}

function firmwareUpdate() {
  var args = new Object();
  args['object'] = current;
  args['detached'] = isundefined(parent.detached, false);
  var d = new dock.cvDialog(dock.myVersion + firmwareupdatefile, args);
  d.onOk = function() {
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function applyDellSettings() {
  var args = new Object();
  args['title'] = "Apply Dell Settings";
  args['question'] = "Proceeding with this operation will apply the changes in Dell settings, if any and may cause a reboot of all the nodes in the category.\n\nDo you really want to continue?";
  args['check'] = "View task manager";
  var d = new dock.cvDialog("question.xul", args);
  d.onOk = function(checked) {
    current.applyDellSettings();
    if (checked) {
      viewTaskManager();
    }
  }
  dock.dialogs.showDialog(d);
}

function viewTaskManager() {
  var taskmanager = dock.dialogs.find('taskmanager.xul');
  if(taskmanager) {
    if(taskmanager.closed) taskmanager.focus();
    taskmanager.refresh();
  } else {
    var d = new dock.cvDialog('taskmanager.xul', []);
    dock.dialogs.showDialog(d);
  }
}
 
