/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var globalCurrent;
var current;
var currentDellSettings;
var currentLiveDellSettings;
var bootOrderCache = new Object();
var currentBootOrder = new Object();
var liveBootOrder = new Object();

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) {
    globalCurrent = use;
  }
  println("-- Refresh Dell BIOS settings");

  parent.document.getElementById('revert').setAttribute('disabled', globalCurrent.get('uniqueKey') < dock.MAX_LOCAL_KEY);
  parent.document.getElementById('save').setAttribute('disabled', !globalCurrent.get('modified'));

  currentDellSettings = globalCurrent.getDellSettings();
  currentLiveDellSettings = globalCurrent.getLiveDellSettings();
  if ( currentDellSettings ) {
    currentBootOrder = currentDellSettings.biosBootOrder.split(',');
    current = new dock.cvObject(globalCurrent.cluster, currentDellSettings, globalCurrent);
    kenum("sysprofile", ['PerfPerWattOptimizedDapc', 'PerfPerWatOptimizedOs', 'PerfOptimized', 'DenseCfgOptimized', 'Custom']);     
    kenum("failsafeBaudRate", ['115200', '19200', '57600', '9600']);     
    kenum("remoteTerminal", ['Ansi', 'Vt100Vt220']);     
    kenum("serialCommunication", ['Off' , 'OnConRedirAuto', 'OnConRedirCom', 'OnConRedirCom1', 'OnConRedirCom2', 'OnNoConRedir']);     
    kenum("serialPort", ['Com1', 'Com2', 'Serial1Com2Serial2Com1', 'Serial1Com1Serial2Com2']);     
    kenum("memFrequency", ['1067mhz', '1333mhz','1600mhz', '800mhz', 'MaxPerf', 'MaxReliability']);     
    kenum("memPatrolScrub", ['Disabled', 'Extended', 'Standard']);     
    kenum("memRefreshRate", ['1x', '2x']);     
    kenum("snoopMode", ['HomeSnoop', 'EarlySnoop', 'ClusterOnDie', 'OpportunisticSnoopBroadcast']);     
    kenum("energyPerformanceBias", ['BalancedEfficiency', 'BalancedPerformance', 'LowPower', 'MaxPower']);     
    check("redirectionAfterBoot", false);     
    check("collaborativeCpuPerfCtrl", false);     
    check("l3Cache", false);     
    check('logicalProcessor', false);
    check("monitorMWait", false);     
    check("mmioAbove4Gb", false);     
    check("nodeInterleaving", false);     
    check("energyEfficientTurbo", false);     
    check("ioNonPostedPrefetch", false);     
  }

  /* Populate bios boot order */
  loadBootOrder(); 
  println("-- Refresh live Dell BIOS settings");
  showLive(parent.document.getElementById('showlive'));
}

function loadBootOrder(live) {
  var dell13gbiosbootorderrows = document.getElementById('dell13gbiosbootorder');
  while(dell13gbiosbootorderrows.firstChild) dell13gbiosbootorderrows.removeChild(dell13gbiosbootorderrows.firstChild);   
  for ( var j = 0; j < currentBootOrder.length; j++ ) {
    var row = document.createElement("row");
    row.setAttribute('align', 'center');
    var shadowlabel = document.createElement('shadowlabel');
    shadowlabel.setAttribute('width', '20px');
    shadowlabel.setAttribute('value', parseInt(j + 1));
    var menulist = document.createElement('menulist');
    var menupopup = document.createElement('menupopup');
    menulist.setAttribute('selectedIndex', 0); 
    menulist.setAttribute('id', 'bootorderlist_' + j); 
    menupopup.setAttribute('id', 'bootorder_' + j); 
    for ( var k = 0; k < currentBootOrder.length; k++ ) {
      var trimmed = currentBootOrder[k];
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute('label', trimmed);
      menuitem.setAttribute('id', 'bootdevice_' + trimmed + '_' + j);
      menuitem.setAttribute('oncommand', "updateBootOrder(this);");
      menupopup.appendChild(menuitem);
    }
    menulist.appendChild(menupopup);
    row.appendChild(shadowlabel);
    row.appendChild(menulist);
    dell13gbiosbootorderrows.appendChild(row);
    menulist.selectedIndex = parseInt(j); 
  }
}

function loadLiveBootOrder(live, id) {
  var showlive = (live.getAttribute('label') == 'Show live' ? true:false);
  if (!showlive) {
    document.getElementById(id).removeAttribute('hidden');
  } else {
    document.getElementById(id).setAttribute('hidden', showlive);
  }
  if (showlive) {
    var livedell13gbiosbootorderrows = document.getElementById('livedell13gbiosbootorder');
    while(livedell13gbiosbootorderrows.firstChild) livedell13gbiosbootorderrows.removeChild(livedell13gbiosbootorderrows.firstChild);   
    for ( var j = 0; j < liveBootOrder.length; j++ ) {
      var row = document.createElement("row");
      row.setAttribute('align', 'center');
      var shadowlabel = document.createElement('shadowlabel');
      shadowlabel.setAttribute('width', '20px');
      shadowlabel.setAttribute('value', '[' + parseInt(j + 1) + '] ' + liveBootOrder[j]);
      shadowlabel.setAttribute('style', 'padding-top:4px;');
      row.appendChild(shadowlabel);
      livedell13gbiosbootorderrows.appendChild(row);
    }
  }
}

function updateBootOrder(elem) {
  var curselid = elem.id.split('_')[2];
  var newsel = elem.id.split('_')[1];
  var newselid = currentBootOrder.indexOf(newsel);
  var prevsel = currentBootOrder[curselid];
  if ( newsel != prevsel ) {
    currentBootOrder[curselid] = newsel;
    currentBootOrder[newselid] = prevsel;
    loadBootOrder();
    var newbootorder = "";
    for ( var i = 0; i < currentBootOrder.length; i++ ) {
      var trimmed = currentBootOrder[i];
      newbootorder += trimmed + ','; 
    }
    newbootorder = newbootorder.replace(/^,+|,+$/g,"");
    //println("New bootorder " + newbootorder);
    var save = parent.document.getElementById('save');
    if(save) save.removeAttribute('disabled');
    current.set('biosBootOrder', newbootorder);
  }
}

function enableLiveElem (id, value, attr, live) {
  var showlive = (live.getAttribute('label') == 'Show live' ? true:false);
  if (!showlive) {
    document.getElementById(id).setAttribute(attr, value);
    document.getElementById(id).removeAttribute('hidden');
  } else {
    document.getElementById(id).setAttribute('hidden', showlive);
  }
}

function showLive(showlive) {
  currentLiveDellSettings = globalCurrent.getLiveDellSettings();
  if (currentLiveDellSettings) {
    liveBootOrder = currentLiveDellSettings.biosBootOrder.split(',');
    enableLiveElem('livesysprofile', currentLiveDellSettings.sysprofile, 'value', showlive); 
    enableLiveElem('livefailsafeBaudRate', currentLiveDellSettings.failsafeBaudRate, 'value', showlive); 
    enableLiveElem('liveremoteTerminal', currentLiveDellSettings.remoteTerminal, 'value', showlive); 
    enableLiveElem('liveserialCommunication', currentLiveDellSettings.serialCommunication, 'value', showlive); 
    enableLiveElem('liveserialPort', currentLiveDellSettings.serialPort, 'value', showlive); 
    enableLiveElem('livememFrequency', currentLiveDellSettings.memFrequency, 'value', showlive); 
    enableLiveElem('livememPatrolScrub', currentLiveDellSettings.memPatrolScrub, 'value', showlive); 
    enableLiveElem('livememRefreshRate', currentLiveDellSettings.memRefreshRate, 'value', showlive); 
    enableLiveElem('livesnoopMode', currentLiveDellSettings.snoopMode, 'value', showlive); 
    enableLiveElem('liveenergyPerformanceBias', currentLiveDellSettings.energyPerformanceBias, 'value', showlive); 
    enableLiveElem('liveredirectionAfterBoot', currentLiveDellSettings.redirectionAfterBoot, 'checked', showlive); 
    enableLiveElem('livecollaborativeCpuPerfCtrl', currentLiveDellSettings.collaborativeCpuPerfCtrl, 'checked', showlive); 
    enableLiveElem('livel3Cache', currentLiveDellSettings.l3Cache, 'checked', showlive); 
    enableLiveElem('livelogicalProcessor', currentLiveDellSettings.logicalProcessor, 'checked', showlive); 
    enableLiveElem('livemonitorMWait', currentLiveDellSettings.monitorMWait, 'checked', showlive); 
    enableLiveElem('livemmioAbove4Gb', currentLiveDellSettings.mmioAbove4Gb, 'checked', showlive); 
    enableLiveElem('livenodeInterleaving', currentLiveDellSettings.nodeInterleaving, 'checked', showlive); 
    enableLiveElem('liveenergyEfficientTurbo', currentLiveDellSettings.energyEfficientTurbo, 'checked', showlive);
    loadLiveBootOrder(showlive, 'livedell13gbiosbootorder'); 
  }
}

