/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function cvDialogs() {
  this.count = 0;
  this.list = 0;
}

cvDialogs.prototype.init = function() {
  this.list = new Object();
}

cvDialogs.prototype.find = function(file) {
  if(!startswith(file, "chrome://")) file = "chrome://cmgui/content/" + file;
  for(var w in this.list) {
    if(this.list[w].file == file) {
      if(!this.list[w].window.closed) {
        return this.list[w].window;
      }
    }
  }
  return 0;
}

cvDialogs.prototype.show = function(dialog) {
  this.count++;
  this.list[this.count] = dialog;
  dialog.show(this, this.count);
}

cvDialogs.prototype.showDialog = function(dialog) {
  this.count++;
  this.list[this.count] = dialog;
  dialog.showDialog(this, this.count);
}

cvDialogs.prototype.showFixed = function(dialog) {
  this.count++;
  this.list[this.count] = dialog;
  dialog.showFixed(this, this.count);
}

cvDialogs.prototype.showShell = function(dialog) {
  this.count++;
  this.list[this.count] = dialog;
  dialog.showShell(this, this.count);
}

cvDialogs.prototype.close = function() {
  for(var w in this.list) {
    if(!this.list[w].window.closed && (typeof(this.list[w].close) == 'function')) {
      this.list[w].close();
    }
  }
  this.list = new Object();
}

function cvDialog(file, args) {
  this.file = file;
  if(!startswith(this.file, "chrome://")) this.file = "chrome://cmgui/content/" + this.file;
  this.arguments = args;
  this.output = 0;
  this.index = 0;
  this.parent = 0;
  this.windowWindow = 0;
  this.window = 0;
  this.onOk = 0;
  this.onNo = 0;
  this.onCancel = 0;
  this.onClose = 0;
  this.onOpened = 0;
  this.showOptions = 'chrome,extrachrome,resizable,centerscreen,scrollbars,status,toolbar';
  // KDR: non modal!! Worker threads are suspended
  /* http://www.w3.org/TR/workers/#active-needed-worker
      A worker is said to be an active needed worker if any of the Document objects in the worker's Documents are fully active.
      A worker is said to be a suspendable worker if it is not an active needed worker but it is a permissible worker.
  */
//   this.dialogOptions = 'chrome,extrachrome,resizable,modal,centerscreen';
  this.dialogOptions = 'chrome,extrachrome,resizable,centerscreen';
  this.fixedOptions = 'resizable=no,width=600,height=400';
  this.shellOptions = 'chrome,extrachrome,resizable,centerscreen,scrollbars,status,toolbar,width=700,height=400';
  this.timeout = 0;
}

cvDialog.prototype.show = function(parent, index) {
  this.parent = parent;
  this.index = index;
  if(this.parentWindow) {
    this.window = this.parentWindow.open(this.file, "dialog[" + index + "]", this.showOptions);
  } else {
    this.window = window.open(this.file, "dialog[" + index + "]", this.showOptions);
  }
}

cvDialog.prototype.showDialog = function(parent, index) {
  this.parent = parent;
  this.index = index;
  if(this.parentWindow) {
    this.window = this.parentWindow.openDialog(this.file, "dialog[" + index + "]", this.dialogOptions);
  } else {
    this.window = window.openDialog(this.file, "dialog[" + index + "]", this.dialogOptions);
  }
}

cvDialog.prototype.showFixed = function(parent, index) {
  this.parent = parent;
  this.index = index;
  if(this.parentWindow) {
    this.window = this.parentWindow.openDialog(this.file, "dialog[" + index + "]", this.fixedOptions);
  } else {
    this.window = window.openDialog(this.file, "dialog[" + index + "]", this.fixedOptions);
  }
}

cvDialog.prototype.showShell = function(parent, index) {
  this.parent = parent;
  this.index = index;
  if(this.parentWindow) {
    this.window = this.parentWindow.open(this.file, "dialog[" + index + "]", this.shellOptions);
  } else {
    this.window = window.open(this.file, "dialog[" + index + "]", this.shellOptions);
  }
}

cvDialog.prototype.close = function() {
  if(this.window && (typeof(this.window.close) == 'function')) {
    this.window.close();
  }
  delete this.parent.list[this.index];
}

cvDialog.prototype.doOk = function() {
  if(this.onOk) this.onOk();
  if(this.onClose) this.onClose();
}

cvDialog.prototype.doNo = function() {
  if(this.onNo) this.onNo();
  if(this.onClose) this.onClose();
}

cvDialog.prototype.doCancel = function() {
  if(this.onCancel) this.onCancel();
  if(this.onClose) this.onClose();
}
