/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var GRIDROWS = 2;
/* Drag-Drop functionality for the profiles
 * allows new profiles to be dragged onto trees
 */
var listObserver =   {
  onDragStart: function (evt,transferData,action) {
    var txt = evt.target.getAttribute("label");
    transferData.data=new TransferData();
    transferData.data.addDataForFlavour("text/unicode",txt);
  }
};
var partitionObserver = {
  getSupportedFlavours : function () {
    var flavours = new FlavourSet();
    flavours.appendFlavour("text/unicode");
    return flavours;
  },
  onDragOver: function(evt,flavour,partition){},
  onDrop: function(evt,dropdata,partition) {
    if (dropdata.data!="") {
      println(dropdata.data);
    }
  }
};


function onLoad() {
  var par1 = {name: 'p1', size: 400, fs: 'xfs'}
  var par2 = {name: 'p2', size: 300, fs: 'xfs'}
  var disk1 = {name: '/dev/sda', size: 800, partitions: [par1, par2]}

  var par1 = {name: 'p1', size: 200, fs: 'xfs'}
  var par2 = {name: 'p2', size: 800, fs: 'xfs'}
  var disk2 = {name: '/dev/sdb', size: 1000, partitions: [par1, par2]}
  var disks = [disk1, disk2];
  createDisks(disks);

  var raid0 = {name: 'raid0', level: 0, partitions: ['P1 (20GB)', 'P2 (20GB)', 'P3 (20GB)', 'P4 (20GB)', 'P5 (20GB)']}
  var raid1 = {name: 'raid1', level: 1, partitions: ['P1 (20GB)', 'P2 (20GB)', 'P3 (20GB)', 'P4 (20GB)', 'P5 (20GB)']}
  var raid5 = {name: 'raid5', level: 5, partitions: ['P1 (20GB)', 'P2 (20GB)', 'P3 (20GB)', 'P4 (20GB)', 'P5 (20GB)']}
  var raid6 = {name: 'raid6', level: 6, partitions: ['P1 (20GB)', 'P2 (20GB)', 'P3 (20GB)', 'P4 (20GB)', 'P5 (20GB)']}
  var raids = [raid0, raid1, raid5, raid6];
  createRaids(raids);
}

function createDisks(disks) {
  var group = document.getElementById("disks");
  for(var i = 0; i < disks.length; i++) {
    group.appendChild(createDisk(disks[i]));
  }
}

function createDisk(disk) {
  var main = document.createElement("groupbox");
  main.setAttribute("class", "disks");
  main.setAttribute("orient", "horizontal");
  main.setAttribute("flex", "1");
  main.setAttribute("context", "diskPopup");
  main.setAttribute("id", "disk:" + disk.name);
  var caption = document.createElement("caption");
  caption.setAttribute("label", disk.name + " (" + disk.size + ")");
  main.appendChild(caption);
  var size = 0;
  for(var i = 0; i < disk.partitions.length; i++) {
    var name = "partition:" + disk.partitions[i].name;
    var part = document.createElement("groupbox");
    part.setAttribute("class", "disks");
    part.setAttribute("context", "partitionPopup");
    part.setAttribute("flex", disk.partitions[i].size);
    part.setAttribute("id", name);
    part.appendChild(createOptions(disk.partitions[i], GRIDROWS));
    part.setAttribute("ondraggesture", "nsDragAndDrop.startDrag(event,listObserver);");
    size += disk.partitions[i].size;
    main.appendChild(part);
  }
  if(size < disk.size) {
    var spacer = document.createElement("spacer");
    spacer.setAttribute("flex", disk.size - size);
    main.appendChild(spacer);
  } else if(size > disk.size) {
  }
  return main;
}

function createRaids(raids) {
  var group = document.getElementById("raids");
  for(var i = 0; i < raids.length; i++) {
    group.appendChild(createRaid(raids[i]));
  }
}

function createRaid(raid) {
  var main = document.createElement("groupbox");
  main.setAttribute("class", "disks");
  main.setAttribute("ondragover","nsDragAndDrop.dragOver(event,boardObserver);");
  main.setAttribute("ondragdrop","nsDragAndDrop.drop(event,boardObserver);");
  main.setAttribute("orient", "vertical");
  main.setAttribute("flex", "1");
  main.setAttribute("context", "raidPopup");
  main.setAttribute("id", "raid:" + raid.name);
  var caption = document.createElement("caption");
  caption.setAttribute("label", raid.name + " (" + raid.level + ")");
  main.appendChild(caption);
  var h = raid.partitions.length;
/*  switch(raid.level) {
    case(0): h = 0; break;
    case(1): h = raid.partitions.length; break;
    case(5): h = raid.partitions.length - 1; break;
    case(6): h = raid.partitions.length - 2; break;
  }*/
  if(h > 0) {
    var hbox = document.createElement("hbox");
    main.appendChild(hbox);
    for(var i = 0; i < h; i++) {
      var name = "raidpartition:" + raid.partitions[i];
      var part = document.createElement("groupbox");
      part.setAttribute("class", "disks");
      part.setAttribute("context", "raidPartitionPopup");
      var label = document.createElement("label");
      label.setAttribute("value", raid.partitions[i]);
      part.appendChild(label);
      part.setAttribute("flex", "1");
      part.setAttribute("id", name);
      hbox.appendChild(part);
    }
  }
  for(var i = h; i < raid.partitions.length; i++) {
    var name = "raidpartition:" + raid.partitions[i];
    var part = document.createElement("groupbox");
    part.setAttribute("class", "disks");
    part.setAttribute("context", "raidPartitionPopup");
    var label = document.createElement("label");
    label.setAttribute("value", raid.partitions[i]);
    part.appendChild(label);
    part.setAttribute("flex", "1");
    part.setAttribute("id", name);
    main.appendChild(part);
  }
  main.appendChild(createOptions(raid, GRIDROWS));
  return main;
}

function createOptions(object, lines) {
  var grid = document.createElement("grid");
  var rows = document.createElement("rows");
  grid.appendChild(rows);
  for(var i = 0; i < lines; i++) {
    var row = document.createElement("row");
    rows.appendChild(row);
  }
  var cols = document.createElement("columns");
  grid.appendChild(cols);
  var col1;
  var col2;
  var j = 0;
  for(var i in object) {
    if(j == 0) {
      var col1 = document.createElement("column");
      cols.appendChild(col1);
      var col2 = document.createElement("column");
      cols.appendChild(col2);
    }
    var prop = document.createElement("label");
    prop.setAttribute("value", i + ":");
    col1.appendChild(prop);
    var val = document.createElement("label");
    val.setAttribute("value", object[i]);
    col2.appendChild(val);
    j = (j + 1) % lines;
  }
  return grid;
}

function editPartition(event) {
  printall(event);
}

function isundefined(x, def) {
  return (typeof x == 'undefined') ? def : x;
}

function header(txt, len, char) {
  txt = isundefined(txt, "");
  len = isundefined(len, HEADER);
  char = isundefined(char, '=');
  if(txt.length == 0) {
    for(var k = 0; k < len; k++) {
      print(char);
    }
  } else {
    var l = txt.length + 2;
    if(len < l) {
      l = len;
    }
    for(var k = 0; k < (len - 1 - l) / 2; k++) {
      print(char);
    }
    print(" " + txt + " ");
    for(var k = 0; k < (len - l) / 2; k++) {
      print(char);
    }
  }
  print("\n");
}

function isArray(obj) {
  if((typeof obj) == "undefined") return false;
  if(! !!obj) return false;
  return (obj.constructor.toString().indexOf("Array") != -1);
}

function printall(obj) {
  if(isArray(obj)) {
    for(var i = 0; i < obj.length; i++) {
      var t = i.toString();
      print(t + " ");
      for(var k = 0; k < (8 - t.length); k++) { print("."); }
      println(obj[i]);
    }
  } else if(typeof(obj) == "object") {
    var name = obj.toString();
    header(name);
    for(var i in obj) {
      try {
        var t = i.toString();
        print(t + " ");
        for(var k = 0; k < (20 - t.length); k++) { print("."); }
        print(" = ");
        t = typeof(obj[i]);
        if(t == "function") {
          println(t);
        } else {
          println(obj[i]);
        }
      } catch(e) {
        println("Error: " + i);
      }
    }
    header();
  } else {
    println(obj);
  }
}

function println(s) {
	dump(s + "\n");
}

function print(s) {
	dump(s);
}
