var dock;
var current;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function build_registry_layout(hostname, registry_settings) {
  var main_box = document.createElement("hbox");
  main_box.setAttribute('equalsize', 'always');

 
  var vbox1 = document.createElement("vbox");
  vbox1.setAttribute('pack', 'end');
  vbox1.setAttribute('class', 'staticdata');
  vbox1.setAttribute('align', 'left');
  vbox1.setAttribute('flex', '1');
  main_box.appendChild(vbox1);

  var title_label = document.createElement("label");
  title_label.setAttribute('style', 'font-weight: bold; font-size: 12px;');
  title_label.setAttribute("value", hostname + " Docker Registry Settings");
  vbox1.appendChild(title_label);

  var address_element = build_registry_property("Address", registry_settings.address);
  var prefix_element = build_registry_property("Prefix", registry_settings.prefix || "n/a");

  vbox1.appendChild(address_element);
  vbox1.appendChild(prefix_element);
  vbox1.appendChild(build_storage_driver_layout(registry_settings.storage_driver));

  return main_box;
}

function build_storage_driver_layout(storage_driver) {
  var vbox = document.createElement("vbox");
  vbox.setAttribute('pack', 'end');
  vbox.setAttribute('align', 'left');
  vbox.setAttribute('flex', '1');
  
  var title_label = document.createElement("label");
  title_label.setAttribute('style', 'font-weight: bold; font-size: 12px;');
  title_label.setAttribute("value", 'Storage Driver:');
  vbox.appendChild(title_label);

  if (storage_driver.childType == 'DockerRegistryFilesystemStorageDriver') {
    var sd_type_element = build_registry_property('Type', 'FileSystem');
    var sd_type_rootdirectory = build_registry_property('Root Directory:', storage_driver.rootdirectory);
    vbox.appendChild(sd_type_element);
    vbox.appendChild(sd_type_rootdirectory);
  }
  else if (storage_driver.childType == 'DockerRegistryInmemoryStorageDriver') {
    var sd_type_element = build_registry_property('Type', 'In Memory');
    vbox.appendChild(sd_type_element);
  }
  
  return vbox;
}

function build_registry_property(property_label, property_value) {
  var hbox = document.createElement("hbox");
  hbox.setAttribute("align", "center");

  var hbox2 = document.createElement("hbox");
  hbox2.setAttribute("width", "130px");
  hbox2.setAttribute("pack", "end");
  hbox.appendChild(hbox2);

  var shadowlabel = document.createElement("shadowlabel");
  shadowlabel.setAttribute("value", property_label + ":");
  shadowlabel.setAttribute("class", "staticdataheader");
  hbox2.appendChild(shadowlabel);

  var label = document.createElement("label");
  label.setAttribute("value", property_value);
  hbox.appendChild(label);
  
  return hbox;
}

function clear_overview(root) {
  while (root.firstChild) {
      root.removeChild(root.firstChild);
    }
}

function build_default_overview(root) {
  var hbox = document.createElement("hbox");
  hbox.setAttribute('align', 'center');

  var label = document.createElement("shadowlabel");
  label.setAttribute("value", "No docker registries deployed on this cluster.");
  hbox.appendChild(label);
  root.appendChild(hbox);
}

function build_overview() {
  var nodesWithDockerRegistryRoleAssigned = nodesWithRole('DockerRegistryRole');

  var root = document.getElementById("node_overviews");

  clear_overview(root);
  if (nodesWithDockerRegistryRoleAssigned.length == 0) {
    build_default_overview(root);
    return;
  }
  
  for (var node of nodesWithDockerRegistryRoleAssigned) {
    var hostname = node.get('hostname');

    var registryRole = getRoleForNode(node, 'DockerRegistryRole');

    var current_storage_driver = registryRole.storagedrivers[0];
    var registry_settings = {
      address: registryRole.address,
      prefix: registryRole.prefix,
      storage_driver: current_storage_driver
    }
    root.appendChild(build_registry_layout(hostname, registry_settings));
  }
}

function refresh(use) {
  println('Refeshing docker-registry overview ...');
  if (!!use) {
    current = use;
  }

  var cluster = current;
  if (!current.isCluster) {
    cluster = current.cluster;
  }

  build_overview();
}

