/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var wizard = new DockerRegistryWizard();

var registry = {
  address: ":5000",
  headers: ["X-Content-Type-Options=nosniff"],
  host: 'docker-registry',
  prefix: "",
  certificate: "/etc/docker-registry/cert/registry.crt",
  key: "/etc/docker-registry/cert/registry.key",
  storagedriver: {
    filesystem: {
      rootdirectory: "/var/lib/docker-registry"
    }
  }
}

var userChoices = {
  nodes: new Array(),
  registry: registry,

  is_valid: function() {
    return ((this.nodes.length > 0)
            && (this.registry.address.length > 0)
            && (this.registry.storagedriver.filesystem.rootdirectory.length > 0));
  },

  node_names: function() {
    return this.nodes.map(function(node) {
      return node.hostname;
    });
  }
}

function setRegistryValues() {
  var address = document.getElementById('registryAddress').value || "";
  var prefix = document.getElementById('registryPrefix').value || "";
  var root_directory = document.getElementById('registryRootDirectory').value || "";

  userChoices.registry.address = address;
  userChoices.registry.prefix = prefix;
  userChoices.registry.storagedriver.filesystem.rootdirectory = root_directory;

  validate();
}



function Node(node) {
  this.node = node;
};

Node.prototype.createTreeItem = function() {
  return {
    key: this.node.get('uniqueKey'),
    hostname: this.node.get('hostname')
  };
}


function onLoad() {
  dock = getDock();
  println("Docker Registry wizard initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  current = myDialog.arguments.cluster;

  window.addEventListener('close', function (event) {
      event.preventDefault();
      wizard.common.cancel();
    },
    true
  );
  wizard.common.initDefaultConfig();
  wizard.common.go(wizard.common.getNextStage.bind(undefined, null));
}

function nodesWithRole(role_name) {
  var nodes = current.find('SlaveNode');
  var nodesWithSpecifiedRole = [];
  
  nodesWithSpecifiedRole = nodes.filter(function(node) {
    var roles = node.get('roles');

    var role = undefined;
    for (var i = 0; i < roles.length; i++) {
      if (roles[i].childType === role_name) {
        role = roles[i];
        break;
      }
    }

    return role !== undefined;
  });
  
  return nodesWithSpecifiedRole;
}

function getRoleForNode(node, role_name) {
  var roles = node.get('roles');

  var role = undefined;
  for (var i = 0; i < roles.length; i++) {
    if (roles[i].childType === role_name) {
      role = roles[i];
      break;
    }
  }

  return role;
}

function contains(array, obj, comparison_function) {
    var i = array.length;
    while (i--) {
      if (comparison_function(array[i], obj)) {
           return true;
       }
    }
    return false;
}


function validate() {
  if (userChoices.is_valid()) {
    wizard.common.enable('nextButton');
  }
  else
    wizard.common.disable('nextButton');

  var remove_node_element = document.getElementById('removeNodesButton');
  if (userChoices.nodes.length == 0) {
    remove_node_element.setAttribute("disabled", true);
  }
  else {
    remove_node_element.setAttribute("disabled", false);
  }
  
}

function selectNodes() {

  var nodesWithDockerRegistryRole = nodesWithRole('DockerRegistryRole');
  var nodesWithDockerRegistryRoleKeys = nodesWithDockerRegistryRole.map(function (node) {
    return node.get('uniqueKey');
  });
  
  var args = {
    cluster: current,
    baseType: 'Node',
    currentnodes: [],
    excludeFunction: function (node) {
      var found = false;

      var currentNodeHostname = node.get('hostname');
      var comparisonOnCvNode = function(nodeObject, hostname) {
        return nodeObject.get('hostname') === hostname;
      };

      var comparisonOnNode = function(nodeObject, hostname) {
        return hostname === nodeObject.hostname;
      };
      
      var hasNodeAlreadyDockerRegistry = contains(nodesWithDockerRegistryRole, currentNodeHostname, comparisonOnCvNode);
      var isNodeAlreadySelected = contains(userChoices.nodes, currentNodeHostname, comparisonOnNode);
      var isMaster = 'MasterNode' == node.getType();

      return hasNodeAlreadyDockerRegistry || isNodeAlreadySelected || isMaster;
    } 
  };
  
  var d = new dock.cvDialog('chrome://cmgui/content/cephwizard/selectitems.xul', args);
  d.onOk = function(keys) {

    tree = document.getElementById('nodes');
    var allNodes = current.find('SlaveNode');

    for (var k = 0; k < keys.length; k++) {
      for (var i = 0; i < allNodes.length; i++) {
        if (allNodes[i].get('uniqueKey') === keys[k]) {
          userChoices.nodes.push(new Node(allNodes[i]).createTreeItem());
        }
      }
    }
    loadTable(tree, userChoices.nodes);
    
    validate();
  };
  dock.dialogs.showDialog(d);
}

function removeNodes() {
  println("Removing nodes ...");

  var selected_nodes = selected('dockerRegistryNodes');
  println(selected_nodes[0]);

  for(var selected_index = 0; selected_index < selected_nodes.length; selected_index++) {
    var current_key = selected_nodes[selected_index];

    var index = userChoices.nodes.findIndex(function(node) {
      return node.key == current_key;
    });

    println(index);
    if (index >= 0)
      userChoices.nodes.splice(index);
  }

  tree = document.getElementById('nodes');
  loadTable(tree, userChoices.nodes);
  validate();
}

function MainWizardScreen(wizard) {
  WizardScreen.call(this, wizard);

  this.configIsLoaded = false;

  this.preEnter = function() {
    wizard.common.unhide('loadConfigButton');
  };

  this.enter = function() {
    println('enter wizard.main.enter');
  };

  this.leave = function(automatic) {
    wizard.common.hide('loadConfigButton');
  };

  this.asyncForward = function(automatic, followup) {
    
    wizard.config.modules = {
      'docker-registry': {
        nodes: userChoices.node_names(),
        registry: userChoices.registry,
        rpms: ['cm-docker-registry'],
        skip_rpms: false,
        skip_reboot: false
      }
    };
    
    followup();
  };
}

function NodeSelectionWizardScreen(wizard) {
  WizardScreen.call(this, wizard);

  this.configIsLoaded = false;

  this.preEnter = function() {
    wizard.common.unhide('loadConfigButton');

    var elem = document.getElementById('nextButton');
    elem.setAttribute('label', 'Next');
    elem.setAttribute('accesskey', 'N');
  };

  this.enter = function() {
    println('enter wizard.nodeselection.enter');
    validate();
  };

  this.leave = function(automatic) {
    wizard.common.hide('loadConfigButton');
  };

  this.forward = function() {
    wizard.deck['nodes'].enable_click = true;
    wizard.deck['summary'].enable_click = true;

    var elem = document.getElementById('nextButton');
    elem.setAttribute('label', 'Deploy');
    elem.setAttribute('accesskey', 'D');

    wizard.config.modules = {
      'docker-registry': {
        nodes: userChoices.node_names(),
        registry: userChoices.registry,
        rpms: ['cm-docker-registry'],
        skip_rpms: false,
        skip_reboot: false
      }
    };
  };
}

function SummaryWizardScreen(wizard) {
  WizardScreen.call(this, wizard);

  this.enter = function() {
    println('enter wizard.summary.enter');

    var selected_nodes_element = document.getElementById('summary-selected-nodes');
    selected_nodes_element.value = userChoices.node_names().join(", ");

    var registry_address_element = document.getElementById('summary-registry-address');
    registry_address_element.value = userChoices.registry.address || 'n/a';

    var registry_prefix_element = document.getElementById('summary-registry-prefix');
    registry_prefix_element.value = userChoices.registry.prefix || 'n/a';

    var registry_root_directory_element = document.getElementById('summary-registry-root-directory');
    registry_root_directory_element.value = userChoices.registry.storagedriver.filesystem.rootdirectory || 'n/a';
  };

  this.leave = function(automatic) {
    wizard.common.hide('loadConfigButton');
  };

  this.forward = function() {
    wizard.deck['summary'].enable_click = true;
    wizard.deck['deploy'].enable_click = true;
  };
}


function DockerRegistryWizard() {
  Wizard.call(this);
  this.dryMode = false;
  this.forceDryMode = false;
  this.advancedMode = false;
  this.main = new MainWizardScreen(this);
  this.nodes = new NodeSelectionWizardScreen(this);
  this.summary = new SummaryWizardScreen(this);
  this.deploy = new DeployWizardScreen(this);
  
  this.deploy.cm_setup_tool_script = 'cm-docker-registry-setup';
  this.deploy.cm_setup_tool_name   = 'Docker Registry';
  this.deck = this.createDeck( [ { 'name' : 'main'},
                                 { 'name' : 'nodes'},
                                 { 'name' : 'summary'},
                                 { 'name' : 'deploy'}] );
  this.currentStage = null;
  this.previousStage = null;
}
