/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var cluster;
var nodes;
var hdfskey;
var allroles;
var nmroles;
var myDialog;
var configs2pass;
var nodesapplied2pass;
var lRoles;
var mapCoRoles;
var old;
var hservices;
var okPressed = false;

var configOnLoad;


Array.prototype.diff = function(a) {
    return this.filter(function(i) {return a.indexOf(i) < 0;});
};

function setAttributes(el, attrs) {
  for(var key in attrs)   el.setAttribute(key, attrs[key]);
}

function changeTabFocus(ind) {
  document.getElementById("tabbox").selectedIndex = ind;
}

function getTabFocus() {
  return document.getElementById("tabbox").selectedIndex;
}


function onLoad() {
  dock = getDock();
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  refresh(myDialog.arguments.object);

  // Commented since New and Clone buttons are hidden when all config-overlays 
  // displayed (in the Cluster tab)
  /*
  if ( location.hash == "#clone" ) {
    document.title = "Clone Configuration Overlay";
    document.getElementById('name').removeAttribute("readonly");
  }

  if ( location.hash == "#new" ) {
    document.title = "New Configuration Overlay";
    document.getElementById('name').removeAttribute("readonly");
  }
  */

  // makes 'name' field changeable for non-committed yet overlays
  if ( myDialog.arguments.object.get('uniqueKey') < dock.MAX_LOCAL_KEY ) {
    document.getElementById('name').removeAttribute("readonly");
  }

  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) {
     current = use;
     old = current.old(false, true);
  }

  println("Refresh Configuration Overlay" + ' - ' + document.documentURI);
  
  // uncomment to print out configuration group with assigned roles
  // printall(current.object);
  // for(var j = 0; j < current.object.roles.length; j++)    printall(current.object.roles[j]);
  
  cluster = current.cluster;
  nodes = current.get('nodes');

  // uncomment to print out node-role mapping for all nodes used by a hadoop instance  
  // for (var key in mapCoRoles) {
  //  println("Key: " + key);
  //  for (var k = 0; k < mapCoRoles[key].length; k++)  println("      " + mapCoRoles[key][k].name + ", " + mapCoRoles[key][k].priority);
  // }

  // text('priority', '^[0-9]+$');
  text('priority', dock.REGEX_COPriority);
  text('name', '');

  var elem = document.getElementById('nodesconfigured'); 
  elem.setAttribute("value", dotFormatKeys(cluster, nodes) );
 
  var tabs = document.getElementById('tabs');   
  var tabpanels = document.getElementById('tabpanels');   

  // when refreshing remove tabs created earlier if any
  while(tabs.firstChild) tabs.removeChild(tabs.firstChild);
  while(tabpanels.firstChild) tabpanels.removeChild(tabpanels.firstChild);

  // Split button is not shown since its precise functionality has not been agreed on
  // uncomment to have the button
  // var splitconfgroup = document.getElementById('splitconfgroupbutton');  
  // splitconfgroup.hidden = true;

  var roles = current.get('roles')
  lRoles = new Object();
  configs2pass = new Object();

  for(var j = 0; j < roles.length; j++) {
    var roleType = roles[j].childType;
    var role = new dock.cvObject(cluster, roles[j], current, 'roles');
    lRoles[roleType] = role;
  }

  hservices = new Array();

  var k = 0;

  // for each role in configuration overlay create tab with its configuration settings window  
  Object.keys(lRoles).forEach(function(key) {
    // some roles (to be removed) marked for removal and hence not shown
    if (!lRoles[key].get('toBeRemoved') ) {
      var shadowtab = document.createElement('shadowtab');
      var iframeelem = document.createElement('iframe');
      var iframesrc = key.substring(0, key.length - 4).toLowerCase() + 'settings.xul';
      setAttributes(shadowtab, {'subtab': 'true', 'id': 'tab_' + k, 'label': 'Configure ' + convertRoleType2ServiceName(key), 'context':'removerole' });
      hservices.push(key);
      // setAttributes(iframeelem, {'id': 'iframe_' + k, 'src': iframesrc });
      tabs.appendChild(shadowtab);
      // tabpanels.appendChild(iframeelem);
      k++;
    }
  });

  //current['hservices'] = hservices;

  changeTabFocus(0);
  
  /*
  println("Refresh Hadoop Role Configuration Tabs");
  for(var i = 0; i < window.frames.length; i++) {
    println("Frame" + i); 
    //alert(ee.contentDocument);

    //ee.contentWindow.location.reload(true);
    if(!!window.frames[i].refresh) {
      println("-Frame" + i);
      window.frames[i].refresh(current.configobj);
    }
  }
  */

}


// has to be checked; in the form below it is taken from edithadoopconfgroupsettings.js and 
// has the following logic: on the left it shows all nodes of a cluster, on the right it is 
// nodes set to a configuration overlay
function manageNodes() {
    var xulfilename = 'hadoop/hadoopnodeselection.xul';
    var args = new Object();
    args['configoverlay'] = current;
    args['mapcoroles'] = mapCoRoles;
    var d = new dock.cvDialog(xulfilename,args);

    d.onOk = function() {
      refresh();
    }

    dock.dialogs.showDialog(d);
}


function buttonCancel() {
  // if 'OK' button (that keeps changes done on config-group) 
  // pressed then window should simply be closed; otherwise original 
  // object (config-group at the time of window pop-up) restored
  if (!okPressed) {
    if ( location.hash == "#clone" || location.hash == "#new"   ) { 
      cluster.removeObject(current.get('uniqueKey'));
    }
    current.old(old);
    close();
  }
  else {
    close();
  }
}


function buttonOk() {
  // save();
  if ( !checkConfOverlayPriority(current.get('priority')) ) {
    dock.displayError('Priority value is out of range: Change priority value.');
    return;
  }  

  if ( !checkConfOverlayName() ) {
    okPressed = true;
    myDialog.onOk();
    close();

    // warnings section
    if ( current.get('nodes').length == 0 )    dock.displayInfo("Warning: No nodes have been defined for configuration group.");
  } 

}


function checkConfOverlayName() {
  var overlays = cluster.find('ConfigurationOverlay');
  var uniqueKey = current.get('uniqueKey');
  var coNames = [];   
  for(var k = 0; k < overlays.length; k++) {
    if ( overlays[k].get('uniqueKey') != uniqueKey )
      coNames.push(overlays[k].resolveName());
  }
  return ( coNames.indexOf(current.resolveName()) > -1 );  
}

