/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var newEdit;
var object;
var old;
var current;
var sysInfo;
var myDialog;
var dock;

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  object = myDialog.arguments.object;
  sysInfo = object.sysInfo;

  if(myDialog.arguments.subobject <= 0) {
    newEdit = true;
    setDialogTitle("New GPU Setting");
    current = new dock.cvGPUSettings();
    current = new dock.cvObject(object.cluster, current);
    refresh();
  } else {
    newEdit = false;
    current = object.subObjectFromKey('gpuSettings', myDialog.arguments.subobject);
    if(isObject(current)) {
      old = current.old(false, true);
      refresh();
    } else {
      dock.displayError("Key no longer valid");
      myDialog.doCancel();
      close();
    }
  }
}

function refresh() {
  var newKey = 0;
  if (newEdit) newKey = current.get('uniqueKey');
  
  text('name', '^all$|^[0-9]+$|[0-9]+-[0-9]+$');
  text('powerLimit', '^[0-9]{0,3}$');

  var powerModes = new Array();
  var eccModes = new Array();
  var computeModes = new Array();
  var operationModes = new Array();
  var persistenceModes = new Array();
  var accountingModes = new Array();
  var clockAutoBoostModes = new Array();
  var clocksSettings = new Array();

  if(isSysInfoAvailable()) {
    for (var i = 0; i < sysInfo.gpuUnits.length; i++) {
      var unit = sysInfo.gpuUnits[i];
      for (var j = 0; j < unit.gpu.length; j++) {
        var gpu = unit.gpu[j];
        powerModes = addValuesToList(powerModes, gpu.powerMode);
        eccModes = addValuesToList(eccModes, gpu.eccMode);
        computeModes = addValuesToList(computeModes, gpu.computeMode);
        operationModes = addValuesToList(operationModes, gpu.operationMode);
        persistenceModes = addValuesToList(persistenceModes, gpu.persistenceMode);
        accountingModes = addValuesToList(accountingModes, gpu.accountingMode);
        clockAutoBoostModes = addValuesToList(clockAutoBoostModes, gpu.clockAutoBoostMode);
        clocksSettings = addCouplesValuesToList(clocksSettings, gpu.clocksSettings);
      }
    }
    powerModes = to_human(sort_unique(powerModes));
    eecModes = to_human(sort_unique(eecModes));
    computeModes = to_human(sort_unique(computeModes));
    operationModes = to_human(sort_unique(operationModes));
    persistenceModes = to_human(sort_unique(persistenceModes));
    accountingModes = to_human(sort_unique(accountingModes));
    clockAutoBoostModes = to_human(sort_unique(clockAutoBoostModes));
    clocksSettings = to_human(sort_unique(clocksSettings));
  } else {
    powerModes = from_human(['', 'Supported']);
    eccModes = from_human(['', 'Enabled', 'Disabled']);
    computeModes = from_human(['', 'Default', 'Exclusive thread', 'Exclusive process', 'Prohibited']);
    operationModes = from_human(['', 'All on', 'Compute', 'Low double precision']);
    persistenceModes = from_human(['', 'Enabled', 'Disabled']);
    accountingModes = from_human(['', 'Enabled', 'Disabled']);
    clockAutoBoostModes = from_human(['', 'Enabled', 'Disabled']);
    clocksSettings =  from_human(['', 'min / min', 'max / max']);
  }
  
  kenum('powerMode', powerModes);
  kenum('eccMode', eccModes);
  kenum('computeMode', computeModes);
  kenum('operationMode', operationModes);
  kenum('persistenceMode', persistenceModes);
  kenum('accountingMode', accountingModes);
  kenum('clockAutoBoostMode', clockAutoBoostModes);
  kenum('clockSpeeds', clocksSettings);
}

function isSysInfoAvailable() {
  if (sysInfo && sysInfo.gpuUnits && (sysInfo.gpuUnits.length > 0)) {
    return true;
  }
  return false;
}

function ok() {
  if(!window.closed) {
    if(blank(current.get('name'))) {
      dock.displayError("GPU range cannot be empty");
      return;
    }
    if(newEdit) {
      var list = object.get('gpuSettings');
      list.push(current.object);
      object.set('gpuSettings', list);
    } else if(current.get('modified')) {
      object.set('modified', true);
    }
    myDialog.doOk();
    close();
  }
}

function cancel() {
  if(!window.closed) {
    if(!newEdit) current.old(old);
    myDialog.doCancel();
    close();
  }
}

function addValuesToList(list, str) {
  var sep = ',';
  if((str === "N/A") || (str === "") || (str.indexOf(sep) == -1)) {
    list.push(str);
  } else {
    var splitted = str.split(sep);
    for (var i = 0; i < splitted.length; i++) {
      list = addValuesToList(list, splitted[i]);
    }
  }
  return list;
}

function addCouplesValuesToList(list, couples) {
  if (couples.length == 0) {
    list.push("");
  } else {
    list = couples;
  }  
  return list;
}

function from_human(list) {
  var r = new Array();
  for(var i = 0; i < list.length; i++) {
    r.push(one_from_human(list[i]));
  }
  return r;
}

function one_from_human(text) {
  return [text.replace(/ \/ /g, '#').replace(/ /g, '_'), text];
}

function to_human(list) {
  var r = new Array();
  for(var i = 0; i < list.length; i++) {
    r.push(one_to_human(list[i]));
  }
  return r;
}

function one_to_human(text) {
  return [text, text.replace(/_/g, ' ').replace(/#/g, ' / ')];
}
