/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var newEdit;
var object;
var old = 0;
var current;
var myDialog;
var dock;

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  object = myDialog.arguments.object;
  if(myDialog.arguments.subobject <= 0) {
    newEdit = true;
    setDialogTitle("New Health Check Configuration")
    current = new dock.cvMonHealthConf();
    current = new dock.cvObject(object.cluster, current);
    refresh();
  } else {
    newEdit = false;
    current = object.subObjectFromKey('checks', myDialog.arguments.subobject);
    if(isObject(current)) {
      old = current.old(false, true);
      refresh();
    } else {
      dock.displayError("Key no longer valid");
      myDialog.doCancel();
      close();
    }
  }
}

function refresh() {
  var title = getDialogTitle();
  resolve('healthcheck', 'HealthCheck');
  if(!newEdit) document.getElementById("healthcheck").setAttribute('disabled', true);
  text('healthcheckParam', '^');
  specialvalues('samplingInterval', [[0, 'N/A'],[4294967295, 'prejob']]);
  text('samplingInterval', '^([0-9]+|prejob)$');
  text('rawTableLength', '^[0-9]+$');
  text('gapThreshold', '^[0-9]+$');
  text('thresholdDuration', '^[0-9]+$');
  text('revision');
  var parent = current.cluster.find(current.get('healthcheck'));
  var dis = false;
  var disl = false;
  if(parent) {
    dis = parent.get('disabled');
    disl = parent.get('disableOnHighLoad');
  }
  check('storeInDatabase');
  check('disabled', dis);
  check('disableOnHighLoad', disl);
  text('failSeverity', '^[0-9]+$');
  text('unknownSeverity', '^[0-9]+$');

  threshActionList('stateFlapping', 'State Flapping:');
  threshActionList('passActionConfs', 'Pass action:');
  threshActionList('failActionConfs', 'Fail action:');
  threshActionList('unknownActionConfs', 'Unknown action:');
  setfocus('healthcheck');
}

function prejob() {
  current.set('samplingInterval', 4294967295);
  refresh();
}

function ok() {
  if(!window.closed) {
    if(current.get('healthcheck') == 0) {
      dock.displayError("Health Check cannot be empty");
      return;
    }
    var healthcheck = current.cluster.find(current.get('healthcheck'));
    switch(healthcheck.get('paramPermission')) {
    case('PARAM_DISALLOWED') :
      if(!blank(current.get('healthcheckParam'))) {
        dock.displayError("Health Check parameter cannot be set");
        return;
      }
      break;
    case('PARAM_REQUIRED') :
      if(blank(current.get('healthcheckParam'))) {
        dock.displayError("Health Check parameter has to be set");
        return;
      }
      break;
    case('PARAM_OPTIONAL') :
    }
    if(newEdit) {
      var list = object.get('checks');
      var key = current.get('healthcheck');
      var param = current.get('healthcheckParam');
      for(var i = 0; i < list.length; i++) {
        if((list[i].healthcheck == key) && (list[i].healthcheckParam == param)) {
          dock.displayError("A healthcheck configuration with that healthcheck/parameter already exits");
          return;
        }
      }
      list.push(current.object);
      object.set('checks', list);
    } else if(current.get('modified')) {
      object.set('modified', true);
    }
    if((current.get('samplingInterval') == 4294967295) && (!old || (old['samplingInterval'] != 4294967295))) {
      var drainKey = current.cluster.find('ThreshAction', 'Drain node');
      if(drainKey) {
        drainKey = drainKey.get('uniqueKey');
        var found = false;
        var list = current.get('failActionConfs');
        for(var i = 0; !found && (i < list.length); i++) {
          found = (list.threshAction == drainKey);
        }
        if(!found) {
          var one = new dock.cvThreshActionConf();
          one.threshAction = drainKey;
          list.push(one);
          current.set('failActionConfs', list);
        }
      }
    }
    myDialog.doOk();
    close();
  }
}

function cancel() {
  if(!window.closed) {
    if(!newEdit) current.old(old);
    myDialog.doCancel();
    close();
  }
}
