/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var newEdit;
var object;
var old;
var current;
var myDialog;
var dock;

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  object = myDialog.arguments.object;
  if(myDialog.arguments.subobject <= 0) {
    newEdit = true;
    setDialogTitle("New Network Interface")
  } else {
    newEdit = false;
    current = object.subObjectFromKey('networks', myDialog.arguments.subobject);
    if(isObject(current)) {
      old = current.old(false, true);
      refresh();
    } else {
      dock.displayError("Key no longer valid");
      myDialog.doCancel();
      close();
    }
  }
}

function refresh() {
  var newKey = 0;
  if(newEdit) newKey = current.get('uniqueKey');
  var title = getDialogTitle();
  var name;
  resolve('network', 'Network', '', true);
  partial('ip', '.', dock.REGEX_IPv4_PARTIAL);  
//   text('ip', dock.REGEX_IPv4);
  check('dhcp');
  text('ipv6Ip', dock.REGEX_IPv6);
  check('ipv6Dhcp');
  partial('mac', ':', '^[0-9a-fA-F]{2}$', '', false, '00:00:00:00:00:00', 0, '6..21', '00');  
  text('revision');
  stringlist('additionalHostnames', "Additional Hostname:", dock.REGEX_HOSTNAME);
  kenum('startIf', [['ALWAYS', 'Always'],['ACTIVE', 'Active'],['PASSIVE', 'Passive'],['PREFERPASSIVE', 'Prefer passive']]);
  kenum('bringupduringinstall', [['NO', 'No'],['BRINGUP', 'Bring up'],['BRINGUPANDKEEP', 'Bring up and keep']]);
  switch(current.object.childType) {
    case("NetworkPhysicalInterface"):
      name = "physical";
      text('name', dock.REGEX_IF_PHYSICAL, '', true, '', newKey);
      text('speed');
      break;
    case("NetworkAliasInterface"):
      name = "alias";
      text('name', dock.REGEX_IF_ALIAS, '', true, '', newKey);
      break;
    case("NetworkBondInterface"):
      name = "bond";
      text('name', '^bond[0-9]{1,4}$');
      number('mode', 0, 6);
      text('options');
//       checkboxlist('interfaces', freeInterfaces());
      stringlist('interfaces', "");
      break;
    case("NetworkVLANInterface"):
      name = "VLAN";
      text('name', dock.REGEX_IF_VLAN, '', true, '', newKey);
      break;
    case("NetworkBmcInterface"):
      name = "BMC";
      text('name', '^(ipmi|ilo|cimc|drac)[0-9]{1,4}$', '', true, '', newKey);
//       partial('gateway', '.', dock.REGEX_IPv4);
      text('gateway', dock.REGEX_IPv6);
      text('vlanid', '^[0-9]+$');
      break;
    case("NetworkBridgeInterface"):
      name = "bridge";
      text('name', '^br[0-9]{1,4}$', '', true, '', newKey);
      text('forward_delay', '^[0-9]+$');
      check('stp');
      stringlist('interfaces', "");
      break;
    case("NetworkNetMapInterface"):
      name = "NetMap";
      text('name', '^map[0-9]{1,4}$', '', true, '', newKey);
      break;
    case("NetworkTunnelInterface"):
      name = "tunnel";
      text('name', '^tun[0-9]{1,4}$', '', true, '', newKey);
      break;
    default:
  }
  
  var external = false;
  var network = current.cluster.find(current.get('network'));
  if(network) external = (network.get('type') == 'EXTERNAL') && network.get('IPv6');
  
  document.getElementById('speedrow').setAttribute('hidden', (current.object.childType != "NetworkPhysicalInterface"));
  document.getElementById('gatewayrow').setAttribute('hidden', (current.object.childType != "NetworkBmcInterface"));
  document.getElementById('vlanidrow').setAttribute('hidden', (current.object.childType != "NetworkBmcInterface"));
  document.getElementById('moderow').setAttribute('hidden', (current.object.childType != "NetworkBondInterface"));
  document.getElementById('optionsrow').setAttribute('hidden', (current.object.childType != "NetworkBondInterface"));
  document.getElementById('interfacesrow').setAttribute('hidden', (current.object.childType != "NetworkBondInterface") && (current.object.childType != "NetworkBridgeInterface"));
  document.getElementById('forward_delayrow').setAttribute('hidden', (current.object.childType != "NetworkBridgeInterface"));
  document.getElementById('macrow').setAttribute('hidden', (current.object.childType != "NetworkPhysicalInterface"));
  document.getElementById('ipv6row').setAttribute('hidden', !external);
  document.getElementById("maindeck").selectedIndex = 1;
  setDialogTitle(title + " (" + name + ")");
  
  autorefresh('network');
  setfocus('name');
}

function freeInterfaces() {
  var interfaces = object.get('networks');
  var list = new Array();
  for(var i = 0; i < interfaces.length; i++) {
    if(interfaces[i].childType == "NetworkPhysicalInterface") {
      //if(interfaces[i].network == 0) { // Fix #2116
        list.push(interfaces[i].name);
      //}
    }
  }
  return list;
}

function ok() {
  if(!window.closed) {
    if(!!current) {
      if(blank(current.get('name'))) {
        dock.displayError("Name cannot be empty");
        return;
      }
      if(newEdit) {
        var list = object.get('networks');
        list.push(current.object);
        object.set('networks', list);
      } else if(current.get('modified')) {
        object.set('modified', true);
      }
      myDialog.doOk();
      close();
    } else {
      var list = document.getElementById("networkinterfacetypes");
      var type = list.getSelectedItem(0);
      if(!!type) {
        type = type.getAttribute("label");
        switch(type) {
          case("Physical"):
            current = new dock.cvNetworkPhysicalInterface();
            break;
          case("Alias"):
            current = new dock.cvNetworkAliasInterface();
            break;
          case("Bond"):
            current = new dock.cvNetworkBondInterface();
            break;
          case("VLAN"):
            current = new dock.cvNetworkVLANInterface();
            break;
          case("CIMC"):
            object.cluster.hasUCS = true;
          case("IPMI"):
          case("iLO"):
            current = new dock.cvNetworkBmcInterface();
            break;
          case("Bridge"):
            current = new dock.cvNetworkBridgeInterface();
            break;
          case("Tunnel"):
            current = new dock.cvNetworkTunnelInterface();
            break;
          case("NetMap"):
            current = new dock.cvNetworkNetMapInterface();
            break;
          default:
        }
        current = new dock.cvObject(object.cluster, current);
        refresh();
      }
    }
  }
}

function cancel() {
  if(!window.closed) {
    if(!newEdit) current.old(old);
    myDialog.doCancel();
    close();
  }
}
