/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var AVAIL_AUTO_LABEL = '<automatic>';

var newEdit;
var object;
var old;
var current;
var defaultRegion;
var myDialog;
var dock;
var newSubnets; // subnets not yet committed (possibily to be deleted)

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  object = myDialog.arguments.object;
  defaultRegion = object.get('defaultRegion');
  if (myDialog.arguments.subobject <= 0) {
    newEdit = true;
    setDialogTitle("New Private Cloud")
    current = new dock.cvEC2PrivateCloud();
    current = new dock.cvObject(object.cluster, current);

    document.getElementById('ok').setAttribute('hidden', true);
    refresh();
  } else {
    newEdit = false;
    current = object.subObjectFromKey('privateClouds', myDialog.arguments.subobject);
    if (isObject(current)) {
      old = current.old(false, true);
      document.getElementById('next').setAttribute('hidden', true);
      document.getElementById('ok').removeAttribute('disabled');
      refresh();
    } else {
      dock.displayError("Key no longer valid");
      myDialog.doCancel();
      close();
    }
  }
  newSubnets = new Array();
  
  // various text messages
  document.getElementById('header').value = "Basic Private Cloud Information";
  document.getElementById('old-header').value = "Subnet Assignment";
  document.getElementById('new-header').value = "Subnet Creation";
  var oldmessage = "Specify which subnets are inside the private cloud. It is advised to have at least two subnets: one for the cloud director, and one for the cloud compute nodes.";
  if (newEdit) {
    document.getElementById('old-message').height = '72px';
    oldmessage += " You can always manage subnets later by editing the private cloud.";
    document.getElementById('warn-message').value = "Click 'Next' before assigning subnets.";
  }
  document.getElementById('old-message').value = oldmessage;

}

function refresh() {
  var newKey = 0;
  if (newEdit) newKey = current.get('uniqueKey');

  // name
  text('name', '^');
  setfocus('name');

  // region
  if (newEdit) {
    resolve('region', 'EC2Region');
    // select default region
    var regionDrop = document.getElementById('region');
    for (var i = 0; i < regionDrop.itemCount; i++) {
      if (regionDrop.getItemAtIndex(i).getAttribute('key') == defaultRegion) {
        regionDrop.selectedIndex = i;
        break;
      }
    }
  } else {
    var found = current.cluster.find(current.get('region')).resolveName();
    var hbox = document.getElementById('regionbox');
    hbox.removeChild(document.getElementById('region'));
    var textbox = document.createElement('textbox');
    textbox.setAttribute('id', 'region');
    textbox.setAttribute('value', found);
    hbox.appendChild(textbox);
  }

  // base
  partial('baseAddress', '.', '^(25[0-5]|2[0-4][0-9]|[0-1]?[0-9]?[0-9])$');

  // mask  
  if (newEdit) {
    number('netmaskBits', 16, 28);
  } else {
    var hbox = document.getElementById('iprange');
    hbox.removeChild(document.getElementById('netmaskBits'));
    var textbox = document.createElement('textbox');
    textbox.setAttribute('id', 'netmaskBits');
    textbox.setAttribute('class', 'ip');
    hbox.appendChild(textbox);    
    text('netmaskBits', '');
  }

  // subnets
  if (newEdit) {
    refreshSubnetsForNewVPC();
  } else {
    disableEditVPC();
    fillNewSubnetSection(current.get('region'), current.get('netmaskBits'), 28);
    refreshSubnetsForEditingVPC();
  }
}

function checkOverlappingSubnets(subnets) {
  // check for duplicates and overlapping subnets
  if (subnets.length > 1) {
    for (var i = 0; i < subnets.length; i++) {
      for (var j = i + 1; j < subnets.length; j++) {
        var sub1 = current.cluster.find(subnets[i]);
        var sub2 = current.cluster.find(subnets[j]);
        if (overlappingNetworks(sub1, sub2)) {
          dock.displayError("Subnets '" + sub1.resolveName() + "' and '" + sub2.resolveName() + "' overlap");
          return false; 
        }
      }
    }
  }
  return true;
}


function refreshSubnets(filter_func, display_func) {
  function checkEventListener(event) {
    if(event.target) {
      var netKey = parseInt(event.target.getAttribute("netKey"));
      var checked = event.target.checked;
      var list = current.get('subnets');
      if(checked) {
	      if(list.indexOf(netKey) == -1) {
	        list.push(netKey);
          checkOverlappingSubnets(list);
	        current.set('subnets', list);
	      }
      }
      else {
	      var idx = list.indexOf(netKey);
	      if(idx != -1) {
	        list.splice(idx, 1);
	        current.set('subnets', list);
	      }
      }
    }
  }

  var subnets = document.getElementById("subnets");
  while (subnets.firstChild) {
    subnets.removeChild(subnets.firstChild);
  }
  var networks = current.cluster.find("Network");
  var list = current.get('subnets');
  for(var i = 0; i < networks.length; i++) {
    if(!filter_func(networks[i])) {
      var hbox = document.createElement("hbox");
      var checkbox = document.createElement('checkimage');
      var netKey = networks[i].get('uniqueKey');
      checkbox.addEventListener("custom-change", checkEventListener, true);
      checkbox.setAttribute('netKey', netKey);
      checkbox.setAttribute('checked', (list.indexOf(netKey) != -1));
      hbox.appendChild(checkbox);
      var label = document.createElement("shadowlabel");
      label.setAttribute("class", "staticdataheader");
      label.setAttribute("value", display_func(networks[i]));
      hbox.appendChild(label);
      subnets.appendChild(hbox);
    }
  }
}

function refreshSubnetsForNewVPC() {
  /* only suggest networks that:
   * -- have "CLOUD" type (network.get("type") != "CLOUD")
   *    AND
   * -- have range included in VPC range (netStart >= currStart) && (netEnd <= currEnd)
   *    AND
   * -- are not used by other VPCs (network.get("EC2SubnetID") == "")
   */
  function filter(network) { 
    var currStart = ip2int(current.get("baseAddress")); 
    var currEnd = lastIP(currStart, current.get("netmaskBits")); 
    var netStart = ip2int(network.get("baseAddress")); 
    var netEnd = lastIP(netStart, network.get("netmaskBits")); 
    return ( (network.get("type") != "CLOUD") || (netStart < currStart) || 
             (netEnd > currEnd) || (network.get("EC2SubnetID") != "") ); 
  };
 
  function display(subnet) { 
    return subnet.resolveName() + " (" + subnet.get("baseAddress") + " / " + subnet.get("netmaskBits") + ")"; 
  }
  if(document.getElementById('next').getAttribute('hidden') == false) {
    return;
  }
  return refreshSubnets(filter, display);
  //resolveList('subnets', "Subnet:", "Network", '', '', '', filter, display);
}

function refreshSubnetsForEditingVPC() {
  /* only suggest networks that:
   * -- have "CLOUD" type (network.get("type") != "CLOUD")
   *    AND
   * -- have range included in VPC range (netStart >= currStart) && (netEnd <= currEnd)
   *    AND
   * -- are not used by other VPCs or are already in this VPC ((network.get("EC2SubnetID") == "") || (current.get('subnets').indexOf(netKey) != -1))
   */
  function filter(network) { 
    var currStart = ip2int(current.get("baseAddress")); 
    var currEnd = lastIP(currStart, current.get("netmaskBits")); 
    var netStart = ip2int(network.get("baseAddress")); 
    var netEnd = lastIP(netStart, network.get("netmaskBits")); 
    var netKey = network.get("uniqueKey"); 
    return ((network.get("type") != "CLOUD") || 
             ((netStart < currStart) || 
              (netEnd > currEnd)) || 
             ((network.get("EC2SubnetID") != "") &&
              (current.get("subnets").indexOf(netKey) == -1))); 
  } 
  function display(subnet) { 
    return subnet.resolveName() + 
      " (" + subnet.get("baseAddress") + " / " + subnet.get("netmaskBits") + ")"; 
  }
  refreshSubnets(filter, display);
}

function next() {
  if(blank(current.get('name'))) {
    dock.displayError("Name cannot be empty");
    return;
  }
  var region = parseInt(document.getElementById('region').selectedItem.getAttribute('key'));
  if(region == 0) {
    dock.displayError("Region cannot be empty");
    return;
  }

  disableEditVPC();
  document.getElementById('warn-message').value = "";
  document.getElementById('next').setAttribute('hidden', true);
  document.getElementById('ok').removeAttribute('hidden');
  document.getElementById('ok').removeAttribute('disabled');

  //add new subnet section
  fillNewSubnetSection(region, current.get('netmaskBits'), 28);

  // other subnets section
  if(newEdit) {
    refreshSubnetsForNewVPC();
  }
  else {
    refreshSubnetsForEditingVPC();
  }
}

function ok() {
  if (!window.closed) {
    if (blank(current.get('name'))) {
      dock.displayError("Name cannot be empty");
      return;
    }

    if (newEdit) {
      var region = parseInt(document.getElementById('region').selectedItem.getAttribute('key'));
      if(region == 0) {
        dock.displayError("Region cannot be empty");
        return;
      }
      current.set('region', region);
    }

    // remove empty subnets (0 key)
    var subnets = current.get('subnets').removeall(0);

    // check for duplicates and overlapping subnets
    if (subnets.length > 1) {
      for (var i = 0; i < subnets.length; i++) {
        for (var j = i + 1; j < subnets.length; j++) {
          if (subnets[i] == subnets[j]) {
            dock.displayError("Subnets cannot be duplicated");
            return;
          }
          var sub1 = current.cluster.find(subnets[i]);
          var sub2 = current.cluster.find(subnets[j]);
          if (overlappingNetworks(sub1, sub2)) {
            dock.displayError("Subnets '" + sub1.resolveName() + "' and '" + sub2.resolveName() + "' overlap");
            return;
          }
        }
      }
    }

    if (newEdit) {
      var list = object.get('privateClouds');
      list.push(current.object);
      object.set('privateClouds', list);
    } else if (current.get('modified')) {
      object.set('modified', true);
    }

    // remove subnets added to cluster, but not to the vpc
    for (var i = 0; i < newSubnets.length; i++) {
      if (subnets.find(newSubnets[i]) == -1) {
        current.cluster.removeObject(newSubnets[i]);
      }
    }
    myDialog.doOk();
    close();
  }
}

function cancel() {
  if (!window.closed) {
    if (!newEdit) current.old(old);
    // remove subnets added to cluster
    for (var i = 0; i < newSubnets.length; i++) {
      current.cluster.removeObject(newSubnets[i]);
    }
    myDialog.doCancel();
    close();
  }
}

function disableEditVPC() {
  document.getElementById('region').disabled = true;
  document.getElementById('baseAddress-0').disabled = true;
  document.getElementById('baseAddress-1').disabled = true;
  document.getElementById('baseAddress-2').disabled = true;
  document.getElementById('baseAddress-3').disabled = true;
  document.getElementById('netmaskBits').disabled = true;
}

function fillNewSubnetSection(regionKey, min, max) {
  // name
  document.getElementById('subnetname').removeAttribute('disabled');

  // netmask
  document.getElementById('subBaseAddress-0').value = document.getElementById('baseAddress-0').value;
  document.getElementById('subBaseAddress-1').value = document.getElementById('baseAddress-1').value;
  document.getElementById('subBaseAddress-2').value = document.getElementById('baseAddress-2').value;
  if (min < 24) {
    document.getElementById('subBaseAddress-2').removeAttribute('disabled');
  }
  document.getElementById('subBaseAddress-3').value = document.getElementById('baseAddress-3').value;
  document.getElementById('subBaseAddress-3').removeAttribute('disabled');

  // netmask bits
  var obj = document.getElementById('subNetmaskBits');
  var popup = obj.firstChild;
  while (popup.firstChild) popup.removeChild(popup.firstChild);
  for (var i = min; i <= max; i++) {
    var item = document.createElement("menuitem");
    item.setAttribute('label', i);
    item.setAttribute('number', i);
    popup.appendChild(item);
  }
  obj.selectedIndex = 0;
  obj.removeAttribute('disabled');
  
  // availability zones
  var obj = document.getElementById('availzone');
  var popup = obj.firstChild;
  while (popup.firstChild) popup.removeChild(popup.firstChild);
  var item = document.createElement("menuitem");
  item.setAttribute('label', AVAIL_AUTO_LABEL);
  popup.appendChild(item);
  var region = current.cluster.find(regionKey);
  var zones = region.get('availabilityZones');
  for (var i = 0; i < zones.length; i++) {
    var item = document.createElement("menuitem");
    item.setAttribute('label', zones[i].name);
    popup.appendChild(item);
  }
  obj.selectedIndex = 0;
  obj.removeAttribute('disabled');

  document.getElementById('create').removeAttribute('disabled');
}

function createSubnet() {
  var name = document.getElementById('subnetname').value;

  if(blank(name)) {
    dock.displayError("Subnet name cannot be empty");
    return;
  }

  if (!regexMatch(document.getElementById('subBaseAddress-2').value, '^(25[0-5]|2[0-4][0-9]|[0-1]?[0-9]?[0-9])$') ||
      !regexMatch(document.getElementById('subBaseAddress-3').value, '^(25[0-5]|2[0-4][0-9]|[0-1]?[0-9]?[0-9])$')) {
    dock.displayError("Subnet base address is invalid");
    return;
  }

  var netmaskBits = document.getElementById('subNetmaskBits').label;
  var baseAddress = document.getElementById('subBaseAddress-0').value + "." +
    document.getElementById('subBaseAddress-1').value + "." +
    document.getElementById('subBaseAddress-2').value + "." +
    document.getElementById('subBaseAddress-3').value;

  // validation against VPC
  var vpcStart = ip2int(current.get("baseAddress"));
  var vpcEnd = lastIP(vpcStart, current.get("netmaskBits"));
  var netStart = ip2int(baseAddress);
  var netEnd = lastIP(netStart, netmaskBits);
  if ((netStart < vpcStart) || (netEnd > vpcEnd)) {
    dock.displayError("Subnet range not compatible with Private cloud range");
    return;
  }

  var subnet = new dock.cvNetwork();
  // create boxed objects
  subnet = current.cluster.addObject(subnet, false, true);
  //subnet = new dock.cvObject(current.cluster, subnet);
  subnet.set('name', name);
  subnet.set('netmaskBits', netmaskBits);
  subnet.set('baseAddress', baseAddress);
  subnet.set('domainName', name + '.cluster');
  subnet.set('type', 'CLOUD');
  subnet.set('management', true);
  subnet.set('gateway', int2ip(ip2int(baseAddress) + 1));
  if (document.getElementById('availzone').label != AVAIL_AUTO_LABEL) {
    subnet.set('EC2AvailabilityZone', document.getElementById('availzone').label);
  }
  var subnetKey = subnet.get('uniqueKey');
  newSubnets.push(subnetKey);
  var list = current.get('subnets');
  list.push(subnetKey);
  current.set('subnets', list);
  // the subnet is submitted in the caller (uncomment the following for testing purposes)
  // subnet.commitNonThreaded();

  // clean for another subnet
  document.getElementById('subnetname').value = '';

  // other subnets section
  if(newEdit) {
    refreshSubnetsForNewVPC();
  }
  else {
    refreshSubnetsForEditingVPC();
  }
}

function overlappingNetworks(net1, net2) {
  if ((typeof net1 == 'object') && (typeof net2 == 'object')) {
    var start1 = net1.get('baseAddress');
    var mask1 = net1.get('netmaskBits');
    var start2 = net2.get('baseAddress');
    var mask2 = net2.get('netmaskBits');

    return overlappingRanges(start1, mask1, start2, mask2);
  } else {
    return false;
  }
}
