/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var NEW_TAB = 1000000;

var allEvents = new Array();
var eventFilters;
var eventViewers;
var ackedEvents = new Object();
var nextEventKey = 0;

function saveAckedEvents() {
  var file = BASE_PATH + "ackedevents.dat";
  var data = ackedEvents.toSource();
  savefile(file, data);
}

function loadAckedEvents() {
  var file = BASE_PATH + "ackedevents.dat";
  var data = readfile(file);
  if(data.length > 0) {
    eval('ackedEvents = ' + data + ';');
  }
}

function saveEventFilters() {
  var file = BASE_PATH + "filters.dat";
  var data = "";
  for(var i = 0; i < eventFilters.length; i++) {
    data += eventFilters[i].save();
  }
  savefile(file, data);
}

function loadEventViewers() {
  eventViewers = new Array();
  var file = BASE_PATH + "eventviewers.dat";
  var data = readListFromFile(file);
  if(isArray(data) && (data.length > 0)) {
    while(data.length > 0) {
      var one = new cvEventViewer();
      one.filter = new cvEventFilter('');
      if(!one.load(data)) {
        println("Failed to load viewer:");
        println(data);
      }
      eventViewers.push(one);
    }
  }
  if(eventViewers.length == 0) addBlankFilter('All Events');
}

function loadEventFilters() {
  eventFilters = new Array();
  var file = BASE_PATH + "filters.dat";
  var data = readListFromFile(file);
  if(isArray(data) && (data.length > 0)) {
    while(data.length > 0) {
      var one = new cvEventFilter();
      if(!one.load(data)) {
        print(data);
      }
      eventFilters.push(one);
    }
  }
}

function saveEventViewers() {
  var file = BASE_PATH + "eventviewers.dat";
  var data = "";
  for(var i = 0; i < eventViewers.length; i++) {
    if(eventViewers[i].tab >= 0) data += eventViewers[i].save();
  }
  savefile(file, data);
}

function addBlankFilter(name) {
  var one = new cvEventViewer();
  one.filter = new cvEventFilter(name);
  one.tab = NEW_TAB;
  eventViewers.push(one);
}

function openViewers() {
  function compareViewer(a, b) {
    return a.tab - b.tab;
  }
  var tabs = document.getElementById('eventTabs');
  var panels = document.getElementById('eventPanels');
  eventViewers = eventViewers.sort(compareViewer);
  var k = 0;
  for(var i = 0; i < eventViewers.length; i++) {
    if(eventViewers[i].tab >= 0) {
      eventViewers[i].tab = k;
      k++;
    }
    if(!eventViewers[i].frame) {
      if(eventViewers[i].tab < 0) {
        var args = new Object();
        args['viewer'] = eventViewers[i];
        var viewer = new cvDialog("eventwindow.xul", args);
        dialogs.show(viewer);
      } else {
        var tab = document.createElement('shadowtab');
        tab.setAttribute('label', eventViewers[i].filter.name);
        tabs.appendChild(tab);
        var frame = document.createElement('iframe');
        frame.setAttribute('id', 'eventviewer[' + i + ']');
        frame.setAttribute('src', 'eventviewer.xul');
        panels.appendChild(frame);
        if(tabs.childNodes.length == 1) tabs.selectedIndex = 0;
      }
    }
  }
  saveEventViewers();
}

function openNewEventViewer() {
  var viewer = new cvEventViewer();
  viewer.tab = NEW_TAB;
  viewer.filter = new cvEventFilter('');
  if(clusterlist.length == 1) viewer.cluster = clusterlist[0];
  viewer.showFilter();
}

function cvLocalEvent() {
  this.key = 0;
  this.image = 0;
  this.date = 0;
  this.cluster = 0;
  this.clusterObject = 0;
  this.event = 0;
  this.source = 0;
  this.tag = '';
  this.message = 0;
  this.details = 0;
  this.severity = '';
  this.func = 0;
  this.ack = false;
}

cvLocalEvent.prototype.show = function() {
  if(this.func) {
    eval(this.func);
  } else {
    var args = new Object();
    args['event'] = this;
    var d = new cvDialog("eventdetails.xul", args);
    d.onOk = function() {
      var bid = this.arguments.event.event.bid;
      var index = -1;
      var cluster = dock.ackedEvents[this.arguments.event.clusterObject.key];
      if(cluster) index = cluster.find(bid);
      if(this.arguments.event.ack) {
        this.arguments.event.ack = false;
        if(index >= 0) {
          dock.ackedEvents[this.arguments.event.clusterObject.key].splice(index, 1);
          dock.saveAckedEvents();
        }
      } else {
        this.arguments.event.ack = true;
        if(index < 0) {
          if(!cluster) dock.ackedEvents[this.arguments.event.clusterObject.key] = new Array();
          if(bid > 0) {
            dock.ackedEvents[this.arguments.event.clusterObject.key].push(bid);
            dock.saveAckedEvents();
          }
        }
      }
      for(var i = 0; i < dock.eventViewers.length; i++) dock.eventViewers[i].refresh();
    }
    dialogs.showDialog(d);
  }
}

function showEvent(key) {
  for(var i = 0; i < allEvents.length; i++) {
    if(key == allEvents[i].key) {
      allEvents[i].show();
      break;
    }
  }
}

function refreshEventViewers() {
  for(var i = 0; i < eventViewers.length; i++) {
    eventViewers[i].refresh();
  }
}

function addEvent(cluster, event, latest) {
  /* Do not add events twice during reconnect */
  if(event.bid > 0) {
    for(var i = 0; i < allEvents.length; i++) {
      if((allEvents[i].event.bid == event.bid) && (allEvents[i].clusterObject == cluster)) {
//         println("*** Duplicate event ***");
//         if(latest > 0) refreshEventViewers();
        return;
      }
    }
  }
  var v = eventmessage(cluster, event);
  if(!isArray(v)) {
//     if(latest > 0) refreshEventViewers();
    return;
  }
  if(!isArray(v[0])) v = [v];

  var source = event.sourceDevice;
  var tag = '';
  if(source == 0) {
    source = "Unknown";
  } else {
    source = cluster.find(source);
    if(isObject(source)) {
      tag = source.get('tag');
      source = source.resolveName();
    } else {
      source = "Unknown";
    }
  }

  for(var i = 0; i < v.length; i++) {
    var current = new cvLocalEvent();
    current['key'] = (++nextEventKey);
    var d = new Date();
    d.setTime(event.creationTime * 1000);
    current['date'] = d;
    current['clusterObject'] = cluster;
    current['cluster'] = cluster.name;
    current['event'] = event;
    current['source'] = source;
    current['tag'] = tag;
    if(v[i][0] == '') {
      current['image'] = severity2icon(event.severity);
    } else {
      current['image'] = v[i][0];
    }
    if((v[i].length > 3) && !!v[i][3]) {
      current['func'] = v[i][3];
    }
    current['severity'] = severity2human(event.severity);
    current['message'] = v[i][1];
    current['details'] = v[i][2];
    var acks = ackedEvents[cluster.key];
    if(acks) {
      current['ack'] = (acks.find(event.bid) >= 0);
    } else {
      current['ack'] = false;
    }
    cluster.dispatchEvent(event);
    allEvents.push(current);
    if(allEvents.length > dock.generalSettings.eventCache) allEvents.shift();
  }

//   if(latest >= 0) refreshEventViewers();
}

function removeEvent(key) {
  for(var i = 0; i < allEvents.length; i++) {
    if(allEvents[i].key == key) {
      allEvents[i].splice(i, 1);
      for(var j = 0; j < eventViewers.length; j++) eventViewers[j].refresh();
      break;
    }
  }
}

function findFilter(name) {
  for(var i = 0; i < eventFilters.length; i++) {
    if(eventFilters[i].name == name) return eventFilters[i];
  }
  return 0;
}

function removeFilter(name) {
  for(var i = 0; i < eventFilters.length; i++) {
    if(eventFilters[i].name == name) {
      eventFilters.splice(i, 1);
      openViewers(); // Set tab index
      return true;
    }
  }
  return false;
}

function findViewer(tab) {
  for(var i = 0; i < eventViewers.length; i++) {
    if(eventViewers[i].tab == tab) return eventViewers[i];
  }
  return 0;
}

function refreshCurrentEventViewers() {
  for(var i = 0; i < eventViewers.length; i++) {
    if(eventViewers[i].filter.source == -1) eventViewers[i].refresh();
  }
  return 0;
}

function closeAllEventViewers() {
  for(var i = 0; i < eventViewers.length; i++) {
    if(eventViewers[i].tab < 0) eventViewers[i].close();
  }
}

function cvEventFilter(name) {
  this.name = name;
  this.cluster = 0;
  this.source = 0;
  this.sourceFilter = '';
  this.severity = 0;
  this.maxseverity = 3000;
  this.time = [259200, 0]; // 72 hours
  this.message = '';
  this.showAcked = false;
}

cvEventFilter.prototype.clone = function() {
  var one = new cvEventFilter();
  one.name = this.name;
  one.time = this.time;
  one.cluster = this.cluster;
  one.source = this.source;
  one.sourceFilter = this.sourceFilter;
  one.message = this.message;
  one.severity = this.severity;
  return one;
}

cvEventFilter.prototype.apply = function() {
  var now = new Date();
  now = now.getTime() / 1000;
  var key = 0;
  if(currentObject && currentObject.object) key = currentObject.get('uniqueKey');
  var list = new Array();
  for(var i = 0; i < allEvents.length; i++) {
    var one = allEvents[i];
    if(!this.showAcked && one.ack) continue;
    if((this.cluster != 0) && (this.cluster != one.clusterObject)) continue;
    if((this.source > 0) && (this.source != one.event.sourceDevice)) continue;
    if((this.source < 0) && (key != one.event.sourceDevice)) continue;
    if(one.event.severity < this.severity) continue;
    if(one.event.severity > this.maxseverity) continue;
    var et = one.event.creationTime - one.clusterObject.timeOffset;
    if(this.time[1] == 0) {
      if(et < (now - this.time[0])) continue;
    } else {
      if((et < this.time[0]) || (et > this.time[1])) continue;
    }
    if(this.sourceFilter && !regexMatch(one.source, this.sourceFilter)) continue;
    if(this.message && !regexMatch(one.message, this.message)) continue;
    list.push(one);
  }
  return list;
}

cvEventFilter.prototype.save = function() {
  var data = "++++BEGIN EVENT FILTER++++\n";
  data += "name = '" + this.name + "';\n";
  if(this.cluster) {
    data += "cluster = '" + this.cluster.key + "';\n";
  } else {
    data += "cluster = " + this.cluster + ";\n";
  }
  data += "source = " + this.source + ";\n";
  data += "sourceFilter = '" + this.sourceFilter + "';\n";
  data += "time = " + this.time.toSource() + ";\n";
  data += "message = '" + this.message + "';\n";
  data += "showAcked = " + this.showAcked + ";\n";
  data += "severity = " + this.severity + ";\n";
  data += "maxseverity = " + this.maxseverity + ";\n";
  data += "++++END EVENT FILTER++++\n";
  return data;
}

cvEventFilter.prototype.load = function(data) {
  if(data[0] != "++++BEGIN EVENT FILTER++++") return false;
  var i = 1;
  while(i < data.length) {
    if(data[i] == "++++END EVENT FILTER++++") {
      data.splice(0, i + 1);
      if(this.cluster) {
        this.cluster = findObject(this.cluster);
        if(!this.cluster) {
          displayInfo("Event filter " + this.name + " no longer valid, cluster not found");
          return false;
        }
      }
      return true;
    }
    eval("this." + data[i]);
    i++;
  }
  return false;
}

function cvEventViewer() {
  this.filter = 0;
  this.tab = 0;
  this.order = [['date', 'descending'], ['bid', 'descending'], ['cluster', 'ascending'], ['source', 'ascending'], ['severity', 'ascending'], ['message', 'ascending']];
  this.tab = 0;
  this.frame = 0;
}

cvEventViewer.prototype.refresh = function() {
  if(this.frame) {
    if(this.frame.refresh) {
      this.frame.refresh();
    }
  }
}

cvEventViewer.prototype.save = function() {
  var data = "++++BEGIN EVENT VIEWER++++\n";
  data += "tab = " + this.tab + ";\n";
  data += "order = " + this.order.toSource() + ";\n";
  data += "filter.name = '" + this.filter.name + "';\n";
  if(typeof(this.filter.cluster) != 'undefined') {
    data += "filter.cluster = 0;\n";
  } else if(this.filter.cluster) {
    data += "filter.cluster = '" + this.filter.cluster.key + "';\n";
  } else {
    data += "filter.cluster = " + this.filter.cluster + ";\n";
  }
  data += "filter.source = " + this.filter.source + ";\n";
  data += "filter.sourceFilter = '" + this.filter.sourceFilter + "';\n";
  data += "filter.time = " + this.filter.time.toSource() + ";\n";
  data += "filter.message = '" + this.filter.message + "';\n";
  data += "filter.showAcked = " + this.filter.showAcked + ";\n";
  data += "filter.severity = " + this.filter.severity + ";\n";
  data += "filter.maxseverity = " + this.filter.maxseverity + ";\n";
  data += "++++END EVENT VIEWER++++\n";
  return data;
}

cvEventViewer.prototype.load = function(data) {
  if(data[0] != "++++BEGIN EVENT VIEWER++++") return false;
  var i = 1;
  while(i < data.length) {
    if(data[i] == "++++END EVENT VIEWER++++") {
      data.splice(0, i + 1);
      var k = -1;
      for(var j = 0; j < this.order.length; j++) {
        if(this.order[j][0] == 'bid') {
          k = -1;
          break;
        } else if(this.order[j][0] == 'date') {
          k = j + 1;
        }
      }
      if(k >= 0) this.order.splice(k, 0, ['bid', 'descending']);
      return true;
    }
    eval("this." + data[i]);
    i++;
  }
  return false;
}

cvEventViewer.prototype.detach = function() {
  var tabs = document.getElementById("eventTabs");
  var panels = document.getElementById("eventPanels");
  panels.removeChild(panels.childNodes[this.tab]);
  if(tabs.childNodes.length > 1) tabs.advanceSelectedTab(1, true);
  tabs.removeItemAt(this.tab);
  this.tab = -1;
  this.frame = 0;
}

cvEventViewer.prototype.showFilter = function() {
  var args = new Object();
  args['viewer'] = this;
  var d = new cvDialog("eventfilter.xul", args);
  d.onOk = function() {
    if(this.arguments.viewer.tab == NEW_TAB) {
      dock.eventViewers.push(this.arguments.viewer);
      dock.openViewers();
      var tabs = document.getElementById("eventTabs").selectedIndex = this.arguments.viewer.tab;
    } else if(this.arguments.viewer.tab >= 0) {
      var tabs = document.getElementById("eventTabs");
      tabs.childNodes[this.arguments.viewer.tab].setAttribute('label', this.arguments.viewer.filter.name);
      dock.saveEventViewers();
    } else {
      this.arguments.viewer.frame.parent.document.getElementById('filtername').setAttribute('value', this.arguments.viewer.filter.name);
    }
  }
  dialogs.showDialog(d);
}

cvEventViewer.prototype.clearEvents = function(all) {
  all = isundefined(all, false);
  var select = this.frame.selected('list');
  var changed = false;
  var checkEvents;
  if(all && this.filter) {
    checkEvents = this.filter.apply();
  } else {
    checkEvents = allEvents;
  }
  for(var i = 0; i < checkEvents.length; i++) {
    if(all || select.find(checkEvents[i].key) >= 0) {
      if(!checkEvents[i].ack) {
        checkEvents[i].ack = true;
        if(!isArray(ackedEvents[checkEvents[i].clusterObject.key])) ackedEvents[checkEvents[i].clusterObject.key] = new Array();
        if(checkEvents[i].event.bid > 0) dock.ackedEvents[checkEvents[i].clusterObject.key].push(checkEvents[i].event.bid);
        changed = true;
      }
    }
  }
  if(changed) {
    saveAckedEvents();
    for(var i = 0; i < dock.eventViewers.length; i++) dock.eventViewers[i].refresh();
  }
}

cvEventViewer.prototype.close = function() {
  if(this.frame) {
    if(typeof(saveOrder) == 'function') this.frame.saveOrder();
  }
  if(this.tab < 0) {
    if(this.frame) this.frame.close();
  } else {
    var tabs = document.getElementById("eventTabs");
    var panels = document.getElementById("eventPanels");
    panels.removeChild(panels.childNodes[this.tab]);
    if(tabs.childNodes.length > 1) tabs.advanceSelectedTab(1, true);
    tabs.removeItemAt(this.tab);
    this.frame = 0;
  }
  for(var i = 0; i < eventViewers.length; i++) {
    if(eventViewers[i] == this) {
      eventViewers.splice(i, 1);
      break;
    }
  }
}

cvEventViewer.prototype.reatach = function() {
  this.tab = NEW_TAB;
  this.frame = 0;
}

function translateSpecialWarningEvent(info) {
  if((info.length > 4) && startswith(info, "@@") && endswith(info, "@@")) {
    if(string_case_equal(info, "@@INDENTIFIED@@")) {
      return [". Double click event to assign.", 'this.clusterObject.select("NodeFolder", "newnodes");'];
    } else if(string_case_equal(info, "@@REFRESH@@CLUSTER@@")) {
      if(currentTab && (currentTab.id == 'clustertab')) timedOutRefresh(100);
      return false;
    } else if(string_case_equal(info, "@@REFRESH@@")) {
      timedOutRefresh();
      return false;
    }
  }
  return true;
}

function puppetEventDetails(event, msg, cluster) {
  var ret = msg + "\n";
  if(event.childType == "PuppetApplyStartedEvent" || 
     event.childType == "PuppetApplyDoneEvent") 
  {
    if(event.hasOwnProperty('resultSession')) {
      var n = 0;
      for(var i = 0; i < event.resultSession.resultStatistics.length-2; i++) {
        n+= event.resultSession.resultStatistics[i];
      }
      ret+= "Number of nodes: " + n + "\n";
    }
    if(event.hasOwnProperty('result')) {
      var first = true;
      for(var i = 0; i < event.result.length; i++) {
        if(first) first = false;
        else ret+= ", ";
        ret+= cluster.resolveName(event.result[i].nodeKey);
        if(event.result[i].numberOfClasses > 0 || 
           event.result[i].numberOfResources > 0) {
          ret+= " (";
          var has_classes = false;
          if(event.result[i].numberOfClasses > 0) {
            has_classes = true;
            ret+= event.result[i].numberOfClasses + " classes ";
          }
          if(event.result[i].numberOfResources > 0) {
            if(has_classes) ret+= ", ";
            ret+= event.result[i].numberOfResources + " resources ";
          }
          ret+= " )";
        }
      }
    }
  }
  else if(event.childType == "PuppetForgeSearchDoneEvent") {
    ret+  "\n\n";
    ret+= "search mode:  " +   event.request.searchMode + "\n";
    ret+= "search state: " +  event.request.searchState + "\n";
    if(event.request.message) {
      ret+= "message:      " +  event.request.message + "\n";
    }
    for(var i = 0; i < event.request.options.length; i++) {
      event.request.options[i].key + ": " + event.request.options[i].value + "\n";
    }
  }
  else if(event.childType == "PuppetForgeInstallationDoneEvent") {
    ret+= "\n";
    ret+= "command: " + event.request.command ;
    for(var i = 0; i < event.request.flags.length; i++) {
      ret+= " " + event.request.flags[i];
    }
    ret+= "\n";
    ret+= 'state:   ' + event.request.installState + "\n";
    ret+= 'module:  ' + event.request.module + "\n";
    ret+= 'version: ' + event.request.version + "\n";
    ret+= 'path:    ' + event.request.modulepath + "\n\n";
    var long_error_msg = event.request.longError;
    var res = long_error_msg.split("\n");
    if(res.length > 0) {
      ret+= "errors\n";
      ret+= "------\n";
      for(var i = 0; i < res.length; i++) {
        ret+= res[i].replace(/\x1B.*?m/g,'');
      }
    }
    if(event.request.results.length > 0) {
      ret+= "\n";
      ret+= "result\n";
      ret+= "------\n";
      for(var i = 0; i < event.request.results.length; i++) {
        ret+= event.request.results[i] + "\n";
      }
    }
  }
  else if(event.childType == "PuppetRescanDoneEvent") {
    ret+= "\n";
    if(event.result.failedManifests.length > 0) {
      ret+= "scan failures\n";
      ret+= "-------------\n";
      for(var i = 0; i < event.result.failedManifests.length; i++) {
        ret+= event.result.failedManifests[i] + ":\n";
        if(i < event.result.failedManifestsMessages.length) {
          ret+= event.result.failedManifestsMessages[i] + "\n";
        }
      }
      ret+= "\n\n";
    }
    if(event.result.moduleValidations.length > 0 || 
       event.result.classValidations.length > 0) 
    {
      ret+= "warnings\n";
      ret+= "--------\n";
      for(var i = 0; i < event.result.moduleValidations.length; i++) {
        ret+= event.result.moduleValidations[i].msg + "\n";
      }
      for(var i = 0; i < event.result.classValidations.length; i++) {
        ret+= event.result.classValidations[i].msg + "\n";
      }
    }
  }
  return ret;
}

