/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */


/* Open flags
#define PR_RDONLY       0x01
#define PR_WRONLY       0x02
#define PR_RDWR         0x04
#define PR_CREATE_FILE  0x08
#define PR_APPEND       0x10
#define PR_TRUNCATE     0x20
#define PR_SYNC         0x40
#define PR_EXCL         0x80
*/
/*
** File modes ....
**
** CAVEAT: 'mode' is currently only applicable on UNIX platforms.
** The 'mode' argument may be ignored by PR_Open on other platforms.
**
**   00400   Read by owner.
**   00200   Write by owner.
**   00100   Execute (search if a directory) by owner.
**   00040   Read by group.
**   00020   Write by group.
**   00010   Execute by group.
**   00004   Read by others.
**   00002   Write by others
**   00001   Execute by others.
**
*/

var oct0600 = parseInt('0600', 8); // octal literals and octal escape sequences are deprecated

/* savefile
 *  write entrire string 'output' to specified file 'filename'
 */
function savefile(filename, output, permissions) {
  permissions = isundefined(permissions, oct0600);
  println("filename: " + filename);
  try {
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(filename);
    if(!file.exists()) {
      println("Creating file... ");
      file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, permissions);
    }
    var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
    outputStream.init(file, 0x04 | 0x08 | 0x20, permissions, 0);
    var result = outputStream.write(output, output.length);
    outputStream.close();
    return true;
  } catch (e) {
    println("Unable to write file: '" + filename + "'");
    return false;
  }
}

/* readfile
 *  returns entire content of file 'filename' as a single string
 */
function readfile(filename) {
  var permissions = oct0600;
  var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
  try {
    file.initWithPath(filename);
    if (!file.exists()) {
      println("File '" + filename + "' does not exist");
      return "";
    }
    var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
      .createInstance(Components.interfaces.nsIFileInputStream );
    is.init(file, 0x01, permissions, null);
    var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
      .createInstance(Components.interfaces.nsIScriptableInputStream );
    sis.init(is);
    var output = sis.read(sis.available());
    sis.close();
    return output;
  } catch(e) {
    println("File '" + filename + "' does not exist");
    return 0;
  }
}

function fileExists(filename) {
//   try {
//     netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
//   } catch (e) {
//     alert("Permission to read file was denied.");
//     return "";
//   }
  var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
  try {
    file.initWithPath(filename);
    return file.exists();
  } catch(e) {
    println(e);
  }
  return false;
}

function allfiles(directory) {
  var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
  var array = new Array();
  try {
    file.initWithPath(directory);
    if (file.exists() && file.isDirectory()) {
      var entries = file.directoryEntries;
      while(entries.hasMoreElements()) {
        var entry = entries.getNext();
        entry.QueryInterface(Components.interfaces.nsIFile);
        array.push(entry.path);
      }
    } else {
      println("Directory " + directory + " not found");
    }
  } catch(e) {
    println(e);
  }
  return array;
}

function readListFromFile(filename, joinchar) {
  joinchar = isundefined(joinchar, '\n');
  var list = readfile(filename);
  if(list) {
    list = list.split(joinchar);
    var i = 0;
    while(i < list.length) {
      if(list[i].length == 0) {
        list.splice(i, 1);
      } else {
        i++;
      }
    }
    return list;
  }
  return [];
}

function saveListToFile(filename, list, joinchar) {
  joinchar = isundefined(joinchar, '\n');
  savefile(filename, list.join(joinchar));
}

function read2DListFromFile(filename, joinchar1, joinchar2, stripblank) {
  joinchar1 = isundefined(joinchar1, '\n');
  joinchar2 = isundefined(joinchar2, ' | ');
  stripblank = isundefined(stripblank, false);
  var list = readfile(filename);
  if(list) {
    list = list.split(joinchar1);
    if(stripblank) {
      var i = 0;
      while(i < list.length) {
        if(list[i].length == 0) {
          list.splice(i, 1);
        } else {
          i++;
        }
      }
    }
    for(i = 0; i < list.length; i++) {
      list[i] = list[i].split(joinchar2);
      if(stripblank) {
        var j = 0;
        while(j < list[i].length) {
          if(list[i][j].length == 0) {
            list[i].splice(j, 1);
          } else {
            j++;
          }
        }
      }
    }
    return list;
  }
  return [];
}

function save2DListToFile(filename, list, joinchar1, joinchar2) {
  joinchar1 = isundefined(joinchar1, '\n');
  joinchar2 = isundefined(joinchar2, ' | ');
  var out = new Array();
  for(var i = 0; i < list.length; i++) {
    out.push(list[i].join(joinchar2));
  }
  savefile(filename, out.join(joinchar1));
}
