/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function createTreeItem(one) {
  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;

  var ip = getManagementIP(one);
  var item = new Object();
  item['key'] = one.get('uniqueKey');
  item['modified'] = one.get('modified');
  item['hostname'] = one.get('hostname');
  item['mac'] = one.get('mac');
  item['tag'] = one.get('tag');
  item['category'] = cluster.resolveName(one.get('category'));
  item['ip'] = ip[1];
  var cloudsettings = one.get('cloudSettings');
  if(cloudsettings) {
    if(cloudsettings.childType == 'EC2Settings') {
      var provider = cluster.find(cloudsettings.provider);
      if(provider) item['provider'] = provider.resolveName();
      item['cloudid'] = cloudsettings.instanceId;
      var type = cloudsettings.type;
      if((type == 0) && provider) {
        if(one.hasCloudDirectorRole()) {
          type = provider.get('defaultDirectorType');
        } else {
          type = provider.get('defaultType');
        }
      }
      item['type'] = cluster.resolveName(type);
      item['storage'] = '';
      var storage = cloudsettings.storage;
      for(var i = 0; i < storage.length; i++) {
        if(storage[i].childType == 'EC2EBSStorage') {
          if(item['storage'].length > 0) item['storage'] += ', ';
          item['storage'] += storage[i].name + ' ('+ formatKMGselect(storage[i].size, 'B') + ')';
        }
      }
    }
  }
  return item;
}


function cloneObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a " + singular + " for cloning");
  } else if(select.length > 1) {
    dock.displayError("Please select only one " + singular);
  } else {
    var cluster = current;
    if(!!current.cluster) cluster = current.cluster;
    var original = cluster.find(select[0]);
    if(isObject(original)) {
      var args = new Object();
      var host = original.get('hostname');
      args['title'] = 'Clone ' + host;
      args['question'] = 'Hostname';
      var nn = nameNumber(host);
      if(nn[1].length == 0) nn[1] = '0';
      var name;
      var index = myParseInt(nn[1]);
      while(true) {
        index++;
        name = nn[0] + sprintf('%0' + nn[1].length + 'd', index);
        if(!cluster.find('Device', name)) break;
      }
      args['default'] = name;
      args['known'] = cluster.known('Device', 'hostname');
      var d = new dock.cvDialog("newobject.xul", args);
      d.onOk = function(name) {
        var object = original.clone();
        object.modified = true;
        object.toBeRemoved = false;
        object.hostname = name;
        object.mac = '00:00:00:00:00:00';
        object.tag = '00000000a000';
        var cloudsettings = object.cloudSettings;
        if(cloudsettings) {
          if(cloudsettings.childType == 'EC2Settings') {
            cloudsettings.instanceID = '';
          }
        }
        object.ethernetSwitch = new dock.cvSwitchPort();
        object.powerDistributionUnits = new Array();
        cloneNodeIP(original.object, object);
        cluster.addObject(object, dock.openAfterCreate);
        refresh();
      }
      dock.dialogs.showDialog(d);
    } else {
      dock.displayError("Original not found, already removed by someone else");
    }
  }
}
