/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var nodeMap;
var filter = [[0, '', '', false]];
var notOfType = [];
var nodeCache = 0;

/**
 * @param excludeFunction - excludeFunction returning [false | true] or [ -1 | 0 | 1 ]
 *
 * Example:
 *
 * excludeFunction should return a boolean, true or false.
 *  true) exclude it
 *  false) don't exclude it.
 *
 * or one of, -1, 0, 1.
 *   0) exclude
 *  -1) include on the left-column
 *   1) include on the right-column
 */
function deviceselect(excludeFunction, nodeLabels) {
  excludeFunction = isundefined(excludeFunction, false);
  nodeLabels = nodeLabels ? nodeLabels : {};
  var n;
  if(fieldname) {
    n = current.get(fieldname);
    var list = current.cluster.resolveList(n);
    var obj = document.getElementById(fieldname);
    obj.value = list;
    obj.setAttribute('value', list);
    obj.addEventListener("command", nodes_command, true);
    obj.addEventListener("change", nodes_change, true);
    obj.addEventListener("keypress", nodes_keypress, true);
  } else {
    n = currentnodes;
  }  
  var no = new Object();
  for(var i = 0; i < n.length; i ++) {
    no[n[i]] = true;
  }
//   n.sort();

  var allnodesID = 'allnodes' + (typeof(nodesIndex) == 'undefined' ? '' : nodesIndex);
  var groupnodesID = 'groupnodes' + (typeof(nodesIndex) == 'undefined' ? '' : nodesIndex);
  var lba = document.getElementById(allnodesID);
  while(lba.childNodes.length > 1) lba.removeChild(lba.lastChild);
  var lbg = document.getElementById(groupnodesID);
  while(lbg.childNodes.length > 1) lbg.removeChild(lbg.lastChild);

  nodeMap = new Object();
  var currentKey = 0;
  var a;
  if(subtype.length) {
    if(current.isCluster) {
      a = current.find(subtype);
    } else {
      a = current.cluster.find(subtype);
      currentKey = current.get('uniqueKey');
    }
  } else {
    a = subtypeList;
  }
  nodeCache = a;
//   for(var i = 0; i < a.length; i++) {
//     a[i] = [a[i].resolveName(), a[i]];
//   }
//   a.sort();
  if(excludeFunction && (typeof(excludeFunction) == 'string')) eval('excludeFunction = ' + excludeFunction);
  var hasSpecialIcon = (typeof(nodesicon) != 'undefined');
  for(var i = 0; i < a.length; i++) {
    var key = a[i].get('uniqueKey');
    var type = a[i].getType();
    if((notOfType.find(type) < 0) && (key != currentKey)) {
      var doExclude = false;
      if (excludeFunction) {
        doExclude = excludeFunction(a[i]);
      }
      if (doExclude === true || doExclude === 0) continue;
      if (doExclude === 1) {
        no[key] = true; // move to right-hand side
      }

      var item = document.createElement('listitem');
      item.setAttribute('key', key);
      var name = a[i].resolveName();
      var namePostfix = '';
      if (nodeLabels && nodeLabels[key]) {
        namePostfix = " (" + nodeLabels[key] + ")";
      } else {
        namePostfix = eval(extraName); // not really flexible because it's const
      }
      item.setAttribute('label', name);
      var image = document.createElement('image');
      if(hasSpecialIcon) {
        image.setAttribute('src', 'chrome://cmgui/content/icons/' + nodesicon);
      } else {
        image.setAttribute('src', 'chrome://cmgui/content/icons/' + type + '_16.png');
      }
      var label = document.createElement('label');
      label.setAttribute('value', name + namePostfix);
      item.appendChild(image);
      item.appendChild(label);
      nodeMap[key] = item;
      if(!no.hasOwnProperty(key)) {
        if((eval(belongs) == 0) && eval(noloop)) {
          lba.appendChild(item);
        }
      } else {
        lbg.appendChild(item);
      }
    }
  }
  createFilter();
  runFilter(false);
}

function addToGroup() {
  var nodes;
  if(fieldname) {
    nodes = current.get(fieldname);
  } else {
    nodes = currentnodes;
  }
  var nodeMap = new Object();
  for(var i = 0; i < nodes.length; i++) nodeMap[nodes[i]] = true;
  var cluster;
  if(current.isCluster) {
    cluster = current;
  } else {
    cluster = current.cluster;
  }
  var allnodesID = 'allnodes' + (typeof(nodesIndex) == 'undefined' ? '' : nodesIndex);
  var lba = document.getElementById(allnodesID);
  for(var i = 0; i < lba.childNodes.length; i++) {
    var item = lba.childNodes[i];
    if(item.getAttribute('selected')) {
      var key = myParseInt(item.getAttribute('key'));
      if(!nodeMap.hasOwnProperty(key)) {
        nodes.push(key);
        nodeMap[key] = true;
        if(dock.SUBTREE && fieldname && displayInTree) {
          var node = cluster.find(key);
          if(node) cluster.addOneObject(node, current.branch);
        }
      } else {
        println('Already added ' + singular + ' ' + key);
      }
    }
  }
  nodes.sort();
  if(fieldname) {
    current.set(fieldname, nodes);
  } else {
    currentnodes = nodes;
  }

  if(dock.SUBTREE && fieldname && displayInTree) {
    parent.refresh();
  } else {
    refresh();
  }
  if(extraEvalOnAdd.length) eval(extraEvalOnAdd);
}

function removeFromGroup() {
  var nodes;
  if(fieldname) {
    nodes = current.get(fieldname);
  } else {
    nodes = currentnodes;
  }
  var cluster;
  if(current.isCluster) {
    cluster = current;
  } else {
    cluster = current.cluster;
  }
  var groupnodesID = 'groupnodes' + (typeof(nodesIndex) == 'undefined' ? '' : nodesIndex);
  var lbg = document.getElementById(groupnodesID);
  for(var i = 0; i < lbg.childNodes.length; i++) {
    var item = lbg.childNodes[i];
    if(item.getAttribute('selected')) {
      var key = myParseInt(item.getAttribute('key'));
      var index = nodes.find(key);
      if(index >= 0) {
        nodes.splice(index, 1);
        if(dock.SUBTREE && fieldname) {
          var node = cluster.find(key);
          if(node) node.removeTreeitemFromParent(current.branch);
        }
      }
    }
  }
  nodes.sort();
  if(fieldname) {
    current.set(fieldname, nodes);
  } else {
    currentnodes = nodes;
  }

  if(dock.SUBTREE && fieldname) {
    parent.refresh();
  } else {
    refresh();
  }
  if(extraEvalOnRemove.length) eval(extraEvalOnRemove);
}

function createFilter() {
  var filterID = 'filter' + (typeof(nodesIndex) == 'undefined' ? '' : nodesIndex);
  var rows = document.getElementById(filterID);
  while(rows.firstChild) rows.removeChild(rows.firstChild);
  var row;
  for(var i = 0; i < filter.length; i++) {
    row = document.createElement('row');
    row.setAttribute('align', 'center');
    var label = document.createElement('label');
    if(i == 0) {
      label.setAttribute('class', 'staticdataheader');
      label.setAttribute('value', 'Filter:');
    } else if(filter[i][0] == 0) {
      label.setAttribute('class', 'clustervalue');
      label.setAttribute('value', 'and');
    } else if(filter[i][0] == 1) {
      label.setAttribute('class', 'clustervalue');
      label.setAttribute('value', 'or');
    } else {
      label.setAttribute('class', 'clustervalue');
      label.setAttribute('value', '???');
    }
    row.appendChild(label);
    var combo = document.createElement('menulist');
    combo.setAttribute('oncommand', 'setParam(' + i + ');');
    var comboID = 'param' + (typeof(nodesIndex) == 'undefined' ? '' : nodesIndex) + i;
    combo.setAttribute('id', comboID);
    var popup = document.createElement('menupopup');
    addNodeProperties(popup, filter[i][1]);
    combo.appendChild(popup);
    row.appendChild(combo);
    //combo.setAttrubute('label', filter[i][1]);
    var spacer = document.createElement('spacer');
    spacer.setAttribute('width', '20px');
    row.appendChild(spacer);
    var label = document.createElement('label');
    label.setAttribute('class', 'staticdataheader');
    label.setAttribute('value', 'matches:');
    row.appendChild(label);
    var textbox = document.createElement('textbox');
    textbox.setAttribute('oncommand', 'setValue(' + i + ');');
    textbox.setAttribute('value', filter[i][2]);
    textbox.setAttribute('type', 'timed');
    textbox.setAttribute('timeout', '100');
    var textboxID = 'value' + (typeof(nodesIndex) == 'undefined' ? '' : nodesIndex) + i;
    textbox.setAttribute('id', textboxID);
    row.appendChild(textbox);
    var minus = document.createElement("button");
    minus.setAttribute('class', 'minus-button');
    minus.setAttribute('oncommand', "removeFilter(" + i + ");");
    row.appendChild(minus);
    rows.appendChild(row);
    if(i != filter.length - 1) {
      row = document.createElement('hbox');
      var box = document.createElement('hbox');
      box.setAttribute('style', 'border-top: dotted #bdbdbd 1px; border-bottom: dotted #efefef 1px; margin: 3px;');
      box.setAttribute('width', '380px');
      row.appendChild(box);
      rows.appendChild(row);
    }
  }
  var and = document.createElement("button");
  and.setAttribute('class', 'and-button');
  and.setAttribute('oncommand', "addFilter(0);");
  row.appendChild(and);
  var or = document.createElement("button");
  or.setAttribute('class', 'or-button');
  or.setAttribute('oncommand', "addFilter(1);");
  row.appendChild(or);
  var search = document.createElement("button");
  search.setAttribute('class', 'search-button');
  search.setAttribute('oncommand', "runFilter();");
  row.appendChild(search);
}

function addNodeProperties(popup, name) {
  for(var i = 0; i < nodeSelectionProperties.length; i++) {
    var item = document.createElement('menuitem');
    item.setAttribute('label', nodeSelectionProperties[i][0]);
    item.setAttribute('param', nodeSelectionProperties[i][1]);
    item.setAttribute('selected', (nodeSelectionProperties[i][1] == name));
    item.setAttribute('resolve', nodeSelectionProperties[i][2]);
    popup.appendChild(item);
  }
}

function setParam(index) {
  var paramID = 'param' + (typeof(nodesIndex) == 'undefined' ? '' : nodesIndex) + index;
  var param = document.getElementById(paramID);
  filter[index][1] = param.selectedItem.getAttribute('param');
  filter[index][3] = eval(param.selectedItem.getAttribute('resolve'));
  runFilter(false);
}

function setValue(index) {
  var valueID = 'value' + (typeof(nodesIndex) == 'undefined' ? '' : nodesIndex) + index;
  var param = document.getElementById(valueID);
  filter[index][2] = param.value
  runFilter(false);
}

function addFilter(type) {
  filter.push([type, '', '', false]);
  createFilter();
  runFilter(false);
}

function removeFilter(index) {
  filter.splice(index, 1);
  if(filter.length == 0) filter.push([0, '', '']);
  createFilter();
  runFilter(false);
}

function runFilter(verbose) {
  verbose = isundefined(verbose, true);
  var first = -1;
  var cluster;
  if(current.isCluster) {
    cluster = current;
  } else {
    cluster = current.cluster;
  }  
  var nodes;
  if(nodeCache) {
    nodes = nodeCache;
  } else {
    nodes = cluster.find(subtype);
    nodeCache = nodes;
  }
  for(var j = 0; j < nodes.length; j++) {
    var text = '';
    var key = nodes[j].get('uniqueKey');
    var item = nodeMap[key];
    if(item) {
      for(var i = 0; i < filter.length; i++) {
        if(blank(filter[i][1]) || blank(filter[i][2])) {
          if(verbose) {
            dock.displayInfo('Incomplete filter, all fields should be set');
          }
          return;
        }
        if(i > 0) {
          if(filter[i][0] == 0) {
            text += " && ";
          } else {
            text += " || ";
          }
        }
        if(filter[i][1] == '@JOB@') {
          var jobs = cluster.getJobCache(key);
          text += "/" + jobs.join('|') + "/i.test('" + filter[i][2] + "')";
        } else {
          var words = filter[i][1].split('.');
          var value = nodes[j].get(words[0]);
          for(var k = 1; k < words.length; k++) {
            value = value[words[k]];
          }
          if(filter[i][3]) {
            var object = cluster.find(value);
            if(object) {
              value = object.resolveName();
            } else {
              value = '';
            }
          }
          text += "/" + filter[i][2] + "/i.test('" + value + "')";
        }
      }
      var selected = eval(text);
      if(selected) {
        item.setAttribute('selected', selected);
        var index = item.parentNode.getIndexOfItem(item);
        if(first < 0) {
          item.parentNode.ensureElementIsVisible(item);
          first = index;
        } else if(index - first < item.parentNode.getNumberOfVisibleRows()) {
          item.parentNode.ensureElementIsVisible(item);
        }
      } else {
        item.removeAttribute('selected');
      }
    } else {
      println("No map: " + nodes[j].get('uniqueKey'));
    }
  }
}

function nodes_keypress(event) {
  var text = event.target;
  var value = text.value;
  var format = "";

  switch(event.keyCode) {
    case KEY_ENTER:
      if(regexMatch(value, format)) {
        var result = current.cluster.resolveText(value, subtype);
        if(result[0]) {
          if(dock.SUBTREE && fieldname) {
            current.updateBranch(fieldname, result[1]);
            parent.refresh();
          } else {
            refresh();
          }
        } else {
          var n = current.get(fieldname);
          var list = current.cluster.resolveList(n);
          text.value = list;
        }
      } else {
        var n = current.get(fieldname);
        var list = current.cluster.resolveList(n);
        text.value = list;
      }
      break;
    case KEY_ESCAPE:
      text.setAttribute("style", "");
      var n = current.get(fieldname);
      var list = current.cluster.resolveList(n);
      text.value = list;
  }
}

function nodes_change(event) {
  var text = event.target;
  var value = text.value;
  var format = "";
  if(regexMatch(value, format)) {
    var result = current.cluster.resolveText(value, subtype);
    if(result[0]) {
      if(dock.SUBTREE && fieldname) {
        current.updateBranch(fieldname, result[1]);
        parent.refresh();
      } else {
        refresh();
      }
    } else {
      var n = current.get(fieldname);
      var list = current.cluster.resolveList(n);
      text.value = list;
    }
  } else {
    var n = current.get(fieldname);
    var list = current.cluster.resolveList(n);
    text.value = list;
  }
  text.setAttribute("style", "");
}

function nodes_command(event) {
  var text = event.target;
  var value = text.value;
  var format = "";

  if(regexMatch(value, format)) {
    text.setAttribute("style", "");
  } else {
    text.setAttribute("style", "color: red;");
  }
}

function jobFilter() {
  var file = "workloadgroup.xul";
  var old = dock.dialogs.find(file);
  if(old) {
    if(!old.closed) {
      dock.displayError("Add workload grouping window is already open", window);
      return;
    }
  }
  var args = new Object();
  args['cluster'] = current;
  var d = new dock.cvDialog(file, args);
  d.parentWindow = window;
  d.onOk = function(groups) {
    if(groups.length) {
      filter = new Array();
      for(var i = 0; i < groups.length; i++) {
        filter.push([(i == 0) ? 0 : 1, '@JOB@', groups[i][0], false]);
      }
      deviceselect();
    }
  }
  dock.dialogs.show(d);  
}

function eliminateDuplicates(arr) {
  var i,
      len = arr.length,
      out = [],
      obj = {};
  for (i=0; i < len; i++) {
    obj[arr[i]] = 0;
  }
  for ( i in obj ) {
    out.push(i);
  }
  return out;
}
