/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function powerOff() {
  var nodes = allNodes();
  if(nodes.length > 0) {
    var cluster = current;
    if(cluster.hasOwnProperty('cluster')) cluster = cluster.cluster;
    var delay = cluster.setup.powerOffDelay;
    var args = new Object();
    args['question'] = "Are you sure you want to power off: " + cluster.resolveList(nodes);
    args['check'] = "Also closed " + deviceNamePlural;
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function(force) {
      dock.workerpool.getWorker(cluster.actualurl).call('cmdevice', 'powerOff', [nodes, delay, cluster.sessionid, force, true], function(code, data) {
        if(code == 0) {
          if(!data) {
            dock.displayError("Power off failed");
          }
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
    }
    dock.dialogs.showDialog(d);
  } else {
    dock.displayInfo("No (commited) nodes");
  }
}

function powerOn() {
  var nodes = allNodes();
  if(nodes.length > 0) {
    var cluster = current;
    if(cluster.hasOwnProperty('cluster')) cluster = cluster.cluster;
    var delay = cluster.setup.powerOnDelay;
    var args = new Object();
    args['question'] = "Are you sure you want to power on: " + cluster.resolveList(nodes);
    args['check'] = "Also closed " + deviceNamePlural;
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function(force) {
      dock.workerpool.getWorker(cluster.actualurl).call('cmdevice', 'powerOn', [nodes, delay, cluster.sessionid, force, true], function(code, data) {
        if(code == 0) {
          if(!data) {
            dock.displayError("Power on failed");
          }
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
    }
    dock.dialogs.showDialog(d);
  } else {
    dock.displayInfo("No (commited) nodes");
  }
}
function powerReset() {
  var nodes = allNodes();
  if(nodes.length > 0) {
    var cluster = current;
    if(cluster.hasOwnProperty('cluster')) cluster = cluster.cluster;
    var delay = cluster.setup.powerOffDelay;
    var args = new Object();
    args['question'] = "Are you sure you want to power reset: " + cluster.resolveList(nodes);
    args['check'] = "Also closed " + deviceNamePlural;
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function(force) {
      dock.workerpool.getWorker(cluster.actualurl).call('cmdevice', 'powerCycle', [nodes, delay, cluster.sessionid, force, true], function(code, data) {
        if(code == 0) {
          if(!data) {
            dock.displayError("Power reset failed");
          }
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
    }
    dock.dialogs.showDialog(d);
  } else {
    dock.displayInfo("No (commited) nodes");
  }
}

function osShutdown() {
  var nodes = allNodes();
  if(nodes.length > 0) {
    var cluster = current;
    if(cluster.hasOwnProperty('cluster')) cluster = cluster.cluster;
    var args = new Object();
    args['question'] = "Are you sure you want to shutdown: " + cluster.resolveList(nodes);
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      dock.workerpool.getWorker(cluster.actualurl).call('cmdevice', 'pshutdown', [nodes], function(code, data) {
        if(code == 0) {
          showPresult(nodes, data, "Shutdown");
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
    }
    dock.dialogs.showDialog(d);
  } else {
    dock.displayInfo("No (commited) nodes");
  }
}
function osRestart() {
  var nodes = allNodes();
  if(nodes.length > 0) {
    var cluster = current;
    if(cluster.hasOwnProperty('cluster')) cluster = cluster.cluster;
    var args = new Object();
    args['question'] = "Are you sure you want to restart: " + cluster.resolveList(nodes);
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      dock.workerpool.getWorker(cluster.actualurl).call('cmdevice', 'preboot', [nodes], function(code, data) {
        if(code == 0) {
          showPresult(nodes, data, "Reboot");
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
    }
    dock.dialogs.showDialog(d);
  } else {
    dock.displayInfo("No (commited) nodes");
  }
}

function syncNode() {
  var nodes = allNodes(true, false);
  if(nodes.length > 0) {
    var cluster = current;
    if(cluster.hasOwnProperty('cluster')) cluster = cluster.cluster;
    var args = new Object();
    args['question'] = "Are you sure you want update: " + cluster.resolveList(nodes);
    args['check'] = "Dry run";
    args['checked'] = true;
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function(dryrun) {
      dock.workerpool.getWorker(cluster.actualurl).call('cmprov', 'imageUpdate', [nodes, dryrun, cluster.sessionid], function(code, data) {
        if(code == 0) {
          showPresult(nodes, data, "Image synchronization");
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
    }
    dock.dialogs.showDialog(d);
  } else {
    dock.displayInfo("No (commited) nodes");
  }
}

function reinstallNode() {
  var nodes = allNodes(true, false);
  if(nodes.length > 0) {
    var cluster = current;
    if(cluster.hasOwnProperty('cluster')) cluster = cluster.cluster;
    var args = new Object();
    args['question'] = "Are you sure you want reinstall: " + cluster.resolveList(nodes);
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      for(var i = 0; i < nodes.length; i++) {
        var node = cluster.find(nodes[i]);
        node.set('nextBootInstallMode', 'FULL');
        if(node.status.status == 'UP') {
          node.commit(1);
        } else {
          node.commit(2);
        }
      }
    }
    dock.dialogs.showDialog(d);
  } else {
    dock.displayInfo("No (commited) nodes");
  }
}

function openDevices() {
  var nodes = allNodes(false);
  if(nodes.length > 0) {
    var cluster = current;
    if(cluster.hasOwnProperty('cluster')) cluster = cluster.cluster;
    var status = new dock.cvDevStatus();
    status.status = 'OPENING';
    dock.workerpool.getWorker(cluster.actualurl).call('cmdevice', 'setStatusForDeviceArray', [nodes, status], function(code, data) {
      if(code == 0) {
        showPresult(nodes, data, "Open");
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
  } else {
    dock.displayInfo("No (commited) nodes");
  }
}

function closeDevices() {
  var nodes = allNodes(false);
  if(nodes.length > 0) {
    var cluster = current;
    if(cluster.hasOwnProperty('cluster')) cluster = cluster.cluster;
    var status = new dock.cvDevStatus();
    status.status = 'CLOSED';
    dock.workerpool.getWorker(cluster.actualurl).call('cmdevice', 'setStatusForDeviceArray', [nodes, status], function(code, data) {
      if(code == 0) {
        showPresult(nodes, data, "Close");
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
  } else {
    dock.displayInfo("No (commited) nodes");
  }
}

function drain() {
  var nodes = allNodes(false);
  if(nodes.length > 0) {
    var cluster = current;
    if(cluster.hasOwnProperty('cluster')) cluster = cluster.cluster;
    dock.workerpool.getWorker(cluster.actualurl).call('cmjob', 'drainNodes', ['', '', nodes, 1, [], [], false], function(code, data) {
      if(code == 0) {
        var args = new Object();
        args['drain'] = true;
        args['drainresult'] = data;
        args['cluster'] = this.current;
        var d = new dock.cvDialog("drainresult.xul", args);
        dock.dialogs.showDialog(d);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
  } else {
    dock.displayInfo("No (commited) nodes");
  }
}

function undrain() {
  var nodes = allNodes(false);
  if(nodes.length > 0) {
    var cluster = current;
    if(cluster.hasOwnProperty('cluster')) cluster = cluster.cluster;
    dock.workerpool.getWorker(cluster.actualurl).call('cmjob', 'drainNodes', ['', '', nodes, 0, [], [], false], function(code, data) {
      if(code == 0) {
        var args = new Object();
        args['drain'] = true;
        args['drainresult'] = data;
        args['cluster'] = this.current;
        var d = new dock.cvDialog("drainresult.xul", args);
        dock.dialogs.showDialog(d);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
  } else {
    dock.displayInfo("No (commited) nodes");
  }
}

function terminateInstance() {
  var nodes = allNodes(true);
  if(nodes.length > 0) {
    var cluster = current;
    if(cluster.hasOwnProperty('cluster')) cluster = cluster.cluster;
    var args = new Object();
    args['question'] = "Are you sure you want to terminate: " + cluster.resolveList(nodes);
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      dock.workerpool.getWorker(cluster.actualurl).call('cmcloud', 'terminate', [nodes], function(code, data) {
        if(code == 0) {
          showPresult(nodes, data, "Terminate");
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
    }
    dock.dialogs.showDialog(d);
  } else {
    dock.displayInfo("No (commited) nodes");
  }
}
