/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function overviewValue(name, value, format) {
  if((value == null) || (typeof(value) == 'undefined') || (value.length == 0)) {
    value = overview[name];
    if(typeof(format) != 'undefined') {
      if(kisNaN(value) && (typeof(value) != 'string')) {
        value = 'N/A';
      } else {
        value = sprintf(format, value)
      }
    }
  } else if(typeof(format) != 'undefined') {
    value = sprintf(format, value)
  }
  var obj = document.getElementById(name);
  if(obj) {
    obj.setAttribute('value', value);
  } else {
    println('++ overviewValue, unable to find: ' + name);
  }
}

function overviewList(name, names, sep) {
  var value = '';
  for(var i = 0; i < names.length; i++) {
    if(value.length > 0) value += sep;
    value += overview[names[i]];
  }
  document.getElementById(name).setAttribute('value', value);
}

function overviewUptime(name) {
  var up = overview[name];
  var days = Math.floor(up / 86400); up -= days * 86400;
  var hours = Math.floor(up / 3600); up -= hours * 3600;
  var mins = Math.floor(up / 60);
  document.getElementById(name + "Days").setAttribute('value', days);
  document.getElementById(name + "Hours").setAttribute('value', hours);
  document.getElementById(name + "Minutes").setAttribute('value', mins);
}

function overviewDisks() {
  var obj = document.getElementById('disks');
  while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
  var data = overview.disks;
  for(var i = 0; i < data.length; i++) {
    obj.appendChild(diskRow(data[i]));
  }

  function diskRow(data) {
    var row = document.createElement('row');
    row.setAttribute("class", "staticdatarow");
    row.setAttribute("flex", "1");
    row.setAttribute("align", "center");
    row.setAttribute("height", "19px");
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.mountpoint);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    if((data.used < 0) || (data.free < 0)) {
      label.setAttribute("value", '0 B');
    } else {
      label.setAttribute("value", formatKMG(data.used) + 'B');
    }
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    if((data.used < 0) || (data.free < 0)) {
      label.setAttribute("value", '0 B');
    } else {
      label.setAttribute("value", formatKMG(data.free + data.used) + 'B');
    }
    row.appendChild(label);
    if((data.used < 0) || (data.free < 0)) {
      row.appendChild(createStatus2("10px", 0, 1));
    } else {
      row.appendChild(createStatus2("10px", data.used, data.free + data.used));
    }
    return row;
  }
}

function overviewJobs() {
  var obj = document.getElementById('jobs');
  while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
  var data = overview.jobs;
  for(var i = 0; i < data.length; i++) {
    obj.appendChild(jobRow(data[i]));
  }

  function jobRow(data) {
    var row = document.createElement('row');
    row.setAttribute("flex", "1");
    row.setAttribute("align", "center");
    row.setAttribute("height", "19px");
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.name);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.user);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", formatDHM(data.runtime));
    row.appendChild(label);
    return row;
  }
}

function overviewGenericPage(title, header, matrix) {
  var vbox = document.createElement('vbox');
  vbox.setAttribute('class', 'staticdata');
  var label = document.createElement('shadowlabel');
  label.setAttribute('value', title);
  label.setAttribute('height', '26px');
  label.setAttribute('class', 'staticdatabigheader');
  vbox.appendChild(label);
  var grid = document.createElement('grid');
  vbox.appendChild(grid);
  var columns = document.createElement('columns');
  var rows = document.createElement('rows');
  var row = document.createElement('row');
  for(var i = 0; i < header.length; i++) {
    var column = document.createElement('column');
    column.setAttribute('flex', 1);
    columns.appendChild(column);
    var label = document.createElement('shadowlabel');
    label.setAttribute('value', header[i]);
    label.setAttribute('class', 'staticdatatableheader');
    row.appendChild(label);
  }
  rows.appendChild(row);
  var line = document.createElement('hbox');
  line.setAttribute('class', 'staticdottedline');
  rows.appendChild(line);
  for(var i = 0; i < matrix.length; i++) {
    var row = document.createElement('row');
    for(var j = 0; j < matrix[i].length; j++) {
      var label = document.createElement('shadowlabel');
      label.setAttribute('value', matrix[i][j]);
      label.setAttribute('class', 'staticdatavaluesmall');
      row.appendChild(label);      
    }
    rows.appendChild(row);
  }
  grid.appendChild(columns);
  grid.appendChild(rows);
  return vbox;
}

function overviewWorkloads() {
  var obj = document.getElementById('workload');
  while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
  var data = overview.workload;
  var nr = 0;
  var nw = 0;
  for(var i = 0; i < data.length; i++) {
    nr += data[i].running;
    nw += data[i].queued;
    obj.appendChild(workloadRow(data[i]));
  }
  document.getElementById('jobsrunning').setAttribute('value', nr);
  document.getElementById('jobswaiting').setAttribute('value', nw);

  function workloadRow(data) {
    var row = document.createElement('row');
    row.setAttribute("flex", "1");
    row.setAttribute("align", "center");
    row.setAttribute("height", "19px");
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.name);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.running);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.queued);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.error);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.completed);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", formatDHMS(data.averageDuration));
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", formatDHMS(data.estimatedDelay));
    row.appendChild(label);
    return row;
  }
}

function overviewInterfaces() {
  var obj = document.getElementById('interfaces');
  while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
  var data = overview.interfaces;
  for(var i = 0; i < data.length; i++) {
    obj.appendChild(interfaceRow(data[i]));
  }

  function interfaceRow(data) {
    var row = document.createElement('row');
    row.setAttribute("flex", "1");
    row.setAttribute("align", "center");
    row.setAttribute("height", "19px");
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.name);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", formatKMG(data.rx) + 'B');
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", formatKMG(data.tx) + 'B');
    row.appendChild(label);
    return row;
  }
}

function createStatus2(height, used, total, percent) {
  var stack = document.createElement('stack');
  var status = document.createElement('hbox');
  status.setAttribute("flex", "1");
  status.setAttribute("height", height);
  var left = document.createElement('hbox');
  var right = document.createElement('hbox');
  if(total < 1e-4) total = 1;
  var size = 100 / total;
  left.setAttribute("flex", used * size);
  left.setAttribute("class", "leftstatus");
  right.setAttribute("flex", (total - used) * size);
  right.setAttribute("class", "rightstatus");
  if(used > 0) status.appendChild(left);
  if((total - used) > 0) status.appendChild(right);
  stack.appendChild(status);
  var mask = document.createElement('hbox');
  mask.setAttribute("flex", "1");
  mask.setAttribute("height", height);
  var img = document.createElement('image');
  img.setAttribute('src', 'chrome://cmgui/content/skin/gradient/blank' + height + 'l.png');
  img.setAttribute("height", height);
  img.setAttribute("style", 'padding: 0px; margin: 0px; max-height: ' + height);
  mask.appendChild(img);
  var spacer = document.createElement('image');
  spacer.setAttribute('flex', 1);
  mask.appendChild(spacer);
  var img = document.createElement('image');
  img.setAttribute('src', 'chrome://cmgui/content/skin/gradient/blank' + height + 'r.png');
  img.setAttribute("height", height);
  img.setAttribute("style", 'padding: 0px; margin: 0px; max-height: ' + height);
  mask.appendChild(img);
  stack.appendChild(mask);

  if(isundefined(percent, true)) {
    var h = (parseInt(height) - 2);  
    var lbox = document.createElement('hbox');
    lbox.setAttribute("flex", "1");
    lbox.setAttribute("align", "top");
    lbox.setAttribute("pack", "center");
    lbox.setAttribute("style", 'padding: 0px; margin: 0px; max-height: ' + h + 'px;');
    var label = document.createElement('label');
    label.setAttribute('value', sprintf("%.1f", 100. * used / total) + '%');
    label.setAttribute('style', 'padding: 0px; padding-top: 0px; margin: 0px; color: #efefef; font-size: ' + h + 'px; max-height: ' + h + 'px;');
    stack.appendChild(label);
    lbox.appendChild(label);
    stack.appendChild(lbox);
  }
  
  return stack;
}

function createStatus3(height, a, b, c) {
  var stack = document.createElement('stack');
  var status = document.createElement('hbox');
  status.setAttribute("flex", "1");
  status.setAttribute("height", height);
  var left = document.createElement('hbox');
  var middle = document.createElement('hbox');
  var right = document.createElement('hbox');
  var total = a + b + c;
  if(total < 1e-4) {
    total = 1;
    c = 1;
  }
  var size = 100 / total;
  left.setAttribute("flex", a * size);
  left.setAttribute("class", "leftstatus");
  middle.setAttribute("flex", b * size);
  middle.setAttribute("class", "middlestatus");
  right.setAttribute("flex", c * size);
  right.setAttribute("class", "rightstatus");
  if(a > 0) status.appendChild(left);
  if(b > 0) status.appendChild(middle);
  if(c > 0) status.appendChild(right);
  stack.appendChild(status);
  var mask = document.createElement('hbox');
  mask.setAttribute("flex", "1");
  mask.setAttribute("height", height);
  var img = document.createElement('image');
  img.setAttribute('src', './skin/gradient/blank' + height + 'l.png');
  mask.appendChild(img);
  var spacer = document.createElement('image');
  spacer.setAttribute('flex', 1);
  mask.appendChild(spacer);
  var img = document.createElement('image');
  img.setAttribute('src', './skin/gradient/blank' + height + 'r.png');
  mask.appendChild(img);
  stack.appendChild(mask);
  return stack;
}

function createStatus4(height, a, b, c, d) {
  var stack = document.createElement('stack');
  var status = document.createElement('hbox');
  status.setAttribute("flex", "1");
  status.setAttribute("height", height);
  var left = document.createElement('hbox');
  var middle = document.createElement('hbox');
  var other = document.createElement('hbox');
  var right = document.createElement('hbox');
  var total = a + b + c + d;
  if(total < 1e-4) {
    total = 1;
    d = 1;
  }
  var size = 100 / total;
  left.setAttribute("flex", a * size);
  left.setAttribute("class", "leftstatus");
  middle.setAttribute("flex", b * size);
  middle.setAttribute("class", "middlestatus");
  other.setAttribute("flex", c * size);
  other.setAttribute("class", "otherstatus");
  right.setAttribute("flex", d * size);
  right.setAttribute("class", "rightstatus");
  if(a > 0) status.appendChild(left);
  if(b > 0) status.appendChild(middle);
  if(c > 0) status.appendChild(other);
  if(d > 0) status.appendChild(right);
  stack.appendChild(status);
  var mask = document.createElement('hbox');
  mask.setAttribute("flex", "1");
  mask.setAttribute("height", height);
  var img = document.createElement('image');
  img.setAttribute('src', './skin/gradient/blank' + height + 'l.png');
  mask.appendChild(img);
  var spacer = document.createElement('image');
  spacer.setAttribute('flex', 1);
  mask.appendChild(spacer);
  var img = document.createElement('image');
  img.setAttribute('src', './skin/gradient/blank' + height + 'r.png');
  mask.appendChild(img);
  stack.appendChild(mask);
  return stack;
}

function overviewBar(name, used, total, unit, kmg) {  
  unit = isundefined(unit, '');
  kmg = isundefined(kmg, false);
  if(used < 0) used = 0;
  if((total < 0) && ((total > -1) || (total < -1e12))) total = Math.max(0, used);
  var obj = document.getElementById(name);
  obj.replaceChild(createStatus2("12px", used, Math.abs(total), total >= 0), obj.firstChild.nextSibling);
  obj = obj.lastChild;
  var usedkmg = used;
  if(kmg) {
    usedkmg = formatKMGselect(usedkmg, unit);
  } else if(unit == '%') {
    usedkmg = sprintf("%.2f", used) + unit;
  }
  obj.firstChild.setAttribute('value', usedkmg);
  if(total >= 0) {
    var totalkmg = total;
    if(kmg) {
      totalkmg = formatKMGselect(totalkmg, unit);
    } else {
      totalkmg += unit
    }
    obj.lastChild.setAttribute('value', totalkmg);
  }
}

function overviewUsage(name, user, system, idle) {
  var obj = document.getElementById(name);
  var other = (1 - user - system - idle);
  if((user == 0) && (system == 0) && (idle == 0)) other = 0;
  obj.replaceChild(createStatus4("12px", user, system, other, idle), obj.firstChild.nextSibling);
  obj = obj.lastChild;
  obj.firstChild.setAttribute('value', sprintf("%.2f", 100 * user) + '%');
  obj.childNodes[2].setAttribute('value', sprintf("%.2f", 100 * system) + '%');
  obj.childNodes[4].setAttribute('value', sprintf("%.2f", 100 * other) + '%');
  obj.childNodes[6].setAttribute('value', sprintf("%.2f", 100 * idle) + '%');
}

function overviewFans() {
  var obj = document.getElementById('fanspeed');
  while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
  var data = overview.fanSpeed;
  for(var i = 0; i < data.length; i++) {
    obj.appendChild(diskRow(i, data[i]));
  }

  function diskRow(i, data) {
    var row = document.createElement('row');
    row.setAttribute("class", "staticdatarow");
    row.setAttribute("flex", "1");
    row.setAttribute("align", "center");
    row.setAttribute("height", "19px");
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", i);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    if(kisNaN(data)) {
      label.setAttribute("value", 'no data');
    } else {
      label.setAttribute("value", data + ' RPM');
    }
    row.appendChild(label);
    return row;
  }
}

function overviewGPU() {
  var obj = document.getElementById('gpusage');
  while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
  var x = overview.cpuUtilGPU; // BUG in xulrunner ?
  var data = [x,
              overview.memUtilGPU,
              overview.memFreeGPU,
              overview.memUsedGPU,
              overview.FanSpeedPercGPU, // KDR: yes caps
              overview.tempGPU,
              overview.powerUsageGPU,
              overview.EccSBitGPU,
              overview.EccDBitGPU,
              overview.pciIndex];

  var n = 0;
  for(var j = 0; j < data.length; j++) {
    n = Math.max(n, data[j].length);
  }
  for(var i = 0; i < n; i++) {
    var x = new Array();
    for(var j = 0; j < data.length; j++) {
      if(data[j].length > i) {
        x.push(data[j][i]);
      } else {
        x.push(NaN);
      }
    }
    obj.appendChild(cmRow(i, x));
  }

  function cmRow(i, x) {
/*    x[1] = 37 * 1024 * 1024;
    x[2] = 219 * 1024 * 1024*/
    var row = document.createElement('row');
    row.setAttribute("class", "staticdatarow");
    row.setAttribute("flex", "1");
    row.setAttribute("align", "center");
    row.setAttribute("height", "19px");
    addLabel(row, 'GPU ' + i);
    addLabel(row, x[9]);
    addLabel(row, x[5], '%.2f', ' C');
    addLabel(row, x[4], '%.2f', '%');
    addLabel(row, x[0], '%.2f', '%');
    if(!kisNaN(x[1]) && !kisNaN(x[2])) {
      var hbox = document.createElement('hbox');
      hbox.setAttribute("align", "center");
      var label = document.createElement('label');
      label.setAttribute("class", 'staticdatavaluesmall');
      label.setAttribute("value", formatKMG(x[3]) + 'B');
      hbox.appendChild(label);
      var bar = createStatus2("10px", x[3], x[3] + x[2]);
      bar.setAttribute("flex", "1");
      hbox.appendChild(bar);
      var label = document.createElement('label');
      label.setAttribute("class", 'staticdatavaluesmall');
      label.setAttribute("value", formatKMG(x[2] + x[3]) + 'B');
      hbox.appendChild(label);
      row.appendChild(hbox);
    } else {
      addLabel(row, x[1], '%.2f', '%');
    }
    addPlaceHolder(row, 'PowerModeGPU:' + i);
    addPlaceHolder(row, 'PowerLimitGPU:' + i);
    addLabel(row, x[6], '%.2f', 'W');
    addPlaceHolder(row, 'DisplayModeGPU:' + i);
    addPlaceHolder(row, 'PersistenceModeGPU:' + i);
    addPlaceHolder(row, 'ComputeModeGPU:' + i);
    addPlaceHolder(row, 'EccModeGPU:' + i);
    addLabel(row, x[7], '%.2f');
    addLabel(row, x[8], '%.2f');

    function addPlaceHolder(row, id) {
      var label = document.createElement('label');
      label.setAttribute("class", 'staticdatavaluesmall');
      label.setAttribute("id", id);
      row.appendChild(label);
    }
    function addLabel(row, x, format, unit) {
      format = isundefined(format, '');
      unit = isundefined(unit, '');
      var label = document.createElement('label');
      label.setAttribute("class", 'staticdatavaluesmall');
      if(isNumeric(x) && kisNaN(x)) {
        label.setAttribute("value", 'N/A');
      } else if(format != '') {
        label.setAttribute("value", sprintf(format, x) + unit);
      } else {
        label.setAttribute("value", x + unit);
      }
      row.appendChild(label);
    }
    
    return row;
  }
}

function overviewLines(name, value) {
  if((value == null) || (typeof(value) == 'undefined') || (value.length == 0)) {
    value = overview[name];
  }
  var list = document.getElementById(name);
  while(list.firstChild) list.removeChild(list.firstChild);
  for(var i = 0; i < value.length; i++) {
    var label = document.createElement('shadowlabel');
    label.setAttribute('class', 'staticdatavalue');
    label.setAttribute('value', value[i]);
    list.appendChild(label);
  }
}
