/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function resolveName(obj, cluster) {
  if(isObject(obj) && obj.hasOwnProperty('object')) {
    cluster = obj.cluster;
    obj = obj.object;
  }
  if((obj != null) && isObject(obj) && obj.hasOwnProperty('baseType')) {
    switch(obj.baseType) {
      case('Device') : return obj.hostname;
      case('CloudNode') : return obj.hostname;
      case('MasterNode'): return obj.hostname;
      case('MonConf'): return cluster.getMonConfName(obj.uniqueKey);
      case('MonMetricConf'): {
          var name = cluster.find(obj.metric);
          if(name) {
            name = resolveName(name, cluster);
            var param = obj.metricParam;
            if(param.length > 0) {
              name += ':' + param;
            }
            return name;
          } else {
            return '';
          }
        }
      case('MonHealthConf'): {
          var name = cluster.find(obj.healthcheck);
          if(name) {
            name = resolveName(name, cluster);
            var param = obj.healthcheckParam;
            if(param.length > 0) {
              name += ':' + param;
            }
            return name;
          } else {
            return '';
          }
        }
    }
    if(obj.hasOwnProperty('name')) return obj.name;
    if(obj.hasOwnProperty('userID')) return obj.userID;
    if(obj.hasOwnProperty('group')) return obj.group;
    if(obj.hasOwnProperty('mountpoint')) return obj.mountpoint;
    if(obj.hasOwnProperty('uniqueKey')) return obj.uniqueKey;
  }
  return 'Unknown';
}

function inverseResolveName(obj, cluster, name) {
  if(isObject(obj) && obj.hasOwnProperty('object')) {
    cluster = obj.cluster;
    obj = obj.object;
    var rname = resolveName(obj, cluster);
    if(rname == name) return 1;
    if(obj.uniqueKey && (name == obj.uniqueKey)) return 2;
  }
  return -1;
}

function baseChildTypeCheck(obj, what) {
  if(isObject(obj) && obj.hasOwnProperty('baseType')) {
    if(isArray(what)) {
      for(var i = 0; i < what.length; i++) {
        var ret = baseChildTypeCheck(obj, what[i]);
        if(ret) {
          return true;
        }
      }
      return false;
    } else {
      if(what == obj.baseType) return true;
      if(obj.hasOwnProperty('childType')) {
        if(what == obj.childType) return true;
        if(obj.baseType == 'Device') {
          if((what == 'Switch') && ((obj.childType == 'EthernetSwitch') || (obj.childType == 'MyrinetSwitch') || (obj.childType == 'IBSwitch'))) return true;
          if((what == 'Node') && ((obj.childType == 'MasterNode') || (obj.childType == 'PhysicalNode') || (obj.childType == 'CloudNode') || (obj.childType == 'MICNode') || (obj.childType == 'VirtualSMPNode') || (obj.childType == 'VirtualNode'))) return true;
          if((what == 'SlaveNode') && ((obj.childType == 'PhysicalNode') || (obj.childType == 'CloudNode') || (obj.childType == 'VirtualSMPNode') || (obj.childType == 'VirtualNode'))) return true;
        }
      }
    }    
  }
  return false;
}
