/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var KEY_ENTER = 13;
var KEY_SPACE = 32;
var KEY_ESCAPE = 27;
var KEY_TAB = 9;

/* autorefresh currently only works for
    - resolve
    - text
    - check
    - number
    - enum
*/

function setfocus(what, subscript) {
  subscript = isundefined(subscript, '');
  var obj = document.getElementById(what + subscript);
  if(obj) obj.focus();
}

function autorefresh(what, subscript) {
  subscript = isundefined(subscript, '');
  var obj = document.getElementById(what + subscript);
  obj.setAttribute('autorefresh', true);
}

function extraeval(what, extraeval, subscript) {
  subscript = isundefined(subscript, '');
  var obj = document.getElementById(what + subscript);
  obj.setAttribute('extraeval', extraeval);
}

function specialvalues(what, specialvalues, subscript) {
  subscript = isundefined(subscript, '');
  var obj = document.getElementById(what + subscript);
  obj.setAttribute('specialvalues', specialvalues.toSource());
}

function displayfunction(what, displayfunction, subscript) {
  subscript = isundefined(subscript, '');
  var obj = document.getElementById(what + subscript);
  obj.setAttribute('displayfunction', displayfunction);
}

function parsefunction(what, parsefunction, subscript) {
  subscript = isundefined(subscript, '');
  var obj = document.getElementById(what + subscript);
  obj.setAttribute('parsefunction', parsefunction);
}

function value2human(value, specialvalues) {
  eval('specialvalues = ' + specialvalues);
  for(var i = 0; i < specialvalues.length; i++) {
    if(specialvalues[i][0] == value) return specialvalues[i][1];
  }
  return value;
}

function human2value(value, specialvalues) {
  eval('specialvalues = ' + specialvalues);
  for(var i = 0; i < specialvalues.length; i++) {
    if(specialvalues[i][1] == value) return specialvalues[i][0];
  }
  return value;
}

function textObjValue(obj, value) {
  var specialvalues = obj.getAttribute('specialvalues');
  if(specialvalues.length > 0) value = value2human(value, specialvalues);
  var displayfunction = obj.getAttribute('displayfunction');
  if(displayfunction.length > 0) value = eval(displayfunction);
  obj.value = value;
}

/**
 * wrapper around text() for text input boxes for floats.
 *
 * this will initialize the textbox with:
 *  - rounding
 *  - regex for floats
 */
function text_float(what) {
    // two conversion functions that allow to show 0.12345 as 0.12, the original value is stored in actual_value,
    //  so this value can be displayed during editing of the textbox.
    var roundTwoDecimals = function () {
          this.actual_value = this.value;
          this.value = parseFloat(this.value).toFixed(2);
        },
        unroundDecimals = function () {
          this.value = this.actual_value;
        };

    // use the existing text() function to construct input element etc. (this function doesn't have a return value,
    // just in case that changes, we'll keep it.)
    var ret = text({
      'what'            : what,
      'format'          : '^0*1(\\.0+)?$|^0+(\\.[0-9]+)?$',
      'onFocusFunction' : unroundDecimals,
      'onBlurFunction'  : roundTwoDecimals
    });
    var element = document.getElementById(what);
    if (element) {
        roundTwoDecimals.bind(element)();
    }
    return ret;
}

var textEventListeners = {};
function text(what, format, subscript, nodups, dupnull, newKey, elementNodeId, currentObject) {
  var onCommandFunction, onFocusFunction, onBlurFunction, keyUpIsChange; // only to be set with object syntax
  if(typeof what === 'object') {
    var params = what;
    what = params.what || undefined;
    format = params.format || undefined;
    subscript = params.subscript || undefined;
    nodups = params.nodups || undefined;
    dupnull = params.dupnull || undefined;
    newKey = params.newKey || undefined;
    elementNodeId = params.elementNodeId || undefined;
    currentObject = params.currentObject || undefined;
    onCommandFunction = params.onCommandFunction || undefined;
    onFocusFunction = params.onFocusFunction || undefined;
    onBlurFunction = params.onBlurFunction || undefined;
    keyUpIsChange = params.keyUpIsChange || false;
  }
  currentObject = isundefined(currentObject, current);
  format = isundefined(format, false);
  subscript = isundefined(subscript, '');
  nodups = isundefined(nodups, false);
  dupnull = isundefined(dupnull, '');
  newKey = isundefined(newKey, 0);
  var i = what.indexOf('_');
  var index = -1;
  var name;
  if((i > 0) && isNumeric(what.substr(i + 1))) {
    index = parseInt(what.substr(i + 1));
    name = what.substr(0, i);
  } else {
    name = what;
  }
  var obj;
  var value;
  var subscriptList = subscript.split('|');

  if(elementNodeId===undefined) {
    if(subscriptList.length == 2) {
      elementNodeId = what + subscriptList[1];
    } else {
      elementNodeId = what + subscript;
    }
  }

  if(subscriptList.length == 2) {
    if(endswith(name, subscriptList[0])) {
      value = currentObject.get(name.substr(0, name.length - subscriptList[0].length));
      value = new dock.cvObject(currentObject.cluster, value);
      value = value.get(subscriptList[0]);
    } else {
      println('Error in subscriptList: ' + name + ' ' + subscript);
      return;
    }
  } else {
    value = currentObject.get(name);
  }


  var hash = elementNodeId + '-' + new Date().getTime() + '-' + Math.random();
  obj = document.getElementById(elementNodeId);
  if(!obj) {
    println('!!! Error not found: ' + elementNodeId);
    return;
  }

  if (onCommandFunction) obj.addEventListener("command", onCommandFunction, false);
  if (onFocusFunction) obj.addEventListener("focus", onFocusFunction, false);
  if (onBlurFunction) obj.addEventListener("blur", onBlurFunction, false);

  if(isArray(value)) {
    if(index < value.length) {
      value = value[index];
    } else {
      println('!!! Error value out of bounds: ' + what + subscript + ' ' + value + ' ' + index);
    }
  }
  if(subscript) {
    if(subscriptList.length == 2) {
      value = value[subscriptList[1]];
    } else {
      value = value[subscript];
    }
  }

  textObjValue(obj, value);

  obj.setAttribute('what', what);
  obj.setAttribute('subscript', subscript);
  obj.setAttribute('nodups', nodups);
  obj.setAttribute('dupnull', dupnull);
  obj.setAttribute('newKey', newKey);

  if (typeof obj.hash !== 'undefined') {
    obj.removeEventListener("command", textEventListeners[obj.hash].textbox_command, true);
    obj.removeEventListener("change", textEventListeners[obj.hash].textbox_change, true);
    obj.removeEventListener("keyup", textEventListeners[obj.hash].textbox_change, true);
    obj.removeEventListener("keypress", textEventListeners[obj.hash].textbox_keypress, true);
    delete textEventListeners[obj.hash];
  }

  textEventListeners[hash] = {
    textbox_command: textbox_command.bind(undefined, currentObject),
    textbox_change: textbox_change.bind(undefined, currentObject),
    textbox_keypress: textbox_keypress.bind(undefined, currentObject)
  };

  if(format) {
    obj.setAttribute('value', obj.value);
    if(obj.getAttribute('type') != 'password') obj.setAttribute('type', 'timed');
    obj.setAttribute('format', format);
    obj.setAttribute('timeout', '100');
    obj.addEventListener("command", textEventListeners[hash].textbox_command, true);
  }
  // obj.addEventListener("blur", textbox_blur, true); /* FIX xul, blur is called after refresh of the next object */
  obj.addEventListener("change", textEventListeners[hash].textbox_change, true);
  if (keyUpIsChange) {
      obj.addEventListener("keyup", textEventListeners[hash].textbox_change, true);
  }
  obj.addEventListener("keypress", textEventListeners[hash].textbox_keypress, true);
  obj.hash = hash;
}

function partial(what, sep, format, subscript, nodups, dupnull, newKey, varlen, digitadd) {
  format = isundefined(format, false);
  subscript = isundefined(subscript, '');
  nodups = isundefined(nodups, false);
  dupnull = isundefined(dupnull, '');
  varlen = isundefined(varlen, '');
  digitadd = isundefined(digitadd, '');
  newKey = isundefined(newKey, 0);
  var i = what.indexOf('_');
  var index = -1;
  var name;
  if(i > 0) {
    index = parseInt(what.substr(i + 1));
    name = what.substr(0, i);
  } else {
    name = what;
  }
  var value = current.get(name);
  if(isArray(value)) value = value[index];
//   if(i > 0) value = value[index];
  if(subscript) {
    value = value[subscript];
  }
  value = value.split(sep);
  var partlist;
  if(varlen.length > 0) {
    partlist = partialfields(what + subscript, value.length);
  }
  for(var i = 0; i < value.length; i++) {
    var obj = document.getElementById(what + subscript + "-" + i);
    if(!obj) {
      if(isArray(partlist)) {
        obj = partlist[i];
      } else {
        println("ERROR: not found: " + what + subscript + "-" + i);
        continue;
      }
    }
    obj.value = value[i];
    obj.setAttribute('what', what);
    obj.setAttribute('subscript', subscript);
    obj.setAttribute('separator', sep);
    obj.setAttribute('nodups', nodups);
    obj.setAttribute('dupnull', dupnull);
    obj.setAttribute('newKey', newKey);
    obj.setAttribute('varlen', varlen);
    obj.setAttribute('digitadd', digitadd);
    obj.setAttribute('index', i);
    if(!!format) {
      obj.setAttribute('value', value[i]);
      obj.setAttribute('type', 'timed');
      obj.setAttribute('timeout', '100');
      if(isArray(format)) {
        obj.setAttribute('format', format[i]);
      } else {
        obj.setAttribute('format', format);
      }
      obj.addEventListener("command", separated_textbox_command, true);
    }
    obj.addEventListener("change", separated_textbox_change, true);
    obj.addEventListener("keypress", separated_textbox_keypress, true);
  }
  var plus = document.getElementById(what + "-plus");
  if(!!plus) plus.setAttribute('hidden', varlen.length == 0);
  var minus = document.getElementById(what + "-minus");
  if(!!minus) minus.setAttribute('hidden', varlen.length == 0);
}

function number(what, min, max, unit, blank, subscript) {
  unit = isundefined(unit, '');
  blank = isundefined(blank, (min - 1));
  subscript = isundefined(subscript, '');
  var i = what.indexOf('_');
  var index = -1;
  var name;
  if(i > 0) {
    index = parseInt(what.substr(i + 1));
    name = what.substr(0, i);
  } else {
    name = what;
  }
  var obj = document.getElementById(what + subscript);
  if(!obj) println("[genericsettings::number] not found: " + what + subscript);
  var popup = obj.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  for(var i = min; i <= max; i++) {
    var item = document.createElement("menuitem");
    if(i == blank) {
      item.setAttribute('label', '');
    } else {
      item.setAttribute('label', i + unit);
    }
    item.setAttribute('number', i);
    popup.appendChild(item);
  }
  var found = current.get(name);
  if(isArray(found)) found = found[index];
  if(subscript) {
    found = found[subscript];
  }
  found = parseInt(found);
  if((found >= min) && (found <= max)) {
    found = Math.max(min, Math.min(max, parseInt(found)));
    obj.selectedIndex = (found - min);
  } else {
    var item = document.createElement("menuitem");
    item.setAttribute('label', i + unit + ' *');
    item.setAttribute('number', i);
    popup.appendChild(item);
    obj.selectedIndex = popup.childNodes.length - 1;
    obj.setAttribute('tooltiptext', 'The current value is not allowed');
  }
  obj.setAttribute('what', what);
  obj.setAttribute('subscript', subscript);
  popup.addEventListener("command", number_command, true);
}

var kenumEventListeners = {};
function kenum(what, list, subscript, elementNodeId, currentObject, onCommandFunction) {
  if(typeof what === 'object') {
    var params = what;
    what = params.what || undefined;
    list = params.list || undefined;
    subscript = params.subscript || undefined;
    elementNodeId = params.elementNodeId || undefined;
    currentObject = params.currentObject || undefined;
    onCommandFunction = params.onCommandFunction || undefined;
  }
  currentObject = isundefined(currentObject, current);
  subscript = isundefined(subscript, '');
  var subscriptList = subscript.split('|');
 if(elementNodeId===undefined) {
    if(subscriptList.length == 2) {
      elementNodeId = what + subscriptList[1];
    } else {
      elementNodeId = what + subscript;
    }
  }

  var hash = elementNodeId + '-' + new Date().getTime() + '-' + Math.random();
  var obj = document.getElementById(elementNodeId);
  if(!obj) {
    println("!!! ERROR !!!: not found: " + what + subscript);
    return;
  }
  var popup = obj.firstChild;
  if(!popup) {
    println("!!! ERROR !!!: " + what + subscript + " : has no menupopup");
    return;
  }
  while(popup.firstChild) popup.removeChild(popup.firstChild);
//   var i = what.indexOf('_');
//   var index = -1;
//   var name;
//   if(i > 0) {
//     index = parseInt(what.substr(i + 1));
//     name = what.substr(0, i);
//   } else {
//     name = what;
//   }
//   var find = currentObject.get(name);

  var i = what.indexOf('_');
  var index = -1;
  var name;
  if((i > 0) && isNumeric(what.substr(i + 1))) {
    index = parseInt(what.substr(i + 1));
    name = what.substr(0, i);
  } else {
    name = what;
  }
  var obj;
  var find;

  if(subscriptList.length == 2) {
    if(endswith(name, subscriptList[0])) {
      find = currentObject.get(name.substr(0, name.length - subscriptList[0].length));
      find = new dock.cvObject(currentObject.cluster, find);
      find = value.get(subscriptList[0]);
      obj = document.getElementById(elementNodeId);
    } else {
      println('Error in subscriptList: ' + name + ' ' + subscript);
      return;
    }
  } else {
    find = currentObject.get(name);
    obj = document.getElementById(elementNodeId);
  }

  if (onCommandFunction) {
    obj.addEventListener("command", onCommandFunction, false);
  }

  if(isArray(find)) find = find[index];
  if(subscript) {
    find = find[subscript];
  }

  if (typeof popup.hash !== 'undefined') {
    popup.removeEventListener("command", kenumEventListeners[popup.hash].enum_command, true);
    delete kenumEventListeners[popup.hash];
  }

  kenumEventListeners[hash] = {
    enum_command: enum_command.bind(undefined, currentObject),
  };

  // KDR: hack to display at least something
  if(list === false) list = [find];

  var found = -1;
  var val = '';
  for(var i = 0; i < list.length; i++) {
    var item = document.createElement("menuitem");
    if(isArray(list[i])) {
      item.setAttribute('key', list[i][0]);
      item.setAttribute('value', list[i][0]);
      item.setAttribute('label', list[i][1]);
      if(typeof(find) == 'string') {
        if(string_case_compare(find, list[i][0])) found = i;
      } else {
        if(find == list[i][0]) found = i;
      }
    } else {
      item.setAttribute('key', list[i]);
      item.setAttribute('value', list[i]);
      item.setAttribute('label', list[i]);
      if(typeof(find) == 'string') {
        if(string_case_compare(find, list[i])) found = i;
      } else {
        if(find == list[i]) found = i;
      }
    }
    popup.appendChild(item);
  }
  if(found >= 0) {
    obj.selectedIndex = found;
  } else {
    obj.setAttribute('label', val);
  }
  obj.setAttribute('what', what);
  obj.setAttribute('subscript', subscript);
  popup.addEventListener("command", kenumEventListeners[hash].enum_command, true);
  popup.hash = hash;
}

var elementOfEventListeners = {};
function elementOf(what, list) {
  var hash = what + '-' + new Date().getTime() + '-' + Math.random();
  var obj = document.getElementById(what);
  var popup = obj.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  var find = current.get(what);
  var val = '';

  if (typeof popup.hash !== 'undefined') {
    popup.removeEventListener("command", elementOfEventListeners[popup.hash].enum_command, true);
    delete elementOfEventListeners[popup.hash];
  }

  elementOfEventListeners[hash] = {
    enum_command: enum_command.bind(undefined, current),
  };

  list = current.get(list);
  for(var i = 0; i < list.length; i++) {
    var item = document.createElement("menuitem");
    item.setAttribute('key', list[i].uniqueKey);
    var x = new dock.cvObject(current.cluster, list[i]);
    var x = x.resolveName();
    item.setAttribute('label', x);
    popup.appendChild(item);
    if(list[i].uniqueKey == find) val = x;
  }
  if(list.length == 0) {
    var item = document.createElement("menuitem");
    item.setAttribute('key', 0);
    item.setAttribute('label', '');
    popup.appendChild(item);
  }
  obj.setAttribute('label', val);
  obj.setAttribute('what', what);
  popup.addEventListener("command", elementOfEventListeners[hash].enum_command, true);
  popup.hash = hash;
}

function resolve(what, type, subscript, blank, specialChar, excludeFunction, displayFunction, getCurrentFunction) {
  subscript = isundefined(subscript, '');
  specialChar = isundefined(specialChar, '');
  excludeFunction = isundefined(excludeFunction, false);
  displayFunction = isundefined(displayFunction, false);
  getCurrentFunction = isundefined(getCurrentFunction, '');
  var i = what.indexOf('_');
  var index = -1;
  var name;
  if(i > 0) {
    index = parseInt(what.substr(i + 1));
    name = what.substr(0, i);
  } else {
    name = what;
  }
  var obj;
  var value = current.get(name);
  if(specialChar.length > 0) {
    value = parseInt(value.substr(specialChar.length, value.length - specialChar.length * 2));
    obj = document.getElementById(what + type);
  } else {
    if(isArray(value)) value = value[index];
    if(subscript) {
      value = value[subscript];
    }
    obj = document.getElementById(what + subscript);
  }
  if(type.indexOf('@') > 0) {
    if(!current.cluster.subFind(type, value)) value = 0;
  } else {
    if(!current.cluster.find(value)) value = 0;
  }

  var popup = obj.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  if(blank || (value == 0)) {
    var item = document.createElement("menuitem");
    item.setAttribute('key', 0);
    if(typeof(blank) == 'string') {
      item.setAttribute('label', blank);
    } else {
      item.setAttribute('label', '');
    }
    popup.appendChild(item);
    if(value == 0) obj.selectedIndex = 0;
  }
  if(excludeFunction && (typeof(excludeFunction) == 'string')) eval('excludeFunction = ' + excludeFunction);
  if(displayFunction && (typeof(displayFunction) == 'string')) eval('displayFunction = ' + displayFunction);
  var found = current.cluster.find(type);
  for(var i = 0; i < found.length; i++) {
    if(excludeFunction && excludeFunction(found[i])) continue;
    var item = document.createElement("menuitem");
    item.setAttribute('key', found[i].object.uniqueKey);
    if(displayFunction) {
      item.setAttribute('label', displayFunction(found[i]));
    } else {
      item.setAttribute('label', found[i].resolveName());
    }
    popup.appendChild(item);
    if(value == found[i].object.uniqueKey) obj.selectedIndex = popup.childNodes.length - 1;
  }
  obj.setAttribute('what', what);
  obj.setAttribute('subscript', subscript);
  obj.setAttribute('specialChar', specialChar);
  obj.setAttribute('getCurrentFunction', getCurrentFunction);
  obj.setAttribute('currentKey', current.get('uniqueKey'));
  popup.addEventListener("command", resolve_command, true);
}

function conditional_resolve(what, type, func, good) {
  var i = what.indexOf('_');
  var index = -1;
  var name;
  if(i > 0) {
    index = parseInt(what.substr(i + 1));
    name = what.substr(0, i);
  } else {
    name = what;
  }
  var obj = document.getElementById(what);
  var popup = obj.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  var found = current.cluster.find(type);
  for(var i = 0; i < found.length; i++) {
    if(found[i].object[func] == good) {
      var item = document.createElement("menuitem");
      item.setAttribute('key', found[i].object.uniqueKey);
      item.setAttribute('label', found[i].resolveName());
      popup.appendChild(item);
    }
  }
  found = current.get(name);
  if(isArray(found)) found = found[index];
  if(found == 0) {
    var item = document.createElement("menuitem");
    item.setAttribute('key', 0);
    item.setAttribute('label', '');
    popup.appendChild(item);
    found = '';
  } else {
    found = current.cluster.find(found);
    if(isObject(found)) {
      found = found.resolveName();
    } else {
      println("Not found " + type + " " + current.get(name) + " (" + name + ")");
      found = '';
    }
  }
  obj.setAttribute('label', found);
  obj.setAttribute('what', what);
  popup.addEventListener("command", resolve_command, true);
}

function lookup(what, service, call) {
  var i = what.indexOf('_');
  var index = -1;
  var name;
  if(i > 0) {
    index = parseInt(what.substr(i + 1));
    name = what.substr(0, i);
  } else {
    name = what;
  }
  var obj = document.getElementById(what);
  var popup = obj.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  dock.workerpool.getWorker(this.current.cluster.actualurl).call(service, call, current.object, function(code, data) {
      if(code == 0) {
        for(var i = 0; i < data.length; i++) {
          var item = document.createElement("menuitem");
          item.setAttribute('label', data[i]);
          popup.appendChild(item);
        }
        var value = this.current.get(name);
        if(data.length == 0) {
          var item = document.createElement("menuitem");
          item.setAttribute('label', 'none');
          popup.appendChild(item);
          value = 'none';
        } else {
          if(data.find(value) < 0) {
            value += '(not found)';
            var item = document.createElement("menuitem");
            item.setAttribute('label', value);
            popup.appendChild(item);
          }
        }
        obj.setAttribute('label', value);
        obj.setAttribute('what', what);
        popup.addEventListener("command", lookup_command, true);
      } else {
        dock.jsonCallError(this, code, data);
      }
      this.current.extraComplete++;
    }, current);
}

var checkEventListeners = {};
function check(what, parent, subscript, elementNodeId, currentObject) {
  if(typeof what === 'object') {
    var params = what;
    what = params.what || undefined;
    parent = params.parent || undefined;
    subscript = params.subscript || undefined;
    elementNodeId = params.elementNodeId || undefined;
    currentObject = params.currentObject || undefined;
  }
  currentObject = isundefined(currentObject, current);
  parent = isundefined(parent, false);
  subscript = isundefined(subscript, '');

  if(elementNodeId===undefined) {
    elementNodeId = what + subscript;
  }
  var hash = elementNodeId + '-' + new Date().getTime() + '-' + Math.random();
  var obj = document.getElementById(elementNodeId);
  var value = currentObject.get(what);
  if(subscript) {
    value = value[subscript];
  }
  if(obj) {
    obj.setAttribute('checked', value);
    obj.setAttribute('parentChecked', parent);
    obj.setAttribute('what', what);

    if (typeof obj.hash !== 'undefined') {
      obj.removeEventListener("custom-change", checkEventListeners[obj.hash]['custom-change'], true);
      delete checkEventListeners[obj.hash];
    }

    checkEventListeners[hash] = {
      'custom-change': check_command.bind(undefined, currentObject)
    }

    obj.addEventListener("custom-change", checkEventListeners[hash]['custom-change'], true);
    obj.setAttribute('subscript', subscript);

    obj.hash = hash;
  } else {
    println('unable to find check: ' + what + subscript + ' elementNodeId ' + elementNodeId);
  }
}

function radio(what, list, subscript) {
  subscript = isundefined(subscript, '');
  var obj = document.getElementById(what + subscript);
  var i = what.indexOf('_');
  var index = -1;
  var name;
  if(i > 0) {
    index = parseInt(what.substr(i + 1));
    name = what.substr(0, i);
  } else {
    name = what;
  }
  var find = current.get(name);
  if(isArray(find)) find = find[index];
  if(subscript) {
    find = find[subscript];
  }
  var found = -1;
  var val = '';
  for(var i = 0; i < list.length; i++) {
    var hbox = document.createElement("hbox");
    hbox.setAttribute('align', 'center');
    var item = document.createElement("image");
    item.setAttribute('class', 'radioimage');
    var label = document.createElement("label");
    if(isArray(list[i])) {
      if(typeof(list[i][0]) == 'boolean') {
        hbox.setAttribute('key', 'boolean:' + list[i][0]);
      } else {
        hbox.setAttribute('key', list[i][0]);
      }
      label.setAttribute('value', list[i][1]);
      if(typeof(find) == 'string') {
        if(string_case_compare(find, list[i][0])) found = i;
      } else {
        if(find == list[i][0]) found = i;
      }
    } else {
      hbox.setAttribute('key', list[i]);
      label.setAttribute('value', list[i]);
      if(typeof(find) == 'string') {
        if(string_case_compare(find, list[i])) found = i;
      } else {
        if(find == list[i]) found = i;
      }
    }
    hbox.appendChild(item);
    hbox.appendChild(label);
    obj.appendChild(hbox);
    if(found == i) {
      item.setAttribute('on', true);
    }
    hbox.addEventListener("click", radioimage_command, true);
//     item.addEventListener("command", radio_command, true);
  }
  obj.setAttribute('what', what);
  obj.setAttribute('subscript', subscript);
}

function textbox_keypress(currentObject, event) {
  currentObject = isundefined(currentObject, current);
  var text = event.target;
  var value = text.value;
  var format = text.getAttribute('format');
  var what = text.getAttribute('what');
  var subscript = text.getAttribute('subscript');
  var nodups = eval(text.getAttribute('nodups'));
  var dupnull = text.getAttribute('dupnull');
  var newKey = text.getAttribute('newKey');
  var specialvalues = text.getAttribute('specialvalues');
  var parsefunction = text.getAttribute('parsefunction');
  var displayfunction = text.getAttribute('displayfunction');

  var i = what.indexOf('_');

  var extraeval = text.getAttribute('extraeval');
  var old;
  switch(event.keyCode) {
    case KEY_ENTER:
      if(text.getAttribute('class') == "textarea") break;
      if(functionRegexMatch(value, format)) {
        if(i > 0) {
          var index = parseInt(what.substr(i + 1));
          var name = what.substr(0, i);
          var list, listParent, parentName;
          var subscriptList = subscript.split('|');
          if(subscriptList.length == 2) {
            if(endswith(name, subscriptList[0])) {
              parentName = name.substr(0, name.length - subscriptList[0].length)
              listParent = currentObject.get(parentName);
              listParent = new dock.cvObject(currentObject.cluster, listParent);
              list = listParent.get(subscriptList[0]);
            } else {
              println('Error in subscriptList: ' + name + ' ' + subscript);
              return;
            }
          } else {
            list = currentObject.get(name);
          }
          if(index > list.length) {
            println("Attemp to write past the last element of: " + name);
            println("  length = " + list.length + ", index = " + index);
          } else {
            if(subscript) {
              if(listParent) listParent.modified = true;
              list[index].modified = true;
              if(specialvalues.length > 0) value = human2value(value, specialvalues);
              if(parsefunction.length > 0) value = eval(parsefunction);
              if(subscriptList.length == 2) {
                list[index][subscriptList[1]] = value;
              } else {
                list[index][subscript] = value;
              }
            } else {
              if(specialvalues.length > 0) value = human2value(value, specialvalues);
              if(parsefunction.length > 0) value = eval(parsefunction);
              list[index] = value;
            }
            if(listParent) {
              listParent.set(subscriptList[0], list);
              currentObject.set('modified', true);
//               currentObject.set(parentName, listParent);
            } else {
              if(extraeval.length) old = currentObject.get(name);
              currentObject.set(name, list);
            }
            if(eval(text.getAttribute('autorefresh'))) parent.refresh();
          }
        } else {
          if(subscript) {
            var one = currentObject.get(what);
            one.modified = true;
            one[subscript] = value;
            //currentObject.set(what, one);
            if(eval(text.getAttribute('autorefresh'))) parent.refresh();
          } else {
            var dupkey = 0;
            if(nodups) dupkey = currentObject.cluster.getKeyFor(currentObject.relevantBaseType(), value, what, newKey);
            if(nodups && (value != dupnull) && (dupkey != currentObject.get('uniqueKey') && (dupkey != 0))) {
              textObjValue(text, currentObject.get(what));
              dock.displayError("A " + currentObject.relevantBaseType() + " with that " + what + " already exists");
            } else {
              if(specialvalues.length > 0) value = human2value(value, specialvalues);
              if(parsefunction.length > 0) value = eval(parsefunction);
              if(extraeval.length) old = currentObject.get(what);
              currentObject.set(what, value);
              if(eval(text.getAttribute('autorefresh'))) parent.refresh();
            }
          }
        }
      } else {
        if(i > 0) {
          var index = parseInt(what.substr(i + 1));
          var name = what.substr(0, i);
          var list, listParent, parentName;
          var subscriptList = subscript.split('|');
          if(subscriptList.length == 2) {
            if(endswith(name, subscriptList[0])) {
              parentName = name.substr(0, name.length - subscriptList[0].length)
              listParent = currentObject.get(parentName);
              listParent = new dock.cvObject(currentObject.cluster, listParent);
              list = listParent.get(subscriptList[0]);
            } else {
              println('Error in subscriptList: ' + name + ' ' + subscript);
              return;
            }
          } else {
            list = currentObject.get(name);
          }
          if(index > list.length) {
            println("Attemp to write past the last element of: " + name);
            println("  length = " + list.length + ", index = " + index);
          } else {
            if(subscript) {
              if(subscriptList.length == 2) {
                value = list[index][subscriptList[1]];
              } else {
                value = list[index][subscript];
              }
            } else {
              value = list[index];
            }
            if(specialvalues.length > 0) value = value2human(value, specialvalues);
            if(displayfunction.length > 0) value = eval(displayfunction);
            text.value = value;
          }
        } else {
          if(subscript) {
            var one = currentObject.get(what);
            value = one[subscript];
          } else {
            value = currentObject.get(what);
          }
          if(specialvalues.length > 0) value = value2human(value, specialvalues);
          if(displayfunction.length > 0) value = eval(displayfunction);
          text.value = value;
        }
      }
      document.commandDispatcher.advanceFocus();
      break;
    case KEY_ESCAPE:
      text.setAttribute("style", "");
      if(i > 0) {
        var index = parseInt(what.substr(i + 1));
        var name = what.substr(0, i);
        var list, listParent, parentName;
        var subscriptList = subscript.split('|');
        if(subscriptList.length == 2) {
          if(endswith(name, subscriptList[0])) {
            parentName = name.substr(0, name.length - subscriptList[0].length)
            listParent = currentObject.get(parentName);
            listParent = new dock.cvObject(currentObject.cluster, listParent);
            list = listParent.get(subscriptList[0]);
          } else {
            println('Error in subscriptList: ' + name + ' ' + subscript);
            return;
          }
        } else {
          list = currentObject.get(name);
        }
        if(index > list.length) {
          println("Attemp to write past the last element of: " + name);
          println("  length = " + list.length + ", index = " + index);
        } else {
          if(subscript) {
            if(subscriptList.length == 2) {
              value = list[index][subscriptList[1]];
            } else {
              value = list[index][subscript];
            }
          } else {
            value = list[index];
          }
          if(specialvalues.length > 0) value = value2human(value, specialvalues);
          if(displayfunction.length > 0) value = eval(displayfunction);
          text.value = value;
        }
      } else {
        if(subscript) {
          var one = currentObject.get(what);
          value = one[subscript];
        } else {
          value = currentObject.get(what);
        }
        if(specialvalues.length > 0) value = value2human(value, specialvalues);
        if(displayfunction.length > 0) value = eval(displayfunction);
        text.value = value;
      }
      break;
    case KEY_TAB:
      break;
    default:
      var save = document.getElementById('save');
      if(!save && document.parent) save = document.parent.getElementById('save');
      if(save) save.removeAttribute('disabled');
  }

  if(extraeval.length > 0) eval(extraeval);
}

/* FIX xul, not used yet */
function textbox_blur(event) {
  var text = event.target;
  var value = text.value;
}

function textbox_change(currentObject, event) {
  currentObject = isundefined(currentObject, current);
  var text = event.target;
  var value = text.value;
  var format = text.getAttribute('format');
  var what = text.getAttribute('what');
  var subscript = text.getAttribute('subscript');
  var nodups = eval(text.getAttribute('nodups'));
  var dupnull = text.getAttribute('dupnull');
  var newKey = text.getAttribute('newKey');
  var specialvalues = text.getAttribute('specialvalues');
  var parsefunction = text.getAttribute('parsefunction');
  var displayfunction = text.getAttribute('displayfunction');

  var i = what.indexOf('_');

  var extraeval = text.getAttribute('extraeval');
  var old;
  if(i > 0) {
    var index = parseInt(what.substr(i + 1));
    var name = what.substr(0, i);
    var list, listParent, parentName;
    var subscriptList = subscript.split('|');
    if(subscriptList.length == 2) {
      if(endswith(name, subscriptList[0])) {
        parentName = name.substr(0, name.length - subscriptList[0].length)
        listParent = currentObject.get(parentName);
        listParent = new dock.cvObject(currentObject.cluster, listParent);
        list = listParent.get(subscriptList[0]);
      } else {
        println('Error in subscriptList: ' + name + ' ' + subscript);
        return;
      }
    } else {
      list = currentObject.get(name);
    }
    if(index > list.length) {
      println("Attemp to write past the last element of: " + name);
      println("  length = " + list.length + ", index = " + index);
    } else {
      if(functionRegexMatch(value, format)) {
        if(subscript) {
          if(listParent) listParent.modified = true;
          list[index].modified = true;
          if(specialvalues.length > 0) value = human2value(value, specialvalues);
          if(parsefunction.length > 0) value = eval(parsefunction);
          if(subscriptList.length == 2) {
            list[index][subscriptList[1]] = value;
          } else {
            list[index][subscript] = value;
          }
        } else {
          if(parsefunction.length > 0) value = eval(parsefunction);
          if(specialvalues.length > 0) value = human2value(value, specialvalues);
          list[index] = value;
        }
        if(listParent) {
          listParent.set(subscriptList[0], list);
          currentObject.set('modified', true);
//           currentObject.set(parentName, listParent);
        } else {
          if(extraeval.length) old = currentObject.get(name);
          currentObject.set(name, list);
        }
        if(currentObject.parent) currentObject.parent.set('modified', true);
        if(eval(text.getAttribute('autorefresh'))) parent.refresh();
      } else {
        var value;
        if(subscript) {
          if(subscriptList.length == 2) {
            value = list[index][subscriptList[1]];
          } else {
            value = list[index][subscript];
          }
        } else {
          value = list[index];
        }
        textObjValue(text, value);
      }
    }
  } else {
    if(functionRegexMatch(value, format)) {
      if(subscript) {
        var subscriptList = subscript.split('|');
        if(subscriptList.length == 2) {
          var parentWhat = what.substr(0, what.length - subscriptList[0].length);
          var one = currentObject.get(parentWhat);
          one.modified = true;
          if(specialvalues.length > 0) value = human2value(value, specialvalues);
          if(parsefunction.length > 0) value = eval(parsefunction);
          one[subscriptList[0]][subscriptList[1]] = value;
          currentObject.set('modified', true);
          if(currentObject.parent) currentObject.parent.set('modified', true);
        } else {
          var one = currentObject.get(what);
          one.modified = true;
          if(specialvalues.length > 0) value = human2value(value, specialvalues);
          if(parsefunction.length > 0) value = eval(parsefunction);
          one[subscript] = value;
          currentObject.set('modified', true);
          if(currentObject.parent) currentObject.parent.set('modified', true);
//         currentObject.set(what, one);
        }
        if(eval(text.getAttribute('autorefresh'))) parent.refresh();
      } else {
        var dupkey = 0;
        if(nodups) dupkey = currentObject.cluster.getKeyFor(currentObject.relevantBaseType(), value, what, newKey);
        if(nodups && (value != dupnull) && (dupkey != currentObject.get('uniqueKey') && (dupkey != 0))) {
          textObjValue(text, currentObject.get(what));
          dock.displayError("A " + currentObject.relevantBaseType() + " with that " + what + " already exists");
        } else {
          if(specialvalues.length > 0) value = human2value(value, specialvalues);
          if(parsefunction.length > 0) value = eval(parsefunction);
          if(extraeval.length) old = currentObject.get(what);
          currentObject.set(what, value);
          if(currentObject.parent) currentObject.parent.set('modified', true);
          if(eval(text.getAttribute('autorefresh'))) parent.refresh();
        }
      }
    } else {
      var value;
      if(subscript) {
        var subscriptList = subscript.split('|');
        if(subscriptList.length == 2) {
          var parentWhat = what.substr(0, what.length - subscriptList[0].length);
          var parent = currentObject.get(parentWhat);
          value = parent[subscriptList[0]][subscriptList[1]];
        } else {
          var one = currentObject.get(what);
          value = one[subscript];
        }
      } else {
        value = currentObject.get(what);
      }
      textObjValue(text, value);
    }
  }
  text.setAttribute("style", "");

  if(extraeval.length > 0) eval(extraeval);
}

function textbox_command(currentObject, event) {
  currentObject = isundefined(currentObject, current);
  var text = event.target;
  var value = text.value;
  var what = text.getAttribute('what');
  var format = text.getAttribute('format');
  var nodups = eval(text.getAttribute('nodups'));
  var dupnull = text.getAttribute('dupnull');
  var newKey = text.getAttribute('newKey');
  if(functionRegexMatch(value, format)) {
    var dupkey = 0;
    if(nodups) dupkey = currentObject.cluster.getKeyFor(currentObject.relevantBaseType(), value, what, newKey);
    if(nodups && (value != dupnull) && (dupkey != currentObject.get('uniqueKey') && (dupkey != 0))) {
      text.setAttribute("style", "color: red;");
    } else {
      text.setAttribute("style", "");
    }
  } else {
    text.setAttribute("style", "color: red;");
  }
}

function separated_textbox_keypress(event) {
  var text = event.target;
  var value = text.value;
  var format = text.getAttribute('format');
  var subscript = text.getAttribute('subscript');
  var what = text.getAttribute('what');
  var sep = text.getAttribute('separator');
  var index = text.getAttribute('index');
  var nodups = eval(text.getAttribute('nodups'));
  var dupnull = text.getAttribute('dupnull');
  var newKey = text.getAttribute('newKey');
  var i = what.indexOf('_');

  var myCode = 0;
  if(sep == String.fromCharCode(event.charCode)) {
    myCode = KEY_ENTER - event.keyCode;
    text.setAttribute("style", "");
    event.stopPropagation();
  }

  var extraeval = text.getAttribute('extraeval');
  var old;
  switch(event.keyCode + myCode) {
    case KEY_ENTER:
      if(functionRegexMatch(value, format)) {
        if(i > 0) {
          var idx = parseInt(what.substr(i + 1));
          var name = what.substr(0, i);
          var one;
          var alist;
          if(subscript) {
            one = current.get(name);
            alist = one[subscript];
          } else {
            alist = current.get(name);
          }
          if(idx > alist.length) {
            println("Attemp to write past the last element of: " + name);
            println("  length = " + alist.length + ", index = " + idx);
          } else {
            var list = alist[idx].split(sep);
            list[index] = value;
            alist[idx] = list.join(sep);
            if(subscript) {
              one.modified = true;
              one['set' + firstUpper(subscript)](alist.length, alist);
              current.set('modified', true);
//               current.set(name, one);
            } else {
              if(extraeval.length) old = current.get(name);
              current.set(name, alist);
            }
            if(eval(text.getAttribute('autorefresh'))) {
              parent.refresh();
            } else if(myCode > 0) {
              text.value = value;
            }
          }
        } else {
          var one;
          var list;
          if(subscript) {
            one = current.get(what);
            list = one[subscript].split(sep);
          } else {
            list = current.get(what).split(sep);
          }
          list[index] = value;
          var data = list.join(sep);
          var dupkey = 0;
          if(nodups) dupkey = current.cluster.getKeyFor(current.relevantBaseType(), data, what, newKey);
          if(nodups && (data != dupnull) && (dupkey != current.get('uniqueKey') && (dupkey != 0))) {
            text.value = list[index];
          } else {
            if(subscript) {
              one.modified = true;
              one[subscript] = data;
              current.set('modified', true);
//               current.set(what, one);
            } else {
              if(extraeval.length) old = current.get(what);
              current.set(what, data);
            }
            if(eval(text.getAttribute('autorefresh'))) {
              parent.refresh();
            } else if(myCode > 0) {
              text.value = value;
            }
          }
        }
      } else {
        if(i > 0) {
          var idx = parseInt(what.substr(i + 1));
          var name = what.substr(0, i);
          var one;
          var alist;
          if(subscript) {
            one = current.get(name);
            alist = one[subscript];
          } else {
            alist = current.get(name);
          }
          if(idx > alist.length) {
            println("Attemp to write past the last element of: " + name);
            println("  length = " + alist.length + ", index = " + idx);
          } else {
            text.value =  alist[idx].split(sep)[index];
          }
        } else {
          var one;
          var list;
          if(subscript) {
            one = current.get(what);
            list = one[subscript].split(sep);
          } else {
            list = current.get(what).split(sep);
          }
          text.value = list[index];
        }
      }
      document.commandDispatcher.advanceFocus();
      break;
    case KEY_ESCAPE:
      text.setAttribute("style", "");
      text.value = current.get(what).split(sep)[index];
      break;
    case KEY_TAB:
      break;
    default:
      var save = document.getElementById('save');
      if(!save && document.parent) save = document.parent.getElementById('save');
      if(save) save.removeAttribute('disabled');
  }

  if(extraeval.length > 0) eval(extraeval);
}

function separated_textbox_change(event) {
  var text = event.target;
  var value = text.value;
  var format = text.getAttribute('format');
  var subscript = text.getAttribute('subscript');
  var what = text.getAttribute('what');
  var sep = text.getAttribute('separator');
  var index = text.getAttribute('index');
  var nodups = eval(text.getAttribute('nodups'));
  var dupnull = text.getAttribute('dupnull');
  var newKey = text.getAttribute('newKey');
  var i = what.indexOf('_');

  if(sep == String.fromCharCode(event.charCode)) {
    myCode = KEY_ENTER - event.keyCode;
    value = value.substr(0, value.length - 1);
  }

  var extraeval = text.getAttribute('extraeval');
  var old;
  if(functionRegexMatch(value, format)) {
    if(i > 0) {
      var idx = parseInt(what.substr(i + 1));
      var name = what.substr(0, i);
      var one;
      var alist;
      if(subscript) {
        one = current.get(name);
        alist = one[subscript];
      } else {
        alist = current.get(name);
      }
      if(idx > alist.length) {
        println("Attemp to write past the last element of: " + name);
        println("  length = " + alist.length + ", index = " + idx);
      } else {
        var list = alist[idx].split(sep);
        list[index] = value;
        alist[idx] = list.join(sep);
        if(subscript) {
          one.modified = true;
          one['set' + firstUpper(subscript)](alist.length, alist);
          current.set('modified', true);
//           current.set(name, one);
        } else {
          if(extraeval.length) old = current.get(name);
          current.set(name, alist);
        }
        if(eval(text.getAttribute('autorefresh'))) parent.refresh();
      }
    } else {
      var one;
      var list;
      if(subscript) {
        one = current.get(what);
        list = one[subscript].split(sep);
      } else {
        list = current.get(what).split(sep);
      }
      list[index] = value;
      var data = list.join(sep);
      var dupkey = 0;
      if(nodups) dupkey = current.cluster.getKeyFor(current.relevantBaseType(), data, what, newKey);
      if(nodups && (data != dupnull) && (dupkey != current.get('uniqueKey') && (dupkey != 0))) {
        text.value = list[index];
      } else {
        if(subscript) {
          one.modified = true;
          one[subscript] = data;
          current.set('modified', true);
//           current.set(what, one);
        } else {
          if(extraeval.length) old = current.get(what);
          current.set(what, data);
        }
        if(eval(text.getAttribute('autorefresh'))) parent.refresh();
      }
    }
  } else {
    if(i > 0) {
      var idx = parseInt(what.substr(i + 1));
      var name = what.substr(0, i);
      var alist = current.get(name);
      if(idx > alist.length) {
        println("Attemp to write past the last element of: " + name);
        println("  length = " + alist.length + ", index = " + idx);
      } else {
        text.value =  alist[idx].split(sep)[index];
      }
    } else {
      text.value = current.get(what).split(sep)[index];
    }
  }
  text.setAttribute("style", "");

  if(extraeval.length > 0) eval(extraeval);
}

function separated_textbox_command(event) {
  var text = event.target;
  var value = text.value;
  var format = text.getAttribute('format');
  if(functionRegexMatch(value, format)) {
    text.setAttribute("style", "");
  } else {
    text.setAttribute("style", "color: red;");
  }
}

function partialfields(field, n) {
  var box = document.getElementById(field + '-box');
  if(!!box) {
    if(box.childNodes.length == n) return box.childNodes;
    var first = box.firstChild;

    var kclass = first.getAttribute('class');
    for(var i = box.childNodes.length; i < n; i++) {
      var item = document.createElement("textbox");
      item.setAttribute('id', field + '-' + i);
      item.setAttribute('type', 'timed');
      item.setAttribute('class', kclass);
      box.appendChild(item);
    }
    while(box.childNodes.length > n) {
      box.removeChild(box.lastChild);
    }
    return box.childNodes;
  }
  return 0;
}

function changepartial(field, direction) {
  var box = document.getElementById(field + '-box');
  var first = box.firstChild;

  var varlen = first.getAttribute('varlen');
  var len = undotFormat(varlen);
  if(len[0]) {
    len = len[1];
    var n = box.childNodes.length;
    var i;
    for(i = 0; i < len.length; i++) {
      if(len[i] == n) break;
    }
    i = i + direction;
    if((i >= 0) && (i < len.length)) {
      n = len[i];
      var format = first.getAttribute('format');
      var subscript = first.getAttribute('subscript');
      var what = first.getAttribute('what');
      var sep = first.getAttribute('separator');
      var nodups = eval(first.getAttribute('nodups'));
      var dupnull = first.getAttribute('dupnull');
      var newKey = first.getAttribute('newKey');
      var digitadd = first.getAttribute('digitadd');

      var data = current.get(field);
      data = data.split(sep);
      for(var i = data.length; i < n; i++) data.unshift(digitadd);
      while(data.length > n) data.shift();
      current.set(field, data.join(sep));

      partial(what, sep, format, subscript, nodups, dupnull, newKey, varlen, digitadd);
    } else {
      var tmp = varlen;
      tmp = tmp.replace(/\.\./g, ' to ');
      tmp = tmp.replace(/,/g, ' or ');
      dock.displayError("The length of " + field + " can only be: " + tmp);
    }
  } else {
    println("Error: bad variable length: " + first.getAttribute('varlen'));
  }
}

function resolve_command(event) {
  var item = event.target;
  var value = parseInt(item.getAttribute('key'));
  var what = item.parentNode.parentNode.getAttribute('what');
  var subscript = item.parentNode.parentNode.getAttribute('subscript');
  var specialChar = item.parentNode.parentNode.getAttribute('specialChar');
  var autorefresh = eval(item.parentNode.parentNode.getAttribute('autorefresh'));
  var extraeval = item.parentNode.parentNode.getAttribute('extraeval');

  // Replace current object if defined:
  var getCurrentFunction = item.parentNode.parentNode.getAttribute('getCurrentFunction');
  var currentKey = item.parentNode.parentNode.getAttribute('currentKey');
  var oldCurernt = current;
  if(getCurrentFunction && (typeof(getCurrentFunction) == 'string') && getCurrentFunction.length) {
    println(" +++ getCurrentFunction = '" + getCurrentFunction + "'");
    eval('getCurrentFunction = ' + getCurrentFunction);
    current = getCurrentFunction(current, currentKey);
  }

  var old;
  var i = what.indexOf('_');
  if(i > 0) {
    var index = parseInt(what.substr(i + 1));
    var name = what.substr(0, i);
    var list = current.get(name);
    if(typeof(list) == "number") {
      current.set(name, value);
      if(autorefresh) parent.refresh();
    } else if(index > list.length) {
      println("Attemp to write past the last element of: " + name);
      println("  length = " + list.length + ", index = " + index);
    } else {
      if(subscript) {
        if(extraeval.length > 0) old = list[index][subscript];
        list[index].modified = true;
        list[index][subscript] = value;
      } else {
        if(extraeval.length > 0) old = list[index];
        list[index] = value;
      }
      current.set(name, list);
      if(autorefresh) parent.refresh();
    }
  } else {
    if(specialChar) value = specialChar + value + specialChar;
    if(subscript) {
      var work = current.get(what);
      if(extraeval.length > 0) old = work[subscript];
      work.modified = true;
      work[subscript] = value;
      current.set('modified', true);
    } else {
      if(extraeval.length > 0) old = current.get(what);
      current.set(what, value);
    }
    if(autorefresh) parent.refresh();
  }

  if(extraeval.length > 0) eval(extraeval);
  current = oldCurernt;
}

function lookup_command(event) {
  var item = event.target;
  var value = item.getAttribute('label');
  var what = item.parentNode.parentNode.getAttribute('what');
  current.set(what, value);
}

function number_command(event) {
  var item = event.target;
  var value = parseInt(item.getAttribute('number'));
  var what = item.parentNode.parentNode.getAttribute('what');
  var subscript = item.parentNode.parentNode.getAttribute('subscript');
  var extraeval = item.parentNode.parentNode.getAttribute('extraeval');
  var old = 0;

  var i = what.indexOf('_');
  if(i > 0) {
    var index = parseInt(what.substr(i + 1));
    var name = what.substr(0, i);
    var list = current.get(name);
    if(index > list.length) {
      println("Attemp to write past the last element of: " + name);
      println("  length = " + list.length + ", index = " + index);
    } else {
      if(subscript) {
        if(extraeval.length > 0) old = list[index][subscript];
        list[index][subscript] = value;
        list[index].modified = true;
      } else {
        if(extraeval.length > 0) old = list[index];
        list[index] = value;
      }
      current.set(name, list);
    }
  } else {
    if(subscript) {
      var work = current.get(what);
      work.modified = true;
      if(extraeval.length > 0) old = work[subscript]
      work[subscript] = value;
      current.set('modified', true);
    } else {
      if(extraeval.length > 0) old = current.get(what);
      current.set(what, value);
    }
  }
  if(eval(item.parentNode.parentNode.getAttribute('autorefresh'))) parent.refresh();
  if(extraeval.length > 0) eval(extraeval);

  if(current.get('modified')) {
    var save = document.getElementById('save');
    if(!save && document.parent) save = document.parent.getElementById('save');
    if(save) save.removeAttribute('disabled');
  }
}

function enum_command(currentObject, event) {
  currentObject = isundefined(currentObject, current);
  var item = event.target;
  var value = item.getAttribute('key');
  var what = item.parentNode.parentNode.getAttribute('what');
  var subscript = item.parentNode.parentNode.getAttribute('subscript');
  var i = what.indexOf('_');
  var forceString = false;
  if(item.parentNode.parentNode.hasAttribute('forceString')) {
    forceString = eval(item.parentNode.parentNode.getAttribute('forceString'));
  }
  if(i > 0) {
//     var index = parseInt(what.substr(i + 1));
//     var name = what.substr(0, i);
//     var list = currentObject.get(name);

    var index = parseInt(what.substr(i + 1));
    var name = what.substr(0, i);
    var list, listParent, parentName;
    var subscriptList = subscript.split('|');
    if(subscriptList.length == 2) {
      if(endswith(name, subscriptList[0])) {
        parentName = name.substr(0, name.length - subscriptList[0].length)
        listParent = currentObject.get(parentName);
        listParent = new dock.cvObject(currentObject.cluster, listParent);
        list = listParent.get(subscriptList[0]);
      } else {
        println('Error in subscriptList: ' + name + ' ' + subscript);
        return;
      }
    } else {
      list = currentObject.get(name);
    }

    if(index > list.length) {
      println("Attemp to write past the last element of: " + name);
      println("  length = " + list.length + ", index = " + index);
    } else {
      if(subscript) {
        if(isNumeric(value) && !forceString) {
          list[index][subscript] = parseInt(value);
        } else {
          list[index][subscript] = value;
        }
        list[index].modified = true;
      } else {
        if(isNumeric(value) && !forceString) {
          list[index] = parseInt(value);
        } else {
          list[index] = value;
        }
      }
      if(listParent) {
        listParent.set(subscriptList[0], list);
//         currentObject.set('modified', true);
        currentObject.setModified();
      } else {
        currentObject.set(name, list);
      }
    }
  } else {
    if(subscript) {
      var work = currentObject.get(what);
      work.modified = true;
      if(isNumeric(value) && !forceString) {
        work[subscript] = parseInt(value);
      } else {
        work[subscript] = value;
      }
      currentObject.set('modified', true);
    } else {
      if(isNumeric(value) && !forceString) {
        currentObject.set(what, parseInt(value));
      } else {
        currentObject.set(what, value);
      }
    }
    if(currentObject.parent) currentObject.parent.set('modified', true);
    currentObject.setModified();
  }
  if(eval(item.parentNode.parentNode.getAttribute('autorefresh'))) parent.refresh();
  if(currentObject.get('modified')) {
    var save = document.getElementById('save');
    if(!save && document.parent) save = document.parent.getElementById('save');
    if(save) save.removeAttribute('disabled');
  }
}

function radio_command(event) {
  var item = event.target;
  var value = item.getAttribute('key');
  if(value == 'boolean:true') {
    value = true;
  } else if(value == 'boolean:false') {
    value = false;
  }
  var what = item.parentNode.getAttribute('what');
  var subscript = item.parentNode.getAttribute('subscript');
  var i = what.indexOf('_');
  if(i > 0) {
    var index = parseInt(what.substr(i + 1));
    var name = what.substr(0, i);
    var list = current.get(name);
    if(index > list.length) {
      println("Attemp to write past the last element of: " + name);
      println("  length = " + list.length + ", index = " + index);
    } else {
      if(subscript) {
        if(isNumeric(value)) {
          list[index][subscript] = parseInt(value);
        } else {
          list[index][subscript] = value;
        }
        list[index].modified = true;
      } else {
        if(isNumeric(value)) {
          list[index] = parseInt(value);
        } else {
          list[index] = value;
        }
      }
      current.set(name, list);
    }
  } else {
    if(subscript) {
      var work = current.get(what);
      work.modified = true;
      if(isNumeric(value)) {
        work[subscript] = parseInt(value);
      } else {
        work[subscript] = value;
      }
      current.set('modified', true);
    } else {
      if(isNumeric(value)) {
        current.set(what, parseInt(value));
      } else {
        current.set(what, value);
      }
    }
  }
  if(eval(item.parentNode.getAttribute('autorefresh'))) parent.refresh();

  if(current.get('modified')) {
    var save = document.getElementById('save');
    if(!save && document.parent) save = document.parent.getElementById('save');
    if(save) save.removeAttribute('disabled');
  }
}

function radioimage_command(event) {
  var item = event.target;
  var value;
  while(!item.hasAttribute('key')) {
    item = item.parentNode;
  }
  var value = item.getAttribute('key');
  if(value == 'boolean:true') {
    value = true;
  } else if(value == 'boolean:false') {
    value = false;
  }
  var what = item.parentNode.getAttribute('what');
  var subscript = item.parentNode.getAttribute('subscript');
  var i = what.indexOf('_');
  if(i > 0) {
    var index = parseInt(what.substr(i + 1));
    var name = what.substr(0, i);
    var list = current.get(name);
    if(index > list.length) {
      println("Attemp to write past the last element of: " + name);
      println("  length = " + list.length + ", index = " + index);
    } else {
      if(subscript) {
        if(isNumeric(value)) {
          list[index][subscript] = parseInt(value);
        } else {
          list[index][subscript] = value;
        }
        list[index].modified = true;
      } else {
        if(isNumeric(value)) {
          list[index] = parseInt(value);
        } else {
          list[index] = value;
        }
      }
      current.set(name, list);
    }
  } else {
    if(subscript) {
      var work = current.get(what);
      work.modified = true;
      if(isNumeric(value)) {
        work[subscript] = parseInt(value);
      } else {
        work[subscript] = value;
      }
      current.set('modified', true);
    } else {
      if(isNumeric(value)) {
        current.set(what, parseInt(value));
      } else {
        current.set(what, value);
      }
    }
  }
  if(eval(item.parentNode.getAttribute('autorefresh'))) {
    parent.refresh();
  } else {
    var child = item.parentNode.firstChild;
    while(child) {
      var cv = child.getAttribute('key');
      if(cv == 'boolean:true') {
        cv = true;
      } else if(cv == 'boolean:false') {
        cv = false;
      }
      if(cv == value) {
        child.firstChild.setAttribute('on', true);
      } else {
        child.firstChild.removeAttribute('on');
      }
      child = child.nextSibling;
    }
  }
  if(current.get('modified')) {
    var save = document.getElementById('save');
    if(!save && document.parent) save = document.parent.getElementById('save');
    if(save) save.removeAttribute('disabled');
  }
}

function check_command(currentObject, event) {
  currentObject = isundefined(currentObject, current);
  var item = event.target;
  var value = item.getAttribute('checked');
  var what = item.getAttribute('what');
  var subscript = item.getAttribute('subscript');
  if(typeof(value) == "string") {
    value = (value == 'true');
  } else {
    value = eval(value);
  }
  if(subscript) {
    var one = currentObject.get(what);
    one[subscript] = value;
    one.modified = true;
    currentObject.set('modified', true);
  } else {
    currentObject.set(what, value);
  }
  var extraeval = item.getAttribute('extraeval');
  if(extraeval.length > 0) eval(extraeval);
  if(eval(item.getAttribute('autorefresh'))) parent.refresh();

  if(currentObject.get('modified')) {
    var save = document.getElementById('save');
    if(!save && document.parent) save = document.parent.getElementById('save');
    if(save) save.removeAttribute('disabled');
  }
}

function browse(field) {
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
  fp.init(window, "Load: " + field, nsIFilePicker.modeOpen);
  fp.appendFilter("All files", "*.*");
  if(fp.show() == nsIFilePicker.returnOK){
    var data = readfile(fp.file.path);
    if(data != 0) {
      current.set(field, data);
      document.getElementById(field).value = data;
    }
  }
}

function textresolve(what, format, type, specialChar) {
  specialChar = isundefined(specialChar, '!');
  var value = current.get(what);
  var isSpecial = (startswith(value, specialChar) && endswith(value, specialChar) && (value.length > 2 * specialChar.length));
  document.getElementById('is' + type).setAttribute('on', isSpecial);
  document.getElementById('isnot' + type).setAttribute('on', !isSpecial);
  resolve(what, type, '', '', specialChar);
  text(what, format);
  if(isSpecial) {
    var obj = document.getElementById(what);
    obj.value = '';
  }
}

function addViewConditional(args) {
  this.addViewConditionalCheck = function(elementContext, elementId, callName, initialValue, inverseCallName) {
    elementContext.getElementById(elementId).addEventListener('click', function(event) {
      var hidden = !event.target.checked;
      setViewConditional(elementContext, callName, hidden, inverseCallName);
    }, false);
    setViewConditional(elementContext, callName, !initialValue, inverseCallName);
  };

  this.addViewConditionalEnum = function(elementContext, elementId, callName, initialValue, evalFunc) {
    elementContext.getElementById(elementId).addEventListener('command', function(event) {
      var hidden = evalFunc(event.target.getAttribute('key'));
      setViewConditional(elementContext, callName, hidden);
    }, false);
    var changeKeypressCallback = function(event) {
      var key = '';
      if(event.target.selectedItem) {
        key = event.target.selectedItem.getAttribute('key');
      } else {
        key = event.target.value;
      }
      var hidden = evalFunc(key);
      setViewConditional(elementContext, callName, hidden);
    };
    elementContext.getElementById(elementId).addEventListener('change', changeKeypressCallback, false);
    elementContext.getElementById(elementId).addEventListener('keyup', changeKeypressCallback, false);
    setViewConditional(elementContext, callName, evalFunc(initialValue));
  };

  function setViewConditional(elementContext, className, hidden, inverseClassName=null){
    if(className) {
      var elements = elementContext.getElementsByClassName(className);
      for(var i = 0; i < elements.length; i++) {
        elements[i].setAttribute('hidden', hidden);
      }
    }
    if(inverseClassName) {
      var elements = elementContext.getElementsByClassName(inverseClassName);
      for(var i = 0; i < elements.length; i++) {
        elements[i].setAttribute('hidden', !hidden);
      }
    }
  }
  if(!args.hasOwnProperty('inverseCallName')) {
    args.inverseCallName = null;
  }
  if(args.type==='check') {
    this.addViewConditionalCheck(args.elementContext, args.elementId, args.callName, args.initialValue, args.inverseCallName);
  } else if(args.type==='enum') {
    this.addViewConditionalEnum(args.elementContext, args.elementId, args.callName, args.initialValue, args.evalFunc);
  }
}

function multipleText(elementNodeId, currentObjectField, currentObjects) {
  this.change = function(currentObjectField, currentObjects, event) {
    var value = event.target.value;
    for(var ii = 0; ii < currentObjects.length; ii++) {
      currentObjects[ii].set(currentObjectField, value);
    }
  };

  this.keypress = this.change;

  var elementNode = document.getElementById(elementNodeId);
  if (elementNode) {
    document.getElementById(elementNodeId).addEventListener("change", this.change.bind(this, currentObjectField, currentObjects), true);
    document.getElementById(elementNodeId).addEventListener("keypress", this.keypress.bind(this, currentObjectField, currentObjects), true);
    var values = {};
    for(var ii = 0; ii < currentObjects.length; ii++) {
      values[currentObjects[ii].get(currentObjectField)] = '1';
    }

    if(Object.keys(values).length===1) {
      document.getElementById(elementNodeId).value = currentObjects[0].get(currentObjectField);
    }
  } else {
    throw "Element ID " + elementNodeId + " not found!";
  }
}

function editableKenum(currentObject, currentObjectField, elementNodeId, options, format) {
  var currentObjectParentField = currentObjectField;
  if (typeof currentObjectField === 'object') {
    currentObjectParentField = currentObjectField.parent;
    var currentObjectChildField = currentObjectField.child;
  }
  format = isundefined(format, false);

  function getValue(currentObject, currentObjectField) {
    if (typeof currentObjectField === 'object') {
      return currentObject.get(currentObjectField.parent)[currentObjectField.child];
    } else {
      return currentObject.get(currentObjectField);
    }
  }

  function setValue(currentObject, currentObjectField, value) {
    if (typeof currentObjectField === 'object') {
      var subObject = currentObject.get(currentObjectField.parent);
      subObject.modified = true;
      subObject[currentObjectField.child] = value;
      currentObject.set('modified', true);
    } else {
      currentObject.set(currentObjectField, value);
    }
  }

  this.valueToHuman = function(human, options) {
    for (var i = 0; i < options.length; i++) {
      if (options[i][1]===human) {
        return options[i][0];
      }
    }
    return human;
  };

  this.humanToValue = function(value, options) {
    for (var i = 0; i < options.length; i++) {
      if (options[i][0]==value) {
        return options[i][1];
      }
    }
    return value;
  };

  this.clearFormat = function(elementNodeId, event) {
    document.getElementById(elementNodeId).inputField.style.color = null;
  };

  this.fieldUpdater = function(currentObject, currentObjectField, elementNodeId, options, format, event) {
    var value = this.valueToHuman(event.target.value, options);
    if(functionRegexMatch(value, format)) {
      document.getElementById(elementNodeId).inputField.style.color = null;
      setValue(currentObject, currentObjectField, value);
    } else {
      document.getElementById(elementNodeId).inputField.style.color = 'red';
    }
  };

  kenum(currentObjectParentField, options, currentObjectChildField, undefined, currentObject);

  document.getElementById(elementNodeId).value = this.humanToValue(getValue(currentObject, currentObjectField), options);
  if(format!==false) {
    document.getElementById(elementNodeId).addEventListener('command', this.clearFormat.bind(this, elementNodeId), true);
  }
  document.getElementById(elementNodeId).addEventListener('change', this.fieldUpdater.bind(this, currentObject, currentObjectField, elementNodeId, options, format), true);
  document.getElementById(elementNodeId).addEventListener('keyup', this.fieldUpdater.bind(this, currentObject, currentObjectField, elementNodeId, options, format), true);
}
