/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function getSelected() {
  return selected('list');
}

function getCurrentSelection() {
  var select = getSelected();
  var list;
  if(select.length > 0) {
    list = current.subObjectsFromKeys(subfield, select);
  } else if(dock.CLONE_SELECT_ALL) {
    list = current.getFromSubfield(subfield);
  } else {
    list = new Array();
  }
  return [list, current, subfield];
}

function refresh(use) {
  println("Refresh " + singular + ' - ' + document.documentURI);
  if(typeof(specialUse) == 'function') {
    if(!specialUse(use)) {
      println("Refresh " + singular + ": specialUse failed");
      return;
    }
  } else if(!!use) {
    current = use;
  }
  document.getElementById('revert').setAttribute('disabled', current.object.uniqueKey < dock.MAX_LOCAL_KEY);
  tree = document.getElementById('tree');
  var list = current.get(subfield);
  data = new Array();
  for(var i = 0; i < list.length; i++) {
    if(!list[i].toBeRemoved || alsoRemoved) {
      var tmp = createTreeItem(list[i], i);
      if(tmp === null) continue;
      data.push(tmp);
    }
  }
  for(var i = list.length; i < MINTREEITEMS; i++) data.push(blankTreeItem());
  var select = getSelected();
  loadTable();
  setSelected('list', select);
  if(typeof(postrefresh) == 'function') {
    return postrefresh(use);
  }
  return true;
}

function newSubObject() {
  var args = new Object();
  args['object'] = current;
  args['subobject'] = 0;
  current.cluster.currentSubObjectType = basetype;
  current.cluster.currentSubObjectList = current.get(subfield);  
  var d = new dock.cvDialog(editfile, args);
  d.onOk = function() {
    refresh();
  }
  d.onClose = function() {
    current.cluster.currentSubObjectType = 0;
    current.cluster.currentSubObjectList = 0;  
  }
  dock.dialogs.showDialog(d);
}
function editSubObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError('No ' + singular + ' selected');
  } else if(select.length == 1) {
    var args = new Object();
    args['object'] = current;
    args['subobject'] = select[0];
    args['detached'] = isundefined(parent.detached, false);
    current.cluster.currentSubObjectType = basetype;
    current.cluster.currentSubObjectList = current.get(subfield);  
    var d = new dock.cvDialog(editfile, args);
    d.onOk = function() {
      refresh();
    }
    d.onClose = function() {
      current.cluster.currentSubObjectType = 0;
      current.cluster.currentSubObjectList = 0;  
    }
    dock.dialogs.showDialog(d);
  } else {
    dock.displayError('Please select only one ' + singular);
  }
}

function cloneSubObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError('No ' + singular + ' selected');
  } else if(select.length == 1) {
    var work = current.subObjectFromKey(subfield, select[0]);
    var clone = work.clone();
    clone = new dock.cvObject(current.cluster, clone, current, subfield);
    var list = current.get(subfield);
    list.push(clone.object);
    current.set(subfield, list);
    refresh();
    var args = new Object();
    args['object'] = current;
    args['subobject'] = clone.get('uniqueKey');
    current.cluster.currentSubObjectType = basetype;
    current.cluster.currentSubObjectList = list;  
    var d = new dock.cvDialog(editfile, args);
    d.onOk = function() {
      refresh();
    }
    d.onClose = function() {
      current.cluster.currentSubObjectType = 0;
      current.cluster.currentSubObjectList = 0;  
    }
    dock.dialogs.showDialog(d);
  } else {
    dock.displayError('Please select only one ' + singular);
  }
}

function removeSubObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError('No ' + singular + ' selected');
  } else {
    var args = new Object();
    if(select.length == 1) {
      args['question'] = 'Are you sure you want to remove this ' + singular +'?';
    } else {
      args['question'] = 'Are you sure you want to remove these ' + plural +'?';
    }
    var d = new dock.cvDialog('question.xul', args);
    d.onOk = function() {
      for(var i = 0; i < select.length; i++) current.deleteSubObjectFromKey(subfield, select[i]);
      refresh();
    }
    dock.dialogs.showDialog(d);
  }
}

function moveup() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError('No ' + singular + ' selected');
  } else {
    var list = current.get(subfield);
    var index = indexes(list, select);
    if(index.length == 0) {
      dock.displayError('One or more selected items were not found');
    } else {
      var j = 0;
      while((j < index.length) && (index[j] == j)) j++;
      for(var i = 1; (j < index.length) && (i < list.length); i++) {
        if(index[j] == i) {
          var t = list[i - 1];
          list[i - 1] = list[i];
          list[i] = t;
          j++;
        }
      }
      current.set(subfield, list);
    }
  }
  refresh();
}

function movetop() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError('No ' + singular + ' selected');
  } else {
    var list = current.get(subfield);
    var index = indexes(list, select);
    if(index.length == 0) {
      dock.displayError('One or more selected items were not found');
    } else {
      var j = 0;
      for(var i = index.length - 1; i >= 0; i--) {
        var t = list.splice(j + index[i], 1)[0];
        list.unshift(t);
        j++;
      }
      current.set(subfield, list);
    }
  }
  refresh();
}

function movedown() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError('No ' + singular + ' selected');
  } else {
    var list = current.get(subfield);
    var index = indexes(list, select);
    if(index.length == 0) {
      dock.displayError('One or more selected items were not found');
    } else {
      var j = index.length - 1;
      while((j >= 0) && (index[j] == (list.length + j - index.length))) j--;
      for(var i = list.length - 2; (j >= 0) && (i >= 0); i--) {
        if(index[j] == i) {
          var t = list[i + 1];
          list[i + 1] = list[i];
          list[i] = t;
          j--;
        }
      }
      current.set(subfield, list);
    }
  }
  refresh();
}

function movebottom() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError('No ' + singular + ' selected');
  } else {
    var list = current.get(subfield);
    var index = indexes(list, select);
    if(index.length == 0) {
      dock.displayError('One or more selected items were not found');
    } else {
      var j = 0;
      for(var i = 0; i < index.length; i++) {
        var t = list.splice(index[i] - j, 1)[0];
        list.push(t);
        j++;
      }
      current.set(subfield, list);
    }
    current.set(subfield, list);
  }
  refresh();
}

