/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var loadtimer = 0;
var detached = false;

// copy of util.js : for speedup?
function getDock() {
  var old = 0;
  var dock = window;
  while(dock != old) {
    old = dock;
    if(typeof(dock.dialogs) != 'undefined') {
      break;
    } else if(!!dock.opener) { // Separate window
      println(dock.opener.document.title + " == " + dock.document.title);
      if(!dock.opener.document || (dock.opener.document.title == dock.document.title)) break;
      dock = dock.opener;
    } else if(!!dock.parent) { // docked inside tab
      dock = dock.parent;
    } else {
      break;
    }
  }
  if(typeof(dock.dialogs) == 'undefined') {
    println("Bad dock: " + window.document.title);
  }
  return dock;
}

function onLoad() {
  dock = getDock();
  println("Load (generictab): " + document.title);
}

function clearPanels() {
  var loading = 0;
  var panels = document.getElementById("tabpanels");
  if(panels) {
    var child = panels.firstChild;
    while(child) {
      if(child.hasAttribute('loadsrc')) {
        child.removeAttribute('src');
        loading++;
      }
      child = child.nextSibling;
    }
  }
  return loading;
}

function loadPanels() {
  var loading = 0;
  var panels = document.getElementById("tabpanels");
  if(panels) {    
    var child = panels.firstChild;
    while(child) {
      if(!child.hasAttribute('src')) {
        child.setAttribute('src', child.getAttribute('loadsrc'));
        loading++;
      } else if(!child.contentWindow.dock) {
        println("No dock: " + child.contentDocument.title);
        loading++;
      }
      child = child.nextSibling;
      dock.mainthread.processNextEvent(true);
    }
  }
  return loading;
}

function resetPanels() {
  var panels = document.getElementById("tabpanels");
  if(panels) {    
    var child = panels.firstChild;
    while(child) {
      child.removeAttribute('src');
      child = child.nextSibling;
    }
  }
}


function refresh(use, page, wantDetached) {
  if(!!use) {
    if(!current) {
      println("generic tab: no current -> resetPanels()");
      resetPanels();
    }
    else {
      var current_cluster;
      if(!!current.cluster) current_cluster = current.cluster;
      else current_cluster = current;
      var use_cluster;
      if(!!use.cluster) use_cluster = use.cluster;
      else use_cluster = use;
      if(current_cluster.actualurl != use_cluster.actualurl) {
        println("Cluster switch  '" + current_cluster.actualurl + " -> " + use_cluster.actualurl + ' -> resetPanels');
        resetPanels();
      }
    }
    current = use;
  }
  if(loadtimer) {
    return;
  }
  
  var loading = loadPanels();
  
  detached = isundefined(wantDetached, detached);
  var tabs = document.getElementById("tabs");
  if(detached) {
    var child = tabs.firstChild;
    while(child) {
      if(child.hasAttribute('detached') && !eval(child.getAttribute('detached'))) {
        child.setAttribute('hidden', true);
      }
      child = child.nextSibling;
    }
  }
  var forced = false;
  if(!!current) {
    var cluster = current;
    if(!!current.cluster) cluster = current.cluster;
        
    var tabs = document.getElementById("tabs");
    if(tabs) {
      var child = tabs.firstChild;
      var isUCS = current.isUCS();
      var isDellSettings = current.isDell13G();
      var isHadoop = cluster.isHadoop();
      var isCeph = cluster.isCeph();
      var isOpenStack = cluster.isOpenStack();
      var isKube = cluster.isKubernetes();

      var isPuppet = cluster.isPuppet();
      var hidden = false;
      var i = 0;
      if(!tabs.selectedIndex) {
        tabs.selectedIndex = 0;
      }
      while(child) {
        if(child.hasAttribute('ucs')) {
          if(isUCS && eval(child.getAttribute('ucs'))) {
            child.removeAttribute('hidden');
          } else {
            child.setAttribute('hidden', true);
          }
        } else if(child.hasAttribute('dellsettings')) {
          if(isDellSettings && eval(child.getAttribute('dellsettings'))) {
            child.removeAttribute('hidden');
          } else {
            child.setAttribute('hidden', true);
          }
        } else if(child.hasAttribute('ceph')) {
          if(isCeph && eval(child.getAttribute('ceph'))) {
            child.removeAttribute('hidden');
          } else {
            child.setAttribute('hidden', true);
          }
        } else if(child.hasAttribute('hadoop')) {
          if(isHadoop && eval(child.getAttribute('hadoop'))) {
            child.removeAttribute('hidden');
          } else {
            child.setAttribute('hidden', true);
          }
        } else if(child.hasAttribute('openstack')) {
          if(isOpenStack && eval(child.getAttribute('openstack'))) {
            child.removeAttribute('hidden');
          } else {
            child.setAttribute('hidden', true);
          } 
        } else if(child.hasAttribute('puppet')) {
          if(isPuppet && eval(child.getAttribute('puppet'))) {
            child.removeAttribute('hidden');
          } else {
            child.setAttribute('hidden', true);
          }
        } else if(child.hasAttribute('kubernetes')) {
          if(isKube && eval(child.getAttribute('kubernetes'))) {
            child.removeAttribute('hidden');
          } else {
            child.setAttribute('hidden', true);
          }
        }
        hidden = hidden || ((tabs.selectedIndex == i) && child.hasAttribute('hidden') && (child.getAttribute('hidden') == 'true'));
        child = child.nextSibling;
        ++i;
      }
    }
    if(hidden) tabs.selectedIndex = 0;
    forced = cluster.forcedDisconnect;
  }
  if(!forced) {
    page = isundefined(page, '');
    dock.waitIcon();
    if(loading) {
      println("Loading " + loading + " frames");
      loadtimer = window.setTimeout('loadFrames("' + page + '", 0);', 100);
    } else {
      loadtimer = window.setTimeout('update("' + page + '");', 25);
    }
  }
  if(typeof(postrefresh) == 'function') {
    if(postrefresh(use)) return;
  }
}

function loadFrames(page, count) {
  for(var i = 0; i < window.frames.length; i++) {
    if(!window.frames[i].dock) {
      println('loadFrames: ' + count + " @ " + i + " / " + window.frames.length);
      loadtimer = window.setTimeout('loadFrames("' + page + '",' + (count + 1) + ');', 100);
      return;
    }
    if(!!window.frames[i].loadFramesWaitForChildFrames) {
      println('wainting for sub frames');
      for(var j = 0; j < window.frames[i].frames.length; j++) {
        if(!window.frames[i].frames[j].dock) {
          println('loadFrames: ' + count + " @ " + i + ":" + j + " / " + window.frames.length + ":" + window.frames[i].frames.length);
          loadtimer = window.setTimeout('loadFrames("' + page + '",' + (count + 1) + ');', 100);
          return;
        }
      }
    }
  }
  loadtimer = 0;
  update(page);
}

function waitIcon() {
  for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("wait");
}

function unwaitIcon() {
  for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("auto");
}

function update(page) {
  page = isundefined(page, '');
  if(page.length > 0) setTab(page);
  var index = -1;
  var tabs = document.getElementById("tabs");
  if(tabs) index = tabs.selectedIndex;
  if(current) {
    var cluster = current;
    if(!!current.cluster) cluster = current.cluster;
    var forced = cluster.forcedDisconnect;
    if(forced) {
      println("Main refresh '" + cluster.actualurl + "' aborted");
    } else {
      if(cluster.connected || (cluster.cluster && cluster.cluster.connected)) {
        println("Main refresh '" + cluster.actualurl + "' tabs = " + window.frames.length + ", index = " + index);
        if(index >= 0) {
          if(!!window.frames[index].refresh) window.frames[index].refresh(current);
          dock.unwaitIcon();
        }
        for(var i = 0; i < window.frames.length; i++) {
          if((i != index) && !!window.frames[i].refresh) {
            window.frames[i].refresh(current);
          }
          //if((i != index) && !window.frames[i].refresh) {
          //}
        }
        setSave();
      }
    }
  } else {
    if(index >= 0) {
      if(!!window.frames[index].refresh) window.frames[index].refresh(current);
      dock.unwaitIcon();
    }
    println("ClustersTab refresh (" + index + ")");    
    for(var i = 0; i < window.frames.length; i++) {
      if((i != index) && !!window.frames[i].refresh) {
        window.frames[i].refresh(current);
      }
    }
  }
  if(index < 0) dock.unwaitIcon();
  if(loadtimer) window.clearTimeout(loadtimer);
  loadtimer = 0;
}

function genericFrameUpdate(functionCall) {
  for(var i = 0; i < window.frames.length; i++) {
    var f = eval('window.frames[i].' + functionCall);
    if(!!f) f();
  }
}

function setTab(page) {
  var tabs = document.getElementById("tabs");
  if(tabs) {
    for(var i = 0; i < tabs.childNodes.length; i++) {
      if(tabs.childNodes[i].id == page) {
        tabs.selectedIndex = i;
        break;
      }
    }
  }
}

function getTab() {
  var tabs = document.getElementById("tabs");
  if(tabs) return tabs.childNodes[tabs.selectedIndex].id;
  return '';  
}

function getTitle() {
  return document.title;
}

function setSave() {
  for(var i = 0; i < window.frames.length; i++) {
    if(!!window.frames[i].refresh) {
      var save = window.frames[i].document.getElementById('save');
      if(save) {
        var current = 0;
        if(typeof(window.frames[i].globalCurrent) != 'undefined') {
          current = window.frames[i].globalCurrent;
        } else if(typeof(window.frames[i].current) != 'undefined') {
          current = window.frames[i].current;
        }
        var cluster = 0;
        if(!current && (typeof(window.frames[i].cluster) != 'undefined')) cluster = window.frames[i].cluster;
        if(!!current && !!current.object) {
          var objs = 0;
          var type = current.get('baseType');
          if(type == 'CloudProvider') {
            objs = current.cluster.findNodesIn(type, current.get('uniqueKey'));
          }
          if(isArray(objs) && (objs.length > 0)) {
            var mod = false;
            for(var j = 0; !mod && (j < objs.length); j++) {
              var one = current.cluster.find(objs[j]);
              if(one) mod = one.get('modified')
            }
            if(mod) {
              save.removeAttribute("disabled");
            } else {
              save.setAttribute("disabled", true);
            }            
          } else {
            if(current.get('modified')) {
              save.removeAttribute("disabled");
            } else {
              save.setAttribute("disabled", true);
            }
          }
        } else if(!!cluster) {
          // FIXME: KDR what should this be?
//           if(cluster.get('modified')) {
//             save.removeAttribute("disabled");
//           } else {
//             save.setAttribute("disabled", true);
//           }
        }
      }
    }
  }
}

function list(clusterid) {
}

function stop() {
  dock.clearRefreshTimer();
  var forced = false;
  if(!!current) {
    if(!!current.current) {
      forced = current.current.forcedDisconnect;
    } else {
      forced = current.forcedDisconnect;
    }
  }
//   if(forced) {
//     println("Main stop, forced: " + forced);
//   } else {
//     println("Main stop");
    var panels = document.getElementById("tabpanels");
    if(panels) {
      println("Main stop, forced: " + forced + ", panels: " + panels.childNodes.length + ", frames: " + window.frames.length);
      var child = panels.firstChild;
      var i = 0;
      while(child && i < window.frames.length) {
        if(child.hasAttribute('src') && child.contentWindow.dock && (typeof(window.frames[i].stop) == 'function')) {
          window.frames[i].stop();
        }
        i++;
        child = child.nextSibling;
      }
    } else {
      println("Main stop, forced: " + forced + ", frames: " + window.frames.length);
      for(var i = 0; i < window.frames.length; i++) {
        if(typeof(window.frames[i].stop) == 'function') {
          window.frames[i].stop();
        }
      }
    }
    println("Main stop (done)");
//   }
}

function getCurrentSelection() {
  var tabs = document.getElementById("tabs");
  if(tabs) {
    var frame = tabs.selectedIndex;
    if((frame >= 0) && (frame < window.frames.length)) {
      frame = window.frames[frame];
      if(typeof(frame.getCurrentSelection) == 'function') {
        return frame.getCurrentSelection();
      } else if(typeof(window.frames[0].getCurrentSelection) == 'function') {
        return window.frames[0].getCurrentSelection();
      } else {
        dock.displayInfo('TODO getCurrentSelection for ' + frame.title);
      }
    } else {
      println("Frame index out of bounds");
    }
  }
  return [0, 0];
}

function findButton(name, doClick) {
  if(typeof(window[name]) === 'function') {
    if(doClick) {
      window[name]();
    }
    return true;
  }
  for(var i = 0; i < window.frames.length; i++) {
    if(typeof(window.frames[i][name]) === 'function') {
      if(doClick) {
        window.frames[i][name]();
      }
      return true;
    }
  }
  return false;
}
