/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var myDialog;
var current;
var dock;

function onLoad() {
  dock = getDock();
  println("Check now initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  current = myDialog.arguments.current;
  refresh();

  document.getElementById('refresh').focus();

}

function refresh() {
  var key = current.get('uniqueKey');
  current.cluster.clearGenericInfoWait(key);
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmgui', 'getGpuUnitOverview', [current.cluster.sessionid, key], function(code, data) {
      if(code == 0) {
        if(data) {
          var values = this.current.cluster.setGenericInfoWait(key, gpudatatable);
          if(values) gpudatatable(values);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);  
}

function parseNameIndexValue(line, parsed) {
  var idx = line.indexOf(' ');
  if((idx < 0) || (idx + 1 > line.length)) return false;
  var jdx = line.indexOf(' ', idx + 1);
  if((jdx < 0) || (jdx + 1 > line.length)) return false;
  parsed['type'] = line.substr(0, idx).toLowerCase();
  parsed['what'] = line.substr(idx + 1, jdx - idx - 1);
  parsed['value'] = line.substr(jdx + 1);
  parsed['index'] = -1;
  var kdx = parsed['what'].indexOf(':');
  if(kdx >= 0) {
    parsed['index'] = parseInt(parsed['what'].substr(kdx + 1));
    parsed['what'] = parsed['what'].substr(0, kdx);    
  }
  var vl = parsed['value'].length;
  if((vl > 2) && (parsed['value'][0] == '"') && (parsed['value'][vl - 1] == '"')) {
    parsed['value'] = trim(parsed['value'].substr(1, vl - 2));    
  }
  return true;
}

function gpudatatable(data) {
  var names = new Object();
  var values = new Object();
  var supported = new Object();
  for(var i = 0; i < data.length; i++) {
    var parsed = new Object();
    if(parseNameIndexValue(data[i], parsed) && (parsed['index'] >= 0)) {
      var whatl = parsed['what'].toLowerCase();
      var idx = whatl.indexOf('gpu');
      if(idx == whatl.length - 3) {
        parsed['what'] = parsed['what'].substr(0, idx);
        whatl = parsed['what'].toLowerCase();
      }
      if(whatl == 'name') {
        names[parsed['index']] = parsed['value'];
      } else if(parsed['type'] == 'status') {
        if(!values.hasOwnProperty(parsed['index'])) values[parsed['index']] = new Object();
        values[parsed['index']][parsed['what']] = parsed['value'];
      } else if(parsed['type'] == 'supported') {
        if(!supported.hasOwnProperty(parsed['index'])) supported[parsed['index']] = new Object();
        if(!supported[parsed['index']].hasOwnProperty(parsed['what'])) {
          supported[parsed['index']][parsed['what']] = '';
        } else {
          supported[parsed['index']][parsed['what']] += ', ';
        }
        supported[parsed['index']][parsed['what']] += parsed['value'];
      }
    }
  }
    
  var obj = document.getElementById('gpustatusrows');
  while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
  
  for(var index in names) {
    if(!values.hasOwnProperty(index)) {
      gpurow(obj, index, names[index], 'none detected', '', '');
    } else {
      for(var param in values[index]) {
        var s = '';
        if(supported.hasOwnProperty(index) && supported[index].hasOwnProperty(param)) s = supported[index][param];
        gpurow(obj, index, names[index], param, values[index][param], s);
      }
    }
  }
}

function gpurow(obj, index, name, param, value, supported) {
  var row = document.createElement('row');
  row.setAttribute("class", "staticdatarow");
  row.setAttribute("flex", "1");
  row.setAttribute("align", "center");
  row.setAttribute("height", "19px");
  
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", index);
  row.appendChild(label);
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", name);
  row.appendChild(label);
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", param);
  row.appendChild(label);
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", value);
  row.appendChild(label);
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", supported);
  row.appendChild(label);
  
  obj.appendChild(row);
}
