/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var overview;
var health;
var timer = 0;
var extraValues = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function stop() {
  println("Stop gpuunit overview");
  if(timer) window.clearTimeout(timer);
}

function refresh(use) {
  if(!!use) {
    current = use;
    extraValues = 0;
    setCallbackResult(new dock.cvGuiGpuUnitOverview());
  }
  println("Refresh gpuunit overview" + ' - ' + document.documentURI);
  
  if(current.stateString() == 'Closed') {
    var no = new dock.cvGuiGpuUnitOverview();
    setCallbackResult(no);
    updateHealth(new Array());
  } else if(current.cluster.connected) {
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmgui', 'getGpuUnitOverview', [current.cluster.sessionid, current.get('uniqueKey')], function(code, data) {
      if(code == 0) {
        setCallbackResult(data);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);

    var mpi = current.cluster.allChecksMetricsAsMetricPrmId(current.object.uniqueKey);
    var gpuunits = [current.object.uniqueKey];
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmmon', 'getLatestPickedRates', [gpuunits, mpi], function(code, data) {
      if(code == 0) {
        updateHealth(data);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
  }
}

function setCallbackResult(value) {
  overview = value;

  if(overview != null) {
    overviewValue('hostname', current.object.hostname);
    overviewValue('state', current.fullStateString());
    if(overview.voltage == 0) {
      overviewValue('voltage', 'N/A');
    } else {
      overviewValue('voltage', null, '%.2f V');
    }
    if(overview.current == 0) {
      overviewValue('current', 'N/A');
    } else {
      overviewValue('current', null, '%.2f A');
    }
    var list = overview.powerUsagePSU;
    if(list.length == 0) {
      overviewValue('wattage', 'N/A');
    } else {
      var power = 0;
      for(var i = 0; i < list.length; i++) {
        if(!kisNaN(list[i])) power += list[i];
      }
      if(kisNaN(power) || (power == 0)) {
        overviewValue('wattage', 'N/A');
      } else {
        overviewValue('wattage', power, '%.2f W');
      }
    }
    overviewValue('intakeTemp', null, '%.2f C');
    overviewValue('driverVersion');
    if(overview.pciSpeed == '0') {
      overviewValue('pciSpeed', 'N/A');
    } else {
      overviewValue('pciSpeed');
    }

    overviewFans();
    overviewGPU();

    var power = document.getElementById('power');
    while(power.firstChild) power.removeChild(power.firstChild);
    var pdu = current.get('powerDistributionUnits');
    for(var i = 0; i < pdu.length; i++) {
      var image = document.createElement('image');
      image.setAttribute('src', './skin/image/LED_blank.png');
      image.setAttribute('id', pdu[i].pdu + '_' + pdu[i].prt);
      power.appendChild(image);
      var spacer = document.createElement('spacer');
      spacer.setAttribute('width', '3px');
      power.appendChild(spacer);
      var label = document.createElement('label');
      label.setAttribute('value', current.cluster.resolveName(pdu[i].pdu) + ':' + pdu[i].prt);
      label.setAttribute('class', 'staticdataunit');
      power.appendChild(label);
    }
    var pctl = current.get('powerControl');
    if(startswith(pctl, 'ipmi')) {
      var name = 'ipmi';
      var image = document.createElement('image');
      image.setAttribute('src', './skin/image/LED_blank.png');
      image.setAttribute('id', name);
      power.appendChild(image);
      var spacer = document.createElement('spacer');
      spacer.setAttribute('width', '3px');
      power.appendChild(spacer);
      var label = document.createElement('label');
      label.setAttribute('class', 'staticdataunit');
      label.setAttribute('value', name);
      power.appendChild(label);
    } else if(pctl == 'custom') {
      var name = 'custom';
      var image = document.createElement('image');
      if(dock.DEMO) {
        image.setAttribute('src', './skin/image/LED_green.png');
      } else {
        image.setAttribute('src', './skin/image/LED_blank.png');
      }
      image.setAttribute('id', name);
      power.appendChild(image);
      var spacer = document.createElement('spacer');
      spacer.setAttribute('width', '3px');
      power.appendChild(spacer);
      var label = document.createElement('label');
      label.setAttribute('class', 'staticdataunit');
      label.setAttribute('value', name);
      power.appendChild(label);
    }
    if(power.childNodes.length == 0) {
      var label = document.createElement('label');
      label.setAttribute('value', 'N/A');
      label.setAttribute('class', 'staticdatavalue');
      power.appendChild(label);
      power.setAttribute('style', '');
    } else {
      power.setAttribute('style', 'padding-left: 4px;');
    }
  }
  if(extraValues) genericUpdate(extraValues);

  if(current.stateString() != 'Closed') {
    if(timer) window.clearTimeout(timer);
    timer = window.setTimeout('refresh();', dock.generalSettings.statusRefresh * 1000);
  }
}

function genericUpdate(values) {
  for(var i = 0; i < values.length; i++) {
    var j0 = values[i].indexOf(' ');
    if(j0 <= 0) continue;
    var j1 = values[i].indexOf(' ', j0 + 1);
    if(j1 <= 0) continue;
    var p = trim(values[i].substr(j0 + 1, j1 - j0 - 1));
    var v = trim(values[i].substr(j1 + 1));
    var obj = document.getElementById(p);
    if(!!obj) obj.setAttribute('value', v);
  }
  extraValues = values;
}

// function genericUpdate(values) {
//   var semigpu = document.getElementById('tempgpu');
//   var i = 0;
//   for(; i < values.length; i++) {
//     var j0 = values[i].indexOf(' ');
//     if(j0 <= 0) continue;
//     var j1 = values[i].indexOf('=', j0 + 1);
//     if(j1 <= 0) continue;
//     var p = trim(values[i].substr(j0 + 1, j1 - j0 - 1)) + ':';
//     var v = trim(values[i].substr(j1 + 1));
//     var l1 = document.createElement('shadowlabel');
//     l1.setAttribute('value', p);
//     l1.setAttribute('class', 'staticdatabigheader');
//     var box = document.createElement('hbox');
//     box.setAttribute('align', 'center');
//     if((v.length > 0) && (v[0] == '(')) {
//       var k0 = 0;
//       while((k0 < v.length) && (k0 >= 0)) {
//         var k1 = v.indexOf(')', k0);
//         if(k1 < 0) {
//           var l2 = document.createElement('shadowlabel');
//           l2.setAttribute('value', v);
//           l2.setAttribute('class', 'staticdatavalue');
//           box.appendChild(l2);
//           break;
//         } else {
//           var k2 = v.indexOf(':', k0);
//           var c, t;
//           if(k2 > 0 && (k2 < k1)) {
//             c = trim(v.substr(k0 + 1, k2 - k0 - 1));
//             t = trim(v.substr(k2 + 1, k1 - k2 - 1));          
//           } else {
//             c = trim(v.substr(k0 + 1, k1 - k0 - 1));
//             t = '';
//           }
//           var vbox = document.createElement('vbox');
//           var spacer =document.createElement('spacer');
//           spacer.setAttribute('height', '3px');
//           vbox.appendChild(spacer);
//           var image = document.createElement('image');
//           image.setAttribute('src', './skin/image/LED_' + c + '.png');
//           image.setAttribute('tooltiptext', t);
//           vbox.appendChild(image);
//           box.appendChild(vbox);
//         }
//         k0 = v.indexOf('(', k1);
//       }
//     } else {
//       var l2 = document.createElement('shadowlabel');
//       l2.setAttribute('value', v);
//       l2.setAttribute('class', 'staticdatavalue');
//       box.appendChild(l2);
//     }
//     
//     if(semigpu.childNodes[i].childNodes.length >= 3) {
//       semigpu.childNodes[i].replaceChild(l1, semigpu.childNodes[i].childNodes[2]);
//     } else {
//       semigpu.childNodes[i].appendChild(l1);
//     }
//     if(semigpu.childNodes[i].childNodes.length >= 4) {
//       semigpu.childNodes[i].replaceChild(box, semigpu.childNodes[i].childNodes[3]);
//     } else {
//       semigpu.childNodes[i].appendChild(box);
//     }
//   }
//   for(; i < semigpu.childNodes.length; i++) {
//     while(semigpu.childNodes[i].childNodes.length > 2) semigpu.childNodes[i].removeChild(semigpu.childNodes[i].lastChild);
//   }
// }
