/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var MAX = 1E+308;
var ZOOM_COLOR = "#333333";
var MIN_HEIGHT = 140;
var MIN_ZOOM_BOX = 8;
var PX = -5; // TODO: keep correct
var PY = -27;

var ADD_INFO = false;

function cvGraphDeviceMetric(device, metric, param, nodes, startTime, endTime, fill, translatedParam, cgroups, user) {
  if(isNumeric(device)) {
    this.device = parseInt(device);
  } else {
    this.device = device;
  }
  if(typeof('cgroups') == 'undefined') {
    this.cgroups = 0;
  } else {
    try {
      this.cgroups = JSON.parse(cgroups);
    } catch(e) {
      this.cgroups = cgroups;
    }
  }
  this.metric = parseInt(metric);
  this.param = param;
  this.translatedParam = isundefined(translatedParam, '');
  this.activePassiveMetric = 0;
  this.activePassiveParam = '';
  this.startTime = isundefined(startTime, 0);
  this.endTime = isundefined(endTime, 0);
  this.fill = isundefined(fill, true);
  this.unit = '';
  this.isCheck = false;
  this.isBar = false;
  this.isAverage = false;
  this.isGroup = false;
  this.isStoragePair = false;
  this.kind = 0;
  this.outliers = -1;
  this.title = '';
//   this.groupType = dock.GRAPH_GROUP_PIE;
  this.user = isundefined(user, 0);
  if(this.cgroups) {
    if(!isArray(this.cgroups) || this.cgroups.length == 1) {
      this.groupType = dock.GRAPH_GROUP_MIN_MED_MAX;
    } else {
      this.groupType = dock.GRAPH_GROUP_SUM;
    }
  } else {
    this.groupType = dock.GRAPH_GROUP_WHISKER;
  }
  if(isArray(nodes)) {
    this.nodes = nodes;
  } else if(typeof(nodes) == 'string') {
    nodes = nodes.split(',');
    if(nodes.length && isNumeric(nodes[0]) && nodes[0] != 0) {
      this.nodes = new Array();
      for(var i = 0; i < nodes.length; i++) this.nodes.push(parseInt(nodes[i]));
    } else {
      this.nodes = 0;
    }
  }
}

cvGraphDeviceMetric.prototype.isBoxWhisker = function() {
  return (this.groupType == dock.GRAPH_GROUP_WHISKER);
}

cvGraphDeviceMetric.prototype.isMinMedMax = function() {
  return (this.groupType == dock.GRAPH_GROUP_MIN_MED_MAX);
}

cvGraphDeviceMetric.prototype.isHeatmap = function() {
  return (this.groupType == dock.GRAPH_GROUP_HEATMAP);
}

cvGraphDeviceMetric.prototype.isPie = function() {
  return (this.groupType == dock.GRAPH_GROUP_PIE);
}

cvGraphDeviceMetric.prototype.isSingle = function() {
  return (this.groupType == dock.GRAPH_GROUP_HEATMAP) || (this.groupType == dock.GRAPH_GROUP_PIE);
}

cvGraphDeviceMetric.prototype.isOperator = function() {
  return (this.groupType == dock.GRAPH_GROUP_SUM) || (this.groupType == dock.GRAPH_GROUP_MIN) || (this.groupType == dock.GRAPH_GROUP_MAX) || (this.groupType == dock.GRAPH_GROUP_AVG);
}

cvGraphDeviceMetric.prototype.hasCGroups = function() {
  return isArray(this.cgroups);
}

function cvGraph(cluster, svg, box) {
  this.cluster = cluster;
  this.endTime = 0;
  this.startTime = 0;
  this.xzoom = 0;
  this.yzoom = 0;
  this.xzoomBroadcast = 0;
  this.yzoomBroadcast = 0;
  this.scale = [0, 0, 0, 0];
  this.drawbox = 0;
  this.xrange = 0;
  this.yrange = 0;
  this.intervals = dock.generalSettings.intervals;
  this.minValue = 0;
  this.maxValue = 0;
  this.bins = 10;
  this.normalize = 1;
  this.smooth = dock.generalSettings.heatmapSmooth;
  this.stacked = false;
  this.nanbin = 0;
  this.deviceMetric = 0;
  this.svg = svg;
  this.box = box;
  this.tooltip = 0;
  this.tooltiptext = 0;
  this.zoombox = 0;
  this.index = -1;
  this.data = 0;
  this.names = 0;
  this.pieInfo = 0;
  this.pieTip = 0;
  this.pieDate = 0;
  this.title = '';
  this.colors = [[['#84bf1c', '#a6e66c', '#a6e66c'], ['#ffff4c', '#4cff4c', '#ff4c4c'], ['#84bf1c', '#a6e66c', '#e78181'], ['#717e95', '#84bf1c', '#dc7c7c']],
                 [['#717e95', '#95a1b4', '#95a1b4'], ['#ffff4c', '#4cff4c', '#ff4c4c'], ['#717e95', '#95a1b4', '#6cdfe6'], ['#717e95', '#84bf1c', '#dc7c7c']],
                 [['#2aa4dd', '#6cdfe6', '#6cdfe6'], ['#ffff4c', '#4cff4c', '#ff4c4c'], ['#2aa4dd', '#6cdfe6', '#95a1b4'], ['#717e95', '#84bf1c', '#dc7c7c']],
                 [['#dc7c7c', '#e78181', '#e78181'], ['#ffff4c', '#4cff4c', '#ff4c4c'], ['#dc7c7c', '#e78181', '#a6e66c'], ['#717e95', '#84bf1c', '#dc7c7c']],
                 [['#e3e300', '#868a08', '#868a08'], ['#ffff4c', '#4cff4c', '#ff4c4c'], ['#e3e300', '#868a08', '#868a08'], ['#717e95', '#84bf1c', '#dc7c7c']],
                 [['#7c7cdc', '#8181e7', '#8181e7'], ['#ffff4c', '#4cff4c', '#ff4c4c'], ['#7c7cdc', '#8181e7', '#6ca6e6'], ['#717e95', '#84bf1c', '#dc7c7c']],
                ];

  this.refreshRate = dock.generalSettings.monitoringRefresh;
  this.timer = 0;
  this.playPause = true;
  this.WM = 14;
  this.HM = 36;
  this.last = 0;
  this.updating = false;
  this.needredraw = true;
  this.allowredraw = true;
  this.ownerWindow = dock;
  this.cmmon = 0;
  this.broadcast = false;
  this.legend = 1;
  this.minHeight = MIN_HEIGHT;
  this.dataIn = 0;
  this.dataWait = 0;
}

cvGraph.prototype.save = function() {
  var data = "++++BEGIN GRAPH++++\n";
  data += "title = '" + this.title + "';\n";
  data += "xzoom = " + this.xzoom.toSource() + ";\n";
  data += "yzoom = " + this.yzoom.toSource() + ";\n";
  data += "scale = " + this.scale.toSource() + ";\n";
  data += "intervals = " + this.intervals + ";\n";
  data += "bins = " + this.bins + ";\n";
  data += "normalize = " + this.normalize + ";\n";
  data += "nanbin = " + this.nanbin + ";\n";
  data += "minValue = " + this.maxValue + ";\n";
  data += "maxValue = " + this.minValue + ";\n";
  data += "smooth = " + this.smooth.toSource() + ";\n";
  data += "refreshRate = " + this.refreshRate + ";\n";
  data += "playPause = " + this.playPause + ";\n";
  data += "stacked = " + this.stacked + ";\n";
  if(this.deviceMetric) {
    var clone_metrics = [];
    println(this.deviceMetric.toSource());
    for(var i = 0; i < this.deviceMetric.length; i++) {
      var graph = clone(this.deviceMetric[i]);
      delete graph.isBoxWhisker;
      delete graph.isMinMedMax;
      delete graph.isHeatmap;
      delete graph.isPie;
      delete graph.isOperator;
      delete graph.isSingle;
      delete graph.hasCGroups;
      clone_metrics.push(graph);
    }
    data += "deviceMetric = " + clone_metrics.toSource() + ";\n";
    println(this.deviceMetric.toSource());
  } else {
    data += "deviceMetric = " + this.deviceMetric + ";\n";
  }
  data += "colors = " + this.colors.toSource() + ";\n";
  data += "legend = " + this.legend + ";\n";
  data += "++++END GRAPH++++\n";
  return data;
}

cvGraph.prototype.load = function(data) {
  if(data[0] != "++++BEGIN GRAPH++++") return false;
  var i = 1;
  while(i < data.length) {
    if(data[i] == "++++END GRAPH++++") {
      for(var j = 0; j < this.deviceMetric.length; j++) {
        if(isObject(this.deviceMetric[j])) {
          this.deviceMetric[j].isBoxWhisker = function() {
            return (this.groupType == dock.GRAPH_GROUP_WHISKER);
          }
          this.deviceMetric[j].isMinMedMax = function() {
            return (this.groupType == dock.GRAPH_GROUP_MIN_MED_MAX);
          }
          this.deviceMetric[j].isHeatmap = function() {
            return (this.groupType == dock.GRAPH_GROUP_HEATMAP);
          }
          this.deviceMetric[j].isPie = function() {
            return (this.groupType == dock.GRAPH_GROUP_PIE);
          }
          this.deviceMetric[j].isOperator = function() {
            return (this.groupType == dock.GRAPH_GROUP_SUM) || (this.groupType == dock.GRAPH_GROUP_MIN) || (this.groupType == dock.GRAPH_GROUP_MAX) || (this.groupType == dock.GRAPH_GROUP_AVG);
          }
          this.deviceMetric[j].isSingle = function() {
            return (this.groupType == dock.GRAPH_GROUP_HEATMAP) || (this.groupType == dock.GRAPH_GROUP_PIE);
          }
          this.deviceMetric[j].hasCGroups = function() {
            return isArray(this.cgroups);
          }
        }
        for(var k = 0; k < this.colors.length; k++) {
          if(this.colors[k].length < 4) {
            this.colors[k].push(['#717e95', '#84bf1c', '#dc7c7c']);
          }
        }
      }
      data.splice(0, i + 1);
      return true;
    }
    try {
      eval("this." + data[i]);
    } catch(err) {
      println("Error reading graph: " + err);
    }    
    i++;
  }
  return false;
}

cvGraph.prototype.showProperties = function() {
  var args = new Object();
  args['graph'] = this;
  var d = new dock.cvDialog("graphsettings.xul", args);
  d.parentWindow = window;
  dock.dialogs.showDialog(d);
}

cvGraph.prototype.handleDeviceChange = function(graphDeviceMetric, obj) {
  var work = 0;
  var name;
  if(graphDeviceMetric.device && isNumeric(graphDeviceMetric.device)) work = this.cluster.find(graphDeviceMetric.device);
  graphDeviceMetric.isStoragePair = false;
  graphDeviceMetric.activePassiveMetric = 0;
  if(work) {
    name = work.resolveName();
    if((work.get('baseType') == 'NodeGroup' && work.get('type') == 'STORAGEPAIR') || (work.get('baseType') == 'CMDaemonFailoverGroup')) {
      graphDeviceMetric.isStoragePair = true;
      var activePassiveMetricParam = this.cluster.findActivePassiveMetric(work);
      if(activePassiveMetricParam) {
        graphDeviceMetric.activePassiveMetric = activePassiveMetricParam[0];
        graphDeviceMetric.activePassiveParam = activePassiveMetricParam[1];
      }
    } else {
      graphDeviceMetric.isGroup = work.get('baseType') != 'Device';
    }
  } else if(graphDeviceMetric.device == 0) {
    name = 'All nodes';
    graphDeviceMetric.isGroup = true;
  } else {
    name = graphDeviceMetric.device;
    graphDeviceMetric.isGroup = true;
  }
  if(graphDeviceMetric.isStoragePair && (this.intervals == 0)) {
    this.intervals = 200;  // no raw, can't multiply
  } else if(graphDeviceMetric.isGroup) {
    if(graphDeviceMetric.isBoxWhisker()) {
      this.intervals = 25;
    } else if(graphDeviceMetric.isHeatmap()) {
      this.intervals = 50;
      this.minValue = obj.get('minimum');
      this.maxValue = obj.get('maximum');
    } else {
      this.intervals = 100;
    }
  }
  return [work, name];
}

cvGraph.prototype.add = function(graphDeviceMetric) {
  if(isNaN(graphDeviceMetric.metric)) return -100000;
  
  if(!this.deviceMetric || (this.deviceMetric.length == 0)) {
    if(graphDeviceMetric.startTime && graphDeviceMetric.endTime) {
      this.xzoom = [[graphDeviceMetric.startTime, graphDeviceMetric.endTime]];
    } else if(graphDeviceMetric.startTime) {
      this.xzoom = [[-graphDeviceMetric.startTime, 0]];
    } else if(graphDeviceMetric.endTime) {
      this.xzoom = [[dock.generalSettings.timespan, -graphDeviceMetric.endTime]];
    } else {
      this.xzoom = [[dock.generalSettings.timespan, 0]];
    }
    this.yzoom = [ [[NaN, NaN],[NaN, NaN],[NaN, NaN],[NaN, NaN]] ];
  }
  if(!isArray(this.deviceMetric)) this.deviceMetric = new Array();
  // TODO: KDR is this check needed ?
//   if((graphDeviceMetric.isAverage && (this.deviceMetric.length > 0)) ||
//      ((this.deviceMetric.length > 0) && this.deviceMetric[i].isAverage)) {
//      return -100001;
//   }
  if((this.deviceMetric.length > 0) && (graphDeviceMetric.isSingle() || this.deviceMetric[0].isSingle())) {
    return -100002;
  }
  
  var index = this.deviceMetric.length % this.colors.length;
  var i;
  for(i = 0; i < this.colors.length; i++) {
    if((index >= this.deviceMetric.length) || !isObject(this.deviceMetric[index])) break;
    index = (++index) % this.colors.length;
  }
  if(i == this.colors.length) return -this.colors.length;
  this.last = 0;
  this.deviceMetric[index] = graphDeviceMetric;
  
  var obj = this.cluster.find(graphDeviceMetric.metric);  
  if(obj.get('baseType') == 'Metric') {
    graphDeviceMetric.unit = obj.get('unit');
  } else {
    graphDeviceMetric.unit = '';
    graphDeviceMetric.isCheck = true;
  }
  this.scale[index] = index;
  for(var j = 0; j < this.yzoom.length; j++) {
    this.yzoom[j][index] = [NaN, NaN];
  }
  for(var j = 0; j < this.deviceMetric.length; j++) {
    if(isObject(this.deviceMetric[j])) {
      if((this.deviceMetric[j].unit == graphDeviceMetric.unit) && (this.deviceMetric[j].isCheck == graphDeviceMetric.isCheck)) {
        this.scale[index] = j;
        break;
      }
    }
  }
  
  var work = this.handleDeviceChange(graphDeviceMetric, obj);
  var name = work[1];
  work = work[0];
  
  var txt = name + ":" + this.cluster.resolveName(graphDeviceMetric.metric);
  if(graphDeviceMetric.cgroups && isArray(graphDeviceMetric.cgroups) && graphDeviceMetric.cgroups.length > 1) {
    txt += " (all " + graphDeviceMetric.cgroups.length + " jobs for " + graphDeviceMetric.user + " on " + graphDeviceMetric.nodes.length + " nodes)";
  } else if(graphDeviceMetric.translatedParam.length) {
    txt += " (" + graphDeviceMetric.translatedParam + ")";
  } else if(graphDeviceMetric.param.length) {
    txt += " (" + graphDeviceMetric.param + ")";
  }
  graphDeviceMetric.title = txt;
  if(blank(this.title)) {
    this.title = graphDeviceMetric.title;
    if(!this.timer) this.start();
  } else {
    var k = 0;
    var n = 0;
    for(var i = 0; i < this.deviceMetric.length; i++) {
      if(isObject(this.deviceMetric[i])) {
        n++
        if(i < index) k++;
      }
    }
    var list = this.title.split(/,\s/);
    if(list.length == n - 1) {
      list.splice(k, 0, txt);
      this.title = list.join(', ');
    }    
//     var list = this.title.split(/,\s/);
//     if(list.length == n - 1) {
//       var txt = name + ":" + this.cluster.resolveName(graphDeviceMetric.metric);
//       if(graphDeviceMetric.param) txt += " (" + graphDeviceMetric.param + ")";
//       graphDeviceMetric.title = txt;
//       list.splice(k, 0, txt);
//       this.title = list.join(', ');
//     }
  }
  return 0;
}

cvGraph.prototype.set = function(graphDeviceMetric) {
  this.clear();
  return this.add(graphDeviceMetric);
}

cvGraph.prototype.remove = function(index, updateTitle) {
  if(index > this.deviceMetric.length) return false;
  if(!isObject(this.deviceMetric[index])) return false;

  updateTitle = isundefined(updateTitle, true);
  if(updateTitle) {
    var k = 0;
    var n = 0;
    for(var i = 0; i < this.deviceMetric.length; i++) {
      if(isObject(this.deviceMetric[i])) {
        n++
        if(i < index) k++;
      }
    }
    var list = this.title.split(/,\s/);
    if(list.length == n) {
      list.splice(k, 1);
      this.title = list.join(', ');
    }
  }

  this.deviceMetric[index] = 0;
  if(this.scale[index] == index) {
    var j = -1;
    for(var i = 0; i < this.deviceMetric.length; i++) {
      if(isObject(this.deviceMetric[i])) {
        if(this.scale[i] == index) {
          if(j < 0) {
            for(var k = 0; k < this.yzoom.length; k++) {
              this.yzoom[k][j] = this.yzoom[k][i];
            }
            j = i;
          }
          this.scale[i] = j;
        }
      }
    }
  }

  this.last = 0;
  this.scale[index] = 0;
  var n = this.deviceMetric.length - 1;
  var i;
  for(i = n; i >= 0; i--) {
    if(isObject(this.deviceMetric[i])) {
      this.deviceMetric.splice(i + 1, n - i);
      return true;
    }
  }
  this.clear();
  return true;
}

cvGraph.prototype.start = function() {
  this.timer = window.setInterval(createObjectCallback(this, this.update), this.refreshRate * 1000);
  this.playPause = true;
}

cvGraph.prototype.setRefreshRate = function(rate) {
  this.refreshRate = rate;
  if(this.timer) {
    this.stop(true);
  }
  if(this.playPause) this.start();
}

cvGraph.prototype.allowRedraw = function(set) {
  this.allowredraw = set;
  if(this.allowredraw && this.needredraw) this.redraw();
}

cvGraph.prototype.redraw = function() {
  if(this.allowredraw && this.svg) {
    var h = Math.max(this.minHeight, this.svg.parentNode.boxObject.height - this.HM);
    this.svg.setAttribute("height", h + "px");
    this.draw(0, 1, 1, this.svg.parentNode.boxObject.width - this.WM, h);
    if(isArray(this.index)) {
      if(this.index[0] >= 0) document.getElementById('title' + this.index[0] + '_' + this.index[1]).setAttribute('value', this.title);
    } else {
      if(this.index >= 0) document.getElementById('title' + this.index).setAttribute('value', this.title);
    }
    this.needredraw = false;
  } else if(this.svg) {
    this.needredraw = true;
  }
}

cvGraph.prototype.update = function() {
  if(this.deviceMetric.length == 0) {
    this.clear();
  } else {
    if(!this.updating) {
      this.updating = true;
      var now = new Date();
      now = Math.floor(now.getTime() / 1000 + this.cluster.timeOffset);
      if(now - this.last > 30) {
        this.refresh(now);        
      } else {
        this.redraw();
        this.updating = false;
      }
    }
  }
}

cvGraph.prototype.stop = function(restart) {
  restart = isundefined(restart, false);
  if(this.timer) {
    this.last = 0;
    window.clearInterval(this.timer);
    this.timer = 0;
    if(!restart) {
      this.playPause = false;
      if(this.index >= 0) replaceOne(this.index);
    }
    return true;
  }
  return false;
}

cvGraph.prototype.info = function(x, y) {
  var t = Math.floor((this.xrange[0] * (this.drawbox[2] - x) + this.xrange[1] * (x - this.drawbox[0])) / (this.drawbox[2] - this.drawbox[0]));
  var data = new Array();
  var n = 0;
  for(var i = 0; i < this.deviceMetric.length; i++) {
    if(isObject(this.deviceMetric[i])) n++;
  }
  var dd = new Date();
  dd.setTime(t * 1000);
  var info = '';
  for(var i = 0; i < this.data.length; i++) {
    if(isObject(this.deviceMetric[i])) {
      var unit = '';
      if(!this.deviceMetric[i].isCheck) {
        unit = this.deviceMetric[i].unit;
        if(unit == '') unit = '-';
      }
      if(this.deviceMetric[i].isGroup && this.deviceMetric[i].isBoxWhisker()) {
        for(var j = 1; j < this.data[i].length; j++) {
          for(var k = 7; k < this.data[i][j].length; k++) {
            if(this.data[i][j][k].length == 4) {
              var dx = this.data[i][j][k][2] - x;
              var dy = this.data[i][j][k][3] - y;
              if(dx * dx + dy * dy <= 16) {
                var work = this.cluster.find(this.data[i][j][k][0]);
                if(work) {
                  var du = formatKMGselect(this.data[i][j][k][1], unit)
                  data.push(work.resolveName() + ': ' + du);
                }
                break;
              }
            }
          }
        }
      } else if(this.deviceMetric[i].isGroup && this.deviceMetric[i].isMinMedMax()) {
        for(var j = 1; j < this.data[i].length; j++) {
          if((this.data[i][j - 1][0] <= t) && (this.data[i][j][0] >= t)) {
            if(!kisNaN(this.data[i][j - 1][1]) && !kisNaN(this.data[i][j][1])) {
              var ddf = this.data[i][j][0] - this.data[i][j - 1][0];
              for(var im = 1; im <= 3; im++) {
                var val;
                if(ddf == 0) {
                  val = this.data[i][j - 1][im];
                } else {
                  val = ((this.data[i][j][0] - t) / ddf) * this.data[i][j - 1][im] + ((t - this.data[i][j - 1][0]) / ddf) * this.data[i][j][im];
                }
                data.push(formatKMGselect(val, unit));
              }
              break;
            }
          }
        }      
      } else if(this.deviceMetric[i].isGroup && this.deviceMetric[i].isPie()) {
        if(this.pieInfo) {
          dd = this.pieDate;
          var dx = x - this.pieInfo[0];
          var dy = y - this.pieInfo[1];
          if(dx * dx + dy * dy <= this.pieInfo[2] * this.pieInfo[2]) {
            var r = Math.sqrt(dx * dx + dy * dy);
            var a;
            if(r <= 1e-9) {
              a = 0;
            } else {
              a = Math.acos(-dy / r);
            }
            if(dx < 0) a = 2 * Math.PI - a;
            for(var j = 0; j < this.pieInfo[3].length; j++) {
              if(a >= this.pieInfo[3][j][0] && a < this.pieInfo[3][j][1]) {
                var k = this.pieInfo[4][j];
                var du = formatKMGselect(this.data[i][k], unit)
                data.push(this.names[k] + ': ' + du);
                break;
              }
            }
          }
        }
      } else if(this.deviceMetric[i].isBar) {
        for(var j = 0; j < this.data[i].length; j++) {
          var d = Math.abs(this.data[i][j][0] - t);
          var dl, dr;
          if(j == 0) {
            dl = 2 * d + 1;
          } else {
            dl = Math.abs(this.data[i][j - 1][0] - t);
          }
          if(j + 1 >= this.data[i].length) {
            dr = 2 * d + 1;
          } else {
            dr = Math.abs(this.data[i][j + 1][0] - t);
          }
          if(d <= dl && d <= dr) {
            if(!kisNaN(this.data[i][j][1])) {
              data.push(formatKMGselect(this.data[i][j][1], unit));
              info = this.data[i][j][2];
            }
            break;
          }
        }
      } else {
        for(var j = 1; j < this.data[i].length; j++) {
          if((this.data[i][j - 1][0] <= t) && (this.data[i][j][0] >= t)) {
            if(!kisNaN(this.data[i][j - 1][1]) && !kisNaN(this.data[i][j][1])) {
              if(this.deviceMetric[i].isCheck && this.deviceMetric[i].fill) {
                var val;
                if((t - this.data[i][j - 1][0]) > (this.data[i][j][0] - t)) {
                  val = this.data[i][j][1];
                } else {
                  val = this.data[i][j - 1][1];
                }
                if(val < 0.5) {
                  data.push('PASS');
                } else if(val > 1.5) {
                  data.push('FAIL');
                } else {
                  data.push('UNKNOWN');
                }
              } else {
                var ddf = this.data[i][j][0] - this.data[i][j - 1][0];
                var val;
                if(ddf == 0) {
                  val = this.data[i][j - 1][1];
                } else {
                  val = ((this.data[i][j][0] - t) / ddf) * this.data[i][j - 1][1] + ((t - this.data[i][j - 1][0]) / ddf) * this.data[i][j][1];
                }
                data.push(formatKMGselect(val, unit));
              }
              if((t - this.data[i][j - 1][0]) > (this.data[i][j][0] - t)) {
                if(this.data[i][j].length >= 2) info = this.data[i][j][2];
              } else {
                if(this.data[i][j - 1].length >= 2) info = this.data[i][j - 1][2];
              }
            }
            break;
          }
        }
      }
      if((n > 1) && (data.length == i)) data.push('N/A');
    } else {
      if(n > 1) data.push('N/A');
    }
  }
  if(isArray(info)) {
    info = this.cluster.composeInfoMessage(info);
    this.svg.parentNode.setAttribute('tooptiptext', info);
  } else {
    this.svg.parentNode.removeAttribute('tooptiptext');
  }
  if(data.length == 0) {
    data = "N/A";
  } else if(data.length == 1) {
    data = data[0];
  } else {
    data = '(' + data.join(', ') + ')';
  }
  return [dd.format("dd/mmm/yyyy HH:MM:ss") + ": " + data, info];
}

cvGraph.prototype.zoomIn = function(x0, y0, x1, y1) {
  if(typeof(x0) == "undefined") {
    var add = new Array();
    var l = this.xzoom.length - 1;
    println("==== IN ====");
    printall(this.xzoom);
    if(this.xzoom[l][1] == 0) {
      if(this.xzoom[l][0] < 0) {
        var now = Date.now() / 1000;
        add = [this.xzoom[l][0] - (now + this.xzoom[l][0]) / 2, 0];
      } else {
        add = [this.xzoom[l][0] / 2, 0];
      }
    } else if(this.xzoom[l][1] >= 0xffffffff) {
      var now = Date.now() / 1000;
      add = [(now - this.xzoom[l][0]) / 2, this.xzoom[l][1]];
    } else if(this.xzoom[l][1] < 0) {
      // TODO: endtime set
    } else {
      var x = (this.xzoom[l][1] - this.xzoom[l][0]) / 4;
      add = [this.xzoom[l][0] + x, this.xzoom[l][1] - x];
    }
    add[0] = Math.floor(add[0]);
    add[1] = Math.floor(add[1]);
    this.xzoom.push(add);
    println("------------");
    printall(this.xzoom);
    add = this.yzoom[0].slice();
    this.yzoom.push(add);
  } else {
    var e;
    if(x1 < x0) {
      e = x1;
      x1 = x0;
      x0 = e;
    }
    if(y1 < y0) {
      e = y1;
      y1 = y0;
      y0 = e;
    }
    var l = this.xzoom.length - 1;
    var addx = new Array();
    if((x1 - x0) <= MIN_ZOOM_BOX) {
      addx = this.xzoom[l].slice();
    } else {
      if(x0 > this.drawbox[0] + MIN_ZOOM_BOX) {
        addx[0] = Math.floor((this.xrange[0] * (this.drawbox[2] - x0) + this.xrange[1] * (x0 - this.drawbox[0])) / (this.drawbox[2] - this.drawbox[0]));
      } else {
        addx[0] = this.xzoom[l][0];
      }
      if(x1 < this.drawbox[2] - MIN_ZOOM_BOX) {
        addx[1] = Math.floor((this.xrange[0] * (this.drawbox[2] - x1) + this.xrange[1] * (x1 - this.drawbox[0])) / (this.drawbox[2] - this.drawbox[0]));
      } else {
        addx[1] = this.xzoom[l][1];
      }
      if(((addx[1] == 0) && (addx[0] != this.xzoom[l][0])) || ((addx[1] != 0) && (this.xzoom[l][1] == 0) && (addx[0] == this.xzoom[l][0]))) { // TODO starttime
        addx[0] = this.xrange[1] - addx[0];
      }
    }
    var addy;
    if((y1 - y0) <= MIN_ZOOM_BOX) {
      addy = this.yzoom[l].slice();
    } else {
      addy = new Array();
      for(var i = 0; i < this.scale.length; i++) {
        if(this.scale[i] == i) {
          var one = this.yzoom[l][i].slice();
          if(y0 > this.drawbox[1] + MIN_ZOOM_BOX) {
            one[1] = (this.yrange[i][1] * (this.drawbox[3] - y0) + this.yrange[i][0] * (y0 - this.drawbox[1])) / (this.drawbox[3] - this.drawbox[1]);
          }
          if(y1 < this.drawbox[3] - MIN_ZOOM_BOX) {
            one[0] = (this.yrange[i][1] * (this.drawbox[3] - y1) + this.yrange[i][0] * (y1 - this.drawbox[1])) / (this.drawbox[3] - this.drawbox[1]);
          }
          if((addx[0] == this.xzoom[l][0]) && (addx[1] == this.xzoom[l][1]) && ((one[0] == this.yzoom[l][i][0]) || (kisNaN(one[0]) && kisNaN(this.yzoom[l][i][0])))
                                                                            && ((one[1] == this.yzoom[l][i][1]) || (kisNaN(one[1]) && kisNaN(this.yzoom[l][i][1])))) return;
          addy.push(one);
        } else {
          addy.push([NaN, NaN]);
        }
      }
    }
    this.xzoom.push(addx);
    this.yzoom.push(addy);
  }
  this.last = 0;
  this.xzoomBroadcast = 0;
  this.yzoomBroadcast = 0;
  if(this.broadcast) {
    window.setTimeout('setZoomBroadcast(' + this.index + ');', 100);
  }
}

cvGraph.prototype.zoomOut = function() {
  var l = this.xzoom.length - 1;
  if(l == 0) {
    var add;
    println("==== OUT ====");
    printall(this.xzoom);
    if(this.xzoom[0][1] == 0) {
      if(this.xzoom[0][0] < 0) {
        var now = Date.now() / 1000;
        add = [2 * this.xzoom[0][0] + now, 0];
      } else {
        add = [this.xzoom[0][0] * 2, 0];
      }
    } else if(this.xzoom[0][1] < 0) {
      // TODO: endtime set
    } else if(this.xzoom[0][1] >= 0xffffffff) {
      now = Date.now() / 1000;
      add = [2 * this.xzoom[0][0] - now, this.xzoom[0][1]];
    } else {
      var x = (this.xzoom[0][1] - this.xzoom[0][0]) / 2;
      add = [this.xzoom[0][0] - x, this.xzoom[0][1] + x];
      var now = Date.now() / 1000;
      if(add[1] > (now + this.cluster.timeOffset)) add = [3 * x, 0];
    }
    add[0] = Math.floor(add[0]);
    add[1] = Math.floor(add[1]);
    this.xzoom[0] = add;
    println("------------");
    printall(this.xzoom);
  } else {
    this.xzoom.pop();
    this.yzoom.pop();
  }
  this.last = 0;
  this.xzoomBroadcast = 0;
  this.yzoomBroadcast = 0;
  if(this.broadcast) {
    window.setTimeout('setZoomBroadcast(' + this.index + ');', 100);
  }
}

cvGraph.prototype.clear = function() {
  this.deviceMetric = new Array();
  this.stop();
  this.title = '';
  this.xzoom = 0;
  this.yzoom = 0;
  this.xzoomBroadcast = 0;
  this.yzoomBroadcast = 0;
  if(this.index >= 0) {
    replaceOne(this.index);
    this.index = -1;
  }
}

cvGraph.prototype.deviceList = function(i) {
  var devices = new Array();
  if(this.deviceMetric[i].nodes) {
    devices = this.deviceMetric[i].nodes;
  } else {
    var work = this.cluster.find(this.deviceMetric[i].device);
    if(work) {
      devices = this.cluster.findNodesIn(work.getType(), work.get('uniqueKey'));
    } else if(this.deviceMetric[i].device == 0) {
      var nodes = this.cluster.find('SlaveNode');
      for(var j = 0; j < nodes.length; j++) {
        devices.push(nodes[j].get('uniqueKey'));
      }
    }
  }
  return devices;
}

cvGraph.prototype.fullTitle = function(i) {
  if(this.deviceMetric[i].groupType == dock.GRAPH_GROUP_SUM) return this.deviceMetric[i].title + ' (sum)';
  if(this.deviceMetric[i].groupType == dock.GRAPH_GROUP_MAX) return this.deviceMetric[i].title + ' (max)';
  if(this.deviceMetric[i].groupType == dock.GRAPH_GROUP_MIN) return this.deviceMetric[i].title + ' (min)';
  if(this.deviceMetric[i].groupType == dock.GRAPH_GROUP_AVG) return this.deviceMetric[i].title + ' (avg)';
  return this.deviceMetric[i].title;
}

cvGraph.prototype.refresh = function(now) {
  if(!this.deviceMetric || (this.deviceMetric.length == 0)) return;
  if(!this.cluster.connected) return;

  if(this.xzoomBroadcast) {
    if(this.xzoomBroadcast[1] == 0) {
      this.endTime = now;
      this.startTime = this.endTime - this.xzoomBroadcast[0];
    } else {
      this.startTime = this.xzoomBroadcast[0];
      this.endTime = this.xzoomBroadcast[1];
    }
  } else {
    var l = this.xzoom.length - 1;
    if(this.xzoom[l][1] == 0) {
      this.endTime = now;
      if(this.xzoom[l][0] < 0) {
        this.startTime = -this.xzoom[l][0];
      } else {
        this.startTime = this.endTime - this.xzoom[l][0];
      }
    } else if(this.xzoom[l][1] < 0) {
      this.endTime = -this.xzoom[l][1];
      this.startTime = this.endTime - this.xzoom[l][0];
    } else if(this.xzoom[l][1] == 0xffffffff) {
      this.startTime = this.xzoom[l][0];
      this.endTime = now;
    } else {
      this.startTime = this.xzoom[l][0];
      this.endTime = this.xzoom[l][1];
    }
  }
  if(dock.DEMO) {
    var alldata = new Array();
    var N = 100;
    for(var i = 0; i < this.deviceMetric.length; i++) {
      alldata[i] = new Array();
      var name = this.deviceMetric[i].metric;
      name = this.cluster.find(name);
      name = name.resolveName().toLowerCase();
      println("Fake data: " + name);
      if(name == "loadone") {
        var S = 7000 + Math.random() * 1000;
        var w = 10;
        var z = 4;
        var zz = 0.5 + Math.random() * 0.5;
        var d = Math.floor((1.5 - Math.random()) * N / z);
        var p = 0;
        for(var k = 0; k < N; k++) {
          var x = Math.floor((this.startTime * (N - k) + this.endTime * k) / N);
          var y = Math.random() * 0.05 + 4 / (1 + Math.exp((x - now + S) * -0.01));
          if((k >= d - w) && (k < d + w)) {
            y = Math.max(0, y - zz * (1 - (k - d) * (k - d) / (w * w)));
          } else if(k == d + w) {
            zz = 0.5 + Math.random() * 0.5;
            d += Math.max(2 * w + 1, Math.floor((1.5 - Math.random()) * N / z));
          }
          alldata[i].push([x, Math.max(0, y + p)]);
          p += (Math.random() - 0.5) * 0.005;
        }
      } else if(name == "memused") {
        var k = 0; var m; var v;
        m = Math.floor(Math.random() * N / 16 + N / 10); v = 31e+6;
        while(k < m) {
          var x = Math.floor((this.startTime * (N - k) + this.endTime * k) / N);
          alldata[i].push([x, v + Math.random() * 1e+7]);
          k++;
        }
        m = Math.floor(Math.random() * N / 16 + 7 * N / 16); v = 2.1e+9;
        while(k < m) {
          var x = Math.floor((this.startTime * (N - k) + this.endTime * k) / N);
          alldata[i].push([x, v + Math.random() * 2e+7]);
          k++;
        }
        m = Math.floor(Math.random() * N / 4 + 5 * N / 9); v = 1.7e+9;
        while(k < m) {
          var x = Math.floor((this.startTime * (N - k) + this.endTime * k) / N);
          alldata[i].push([x, v + Math.random() * 2e+7]);
          k++;
        }
        m = N; v = 8.1e+9;
        while(k < m) {
          var x = Math.floor((this.startTime * (N - k) + this.endTime * k) / N);
          alldata[i].push([x, v + Math.random() * 2e+7]);
          k++;
        }
      } else if(name == "runningjobs") {
        var k = 0; var m; var v;
        m = 0;
        v = 45;
        while(k < N) {
          m += Math.floor(Math.random() * N / 10 + N / 20);
          while((k < m) && (k < N)) {
            var x = Math.floor((this.startTime * k + this.endTime * (N - k)) / N);
            alldata[i].unshift([x, v]);
            k++;
          }
          v += 3 - Math.floor(Math.random() * 6);
        }
      } else if(name == "bytessend") {
        var w = 7;
        var z = Math.floor(Math.random() * 5) + 3;
        var zz = (1 + Math.random()) * 1e+6;
        var d = Math.floor((1.5 - Math.random()) * N / z);
        for(var k = 0; k < N; k++) {
          var x = Math.floor((this.startTime * (N - k) + this.endTime * k) / N);
          var y = Math.random() * 1e+4;
          if((k >= d - w) && (k < d + w)) {
            y = zz * (Math.random() + 1 - (k - d) * (k - d) / (w * w));
          } else if(k == d + w) {
            zz = (1 + Math.random()) * 1e+6;
            d += Math.max(2 * w + 1, Math.floor((1.5 - Math.random()) * N / z));
          }
          alldata[i].push([x, y]);
        }
      } else if(name == "eccerrors") {
        var y = 0;
        for(var k = 0; k < N; k++) {
          var x = Math.floor((this.startTime * (N - k) + this.endTime * k) / N);
          var r = Math.random();
          if(r > 0.96) y++;
          alldata[i].push([x, y]);
        }
      } else if(name == "") { // EccErrors ?
        var y = 0;
        for(var k = 0; k < N; k++) {
          var x = Math.floor((this.startTime * (N - k) + this.endTime * k) / N);
          var y = 53 + Math.random() * 7;
          if(r > 0.96) y += Math.random() * 7;
          alldata[i].push([x, y]);
        }
      } else if(name == "cputemp") {
        N = 20;
        rm = [Math.random() * 8 + 8, Math.random() * 5 + 4];
        rp = [Math.random() * 8 + 10, Math.random() * 5 + 4];
        var node = this.cluster.find('Device', 'node041');
        node = node.get('uniqueKey');
        for(var k = 0; k < N; k++) {
          var x = k + 0.5;
          x = [Math.floor((this.startTime * (N - x) + this.endTime * x) / N)];
          var y0 = 1 + Math.exp(8 * (3 * N / 7 - k) / N);
          y0 = (40 / y0) + 30;
          for(var kk = 0; kk < 2; kk++) {
            rm[kk] += (Math.random() - 0.5) * 2;
            if(rm[kk] < 0) rm[kk] = -rm[kk];
            rp[kk] += (Math.random() - 0.45) * 2 * (1 + kk);
            if(rp[kk] < 0) rp[kk] = -rp[kk];
          }
          rm.sort(function(a,b) { return a > b; });
          rp.sort(function(a,b) { return a > b; });
          var y = [y0 - rm[1], y0 - rm[0], y0, y0 + rp[0], y0 + rp[1]];
          var t = x.concat(y).concat([Math.random() * 0.1, [node, y[0]], [node, y[4]], [node, y[0]], [node, y[4]]]) ;
          alldata[i].push(t);
        }
      } else if(name == "freespace") {
        N = 20;
        rm = [Math.random() * 8 + 8, Math.random() * 5 + 4];
        rp = [Math.random() * 8 + 10, Math.random() * 5 + 4];
        var p = 10e8;
        var y0 = 423 * p;
        var node = this.cluster.find('Device', 'node041');
        node = node.get('uniqueKey');
        for(var k = 0; k < N; k++) {
          var x = k + 0.5;
          x = [Math.floor((this.startTime * (N - x) + this.endTime * x) / N)];
          y0 -= Math.random() * 10 * p;
          for(var kk = 0; kk < 2; kk++) {
            rm[kk] += (Math.random()) * Math.pow(kk + 1, 4);
            if(rm[kk] < 0) rm[kk] = -rm[kk];
            rp[kk] += (Math.random() - 0.5) * 2;
            if(rp[kk] < 0) rp[kk] = -rp[kk];
          }
          rm.sort(function(a,b) { return a > b; });
          rp.sort(function(a,b) { return a > b; });
          println(rm);
          var y = [y0 - rm[1] * p, y0 - rm[0] * p, y0, y0 + rp[0] * p, y0 + rp[1] * p];
          var t = x.concat(y).concat([Math.random() * 0.1, [node, y[0]], [node, y[4], node, y[0] + p], [node, y[4] - p]]);
          alldata[i].push(t);
        }
/*        var y = [100.0, 200.0, 300.0, 400.0, 500.0];
        for(var k = 0; k < N; k++) {
          var x = k + 0.5;
          x = [Math.floor((this.startTime * (N - x) + this.endTime * x) / N)];
          for(var kk = 0; kk < y.length; kk++) {
            y[kk] += Math.random() * 20 - 10;
          }
          y.sort(function(a,b) { return a > b; });
          var t = x.concat(y).concat([Math.random() * 0.1, [0, y[0]], [0, y[4]]]);
          alldata[i].push(t);
        }*/
      }
    }
    this.data = alldata;
  } else {
    var smdi = 0;
    var rmdi = new Array();
    var allrmdi = new Array();
    this.data = new Array();
    this.startTime = Math.max(0, this.startTime);
    if(this.endTime < this.startTime) this.endTime = this.startTime;
    for(var i = 0; i < this.deviceMetric.length; i++) {
      this.data.push(new Array());
      if(isObject(this.deviceMetric[i])) {
        if(this.deviceMetric[i].isGroup) {
          if(this.deviceMetric[i].isOperator()) {
            var devices = this.deviceList(i);
            if(devices.length) {
              var list = new Array();
              var deviceBit = 0;
              if(this.deviceMetric[i].hasCGroups() && this.cluster.isCGroupMetric(this.deviceMetric[i].metric)) {
                for(var j = 0; j < devices.length; j++) {
                  for(var k = 0; k < this.deviceMetric[i].cgroups.length; k++) {
                    var one = new dock.cvReadMonDataId();
                    one.devId = devices[j];
                    one.metricId = this.deviceMetric[i].metric;
                    one.metricParam = this.deviceMetric[i].cgroups[k];
                    if(this.deviceMetric[i].param.length && (this.deviceMetric[i].param != one.metricParam)) {
                      one.metricParam += ":" + this.deviceMetric[i].param;
                    }
                    one.kind = this.deviceMetric[i].kind;
                    one.begTime = this.startTime;
                    one.endTime = this.endTime;
                    list.push(one);
                  }
                }
                if(parseInt(this.cluster.versionInfo.cmdaemonBuild) >= 30703) {
                  deviceBit = 1;
                }
              } else {
                for(var j = 0; j < devices.length; j++) {
                  var one = new dock.cvReadMonDataId();
                  one.devId = devices[j];
                  one.metricId = this.deviceMetric[i].metric;
                  one.metricParam = this.deviceMetric[i].param;
                  one.kind = this.deviceMetric[i].kind;
                  one.begTime = this.startTime;
                  one.endTime = this.endTime;
                  list.push(one);
                }
              }
              println("++++ LIST: " + list.length + " : " + this.deviceMetric[i].hasCGroups() + ", " + this.cluster.isCGroupMetric(this.deviceMetric[i].metric) + ", devicebit: " + deviceBit);
              if(this.deviceMetric[i].groupType == dock.GRAPH_GROUP_MAX) {
                allrmdi.push([i, (14 + deviceBit) << 20, list]);
              } else if(this.deviceMetric[i].groupType == dock.GRAPH_GROUP_MIN) {
                allrmdi.push([i, (22 + deviceBit) << 20, list]);
              } else if(this.deviceMetric[i].groupType == dock.GRAPH_GROUP_AVG) {
                allrmdi.push([i, (38 + deviceBit) << 20, list]);
              } else { // sum
                allrmdi.push([i, (6 + deviceBit) << 20, list]);
              }
            }
          } else if(this.deviceMetric[i].isBoxWhisker() || this.deviceMetric[i].isMinMedMax()) {
            smdi += 1;
          } else if(this.deviceMetric[i].isHeatmap()) {
            // MOST LIKELY NOTHING
          } else if(this.deviceMetric[i].isPie()) {
            // MOST LIKELY NOTHING
          }
        } else if(this.deviceMetric[i].isAverage) { // MOST LIKELY NOTHING
        } else if(this.deviceMetric[i].isStoragePair) {
          var devices = this.deviceList(i);
//           println(' ++++ KDR: Product devices ' + i + ' : ' + devices.length);
          if(devices.length) {
            var list = new Array();
            for(var j = 0; j < devices.length; j++) {
              var one = new dock.cvReadMonDataId();
              one.devId = devices[j];
              one.metricId = this.deviceMetric[i].metric;
              one.metricParam = this.deviceMetric[i].param;
              one.kind = this.deviceMetric[i].kind;
              one.begTime = this.startTime;
              one.endTime = this.endTime;
              list.push(one);
              if(this.deviceMetric[i].activePassiveMetric) {
                var two = new dock.cvReadMonDataId();
                two.devId = devices[j];
                two.metricId = this.deviceMetric[i].activePassiveMetric;
                two.metricParam = this.deviceMetric[i].activePassiveParam;
                two.kind = this.deviceMetric[i].kind;
                two.begTime = this.startTime;
                two.endTime = this.endTime;
                list.push(two);
              }
            }
//             println(' ++++ KDR: Product list ' + i + ' : ' + list.length);
            if(this.deviceMetric[i].activePassiveMetric) {
              allrmdi.push([i | 0x10000, 0, list]);
            } else {
              allrmdi.push([i, 0, list]); // fallback to sum
            }
          }          
        } else {
          var one = new dock.cvReadMonDataId();
          one.devId = this.deviceMetric[i].device;
          one.metricId = this.deviceMetric[i].metric;
          one.metricParam = this.deviceMetric[i].param;
          one.kind = this.deviceMetric[i].kind;
          one.begTime = this.startTime;
          one.endTime = this.endTime;
          rmdi.push(one);
        }
      }
    }
    if(rmdi.length) allrmdi.unshift([-1, 0, rmdi]);
    
    this.dataIn = 0;
    this.dataWait = 0;
    for(var i = 0; i < this.deviceMetric.length; i++) {
      if(isObject(this.deviceMetric[i])) {
        if(this.deviceMetric[i].isGroup) this.dataWait++;
      }
    }
    if(rmdi.length > 0) this.dataWait++;
    this.ownerWindow.waitIcon();
    if(smdi > 0) {
      for(var i = 0; i < this.deviceMetric.length; i++) {
        if(isObject(this.deviceMetric[i])) {
          if(this.deviceMetric[i].isGroup && (this.deviceMetric[i].isBoxWhisker() || this.deviceMetric[i].isMinMedMax())) {
            var promilles;
            var outliers;
            if(this.deviceMetric[i].isMinMedMax()) {
              outliers = 0;
              promilles = [0, 500, 1000];
            } else {
              outliers = isundefined(this.deviceMetric[i].outliers, 2);
              promilles = [0, 250, 500, 750, 1000];
            }
            var devices = this.deviceList(i);
            var data = new Object();
            var errors = new Object();
            dock.workerpool.getWorker(this.cluster.actualurl).call('cmmon', 'getStatisticalData', [devices, this.deviceMetric[i].metric, this.deviceMetric[i].param, this.startTime, this.endTime, this.intervals, promilles, outliers], function(code, data) {
              if(code == 0) {
                for(var j = 0; j < data.statisticData.length; j++) {
                  var one = [(data.statisticData[j].begTime + data.statisticData[j].endTime) / 2].concat(data.statisticData[j].statisticValues);
                  one.push(data.statisticData[j].percentageNan);
                  if(outliers != 0) {
                    var nodes = data.statisticData[j].exactDevices;
                    var statisticDatas = data.statisticData[j].exactValues;
                    for(var k = 0; k < nodes.length; k++) {
                      one.push([nodes[k], statisticDatas[k]]);
                    }
                  }
                  this.current[0].data[this.current[1]].push(one);
                }
              } else {
                dock.jsonCallError(this, code, data);
              }
              this.current = this.current[0];
              this.current.dataIn++;
              if(this.current.dataIn >= this.current.dataWait) {
                this.current.redraw();
                this.current.last = now;
                this.current.updating = false;
                this.current.ownerWindow.unwaitIcon();
              }
            }, [this, i]); // KDR: pass i, very important!
          }
        }
      }
    }
    
    if(this.deviceMetric.length > 0 && isObject(this.deviceMetric[0]) && this.deviceMetric[0].isHeatmap()) {
      var i = 0; // only 1 allowed
      var devices = this.deviceList(i);
      var flags = new Array();
      if(this.normalize) flags.push(0x100000000 + this.normalize);
      if(this.nanbin) flags.push(0x200000001);
      if((this.smooth.length == 2) && this.smooth[0] > 0 && this.smooth[1] > 0) {
        flags.push(0x300000000 + (Math.floor(Math.min(this.smooth[0], 0xFFFF)) << 16) + Math.floor(Math.min(this.smooth[1], 0xFFFF)));
      }
      dock.workerpool.getWorker(this.cluster.actualurl).call('cmmon', 'getHeatMap', [devices, this.deviceMetric[i].metric, this.deviceMetric[i].param, this.startTime, this.endTime, this.intervals, this.minValue, this.maxValue, this.bins, flags], function(code, data) {
        if(code == 0) {
          if(data.heatMapData == null) {
            this.current[0].data[this.current[1]] = new dock.cvHeatMapData();
          } else {
            this.current[0].data[this.current[1]] = data.heatMapData;
          }
        } else {
          dock.jsonCallError(this, code, data);
        }
        this.current = this.current[0];
        this.current.dataIn++;
        if(this.current.dataIn >= this.current.dataWait) {
          this.current.redraw();
          this.current.last = now;
          this.current.updating = false;
          this.current.ownerWindow.unwaitIcon();
        }
      }, [this, i]);
    }
    if(this.deviceMetric.length > 0 && isObject(this.deviceMetric[0]) && this.deviceMetric[0].isPie()) {
      var i = 0; // only 1 allowed
      var devices = this.deviceList(i);
      this.names = this.cluster.resolveAll(devices);
      var one = new dock.cvMetricPrmId();
      one.metricId = this.deviceMetric[i].metric;
      one.metricParam = this.deviceMetric[i].param;
      var mpi = [one];
      this.pieDate = new Date();
      dock.workerpool.getWorker(this.cluster.actualurl).call('cmmon', 'getLatestPickedRates', [devices, mpi], function(code, data) {
        if(code == 0) {
          var nd = new Array();
          for(var i = 0; i < data.length; i++) {
            nd.push(data[i].rate);
          }
          this.current[0].data[this.current[1]] = nd;
        } else {
          dock.jsonCallError(this, code, data);
        }
        this.current = this.current[0];
        this.current.dataIn++;
        if(this.current.dataIn >= this.current.dataWait) {
          this.current.redraw();
          this.current.last = now;
          this.current.updating = false;
          this.current.ownerWindow.unwaitIcon();
        }
      }, [this, i]);      
    }
    for(var i0 = 0; i0 < allrmdi.length; i0++) {
      var intervals = this.intervals;
      if(allrmdi[i0][1] > 0 && intervals == 0) intervals = 100;
      dock.workerpool.getWorker(this.cluster.actualurl).call('cmmon', 'readDataByIntervalNum', [allrmdi[i0][2], allrmdi[i0][1] + intervals], function(code, data) {
        if(code == 0) {
          var allInfo = new Array();
          var j = 0;
          var i = 0;
          var products = new Object();
          if(this.current[1][0] < 0) {
            while((i < data.length) && (j < this.current[0].deviceMetric.length)) {
              var cur = data[i].readMonDataId;
              if((cur.devId == this.current[0].deviceMetric[j].device) && (cur.metricId == this.current[0].deviceMetric[j].metric) && ((cur.metricParam == this.current[0].deviceMetric[j].param) || (this.current[0].deviceMetric[j].param.length && (cur.metricParam == this.current[0].deviceMetric[j].translatedParam))) && (cur.kind == this.current[0].deviceMetric[j].kind)) {
                var rates = data[i].rates;
                if(ADD_INFO) {
                  for(var k = 0; k < rates.length; k++) {
                    var info = rates[k].infoMessageIds;
                    this.current[0].data[j].push([rates[k].timeStampBegin, rates[k].rate, info]);
                    for(var ii = 0; ii < info.length; ii++) {
                      if(info[ii] > 0) allInfo.push(info[i]);
                    }
                    if(rates[k].timeStampEnd != rates[k].timeStampBegin) {
                      this.current[0].data[j].push([rates[k].timeStampEnd, rates[k].rate, info]);
                    }
                  }
                } else {
                  for(var k = 0; k < rates.length; k++) {
                    this.current[0].data[j].push([rates[k].timeStampBegin, rates[k].rate]);
                    if(rates[k].timeStampEnd != rates[k].timeStampBegin) {
                      this.current[0].data[j].push([rates[k].timeStampEnd, rates[k].rate]);
                    }
                  }
                }
                i++;
              }
              j++;
            }
          } else if(this.current[1][0] & 0x10000) { // product
            for(var i = 0; i < data.length; i++) {
              var cur = data[i].readMonDataId;
              var rates = data[i].rates;
              var j = this.current[1][0] & ~0x10000;
              var one = [cur.devId, (cur.metricId == this.current[0].deviceMetric[j].activePassiveMetric) && (cur.metricParam == this.current[0].deviceMetric[j].activePassiveParam), rates];
//               println(' ++++ KDR: Product ' + one[0] + ' : ' + one[1] + ' : ' + one[2].length);
              if(products.hasOwnProperty(j)) {
                products[j].push(one);
              } else {
                products[j] = [one];
              }
              if(ADD_INFO) {
                for(var k = 0; k < rates.length; k++) {
                  var info = rates[k].infoMessageIds;
                  for(var ii = 0; ii < info.length; ii++) {
                    if(info[ii] > 0) allInfo.push(info[i]);
                  }
                }
              }
            }
          } else { // sum/max/min/avg            
            var rates = data[i].rates;
            var j = this.current[1][0];
            if(ADD_INFO) {
              for(var k = 0; k < rates.length; k++) {
                var info = rates[k].infoMessageIds;
                this.current[0].data[j].push([rates[k].timeStampBegin, rates[k].rate, info]);
                for(var ii = 0; ii < info.length; ii++) {
                  if(info[ii] > 0) allInfo.push(info[i]);
                }
                if(rates[k].timeStampEnd != rates[k].timeStampBegin) {
                  this.current[0].data[j].push([rates[k].timeStampEnd, rates[k].rate, info]);
                }
              }
            } else {
              for(var k = 0; k < rates.length; k++) {
                this.current[0].data[j].push([rates[k].timeStampBegin, rates[k].rate]);
                if(rates[k].timeStampEnd != rates[k].timeStampBegin) {
                  this.current[0].data[j].push([rates[k].timeStampEnd, rates[k].rate]);
                }
              }
            }
          }
          for(var j in products) {
            var times = new Object();
            if(this.current[0].deviceMetric[j].isCheck) {
//               println(' ++++ KDR: Product ' + j + ' : ' + products[j].length);
              var map = new Object();
              var ll = -1;
              for(var j0 = 0; j0 < products[j].length; j0++) {
                var l0 = products[j][j0][2].length;
//                 println(' ++++ KDR: Product j0 = ' + j0 + ', l0 ' + l0);
                if(products[j][j0][1] && l0) {
                  for(var j1 = 0; j1 < products[j].length; j1++) {
                    if(!products[j][j1][1] && products[j][j1][2].length && (products[j][j0][0] == products[j][j1][0])) {
                      map[j0] = j1;
                      if(ll < 0) {
                        ll = l0;
                      } else if(ll != l0) {
                        ll = -1;
                        break;
                      }
                    }
                  }
                }
              }
//               println(' ++++ KDR: Product ll ' + ll);
              if(ll > 0) {
                var active = new Array();
                var indexes = new Object();
                var values = new Object();
                for(var j0 in map) {
                  var j1 = map[j0];
                  indexes[j1] = 0;
                  values[j1] = NaN;
//                   println(' ++++ KDR: Product map: ' + j0 + ' -> ' + j1);
                  var v1 = -1;
                  for(var k = 0; k < ll; k++) {
                    var t = products[j][j0][2][k].timeStampBegin;
                    var v0 = products[j][j0][2][k].rate;
//                     println('t = ' + t + ' : ' + j0 + ',' + j1 + ' = ' + v0);
                    if(kisNaN(v0)) {
                      active.push([t, -1]);
                    } else if(v1 >= 0) {
                      if((v1 >= v0) && ((v1 + v0) >= 1)) {
                        active.push([t, j1]);
                      }
                    } else if(v0 >= 0.5) {
                      active.push([t, j1]);
                    }
                    v1 = v0;
                  }
                }
                active.sort();
                for(var j0 = 0; j0 < active.length; j0++) {
                  var t = active[j0][0];
                  var j1 = active[j0][1];
                  if(j1 < 0) {
//                     println('t = ' + t + ' : ' + j1);
                    if(!times.hasOwnProperty(t)) times[t] = NaN;
                    continue;
                  }
//                   if(indexes[j1] < products[j][j1][2].length) {
//                     println('t = ' + t + ' : ' + j1 + ' , ' + indexes[j1] + ' < ' + products[j][j1][2].length + ' @ ' + products[j][j1][2][indexes[j1]].timeStampBegin);
//                   } else {
//                     println('t = ' + t + ' : ' + j1 + ' , ' + indexes[j1] + ' < ' + products[j][j1][2].length + ' @ end');
//                   }
                  while((indexes[j1] < products[j][j1][2].length) && (t >= products[j][j1][2][indexes[j1]].timeStampBegin)) {
                    values[j1] = products[j][j1][2][indexes[j1]].rate;                    
                    indexes[j1] += 1;
                  }
//                   println(' v = ' + values[j1]);
                  if(times.hasOwnProperty(t) && !isNaN(times[t])) {
                    println('Duplicate value for heatlthcheck product');
                  } else {
                    times[t] = values[j1];
                  }
                }
              }
            } else {
              var map = new Object();
              var ll = -1;
              for(var j0 = 0; j0 < products[j].length; j0++) {
                var l0 = products[j][j0][2].length;
                if(products[j][j0][1] && l0 && ((ll < 0) || (l0 == ll))) {
                  for(var j1 = 0; j1 < products[j].length; j1++) {
                    if(!products[j][j1][1] && products[j][j1][2].length && (l0 == products[j][j1][2].length)  && (products[j][j0][0] == products[j][j1][0])) {
                      map[j0] = j1;
                      ll = l0;
                    }
                  }
                }
              }
              if(ll > 0) {
                for(var j0 in map) {
                  var j1 = map[j0];
  //                 println(' ++++ KDR: Product map: ' + j0 + ' -> ' + j1);
                  for(var k = 0; k < ll; k++) {
                    var t = products[j][j0][2][k].timeStampBegin;
                    if(t == products[j][j1][2][k].timeStampBegin) {
                      var v0 = products[j][j0][2][k].rate;
                      var v1 = products[j][j1][2][k].rate;
                      if(!kisNaN(v0) && !kisNaN(v1)) {
                        var v = Math.max(0, v0) * v1;
                        if(times.hasOwnProperty(t)) {
                          times[t] += v;
                        } else {
                          times[t] = v;
                        }
                      }
                    }
                  }
                }
              }
            }
            for(var t in times) {
              this.current[0].data[j].push([t, times[t]]);
            }
            this.current[0].data[j].sort();            
          }
          if(allInfo.length > 0) this.cluster.ensureInfoMessage(allInfo); // TODO: NOT threaded!!
        } else {
          dock.jsonCallError(this, code, data);
        }
        this.current[0].dataIn++;
        if(this.current[0].dataIn >= this.current[0].dataWait) {
          this.current[0].redraw();
          this.current[0].last = now;
          this.current[0].updating = false;
          this.current[0].ownerWindow.unwaitIcon();
        }
      }, [this, allrmdi[i0]]);
    }
  }
}

cvGraph.prototype.leftSpace = function() {
//   for(var i = 0; i < this.deviceMetric.length; i++) {
//     if(!!this.deviceMetric[i]) {
//       var metric = this.deviceMetric[i].metric;
//       metric = this.cluster.find(metric);
//       if(metric) {
//         var name = metric.resolveName();
//         if(name == "NetworkUtilization") return 64;
//         var unit = metric.get('unit');
//         if(unit.length >= 3) return 64;
//       }
//     }
//   }
  return 64;
}

cvGraph.prototype.clearScreen = function() {
  while(this.box.firstChild) this.box.removeChild(this.box.firstChild);
}

cvGraph.prototype.needAxis = function() {
  return (this.deviceMetric.length != 1) || (this.deviceMetric[0].groupType != dock.GRAPH_GROUP_PIE);
}

cvGraph.prototype.needLegend = function() {
  return (this.deviceMetric.length != 1) || (this.deviceMetric[0].groupType != dock.GRAPH_GROUP_PIE);
}

cvGraph.prototype.draw = function(index, nx, ny, w, h) {
  var sx = w * (index % nx);
  var sy = h * Math.floor(index / nx);
  var s = Math.min(w, h);
  var x0 = sx + this.leftSpace();
  var x1 = w - 10;
  var y1 = 10;
  var y0 = h - 20;

  if(this.legend >= 2) {
    var l = 0;
    for(var i = 0; i < this.data.length; i++) {
      if(isObject(this.deviceMetric[i])) {
        var text = svgtext(this.box, 0, 0, this.fullTitle(i), SVG_FONT_SIZE);
        l = Math.max(l, text.getComputedTextLength());
        this.box.removeChild(text);
      }
    }
    x1 -= (l + 2 * SVG_FONT_SIZE);
  }
  this.drawbox = [x0, y1, x1, y0];
  
  while(this.box.firstChild) this.box.removeChild(this.box.firstChild);
  svgrect(this.box, x0, y1, x1 - x0, y0 - y1, "#f4f4f4", 1, "#d2d2d2"); // LINE WIDTH

  this.xrange = [this.startTime, this.endTime];
  var xunit = "time";
  var xaxis = autoIntAxis(this.xrange, xunit);
  if(this.needAxis()) svgxaxis(this.box, x0, y0, x1, y1, s, this.xrange, xaxis, w);

  var stackedLineSum = [];
  var stackedBarSum = [];
  var bars = 0;
  
  for(var i = 0; i < this.data.length; i++) {
    if(!isObject(this.deviceMetric[i])) continue;
    if(this.deviceMetric[i].isBar) {
      bars++;
      if(this.stacked) {
        for(var j = 0; j < this.data[i].length; j++) {
          if(!kisNaN(this.data[i][j][1])) {
            for(var k = stackedBarSum.length; k <= j; k++) stackedBarSum.push(0);
            stackedBarSum[j] = stackedBarSum[j] += this.data[i][j][1];
          }
        }
      }
    }
    // TODO: line sum
  }
  
  var metricCount = 0;
  var checkCount = 0;
  var l = this.xzoom.length - 1;
  this.yrange = [[MAX, -MAX], [MAX, -MAX], [MAX, -MAX], [MAX, -MAX]];
  for(var i = 0; i < this.data.length; i++) {
    if(!isObject(this.deviceMetric[i])) continue;
    
    var scale = this.scale[i];    
    if(this.deviceMetric[i].isCheck) {
      if(this.deviceMetric[i].fill) {
        checkCount++;
      } else {
        metricCount++;
      }
    } else if(!this.deviceMetric[i].isHeatmap()) {
      metricCount++;
      if(this.yrange[scale][0] == MAX) {
        var metric = this.deviceMetric[i].metric;
        if(metric) {
          metric = this.cluster.find(metric);
          if(metric && ((metric.get('minimum') != 0) || (metric.get('maximum') != 0))) {
            this.yrange[scale][0] = metric.get('minimum') * (this.stacked ? bars : 1);
            this.yrange[scale][1] = metric.get('maximum') * (this.stacked ? bars : 1);
          } else if(this.deviceMetric[i].isBar && stackedBarSum.length) {
            this.yrange[scale][0] = 0
            this.yrange[scale][1] = Math.max.apply(null, stackedBarSum);
          }
        }
      }
    }
    if(!this.deviceMetric[i].isHeatmap() && !this.deviceMetric[i].isPie() && (!this.deviceMetric[i].isCheck || !this.deviceMetric[i].fill)) {
      if((!this.yzoomBroadcast || (typeof(this.yzoomBroadcast[scale]) == 'undefined') || kisNaN(this.yzoomBroadcast[scale][0])) && kisNaN(this.yzoom[l][scale][0])) {
        for(var j = 0; j < this.data[i].length; j++) {
          if(!kisNaN(this.data[i][j][1])) {
            this.yrange[scale][0] = Math.min(this.yrange[scale][0], this.data[i][j][1]);
          }
        }
      } else if(this.yzoomBroadcast && typeof(this.yzoomBroadcast[scale]) != 'undefined') {
        this.yrange[scale][0] = this.yzoomBroadcast[scale][0];
      } else {
        this.yrange[scale][0] = this.yzoom[l][scale][0];
      }
      if((!this.yzoomBroadcast || (typeof(this.yzoomBroadcast[scale]) == 'undefined') || kisNaN(this.yzoomBroadcast[scale][1])) && kisNaN(this.yzoom[l][scale][1])) {
        var last = 1;
        if(this.deviceMetric[i].isGroup && this.deviceMetric[i].isBoxWhisker()) {
          last = 5;
        } else if(this.deviceMetric[i].isGroup && this.deviceMetric[i].isMinMedMax()) {
          last = 3;
        } else if(this.data[i].length > 0) {
          last = this.data[i][0].length - 1;
        }
        for(var j = 0; j < this.data[i].length; j++) {
          if(!kisNaN(this.data[i][j][last])) {
            this.yrange[scale][1] = Math.max(this.yrange[scale][1], this.data[i][j][last]);
          }
        }
      } else if(this.yzoomBroadcast && typeof(this.yzoomBroadcast[scale]) != 'undefined') {
        this.yrange[scale][1] = this.yzoomBroadcast[scale][1];
      } else {
        this.yrange[scale][1] = this.yzoom[l][scale][1];
      }
    } else if(this.deviceMetric[i].isHeatmap()) {
      var n = this.data[i].yBins.length;
      if(kisNaN(this.data[i].yBins[n - 1])) {
        this.yrange[scale] = [this.data[i].yBins[0], this.data[i].yBins[n - 2]];
      } else {
        this.yrange[scale] = [this.data[i].yBins[0], this.data[i].yBins[n - 1]];
      }
    }
  }
  if((metricCount == 0) && (checkCount > 0)) {
    this.yrange = [[0, 1], [0, 1], [0, 1], [0, 1]];
  } else {
    for(var i = 0; i < this.data.length; i++) {
      if(this.deviceMetric[i].isHeatmap()) continue;
      var scale = this.scale[i];
      if(scale == i) {
        var diff = this.yrange[scale][1] - this.yrange[scale][0];
        if(diff <= EPS) {
          if(Math.floor(this.yrange[scale][1]) == this.yrange[scale][1]) {
            diff = 1;
          } else {
            diff = 0.1;
          }
        }
        var avg = (this.yrange[scale][0] + this.yrange[scale][1]) / 2;
        if(kisNaN(this.yzoom[l][scale][0])) {
          var y = this.yrange[scale][0] - Math.max(diff * 0.1, avg / 10);
          if((y < EPS) && (this.yrange[scale][0] > -EPS)) {
            this.yrange[scale][0] = 0;
          } else {
            this.yrange[scale][0] = y;
          }
        }
        if(kisNaN(this.yzoom[l][scale][1])) {
          var y = this.yrange[scale][1] + Math.max(diff * 0.1, avg / 10);
          if((y > -EPS) && (this.yrange[scale][1] < EPS)) {
            this.yrange[scale][1] = 0;
          } else {
            this.yrange[scale][1] = y;
          }
        }
        if(((this.yrange[scale][0] == 0) && (this.yrange[scale][1] == 0)) || ((this.yrange[scale][0] == MAX) && (this.yrange[scale][1] == -MAX))) {
          this.yrange[scale][0] = 0;
          this.yrange[scale][1] = 1;
        }
      }
    }
  }

  var first = -1;
  for(var i = 0; i < this.deviceMetric.length; i++) {
    if(isObject(this.deviceMetric[i])) {
      if(!this.deviceMetric[i].isCheck) {
        first = i;
        break;
      }
    }
  }

  if(first >= 0) {
    var yunit = this.deviceMetric[first].unit; // All te same;
    if(yunit.length > 2) {
      var idx = yunit.indexOf('/');
      if(idx > 0) {
        yunit = yunit.substr(0, 1) + yunit.substr(idx, 2);
      } else {
        yunit = yunit.substr(0, 2);
      }
    }
    var yaxis = autoIntAxis(this.yrange[first], yunit);
    if(this.needAxis()) svgyaxis(this.box, x0, y0, x1, y1, s, this.yrange[first], yaxis);
  }

  var hasAverageFill = false;
  var defs = document.createElementNS(XUL_SVG, "svg:defs");
  for(var i = 0; i < this.data.length; i++) {
    if(this.deviceMetric[i].isAverage) {
      if(this.deviceMetric[i].fill) {
        hasAverageFill = true;
        var n = this.data[i][0].length - 1;
        for(var j = 1; j < n; j++) {
          var grad = document.createElementNS(XUL_SVG, "svg:linearGradient");
          grad.id = 'fill' + (this.index * 100 + j);
          grad.setAttribute('x1',"0%");
          grad.setAttribute('y1',"0%");
          grad.setAttribute('x2',"0%");
          grad.setAttribute('y2',"100%");
          var stop = document.createElementNS(XUL_SVG, "svg:stop");
          stop.setAttribute('offset', '0%');
//        var x = j / (n - 1);
          var x = 1 - Math.abs(2 * (j) / (n - 1) - 1);
          stop.setAttribute('stop-color', colorbar(x));
          stop.setAttribute('stop-opacity', 1);
          grad.appendChild(stop);

          var stop = document.createElementNS(XUL_SVG, "svg:stop");
          stop.setAttribute('offset', '100%');
//        var x = (j - 1) / (n - 1);
          var x = 1 - Math.abs(2 * (j - 1) / (n - 1) - 1);
          stop.setAttribute('stop-color', colorbar(x));
          stop.setAttribute('stop-opacity', 1);
          grad.appendChild(stop);

          defs.appendChild(grad);
        }
      }
    } else {
      var colors;
      if(this.deviceMetric[i].isGroup) {
        if(this.deviceMetric[i].isOperator() || (this.colors[i].length < 2)) {
          colors = this.colors[i][0];
        } else {
          colors = this.colors[i][2];
        }
      } else {
        colors = this.colors[i][0];
      }
      if(colors[1] != colors[2]) {
        var grad = document.createElementNS(XUL_SVG, "svg:linearGradient");
        grad.id = 'fill' + (this.index * 10 + i);
//       grad.setAttribute('gradientTransform', "rotate(0)");
        grad.setAttribute('x1',"0%");
        grad.setAttribute('y1',"0%");
        grad.setAttribute('x2',"0%");
        grad.setAttribute('y2',"100%");
        var stop = document.createElementNS(XUL_SVG, "svg:stop");
        stop.setAttribute('offset', '0%');
        stop.setAttribute('stop-color', colors[1]);
        stop.setAttribute('stop-opacity', 1);
        grad.appendChild(stop);
        var stop = document.createElementNS(XUL_SVG, "svg:stop");
        stop.setAttribute('offset', '100%');
        stop.setAttribute('stop-color', colors[2]);
        stop.setAttribute('stop-opacity', 1);
        grad.appendChild(stop);
        defs.appendChild(grad);
      }
    }
  }
  if(defs.childNodes.length > 0) this.box.appendChild(defs);

  for(var i = 0; i < stackedBarSum.length; i++) stackedBarSum[i] = 0;
  for(var i = 0; i < stackedLineSum.length; i++) stackedLineSum[i] = 0;
  
  var c = 0;
  var n = 0;
  var l = 0;
  var b = 0;
  for(var i = 0; i < this.data.length; i++) {
    if(isObject(this.deviceMetric[i])) {
      if(this.legend >= 0) {
        var text = svgtext(this.box, 0, 0, this.fullTitle(i), SVG_FONT_SIZE);
        l = Math.max(l, text.getComputedTextLength());
        this.box.removeChild(text);
      }
      n++;
      var scale = this.scale[i];
      if(this.deviceMetric[i].isGroup && this.deviceMetric[i].isHeatmap()) {
        svgheatmap(this.box, x0, y0, x1, y1, this.xrange, this.yrange[scale], this.data[i]);
      } else if(this.deviceMetric[i].isGroup && this.deviceMetric[i].isPie()) {
        this.pieInfo = svgpiechart(this.box, x0, y1, x1, y0, this.data[i], this.names, this.deviceMetric[i].unit, 100, 0.5); // max 100, or 0.5%
      } else if(this.deviceMetric[i].isGroup && this.deviceMetric[i].isBoxWhisker()) {
        svggraphboxwhisker(this.box, x0, y0, x1, y1, this.xrange, this.yrange[scale], this.data[i], this.colors[i][2][0], 1, this.colors[i][2][1], this.deviceMetric[i].fill, this.colors[i][2][2]);
      } else if(this.deviceMetric[i].isGroup && this.deviceMetric[i].isMinMedMax()) {
        if(this.deviceMetric[i].fill && false) {
          // TODO, remove false!
        } else {
          for(var im = 1; im <= 3; im++) {
            var data = [];
            for(var ix = 0; ix < this.data[i].length; ix++) {
              data.push([this.data[i][ix][0], this.data[i][ix][im]]);
            }
            svggraphline(this.box, x0, y0, x1, y1, this.xrange, this.yrange[scale], data, this.colors[i][3][im - 1], "none", 2);
          }
        }
      } else if(this.deviceMetric[i].isAverage) { // KDR: does nothing?
        if(this.deviceMetric[i].fill) {
          svggraphaverage(this.box, x0, y0, x1, y1, this.xrange, this.yrange[scale], this.data[i], this.index * 100);
        } else {
          if(this.data[i].length > 0) {
            var n = this.data[i][0].length;
            for(var j = 1; j < n; j++) {
              var tmp = new Array();
              for(var k = 0; k < this.data[i].length; k++) {
                tmp.push([this.data[i][k][0], this.data[i][k][j]]);
              }
              var x = 1 - Math.abs(2 * (j - 1) / (n - 2) - 1);
              svggraphline(this.box, x0, y0, x1, y1, this.xrange, this.yrange[scale], tmp, colorbar(x), "none", 2);
            }
          }
        }
      } else if(this.deviceMetric[i].isCheck && this.deviceMetric[i].fill) {
        svggraphblock(this.box, x0, y0, x1, y1, this.xrange, this.yrange[scale], this.data[i], c, checkCount, [this.colors[i][1][1], this.colors[i][1][0], this.colors[i][1][2]]);
        c++;
      } else if(this.deviceMetric[i].fill) {
        if(this.deviceMetric[i].isBar) {
          if(!this.stacked) stackedBarSum = [];
          b++;
          if(this.colors[i][0][1] != this.colors[i][0][2]) {
            svggraphbar(this.box, x0, y0, x1, y1, this.xrange, this.yrange[scale], this.data[i], this.colors[i][0][0], 'url(#fill' + (this.index * 10 + i) + ')', 1, b, this.stacked ? 1 : bars, stackedBarSum);
          } else {
            svggraphbar(this.box, x0, y0, x1, y1, this.xrange, this.yrange[scale], this.data[i], this.colors[i][0][0], this.colors[i][0][1], 1, b, this.stacked ? 1 : bars, stackedBarSum);
          }          
        } else {
          if(!this.stacked) stackedLineSum = []; // TODO
          if(this.colors[i][0][1] != this.colors[i][0][2]) {
            svggraphline(this.box, x0, y0, x1, y1, this.xrange, this.yrange[scale], this.data[i], this.colors[i][0][0], 'url(#fill' + (this.index * 10 + i) + ')', 1);
          } else {
            svggraphline(this.box, x0, y0, x1, y1, this.xrange, this.yrange[scale], this.data[i], this.colors[i][0][0], this.colors[i][0][1], 1);
          }
        }
      } else {
        if(this.deviceMetric[i].isBar) {
          if(!this.stacked) stackedBarSum = [];
          b++;
          svggraphbar(this.box, x0, y0, x1, y1, this.xrange, this.yrange[scale], this.data[i], this.colors[i][0][0], "none", 2, b, this.stacked ? 1 : bars, stackedBarSum);
        } else {
          if(!this.stacked) stackedLineSum = []; // TODO
          svggraphline(this.box, x0, y0, x1, y1, this.xrange, this.yrange[scale], this.data[i], this.colors[i][0][0], "none", 2);
        }
      }
    }
  }
  if(this.zoombox) this.box.appendChild(this.zoombox);
  if(this.legend >= 0 && this.needLegend()) {
    var bs = 1.5;
    var xl, yl, wl = l + SVG_FONT_SIZE * 2, hl = n * SVG_FONT_SIZE + 6;
    switch(this.legend) {
      case(0) : {
        xl = x0 + 10;
        yl = y1 + 10;        
        break;
      } case(1): {
        xl = x1 - wl - 6;
        yl = y1 + 10;
        break;
      } case(2): {
        xl = x1 + 8;
        yl = y1 + 2 * bs;
        break;
      } case(3): {
        xl = x1 + 8;
        yl = y0 - hl + 2 * bs;
        break;
      }
    }        
    svgrect(this.box, xl - 2 * bs, yl - 2 * bs, wl, hl, "#f8f8f8", 1, "#7e7e7e"); // LINE WIDTH
    var boxsize = SVG_FONT_SIZE - 2 * bs;
    for(var i = 0; i < this.data.length; i++) {
      if(isObject(this.deviceMetric[i])) {
        if(this.deviceMetric[i].isGroup) {
          if(this.deviceMetric[i].isOperator()) {
            if(this.deviceMetric[i].fill) {
              if(this.colors[i][0][1] != this.colors[i][0][2]) {
                svgrect(this.box, xl + bs, yl + bs, boxsize, boxsize, 'url(#fill' + (this.index * 10 + i) + ')', 1, this.colors[i][0][0]);
              } else {
                svgrect(this.box, xl + bs, yl + bs, boxsize, boxsize, this.colors[i][0][1], 1, this.colors[i][0][0]);
              }
            } else {
              svgrect(this.box, xl + bs, yl + bs, boxsize, boxsize, this.colors[i][0][0], 1);
            }            
          } else if(this.deviceMetric[i].isHeatmap()) {
            // TODO
            svgrect(this.box, xl + bs, yl + bs, boxsize, boxsize, this.colors[i][2][1], 1, this.colors[i][2][0]);
          } else {            
            svgrect(this.box, xl + bs, yl + bs, boxsize, boxsize, this.colors[i][2][1], 1, this.colors[i][2][0]);
          }
        } else if(this.deviceMetric[i].isAverage) {
          svgrect(this.box, xl + bs, yl + bs, boxsize, boxsize, this.colors[i][1][0], 1, this.colors[i][1][1]);
        } else if(this.deviceMetric[i].isCheck && this.deviceMetric[i].fill) {
          svgrect(this.box, xl + bs + (0 * boxsize / 3), yl + bs, boxsize / 3, boxsize, this.colors[i][1][1], 0);
          svgrect(this.box, xl + bs + (1 * boxsize / 3), yl + bs, boxsize / 3, boxsize, this.colors[i][1][0], 0);
          svgrect(this.box, xl + bs + (2 * boxsize / 3), yl + bs, boxsize / 3, boxsize, this.colors[i][1][2], 0);
        } else if(this.deviceMetric[i].fill) {
          if(this.colors[i][0][1] != this.colors[i][0][2]) {
            svgrect(this.box, xl + bs, yl + bs, boxsize, boxsize, 'url(#fill' + (this.index * 10 + i) + ')', 1, this.colors[i][0][0]);
          } else {
            svgrect(this.box, xl + bs, yl + bs, boxsize, boxsize, this.colors[i][0][1], 1, this.colors[i][0][0]);
          }
        } else {
          svgrect(this.box, xl + bs, yl + bs, boxsize, boxsize, this.colors[i][0][0], 1);
        }
        svgtext(this.box, xl + SVG_FONT_SIZE * 1.25, yl + SVG_FONT_SIZE - bs, this.fullTitle(i), SVG_FONT_SIZE, 0, '5e5e5e');
        yl += SVG_FONT_SIZE;
      }
    }
  }
}
