/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var dock;
var myDialog;
var graph;

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  graph = myDialog.arguments.graph;

  if(parseInt(graph.cluster.versionInfo.cmdaemonBuild) < 17984) {
    document.getElementById('sumw').setAttribute('hidden', true);
    document.getElementById('sumhw').setAttribute('hidden', true);
    for(var i = 0; i < graph.colors.length; i++) {
      document.getElementById('sum' + i + 'w').parentNode.setAttribute('hidden', true);
    }
  }
  
  refresh();
}

function refresh() {
  var intervals = [[0, 'Best'], 10, 25, 50, 100, 200, 500, 1000];
  var rates = [[60, '1 minute'], [120, '2 minutes'], [300, '5 minutes'], [600, '10 minutes'], [1800, '30 minutes'], [3600, '1 hour']];
  var timeList = [[3600, '1 hour'], [43200, '12 hours'], [86400, '1 day'], [259200, '3 days'],
                  [608400, '7 days'], [2678400, '1 month'], [31526000, '1 year'], [94578000, '3 years']];
  var outliers = [[-1, '<auto>'], [0, '0'], [1, '1'], [2, '2'], [4, '4'], [10, '10'], [50, '50'], [100, '100']];
  var legends = [[-1, 'None'], [0, 'Top left'], [1, 'Top right'], [2, 'Top right outside'], [3, 'Bottom right outside']];
  var myTime = 3600;
  var groupTypes = [[dock.GRAPH_GROUP_WHISKER, 'BoxWhisker'], [dock.GRAPH_GROUP_MIN_MED_MAX, 'Min, median, max'],
                    [dock.GRAPH_GROUP_MAX, 'Maximum'], [dock.GRAPH_GROUP_MIN, 'Minimum'], 
                    [dock.GRAPH_GROUP_SUM, 'Sum'], [dock.GRAPH_GROUP_AVG, 'Average']];
  var basicTypes = [[0, 'Line'], [1, 'Bar']];
  if(graph.deviceMetric.length == 1) {
    groupTypes.push([dock.GRAPH_GROUP_HEATMAP, 'Heatmap']);
    groupTypes.push([dock.GRAPH_GROUP_PIE, 'Pie chart']);
  }

  document.getElementById('title').setAttribute('value', graph.title);
  if((graph.xzoom[graph.xzoom.length - 1][1] == 0) && (graph.xzoom[graph.xzoom.length - 1][0] >= 0)) {
    setTimeType(1);
    var d = new Date();

    datetext('enddate', DATETIME_FORMAT, d);
    myTime = graph.xzoom[graph.xzoom.length - 1][0];
    d.setTime(d.getTime() - myTime * 1000);
    for(var i = 0; i < timeList.length; i++) {
      if(timeList[i][0] == myTime) break;
      if(timeList[i][0] > myTime) {
        timeList.splice(i, 0, [myTime, formatDHM(myTime)]);
        break;
      }
    }
    datetext('startdate', DATETIME_FORMAT, d);
  } else if(graph.xzoom[graph.xzoom.length - 1][1] == 0) {
    setTimeType(0);
    var d = new Date();
    datetext('enddate', DATETIME_FORMAT, d);
    var d = new Date();
    d.setTime(-graph.xzoom[graph.xzoom.length - 1][0] * 1000);
    datetext('startdate', DATETIME_FORMAT, d);
  } else {
    setTimeType(0);
    var d = new Date();
    d.setTime(graph.xzoom[graph.xzoom.length - 1][1] * 1000);
    datetext('enddate', DATETIME_FORMAT, d);
    var d = new Date();
    d.setTime(graph.xzoom[graph.xzoom.length - 1][0] * 1000);
    datetext('startdate', DATETIME_FORMAT, d);
  }
  var popup = document.getElementById('lasttime').firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  for(var i = 0; i < timeList.length; i++) {
    var item = document.createElement('menuitem');
    item.setAttribute('label', timeList[i][1]);
    item.setAttribute('time', timeList[i][0]);
    popup.appendChild(item);
    if(timeList[i][0] == myTime) document.getElementById('lasttime').selectedIndex = i;
  }

  var popup = document.getElementById('legend').firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  for(var i = 0; i < legends.length; i++) {
    var item = document.createElement('menuitem');
    var val = legends[i];
    if(isArray(legends[i])) {
      item.setAttribute('legend', legends[i][0]);
      item.setAttribute('label', legends[i][1]);
      val = legends[i][0];
    } else {
      item.setAttribute('label', legends[i]);
      item.setAttribute('legend', legends[i]);
    }
    popup.appendChild(item);
    if(val == graph.legend) document.getElementById('legend').selectedIndex = i;
  }
  
  for(var i = 0; i < intervals.length; i++) {
    if(timeList[i][0] == graph.intervals) break;
    if(timeList[i][0] > graph.intervals) {
      timeList.splice(i, 0, graph.intervals);
      break;
    }
  }
  var popup = document.getElementById('intervals').firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  for(var i = 0; i < intervals.length; i++) {
    var item = document.createElement('menuitem');
    var val = intervals[i];
    if(isArray(intervals[i])) {
      item.setAttribute('intervals', intervals[i][0]);
      item.setAttribute('label', intervals[i][1]);
      val = intervals[i][0];
    } else {
      item.setAttribute('label', intervals[i]);
      item.setAttribute('intervals', intervals[i]);
    }
    popup.appendChild(item);
    if(val == graph.intervals) document.getElementById('intervals').selectedIndex = i;
  }

  for(var i = 0; i < rates.length; i++) {
    if(rates[i][0] == graph.refreshRate) break;
    if(rates[i][0] > graph.refreshRate) {
      rates.splice(i, 0, [graph.refreshRate, formatDHM(graph.refreshRate)]);
      break;
    }
  }
  var popup = document.getElementById('rate').firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  for(var i = 0; i < rates.length; i++) {
    var item = document.createElement('menuitem');
    item.setAttribute('label', rates[i][1]);
    item.setAttribute('time', rates[i][0]);
    popup.appendChild(item);
    if(rates[i][0] == graph.refreshRate) document.getElementById('rate').selectedIndex = i;
  }

  document.getElementById('stacked').checked = graph.stacked;
  
  var hc = 0;
  var mc = 0;
  var wc = 0;
  var mamc = 0;
  for(var i = 0; i < graph.colors.length; i++) {
    if(isObject(graph.deviceMetric[i])) {
      var all = ['', 'h', 'w', 'mam'];
      var y = ''; 
      if(graph.deviceMetric[i].isCheck) {
        y = 'h';
        hc++;
      } else if(graph.deviceMetric[i].isMinMedMax()) {
        y = 'mam';
        mamc++;
        var popup = document.getElementById('type' + i + y).firstChild;
        while(popup.firstChild) popup.removeChild(popup.firstChild);
        for(var j = 0; j < groupTypes.length; j++) {
          var item = document.createElement('menuitem');
          item.setAttribute('label', groupTypes[j][1]);
          item.setAttribute('type', groupTypes[j][0]);
          popup.appendChild(item);
          if(groupTypes[j][0] == graph.deviceMetric[i].groupType) popup.parentNode.selectedIndex = j;
        }
      } else if(graph.deviceMetric[i].isGroup) {
        y = 'w';
        wc++;
        var popup = document.getElementById('outliers' + i + y).firstChild;
        while(popup.firstChild) popup.removeChild(popup.firstChild);
        for(var j = 0; j < outliers.length; j++) {
          var item = document.createElement('menuitem');
          item.setAttribute('label', outliers[j][1]);
          item.setAttribute('outlier', outliers[j][0]);
          popup.appendChild(item);
          if(outliers[j][0] == graph.deviceMetric[i].outliers) popup.parentNode.selectedIndex = j;
        }
        var popup = document.getElementById('type' + i + y).firstChild;
        while(popup.firstChild) popup.removeChild(popup.firstChild);
        for(var j = 0; j < groupTypes.length; j++) {
          var item = document.createElement('menuitem');
          item.setAttribute('label', groupTypes[j][1]);
          item.setAttribute('type', groupTypes[j][0]);
          popup.appendChild(item);
          if(groupTypes[j][0] == graph.deviceMetric[i].groupType) popup.parentNode.selectedIndex = j;
        }
      } else {
        var popup = document.getElementById('type' + i + y).firstChild;
        while(popup.firstChild) popup.removeChild(popup.firstChild);
        for(var j = 0; j < basicTypes.length; j++) {
          var item = document.createElement('menuitem');
          item.setAttribute('label', basicTypes[j][1]);
          item.setAttribute('type', basicTypes[j][0]);
          popup.appendChild(item);
          if(basicTypes[j][0] == graph.deviceMetric[i].isBar) popup.parentNode.selectedIndex = j;
        }
        mc++;
      }
      var metric = graph.cluster.resolveName(graph.deviceMetric[i].metric);
      if(graph.deviceMetric[i].param) metric += ' (' + graph.deviceMetric[i].param + ')';
      document.getElementById('device' + i + y).setAttribute('value', graph.cluster.resolveName(graph.deviceMetric[i].device));
      document.getElementById('metric' + i + y).setAttribute('value', metric);
      document.getElementById('fill' + i + y).checked = graph.deviceMetric[i].fill;
      document.getElementById('graph' + i + y).setAttribute('hidden', false);
      for(var j = 0; j < all.length; j++) {
        if(y != all[j]) {
          document.getElementById('graph' + i + all[j]).setAttribute('hidden', true);
        }
      }
    } else {
      for(var j = 0; j < all.length; j++) {
        document.getElementById('graph' + i + all[j]).setAttribute('hidden', true);
      }
    }
    document.getElementById('colorStart' + i).color = graph.colors[i][0][1];
    document.getElementById('colorEnd' + i).color = graph.colors[i][0][2];
    document.getElementById('colorLine' + i).color = graph.colors[i][0][0];
    document.getElementById('colorStart' + i + 'h').color = graph.colors[i][1][1];
    document.getElementById('colorEnd' + i + 'h').color = graph.colors[i][1][2];
    document.getElementById('colorLine' + i + 'h').color = graph.colors[i][1][0];
    if(isObject(graph.deviceMetric[i]) && (graph.deviceMetric[i].isOperator())) {
      document.getElementById('colorStart' + i + 'w').color = graph.colors[i][0][0];
      document.getElementById('colorEnd' + i + 'w').color = graph.colors[i][0][1];
      document.getElementById('colorLine' + i + 'w').color = graph.colors[i][0][1]; // KDR: correct
    } else if(isObject(graph.deviceMetric[i]) && (graph.deviceMetric[i].isMinMedMax())) {
      document.getElementById('colorStart' + i + 'mam').color = graph.colors[i][3][0];
      document.getElementById('colorEnd' + i + 'mam').color = graph.colors[i][3][1];
      document.getElementById('colorLine' + i + 'mam').color = graph.colors[i][3][2];
    } else {
      document.getElementById('colorStart' + i + 'w').color = graph.colors[i][2][0];
      document.getElementById('colorEnd' + i + 'w').color = graph.colors[i][2][1];
      document.getElementById('colorLine' + i + 'w').color = graph.colors[i][2][2];
    }
  }
  document.getElementById('metricstrip').setAttribute('hidden', (mc == 0));
  document.getElementById('healthstrip').setAttribute('hidden', (hc == 0));
  document.getElementById('healthstrip').setAttribute('class', ((mc == 0) ? 'datastripeven' : 'datastripodd'));
  document.getElementById('whiskerstrip').setAttribute('hidden', (wc == 0));
  document.getElementById('whiskerstrip').setAttribute('class', (!(mc == 0) ^ (hc == 0) ? 'datastripeven' : 'datastripodd'));
  document.getElementById('minmedmaxstrip').setAttribute('hidden', (mamc == 0));
  document.getElementById('minmedmaxstrip').setAttribute('class', (!(mc == 0) ^ (hc == 0) ^ (wc == 0) ? 'datastripeven' : 'datastripodd'));
}

function setTimeType(index) {
  document.getElementById('between').setAttribute('on', index == 0);
  document.getElementById('last').setAttribute('on', index == 1);
}

function setDate(name) {
  var args = new Object();
  var t = document.getElementById(name).getAttribute('time');
  var d = new Date();
  d.setTime(t * 1000);
  args['date'] = d;
  var d = new dock.cvDialog("datepickerwindow.xul", args);
  d.parentWindow = window;
  d.onOk = function(date) {
    document.getElementById(name).setAttribute('value', date.format(DATETIME_FORMAT));
    document.getElementById(name).setAttribute('time', Math.floor(date.getTime() / 1000));
  }
  dock.dialogs.showDialog(d);
}

function removeMetric(index) {
  document.getElementById('device' + index).setAttribute('value', '');
  document.getElementById('metric' + index).setAttribute('value', '');
  document.getElementById('fill' + index).setAttribute('hidden', true);
  document.getElementById('graph' + index).setAttribute('hidden', true);
}

function ok() {
  apply();
  cancel();
}

function apply() {
  var updateTitle = true;
  if(graph.title != document.getElementById('title').value) {
    graph.title = document.getElementById('title').value;
    updateTitle = false;
  }
  for(var i = 0; i < graph.deviceMetric.length; i++) {
    var y = '';
    if(graph.deviceMetric[i].isCheck) {
      y = 'h';
    } else if(graph.deviceMetric[i].isMinMedMax()) {
      y = 'mam';
    } else if(graph.deviceMetric[i].isGroup) {
      y = 'w';
    }
    if(graph.deviceMetric[i].isGroup) {
      var item = document.getElementById('type' + i + y).selectedItem;
      if(item) graph.deviceMetric[i].groupType = parseInt(item.getAttribute('type'));
    } else {
      var item = document.getElementById('type' + i).selectedItem;
      if(item) graph.deviceMetric[i].isBar = parseInt(item.getAttribute('type'));
    }
    graph.deviceMetric[i].fill = document.getElementById('fill' + i + y).checked;
    if(eval(document.getElementById('fill' + i + y).getAttribute('hidden'))) {
      graph.remove(i, updateTitle);
      if(updateTitle) document.getElementById('title').value = graph.title;
    }
  }
  var index = graph.xzoom.length - 1;
  if(index >= 0) {
    var old0 = graph.xzoom[index][0];
    var old1 = graph.xzoom[index][1];
    if(eval(document.getElementById('between').getAttribute('on'))) {
      graph.xzoom[index][0] = parseInt(document.getElementById('startdate').getAttribute('time'));
      graph.xzoom[index][1] = parseInt(document.getElementById('enddate').getAttribute('time'));
    } else {
      var item = document.getElementById('lasttime').selectedItem;
      if(item) {
        graph.xzoom[index][0] = item.getAttribute('time');
        graph.xzoom[index][1] = 0;
      }
    }
    if((old0 != graph.xzoom[index][0]) || (old1 != graph.xzoom[index][1])) {
      if((myDialog.parentWindow.broadcast >= 0) && (myDialog.parentWindow.broadcast == graph.index)) {
        myDialog.parentWindow.setZoomBroadcast(graph.index);
      }
    }
  }

  var item = document.getElementById('rate').selectedItem;
  if(item) graph.setRefreshRate(parseInt(item.getAttribute('time')));
  var item = document.getElementById('intervals').selectedItem;
  if(item) graph.intervals = parseInt(item.getAttribute('intervals'));
  var item = document.getElementById('legend').selectedItem;
  if(item) graph.legend = parseInt(item.getAttribute('legend'));
  var item = document.getElementById('stacked');
  if(item) graph.stacked = item.checked;

  for(var i = 0; i < graph.colors.length; i++) {
    if(isObject(graph.deviceMetric[i])) {
      if(graph.deviceMetric[i].isCheck) {
        graph.colors[i][1][1] = document.getElementById('colorStart' + i + 'h').color;
        graph.colors[i][1][2] = document.getElementById('colorEnd' + i + 'h').color;
        graph.colors[i][1][0] = document.getElementById('colorLine' + i + 'h').color;
      } else if(graph.deviceMetric[i].isGroup) {
        if(graph.deviceMetric[i].isOperator()) {
          graph.colors[i][0][1] = document.getElementById('colorEnd' + i + 'w').color;
          graph.colors[i][0][2] = document.getElementById('colorEnd' + i + 'w').color; // KDR: correct
          graph.colors[i][0][0] = document.getElementById('colorStart' + i + 'w').color;
        } else if(graph.deviceMetric[i].isMinMedMax()) {
          graph.colors[i][3][0] = document.getElementById('colorStart' + i + 'mam').color;
          graph.colors[i][3][1] = document.getElementById('colorEnd' + i + 'mam').color;
          graph.colors[i][3][2] = document.getElementById('colorLine' + i + 'mam').color;
        } else {
          graph.colors[i][2][0] = document.getElementById('colorStart' + i + 'w').color;
          graph.colors[i][2][1] = document.getElementById('colorEnd' + i + 'w').color;
          graph.colors[i][2][2] = document.getElementById('colorLine' + i + 'w').color;
        }
        var item = document.getElementById('outliers' + i + 'w').selectedItem;
        if(item) graph.deviceMetric[i].outliers = parseInt(item.getAttribute('outlier'));
      } else if(!graph.deviceMetric[i].isOperator()) {
        graph.colors[i][0][1] = document.getElementById('colorStart' + i).color;
        graph.colors[i][0][2] = document.getElementById('colorEnd' + i).color;
        graph.colors[i][0][0] = document.getElementById('colorLine' + i).color;
      }
    }
  }
  graph.update();
}

function cancel() {
  close();
}

function changeType(event, i) {
  var index = event.target.getAttribute('type');
  if((index == dock.GRAPH_GROUP_WHISKER) || (index == dock.GRAPH_GROUP_HEATMAP)) {
    document.getElementById('colorStart' + i + 'w').color = graph.colors[i][2][0];
    document.getElementById('colorEnd' + i + 'w').color = graph.colors[i][2][1];
    document.getElementById('colorLine' + i + 'w').color = graph.colors[i][2][2];
  } else if(index == dock.GRAPH_GROUP_MIN_MED_MAX) {
    document.getElementById('colorStart' + i + 'mam').color = graph.colors[i][3][0];
    document.getElementById('colorEnd' + i + 'mam').color = graph.colors[i][3][1];
    document.getElementById('colorLine' + i + 'mam').color = graph.colors[i][3][2];
  } else {
    document.getElementById('colorStart' + i + 'w').color = graph.colors[i][0][1];
    document.getElementById('colorEnd' + i + 'w').color = graph.colors[i][0][2];
    document.getElementById('colorLine' + i + 'w').color = graph.colors[i][0][0];
  }
}

function changeBasicType(event, i) {
}

