/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */


 // this file relies on these files to be previously loaded:
 // . hadoop/objecthelpers.js
 // . hadoop/addbigdatainstancewizard.js
 // (note that the requisites of those files aren't listed here; check them to know what they require). 

function CassandraBigDataWizardBuilder() {
  this.toString = function() {
    return "Cassandra";
  }
  this.getRemoteCallParameters = function()  {
    var remoteCallParameters = {};
    remoteCallParameters.service        = "cmhadoop";
    remoteCallParameters.remoteCall     = "executeHadoopSetup";
    remoteCallParameters.remoteCallType = 2; // 0 - Hadoop, 1 - Spark, 2 - Cassandra; check executeHadoopSetup at src/autogen/services/cmhadoop.xml in cmdaemon.
    return remoteCallParameters;
  }

  var userChoices = {};
  this.getUserChoices = function() {
    return userChoices;
  }

  this.mapElementIdUserChoices = {
    "main" : {
      "dialogtitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Cassandra main details"
      },
      "mainMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "Please set the name of the instance and the Java home directory. Specify where to find the Cassandra tarball and the Jolokia jar; below are suggested the most recent ones, which are provided by the package cm-apache-hadoop-extras and also available at\nhttp://support.brightcomputing.com/bigdata/"
      },
      "instanceName" : {
        validate_as         : "instanceName",
        humanReadable       : "Instance name",
        usersChoicesEntry   : "instanceName",
        attributeToSet      : "value",
        attributeToSetValue : "cass1"
      },
      "cassandraBigDataWizardTarball" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "Cassandra tarball",
        usersChoicesEntry   : "cassandraTarball",
        checkFileOnHeadnode : true,
        attributeToSet      : "value",
        attributeToSetValue : CM_LOCAL_PREFIX + NEWEST_CASSANDRA
      },
      "cassandraBigDataWizardJolokiaJar" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "Jolokia jar",
        usersChoicesEntry   : "jolokiaJar",
        checkFileOnHeadnode : true,
        attributeToSet      : "value",
        attributeToSetValue : CM_LOCAL_PREFIX + NEWEST_JOLOKIA_FOR_CASSANDRA
      },
      "useDefaultJavaHome" : {
        humanReadable       : "Use default Java home",
        usersChoicesEntry   : "useDefaultJavaHome",
        attributeToSet      : "value",
      },
      "javaHome" : {
        validate_as         : validateJavaHome.bind(this, 'useDefaultJavaHome', 'defaultJavaHome', 'customJavaHome', userChoices),
        humanReadable       : "Java home",
        usersChoicesEntry   : "javaHome",
        attributeToSet      : "value",
        // will use extraSteps for complex setDefaultJavaHome('javaHome'); instead of setting attributeToSetValue.
      },
      "secondMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "Please set the directories for Cassandra. Multiple Data File Directories can be set using a comma to separate them. Example: '/data1,/data2'"
      },
      "cassandraBigDataWizardCommitlogDir" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "Commit Log Directory",
        usersChoicesEntry   : "commitlog",
        attributeToSet      : "value",
        attributeToSetValue : "/var/lib/cassandra/cass1/commitlog"
      },
      "cassandraBigDataWizardDataFileDirs" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "Data File Directories",
        usersChoicesEntry   : "datafile",
        attributeToSet      : "value",
        attributeToSetValue : "/var/lib/cassandra/cass1/data"
      },
      "cassandraBigDataWizardSavedCachesDir" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "Saved Caches Directory",
        usersChoicesEntry   : "savedcaches",
        attributeToSet      : "value",
        attributeToSetValue : "/var/lib/cassandra/cass1/saved_caches"
      },
      "cassandraBigDataWizardHintsDir" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "Hints Directory",
        usersChoicesEntry   : "hints",
        attributeToSet      : "value",
        attributeToSetValue : "/var/lib/cassandra/cass1/hints"
      },
    }, // main

    "nodesDeck" : {
      "dialogtitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Cassandra nodes selection"
      },
      "nodesMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "Please choose at least one node for Cassandra."
      },
      "groupnodesNodes" : {
        humanReadable       : "Cassandra nodes",
        usersChoicesEntry   : "nodes",
      },
    }, // nodes

    "seedsDeck" : {
      "dialogtitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Cassandra seeds selection"
      },
      "seedsMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "Please choose at least one node as seed for Cassandra. A common production environment uses 2 seeds. The seeds should be a subset of the nodes chosen in the previous page."
      },
      "groupnodesNodes" : {
        humanReadable       : "Cassandra seeds",
        usersChoicesEntry   : "seeds",
      },
    }, // seeds
    
    "paramsDeck" : {
      "dialogtitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Cassandra parameters"
      },
      "paramsMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "This page shows most common Cassandra configuration parameters.\nPlease choose the number of vnodes, the snitch, and Java settings.\nMAX_HEAP_SIZE is the total amount of memory dedicated to the Java heap. Both 'Maximum heap size' and 'Young generation size' should be either set or not (if you set one, set the other)."
      },
      "numTokens" : {
        validate_as         : "integerBiggerThanZero",
        humanReadable       : "Number of tokens",
        usersChoicesEntry   : "numTokens",
        attributeToSet      : "value",
        attributeToSetValue : "256"
      },
      "maxHeapSize" : {
        validate_as         : "memorySettingOrEmpty",
        humanReadable       : "Maximum heap size",
        usersChoicesEntry   : "maxHeapSize",
        attributeToSet      : "value",
        attributeToSetValue : ""
      },
      "maxHeapSizeLabel" : {
        attributeToSet      : "value",
        attributeToSetValue : "e.g. 4G (leave empty for automatic calculation)"
      },
      "heapNewsize" : {
        validate_as         : "memorySettingOrEmpty",
	humanReadable       : "Young generation size",
        usersChoicesEntry   : "heapNewsize",
        attributeToSet      : "value",
        attributeToSetValue : ""
      },
      "heapNewsizeLabel" : {
        attributeToSet      : "value",
        attributeToSetValue : "e.g. 800M (leave empty for automatic calculation)"
      },
      "snitchMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "Cassandra's replication strategy relies on the information provided by the endpoint snitch.\nNote: Changing snitches after Cassandra deployment may require some maintenance operations."
      },
      "endpointSnitch" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "Endpoint snitch",
        usersChoicesEntry   : "endpointSnitch",
        attributeToSet      : "selectedItem",
        transformations     : [ {"name": "elementGetSubElement", "accessor":  ".value"} ]
      },
    },
    
    "summaryDeck" : {
      "dialogtitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Summary of choices"
      },
      "summaryChoicesMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "The selected choices are shown below. They will be used to create an XML configuration file.\n",
      },
      "summaryMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "\n\nBy pressing 'Next' a temporary XML file will be created with the selected configuration and a Cassandra instance will be installed using this XML file.\n\nAlternatively, clicking the 'Save Configuration' button will produce an XML configuration file which can be further customized, if needed, and then used as the input configuration file for the cm-cassandra-setup command line utility.\nPress 'Show Configuration' to get the Cassandra deployment configuration as XML.",
      },
      "prerequisitesHadoopDialogTitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Prerequisites checking Cassandra",
      },
      "prerequisitesHadoopMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : 'Before starting the installer two things will be checked:\n' +
                              '- The JRE folder should contain the Java executable (bin/java) on all nodes that are part of this Hadoop instance.\n' +
                              '- The required tarballs should exist on the head node(s).\n'
      },
    }, // summaryDeck
    "deployDeck" : {
      "dialogtitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Deploy details"
      },
    } // deployDeck
  }

  this.deckStateMachine = {
    "main"          : {
      "elementId"     : "main", 
      "next"          : {
        "nodesDeck" : true
      }, 
      "extraSteps"    : [ "setDefaultJava180Home('defaultJavaHome', 'useDefaultJavaHome');"],
      "beforeNext"    : [ "setJavaHomeToUse(userChoices);" ]
    }, // main
    
    "nodesDeck"       : {
      "elementId"     : "cassandraNodes", 
      "previous"      : {
        "main"   : true
      },
      "previous"      : {
        "main"          : true
      },
      "nodesRules"    : [
        { "condition": " < 1", "errorMessage": "At least one host should be selected"},
      ],
      "extraSteps"    : [ "displayNodesSelection('Nodes');"],  // must match userChoices of this.nodesHostsNode (and have allnodesNodes, groupnodesNodes in xul file) 
      "next"          : {
        "seedsDeck" : true
      }, 
    }, // nodesDeck
    
    "seedsDeck"       : {
      "elementId"     : "seeds", 
      "previous"      : {
        "nodesDeck"   : true
      },
      "nodesRules"    : [
        { "condition": " < 1", "errorMessage": "At least one host should be selected"},
      ],
      "extraSteps"    : [ "displayNodesSelection('Seeds');"],  // must match userChoices of this.seedsHostsNode (and have allnodesSeeds, groupnodesSeeds in xul file) 
      "next"          : {
        "paramsDeck"   : "checkNodesSeeds('Seeds', 'Nodes', 'The set of seeds should be a subset of the nodes chosen for Cassandra.')"
      }, 
    }, // seedsDeck
    
    "paramsDeck"       : {
      "elementId"     : "params", 
      "previous"      : {
        "seedsDeck"   : true
      },
      "next"          : {
        "summaryDeck" : "wizardType.checkJavaParams()"
      }, 
    }, // seedsDeck
    
    "summaryDeck"   : {
      "elementId"     : "summary", 
      "previous"      : {
        "paramsDeck"   : true
      },
      "next"          : {
        "deployDeck"   : "remoteRequisitesSatisfiedOrPrompt()"
      }, 
      "extraSteps"    : [ " printUserChoicesOnElementId('summaryChoices');", "displayPreconditionsCheckAtElement('verificationSteps');"]
    }, // summaryDeck 
    "deployDeck"   : {
        "elementId"     : "deploy", 
        "previous"      : {
          "summaryDeck"   : true
        },
      "extraSteps"    : [ "deploy();"]
    } // deployDeck
  }

  this.cassandraArchiveXMLNode = {
    "tagName"                        : "archive",
    "text"                           : "userChoices[mapElementIdUserChoices.main.cassandraBigDataWizardTarball.usersChoicesEntry]",
    "textEval"                       : true,
  }
  this.jolokiaJarNode = {
    "tagName"                        : "jolokiajar",
    "text"                           : "userChoices[mapElementIdUserChoices.main.cassandraBigDataWizardJolokiaJar.usersChoicesEntry]",
    "textEval"                       : true,
  }
  this.javaHomeXMLNode = {
    "tagName"                        : "javahome",
    "text"                           : "userChoices[mapElementIdUserChoices.main.javaHome.usersChoicesEntry]",
    "textEval"                       : true,
  }
  this.instanceNameNode = {
    "tagName"                        : "name",
    "text"                           : "userChoices[mapElementIdUserChoices.main.instanceName.usersChoicesEntry]",
    "textEval"                       : true,
  }
  this.nodesHostsNode = {
    "tagName"                        : "nodes",
    "text"                           : "userChoices['Nodes']",  // must match displayNodesSelection of this.deckStateMachine.nodesDeck.extraSteps (and have allnodesNodes, groupnodesNodes in xul file) 
    "textEval"                       : true,
  }
  this.seedsHostsNode = {
    "tagName"                        : "seeds",
    "text"                           : "userChoices['Seeds']",  // must match displayNodesSelection of this.deckStateMachine.seedsDeck.extraSteps (and have allnodesSeeds, groupnodesSeeds in xul file) 
    "textEval"                       : true,
  }
  this.topologyNode = {
    "tagName"                        : "topology",
    "nodesArray"                     : [ this.nodesHostsNode, this.seedsHostsNode ]
  }
  this.storagePortNode = {
    "tagName"                        : "storageport",
    "text"                           : "7000"
  }
  this.sslStoragePortNode = {
    "tagName"                        : "sslstorageport",
    "text"                           : "7001"
  }
  this.nativeTransportPortNode = {
    "tagName"                        : "nativetransportport",
    "text"                           : "9042"
  }
  this.nativeTransportPortSslNode = {
    "tagName"                        : "nativetransportportssl",
    "text"                           : "9042"
  }
  this.jmxPortNode = {
    "tagName"                        : "jmxport",
    "text"                           : "7199"
  }
  this.jolokiaPortNode = {
    "tagName"                        : "jolokiaport",
    "text"                           : "8778"
  }
  this.portsNode = {
    "tagName"                        : "ports",
    "nodesArray"                     : [ this.storagePortNode, this.sslStoragePortNode, this.nativeTransportPortNode, this.nativeTransportPortSslNode, this.jmxPortNode, this.jolokiaPortNode ]
  }
  this.commitLogNode = {
    "tagName"                        : "commitlog",
    "text"                           : "userChoices[mapElementIdUserChoices.main.cassandraBigDataWizardCommitlogDir.usersChoicesEntry]",
    "textEval"                       : true,
  }
  this.dataFileNode = {
    "tagName"                        : "datafile",
    "text"                           : "userChoices[mapElementIdUserChoices.main.cassandraBigDataWizardDataFileDirs.usersChoicesEntry]",
    "textEval"                       : true,
  }
  this.savedCachesNode = {
    "tagName"                        : "savedcaches",
    "text"                           : "userChoices[mapElementIdUserChoices.main.cassandraBigDataWizardSavedCachesDir.usersChoicesEntry]",
    "textEval"                       : true,
  }
  this.hintsNode = {
    "tagName"                        : "hints",
    "text"                           : "userChoices[mapElementIdUserChoices.main.cassandraBigDataWizardHintsDir.usersChoicesEntry]",
    "textEval"                       : true,
  }
  this.directoriesNode = {
    "tagName"                        : "directories",
    "nodesArray"                     : [ this.commitLogNode, this.dataFileNode, this.savedCachesNode, this.hintsNode ]
  }
  this.numTokensNode = {
    "tagName"                        : "numtokens",
    "text"                           : "userChoices[mapElementIdUserChoices.paramsDeck.numTokens.usersChoicesEntry]",
    "textEval"                       : true,
  }
  this.endPointSnitchNode = {
    "tagName"                        : "endpointsnitch",
    "text"                           : "userChoices[mapElementIdUserChoices.paramsDeck.endpointSnitch.usersChoicesEntry]",
    "textEval"                       : true,
  }
  this.maxHeapSizeNode = {
    "tagName"                        : "maxheapsize",
    "text"                           : "userChoices[mapElementIdUserChoices.paramsDeck.maxHeapSize.usersChoicesEntry]",
    "textEval"                       : true,
  }
  this.heapNewSizeNode = {
    "tagName"                        : "heapnewsize",
    "text"                           : "userChoices[mapElementIdUserChoices.paramsDeck.heapNewsize.usersChoicesEntry]",
    "textEval"                       : true,
  }
  this.paramsNode = {
    "tagName"                        : "params",
    "nodesArray"                     : [ this.numTokensNode, this.endPointSnitchNode, this.maxHeapSizeNode, this.heapNewSizeNode ]
  }
  this.instanceXMLNode = {
    "tagName"                        : "instance",
    "nodesArray"                     : [ this.instanceNameNode, this.topologyNode, this.portsNode, this.directoriesNode, this.paramsNode ]
  }
  this.configurationFileStructure  = [
    {
      "tagName"                        : "cassandraConfig",
      "nodesArray"                     : [ this.cassandraArchiveXMLNode, this.javaHomeXMLNode, this.jolokiaJarNode, this.instanceXMLNode ]
    }
  ] // this.configurationFileStructure

  // the file to be displayed on the screen
  this.getConfigurationFileToBeDisplayedStructure = function() {
    var configurationFileStructureToBeDisplayed = this.configurationFileStructure;
    return configurationFileStructureToBeDisplayed;
  }

  // Update some inputs as user changes the instance name
  this.changeAllDataRoot = function() {
    if (document.getElementById('instanceName').value.length > 0) {
      var instanceName = document.getElementById('instanceName').value.trim();
      this.mapElementIdUserChoices.main.cassandraBigDataWizardCommitlogDir.attributeToSetValue = "/var/lib/cassandra/" + instanceName + "/commitlog";
      document.getElementById('cassandraBigDataWizardCommitlogDir').value = "/var/lib/cassandra/" + instanceName + "/commitlog";
      this.mapElementIdUserChoices.main.cassandraBigDataWizardDataFileDirs.attributeToSetValue = "/var/lib/cassandra/" + instanceName + "/data";
      document.getElementById('cassandraBigDataWizardDataFileDirs').value = "/var/lib/cassandra/" + instanceName + "/data";
      this.mapElementIdUserChoices.main.cassandraBigDataWizardSavedCachesDir.attributeToSetValue = "/var/lib/cassandra/" + instanceName + "/saved_caches";
      document.getElementById('cassandraBigDataWizardSavedCachesDir').value = "/var/lib/cassandra/" + instanceName + "/saved_caches";
      this.mapElementIdUserChoices.main.cassandraBigDataWizardHintsDir.attributeToSetValue = "/var/lib/cassandra/" + instanceName + "/hints";
      document.getElementById('cassandraBigDataWizardHintsDir').value = "/var/lib/cassandra/" + instanceName + "/hints";
    }
  }

  this.checkJavaParams = function() {
    if (((document.getElementById('maxHeapSize').value != '') && (document.getElementById('heapNewsize').value == '')) ||
        ((document.getElementById('maxHeapSize').value == '') && (document.getElementById('heapNewsize').value != ''))) {
      dock.displayError("Both 'Maximum heap size' and 'Young generation size' should be either set or not (if you set one, set the other).");
      return false;
    }
    return true;
  }
} // CassandraBigDataWizardBuilder



var cassandraBigDataWizard;

function onLoad() {
  println("caller is " + arguments.callee.caller.toString());
  cassandraBigDataWizard = new BigDataWizard(this, new CassandraBigDataWizardBuilder());
}

function onClose() {
  cassandraBigDataWizard.onClose();
  cassandraBigDataWizard = null;
}
