/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */


 // this file relies on these files to be previously loaded:
 // . hadoop/objecthelpers.js
 // . hadoop/addbigdatainstancewizard.js
 // (note that the requisites of those files aren't listed here; check them to know what they require). 


function SparkBigDataWizardBuilder() {
  this.toString = function() {
    return "Spark";
  }
  this.getRemoteCallParameters = function()  {
    var remoteCallParameters = {};
    remoteCallParameters.service        = "cmhadoop";
    remoteCallParameters.remoteCall     = "executeHadoopSetup";
    remoteCallParameters.remoteCallType = 1; // 0 - Hadoop, 1 - Spark; check executeHadoopSetup at src/autogen/services/cmhadoop.xml in cmdaemon.
    return remoteCallParameters;
  }

  var userChoices = {};
  this.getUserChoices = function() {
    return userChoices;
  }

  // this.mapElementIdUserChoices is one of the 3 structures that must be defined.
  this.mapElementIdUserChoices = {

    "main" : {
      "dialogtitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Spark main details"
      },

      "mainMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "Please set the name of the instance and the Java home directory. Specify where to find the Spark tarball; below is suggested the most recent one, which is provided by the package cm-apache-hadoop-extras and also available at\nhttp://support.brightcomputing.com/bigdata/"
      },

      "instanceName" : {
        validate_as         : "instanceName",
        humanReadable       : "Instance name",
        usersChoicesEntry   : "instanceName",
        attributeToSet      : "value",
        attributeToSetValue : "spark1"
      },

      "sparkBigDataWizardTarball" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "Spark tarball",
        usersChoicesEntry   : "sparkTarball",
        checkFileOnHeadnode : true,
        attributeToSet      : "value",
        attributeToSetValue : CM_LOCAL_PREFIX + NEWEST_SPARK_STANDALONE

      },

      "useDefaultJavaHome" : {
        humanReadable       : "Use default Java home",
        usersChoicesEntry   : "useDefaultJavaHome",
        attributeToSet      : "value",
      },

      "javaHome" : {
        validate_as         : validateJavaHome.bind(this, 'useDefaultJavaHome', 'defaultJavaHome', 'customJavaHome', userChoices),
        humanReadable       : "Java home",
        usersChoicesEntry   : "javaHome",
        attributeToSet      : "value",
        // will use extraSteps for complex setDefaultJavaHome('javaHome'); instead of setting attributeToSetValue.
      },

      "secondMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "Choose 'ZooKeeper' recovery method to enable Spark Master High Availability. The number of Masters can be set during the installation and changed after the deployment."
      },

      "sparkBigDataWizardRecoveryMethod" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "Recovery method",
        usersChoicesEntry   : "recoveryMethod",
        attributeToSet      : "selectedItem",
        // will set the default value (NONE) in .xul file, instead of setting attributeToSetValue.
        transformations     : [ {"name": "elementGetSubElement", "accessor":  ".value"} ]  // I need sparkBigDataWizardRecoveryMethod.selectedItem.value in userChoices
      },

    }, // main

    "zooKeeperDesk" : {
      "dialogtitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "ZooKeeper details"
      },

      "zookeeperMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "ZooKeeper works as an ensemble of ZooKeeper hosts. There must be at least 3 hosts. Selecting more than 3 nodes is possible, provided that the number of selected hosts is an odd number (e.g. 3, 5, 7).\n\nSpecify where to find the ZooKeeper tarball:"
      },

      "sparkBigDataWizardZooKeeperTarball" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "ZooKeeper tarball",
        usersChoicesEntry   : "zooKeeperTarball",
        checkFileOnHeadnode : true,
        attributeToSet      : "value",
        attributeToSetValue : CM_LOCAL_PREFIX + ZOOKEEPER_FOR_NEWEST_SPARK_STANDALONE
      },

      "zookeeperMessage3" : {
        attributeToSet      : "value",
        attributeToSetValue : "Please choose ZooKeeper nodes:"
      },

      "groupnodesZookeeper" : {
        humanReadable       : "Zookeeper nodes",
        usersChoicesEntry   : "Zookeeper",
      },

    }, // zooKeeperDesk

    "mastersDesk" : {
      "dialogtitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Masters details"
      },

      "masternodesMessage1" : {
        attributeToSet      : "value",
        attributeToSetValue : "Please choose one node as Spark Master. When the recovery method is ZooKeeper, more nodes can be selected:"
      },

      "sparkBigDataWizardMasterLocalDir" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "Master local directory",
        usersChoicesEntry   : "masterLocalDir",
        attributeToSet      : "value",
        attributeToSetValue : "/tmp/spark/spark1/"
      },

      "masternodesMessage3" : {
        attributeToSet      : "value",
        attributeToSetValue : "Please specify the directory that Spark must use for scratch space, like map output files and RDDs that get stored on disk. It must be on a fast, local disk in your system:"
      },

      "groupnodesMasters" : {
        humanReadable       : "Spark masters",
        usersChoicesEntry   : "Masters",
      },

    }, // mastersDesk

    "workersDesk" : {
      "dialogtitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Workers details"
      },

      "datanodesMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "Please choose at least one node as Spark Worker:"
      },

      "groupnodesDatanodes" : {
        humanReadable       : "Spark workers",
        usersChoicesEntry   : "Datanodes",
      },

      "datanodesMessage3" : {
        attributeToSet      : "value",
        attributeToSetValue : "Specify the directory used by workers to to run applications (scratch space):"
      },

      "sparkBigDataWizardWorkerDir" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "Worker directory",
        usersChoicesEntry   : "workerDir",
        attributeToSet      : "value",
        attributeToSetValue : "/tmp/spark/spark1/"
      },

      "datanodesMessage5" : {
        attributeToSet      : "value",
        attributeToSetValue : "Set the number of worker instances and the number of cores available to Spark applications." +
                                " The number of worker instances can be bigger than 1 for very large machines, to spread multiple worker processes:"
      },

      "sparkBigDataWizardNumCores" : {
        validate_as         : "lenghtNotZero",
        humanReadable       : "Number of cores",
        usersChoicesEntry   : "numCores",
        attributeToSet      : "value",
        attributeToSetValue : "0"
      },

      "sparkBigDataWizardNumCoresPostLabel" : {
        attributeToSet      : "value",
        attributeToSetValue : "(0 to use all available cores)"
      },

      "sparkBigDataWizardNumInstances" : {
        validate_as         : "integerBiggerThanZero",
        humanReadable       : "Number of worker instances",
        usersChoicesEntry   : "numInstances",
        attributeToSet      : "value",
        attributeToSetValue : "1"
      },

    }, // workersDesk

    "summaryDesk" : {
      "dialogtitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Summary of choices"
      },

      "summaryChoicesMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "The selected choices are shown below. They will be used to create an XML configuration file.\n",
      },

      "summaryMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : "\n\nBy pressing 'Next' a temporary XML file will be created with the selected configuration and a Spark instance will be installed using this XML file.\n\nAlternatively, clicking the 'Save Configuration' button will produce an XML configuration file which can be further customized, if needed, and then used as the input configuration file for the cm-spark-setup command line utility.\nPress 'Show Configuration' to get the Spark deployment configuration as XML.",
      },

      "prerequisitesHadoopDialogTitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Prerequisites checking Spark",
      },

      "prerequisitesHadoopMessage" : {
        attributeToSet      : "value",
        attributeToSetValue : 'Before starting the installer two things will be checked:\n' +
                              '- The JRE folder should contain the Java executable (bin/java) on all nodes that are part of this Hadoop instance.\n' +
                              '- The required tarballs should exist on the head node(s).\n'
      },

    }, // summaryDesk

    "deployDesk" : {
      "dialogtitle" : {
        attributeToSet      : "value",
        attributeToSetValue : "Deploy details"
      },

    } // deployDesk

  } // this.mapElementIdUserChoices

  // this.deckStateMachine is one of the 3 structures that must be defined.
  this.deckStateMachine = {
      "main"          : {
        "elementId"     : "main", 
        "next"          : {
          "zooKeeperDesk" : "document.getElementById('sparkBigDataWizardRecoveryMethod').selectedIndex == 2",
          "mastersDesk"   : "document.getElementById('sparkBigDataWizardRecoveryMethod').selectedIndex != 2"
        }, 
        "extraSteps"    : [ "setDefaultJava180Home('defaultJavaHome', 'useDefaultJavaHome');"],
        "beforeNext"    : [ "setJavaHomeToUse(userChoices);" ]
      }, // main

      "zooKeeperDesk" : {
        "elementId"     : "zookeeper", 
        "previous"      : {
          "main"          : true
        },
        "next"          : {
          "mastersDesk"   : true
        },
        "nodesRules"    : [
          { "condition": " < 3", "errorMessage": "At least 3 hosts should be selected"},
          { "condition": "  % 2 - 1", "errorMessage": "An odd number of hosts should be selected"},
        ],
        "extraSteps"    : [ "displayNodesSelection('Zookeeper');"]
      }, // zooKeeperDesk

      "mastersDesk"   : {
        "elementId"     : "masternodes",
        "previous"      : {
          "zooKeeperDesk" : "document.getElementById('sparkBigDataWizardRecoveryMethod').selectedIndex == 2",
          "main"          : "document.getElementById('sparkBigDataWizardRecoveryMethod').selectedIndex != 2"
        },
        "nodesRules"    : [
          { "condition": " < 1", "errorMessage": "At least one host should be selected"},
          { "condition": " *( document.getElementById('sparkBigDataWizardRecoveryMethod').selectedIndex == 2 ? 0 : 1)  > 1 ", "errorMessage": "Only ZooKeeper recovery method allows more than one master host."},
        ],
        "extraSteps"    : [ "displayNodesSelection('Masters');"],  // must match userChoices of this.masterHostsNode (and have allnodesMasters, groupnodesMasters in xul file) 
        "next"          : {
          "workersDesk"   : true
        }
      }, //  mastersDesk

      "workersDesk"   : {
        "elementId"     : "datanodes", 
        "previous"      : {
          "mastersDesk"   : true
        },
        "next"          : {
          "summaryDesk"   : true
        },
        "nodesRules"    : [
          { "condition": " < 1", "errorMessage": "At least one host should be selected"},
        ],
        "extraSteps"    : [ "displayNodesSelection('Datanodes');"]
      }, //  workersDesk

      "summaryDesk"   : {
        "elementId"     : "summary", 
        "previous"      : {
          "workersDesk"   : true
        },
        "next"          : {
          "deployDesk"   : "remoteRequisitesSatisfiedOrPrompt()"
        }, 
        "extraSteps"    : [ " printUserChoicesOnElementId('summaryChoices');", "displayPreconditionsCheckAtElement('verificationSteps');"]
      }, // summaryDesk 

      "deployDesk"   : {
        "elementId"     : "deploy", 
        "previous"      : {
          "summaryDesk"   : true
        },
        "extraSteps"    : [ "deploy();"]
      } // deployDesk

  } // this.deckStateMachine

  // this.configurationFileStructure is one of the 3 structures that must be defined. The Spark XML file needs these other XML structures:
  this.sparkArchiveXMLNode = {
     "tagName"                        : "archive",
     "text"                           : "userChoices[mapElementIdUserChoices.main.sparkBigDataWizardTarball.usersChoicesEntry]",
     "textEval"                       : true,
   } // this.sparkArchiveXMLNode
   this.javaHomeXMLNode = {
     "tagName"                        : "javahome",
     "text"                           : "userChoices[mapElementIdUserChoices.main.javaHome.usersChoicesEntry]",
     "textEval"                       : true,
   } // this.javaHomeXMLNode

   // instance xml nodes:
   this.instanceNameNode = {
     "tagName"                        : "name",
     "text"                           : "userChoices[mapElementIdUserChoices.main.instanceName.usersChoicesEntry]",
     "textEval"                       : true,
   } // this.instanceNameNode

   // masters xml nodes:
   this.masterHostsNode = {
     "tagName"                        : "hosts",
     "text"                           : "userChoices['Masters']",  // must match displayNodesSelection of this.deckStateMachine.mastersDesk.extraSteps  (and have allnodesMasters, groupnodesMasters in xul file) 
     "textEval"                       : true,
   } // this.masterHostsNode
   this.masterPortNode = {
     "tagName"                        : "port",
     "text"                           : "7070"
   } // this.masterPortNode
   this.masterWebPortNode = {
     "tagName"                        : "webport",
     "text"                           : "9080"
   } // this.masterWebPortNode
   this.masterHistoryWebPortNode = {
     "tagName"                        : "historywebport",
     "text"                           : "19080"
   } // this.masterHistoryWebPortNode
   this.masterLocalDir = {
     "tagName"                        : "localdir",
     "text"                           : "userChoices[mapElementIdUserChoices.mastersDesk.sparkBigDataWizardMasterLocalDir.usersChoicesEntry]",
     "textEval"                       : true
   } // this.masterLocalDir
   this.mastersXMLNode = {
     "tagName"                        : "masters",
     "attributesArray"                : [
       {
         "attributeName"                 : "recovery" ,
         "attributeValue"                : "userChoices[mapElementIdUserChoices.main.sparkBigDataWizardRecoveryMethod.usersChoicesEntry]",
         "attributeValueEval"            : true
       },
     ], // attributesArray
     "nodesArray"                     : [ this.masterHostsNode, this.masterPortNode, this.masterWebPortNode, this.masterHistoryWebPortNode, this.masterLocalDir ]
   } // this.mastersXMLNode

   // workers xml nodes:
   this.workerHostsNode = {
     "tagName"                        : "hosts",
     "text"                           : "userChoices['Datanodes']",  // must match displayNodesSelection of this.deckStateMachine.mastersDesk.extraSteps  (and have allnodesDatanodes, groupnodesDatanodes in xul file) 
     "textEval"                       : true,
   } // this.workerHostsNode
   this.workerPortNode = {
     "tagName"                        : "port",
     "text"                           : "7071"
   } // this.workerPortNode
   this.workerWebPortNode = {
     "tagName"                        : "webport",
     "text"                           : "9081"
   } // this.workerWebPortNode
   this.numcoresNode = {
     "tagName"                        : "numcores",
     "text"                           : "userChoices[mapElementIdUserChoices.workersDesk.sparkBigDataWizardNumCores.usersChoicesEntry]",
     "textEval"                       : true
   } // this.numcoresNode
   this.numinstancesNode = {
     "tagName"                        : "numinstances",
     "text"                           : "userChoices[mapElementIdUserChoices.workersDesk.sparkBigDataWizardNumInstances.usersChoicesEntry]",
     "textEval"                       : true
   } // this.numinstancesNode
   this.workerDirNode = {
     "tagName"                        : "workerdir",
     "text"                           : "userChoices[mapElementIdUserChoices.workersDesk.sparkBigDataWizardWorkerDir.usersChoicesEntry]",
     "textEval"                       : true
   } // this.workerDirNode
   this.cleanupEnabledNode = {
     "tagName"                        : "cleanupenabled",
     "text"                           : "true"
   } // this.cleanupEnabledNode
   this.cleanupintervalNode = {
     "tagName"                        : "cleanupinterval",
     "text"                           : "1800"
   } // this.cleanupintervalNode
   this.cleanupappdatattlNode = {
     "tagName"                        : "cleanupappdatattl",
     "text"                           : "604800"
   } // this.cleanupappdatattlNode
   this.workersXMLNode = {
     "tagName"                        : "workers",
     "nodesArray"                     : [ this.workerHostsNode, this.workerPortNode, this.workerWebPortNode, this.numcoresNode, this.numinstancesNode, this.workerDirNode, this.cleanupEnabledNode, this.cleanupintervalNode, this.cleanupappdatattlNode ]
   } // this.workersXMLNode

   // zookeeper xml nodes:
   this.zookeeperHostsNode = {
     "tagName"                        : "hosts",
     "text"                           : "userChoices['Zookeeper']",  // must match displayNodesSelection of this.deckStateMachine.mastersDesk.extraSteps  (and have allnodesDatanodes, groupnodesDatanodes in xul file) 
     "textEval"                       : true,
   } // this.zookeeperHostsNode
   this.zookeeperTarballNode = {
     "tagName"                        : "archive",
     "text"                           : "userChoices[mapElementIdUserChoices.zooKeeperDesk.sparkBigDataWizardZooKeeperTarball.usersChoicesEntry]",
     "textEval"                       : true
   } // this.numcoresNode
   this.clientportNode = {
     "tagName"                        : "clientport",
     "text"                           : "12181"
   } // this.clientportNode
   this.serverportNode = {
     "tagName"                        : "serverport",
     "text"                           : "12888"
   } // this.serverportNode
   this.electionportNode = {
     "tagName"                        : "electionport",
     "text"                           : "13888"
   } // this.electionportNode
   this.zookeeperXMLNode = {
     "tagName"                        : "zookeeper",
     "condition"                      : "document.getElementById('sparkBigDataWizardRecoveryMethod').selectedItem.value == 'ZOOKEEPER'",
     "nodesArray"                     : [ this.zookeeperTarballNode, this.zookeeperHostsNode, this.clientportNode, this.serverportNode, this.electionportNode  ]
   } // this.zookeeperXMLNode

   this.instanceXMLNode = {
     "tagName"                        : "instance",
     "nodesArray"                     : [ this.instanceNameNode, this.mastersXMLNode, this.workersXMLNode, this.zookeeperXMLNode ]
   } // this.instanceXMLNode
   this.configurationFileStructure  = [
   {
     "tagName"                        : "sparkConfig",
     "nodesArray"                     : [ this.sparkArchiveXMLNode, this.javaHomeXMLNode, this.instanceXMLNode ]
   }
   ] // this.configurationFileStructure

   // the file to be displayed on the screen
   this.getConfigurationFileToBeDisplayedStructure = function() {
     var configurationFileStructureToBeDisplayed = this.configurationFileStructure;
     return configurationFileStructureToBeDisplayed;
   }

    // Update some inputs as user changes the instance name
    this.changeAllDataRoot = function() {
      if (document.getElementById('instanceName').value.length > 0) {
        var instanceName = document.getElementById('instanceName').value.trim();
        this.mapElementIdUserChoices.mastersDesk.sparkBigDataWizardMasterLocalDir.attributeToSetValue = "/tmp/spark/" + instanceName;
        document.getElementById('sparkBigDataWizardMasterLocalDir').value =  "/tmp/spark/" + instanceName;
        this.mapElementIdUserChoices.workersDesk.sparkBigDataWizardWorkerDir.attributeToSetValue = "/tmp/spark/" + instanceName;
        document.getElementById('sparkBigDataWizardWorkerDir').value =  "/tmp/spark/" + instanceName;
      }
    }

} // SparkBigDataWizardBuilder




var sparkBigDataWizard;
function onLoad() {
  println("caller is " + arguments.callee.caller.toString());
  sparkBigDataWizard = new BigDataWizard(this, new SparkBigDataWizardBuilder());
}


function onClose() {
  sparkBigDataWizard.onClose();
  sparkBigDataWizard = null;
}
