/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

/**
 * Helper class to store user preferences in $HOME/.cm/cmgui/BigDataPreferences.mine.
 */
var BigDataPreferences = (function () {
    var dock = false, file = false, data = {};
    return {
        /**
         * Get value from User Preferences by some index. Note that you can pass multiple values in the get function.
         *
         * Example: get('cluster1', 0, 1) will return what' stored with set('cluster1', 0, 1, 'THIS IS THE VALUE').
         *
         * @returns mixed - false in case there is nothing
         */
        'get' : function () {
            this.initialize();
            var index = ''; // this doesn't work: = arguments.join('_');
            for (var i=0; i<arguments.length; i++) {
                index += (i > 0 ? '_' : '') + arguments[i];
            }
            if (data[index]) {
                return data[index];
            }
            return false;
        },
        /**
         * Set value for User Preferences for some index. Note that you can pass multiple values for the index.
         *
         * Example: set('cluster1', 0, 1, 'test') will store the value 'test' for the index 'cluster1_0_1'
         *  (internally the variables are concatinated to create a single string for index).
         *
         * @returns void
         */
        'set' : function () {
            this.initialize();
            var value = arguments[arguments.length - 1];
            var index = '';
            for (var i=0; i<(arguments.length - 1); i++) {
                index += (i > 0 ? '_' : '') + arguments[i];
            }
            data[index] = value;
            this.save();
        },
        /**
         * Lazy initialize the dependencies for this object.
         *
         * @return void
         */
        'initialize' : function () {
            if (!!dock) return;
            dock = getDock();
            file = dock.BASE_PATH + 'BigDataPreferences.mine';
            if (! fileExists(file)) {
                println("BigDataPreferences.mine does not exist");
                return;
            }
            this.load();
        },
        /**
         * Load disk from file and merge it with the in-memory data.
         */
        'load' : function () {
            var contents = readfile(file);
            if (! contents) {
                println("BigDataPreferences.mine could not be read");
                return;
            }
            var newdata = JSON.parse(contents);
            if (! newdata) {
                println("BigDataPreferences.mine did not contain valid JSON");
                return;
            }
            for (var i in newdata) {
                if (!data[i]) {
                    data[i] = newdata[i];
                }
            }
        },
        /**
         * Save storage from this object to file.
         *
         * @return void
         */
        'save' : function () {
            this.initialize();
            this.load(); // make sure any other change that was made on disk in the meantime is present in data,
                         // or we would overwrite/delete it at this point.
            savefile(file, JSON.stringify(data));
        }
    };
})();
