/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var cluster;
var nodes;
var hdfskey;
var allroles;
var nmroles;
var myDialog;
var myDialogIndex;
var configs2pass;
var nodesapplied2pass;
var lRoles;
var mapCoRoles;
var old;
var hservices;
var okPressed = false;

var configOnLoad;


Array.prototype.diff = function(a) {
    return this.filter(function(i) {return a.indexOf(i) < 0;});
};

function setAttributes(el, attrs) {
  for(var key in attrs)   el.setAttribute(key, attrs[key]);
}

function changeTabFocus(ind) {
  document.getElementById("tabbox").selectedIndex = ind;
}

function getTabFocus() {
  return document.getElementById("tabbox").selectedIndex;
}


function onLoad() {
  dock = getDock();
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  hdfskey = myDialog.arguments.hdfs;
  allroles = myDialog.arguments.availroles;
  nmroles = myDialog.arguments.nonmovroles;
  myDialogIndex = index;

  var select_tab = null;
  if(myDialog.arguments.hasOwnProperty('select_tab')) {
    select_tab = myDialog.arguments.select_tab;
  }

  refresh(myDialog.arguments.object, select_tab);

  // select tab

  // configOnLoad = myDialog.arguments.config;

  if ( location.hash == "#clone" ) {
    document.title = "Clone Hadoop Configuration Group";
    document.getElementById('name').removeAttribute("readonly");
  }

  if ( location.hash == "#new" ) {
    document.title = "New Hadoop Configuration Group";
    document.getElementById('name').removeAttribute("readonly");
  }
  
  // makes 'name' field changeable for non-committed yet overlays
  if ( myDialog.arguments.object.get('uniqueKey') < dock.MAX_LOCAL_KEY ) {
    document.getElementById('name').removeAttribute("readonly");
  }

  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use, select_tab=null) {
  if(!!use) {
     current = use;
     old = current.old(false, true);
  }

  println("Refresh Hadoop Configuration Group" + ' - ' + document.documentURI);
  
  // uncomment to print out configuration group with assigned roles
  // printall(current.object);
  // for(var j = 0; j < current.object.roles.length; j++)    printall(current.object.roles[j]);
  
  cluster = current.cluster;
  nodes = current.get('nodes');
  mapCoRoles = current.buildConfigurationOverlayNodeRoleMapping(hdfskey);  
  
  // uncomment to print out node-role mapping for all nodes used by a hadoop instance  
  // for (var key in mapCoRoles) {
  //  println("Key: " + key);
  //  for (var k = 0; k < mapCoRoles[key].length; k++)  println("      " + mapCoRoles[key][k].name + ", " + mapCoRoles[key][k].priority);
  // }

  // text('priority', '^[0-9]+$');
  text('priority', dock.REGEX_COPriority);
  text('name', '');

  var elem = document.getElementById('nodesconfigured'); 
  elem.setAttribute("value", dotFormatKeys(cluster, nodes) );
 
  var tabs = document.getElementById('tabs');   
  var tabpanels = document.getElementById('tabpanels');   

  // when refreshing remove tabs created earlier if any
  while(tabs.firstChild) tabs.removeChild(tabs.firstChild);
  while(tabpanels.firstChild) tabpanels.removeChild(tabpanels.firstChild);

  // Split button is not shown since its precise functionality has not been agreed on
  // uncomment to have the button
  // var splitconfgroup = document.getElementById('splitconfgroupbutton');  
  // splitconfgroup.hidden = true;

  var roles = current.getHadoopRoles(hdfskey, false);
  var hadoopservices = new Array();
  var change_tab_focus_to = 0;
  lRoles = new Object();
  configs2pass = new Object();
  nodesapplied2pass = new Object();

  for(var j = 0; j < roles.length; j++) {
    var roleType = roles[j].childType;
    var role = new dock.cvObject(cluster, roles[j], current, 'roles');

    for(var i = 0; i < roles[j].config.length; i++) {
        lRoles[roleType] = role;
        configs2pass[roleType] = new dock.cvObject(cluster, roles[j].config[i], role, 'config');
        nodesapplied2pass[roleType] = determineNodesApplied(mapCoRoles,nodes,roleType);
    }
  }

  Object.keys(lRoles).forEach(function(key) {
    if (!lRoles[key].get('toBeRemoved') ) {
      hadoopservices.push(hadoopservicenames[key]);
    }
  });

  hservices = new Array();

  var k = 0;

  // for each role in configuration overlay create tab with its configuration settings window  
  Object.keys(lRoles).forEach(function(key) {
    // some roles (to be removed) marked for removal and hence not shown
    if (!lRoles[key].get('toBeRemoved') ) {
      var shadowtab = document.createElement('shadowtab');
      var iframeelem = document.createElement('iframe');
      var iframesrc = dock.myVersion + key.substring(0, key.length - 4).toLowerCase() + 'settings.xul';
      setAttributes(shadowtab, {'subtab': 'true', 'id': 'tab_' + k, 'label': 'Configure ' + hadoopservicenames[key], 'context':'removerole' });
      hservices.push(key);
      if(select_tab && select_tab == lRoles[key].get('uniqueKey')) {
        change_tab_focus_to = k;
      }
      //select_tab=null
      setAttributes(iframeelem, {'id': 'iframe_' + k, 'src': iframesrc });
      tabs.appendChild(shadowtab);
      tabpanels.appendChild(iframeelem);
      k++;
    }
  });
  // 'Remove role' button shown if there are more than one role
  var removerole = document.getElementById('removerolebutton');
  if ( k <= 1 ) removerole.hidden = true;
  else  removerole.hidden = false;

  //current['hservices'] = hservices;

  // To define roles that can be added to configuration group
  var existingroles = Object.keys(lRoles).map(function(key) {   if (!lRoles[key].get('toBeRemoved') ) return key;   });
  var roles2show = allroles.diff(existingroles);

  var elem = document.getElementById('selecthadooprole');   
  // when refreshing remove items created earlier if any
  while(elem.firstChild) elem.removeChild(elem.firstChild);
 
  for (var i=0; i < roles2show.length; i++) {
    var menuItem = document.createElement("menuitem");
    menuItem.setAttribute("value", roles2show[i]);
    menuItem.setAttribute("label", hadoopservicenames[roles2show[i]]);
    elem.appendChild(menuItem);
  }
  //document.getElementById("selecthadooprole").selectedIndex = 0;
  /*
  // add 'Manage roles in configuration group' tab
  shadowtab = document.createElement('shadowtab');
  iframeelem = document.createElement('iframe');
  iframesrc = dock.myVersion + 'managerolesinconfiggroup.xul';
  setAttributes(shadowtab, {'subtab': 'true', 'id': 'tab' + k, 'label': 'Manage roles in ' + current.configgroupname });
  setAttributes(iframeelem, {'id': 'iframe_' + k, 'src': iframesrc });
  tabs.appendChild(shadowtab);
  tabpanels.appendChild(iframeelem);
  */
  changeTabFocus(change_tab_focus_to);
  
  /*
  println("Refresh Hadoop Role Configuration Tabs");
  for(var i = 0; i < window.frames.length; i++) {
    println("Frame" + i); 
    //alert(ee.contentDocument);

    //ee.contentWindow.location.reload(true);
    if(!!window.frames[i].refresh) {
      println("-Frame" + i);
      window.frames[i].refresh(current.configobj);
    }
  }
  */

}

function manageNodes() {
    var xulfilename = dock.myVersion + 'hadoop/hadoopnodeselection.xul';
    var args = new Object();
    args['configoverlay'] = current;
    args['mapcoroles'] = mapCoRoles;
    var d = new dock.cvDialog(xulfilename,args);

    d.onOk = function() {
      refresh();
    }

    dock.dialogs.showDialog(d);
}

function doOk()
{
  top.closeManageCustomizations();
}
function doCancel()
{
  top.closeManageCustomizations();
}

function detachrole() {
  var where = getTabFocus();

  // println(where);
  // println(hservices[where]);
  // println(hadoopservicenames[hservices[where]]);
  
  if ( nmroles.some(function(r) { return r == hservices[where]; }) ) {
    var args = new Object();
    args['title'] = "Warning";
    args['text'] = hadoopservicenames[hservices[where]] + " cannot be removed.";
    var d = new dock.cvDialog(dock.myVersion + "showtext.xul", args);
    dock.dialogs.showDialog(d);
  }
  else {

    var roles = current.get('roles');
    var removed;
    for(var j = 0; j < roles.length; j++) {
      // printall(roles[j]);
      if ( roles[j].childType == hservices[where] ) {
         for(var m = 0; m < roles[j].config.length; m++) {
             // printall(roles[j].config[m]);
             if ( roles[j].config[m].hdfs == hdfskey ) {
                roles[j].config[m].toBeRemoved = true;
                roles[j].modified = true;
                roles[j].config.splice(m, 1);
                break;
             }   
         }

         if ( roles[j].config.length == 0 )    roles.splice(j,1);
         break;
      }
    }
    current.set('roles',roles);
    println(".. num of Hadoop/Spark roles in configuration group: " + current.getHadoopRoles(hdfskey, false).length);
    refresh();

  }

}

function attachrole(val) {
    // val, hadoopservicenames[val]
    if ( val in lRoles ) { 
      lRoles[val].set('toBeRemoved',false);
    }
    else {
      var hadoopRoleFunction = "cv" + val;
      var hadoopConfigurationFunction = "cv" + val.substring(0, val.length - 4) + "HDFSConfiguration";

      var roles = current.get('roles'); 

      var newrole = new window["dock"][hadoopRoleFunction]();
      var newconfig = new window["dock"][hadoopConfigurationFunction]();
      newconfig.hdfs = hdfskey;
      newrole.config.push( newconfig );
      roles.push(newrole);
      current.set('roles', roles);

      println("..#roles in configuration group: " + current.getHadoopRoles(hdfskey, false).length);
      // overlay.commit()
    }
    refresh();
    // myDialog.onAddRole();
}

// Split config-group functionality not defined
// uncomment to implement 'Split' button functionality
/*
function splitconfgroup() {
    var xulfilename = 'splitconfgroupwindow.xul';
    
    current['roleleft'] = current.hservices[getTabFocus()];
    var d = new dock.cvDialog(xulfilename,current);
    d.onOk = function() {
      refresh();
    }
    dock.dialogs.showDialog(d);
}
*/

function buttonCancel() {
  // if 'OK' button (that keeps changes done on config-group) 
  // pressed then window should simply be closed; otherwise original 
  // object (config-group at the time of window pop-up) restored
  if (!okPressed) {
    if ( location.hash == "#clone" || location.hash == "#new"   ) { 
      cluster.removeObject(current.get('uniqueKey'));
    }
    current.old(old);
    close();
  }
  else {
    close();
  }
}


function buttonOk() {
  // save();
  if ( current.getHadoopRoles(hdfskey, false).length == 0 ) {
    dock.displayError('No roles assigned: Add at least one role.');
    return;
  }

  var priConflicts = checkPriorityConflicts();
  if ( priConflicts.length > 0 ) {
    var text = '';
    for(var k = 0; k < priConflicts.length; k++) {
      text += '\t' + priConflicts[k] + '\n';      
    }
    dock.displayError('Priority conflict(-s): Change priority value to resolve.\n\nFound conflict(-s) with \n' + text);
    return;
  }
  
  if ( !checkConfOverlayPriority(current.get('priority')) ) {
    dock.displayError('Priority value is out of range: Change priority value.');
    return;
  }  

  if ( !checkConfOverlayName() ) {
    var hiName = cluster.find(hdfskey).get('name');
    okPressed = true;
    myDialog.onOk();
    close();

    // warnings section
    if ( !(current.resolveName().indexOf(hiName) === 0) )	dock.displayInfo("Warning: Name of configuration group does not start with " + hiName);
    if ( current.get('nodes').length == 0 )	dock.displayInfo("Warning: No nodes have been defined for configuration group.");

  }
  else {
    dock.displayError('This name is used: Change configuration group name.\nNote: see all configuration groups (overlays) in the Cluster Tab or\n run \'configurationoverlay list\' in cmsh.');
    return;
  }
}


function checkConfOverlayName() {
  var overlays = cluster.find('ConfigurationOverlay');
  var uniqueKey = current.get('uniqueKey');
  var coNames = [];   
  for(var k = 0; k < overlays.length; k++) {
    if ( overlays[k].get('uniqueKey') != uniqueKey )
      coNames.push(overlays[k].resolveName());
  }
  return ( coNames.indexOf(current.resolveName()) > -1 );  
}

function determineNodesApplied(nodeRoleMapping,nodes,roleType) {
  var coname = current.resolveName();
  var copriority = current.get('priority')
  var nodesapplied = new Array();
  if ( nodes.length == 0 || parseInt(copriority) == -1 )    return nodesapplied;

  for(var k = 0; k < nodes.length; k++) {
    var key = nodes[k] + "/" + roleType;
    var pr = 0, 
        ind = 0;
    for(var m = 0; m < nodeRoleMapping[key].length; m++) {
      if( parseInt(nodeRoleMapping[key][m].priority) > pr ) {
        pr = nodeRoleMapping[key][m].priority;
        ind = m;
      }
    }

    if( coname == nodeRoleMapping[key][ind].name)      nodesapplied.push(nodes[k]);
  }

  return nodesapplied;
}

function checkPriorityConflicts() {

  var coname = current.resolveName();
  var conodes = current.get('nodes');
  var coroles = current.getHadoopRoles(hdfskey, false);
  var copriority = current.get('priority');
  var map = current.buildConfigurationOverlayNodeRoleMapping(hdfskey);  

  var conflicts = new Array();
  if ( conodes.length == 0 || parseInt(copriority) == -1 )    return conflicts;

  for(var k = 0; k < conodes.length; k++) {
    for(var z = 0; z < coroles.length; z++) {

      var key = conodes[k] + "/" + coroles[z].childType;
      for(var m = 0; m < map[key].length; m++) {
        if ( coname != map[key][m].name && parseInt(copriority) == map[key][m].priority )   conflicts.push(map[key][m].name + " [" + map[key][m].priority + "]: " + hadoopservicenames[coroles[z].childType] + " on " + cluster.find(conodes[k]).resolveName());
      }
    }

  }

  return conflicts;
}
