/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var basetype = 'ConfigurationGroup';
var singular = 'Configuration Group';
var plural = 'Configuration Groups';

var dock;
var cluster;
var current;
var hdfskey;
var timer = 0;
var listRoles;
var nonmovableRoles;


function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}


function refresh(use) {
  if(!!use) current = use; 
  cluster = current.cluster;
  hdfskey = getHadoopKey();
  println("Refresh Hadoop Configuration Group list" + ' - ' + document.documentURI);

  tree = document.getElementById('tree');
  data = new Array();
  var tt;
  
  // ConfigurationOverlays to be shown are defined by an argument passed after hash
  // filtering conditions are below  
  if ( location.hash == "#HDFS" ) tt = getFilteredOverlays( hadoopTypesToRoles['HDFS'] );
  else if ( location.hash == "#MRv1" ) tt = getFilteredOverlays( hadoopTypesToRoles['MRv1'] );
  else if ( location.hash == "#YARN" ) tt = getFilteredOverlays( hadoopTypesToRoles['YARN'] );
  else if ( location.hash == "#HBase" ) tt = getFilteredOverlays( hadoopTypesToRoles['HBase'] );
  else if ( location.hash == "#Spark" ) tt = getFilteredOverlays( hadoopTypesToRoles['Spark'] );
  else if ( location.hash == "#Hive" ) tt = getFilteredOverlays( hadoopTypesToRoles['Hive'] );
  else if ( location.hash == "#Sqoop" ) tt = getFilteredOverlays( hadoopTypesToRoles['Sqoop'] );
  else if ( location.hash == "#ZooKeeper" ) tt = getFilteredOverlays( hadoopTypesToRoles['ZooKeeper'] );
  else tt = getFilteredOverlays( hadoopservicenames );
  
  //println("CO-length " + location.hash + " :" + tt.length);
  
  // depending on installed Hadoop Services (e.g., HDFS, YARN, HBase, etc.) build 
  // a list of roles available for manipulation and a list of 'nonmovable' roles  
  listRoles = new Array();
  nonmovableRoles = new Array();

  if (current.object.enableFailover) {
    listRoles = listRoles.concat(Object.keys(hadoopTypesToRoles['HDFS-HA-movable']));
    nonmovableRoles = nonmovableRoles.concat(Object.keys(hadoopTypesToRoles['HDFS-HA-non-movable']));
  }
  else {
    listRoles = listRoles.concat(Object.keys(hadoopTypesToRoles['HDFS-movable']));
    nonmovableRoles = nonmovableRoles.concat(Object.keys(hadoopTypesToRoles['HDFS-non-movable']));
  } 
  if ( current.checkHadoopService('MRv1') ) { 
    listRoles = listRoles.concat(Object.keys(hadoopTypesToRoles['MRv1-movable']));
    nonmovableRoles = nonmovableRoles.concat(Object.keys(hadoopTypesToRoles['MRv1-non-movable']));
  }  
  if ( current.checkHadoopService('YARN') ) {
    listRoles = listRoles.concat(Object.keys(hadoopTypesToRoles['YARN-movable']));  
    nonmovableRoles = nonmovableRoles.concat(Object.keys(hadoopTypesToRoles['YARN-non-movable']));
  }  
  if ( current.checkHadoopService('HBase') ) { 
    listRoles = listRoles.concat(Object.keys(hadoopTypesToRoles['HBase-movable']));  
    nonmovableRoles = nonmovableRoles.concat(Object.keys(hadoopTypesToRoles['HBase-non-movable']));
  }  
  if ( current.checkHadoopService('Spark') ) {
    listRoles = listRoles.concat(Object.keys(hadoopTypesToRoles['Spark-movable']));  
    nonmovableRoles = nonmovableRoles.concat(Object.keys(hadoopTypesToRoles['Spark-non-movable']));
  }  
  if ( current.checkHadoopService('Hive') )  listRoles = listRoles.concat(Object.keys(hadoopTypesToRoles['Hive']));  
  if ( current.checkHadoopService('Sqoop') )  listRoles = listRoles.concat(Object.keys(hadoopTypesToRoles['Sqoop']));  
  if ( current.checkHadoopService('ZooKeeper') )  listRoles = listRoles.concat(Object.keys(hadoopTypesToRoles['ZooKeeper']));  
  if ( current.checkHadoopService('KMS') )  listRoles = listRoles.concat(Object.keys(hadoopTypesToRoles['KMS']));  

  for(var k = 0; k < tt.length; k++)    data.push( createTreeItem(tt[k],hdfskey) );

  for(var i = data.length; i < MINTREEITEMS; i++)   data.push(blankTreeItem());
  var select = selected('list');
  loadTable();
  setSelected('list', select);
  
  // ML: disable save button if no modification pending
  var modified = false;
  for(var i = 0; i < data.length; i++) {
    if(data[i].modified) {
      modified = true;
      break;
    }
  }
  document.getElementById('saveCO').setAttribute('disabled', !modified);
}


function getSelected() {
  return selected('list');
}


function createTreeItem(one,hdfsKey) {
  var item = new Object();

  item['clusterref'] = cluster; 
  
  item['hdfskey'] = hdfsKey;
  item['key'] = one.get('uniqueKey');
  item['modified'] = one.get('modified');

  if ( item['modified'] ) {
    document.getElementById('saveCO').removeAttribute('disabled');
  }

  item['configgroupname'] = one.resolveName();

  item['nodesconfigured'] = dotFormatKeys(cluster, one.get('nodes'));

  item['priority'] = one.get('priority');

  //var roles = one.get('roles');
  var roles = one.getHadoopRoles(hdfsKey, false);

  item['hadoopservice'] = new Array();
  // item['hadoopservices'] = new Object();
  // item['configs'] = new Object();

  for(var j = 0; j < roles.length; j++) {
    if (!roles[j].toBeRemoved ) {
      item['hadoopservice'].push(hadoopservicenames[roles[j].childType]);
    }
  }  

  return item;
}


function editObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a configuration overlay");
  } else if(select.length > 1) {
    dock.displayError("Please select only one configuration overlay");
  } else {
    var k = 0;
    while ( data[k].key != select[0] )  k++;
    
    var args = new Object();
    args['hdfs'] = hdfskey;
    args['object'] = cluster.find(select[0]);
    args['availroles'] = listRoles;
    args['nonmovroles'] = nonmovableRoles;

    var xulfilename = dock.myVersion + 'hadoop/edithadoopconfgroupsettings.xul';

    var d = new dock.cvDialog(xulfilename,args);
    d.onOk = function() {

      // if location hash defined (e.g., set to "#HDFS") then parent of parent window 
      // has to be refreshed
      // similar in other places for New, Clone, Revert operations
      // Note: refreshing for Revert operation is defined in generictreecell.js
      if ( location.hash )    parent.parent.refresh();
      else    parent.refresh();   
    }

    dock.dialogs.showDialog(d);
  }
}


function newObject() {
    
  var args = new Object();
  args['hdfs'] = hdfskey;
  args['availroles'] = listRoles;
  args['nonmovroles'] = nonmovableRoles;

  var newconfgroup = new dock.cvConfigurationOverlay();
  newconfgroup.name = current.get('name') + "-";
  newconfgroup.priority = 500;
    
  // setting false to second and third arguments is important as it avoids 
  // switching to main cluster overview window when a configuration group 
  // window is closed or cancelled
  args['object'] = cluster.addObject(newconfgroup,false,false);

  var xulfilename = dock.myVersion + 'hadoop/edithadoopconfgroupsettings.xul#new';

  var d = new dock.cvDialog(xulfilename,args);
  d.onOk = function() {
    if ( location.hash )    parent.parent.refresh();
    else    parent.refresh();     
  }

  dock.dialogs.showDialog(d);
}


function cloneObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a configuration overlay");
  } else if(select.length > 1) {
    dock.displayError("Please select only one configuration overlay");
  } else {
    var k = 0;
    while ( data[k].key != select[0] )  k++;

    var args = new Object();
    args['hdfs'] = hdfskey;
    args['availroles'] = listRoles;
    args['nonmovroles'] = nonmovableRoles;

    var newcloned = cluster.find(select[0]).clone();
    newcloned.name = data[k].configgroupname + "-cloned";
     
    var intPriority = parseInt(data[k].priority);
    if ( intPriority == -1  )   newcloned.priority = -1;
    else if ( intPriority >= 990 )    newcloned.priority = 1000;
    else    newcloned.priority = intPriority + 10;

    // setting false to second and third arguments is important as it avoids 
    // switching to main cluster overview window when a configuration group 
    // window is closed or cancelled
    args['object'] = cluster.addObject(newcloned,false, false);

    var xulfilename = dock.myVersion + 'hadoop/edithadoopconfgroupsettings.xul#clone';

    var d = new dock.cvDialog(xulfilename,args);
    d.onOk = function() {
      if ( location.hash )    parent.parent.refresh();
      else    parent.refresh();   
    }

    dock.dialogs.showDialog(d);
  }
}

function removeObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError('Please select a ' + singular);
  } else {
    var cluster = current;
    if(!!current.cluster) cluster = current.cluster;
    var args = new Object();
    if(select.length == 1) {
      args['question'] = 'Are you sure you want to remove this ' + singular + '?';
    } else {
      args['question'] = 'Are you sure you want to remove these ' + plural + '?';
    }
    var d = new dock.cvDialog(dock.myVersion + 'question.xul', args);
    d.onOk = function() {
      for(var i = 0; i < select.length; i++)   cluster.removeObject(select[i]);
      refresh();
    }
    dock.dialogs.showDialog(d);
  }
}

function getHadoopKey() {
  if ( current.getType() == 'HadoopHDFS' )   return current.get('uniqueKey');
  else   return;
}

function getFilteredOverlays(baseTypes) {    
  var overlays = cluster.find('ConfigurationOverlay');

  var filteredOverlays = new Array();
  
  for(var k = 0; k < overlays.length; k++) {
    var roles = overlays[k].getHadoopRoles(hdfskey, false);
 
    for(var j = 0; j < roles.length; j++) {
      if ( baseTypes[roles[j].childType] ) {
        filteredOverlays.push(overlays[k]); break;      
      } 
    }
  }
  return filteredOverlays;    
}

function checkHadoopRoleAssignment() {
  var overlays = cluster.find('ConfigurationOverlay');
  var hadoopRoleWarnings = new Array();
  var text = 'Warning: Hadoop role(-s) below are not assigned/configured via Hadoop configuration groups\n\n';

  if ( overlays.length > 0) {
    var map = overlays[0].buildConfigurationOverlayNodeRoleMapping(hdfskey);

    for (var key in map) {
      for(var m = 0; m < map[key].length; m++) {
        if ( map[key][m].name == '___NODE' ) {
          var dat = key.split('/');
          hadoopRoleWarnings.push('Role ' + hadoopservicenames[dat[1]] + ' explicitly assigned to ' + cluster.find(dat[0]).resolveName());
        }
        if ( map[key][m].name == '___CATEGORY' ) {
          var dat = key.split('/');
          hadoopRoleWarnings.push('Role ' + hadoopservicenames[dat[1]] + ' implicitly assigned (via category) to ' + cluster.find(dat[0]).resolveName());
        }
      }
    }
  }
 
  if ( hadoopRoleWarnings.length > 0) {
    for(var k = 0; k < hadoopRoleWarnings.length; k++)    text += '\t' + hadoopRoleWarnings[k] + '\n';      
    dock.displayInfo(text + '\nUse cmsh to assign/unassign/configure these roles.');
  }
  
  return;
}


// not in use; left for reference
/*
function getConfigsForRole(rolename) {

  var configs = new Array();

  var overlays = current.cluster.find('ConfigurationOverlay');
  for(var k = 0; k < overlays.length; k++) {
    var roles = overlays[k].get('roles');
    for(var j = 0; j < roles.length; j++) {
      if(roles[j].childType == rolename) {
        for(var i = 0; i < roles[j].config.length; i++) {
          // println ("######################" + roles[j].config[i].hdfs + " ---- " + current.get('uniqueKey'));
          if(roles[j].config[i].hdfs == current.get('uniqueKey')) {
            var role = new dock.cvObject(current.cluster, roles[j], overlays[k], 'roles');
            var config = new dock.cvObject(current.cluster, roles[j].config[i], role, 'config');
            configs.push(config);
            printall(roles[j].config[i]);
          } 
          // else {
          //   printall(roles[j]);
          //   printall(roles[j].config[i]);
          // }
        }
      }
    }
  }
  return configs;
}
*/
