/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var overview;

var oldcluster = 0;
var allHadoopNodeKeys;

var PATTERN_ID = /^metriclist(\d+)$/g;

var timer = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function getCurrentSelection() {
  return [[current], 0, ""];
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh HDFS Tab" + ' - ' + document.documentURI);

  var isHDFS = current.checkHadoopService('HDFS');
  if (isHDFS) {
    for(var i = 0; i < window.frames.length; i++)  window.frames[i].refresh(current);
    update();
    var cluster = current.cluster;
    if(cluster != oldcluster) {
      setCallbackResult(new dock.cvGuiClusterOverview());    
      oldcluster = cluster;
    }  
  }
}

function update() {
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmgui', 'getHadoopHdfsDetailHDFS', [current.cluster.sessionid, current.get('uniqueKey')], function(code, data) {
    if(code == 0) {
      setCallbackResult(data);
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);
}

function setCallbackResult(value) {
  overview = value;
  
  overviewValue('blocksTotal');
  overviewValue('filesTotal');
  
  // to get percentage values for capacities: dfs-used, dfs-remaining, non-dfs-used, where
  // capacityTotal is 100% 
  var cu = 100 * overview.capacityUsed/overview.capacityTotal;
  var cr = 100 * overview.capacityRemaining/overview.capacityTotal;
  var cndfs = 100 * (overview.capacityTotal - overview.capacityUsed - overview.capacityRemaining)/overview.capacityTotal;
  
  document.getElementById('capacityUsed').setAttribute('value', formatKMGselect(overview.capacityUsed, 'B') + " (" + cu.toFixed(2) + "%)");
  document.getElementById('capacityRemaining').setAttribute('value', formatKMGselect(overview.capacityRemaining, 'B') + " (" + cr.toFixed(2) + "%)");
  document.getElementById('capacityNonDFSUsed').setAttribute('value', formatKMGselect(overview.capacityTotal - overview.capacityUsed - overview.capacityRemaining, 'B') + " (" + cndfs.toFixed(2) + "%)");

  document.getElementById('blocksErrorStatus').setAttribute('value', overview.corruptBlocks + " corrupt, " + overview.missingBlocks + " missing");
  document.getElementById('blocksReplStatus1').setAttribute('value', overview.underReplicatedBlocks + " under-replicated");
  document.getElementById('blocksReplStatus2').setAttribute('value', overview.pendingReplicationBlocks + " pending replication");
  document.getElementById('blocksReplStatus3').setAttribute('value', overview.scheduledReplicationBlocks + " scheduled for replication");

  document.getElementById('datanodesLiveDead').setAttribute('value', overview.available_datanodes + " live / " + overview.dead_datanodes + " dead");
  document.getElementById('datanodesDecommission').setAttribute('value', overview.decommissioned_datanodes + " decommissioned, " + overview.decommission_in_progress_datanodes + " decommission in progress");

  overviewBar('namenodeJVMMemory', overview.namenode_MemHeapUsed+overview.namenode_MemNonHeapUsed, overview.namenode_MemHeapCommitted+overview.namenode_MemNonHeapCommitted, 'B', true);

  if (current.object.extraField1 == 'gpfs') {
    document.getElementById('standbynn-label').setAttribute('hidden', true);
    document.getElementById('standbynamenodeJVMMemory').setAttribute('hidden', true);
    document.getElementById('snn-label').setAttribute('hidden', true);
    document.getElementById('secnamenodeJVMMemory').setAttribute('hidden', true);
  } else if (current.object.enableFailover) {
    document.getElementById('standbynn-label').removeAttribute('hidden');
    document.getElementById('standbynamenodeJVMMemory').removeAttribute('hidden');
    document.getElementById('snn-label').setAttribute('hidden', true);
    document.getElementById('secnamenodeJVMMemory').setAttribute('hidden', true);
    overviewBar('standbynamenodeJVMMemory', overview.secondarynamenode_MemHeapUsed+overview.secondarynamenode_MemNonHeapUsed, 
                overview.secondarynamenode_MemHeapCommitted+overview.secondarynamenode_MemNonHeapCommitted, 'B', true);
  } else {
    document.getElementById('standbynn-label').setAttribute('hidden', true);
    document.getElementById('standbynamenodeJVMMemory').setAttribute('hidden', true);
    document.getElementById('snn-label').removeAttribute('hidden');
    document.getElementById('secnamenodeJVMMemory').removeAttribute('hidden');
    overviewBar('secnamenodeJVMMemory', overview.secondarynamenode_MemHeapUsed+overview.secondarynamenode_MemNonHeapUsed, 
                overview.secondarynamenode_MemHeapCommitted+overview.secondarynamenode_MemNonHeapCommitted, 'B', true);
  }

  overviewBar('totaldatanodeJVMMemory', overview.total_datanode_MemHeapUsed+overview.total_datanode_MemNonHeapUsed, overview.total_datanode_MemHeapCommitted+overview.total_datanode_MemNonHeapCommitted, 'B', true);
  
  var ra_elem = "OFF";
  if ( current.get('topology') != 'NONE' ) ra_elem = "ON";
  document.getElementById('rackAwarenessStatus').setAttribute('value', ra_elem);

  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 500);
}
