/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

// common functions for svg graphs

function refreshGraph(cluster, index) {
  var list = document.getElementById("metriclist" + index);
  var oldkey = parseInt(list.getAttribute('metricKey'));
  var oldparam = list.getAttribute('param');
  var oldlabel = list.getAttribute('label');
  var groupType = DEFAULT_TYPES[index];

  var noOldKey = 0;
  if(oldkey) {
    var metric = cluster.find(oldkey);
    if(metric) {
      var name = metric.resolveName() + "_";
      if(startswith(metric.get('metricClass'), 'HADOOP_') && !startswith(name, current.resolveName())) {
        println("Old metric for different Hadoop " + name + " " + current.resolveName());
        noOldKey = oldkey;
        oldkey = 0;
        oldparam = '';
        oldlabel = '';
      }
    }
  }

  if(kisNaN(oldkey) || (oldkey == 0)) {
    if(isArray(cluster.hadoopMetric)) {
      if((noOldKey == 0) || (noOldKey != cluster.hadoopMetric[0])) {
        oldkey = cluster.hadoopMetric[0];
        oldparam = cluster.hadoopMetric[1];
        oldlabel = '';
        var metric = cluster.find(cluster.hadoopMetric[0]);
        if(metric) {
          var text = metric.resolveName();
          if(cluster.hadoopMetric[1].length > 0) text += "[" + cluster.hadoopMetric[1].metricParam + "]";
          if(metric.get('unit')) text += " (" + metric.get('unit') + ")";
          oldlabel = text;
        }
      }
    }
  }

  var mmc = cluster.allMetrics();
   
  var hiName = current.resolveName();

  // used in createMetricPopup() to filter out Hadoop metrics for another Hadoop instances
  // E.g., in case of two or more instances, hdfs1 (current), hdfs2, ...: 
  //     metrics for hdfs2, ... should not be shown)
  // currently, it only checks metrics from classes start with 'HADOOP_*'; 
  // those from 'SPARK_*' classes are not checked
  function hidefunction(metric) {
    if ( metric._mclass.indexOf('HADOOP') === 0 &&  !(metric._mtext.indexOf(hiName) === 0) ) return true;
  }

  createMetricPopup(cluster, mmc, 'metriclist' + index, false, true, ['OPENSTACK','CEPH','AMAZONEC2','CLOUD','DELLNSS', 'LUSTRE'], hidefunction);

  var found = false;
  if(oldkey) {
    for(var i = 0; (i < mmc.length) && !found; i++) {
      found = ((mmc[i].metric == oldkey) && (mmc[i].metricParam == oldparam));
    }
  }

  if(!found) {
    if(mmc.length == 0) {
      var item = document.createElement("menuitem" + index);
      item.setAttribute('metricKey', 0);
      item.setAttribute('param', '');
      item.setAttribute('label', '');
      list.firstChild.appendChild(item);
      oldkey = 0;
      oldparam = '';
      oldlabel = '';
    } else {
      oldkey = mmc[0].metric;
      oldparam = mmc[0].metricParam;
      var metric = null;

      // initialize metric via user preferences
      var def = BigDataPreferences.get(
          current.cluster.resolveName(),
          BIGDATA_GRAPHS_FOR,
          current.resolveName(),
          index /*graphIndex*/,
          0 /*metricIndex*/
      );
      if (def.length === 3) {
        oldkey = def[0];
        oldparam = def[1];
        groupType = def[2];
        metric = cluster.find(oldkey);
      }

      // initialize metric via defaults
      if (!metric) {
        if (current.object.lustreFS) {
          def = current.resolveName() + '_resourcemanager_SnapshotAvgTime';
        } else if (current.object.isSparkInstance) {
          def = current.resolveName() + DEFAULT_METRICS_SPARK[index];
        } else if (current.object.isCassandraInstance) {
          def = current.resolveName() + DEFAULT_METRICS_CASSANDRA[index];
        } else {
          def = current.resolveName() + DEFAULT_METRICS[index];
        }
        def = cluster.find('Metric', def);
        if(def) {
          def = def.get('uniqueKey');
          for(var i = 0; i < mmc.length; i++) {
            if(mmc[i].metric == def) {
              oldkey = def
              oldparam = mmc[i].metricParam;
              break;
            }
          }
        }
        metric = cluster.find(oldkey);
      }

      if(metric) {
        oldlabel = metric.resolveName();
        if(oldparam.length > 0) oldlabel += "[" + oldparam + "]";
        if(metric.get('unit')) oldlabel += " (" + metric.get('unit') + ")";
      }
    }
  }
  list.setAttribute('metricKey', oldkey);
  list.setAttribute('param', oldparam);
  list.setAttribute('label', oldlabel);

  if(graphs[index].deviceMetric.length > 0) {
    graphs[index].deviceMetric[0].nodes = allHadoopNodeKeys;
    graphs[index].deviceMetric[0].metric = oldkey;
    graphs[index].deviceMetric[0].param = oldparam;
  } else {
    var dmp = new cvGraphDeviceMetric(0, oldkey, oldparam, allHadoopNodeKeys);
    dmp.groupType = groupType;
    graphs[index].allowRedraw(true);
    var r = graphs[index].add(dmp);
  }
  redraw(index);
}

function redraw(index) {
  if(graphs[index]) {
    graphs[index].update();
  }
}

function selectMetric(event) {
  var metric = event.target;
  if(!metric.parentNode) {
    return;
  }
  var metricListIndex;
  var currParent = metric.parentNode;
  while(currParent) {
    if(currParent.nodeName == 'menulist') {
      var parentId = currParent.id;
      if(parentId.match(PATTERN_ID)) {
        var m = PATTERN_ID.exec(parentId);
        if(m) {
          metricListIndex = m[1];
          break;
        }
      }
    }
    currParent = currParent.parentNode;
  }

  if(!!metricListIndex) {
    graphs[metricListIndex].last = 0;
    var old = graphs[metricListIndex].deviceMetric[0].device;
    graphs[metricListIndex].deviceMetric[0].metric = parseInt(metric.getAttribute('metricKey'));
    graphs[metricListIndex].deviceMetric[0].param = metric.getAttribute('param');
    graphs[metricListIndex].deviceMetric[0].unit = current.cluster.find(graphs[metricListIndex].deviceMetric[0].metric).get('unit');
    var name = metric.getAttribute('label');
    var list = document.getElementById("metriclist" + metricListIndex);
    list.setAttribute('metricKey', graphs[metricListIndex].deviceMetric[0].metric);
    list.setAttribute('param', graphs[metricListIndex].deviceMetric[0].param);
    list.setAttribute('label', name);
    if(old != graphs[metricListIndex].deviceMetric[0].device) {
      graphs[metricListIndex].handleDeviceChange(graphs[metricListIndex].deviceMetric[0]);
    }
    redraw(metricListIndex);
    current.hadoopMetric = [graphs[metricListIndex].deviceMetric[0].metric, graphs[metricListIndex].deviceMetric[0].param];
    for (var i=0; i<graphs.length; i++) {
      if (graphs[i]['deviceMetric'].length > 0) {
        BigDataPreferences.set(
          current.cluster.resolveName(),
          BIGDATA_GRAPHS_FOR,
          current.resolveName(),
          i,
          0,
          [ graphs[i]['deviceMetric'][0].metric,
            graphs[i]['deviceMetric'][0].param,
            graphs[i]['deviceMetric'][0].groupType ]
        );
      }
    }
  }
}
