var dock;
var current;

var nodesUp;
var nodesDown;
var nodesClosed;
var decommConfOverlay;

var overview;
var timer = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function getCurrentSelection() {
  return [[current], 0, ""];
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh HDFS operations" + ' - ' + document.documentURI);

  // this call provides info about the Hadoop instance, particularly, if HDFS is in HA-mode
  // 
  update();
}


function update() {
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmgui', 'getHadoopHdfsOverview', [current.cluster.sessionid, current.get('uniqueKey')], function(code, data) {
      if(code == 0) {
        setCallbackResult(data);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
}


function setCallbackResult(value) {
  overview = value;

  // println("=== HDFS operations ===");
  // printall(overview);
  // println("======");  
  // printall(current.object);

  var obj = document.getElementById('oneFailoverInfo');
  if (current.object) {
    if (current.object.enableFailover && current.object.automaticFailover == "DISABLED") {
      var hdfsKey = current.get('uniqueKey');
      if (overview.haPairs.length > 0) {
        while (obj.childNodes.length > 2) obj.removeChild(obj.lastChild);

        for (var i = 0; i < overview.haPairs.length; i += 4) {
          var row = document.createElement('row');
          row.setAttribute("class", "staticdatarow");
          row.setAttribute("align", "center");

          var label = document.createElement('label');
          label.setAttribute("class", "staticdatavaluesmall");

          var nn1 = document.createElement('button');
          var nn2 = document.createElement('button');
          var currentState = document.createElement('textbox');

          nn1.setAttribute('class', 'bluebutton');
          nn1.setAttribute("align", "center");

          nn2.setAttribute('class', 'bluebutton');
          nn2.setAttribute("align", "center");
          currentState.setAttribute("flex", "1");

          var info = "";
          var group = "";
          var to = 0, from = 0;
          
          var node1 = current.cluster.find(overview.haPairs[i]);
          if (node1) {
            nn1.setAttribute('label', node1.get('hostname'));
            group += node1.get('hostname');
            info += node1.get('hostname')
            if (overview.haPairs[i+1] == 0) {
              nn1.setAttribute('disabled', false);
              info += ":standby ";
              to = overview.haPairs[i];
            } else if (overview.haPairs[i+1] == 1) {
              nn1.setAttribute('disabled', true);
              info += ":active ";
            } else {
              nn1.setAttribute('disabled', false);
              info += " :unknown ";
            }
          }
          if (i+2 < overview.haPairs.length) {
            var node2 = current.cluster.find(overview.haPairs[i+2]);
            if (node2) {
              nn2.setAttribute('label', node2.get('hostname'));
              group += "/" + node2.get('hostname');
              info += node2.get('hostname');
              if (overview.haPairs[i+3] == 0) {
                nn2.setAttribute('disabled', false);
                info += ":standby ";
                to = overview.haPairs[i+2];
                from = overview.haPairs[i];
              } else if (overview.haPairs[i+3] == 1) {
                nn2.setAttribute('disabled', true);
                info += ":active ";
                to = overview.haPairs[i];
                from = overview.haPairs[i+2];                
              } else {
                from = overview.haPairs[i+2];
                to = overview.haPairs[i];
                info += ": unknown ";
                if (overview.haPairs[i+1] == 0) { // both are in standby
                  nn1.setAttribute('disabled', false);
                } else {
                  nn2.setAttribute('disabled', false);  // TODO: enable/disable if unknown ?
                }
              }
            }
          }

          label.setAttribute("value", group);
          currentState.setAttribute('value', info);
          nn1.setAttribute('oncommand', "hdfsManualNameNodeFailover("+from +", "+to+");");
          nn2.setAttribute('oncommand', "hdfsManualNameNodeFailover("+from +", "+to+");");

          row.appendChild(label);
          row.appendChild(nn1);
          row.appendChild(nn2);
          row.appendChild(currentState);

          obj.appendChild(row);
          obj.setAttribute('hidden', false);
        }
        obj = document.getElementById('failoverInfo');
        obj.setAttribute('hidden', false);
      } else {
        obj.setAttribute('hidden', true);
        obj = document.getElementById('failoverInfo');
        obj.setAttribute('hidden', true);
      }
    } else {
      obj.setAttribute('hidden', true);
      obj = document.getElementById('failoverInfo');
      obj.setAttribute('hidden', true);
    }
    if (current.object.lustreFS) {
      document.getElementById('hdfsstartbalance').setAttribute('disabled', true);
      document.getElementById('hdfsstopbalance').setAttribute('disabled', true);
    } else {
      document.getElementById('hdfsstartbalance').setAttribute('disabled', false);
      document.getElementById('hdfsstopbalance').setAttribute('disabled', false);
    }
  }

  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 1000);  
}



function startHDFS() {
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  var hdfsname = current.get('name');

  args['question'] = "Start all HDFS services on " + hdfsname + "?";
  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();

    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'startServiceForHDFS', [hdfskey, 0], function(code, data) {
      if(code == 0) {
        if(data.output.length == 0) {
          dock.displayInfo('HDFS start returned no output and error code: ' + data.code);
        } else {
          dock.displayInfo(data.output);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
  dock.dialogs.showDialog(d);

}

function stopHDFS() {
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  var hdfsname = current.get('name');

  args['question'] = "Stop all HDFS services on " + hdfsname + "?";
  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();

    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'stopServiceForHDFS', [hdfskey, 0], function(code, data) {
      if(code == 0) {
        if(data.output.length == 0) {
          dock.displayInfo('HDFS stop returned no output and error code: ' + data.code);
        } else {
          dock.displayInfo(data.output);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
  dock.dialogs.showDialog(d);

}


function decommissionHDFS() {
  var hdfskey = current.get('uniqueKey');
  var hdfsname = current.get('name');
  
  current.object.nodes = [];
  // printall( current.object );

  var overlays = current.cluster.find('ConfigurationOverlay');
  /*
  for(var k = 0; k < overlays.length; k++) {
    if ( overlays[k].resolveName() == hdfsname + "-decommissioned-datanodes")	break;
  }
  */
  var xulfilename = dock.myVersion + 'hadoop/hadoopnodeselection.xul#datanodesDecomm';
  var args = new Object();
  args['hi'] = current;
  args['mapcoroles'] = overlays[0].buildConfigurationOverlayNodeRoleMapping(hdfskey);
  args['initialize'] = function (refreshFunc) {
    dock.waitIcon();
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'getHadoopDecommissionStatus', [hdfskey, 'hdfs', HADOOP_DECOMMISSION_STATUS_INCLUDE_JMX], handle_getHadoopDecommissionStatusResponse.bind({'refreshFunc' : refreshFunc, 'dock' : dock}));
  };
  var d = new dock.cvDialog(dock.myVersion + xulfilename,args);

  var self = this;
  d.onOk = function() {
    var nodes = current.get('nodes');
    if (nodes.length === 0) {
      return;
    }
    dock.waitIcon();
    var decommissionHdfsOnly = getHadoopDrainAction(['decommissionHdfs'], HADOOP_DRAIN_MERGE);
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmjob', 'drainNodes', ['', '', nodes, decommissionHdfsOnly, [], [], false], 
        handle_drainNodesResponse.bind({'self' : self, 'hdfskey' : hdfskey, 'filter' : 'hdfs'}), current);
  }
  dock.dialogs.showDialog(d);
  // if ( decommConfOverlay.get('uniqueKey') )  current.cluster.removeObject(decommConfOverlay.get('uniqueKey'));

}

function decommissionStatusOverview() {
  var self = this;
  var hdfskey = current.get('uniqueKey');
  showDecommissionStatusOverviewWindow(self, hdfskey, 'hdfs');
}

function commissionHDFS() {
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  var hdfsname = current.get('name');

  current.object.nodes = [];
  var overlays = current.cluster.find('ConfigurationOverlay');

  var xulfilename = dock.myVersion + 'hadoop/hadoopnodeselection.xul#datanodesComm';
  var args = new Object();
  args['hi'] = current;
  args['mapcoroles'] = overlays[0].buildConfigurationOverlayNodeRoleMapping(hdfskey);
  args['initialize'] = function (refreshFunc) {
    dock.waitIcon();
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'getHadoopDecommissionStatus', [hdfskey, 'hdfs', HADOOP_DECOMMISSION_STATUS_INCLUDE_JMX], handle_getHadoopDecommissionStatusResponse.bind({'refreshFunc' : refreshFunc, 'dock' : dock}));
  };
  var d = new dock.cvDialog(xulfilename,args);

  var self = this;
  d.onOk = function() {
    var nodes = current.get('nodes');
    if (nodes.length === 0) {
      return;
    }
    dock.waitIcon();
    var commissionHdfsOnly = getHadoopDrainAction(['decommissionHdfs'], HADOOP_DRAIN_SUBSTRACT);
      dock.workerpool.getWorker(current.cluster.actualurl).call('cmjob', 'drainNodes', ['', '', nodes, commissionHdfsOnly, [], [], false],
      handle_drainNodesResponse.bind({'self' : self, 'hdfskey' : hdfskey, 'filter' : 'hdfs'}), current
    );
  }
  dock.dialogs.showDialog(d);
}

function restartHDFS() {
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  var hdfsname = current.get('name');

  args['question'] = "Restart all HDFS services on " + hdfsname + "?";
  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();

    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'restartServiceForHDFS', [hdfskey, 0], function(code, data) {
      if(code == 0) {
        if(data.output.length == 0) {
          dock.displayInfo('HDFS restart returned no output and error code: ' + data.code);
        } else {
          dock.displayInfo(data.output);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
  dock.dialogs.showDialog(d);

}

function startBalancer() {
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  // var hdfsname = current.get('name');

  args['question'] = "Start HDFS balancer?";
  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();

    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'startBalancer', hdfskey, function(code, data) {
      if(code == 0) {
        if(data.output.length == 0) {
          dock.displayInfo('Starting HDFS balancer returned no output and error code: ' + data.code);
        } else {
          dock.displayInfo(data.output);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
  dock.dialogs.showDialog(d);
}

function stopBalancer() {
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  // var hdfsname = current.get('name');

  args['question'] = "Stop HDFS balancer?";
  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();

    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'stopBalancer', hdfskey, function(code, data) {
      if(code == 0) {
        if(data.output.length == 0) {
          dock.displayInfo('Stopping HDFS balancer returned no output and error code: ' + data.code);
        } else {
          dock.displayInfo(data.output);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
  dock.dialogs.showDialog(d);
}

function safemodeOn() {
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  // var hdfsname = current.get('name');

  args['question'] = "Enter HDFS safe mode?";
  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();

    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'enterSafeMode', hdfskey, function(code, data) {
      if(code == 0) {
        if(data.output.length == 0) {
          dock.displayInfo('Entering safe mode returned no output and error code: ' + data.code);
        } else {
          dock.displayInfo(data.output);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
  dock.dialogs.showDialog(d);
}

function safemodeOff() {
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  // var hdfsname = current.get('name');

  args['question'] = "Leave HDFS safe mode?";
  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();

    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'leaveSafeMode', hdfskey, function(code, data) {
      if(code == 0) {
        if(data.output.length == 0) {
          dock.displayInfo('Leaving safe mode returned no output and error code: ' + data.code);
        } else {
          dock.displayInfo(data.output);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
  dock.dialogs.showDialog(d);
}


function hdfsFormat() {
  var args = new Object();
  args['question'] = "Format HDFS?\nThis operation will delete all data from all datanodes!";
  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();
    var key = current.get('uniqueKey');
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'formatHDFS', [key, 3], function(code, data) {
      if(code == 0) {
        dock.displayInfo(data.output);
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }

  dock.dialogs.showDialog(d);
}

function hdfsManualNameNodeFailover(nnFrom, nnTo) {
  var key = current.get('uniqueKey');
  dock.waitIcon();
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'manualNameNodeFailover', [key, nnFrom, nnTo, []], function(code, data) {
    if (code == 0) {
      dock.displayInfo(data.output);
    } else {
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
  }, current);

}
