/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var overview;

var oldcluster = 0;
var allHadoopNodeKeys;

var graphs;
var NUM_GRAPHS = 1;
var PATTERN_ID = /^metriclist(\d+)$/g;
var DEFAULT_METRICS = ['_datanode_dfs_used'];
var DEFAULT_METRICS_SPARK = ['_sparkworker_MemHeapUsage'];
var DEFAULT_METRICS_CASSANDRA = ['_cassandra_MemHeapUsed'];
var DEFAULT_TYPES = [1];
var BIGDATA_GRAPHS_FOR = 'Overview tab';

var timer = 0;

function onLoad() {
  dock = getDock();
  graphs = new Array();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
  addEventListener("resize", onResize, false);
}

function onResize() {
  for(var i = 0; i < NUM_GRAPHS; i++) {
    var svg = document.getElementById("svg" + i);
    svg.setAttribute("height", "25px");
    svg.setAttribute("width", svg.parentNode.boxObject.width);
    svg.setAttribute("height", svg.parentNode.boxObject.height);
    redraw(i);
  }
}

function getCurrentSelection() {
  return [[current], 0, ""];
}

function stop() {
  println("Stop Hadoop hdfs overview");
  for (var i = 0; i < graphs.length; i++) {
    //graphs[i].stop(); ML: enable it?
  }
  if(timer) window.clearTimeout(timer);
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh hadoophdfs overview" + ' - ' + document.documentURI);

  if(!isCorrectlyConfigured(current.cluster, current.object.uniqueKey)) {
    document.getElementById('nohadoop').removeAttribute('hidden');
    document.getElementById('leftcolumn').setAttribute('hidden', true);
    document.getElementById('centralcolumn').setAttribute('hidden', true);
    document.getElementById('rightcolumn').setAttribute('hidden', true);
    document.getElementById('metricsBox').setAttribute('hidden', true);
    document.getElementById('nodesBox').setAttribute('hidden', true);
    return;
  } else {
    document.getElementById('nohadoop').setAttribute('hidden', true);
    document.getElementById('leftcolumn').removeAttribute('hidden');
    document.getElementById('centralcolumn').removeAttribute('hidden');
    document.getElementById('rightcolumn').removeAttribute('hidden');
    document.getElementById('metricsBox').removeAttribute('hidden');
    document.getElementById('nodesBox').removeAttribute('hidden');
  }
  
  //printall(current);
  var list = document.getElementById('nodes');
  while(list.childNodes.length > 2) list.removeChild(list.lastChild);

  // Obtaining nodes running Hadoop services by looping through all configuration overlays  
  var hdfsKey = current.get('uniqueKey');
  var overlays = getAllOverlays(current);
  allHadoopNodeKeys = getAllHadoopNodeKeys(current);
  
  var listRoles = new Object();
  
  if ( overlays.length > 0 ) { 
    var nodeToRoles = overlays[0].buildConfigurationOverlayNodeRoleMapping(hdfsKey);

    for (var nodekey in nodeToRoles) {
      var kparts = nodekey.split("/");
      if ( !(kparts[0] in listRoles) ) listRoles[kparts[0]] = new Array();
      listRoles[kparts[0]].push( hadoopservicenames[kparts[1]] );
    }
    //for (var nodekey in listRoles) {
    //  println(dotFormatKeys(overlays[0].cluster, nodekey) + "::" + listRoles[nodekey].toString());
    //}    

    var elem = document.getElementById('nodes');
    for (var nodekey in listRoles) {
      var row = document.createElement('row');
      row.setAttribute("flex", "1");
      row.setAttribute("align", "center");
      row.setAttribute("height", "15px");
      var label = document.createElement('label');
      label.setAttribute("class", "staticdatavaluesmall");
      label.setAttribute("value", current.cluster.find(nodekey).resolveName());
      row.appendChild(label);
      var label = document.createElement('label');
      label.setAttribute("class", "staticdatavaluesmall");
      label.setAttribute("value", listRoles[nodekey].toString());
      row.appendChild(label);
      elem.appendChild(row);  
    }
  }

  update();
  printall("----->" + allHadoopNodeKeys);

  var cluster = current.cluster;
  if(cluster != oldcluster) {
    setCallbackResult(new dock.cvGuiClusterOverview());
    if(graphs.length == 0) { // init
      for (var i = 0; i < NUM_GRAPHS; i++) {
        graphs[i] = new cvGraph(cluster, document.getElementById("svg" + i), document.getElementById("box" + i));
        graphs[i].WM = 32;
        graphs[i].HM = 61;
        graphs[i].legend = -1;
      }
    } else {
      for (var i = 0; i < graphs.length; i++) {
        graphs[i].cluster = cluster;
        graphs[i].deviceMetric[0].nodes = allHadoopNodeKeys;
        graphs[i].clearScreen();
      }
    }
    oldcluster = cluster;
  }  

  for(var i = 0; i < graphs.length; i++) {
    refreshGraph(cluster, i);
  }
}

function update() {
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmgui', 'getHadoopHdfsOverview', [current.cluster.sessionid, current.get('uniqueKey')], function(code, data) {
    if(code == 0) {
      setCallbackResult(data);
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);
}

function setCallbackResult(value) {
  overview = value;
  
  //printall(overview);

  var isGpfs = (current.object.extraField1 == 'gpfs');

  println("  - " + current.get('name') + " Hadoop instance overview status updated");
  
  if ( current.checkHadoopService('HDFS') ) {
    document.getElementById('hdfs').removeAttribute("hidden");
    overviewValue('available_datanodes');
    overviewValue('dead_datanodes');
    overviewValue('decommissioned_datanodes');
    //overviewValue('decommission_in_progress_datanodes');
    //overviewValue('nameNodeUI');
  } else {
    document.getElementById('hdfs').setAttribute("hidden", true);
  }

  if ( current.checkHadoopService('MRv1') ) {
    document.getElementById('mapreduce').removeAttribute("hidden");
    overviewValue('tasktracker_totalNodes');
    overviewValue('tasktracker_lostNodes');
    overviewValue('jobtracker_trackers_decommissioned');
    //document.getElementById('mapreduce-ui').setAttribute("hidden", false);
    //overviewValue('jobTrackerUI');
  } else {
    document.getElementById('mapreduce').setAttribute("hidden", true);
  }

  if ( current.checkHadoopService('YARN') ) {
    document.getElementById('yarn').removeAttribute("hidden");
    overviewValue('resourcemanager_lostNodes');
    overviewValue('resourcemanager_activeNodes');
    overviewValue('resourcemanager_decommissionedNodes');
    //document.getElementById('yarn-ui').setAttribute("hidden", false);
    //overviewValue('resourceManagerUI');
  } else {
    document.getElementById('yarn').setAttribute("hidden", true);
  }

  if ( current.checkHadoopService('HBase') ) {
    document.getElementById('hbase').removeAttribute("hidden");
    overviewValue('hbaseclient_lostNodes');
    overviewValue('hbaseclient_decommissioned');
    overviewValue('hbaseclient_totalNodes');
    //document.getElementById('hbase-ui').setAttribute("hidden", false);
    //overviewValue('hbaseMasterUI');
  } else {
    document.getElementById('hbase').setAttribute("hidden", true);
  }

  if ( current.checkHadoopService('Spark') && !current.checkHadoopService('Spark-YARN') ) {
    document.getElementById('spark').removeAttribute("hidden");
    overviewValue('sparkworker_totalNodes');
    overviewValue('sparkworker_lostNodes');
    //document.getElementById('spark-ui').setAttribute("hidden", false);
    //overviewValue('sparkMasterUI');
  } else {
    document.getElementById('spark').setAttribute("hidden", true);
  }

  if ( current.checkHadoopService('ZooKeeper') ) {
    document.getElementById('zookeeper').removeAttribute("hidden");
    document.getElementById('zkprServersInfo').setAttribute('value', (overview.zookeeper_running_servers-overview.zookeeper_followers) + " leader, " + overview.zookeeper_followers + " followers");
  } else {
    document.getElementById('zookeeper').setAttribute("hidden", true);
  }

  if (current.object.isSparkInstance) {
    document.getElementById('centralcolumn').setAttribute("hidden", true);
    document.getElementById('rightcolumn').setAttribute("hidden", true);
    document.getElementById('criticalEventsLabel').removeAttribute("hidden");
    document.getElementById('criticalEventsLabel').setAttribute('value',  'Spark services critical events:');
    document.getElementById('criticalDatanodeEvents').removeAttribute("hidden");
    document.getElementById('moregraphs').setAttribute("hidden", true);
  } else if (current.object.isCassandraInstance) {
    document.getElementById('centralcolumn').setAttribute("hidden", true);
    document.getElementById('rightcolumn').setAttribute("hidden", true);
    document.getElementById('criticalEventsLabel').setAttribute("hidden", true);
    document.getElementById('criticalDatanodeEvents').setAttribute("hidden", true);
    document.getElementById('moregraphs').setAttribute("hidden", true);
  } else {
    document.getElementById('centralcolumn').removeAttribute("hidden");
    document.getElementById('rightcolumn').removeAttribute("hidden");
    document.getElementById('criticalEventsLabel').removeAttribute("hidden");
    document.getElementById('criticalEventsLabel').setAttribute('value',  'Hadoop services critical events:');
    document.getElementById('criticalDatanodeEvents').removeAttribute("hidden");
    document.getElementById('moregraphs').removeAttribute("hidden");
  }

  if (current.object.alternativeWLM.indexOf("SLURM") != -1) {
    document.getElementById('rightcolumn').setAttribute("hidden", true);
  } else {
    document.getElementById('rightcolumn').removeAttribute("hidden");
  }  
  
  //overviewValue('appsRunning');
  //overviewValue('appsPending');
  //overviewValue('appsSubmitted');
  //overviewValue('appsCompleted');
  //overviewValue('appsFailed');

  overviewValue('capacityTotal', formatKMGselect(overview.capacityTotal, 'B'));
  overviewValue('capacityUsed', formatKMGselect(overview.capacityUsed, 'B'));
  overviewValue('capacityRemaining', formatKMGselect(overview.capacityRemaining, 'B'));

  if (current.object.lustreFS) { 
    document.getElementById('blocksTotalRowId').setAttribute('hidden', true);
    document.getElementById('underReplicatedBlocksRowId').setAttribute('hidden', true);
    document.getElementById('pendingReplicationBlocksRowId').setAttribute('hidden', true);
    document.getElementById('filesTotalRowId').setAttribute('hidden', true);
    document.getElementById('missingBlocksRowId').setAttribute('hidden', true);
    document.getElementById('sendHeartbeatAvgTimeRowId').setAttribute('hidden', true);
    document.getElementById('blockReportAvgTimeRowId').setAttribute('hidden', true);
  } else {
    document.getElementById('blocksTotalRowId').setAttribute('hidden', false);
    document.getElementById('underReplicatedBlocksRowId').setAttribute('hidden', false);
    document.getElementById('pendingReplicationBlocksRowId').setAttribute('hidden', false);
    document.getElementById('filesTotalRowId').setAttribute('hidden', false);
    document.getElementById('missingBlocksRowId').setAttribute('hidden', false);
    document.getElementById('sendHeartbeatAvgTimeRowId').setAttribute('hidden', false);
    document.getElementById('blockReportAvgTimeRowId').setAttribute('hidden', false);

    overviewValue('blocksTotal', formatKMG10(overview.blocksTotal));
    overviewValue('underReplicatedBlocks', formatKMG10(overview.underReplicatedBlocks));
    overviewValue('pendingReplicationBlocks', formatKMG10(overview.pendingReplicationBlocks));
  
    overviewValue('filesTotal', formatKMG10(overview.filesTotal));
    overviewValue('missingBlocks', formatKMG10(overview.missingBlocks));
    overviewValue('sendHeartbeatAvgTime', overview.sendHeartbeatAvgTime, '%u ms');
    overviewValue('blockReportAvgTime', overview.blockReportAvgTime, '%u ms');
  }

  if (current.object.isCassandraInstance) {
    document.getElementById('cassandraNodes').setAttribute('hidden', false);
    overviewValue('cassandra_nodes_joining');
    overviewValue('cassandra_nodes_leaving');
    overviewValue('cassandra_nodes_live');
    overviewValue('cassandra_nodes_moving');
    overviewValue('cassandra_nodes_unreachable');
  } else {
    document.getElementById('cassandraNodes').setAttribute('hidden', true);
  }

//  TODO:
//  overviewValue('sendDataPacketBlockedAvgTime');  
//  overviewValue('ackPacketRtt');  
//  overviewValue('avgDiskFlushTime');  
  overviewValue('criticalDatanodeEvents');

  overviewBar('heapMemory', overview.memHeapUsed, overview.memHeapCommitted, 'B', true);
  overviewBar('nonHeapMemory', overview.memNonHeapUsed, overview.memNonHeapCommitted, 'B', true);
  
//  overviewValue('memHeapUsedMin', formatKMGselect(overview.memHeapUsedMin, 'B'));
//  overviewValue('memHeapUsedMax', formatKMGselect(overview.memHeapUsedMax, 'B'));
  
//  overviewValue('memNonHeapUsedMin', formatKMGselect(overview.memNonHeapUsedMin, 'B'));
//  overviewValue('memNonHeapUsedMax', formatKMGselect(overview.memNonHeapUsedMax, 'B'));
  if (current.object.useFederation) {
    document.getElementById('federationRowId').setAttribute('hidden', false);
    document.getElementById('federationInfo').setAttribute("value", "Yes");
  } else { 
    document.getElementById('federationRowId').setAttribute('hidden', true);
  }

  if (current.object.enableFailover) {
    document.getElementById('haRowId').setAttribute('hidden', false);
    var obj = document.getElementById('haInfo');
    var info = "Yes";
    var haInfo = "";
    if (!(typeof overview.haPairs === "undefined") && overview.haPairs.length > 0) {
      var nodes = current.cluster.find('Node');
      for (var i = 0; i < overview.haPairs.length; i += 2) {
        var node1 = current.cluster.find(overview.haPairs[i]);
        if (node1) {
          haInfo += node1.get('hostname');
          if (overview.haPairs[i+1] == 1) {
            haInfo += "* ";
          }
          haInfo += " ";
        }
      }
      if (haInfo) {
        info += " (" + haInfo;
      }
    }
    info += haInfo ? ", " : " (";
    if (current.object.automaticFailover == "DISABLED") {
      info += "manual failover";
    } else {
      info += "automatic failover";
    }
    /*
    if (current.object.automaticFailover == "DISABLED") {
      info += "automatic failover: Disabled";
    } else {
      info += "automatic failover managed by: " + current.object.automaticFailover
    }*/
    info += ")";
    obj.setAttribute("value", info);
  } else {
    document.getElementById('haRowId').setAttribute('hidden', true);
  }
  if (current.object.enableYARNfailover) {
    document.getElementById('yarnHaRowId').setAttribute('hidden', false);
    var obj = document.getElementById('yarnHaInfo');
    var info = "Yes";
    info += " (";
    if (current.object.automaticYARNfailover == "DISABLED") {
      info += "manual failover";
    } else {
      info += "automatic failover";
    }
    info += ")";
    obj.setAttribute("value", info);
  } else {
    document.getElementById('yarnHaRowId').setAttribute('hidden', true);
  }
  if (current.object.lustreFS) {
    document.getElementById('lustreHadoopRowId').setAttribute("hidden", false);
    var obj = document.getElementById('lustreHadoopInfo');
    var info = "Yes (" + current.object.lustreRootDir + ")";
    obj.setAttribute("value", info);
  } else {
    document.getElementById('lustreHadoopRowId').setAttribute("hidden", true);
  }
 
  if (isGpfs) {
    document.getElementById('gpfsHadoopRowId').setAttribute("hidden", false);
    var obj = document.getElementById('gpfsHadoopInfo');
    var info = "Yes";
    obj.setAttribute("value", info);
  } else {
    document.getElementById('gpfsHadoopRowId').setAttribute("hidden", true);
  }
  
  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 500);  
}

function hadoopNodeRow(roles, hosts) {
  var row = document.createElement('row');
  row.setAttribute("flex", "1");
  row.setAttribute("align", "center");
  row.setAttribute("height", "17px");
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", roles);
  row.appendChild(label);
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", hosts);
  row.appendChild(label);
  return row;
}

function moreGraphs() {
  dock.waitIcon();
  var args = new Object();
  args['cluster'] = current.cluster;
  args['current'] = current;
  args['devices'] = allHadoopNodeKeys;
  var d = new dock.cvDialog(dock.myVersion + "tabbedgraphs.xul", args);
  d.onOk = function() {
  }
  d.onOpened = function() {
    dock.unwaitIcon();
  }
  dock.dialogs.showDialog(d);
}
