/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;

var nodesUp;
var nodesDown;
var nodesClosed;

var overview;
var timer = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function getCurrentSelection() {
  return [[current], 0, ""];
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh MapReduce operations" + ' - ' + document.documentURI);
  // update();
}

function startMapReduce() {
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  var hdfsname = current.get('name');
  args['question'] = "Start all MapReduce services on " + hdfsname + "?";

  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();

     dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'startServiceForHDFS', [hdfskey, 1], function(code, data) {
      if(code == 0) {
        if(data.output.length == 0) {
          dock.displayInfo('MapReduce start returned no output and error code: ' + data.code);
        } else {
          dock.displayInfo(data.output);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
  dock.dialogs.showDialog(d);
}

function stopMapReduce() {
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  var hdfsname = current.get('name');
  args['question'] = "Stop all MapReduce services on " + hdfsname + "?";
  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();

     dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'stopServiceForHDFS', [hdfskey, 1], function(code, data) {
      if(code == 0) {
        if(data.output.length == 0) {
          dock.displayInfo('MapReduce stop returned no output and error code: ' + data.code);
        } else {
          dock.displayInfo(data.output);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
  dock.dialogs.showDialog(d);
}

function restartMapReduce() {
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  var hdfsname = current.get('name');
  args['question'] = "Restart all MapReduce services on " + hdfsname + "?";

  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();

     dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'restartServiceForHDFS', [hdfskey, 1], function(code, data) {
      if(code == 0) {
        if(data.output.length == 0) {
          dock.displayInfo('MapReduce restart returned no output and error code: ' + data.code);
        } else {
          dock.displayInfo(data.output);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
  dock.dialogs.showDialog(d);
}

function decommissionStatusOverview() {
  var self = this;
  var hdfskey = current.get('uniqueKey');
  showDecommissionStatusOverviewWindow(self, hdfskey, 'mapreduce');
}

function commissionMR() {
  var hdfskey = current.get('uniqueKey');
  var hdfsname = current.get('name');
  
  current.object.nodes = [];
  // printall( current.object );

  var overlays = current.cluster.find('ConfigurationOverlay');

  var xulfilename = dock.myVersion + 'hadoop/hadoopnodeselection.xul#tasktrackersComm';
  var args = new Object();
  args['hi'] = current;
  args['mapcoroles'] = overlays[0].buildConfigurationOverlayNodeRoleMapping(hdfskey);
  args['initialize'] = function (refreshFunc) {
    dock.waitIcon();
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'getHadoopDecommissionStatus', [hdfskey, 'mapreduce', HADOOP_DECOMMISSION_STATUS_INCLUDE_JMX], handle_getHadoopDecommissionStatusResponse.bind({'refreshFunc' : refreshFunc, 'dock' : dock}));
  };
  var d = new dock.cvDialog(xulfilename,args);

  var self = this;
  d.onOk = function() {
    var nodes = current.get('nodes');
    if (nodes.length === 0) {
      return;
    }
    dock.waitIcon();
    var commissionMapreduceOnly = getHadoopDrainAction(['decommissionMapreduce'], HADOOP_DRAIN_SUBSTRACT);


    dock.workerpool.getWorker(current.cluster.actualurl).call('cmjob', 'drainNodes', ['', '', nodes, commissionMapreduceOnly, [], [], false],
        handle_drainNodesResponse.bind({'self' : self, 'hdfskey' : hdfskey, 'filter' : 'mapreduce'}), current);
  }
  dock.dialogs.showDialog(d);

}

function decommissionMR() {

  var hdfskey = current.get('uniqueKey');
  var hdfsname = current.get('name');
  
  current.object.nodes = [];
  // printall( current.object );

  var overlays = current.cluster.find('ConfigurationOverlay');

  var xulfilename = dock.myVersion + 'hadoop/hadoopnodeselection.xul#tasktrackersDecomm';
  var args = new Object();
  args['hi'] = current;
  args['mapcoroles'] = overlays[0].buildConfigurationOverlayNodeRoleMapping(hdfskey);
  args['initialize'] = function (refreshFunc) {
    dock.waitIcon();
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'getHadoopDecommissionStatus', [hdfskey, 'mapreduce', HADOOP_DECOMMISSION_STATUS_INCLUDE_JMX], handle_getHadoopDecommissionStatusResponse.bind({'refreshFunc' : refreshFunc, 'dock' : dock}));
  };
  var d = new dock.cvDialog(xulfilename,args);

  var self = this;
  d.onOk = function() {
    var nodes = current.get('nodes');
    if (nodes.length === 0) {
      return;
    }
    dock.waitIcon();
    var decommissionMapreduceOnly = getHadoopDrainAction(['decommissionMapreduce'], HADOOP_DRAIN_MERGE);
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmjob', 'drainNodes', ['', '', nodes, decommissionMapreduceOnly, [], [], false], 
       handle_drainNodesResponse.bind({'self' : self, 'hdfskey' : hdfskey, 'filter' : 'mapreduce'}), current);
  }
  dock.dialogs.showDialog(d);

}

