/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var timer = 0;
var myDialog;
var hadoopRoleMapping;
var hadoopIntentionMapping = {};
var hadoopJmxStatusMapping = {};

var allnodeslabel;
var groupnodeslabel;

var fieldname = 'nodes';
var displayInTree = false;
var subtype = 'Node';
var singular = 'node';
var plural = 'nodes';
var extraName = "''";
var nodeSelectionProperties = [['','', false],
            ['hostname', 'hostname', false],
            ['MAC', 'mac', false],
            ['tag', 'tag', false],
            ['category', 'category', true],
            ['installmode', 'installMode', false],
            ['rack', 'rack', true],
            ['position', 'rackPosition', false]];

var belongs = '0';
var noloop = 'true';
var extraEvalOnAdd = '';
var extraEvalOnRemove = '';

var currentnodes = new Array();
var nodesIndex = '';
var excludeFunction = undefined;

var excludeFunctionNodesUP = function(node) {
  if (node.status.status != 'UP')	return true;
  else	return false;
};

var excludeFunctionDatanodesCommissioning = function(node) {
  var nodeintention = hadoopIntentionMapping[node.get('uniqueKey')] || '';
  if (!( (node.get('uniqueKey') + '/HadoopDataNodeRole') in hadoopRoleMapping)) return 0;
  return (nodeintention === 'DECOMMISSIONING') ? -1 : 1;
};

var excludeFunctionDatanodesDecommissioning = function(node) {
  var nodeintention = hadoopIntentionMapping[node.get('uniqueKey')] || '';
  if (!( (node.get('uniqueKey') + '/HadoopDataNodeRole') in hadoopRoleMapping)) return 0;
  return (nodeintention === 'DECOMMISSIONING') ? 1 : -1;
};

var excludeFunctionNodemanagersCommissioning = function(node) {
  var nodeintention = hadoopIntentionMapping[node.get('uniqueKey')] || '';
  if (!( (node.get('uniqueKey') + '/HadoopYARNClientRole') in hadoopRoleMapping)) return 0;
  return (nodeintention === 'DECOMMISSIONING') ? -1 : 1;
};
var excludeFunctionNodemanagersDecommissioning = function(node) {
  var nodeintention = hadoopIntentionMapping[node.get('uniqueKey')] || '';
  if (!( (node.get('uniqueKey') + '/HadoopYARNClientRole') in hadoopRoleMapping)) return 0;
  return (nodeintention === 'DECOMMISSIONING') ? 1 : -1;
};

var excludeFunctionTasktrackersCommissioning = function(node) {
  var nodeintention = hadoopIntentionMapping[node.get('uniqueKey')] || '';
  if (!( (node.get('uniqueKey') + '/HadoopTaskTrackerRole') in hadoopRoleMapping))	return 0;
  return (nodeintention === 'DECOMMISSIONING') ? -1 : 1;
};
var excludeFunctionTasktrackersDecommissioning = function(node) {
  var nodeintention = hadoopIntentionMapping[node.get('uniqueKey')] || '';
  if (!( (node.get('uniqueKey') + '/HadoopTaskTrackerRole') in hadoopRoleMapping))	return 0;
  return (nodeintention === 'DECOMMISSIONING') ? 1 : -1;
};

var excludeFunctionRegionservers = function(node) {
  if (!( (node.get('uniqueKey') + '/HadoopHBaseClientRole') in hadoopRoleMapping))	return false;
  else	return true;
};


function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  // printall(myDialog.arguments.configoverlay);
  hadoopRoleMapping = myDialog.arguments.mapcoroles;

  if ( location.hash == "#datanodesComm" ) {
    allnodeslabel = "All DataNodes";
    groupnodeslabel = "To be commissioned";
  } else if ( location.hash == "#datanodesDecomm" ) {
    allnodeslabel = "All DataNodes";
    groupnodeslabel = "To be decommissioned";
  } else if ( location.hash == "#nodemanagersComm" ) {
    allnodeslabel = "All NodeManagers";
    groupnodeslabel = "To be commissioned";
  } else if ( location.hash == "#nodemanagersDecomm" ) {
    allnodeslabel = "All NodeManagers";
    groupnodeslabel = "To be decommissioned";
  } else if ( location.hash == "#tasktrackersComm" ) {
    allnodeslabel = "All TaskTrackers";
    groupnodeslabel = "To be commissioned";
  } else if ( location.hash == "#tasktrackersDecomm" ) {
    allnodeslabel = "All TaskTrackers";
    groupnodeslabel = "To be decommissioned";
  } else if ( location.hash == "#regionserversComm" ) {
    allnodeslabel = "All RegionServers";
    groupnodeslabel = "To be commissioned";
  } else if ( location.hash == "#regionserversDecomm" ) {
    allnodeslabel = "All RegionServers";
    groupnodeslabel = "To be decommissioned";
  } else {
    allnodeslabel = "All Nodes";
    groupnodeslabel = "Nodes in Configuration Group";
  }
 
  document.getElementById('allnodeslabel').value = allnodeslabel;
  document.getElementById('groupnodeslabel').value = groupnodeslabel;

  var refreshFunc = function (nodeintentions, JMX_stati) {
      if(!nodeintentions) nodeintentions = {};
      if(!JMX_stati) JMX_stati = {};
      hadoopIntentionMapping = nodeintentions;
      hadoopJmxStatusMapping = JMX_stati;

      if ( location.hash ) {
        refresh(myDialog.arguments.hi);
      } else {
        refresh(myDialog.arguments.configoverlay);
      }
      var throbbers = document.querySelectorAll('hbox.throbber');
      for (var i=0; i<throbbers.length; i++) {
        throbbers[i].style.display = 'none';
      }
  };

  if (myDialog.arguments.initialize) {
    // Call custom initialize function first, and make it continue the refresh..
    myDialog.arguments.initialize(refreshFunc);
  }
  else {
    // We can refresh immediately
    refreshFunc();
  }
}


function getCurrentSelection() {
  return [[current], 0, ""];
}

function stop() {
  println("Stop hadoop node selection");
  if(timer) window.clearTimeout(timer);
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh hadoop node selection window" + ' - ' + document.documentURI);
  nodesIndex = 'Hadoopnodes';

  //deviceselect(excludeFunction);  
  currentnodes = new Array();
  //var userChoices;
  //if (userChoices.dataNodeKeys && (userChoices.dataNodeKeys.length > 0)) {
  //  currentnodes = userChoices.dataNodeKeys;
  //} else {
  //  currentnodes = new Array(); 
  // }

  if ( location.hash == "#datanodesComm" )	excludeFunction = excludeFunctionDatanodesCommissioning;
  else if ( location.hash == "#datanodesDecomm" )	excludeFunction = excludeFunctionDatanodesDecommissioning;
  else if ( location.hash == "#nodemanagersComm" )	excludeFunction = excludeFunctionNodemanagersCommissioning;
  else if ( location.hash == "#nodemanagersDecomm" )	excludeFunction = excludeFunctionNodemanagersDecommissioning;
  else if ( location.hash == "#tasktrackersComm" )	excludeFunction = excludeFunctionTasktrackersCommissioning;
  else if ( location.hash == "#tasktrackersDecomm" )	excludeFunction = excludeFunctionTasktrackersDecommissioning;  
    // uncomment for HBase RegionServer commissioning/decommissioning
    // else if ( location.hash == "#regionserversComm" )	excludeFunction = excludeFunctionRegionservers;
    // else if ( location.hash == "#regionserversDecomm" )	excludeFunction = excludeFunctionRegionservers;  
  //else	excludeFunction = excludeFunctionNodesUP;

  deviceselect(excludeFunction, hadoopJmxStatusMapping);

  // document.getElementById('revert').setAttribute('disabled', current.object.uniqueKey < dock.MAX_LOCAL_KEY);
}

function buttonCancel() {
  close();
}

function buttonOk() {
  // save();
  myDialog.onOk();
  close();
}

