/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var overview;
var timer;
var timer2;

function onLoad() {
  dock = getDock();
  println('.Load : ' + document.title + ' - ' + document.documentURI);
  stop();
}

// Make sure all timers are stopped
function stop() {
  if (timer)
    window.clearTimeout(timer);
  if (timer2)
    window.clearTimeout(timer2);
}

/**
 * Update UI based on BigDataSecurity subobject (HadoopHdfs::security)
 */
function update_ui_based_on_security_properties()
{
  var security = current.get('security');
  if (!security.kerberosEnabled) {
    // Help the user by explaining cmhadoop-security-setup has to be called on the commandline for Kerberos
    document.getElementById('noKerberosWizardYet').removeAttribute('hidden');
    document.getElementById('kerberosSpecificOptions').setAttribute('hidden', true);
  }
  else {
    // Remove instructions and show Kerberos specific options (regenerate keytabs, etc.)
    document.getElementById('noKerberosWizardYet').setAttribute('hidden', 'true');
    document.getElementById('kerberosSpecificOptions').removeAttribute('hidden');
  }

  // If no security at all is enabled do not show buttons and console output
  var noSecurityEnabledAtAll = !security.kerberosEnabled && !security.sslEnabled && !security.wireEncryptionEnabled;
  if (noSecurityEnabledAtAll)
    document.getElementById('kerberosOptions').setAttribute('hidden', true);
  else
    document.getElementById('kerberosOptions').removeAttribute('hidden');

  // Generate the actual command the user can copy & paste for installing Kerberos
  var hdfsname = current.get('name');
  document.getElementById('cmhadoopsecuritysetupcommand').value = 'cmhadoop-security-setup -i ' + hdfsname + ' --kerberos';
  
  // Make sure we periodically update
  timer = setTimeout(update_ui_based_on_security_properties, 1000);
}

/**
 * Update UI with output for any security command running or previous (stopped) command
 */
function check_status_of_security_commands()
{
  if (current) {
    var hdfskey = current.get('uniqueKey');
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'executeSecurityCommand', [hdfskey, "status", []], function (code, data) {
      if (code == 0) {
        if (data.output.length != 0) {
          document.getElementById('securitycommandstatus').value = data.output[0];
          document.getElementById('securitycommandoutput').value = data.output.splice(3).join("\n");
        }
      }
      // Update again after 5 seconds
      timer2 = setTimeout(check_status_of_security_commands, 5000);
    }, current);
  } else {
    // refresh() has not been called yet by the parent window, retry later
    timer2 = setTimeout(check_status_of_security_commands, 1000);
  }
}

/**
 * Request CMDaemon to restart *all* services related to this Hadoop instance (not just HDFS!)
 */
function restartAllServicesForHDFS() {
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  var hdfsname = current.get('name');

  args['question'] = "Restart all HDFS services on " + hdfsname + "?";
  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'restartAllServiceForHDFS', [hdfskey, 0], function(code, data) {
      if(code == 0) {
        if(data.output.length == 0) {
          dock.displayInfo('HDFS restart returned no output and error code: ' + data.code);
        } else {
          dock.displayInfo(data.output);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
  dock.dialogs.showDialog(d);
}

/**
 * Request CMDaemon to execute a security command (i.e., uninstall Kerberos, enableSSL, ...)
 * 
 * @param string command - invalid commands are ignored by CMDaemon
 * @param string questionString - what to prompt the user before executing
 */
function executeSecurityCommand(command, questionString)
{
  var args = new Object();
  var hdfskey = current.get('uniqueKey');
  var hdfsname = current.get('name');

  args['question'] = questionString;
  var d = new dock.cvDialog(dock.myVersion + "question.xul", args);

  d.onOk = function() {
    dock.waitIcon();
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'executeSecurityCommand', [hdfskey, command, []], function(code, data) {
      if(code == 0) {
        if(data.output.length == 0) {
          dock.displayInfo('Security command returned no output and error code: ' + data.code);
        } else {
          // dock.displayInfo(data.output);
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
  dock.dialogs.showDialog(d);
}

function regenerateAllKeytabs()
{
  var hdfsname = current.get('name');
  executeSecurityCommand('regeneratekeytabs', "Regenerate all keytabs for " + hdfsname + "? (This requires a restart of all HDFS services..)");
}
function uninstallKerberos()
{
  var hdfsname = current.get('name');
  executeSecurityCommand('uninstallkerberos', "Remove kerberos for " + hdfsname + "? (This requires a temporary shutdown of all HDFS services..)");
}
function installSSL()
{
  var hdfsname = current.get('name');
  executeSecurityCommand('enablessl', "Enable SSL for " + hdfsname + "? (This requires a restart of all HDFS services..)");
}
function uninstallSSL()
{
  var hdfsname = current.get('name');
  executeSecurityCommand('disablessl', "Disable SSL for " + hdfsname + "? (This requires a restart of all HDFS services..)");
}
function installWireEncryption()
{
  var hdfsname = current.get('name');
  executeSecurityCommand('enablewireencryption', "Enable Wire-encryption for " + hdfsname + "? (This requires a restart of all HDFS services..)");
}
function uninstallWireEncryption()
{
  var hdfsname = current.get('name');
  executeSecurityCommand('disablewireencryption', "Disable Wire-encryption for " + hdfsname + "? (This requires a restart of all HDFS services..)");
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh Security operations" + ' - ' + document.documentURI);
  update_ui_based_on_security_properties();
  check_status_of_security_commands();
}

