/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var timer;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
  stop();
}

function stop()
{
  if (timer) window.clearTimeout(timer);
}

function update_ipc_values_in_ui(metrics, security_open_ipc, security_kerb_ipc)
{
  var allHadoopNodeKeys = getAllHadoopNodeKeys(current);
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmmon', 'sampleOndemandOnDevices', [allHadoopNodeKeys, metrics], function(code, data) {
    if(code == 0) {
      var total = 0;
      var kerberized = 0;
      var nonkerberized = 0;
      for (var i=0; i<data.length; i++) {
        if (data[i].metricId == security_open_ipc) {
          nonkerberized += data[i].rate;
        }
        else if (data[i].metricId == security_kerb_ipc) {
          kerberized += data[i].rate;
        }
        //println(data[i].timeStamp + ' | ' + data[i].metricId + ' | ' + data[i].rate);
      }
      total = nonkerberized + kerberized;
      // Display the values in the UI
      overviewBar('kerberizedPercentage', kerberized, total, ' IPC services', true);
      // request every 10 seconds..
      timer = setTimeout(update_ipc_values_in_ui.bind(this, metrics, security_open_ipc, security_kerb_ipc), 10000);
    } else {
      //dock.jsonCallError(this, code, data);
      // retry after 30 seconds..
      //timer = setTimeout(update_ipc_values_in_ui.bind(this, metrics, security_open_ipc, security_kerb_ipc), 30000);
    }
  }, current);
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh Security Tab" + ' - ' + document.documentURI);

  var hdfsname = current.get('name');
  var security = current.get('security');

  // Update some generic information in the top of the UI
  document.getElementById('kerberosEnabled').value = security.kerberosEnabled ? "Yes" : "No";
  document.getElementById('sslEnabled').value = security.sslEnabled ? "Yes" : "No";
  document.getElementById('wireEncryptionEnabled').value = security.wireEncryptionEnabled ? "Yes" : "No";

  // Get uniqueId's for the two IPC related security metrics
  var security_open_ipc = current.resolveName() + '_security_open_ipc';
  var security_kerb_ipc = current.resolveName() + '_security_kerberized_ipc';
  security_open_ipc = current.cluster.find('Metric', security_open_ipc);
  security_kerb_ipc = current.cluster.find('Metric', security_kerb_ipc);
  if(security_open_ipc || security_kerb_ipc) {
    var mmc = current.cluster.allMetrics();
    security_open_ipc = security_open_ipc.get('uniqueKey');
    security_kerb_ipc = security_kerb_ipc.get('uniqueKey');
    var metrics = new Array();
    for(var i = 0; i < mmc.length; i++) {
      if(mmc[i].metric == security_open_ipc || mmc[i].metric == security_kerb_ipc) {
       // printall(mmc[i]);
        var mpi = new dock.cvMetricPrmId();
        mpi.metricId = parseInt(mmc[i].metric); // unsure
        mpi.metricParam = mmc[i].metricParam;
        metrics.push(mpi);
      }
    }
  }
  
  // Request CMDaemon for the values for these Metrics, update the UI in the callback.
  update_ipc_values_in_ui(metrics, security_open_ipc, security_kerb_ipc);
  
  setTimeout( update, 100 ); // trigger refresh() inside iframe(s)
}


