/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */


// helper functions to transform object and arrays. 
// probably helpful more than for big data, but for while it will be kept here.


function stringify(object, prepend = null, instructions = {"indentation" : 2, "println": false, "printall" : false }) {
  // returns a string out of object. 
  // when prepend is passed, add prepend + "="  to the beginning of the resulting string.
  // uses JSON, so, object must be JSON object (functions, e.g, inside that object, won't get printed into the string).

  var finalString = "";
  if (prepend != null) {
    finalString += prepend + " = ";
  }
  var indentation = instructions.indentation != null ? instructions.indentation : 2;
  finalString += JSON.stringify(object, null, indentation);
  if (instructions.println) {
    println(finalString);
  }
  if (instructions.printall) {
    printall(object);
  }
  return finalString;
}


function bidimensionalObjectToFlatArray(bidimensionalObject) {
  // object { array1: [e1, e2 ] , array2: [e2, e3] , array3 : [e1, e2, e3, e4] } => returns [e1, e2, e3, e4]

  var singleArrayOfNodesObject = new Object();
  var bidimensionalObjectKeys  = Object.keys(bidimensionalObject);
  bidimensionalObjectKeys.forEach(function(key1, index1){
      bidimensionalObject[key1].forEach(function(key2, index2){
          singleArrayOfNodesObject[bidimensionalObject[key1][index2]] = true; // just any random value, will be discarded
        });
  });
  var singleArrayOfNodeKeys    = Object.keys(singleArrayOfNodesObject);
  // println("singleArrayOfNodeKeys=" + JSON.stringify(singleArrayOfNodeKeys, null, 2));
  return singleArrayOfNodeKeys;
}


function arrayOfObjectsToFlatObject(arrayOfObjects) {
  // example:
  // var arrayOfObjects = [ { a: v1 , b : v2 }, { c : v3 }, { c : v4} ]
  // will return { a: v1,  b: v2, c : v4 } // note that duplicates will be overwritten.

  var flatObject = new Object();
  Object.keys(arrayOfObjects).forEach( function (key1, index1) {
      Object.keys(arrayOfObjects[index1]).forEach ( function (key2, index2) {
          flatObject[key2] = arrayOfObjects[index1][key2];
        });
    });
  return flatObject;
}


function hasPrimitiveType(object) {
  // returns true if type of object is primitive (bool, string, number, undefined, null)

  return typeof object != "object";
}


function objectGetClassName(object) {
  // returns the class name of the object.

  var classNameWrapper = {}.toString;
  var classOfObject = classNameWrapper.call(object);
  classOfObject = classOfObject.replace(/.* /g, '').replace(/]$/g, '');
  //println("classOfObject = " + classOfObject);
  return classOfObject;
}


function objectCloneRespectingItsReferences(object) {
  // clones object, but not deeply (both object and the returned object will share internal references)
  // the advantage of this function is that the type of object does not have to be provided.

  var clone;
  var cloneCode = "new " + objectGetClassName(object) + "(object);";
  clone = eval(cloneCode);
  return clone;
}


function objectDeepCopy(originalObject) {
  // a = objectDeepCopy(b)
  // makes a to get a copy of b, instead of referencing the same object.
  // all references are recursively copied
  // TODO: this function hasn't been tested with objects having functions, and more complex things

  var objectDeepCopied;
  if (hasPrimitiveType(originalObject)) {
    //println(stringify(originalObject, "primitive object " + originalObject));
    objectDeepCopied = originalObject;
  } else {
    objectDeepCopied = objectCloneRespectingItsReferences(originalObject);
    // now, having cloned the object, go deep changing references by their values:
    var originalObjectKeys = Object.keys(originalObject);
    originalObjectKeys.forEach( function (key, index){
        objectDeepCopied[key] = objectDeepCopy(originalObject[key]);
        //println(stringify(originalObject[key], "originalObject["   + key + "]"));
        //println(stringify(objectDeepCopied[key], "objectDeepCopied[" + key + "]"));
      });
    }
  return objectDeepCopied;
}


function objectTwoValuesToKeyValueArray(srcObject, keyA, keyB, recursive = false) {
  // example:
  // var value3 = { keyA : "value3A", keyB : "value3B" , keyD : "value4" };
  // var srcObject = { keyA : "valueA", keyB : "valueB", key3 : value3 };
  // calling objectTwoValuesToKeyValueArray(srcObject, "keyA", "keyB")
  // will return =   [{ valueA : valueB }]
  // if recursive is true, it will deeply search for sub-objects having keyA, keyB. so it will return:
  // will make this return =  [{ valueA : valueB }, {value3A: value3B} ]

  // return value is an Array
  var keyValueArray = new Array();
  if (!hasPrimitiveType(srcObject)) {
    // generate the next object to be pushed back into keyValueArray
    var firstObjectOfKeyValueArray = new  Object();
    if ( (srcObject[keyA] != null) && srcObject[keyB] != null ) {
      firstObjectOfKeyValueArray[srcObject[keyA]] = srcObject[keyB];
    }
    keyValueArray.push(firstObjectOfKeyValueArray);
    // if recursive, trawl for the same pattern inside the objects
    if (recursive) {
      var srcObjectKeys = Object.keys(srcObject);
      srcObjectKeys.forEach( function (key, index) {
          println(stringify(srcObject[key], key));
          keyValueArray = keyValueArray.concat(objectTwoValuesToKeyValueArray(srcObject[key], keyA, keyB, true));
        });
    }
  }
  return keyValueArray;
}


function objectKeys(object) {
  // just like Object.keys(object), but, succeeds also for primitive types (string, bool, number) --  it returns empty vector.
  // simple, but saves a lot of ifs in the code.
  // user wisely: better don't use this function where objects can'thave primitive types, so interpreter 
  // can help you find mistakes

  var keys = new Array();
  if (!hasPrimitiveType(object)) {
    keys = Object.keys(object);
  }
  return keys;

}


function valuesOfTheKey(object, key, recursive = false) {
  // returns the value of the key (ie, object[key]), within an array, when recursive is false.
  // if recursive is true, it will search deeply for the key inside the object (values will be pushed back)

  var values = new Array();
  if (object[key] != null) {
    values.push(object[key]);
  }
  if (recursive) {
    objectKeys(object).forEach( function(currentKey, currentIndex) {
      values = values.concat(valuesOfTheKey(object[currentKey], key, recursive));
    });
  }
  return values;
}


function arrayRemoveObjectsHavingKey(array, key, recursive = false) {
  // examples:
  //
  // arrayRemoveObjectsHavingKey([ {a:b},  {c:d},  {e:a, a:c},  { f: {a:b} },  { g:{ h:{a:b} } } ], "a" );
  // returns [ {c:d},  {e:a, a:c},  { f:{a:b} },  { g:{ h:{a:b} } } ]  // note removed {a:b}
  //
  // arrayRemoveObjectsHavingKey([ {a : b},  {c: d},  {e: a, a: c},  { f: {a : b} } ], "a" , true);
  // returns  [ {c:d}  ] // note removed also {e:a, a:c},  { f:{a:b} },  { g:{ h:{a:b} } }, beyond {a:b}

  var objectsHavingKey = new Array();
  objectKeys(array).forEach( function (currentKey, currentIndex) {
      if (valuesOfTheKey(array[currentIndex], key, recursive).length == 0) {
        objectsHavingKey.push(array[currentIndex]);
      }
  });
  return objectsHavingKey;
}


function hasKeyValue(object, key, value, recursive = false) {
  // returns true if object[key] = value, when recursive is false.
  // if recursive is true, it will search deeply for the key-value inside the object (values will be pushed back)

  var keyValueFound = (object[key] != null) && (object[key] == value);
  //println(stringify(object[key], "object[" + key + "] -- is value=[" + value + "] already found? " + keyValueFound));
  if (recursive && !keyValueFound) {
    keyValueFound = objectKeys(object).some( function (currentKey, currentIndex) {
        return hasKeyValue(object[currentKey], key, value, true);
    });
  }
  //println(object[key] + " -- was it found? => " + keyValueFound);
  return keyValueFound;
}


function arrayRemoveObjectsHavingKeyValue(array, key, value, recursive = false) {
  // examples:
  //
  // arrayRemoveObjectsHavingKey([ {a:b},  {c:d},  {e:a, a:c},  { f: {a:b} },  { g:{ h:{a:b} } } ], "a", "b" );
  // returns [ {c:d},  {e:a, a:c},  { f:{a:b} },  { g:{ h:{a:b} } } ]  // note removed {a:b}
  //
  // arrayRemoveObjectsHavingKey([ {a : b},  {c: d},  {e: a, a: c},  { f: {a : b} } ], "a", "b", true);
  // returns  [ {c:d}, {e:a, a:c}   ] // note removed also { f:{a:b} },  { g:{ h:{a:b} } }, beyond {a:b}

  var objectsHavingKeyValue = new Array();
  objectKeys(array).forEach( function (currentKey, currentIndex) {
      //println(stringify(array[currentIndex], "currentIndex=" + currentIndex));
      if (!hasKeyValue(array[currentIndex], key, value, recursive)) {
        //println(stringify(array[currentIndex], "array[" + currentIndex + "] DOES NOT have key=[" + key + "] with value=[" + value + "]"));
        objectsHavingKeyValue.push(array[currentIndex]);
      }
  });
  return objectsHavingKeyValue;
}

