/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var DEBUG = false;

var dock;
var myDialog;
var current;
var userChoices;
var removalConfirmed = false;
var allHadoops = new Array();
var allOverlays = new Array();
var instanceNodes = {};

var timer = 0;
var count = 0;

// stuff for deck management START
var DECK_MAIN = 'main';
var DECK_REMOVE = 'remove';
var currentDeck = '';
// stuff for deck management END

// START for progress bar and steps
var PATTERN_FIRST_LINE = /^##### cmgui #####$/g;
var PATTERN_GENERIC = /^### ([^#]+) ### (STARTED|FINISHED|FAILED)$/g;
var PATTERN_PERC = /^### (\d{1,3})$/g;
var stepsList;
var progressPerc = 0;
// END for progress bar and steps

function onLoad() {
  dock = getDock();
  userChoices = {};

  println("Remove Big Data Instance Wizard initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  current = myDialog.arguments.cluster;

  allHadoops = current.find('HadoopHDFS');
  allOverlays = current.find('ConfigurationOverlay');
  for (var i = 0; i < allOverlays.length; i++) {
    for (var j = 0; j < allHadoops.length; j++) {
      var hdfsKey = allHadoops[j].get('uniqueKey');
      var hdfsName = allHadoops[j].resolveName();
      var roles = allOverlays[i].getHadoopRoles(hdfsKey);
      if (roles.length > 0) {
        var nodes = allOverlays[i].get('nodes');
        if (hdfsName in instanceNodes) {
          instanceNodes[hdfsName] = instanceNodes[hdfsName].concat(nodes);
          instanceNodes[hdfsName] = sort_unique(instanceNodes[hdfsName]);
        } else {
          instanceNodes[hdfsName] = nodes;
        }
      }
    }
  }

  mainScreen();
}

function next() {
  println('next from ' + currentDeck);
  var pages = document.getElementById('deck').childNodes.length - 1;
  var index = parseInt(document.getElementById('deck').selectedIndex);

  if (currentDeck == DECK_MAIN) {
    if (!checkMain()) return;
    if (!askConfirmation(userChoices.instance) && !removalConfirmed) return;
    currentDeck = DECK_REMOVE;
    remove();
  }
  index = getNumericIndexFromDeckString(currentDeck);

  println('going to ' + currentDeck + ' (' + index + ')');

  document.getElementById('deck').selectedIndex = index;
  document.getElementById('prev').setAttribute('hidden', index == 0);
  document.getElementById('prev').setAttribute('disabled', index == 0);
  document.getElementById('next').setAttribute('hidden', index == pages);
  document.getElementById('finish').setAttribute('hidden', index < pages);
  document.getElementById('finish').setAttribute('disabled', true);
}

function prev() {
  println('prev from ' + currentDeck);
  var pages = document.getElementById('deck').childNodes.length - 1;
  var index = parseInt(document.getElementById('deck').selectedIndex);

  if (currentDeck == DECK_REMOVE) {
    currentDeck = DECK_MAIN;
    mainScreen();
  }

  index = getNumericIndexFromDeckString(currentDeck);

  println('going to ' + currentDeck + ' (' + index + ')');

  document.getElementById('deck').selectedIndex = index;
  document.getElementById('canc').setAttribute('disabled', index == pages);
  document.getElementById('prev').setAttribute('disabled', index == 0);
  document.getElementById('prev').setAttribute('hidden', index == 0);
  document.getElementById('next').setAttribute('hidden', index == pages);
  document.getElementById('next').removeAttribute('disabled');
  document.getElementById('finish').setAttribute('hidden', index < pages);
  document.getElementById('finish').setAttribute('disabled', true);
}

function getNumericIndexFromDeckString(deckString) {
  // get index for nextDeck
  var deck = document.getElementById('deck');
  for (var i = 0; i < deck.childNodes.length; i++) {
    var hadoopindex = deck.childNodes[i].getAttribute('hadoopindex');
    if (deckString == hadoopindex) {
      return i;
    }
  }
}

function cancel() {
  var args = new Object();
  args['question'] = 'Are you sure you want to quit the wizard?';
  var d = new cvDialog(dock.myVersion + "question.xul", args);
  d.onOk = function() {
    close();
  }
  dock.dialogs.showDialog(d);
}

function askConfirmation(instance) {
  if (removalConfirmed) {
    return true;
  } else {
    var args = new Object();
    args['question'] = "Are you sure you want to remove the instance '" + instance + "'?";
    var d = new cvDialog(dock.myVersion + "question.xul", args);
    d.onOk = function() {
      removalConfirmed = true;
      next();
    }
    dock.dialogs.showDialog(d);
  }
  return false;
}

function finish() {
  close();
}

// START deck MAIN
function mainScreen() {
  currentDeck = DECK_MAIN;

  document.getElementById('dialogtitle').setAttribute('value', 'Big Data instances');
  var text = "Please choose the Big Data instance to remove. All related Big Data services will be stopped. For Hadoop instances, all the data contained in HDFS will be lost.";
  document.getElementById('mainMessage').setAttribute('value', text);

  var instanceSelect = document.getElementById('instance');
  var popup = instanceSelect.firstChild;
  if (!popup) {
    println("!!! ERROR !!!: has no menupopup");
    return;
  }
  while (popup.firstChild) popup.removeChild(popup.firstChild);

  var item = document.createElement('menuitem');
  item.setAttribute('key', '');
  item.setAttribute('label', '');
  popup.appendChild(item);
  for (var i = 0; i < allHadoops.length; i++) {
    var name = allHadoops[i].resolveName();
    var text = name + " (" + allHadoops[i].get('version') + " " + allHadoops[i].get('distribution') + ")";
    var item = document.createElement('menuitem');
    item.setAttribute('key', name);
    item.setAttribute('label', text);
    if (allHadoops[i].get('isSparkInstance')) {
      item.setAttribute('type', 'spark');
    } else {
      item.setAttribute('type', 'hadoop');
    }
    popup.appendChild(item);
  }
  popup.addEventListener('command', instance_command, true);
}

function instance_command(event) {
  var item = event.target;
  var selectedItem = item.parentNode.parentNode.selectedItem;
  showNodes(selectedItem.getAttribute('key'));
}

function showNodes(instance) {
  var text = "";
  document.getElementById('next').setAttribute('disabled', true);
  if (instance != '') {
    var nodeKeys = instanceNodes[instance];
    var nodeList = new Array();
    var notUp = new Array();
    if (nodeKeys) {
      for (var i = 0; i < nodeKeys.length; i++) {
        var node = current.find(nodeKeys[i]);
        nodeList.push(node.resolveName());
        if (node.status.status != 'UP') {
          notUp.push(node.resolveName());
        }
      }
    }
    var str = dotFormat(nodeList.sort()) || '(none applicable).';
    var text = "The selected Big Data instance is deployed on the following nodes:\n\n" + str;
    if (notUp.length > 0) {
      text += "\n\nPlease make sure that the following nodes are in status 'UP' before proceeding with the removal:\n" + dotFormat(notUp.sort());
    } else {
      document.getElementById('next').removeAttribute('disabled');
    }
  }
  document.getElementById('nodesMessage').setAttribute('value', text);
}

function checkMain() {
  if (document.getElementById('instance').label.length == 0) {
    dock.displayError('A Big Data instance should be selected');
    return false;
  }
  userChoices.instance = document.getElementById('instance').selectedItem.getAttribute('key');
  userChoices.isSparkInstance = (document.getElementById('instance').selectedItem.getAttribute('type') == 'spark');
  
  return true;
}
// END deck MAIN

// START deck REMOVE
function remove() {
  var xmlConfig = "";
  try {
    xmlConfig = createXMLconfigFile();
  } catch (err) {
    println('exception ' + err + ' while generation xml file');
    dock.displayError('Error while generating XML file');
    return false;
  }

  document.getElementById('dialogtitle').setAttribute('value', 'Removal progress');
  var text = "\nRemoval progress for Big Data instance '" + userChoices.instance + "' is shown below.";
  document.getElementById('removeMessage').setAttribute('value', text);

  if (xmlConfig != "") {
    var id = 0;
    overviewBar('progressBar', 0, 100);
    stepsList = new Array();

    var remoteCallType = 0;
    if (userChoices.isSparkInstance) {
      remoteCallType = 1;
    }
    
    dock.workerpool.getWorker(current.actualurl).call('cmhadoop', 'executeHadoopSetup', [remoteCallType, xmlConfig], function(code, data) {
      if(code == 0) {
        id = data.setupExecutionID;
        timer = window.setTimeout('checkStatus(' + id +');', 1000);
      } else {
        document.getElementById('canc').setAttribute('disabled', false);
        document.getElementById('prev').setAttribute('disabled', false);
        dock.jsonCallError(this, code, data);
      }
    }, current);
  } else {
    println('not running');
  }
}

function checkStatus(id) {
  timer = 0;
  count++;
  if (count > 2000) {
    return;
  }

  dock.workerpool.getWorker(current.actualurl).call('cmhadoop', 'getHadoopSetupExecution', [id], function(code, data) {
    if(code == 0) {
      if (!data) {
        dock.displayError('A Big Data instance is already being removed.');
        return;
      }
      var currentProgress = "";
      var stepsCounter = 0;
      for (var i = 0; i < data.datacout.length; i++) {
        var matched = false;

        if (data.datacout[i].match(PATTERN_FIRST_LINE)) {
          document.getElementById('removeProgress').removeAttribute('hidden');
          matched = true;
        }
        var m;
        do {
          m = PATTERN_GENERIC.exec(data.datacout[i]);
          if (m) {
            matched = true;
            //println('matching m = ' + m);
            stepsList[stepsCounter++] = m;
          }
        } while (m);
        var m2 = PATTERN_PERC.exec(data.datacout[i]);
        if (m2) {
          matched = true;
          //println('matching m2 = ' + m2);
          progressPerc = m2[1];
        }

        if (!matched) {
          currentProgress += data.datacout[i];
          currentProgress += "\n";
        }
      }
      var removeSteps = document.getElementById('removeSteps');
      while (removeSteps.childNodes.length > 2) {
        removeSteps.removeChild(removeSteps.lastChild);
      }

      // merge stepsList
      var mergedStepsList = new Array();
      for (var i = 0; i < stepsList.length; i++) {
        var item = stepsList[i];
        if (stepsList[i-1]) {
          var previousItem = stepsList[i-1];
          if (item[1] == previousItem[1]) {
            mergedStepsList[mergedStepsList.length-1] = item;
          } else {
            mergedStepsList.push(item);
          }
        } else {
          mergedStepsList.push(item);
        }
      }
      for (var i = 0; i < mergedStepsList.length; i++) {
        var row = document.createElement('row');
        row.setAttribute('align', 'center');
        var image = document.createElement('image');
        if (mergedStepsList[i][2] == 'STARTED') {
          image.setAttribute('src', 'chrome://cmgui/content/icons/throbber.gif');
        } else if (mergedStepsList[i][2] == 'FINISHED') {
          image.setAttribute('src', 'chrome://cmgui/content/icons/ok.png');
          //addLatestThrobber = true;
        } else if (mergedStepsList[i][2] == 'FAILED') {
          image.setAttribute('src', 'chrome://cmgui/content/icons/alert.png');
        }
        row.appendChild(image);
        var stepDesc = document.createElement('shadowlabel');
        stepDesc.setAttribute('value', mergedStepsList[i][1]);
        row.appendChild(stepDesc);
        removeSteps.appendChild(row);
      }
      overviewBar('progressBar', progressPerc, 100);

      var removeOutput = document.getElementById('removeOutput');
      removeOutput.value = currentProgress;
      var ti = document.getAnonymousNodes(removeOutput)[0].childNodes[0];
      ti.scrollTop = ti.scrollHeight;

      if (data.finishedAt == 0) {
        document.getElementById('canc').setAttribute('disabled', true);
        document.getElementById('prev').setAttribute('disabled', true);
        timer = window.setTimeout('checkStatus(' + id +');', 3000);
      } else if (data.successful) {
        overviewBar('progressBar', 100, 100);
        document.getElementById('finish').setAttribute('disabled', false);
      } else { // failed
        document.getElementById('canc').setAttribute('disabled', false);
        document.getElementById('prev').setAttribute('disabled', false);
      }
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);
}
// END deck REMOVE

// START xml stuff
function createXMLconfigFile() {
  var xmlConfig = undefined;

  var prefix = userChoices.isSparkInstance ? "spark" : "hadoop";
  var doc = document.implementation.createDocument("", prefix + "Remove", null);

  addTextNode(doc, doc.documentElement, "fromcmgui", true);

  // instance
  var instanceElem = doc.createElement("instance");
  doc.documentElement.appendChild(instanceElem);

  // name
  addTextNode(doc, instanceElem, "name", userChoices.instance);

  xmlConfig = new XMLSerializer().serializeToString(doc);
  return xmlConfig;
}

function addTextNode(doc, node, name, value) {
  var elem = doc.createElement(name);
  var text = doc.createTextNode(value);
  elem.appendChild(text);
  node.appendChild(elem);
}
// END xml stuff
