/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function HadoopWizardExtenderWithActualTarballInformation(returnValues)
{
  this.tarballs = [];
  this.tarballsPerPopup = {};
  this.active = false;

  this.isActive = function () {
    return this.active;
  };

  this.getDropDownValue = function (popupId, index) {
    if (index === -1) {
      return false;
    }
    if (!(popupId in this.tarballsPerPopup)) {
      return false;
    }
    if (index >= this.tarballsPerPopup[popupId].length) {
      return false;
    }
    return this.tarballsPerPopup[popupId][index];
  };

  this.preparePopup = function (popupId, startsWithFilter, popup_command_fun) {
    // Get all tarballs, put them in the Tarballs on server dropdown, and store them
    var tarballsAvail = document.getElementById(popupId + 'Avail');
    var popup = tarballsAvail.firstChild;
    if (!popup) {
      println("!!! ERROR !!!: has no menupopup");
      return;
    }
    while (popup.firstChild) popup.removeChild(popup.firstChild);

    this.tarballsPerPopup[popupId] = [];
    for (var i=0; i<this.tarballs.length; i++) {
      if (this.tarballs[i].indexOf(startsWithFilter) !== 0)
        continue;
      var item = document.createElement('menuitem');
      var name = this.tarballs[i];
      item.setAttribute('key', name);
      item.setAttribute('label', name);
      popup.appendChild(item);
      this.tarballsPerPopup[popupId].push(this.tarballs[i]);
    }
    popup.addEventListener('command', popup_command_fun, true);
  };

  this.getReturnValues = function (isCompleted, returnValues, error) {
    if (!isCompleted)
      return;
    if (error) {
      return;
    }

    this.tarballs = returnValues;

    function avail_tarballs_command(event) {
      var item = event.target;
      var selectedIndex = item.parentNode.parentNode.selectedIndex;
      document.getElementById(this.textboxElem).value = CM_LOCAL_PREFIX + tarballWizardExtender.tarballsPerPopup[this.popupId][selectedIndex];
      showSelectedTarballs();
    }
    function avail_hadoop_tarballs_command(event) {
      return avail_tarballs_command.bind({'textboxElem' : 'hadoopTarballCorrected', 'popupId' : 'hadoopTarball'})(event);
    }
    function avail_zk_tarballs_command(event) {
      return avail_tarballs_command.bind({'textboxElem' : 'zooKeeperTarballCorrected', 'popupId' : 'zooKeeperTarball'})(event);
    }
    function avail_hbase_tarballs_command(event) {
      return avail_tarballs_command.bind({'textboxElem' : 'hbaseTarballCorrected', 'popupId' : 'hbaseTarball'})(event);
    }

    this.preparePopup('hadoopTarball', 'hadoop', avail_hadoop_tarballs_command);
    this.preparePopup('zooKeeperTarball', 'zookeeper', avail_zk_tarballs_command);
    this.preparePopup('hbaseTarball', 'hbase', avail_hbase_tarballs_command);

    // Re consider the correct tarballs by invoking setDistro (which has more information available now)
    var selectedIndex = document.getElementById('distro').selectedIndex;
    if (selectedIndex != -1) {
      setDistro(selectedIndex);
    }

    // Now it is safe to manipulate the UI for the extra features that just became available.

    document.getElementById('hadoopTarballAvail').parentNode.hidden = '';
    document.getElementById('specifyCustomTarball').parentNode.hidden = '';

    document.getElementById('zooKeeperTarball').parentNode.className = '';
    document.getElementById('zooKeeperTarballAvail').parentNode.className = 'zooKeeperTarballBox';
    document.getElementById('specifyCustomTarballZk').parentNode.className = 'zooKeeperTarballBox';
    document.getElementById('zooKeeperTarballNotAvailableWarningText').parentNode.className = 'zooKeeperTarballNotAvailableWarning zooKeeperTarballBox';

    document.getElementById('hbaseTarball').parentNode.className = '';
    document.getElementById('hbaseTarballAvail').parentNode.className = 'hbaseTarballBox';
    document.getElementById('specifyCustomTarballHBase').parentNode.className = 'hbaseTarballBox';
    document.getElementById('hbaseTarballNotAvailableWarningText').parentNode.className = 'hbaseTarballNotAvailableWarning hbaseTarballBox';


    function disableTarballElements(elementId) {
      document.getElementById(elementId).disabled = 'true';
      document.getElementById(elementId).parentNode.hidden = 'true';
      document.getElementById(elementId).parentNode.getElementsByTagName('shadowlabel')[0].value = '';
    }

    if (!document.getElementById('specifyCustomTarball').checked)
      disableTarballElements('hadoopTarball');
    if (!document.getElementById('specifyCustomTarballZk').checked)
      disableTarballElements('zooKeeperTarball');
    if (!document.getElementById('specifyCustomTarballHBase').checked)
      disableTarballElements('hbaseTarball');

    this.active = true;
  };

  this.showTarballNotAvailableWarning = function (popupId, flag, text, img_src) {
    var elems = document.getElementsByClassName(popupId + 'NotAvailableWarning');
    var textelem = document.getElementById(popupId + 'NotAvailableWarningText');
    img_src = img_src || 'chrome://cmgui/content/icons/warning.png';
    flag = flag || false;
    textelem.innerHTML = text;
    for (var i=0; i<elems.length; i++) {
      elems[i].style.display = flag ? '' : 'none';
    }
    textelem.parentNode.getElementsByTagName('image')[0].src = img_src;
  }

  this.showTarballNotAvailableInfo = function (popupId, flag, tarball) {
    return this.showTarballNotAvailableWarning(popupId, flag, tarball, 'chrome://cmgui/content/icons/info.png');
  }

  this.clear = function() {
    var popups = ['hadoopTarball', 'zooKeeperTarball', 'hbaseTarball'];
    for (var i=0; i<popups.length; i++) {
      var popupId = popups[i];
      var o = document.getElementById(popupId);
      if (o) {
        o.selectedIndex = 0;
        this.showTarballNotAvailableWarning(popupId, false);
      }
    }
  };

  this.resolve = function (popupId, tarball) {
    if (!(popupId in this.tarballsPerPopup))
      return false;

    var o = document.getElementById(popupId);
    if (o) o.selectedIndex = 0;

    var warning = function (tarball) { return 'The tarball ' + tarball + ' is not available on the server.'; };

    this.showTarballNotAvailableWarning(popupId, false);

    var index = this.tarballsPerPopup[popupId].indexOf(tarball);
    if (index === -1) {
      var result = false,
        result_index = -1,
        best_match = 0,
        regex = tarball;

      // create a regex that can match tarballs without looking at exact numbers used in versioning
      regex = regex.replace(/\./g, '\\.');
      regex = regex.replace(/[0-9]/g, '[0-9]*');

      for (var i=this.tarballsPerPopup[popupId].length - 1; i>=0; i--) {
        // this tarball is not even close..
        if (!this.tarballsPerPopup[popupId][i].match(regex)) {
          continue;
        }

        // determine after how many characters the tarballs are no longer similar, the more literal characters
        // match, the better. (i.e., in the case of "hadoop-2.4.0.tar.gz", an alternative "hadoop-2.4.1.tar.gz"
        // is a better match than "hadoop-2.5.1.tar.gz", because more characters from the start are exactly the same.
        var a = this.tarballsPerPopup[popupId][i].split(''); // before checking make arrays with characters out of the strings
        var b = tarball.split('');
        for (var similar=0, j=0; j<Math.max(a.length, b.length); j++) {
          if (a.length < j || b.length < j) {
            break;
          }
          // do not suggest hadoop 2.7 when we're interested in 2.6!
          if (a[j] > b[j]) {
            similar = 0;
            break;
          }
          // we've encountered a character that is not similar, stop checking
          if (a[j] !== b[j]) {
            break;
          }
          similar++;
        }

        // cdh tarballs contain important version numbers at the end of the tarball.
        // in the form of hadoop-2.6.x-cdh5.5.x.tar.gz, we need to be extra sure that we're not suggesting
        // a hadoop 2.6 for an older cdh!
        var cdh_version = tarball.match(/cdh.\../); // we only check "cdh5.5", and assume the 5.5.* to be OK as an alternative
        if (cdh_version !== null) {
          if (this.tarballsPerPopup[popupId][i].indexOf(cdh_version) === -1) {
            // the match looked good, but is for an older cdh version.
            similar = 0;
          }
        }

        var get_hdp_version = function (tarball) {
          // hortonworks tarballs contain important version numbers in the tarball names as well.
          var hw_version = tarball.match(/[a-zA-Z]*-([0-9]*\.){3}(([0-9]*\.){2})/);
          if (hw_version !== null && hw_version.length === 4) {
            hw_version = hw_version[2]; // i.e., (HDP) "2.4."
            return hw_version;
          }
          return false;
        }

        // hortonworks tarballs contain important version numbers in the tarball names as well.
        var hdp1 = get_hdp_version(tarball);
        var hdp2 = get_hdp_version(this.tarballsPerPopup[popupId][i]);
        if (hdp1 !== false && hdp1 != hdp2) {
          similar = 0;
        }

        // a match we're not interested in
        if (similar == 0) continue;


        // we found a better match
        if (similar > best_match) {
          best_match = similar;
          result = this.tarballsPerPopup[popupId][i];
          result_index = i;
        }
        // ambigious, we cannot decide
        else if (similar == best_match) {
          this.showTarballNotAvailableWarning(popupId, true, warning(tarball));
          return [-1, tarball];
        }
      }
      if (best_match > 0) {
        this.showTarballNotAvailableWarning(popupId, true, warning(tarball));
        return [result_index, result];
      }
      this.showTarballNotAvailableWarning(popupId, true, warning(tarball));
    }
    return [index, tarball];
  }
}

// some event handlers to be used in xul

function onChangeAvailableTarballsOnlyCheckbox(elem, userChoices)
{
  var tarballsAvailElem = document.getElementById('hadoopTarballAvail');
  tarballsAvailElem.parentNode.style.display = elem.checked ? 'none' : '';
  var hadoopTarballElem = document.getElementById('hadoopTarball');
  hadoopTarballElem.disabled = elem.checked ? '' : 'true';
  hadoopTarballElem.parentNode.hidden = elem.checked ? '' : 'true'
  userChoices.useCustomHadoopTarball = elem.checked;
  showSelectedTarballs();
}

function onChangeAvailableZooKeeperTarballsOnlyCheckbox(elem, userChoices)
{
  var tarballsAvailElem = document.getElementById('zooKeeperTarballAvail');
  tarballsAvailElem.parentNode.style.display = elem.checked ? 'none' : '';
  var zooKeeperTarballElem = document.getElementById('zooKeeperTarball');
  zooKeeperTarballElem.disabled = elem.checked ? '' : 'true';
  zooKeeperTarballElem.parentNode.hidden = elem.checked ? '' : 'true';
  // set appropriate class, so the Y/N toggle will toggle this appropriately
  zooKeeperTarballElem.parentNode.className = elem.checked ? 'zooKeeperTarballBox' : '';
  userChoices.useCustomZooKeeperTarball = elem.checked;
  showSelectedTarballs();
}

function onChangeAvailableHBaseTarballsOnlyCheckbox(elem, userChoices)
{
  var tarballsAvailElem = document.getElementById('hbaseTarballAvail');
  tarballsAvailElem.parentNode.style.display = elem.checked ? 'none' : '';
  var hbaseTarballElem = document.getElementById('hbaseTarball');
  hbaseTarballElem.disabled = elem.checked ? '' : 'true';
  hbaseTarballElem.parentNode.hidden = elem.checked ? '' : 'true'
  // set appropriate class, so the Y/N toggle will toggle this appropriately
  hbaseTarballElem.parentNode.className = elem.checked ? 'hbaseTarballBox' : '';
  userChoices.useCustomHBaseTarball = elem.checked;
  showSelectedTarballs();
}
