/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'MasterNode';
var singular = 'head node';
var plural = 'head nodes';

function createTreeItem(one) {
  var ip = '0.0.0.0';
  var ips = one.getIPs();
  if(ips.length >= 1) ip = ips[0][1];

  var item = new Object();
  item['key'] = one.get('uniqueKey');
  item['modified'] = one.get('modified');
  item['hostname'] = one.get('hostname');
  item['mac'] = one.get('mac');
  item['tag'] = one.get('tag');
  item['ip'] = ip;
  var sw = one.get('ethernetSwitch');
  if(sw && sw.ethernetSwitch) {
    item['switchPort'] = one.cluster.resolveName(sw.ethernetSwitch) + ':' + sw.prt;
  } else {
    item['switchPort'] = '';
  }
  if(one.get('powerControl').toLowerCase() == 'apc') {
    item['power'] = '';
    var pdu = one.get('powerDistributionUnits');
    for(var i = 0; i < pdu.length; i++) {
      if(item['power'].length > 0) item['power'] += ', ';
      item['power'] += one.cluster.resolveName(pdu[i].pdu) + ':' + pdu[i].prt;
    }
  } else {
    item['power'] = one.get('powerControl');
  }
  return item;
}

function addObject() {
  var args = new Object();
  args['title'] = 'Add new ' + singular;
  args['question'] = 'Hostname';
  args['default'] = 'headnode';
  args['known'] = current.known('Device', 'hostname');
  var d = new dock.cvDialog("newobject.xul", args);
  d.onOk = function(name) {
    var object = new dock.cvMasterNode();
    object.hostname = name;
    var base = current.getBasePartition();
    if(isObject(base)) {
      object.partition = base.object.uniqueKey;
    }
    current.addObject(object, dock.openAfterCreate);
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function cloneObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a " + singular + " for cloning");
  } else if(select.length > 1) {
    dock.displayError("Please select only one " + singular);
  } else {
    var original = current.find(select[0]);
    if(isObject(original)) {
      var args = new Object();
      var host = original.get('hostname');
      args['title'] = 'Clone ' + host;
      args['question'] = 'Hostname';
      var nn = nameNumber(host);
      if(nn[1].length == 0) nn[1] = '0';
      var name;
      var index = myParseInt(nn[1]);
      while(true) {
        index++;
        name = nn[0] + sprintf('%0' + nn[1].length + 'd', index);
        if(!current.find('Device', name)) break;
      }
      args['default'] = name;
      args['known'] = current.known('Device', 'hostname');
      var d = new dock.cvDialog("newobject.xul", args);
      d.onOk = function(name) {
        var object = original.clone();
        object.hostname = name;
        object.mac = '00:00:00:00:00:00';
        object.tag = '00000000a000';
        object.ethernetSwitch = new dock.cvSwitchPort();
        object.powerDistributionUnits = new Array();
        cloneNodeIP(original.object, object);
        current.addObject(object, dock.openAfterCreate);
        refresh();
      }
      dock.dialogs.showDialog(d);
    } else {
      dock.displayError("Original not found, already removed by someone else");
    }
  }
}
