/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var MIN_HEIGHT = 140;

function cvHealth(cluster, svg, box, devices, startlabel, endlabel, info) {
  this.cluster = cluster;
  this.endTime = 0;
  this.startTime = 0;
  this.svg = svg;
  this.box = box;
  this.drawbox = 0;
  this.devices = devices;
  this.alldevices = devices;
  this.data = 0;

  this.WM = 14;
  this.HM = 36;

  this.refreshRate = dock.generalSettings.monitoringRefresh;
  this.timer = 0;
  this.last = 0;
  this.updating = false;

  this.alertLevel = 0;
  this.alertParam = 'max';
  this.period = 0;
  this.sort = 1;
  this.startlabel = startlabel;
  this.endlabel = endlabel;
  this.info = info

  this.showhealthRows = 0;
  this.processData = 0;
  this.processTimer = 0;
  this.processIndex = 0;
  this.processIndexJ = 0;
  this.processStop = false;
  
  this.showhealthIndex = 0;
  this.showhealthTimer = 0;
  
  this.perPage = 25;
  this.page = 1;
}

cvHealth.prototype.setDeviceSelection = function(devices) {
  if(devices && isArray(devices) && devices.length) {
    this.devices = devices;
  } else {
    this.devices = this.cluster.allKeys('Device');
  }
}

cvHealth.prototype.start = function() {
  this.alertLevel = this.cluster.find('Metric', 'AlertLevel');
  if(!!this.alertLevel) {
    this.alertLevel = this.alertLevel.get('uniqueKey');
    this.timer = window.setInterval(createObjectCallback(this, this.update), this.refreshRate * 1000);
  } else {
    dock.displayInfo('Unable to find AlertLevel metric');
  }
}

cvHealth.prototype.stop = function() {
  println('health stop ' + this.timer);
  this.stopProcess();
  if(this.showhealthTimer) window.clearTimeout(this.showhealthTimer);  
  if(this.timer) {
    this.last = 0;
    window.clearInterval(this.timer);
    this.timer = 0;
    return true;
  }
  return false;
}

cvHealth.prototype.stopProcess = function() {
  if(this.processTimer) window.clearInterval(this.processTimer);  
  this.processStop = true;
}

cvHealth.prototype.setRefreshRate = function(rate) {
  this.refreshRate = rate;
  if(this.timer) {
    this.stop();
  }
  this.start();
}


cvHealth.prototype.processLatestPickedRates = function(data) {
  if(this.processIndex == 0) {
    this.data = new Array();
    this.processIndexJ = 0;
  }
  println("processLatestPickedRates: " + this.processIndex + " / " + this.devices.length);

  var i = 0;
  while(!this.processStop && (this.processIndex < this.devices.length) && (this.processIndexJ < this.processData.length)) {
    if(i >= dock.FILL_COUNT) {
      window.setTimeout(createObjectCallback(this, this.processLatestPickedRates), dock.FILL_TIMEOUT);
      return;
    }
    var one = new Array();
    one.push(''); // SORT!
    one.push(this.cluster.resolveName(this.devices[this.processIndex]));
    if((this.processIndexJ < this.processData.length) && (this.devices[this.processIndex] == this.processData[this.processIndexJ].devId)) {
      one.push(this.processData[this.processIndexJ].rate);
      var info = this.processData[this.processIndexJ].infoMessage;
      var idx = info.indexOf('FAIL');
      if(idx < 0) {
        one.push('');
      } else {
        var jdx = info.indexOf("\n", idx);
        if(jdx < 0) jdx = info.length;
        one.push(info.substr(idx + 5, jdx - idx - 5).ltrim());
      }
      var idx = info.indexOf('THRESH');
      if(idx < 0) {
        one.push('');
      } else {
        var jdx = info.indexOf("\n", idx);
        if(jdx < 0) jdx = info.length;
        one.push(info.substr(idx + 7, jdx - idx - 7).ltrim());
      }
      var idx = info.indexOf('UNKNOWN');
      if(idx < 0) {
        one.push('');
      } else {
        var jdx = info.indexOf("\n", idx);
        if(jdx < 0) jdx = info.length;
        one.push(info.substr(idx + 8, jdx - idx - 8).ltrim());
      }
      this.processIndexJ++;
    } else {
      one.push('no data');
      one.push('');
      one.push('');
      one.push('');
    }
    one.push(this.devices[this.processIndex]);
    if(this.sort == 0) {
      one[0] = one[1];
    } else if(this.sort == 1) {
      if(one[2] == 'no data') {
        one[0] = -1;
      } else {
        one[0] = one[2];
      }
    }
    this.data.push(one);
    this.processIndex++;
    i++;
  }
  if(!this.processStop) {
    if(this.sort == 1) {
      this.data.sort(function(a,b) { return a[0] < b[0]; } );
    } else {
      this.data.sort();
    }
    this.showhealth();
  }
  this.updating = false;
  dock.unwaitIcon();
}

cvHealth.prototype.processDataByIntervalNum = function() {
  if(this.processIndex == 0) this.data = new Array();
  var i = 0;
  println("processDataByIntervalNum: " + this.processIndex + " / " + this.processData.length);
  while(!this.processStop && (this.processIndex < this.processData.length)) {
    if(i >= dock.FILL_COUNT) {
      window.setTimeout(createObjectCallback(this, this.processDataByIntervalNum), dock.FILL_TIMEOUT);
      return;
    }
    var cur = this.processData[this.processIndex].readMonDataId;
    if(cur.devId == this.devices[this.processIndex]) {
      var rates = this.processData[this.processIndex].rates;
      this.data[this.processIndex] = new Array();
      for(var k = 0; k < rates.length; k++) {
        this.data[this.processIndex].push([rates[k].timeStampBegin, rates[k].rate]);
        if(rates[k].timeStampEnd != rates[k].timeStampBegin) {
          this.data[this.processIndex].push([rates[k].timeStampEnd, rates[k].rate]);
        }
      }
    }
    this.processIndex++;
    i++;
  }
  if(!this.processStop) {
    var h = this.setH();
    this.draw(this.svg.parentNode.boxObject.width - this.WM, h);
  }
  this.updating = false;
  dock.unwaitIcon();
}

cvHealth.prototype.update = function(refresh) {
  refresh = isundefined(refresh, 1);
  if(this.devices.length == 0) {
    this.clear();
  } else if(!this.cluster.connected) {
    println("Health update, no longer connected");
    window.clearInterval(this.timer);
    this.timer = 0;    
  } else {
    if(!this.updating) {
      this.updating = true;
      var now = new Date();
      now = Math.floor(now.getTime() / 1000) + this.cluster.timeOffset;
      if((refresh == 2) || ((refresh == 1) && (now - this.last > 5))) {
        this.last = now;
        this.stopProcess();
        if(this.period == 0) {
          var alertlevel = new Array();
          var mpi = new dock.cvMetricPrmId();
          mpi.metricId = this.alertLevel;
          mpi.metricParam = this.alertParam;
          alertlevel.push(mpi);
          dock.waitIcon();
          dock.workerpool.getWorker(current.actualurl).call('cmmon', 'getLatestPickedRates', [this.devices, alertlevel], function(code, data) {
              if(code == 0) {
                this.current.processData = data;
                this.current.processIndex = 0;
                this.current.processStop = false;
                this.current.processLatestPickedRates(data);
              } else {
                dock.jsonCallError(this, code, data);
                this.current.updating = false;
                dock.unwaitIcon();
              }
            }, this);
        } else {
          this.endTime = now;
          if(this.period) {
            this.startTime = this.endTime - this.period;
          } else {
            this.startTime = 0;
          }
          var rmdi = new Array();
          for(var i = 0; i < this.devices.length; i++) {
            if(this.devices[i] > 0) {
              var one = new dock.cvReadMonDataId();
              one.devId = this.devices[i];
              one.metricId = this.alertLevel;
              one.metricParam = this.alertParam;
              one.begTime = this.startTime;
              one.endTime = this.endTime;
              rmdi.push(one);
            }
          }
          this.data = new Array();
          if(rmdi.length > 0) {
            dock.waitIcon();
            dock.workerpool.getWorker(current.actualurl).call('cmmon', 'readDataByIntervalNum', [rmdi, 0], function(code, data) {
                this.current.last = now;
                if(code == 0) {
                  this.current.processData = data;
                  this.current.processIndex = 0;
                  this.current.processStop = false;
                  this.current.processDataByIntervalNum();
                } else {
                  dock.jsonCallError(this, code, data);
                  this.current.updating = false;
                  dock.unwaitIcon();                
                }            
              }, this);
          } else {
            this.last = now;
            this.updating = false;
            dock.unwaitIcon();
          }
        }
      } else {
        dock.waitIcon();
        if(this.period == 0) {
          this.showhealth();
        } else {
          var h = this.setH();
          this.draw(this.svg.parentNode.boxObject.width - this.WM, h);
        }
        this.last = now;
        this.updating = false;
        dock.unwaitIcon();
      }
    } else {
      this.updating = false;
    }
  }
}

cvHealth.prototype.setH = function(set) {
  if(typeof(set) == 'undefined') {
    var b = Math.max(this.perPage * (this.page - 1), 0);
    var e = Math.min(this.perPage * this.page - 1, this.showhealthRows.length);    
    var h = Math.max(MIN_HEIGHT, this.svg.parentNode.boxObject.height - this.HM, (e - b + 1) * 24 + 20);
    this.svg.setAttribute("height", h + "px");
    return h;
  } else {
    this.svg.setAttribute("height", "10px");
  }
  return 0;
}

cvHealth.prototype.clear = function() {
  this.devices = new Array();
}

cvHealth.prototype.draw = function(w, h) {
  var sx = w;
  var sy = h;
  var s = Math.min(w, h);
  var x0 = 20;
  var x1 = w - 10;
  var y1 = 0;
  var y0 = h - 20;
  this.drawbox = [x0, y1, x1, y0];

  var bh = 24;
  var sp = 2;

  var x2 = x0 + 96;
  var x3 = x1 - sp * 2 - x2;
  var y3 = bh - 2 * sp;

  var T = this.endTime - this.startTime;

  var dd = new Date();
  dd.setTime(this.startTime * 1000);
  this.startlabel.setAttribute('value', dd.format("dd/mmm/yyyy HH:MM:ss"));
  this.startlabel.previousSibling.setAttribute('width', x2 - 8);
  var dd = new Date();
  dd.setTime(this.endTime * 1000);
  this.endlabel.setAttribute('value', dd.format("dd/mmm/yyyy HH:MM:ss"));
  this.endlabel.nextSibling.setAttribute('width', 28);

  this.info.setAttribute('value', 'Alert Level:' + this.alertParam);

  while(this.box.firstChild) this.box.removeChild(this.box.firstChild);
  var sorted = new Array();
  var n = 0;
  if(this.sort == 0) {
    for(var is = 0; is < this.data.length; is++) {
      var work = this.data[is];
      var name = this.cluster.resolveName(this.devices[is]);
      sorted.push([name, is]);
    }
    sorted.sort(function(a,b) { return a[0] > b[0]; });
    n = 1;
  } else if(this.sort == 1) {
    for(var is = 0; is < this.data.length; is++) {
      var work = this.data[is];
      var name = this.cluster.resolveName(this.devices[is]);
      if(work.length > 0) {
        sorted.push([work[work.length - 1][1], name, is]);
      } else {
        sorted.push([-1, name, is]);
      }
    }
    sorted.sort(function(a,b) { if(a[0] == b[0]) { return a[1] > b[1]; } else { return b[0] - a[0]; } });
    n = 2;
  } else {
    for(var is = 0; is < this.data.length; is++) {
      sorted.push(is);
    }
  }
  var alertMax = this.cluster.find(this.alertLevel);
  alertMax = Math.max(10, alertMax.get('maximum'));
  document.getElementById('alertMax').setAttribute('value', alertMax);
  var b = Math.max(this.perPage * (this.page - 1), 0);
  var e = Math.min(this.perPage * this.page - 1, this.data.length);  
  for(var is = b; is < e; is++) {
    var i = sorted[is][n];
    var y9 = y1 + (is - b) * bh;
    if(y9 + bh > y0) continue;
    var work = this.data[i];
    var name = this.cluster.resolveName(this.devices[i]);
    svgtext(this.box, x0 + 8, y9 + 3 * SVG_FONT_SIZE / 2, name, SVG_FONT_SIZE, 0, FONT_COLOR, "start");
    var y2 = y9 + sp;
    svgrect(this.box, x2, y2, x3, y3, "none", 0.3, "#000000");

    var xrange = [this.startTime, this.endTime];
    var yrange = [0, 1];
    svggraphblock(this.box, x2, y2, x2 + x3, y2 + y3, xrange, yrange, work, 0, 1, alertMax, 0.5, true);
  }
}

cvHealth.prototype.clearScreen = function() {
  var obj = document.getElementById('healthstatusrows');
  while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
}

cvHealth.prototype.showhealth = function() {
  if(this.showhealthTimer) window.clearTimeout(this.showhealthTimer);
  this.showhealthRows = new Array();
  this.showhealthIndex = 0;
  this.showhealthTimer = window.setTimeout(createObjectCallback(this, this.showhealthUpdate), dock.FILL_TIMEOUT);
}

cvHealth.prototype.pages = function(perPage) {
  this.perPage = isundefined(perPage, this.perPage);
  return Math.ceil(this.showhealthRows.length / this.perPage);
}

cvHealth.prototype.displayPage = function(page) {
  this.page = Math.min(Math.max(1, isundefined(page, this.page)), this.pages());
  if(this.period == 0) {  
    var obj = document.getElementById('healthstatusrows');
    while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
    
    var b = Math.max(this.perPage * (this.page - 1), 0);
    var e = Math.min(this.perPage * this.page - 1, this.showhealthRows.length);
//     println('showhealth page ' + this.page + ' = [' + b + ',' + e + ')');
    
    for(var i = b; i < e; i++) {
      for(var j = 0; j < this.showhealthRows[i].length; j++) {
        obj.appendChild(this.showhealthRows[i][j]);
      }
    }
  } else {
    var b = Math.max(this.perPage * (this.page - 1), 0);
    var e = Math.min(this.perPage * this.page - 1, this.showhealthRows.length);
    
    var h = Math.max(MIN_HEIGHT, this.svg.parentNode.boxObject.height - this.HM, (e - b + 1) * 24 + 20);
    this.svg.setAttribute("height", h + "px");
    this.draw(this.svg.parentNode.boxObject.width - this.WM, h);
  }
  updatePageMenu(this.page);
}

cvHealth.prototype.showhealthUpdate = function() {
  var n = Math.min(this.data.length, this.devices.length);
  println('showhealth ' + this.showhealthIndex + ' / ' + n);
  for(var j = 0; j < dock.FILL_COUNT * 5; j++) {
    if(this.showhealthIndex >= n) {
      this.showhealthTimer = 0;
      createPageMenu(this.pages());
      this.displayPage();
      println('showhealth [DONE]');
      return;
    }
    healthRows(this.showhealthRows, this.data[this.showhealthIndex], this.showhealthIndex);
    this.showhealthIndex++;
  }
  this.showhealthTimer = window.setTimeout(createObjectCallback(this, this.showhealthUpdate), dock.FILL_TIMEOUT);
}

function healthRows(rows, data, j) {
  var list = new Array();
  var n = 0;
  for(var i = 1; i <= 6; i++) {
    if(typeof(data[i]) == 'string') {
      var one = data[i].split(/,\s*/);
      list.push(one);
      n = Math.max(n, one.length);
    } else {
      list.push([data[i]]);
//         n = 1;
      n = Math.max(n, 1);
    }
  }
  var subrows = new Array();
  for(var i = 0; i < n; i++) {
    var row = document.createElement('row');
    row.setAttribute("class", "staticdatarow");
    row.setAttribute("flex", "1");
    row.setAttribute("align", "center");
    row.setAttribute("height", "19px");
    if(j % 2 == 1) row.setAttribute("style", "background-color: #e3e3e3;");
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    if(i < list[0].length) {
      label.setAttribute("value", list[0][i]);
      label.setAttribute("ondblclick", "opendevice(" + list[5][0] +");");
    }
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    if(i < list[1].length) label.setAttribute("value", list[1][i]);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    if(i < list[2].length) label.setAttribute("value", list[2][i]); // FAIL
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    if(i < list[4].length) label.setAttribute("value", list[4][i]); // UNKNOWN
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    if(i < list[3].length) label.setAttribute("value", list[3][i]); // THRESH
    row.appendChild(label);
    if(i == 0) {
      var label = document.createElement('button');
      label.setAttribute("label", 'Show');
      label.setAttribute("class", 'graybutton');
      label.setAttribute("oncommand", 'showdetails(' + list[5][0] + ');');
      row.appendChild(label);
    } else {
      var label = document.createElement('spacer');
      row.appendChild(label);
    }
    subrows.push(row);
  }
  rows.push(subrows);
}
