/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var overview;

var force = false;

var perPage = 25;
var page = 1;
var pages = 1;
var sortedlist = 0;
var timer = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  println(".Refresh: " + document.title + ' - ' + document.documentURI);
  if(!!use) {
    current = use;
  }
  update();
}

function update() {
  if(!current.connected) return;
  dock.workerpool.getWorker(current.actualurl).call('lustre', 'getAlerts', [0, 0, false], function(code, data) {
    if ( data.errorMessage && ! blank(data.errorMessage) ) {
      current.lustreErrorCache(data.errorMessage);
    }
    if(code == 0) {
      setCallbackResult(data.alerts);
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);
}

function setCallbackResult(value) {
  overview = value;
  if(overview == null) return;

  var list = document.getElementById('alerts');
  while(list.childNodes.length > 2) list.removeChild(list.lastChild);

  var timecol = document.getElementById('sort1');
  sortedlist = new Array();

  for(var i = 0; i < overview.length; i++) {
    var one = [0, overview[i].begin, overview[i].end, overview[i].alertItemStr, overview[i].severity, overview[i].id, overview[i].alertType, overview[i].message];
    one[0] = one[dock.generalSettings.ucsfaultsort];
    if(typeof(one[0]) == 'string') one[0] = one[0].toLowerCase();
    sortedlist.push(one);
  }
  sortedlist.sort();

  var n = Math.max(1, Math.ceil(sortedlist.length / perPage));
  createPageMenu(n);
  displayPage();

  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 1000);
}

function alertRow(alertr, i0) {
    var begin = alertr[1];
    var end = alertr[2];
    var host = alertr[3];
    var severity = alertr[4];
    var id = alertr[5];
    var alerttype = alertr[6];
    var message = alertr[7];

    var row = document.createElement("row");
    row.setAttribute('align', 'center');
    row.setAttribute("height", "19px");
    if(i0 % 2 == 0) row.setAttribute("style", "background-color: #dcdcdc;");

    var bdate = new Date();
    bdate.setTime(begin * 1000);
    var label = document.createElement("label");
    label.setAttribute('value', bdate.format("dd/mmm/yyyy HH:MM:ss"));
    row.appendChild(label);

    var edate = new Date();
    edate.setTime(end * 1000);
    var label = document.createElement("label");
    label.setAttribute('value', edate.format("dd/mmm/yyyy HH:MM:ss"));
    row.appendChild(label);

    var label = document.createElement("label");    
    label.setAttribute('value', host);      
    row.appendChild(label);    
    var label = document.createElement("label");    
    label.setAttribute('value', severity);      
    row.appendChild(label);    
    var label = document.createElement("label");    
    label.setAttribute('value', id);      
    row.appendChild(label);    
    var label = document.createElement("label");
    label.setAttribute('value', alerttype);
    row.appendChild(label);    
    var label = document.createElement("label");
    label.setAttribute('value', message);
    row.appendChild(label);

    return row;
}

function displayPage(index) {
  page = isundefined(index, page);
  var list = document.getElementById('alerts');
  while(list.childNodes.length > 2) list.removeChild(list.lastChild);
  
  var b = Math.max(perPage * (page - 1), 0);
  var e = Math.min(perPage * page - 1, sortedlist.length);
  for(var i = b; i < e; i++) {
    list.appendChild(alertRow(sortedlist[i], i));
  }
  updatePageMenu(page);
}

function createPageMenu(n) {
  var pagespopup = document.getElementById("pages");
  var m = pagespopup.childNodes.length;
  for(var i = n; i < m; i++) pagespopup.removeChild(pagespopup.lastChild);
  for(var i = m + 1; i <= n; i++) {
    var item = document.createElement('menuitem');
    item.setAttribute('page', i);
    item.setAttribute('label', i);
    pagespopup.appendChild(item);
  }
  pages = n;
}

function updatePageMenu(page) {
  var pagespopup = document.getElementById("pages");
  pagespopup.parentNode.selectedIndex = page - 1;
}

function setPage(index) {
  if((index >= 1) && (index <= pages)) {
    displayPage(index);
  }
}

function perpagemenu(event) {
  var perPageX = parseInt(event.target.getAttribute('page'));
  if(perPageX != perPage) {
    var old = perPage * (page - 1);
    perPage = perPageX
    var n = Math.max(1, Math.ceil(sortedlist.length / perPageX));    
    createPageMenu(n);
    displayPage(Math.min(1 + Math.floor(old / perPageX), n));
  }
}

function pagemenu(event) {
  setPage(parseInt(event.target.getAttribute('page')));
}

function pagetop() {
  setPage(1);
}

function pagebottom() {
  setPage(pages);
}

function pageup() {
  setPage(page - 1)
}

function pagedown() {
  setPage(page + 1);
}

function starSort(index) {
  var i = 1;
  while(true) {
    var obj = document.getElementById('sort' + i);
    if(!obj) break;
    var value = obj.getAttribute('value');
    var star = value.indexOf('*');
    if((index == i) && (star < 0)) {
      obj.setAttribute('value', value + ' *');
    } else if((index != i) && (star >= 0)) {
      star = value.lastIndexOf(' ', star);
      obj.setAttribute('value', value.substr(0, star));
    }
    println(obj.getAttribute('value') + ' ' + star);
    i++;
  }
}

function sortBy(event) {
  var index = 1;
  if(typeof(event) == 'number') {
    index = event;
  } else {
    var item = event.target;
    index = parseInt(item.getAttribute('sort'));
  }
  starSort(index);
  dock.generalSettings.ucsfaultsort = index;
  setCallbackResult(overview);
}
