/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var overview;
var timer = 0;
var forcerefresh = true;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  println(".Refresh: " + document.title + ' - ' + document.documentURI);
  if(!!use) {
    current = use;
  }
  update();
}

function update() {
  if(!current.connected) return;
  dock.workerpool.getWorker(current.actualurl).call('lustre', 'getFileSystems', [0, 0, false], function(code, data) {
      if ( ! blank(data.errorMessage) ) {
        current.lustreErrorCache(data.errorMessage);
      }
      if(code == 0) {
        setCallbackResult(data.fileSystems);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
    forcerefresh = false;
}

function setCallbackResult(value) {
  overview = value;
  if(overview == null) return;
  var list = document.getElementById('filesystems');
  //FIXME (JD): Fix reloading of existing filesystems, which resets hidden attribute
  //Typically, we want to keep track of whether it is an update of full reload.
  if (forcerefresh) {
    while(list.childNodes.length > 2) list.removeChild(list.lastChild);
  }
  for(var j = 0; j < overview.length; j++) {
    if ( document.getElementById(overview[j].name + '_filesystem') == null ) {
      list.appendChild(fileSystemRow(overview[j]), j);
      addTargetsRow(overview[j], j, list);
    }
  }
  
  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 1000);
}

function fileSystemRow(filesystem, j) {

  var managementserver = filesystem.mgt.primaryServerName;
  var metadataserver = filesystem.mdts[0].primaryServerName;

  var row = document.createElement('row');
  row.setAttribute("class", "staticdatarow");
  row.setAttribute("flex", "1");
  row.setAttribute("align", "center");
  row.setAttribute("height", "19px");
  row.setAttribute("id", filesystem.name + '_filesystem');
  if(j % 2 == 1) row.setAttribute("style", "background-color: #e3e3e3;");

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmallurl");
  label.setAttribute("value", filesystem.name);
  label.setAttribute('onclick', 'showFileSystemDetails("' + filesystem.name + '");');
  //if(index >= 0) label.setAttribute('onclick', 'setMinMax(' + index + ',' + low + ',"' + name + '","' + unit + '");');
  row.appendChild(label);

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", managementserver); 
  row.appendChild(label);

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", metadataserver);
  row.appendChild(label);

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", filesystem.clientCount);
  row.appendChild(label);

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value",  formatKMGselect(filesystem.bytesTotal, 'B'));
  row.appendChild(label);

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value",  formatKMGselect(filesystem.bytesFree, 'B'));
  row.appendChild(label);

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value",  formatKMG10(filesystem.filesTotal));
  row.appendChild(label);

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", filesystem.filesFree);
  row.appendChild(label);

  return row;
}

function addTargetsRow(fs, j, l) {
  /* Implement differently, possibly using overlays ? */
  var targets = document.getElementById('targets');
  var clone = targets.cloneNode(true);
  clone.setAttribute("id", fs.name + '_targets');
  if (document.getElementById(fs.name + '_targets') != null) {
    clone.setAttribute("hidden", document.getElementById(fs.name + '_targets').hidden);
  }
  l.appendChild(clone);

  // Management target
  var mgtrows = document.getElementById('managementtargets');
  mgtrows.setAttribute("id", fs.name + '_managementtargets');
  while(mgtrows.childNodes.length > 2) mgtrows.removeChild(mgtrows.lastChild);
  

  var row = document.createElement('row');
  row.setAttribute("class", "staticdatarow");
  row.setAttribute("flex", "1");
  row.setAttribute("align", "center");
  row.setAttribute("height", "19px");
  if(j % 2 == 1) row.setAttribute("style", "background-color: #e3e3e3;");

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", fs.mgt.name);
  row.appendChild(label);

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", fs.mgt.volumeName);
  row.appendChild(label);

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", fs.mgt.primaryServerName);
  row.appendChild(label);

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", fs.mgt.failoverServerName);
  row.appendChild(label);

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", fs.mgt.inodeCount);
  row.appendChild(label);

  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", fs.mgt.inodeSize);
  row.appendChild(label);
 
  mgtrows.appendChild(row)
  
  // Metadata targets
  var mdtrows = document.getElementById('metadatatargets');
  mdtrows.setAttribute("id", fs.name + '_metadatatargets');
  while(mdtrows.childNodes.length > 2) mdtrows.removeChild(mdtrows.lastChild);
  for ( var i = 0; i < fs.mdts.length; i++ ) {
    var row = document.createElement('row');
    row.setAttribute("class", "staticdatarow");
    row.setAttribute("flex", "1");
    row.setAttribute("align", "center");
    row.setAttribute("height", "19px");
    if(j % 2 == 1) row.setAttribute("style", "background-color: #e3e3e3;");

    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", fs.mdts[i].name);
    row.appendChild(label);

    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", fs.mdts[i].volumeName);
    row.appendChild(label);

    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", fs.mdts[i].primaryServerName);
    row.appendChild(label);

    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", fs.mdts[i].failoverServerName);
    row.appendChild(label);

    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", fs.mdts[i].inodeCount);
    row.appendChild(label);

    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", fs.mdts[i].inodeSize);
    row.appendChild(label);

    mdtrows.appendChild(row)
  }

  // Object storage targets
  var ostrows = document.getElementById('objectstoragetargets');
  ostrows.setAttribute("id", fs.name + '_objectstoragetargets');
  while(ostrows.childNodes.length > 2) ostrows.removeChild(ostrows.lastChild);
  for ( var i = 0; i < fs.osts.length; i++ ) {
    var row = document.createElement('row');
    row.setAttribute("class", "staticdatarow");
    row.setAttribute("flex", "1");
    row.setAttribute("align", "center");
    row.setAttribute("height", "19px");
    if(j % 2 == 1) row.setAttribute("style", "background-color: #e3e3e3;");

    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", fs.osts[i].name);
    row.appendChild(label);

    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", fs.osts[i].volumeName);
    row.appendChild(label);

    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", fs.osts[i].primaryServerName);
    row.appendChild(label);

    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", fs.osts[i].failoverServerName);
    row.appendChild(label);

    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", fs.osts[i].inodeCount);
    row.appendChild(label);

    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", fs.osts[i].inodeSize);
    row.appendChild(label);

    ostrows.appendChild(row)

  }
}

function showFileSystemDetails(name) {
  var elem = document.getElementById(name + '_targets');
  //elem.hidden = !elem.hidden;
  elem.setAttribute("hidden", !elem.hidden);
}
