/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var overview;
var graph;
var monconf;
var timer = 0;

function onLoad() {
  dock = getDock();
  addEventListener("resize", onResize, false);
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function getCurrentSelection() {
  return [[current.find('Partition', 'base')], 0, ""];
}

function refresh(use) {
  println(".Refresh: " + document.title + ' - ' + document.documentURI);
  if(!!use) {
    var active = use.activeHeadNode();
    if(!active) return;
    monconf = active.getMonitoringConfiguration();
    if(!monconf) return;
    current = use;
  }
  update();
  
  // Load graph
  if(graph) {
    graph.cluster = current;
  } else {
    graph = new cvGraph(current, document.getElementById("svg"), document.getElementById("box"));
    graph.colors = [[['#7cc300', '#7dc200', '#4d7900']]];
    graph.WM = 32;
    graph.HM = 61;
    graph.legend = -1;
  }
  var list = document.getElementById("metriclist_lustre");
  
  var mmc = monconf.get("metrics");
  createMetricPopupShowClass(current, mmc, 'metriclist_lustre', false, true, ['LUSTRE']);
  var found = false;
  for(var i = 0; (i < mmc.length) && !found; i++) {
    found = ((mmc[i].metric == key) && (mmc[i].metricParam == param));
  }

  var key = 0;
  var param = '';
  var label = '';
  var object = 0;
  if(object == 0 || isNaN(object)) object = current.activePassiveKeys[0];
  //println('object: "' + object + '" ' + typeof(object));

  if(!found) {
    if(mmc.length == 0) {
      var item = document.createElement("menuitem");
      item.setAttribute('metricKey', 0);
      item.setAttribute('param', '');
      item.setAttribute('label', '');
      list.firstChild.appendChild(item);
      key = 0;
      param = '';
      label = '';
    } else {
      key = mmc[0].metric;
      param = mmc[0].metricParam;
      var metric = current.find(key);
      if(metric) {
        label = metric.resolveName();
        if(param.length > 0) label += "[" + param + "]";
        if(metric.get('unit')) label += " (" + metric.get('unit') + ")";
      }
    }
  }
  list.setAttribute('metricKey', key);
  list.setAttribute('param', param);
  list.setAttribute('label', label);

  if(object) {
    if(key) {
      var dmp = new cvGraphDeviceMetric(object, key, param);
      var r = graph.add(dmp);
      redraw();
    } else {
      dock.displayError("No metrics defined");
    }
  } else {
    dock.displayError("Primary head node not defined");
  }
}

function update() {
  if(!current.connected) return;
  dock.workerpool.getWorker(current.actualurl).call('lustre', 'getOverview', [false], function(code, data) {
      if ( data.errorMessage && ! blank(data.errorMessage) ) {
        current.lustreErrorCache(data.errorMessage);
      }
      if(code == 0) {
        setCallbackResult(data.overview);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
}

function setCallbackResult(value) {
  overview = value;
  if(overview == null) return;
  if(dock.IEELDEMO2) {
    overview.ieelVersion = "2.2.0.0"; 
    overview.lustreServers = 5; 
    overview.lustreFileSystems = 10; 
    overview.ieelUsers = 20; 
    overview.clientMounts = 50; 
    overview.lustreVolumes = 20; 
    overview.lustreVolumeNodes = 10; 
    overview.memoryTotal = 16e+9 * 512;
    overview.memoryUsageMdts = 16e+9 * 503;
    overview.memoryUsageOsts = 16e+9 * 503;
    overview.cpuUsageMdts = 0.15
    overview.cpuUsageOsts = 0.10;
    overview.cpuUsageSystem = 0.29;
    overview.cpuUsageIdle = 0.10;                       
    overview.spaceUsed = 20;
    overview.spaceTotal = 200;
    overview.filesUsed = 200;
    overview.filesTotal = 200000;
  }
  overviewValue("ieelVersion");
  overviewValue("lustreServers");
  overviewValue("lustreFileSystems");
  overviewValue("ieelUsers");
  overviewValue("clientMounts");
  overviewValue("lustreVolumes");
  overviewValue("lustreVolumeNodes");

  overviewBar('memoryUsageMdts', overview.memoryUsageMdts, overview.memoryTotal, 'B', true);
  overviewUsageSingle('cpuUsageMdts', overview.cpuUsageMdts);
  overviewBar('memoryUsageOsts', overview.memoryUsageOsts, overview.memoryTotal, 'B', true);
  overviewUsageSingle('cpuUsageOsts', overview.cpuUsageOsts);
  overviewBar('spaceUsage', overview.spaceUsed, overview.spaceTotal, 'B', true);
  overviewBar('filesUsage', overview.filesUsed, overview.filesTotal);
  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 1000);
}

function overviewUsageSingle(name, system) {
  var obj = document.getElementById(name);
  obj.replaceChild(createStatusSingle("12px", system), obj.firstChild.nextSibling);
  obj = obj.lastChild;
  obj.firstChild.setAttribute('value', sprintf("%.2f", 100 * system) + '%');
}

function createStatusSingle(height, a) {
  var stack = document.createElement('stack');
  var status = document.createElement('hbox');
  status.setAttribute("flex", "1");
  status.setAttribute("height", height);
  var left = document.createElement('hbox');
  var total = 0.9;
  var size = 100 / total;
  left.setAttribute("flex", a * size);
  left.setAttribute("class", "leftstatus");
  status.appendChild(left);
  stack.appendChild(status);
  var mask = document.createElement('hbox');
  mask.setAttribute("flex", "1");
  mask.setAttribute("height", height);
  var img = document.createElement('image');
  img.setAttribute('src', './skin/gradient/blank' + height + 'l.png');
  mask.appendChild(img);
  var spacer = document.createElement('image');
  spacer.setAttribute('flex', 1);
  mask.appendChild(spacer);
  var img = document.createElement('image');
  img.setAttribute('src', './skin/gradient/blank' + height + 'r.png');
  mask.appendChild(img);
  stack.appendChild(mask);
  return stack;
}

function onResize() {
  var svg = document.getElementById("svg");
  svg.setAttribute("height", "25px");
  svg.setAttribute("width", svg.parentNode.boxObject.width);
  svg.setAttribute("height", svg.parentNode.boxObject.height - 128); // 128 ?? KDR
  redraw();
}

function selectMetric(event) {
  var list = document.getElementById("metriclist_lustre");
  var metric = event.target;
  var name = metric.getAttribute('label');
  list.setAttribute('metricKey', graph.deviceMetric[0].metric);
  list.setAttribute('param', graph.deviceMetric[0].param);
  list.setAttribute('label', name);
  redraw();
  if(!!metric) {
    graph.last = 0;
    var old = graph.deviceMetric[0].device;
    graph.deviceMetric[0].metric = parseInt(metric.getAttribute('metricKey'));
    graph.deviceMetric[0].param = metric.getAttribute('param');
    graph.deviceMetric[0].unit = current.find(graph.deviceMetric[0].metric).get('unit');
    var name = metric.getAttribute('label');
    list.setAttribute('metricKey', graph.deviceMetric[0].metric);
    list.setAttribute('param', graph.deviceMetric[0].param);
    list.setAttribute('label', name);
    redraw();
  }
}

function redraw() {
  if (graph) {
    graph.update();
  }
}

function stop() {
  if(graph) graph.stop();
  if(timer) window.clearTimeout(timer);
}

