/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var overview;
var timer = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  println(".Refresh: " + document.title + ' - ' + document.documentURI);
  if(!!use) {
    current = use;
  }
  update();
}

function update() {
  if(!current.connected) return;
  dock.workerpool.getWorker(current.actualurl).call('lustre', 'getServers', [0, 0, false], function(code, data) {
    if ( data.errorMessage && ! blank(data.errorMessage) ) {
      current.lustreErrorCache(data.errorMessage);
    }
    if(code == 0) {
      setCallbackResult(data.servers);
    } else {
      dock.jsonCallError(this, code, data.errorMessage);
    }
  }, current);
}

function setCallbackResult(value) {
  overview = value;
  if(overview == null) return;
  var list = document.getElementById('lustreservers');
  while(list.childNodes.length > 2) list.removeChild(list.lastChild);

  for(var j = 0; j < overview.length; j++) {
    list.appendChild(serverRow(overview[j], j));
  }

  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 1000);
}

function serverRow(server, j) {
  var row = document.createElement('row');
  row.setAttribute("class", "staticdatarow");
  row.setAttribute("flex", "1");
  row.setAttribute("align", "center");
  row.setAttribute("height", "19px");
  if(j % 2 == 1) row.setAttribute("style", "background-color: #e3e3e3;");

  // Hostname
  var label = document.createElement('label');
  //label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("class", "staticdatavaluesmallurl");
  label.setAttribute("value", server.nodename);
  label.setAttribute("onclick", "selectObject(this);");
  row.appendChild(label);

  // Software image
  var node = current.find('Node', server.nodename);
  var softwareimage = current.resolveName(node.getSoftwareImage());
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmallurl");
  label.setAttribute("value", softwareimage); 
  label.setAttribute("onclick", "selectObject(this);");
  row.appendChild(label);
 
  // Status
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", "N/A"); //TODO: Set value based on alerts for this node
  row.appendChild(label);

  // Profile name 
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", server.serverProfileUiName);
  row.appendChild(label);

  // LNet state 
  var hbox = document.createElement('hbox');
  hbox.setAttribute("align", "center");
  hbox.setAttribute("style", "padding-left:20px");
  var image = document.createElement('image');
  image.setAttribute("src", lnetState(server.state));
  hbox.appendChild(image);
  row.appendChild(hbox);
  
  return row;
}

function lnetState(s) {
  if ( s == "lnet_up" ) return "../icons/ok.png";
  if ( s == "lnet_down" ) return "../skin/image/exclamation.png";
  if ( s == "removed" ) return "../icons/closetab.png";
  if ( s == "lnet_unloaded" ) return "../skin/image/Status_closed.png";
  return "../skin/image/LED_blank.png";
}

function selectObject(item) {
  current.select(current.resolveKey(item.value));
}
