(function() {
  var netaddr, root;

  netaddr = {};

  root = this;

  if ((typeof module !== "undefined" && module !== null) && module.exports) {
    module.exports = netaddr;
  } else {
    root['netaddr'] = netaddr;
  }

  netaddr.IPv4 = (function() {
    function IPv4(ip, mask) {
      this.ip = ip;
      this.netmaskbits = mask;
      if (!parseInt(this.netmaskbits)) {
        // TODO
      }
    }

    IPv4.prototype.network = function() {
      return long2ip(ip2long(this.ip) & ip2long(this.netmask()));
    };

    IPv4.prototype.netmask = function() {
      return long2ip(ip2long('255.255.255.255') << (32 - this.netmaskbits));
    };

    IPv4.prototype.size = function() {
      return Math.pow(2, 32 - this.netmaskbits);
    };

    IPv4.prototype.broadcast = function() {
      return long2ip(ip2long(this.network()) | (~ip2long(this.netmask())));
    };

    IPv4.prototype.add = function(value) {
      return new IPv4(long2ip(ip2long(this.ip) + value), this.netmaskbits);
    };

    return IPv4;
  })();

  netaddr.create = function(ip, netmaskbits) {
    return new netaddr.IPv4(ip, netmaskbits);
  };


  function ip2long(ip) {
    var i = 0;
    ip = ip.match(/^([1-9]\d*|0[0-7]*|0x[\da-f]+)(?:\.([1-9]\d*|0[0-7]*|0x[\da-f]+))?(?:\.([1-9]\d*|0[0-7]*|0x[\da-f]+))?(?:\.([1-9]\d*|0[0-7]*|0x[\da-f]+))?$/i);
    if (!ip) {
      return false;
    }
    ip[0] = 0;
    for (i = 1; i < 5; i += 1) {
      ip[0] += !!((ip[i] || '').length);
      ip[i] = parseInt(ip[i]) || 0;
    }
    ip.push(256, 256, 256, 256);
    ip[4 + ip[0]] *= Math.pow(256, 4 - ip[0]);
    if (ip[1] >= ip[5] || ip[2] >= ip[6] || ip[3] >= ip[7] || ip[4] >= ip[8]) {
      return false;
    }
    return ip[1] * (ip[0] === 1 || 16777216) + ip[2] * (ip[0] <= 2 || 65536) + ip[3] * (ip[0] <= 3 || 256) + ip[4] * 1;
  }

  function long2ip(ip) {
    if (!isFinite(ip)) {
      return false;
    }
    return [ip >>> 24, ip >>> 16 & 0xFF, ip >>> 8 & 0xFF, ip & 0xFF].join('.');
  }


}).call(this);
