/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function checkErrorXML(x) {
  return checkXML(x, 1, '').xt;
}

function checkXML(n, h3OK, xt) {  
  if(n.nodeName == "h3") {
    if(h3OK == 0) {
      return {c: h3OK, xt: xt};
    }
    h3OK = 0;
  }
  if(n.nodeName == "#text") {
    var a = strip_beginning_trailing_newlines(n.nodeValue);
    if(xt.indexOf(a) < 0) {
      xt += a + "\n";
    }
  }
  for(var i = 0; i < n.childNodes.length; i++) {
    var r = checkXML(n.childNodes[i], h3OK, xt);
    h3OK = r.c;
    xt = r.xt;
  }
  return {c: h3OK, xt: xt};
}

function getAttribute(xmlDom, param) {
  if(!xmlDom || !xmlDom.attributes) return null;
  var v = xmlDom.attributes.getNamedItem(param);
  if(v) return v.nodeValue;
  return null;
}

function setNotNullAttribute(xmlDom, param, obj, varname) {
  var x = getAttribute(xmlDom, param);
  if(x != null) obj[varname] = x;
  return x;
}

function setTextValue(xmlDom, param, obj, varname) {
  var x = xmlDom.getElementsByTagName(param);
  if(x.length > 0) obj[varname] = x[0].textContent;
}

function diskSetup_empty(ds) {
  for(var i = 0; i < ds.length; i++) {
    if((ds[i] != ' ') && (ds[i] != '\t') && (ds[i] != '\n') && (ds[i] != '\r')) return false;
  }
  return true;
}

function xml_to_diskSetup(xml) {  
  var parser = new DOMParser();
  var xmlDoc = parser.parseFromString(xml, "text/xml");
  var errors = '';
  
  if(!xmlDoc) {
    errors = 'Failed to parse XML data';
  } else if(xmlDoc.getElementsByTagName("parsererror").length) {
    errors = checkErrorXML(xmlDoc.getElementsByTagName("parsererror")[0]);
  }

  return {value: xml, errors: errors};
}

function validate() {
  var xml = document.getElementById("disksetup").value;
  var ds = xml_to_diskSetup(xml);
  if(ds.errors.length == 0) {
//     var old = current.object.diskSetup.modified;
//     current.object.diskSetup.modified = true;
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'validate' + current.get('baseType'), current.object, function(code, data) {
//         this.current.object.diskSetup.modified = old;
        if(code == 0) {
          var validate = new Array();
          for(var i = 0; i < data.length; i++) {
            if(startswith(data[i].field, 'disksetup')) {
              validate.push(data[i]);
            }
          }
          if(validate.length > 0) {
            var args = new Object();
            args['object'] = this.current;
            args['validate'] = validate;
            var d = new dock.cvDialog("validate.xul", args);
            dock.dialogs.show(d);         
          } else {
            dock.displayInfo('No errors');
          }
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);    
  } else {
    dock.displayInfo(ds.errors);
  }
}

function disksetup_keypress(event) {
  var text = document.getElementById("disksetup");
  var value = text.value;
  switch(event.keyCode) {
    case KEY_ESCAPE:
      text.value = current.get("disksetup");
      break;
    default:
      var save = document.getElementById('save');
      if(save) {
        save.setAttribute('disabled', false);
        //save.clearAttribute('disabled');
      }
  }
}

function disksetup_change(event) {
//   var text = document.getElementById("disksetup");
//   var value = text.value;
  var xml = document.getElementById("disksetup").value;
  current.set("disksetup", xml);
}

function copy() {
  var category = current.cluster.find(current.get('category'));
  if(category) {
    var old = category.get('disksetup');
    current.set("disksetup", old);
    refresh();
  }
}

function clear() {
  current.set("disksetup", '');
  refresh();
}

function load() {
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
  fp.init(window, "Select a file", nsIFilePicker.modeOpen);
  var path = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
  path.initWithPath(dock.getProcessDirectory() + "disk-setup");
  fp.displayDirectory = path;
  fp.appendFilters(nsIFilePicker.filterAll);
  fp.appendFilter("xml disksetup files","*.xml");

  var res = fp.show();
  if((res == nsIFilePicker.returnOK)) {
    var thefile = fp.fileURL;
    var path = thefile.path;
    if(startswith(getOS(), "WIN")) {
      if(startswith(path, "/")) {
        path = path.substr(1);
      }
      path = path.replace(/%20/g, " ");
      path = path.replace(/\//g, "\\\\");
    }
    var xml = readfile(path);
    var ds = xml_to_diskSetup(xml);
    if(!!ds && !!ds.value && (ds.errors.length == 0)) {
      current.set('disksetup', ds.value);      
      refresh();
    }
  }
}

function parseDiskSetupXML(xml) {
  if(typeof xml === 'string') {
    var parser = new DOMParser();
    var xml_doc = parser.parseFromString(xml, "text/xml");
    var errors = '';
    if(!xml_doc) {
      errors = 'Failed to parse XML data';
    } else if(xml_doc.getElementsByTagName("parsererror").length) {
      errors = checkErrorXML(xmlDoc.getElementsByTagName("parsererror")[0]);
    }
    if(errors) {
      throw {value:xml, errors: errors };
    }
    for(var i = 0; i < xml_doc.childNodes.length; i++) {
      if(xml_doc.childNodes[i].tagName && 
         xml_doc.childNodes[i].tagName.toLowerCase() == "disksetup") {
        return xml_doc.childNodes[i];
      }
    }
    throw { value:xml, errors: "missing element 'diskSetup'" }
  }
  else {
    return xml;
  }
}

function isFSSemiDiskless(xml) {
  var xml_doc = parseDiskSetupXML(xml);
  for(var i = 0; i < xml_doc.childNodes.length; i++) {
    if(xml_doc.childNodes[i].tagName && 
       xml_doc.childNodes[i].tagName.toLowerCase() == "diskless") {
      return true;
    }
  }
  return false;
}

function getDiskedMountPoints(xml) {
  function findDevicePartition(xml_doc) {
    for(var i = 0; i < xml_doc.childNodes.length; i++) {
      if(xml_doc.childNodes[i].tagName && 
         xml_doc.childNodes[i].tagName.toLowerCase() == "device") {
        for(var j = 0; j < xml_doc.childNodes[i].childNodes.length; j++) {
          if(xml_doc.childNodes[i].childNodes[j].tagName && 
             xml_doc.childNodes[i].childNodes[j].tagName.toLowerCase() == "partition") { 
            return xml_doc.childNodes[i].childNodes[j];
          }
        }
      }
    }
    return null;
  }
  var xml_doc = parseDiskSetupXML(xml);
  var xml_partition = findDevicePartition(xml_doc);
  var ret = [];
  if(xml_partition) {
    for(var k = 0; k < xml_partition.childNodes.length; k++) {
      if(xml_partition.childNodes[k].tagName && 
         xml_partition.childNodes[k].tagName.toLowerCase() == "mountpoint") { 
        if(xml_partition.childNodes[k].firstChild) {
          if(xml_partition.childNodes[k].firstChild.nodeValue) {
	    ret.push(xml_partition.childNodes[k].firstChild.nodeValue.trim());
          }
        }
      }
    }
  }
  return ret;
}

function isMountpointDiskless(xml, mount_point) {
  if(!isFSSemiDiskless(xml)) {
    return false;
  }
  var mp = getDiskedMountPoints(xml);
  var ret = true;
  mp.forEach(function(path) {
    if(path == mount_point) ret = false;
  });
  return ret;
}

function isPathDiskless(xml, path) {
  if(path != "/") {
    path = path.replace(/\/+$/, '');
  }
  if(!isFSSemiDiskless(xml)) {
    return false;
  }
  var tmp = getDiskedMountPoints(xml);
  var mp = [];
  var ret = true;
  tmp.forEach(function(mount_point) {
    if(mount_point != "/") {
      mount_point = mount_point.replace(/\/$/, '');
    }
    if(path.substr(0, mount_point.length) == mount_point) {
      ret = false;
    } 
  });
  return ret;
}

function isRootFSDiskless(xml) {
  return isPathDiskless(xml, "/");
}

