/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

// var debug = true;
var debug = false;
var HttpRequest = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"];

var fus = Components.classes["@mozilla.org/docshell/urifixup;1"].getService(Components.interfaces.nsIURIFixup);
var overrideService = Components.classes["@mozilla.org/security/certoverride;1"].getService(Components.interfaces.nsICertOverrideService);

function getURI(targetHost) {
  // Use fixup service instead of just ioservice's newURI since it's quite likely
  // that the host will be supplied without a protocol prefix, resulting in malformed
  // uri exceptions being thrown.
  var fus = Components.classes["@mozilla.org/docshell/urifixup;1"].getService(Components.interfaces.nsIURIFixup);
  var uri = fus.createFixupURI(targetHost, 0);
  
  if(!uri)
    return null;

  if(uri.scheme == "http")
    uri.scheme = "https";

  if (uri.port == -1)
    uri.port = 8081;

  return uri;
}

function addException(gSSLStatus, targetHost) {
  if(!gSSLStatus)
    return;
  var gCert = gSSLStatus.QueryInterface(Components.interfaces.nsISSLStatus).serverCert;
  if(!gCert || !gSSLStatus)
    return;

  var flags = 0;
  if(gSSLStatus.isUntrusted)
    flags |= overrideService.ERROR_UNTRUSTED;
  if(gSSLStatus.isDomainMismatch)
    flags |= overrideService.ERROR_MISMATCH;
  if(gSSLStatus.isNotValidAtThisTime)
    flags |= overrideService.ERROR_TIME;

  var uri = getURI(targetHost);
  overrideService.rememberValidityOverride(uri.asciiHost, uri.port, gCert, flags, false); // KDR: false = temporary
}


function badCertListener() {}
badCertListener.prototype = { 
  getInterface: function (aIID) { return this.QueryInterface(aIID); },  
  QueryInterface: function(aIID) {
    if (aIID.equals(Components.interfaces.nsIBadCertListener2) ||
        aIID.equals(Components.interfaces.nsIInterfaceRequestor) ||
        aIID.equals(Components.interfaces.nsISupports))
      return this;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },
  notifyCertProblem: function MSR_notifyCertProblem(socketInfo, sslStatus, targetHost) {
    if(sslStatus) {
      println('bad certificate, add exception');
      addException(sslStatus, targetHost);
    }
    return true; // suppress error UI
  }
}

// searches in the exceptions db for the SSL certificate
// if not found, makes a fake login call to add the exception
function fixSSLCertificateException(url) {
  var uri = getURI(url);
  try {
    var found = overrideService.getValidityOverride(uri.asciiHost, uri.port, {}, {}, {}, {});
    if (!found) {
      println("fixing bad certificate using a fictitious login call");
      var current = HttpRequest.createInstance();
      current.open("POST", url + "/json", false);
      current.channel.loadFlags = Components.interfaces.nsIRequest.VALIDATE_NEVER | Components.interfaces.nsIRequest.LOAD_BYPASS_CACHE | Components.interfaces.nsIRequest.INHIBIT_PERSISTENT_CACHING;
      var params = {"service": "login", "username": "", "password": "", "token": ""};
      params = JSON.stringify(params);
      current.channel.notificationCallbacks = new badCertListener();
      current.onerror = function (aEvt) {
        println('fixBadCertificate: error (1)');
      }
      try {
        current.send(params);
      } catch(e) {
        println('fixBadCertificate: error (2)');
      }
    }
  } catch (ex) {
    println('fixBadCertificate EXCEPTION');
  }
}

function discoverRedirect(url) {
  var current = HttpRequest.createInstance();

  println("JSON discoverRedirect");
  current.open("POST", url + "/json", false);
  current.channel.loadFlags = Components.interfaces.nsIRequest.VALIDATE_NEVER | Components.interfaces.nsIRequest.LOAD_BYPASS_CACHE | Components.interfaces.nsIRequest.INHIBIT_PERSISTENT_CACHING;
  var params = {"redirect": true, "url": url};
  params = JSON.stringify(params);

  current.channel.notificationCallbacks = new badCertListener();
  current.channel.QueryInterface(Components.interfaces.nsIHttpChannel).redirectionLimit = 0; // will not redirect, so we can retrieve the "Location" header

  var redirectedUrl;
  current.onreadystatechange = function (aEvt) {  
    println("Login-responseheaders:\n" + current.getAllResponseHeaders());
    if (current.readyState == 4) {
      redirectedUrl = current.getResponseHeader('Location');
    }
  }
  try {
    current.send(params);
  } catch(e) {
    println('discoverRedirect: expected error');
  }

  return redirectedUrl;
}

function json_login(url, username, password, token, redirect) {
  var originalUrl = url;
  var redirectedUrl;
  if(redirect) {
    println('discovering redirect');
    redirectedUrl = discoverRedirect(url, username, password, token, redirect);
    println('redirectedUrl: ' + redirectedUrl);
    if(!!redirectedUrl && (redirectedUrl.length > 0)) {
      url = redirectedUrl;
    }
  }

  var current = HttpRequest.createInstance();

  println("JSON login: " + url);
  current.open("POST", url + "/json", false);
  current.channel.loadFlags = Components.interfaces.nsIRequest.VALIDATE_NEVER | Components.interfaces.nsIRequest.LOAD_BYPASS_CACHE | Components.interfaces.nsIRequest.INHIBIT_PERSISTENT_CACHING;

  var params = {"service": "login", "username": username, "password": password, "token": token, "redirect": redirect, "url": url, "originalUrl": originalUrl };
  params = JSON.stringify(params);

  //println("params" + params);
  
  current.channel.notificationCallbacks = new badCertListener();

  var login = false;
  var error = '';
  var data;
  current.onreadystatechange = function (aEvt) {  
    if (current.readyState == 4) {
      if(debug) {
        dumpSecurityInfo(current.channel);
      }
      if(current.responseText == 'true') {
        login = true;
      } else {
        println('===== Login status =====');
        println(current.responseText);
        println('========================');
        try {
          data = JSON.parse(current.responseText);
          if(data.hasOwnProperty('errormessage')) error = data.errormessage
        } catch(e) {}
      }
    }
  }
  current.onerror = function (aEvt) {
    println('login: error (1)');
//     printall(aEvt);
  }
  try {
    current.send(params);
  } catch(e) {
    println('login: error (2)');
//     printall(e);
    return [false, error];
  }
  return [login, error, data, redirectedUrl];
}


function dumpSecurityInfo(channel) {
  try {
    var Ci = Components.interfaces;
    
    // Do we have a valid channel argument?
    if (! channel instanceof  Ci.nsIChannel) {
      dump("No channel available\n");
      return;
    }
    
    var secInfo = channel.securityInfo;
    
    
    // Print general connection security state
    dump("Security Info:\n");
    
    if (secInfo instanceof Ci.nsITransportSecurityInfo) {
      
      secInfo.QueryInterface(Ci.nsITransportSecurityInfo);
      
      dump("\tSecurity state: ");
      
      // Check security state flags
      if ((secInfo.securityState & Ci.nsIWebProgressListener.STATE_IS_SECURE) == Ci.nsIWebProgressListener.STATE_IS_SECURE)
        dump("secure\n");
      
      else if ((secInfo.securityState & Ci.nsIWebProgressListener.STATE_IS_INSECURE) == Ci.nsIWebProgressListener.STATE_IS_INSECURE)
        dump("insecure\n");
        
      else if ((secInfo.securityState & Ci.nsIWebProgressListener.STATE_IS_BROKEN) == Ci.nsIWebProgressListener.STATE_IS_BROKEN)
        dump("unknown\n");
      
      dump("\tSecurity description: " + secInfo.shortSecurityDescription + "\n");
      dump("\tSecurity error message: " + secInfo.errorMessage + "\n");
    }
    else {
      
      dump("\tNo security info available for this channel\n");
    }
    
    // Print SSL certificate details
    if (secInfo instanceof Ci.nsISSLStatusProvider) {
      
      var cert = secInfo.QueryInterface(Ci.nsISSLStatusProvider).
      SSLStatus.QueryInterface(Ci.nsISSLStatus).serverCert;
      
      dump("\nCertificate Status:\n");
      
      var verificationResult = cert.verifyForUsage(Ci.nsIX509Cert.CERT_USAGE_SSLServer);
      dump("\tVerification: ");
      
      switch (verificationResult) {
        case Ci.nsIX509Cert.VERIFIED_OK:
          dump("OK");
          break;
        case Ci.nsIX509Cert.NOT_VERIFIED_UNKNOWN:
          dump("not verfied/unknown");
          break;
        case Ci.nsIX509Cert.CERT_REVOKED:
          dump("revoked");
          break;
        case Ci.nsIX509Cert.CERT_EXPIRED:
          dump("expired");
          break;
        case Ci.nsIX509Cert.CERT_NOT_TRUSTED:
          dump("not trusted");
          break;
        case Ci.nsIX509Cert.ISSUER_NOT_TRUSTED:
          dump("issuer not trusted");
          break;
        case Ci.nsIX509Cert.ISSUER_UNKNOWN:
          dump("issuer unknown");
          break;
        case Ci.nsIX509Cert.INVALID_CA:
          dump("invalid CA");
          break;
        default:
          dump("unexpected failure");
          break;
      }
      dump("\n");
      
      dump("\tCommon name (CN) = " + cert.commonName + "\n");
      dump("\tOrganisation = " + cert.organization + "\n");
      dump("\tIssuer = " + cert.issuerOrganization + "\n");
      dump("\tSHA1 fingerprint = " + cert.sha1Fingerprint + "\n");
      
      var validity = cert.validity.QueryInterface(Ci.nsIX509CertValidity);
      dump("\tValid from " + validity.notBeforeGMT + "\n");
      dump("\tValid until " + validity.notAfterGMT + "\n");
    }
  } catch(err) {
    alert(err);
  }
}
