/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function checkErrorXML(x) {
  return checkXML(x, 1, '').xt;
}

function checkXML(n, h3OK, xt) {  
  if(n.nodeName == "h3") {
    if(h3OK == 0) {
      return {c: h3OK, xt: xt};
    }
    h3OK = 0;
  }
  if(n.nodeName == "#text") {
    var a = strip_beginning_trailing_newlines(n.nodeValue);
    if(xt.indexOf(a) < 0) {
      xt += a + "\n";
    }
  }
  for(var i = 0; i < n.childNodes.length; i++) {
    var r = checkXML(n.childNodes[i], h3OK, xt);
    h3OK = r.c;
    xt = r.xt;
  }
  return {c: h3OK, xt: xt};
}

function getAttribute(xmlDom, param) {
  if(!xmlDom || !xmlDom.attributes) return null;
  var v = xmlDom.attributes.getNamedItem(param);
  if(v) return v.nodeValue;
  return null;
}

function setNotNullAttribute(xmlDom, param, obj, varname) {
  var x = getAttribute(xmlDom, param);
  if(x != null) obj[varname] = x;
  return x;
}

function setTextValue(xmlDom, param, obj, varname) {
  var x = xmlDom.getElementsByTagName(param);
  if(x.length > 0) obj[varname] = x[0].textContent;
}

function raidSetup_empty(ds) {
  for(var i = 0; i < ds.length; i++) {
    if((ds[i] != ' ') && (ds[i] != '\t') && (ds[i] != '\n') && (ds[i] != '\r')) return false;
  }
  return true;
}

function xml_to_raidSetup(xml) {  
  var parser = new DOMParser();
  var xmlDoc = parser.parseFromString(xml, "text/xml");
  var errors = '';
  
  if(!xmlDoc) {
    errors = 'Failed to parse XML data';
  } else if(xmlDoc.getElementsByTagName("parsererror").length) {
    errors = checkErrorXML(xmlDoc.getElementsByTagName("parsererror")[0]);
  }

  return {value: xml, errors: errors};
}


function validate() {
  var xml = document.getElementById("raidsetup").value;
  var ds = xml_to_raidSetup(xml);
  if(ds.errors.length == 0) {
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'validate' + current.get('baseType'), current.object, function(code, data) {
        if(code == 0) {
          var validate = new Array;
          for(var i = 0; i < data.length; i++) {
            if(startswith(data[i].field, 'raidconf')) {
              validate.push(data[i]);
            }
          }
          if(validate.length > 0) {
            var args = new Object();
            args['object'] = this.current;
            args['validate'] = validate;
            var d = new dock.cvDialog("validate.xul", args);
            dock.dialogs.show(d);         
          } else {
            dock.displayInfo('No errors');
          }
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);    
  } else {
    dock.displayInfo(ds.errors);
  }
}

function raidsetup_keypress(event) {
  var text = document.getElementById("raidsetup");
  var value = text.value;
  switch(event.keyCode) {
    case KEY_ESCAPE:
      text.value = current.get("raidconf");
      break;
    default:
      var xml = document.getElementById("raidsetup").value;
      current.set("raidconf", xml);
      var save = document.getElementById('save');
      if(save) {
        save.setAttribute('disabled', false);
        //save.clearAttribute('disabled');
      }
  }
}

function raidsetup_change(event) {
//   var text = document.getElementById("raidsetup");
//   var value = text.value;
  var xml = document.getElementById("raidsetup").value;
  current.set("raidconf", xml);
}

function copy() {
  var category = current.cluster.find(current.get('category'));
  if(category) {
    var old = category.get('raidconf');
    current.set("raidconf", old);
    refresh();
  }
}

function clear() {
  current.set("raidconf", '');
  refresh();
}

function load() {
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
  fp.init(window, "Select a file", nsIFilePicker.modeOpen);
  var path = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
  path.initWithPath(dock.getWorkDirectory() + "raid-setup");
  fp.displayDirectory = path;
  fp.appendFilters(nsIFilePicker.filterAll);
  fp.appendFilter("xml raidsetup files","*.xml");

  var res = fp.show();
  if((res == nsIFilePicker.returnOK)) {
    var thefile = fp.fileURL;
    var path = thefile.path;
    if(startswith(getOS(), "WIN")) {
      if(startswith(path, "/")) {
        path = path.substr(1);
      }
      path = path.replace(/%20/g, " ");
      path = path.replace(/\//g, "\\\\");
    }
    var xml = readfile(path);
    var ds = xml_to_raidSetup(xml);
    if(!!ds && !!ds.value && (ds.errors.length == 0)) {
      current.set('raidconf', ds.value);      
      refresh();
    }
  }
}
