/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

importScripts('logging.js', 'jsontimeouts.js');

var timeout = 45;

var randomStart = 0;
var randomError = null; // ONLY disable for bad connection testing
// var randomError = [[0.001, 1234567, {"errormessage":"hello world"}],
//                    [0.040, -1, {"errormessage":"fake error"}],
//                    [0.080, -2, {"errormessage":"fake timeout"}],
//                   ];

var request = null;
var debug = false;
var url = '';
var wget = false;

function jsoncall(data, call) {
  function ondata() {
    if(debug) {
      println("jsoncall: output");
      println("=========================================================================");
      println(request.responseText);
      println("\n=========================================================================");
    }
    if(request && request.responseText) {  
      if(call != 0) {
        try {
          var data = JSON.parse(request.responseText);
          var code = 0;
          if(request.readyState == 4) {
            if(request.status >= 400) {
              code = request.status;
            } else if(randomError) {
              if(randomStart == 0) {
                randomStart = Date.now();
              } else if(Date.now() - randomStart > 5000) {
                var r = Math.random();
                for(var i = 0; i < randomError.length; i++) {
                  if(r < randomError[i][0]) {
                    code = randomError[i][1];
                    data = randomError[i][2];
                    println("Random communication error, code " + code);
                    printall(data);
                    break;
                  }
                }
              }
            }
          }
          postMessage([code, data]);
        } catch(e) {
          if(debug) {
            printall(e);
            printall(request);
            println(request.responseText);
          }
          var code = 2;
          if((request.readyState == 4) && (request.status >= 400)) code = request.status;
          postMessage([code, 'Error: ' + e]);
        }
      } else {
        postMessage([0, request.responseText]);
      }
    } else {
      postMessage([1, 'Call returned no data']);
    }
    request = null;
  }
  function ontimeout() {
    if(debug) println("jsoncall: timeout");
    postMessage([-2, 'Timeout in XMLHttpRequest']);
    request = null;
  }
  function onerror() {
    if(debug) println("jsoncall: error");
    postMessage([-1, 'Error in XMLHttpRequest']);
    request = null;
  }
  if(!request) {
    request = new XMLHttpRequest();
    request.onload = ondata;  
    request.onerror = onerror;
    if(typeof(request.ontimeout) != 'undefined') { // not for gecko < 12.0
      request.ontimeout = ontimeout;
      initTimeoutMap();
    }
  }
  if(timeoutMap && call) {
    if(timeoutMap.hasOwnProperty(call)) {
      request.timeout = timeoutMap[call] * 1000;
      if(debug) println(['timeout', call, timeoutMap[call]]);
    } else {
      request.timeout = timeout * 1000;
    }
  }
  if(debug) println("Sending to " + url + "json, bytes = " + data.length + "");
  if(wget) {
    request.open("GET", wget, true);
    wget = false;
  } else {  
    request.open("POST", url + 'json', true);
  }
  request.setRequestHeader('Cache-Control', 'no-cache');
  request.setRequestHeader('Content-Type', 'application/json; charset=utf-8');
  request.setRequestHeader('Accept-Encoding', 'gzip');
//   request.setRequestHeader('Content-Encoding', 'gzip'); // doesn't automatically encode
  request.setRequestHeader('Content-Length', data.length);
  request.setRequestHeader('Connection', 'Keep-Alive');
  request.send(data);
}  

onmessage = function(event) {  
  if(event.data && isArray(event.data) && (typeof(event.data[0]) == 'string')) {
    if(event.data[0] == 'ABORT') {
      if(request) request.abort();
      if(debug) println("JsonWorker: abort");
    } else if(event.data[0] == 'DEBUG') {
      debug = (event.data[1] == 'ON');
    } else if(event.data[0] == 'TIMEOUT') {
      timeout = event.data[1] * 1000;
    } else if(event.data[0] == 'URL') {      
      url = event.data[1];
      if(debug) println("JsonWorker: Set url: '" + url + "'");
    } else if(event.data[0] == 'WGET') {
      wget = event.data[1];
      if(debug) println("JsonWorker: wget: '" + wget + "'");
      jsoncall('', 0);
    } else {
      if(debug) {
        println("JsonWorker: call:");
        println(event.data);
        println("\n=========================================================================\n");
      }
      jsoncall(event.data[0], event.data[1]); /* send already json-ed */
    }
  } else {
    printall(event.data);
    postMessage([-1000, 'bad data ' + event.data]);
  }
}
