/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var myDialog;
var current;

var currentDeck;
var DECK_GENERAL = 'general';
var DECK_CLUSTER = 'cluster';
var DECK_NODES = 'compute';
var DECK_DOCKER = 'docker';
var DECK_NETWORK = 'network';
var DECK_SUMMARY = 'summary';
var DECK_DEPLOY = 'deploy';

var timer = 0;
var count = 0;

var advancedMode = false;

var PATTERN_FIRST_LINE = /^##### cmgui #####$/g;
var PATTERN_GENERIC = /^### ([^#]+) ### (STARTED|FINISHED|FAILED)$/g;
var PATTERN_PERC = /^### (\d{1,3})$/g;
var stepsList;
var progressPerc = 0;

var userChoices = {
  clusterName: "default",
  runOnHead: false,
  podNetworkName: "kubepodnet",
  podNetworkBaseAddress: "172.29.0.0",
  podNetworkBits: "16",
  svcNetworkName: "kubeservicenet",
  svcNetworkBaseAddress: "10.150.0.0",
  svcNetworkBits: "16",
  insecureApiPort: "8085",
  secureApiPort: "6443",
  kubeletPort: "10250",
  healthCheckPort: "10251",
  controllerPort: "10252",
  schedulerPort: "10253",
  categories: "default",
  categoriesToDeployDocker: [],
  nodesToDeployDocker: [],
  containerMap: {},
  dryMode: 1,

  addContainers: function(baseType, keys) {
    for (var i = 0; i < keys.length; i++) {
      var key = keys[i];
      if (!(key in this.containerMap)) {
        this.containerMap[key] = new CategoryTree(baseType, key);
      }
    }
  },
  getContainer: function(uniqueKey) {
    return this.containerMap[uniqueKey];
  },
  getContainers: function(baseType) {
    var that = this;
    var containers = Object.keys(that.containerMap).map(function(key) {
      return that.containerMap[key];
    });
    if (typeof(baseType) !== 'undefined') {
      containers = containers.filter(function(item) { return item.baseType === baseType });
    }
    return containers;
  },
  removeContainers: function(keys) {
    var that = this;
    keys.forEach(function(key) {
      delete that.containerMap[key];
    });
  },

  getObjects: function() {
    var nodes = this.getContainers('Node').map(function(nodeContainer) {
      return current.find(nodeContainer.uniqueKey);
    });
    this.getContainers('Category').forEach(function(catContainer) {
      var nodeKeys = current.findNodesIn('category', catContainer.uniqueKey);
      nodes = nodes.concat(nodeKeys.map(function(nodeKey) {
        return current.find(nodeKey);
      }));
    });
    return nodes;
  },

  getNodeObjectsFiltered: function(filterF) {
    var nodes = this.getContainers('Node').filter(filterF).map(function(nodeContainer) {
      return current.find(nodeContainer.uniqueKey);
    });
    this.getContainers('Category').filter(filterF).forEach(function(catContainer) {
      var nodeKeys = current.findNodesIn('category', catContainer.uniqueKey);
      nodes = nodes.concat(nodeKeys.map(function(nodeKey) {
        return current.find(nodeKey);
      }));
    });
    return nodes;
  },

  toXML: function() {
    var xml = toXML(this);
    return vkbeautify.xml(xml);
  }
};

function _get_name() {
  var fieldName = (this.baseType === 'Node') ? 'hostname' : 'name';
  return current.find(this.uniqueKey).get(fieldName);
}
function CategoryTree(baseType, uniqueKey) {
  this.uniqueKey = uniqueKey;
  this.baseType = baseType; // 'Node' or 'Category'
};
CategoryTree.prototype.bootstrap_options = ['auto', 'true', 'false'];
CategoryTree.prototype.getName = function() {
  return _get_name.call(this);
}
CategoryTree.prototype.createTreeItem = function() {
  return {
    key: this.uniqueKey,
    baseType: this.baseType,
    name: this.getName(),
  };
}
CategoryTree.prototype.validate = function() {
  var results = [];
  return results;
}
CategoryTree.prototype.clone = function() {
  var newContainer = new CategoryTree(this.baseType, this.uniqueKey);
  return newContainer;
}

function getNumericIndexFromDeckString(deckString) {
  var deck = document.getElementById('deck');
  for (var i = 0; i < deck.childNodes.length; i++) {
    var kubeindex = deck.childNodes[i].getAttribute('kubeindex');
    if (deckString == kubeindex) {
      return i;
    }
  }
}

function setStage(num) {
  println("Set stage " + num);
  for (var i = 1; i <= 7; i++) {
    var stageid = 'stage' + i;
    if (i == num) {
      setAttr(stageid,'style', 'margin: 0px; padding: 1px; font-size: 12px; color: black; background-color: #7cc200;');
    } else {
      setAttr(stageid, 'style', 'margin: 0px; padding: 1px; font-size: 12px;');
    }
  }
}

function fillListDropdown(popupId, list, excludeFunction) {
  excludeFunction = isundefined(excludeFunction, false);
  var drop = document.getElementById(popupId);
  var popup = drop.firstChild;
  if (!popup) {
    println("!!! ERROR !!!: has no menupopup");
    return;
  }
  while (popup.firstChild) popup.removeChild(popup.firstChild);
  if (excludeFunction && (typeof(excludeFunction) == 'string')) eval('excludeFunction = ' + excludeFunction);
  for (var i = 0; i < list.length; i++) {
    var key = (typeof(list[i].uniqueKey) != 'undefined' ? list[i].uniqueKey : list[i].get('uniqueKey'));
    var name = (typeof(list[i].resolveName) != 'undefined' ? list[i].resolveName() : list[i].name);
    if (excludeFunction && excludeFunction(name)) continue;
    var item = document.createElement('menuitem');
    item.setAttribute('key', key);
    item.setAttribute('label', name);
    popup.appendChild(item);
  }
}

function setDefaultIfPresent(dropDownID, defaultVal) {
  var dropDown = document.getElementById(dropDownID);
  for (var i = 0; i < dropDown.itemCount; i++) {
    if (dropDown.getItemAtIndex(i).getAttribute('label') == defaultVal) {
      dropDown.selectedIndex = i;
      break;
    }
  }
}

function onLoad() {
  dock = getDock();
  println("Kubernetes Cluster Wizard initialized: " + this.name);
  var DIALOG_NUM = /.*\[(\d+)\]/g;
  var index = parseInt(DIALOG_NUM.exec(this.name)[1]);
  myDialog = dock.dialogs.list[index];
  current = myDialog.arguments.cluster;
  currentDeck = DECK_GENERAL;
  generalScreen();

  if(getOSFlavor().match(/CENTOS7u0|CENTOS7u1|CENTOS7u2|CENTOS7u3|RedHat7u1|RedHat7u2|RedHat7u3|SL7u1|SL7u2|SL7u3|SLES12/i)===null) {
    document.getElementById('unsupportedOSFlavorLabel').textContent = getOSFlavor() + " distribution is not supported.";
    unhide('unsupportedOSFlavor');
  }

  if (advancedMode===true) {
    setAdvancedMode(1);
  } else {
    setAdvancedMode(0);
  }

  if (userChoices.runOnHead===true) {
    setRunOnHead(1);
  } else {
    setRunOnHead(0);
  }

  var cat = current.find('Category', 'default');
  if(cat) {
    var keys = [cat.get('uniqueKey')];
    userChoices.addContainers('Category', keys);
  }
}

function getRolesForCategory(categoryName) {
  var roleTypesAssignedToCategory = []
  var category = current.find('Category', categoryName);
  
  if(!category) {
    return [];
  }
  
  var categoryRoles = category.get('roles');

  for(var i = 0; i < categoryRoles.length; i++) {
    roleTypesAssignedToCategory.push(categoryRoles[i].childType);
  }

  return roleTypesAssignedToCategory;
}

function getRolesForNode(nodeName) {
  var roleTypesAssignedToCategory = []
  var node = current.find('Node', nodeName);
  
  if(!node) {
    return [];
  }
  
  var categoryRoles = node.find('roles');

  for(var i = 0; i < categoryRoles.length; i++) {
    roleTypesAssignedToCategory.push(categoryRoles[i].childType);
  }

  return roleTypesAssignedToCategory;
}

function isDockerAssignedToCategory(categoryName) {
  var roles = getRolesForCategory(categoryName);

  var found = false;
  for(var i = 0; i < roles.length; i++) {
    if (roles[i] === "DockerHostRole") {
      found = true;
      break;
    }
  }
  
  return found;
}

function isDockerAssignedToNode(nodeName) {
  var roles = getRolesForNode(nodeName);

  var found = false;
  for(var i = 0; i < roles.length; i++) {
    if (roles[i] === "DockerHostRole") {
      found = true;
      break;
    }
  }
  return found;
}

function getCategoriesWithoutDockerAssigned() {
  var categoriesWithoutDockerAssigned = userChoices.getContainers('Category').filter(
    function(category) {
      println(category.getName() + " " + isDockerAssignedToCategory(category.getName()));
      return !isDockerAssignedToCategory(category.getName());
    });

  return categoriesWithoutDockerAssigned;
}

function getNodesWithoutDockerAssigned(nodes) {
  var nodesWithoutDockerAssigned = nodes.filter(
    function(node) {
      return !isDockerAssignedToNode(node.name);
    });

  return nodesWithoutDockerAssigned;
}

function deploymentValid() {
  var categoriesFlag = userChoices.categoriesToDeployDocker.length === userChoices.getContainers('Category').length;

  var masterNodes = current.find('MasterNode');
  var mastersFlag = true;
  if (userChoices.runOnHead)
    mastersFlag = userChoices.nodesToDeployDocker.length === masterNodes.length;

  return categoriesFlag && mastersFlag;
}

function haveToDeployDocker() {
  userChoices.categoriesToDeployDocker = getCategoriesWithoutDockerAssigned();

  var hasToDeployDockerOnMasters = false;
  if (userChoices.runOnHead) {
    var masterNodes = current.find('MasterNode');
    userChoices.nodesToDeployDocker = getNodesWithoutDockerAssigned(masterNodes);

    hasToDeployDockerOnMasters = userChoices.nodesToDeployDocker.length > 0;
  }

  return userChoices.categoriesToDeployDocker.length != 0;
}

function cleanDeploymentErrorSection() {
  setAttr('next', 'disabled', 'false');
  var errorSection = document.getElementById("errorSection");
   while(errorSection.hasChildNodes()){
    errorSection.removeChild(errorSection.firstChild);
  }
}

function checkDeploymentValidity() {
  cleanDeploymentErrorSection();
  if (haveToDeployDocker() && !deploymentValid()) {
    buildDeploymentErrorSection();
  }
}

function error(parent, title, text) {
  var hbox = document.createElement('hbox');
  hbox.setAttribute("style", "width: 150px;");
  hbox.setAttribute("class", "warning");

  var image = document.createElement('image');
  image.setAttribute("style", "margin-left: 5px;");
  image.setAttribute("src", "chrome://cmgui/content/icons/error.png");

  var label = document.createElement("label");
  label.setAttribute("value", title);  
  
  hbox.appendChild(image);
  hbox.appendChild(label);

  parent.appendChild(hbox);
}

function buildDeploymentErrorSection() {
  var errorSection = document.getElementById("errorSection");
  
  var header = document.createElement("shadowlabel");
  header.setAttribute("class", "plain");
  header.setAttribute("value", "Detected issues:");
  header.setAttribute("style", "font-size: 14px;");

  errorSection.appendChild(header);
  error(errorSection, "Some selected categories/master nodes have already docker setup.", "");

  setAttr('next', 'disabled', 'true');
}

function next() {
  println('next from ' + currentDeck);

  switch (currentDeck) {
    case DECK_GENERAL:
      if(!checkAndSaveGeneralSettings()) return;
      if(advancedMode) {
        currentDeck = DECK_CLUSTER;
        clusterScreen();
      } else {
        currentDeck = DECK_SUMMARY;
        summaryScreen();
      }
      break;
    case DECK_CLUSTER:
      if(!checkAndSaveClusterSettings()) return;
      currentDeck = DECK_NODES;
      computeScreen();
      break;
    case DECK_NODES:
      if(!checkAndSaveComputeSettings()) return;

      if (haveToDeployDocker()) {
        currentDeck = DECK_DOCKER;
        dockerScreen();
      } else {
        currentDeck = DECK_NETWORK;
        networkScreen();
      }
      break;
    case DECK_DOCKER:
      currentDeck = DECK_NETWORK;

      if (dockerDeployment.storageBackend !== undefined) {
        dockerDeployment.storageBackend.device.buildFromUI(document);
      }
    
      networkScreen();
      break;
    case DECK_NETWORK:
      if(!checkAndSaveNetworkSettings()) return;
      currentDeck = DECK_SUMMARY;
      summaryScreen();
      break;
    case DECK_SUMMARY:
      currentDeck = DECK_DEPLOY;
      userChoices.dryMode = dryMode();
      deploy();
      break;
    case DECK_DEPLOY:
      userChoices.dryMode = 0;
      currentDeck = DECK_DEPLOY;
      deploy();
      break;
  }

  configureButtons();
  refresh();
}

function prev() {
  println('prev from ' + currentDeck);
  cleanDeploymentErrorSection();

  switch (currentDeck) {
    case DECK_CLUSTER:
      currentDeck = DECK_GENERAL;
      generalScreen();
      break;
    case DECK_NODES:
      currentDeck = DECK_CLUSTER;
      clusterScreen();
      break;
    case DECK_DOCKER:
      currentDeck = DECK_NODES;
      computeScreen();
      break;
    case DECK_NETWORK:
      if (haveToDeployDocker()) {
          currentDeck = DECK_DOCKER;
          dockerScreen();
      } else {
        currentDeck = DECK_NODES;
        computeScreen();
      }
      break;
  case DECK_SUMMARY:
      document.getElementById('next').setAttribute('label', 'Next');
      document.getElementById('next').setAttribute('accesskey', 'N');
      if(advancedMode) {
        currentDeck = DECK_NETWORK;
        document.getElementById('next').setAttribute('disabled', false);
        networkScreen();
      } else {
        currentDeck = DECK_GENERAL;
        document.getElementById('next').setAttribute('enabled', false);
        generalScreen();
      }
      break;
    case DECK_DEPLOY:
      document.getElementById('next').setAttribute('label', 'Deploy');
      document.getElementById('next').setAttribute('accesskey', 'D');
      currentDeck = DECK_SUMMARY;
      summaryScreen();
      break;
  }

  configureButtons();
  refresh();
}

function cancel() {
  var args = { question: 'Are you sure you want to quit the wizard?' };
  var d = new cvDialog("chrome://cmgui/content/question.xul", args);
  d.onOk = function() {
    close();
  };
  dock.dialogs.showDialog(d);
}

function checkAndSaveGeneralSettings() {
  advancedMode = (document.getElementById('isAdvancedWizardMode').getAttribute('on') === 'true') ? true : false;
  return true;
}

function checkAndSaveClusterSettings() {
  var clusterName = getAttr('clusterName', 'value');
  if (clusterName.length === 0) {
    dock.displayError("The 'Cluster Name' field must contain a astring");
    return false;
  }
  userChoices.clusterName = clusterName;
  return true;
}

function checkAndSaveComputeSettings() {
  userChoices.categories = userChoices.getContainers('Category').map(
    function(cat) {
      return cat.getName();
    }
  ).join(',');
  return true;
}

String.prototype.isEmpty = function() {
    return (this.length === 0 || !this.trim());
};

function checkAndSaveNetworkSettings() {
  var podNetworkName = getAttr('podNetworkName', 'value');
  var podNetworkBaseAddress = getAttr('podNetworkBaseAddress', 'value');
  var podNetworkBits = getAttr('podNetworkBits', 'value');
  var svcNetworkName = getAttr('svcNetworkName', 'value');
  var svcNetworkBaseAddress = getAttr('svcNetworkBaseAddress', 'value');
  var svcNetworkBits = getAttr('svcNetworkBits', 'value');

  userChoices.podNetworkName = podNetworkName;
  userChoices.podNetworkBaseAddress = podNetworkBaseAddress;
  userChoices.podNetworkBits = podNetworkBits;
  userChoices.svcNetworkName = svcNetworkName;
  userChoices.svcNetworkBaseAddress = svcNetworkBaseAddress;
  userChoices.svcNetworkBits = svcNetworkBits;

  userChoices.insecureApiPort = getAttr('insecureApiPort', 'value');
  userChoices.secureApiPort = getAttr('secureApiPort', 'value');
  userChoices.kubeletPort = getAttr('kubeletPort', 'value');
  userChoices.healthCheckPort = getAttr('healthCheckPort', 'value');
  userChoices.controllerPort = getAttr('controllerPort', 'value');
  userChoices.schedulerPort = getAttr('schedulerPort', 'value');

  if (podNetworkName.isEmpty() || podNetworkBaseAddress.isEmpty() || podNetworkBits.isEmpty() ||
      svcNetworkName.isEmpty() || svcNetworkBaseAddress.isEmpty() || svcNetworkBits.isEmpty()) {
    alert('Empty fields are not allowed. Please review the networking parameters.');
    return false;
  }
  return true;
}

function configureButtons() {
  var pages = document.getElementById('deck').childNodes.length - 1;
  var index = parseInt(document.getElementById('deck').selectedIndex);
  println("current deck: " + currentDeck + ", pages: " + pages + ", index: " + index);
  index = getNumericIndexFromDeckString(currentDeck);
  println('going to ' + currentDeck + ' (' + index + ')');

  document.getElementById('deck').selectedIndex = index;
  document.getElementById('prev').setAttribute('hidden', index === 0);
  document.getElementById('prev').setAttribute('disabled', index === 0);
  document.getElementById('next').setAttribute('hidden', index === pages);
  document.getElementById('finish').setAttribute('hidden', index < pages);
  document.getElementById('finish').setAttribute('disabled', true);

  var isNodeSettings = (index === 2);
  setAttr('selectCategoriesButton', 'hidden', !isNodeSettings);
  setAttr('selectRemoveButton', 'hidden', !isNodeSettings);

  setAttr('selectCategoriesButton', 'oncommand', "selectItems('Category');");
  setAttr('selectRemoveButton', 'oncommand', "removeItems('Category');");
}

function generalScreen() {
  setStage(1);
}

function clusterScreen() {
  setStage(2);
  setAttr('clusterName', 'value', userChoices.clusterName);
}

function computeScreen() {
  setStage(3);
}

function dockerScreen() {
  setStage(4);

  if (userChoices.categoriesToDeployDocker.length !== 0) {

    setAttr("docker_categories", "hidden", false);
    
    var categoriesToDeployDocker = userChoices.categoriesToDeployDocker
        .map(function(category) {
          return category.getName();
        });
    
    var categoryLabel = "Category";
    if (userChoices.categoriesToDeployDocker.length > 1)
      categoryLabel = "Categories";
    
    setAttr("docker_categories", "value", categoryLabel + ": " + categoriesToDeployDocker.join(", "));
  }

  if (userChoices.nodesToDeployDocker.length !== 0) {
    var nodesToDeployDocker = userChoices.nodesToDeployDocker
        .map(function(node) {
          return node.shortHostname();
        });
    
    var nodeLabel = "Master";
    if (userChoices.nodesToDeployDocker.length > 1)
      nodeLabel = "Masters";
    
    setAttr("docker_headnodes", "hidden", false);
    setAttr("docker_headnodes", "value", nodeLabel + ": " + nodesToDeployDocker.join(", "));
  }
  
}

function validNetworks() {
  var networks = current.find('Network');
  return networks.filter(function(network) {
    return network.get('type') != 'GLOBAL';
  });
}

function updateNetworkInformation(networkId, baseAddressId, networkBitsId, networkName, defaultBaseAddress, defaultNetworkBits) {
  setAttr(networkId, 'value', networkName);

  var baseAddress = defaultBaseAddress;
  var networkBits = defaultNetworkBits;
  var networkReadOnly = false;
  
  var network = current.find('Network', networkName);
  
  if (network && networkName !== '') {
    baseAddress = network.get('baseAddress');
    networkBits = network.get('netmaskBits');

    networkReadOnly = true;
  }

  var baseAddressElement = document.getElementById(baseAddressId);
  var networkBitsElement = document.getElementById(networkBitsId);

  baseAddressElement.value = baseAddress;
  networkBitsElement.value = networkBits;
  
  if (networkReadOnly) {
    baseAddressElement.setAttribute('readonly', networkReadOnly);
    networkBitsElement.setAttribute('readonly', networkReadOnly);
  }
  else {
    baseAddressElement.removeAttribute('readonly');
    networkBitsElement.removeAttribute('readonly');
  }
  
}

function updatePodNetworkInformation(networkName) {
  updateNetworkInformation('podNetworkName', 'podNetworkBaseAddress', 'podNetworkBits',
                           networkName, userChoices.podNetworkBaseAddress, userChoices.podNetworkBits);
}

function updateServiceNetworkInformation(networkName) {
  updateNetworkInformation('svcNetworkName', 'svcNetworkBaseAddress', 'svcNetworkBits',
                           networkName, userChoices.svcNetworkBaseAddress, userChoices.svcNetworkBits);
}

function networkScreen() {
  setStage(5);

  updatePodNetworkInformation(userChoices.podNetworkName);
  updateServiceNetworkInformation(userChoices.svcNetworkName);

  setAttr('insecureApiPort', 'value', userChoices.insecureApiPort);
  setAttr('secureApiPort', 'value', userChoices.secureApiPort);
  setAttr('kubeletPort', 'value', userChoices.kubeletPort);
  setAttr('healthCheckPort', 'value', userChoices.healthCheckPort);
  setAttr('controllerPort', 'value', userChoices.controllerPort);
  setAttr('schedulerPort', 'value', userChoices.schedulerPort);
}

function dockerSummary() {
  var box = document.getElementById('docker-summary-rows');
  while (box.firstChild) box.removeChild(box.firstChild);

  var registries = dockerDeployment.registries.join(", ");
  appendChoice(box, 'Docker Registries: ', registries);

  if (dockerDeployment.storageBackend === undefined)
    appendChoice(box, 'Storage Backend: ', 'Default');
  else {
    appendChoice(box, 'Storage Backend: ', dockerDeployment.storageBackend.name());
    var f = function(fieldName, fieldValue) {
      appendChoice(box, fieldName, fieldValue);
    }
    
    dockerDeployment.storageBackend.device.summary(f);
  }
}

function summaryScreen() {
  setStage(6);
  var good = 1;
  dockerSummary();

  document.getElementById('next').setAttribute('label', 'Deploy');
  document.getElementById('next').setAttribute('accesskey', 'D');
  
  var box = document.getElementById('summary-rows');
  while (box.firstChild) box.removeChild(box.firstChild);
  appendChoice(box, 'Cluster name: ', userChoices.clusterName);
  appendChoice(box, 'Run pods on head: ', (userChoices.runOnHead?"yes":"no"));

  var cats = userChoices.getContainers('Category').map(function(cat) {
    return cat.getName();
  }).join(', ');
  if (cats === '') {
    var warning = "No categories are defined to run pods. " +
                  "While it is possible to deploy a Kubernetes cluster without them, " +
                  "it won't be possible to run any pods on it. " +
                  "Please add node categories by going back to the 'Nodes Settings' section.\n\n";
    setAttr('summaryMessage', 'value', warning + getAttr('summaryMessage', 'value'));
    setAttr('summaryMessage', 'style', 'color: red; background-color: transparent !important; font-size: 14px;');
    good = 0;
  } else {
    appendChoice(box, 'Node categories: ', cats);
  }
  appendChoice(box, 'Pod network name: ', userChoices.podNetworkName);
  appendChoice(box, 'Pod network base address: ', userChoices.podNetworkBaseAddress);
  appendChoice(box, 'Pod network bits: ', userChoices.podNetworkBits);
  appendChoice(box, 'Service network name: ', userChoices.svcNetworkName);
  appendChoice(box, 'Service network base address: ', userChoices.svcNetworkBaseAddress);
  appendChoice(box, 'Service network bits: ', userChoices.svcNetworkBits);
  appendChoice(box, 'Insecure API port: ', userChoices.insecureApiPort);
  appendChoice(box, 'Secure API port: ', userChoices.secureApiPort);
  appendChoice(box, 'Kubelet port: ', userChoices.kubeletPort);
  appendChoice(box, 'Health check port: ', userChoices.healthCheckPort);
  appendChoice(box, 'Controller port: ', userChoices.controllerPort);
  appendChoice(box, 'Scheduler port: ', userChoices.schedulerPort);

  if(good==1) {
    var msg = "By pressing 'Next' a temporary XML file will be created with " +
              "the selected configuration and Kubernetes cluster will be deployed " +
              "using this XML file.\n\n" + 
              "Alternatively, clicking the 'Save Configuration' button will produce " +
              "an XML configuration file which can be further customized, if needed, " +
              "and then used as the input configuration file for the cm-kubernetes-setup " +
              "command line utility.\n\n" +
              "Press 'Show Configuration' to get the Kubernetes deployment configuration as XML.";
    setAttr('summaryMessage', 'value', msg);
    setAttr('summaryMessage', 'style', 'background-color: transparent !important; font-size: 14px;');
  } else {
    document.getElementById('next').setAttribute('disabled', true);
  }
}

function appendChoice(rows, fieldName, fieldValue) {
  var row = document.createElement("row");
  row.setAttribute('align', 'center');
  rows.appendChild(row);

  var vbox1 = document.createElement("vbox");
  vbox1.setAttribute('pack', 'end');
  row.appendChild(vbox1);

  var field = document.createElement('shadowlabel');
  field.setAttribute('class', 'staticdataheader');
  field.setAttribute('value', fieldName);
  vbox1.appendChild(field);

  var vbox2 = document.createElement("hbox");
  row.appendChild(vbox2);
  var label = document.createElement('label');
  label.setAttribute('value', fieldValue);
  vbox2.appendChild(label);
}

function refresh() {
  tree = document.getElementById('runPodsTree');
  data = userChoices.getContainers().map(function(container) {
    return container.createTreeItem();
  });
  loadTable();


  if (currentDeck === DECK_NODES)
    checkDeploymentValidity();
  return true;
}

function showConfiguration() {
  var xmlConfig = "";
  var xmlDockerDeployment = "";
  
  try {
    userChoices.dryMode = dryMode();
    xmlConfig = userChoices.toXML();

    if (haveToDeployDocker())
      xmlDockerDeployment = dockerDeployment.serialize();
  } catch (err) {
    println('exception ' + err + ' while generation xml file');
    dock.displayError('Error while generating XML file');
    return false;
  }

  var args = {
    title: 'KubeCluster configuration',
    style: 'background-color: white !important;',
    text: xmlConfig + '\n\n' + xmlDockerDeployment
  };
  var d = new cvDialog(dock.myVersion + "showtext.xul", args);
  dock.dialogs.showDialog(d);
}

function saveConfiguration() {
  var xmlConfig = "";
  var xmlDockerDeployment = "";
  try {
    xmlConfig = userChoices.toXML();
    if (haveToDeployDocker())
      xmlDockerDeployment = dockerDeployment.serialize();
  } catch (err) {
    println('exception ' + err + ' while generation xml file');
    dock.displayError('Error while generating XML file');
    return false;
  }

  var saved = true;
  var kubernetesFile = selectFile("Save Kubernetes cluster setup as");
  if (kubernetesFile !== '')
    saved = savefile(kubernetesFile, xmlConfig) && saved;

  if (haveToDeployDocker()) {
    var dockerFile = selectFile("Save docker deployment as");
    if (dockerFile !== '')
      saved = savefile(dockerFile, xmlDockerDeployment) && saved;
  }
  
  if(saved) close();
  return saved;
}

function selectFile(title) {
  var file = '';
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
  fp.init(window, title, nsIFilePicker.modeSave);
  var path = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
  path.initWithPath(dock.BASE_PATH);
  fp.displayDirectory = path;
  fp.appendFilter("Kubernetes setups", "*.xml");
  var res = fp.show();
  if ((res == nsIFilePicker.returnOK) || (res == nsIFilePicker.returnReplace)) {
    file = fp.file.path;
    if (!endswith(file, '.xml')) file += '.xml';
  }
  
  return file;
}

function deploy() {
  setStage(7);

  var xmlConfig = "";
  var dockerDeploymentConfig = "";
  try {
    xmlConfig = userChoices.toXML();
    if (haveToDeployDocker())
      dockerDeploymentConfig = dockerDeployment.serialize();
    
  } catch (err) {
    println('exception ' + err + ' while generation xml file');
    dock.displayError('Error while generating XML file');
    return false;
  }
  var text = "\nKubernetes cluster configuration has been written. A Kubernetes cluster is being installed. Installation progress is shown below.";
  setAttr('deployMessage', 'value', text);

  if (userChoices.dryMode == 1) {
    setAttr('run-mode', 'hidden', 'false');
  }
  else {
    setAttr('run-mode', 'hidden', 'true');
  }

  var id = 0;
  overviewBar('progressBar', 0, 100);
  stepsList = new Array();

  var args = [xmlConfig, dockerDeploymentConfig];

  dock.workerpool.getWorker(current.actualurl).call('cmkube', 'executeKubeSetup', args, function(code, data) {
    if(code == 0) {
      id = data.setupExecutionID;
      timer = window.setTimeout('checkStatus(' + id +');', 1000);
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);
}

function finish() {
  close();
}

function checkStatus(id) {
  timer = 0;
  count++;
  if (count > 2000) {
    return;
  }

  dock.workerpool.getWorker(current.actualurl).call('cmkube', 'getKubeSetupExecution', [id], function(code, data) {
    if(code !== 0) {
      dock.jsonCallError(this, code, data);
      return;
    }
    if (!data) {
      dock.displayError('A Kubernetes cluster is already being installed.');
      return;
    }

    var currentProgress = "";
    var stepsCounter = 0;
    for (var i = 0; i < data.datacout.length; i++) {
      var matched = false;

      if (data.datacout[i].match(PATTERN_FIRST_LINE)) {
        document.getElementById('deployProgress').removeAttribute('hidden');
        matched = true;
      }
      var m;
      do {
        m = PATTERN_GENERIC.exec(data.datacout[i]);
        if (m) {
          matched = true;
          stepsList[stepsCounter++] = m;
        }
      } while (m);
      var m2 = PATTERN_PERC.exec(data.datacout[i]);
      if (m2) {
        matched = true;
        progressPerc = m2[1];
      }

      if (!matched) {
        currentProgress += data.datacout[i];
        currentProgress += "\n";
      }
    }
    var deployStepsDisplay = new ProgressStepDisplayer('deploySteps');
    deployStepsDisplay.clearAll();

    var mergedStepsList = new Array();
    for (var i = 0; i < stepsList.length; i++) {
      var item = stepsList[i];
      if (stepsList[i-1]) {
        var previousItem = stepsList[i-1];
        if (item[1] == previousItem[1]) {
          mergedStepsList[mergedStepsList.length-1] = item;
        } else {
          mergedStepsList.push(item);
        }
      } else {
        mergedStepsList.push(item);
      }
    }

    for (var i = 0; i < mergedStepsList.length; i++) {
      deployStepsDisplay.add(mergedStepsList[i][2], mergedStepsList[i][1]);
    }

    overviewBar('progressBar', progressPerc, 100);

    var deployOutput = document.getElementById('deployOutput');
    deployOutput.value = currentProgress;
    var ti = document.getAnonymousNodes(deployOutput)[0].childNodes[0];
    ti.scrollTop = ti.scrollHeight;

    if (data.finishedAt == 0) {
      document.getElementById('canc').setAttribute('disabled', true);
      document.getElementById('prev').setAttribute('disabled', true);
      timer = window.setTimeout('checkStatus(' + id +');', 3000);
    } else if (data.successful) {
      overviewBar('progressBar', 100, 100);

      if (userChoices.dryMode == 1) {
        document.getElementById('next').setAttribute('disabled', false);
        document.getElementById('next').setAttribute('hidden', false);
        document.getElementById('next').setAttribute('label', 'Deploy');
        document.getElementById('next').setAttribute('accesskey', 'D');
        document.getElementById('prev').setAttribute('disabled', false);
      }

      document.getElementById('finish').setAttribute('disabled', false);
    } else { // failed
      document.getElementById('prev').setAttribute('disabled', false);
      document.getElementById('finish').setAttribute('disabled', false);
    }
  },
  current);
}

function dryMode() {
  var is_drymode = document.getElementById('isDryMode').getAttribute('checked') || false;
  return is_drymode ? 1 : 0;
}

function setAdvancedMode(choice) {
  setSimpleRadio(choice, ['isBasicWizardDryMode', 'isAdvancedWizardMode']);
}

function setRunOnHead(choice) {
  setSimpleRadio(choice, ['isNotRunOnHead', 'isRunOnHead']);
  userChoices.runOnHead = choice;
}

function selectItems(baseType) {
  var args = {
    cluster: current,
    baseType: baseType,
    currentnodes: []
  };
  var d = new dock.cvDialog('chrome://cmgui/content/kube/selectitems.xul', args);
  d.onOk = function(keys) {
    userChoices.addContainers(baseType, keys);
    refresh();
  };
  dock.dialogs.showDialog(d);
}

function removeItems() {
  var keys = selected('runPodsList');
  userChoices.removeContainers(keys);
  refresh();
}


