/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var object;
var old;
var current;
var myDialog;
var dock;
var last = false;

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  object = myDialog.arguments.object;
  current = myDialog.arguments.subobject;
  old = current.old(false, true);
  refresh("true");
}

function refresh(onload) {
  check('debug');
  check('enableSelinux');
  check('iptables');
  check('userNamespaceRemap');
  text('bridge');
  text('mtu', '^[0-9]+$');
  text('spool');
  text('tmpDir');
  text('containerdSocket');
  stringlist('apiSockets', '');
  stringlist('defaultUlimits', '');
  stringlist('insecureRegistries', '');
  stringlist('options', '');

  refreshDeviceMapper(onload);
  refreshTls(onload);
}

function addBackend(backends) {
  backends.push(new cvDockerStorageDeviceMapperBackend());
  current.set('storageBackends', backends);
  current.set('modified', true);
}
function delBackend(backends) {
  for(var i=0; i<backends.length; i++) {
    if(backends[i].childType == "DockerStorageDeviceMapperBackend") {
      delete backends[i];
      current.set('storageBackends', backends);
      current.set('modified', true);
      return;
    }
  }
}

function updateLastBackendState() {
  var obj = document.getElementById('deviceMapperEnabled');
  last = obj.checked;
}

function refreshDeviceMapper(onload) {
  var backends = current.get('storageBackends');
  var found = false;
  for(var i=0; i<backends.length; i++) {
    if(backends[i].childType == "DockerStorageDeviceMapperBackend") {
      found = true;
      break;
    }
  }

  var enabled = false;
  var obj = document.getElementById('deviceMapperEnabled');
  if(onload) {
    if(obj.checked && !found) {
      addBackend(backends);
      enabled = true;
    } else {
      setAttr('deviceMapperEnabled', 'checked', found);
      enabled = found;
    }
  } else {
    if(obj.checked && !last) { // just checked
      addBackend(backends);
    } else if(!obj.checked && last) { // just unchecked
      delBackend(backends);
    }
    enabled = obj.checked;
  }
  setAttr('editDeviceMapperButton', 'hidden', !enabled);
}

function refreshTls(onload) {
  check('enableTls');
  check('verifyTls');
  text('tlsCa');
  text('tlsCertificate');
  text('tlsKey');
  text('certificatesPath');

  var enabled = 'true';
  if(onload) {
    enabled=current.enableTls;
  } else {
    var obj = document.getElementById('enableTls');
    printall(obj);
    enabled = !obj.checked;
  }
  setAttr('certificatesPath', 'disabled', !eval(enabled));
  setAttr('tlsCa', 'disabled', !eval(enabled));
  setAttr('tlsCertificate', 'disabled', !eval(enabled));
  setAttr('tlsKey', 'disabled', !eval(enabled));
  setAttr('verifyTls', 'disabled', !eval(enabled));
}

function editDeviceMapperBackend() {
  var xulfilename = dock.myVersion + 'kube/dockerdevicemapper.xul';
  var args = new Object();
  args['object'] = current;
  var backends = current.get('storageBackends');
  for(var i=0; i<backends.length; i++) {
    if(backends[i].childType == "DockerStorageDeviceMapperBackend") {
      args['subobject'] = new dock.cvObject(current.cluster, backends[i]);
      break;
    }
  }
  var d = new dock.cvDialog(xulfilename, args);
  dock.dialogs.showDialog(d);
}

function ok() {
  if(current.get('modified')) object.set('modified', true);
  close();
  myDialog.onOk();
}

function cancel() {
  if(!window.closed) {
    current.old(old);
    close();
  }
}

