'use strict';

var dock;
var dockerDeployment;

var DeviceMapperLoopbackDevice = (function() {
  function DeviceMapperLoopbackDevice(data_size, metadata_size) {
    if (data_size === undefined)
      this.data_size = '100GB';
    else
      this.data_size = data_size;

    if (metadata_size === undefined)
      this.metadata_size = '2GB';
    else
      this.metadata_size = data_size;
  }

  DeviceMapperLoopbackDevice.prototype.buildFromUI = function(document) {
    this.data_size = document.getElementById("loopbackDataSize").value;
    this.metadata_size = document.getElementById("loopbackMetaDataSize").value;
  }

  DeviceMapperLoopbackDevice.prototype.summary = function(summaryFunction) {
    summaryFunction('Type: ', 'Loopback Device');
    summaryFunction('Data Device: ', this.data_size);
    summaryFunction('Metadata Device: ', this.metadata_size);
  }

  DeviceMapperLoopbackDevice.prototype.serialize = function(document, root) {
    var loopbackDeviceFragment = document.createElement('loopback_device');

    var dataSizeElement = document.createElement('data_size');
    var dataSizeTextNode = document.createTextNode(this.data_size);

    dataSizeElement.appendChild(dataSizeTextNode);
    loopbackDeviceFragment.appendChild(dataSizeElement);

    var metadataSizeElement = document.createElement('metadata_size');
    var metadataSizeTextNode = document.createTextNode(this.metadata_size);

    metadataSizeElement.appendChild(metadataSizeTextNode);
    loopbackDeviceFragment.appendChild(metadataSizeElement);

    root.appendChild(loopbackDeviceFragment);
  }

  return DeviceMapperLoopbackDevice;

})();

var DeviceMapperBlockDevice = (function() {
  function DeviceMapperBlockDevice(data_device) {
    this.data_device = data_device;
  }

  DeviceMapperBlockDevice.prototype.buildFromUI = function(document) {
    this.data_device = document.getElementById("blockDeviceName").value;
  }

  DeviceMapperBlockDevice.prototype.summary = function(summaryFunction) {
    summaryFunction('Type: ', 'Loopback Device');
    summaryFunction('Data Device: ', this.data_device);
  }

  DeviceMapperBlockDevice.prototype.serialize = function(document, root) {
    var blockDeviceFragment = document.createElement('block_device');

    var dataDeviceElement = document.createElement('data_device');
    var dataDeviceTextNode = document.createTextNode(this.data_device);

    dataDeviceElement.appendChild(dataDeviceTextNode);
    blockDeviceFragment.appendChild(dataDeviceElement);

    root.appendChild(blockDeviceFragment);
  }
  
  return DeviceMapperBlockDevice;

})();

var DeviceMapperStorageBackend = (function() {
  function DeviceMapperStorageBackend() {
    this.device = new DeviceMapperLoopbackDevice();
  }

  DeviceMapperStorageBackend.prototype.name = function() {
    return 'devicemapper';
  };

  DeviceMapperStorageBackend.prototype.serialize = function(document, root) {
    var storageBackendFragment = document.createElement('storage_backend');
    storageBackendFragment.setAttribute("name", this.name());

    this.device.serialize(document, storageBackendFragment);

    root.appendChild(storageBackendFragment);
  };

  return DeviceMapperStorageBackend;

})();


var dockerDeployment = {
  registries: [],
  storageBackend: new DeviceMapperStorageBackend(),

  addRegistry: function(name) {
    var registriesSet = new Set(this.registries);

    if (!registriesSet.has(name))
      this.registries.push(name);
  },

  removeRegistry: function(name) {
    var index = this.registries.findIndex(function(registry) {
      return registry == name;
    });

    if (index >= 0) {
      this.registries.splice(index, 1);
    }
  },

  editRegistry: function(at, name) {
    if (index < 0 || index >= this.registries.length)
      throw 'Index out of bound exception: editRegistry';
    
    this.registries[at] = name;
  },

  serialize: function() {
    var doc = document.implementation.createDocument('', 'dockerconfig', null);
    this.serializeRegistries(doc, doc.documentElement);

    if (this.storageBackend !== undefined)
      this.storageBackend.serialize(doc, doc.documentElement);
    
    var xml = new XMLSerializer().serializeToString(doc);
    return vkbeautify.xml(xml);
  },

  serializeRegistries: function(doc, rootFragment) {
    var registriesFragment = doc.createElement('registries');
    rootFragment.appendChild(registriesFragment);

    for(var registry of this.registries) {
      var registryFragment = doc.createElement('registry');
      registryFragment.setAttribute("name", registry);
      registriesFragment.appendChild(registryFragment);
    }
  }
};

function createTreeItem(one) {
  var item = new Object();
  item['key'] = one;
  item['name'] = one;

  return item;
}

function refreshRegistries() {
  tree = document.getElementById('registriestree');
  data = [];

  println(dockerDeployment.registries.length);
  for (var i = 0; i < dockerDeployment.registries.length; i++) {
    var registry = dockerDeployment.registries[i];
    data.push(createTreeItem(registry));
  }
  
  loadTable();
  return true;
}

function addRegistry() {
  var args = {};
  args['docker'] = dockerDeployment;
  args['object'] = 0;
  var d = new dock.cvDialog("chrome://cmgui/content/kube/docker/editregistry.xul", args);
  d.onOk = function() {
    refreshRegistries();
  }
  dock.dialogs.showDialog(d);
}

function removeRegistry() {
  var registries = selected('registries');
  for (var registryTobeDeleted of registries) {
    dockerDeployment.removeRegistry(registryTobeDeleted);
  }
  refreshRegistries();
}

function selectDefaultStorageBackend() {
  var deviceMapperSection = document.getElementById('deviceMapperSection');
  deviceMapperSection.setAttribute("hidden", true);

  dockerDeployment.storageBackend = undefined;
}

function selectDeviceMapperStorageBackend() {
  var deviceMapperSection = document.getElementById('deviceMapperSection');
  deviceMapperSection.setAttribute("hidden", false);

  dockerDeployment.storageBackend = new DeviceMapperStorageBackend();
}


function selectLoopbackDevice() {
  var loopbackDeviceSection = document.getElementById('loopbackDeviceSection');
  loopbackDeviceSection.setAttribute("hidden", false);

  var blockDeviceSection = document.getElementById('blockDeviceSection');
  blockDeviceSection.setAttribute("hidden", true);

  dockerDeployment.storageBackend.device = new DeviceMapperLoopbackDevice();
  dockerDeployment.storageBackend.device.buildFromUI(document);
}

function selectBlockDevice() {
  var loopbackDeviceSection = document.getElementById('loopbackDeviceSection');
  loopbackDeviceSection.setAttribute("hidden", true);

  var blockDeviceSection = document.getElementById('blockDeviceSection');
  blockDeviceSection.setAttribute("hidden", false);

  dockerDeployment.storageBackend.device = new DeviceMapperBlockDevice();
  dockerDeployment.storageBackend.device.buildFromUI(document);
}

