/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var cluster;
var health;
var overview;
var timer = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) {
    cluster = use;
    var kubes = cluster.find("KubeCluster");
    if(kubes.length < 1) {
      current = 0;
      hideKube();
      return;
    } else {
      current = kubes[0];
      showKube();
    }
  }
  println("Refresh kube overview");

  var roles = new Object();
  var nodes = cluster.find('Node');
  for(var i = 0; i < nodes.length; i++) {
    var kube_roles = nodes[i].getRolesByFilter(function(role) {
        return startswith(role.childType, 'Kubernetes') || startswith(role.childType, 'Docker') ||
               (role.childType == 'EtcdHostRole') || (role.childType == 'CgroupSupervisorRole');
      });
    if(kube_roles.length == 0) continue;
    var hostname = nodes[i].resolveName();
    for(var j = 0; j < kube_roles.length; j++) {
      var name = kube_roles[j].name;
      if(roles.hasOwnProperty(name)) {
        roles[name].push(hostname);
      } else {
        roles[name] = [hostname];
      }
    }
  }
  
  var list = document.getElementById('nodes');
  while(list.childNodes.length > 2) list.removeChild(list.lastChild);
  for(var role in roles) {
    list.appendChild(nodeRow(role, dotFormat(roles[role])));
  }

  update();
}

function update() {
  if(current && current.cluster.connected) {
   dock.workerpool.getWorker(current.cluster.actualurl).call('cmgui', 'getKubeClusterOverview', [current.cluster.sessionid, current.get('uniqueKey')], function(code, data) {
       if(code == 0) {
         setCallbackResult(data);
       } else {
         dock.jsonCallError(this, code, data);
       }
     }, current);

    var primary = current.cluster.getBasePartition();
    primary = current.cluster.find(primary.get('primaryMaster'));
    var mpi = current.cluster.allChecksMetricsAsMetricPrmId(primary.object.uniqueKey);
    var nodes = [primary.object.uniqueKey];
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmmon', 'getLatestPickedRates', [nodes, mpi], function(code, data) {
      if(code == 0) {
        updateHealth(data, primary.object.uniqueKey);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
  }
}

function updateHealth(srwi, key, showSeverity, hostname) {
  showSeverity = isundefined(showSeverity, true);
  health = srwi;
  if(typeof(hostname) != 'undefined') {
    var obj = document.getElementById('healthstatustitle');
    obj.setAttribute('value', 'Health Status for ' + hostname);
  }

  var list = new Array();
  var obj = document.getElementById('healthstatusrows');
  while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
  for(var i = 0; i < srwi.length; i++) {
    if(srwi[i].thresholdId > 0) {
      var one = current.cluster.find(srwi[i].metricId);
      if(one) {
        var name = one.resolveName();
        if(srwi[i].metricParam.length > 0) name += ':' + srwi[i].metricParam;
        var severity = srwi[i].severity;
        var value;
        if(one.get('baseType') == 'HealthCheck') {
          var conf = current.cluster.getMonHealthConf(key, srwi[i].thresholdId);
          if(srwi[i].rate < 0.5) {
            value = 'PASS';
          } else if(srwi[i].rate > 1.5) {
            value = 'FAIL';
            if((severity >= MINUSONE) && conf) severity = conf.failSeverity;
          } else if(kisNaN(srwi[i].rate)) {
            value = 'no data';
          } else {
            value = 'UNKNOWN';
            if(conf) severity = conf.unknownSeverity;
          }
        } else {
          var conf = current.cluster.getThreshold(key, srwi[i].thresholdId);
          value = srwi[i].rate;
          if(conf) {
            if((severity >= MINUSONE) && conf) severity = conf.severity;
            if(conf.isUpper) {
              value += ' > ';
            } else {
              value += ' < ';
            }
            value += conf.bound;
          } else {
            println("Unknown threshold " + current.resolveName(), srwi[i].thresholdId);
          }
        }
        if(severity >= MINUSONE) severity = 0;
        var date = new Date();
        date.setTime(srwi[i].timeStamp * 1000);
        list.push([-severity, date.format("dd/mmm/yyyy HH:MM:ss"), name, value, srwi[i].infoMessage, one.get('notes')]);
      }
    }
  }
  list = list.sort();
  for(var i = 0; i < list.length; i++) {
    healthRows(obj, list[i], showSeverity);
  }

  function healthRows(obj, data, showSeverity) {
    var name = data[2];
    if(!startswith(name, "Kube") || string_case_equal(name, "KubernetesChildNode")) {
      return;
    }

    if(string_case_equal(name, "KubernetesComponentsStatus")) name = "Components";
    else if(string_case_equal(name, "KubernetesNodesStatus")) name = "Nodes";
    else if(string_case_equal(name, "KubernetesPodsStatus")) name = "Pods";

    var MAX_INFO_LEN = 96;
    var info = data[4];
    for(var i = 0; i < 10; i++) {
      var row = document.createElement('row');
      row.setAttribute("class", "staticdatarow");
      row.setAttribute("flex", "1");
      row.setAttribute("align", "center");
      row.setAttribute("height", "19px");
      var label = document.createElement('label');
      label.setAttribute("class", "staticdatavaluesmall");
      if(i == 0) label.setAttribute("value", data[1]);
      row.appendChild(label);
      var label = document.createElement('label');
      label.setAttribute("class", "staticdatavaluesmall");
      if(i == 0) {
        label.setAttribute("value", name);
        if(data[5].length > 0) label.setAttribute("tooltiptext", data[5]);
      }
      row.appendChild(label);
      if(showSeverity) {
        var label = document.createElement('label');
        label.setAttribute("class", "staticdatavaluesmall");
        if(i == 0) label.setAttribute("value", -data[0]);
        row.appendChild(label);
      }
      var label = document.createElement('label');
      label.setAttribute("class", "staticdatavaluesmall");
      if(i == 0) label.setAttribute("value", data[3]);
      row.appendChild(label);
      var label = document.createElement('label');
      label.setAttribute("class", "staticdatavaluesmall");
      row.appendChild(label);
      obj.appendChild(row);
      var k = info.indexOf("\n");
      if((k > 0) && (k <= MAX_INFO_LEN)) {
        label.setAttribute("value", info.substr(0, k));
        info = info.substr(k + 1);
      } else {
        if(info.length <= MAX_INFO_LEN) {
          label.setAttribute("value", info);
          break;
        } else {
          var j = info.lastIndexOf(' ', MAX_INFO_LEN);
          if(j >= MAX_INFO_LEN * 0.8) {
            label.setAttribute("value", info.substr(0, j));
            info = info.substr(j + 1);
          } else {
            label.setAttribute("value", info.substr(0, MAX_INFO_LEN));
            info = info.substr(MAX_INFO_LEN);
          }
        }
      }
    }
  }
}

function setCallbackResult(value) {
  overview = value;
  overviewValue('name');
  overviewValue('version');
  overviewValue('numNodes');
  overviewValue('numNamespaces');
  overviewValue('numServices');
  overviewValue('numRcs');
  overviewValue('numPvs');
  overviewValue('numPvcs');

  var podsRunning = 0;
  var podsPending = 0;
  var podsSucceeded = 0;
  var podsFailed = 0;
  var podsUnknown = 0;
  for(var i = 0; i < overview.pods.length; i++) {
    if(string_case_equal(overview.pods[i].status, "Succeeded")) {
      podsSucceeded++;
    } else if(string_case_equal(overview.pods[i].status, "Running")) {
      podsRunning++;
    } else if(string_case_equal(overview.pods[i].status, "Pending")) {
      podsPending++;
    } else if(string_case_equal(overview.pods[i].status, "Failed")) {
      podsFailed++;
    } else {
      podsUnknown++;
    }
  }
  setTextMessages({
    podsRunning: podsRunning,
    podsPending: podsPending,
    podsSucceeded: podsSucceeded,
    podsFailed: podsFailed,
    podsUnknown: podsUnknown
  });

  var jobsRunning = 0;
  var jobsPending = 0;
  var jobsSucceeded = 0;
  var jobsFailed = 0;
  var jobsUnknown = 0;
  for(var i = 0; i < overview.jobs.length; i++) {
    if(string_case_equal(overview.jobs[i].state, "Succeeded")) {
      jobsSucceeded++;
    } else if(string_case_equal(overview.jobs[i].state, "Running")) {
      jobsRunning++;
    } else if(string_case_equal(overview.jobs[i].state, "Pending")) {
      jobsPending++;
    } else if(string_case_equal(overview.jobs[i].state, "Failed")) {
      jobsFailed++;
    } else {
      jobsUnknown++;
    }
  }
  setTextMessages({
    jobsRunning: jobsRunning,
    jobsPending: jobsPending,
    jobsSucceeded: jobsSucceeded,
    jobsFailed: jobsFailed,
    jobsUnknown: jobsUnknown
  });

  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 1000);
}

function nodeRow(role, hosts) {
  var row = document.createElement('row');
  row.setAttribute("flex", "1");
  row.setAttribute("align", "center");
  row.setAttribute("height", "17px");
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  label.setAttribute("value", role);
  row.appendChild(label);
  var label = document.createElement('label');
  label.setAttribute("class", "staticdatavaluesmall");
  if(hosts.length>35) {
    var last = hosts.lastIndexOf(",");
    hosts = hosts.substr(0, last) + ",...";
  }
  label.setAttribute("value", hosts);
  row.appendChild(label);
  return row;
}

function hideKube() {
  if(parent.document.getElementById('nokube').hasAttribute('hidden')) {
    parent.document.getElementById('tabbox').setAttribute('hidden', true);
    parent.document.getElementById('nokube').removeAttribute('hidden');
  }
}

function showKube() {
  if(parent.document.getElementById('tabbox').hasAttribute('hidden')) {
    parent.document.getElementById('tabbox').removeAttribute('hidden');
    parent.document.getElementById('nokube').setAttribute('hidden', true);
  }
}

