/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var cluster;
var current = 0;

var timer = 0;
var count = 0;

var PATTERN_FIRST_LINE = /^##### cmgui #####$/g;
var PATTERN_GENERIC = /^### ([^#]+) ### (STARTED|FINISHED|FAILED)$/g;
var PATTERN_PERC = /^### (\d{1,3})$/g;
var stepsList;
var progressPerc = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) {
    cluster = use;
    var kubes = cluster.find("KubeCluster");
    if(kubes.length < 1) return;
    current = kubes[0];
  }
  println("Refresh Kubernetes settings" + ' - ' + document.documentURI);
}

function kubeDisable() {
  var args = new Object();
  args['question'] = "Are you sure you want to disable Kubernetes?";
  var d = new dock.cvDialog("question.xul", args);
  d.onOk = function() {
    var id = 0;
    var text = "\nKubernetes cluster is being disabled. The progress is shown below.";
    setAttr('disablingMessage', 'value', text);
    overviewBar('progressBar', 0, 100);
    stepsList = new Array();
    var xmlConfig = "<kubeConfig><disable>1</disable></kubeConfig>";
    var dockerDeployment = ""
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmkube', 'executeKubeSetup', [xmlConfig, dockerDeployment], function(code, data) {
      if(code == 0) {
        id = data.setupExecutionID;
        timer = window.setTimeout('checkStatus(' + id +');', 1000);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current.cluster);
  }
  dock.dialogs.showDialog(d);
}

function checkStatus(id) {
  timer = 0;
  count++;
  if (count > 2000) {
    return;
  }

  dock.workerpool.getWorker(current.cluster.actualurl).call('cmkube', 'getKubeSetupExecution', [id], function(code, data) {
    if(code !== 0) {
      dock.jsonCallError(this, code, data);
      return;
    }
    if (!data) {
      dock.displayError('A Kubernetes cluster is already being disabling.');
      return;
    }

    var currentProgress = "";
    var stepsCounter = 0;
    for (var i = 0; i < data.datacout.length; i++) {
      var matched = false;

      if (data.datacout[i].match(PATTERN_FIRST_LINE)) {
        setAttr('disablingProgress', 'hidden', "false");
        matched = true;
      }
      var m;
      do {
        m = PATTERN_GENERIC.exec(data.datacout[i]);
        if (m) {
          matched = true;
          stepsList[stepsCounter] = m;
          stepsCounter++;
        }
      } while (m);
      var m2 = PATTERN_PERC.exec(data.datacout[i]);
      if (m2) {
        matched = true;
        progressPerc = m2[1];
      }

      if (!matched) {
        currentProgress += data.datacout[i];
        currentProgress += "\n";
      }
    }
    var stepsDisplay = new ProgressStepDisplayer('disablingSteps');
    stepsDisplay.clearAll();

    var mergedStepsList = new Array();
    for (var i = 0; i < stepsList.length; i++) {
      var item = stepsList[i];
      if (stepsList[i-1]) {
        var previousItem = stepsList[i-1];
        if (item[1] == previousItem[1]) {
          mergedStepsList[mergedStepsList.length-1] = item;
        } else {
          mergedStepsList.push(item);
        }
      } else {
        mergedStepsList.push(item);
      }
    }

    for (var i = 0; i < mergedStepsList.length; i++) {
      //stepsDisplay.add(mergedStepsList[i][2], mergedStepsList[i][1]);
    }

    overviewBar('progressBar', progressPerc, 100);

    var disablingOutput = document.getElementById('disablingOutput');
    disablingOutput.value = currentProgress;
    var ti = document.getAnonymousNodes(disablingOutput)[0].childNodes[0];
    ti.scrollTop = ti.scrollHeight;

    if (data.finishedAt == 0) {
      timer = window.setTimeout('checkStatus(' + id +');', 3000);
    } else if (data.successful) {
      overviewBar('progressBar', 100, 100);
    } else { // failed
    }
  },
  current);
}

