'use strict';

var Context = require('cm-context');
var test  = require('unit.js');
var fs = require("fs");

var dockerManager;

function testSetup() {
  context = new Context(__dirname + '/../docker/');
  dockerManager = context.load('dockerdeployment.js');
}

describe('Docker Registries Management', function(){
  it('should not allow to add the same registry twice', function() {
    testSetup();
    
    var deployment = dockerManager.dockerDeployment;
    deployment.addRegistry('test1');
    deployment.addRegistry('test2');
    deployment.addRegistry('test1');

    test.number(deployment.registries.length).is(2);
  });

  it('should correctly delete the specified registry', function() {
    testSetup();
    
    var deployment = dockerManager.dockerDeployment;
    deployment.addRegistry('test1');
    deployment.addRegistry('test2');
    deployment.addRegistry('test3');
    deployment.addRegistry('test4');

    deployment.removeRegistry('test2');

    test.number(deployment.registries.length).is(3);
    test.string(deployment.registries[2]).is('test4');
    test.string(deployment.registries[1]).is('test3');
  });

  it('should correctly empty the registries', function() {
    testSetup();
    
    var deployment = dockerManager.dockerDeployment;
    deployment.addRegistry('test1');

    deployment.removeRegistry('test1');

    test.number(deployment.registries.length).is(0);
  });
});
