/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function setAttr(id, attr, value) {
  document.getElementById(id).setAttribute(attr, value);
}

function getAttr(id, attr, notrim) {
  if (notrim === undefined) {
    return document.getElementById(id)[attr].trim();
  } else {
    return document.getElementById(id)[attr];
  }
}

function setTextMessages(messages) {
  for (var fieldId in messages) {
    if (messages.hasOwnProperty(fieldId)) {
      setAttr(fieldId, 'value', messages[fieldId]);
    }
  }
}

function joinPath(prefix, sourcePath) {
  if (sourcePath === '' || sourcePath[0] === '/')
    return sourcePath;
  if (prefix[prefix.length - 1] === '/')
    prefix = prefix.substr(0, prefix.length - 1);
  return prefix + '/' + sourcePath;
}

function addTextNode(doc, node, name, value) {
  var elem = doc.createElement(name);
  var text = doc.createTextNode(value);
  elem.appendChild(text);
  node.appendChild(elem);
}

function toXML(c) {
  var doc = document.implementation.createDocument('', 'kubeConfig', null);

  var script = doc.createElement('script');
  var run_element = doc.createElement('drymode');
  run_element.setAttribute('enabled', c.dryMode);
  script.appendChild(run_element);
  doc.documentElement.appendChild(script);

  addTextNode(doc, doc.documentElement, 'clustername', c.clusterName);
  if (c.runOnHead) {
    addTextNode(doc, doc.documentElement, 'runonhead', "1");
  } else {
    addTextNode(doc, doc.documentElement, 'runonhead', "0");
  }
  addTextNode(doc, doc.documentElement, 'categories', c.categories);
  addTextNode(doc, doc.documentElement, 'podnetworkname', c.podNetworkName);
  addTextNode(doc, doc.documentElement, 'podnetworkbaseaddress', c.podNetworkBaseAddress);
  addTextNode(doc, doc.documentElement, 'podnetworkbits', c.podNetworkBits);
  addTextNode(doc, doc.documentElement, 'svcnetworkname', c.svcNetworkName);
  addTextNode(doc, doc.documentElement, 'svcnetworkbaseaddress', c.svcNetworkBaseAddress);
  addTextNode(doc, doc.documentElement, 'svcnetworkbits', c.svcNetworkBits);
  addTextNode(doc, doc.documentElement, 'insecureapiport', c.insecureApiPort);
  addTextNode(doc, doc.documentElement, 'secureapiport', c.secureApiPort);
  addTextNode(doc, doc.documentElement, 'kubeletport', c.kubeletPort);
  addTextNode(doc, doc.documentElement, 'healthcheckport', c.healthCheckPort);
  addTextNode(doc, doc.documentElement, 'controllerPort', c.controllerPort);
  addTextNode(doc, doc.documentElement, 'schedulerPort', c.schedulerPort);
  return new XMLSerializer().serializeToString(doc);
}

function splitArrayWithPattern(a, pattern) {
  var i = 0;
  for (; i < a.length; i++) {
    if (a[i].match(pattern)) {
      break;
    }
  }
  return {
    before: a.slice(0, i),
    after:  a.slice(i+1, a.length)
  };
}

function checkEnoughMonitorsUP() {
  var monsBSTrue = userChoices.getNodeObjectsFiltered('Monitor', function(container) {
    return container.bootstrap === 'true';
  });
  var allBSTrueUP = monsBSTrue.every(function(node) {
    return node.status && node.status.status === 'UP';
  });

  var monsBSAuto = userChoices.getNodeObjectsFiltered('Monitor', function(container) {
    return container.bootstrap === 'auto';
  });
  var someBSAutoUP = monsBSAuto.some(function(node) {
    return node.status && node.status.status === 'UP';
  });

  if ((monsBSTrue.length > 0 && allBSTrueUP) || (monsBSTrue.length === 0 && someBSAutoUP)) {
    return '';
  } else {
    var msg = 'Not enough Ceph Monitor nodes are up to proceed with the setup.\n\n';
    if (monsBSTrue.length > 0) {
      msg += 'All of the following nodes needs to be up: ' +
             monsBSTrue.map(function(node) { return node.get('hostname')}).join(', ') + '\n';
    }
    if (monsBSAuto.length > 0) {
      msg += 'At least one of the following nodes needs to be up: ' +
             monsBSAuto.map(function(node) { return node.get('hostname')}).join(', ') + '\n';
    }
    return msg;
  }
}

function getOSFlavor() {
    var osFlavor = 'unknown';
    var masterNodes = current.find('MasterNode');
    if(masterNodes.length>0 && masterNodes[0].sysInfo.osFlavor!==undefined) {
      osFlavor = masterNodes[0].sysInfo.osFlavor;
    } else if(masterNodes.length>0 && masterNodes[0].sysInfo.revision!=='') {
      osFlavor = masterNodes[0].sysInfo.revision;
    }
    return osFlavor;
}

function unhide(elementNodeId) {
    document.getElementById(elementNodeId).removeAttribute('hidden');
}

function setSimpleRadio(choice, elementIds) {
  elementIds.forEach(function(elementId, index) {
    var obj = document.getElementById(elementId);
    if (obj.getAttribute('disabled')!=='true') {
      if (choice===index) {
        obj.setAttribute('on', true);
      } else {
        obj.setAttribute('on', false);
      }
    }
  });
}

