/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'MICNode';
var singular = 'MIC';
var plural = 'MICs';

function createTreeItem(one) {
  var ip = '0.0.0.0';
  var ips = one.getIPs();
  if(ips.length >= 1) ip = ips[0][1];
  
  var item = new Object();
  item['key'] = one.get('uniqueKey');
  item['modified'] = one.get('modified');
  item['hostname'] = one.get('hostname');
  item['host'] = current.resolveName(one.get('host'));
  item['ip'] = ip;
  return item;
}

function addObject() {
  var args = new Object();
  args['title'] = 'Add new ' + singular;
  args['question'] = 'Hostname';
  args['known'] = current.known('Device', 'hostname');
  var base = current.getBasePartition();
  args['default'] = 'mic';
  var d = new dock.cvDialog("newobject.xul", args);
  d.onOk = function(name) {
    var mac = randomMAC();
    var object = new dock.cvMICNode();
    object.hostname = name;
    object.powerControl = 'mic';    
    object.mac = mac;
    var base = current.getBasePartition();
    if(isObject(base)) {
      object.partition = base.object.uniqueKey;
      object.category = base.object.defaultMICCategory;
    }
    var work = current.addObject(object, dock.openAfterCreate);
    guessIP(work, 'mic0', mac);
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function cloneObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a " + singular + " for cloning");
  } else if(select.length > 1) {
    dock.displayError("Please select only one " + singular);
  } else {
    var original = current.find(select[0]);
    if(isObject(original)) {
      var args = new Object();
      var host = original.get('hostname');
      args['title'] = 'Clone ' + host;
      args['question'] = 'Hostname';
      var nn = nameNumber(host);
      if(nn[1].length == 0) nn[1] = '0';
      var name;
      var index = myParseInt(nn[1]);
      while(true) {
        index++;
        name = nn[0] + sprintf('%0' + nn[1].length + 'd', index);
        if(!current.find('Device', name)) break;
      }
      args['default'] = name;
      args['known'] = current.known('Device', 'hostname');
      var d = new dock.cvDialog("newobject.xul", args);
      d.onOk = function(name) {
        var object = original.clone();
        object.modified = true;
        object.toBeRemoved = false;
        object.hostname = name;
        object.mac = randomMAC();
        object.tag = '00000000a000';
        object.ethernetSwitch = new dock.cvSwitchPort();
        object.setPowerDistributionUnits(0, new Array());
        cloneNodeIP(original.object, object);
        current.addObject(object, dock.openAfterCreate);
        refresh();
      }
      dock.dialogs.showDialog(d);
    } else {
      dock.displayError("Original not found, already removed by someone else");
    }
  }
}

function createWizard() {
  var args = new Object();
  args['cluster'] = current;
  var d = new dock.cvDialog("micnodewizard.xul", args);
  d.onOk = function() {
    refresh();
  }
  dock.dialogs.showDialog(d);
}
