/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var myDialog;
var dock
var cluster;
var nodes;

var firstnodeIndex;
var lastnodeIndex;
var bridgeName;
var micrange;
var ipoffset;

var waitCount = 0;

function onLoad() {
  dock = getDock();
  println("Node Creation Wizard initialized: " + this.name);
  
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  cluster = myDialog.arguments.cluster;
  nodes = cluster.find('PhysicalNode');
  
  if(nodes.length == 0) {
    dock.displayError('No nodes found to serve as host nodes');
    close();
    return;
  }
  
  waitIcon();
  
  var fn = document.getElementById('firstnode');
  for(var i = 0; i < nodes.length; i++) {
    var item = document.createElement('menuitem');
    item.setAttribute('key', nodes[i].get('uniqueKey'));
    item.setAttribute('index', i);
    item.setAttribute('label', nodes[i].resolveName());
    fn.firstChild.appendChild(item);
  }
  fn.selectedIndex = 0;

  var ln = document.getElementById('lastnode');
  for(var i = 0; i < nodes.length; i++) {
    var item = document.createElement('menuitem');
    item.setAttribute('key', nodes[i].get('uniqueKey'));
    item.setAttribute('index', i);
    item.setAttribute('label', nodes[i].resolveName());
    ln.firstChild.appendChild(item);
  }
  ln.selectedIndex = nodes.length - 1;

  document.getElementById('firstmic').value = 'mic0';
  document.getElementById('lastmic').value = 'mic0';

  document.getElementById('hostname').value = '{host}-{mic}';
  updateCategory();
  
}

function createCategory() {
  waitIcon();
  document.getElementById('next').setAttribute('disabled', true);
  var args = new Object();
  args['title'] = 'Add new MIC node category';
  args['question'] = 'Name';
  args['known'] = [];
  args['default'] = 'mic-default';
  var d = new dock.cvDialog("newobject.xul", args);
  d.onOk = function(name) {
      var category = createDefaultMICNodeCategory(name, cluster);
      category.commit();
      window.setTimeout('updateCategory();', 3000);
    }
  dock.dialogs.showDialog(d);  
}

function updateCategory() {
  var cat = document.getElementById('category');
  var catpop = cat.firstChild;
  while(catpop.firstChild) catpop.removeChild(catpop.firstChild);
  var categories = cluster.find('MICNodeCategory');
  var found = -1;
  var dc = cluster.base.get('defaultMICCategory');
  for(var i = 0; i < categories.length; i++) {
    var item = document.createElement('menuitem');
    item.setAttribute('key', categories[i].get('uniqueKey'));
    item.setAttribute('label', categories[i].resolveName());
    cat.firstChild.appendChild(item);
    if(categories[i].get('uniqueKey') == dc) found = i;
  }
  cat.selectedIndex = found;  
  if(categories.length > 0) {
    var button = document.getElementById('createCategory');
    button.setAttribute('hidden', true);
    if (categories.length == 1) { // forces selection after creation of first category
      cat.selectedIndex = 0;
    }
  }
  document.getElementById('next').removeAttribute('disabled');
  unwaitIcon();
}

function next() {
  var pages = document.getElementById('deck').childNodes.length - 1;
  var index = parseInt(document.getElementById('deck').selectedIndex);
  switch(index) {
  case(0) :
    if(!checkRange()) return;
    var hn = document.getElementById('hostname').value;
    if((hn.indexOf('{host}') < 0) && (lastnodeIndex - firstnodeIndex > 0)) {
      dock.displayError("The {host} hostname needs to be part of the MIC hostname if more than one host is selected", window);
      return;
    }
    micrange = checkMICRange();
    if(micrange.length == 0) return;
    if((hn.indexOf('{mic}') < 0) && (micrange[2] - micrange[1] > 0)) {
      dock.displayError("The {mic} name needs to be part of the MIC hostname if more than one MIC is selected", window);
      return;
    }
    var category = document.getElementById('category');
    if(category) {
      if(!category.selectedItem) {
        dock.displayError("The MIC node category needs to be set", window);
        return;
      }
    }
    break;
  case(1) :
    bridgeName = document.getElementById('hostbridge').value;
    if(!regexMatch(bridgeName, '^br[0-9]+$')) {
      dock.displayError("The bridge name is not valid, it should be brX");
      return;
    }
    var ip = getOffset();
    if(!regexMatch(ip, dock.REGEX_IPv4)) {
      dock.displayError("The base ip offset is not valid.\nUsually something like 0.0.128.0");
      return;
    }
    ipoffset = ip2int(ip);
    break;
  }
  index++;
  switch(index) {
  case(1) :
    MICipOffset();
    break;
  case(2) :
    create(false);
  }
  document.getElementById('deck').selectedIndex = index;
  document.getElementById('prev').setAttribute('disabled', index == 0);
  document.getElementById('next').setAttribute('hidden', index == pages);
  document.getElementById('savefinish').setAttribute('hidden', index < pages);
  document.getElementById('finish').setAttribute('hidden', index < pages);
}

function prev() {
  var pages = document.getElementById('deck').childNodes.length - 1;
  var index = parseInt(document.getElementById('deck').selectedIndex);
  var back = true;
  switch(index) {
    case(3):
      var args = new Object();
      args['question'] = "This will undo all your manual changes, are you sure?";
      var d = new dock.cvDialog("question.xul", args);
      d.onNo = function() {
        back = false;
      }
      dock.dialogs.showDialog(d);
      break;
  }
  if(back) {
    index--;
    document.getElementById('deck').selectedIndex = index;
    document.getElementById('prev').setAttribute('disabled', index == 0);
    document.getElementById('next').setAttribute('hidden', index == pages);
    document.getElementById('savefinish').setAttribute('hidden', index < pages);
    document.getElementById('finish').setAttribute('hidden', index < pages);
  }
}

function checkRange() {
  firstnodeIndex = parseInt(document.getElementById('firstnode').selectedItem.getAttribute('index'));
  lastnodeIndex = parseInt(document.getElementById('lastnode').selectedItem.getAttribute('index'));

  if(lastnodeIndex < firstnodeIndex) {
    dock.displayError("The last node should have a larger number than the first", window);
    return false;
  }
  return true;
}

function checkMICRange() {
  var n0 = nameNumber(document.getElementById('firstmic').value);
  var n1 = nameNumber(document.getElementById('lastmic').value);
  if(n0[0] != n1[0]) {
    dock.displayError("The name part MIC range should be equal");
    return [];
  } else if(n0[1].length != n1[1].length) {
    dock.displayError("The number of digits of the MIC range should be equal");
    return [];
  }
  var s = myParseInt(n0[1]);
  var e = myParseInt(n1[1]);
  if(s > e) {
    dock.displayError("The last MIC should have a larger number than the first");
    return [];
  }
  return [n0[0], s, e, n0[1].length];
}

function MICipOffset() {
  document.getElementById('hostbridge').value = 'br0';
//   var nips = 2 * (micrange[2] - micrange[1] + 1);
  var nips = (micrange[2] - micrange[1] + 1);
  var stride2 = Math.ceil(Math.log(nips) / Math.log(2));  
  var stride = document.getElementById('stride');
  stride.selectedIndex = stride2;
  updateOffset();
}

function getOffset() {
  var offset = document.getElementById('offset-0').value;
  offset += '.' + document.getElementById('offset-1').value;
  offset += '.' + document.getElementById('offset-2').value;
  offset += '.' + document.getElementById('offset-3').value;
  return offset;
}

function updateOffset() {
  var data = document.getElementById('ipoffsets');
  var base = getOffset();
  if(!regexMatch(base, dock.REGEX_IPv4)) {
    data.value = 'The base ip offset is not valid.\nUsually something like 0.0.128.0';
    return;
  }  
  base = ip2int(base);
  var stride = document.getElementById('stride');
  stride = stride.firstChild.childNodes[stride.selectedIndex];
  stride = parseInt(stride.getAttribute('label'));

  var mhnl = 12;
  for(var i = firstnodeIndex; i <= lastnodeIndex; i++) {
    mhnl = Math.max(mhnl, nodes[i].resolveName().length);
  }
  mhnl += 2 + micrange[0].length + micrange[3];

  var spaces = new Array();
  spaces[0] = '';
  for(var i = 1; i < mhnl; i++) {
    spaces[i] = spaces[i - 1] + ' ';
  }
  
  var lines = new Array();
  var hn = document.getElementById('hostname').value;
  var format = micrange[0] + '%0' + micrange[3] + 'd';
  var ip0 = base;
  for(var i = firstnodeIndex; i <= lastnodeIndex; i++) {
    var node = nodes[i];
    var base = 0;
    var nw = node.getManagementNetwork();
    if(nw) {
      nw = cluster.find(nw);
      if(nw) base = ip2int(nw.get('baseAddress'));
    }
    for(var j = micrange[1]; j <= micrange[2]; j++) {
      var mic = sprintf(format, j);
      var hostname = hn.replace('{host}', node.resolveName()).replace('{mic}', mic);
//       var mic0 = int2ip(ip0 + 2 * (j - micrange[1]));
//       var host = int2ip(ip0 + 2 * (j - micrange[1]) + 1);
//       lines.push(hostname + spaces[mhnl - hostname.length] + "mic: " + mic0 + spaces[14 - mic0.length] + "\thost: " + host);
      var mic0 = int2ip(base + ip0 + (j - micrange[1]));
      lines.push(hostname + spaces[mhnl - hostname.length] + "mic: " + mic0);
    }
    ip0 += stride;
  }
  data.value = lines.join("\n");
}

function assignMICHostRole(node, real, result) {
  var roles = node.get('roles');
  for(var i = 0; i < roles.length; i++) {
    if(roles[i].childType == 'MICHostRole') return true;
  }
  if(real) {
    var role = new dock.cvMICHostRole();
    var work = new dock.cvObject(node.cluster, role);
    roles.push(role);
    node.set('roles', roles);
    
    var fin = node.get('finalize');
    if(fin.length == 0) {
      fin = '#!/bin/sh\n';
    }
    if(fin.indexOf('net.ipv4.ip_forward') < 0) {
      fin += '# Added by mic-setup:\n'
      fin += 'perl -pi -e \'s/net\.ipv4\.ip_forward = 0/net.ipv4.ip_forward = 1/\' /localdisk/etc/sysctl.conf\n'
      fin += 'if [ -e /localdisk/etc/sysconfig/sysctl ]; then\n'
      fin += '  perl -pi -e \'s/IP_FORWARD="no"/IP_FORWARD="yes"/\' /localdisk/etc/sysconfig/sysctl\n'
      fin += 'fi\n\n'
      node.set('finalize', fin);
    } else {
      result.push('IP forwarding has been already configured in the host finalize script: it should be enabled');
    }
  }
  return true;
}

function assignBrigdge(node, real, result, base) {
  var intf = node.get('networks');
  var found = false;
  for(var i = 0; !found && (i < intf.length); i++) {
    if(bridgeName == intf[i].name) found = intf[i];
  }
  
  var joff = 0;
  var interfaces = new Array();
  if(found) {
    result.push(node.resolveName() + ' already has ' + bridgeName + ' defined');
    interfaces = found.interfaces.slice(0);
  } else if(real) {
    var ip = node.getIPs(true);
    if(ip.length == 0) {
      result.push(node.resolveName() + ' has no management interface');
      return false;
    } else {
      ip = ip[0];
    }
    found = new dock.cvNetworkBridgeInterface();
    found.name = bridgeName;
    found.network = ip[0]
    found.ip = ip[1];
    interfaces = [ip[2]];
    intf[ip[3]].network = 0;
    intf[ip[3]].ip = '0.0.0.0';
    intf[ip[3]].dhcp = false;
    intf[ip[3]].modified = true;
    var work = new dock.cvObject(node.cluster, found);
    node.set('provisioningNetwork', found.uniqueKey);
    intf.push(found);
    var ip2dot = (ip[2] + '.').toLowerCase();
    var ip2col = (ip[2] + ':').toLowerCase();
    for(var i = 0; i < intf.length; i++) {
      var name0 = intf[i].name;
      var name = name0.toLowerCase();
      if(startswith(name, ip2dot)) {
        var idx = intf[i].name.indexOf('.');
        intf[i].name = bridgeName + '.' + intf[i].name.substr(idx + 1);
        result.push(node.resolveName() + ' renamed ' + name0 + ' to ' + intf[i].name);
      } else if(startswith(name, ip2col)) {
        var idx = intf[i].name.indexOf(':');
        var idx = intf[i].name.substr(idx + 1);
        intf[i].name = bridgeName + ':' + idx;
        result.push(node.resolveName() + ' renamed ' + name0 + ' to ' + intf[i].name);
        if(isNumeric(idx)) joff = Math.max(joff, parseInt(idx) + 1);
      }
    }
  }
    
//   for(var j = micrange[1]; j <= micrange[2]; j++) {
//     var f = false;
//     var name = bridgeName + ':' + (j + joff);
//     for(var i = 0; !f && (i < intf.length); i++) {
//       f = (name == intf[i].name);
//     }
//     if(f) {
//       result.push(node.resolveName() + ' already has ' + name +' defined');
//     } else if(real) {
//       var alias = new dock.cvNetworkAliasInterface();
//       var work = new dock.cvObject(node.cluster, alias);
//       alias.name = name;
//       alias.network = found.network;
//       alias.additionalHostnames = ['no-dns'];
//       var nw = node.cluster.find(found.network);
//       alias.ip = int2ip(ip2int(nw.get('baseAddress')) + base + 2 * (j - micrange[1]) + 1);
//       intf.push(alias);
//     }
//   }

  for(var j = micrange[1]; j <= micrange[2]; j++) {
    var f = false;
    var name = 'mic' + j;
    for(var i = 0; !f && (i < intf.length); i++) {
      f = (name == intf[i].name);
    }
    if(f) {
      result.push(node.resolveName() + ' already has ' + name +' defined');
    } else if(real) {
      var mic0 = new dock.cvNetworkPhysicalInterface();
      var work = new dock.cvObject(node.cluster, mic0);
      mic0.name = 'mic' + j;
      mic0.network = 0;
      mic0.ip = '0.0.0.0';
      intf.push(mic0);
    }
    if(real && interfaces.find(name) < 0) interfaces.push(name);
  }  
  if(real) {
    found.interfaces = interfaces;
    node.set('networks', intf);
  }

  if(!real) return [found.network, '255.255.0.0'];
  return [found.network, found.ip];
}

function createMIC(hostname, real, result, node, network, ip, index) {
  var micnode = cluster.find('Device', hostname);
  if(micnode) {
    if(!micnode.get('childType') != 'isMICNode') {
      result.push(hostname + ' already exists and is not a MIC');
      return false;
    }
    result.push(hostname + ' already exists');
  } else {
    var mac = randomMAC();
    micnode = new dock.cvMICNode();
    micnode = new dock.cvObject(cluster, micnode);
    micnode.set('hostname', hostname);
    micnode.set('partition', node.get('partition'));
    micnode.set('powerControl', 'mic');
    micnode.set('mac', mac);
    
    var mic0 = new dock.cvNetworkPhysicalInterface();
    var work = new dock.cvObject(node.cluster, mic0);
    mic0.name = 'mic0';
    mic0.network = network;
    mic0.ip = ip;
    mic0.mac = mac;
    micnode.set('indexInsideContainer', index);
    micnode.set('networks', [mic0]);
    micnode.set('provisioningNetwork', mic0.uniqueKey);
    micnode.set('host', node.get('uniqueKey'));
    
    var category = document.getElementById('category');
    if(category) {
      category = parseInt(category.selectedItem.getAttribute('key'));
      micnode.set('category', category, true, -1);
    }    
  }  
  return micnode;
}

function create(real, save) {
  save = isundefined(save, false);
  var result = new Array();
  var base = getOffset();
  if(!regexMatch(base, dock.REGEX_IPv4)) {
    data.value = 'The base ip offset is not valid.\nUsually something like 0.0.128.0';
    return;
  }  
  base = ip2int(base);
  var stride = document.getElementById('stride');
  stride = stride.firstChild.childNodes[stride.selectedIndex];
  stride = parseInt(stride.getAttribute('label'));  
  var format = micrange[0] + '%0' + micrange[3] + 'd';
  var hn = document.getElementById('hostname').value;
  for(var i = firstnodeIndex; i <= lastnodeIndex; i++) {
    var node = nodes[i];
    assignMICHostRole(node, real, result);
    var netip = assignBrigdge(node, real, result, base + stride * (i - firstnodeIndex));
    if(netip) {
      var micnodes = new Array();
      var nw = cluster.find(netip[0]);
      var ip0;
      if(nw) {
        ip0 = ip2int(nw.get('baseAddress')) + base + stride * (i - firstnodeIndex);
      } else {
        ip0 = base + stride * (i - firstnodeIndex);
      }
      for(var j = micrange[1]; j <= micrange[2]; j++) {
        var mic = sprintf(format, j);
        var hostname = hn.replace('{host}', node.resolveName()).replace('{mic}', mic);
//         var micip = int2ip(ip0 + 2 * (j - micrange[1]));
        var micip = int2ip(ip0 + (j - micrange[1]));  // NO BRIDGE ALIAS
        var micnode = createMIC(hostname, real, result, node, netip[0], micip, j);
        if(micnode) {
          if(real) {
            micnodes.push(micnode);
            cluster.addObject(micnode, false);
          }
        }
      }
    }
    if(save && real) {
      node.commit();
      for(var j = 0; j < micnodes.length; j++) {
        micnodes[j].commit();
      }
    }
  }
  
  if(result.length == 0) result.push('No conflicts detected, press finish.');
  result.push('');
  result.push('Host nodes need to be rebooted before MIC(s) can be used.');
  document.getElementById('data').value = result.join('\n');
}

function addKernelModulesToOneImage(image) {
  var modules = image.get('modules');
  for(var i = 0; i < modules.length; i++) {
    if(modules[i].name == 'bridge') return;
  }
  modules.push(createKernelModule(cluster, 'bridge', ''));
  image.set('modules', modules);
  image.commit();
}

function addKernelModulesToImages() {
  var images = new Object();
  var categories = new Object();
  for(var i = firstnodeIndex; i <= lastnodeIndex; i++) {
    var node = nodes[i];
    var categoryKey = node.get('category');
    if(categoryKey && !categories.hasOwnProperty(categoryKey)) {
      var category = cluster.find(categoryKey);     
      categories[categoryKey] = category;
      if(category) {
        var imageKey = category.getSoftwareImage();
        if(imageKey && !images.hasOwnProperty(imageKey)) {
          var image = cluster.find(imageKey);
          images[imageKey] = image;
          addKernelModulesToOneImage(image);
        }
      }
    }
  }
}

function finish(save) {
  create(true, save);
  addKernelModulesToImages();
  if(!save) dock.displayInfo('Both MIC nodes and their host nodes need to be saved manually');
  close();
  cluster.updateRack = true;
  myDialog.onOk();
}

function cancel() {
  cluster.updateRack = true;
  close();
}

function waitIcon() {
  if(waitCount == 0) {
    window.setCursor("wait");
    for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("wait");
  }
  waitCount++;
}

function unwaitIcon() {
  waitCount--;
  if(waitCount <= 0) {
    window.setCursor("auto");
    for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("auto");
    if(waitCount < 0) println("Mismatch waitCursor");
    waitCount = 0;
  }
}
