/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function cvMonitoringFilter(cluster, parentWindow) {
  this.cluster = cluster;
  this.query = '';
  this.parentWindow = parentWindow;

  this.devices = 0;  
  this.devicesMap = 0;
  this.metricsMap = 0;
  this.dataMap = 0;
  
  this.refreshRate = dock.generalSettings.monitoringRefresh;
  this.timer = 0;
  this.last = 0;
  this.updating = false;

  this.showfilterRows = 0;
  this.processData = 0;
  this.processTimer = 0;
  this.processIndex = 0;
  this.processStop = false;
  
  this.showfilterIndex = 0;
  this.showfilterTimer = 0;
  
  this.perPage = 25;
  this.page = 1;
}

// cvMonitoringFilter.prototype.setDeviceSelection = function(devices) {
//   if(devices && isArray(devices) && devices.length) {
//     this.devices = devices;
//   } else {
//     this.devices = this.cluster.allKeys('Device');
//   }
// }
//
// cvMonitoringFilter.prototype.clear = function() {
//   this.devices = new Array();
// }

cvMonitoringFilter.prototype.execute = function(query) {
  this.stop();
  this.query = query;
  println('filter execute: ' + this.query);
  this.update(2);
}

cvMonitoringFilter.prototype.start = function() {
  this.timer = window.setInterval(createObjectCallback(this, this.update), this.refreshRate * 1000);
}

cvMonitoringFilter.prototype.stop = function() {
  println('filter stop ' + this.timer);
  this.stopProcess();
  if(this.showfilterTimer) window.clearTimeout(this.showfilterTimer);  
  if(this.timer) {
    this.last = 0;
    window.clearInterval(this.timer);
    this.timer = 0;
    return true;
  }
  return false;
}

cvMonitoringFilter.prototype.stopProcess = function() {
  if(this.processTimer) window.clearInterval(this.processTimer);  
  this.processStop = true;
}

cvMonitoringFilter.prototype.setRefreshRate = function(rate) {
  this.refreshRate = rate;
  if(this.timer) {
    this.stop();
  }
  this.start();
}


cvMonitoringFilter.prototype.processLatestPickedRates = function() {
  if(this.processIndex == 0) {
    this.devices = new Array();
    this.devicesMap = new Object();
    this.metricsMap = new Object();
    this.dataMap = new Object();
  }
  println("processLatestPickedRates: " + this.processIndex + " / " + this.processData.length);
  
  if(this.processData.length == 0) {
    this.clearPage();
    this.updating = false;
    this.parentWindow.unwaitIcon();
    return;
  } else if((this.processData.length == 1) && (this.processData[0].devId < 1024)) { // error
    this.clearPage();
    this.updating = false;
    this.parentWindow.unwaitIcon();
    dock.displayError(this.processData[0].infoMessage, window);
    return;    
  }

  var i = 0;
  while(!this.processStop && (this.processIndex < this.processData.length)) {
    if(i >= dock.FILL_COUNT) {
      window.setTimeout(createObjectCallback(this, this.processLatestPickedRates), dock.FILL_TIMEOUT);
      return;
    }
    var work = this.processData[this.processIndex];
    if(!this.devicesMap.hasOwnProperty(work.devId)) {
      this.devicesMap[work.devId] = this.cluster.find(work.devId);
      this.devices.push(work.devId);
    }
    var mp = [work.metricId, work.metricParam];
    if(!this.metricsMap.hasOwnProperty(mp)) {
      var mh = -1;
      if(work.metricId) {
        var metric = this.cluster.find(work.metricId);
        var mn = '';
        if(metric) {
          mn = metric.resolveName();
          if(metric.get('baseType') == 'Metric') mh =  metric.get('unit');
        }
        if(work.metricParam.length) mn += ':' + work.metricParam;
      } else {
        mn = work.metricParam;
        mh = -2;
      }
      this.metricsMap[mp] = [mn, mh];
      this.dataMap[[work.devId, mn]] = work;
    } else {
      this.dataMap[[work.devId, this.metricsMap[mp][0]]] = work;
    }
    this.processIndex++;
  }
  if(!this.processStop) {
    this.showfilter();
  }
  this.updating = false;
  this.parentWindow.unwaitIcon();
}

cvMonitoringFilter.prototype.update = function(refresh) {
  refresh = isundefined(refresh, 1);
  if(!this.cluster.connected) {
    println("Filter update, no longer connected");
    window.clearInterval(this.timer);
    this.timer = 0;    
  } else {
    if(!this.updating) {
      this.updating = true;
      var now = new Date();
      now = Math.floor(now.getTime() / 1000) + this.cluster.timeOffset;
      if((refresh == 2) || ((refresh == 1) && (now - this.last > 5))) {
        this.last = now;
        this.stopProcess();
        println('filter update: ' + this.query);
        this.parentWindow.waitIcon();
        dock.workerpool.getWorker(current.actualurl).call('cmmon', 'executeFilterLatestPickedRates', [[], this.query], function(code, data) {
            if(code == 0) {
              this.current.processData = data;
              this.current.processIndex = 0;
              this.current.processStop = false;
              this.current.processLatestPickedRates();
            } else {
              dock.jsonCallError(this, code, data);
              this.current.updating = false;
              this.current.parentWindow.unwaitIcon();
            }
          }, this);
      } else {
        this.parentWindow.waitIcon();
        this.showfilter();
        this.last = now;
        this.updating = false;
        this.parentWindow.unwaitIcon();
      }
    } else {
      this.updating = false;
    }
  }
}

cvMonitoringFilter.prototype.clearScreen = function() {
  var obj = document.getElementById('filterstatusrows');
  while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
}

cvMonitoringFilter.prototype.showfilter = function() {
  if(this.showfilterTimer) window.clearTimeout(this.showfilterTimer);
  this.showfilterRows = new Array();
  this.showfilterIndex = 0;
  this.showfilterTimer = window.setTimeout(createObjectCallback(this, this.showfilterUpdate), dock.FILL_TIMEOUT);
}

cvMonitoringFilter.prototype.pages = function(perPage) {
  this.perPage = isundefined(perPage, this.perPage);
  return Math.ceil(this.showfilterRows.length / this.perPage);
}

cvMonitoringFilter.prototype.clearPage = function(page) {
  var obj = document.getElementById('monitoringfilterrows');
  while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
  return obj;
}

cvMonitoringFilter.prototype.displayPage = function(page) {
  this.page = Math.min(Math.max(1, isundefined(page, this.page)), this.pages());

  var obj = this.clearPage();
  
  var b = Math.max(this.perPage * (this.page - 1), 0);
  var e = Math.min(this.perPage * this.page, this.showfilterRows.length);

  for(var i = b; i < e; i++) {
    for(var j = 0; j < this.showfilterRows[i].length; j++) {
      obj.appendChild(this.showfilterRows[i][j]);
    }
  }
  
  updatePageMenu(this.page);
}

cvMonitoringFilter.prototype.showfilterUpdate = function() {
  println('showfilter ' + this.showfilterIndex + ' / ' + this.devices.length);
  if(this.showfilterIndex == 0) {
    var columns = document.getElementById('columns');
    while(columns.childNodes.length > 1) columns.removeChild(columns.lastChild);
    for(var mp in this.metricsMap) {
      var label = document.createElement('column');
      label.setAttribute("flex", "1");
      columns.appendChild(label);
    }
    var rowsheader = document.getElementById('rowsheader');
    while(rowsheader.childNodes.length > 1) rowsheader.removeChild(rowsheader.lastChild);
    for(var mp in this.metricsMap) {
      var label = document.createElement('label');
      label.setAttribute("class", "staticdatatableheader");
      label.setAttribute("value", this.metricsMap[mp][0]);
      rowsheader.appendChild(label);
    }
  }
  for(var j = 0; j < dock.FILL_COUNT * 5; j++) {
    if(this.showfilterIndex >= this.devices.length) {
      this.showfilterTimer = 0;
      createPageMenu(this.pages());
      this.displayPage();
      println('showfilter [DONE]');
      return;
    }
    this.filterRows(this.showfilterRows, this.devices[this.showfilterIndex], this.showfilterIndex);
    this.showfilterIndex++;
  }
  this.showfilterTimer = window.setTimeout(createObjectCallback(this, this.showfilterUpdate), dock.FILL_TIMEOUT);
}

cvMonitoringFilter.prototype.filterRows = function(rows, devId, j) {
  var data = new Array();
  data.push(this.devicesMap[devId].resolveName());
  for(var mp in this.metricsMap) {
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    var value;
    var mn = this.metricsMap[mp][0];
    if(this.dataMap.hasOwnProperty([devId, mn])) {
      var v = this.dataMap[[devId, mn]];
      if(kisNaN(v.rate)) {
        value = 'no-data';
      } else if(this.metricsMap[mp][1] == -2) {
        value = v.infoMessage;
      } else if(this.metricsMap[mp][1] == -1) {
        if(v.rate < 0.5) {
          value = "PASS";
        } else if(v.rate > 1.5) {
          value = "FAIL";
        } else {
          value = "UNKNOWN";
        }        
      } else {
        value = formatKMGselect(v.rate, this.metricsMap[mp][1]);
      }
    } else {
      value = 'no-data';
    }
    data.push(value);
  }
  
  var list = new Array();
  var n = 0;
  for(var i = 0; i < data.length; i++) {
    if(typeof(data[i]) == 'string') {
      var one = data[i].split(/,\s*/);
      list.push(one);
      n = Math.max(n, one.length);
    } else {
      list.push([data[i]]);
//         n = 1;
      n = Math.max(n, 1);
    }
  }
  var subrows = new Array();
  for(var i = 0; i < n; i++) {
    var row = document.createElement('row');
    row.setAttribute("class", "staticdatarow");
    row.setAttribute("flex", "1");
    row.setAttribute("align", "center");
    row.setAttribute("height", "19px");
    if(j % 2 == 1) row.setAttribute("style", "background-color: #e3e3e3;");
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    if(i < list[0].length) {
      label.setAttribute("value", list[0][i]);
      label.setAttribute("ondblclick", "opendevice(" + list[0][0] +");");
    }
    row.appendChild(label);
    for(var k = 1; k < list.length; k++) {
      if(i < list[k].length) {
        var label = document.createElement('label');
        label.setAttribute("class", "staticdatavaluesmall");
        label.setAttribute("value", list[k][i]);
        row.appendChild(label);
      } else {
        var spacer = document.createElement('spacer');
        row.appendChild(spacer);
      }
    }
    subrows.push(row);
  }
  rows.push(subrows);
}
