/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var myDialog;
var cluster;
var graphs = new Array();
var size = [0, 0];
var timer;
var zoombox = 0;
var saveFile = '';
var recentFiles;
var fillList = new Array();
var addList = new Array();
var waitCount = 0;
var broadcast = -1;
var allJobs = new Object();

var ENABLE_TOOLTIP = false;

var nativeToggleOpenState;

function onLoad() {
  dock = getDock();
  println("Monitoring initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  cluster = myDialog.arguments.cluster;
  
  waitIcon();
  document.getElementById('titlename').setAttribute('value', cluster.name);

  for(var i = 0; i < size[0] * size[1]; i++) {
    var one = new cvGraph(cluster);
    one.ownerWindow = this;
    graphs.push(one);
  }

  recentFiles = new Array();
  var file = dock.BASE_PATH + "monitoring.recent"
  var data = readListFromFile(file);
  var i = 0;
  while(i < data.length) {
    if(addRecent(data[i], false)) {
      i++;
    } else {
      data.splice(i, 1);
    }
  }

  var loaded = false;
  if(dock.generalSettings.defaultMonitoringSetup.length) {
    loaded = loadSetup(dock.generalSettings.defaultMonitoringSetup, false);
  }
  if(!loaded) {
    setSize(1, 3);
    buildbox();
  }
  addEventListener("resize", redraw, false);

  window.setTimeout('showlist();', dock.FILL_TIMEOUT);
}

function mon_settings() {
  var args = new Object();
  var d = new dock.cvDialog("monsettingswindow.xul", args);
  dock.dialogs.showDialog(d);
}

function addRecent(file, save) {
  for(var j = 0; j < recentFiles.length; j++) {
    if(recentFiles[j]['file'] == file) {
      return false;
    }
  }
  var popup = document.getElementById('recent');
  var menu = document.createElement('menuitem');
  if(dock.getDirectorySeparator() == '\\') {
    var i = 0;
    while(i < file.length - 1) {
      if((file[i] == '\\') && (file[i + 1] == '\\')) {
        file = file.substr(0, i) + file.substr(i + 1);
        save = true;
      } else {
        i++;
      }
    }
  }
  menu.setAttribute('label', file);
  var fileSlash = file;
  if(dock.getDirectorySeparator() == '\\') fileSlash = fileSlash.replace(/\\/g, "\\\\");
  menu.setAttribute('oncommand', "loadSetup('" + fileSlash + "')");
  popup.appendChild(menu);

  var x = new Object();
  x['file'] = file;
  x['menu'] = menu;
  recentFiles.push(x);
  if(save) {
    saveRecent();
  }
  return true;
}

function saveRecent() {
  var list = new Array();
  for(var i = 0; i < recentFiles.length; i++) {
    list.push(recentFiles[i]['file']);
  }
  var file = dock.BASE_PATH + "monitoring.recent";
  saveListToFile(file, list);
}

function removeRecent(file) {
  for(var j = 0; j < recentFiles.length; j++) {
    if(recentFiles[j]['file'] == file) {
      var popup = document.getElementById('recent');
      popup.removeChild(recentFiles[j]['menu']);
      recentFiles.splice(j, 1);
      saveRecent();
      return;
    }
  }
}

function showlist() {
  var list = document.getElementById('devices');
  while(list.firstChild) list.removeChild(list.firstChild);

  add(list, "Workload", [null], 'Workload_16.png');
  
  var gd = cluster.find('GenericDevice');
  
  var treepdus = rebranding(dock.TreePowerDistributionUnits, 'Power Distribution Units');
//   var treerack = rebranding(dock.TreeRacks, 'Racks');
  var treesensors = rebranding(dock.TreeSensors, 'rack sensors');
  var treechas = rebranding(dock.TreeChassis, 'Chassis');
//   var treesmp = rebranding(dock.TreeVirtualSMPNodes, 'Virtual SMP Nodes');
//   var treenode = rebranding(dock.TreeNodes, 'Nodes');
//   var treecloud = rebranding(dock.TreeCloudNodes, 'Cloud Nodes');
  var treegpu = rebranding(dock.TreeGPUUnits, 'GPU Units');
  var treeotherdevices = rebranding(dock.TreeOtherDevices, 'generic devices');
  
  add(list, null, cluster.find('MasterNode'), 'Node_16.png');
  add(list, null, cluster.find('SlaveNode'), 'Node_16.png');
  add(list, null, cluster.find('MICNode'), 'Node_16.png');
  add(list, null, cluster.find('EthernetSwitch'), 'Switch_16.png');
  add(list, null, cluster.find('IBSwitch'), 'Switch_16.png');
  add(list, null, cluster.find('MyrinetSwitch'), 'Switch_16.png');
  add(list, null, cluster.find('PowerDistributionUnit'), 'PowerDistributionUnit_16.png');
  add(list, null, cluster.find('RackSensor'), 'RackSensor_16.png');
  add(list, null, gd, 'GenericDevice_16.png');
  add(list, null, cluster.find('Chassis'), 'Chassis_16.png');
  add(list, null, cluster.find('GpuUnit'), 'GpuUnit_16.png');
  
  add(list, 'All head nodes', cluster.find('MasterNode'), 'Category_16.png');
  add(list, 'All nodes', cluster.find('Node'), 'Category_16.png', 0);
  add(list, null, cluster.find('Category'), 'Category_16.png');
  
  function nodeGroupIcon(object) {
    var type = object.get('type');
    if(type == 'STORAGEPAIR') return 'StoragePair_16.png';
    return 'NodeGroup_16.png';
  }
  add(list, null, cluster.find('NodeGroup'), nodeGroupIcon);
  
  var fg = new Array();
  var partitions = cluster.find('Partition');
  for(var i = 0; i < partitions.length; i++) {
    var groups = partitions[i].get('failoverGroups');
    for(var j = 0; j < groups.length; j++) {
      var key = groups[j].uniqueKey;
      if(key > dock.MAX_LOCAL_KEY) {
        var one = new dock.cvObject(cluster, groups[i]);
        // KDR: not ideal, but much easier than creating a new object group. Perhaps easy to fix once we have sub object resolve
        if(cluster.objects.hasOwnProperty(key)) {
          if(cluster.objects[key].get('baseType') == 'CMDaemonFailoverGroup') {
            cluster.objects[key] = one;
            fg.push(one);
          }
        } else {
          cluster.objects[key] = one;
          fg.push(one);
        }
      }
    }
  }
  add(list, null, fg, 'NodeGroup_16.png');
  
  add(list, 'All Ethernet switches', cluster.find('EthernetSwitch'), 'Category_16.png');
  add(list, 'All IB switches', cluster.find('IBSwitch'), 'Category_16.png');
  add(list, 'All Myrinet switches', cluster.find('MyrinetSwitch'), 'Category_16.png');
  
  if(treepdus.length) add(list, 'All ' + treepdus, cluster.find('PowerDistributionUnit'), 'Category_16.png');
  if(treesensors.length) add(list, 'All ' + treesensors, cluster.find('RackSensor'), 'Category_16.png');
  if(treechas.length) add(list, 'All ' + treechas, cluster.find('Chassis'), 'Category_16.png');
  if(treegpu.length) add(list, 'All ' + treegpu, cluster.find('GpuUnit'), 'Category_16.png');

  if(treeotherdevices.length) add(list, 'All ' + treeotherdevices, gd, 'Category_16.png');
  
  var models = new Object();
  for(var i = 0; i < gd.length; i++) {
    var model = gd[i].get('model');
    if(model.length > 0) {
      if(models.hasOwnProperty(model)) models[model].push(gd[i]);
      else models[model] = [gd[i]];
    }
  }
  for(var model in models) {
    add(list, 'All ' + model + ' '  + treeotherdevices, models[model], 'Category_16.png');
  }
  
//   if(dock.generalSettings.monitoringAutoFill) {
//     window.setTimeout('fillNext(0);', dock.FILL_TIMEOUT);
//   } else {
//     fillList = new Array();
//     unwaitIcon();
//   }
}

function stopAll() {
  for(var i = 0; i < size[0] * size[1]; i++) {
    if(graphs[i].deviceMetric.length > 0) {
      graphs[i].stop();
      document.getElementById('play' + i).setAttribute('state', 'play');
    }
  }
}
function startAll() {
  for(var i = 0; i < size[0] * size[1]; i++) {
    if(graphs[i].deviceMetric.length > 0) {
      graphs[i].start();
      document.getElementById('play' + i).setAttribute('state', 'pause');
    }
  }
}

function clearAll() {
  for(var i = 0; i < size[0] * size[1]; i++) {
    clearAllMetrics(i);
  }
  saveFile = '';
}

function setSize(x, y) {
  if((x != size[0]) || (y != size[1])) {
    broadcast = -1;
    var n = 0;
    for(var i = 0; i < graphs.length; i++) {
      graphs[i].broadcast = false;
      if(graphs[i].deviceMetric.length > 0) n++;
    }
    if(n > x * y) {
      dock.displayInfo("The number of active monitoring panels is larger than the desired size. Close some monitoring panels before resizing.", window);
      return false;
    } else {
      if(size[0] * size[1] > x * y) {
        var j = x * y - 1;
        var i = size[0] * size[1] - 1;
        while((i > j) && (i >= 0) && (j >= 0)) {
          if(graphs[j].deviceMetric && (graphs[j].deviceMetric.length > 0)) {
            j--;
          } else if(!graphs[i].deviceMetric || (graphs[i].deviceMetric.length == 0)) {
            i--;
          } else {
            println("SWAP " + j + " --> " + i);
            var tmp = graphs[j];
            graphs[j] = graphs[i];
            graphs[i] = tmp;
            i--;
            j--;
          }
        }
        for(var i = x * y; i < size[0] * size[1]; i++) {
          graphs[i].stop();
        }
        graphs.splice(x * y, size[0] * size[1] - x * y);
        println(x * y + " =?= " + graphs.length);
      } else {
        for(var i = size[0] * size[1]; i < x * y; i++) {
          var one = new cvGraph(cluster);
          one.ownerWindow = this;
          graphs.push(one);
        }
      }
      size[0] = x;
      size[1] = y;
      buildbox();
      redraw();
    }
  }
  return true;
}

function cvMonFill(child, metricClass, selected, devices, mmc, mmh, startTime, endTime, cgroups, user) {
  this.child = child;
  this.metricClass = metricClass;
  this.selected = selected;
  this.devices = devices;
  this.mmc = mmc;
  this.mmh = mmh;
  this.startTime = startTime;
  this.endTime = endTime;
  this.cgroups = isundefined(cgroups, 0);
  this.user = isundefined(user, 0);
  this.mmci = 0;
  this.mmhi = 0;  
  this.looping = false;
  this.men = new Object();
}

cvMonFill.prototype.multipleCGroups = function() {
  return this.user;
}

cvMonFill.prototype.fillOne = function(text, metric, param, translatedParam, icon, metricClass) {
  var subitem = document.createElement("treeitem");
  subitem.setAttribute("key", this.selected);
  subitem.setAttribute("devices", this.devices);
  subitem.setAttribute("metric", metric);
  subitem.setAttribute("param", param);
  subitem.setAttribute("translatedParam", translatedParam);
  subitem.setAttribute("startTime", this.startTime);
  subitem.setAttribute("endTime", this.endTime);
  subitem.setAttribute("cgroups", this.cgroups);
  subitem.setAttribute("user", this.user);

  var row = document.createElement("treerow");
  var cell = document.createElement("treecell");
  cell.setAttribute("src", "chrome://cmgui/content/icons/" + icon);
  cell.setAttribute("label", " " + text); // space between image and label, css doesn't work
  row.appendChild(cell);
  subitem.appendChild(row);
  if(this.metricClass.hasOwnProperty(metricClass)) {
    var mclass = this.metricClass[metricClass];
    if(mclass) {
      mclass[3][text] = subitem;
      mclass[2]++;
      while(mclass[4].length && this.metricClass.hasOwnProperty(mclass[4])) {
        mclass = this.metricClass[mclass[4]];
        mclass[2]++;
      }
    }
  }
}

cvMonFill.prototype.createMonitoringFolder = function(parent, text, icon, group, count, extra) {
  if(!this.metricClass.hasOwnProperty(group)) {
    count = isundefined(count, 0);
    var item = document.createElement("treeitem");
    var row = document.createElement("treerow");
    var cell = document.createElement("treecell");
    cell.setAttribute("src", "chrome://cmgui/content/icons/" + icon);
    cell.setAttribute("label", " " + text); // space between image and label, css doesn't work
    row.appendChild(cell);
    item.appendChild(row);
    if(count >= 0) item.setAttribute("container", "true");
    if(typeof(extra) != 'undefined') {
      for(var f in extra) {
        item.setAttribute(f, extra[f]);
      }
    }
    item.setAttribute("open", "false");
    var child = document.createElement("treechildren");
    item.appendChild(child);
    this.metricClass[group] = [item, child, count, new Object(), parent];
    println("Add fake metric class for: '" + group + "' parent: '" + parent + "'");
  }
  return group;
}

cvMonFill.prototype.fakeClass = function(param, translatedParam, metricClass) {
  if(this.metricClass.hasOwnProperty(metricClass)) {
    if(translatedParam.length > 0) {
      if((translatedParam.indexOf('/') > 0) && (metricClass.indexOf('_VM_') > 0)) {
// metric <mymetric1>:<tenant>/<instance>
// class = openstack/VM/<something>
// 
// metric <mymetric2>:<tenant>/<instance>
// class = openstack/VM/<somethingelse>
// 
// node001
// - openstack
// -- VM
// --- <tenant>
// ---- <instance>
// ----- <something>
// ------ <mymetric1>
// ----- <somethingelse>
// ------ <mymetric2>
        var st = translatedParam.split('/');
        var sc = metricClass.split('_');
        var scn = sc[2];
        if(this.men.hasOwnProperty(metricClass)) scn = this.men[metricClass].split('/')[2];
        
        var group = translatedParam + '/' + sc[2];
        
        this.createMonitoringFolder("OPENSTACK_VM", st[0], 'Group_16.png', st[0]);
        this.createMonitoringFolder(st[0], st[1], 'Node_16.png', translatedParam);
        return this.createMonitoringFolder(translatedParam, scn, 'Category_16.png', group);
      } else {
        return this.createMonitoringFolder(metricClass, translatedParam, 'Category_16.png', translatedParam);
      }
    } else if((param.length > 0) && (metricClass == 'OPENSTACK_HYPERVISORS')) {
      return this.createMonitoringFolder(metricClass, param, 'Node_16.png', param);
    } else if(isOpenStackTenantId(param)) {
      var noLonger = 'Unknown tenants';
      this.createMonitoringFolder(metricClass, noLonger, 'Folder_16.png', noLonger);
      return this.createMonitoringFolder(noLonger, param, 'Category_16.png', param);
    }
  }
  return metricClass;
}

function stripCGroupParam(param) {
  var idx = param.indexOf(':');
  if(idx > 0) return param.substr(idx + 1);
  return "";
}

cvMonFill.prototype.fill = function() {
  var multipleCGroups = this.multipleCGroups();
  var n = 0;
  for(; this.mmci < this.mmc.length; this.mmci++) {
    var metric = cluster.find(this.mmc[this.mmci].metric);
    if(metric) {
      var text = metric.resolveName();
      var mclass = metric.get('metricClass');
      var param = this.mmc[this.mmci].metricParam;
      var translated = this.mmc[this.mmci].translatedMetricParam;
      text += metricParamText(mclass, param, translated);      
      if(metric.get('unit')) text += " (" + metric.get('unit') + ")";
      text += osdText(mclass, param, this.selected);      
      var mclass = this.fakeClass(param, translated, mclass);
      if(multipleCGroups && startswith(mclass, 'CGROUPS_')) param = stripCGroupParam(param);
      this.fillOne(text, this.mmc[this.mmci].metric, param, translated, 'Monitoring_16.png', mclass);
    } else {
      println("Metric not found: " + this.mmc[this.mmci].metric);
    }    
    ++n;
    if(this.looping && (n > dock.FILL_COUNT)) {
      this.loop();
      return;
    }
  }
  for(; this.mmhi < this.mmh.length; this.mmhi++) {  
    var check = cluster.find(this.mmh[this.mmhi].healthcheck);
    if(check) {
      var text = check.resolveName();
      var mclass = check.get('healthCheckClass');
      var param = this.mmh[this.mmhi].healthcheckParam;
      var translated = this.mmh[this.mmhi].translatedHealthcheckParam;
      text += metricParamText(mclass, param, translated);
      text += osdText(mclass, param, this.selected);
      var mclass = this.fakeClass(param, translated, mclass);
      if(multipleCGroups && startswith(mclass, 'CGROUPS_')) param = stripCGroupParam(param);
      this.fillOne(text, this.mmh[this.mmhi].healthcheck, param, translated, 'Health_16.png', mclass);
    } else {
      println("health check not found: " + this.mmh[this.mmhi].healthcheck);
    }
    ++n;
    if(this.looping && (n > dock.FILL_COUNT)) {
      this.loop();
      return;
    }
  }
  for(var mclass in this.metricClass) {
    if(this.metricClass[mclass][2] != 0) {
      var add = this.metricClass[mclass][1];
      var work = this.metricClass[mclass][3];
      var parent = this.metricClass[mclass][4];
      if(parent.length && this.metricClass.hasOwnProperty(parent)) {
//         println(' +++ add ' + mclass + ' to ' + parent);
        this.metricClass[parent][1].appendChild(this.metricClass[mclass][0]);
      } else {
//         println(' +++ add ' + mclass);
        this.child.appendChild(this.metricClass[mclass][0]);
      }
      var keys = Object.keys(work).sort();
      for(var i = 0; i < keys.length; i++) {
        add.appendChild(work[keys[i]]);
      }
    }
  }
  if(this.looping) unwaitIcon();
  this.looping = false;
}

cvMonFill.prototype.loop = function() {
  if(!this.looping) waitIcon();
  this.looping = true;
  window.setTimeout(createObjectCallback(this, this.fill), dock.FILL_TIMEOUT);  
}

function populateMetrics(child, selected, devices, startTime, endTime, men, cgroup, user) {
  var m = 0;
  var monconf = new Array();
  var metricsOnly = false;
  cgroup = isundefined(cgroup, '');
  
  if(selected > 0) {
    var work = cluster.find(selected);
    monconf = work.getAllMonitoringConfigurations();
  } else if(isArray(devices) && devices.length > 0) {
    for(var i = 0; i < devices.length; i++) {
      var device = cluster.find(devices[i]);
      if(device) {
        monconf.push(device.getMonitoringConfiguration());
      }
    }
  } else if(selected == 0 || isNaN(selected)) { // all slaves / job based
    var categories = cluster.find('NodeCategory');
    for(var i = 0; i < categories.length; i++) {
      var mcid = cluster.find(categories[i].get('monConfId'));
      if(mcid) monconf.push(mcid);
    }
  } else {
    return 0;
  }
  if(monconf.length > 0 && monconf[0]) {
    var mmc = monconf[0].get('metrics');
    var mmh = monconf[0].get('checks');
    if(monconf.length > 1) {
      // make a clone
      mmc = mmc.slice(0);
      mmh = mmh.slice(0);
      var mapM = new Object();
      var mapH = new Object();
      for(var i = 0; i < mmc.length; i++) {
        mapM[mmc[i].metric + ':' + mmc[i].metricParam] = 1;
      }
      for(var i = 0; i < mmh.length; i++) {
        mapH[mmh[i].healthcheck + ':' + mmh[i].healthcheckParam] = 1;
      }
      for(var j = 1; j < monconf.length; j++) {
        var mmc2 = monconf[j].get('metrics');
        for(var i = 0; i < mmc2.length; i++) {
          var name = mmc2[i].metric + ':' + mmc2[i].metricParam;
          if(!mapM.hasOwnProperty(name)) {
            mmc.push(mmc2[i]);
            mapM[name] = 1;
          }
        }
        var mmh2 = monconf[j].get('checks');
        for(var i = 0; i < mmh2.length; i++) {
          var name = mmh2[i].healthcheck + ':' + mmh2[i].healthcheckParam;
          if(!mapH.hasOwnProperty(name)) {
            mmh.push(mmh2[i]);
            mapH[name] = 1;
          }
        }
      }
    }
    if(cgroup) {
      var list;
      if(isArray(cgroup)) {
        list = cgroup;
      } else {
        list = [cgroup];
      }
      var mmc_filtered;
      var mmh_filtered;
      if(dock.generalSettings.jobOnlyCgroup) {
        mmc_filtered = new Array();
        mmh_filtered = new Array();
      } else {
        var cgroup_metrics = cluster.getCGroupMetrics();
        mmc_filtered = mmc.filter(metricNotInList, cgroup_metrics);
        mmh_filtered = mmh.filter(checkNotInList, cgroup_metrics);
        println("Metrics for non-cgroup, mmc: " + mmc_filtered.length + ", mmh: " + mmh_filtered.length);
      }
      for(var i = 0; i < list.length; i++) {
        var one = list[i];
        var mmc_one = mmc.filter(metricInCGroup, one);
        var mmh_one = mmh.filter(checkInCGroup, one);
        println("Metrics for cgroup: " + one + " (" + i + "/" + list.length + ") mmc: " + mmc_one.length + ", mmh: " + mmh_one.length);
        Array.prototype.push.apply(mmc_filtered, mmc_one);
        Array.prototype.push.apply(mmh_filtered, mmh_one);
      }
      mmc = mmc_filtered;
      mmh = mmh_filtered;
      cgroup = JSON.stringify(list);
    }
    if((mmc.length > 0) || (mmh.length > 0)) {
      if(typeof(men) == 'undefined') {
        println(" +++ men undefined");
        men = cluster.metricEnumName(true, false);
      }
      var metricClass = buildMetricClass('rack', cluster, men, true);
      var filler = new cvMonFill(child, metricClass, selected, devices, mmc, mmh, startTime, endTime, cgroup, user);
      for(var i = 0; i < men.length; i++) {
        filler.men[men[i][0]] = men[i][1];
      }
      filler.loop();
    }
  }
  return 1;
}

function cvAddFill(list, name, devices, image, startTime, endTime, autoselect) {
  this.list = list;
  this.name = name;
  this.devices = devices;
  this.image = image;
  this.startTime = isundefined(startTime, 0);
  this.endTime = isundefined(endTime, 0);
  this.autoselect = isundefined(autoselect, false);
  this.index = 0;
}

cvAddFill.prototype.add = function(max) {
  if(this.devices === null) return 0;
  
  var m = 0;
  while(this.index < this.devices.length) {
    var item = document.createElement("treeitem");
    this.list.appendChild(item);
    var row = document.createElement("treerow");
    var cell = document.createElement("treecell");
    if(typeof(this.image) == 'function') {      
      cell.setAttribute("src", "chrome://cmgui/content/icons/" + this.image(this.devices[this.index]));      
    } else {
      cell.setAttribute("src", "chrome://cmgui/content/icons/" + this.image);
    }
    row.appendChild(cell);
    item.appendChild(row);
//     item.setAttribute("container", "true");
    item.setAttribute("open", this.autoselect);
    var child = document.createElement("treechildren");
    item.appendChild(child);
    item.setAttribute("startTime", this.startTime);
    item.setAttribute("endTime", this.endTime);
    if(this.name === null) {
      fillList.push(this.devices[this.index].get('uniqueKey'));
      item.setAttribute("id", this.devices[this.index].get('uniqueKey'));
      cell.setAttribute("label", " " + this.devices[this.index].resolveName()); // space between image and label, css doesn't work
    } else {
      fillList.push(this.name);
      item.setAttribute("id", this.name);
      if(this.devices.length == 1 && this.devices[0] === null) {
        item.setAttribute("workload", true);
      } else {
        var deviceKeys = [];
        for(var i = 0; i < this.devices.length; i++) {
          deviceKeys.push(this.devices[i].get('uniqueKey'));
        }
        item.setAttribute("devices", deviceKeys);
      }
      cell.setAttribute("label", " " + this.name); // space between image and label, css doesn't work
      this.index = this.devices.length; // all done
    }
    if(this.autoselect) {
      var clusters = document.getElementById("treecluster");
      var index = clusters.contentView.getIndexOfItem(item);
      if(index >= 0) {
        clusters.contentView.selection.clearSelection();
        clusters.contentView.selection.select(index);
      }
    }
    this.index++;
    m++;
    if(m > max) break;
  }
  return m;
}

cvAddFill.prototype.done = function() {
  return (this.index >= this.devices.length);
}

function add(list, name, devices, image, startTime, endTime, autoselect) {
  var n = addList.length;
  var adder = new cvAddFill(list, name, devices, image, startTime, endTime, autoselect);
  addList.push(adder);
  if(n == 0) window.setTimeout('addNext(0);', dock.FILL_TIMEOUT);
}

function redraw(index) {
  if(typeof(index) == "number") {
    if(isArray(graphs[index].deviceMetric) && (graphs[index].deviceMetric.length > 0)) {
      graphs[index].update();
    }
  } else {
    for(var i = 0; i < graphs.length; i++) {
      if(isArray(graphs[i].deviceMetric) && (graphs[i].deviceMetric.length > 0)) {
        graphs[i].svg.setAttribute("height", "25px");
      }
    }
    for(var i = 0; i < graphs.length; i++) {
      if(isArray(graphs[i].deviceMetric) && (graphs[i].deviceMetric.length > 0)) {
        graphs[i].update();
      }
    }
  }
}

function replaceOne(n) {
  var row = Math.floor(n / size[0]);
  var index = n - row * size[0];
  var row = document.getElementById('row' + row);
  row.replaceChild(addOne(n), row.childNodes[index]);
  redraw(n);
}

function clearAllMetrics(n) {
  if(graphs[n].deviceMetric.length > 0) {
    graphs[n].clear();
    graphs[n].broadcast = false;
    if(broadcast == n) {
      broadcast = -1;
    }
  }
}

function playPause(n) {
  if(graphs[n].deviceMetric.length > 0) {
    if(graphs[n].playPause) {
      graphs[n].stop();
    } else {
      graphs[n].start();
    }
    if(graphs[n].playPause) {
      graphs[n].update();
      document.getElementById('play' + n).setAttribute('state', 'pause');
    } else {
      document.getElementById('play' + n).setAttribute('state', 'play');
    }
  } else {
    dock.displayInfo("No metrics/checks defined", window);
  }
}

function zoomIn(n) {
  if(graphs[n].deviceMetric.length > 0) {
    graphs[n].zoomIn();
    graphs[n].update();
  } else {
    dock.displayInfo("No metrics/checks defined", window);
  }
}

function zoomOut(n) {
  if(graphs[n].deviceMetric.length > 0) {
    graphs[n].zoomOut();
    graphs[n].update();
  } else {
    dock.displayInfo("No metrics/checks defined", window);
  }
}

function showProperties(n) {
  if(graphs[n].deviceMetric.length > 0) {
    graphs[n].showProperties();
  } else {
    dock.displayInfo("No metrics/checks defined", window);
  }
}

function setBroadcast(n) {
  if(graphs[n].deviceMetric.length > 0) {
    if(broadcast >= 0) {
      var button = document.getElementById('broadcast' + broadcast);
      if(button) button.removeAttribute('on');
      var graph = document.getElementById('graph' + broadcast);
      if(graph) graph.removeAttribute('broadcast');
      var bottom = document.getElementById('bottom' + broadcast);
      if(bottom) bottom.removeAttribute('broadcast');
      graphs[n].broadcast = false;
    }
    if(n == broadcast) {
      broadcast = -1;
    } else {
      broadcast = n;
      var button = document.getElementById('broadcast' + broadcast);
      if(button) button.setAttribute('on', true);
      var graph = document.getElementById('graph' + broadcast);
      if(graph) graph.setAttribute('broadcast', true);
      var bottom = document.getElementById('bottom' + broadcast);
      if(bottom) bottom.setAttribute('broadcast', true);
      graphs[n].broadcast = true;
    }
  } else {
    dock.displayInfo("No metrics/checks defined", window);
  }
}

function setZoomBroadcast(index) {
  var xzoom = graphs[index].xzoom[graphs[index].xzoom.length - 1];
  var yzoom = graphs[index].yzoom[graphs[index].yzoom.length - 1];
  for(var i = 0; i < graphs.length; i++) {
    if((i != index) && (graphs[i].deviceMetric.length > 0)) {
      graphs[i].xzoomBroadcast = xzoom;
      graphs[i].yzoomBroadcast = yzoom;
      graphs[i].last = 0;
      graphs[i].update();
    }
  }
}

function addOne(n) {
  var box = document.createElement('vbox');
  box.setAttribute('flex', '1');
  box.setAttribute("graphindex", n);
  box.setAttribute("dropzone", true);
  graphs[n].index = n;
  if(isArray(graphs[n].deviceMetric) && (graphs[n].deviceMetric.length > 0)) {
    var box3 = document.createElement('vbox');
    box3.setAttribute('flex', '1');
    box3.setAttribute('class', 'monitoringtop');
    box3.setAttribute('id', 'graph' + n);
    if(broadcast == n) box3.setAttribute('broadcast', true);
    var box2 = document.createElement('hbox');
    box2.setAttribute('overflow', 'auto');
    box2.setAttribute('pack', 'end');
    var spacer = document.createElement('spacer');
    spacer.setAttribute('width', '20px'); // 37px to align
    box2.appendChild(spacer);
    var label = document.createElement('shadowlabel');
    label.setAttribute('class', 'staticdataheader');
    label.setAttribute('id', 'title' + n);
    label.setAttribute('flex', '1');
    label.setAttribute('style', 'overflow: hidden;');
    box2.appendChild(label);
    var button = document.createElement('button');
    button.setAttribute('class', 'closepanel-button');
    button.setAttribute('tooltiptext', 'Clear all');
    button.setAttribute('oncommand', 'clearAllMetrics(' + n + ')');
    box2.appendChild(button);
    box3.appendChild(box2);

    var svg = document.createElementNS(XUL_SVG, "svg:svg");
    svg.setAttribute('class', 'graphbox');
    var svgg = document.createElementNS(XUL_SVG, "svg:g");
    svgg.setAttribute('style', 'fill-opacity:0.50;');

    svg.appendChild(svgg);
    box3.appendChild(svg);
    graphs[n].svg = svg;
    graphs[n].box = svgg;
    graphs[n].tooltip = 0;
    graphs[n].tooltiptext = '';
    graphs[n].zoombox = 0;
    zoomBox(n);

    var box2 = document.createElement('hbox');
    box2.setAttribute('align', 'center');
    box2.setAttribute('class', 'monitoringbottom');
    box2.setAttribute('id', 'bottom' + n);
    if(broadcast == n) box2.setAttribute('broadcast', true);

    var label = document.createElement('label');
    label.setAttribute('id', 'info' + n);
    box2.appendChild(label);
    var spacer = document.createElement('spacer');
    spacer.setAttribute('flex', '1');
    box2.appendChild(spacer);

    var button = document.createElement('button');
    button.setAttribute('id', 'play' + n);
    if(graphs[n].playPause) {
      button.setAttribute('state', 'pause');
    } else {
      button.setAttribute('state', 'play');
    }
    button.setAttribute('class', 'pauseplay-button');
    button.setAttribute('tooltiptext', 'Play/Pause');
    button.setAttribute('oncommand', 'playPause(' + n + ');');
    box2.appendChild(button);

    var button = document.createElement('button');
    button.setAttribute('class', 'zoomin-button');
    button.setAttribute('tooltiptext', 'Zoom in');
    button.setAttribute('oncommand', 'zoomIn(' + n + ');');
    box2.appendChild(button);
    var button = document.createElement('button');
    button.setAttribute('class', 'zoomout-button');
    button.setAttribute('tooltiptext', 'Zoom out');
    button.setAttribute('oncommand', 'zoomOut(' + n + ');');
    box2.appendChild(button);

    var button = document.createElement('button');
    button.setAttribute('class', 'broadcast-button');
    button.setAttribute('tooltiptext', 'broadcast');
    button.setAttribute('id', 'broadcast' + n);
    if(broadcast == n) button.setAttribute('on', true);
    button.setAttribute('oncommand', 'setBroadcast(' + n + ');');
    box2.appendChild(button);

    var button = document.createElement('button');
    button.setAttribute('class', 'properties-button');
    button.setAttribute('tooltiptext', 'Properties');
    button.setAttribute('oncommand', 'showProperties(' + n + ');');
    box2.appendChild(button);

    if(startswith(getOS(), "WIN")) {
      var button = document.createElement('spacer');
      button.setAttribute('width', '4px');
      box2.appendChild(button);
    }

    box.appendChild(box3);
    box.appendChild(box2);
  } else {
    box.setAttribute('class', 'staticdata');
    box.setAttribute('align', 'center');
    var box2 = document.createElement('hbox');
    box2.setAttribute('flex', '1');
    box2.setAttribute('align', 'center');
    var image = document.createElement('image');
    image.setAttribute('src', './skin/gradient/drophere.png');
    box2.appendChild(image);
    box.appendChild(box2);
  }
  return box;
}

function buildbox() {
  var cols = document.getElementById('graphbox');
  while(cols.firstChild) cols.removeChild(cols.firstChild);
  var n = 0;
  for(var j = 0; j < size[1]; j++) {
    var row = document.createElement('hbox');
    row.setAttribute('id', 'row' + j);
    row.setAttribute('equalsize', 'always');
    row.setAttribute('flex', '1');
    for(var i = 0; i < size[0]; i++) {
      row.appendChild(addOne(n));
      n++;
    }
    cols.appendChild(row);
  }
}

function zoomBox(n) {
  var index = n;
  var startPos = 0;
  var mouseDown = function(event) {
    if (event.button == 0) {
      var element = event.target;
      while(!isObject(element.boxObject)) element = element.parentNode;
      var scroll = element.parentNode.parentNode.parentNode;
      var elementX = event.clientX - element.boxObject.x + PX + scroll.scrollLeft;
      var elementY = event.clientY - element.boxObject.y + PY + scroll.scrollTop;
      startPos = [ elementX, elementY];
    } else {
      startPos = 0;
    }
    return false;
  }

  var mouseMove = function(event) {
    var element = event.target;
    while(!isObject(element.boxObject)) element = element.parentNode;
    var scroll = element.parentNode.parentNode.parentNode;
    var elementX = event.clientX - element.boxObject.x + PX + scroll.scrollLeft;
    var elementY = event.clientY - element.boxObject.y + PY + scroll.scrollTop;
    var info = graphs[index].info(elementX, elementY);
    document.getElementById('info' + index).setAttribute('value', info[0]);
    if(ENABLE_TOOLTIP) {
      if(info[1].length) {
        if((graphs[index].tooltiptext != info[1])) {
          if(graphs[index].tooltip) graphs[index].box.removeChild(graphs[index].tooltip);
          var svgt = document.createElementNS(XUL_SVG, "svg:g");
          svgt.setAttribute('opacity', '0.7');
    //       svgt.setAttribute('visibility', 'hidden');
    //       svgt.setAttribute('pointer-events', 'none');
          var svgr = document.createElementNS(XUL_SVG, "svg:rect");
          svgr.setAttribute('x', '0');
          svgr.setAttribute('y', '5');
          svgr.setAttribute('width', '200');
          svgr.setAttribute('height', '20');
          svgr.setAttribute('rx', '2');
          svgr.setAttribute('ry', '2');
          svgr.setAttribute('stroke', 'white');
          svgr.setAttribute('fill', 'yellow');
  /*        var svg1 = document.createElementNS(XUL_SVG, "svg:text");
          svg1.setAttribute('x', '5');
          svg1.setAttribute('y', '20');
          svg1.setAttribute("font-size", 11);
          svg1.setAttribute('fill','black');
          println(info[1]);
          svg1.appendChild(document.createTextNode(info[1]));
          svgr.appendChild(svg1)*/
          svgt.appendChild(svgr);
          graphs[index].box.appendChild(svgt);
          svgtext(svgt, 5, 18, info[1]);

          //return rectangle around text as SVGRect object
          var outline = svgr.getBBox();
          svgt.setAttribute('width', Number(outline.width) + 10);
          svgt.setAttribute('height', Number(outline.height) + 10);
          svgt.setAttribute('transform', 'translate(' + elementX + ',' + elementY + ')');

          graphs[index].tooltip = svgt;
          graphs[index].tooltiptext = info[1];
  //         graphs[index].tooltip = svgtext(graphs[index].box, elementX, elementY, info[1]);
        }
      } else if(graphs[index].tooltip) {
        graphs[index].box.removeChild(graphs[index].tooltip);
        graphs[index].tooltip = 0;
        graphs[index].tooltiptext = '';
      }
    }

    if(startPos != 0) {
      var zoom = graphs[index].zoombox;
      if(zoom) {
        while(zoom.firstChild) zoom.removeChild(zoom.firstChild);
      } else {
        zoom = document.createElementNS(XUL_SVG, "svg:g");
        graphs[index].zoombox = zoom;
        graphs[index].box.appendChild(zoom);
      }
      if(Math.abs(elementX - startPos[0]) <= MIN_ZOOM_BOX) {
        if(Math.abs(elementY - startPos[1]) <= MIN_ZOOM_BOX) {
          svgline(zoom, startPos[0] - MIN_ZOOM_BOX, startPos[1], startPos[0] + MIN_ZOOM_BOX, startPos[1], ZOOM_COLOR);
          svgline(zoom, startPos[0], startPos[1] - MIN_ZOOM_BOX, startPos[0], startPos[1] + MIN_ZOOM_BOX, ZOOM_COLOR);
        } else {
          svgline(zoom, startPos[0] - MIN_ZOOM_BOX, startPos[1], startPos[0] + MIN_ZOOM_BOX, startPos[1], ZOOM_COLOR);
          svgline(zoom, startPos[0], startPos[1], startPos[0], elementY, ZOOM_COLOR);
          svgline(zoom, startPos[0] - MIN_ZOOM_BOX, elementY, startPos[0] + MIN_ZOOM_BOX, elementY, ZOOM_COLOR);
        }
      } else if(Math.abs(elementY - startPos[1]) < MIN_ZOOM_BOX) {
        svgline(zoom, startPos[0], startPos[1] - MIN_ZOOM_BOX, startPos[0], startPos[1] + MIN_ZOOM_BOX, ZOOM_COLOR);
        svgline(zoom, startPos[0], startPos[1], elementX, startPos[1], ZOOM_COLOR);
        svgline(zoom, elementX, startPos[1] - MIN_ZOOM_BOX, elementX, startPos[1] + MIN_ZOOM_BOX, ZOOM_COLOR);
      } else {
        svgrect(zoom, startPos[0], startPos[1], elementX - startPos[0], elementY - startPos[1], '#555555', 1, ZOOM_COLOR);
      }
    }
  }

  var mouseUp = function(event) {
    if(startPos) {
      var zoom = graphs[index].zoombox;
      if(zoom) {
        var element = event.target;
        while(element && !isObject(element.boxObject)) element = element.parentNode;
        if(element) {
          var scroll = element.parentNode.parentNode.parentNode;
          var elementX = event.clientX - element.boxObject.x + PX + scroll.scrollLeft;
          var elementY = event.clientY - element.boxObject.y + PY + scroll.scrollTop;
          if((Math.abs(elementX - startPos[0]) > MIN_ZOOM_BOX) || (Math.abs(elementY - startPos[1]) > MIN_ZOOM_BOX)) {
            graphs[index].zoomIn(startPos[0], startPos[1], elementX, elementY);
            try {
              graphs[index].box.removeChild(zoom);
            } catch(e) {}
            graphs[index].zoombox = 0;
            graphs[index].update();
          } else {
            try {
              graphs[index].box.removeChild(zoom);
            } catch(e) {}
            graphs[index].zoombox = 0;
          }
        }
      }
      startPos=0;
    }
  }

  if(ENABLE_TOOLTIP) {
    var mouseOut = function(event) {
      if(graphs[index].tooltip) {
        graphs[index].box.removeChild(graphs[index].tooltip);
        graphs[index].tooltip = 0;
        graphs[index].tooltiptext = '';
      }
    }
  }

  graphs[index].box.addEventListener("mousedown",mouseDown, false);
  graphs[index].box.addEventListener("mousemove",mouseMove, false);
  if(ENABLE_TOOLTIP) {
    graphs[index].box.addEventListener("mouseout",mouseOut, false);
  }
  window.addEventListener("mouseup",mouseUp, false);
}

function loadSetup(file, recent) {
  recent = isundefined(recent, true);
  if(typeof(file) == 'undefined') {
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    fp.init(window, "Load monitoring setup", nsIFilePicker.modeOpen);
    var path = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
    path.initWithPath(dock.BASE_PATH);
    fp.displayDirectory = path;
    fp.appendFilter("Monitoring setups","*.mon");
    var res = fp.show();
    if((res == nsIFilePicker.returnOK)) {
      file = fp.file.path;
    } else {
      return;
    }
  }
  var data = readListFromFile(file);
  if(data.length < 2) {
    removeRecent(file);
  } else if(cluster.url == data[0]) {
    eval('size = ' + data[1]);
    data.splice(0, 2);
    graphs = new Array();
    for(var i = 0; i < size[0] * size[1]; i++) {
      var one = new cvGraph(cluster);
      one.ownerWindow = this;
      if(!one.load(data)) {
        print(data);
      }
      graphs.push(one);
      if(one.playPause) one.start();
    }
    buildbox();
    redraw();
    saveFile = file;
    if(recent) addRecent(saveFile, true);
    return file;
  } else {
    dock.displayInfo("This monitoring setup does not belong to this cluster", window);
  }
  return false;
}

function setDefaultSetup() {
  var file = loadSetup();
  if(!file) return;
  dock.generalSettings.defaultMonitoringSetup = file;
}

function clearDefaultSetup() {
  dock.generalSettings.defaultMonitoringSetup = '';  
}

function saveSetup(saveas) {
  var file = saveFile;
  if(saveas || (saveFile.length == 0)) {
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    if(saveas) {
      fp.init(window, "Save monitoring setup as", nsIFilePicker.modeSave);
    } else {
      fp.init(window, "Save monitoring setup", nsIFilePicker.modeSave);
    }
    var path = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
    path.initWithPath(dock.BASE_PATH);
    fp.displayDirectory = path;
    fp.appendFilter("Monitoring setups","*.mon");
    var res = fp.show();
    if((res == nsIFilePicker.returnOK) || (res == nsIFilePicker.returnReplace)) {
      file = fp.file.path;
      if(!endswith(file, '.mon')) file += '.mon';
    } else {
      return;
    }
  }
  var data = cluster.url + "\n";
  data += size.toSource() + "\n";
  for(var i = 0; i < graphs.length; i++) {
    data += graphs[i].save();
  }
  println(file);
  savefile(file, data);
  saveFile = file;
  addRecent(saveFile, true);
}

function querySize() {
  var args = new Object();
  args['title'] = 'Monitoring';
  args['question'] = 'Grid size';
  args['default'] = size[0] + ' ' + size[1];
  args['known'] = new Array();
  var d = new dock.cvDialog("newobject.xul", args);
  d.parentWindow = window;
  d.onOk = function(name) {
    var x = name.split(' ');
    if((x.length == 2) || isNumeric[x[0]] || isNumeric[x[1]]) {
      setSize(parseInt(x[0]), parseInt(x[1]));
    } else {
      dock.displayInfo("Invalid size, specify: 'x y'", window);
    }
  }
  dock.dialogs.show(d);
}

function fillResourceUserDay(statistics, item) {
  var metricClass = {'WORKLOAD': [item, item.lastChild, 0, new Object(), 0]};
  var filler = new cvMonFill(item.lastChild, {}, [], [], [], []);
  var users = Object();
  for(var i = 0; i < statistics.length; i++) {
    var user = statistics[i].user;
    if(!users.hasOwnProperty(user)) {
      users[user] = filler.createMonitoringFolder("WORKLOAD", user, 'User_16.png', "WORKLOAD_" + user, 1);
      filler.createMonitoringFolder("WORKLOAD_" + user, "Total", 'sum.png', "WORKLOAD_" + user + "_TOTAL", -1, { 'workload_user': user, 'workload_total': true });
    }
  }
  for(var i = 0; i < statistics.length; i++) {
    var user = statistics[i].user;
    var t = statistics[i].intervalStart;
    var date = formatDateShort(t);
    
    filler.createMonitoringFolder("WORKLOAD_" + user, date, 'calendar.png', "WORKLOAD_" + user + "_" + t, -1, { 'workload_user': user, 'workload_time': t });
  }  
  unwaitIcon();
  filler.loop();
}

function fillResourceUserDayJobs(jobs, item) {
  var metricClass = {'DAY': [item, item.lastChild, 0, new Object(), 0]};
  var filler = new cvMonFill(item.lastChild, {}, [], [], [], []);
  for(var i = 0; i < jobs.length; i++) {
    if(jobs[i].cgroup.length == 0) {
      println(" +++ fillResourceUserDayJobs, no cgroup: " + jobs[i].jobId);
      continue;
    }
    var wlm = jobs[i].wlm;
    var jobId = jobs[i].jobId;
    var name = jobs[i].jobName + ' (' + jobId + ')';
    var id = wlm + '_' + jobId; 
    allJobs[id] = jobs[i];
    var icon;
    if(jobs[i].endTime) {
      if(jobs[i].exitCode) {
        icon = 'application_error.png';
      } else {
        icon = 'application_ok.png';
      }
    } else if(jobs[i].startTime) {
      icon = 'application_run.png';
    } else {
      icon = 'application_new.png';
    }
    filler.createMonitoringFolder("DAY", name, icon, "JOB_" + jobId, -1, { 'workload_id': id });
  }
  unwaitIcon();
  filler.loop();
}

function fillResourceUserTotal(user, jobs, item) {
  var startMin = 0xffffffff;
  var endMax = 0;
  var nodeSet = Object.create(null);
  var cgroups = []
  for(var i = 0; i < jobs.length; i++) {
    var job = jobs[i];
    if(job.cgroup.length == 0) continue;
    cgroups.push(job.cgroup);
    for(var j = 0; j < job.nodes.length; j++) {
      nodeSet[job.nodes[j]] = true;
    }
    if(job.startTime) startMin = Math.min(startMin, job.startTime);
    if(job.endTime) endMax = Math.max(endMax, job.endTime);
  }
  
  var nodes = [];
  for(var i in nodeSet) {
    nodes.push(i);
  }

  var men = cluster.metricEnumName(true, false, dock.generalSettings.jobOnlyCgroup ? -1 : 1);
  var selected = user;
  var startTime = startMin;
  var endTime = endMax;
  if(endTime == 0) endTime = 0xffffffff;
  var deviceKeys = nodes;
  
  unwaitIcon();
  return populateMetrics(item.lastChild, selected, deviceKeys, startTime, endTime, men, cgroups, user);
}

function fillResource(item) {
  if(!item.hasAttribute("container") && (item.lastChild.childNodes.length == 0)) {
    item.setAttribute("container", "true");
    if(item.hasAttribute('workload')) {
      waitIcon();
      dock.workerpool.getWorker(cluster.actualurl).call('cmjob', 'getJobInfoStatistics', ["", "", "", "", "", "", 0, 0xffffffff, 0xffffffffffffffff, 0, 0xffffffffffffffff, ["user", "day"], 0, 0], function(code, data) {
          if(code == 0) {
            fillResourceUserDay(data, item);
          } else {
            unwaitIcon();
            dock.jsonCallError(this, code, data);
          }
        }, cluster);
    } else if(item.hasAttribute('workload_total')) {
      var user = item.getAttribute('workload_user');
      var states = 2 + 4 + 8; // not pending
      waitIcon();
      dock.workerpool.getWorker(cluster.actualurl).call('cmjob', 'filterJobInfo', ["", "", "", "", user, "", 0, 0xffffffff, states, 0, 0xffffffffffffffff], function(code, data) {
        if(code == 0) {
          fillResourceUserTotal(user, data, item);
        } else {
          unwaitIcon();
          dock.jsonCallError(this, code, data);
        }
      }, cluster);      
    } else if(item.hasAttribute('workload_user')) {
      var user = item.getAttribute('workload_user');
      var t0 = parseInt(item.getAttribute('workload_time')); // epoch, starts at day
      var t1 = t0 + 86400;
      var states = 2 + 4 + 8; // not pending
      waitIcon();
      dock.workerpool.getWorker(cluster.actualurl).call('cmjob', 'filterJobInfo', ["", "", "", "", user, "", t0, t1, states, 0, 0xffffffffffffffff], function(code, data) {
          if(code == 0) {
            fillResourceUserDayJobs(data, item);
          } else {
            unwaitIcon();
            dock.jsonCallError(this, code, data);
          }
        }, cluster);      
    } else if(item.hasAttribute('workload_id')) {
      var id = item.getAttribute('workload_id');
      if(!allJobs.hasOwnProperty(id)) {
        println("Error: job not found: " + id);
        return;
      }
      var job = allJobs[id];
      var men = cluster.metricEnumName(true, false, dock.generalSettings.jobOnlyCgroup ? -1 : 1);
      var selected = item.getAttribute("id");
      var startTime = job.startTime;
      var endTime = job.endTime;
      if(endTime == 0) job.endTime = 0xffffffff;
      var deviceKeys = job.nodes;
      return populateMetrics(item.lastChild, selected, deviceKeys, startTime, endTime, men, job.cgroup);
    } else {
      var men = cluster.metricEnumName(true, false);
      var selected = item.getAttribute("id");
      var startTime = item.getAttribute("startTime");
      var endTime = item.getAttribute("endTime");
      var deviceKeys = 0;
      if(item.hasAttribute('devices')) deviceKeys = item.getAttribute("devices").split(',');
      return populateMetrics(item.lastChild, selected, deviceKeys, startTime, endTime, men);
    }
  }
  return 1;
}

function tree_keypress(e) {
  if((e.charCode >= e.DOM_VK_1) && (e.charCode <= e.DOM_VK_9)) {
    var index = e.charCode - e.DOM_VK_1;
    if(index >= graphs.length) return true;
    var tree = e.target;
    var start = new Object();
    var end = new Object();
    for(var t = 0; t < tree.view.selection.getRangeCount(); t++){
      tree.view.selection.getRangeAt(t,start,end);
      for(var v = start.value; v <= end.value; v++) {
        var item = tree.view.getItemAtIndex(v);
        if(item.hasAttribute("metric")) {
          var dmp = new cvGraphDeviceMetric(item.getAttribute("key"), item.getAttribute("metric"), item.getAttribute("param"), 
                                            item.getAttribute("devices"), parseInt(item.getAttribute("startTime")), parseInt(item.getAttribute("endTime")), 
                                            true, item.getAttribute("translatedParam"), item.getAttribute("cgroups"), item.getAttribute("user"));
          graphs[index].allowRedraw(true);
          var r = graphs[index].add(dmp);
          if(r == 0) {
            replaceOne(index);
          } else if(r <= -100000) {
            if(r == -100000) {
              dock.displayError("This element cannot be plotted, expand the tree and plot one of the metrics or health checks", window);
            } else if(r == -100002) {
              dock.displayError("Only one heatmap/pie can be plotted in a single graph", window);
            } else {
              dock.displayError("Unknown drag/drop error code: " + r, window);
            }
          } else if(r < 0) {
            dock.displayError("Too many metrics/checks, only " + -r + " are supported", window);
          } else {
            dock.displayError("Incompatible metrics/checks: " + r, window);
          }
        }
      }
    }    
  }
  return true;
}

function selectResource(event) {
  var tree = document.getElementById("treecluster");
  var start = new Object();
  var end = new Object();
  for(var t = 0; t < tree.view.selection.getRangeCount(); t++){
    tree.view.selection.getRangeAt(t,start,end);
    for(var v = start.value; v <= end.value; v++) {
      fillResource(tree.view.getItemAtIndex(v));
      return;
    }
  }
}

function addNext(n) {
  var m = 0;
  while(n < addList.length) {
    var selected = addList[n];
    m += selected.add(dock.FILL_COUNT - m);
    if(selected.done()) n++;
    if(m >= dock.FILL_COUNT) {
      window.setTimeout('addNext(' + n + ');', dock.FILL_TIMEOUT);
      return;
    }
  }
  addList = new Array();
  
  if(dock.generalSettings.monitoringAutoFill) {
    println('dock.generalSettings.monitoringAutoFill ' + n);
    window.setTimeout('fillNext(0);', dock.FILL_TIMEOUT);
  } else {
    fillList = new Array();
    unwaitIcon();
  }  
}

function fillNext(n) {
  var m = 0;
  while(n < fillList.length) {
    var selected = fillList[n]
    var item = document.getElementById(selected);
    if(item) {
      if(item.lastChild.childNodes.length == 0) {
        m += fillResource(item);
        n++;
        if(m > dock.FILL_COUNT) {
          window.setTimeout('fillNext(' + n + ');', dock.FILL_TIMEOUT);
          return;
        }
      } else {
        // already done
        n++;
      }
    } else {
      println('fillNext, item not found: ' + selected);
      n++;
    }
  }
  fillList = new Array();
  unwaitIcon();
}

function waitIcon() {
  if(waitCount == 0) {
    window.setCursor("wait");
    for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("wait");
  }
  waitCount++;
}

function unwaitIcon() {
  waitCount--;
  if(waitCount <= 0) {
    window.setCursor("auto");
    for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("auto");
    if(waitCount < 0) println("Mismatch waitCursor");
    waitCount = 0;
  }
}

function metricForAllDevice() {
  var file = "metricforalldevice.xul";
  var old = dock.dialogs.find(file);
  if(old) {
    if(!old.closed) {
      dock.displayError("Metric for all device window is already open", window);
      return;
    }
  }
  var args = new Object();
  args['cluster'] = cluster;
  var d = new dock.cvDialog(file, args);
  d.parentWindow = window;
  d.onOk = function(w, h, list, interval, rate, zoom, minheight, legend) {
    clearAll();
    setSize(w, h);
    var heightChanged = false;
    for(var i = 0; i < list.length; i++) {
      graphs[i].title = '';
      graphs[i].intervals = interval;
      graphs[i].refreshRate = rate;
      if(graphs[i].minHeight != minheight) heightChanged = true;
      graphs[i].minHeight = minheight;
      graphs[i].legend = legend;
      for(var j = 0; j < list[i].length; j++) {
        graphs[i].add(list[i][j]);
      }
      graphs[i].xzoom = [zoom];
      replaceOne(i);
    }
    if(heightChanged) redraw();
  }
  dock.dialogs.show(d);
}
  
function customGroup() {
}
