/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var myDialog;
var dock;
var cluster;
var cmdtextbox;
var filterwindow;

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  cluster = myDialog.arguments.cluster;
  cmdtextbox = myDialog.arguments.cmdtextbox;
  filterwindow = myDialog.arguments.filterwindow;
  
  
  refresh();
}

function getSelected() {
  return selected('list');
}

function addOne(all, mn, mcname, image) {
  if(all.hasOwnProperty(mn)) {
    all[mn].monconfs.push(mcname);
  } else {
    all[mn] = {'image': image, 'name': mn, 'monconfs': [mcname]};
  }
}

function refresh() {
  window.setCursor("wait");

  var all = new Object();  
  addOne(all, 'jobs', ['All nodes'], 'Workload_16.png');
  addOne(all, 'queues', ['All nodes'], 'Workload_16.png');
  addOne(all, 'category', ['All compute nodes'], 'Category_16.png');
  addOne(all, 'nodegroups', ['All nodes'], 'NodeGroup_16.png');
  addOne(all, 'chassis', ['All devices'], 'Chassis_16.png');
  addOne(all, 'rack', ['All devices'], 'Rack_16.png');
  addOne(all, 'hostname', ['All devices'], 'Node_16.png');
  
  var monconfs = cluster.find('MonConf'); // TODO: make threaded ??
  for(var i = 0; i < monconfs.length; i++) {
    var mcname = cluster.getMonConfName(monconfs[i].get('uniqueKey'));
    var metrics = monconfs[i].get('metrics');
    for(var j = 0; j < metrics.length; j++) {
      var metric = cluster.find(metrics[j].metric);
      if(!metric || metric.get('metricClass') == 'PROTOTYPE') continue;
      var mn = metric.resolveName();
      if(metrics[j].metricParam.length) mn += ':' + metrics[j].metricParam;
      addOne(all, mn, mcname, 'Monitoring_16.png');
    }
    var healthchecks = monconfs[i].get('checks');
    for(var j = 0; j < healthchecks.length; j++) {      
      var healthcheck = cluster.find(healthchecks[j].healthcheck);
      if(!healthcheck || healthcheck.get('healthCheckClass') == 'PROTOTYPE') continue;
      var mn = healthcheck.resolveName();
      if(healthchecks[j].healthcheckParam.length) mn += ':' + healthchecks[j].healthcheckParam;
      addOne(all, mn, mcname, 'Health_16.png');
    }
  }
  
  tree = document.getElementById('tree');
  data = new Array();
  for(var one in all) {
    data.push(oneTreeItem(all[one]));
  }
  
  var select = selected('list');
  loadTable();
  setSelected('list', select);
  
  window.setCursor("auto");
}

function oneTreeItem(one, state) {
  var item = new Object();
  item['key'] = one.name;
  item['name'] = one.name;
  item['monconfs'] = one.monconfs.join(', ');
  item['image.src'] = "chrome://cmgui/content/icons/" + one.image;
  return item;
}

function keypress(event) {
  if((event.keyCode == event.DOM_VK_RETURN) && !event.altKey && !event.ctrlKey && !event.shiftKey && !event.metaKey) insert();
}

function escapeExpresssion(txt) {
  var s = '-+/%&|!=';
  var r = new Array();
  for(var i = 0; i < txt.length; i++) {
    if(s.indexOf(txt[i]) >= 0) r.push('\\');
    r.push(txt[i]);
  }
  return r.join();
}

function insert() {
  var select = getSelected();
  if(select.length == 0) {
    dock.displayError('Please select a metric or health check');
  } else {  
    for(var i = 0; i < select.length; i++) {
      if(i > 0) cmdtextbox.value += ' ';
      cmdtextbox.value += select[i];
    }
    filterwindow.focus();
    cmdtextbox.setSelectionRange(cmdtextbox.value.length, cmdtextbox.value.length);
    cmdtextbox.focus();
  }
}

function ok() {
  close();
}
