/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

// Category | Metric | Parameter | Threshold Bound  | Action  | Action Parameter

function refresh(use) {
  if(!!use) current = use;
  println("Refresh monitoring overview" + ' - ' + document.documentURI);
  tree = document.getElementById('tree');
  var monconfs = current.find('MonConf');
  data = new Array();
  var modified = false;
  for(var i = 0; i < monconfs.length; i++) {
    modified = modified || monconfs[i].get('modified');
    var cat = current.getMonConfName(monconfs[i].get('uniqueKey'))
    if(cat.length == 0) continue;
    
    var list = monconfs[i].get('metrics');
    for(var j = 0; j < list.length; j++) {
      var unit = current.find(list[j].metric);
      if(!!unit) {
        var metric = unit.resolveName();
        unit = unit.get('unit');
        var param = list[j].metricParam;
        var thresholds = list[j].thresholds;
        for(var k = 0; k < thresholds.length; k++) {
          var bound = thresholds[k].bound;
          if(unit.length > 0) {
            bound = formatKMGselect(bound, unit);
          }
          if(thresholds[k].isUpper) {
            bound = '> ' + bound;
          } else {
            bound = '< ' + bound;
          }
          var actions = thresholds[k].actionConfs;
          for(var l = 0; l < actions.length; l++) {
            var key = 'metrics|' + monconfs[i].object.uniqueKey + '|' + list[j].uniqueKey + '|' + thresholds[k].uniqueKey + '|' + actions[l].uniqueKey;
            var action = current.resolveName(actions[l].threshAction);
            var state = '';
            if(actions[l].doWhen == 'DURING') {
              state = ' (during)';
            } else if(actions[l].doWhen == 'LEAVE') {
              state = ' (leave)';
            }
            data.push(createTreeItem(key, cat, metric, param, bound + state, action, actions[l].actionParam));
          }
        }
      }
    }
    var list = monconfs[i].get('checks');
    for(var j = 0; j < list.length; j++) {
      var metric = current.resolveName(list[j].healthcheck);
      var param = list[j].healthcheckParam;

      var actions = list[j].passActionConfs;
      for(var l = 0; l < actions.length; l++) {
        var key = 'checks|' + monconfs[i].object.uniqueKey + '|' + list[j].uniqueKey + '|' + actions[l].uniqueKey;
        var action = current.resolveName(actions[l].threshAction);
        var state = '';
        if(actions[l].doWhen == 'DURING') {
          state = ' (during)';
        } else if(actions[l].doWhen == 'LEAVE') {
          state = ' (leave)';
        }
        data.push(createTreeItem(key, cat, metric, param, 'PASS' + state, action, actions[l].actionParam));
      }
      var actions = list[j].failActionConfs;
      for(var l = 0; l < actions.length; l++) {
        var key = 'checks|' + monconfs[i].object.uniqueKey + '|' + list[j].uniqueKey + '|' + actions[l].uniqueKey;
        var action = current.resolveName(actions[l].threshAction);
        var state = '';
        if(actions[l].doWhen == 'DURING') {
          state = ' (during)';
        } else if(actions[l].doWhen == 'LEAVE') {
          state = ' (leave)';
        }
        data.push(createTreeItem(key, cat, metric, param, 'FAIL' + state, action, actions[l].actionParam));
      }
      var actions = list[j].unknownActionConfs;
      for(var l = 0; l < actions.length; l++) {
        var key = 'checks|' + monconfs[i].object.uniqueKey + '|' + list[j].uniqueKey + '|' + actions[l].uniqueKey;
        var action = current.resolveName(actions[l].threshAction);
        var state = '';
        if(actions[l].doWhen == 'DURING') {
          state = ' (during)';
        } else if(actions[l].doWhen == 'LEAVE') {
          state = ' (leave)';
        }
        data.push(createTreeItem(key, cat, metric, param, 'UNKNOWN' + state, action, actions[l].actionParam));
      }
    }
  }
//   while(data.length < MINTREEITEMS) data.push(blankTreeItem());
  loadTable();

  println("Refresh monitoring overview DONE");
  if(modified) {
    document.getElementById('save').removeAttribute("disabled");
  } else {
    document.getElementById('save').setAttribute("disabled", true);
  }
  if(modified) parent.frames[1].refreshlist();
}

function createTreeItem(key, cat, metric, param, bound, action, arg) {
  var item = new Object();
  item['key'] = key;
  item['category'] = cat;
  item['metric'] = metric;
  item['parameter'] = param;
  item['bound'] = bound;
  item['action'] = action;
  item['argument'] = arg;
  return item;
}

function addObject() {
  var args = new Object();
  args['cluster'] = current;
  args['select'] = '';
  var d = new dock.cvDialog("thresholdwizard.xul", args);
  d.onOk = function() {
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function editObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select an action");
  } else if(select.length > 1) {
    dock.displayError("Please select only one action");
  } else {
    var index = select[0].split('|');
    var monconf = current.find(parseInt(index[1]));
    var args = new Object();
    args['cluster'] = current;
    args['window'] = window;
    if(index[0] == 'metrics') {
      args['name'] = current.getMonConfName(monconf.get('uniqueKey'));
      args['object'] = monconf.subObjectFromKey(index[0], parseInt(index[2]));
      args['container'] = monconf;
      args['sub'] = parseInt(index[3]);
      var d = new dock.cvDialog("thresholds.xul", args);
      d.onOk = function() {
        monconf.set('modified', true);
        refresh();
      }
      dock.dialogs.showDialog(d);
    } else if(index[0] == 'checks') {
      args['object'] = monconf;
      args['subobject'] = parseInt(index[2]);
      var d = new dock.cvDialog("editmonhealthconf.xul", args);
      d.onOk = function() {
        monconf.set('modified', true);
        refresh();
      }
      dock.dialogs.showDialog(d);
    } else {
      dock.displayError('Unknown type: ' + index[0]);
    }
  }
}

function removeObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select an action");
  } else {
    var args = new Object();
    if(select.length == 1) {
      args['question'] = "Are you sure you want to remove this action?";
    } else {
      args['question'] = "Are you sure you want to remove these actions?";
    }
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      for(var i = 0; i < select.length; i++) {
        var index = select[i].split('|');
        var monconf = current.find(parseInt(index[1]));
        if(index[0] == 'metrics') {
          var metrics = monconf.get('metrics');
          for(var j = 0; j < metrics.length; j++) {
            if(metrics[j].uniqueKey == parseInt(index[2])) {
              var thresholds = metrics[j].thresholds;
              for(var k = 0; k < thresholds.length; k++) {
                if(thresholds[k].uniqueKey == parseInt(index[3])) {
                  var actions = thresholds[k].actionConfs;
                  for(var l = 0; l < actions.length; l++) {
                    if(actions[l].uniqueKey == parseInt(index[4])) {
                      actions.splice(l, 1);
                      thresholds[k].actionConfs = actions;
                      monconf.set('modified', true);
                      break;
                    }
                  }
                  break;
                }
              }
              break
            }
          }
        } else if(index[0] == 'checks') {
          var checks = monconf.get('checks');
          var found = false;
          for(var j = 0; !found && (j < checks.length); j++) {
            var actions = checks[j].passActionConfs;
            for(var l = 0; !found && (l < actions.length); l++) {
              if(actions[l].uniqueKey == parseInt(index[3])) {
                actions.splice(l, 1);
                checks[j].passActionConfs = actions;
                monconf.set('modified', true);
                found = true;
              }
            }
            var actions = checks[j].failActionConfs;
            for(var l = 0; !found && (l < actions.length); l++) {
              if(actions[l].uniqueKey == parseInt(index[3])) {
                actions.splice(l, 1);
                checks[j].failActionConfs = actions;
                monconf.set('modified', true);
                found = true;
              }
            }
            var actions = checks[j].unknownActionConfs;
            for(var l = 0; !found && (l < actions.length); l++) {
              if(actions[l].uniqueKey == parseInt(index[3])) {
                actions.splice(l, 1);
                checks[j].unknownActionConfs = actions;
                monconf.set('modified', true);
                found = true;
              }
            }
          }
        } else {
          dock.displayError('Unknown type: ' + index[0]);
        }
      }
      refresh();
    }
    dock.dialogs.showDialog(d);
  }
}

function getSelected() {
  return selected('list');
}

function getCurrentSelection() {
  var select = getSelected();
  if(select.length > 0) {
    list = current.subObjectsFromKeys(subfield, select);
  } else if(!dock.CLONE_SELECT_ALL) {
    select = new Array();
  }
  return [select, current, '@SPECIAL:ACTION@'];
}

function addMenuItem(parent, text, command) {
  var item = document.createElement("menuitem");
  item.setAttribute('label', text);
  item.setAttribute('oncommand', command);
  parent.appendChild(item);
}

function addMenuSeparator(parent) {
  var item = document.createElement('menuseparator');
  parent.appendChild(item);
}

function changeOverviewTreePopup(event) {
  var popup = event.target;
  while(popup.firstChild) popup.removeChild(popup.firstChild);

  addMenuItem(popup, 'Edit', 'editObject();');
  addMenuItem(popup, 'Remove', 'removeObject();');

  var nc = dock.connectedClustersCount();
  if(nc > 1) {
    addMenuSeparator(popup);
    addMenuItem(popup, 'Export to other clusters', 'dock.cloneWizard(' + dock.CLONE_CREATE + ');');
  }
  return true;
}
