/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var monconf;
var monconfForCluster = new Object();
var alsoRemoved = false;

var basetype = 'MonMetricConf';
var singular = 'metric configuration';
var plural = 'metric configurations';
var subfield = 'metrics';
var editfile = 'editmonmetricconf.xul';

function refreshlist() {
  tree = document.getElementById('tree');
  data = new Array();

  if(monconf) {
    var list = monconf.get('metrics');
    for(var i = 0; i < list.length; i++) {
      if(!list[i].toBeRemoved || alsoRemoved) {
        var item = createTreeItem(list[i]);
        if(item) data.push(item);
      }
    }
    var select = selected('list');
    loadTable();
    setSelected('list', select);

    document.getElementById('save').setAttribute("disabled", !monconf.get('modified'));
    document.getElementById('revert').setAttribute("disabled", false);
    document.getElementById('edit').setAttribute("disabled", false);
    document.getElementById('add').setAttribute("disabled", false);
    document.getElementById('remove').setAttribute("disabled", false);
    document.getElementById('thresholds').setAttribute("disabled", false);
    document.getElementById('consolidators').setAttribute("disabled", false);

    if(monconf.get('modified')) {
      parent.frames[0].document.getElementById('save').setAttribute("disabled", false);
    }
  } else {
    loadTable();

    document.getElementById('save').setAttribute("disabled", true);
    document.getElementById('revert').setAttribute("disabled", true);
    document.getElementById('edit').setAttribute("disabled", true);
    document.getElementById('add').setAttribute("disabled", true);
    document.getElementById('remove').setAttribute("disabled", true);
    document.getElementById('thresholds').setAttribute("disabled", true);
    document.getElementById('consolidators').setAttribute("disabled", true);
  }
}

function createTreeItem(one) {
  var item = new Object();
  var metric = current.find(one.metric);
  if(!metric) return 0;
  item['metric'] = metric.resolveName();
  item['key'] = one.uniqueKey;
  if(one.translatedMetricParam.length) {
    item['parameter'] = one.translatedMetricParam + ' (' + one.metricParam + ')';
  } else {
    item['parameter'] = one.metricParam;
  }
  item['loglength'] = one.rawTableLength;
  if(one.samplingInterval == 0) {
    item['interval'] = 'N/A';
  } else {
    item['interval'] = one.samplingInterval;
  }
  if(one.disabled) {
    item['disabled'] = 'yes';
  } else if(metric.get('disabled')) {
    item['disabled'] = 'metric';
  }
  item['storeInDatabase'] = one.storeInDatabase;
  return item;
}

function thresholds() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("No metric configuration selected");
  } else if(select.length == 1) {
    var one = monconf.subObjectFromKey('metrics', select[0]);
    var metric = current.find(one.get('metric'));
    if(metric.get('metricClass') == 'PROTOTYPE') {
      dock.displayInfo('Thresholds cannot be defined for prototype metrics');
    } else {
      var args = new Object();
      args['name'] = current.getMonConfName(monconf.get('uniqueKey'));
      args['object'] = one;
      args['container'] = monconf;
      args['sub'] = 0;
      var d = new dock.cvDialog("thresholds.xul", args);
      d.onOk = function() {
        refreshlist();
      }
      dock.dialogs.showDialog(d);
    }
  } else {
    dock.displayError("Please select only one metric configuration");
  }
}
function consolidators() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("No metric configuration selected");
  } else if(select.length == 1) {
    var one = monconf.subObjectFromKey('metrics', select[0]);
    var metric = current.find(one.get('metric'));
    if(metric.get('metricClass') == 'PROTOTYPE') {
      dock.displayInfo('Consolidators cannot be defined for prototype metrics');
    } else {
      var args = new Object();
      args['name'] = current.getMonConfName(monconf.get('uniqueKey'));
      args['object'] = one;
      args['container'] = monconf;
      var d = new dock.cvDialog("consolidators.xul", args);
      d.onOk = function() {
        refreshlist();
      }
      dock.dialogs.showDialog(d);
    }
  } else {
    dock.displayError("Please select only one metric configuration");
  }
}
