/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var cluster;
var myDialog;
var current;
var directorInVpc; // status before moving
var directorNet; // status before moving
var directorVpc; // status before moving
var provider;
var region;
var vpcs;
var allSubnets; // all cloudnetworks with EC2SubnetID != ""

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  document.getElementById('message').value = "Please select a private cloud in which to move the selected director.\nOnly private clouds with at least two subnets and without a director are shown. Select \"Move nodes\" to move also nodes.";

  cluster = myDialog.arguments.cluster;
  current = cluster.find(myDialog.arguments.clouddirector);

  if(isObject(current)) {
    refresh();
  } else {
    dock.displayError("Key no longer valid");
    close();
  }
  
}

function refresh() {
  text('hostname', '^');

  provider = cluster.find(current.get('cloudSettings').provider);
  vpcs = provider.get('privateClouds');

  var list = document.getElementById('privateclouds');
  var popup = list.firstChild;
  while (popup.firstChild) popup.removeChild(popup.firstChild);

  var alreadyUsedNets = new Array();
  var cloudNodes = cluster.find('CloudNode');
  for (var i = 0; i < cloudNodes.length; i++) {
    if (cloudNodes[i].hasCloudDirectorRole()) {
      var networks = cloudNodes[i].get('networks');
      for (var j = 0; j < networks.length; j++) {
        if (networks[j].childType == 'NetworkPhysicalInterface') {
          alreadyUsedNets.push(networks[j].network);
        }
      }
    }
  }

  // check if director already in VPC or not
  directorNet = getEth(current);
  directorInVpc = false;
  directorVpc = 0;
  allSubnets = new Array();
  var cloudnets = cluster.findCloudNetworks();
  for (var i = 0; i < cloudnets.length; i++) {
    if (cloudnets[i].get('EC2SubnetID') != "") {
      allSubnets.push(cloudnets[i].get('uniqueKey'));
    }
  }
  if (allSubnets.find(directorNet) != -1) {
    directorInVpc = true;
  }

  region = current.get('cloudSettings').region;
  for (var i = 0; i < vpcs.length; i++) {
    if ((vpcs[i].subnets.length > 1) && (vpcs[i].region == region)) {
      // filter out vpcs with already used subnets
      var used = false;
      for (var j = 0; j < vpcs[i].subnets.length; j++) {
        if (alreadyUsedNets.find(vpcs[i].subnets[j]) != -1) {
          used = true;
        }

        // check if director in subnet
        if (directorInVpc && (directorNet == vpcs[i].subnets[j])) {
          directorVpc = vpcs[i].uniqueKey;
        }
      }

      if (!used) {
        var name = vpcs[i].name;
        var item = document.createElement('menuitem');
        item.setAttribute('key', vpcs[i].uniqueKey);
        item.setAttribute('label', name);
        popup.appendChild(item);
      }
    }
  }
}

function ok() {
  if(!window.closed) {

    var privateItem = document.getElementById('privateclouds').selectedItem;
    if (!privateItem) {
      dock.displayError("Private cloud cannot be empty");
      return;
    } else {
      var selectedvpc;
      var cloudKey = privateItem.getAttribute('key');
      for (var i = 0; i < vpcs.length; i++) {
        if (vpcs[i].uniqueKey == cloudKey) {
          selectedvpc = vpcs[i];
          break;
        }
      }

      // choose subnets
      var subnets = new Array();
      for (var i = 0; i < selectedvpc.subnets.length; i++) {
        subnets.push(cluster.find(selectedvpc.subnets[i]));
      }
      subnets.sort(function(a, b) {
        var x = ip2int(a.get('baseAddress'));
        var y = ip2int(b.get('baseAddress'));
        return x > y;
      });
      var subnetDirector = subnets[0].get('uniqueKey');
      var subnetNodes = subnets[1].get('uniqueKey');

      // check if nodes running
      var running = false;
      running = false || !!(current.get('cloudSettings').instanceId);

      // tell the user which nodes will be moved
      var nodesToMove = new Array();
      var movenodes = document.getElementById('movenodes').checked;
      if (movenodes) {
        var cloudNodes = cluster.find('CloudNode');
        for (var i = 0; i < cloudNodes.length; i++) {
          if (!cloudNodes[i].hasCloudDirectorRole()) {
            if (directorInVpc) {
              if (getVpc(getEth(cloudNodes[i])) == directorVpc) { // same vpc
                nodesToMove.push(cloudNodes[i].resolveName());
                running = false || !!(cloudNodes[i].get('cloudSettings').instanceId);
              }
            } else {
              if (getEth(cloudNodes[i]) == directorNet) { // same network
                nodesToMove.push(cloudNodes[i].resolveName());
                running = false || !!(cloudNodes[i].get('cloudSettings').instanceId);
              }
            }
          }
        }
      }

      var args = new Object();
      if (movenodes && (nodesToMove.length > 0)) {
        args['question'] = 'The selected cloud director and the following nodes will be moved: ' + nodesToMove;
      } else {
        args['question'] = 'The selected cloud director will be moved';
      }
      var d = new dock.cvDialog('question.xul', args);
      d.onOk = function() {
        if (running) {
          dock.displayError("Cloud director and nodes must be terminated before moving them.");
        } else {
          doMove(selectedvpc, subnetDirector, subnetNodes);
          close();
        }
      }
      dock.dialogs.showDialog(d);
    }
  }
}

function cancel() {
  if(!window.closed) {
    close();
  }
}

// get first physical interface
function getEth(device) {
  var nets = device.get('networks');
  for (var i = 0; i < nets.length; i++) {
    if (nets[i].childType == 'NetworkPhysicalInterface') {
      return nets[i].network;
    }
  }
}

function getVpc(subnet) {
  for (var i = 0; i < vpcs.length; i++) {
    if (vpcs[i].subnets.find(subnet) != -1) {
      return vpcs[i].uniqueKey;
    }
  }
}

function assignSubnet(device, subnetKey) {
  var nets = device.get('networks');
  for (var i = 0; i < nets.length; i++) {
    if (nets[i].childType == 'NetworkPhysicalInterface') {
      nets[i].network = subnetKey;
      nets[i].modified = true;
      break;
    }
  }
}

function modifyFSExports(device, selectedvpc) {
  var paths = new Array('/home', '/cm/shared');
  var fsexports = device.get('fsexports');
  for (var i = 0; i < fsexports.length; i++) {
    var path = fsexports[i].path;
    if (paths.indexOf(path) >= 0) {
      fsexports[i].name = path + '@' + selectedvpc.name;
      fsexports[i].hosts = selectedvpc.baseAddress + '/' + selectedvpc.netmaskBits;
      fsexports[i].modified = true;
    }
  }
}

function doMove(selectedvpc, subnetDirector, subnetNodes) {
  assignSubnet(current, subnetDirector);
  modifyFSExports(current, selectedvpc);

  var movenodes = document.getElementById('movenodes').checked;
  if (movenodes) {
    var cloudNodes = cluster.find('CloudNode');
    for (var i = 0; i < cloudNodes.length; i++) {
      if (!cloudNodes[i].hasCloudDirectorRole()) {
        if (directorInVpc) {
          if (getVpc(getEth(cloudNodes[i])) == directorVpc) { // same vpc
            assignSubnet(cloudNodes[i], subnetNodes);
            cloudNodes[i].get('cloudSettings').cloudStaticIP = 0;
            cloudNodes[i].set('modified', true);
          }
        } else {
          if (getEth(cloudNodes[i]) == directorNet) { // same network
            assignSubnet(cloudNodes[i], subnetNodes);
            cloudNodes[i].get('cloudSettings').cloudStaticIP = 0;
            cloudNodes[i].set('modified', true);
          }
        }
      }
    }
  }
  
  current.get('cloudSettings').cloudStaticIP = 0;
  current.set('modified', true);
}
