/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'Network';
var singular = 'network';
var plural = 'networks';

function createTreeItem(one) {
  var item = new Object();
  item['key'] = one.get('uniqueKey');
  item['modified'] = one.get('modified');
  item['name'] = one.get('name');
  item['baseAddress'] = one.get('baseAddress');
  item['broadcastAddress'] = one.get('broadcastAddress');
  item['netmaskBits'] = one.get('netmaskBits');
  switch(one.get('type')) {
    case('INTERNAL'): item['type'] = 'Internal'; break;
    case('EXTERNAL'): item['type'] = 'External'; break;
    case('CLOUD'): item['type'] = 'Cloud'; break;
    case('NETMAP'): item['type'] = 'NetMap'; break;
    case('TUNNEL'): item['type'] = 'Tunnel'; break;
    case('GLOBAL'): item['type'] = 'Global'; break;
  }
  item['bootable'] = one.get('bootable');
  item['IPv6'] = one.get('IPv6');
  item['dynamicRangeStart'] = one.get('dynamicRangeStart');
  item['dynamicRangeEnd'] = one.get('dynamicRangeEnd');
  return item;
}

function addObject() {
  var args = new Object();
  args['title'] = 'Add new ' + singular;
  args['question'] = 'Name';
  args['default'] = '';
  args['known'] = current.known('Network', 'name');
  var d = new dock.cvDialog("newobject.xul", args);
  d.onOk = function(name) {
    var object = new dock.cvNetwork();
    object.name = name;
    object.netmaskBits = 16;
    current.addObject(object, dock.openAfterCreate);
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function cloneObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a " + singular + "for cloning");
  } else if(select.length > 1) {
    dock.displayError("Please select only one " + singular);
  } else {
    var original = current.find(select[0]);
    if(isObject(original)) {
      var args = new Object();
      var name = original.get('name');
      args['title'] = 'Clone ' + name;
      args['question'] = 'Name';
      var nn = nameNumber(name);
      if(nn[1].length == 0) nn[1] = '0';
      var index = myParseInt(nn[1]);
      while(true) {
        index++;
        name = nn[0] + sprintf('%0' + nn[1].length + 'd', index);
        if(!current.find('Network', name)) break;
      }
      args['default'] = name;
      args['known'] = current.known('Network', 'name');
      var d = new dock.cvDialog("newobject.xul", args);
      d.onOk = function(name) {
        var object = original.clone();
        object.modified = true;
        object.toBeRemoved = false;
        object.name = name;
        current.addObject(object, dock.openAfterCreate);
        refresh();
      }
      dock.dialogs.showDialog(d);
    } else {
      dock.displayError("Original not found, already removed by someone else");
    }
  }
}
