/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh node burn" + ' - ' + document.documentURI);

  var nodes = [current.get('uniqueKey')];
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'getBurnStatus', [nodes], function(code, data) {
    if(code == 0) {
      setCallbackResult(data);
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);
}

function setCallbackResult(value) {
  if(value.length == 1) {
    var status = value[0]
    document.getElementById("status").setAttribute('value', status.error);
    if(showDetails(status)) {
      document.getElementById("details").setAttribute('hidden', true);
    } else {
      document.getElementById("details").removeAttribute('hidden');
    }
  } else {
    document.getElementById("status").setAttribute('value', "Error: no data");
  }
}

function showDetails(status) {
  var burn = current.get('burnConfig');
  if(burn) {
    document.getElementById("name").setAttribute('value', burn.name);
    document.getElementById("description").setAttribute('value', burn.description);
  }  
  document.getElementById("warnings").setAttribute('value', status.warnings);
  document.getElementById("phase").setAttribute('value', status.phaseName);
  document.getElementById("time").setAttribute('value', status.phaseTime);

  var list = document.getElementById("list");
  while(list.childNodes.length > 2) list.removeChild(list.lastChild);
  var test = status.testStatusList;
  for(var i = 0; i < test.length; i++) {
    var row = document.createElement('row');
    row.setAttribute("flex", "1");
    row.setAttribute("align", "center");
    row.setAttribute("height", "17px");
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", test[i].name);
    row.appendChild(label);
    var hbox = document.createElement('hbox');
    hbox.setAttribute("align", "left");
    if(test[i].warning) {
      var spacer = document.createElement('spacer');
      spacer.setAttribute("width", "16px");
      hbox.appendChild(spacer);
      var image = document.createElement('image');
      image.setAttribute("src", "./skin/image/exclamation.png");
      hbox.appendChild(image);
    }
    row.appendChild(hbox);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    var text = '';
    if(test[i].started) text += 'S';
    if(test[i].warning) text += 'W';
    if(test[i].failed) text += 'F';
    if(test[i].passed) text += 'P';
    if(test[i].status.length > 0) text += ' (' + test[i].status + ')';
    label.setAttribute("value", text);
    row.appendChild(label);
    list.appendChild(row);
    if(test[i].failed) document.getElementById("status").setAttribute('label', 'Will fail');
  }
  return (test.length == 0);
}

function start() {
  if(current.get('modified')) {
    dock.displayInfo("Local changes were made, commit or revert them first");
  } else {
    var args = new Object();
    args['node'] = current;
    var d = new dock.cvDialog("selectburnconfig.xul", args);
    d.onOk = function(name, description, text) {
      var burnConfig = new dock.cvBurnConfig();
      burnConfig.name = name;
      burnConfig.description = description;
      burnConfig.configuration = text;
      current.set('burnConfig', burnConfig);
      current.set('startNewBurn', true);
      current.commit(2);      
    }
    dock.dialogs.showDialog(d);
  }
}

function cancel() {
  if(current.get('modified')) {
    dock.displayInfo("Local changes were made, commit or revert them first");
  } else {
    current.set('startNewBurn', false);
    current.set('burning', false);
    current.commit(2);
  }
}

function burnLog() {
  var args = new Object();
  args['nodes'] = [current.get('uniqueKey')];
  args['cluster'] = current.cluster;
  var d = new dock.cvDialog("burnlog.xul", args);
  dock.dialogs.showDialog(d);
}
