/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var interactions;

var basetype;
var singular;
var plural;


function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);

  basetype = parent.frames[0].basetype;
  singular = parent.frames[0].singular;
  plural = parent.frames[0].plural;
}

function refresh(use) {	
  if (!!use) {
    if (current != use) {
      current = use;
    }
  }

  populateCategoryDropdown(current);

  println("Refresh node folder installer interactions" + ' - ' + document.documentURI);
  dock.workerpool.getWorker(current.actualurl).call('cmdevice', 'getRemoteNodeInstallerInteractions', null, function(code, data) {
    if (code == 0) {
      update(data);
      dock.unwaitIcon();
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);
  dock.waitIcon();
}

function populateCategoryDropdown(current) {
  var categories = current.find('Category');
  var list = document.getElementById('cataction');
  while (list.childNodes.length > 0) list.removeChild(list.lastChild);
  var popup = document.createElement("menupopup");
  var item = document.createElement("menuitem");
  item.setAttribute('label', '');
  popup.appendChild(item);
  for (var i = 0; i < categories.length; i++) {
    //printall(categories[i].object);
    var catName = categories[i].get('name');
    var catKey = categories[i].get('uniqueKey');
    var text = "Confirm all nodes for category '" + catName + "'";
    var item = document.createElement("menuitem");
    var fun;
    item.setAttribute('label', text);
    eval('fun = function () { applyCategory(' + catKey + ', 1) }');
    item.addEventListener("command", fun, true);
    popup.appendChild(item);
    
    text = "Deny all nodes for category '" + catName + "'";
    item = document.createElement("menuitem");
    item.setAttribute('label', text);
    eval('fun = function () { applyCategory(' + catKey + ', 0) }');
    item.addEventListener("command", fun, true);
    popup.appendChild(item);
  }
  list.appendChild(popup);
  list.selectedIndex = 0;
}

function update(value) {
  interactions = value;
  var list = document.getElementById('list');
  while (list.childNodes.length > 2) list.removeChild(list.lastChild);

  for (var i = 0; i < interactions.length; i++) {
    //printall(interactions[i]);
    var node = current.find(interactions[i].node);
    var hostname = node.get('hostname');
    var categoryKey = node.get('category');
    var categoryName = current.resolveName(categoryKey);
    var row = document.createElement("row"); 
    row.setAttribute('align', 'center');
    var label = document.createElement("label");
    label.setAttribute('value', hostname);
    row.appendChild(label);
    var label = document.createElement("label");
    label.setAttribute('value', categoryName);
    row.appendChild(label);
    var label = document.createElement("label");
    label.setAttribute('value', interactions[i].appeared);
    row.appendChild(label);
    var label = document.createElement("label");
    label.setAttribute('value', interactions[i].description);
    row.appendChild(label);
    var label = document.createElement("label");
    var confirmedOrDenied = "";
    if (interactions[i].wasDenied) {
      confirmedOrDenied = "denied";
    } else if (interactions[i].wasConfirmed) {
      confirmedOrDenied = "confirmed";
    } else {
      confirmedOrDenied = "pending";
    }
    label.setAttribute('value', confirmedOrDenied);
    row.appendChild(label);
    if (confirmedOrDenied == "pending") {
      var menulist = document.createElement("menulist");
      menulist.setAttribute('id', 'action_' + interactions[i].node + '_' + categoryKey);
      var menupopup = document.createElement("menupopup");
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute('label', 'None');
      menuitem.setAttribute('key', -1);
      menupopup.appendChild(menuitem);
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute('label', 'Confirm');
      menuitem.setAttribute('key', 1);
      menupopup.appendChild(menuitem);
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute('label', 'Deny');
      menuitem.setAttribute('key', 0);
      menupopup.appendChild(menuitem);
      menupopup.addEventListener("command", choose_action, true);
      menulist.appendChild(menupopup);
      row.appendChild(menulist);
    }
    list.appendChild(row);
  }
}

function choose_action(event) {
  var item = event.target;
  var actionKey = parseInt(item.getAttribute('key'));
  var nodeKey = item.parentNode.parentNode.getAttribute('key');
  
  for (var i = 0; i < interactions.length; i++) {
    if (nodeKey == interactions[i].node) {
      if (actionKey == 1) { // confirm requested
        if (!interactions[i].wasConfirmed) {
          interactions[i].wasConfirmed = true;
        }
      } else if (actionKey == 0) { // deny requested
        if (!interactions[i].wasDenied) {
          interactions[i].wasDenied = true;
        }
      }
    }
  }
}

function applyCategory(category, action) {
  for (var i = 0; i < interactions.length; i++) {
    var node = current.find(interactions[i].node);
    if (category == node.get('category')) {
      if (action == 1) { // confirm requested
        if (!interactions[i].wasConfirmed) {
          interactions[i].wasConfirmed = true;
        }
      } else if (action == 0) { // deny requested
        if (!interactions[i].wasDenied) {
          interactions[i].wasDenied = true;
        }
      }
    }
  }

  var list = document.getElementsByTagName('menulist');
  for (var i = 0; i < list.length; i++) {
    var id = list[i].id;
    if (startswith(id, 'action_') && endswith(id, '_' + category)) {
      var drop = document.getElementById(id);
      if (action == 1) {
        drop.selectedIndex = 1;
      } else if (action == 0) {
        drop.selectedIndex = 2;
      }
    }
  }
}

function apply() {
  if (interactions.length == 0) {
    dock.displayInfo("You must apply at least one interaction");
  } else {
    var args = new Object();
    args['question'] = "Apply choices to nodes?";
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      applySave();
      parent.refresh();
    }
    d.onNo = function() {
      parent.refresh();
    }
    dock.dialogs.showDialog(d);
  }
}

function applySave() {
  dock.workerpool.getWorker(current.actualurl).call('cmdevice', 'updateRemoteNodeInstallerInteractions', [interactions], function(code, data) {
    if (code == 0) {      
      update(data);
      dock.unwaitIcon();
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);
  dock.waitIcon();
}
