/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var timer = 0;
var busy = false;

var basetype;
var singular;
var plural;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);

  basetype = parent.frames[0].basetype;
  singular = parent.frames[0].singular;
  plural = parent.frames[0].plural;
}

function getCurrentSelection() {
  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;
  var select = getSelected();
  if(select.length > 0) {
    var list = new Array();
    for(var i = 0; i < select.length; i++) {
      var one = cluster.find(select[i]);
      if(!!one) list.push(one);
    }
    return [list, 0];
  } else {
    return [current.find(basetype), 0];
  }
}

function getSelected() {
  return selected('list');
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh node folder status" + ' - ' + document.documentURI);

  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;
  var nodes;  
  if(typeof(parent.frames[0].allKeys) == 'function') {
    nodes = parent.frames[0].allKeys();
  } else {
    nodes = cluster.allKeys(basetype);
  }
  dock.workerpool.getWorker(cluster.actualurl).call('cmgui', 'getNodeStatus', [nodes], function(code, data) {
    if(code == 0) {
      setCallbackResult(data);
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);
}

function setCallbackResult(status) {
  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;
  var nodes;  
  if(typeof(parent.frames[0].allKeys) == 'function') {
    nodes = parent.frames[0].allKeys();
  } else {
    nodes = cluster.allKeys(basetype);
  }
  tree = document.getElementById('tree');
  data = new Array();
  for(var i = 0; i < nodes.length; i++) {
    data.push(oneTreeItem(cluster.find(nodes[i]), status[i]));
  }
  for(var i = status.length; i < MINTREEITEMS; i++) data.push(blankTreeItem());
  var select = selected('list');
  loadTable();
  setSelected('list', select);
  dock.addHistory(plural + ' status refreshed');

  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('refresh();', dock.generalSettings.statusRefresh * 1000);
}

function stop() {
  println("Stop node folder status");
  if(timer) window.clearTimeout(timer);
  timer = 0;
}

function oneTreeItem(one, state) {
  var item = new Object();
  item['key'] = one.get('uniqueKey');
  item['image.src'] = "chrome://cmgui/content/icons/Node-" + one.stateString(false, true) + "_16.png";
  item['hostname'] = one.get('hostname');
  item['state'] = one.fullStateString();
  if(typeof(state) == 'undefined') {
    item['uptime'] = ''
    item['usage'] = '';
    item['loadaverage'] = '';
    item['memory'] = '';
    item['swap'] = '';
  } else {
    item['uptime'] = formatDays(state.uptime);
    if(kisNaN(state.usageUser)) {
      item['usage'] = "Undefined";
    } else {
      item['usage'] = sprintf("%.1f", 100 * state.usageUser) + "% user, " + sprintf("%.1f", 100 * state.usageSystem) + "% system, " + sprintf("%.1f", 100 * state.usageIdle) + "% idle";
    }
    item['loadaverage'] = state.load1 + ", " + state.load5 + ", " + state.load15;
    item['memory'] = formatKMGselect(state.memoryUsed, 'B');
    item['swap'] = formatKMGselect(state.swapUsed, 'B');
  }
  return item;
}

function editObject() {
  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;
  var select = getSelected();
  if(select.length == 0) {
    dock.displayError('Please select a ' + singular);
  } else if(select.length > 1) {
    dock.displayError('Please select only one ' + singular);
  } else {
    cluster.select(select[0]);
  }
}
