/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;

var basetype;
var singular;
var plural;

var deviceName = 'node';
var deviceNamePlural = 'nodes';

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);

  basetype = parent.frames[0].basetype;
  singular = parent.frames[0].singular;
  plural = parent.frames[0].plural;
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh node folder tasks" + ' - ' + document.documentURI);

  var group = document.getElementById("addgroup");
  var popup = group.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);

  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;
  
  var list = cluster.find("NodeGroup");
  for(var i = 0; i < list.length; i++) {
    var item = document.createElement("menuitem");
    item.setAttribute("class", "menuitem-iconic");
    item.setAttribute("type", "nodegroup");
    item.setAttribute("key", list[i].object.uniqueKey);
    item.setAttribute("label", list[i].resolveName());
    popup.appendChild(item);
  }

  var item = document.createElement("menuitem");
  item.setAttribute("class", "menuitem-iconic");
  item.setAttribute("type", "nodegroup");
  item.setAttribute("key", 0);
  item.setAttribute("label", '<new>');
  popup.appendChild(item);
  group.selectedIndex = popup.childNodes.length - 1;

  var group = document.getElementById("setcategory");
  var popup = group.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);

  var list = cluster.find("Category");
  for(var i = 0; i < list.length; i++) {
    var item = document.createElement("menuitem");
    item.setAttribute("class", "menuitem-iconic");
    item.setAttribute("type", "category");
    item.setAttribute("key", list[i].object.uniqueKey);
    item.setAttribute("label", list[i].resolveName());
    popup.appendChild(item);
  }
  group.selectedIndex = -1;
}

function allNodes(needCommit) {
  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;

  needCommit = isundefined(needCommit, false);
  var select = parent.frames[0].getSelected();
  select = cluster.stripKeys(select, false, needCommit);
  if(select.length > 0) {
    return select;
  } else {
    return cluster.allKeys(basetype, false, needCommit);
  }
}

function provisioningLog() {
  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;
  var args = new Object();
  args['nodes'] = allNodes(false);
  args['cluster'] = cluster;
  var d = new dock.cvDialog("synclog.xul", args);
  dock.dialogs.showDialog(d);
}

function removeFromGroup() {
  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;
  var item = document.getElementById("addgroup").selectedItem;
  var group = parseInt(item.getAttribute('key'));
  if(group == 0) {
    dock.displayInfo("Group <new> does not exist");
  } else {
    group = cluster.find(group);
    if(group) {
      var mine = allNodes(false).sort();
      var nodes = group.get('nodes').sort();
      var left = new Array();
      var removed = new Array()
      var i = 0;
      var j = 0;
      while((i < mine.length) && (j < nodes.length)) {
        if(nodes[j] < mine[i]) {
          left.push(nodes[j]);
          j++;
        } else if(nodes[j] > mine[i]) {
          i++;
        } else {
          removed.push(mine[i]);
          j++;
          i++;
        }
      }
      while(j < nodes.length) {
        left.push(nodes[j]);
        j++;
      }
      if(left.length < nodes.length) {
        group.set('nodes', left);
        refresh();
        if(dock.SUBTREE) {
          for(var i = 0; i < removed.length; i++) {
            var work = cluster.find(removed[i]);
            work.addOneObject(work, group.branch);
          }
          cluster.removeTreeitemFromParent(group.branch);
        }
      } else {
        dock.displayInfo("Group does not contain this node");
      }
    } else {
      dock.displayInfo("Group was deleted");
      refresh();
    }
  }
}

function addToGroup() {
  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;
  var item = document.getElementById("addgroup").selectedItem;
  var group = parseInt(item.getAttribute('key'));
  if(group == 0) {
    var args = new Object();
    args['title'] = 'Add new nodegroup';
    args['question'] = 'Name';
    args['default'] = '';
    args['known'] = cluster.known('NodeGroup', 'name');
    var d = new dock.cvDialog("newobject.xul", args);
    d.onOk = function(name) {
      var object = new dock.cvNodeGroup();
      object.name = name;
      var group = cluster.addObject(object, false);
      var nodes = allNodes(false);
      group.set('nodes', nodes);
      refresh();
      if(dock.SUBTREE) {
        for(var i = 0; i < nodes.length; i++) {
          var work = cluster.find(nodes[i]);
          cluster.addOneObject(work, group.branch);
        }
      }
    }
    dock.dialogs.showDialog(d);
  } else {
    group = cluster.find(group);
    if(group) {
      var mine = allNodes(false).sort();
      var nodes = group.get('nodes').sort();
      var join = new Array();
      var added = new Array();
      var i = 0;
      var j = 0;
      while((i < mine.length) && (j < nodes.length)) {
        if(nodes[j] < mine[i]) {
          join.push(nodes[j]);
          j++;
        } else if(nodes[j] > mine[i]) {
          join.push(mine[i]);
          added.push(mine[i]);
          i++;
        } else {
          join.push(nodes[j]);
          j++;
          i++;
        }
      }
      while(i < mine.length) {
        join.push(mine[i]);
        i++;
      }
      while(j < nodes.length) {
        join.push(nodes[j]);
        j++
      }
      if(join.length > nodes.length) {
        group.set('nodes', join);
        refresh();
        if(dock.SUBTREE) {
          for(var i = 0; i < added.length; i++) {
            var work = cluster.find(added[i]);
            work.addOneObject(work, group.branch);
          }
        }
      } else {
        dock.displayInfo("Group already contains this node");
      }
    } else {
      dock.displayInfo("Group was deleted");
      refresh();
    }
  }
}

function setCategory() {
  var cluster = current;
  if(!!current.cluster) cluster = current.cluster;
  var item = document.getElementById("setcategory").selectedItem;
  var category = parseInt(item.getAttribute('key'));
  if(category) {
    var mine = allNodes(false);
    var args = new Object();
    args['question'] = "Change category for:\n"  + cluster.resolveList(mine);
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      for(var i = 0; i < mine.length; i++) {
        var work = cluster.find(mine[i]);
        work.set('category', category);
      }
    }
    dock.dialogs.showDialog(d);
  } else {
    dock.displayInfo("Select a category");
  }
}
