/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var timer = 0;

var fieldname = 'nodes';
var displayInTree = true;
var subtype = 'Node';
var singular = 'node';
var plural = 'nodes';
var nodeSelectionProperties = [['','', false],
            ['hostname', 'hostname', false],
            ['MAC', 'mac', false],
            ['tag', 'tag', false],
            ['category', 'category', true],
            ['installmode', 'installMode', false],
            ['rack', 'rack.rack', true],
            ['position', 'rack.position', false]];
var belongs = '0';
var noloop = 'true';
var extraName = "''";
var extraEvalOnAdd = '';
var extraEvalOnRemove = '';

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function getCurrentSelection() {
  return [[current], 0, ""];
}

function stop() {
  println("Stop nodegroup settings");
  if(timer) window.clearTimeout(timer);
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh nodegroup settings" + ' - ' + document.documentURI);
  document.getElementById('revert').setAttribute('disabled', current.object.uniqueKey < dock.MAX_LOCAL_KEY);
  text('name', '^[a-zA-Z]', '', true);
  text('revision');
  deviceselect();
  
  var disabled = (current.get('type') != 'NORMAL');
  document.getElementById('nodes').setAttribute('disabled', disabled);
  document.getElementById('allnodes').setAttribute('disabled', disabled);
  document.getElementById('groupnodes').setAttribute('disabled', disabled);
  document.getElementById('addToGroup').setAttribute('disabled', disabled);
  document.getElementById('removeFromGroup').setAttribute('disabled', disabled);
  
  if(current.get('type') == 'STORAGEPAIR') {
    updateStoragePair();
  } else {
    document.getElementById('storagepairstate').setAttribute('hidden', true);
  }
}

function updateStoragePair() {
  var nodes = current.get('nodes');
  if(nodes.length > 1) {
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'pgetClustat', [nodes], function(code, data) {
        if(code != 0) {
          dock.jsonCallError(this, code, data);
        }
        storagePairState(nodes, data);
      }, current);
  } else {
    document.getElementById('storagepairstate').setAttribute('hidden', true);
  }
}

function storagePairState(nodes, state) {
  var node1 = current.cluster.find(nodes[0]);
  var node2 = current.cluster.find(nodes[1]);

  if(node1 && node2) {
    var clustat = [false, false];
    
    for(var i = 0; i < state.length; i++) {
      if(state[i].uniqueKey == nodes[0]) {
        clustat[0] = state[i];
      } else if(state[i].uniqueKey == nodes[1]) {
        clustat[1] = state[i];
      }
    }
    
    drawImages(node1, clustat[0], 'node1');
    drawImages(node2, clustat[1], 'node2');
    
    var obj = document.getElementById('clustatgroups');
    while(obj.childNodes.length > 2) obj.removeChild(obj.lastChild);
    
    for(var i = 0; i < state.length; i++) {
      for(var j = 0; j < state[i].groups.length; j++) {
        obj.appendChild(clustatGroupRow(state[i].groups[j]));
      }
    }
  }

  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('updateStoragePair();', dock.generalSettings.statusRefresh * 1000);

  document.getElementById('storagepairstate').removeAttribute('hidden');
  
  function clustatGroupRow(data) {
    var row = document.createElement('row');
    row.setAttribute("flex", "1");
    row.setAttribute("align", "center");
    row.setAttribute("height", "19px");
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.name);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.stateStr);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.owner);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.lastOwner);
    row.appendChild(label);
    var label = document.createElement('label');
    label.setAttribute("class", "staticdatavaluesmall");
    label.setAttribute("value", data.lastTransitionStr);
    row.appendChild(label);
    return row;
  }  
}

function drawImages(node, clustat, name) {
  var hostname = node.get('hostname');
  var state = node.stateString(true);
  
  document.getElementById(name + '_hostname').setAttribute('value', hostname);
  document.getElementById(name + '_icon').setAttribute('src', 'chrome://cmgui/content/icons/Node-' + state + '_32.png');
  if(clustat) {
    var clustatNode = false;
    for(var i = 0; i < clustat.nodes.length; i++) {
      if(clustat.nodes[i].local) {
        clustatNode = clustat.nodes[i];
        break;
      }
    }
    var leds = document.getElementById(name + '_leds');
    var names = [['state', 1], ['rgmanager', 1], ['rgmanagerMaster', 1], ['estranged', 0], ['qdisk', 1]];
    while(leds.firstChild) leds.removeChild(leds.firstChild);
    for(var i = 0; i < names.length; i++) {
      var image = document.createElement('image');
      var value = 'unknown';
      if(clustatNode) value = eval('clustatNode.' + names[i][0]);
      if((value == 'unknown') || (node.status.status != 'UP')) {
        image.setAttribute('src', './skin/image/LED_blank.png');
      } else if(value == names[i][1]) {
        image.setAttribute('src', './skin/image/LED_green.png');
      } else {
        image.setAttribute('src', './skin/image/LED_red.png');
      }
      image.setAttribute('tooltiptext', names[i][0] + ' = ' + value);
      image.setAttribute('id', name + '_' + names[i][0]);
      leds.appendChild(image);
    }
  }
}
