/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'NetworkInterface';
var subfield = 'networks';
var singular = 'network interface';
var plural = 'network interfaces';
var editfile = 'editnetworkinterface.xul';

function createTreeItem(one) {
  var item = new Object();
  item['key'] = one.uniqueKey;
  item['name'] = one.name;
  item['ip'] = one.ip;
  var network = current.cluster.find(one.network);
  if(network) {
    item['network'] = network.resolveName() + ' (' + network.get('baseAddress') + '/' + network.get('netmaskBits') + ')';
  } else {
    item['network'] = '';
  }
  switch(one.childType) {
  case("NetworkPhysicalInterface"):
    item['childType'] = 'Physical';
    break;
  case("NetworkAliasInterface"):
    item['childType'] = 'Alias';
    break;
  case("NetworkBondInterface"):
    item['childType'] = 'Bond';
    var name = one.name + " (";
    var list = one.interfaces;
    for(var i = 0; i < list.length; i++) {
      if(i > 0) name += ", ";
      name += list[i];
    }
    name += ")";
    item['name'] = name;
    break;
  case("NetworkVLANInterface"):
    item['childType'] = 'VLAN';
    break;
  case("NetworkBmcInterface"):
    item['childType'] = 'BMC';
    break;
  case("NetworkBridgeInterface"):
    item['childType'] = 'Bridge';
    var name = one.name + " (";
    var list = one.interfaces;
    for(var i = 0; i < list.length; i++) {
      if(i > 0) name += ", ";
      name += list[i];
    }
    name += ")";
    item['name'] = name;
    break;
  case("NetworkNetMapInterface"):
    item['childType'] = 'NetMap';
    break;
  case("NetworkTunnelInterface"):
    item['childType'] = 'Tunnel';
    break;
  default:
  }
  item['modified'] = one.modified;
  item['toBeRemoved'] = one.toBeRemoved;
  return item;
}

function removeSubObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError('No ' + singular + ' selected');
  } else {
    var args = new Object();
    if(select.length == 1) {
      args['question'] = 'Are you sure you want to remove this ' + singular + '?';
    } else {
      args['question'] = 'Are you sure you want to remove these ' + plural + '?';
    }
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      var networks = current.get(subfield);
      for(var i = 0; i < select.length; i++) {
        var one = 0;
        var j = 0;
        for(; j < networks.length; j++) {
          if(networks[j].uniqueKey == select[i]) {
            one = networks[j];
            break;
          }
        }
        if(one) {
          if(one.childType == "NetworkPhysicalInterface") {
            for(var k = 0; k < networks.length; k++) {
              if((networks[k].childType == "NetworkAliasInterface") || (networks[k].childType == "NetworkVLANInterface")) {
                if(startswith(networks[k].name, one.name)) {
                  networks.splice(k, 1);
                  if(j > k) j--;
                  break;
                }
              } else if(networks[k].childType == "NetworkBondInterface") {
                var list = networks[k].interfaces;
                for(var i0 = 0; i0 < list.length; i0++) {
                  if(list[i0] == one.name) {
                    dock.displayInfo(one.name + " is used by " + networks[k].name);
                    return;
                  }
                }
              }
            }
            networks.splice(j, 1);
          } else {
            networks.splice(j, 1);
          }
        }
      }
      current.set(subfield, networks);
      refresh();
    }
    dock.dialogs.showDialog(d);
  }
}
