/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var overview;
var health;
var timer = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function stop() {
  println("Stop node overview");
  if(timer) window.clearTimeout(timer);
}

function getCurrentSelection() {
  return [[current], 0, ""];
}

function refresh(use) {
  if(!!use) {
    current = use;
    setCallbackResult(new dock.cvGuiNodeOverview());
  }
  println("Refresh node overview" + ' - ' + document.documentURI);

  if(startswith(current.fullStateString(), 'Closed')) {
    var no = new dock.cvGuiNodeOverview();
    setCallbackResult(no);
    updateHealth(new Array());
  } else if(current.cluster.connected) {
    var uniqueKey = current.get('uniqueKey');
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmgui', 'getNodeOverview', [current.cluster.sessionid, uniqueKey], function(code, data) {
        if(code == 0) {
          setCallbackResult(data);
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
    
    var mpi = current.cluster.allChecksMetricsAsMetricPrmId(uniqueKey);
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmmon', 'getLatestPickedRates', [[uniqueKey], mpi], function(code, data) {
        if(code == 0) {
          updateHealth(data);
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);

    if(timer) window.clearTimeout(timer);
    timer = window.setTimeout('refresh();', dock.generalSettings.statusRefresh * 1000);
  }
}

function setCallbackResult(value) {
  overview = value;
  if(dock.DEMO) {
    overview.load1 = 15.8;
    overview.load5 = 15.6;
    overview.load15 = 15.2;
    overview.uptime = 6136183;
    overview.memoryUsed = 29.6 * 1024 * 1024 * 1024;
    overview.memoryTotal = 32 * 1024 * 1024 * 1024;
    overview.swapUsed = 0;
    overview.swapTotal = 0;
    overview.usageUser = 0.91;
    overview.usageSystem = 0.02;
    overview.usageIdle = 0.03;

    var workload = new Array();
    var wl = new dock.cvGuiJob();
    wl.name = "gromacs";
    wl.user = 'alex';
    wl.runtime = 21817;
    workload.push(wl);
    overview.setJobs(workload.length, workload);

    var disks = new Array();
    var dk = new dock.cvGuiDiskUsage();
    dk.mountpoint = '/';
    dk.free = 23e+9;
    dk.used = 17e+9;
    disks.push(dk);
    var dk = new dock.cvGuiDiskUsage();
    dk.mountpoint = '/cm/shared';
    dk.free = 289e+8;
    dk.used = 115e+8;
    disks.push(dk);
    var dk = new dock.cvGuiDiskUsage();
    dk.mountpoint = '/home';
    dk.free = 10000e+9;
    dk.used = 894e+9;
    disks.push(dk);

    overview.setDisks(disks.length, disks);

    var intf = new Array();
    var int = new dock.cvGuiNetworkInterface();
    int.name = 'eth0';
    int.tx = 1177219121;
    int.rx = 217171982781;
    intf.push(int);
    var int = new dock.cvGuiNetworkInterface();
    int.name = 'ib0';
    int.tx = 7493793872983;
    int.rx = 4782349239972;
    intf.push(int);
    overview.setInterfaces(intf.length, intf);
  }

  if(overview != null) {
    overviewValue('hostname', current.object.hostname);
    overviewValue('state', current.fullStateString());
    overviewValue('load1', null, '%.2f');
    overviewValue('load5', null, '%.2f');
    overviewValue('load15', null, '%.2f');

    overviewUptime('uptime');
    overviewBar('memory', overview.memoryUsed, overview.memoryTotal, 'B', true);
    overviewBar('swapmemory', overview.swapUsed, overview.swapTotal, 'B', true);
    overviewUsage('usage', overview.usageUser, overview.usageSystem, overview.usageIdle);

    overviewDisks();
    overviewJobs();
    overviewInterfaces();

    var swap = document.getElementById('swapmemory').parentNode;
    if(current.get('childType') == 'MICNode') {
      swap.setAttribute('hidden', true);
    } else {
      swap.removeAttribute('hidden');
    }    
    
    var pctl = current.get('powerControl');
    var work = current;
    if(dock.baseChildTypeCheck(work.object, 'VirtualSMPNode')) {
      work = work.cluster.find(work.get('primaryNode'));
      if(work) {
        pctl = work.get('powerControl');
      } else {
        work = current;
      }
    }

    var power = document.getElementById('power');
    while(power.firstChild) power.removeChild(power.firstChild);
    var pdu = work.get('powerDistributionUnits');
    for(var i = 0; i < pdu.length; i++) {
      var name = current.cluster.resolveName(pdu[i].pdu);
      if(pdu[i].pdu && name.length) {
        var image = document.createElement('image');
        image.setAttribute('src', './skin/image/LED_blank.png');
        image.setAttribute('id', pdu[i].pdu + '_' + pdu[i].prt);
        power.appendChild(image);
        var spacer = document.createElement('spacer');
        spacer.setAttribute('width', '3px');
        power.appendChild(spacer);
        var label = document.createElement('label');
        label.setAttribute('value', current.cluster.resolveName(pdu[i].pdu) + ':' + pdu[i].prt);
        label.setAttribute('class', 'staticdataunit');
        power.appendChild(label);
      } else {
        println("Unknown pdu for " + current.resolveName() + " " + pdu[i].pdu + ':' + pdu[i].prt);
      }
    }
    if(startswith(pctl, 'ipmi') || startswith(pctl, 'ilo') || startswith(pctl, 'cimc') || startswith(pctl, 'drac')) {
      var intf = work.get('networks');
      for(var i = 0; i < intf.length; i++) {
        if(intf[i].childType == 'NetworkBmcInterface') {
          var name = intf[i].name;
          var image = document.createElement('image');
          image.setAttribute('src', './skin/image/LED_blank.png');
          image.setAttribute('id', name);
          power.appendChild(image);
          var spacer = document.createElement('spacer');
          spacer.setAttribute('width', '3px');
          power.appendChild(spacer);
          var label = document.createElement('label');
          label.setAttribute('class', 'staticdataunit');
          label.setAttribute('value', name);
          power.appendChild(label);
        }
      }
    } else if((pctl == 'custom') || (pctl == 'cloud') || (pctl == 'openstack') || (pctl == 'mic')) {
      var name = pctl;
      var image = document.createElement('image');
      if(dock.DEMO) {
        image.setAttribute('src', './skin/image/LED_green.png');
      } else {
        image.setAttribute('src', './skin/image/LED_blank.png');
      }
      image.setAttribute('id', name);
      power.appendChild(image);
      var spacer = document.createElement('spacer');
      spacer.setAttribute('width', '3px');
      power.appendChild(spacer);
      var label = document.createElement('label');
      label.setAttribute('class', 'staticdataunit');
      label.setAttribute('value', name);
      power.appendChild(label);
    }
    if(power.childNodes.length == 0) {
      var label = document.createElement('label');
      label.setAttribute('value', 'N/A');
      label.setAttribute('class', 'staticdatavalue');
      power.appendChild(label);
      power.setAttribute('style', '');
    } else {
      power.setAttribute('style', 'padding-left: 4px;');
    }
  }

  if(current.stateString() != 'Closed') {
    if(timer) window.clearTimeout(timer);
    timer = window.setTimeout('refresh();', dock.generalSettings.statusRefresh * 1000);
  }
}
