/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var timer = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) {
    var changed = (current != use);
    current = use;
    if(!data || (data.length == 0) || changed) setCallbackResult(new Array());
  }

  document.getElementById('autorefresh').checked = dock.generalSettings.autoProcessesRefresh
  if(dock.generalSettings.autoProcessesRefresh) refreshButton();
}


function autoRefresh() {
  if(!dock.generalSettings.autoProcessesRefresh) refreshButton();
  dock.generalSettings.autoProcessesRefresh = !dock.generalSettings.autoProcessesRefresh
  document.getElementById('autorefresh').checked = dock.generalSettings.autoProcessesRefresh;
}

function refreshButton(use) {
  println("Refresh node process");

  if(current.stateString() == 'Closed') {
    setCallbackResult(new Array());
  } else {
    dock.waitIcon();
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmproc', 'getProcessesOnNode', current.get('uniqueKey'), function(code, data) {
      if(code == 0) {
        setCallbackResult(data);
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  }
}

function setCallbackResult(processes) {
  tree = document.getElementById('tree');
  data = new Array();
  if(processes) {
    for(var i = 0; i < processes.length; i++) {
      data.push(createTreeItem(processes[i]));
    }
  }
  var select = selected('list');
  loadTable();
  setSelected('list', select);

  var state = current.stateString();
  if(state == 'Up') {
    if(timer) window.clearTimeout(timer);
    if(dock.generalSettings.autoProcessesRefresh) {
      timer = window.setTimeout('refreshButton();', dock.generalSettings.statusRefresh * 1000);
    }
    dock.addHistory('Node services refreshed');
  }

}

function createTreeItem(one) {
  var item = new Object();
  item['key'] = one.pid;
  item['user'] = one.username;
  item['pid'] = one.pid;
  item['ppid'] = one.ppid;
  item['uid'] = one.uid;
  item['gid'] = one.gid;
  item['state'] = one.state;
  item['size'] = one.size;
  item['rss'] = one.rss;
  item['command'] = one.cmd;
  return item;
}

function killProcess() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("No process selected");
  } else {
    var signal = document.getElementById('signal');
    signal = parseInt(signal.selectedItem.getAttribute("signal"));

    var args = new Object();
    args['question'] = "Are you sure you want to kill these processes?";
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      var work = {n: 0, m: select.length, text: '', cluster: current.cluster};
      dock.waitIcon();
      for(var i = 0; i < select.length; i++) {
        var pid = parseInt(select[i]);
        dock.workerpool.getWorker(current.cluster.actualurl).call('cmproc', 'sendSignalOnNode', [pid, signal, current.get('uniqueKey')], function(code, data) {
          if(code == 0) {
            if(!data) this.current.text += 'Failed to kill pid: ' + select[i] + '\n';              
          } else {
            dock.jsonCallError(this, code, data);
          }
          this.current.n++;
          if(this.current.n >= this.current.m) {
            if(this.current.text.length > 0) dock.displayInfo(this.current.text);
            refreshButton();
            dock.unwaitIcon();
          }
        }, work);
      }
    }
    dock.dialogs.showDialog(d);
  }
}
function killProcessByName() {
  var signal = document.getElementById('signal');
  signal = parseInt(signal.selectedItem.getAttribute("signal"));

  var args = new Object();
  args['title'] = 'Kill processes by name';
  args['question'] = 'Process name';
  args['default'] = '';
  args['width'] = 600;
  args['height'] = 600;
  args['known'] = new Array();
  var d = new dock.cvDialog("newobject.xul", args);
  d.onOk = function(name) {
    var nodes = [current.get('uniqueKey')];
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmproc', 'psendSignalByName', [name, signal, nodes], function(code, data) {
      if(code == 0) {
        if(!data[0]) dock.displayInfo('Failed to kill: ' + name);
      } else {
        dock.jsonCallError(this, code, data);
      }
      refreshButton();
    }, current);
  }
  dock.dialogs.showDialog(d);
}

function killProcessByUser() {
  var signal = document.getElementById('signal');
  signal = parseInt(signal.selectedItem.getAttribute("signal"));

  var args = new Object();
  args['title'] = 'Kill processes by user';
  args['question'] = 'User name';
  args['default'] = '';
  args['known'] = new Array();
  var d = new dock.cvDialog("newobject.xul", args);
  d.onOk = function(name) {
    var names = name.split(",");
    var nodes = [current.get('uniqueKey')];
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmproc', 'psendSignalByUsers', [names, signal, nodes], function(code, data) {
      if(code == 0) {
        if(!data[0]) dock.displayInfo('Failed to kill for users ' + name);
      } else {
        dock.jsonCallError(this, code, data);
      }
      refreshButton();
    }, current);
  }
  dock.dialogs.showDialog(d);
}

