/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var timer = 0;
var alsoRemoved = false;

var basetype = 'OSServiceConfig';
var subfield = 'services';
var singular = 'OS service config';
var plural = 'OS service config';
var editfile = 'editosserviceconfig.xul';

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function getCurrentSelection() {
  var select = selected('list');
  var list;
  if(select.length > 0) {
    list = current.subObjectsFromKeys(subfield, select);
  } else {
    list = current.getFromSubfield(subfield);
  }
  return [list, current, subfield];
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh node services" + ' - ' + document.documentURI);

  setCallbackResult(new Array());

  document.getElementById('autorefresh').checked = dock.generalSettings.autoServiceRefresh
  if(dock.generalSettings.autoServiceRefresh) refreshStatus();
}

function autoRefresh() {
  if(!dock.generalSettings.autoServiceRefresh) refreshStatus();
  dock.generalSettings.autoServiceRefresh = !dock.generalSettings.autoServiceRefresh
  document.getElementById('autorefresh').checked = dock.generalSettings.autoServiceRefresh;
}

function allNodes(needCommit) {
  return [current.get('uniqueKey')];
}

function refreshStatus() {
  if(current.stateString() == 'Up') {
    dock.waitIcon();
    var nodes = [current.get('uniqueKey')];
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmserv', 'pgetOSServices', [nodes], function(code, data) {
      if(code == 0) {
        setCallbackResult(data);
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  } else {
    setCallbackResult(new Array());
  }
}

function setCallbackResult(value) {
  var services = value;
  tree = document.getElementById('tree');
  data = new Array();
  var config = current.get(subfield).slice();
  var n = config.length;
  var cateext = '';
  var state = current.stateString();
  if(dock.baseChildTypeCheck(current.object, 'SlaveNode')) {
    var category = current.get('category')
    if(category) {
      var category = current.cluster.find(category);
      if(category) {
        var catconf = category.get(subfield);
        cateext = " (" + category.resolveName() + ")";
        for(var i = 0; i < catconf.length; i++) {
          var found = false;
          for(var j = 0; !found && (j < config.length); j++) {
            found = (catconf[i].name == config[j].name);
          }
          if(!found) {
            config.push(catconf[i]);
          }
        }
      }
    }
  }
  if(services.length == 1) services = services[0].osservices;
  for(var i = 0; i < config.length; i++) {
    var status = -1;
    var message = "";
    for(var j = 0; j < services.length; j++) {
      if(config[i].name == services[j].name) {
        status = services[j].status;
        message = services[j].sicknessMessage;
        services.splice(j, 1);
        break;
      }
    }
    if((status != -1) || (config[i].uniqueKey < dock.MAX_LOCAL_KEY)) {
      status = formatStatus(status, message);
      if(!config[i].toBeRemoved || alsoRemoved) {
        if(i < n) {
          data.push(oneTreeItem(config[i], status, 1));
        } else {
          data.push(oneTreeItem(config[i], status, 0, cateext));
        }
      }
    } else if((status == -1) && ((state != 'Up') || value.length == 0)) {
      status = '';
      if(i < n) {
        data.push(oneTreeItem(config[i], status, 1));
      } else {
        data.push(oneTreeItem(config[i], status, 0, cateext));
      }
    }
  }

  for(var j = 0; j < services.length; j++) {
    var status = formatStatus(services[j].status, services[j].sicknessMessage);
    data.push(oneTreeItem(services[j], status, 0, ' (free)'));
  }

//   for(var i = data.length; i < MINTREEITEMS; i++) data.push(blankTreeItem());
  var select = selected('list');
  loadTable();
  setSelected('list', select);

  if(state == 'Up') {
    if(timer) window.clearTimeout(timer);
    if(dock.generalSettings.autoServiceRefresh) {
      timer = window.setTimeout('refreshStatus();', dock.generalSettings.statusRefresh * 1000);
    }
    dock.addHistory('Node services refreshed');
  }
}

function formatStatus(status) {
  switch(status) {
  case('DOWN') :
    return "Down";
  case('UP') :
    return "Up";
  case('STOPPED') :
    return "Stopped";
  case('FAILING') :
    return "Failing";
  case('SICK') :
    return "SICK (" + message + ")";
  case('NOTNEEDED') :
    return "Not used";
  case('UNKNOWN') :
    return "Unknown (/etc/init.d script timeout)";
  case('NOTFOUND') :
    return "Unknown (/etc/init.d script not found/executable)";
  }
  return "Unknown (node down/save first)";
}

function stop() {
  println("Stop node services");
  if(timer) window.clearTimeout(timer);
}

function updateService() {
  refresh();
}

function oneTreeItem(one, status, owned, catext) {
  var item = new Object();
  item['key'] = owned * one.uniqueKey + '|' + one.name + '|' + owned;
  if(!!catext && catext.length > 0) {
    item['name'] = one.name + catext;
  } else {
    item['name'] = one.name;
  }
  item['monitored'] = one.monitored;
  item['autostart'] = one.autostart;
  item['status'] = status;
  item['modified'] = one.modified;
  item['toBeRemoved'] = one.toBeRemoved;
  return item;
}

function newService() {
  var args = new Object();
  args['object'] = current;
  args['osserviceconfig'] = 0;
  current.cluster.currentSubObjectType = basetype;
  current.cluster.currentSubObjectList = current.get(subfield);  
  var d = new dock.cvDialog(editfile, args);
  d.onOk = function() {
    refresh();
  }
  d.onClose = function() {
    current.cluster.currentSubObjectType = 0;
    current.cluster.currentSubObjectList = 0;  
  }  
  dock.dialogs.showDialog(d);
}
function editService() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("No service selected");
  } else if(select.length == 1) {
    var name = select[0].split('|');
    if(name[0] == '0') {
      dock.displayError("This service is not part of the configuration");
    } else if(name[2] == '0') {
      dock.displayError("This service is part of the category configuration");
    } else {
      var args = new Object();
      args['object'] = current;
      args['osserviceconfig'] = name[0];
      current.cluster.currentSubObjectType = basetype;
      current.cluster.currentSubObjectList = current.get(subfield);  
      var d = new dock.cvDialog(editfile, args);
      d.onOk = function() {
        refresh();
      }
      d.onClose = function() {
        current.cluster.currentSubObjectType = 0;
        current.cluster.currentSubObjectList = 0;  
      }  
      dock.dialogs.showDialog(d);
    }
  } else {
    dock.displayError("Please select only one service");
  }
}
function removeService() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("No service selected");
  } else {
    var args = new Object();
    args['question'] = "Are you sure you want to remove these services?";
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      var text = '';
      for(var i = 0; i < select.length; i++) {
        var name = select[i].split('|');
        if(name[0] == '0') {
          text += name[1] + " is not part of the configuration\n";
        } else if(name[2] == '0') {
          text += name[1] + " is not part of the category configuration\n";
        } else {
          var serv = current.subObjectFromKey(subfield, name[0]);
          if(serv) {
            if(serv.get('belongsToRole')) {
              dock.displayInfo(serv.resolveName() + ' is added as part of a role and cannot be removed');
              continue;
            }
          }
          current.deleteSubObjectFromKey(subfield, name[0]);
        }
      }
      refresh();
      if(text.length > 0) dock.displayError(text);
    }
    dock.dialogs.showDialog(d);
  }
}
