/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;

var deviceName = 'node';
var deviceNamePlural = 'nodes';

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh node tasks" + ' - ' + document.documentURI);

  var group = document.getElementById("addgroup");
  var popup = group.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);

  var key = current.get('uniqueKey');
  var list = current.cluster.find("NodeGroup");
  for(var i = 0; i < list.length; i++) {
    var item = document.createElement("menuitem");
    item.setAttribute("class", "menuitem-iconic");
    var nodes = list[i].get('nodes');
    if(nodes.find(key) >= 0) item.setAttribute("image", "chrome://cmgui/content/icons/checked_16.png");
    item.setAttribute("type", "nodegroup");
    item.setAttribute("key", list[i].object.uniqueKey);
    item.setAttribute("label", list[i].resolveName());
    popup.appendChild(item);
  }

  var item = document.createElement("menuitem");
  item.setAttribute("class", "menuitem-iconic");
  item.setAttribute("type", "nodegroup");
  item.setAttribute("key", 0);
  item.setAttribute("label", '<new>');
  popup.appendChild(item);
  group.selectedIndex = popup.childNodes.length - 1;

  var check = document.getElementById("docheck");
  if(check) {
    var popup = check.firstChild;
    while(popup.firstChild) popup.removeChild(popup.firstChild);
   
    var classes = new Object();
    var key = current.get('uniqueKey');
    var list = current.cluster.allChecksMetricsAsMetricPrmId(current.get('uniqueKey'), false, true);
    for(var i = 0; i < list.length; i++) {
      var item = document.createElement("menuitem");
      item.setAttribute("key", list[i].metricId);
      item.setAttribute("param", list[i].metricParam);
      var metric = current.cluster.find(list[i].metricId);
      if(metric) {
        var text = metric.resolveName();
        if(list[i].metricParam.length > 0) {
          text += ':' + list[i].metricParam;
        }
        item.setAttribute("label", text);
        popup.appendChild(item);
        var classname;
        if(metric.get('baseType') == 'Metric') {
          classname = metric.get('metricClass');
        } else {
          classname = metric.get('healthCheckClass');
        }
        if(!classes.hasOwnProperty(classname)) classes[classname] = metricClass2String(classname);
      }
    }

    for(var classname in classes) {
      var item = document.createElement("menuitem");
      item.setAttribute("key", -1);
      item.setAttribute("param", classname);
      item.setAttribute("label", '<' + classes[classname] + '>');
      popup.appendChild(item);    
    }
   
    item.setAttribute("key", 0);
    item.setAttribute("param", '');
    item.setAttribute("label", '<all>');
    popup.appendChild(item);
    check.selectedIndex = popup.childNodes.length - 1;
  }
  
  var work = current;
  if(dock.baseChildTypeCheck(current.object, 'VirtualSMPNode')) {
    work = current.get('primaryNode');
    work = current.cluster.find(work);
    if(!work) work = current;
  }
  if(work.get('powerControl') == 'apc') {
    var apc = work.get('powerDistributionUnits');
    document.getElementById('poweron').setAttribute('disabled', apc.length == 0);
    document.getElementById('poweroff').setAttribute('disabled', apc.length == 0);
    document.getElementById('powerreset').setAttribute('disabled', apc.length == 0);
  } else if(work.get('powerControl') == 'none') {
    document.getElementById('poweron').setAttribute('disabled', true);
    document.getElementById('poweroff').setAttribute('disabled', true);
    document.getElementById('powerreset').setAttribute('disabled', true);
  } else {
    document.getElementById('poweron').setAttribute('disabled', false);
    document.getElementById('poweroff').setAttribute('disabled', false);
    document.getElementById('powerreset').setAttribute('disabled', false);
  }

  var ipmi = (work.get('customRemoteConsoleScript').length > 0);
  if(!ipmi) {
    var networks = current.get('networks');
    for(var i = 0; i < networks.length; i++) {
      if(networks[i].childType == 'NetworkBmcInterface') {
        ipmi = true;
        break;
      }
    }
  }

  if(ipmi) {
    var obj = document.getElementById('startsol');
    if(obj) obj.removeAttribute('disabled');
    var obj = document.getElementById('identify');
    if(obj) obj.removeAttribute('disabled');

    current.isIdentified(createObjectCallback(document, showIdentify));
  } else {
    var obj = document.getElementById('startsol');
    if(obj) obj.setAttribute('disabled', true);
    var obj = document.getElementById('identify');
    if(obj) obj.setAttribute('disabled', true);
    var obj = document.getElementById('identify');
    if(obj) obj.setAttribute('hidden', false);
    var obj = document.getElementById('unidentify');
    if(obj) obj.setAttribute('hidden', true);
  }

  var obj = document.getElementById('rootShell');
  if(obj) {
    if(dock.baseChildTypeCheck(current.object, 'isMICNode')) {
      obj.setAttribute('disabled', true);
    } else {
      obj.removeAttribute('disabled');
    }
  }
  
  var kickstart = document.getElementById('kickstart');
  if(!!kickstart) kickstart.setAttribute('hidden', !current.cluster.isKickStart);
  
  var decommissionRow = document.getElementById("decommissionRow");
  if(decommissionRow) {
    if(current.isHadoop() || current.isCeph()) {
      decommissionRow.removeAttribute('hidden');
    } else {
      decommissionRow.setAttribute('hidden', true);
    }
  }
  
  var puppetRow = document.getElementById("puppetRow");
  if(puppetRow) {
    if(current.cluster.isPuppet()) {
      puppetRow.removeAttribute('hidden');
    } else {
      puppetRow.setAttribute('hidden', true);
    }
  }
}

function osShutdown() {
  var keys = [current.get('uniqueKey')];
  var args = new Object();
  args['question'] = "Are you sure you want to shutdown this node?";
  var d = new dock.cvDialog("question.xul", args);
  d.onOk = function() {
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'pshutdown', [keys], function(code, data) {
      if(code == 0) {
        var nodes = [this.current.get('uniqueKey')];
        showPresult(nodes, data, "Shutdown");
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
  }
  dock.dialogs.showDialog(d);
}
function osRestart() {
  var keys = [current.get('uniqueKey')];
  var args = new Object();
  args['question'] = "Are you sure you want to restart this node?";
  var d = new dock.cvDialog("question.xul", args);
  d.onOk = function() {
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'preboot', [keys], function(code, data) {
      if(code == 0) {
        var nodes = [this.current.get('uniqueKey')];
        showPresult(nodes, data, "Reboot");
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, current);
  }
  dock.dialogs.showDialog(d);
}
// function osKickstart() {
//   var args = new Object();
//   args['question'] = "Are you sure you want to kickstart this node?";
//   var d = new dock.cvDialog("question.xul", args);
//   d.onOk = function() {
//     var cmd = "/cm/local/apps/cluster-tools/bin/kickstart " + current.get('hostname');
//     if(current.cluster.runCommandOnMaster(cmd)) {
//       var nodes = [current.get('uniqueKey')];
//       var delay = Math.max(current.cluster.setup.powerOffDelay, current.cluster.setup.powerOnDelay);
//       var done = current.cluster.services['cmdevice'].powerCycle(nodes.length, nodes, delay, current.cluster.sessionid, true);
//       if(!done) dock.displayError("Power reset failed");
//     } else {
//       dock.displayError("Command: '" + cmd + "' failed");
//     }
//   }
//   dock.dialogs.showDialog(d);
// }

function removeFromGroup() {
  var item = document.getElementById("addgroup").selectedItem;
  var key = current.get('uniqueKey');
  var group = parseInt(item.getAttribute('key'));
  if(group == 0) {
    dock.displayInfo("Group <new> does not exist");
  } else {
    group = current.cluster.find(group);
    if(group) {
      var nodes = group.get('nodes');
      var index = nodes.find(key);
      if(index >= 0) {
        nodes.splice(index, 1);
        group.set('nodes', nodes);
        refresh();
        if(dock.SUBTREE) {
          current.removeTreeitemFromParent(group.branch);
        }
      } else {
        dock.displayInfo("Group does not contain this node");
      }
    } else {
      dock.displayInfo("Group was deleted");
      refresh();
    }
  }
}

function addToGroup() {
  var item = document.getElementById("addgroup").selectedItem;
  var key = current.get('uniqueKey');
  var group = parseInt(item.getAttribute('key'));
  if(group == 0) {
    var args = new Object();
    args['title'] = 'Add new nodegroup';
    args['question'] = 'Name';
    args['default'] = '';
    args['known'] = current.cluster.known('NodeGroup', 'name');
    var d = new dock.cvDialog("newobject.xul", args);
    d.onOk = function(name) {
      var object = new dock.cvNodeGroup();
      object.name = name;
      var group = current.cluster.addObject(object, false);
      var nodes = group.get('nodes');
      nodes.push(key);
      group.set('nodes', nodes);
      refresh();
      if(dock.SUBTREE) {
        current.cluster.addOneObject(current, group.branch);
      }
    }
    dock.dialogs.showDialog(d);
  } else {
    group = current.cluster.find(group);
    if(group) {
      var nodes = group.get('nodes');
      if(nodes.find(key) < 0) {
        var modified = group.get('modified');
        nodes.push(key);
        group.set('nodes', nodes);
        refresh();
        if(dock.SUBTREE) {
          current.cluster.addOneObject(current, group.branch);
        }
        if(!modified) group.commit();
      } else {
        dock.displayInfo("Group already contains this node");
      }
    } else {
      dock.displayInfo("Group was deleted");
      refresh();
    }
  }
}

function docheck() {
  if(current.stateString() != "Closed") {
    var item = document.getElementById("docheck").selectedItem;
    var check = parseInt(item.getAttribute('key'));
    var param = item.getAttribute('param');

    var nodes = [current.get('uniqueKey')];
    var mpi = new Array();
    if(check < 0) {
      mpi = current.cluster.allChecksMetricsAsMetricPrmId(current.get('uniqueKey'), false, true, param);
    } else if(check == 0) {
      mpi = current.cluster.allChecksMetricsAsMetricPrmId(current.get('uniqueKey'), false, true);
    } else {
      var one = new dock.cvMetricPrmId();
      one.metricId = check;
      one.metricParam = param;
      mpi.push(one);
    }
    
    dock.waitIcon();
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmmon', 'sampleOndemandOnDevices', [nodes, mpi], function(code, data) {
      if(code == 0) {
        var args = new Object();
        args['current'] = current;
        args['srwi'] = data;
        var d = new dock.cvDialog("checknow.xul", args);
        dock.dialogs.show(d);
      } else {
        dock.jsonCallError(this, code, data);
      }
      refresh();
      dock.unwaitIcon();
    }, current);
  } else {
    dock.displayInfo("When a device is closed no monitoring data will be retrieved");
  }
}

function rootShell() {
  if(current.stateString() == "Up") {
    dock.waitIcon();
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmproc', 'startShellOnNode', [current.cluster.syslogin, current.get('uniqueKey'), ''], function(code, data) {
      if(code == 0) {
        if(data.length) {
          var args = [this.current.cluster.host, this.current.cluster.port, data, this.current.resolveName(), 'Root Shell'];
          var d = new dock.cvDialog("remoteshell.xul", args); 
          dock.dialogs.showShell(d);
        } else {
          dock.displayInfo("The node did not return a login token");
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
  } else {
    dock.displayInfo("This node is not up");
  }
}

function solShell() {
  dock.waitIcon();
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmproc', 'startSOL', current.get('uniqueKey'), function(code, data) {
    if(code == 0) {
      if(data.length) {
        var args = [this.current.cluster.host, this.current.cluster.port, data, this.current.resolveName(), 'Remote Console', 'false'];
        var d = new dock.cvDialog("remoteshell.xul", args);
        dock.dialogs.showShell(d);
      } else {
        dock.displayInfo("The node did not return a login token");
      }
    } else {
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
  }, current);
}

function locateInRack() {
  current.cluster.locate(current);
}

function showIdentify() {
  var iden = document.getElementById('identify');
  var uniden = document.getElementById('unidentify');
  if(iden) {
    iden.setAttribute('disabled', (current.identified == 4) || (current.identified < 0));
    iden.setAttribute('hidden', (current.identified == 0));
  }
  if(uniden) uniden.setAttribute('hidden', (current.identified != 0));
}

function identifyNode() {
  dock.waitIcon();
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'identify', current.get('uniqueKey'), function(code, data) {
    if(code == 0) {
      current.identified = data;
      parent.refresh();
      if(data >= 2) {
        dock.displayError("Identify failed");
      }
    } else {
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
    refresh();
  }, current);
}

function unidentifyNode() {
  dock.waitIcon();
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'unidentify', current.get('uniqueKey'), function(code, data) {
    if(code == 0) {
      current.identified = data;
      parent.refresh();
      if(data >= 2) {
        dock.displayError("Unable to unidentify");
      }
    } else {
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
    refresh();
  }, current);
}

function openDevice() {
  dock.waitIcon();
  var keys = [current.get('uniqueKey')];
  var status = new dock.cvDevStatus();
  status.status = 'OPENING';
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'setStatusForDeviceArray', [keys, status], function(code, data) {
    if(code == 0) {
      if(!data) {
        dock.displayError("Open failed");
      }
    } else {
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
    refresh();
  }, current);
}

function closeDevice() {
  dock.waitIcon();
  var keys = [current.get('uniqueKey')];
  var status = new dock.cvDevStatus();
  status.status = 'CLOSED';
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'setStatusForDeviceArray', [keys, status], function(code, data) {
    if(code == 0) {
      if(!data) {
        dock.displayError("Close failed");
      }
    } else {
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
    refresh();
  }, current);
}

function drain() {
  dock.waitIcon();
  var nodes = [this.current.get('uniqueKey')];
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmjob', 'drainNodes', ['', '', nodes, 2, [], [], false], function(code, data) {
    if(code == 0) {
      var args = new Object();
      args['drain'] = 0;
      args['drainresult'] = data;
      args['cluster'] = this.current.cluster;
      var d = new dock.cvDialog("drainresult.xul", args);
      dock.dialogs.showDialog(d);
    } else {
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
  }, current);
}

function undrain() {
  dock.waitIcon();
  var nodes = [this.current.get('uniqueKey')];
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmjob', 'drainNodes', ['', '', nodes, 0, [], [], false], function(code, data) {
    if(code == 0) {
      var args = new Object();
      args['drain'] = 1;
      args['drainresult'] = data;
      args['cluster'] = this.current.cluster;
      var d = new dock.cvDialog("drainresult.xul", args);
      dock.dialogs.showDialog(d);
    } else {
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
  }, current);
}

function decommission() {
  dock.waitIcon();
  var nodes = [this.current.get('uniqueKey')];
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmjob', 'drainNodes', ['', '', nodes, 1, [], [], false], function(code, data) {
    if(code == 0) {
      var args = new Object();
      args['drain'] = 2;
      args['drainresult'] = data;
      args['cluster'] = this.current.cluster;
      var d = new dock.cvDialog("drainresult.xul", args);
      dock.dialogs.showDialog(d);
    } else {
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
  }, current);
}

function recommission() {
  dock.waitIcon();
  var nodes = [this.current.get('uniqueKey')];
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmjob', 'drainNodes', ['', '', nodes, 0, [], [], false], function(code, data) {
    if(code == 0) {
      var args = new Object();
      args['drain'] = 3;
      args['drainresult'] = data;
      args['cluster'] = this.current.cluster;
      var d = new dock.cvDialog("drainresult.xul", args);
      dock.dialogs.showDialog(d);
    } else {
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
  }, current);
}


function puppetApply(noop) {
  var node = parseInt(current.get('uniqueKey'));
  var all = current.cluster.find('Puppet');
  var puppet;
  if(all.length == 1) {
    puppet = all[0];
  }
  else {
    dock.displayError('Puppet has not been configured');
    return true;
  }
  var session = current.cluster.sessionid;
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmpuppet', 
                                                            'applyList', 
                                                            [ puppet.get('uniqueKey'), 
                                                              session, 
                                                              [ node ], 
                                                              noop ],
    function(code, data) {
      if(code != 0) {      
        dock.jsonCallError(this, code, data);
      }
      else {
        dock.displayInfo('Apply ' + (noop ? '(noop) ':'') + ' started in the background');
      }
      dock.unwaitIcon();
    }, current);
}


function puppetLog() {
  var node = parseInt(current.get('uniqueKey'));
  var cluster = current.isCluster?current:current.cluster;
  var apply_session = 0;
  var run = -1;
  var noop = true;
  var puppet;
  var all = current.cluster.find('Puppet');
  if(all.length == 1) {
    puppet = all[0];
  }
  else {
    dock.displayError('Puppet has not been configured');
    return true;
  }
  dock.workerpool.getWorker(cluster.actualurl).call('cmpuppet', 'getPuppetOutput', 
                                                    [ node, apply_session, run, noop, [ true, true ] ], 
       function(code, data) {
          if(code == 0) 
           {   
             var args = new Object();
             var d = new dock.cvDialog("chrome://cmgui/content/puppet/puppetlog.xul", args);
             args['config_success'] = data.success[0];
             args['log_success']    = data.success[1];
             args['config']         = data.content[0];
             args['hostname']       = current.get('hostname');
             args['log']            = data.content[1].replace(/\x1B.*?m/g,'');
             dock.dialogs.showDialog(d);
          } else {
              dock.jsonCallError(this, code, data);
          }
          dock.unwaitIcon();
      }, current);
}

