1/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var myDialog;
var dock
var cluster;
var types;
var networks;
var switches;
var basetype;
var cloudprovider;
var openstack;
var spotPersistent;
var spotPrice;
var masterInCloud;

var allTypes = ['eth0', 'eth1', 'eth2', 'eth3', 'eth4', 'BOOTIF', 'ib0', 'ib1', 'ipmi0', 'ilo0', 'cimc0', 'drac0', 'map0', 'tun0'];

function onLoad() {
  dock = getDock();
  println("Node Creation Wizard initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  cluster = myDialog.arguments.cluster;
  basetype = myDialog.arguments.basetype;

//   var nets = cluster.findCloudNetworks();
//   masterInCloud = (nets.length == 0);
  masterInCloud = cluster.masterOnCloud;

  var categoryNodes = cluster.find(basetype);
  var nodeCategoryKey;
  if (categoryNodes.length > 0 && basetype == 'VirtualNode'){
    nodeCategoryKey = categoryNodes[0].get('category');
  }

  var cat = document.getElementById('category');
  var categories = cluster.find('NodeCategory');
  var found = -1;
  for(var i = 0; i < categories.length; i++) {
    var item = document.createElement('menuitem');
    item.setAttribute('key', categories[i].get('uniqueKey'));
    item.setAttribute('label', categories[i].resolveName());
    cat.firstChild.appendChild(item);
    if ( basetype == 'VirtualNode' && categories[i].get('name') == 'virtual-nodes' ){
      found = i;
    } else if ( nodeCategoryKey && nodeCategoryKey == categories[i].get('uniqueKey') ) {
      found = i;
    } else if(categories[i].get('uniqueKey') == cluster.base.get('defaultCategory')) {
      found = i;
    }
  }
  cat.selectedIndex = found;

  var format = cluster.base.get('slaveName') + '%0' + cluster.base.get('slaveDigits') + 'd';
  if(basetype == 'VirtualSMPNode') {
    format = 'vsmp' + format;
  } else if(basetype == 'CloudNode') {
    cloudprovider = myDialog.arguments.cloudprovider;
    format = 'c' + format;
    document.getElementById('cloudspecific').removeAttribute('hidden');
    var box = document.getElementById('type'); // TODO: add support for multiple providers
    var list = cluster.find('CloudType');
    var found = -1;
    var type = cloudprovider.get('defaultType');
    if(type) {
      type = cluster.find(type);
      if(type) {
        var item = document.createElement('menuitem');
        item.setAttribute('key', 0);
        item.setAttribute('label', type.resolveName() + ' (' + cloudprovider.resolveName() + ')');
        box.firstChild.appendChild(item);
      }
    }
    for(var i = 0; i < list.length; i++) {
      var item = document.createElement('menuitem');
      item.setAttribute('key', list[i].get('uniqueKey'));
      item.setAttribute('label', list[i].resolveName());
      box.firstChild.appendChild(item);
    }
    box.selectedIndex = 0;

    var found = 0;
    var box = document.getElementById('region');
    var popup = box.firstChild;
    while(popup.firstChild) popup.removeChild(popup.firstChild);
    var region = cloudprovider.get('defaultRegion');
    if(region) {
      region = cluster.find(region);
      if(region) {
        var item = document.createElement('menuitem');
        item.setAttribute('key', 0);
        item.setAttribute('label', region.resolveName() + ' (' + cloudprovider.resolveName() + ')');
        popup.appendChild(item);
      }
    }
    var list = cluster.find('CloudRegion');
    for(var i = 0; i < list.length; i++) {
      var name = list[i].resolveName();
      var key = list[i].get('uniqueKey');
      var item = document.createElement('menuitem');
      item.setAttribute('key', key);
      item.setAttribute('label', name);
      popup.appendChild(item);
    }
    box.selectedIndex = 0;
  } else if(basetype == 'VirtualNode') {
    document.getElementById('openstackspecific').removeAttribute('hidden');
    openstack = myDialog.arguments.openstack;
    format = 'v' + format;
    var box = document.getElementById('flavor'); // TODO: add support for multiple providers
    var flavors = cluster.find('OpenStackApiFlavor');
    var foundFlavor = 0;
    for (var i = 0; i < flavors.length; i++) {
      if(flavors[i].get('deleted')!==true && flavors[i].get('ephemeral') >= 10737418240) {
        var item = document.createElement('menuitem');
        item.setAttribute('key', flavors[i].get('uniqueKey'));
        item.setAttribute('label', flavors[i].get('name'));
        box.firstChild.appendChild(item);
      }
    }
    box.selectedIndex = foundFlavor;
  }
  document.getElementById('firstnode').value = sprintf(format, 1);
  document.getElementById('lastnode').value = sprintf(format, 16);

}

function next() {
  var pages = document.getElementById('deck').childNodes.length - 1;
  var index = parseInt(document.getElementById('deck').selectedIndex);
  switch(index) {
  case(0) :
    if(checkRange().length == 0) return;
    if(basetype == 'CloudNode') {
      var storage = document.getElementById('storage').selectedItem.getAttribute('label');
      if(storage == 'EBS') {
        var storage = document.getElementById('storagesize').value;
        if(!regexMatch(storage, '^[0-9]+(\.[0-9]*)?\s*[kmgtKMGT]?i?[Bb]?$')) {
          dock.displayError('Invalid storage size: ' + storage);
          return;
        }
      }
      spotPrice = document.getElementById('spotPrice').value;
      if(spotPrice == 'N/A') {
        spotPrice = 0;
      } else if(!isNumeric(spotPrice)) {
        dock.displayError('Spot price needs to be a positive number or N/A for on demand instances');
        return;
      }
      spotPersistent = document.getElementById('spotPersistent').checked;
    }
    break;
  case(1) :
    var n = 0;
    for(var j = 0; j < types.length; j++) {
      var netw = document.getElementById('network' + types[j]).selectedItem;
      if(netw && (netw.getAttribute('key') != '0')) {
        n++;
        if(!document.getElementById(types[j] + '-offset').hasAttribute('hidden')) {
          var ip = getOffset(types[j]);
          if(!regexMatch(ip, dock.REGEX_IPv4)) {
            dock.displayError("The ip offset for " + types[j] + "is not valid.\nUsually something like 0.0.8.0");
            return;
          }
        }
      }
    }
    if(n == 0) {
      dock.displayError("Specify at least one interface");
      return;
    }
    break;
  case(2) :
    var count = document.getElementById('switches').childNodes.length - 1;
    for(var i = 0; i < count; i++) {
      var sw = document.getElementById('switch' + i).selectedItem;
      var ps = document.getElementById('switch' + i + 'start').selectedItem;
      var pe = document.getElementById('switch' + i + 'end').selectedItem;
      var n = 0;
      if(sw) n++;
      if(ps) n++;
      if(pe) n++;
      if(n > 0 && n < 3) {
        dock.displayError("Specify a switch, start port and end port");
        return;
      } else if(n == 3) {
        var s = myParseInt(ps.getAttribute('label'));
        var e = myParseInt(pe.getAttribute('label'));
        if(s > e) {
          dock.displayError("Start port has to be smaller than end port");
          return;
        }
      }
    }
    break;
  }
  index++;
  if((index == 2) && ((basetype == 'CloudNode') || (basetype == 'VirtualNode'))) index++;
  switch(index) {
  case(1) :
    if(!networks) {
      networks = cluster.find('Network');
      for(var i = 0; i < networks.length; i++) {
        if(networks[i].get('type') == 'GLOBAL') {
          networks.splice(i, 1);
          break;
        }
      }
      if(basetype == 'CloudNode') {
        if(masterInCloud) {
          types = ['eth0'];
        } else {
          types = ['eth0', 'tun0'];
        }
      } else if(basetype == 'VirtualNode') {
        types = ['eth0', 'eth1', 'map0'];
      } else {
        types = ['eth0', 'eth1', 'eth2', 'eth3', 'eth4', 'BOOTIF', 'ib0', 'ib1', 'ipmi0', 'ilo0', 'map0'];
        if(cluster.isUCS) types.push('cimc0');
      }
      for(var j = 0; j < types.length; j++) {
        var net = document.getElementById('network' + types[j]).firstChild;
        var item = document.createElement('menuitem');
        item.setAttribute('key', 0);
        item.setAttribute('label', '');
        net.appendChild(item);
        for(var i = 0; i < networks.length; i++) {
          if(networks[i].get('type') != 'GLOBAL') {
            var item = document.createElement('menuitem');
            item.setAttribute('key', networks[i].get('uniqueKey'));
            item.setAttribute('label', networks[i].resolveName());
            net.appendChild(item);
          }
        }
      }
      for(j = 0; j < allTypes.length; j++) {
        if(types.find(allTypes[j]) < 0) {
          var row = document.getElementById(allTypes[j]);
          if(row) {
            row = row.parentNode;
            if(row) row.setAttribute('hidden', true);
          }
        }
      }
      if(basetype == 'CloudNode') {
        var nets;
        if(masterInCloud) {
          nets = cluster.findManagementNetworks();
        } else {
          nets = cluster.findCloudNetworks();
        }
        if(nets.length > 0) {
          for(var i = 0; i < networks.length; i++) {
            if(networks[i] == nets[0]) {
              document.getElementById('networketh0').selectedIndex = i + 1;
              for(j = 0; j < 4; j++) {
                document.getElementById('eth0-offset-' + j).removeAttribute('disabled');
              }
              break;
            }
          }
        }
        var nets = cluster.findTunnelNetworks();
        if(nets.length > 0 && !masterInCloud) {
          var region = document.getElementById('region').selectedItem.getAttribute('label');
          var idx = region.indexOf('(');
          if(idx > 0) region = trim(region.substr(0, idx));
          var found = false;
          var i0 = 0;
          for(var i = 0; i < networks.length; i++) {
            if(networks[i].get('type') != 'GLOBAL') {
              if(networks[i].resolveName() == region) {
                document.getElementById('networktun0').selectedIndex = i0 + 1;
                for(j = 0; j < 4; j++) {
                  document.getElementById('tun0-offset-' + j).removeAttribute('disabled');
                }
                found = true;
                break;
              }
              i0 += 1;
            }
          }
          if(!found) {
            var i0 = 0;
            for(var i = 0; i < networks.length; i++) {
              if(networks[i].get('type') != 'GLOBAL') {
                if(networks[i] == nets[0]) {
                  document.getElementById('networktun0').selectedIndex = i0 + 1;
                  for(j = 0; j < 4; j++) {
                    document.getElementById('tun0-offset-' + j).removeAttribute('disabled');
                  }
                  break;
                }
                i0 += 1;
              }
            }
          }
        }
      } else if (basetype == 'VirtualNode') {
        var net = cluster.findOpenStackNetwork();
        if(net) {
          for(var i = 0; i < networks.length; i++) {
            if(networks[i] == net) {
              document.getElementById('networketh0').selectedIndex = i + 1;
              for(j = 0; j < 4; j++) {
                document.getElementById('eth0-offset-' + j).removeAttribute('disabled');
              }
              document.getElementById('eth0-dhcp').removeAttribute('hidden');
              document.getElementById('eth0-dhcp').setAttribute('checked', true);
              break;
            }
          }
        }
      }
    }
    break;
  case(2):
    if(!switches) {
      switches = cluster.find('EthernetSwitch');
      addSwitch();
    }
    break;
  case(3):
    createTable();
//     winmax();
  }
  document.getElementById('deck').selectedIndex = index;
  document.getElementById('prev').setAttribute('disabled', index == 0);
  document.getElementById('next').setAttribute('hidden', index == pages);
  document.getElementById('finish').setAttribute('hidden', index < pages);
}

function addSwitch() {
  var count = document.getElementById('switches').childNodes.length - 1;
  var row = document.createElement('row');
  row.setAttribute('align', 'center');
  var swl = document.createElement('menulist');
  swl.setAttribute('id', 'switch' + count);
  swl.appendChild(document.createElement('menupopup'));
  var psl = document.createElement('menulist');
  psl.setAttribute('id', 'switch' + count + 'start');
  psl.appendChild(document.createElement('menupopup'));
  var pel = document.createElement('menulist');
  pel.setAttribute('id', 'switch' + count + 'end');
  pel.appendChild(document.createElement('menupopup'));
  var stl = document.createElement('menulist');
  stl.setAttribute('id', 'switch' + count + 'stride');
  stl.appendChild(document.createElement('menupopup'));
  var button = document.createElement('button');
  button.setAttribute('class', 'plus-button');
  button.setAttribute('oncommand', 'addSwitch();');

  var sw = swl.firstChild;
  var ps = psl.firstChild;
  var pe = pel.firstChild;
  var st = stl.firstChild;

  var item = document.createElement('menuitem');
  item.setAttribute('key', 0);
  item.setAttribute('label', '');
  sw.appendChild(item);
  for(var j = 0; j < switches.length; j++) {
    var item = document.createElement('menuitem');
    item.setAttribute('key', switches[j].get('uniqueKey'));
    item.setAttribute('label', switches[j].resolveName());
    sw.appendChild(item);
  }
  var item = document.createElement('menuitem');
  item.setAttribute('label', '');
  ps.appendChild(item);
  var item = document.createElement('menuitem');
  item.setAttribute('label', '');
  pe.appendChild(item);
  for(j = 1; j < dock.MAXSWITCHPORTS; j++) {
    var item = document.createElement('menuitem');
    item.setAttribute('label', j);
    ps.appendChild(item);
    var item = document.createElement('menuitem');
    item.setAttribute('label', j);
    pe.appendChild(item);
  }
  for(j = 1; j < 8; j++) {
    var item = document.createElement('menuitem');
    item.setAttribute('label', j);
    st.appendChild(item);
  }

  row.appendChild(swl);
  row.appendChild(psl);
  row.appendChild(pel);
  row.appendChild(stl);
  row.appendChild(button);
  document.getElementById('switches').appendChild(row);
  stl.selectedIndex = 0;
}

function prev() {
  var pages = document.getElementById('deck').childNodes.length - 1;
  var index = parseInt(document.getElementById('deck').selectedIndex);
  var back = true;
  switch(index) {
    case(3):
      var args = new Object();
      args['question'] = "This will undo all your manual changes, are you sure?";
      var d = new dock.cvDialog("question.xul", args);
      d.onNo = function() {
        back = false;
      }
      dock.dialogs.showDialog(d);
      break;
  }
  if(back) {
    index--;
    document.getElementById('deck').selectedIndex = index;
    document.getElementById('prev').setAttribute('disabled', index == 0);
    document.getElementById('next').setAttribute('hidden', index == pages);
    document.getElementById('finish').setAttribute('hidden', index < pages);
  }
}

function checkRange() {
  var n0 = nameNumber(document.getElementById('firstnode').value);
  var n1 = nameNumber(document.getElementById('lastnode').value);
  if(n0[0] != n1[0]) {
    dock.displayError("The name part node range should be equal");
    return [];
  } else if(n0[1].length != n1[1].length) {
    dock.displayError("The number of digits of the node range should be equal");
    return [];
  }
  var s = myParseInt(n0[1]);
  var e = myParseInt(n1[1]);
  if(s > e) {
    dock.displayError("The last node should have a larger number than the first");
    return [];
  }
  return [n0[0], s, e, n0[1].length];
}

function generateIp(network, index, offset, dhcp) {
  dhcp = isundefined(dhcp, false);
  if(dhcp || (network.get('type') == 'CLOUD')) return '0.0.0.0';
  if(masterInCloud) return '0.0.0.0';

  var ip = network.get('baseAddress');
  if(ip != '0.0.0.0') {
    ip = ip.split('.');
    ip[0] = myParseInt(ip[0]);
    ip[1] = myParseInt(ip[1]);
    ip[2] = myParseInt(ip[2]);
    ip[3] = myParseInt(ip[3]) + index;
    if(ip[3] > 254) {
      ip[2] = ip[2] + Math.floor(ip[3] / 254);
      ip[3] = (ip[3] - 1) % 254 + 1;
    }
    if(isArray(offset)) {
      var carry = 0;
      for(var i = 3; i >= 0; i--) {
        var sum = ip[i] + offset[i] + carry;
        if(sum >= 256) {
          ip[i] = sum - 256;
          carry = 1;
        } else {
          ip[i] = sum;
          carry = 0;
        }
      }
    }
    return ip.join('.');
  }
  return ip;
}

function stringAndSpaces(txt, len, spaces) {
  if(txt.length > len) return txt;
  return txt + spaces[len - txt.length];
}

function createTable() {
  var nodes = checkRange();
  if(nodes.length == 0) return false;
  var spaces = new Array();
  spaces[0] = '';
  var hostnameLen = Math.max(16, nodes[0].length + nodes[3] + 1);
  for(var i = 1; i < Math.max(32, hostnameLen); i++) {
    spaces[i] = spaces[i - 1] + ' ';
  }

  var network = new Array();
  var text = "hostname" + spaces[hostnameLen - 8];
  for(var i = 0; i < types.length; i++) {
    var net = document.getElementById('network' + types[i]).selectedItem;
    if(net && (net.getAttribute('key') != '0')) {
//     if(document.getElementById(types[i]).checked) {
      net = parseInt(net.getAttribute('key'));
      network[i] = cluster.find(net);
      var name = types[i] + '(' + network[i].resolveName() + ')';
      text += name + spaces[24 - name.length];
    }
  }

  var type;
  var storage;
  var flavor;
  var switchPorts = new Array();
  var switchPortLen = 18;
  if(basetype == 'CloudNode') {
    type = document.getElementById('type').selectedItem.getAttribute('label');
    region = document.getElementById('region').selectedItem.getAttribute('label');
    storage = document.getElementById('storage').selectedItem.getAttribute('label');
    if(storage == 'EBS') {
      storage += ' (' + document.getElementById('storagesize').value + ')';
    }
  } else if(basetype == 'VirtualNode') {
    flavor = document.getElementById('flavor').selectedItem.getAttribute('label');
  } else {
    var count = document.getElementById('switches').childNodes.length - 1;
    for(var i = 0; i < count; i++) {
      var sw = document.getElementById('switch' + i).selectedItem;
      var ps = document.getElementById('switch' + i + 'start').selectedItem;
      var pe = document.getElementById('switch' + i + 'end').selectedItem;
      var st = document.getElementById('switch' + i + 'stride').selectedItem;
      if(!!sw && !!ps && !!pe && !!st) {
        var n = sw.getAttribute('label');
        if(n.length > 0) {
          var s = myParseInt(ps.getAttribute('label'));
          var e = myParseInt(pe.getAttribute('label'));
          var st = myParseInt(st.getAttribute('label'));
          for(var j = s; j <= e; j += st) {
            var name = n + ':' + j;
            switchPorts.push(name);
            switchPortLen = Math.max(switchPortLen, name.length + 1);
          }
        }
      }
    }
  }
  
  var sizes;
  if(basetype == 'CloudNode') {
    sizes = [16, 24, 24];
    text += stringAndSpaces("storage", 16, spaces);
    text += stringAndSpaces("type", 24, spaces);
    text += stringAndSpaces("region", 24, spaces);
  } else if(basetype == 'VirtualNode') {
    sizes = [24];
    text += stringAndSpaces("flavor", 24, spaces)
  } else {
    sizes = [switchPortLen];
    text += stringAndSpaces("switch:port", sizes[0], spaces);
  }
  text += "\n";
  var data = '';
  for(var i = 0; i < text.length - 1; i++) data += '-';
  text += data + "\n";

  var offsets = new Object();
  for(var j = 0; j < types.length; j++) {
    var netw = document.getElementById('network' + types[j]).selectedItem;
    if(netw && (netw.getAttribute('key') != '0')) {
      var x = getOffset(types[j]).split('.');
      offsets[types[j]] = new Array();
      for(var i = 0; i < x.length; i++) {
        offsets[types[j]].push(myParseInt(x[i]));
      }
    } else {
      offsets[types[j]] = [0, 0, 0, 0];
    }
  }

  var format = nodes[0] + '%0' + nodes[3] + 'd';
  for(var i = nodes[1]; i <= nodes[2]; i++) {
    var data = stringAndSpaces(sprintf(format, i), hostnameLen, spaces);
    for(var j = 0; j < types.length; j++) {
      var netw = document.getElementById('network' + types[j]).selectedItem;
      if(netw && (netw.getAttribute('key') != '0')) {
        var dhcp = false;
        var obj = document.getElementById(types[j] + '-dhcp');
        if(obj) dhcp = obj.checked;
        var ip = generateIp(network[j], i, offsets[types[j]], dhcp);
        data += stringAndSpaces(ip, 24, spaces);
      }
    }
    if(basetype == 'CloudNode') {
      data += stringAndSpaces(storage, sizes[0], spaces);
      data += stringAndSpaces(type, sizes[1], spaces);
      data += region;
    } else if(basetype == 'VirtualNode') {
      data += stringAndSpaces(flavor, sizes[0], spaces);
    } else {
      var index = i - nodes[1];
      var name = '';
      if(index < switchPorts.length) {
        data += stringAndSpaces(switchPorts[index], sizes[0], spaces);
      }
    }
    text += data + "\n";
  }
  document.getElementById('data').value = text;
}

function parseTable() {
  var data = document.getElementById('data').value.split("\n");
  // TODO: line check
  var offset = new Array();
  var space = true;
  var bracket = false;
  var brace = false;
  var k = 0;
  for(var i = 0; i < data[0].length; i++) {
    if(data[0][i] == '(') {
      brace = true;
    } else if(data[0][i] == ')') {
      brace = false;
    } else if(data[0][i] == '[') {
      bracket = true;
    } else if(data[0][i] == ']') {
      bracket = false;
    } else if(!brace && !bracket) {
      if(data[0][i] == ' ') {
        space = true;
      } else if(space) {
        offset[k] = i;
        k++;
        space = false;
      }
    }
  }
  offset.push(10000);

//   var overwrite = document.getElementById('overwrite').checked;
  var create = new Array();
  var old = new Array();

  var first = trim(data[0].substr(offset[0], offset[1] - offset[0]));
  if(first == 'hostname') {
    for(var i = 2; i < data.length; i++) {
      var hostname = trim(data[i].substr(offset[0], offset[1] - offset[0]));
      if(hostname.length > 0) {
        var node = cluster.find('Device', hostname);
        if(node) {
          old.push(hostname);
//          if(overwrite) create.push(hostname);
        } else {
          create.push(hostname);
        }
      }
    }
  } else {
    dock.displayError('The first column should contain the hostname');
    return false;
  }

  if(create.length == 0) {
    dock.displayError('Nothing to do, all these nodes already exist');
    return false;
  }

  var msg = '';
  if(old.length > 0) {
/*    if(overwrite) {
      msg = 'The following nodes will be removed: ' + old.join(", ") + "\n";
    } else {*/

      msg = 'The following nodes already exists and will not be changed: ' + dotFormat(old) + "\n";
    //}
  }
  msg += 'The following nodes will be created: ' + dotFormat(create) + "\n";
  msg += 'Are you sure?';

  var args = new Object();
  args['question'] = msg;
  var d = new dock.cvDialog("question.xul", args);
  d.onOk = function() {
    createNodes(data, offset)
  }
  dock.dialogs.showDialog(d);
}

function createNodes(data, offset) {
  var ipmi0 = 0;
  var netcount = new Object();
  var todo = new Array();
  var good = true;
  for(var j = 0; j < offset.length - 1; j++) {
    var what = trim(data[0].substr(offset[j], offset[j + 1] - offset[j]));
    if(what == 'hostname') {
      todo.push("node.set('hostname', value, true, 1); good = true;");
    } else if(what == 'switch:port') {
      todo.push("good = setSwitchPort(node, value);");
    } else if(what == 'powerdistributionunit:port[,...]') {
      todo.push("good = setPDUPort(node, value);");
    } else if(what == 'rack:position:height') {
      todo.push("good = setRackPos(node, value);");
    } else if(endswith(what, ')')) { // TODO: better check
      var ii = what.indexOf('(');
      if(ii > 0) {
        var name = what.substr(0, ii);
        var net = what.substr(ii + 1, what.length - ii - 2);
        var network = cluster.find('Network', net);
        if(!network) {
          dock.displayError('Network ' + net + ' not defined');
          return false;
        }
        if(!regexMatch(name, dock.REGEX_IF_ANY)) {
          dock.displayError('Invalid network device name ' + name);
          return false;
        }
      } else {
        dock.displayError('Interfaces should ne defined as: name(network)');
        return false;
      }
      var key = network.get('uniqueKey');
      if(typeof(netcount[key]) == 'undefined') {
        netcount[key] = 1;
      } else {
        netcount[key]++;
      }
      if(name == 'ipmi0') {
        ipmi0 = key;
        todo.push("good = addInterface(node, value, '" + name + "', " + key + ", '@BMC@');");
      } else {
        todo.push("good = addInterface(node, value, '" + name + "', " + key + ", '');");
      }
    } else if(what == 'type') {
      todo.push("good = setCloudType(node, value);");
    } else if(what == 'region') {
      todo.push("good = setCloudRegion(node, value);");
    } else if(what == 'storage') {
      todo.push("good = setCloudStorage(node, value);");
    } else if(what == 'flavor') {
      todo.push("good = setOpenStackFlavor(node, value);");
    } else {
      dock.displayError("Tableheader '" + what + "' is not valid.");
      return false;
    }
  }
  var additionalName = '';
  if(ipmi0 && (netcount[ipmi0] > 1)) {
    println("count = " + netcount[ipmi0]);
    additionalName = '-bmc';
  }
  for(var i = 0; i < todo.length; i++) {
    todo[i] = todo[i].replace('@BMC@', additionalName);
  }

  var category = parseInt(document.getElementById('category').selectedItem.getAttribute('key'));
  for(var i = 2; i < data.length; i++) {
    var hostname = trim(data[i].substr(offset[0], offset[1] - offset[0]));
    if(hostname.length > 0) {
      var node = cluster.find('Device', hostname);
      if(node) {
        continue;
  /*      if(overwrite) {
          if(!cluster.removeObject(node)) {
            // TODO: error?
            continue;
          }
        } else {
          continue;
        }*/
      }
      eval('node = new dock.cv' + basetype + '();');
      node = cluster.addObject(node, false, false);
      node.set('partition', cluster.base.get('uniqueKey'), true, -1);
      node.set('category', category, true, -1);
      if(basetype == 'CloudNode') {
        node.set('powerControl', 'cloud');
      } else if(basetype == 'VirtualNode') {
        node.set('powerControl', 'openstack');
      }
      for(var j = 0; j < offset.length - 1; j++) {
        var value = trim(data[i].substr(offset[j], offset[j + 1] - offset[j]));
        var good = false;
        eval(todo[j]);
        if(!good) {
          // TODO: ERROR
          cluster.removeObject(node);
//           cluster.addOneObject(node);
          return false;
        }
      }
      checkManagementNetwork(node);
      cluster.addOneObject(node);
    }
  }
  allDone();
  return true;
}

function setSwitchPort(node, value) {
  var i = value.indexOf(':');
  if(i > 0) {
    var sw = value.substr(0, i);
    var port = myParseInt(value.substr(i + 1));
    var swi = cluster.find('EthernetSwitch', sw);
    if(!swi) {
      dock.displayError("The ethernetswitch " + sw + " is not defined");
      return false;
    }
    var sp = node.get('ethernetSwitch');
    if(!sp) sp = new dock.switchPort();
    sp.ethernetSwitch = swi.get('uniqueKey');
    sp.prt = port;
    return true;
  } else if(value == "") {
    return true;
  }
  dock.displayError("The switch port property should be set as: switch:port");
  return false;
}

function setPDUPort(node, value) {
  var list = value.split(',');
  var pdus = new Array();
  for(var j = 0; j < list.length; j++) {
    var tmp = trim(list[j]);
    if(tmp.length > 0) {
      var i = tmp.indexOf(':');
      if(i > 0) {
        var sw = tmp.substr(0, i);
        var port = myParseInt(tmp.substr(i + 1));
        var swi = cluster.find('PowerDistributionUnit', sw);
        if(!swi) {
          dock.displayError("The power distribution unit " + sw + " is not defined");
          return false;
        }
        var sp = new dock.cvPDUPort();
        var tmp = new dock.cvObject(node.cluster, sp);
        tmp.set('pdu', swi.get('uniqueKey'));
        tmp.set('prt', port);
        pdus.push(sp);
      } else {
        dock.displayError("The power distribution unit port property should be set as: pdu:port[,pdu:port,...] " + list[j]);
        return false;
      }
    }
  }
  if(list.length > 0) node.set('powerDistributionUnits', pdus, false, -1);
  return true;
}

function setRackPos(node, value) {
  if(value == "") {
    return true;
  } else {
    var list = value.split(':');
    if(list.length == 3) {
      if(!isNumeric(list[0], false) || !isNumeric(list[1], false) || !isNumeric(list[2], false)) {
        dock.displayError("The rackindex:rackposition:height have to be possitive nummeric lists");
        return false;
      }
      var rack = 0;
      var a = list[0];
      if(a) {
        rack = node.cluster.find('Rack', a);
        if(!rack) {
          dock.displayError("Unable to find rack " + a);
          return false;
        }
        a = rack.get('uniqueKey');
      } else {
        a = 0
      }
      var b = myParseInt(list[1]);
      var c = myParseInt(list[2]);
      if((a > 0) && ((b < 1) || (b > rack.get('height')))) {
        dock.displayError("Rack position should be smaller than rack height (" + rack.get('height') + ")");
        return false;
      } else if((c < 1) || (c > dock.MAXDEVICEHEIGHT)) {
        dock.displayError("Device height should be between 1 and " + dock.MAXDEVICEHEIGHT);
      }
      var rr = node.get('rack');
      rr.rack = a;
      rr.position = b;
      rr.height = c;
      rr.modified = true;
      return true;
    } else {
      dock.displayError("The rack position property should be set as: rackindex:rackposition:height");
    }
  }
  return false;
}

function addInterface(node, value, iface, network, additionalName) {
  if(regexMatch(value, dock.REGEX_IPv4)) {
    var intf;
    var provisioning = -1;
    if(regexMatch(iface, '^(ipmi|ilo)[0-9]+$')) {
      intf = new dock.cvNetworkBmcInterface();
      node.set('powerControl', iface, true, -1);
    } else if(regexMatch(iface, dock.REGEX_IF_ALIAS)) {
      intf = new dock.cvNetworkAliasInterface();
    } else if(regexMatch(iface, dock.REGEX_IF_VLAN)) {
      intf = new dock.cvNetworkVLANInterface();
    } else if(regexMatch(iface, '^map0+$')) {
      intf = new dock.cvNetworkNetMapInterface();
    } else if(regexMatch(iface, '^br[0-9]+$')) {
      intf = new dock.cvNetworkBridgeInterface();
    } else if(regexMatch(iface, '^tun[0-9]+$')) {
      intf = new dock.cvNetworkTunnelInterface();
    } else if(regexMatch(iface, dock.REGEX_IF_PHYSICAL)) {
      intf = new dock.cvNetworkPhysicalInterface();
      if(iface == 'BOOTIF') {
        provisioning = 1;
      } else {
        provisioning = 0;
      }
    } else {
      println("If you see this, the primary regex is wrong!");
      return false;
    }
    intf = new dock.cvObject(cluster, intf);
    intf.set('name', iface);
    if(value == '0.0.0.0') intf.set('dhcp', true);
    intf.set('ip', value);
    intf.set('network', network);
    if(additionalName.length > 0) {
      var names = new Array();
      names.push(node.get('hostname') + additionalName);
      intf.set('additionalHostnames', names);
    }
    var list = node.get('networks');
    list.push(intf.object);
    node.set('networks', list, true, -1);

    var oldp = node.get('provisioningNetwork');
    if(((provisioning == 0) && (oldp == 0)) || (provisioning > 0)) {
      node.set('provisioningNetwork', intf.get('uniqueKey'), true, -1);
    }
    if(basetype == 'CloudNode') {
      if(intf.object.childType == 'NetworkTunnelInterface') {
        node.set('managementNetwork', network);
      }
    }
    return true;
  } else {
    dock.displayError(value + ' is not a valid ip');
  }
  return false;
}

function checkManagementNetwork(node) {
  var setman = 0;
  var mannet = node.getManagementNetwork();
  var list = node.get('networks');
  for(var i = 0; i < list.length; i++) {
    var netw = list[i].network;
    if(netw) {
      if(netw == mannet) {
        setman = 0;
        break;
      } else {
        var network = node.cluster.find(netw);
        if(network) {
          var ntype = network.get('type');
          if((ntype == 'INTERNAL') && (list[i].childType != "NetworkAliasInterface") && (list[i].childType != "NetworkBmcInterface")) {
            setman = netw;
          } else if((ntype == 'EXTERNAL') && (setman == 0)) {
            setman = netw;
          }
        }
      }
    }
  }
  if(setman && (setman != mannet)) {
    node.set('managementNetwork', setman);
  }
}

function setCloudType(node, value) {
  var type = 0;
  if(value.indexOf('(') <= 0) {
    type = cluster.find('EC2Type', value);
    if(!type) {
      dock.displayError('Cloud type not found ' + value);
      return false;
    }
    if(isObject(type)) {
      type = type.get('uniqueKey');
    } else {
      dock.displayError('Cloud type not found ' + value);
      return false;
    }
  }
  var cloudsettings = node.get('cloudSettings');
  if(cloudsettings) {
    if(cloudsettings.childType == 'EC2Settings') {
      cloudsettings.provider = cloudprovider.get('uniqueKey');
      cloudsettings.type = type;
      // KDR: do here ?
      cloudsettings.spotPrice = spotPrice;
      cloudsettings.spotPersistent = spotPersistent;
      return true;
    }
  }
  return false;
}

function setCloudRegion(node, value) {
  var region = 0;
  if(value.indexOf('(') <= 0) {
    region = cluster.find('CloudRegion', value);
    if(!region) {
      dock.displayError('Cloud region not found ' + value);
      return false;
    }
    region = region.get('uniqueKey');
  }
  var cloudsettings = node.get('cloudSettings');
  if(cloudsettings) {
    if(cloudsettings.childType == 'EC2Settings') {
      cloudsettings.provider = cloudprovider.get('uniqueKey');
      cloudsettings.region = region;
      return true;
    }
  }
  return false;
}

function setCloudStorage(node, value) {
  var s;
  if(regexMatch(value, '^EBS\\s*\\([0-9]+(\.[0-9]*)?\s*[kmgtKMGT]?i?[Bb]?\\)$')) {
    s = new dock.cvEC2EBSStorage();
    s.name = 'ebs';
    s.drive = 'sdb';
    var i = value.indexOf('(');
    var j = value.indexOf(')');
    s.size = parseKMG(value.substr(i + 1, j - i));
  } else if(value.toLowerCase() == 'ephemeral') {
    s = new dock.cvEC2EphemeralStorage();
    s.name = 'Ephemeral0';
    s.drive = 'sdb';
  } else {
    dock.displayError('Invalid cloud storage: "' + value + '"');
    return false;
  }
  var cloudsettings = node.get('cloudSettings');
  if(cloudsettings) {
    if(cloudsettings.childType == 'EC2Settings') {
      s = new dock.cvObject(cluster, s);
      s.set('toBeRemoved', false);
      s.set('modified', true);
      var storage = cloudsettings.storage;
      storage.push(s.object);
      cloudsettings.storage = storage;
      cloudsettings.modified = true;
      return true;
    }
  }
  return false;
}

function setOpenStackFlavor(node, value) {
  var flavors = cluster.findBy({
    baseType: 'OpenStackApiFlavor',
    filter: function(flavor) {
      if(flavor.get('name') === value) {
        return true;
      }
      return false;
    }
  });
  if(flavors && flavors[0]) {
    var settings = node.get('virtualNodeSettings');
    if(settings) {
      if(settings.childType == 'OpenStackSettings') {
        settings.openStackFlavor = flavors[0].get('uniqueKey');
        return true;
      }
    }
  } else {
    println("Unable to find flavor: '" + value + "'");
  }
  return false;
}

function finish() {
  parseTable();
}

function allDone() {
  close();
  cluster.updateRack = true;
  myDialog.onOk();
}

function cancel() {
  cluster.updateRack = true;
  close();
}

function getOffset(name) {
  var offset = document.getElementById(name + '-offset-0').value;
  offset += '.' + document.getElementById(name + '-offset-1').value;
  offset += '.' + document.getElementById(name + '-offset-2').value;
  offset += '.' + document.getElementById(name + '-offset-3').value;
  return offset;
}

function setOffset(name, a, b, c, d) {
  document.getElementById(name + '-offset-0').value = a;
  document.getElementById(name + '-offset-1').value = b;
  document.getElementById(name + '-offset-2').value = c;
  document.getElementById(name + '-offset-3').value = d;
}

function setNetwork(name) {
  var netw = document.getElementById('network' + name).selectedItem;
  if(netw && (netw.getAttribute('key') != '0')) {
    document.getElementById(name + '-offset-0').removeAttribute('disabled');
    document.getElementById(name + '-offset-1').removeAttribute('disabled');
    document.getElementById(name + '-offset-2').removeAttribute('disabled');
    document.getElementById(name + '-offset-3').removeAttribute('disabled');
  } else {
    document.getElementById(name + '-offset-0').setAttribute('disabled', true);
    document.getElementById(name + '-offset-1').setAttribute('disabled', true);
    document.getElementById(name + '-offset-2').setAttribute('disabled', true);
    document.getElementById(name + '-offset-3').setAttribute('disabled', true);
  }
}

function setStorageType(event) {
  if(document.getElementById('storage').selectedItem.getAttribute('label') == 'EBS') {
    document.getElementById('storagesize').removeAttribute('disabled');
  } else {
    document.getElementById('storagesize').setAttribute('disabled', true);
  }
}
