/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */


function DomElementFactory(args) {
  this.type = null;
  this.attributes = [];
  this.events = [];

  this.create = function() {
    var element = document.createElement(this.type);
    for(var i = 0; i < this.attributes.length; i++) {
       element.setAttribute(this.attributes[i].name, this.attributes[i].value);
    }

    for(var i = 0; i < this.events.length; i++) {
      element.addEventListener(this.events[i].event, this.events[i].func, this.events[i].useCapture);
    }
    return element;
  };

  if (args.type!==undefined) {
    this.type = args.type;
  }
  if (args.attributes!==undefined) {
    this.attributes = args.attributes;
  }
  if (args.events!==undefined) {
    this.events = args.events;
  }
};


/*
 * DomManager Classes
 */
function ElementDomManager() {

  this.createElement = function(type, attributes, events) {
    var elementFactory = new DomElementFactory({
      'type': type,
      'attributes': attributes,
      'events': events
    });
    return elementFactory.create();
  };
};


function TableDomManager() {
  this.table = {};
  this.table.grid = null;
  this.table.columns = null;
  this.table.column = [];
  this.table.rows = null;
  this.table.row = [];
  this.table.cells = [];
  this.nColumns = 0;
  this.nRows = 0;

  this.setCells = function(cells) {
    this.table.cells = cells;
    this.nRows = this.table.cells.length;
    if(this.nRows>0) {
      this.nColumns = this.table.cells[0].length;
    }
    this.create();
  };

  this.get = function() {
    this.table.grid.appendChild(this.table.columns);
    this.table.grid.appendChild(this.table.rows);

    for(var i = 0; i < this.table.column.length; i++) {
      this.table.columns.appendChild(this.table.column[i]);
    }

    for(var i = 0; i < this.table.row.length; i++) {
      this.table.rows.appendChild(this.table.row[i]);
    }

    for(var i = 0; i < this.table.cells.length; i++) {
      for(var j = 0; j < this.table.cells[i].length; j++) {
        this.table.row[i].appendChild(this.table.cells[i][j]);
      }
    }
    return this.table.grid;
  };

  this.create = function() {
    this.table.grid = this.createGridElement();
    this.table.columns = this.createColumnsElement();
    this.table.rows = this.createRowsElement();

    for(var i = 0; i < this.nColumns; i++) {
      this.table.column.push(this.createColumnElement());
    }

    for(var i = 0; i < this.nRows; i++) {
      this.table.row.push(this.createRowElement());
    }
  };

  this.createGridElement = function() {
    return this.createElement('grid', new Array(), new Array());
  };

  this.createColumnsElement = function() {
    return this.createElement('columns', new Array(), new Array());
  };

  this.createColumnElement = function() {
    return this.createElement('column', new Array(), new Array());
  };

  this.createRowsElement = function() {
    return this.createElement('rows', new Array(), new Array());
  };

  this.createRowElement = function() {
    return this.createElement('row', new Array(), new Array());
  };
};

TableDomManager.prototype = new ElementDomManager();
TableDomManager.prototype.constructor = TableDomManager;


function MenuListDomManager() {
  this.menu = {};
  this.menu.menulist = null;
  this.menu.menulistAttributes = [];
  this.menu.menulistEvents = [];
  this.menu.menupopup = null;
  this.menu.menuitems = [];
  this.menu.items = [];

  this.setList = function(items) {
    this.menu.items = items;
    this.create();
  };

  this.get = function() {
    this.menu.menulist.appendChild(this.menu.menupopup);
    for(var i = 0; i < this.menu.menuitems.length; i++) {
      this.menu.menupopup.appendChild(this.menu.menuitems[i]);
    }
    return this.menu.menulist;
  };

  this.create = function() {
    this.menu.menulist = this.createMenuListElement();
    this.menu.menupopup = this.createMenuPopupElement();

    for(var i = 0; i < this.menu.items.length; i++) {
      this.menu.menuitems.push(this.createMenuItemElement(this.menu.items[i]['attributes'], this.menu.items[i]['events']));
    }
  };

  this.setMenuListElementAttributes = function(attributes) {
    this.menu.menulistAttributes = attributes;
  };

  this.setMenuListElementEvents = function(events) {
    this.menu.menulistEvents = events;
  };

  this.createMenuListElement = function() {
    return this.createElement('menulist', this.menu.menulistAttributes, this.menu.menulistEvents);
  };

  this.createMenuPopupElement = function() {
    return this.createElement('menupopup', new Array(), new Array());
  };

  this.createMenuItemElement = function(attributes, events) {
    return this.createElement('menuitem', attributes, events);
  };
};

MenuListDomManager.prototype = new ElementDomManager();
MenuListDomManager.prototype.constructor = MenuListDomManager;

function ListBoxDomManager() {
  this.listBox = {};
  this.listBox.listBox = null;
  this.listBox.listBoxAttributes = null;
  this.listBox.listHead = null;
  this.listBox.listHeader = [];
  this.listBox.listCols = null;
  this.listBox.listCol = [];
  this.listBox.listItem = [];
  this.listBox.listCell = [];
  this.listBox.cellsHeader = [];
  this.listBox.cells = [];
  this.nHeaders = 0;
  this.nColumns = 0;
  this.nRows = 0;

  this.setListBoxAttributes = function(attributes) {
    this.listBox.listBoxAttributes = attributes;
  };

  this.setCells = function(cellsHeader, cells) {
    this.listBox.cellsHeader = cellsHeader;
    this.nHeaders = this.listBox.cellsHeader.length;
    this.listBox.cells = cells;
    this.nRows = this.listBox.cells.length;
    if(this.nRows>0) {
      this.nColumns = this.listBox.cells[0].length;
    }
    this.create();
  };

  this.get = function() {
    this.listBox.listBox.appendChild(this.listBox.listHead);
    this.listBox.listBox.appendChild(this.listBox.listCols);
    for(var i = 0; i < this.listBox.listItem.length; i++) {
      this.listBox.listBox.appendChild(this.listBox.listItem[i]);
    }

    for(var i = 0; i < this.listBox.listHeader.length; i++) {
      this.listBox.listHead.appendChild(this.listBox.listHeader[i]);
    }

    for(var i = 0; i < this.listBox.cellsHeader.length; i++) {
      this.listBox.listHeader[i].appendChild(this.listBox.cellsHeader[i]);
    }

    for(var i = 0; i < this.listBox.listCol.length; i++) {
      this.listBox.listCols.appendChild(this.listBox.listCol[i]);
    }

    for(var i = 0; i < this.listBox.listCell.length; i++) {
      for(var j = 0; j < this.listBox.listCell[i].length; j++) {
        this.listBox.listItem[i].appendChild(this.listBox.listCell[i][j]);
      }
    }

    for(var i = 0; i < this.listBox.cells.length; i++) {
      for(var j = 0; j < this.listBox.cells[i].length; j++) {
        this.listBox.listCell[i][j].appendChild(this.listBox.cells[i][j]);
      }
    }
    return this.listBox.listBox;
  };

  this.getSelected = function() {
    var selected = [];
    for(var i = 0; i < this.listBox.listItem.length; i++) {
      selected.push(this.listBox.listItem[i].selected);
    }
    return selected;
  };


  this.create = function() {
    this.listBox.listBox = this.createListBoxElement();
    this.listBox.listHead = this.createListHeadElement();
    this.listBox.listCols = this.createListColsElement();


    for(var i = 0; i < this.nHeaders; i++) {
      this.listBox.listHeader.push(this.createListHeaderElement());
    }

    for(var i = 0; i < this.nColumns; i++) {
      this.listBox.listCol.push(this.createListColElement());
    }

    for(var i = 0; i < this.nRows; i++) {
      this.listBox.listItem.push(this.createListItemElement());
      this.listBox.listCell[i] = [];
      for(var j = 0; j < this.nColumns; j++) {
        this.listBox.listCell[i].push(this.createListCellElement());
      }
    }
  };

  this.createListBoxElement = function() {
    return this.createElement('listbox', this.listBox.listBoxAttributes, new Array());
  };

  this.createListHeadElement = function() {
    return this.createElement('listhead', new Array(), new Array());
  };

  this.createListHeaderElement = function() {
    return this.createElement('listheader', new Array(), new Array());
  };

  this.createListColsElement = function() {
    return this.createElement('listcols', new Array(), new Array());
  };

  this.createListColElement = function() {
    return this.createElement('listcol', new Array(), new Array());
  };

  this.createListItemElement = function() {
    return this.createElement('listitem', new Array(), new Array());
  };

  this.createListCellElement = function() {
    return this.createElement('listcell', new Array(), new Array());
  };
};

ListBoxDomManager.prototype = new ElementDomManager();
ListBoxDomManager.prototype.constructor = ListBoxDomManager;