/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */


function ComputeRoleFSMountListingController(chromeWindow) {
  this.chromeWindow = chromeWindow;

  this.getField = function() {
    return 'fsmounts';
  };

  this.getNewDataObject = function() {
    return '';
  };

  this.getEditorRule = function(key) {
    return new EditorRuleComputeFSMount(key);
  };

  this.getListing = function(object) {
    return object.get('fsmounts');
  };

  this.createTreeItem = function(one) {
    var item = new Object();
    for(var attribute in one) {
      if(attribute === 'uniqueKey') {
        item['key'] = one['uniqueKey'];
      } else {
        item['l_' + attribute] = one[attribute];
      }
    }
    return item;
  };

  this.addObject = function() {
    var dataobject = new this.dock.cvFSMount();
    dataobject.uniqueKey = ++this.object.cluster.nextLocalKey;
    this.subobject.addSubObject({'field': 'fsmounts', 'object': dataobject});
    this.refresh();
    this.clearSelected();
    this.setSelected([dataobject.uniqueKey]);
  };

  this.removeObject = function() {
    var instance = this;

    if(this.select.length === 0) {
      dock.displayError('Please select a filesystem mount.');
    } else {

      var args = new Object();
      if(this.select.length === 1) {
        args['question'] = 'Are you sure you want to remove this filesystem mount?';
      } else {
        args['question'] = 'Are you sure you want to remove these filesystem mounts?';
      }

      var d = new dock.cvDialog('chrome://cmgui/content/question.xul', args);
      d.onOk = function() {
        var roles = instance.object.get('roles');
        for(var j = 0; j < roles.length; j++) {
          if('OpenStackComputeRole' === roles[j].childType) {
            for(var i = instance.select.length - 1; i >= 0; i--) {
              for(var k = roles[j].fsmounts.length - 1; k >= 0; k--) {
                if(roles[j].fsmounts[k].uniqueKey === parseInt(instance.select[i])) {
                  roles[j].fsmounts[k].toBeRemoved = true;
                  roles[j].modified = true;
                  instance.object.object.modified = true;
                }
              }
            }
          }
        }
        instance.clearSelected();
        instance.refresh();
      };
      dock.dialogs.showDialog(d);
    }
  };

  this.revertFinalize = function() {
    var roles = this.object.get('roles');
    for(var i = 0; i < roles.length; i++) {
      if('OpenStackComputeRole' === roles[i].childType) {
        this.subobject.object = roles[i];
        break;
      }
    }

    this.refresh();
  };
};

ComputeRoleFSMountListingController.prototype = new ListingController();
ComputeRoleFSMountListingController.prototype.constructor = ComputeRoleFSMountListingController;

var listing = new ComputeRoleFSMountListingController(this);

/*
 * WRAPPERS
 */
function addObject(type) {
  listing.addObject(type);
}

/*
 * *******************************************************************************************************************************************************
 */

function EditorRuleComputeFSMount(key) {
  this.addTitle = "Add filesystem mount";
  this.editTitle = "Edit filesystem mount";
  this.api = '';
  this.key = parseInt(key);

  this.getCurrentObject = function(editorController) {
    var cluster = editorController.object.cluster;
    for(var i = 0; i < editorController.myDialog.arguments.subobject.object.fsmounts.length; i++) {
      if(editorController.myDialog.arguments.subobject.object.fsmounts[i].uniqueKey === this.key) {
        var dataobject = editorController.myDialog.arguments.subobject.object.fsmounts[i];
      }
    }
    var role = editorController.myDialog.arguments.subobject;

    return new editorController.dock.cvObject(cluster, dataobject, role, 'fsmounts');
  };

  this.dataBindingAndCheck = function(editorController) {
    this.resetEvents('formContainer');

    text('device', '^');
    text('filesystem', '^');
    text('mountpoint', '^.+');
    check('dump');
    check('rdma');
    text('mountoptions', '^');
    kenum('fsck', [
      ['0', '0'],
      ['1', '1'],
      ['2', '2']
    ]);
  };

  this.inputCheck = function(editorController) {
    if(editorController.currentObject.get('toBeRemoved')===true) {
      return true;
    }
    if(blank(editorController.currentObject.get('mountpoint'))) {
      editorController.dock.displayError("Mountpoint cannot be empty");
      return false;
    }
    return true;
  };
}

EditorRuleComputeFSMount.prototype = new EditorRuleBase();
EditorRuleComputeFSMount.prototype.constructor = EditorRuleComputeFSMount;
