/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function HAProxyRoleEntriesListingController(chromeWindow) {
  this.chromeWindow = chromeWindow;

  this.getField = function() {
    return 'entries';
  };

  this.getNewDataObject = function() {
    return new this.dock.cvHAProxyEntry();
  };

  this.getEditorRule = function(key) {
    return new EditorRuleHaproxyEntry(key);
  };

  this.getListing = function(object) {
    return object.get('entries');
  };

  this.createTreeItem = function(one) {
    var item = new Object();
    for(var attribute in one) {
      if(attribute === 'uniqueKey') {
        item['key'] = one['uniqueKey'];
      } else if(attribute === 'servers') {
        item['l_numservers'] = one[attribute].length;
      } else if(attribute === 'bindLocations') {
        item['l_numbindLocations'] = one[attribute].length;
      } else {
        item['l_' + attribute] = one[attribute];
      }
    }
    return item;
  };

  this.removeObject = function() {
    var instance = this;

    if(this.select.length === 0) {
      dock.displayError('Please select an entry.');
    } else {

      var args = new Object();
      if(this.select.length === 1) {
        args['question'] = 'Are you sure you want to remove this entry?';
      } else {
        args['question'] = 'Are you sure you want to remove these entries?';
      }

      var d = new dock.cvDialog('chrome://cmgui/content/question.xul', args);
      d.onOk = function() {
        var roles = instance.object.get('roles');
        for(var j = 0; j < roles.length; j++) {
          if('HAProxyRole' === roles[j].childType) {
            for(var i = instance.select.length - 1; i >= 0; i--) {
              for(var k = roles[j].entries.length - 1; k >= 0; k--) {
                if(roles[j].entries[k].uniqueKey === parseInt(instance.select[i])) {
                  roles[j].entries[k].toBeRemoved = true;
                  roles[j].modified = true;
                  instance.object.object.modified = true;
                }
              }
            }
          }
        }
        instance.clearSelected();
        instance.refresh();
      };
      dock.dialogs.showDialog(d);
    }
  };
};

HAProxyRoleEntriesListingController.prototype = new ListingController();
HAProxyRoleEntriesListingController.prototype.constructor = HAProxyRoleEntriesListingController;

var listing = new HAProxyRoleEntriesListingController(this);



/*
 * *******************************************************************************************************************************************************
 */

function EditorRuleHaproxyEntry(key) {
  this.addTitle = "Add HAProxy Entry";
  this.editTitle = "Edit HAProxy Entry";
  this.api = 'Domain';
  this.key = parseInt(key);

  this.getCurrentObject = function(editorController) {
    var cluster = editorController.object.cluster;
    for(var i = 0; i < editorController.myDialog.arguments.subobject.object.entries.length; i++) {
      if(editorController.myDialog.arguments.subobject.object.entries[i].uniqueKey === this.key) {
        var dataobject = editorController.myDialog.arguments.subobject.object.entries[i];
      }
    }
    var role = editorController.myDialog.arguments.subobject;

    return new editorController.dock.cvObject(cluster, dataobject, role, 'entries');
  };

  this.dataBindingAndCheck = function(editorController) {
    this.resetEvents('formContainer');

    text('name', '^');
    check('useCustomSettings');
    addViewConditional({
      'type': 'check',
      'elementContext': document,
      'elementId': 'useCustomSettings',
      'callName': 'checkUseCustomSettings',
      'initialValue': editorController.currentObject.get('useCustomSettings')
    });

    check('useCustomServerSettings');
    addViewConditional({
      'type': 'check',
      'elementContext': document,
      'elementId': 'useCustomServerSettings',
      'callName': 'checkUseCustomServerSettingsEntry',
      'initialValue': editorController.currentObject.get('useCustomServerSettings')
    });

    text('customCode');
    text('customCodeAfterServers');


    var menulistCat = [{ 'value': 0, 'label': '' }];
    var found = editorController.currentObject.cluster.find('NodeCategory');
    for(var i=0; i < found.length; i++) {
      menulistCat.push({ 'value': found[i].object.uniqueKey, 'label': found[i].resolveName() });
    }

    var menulistNet = [{ 'value': 0, 'label': '' }];
    var found = editorController.currentObject.cluster.find('Network');
    for(var i=0; i < found.length; i++) {
      menulistNet.push({ 'value': found[i].object.uniqueKey, 'label': found[i].resolveName() });
    }

    function nodeTemplateCloning(caller, elementConfig, row, col) {
      var elementCloned = document.getElementById('template_nodes').cloneNode(true);
      elementCloned.id = 'nodes_' + row;
      elementCloned.hidden = false;


      var list = editorController.currentObject.cluster.find('Node');

      var cvHAProxyServer = new dock.cvObject(editorController.currentObject.cluster, editorController.currentObject.get('servers')[row], editorController.currentObject, 'servers');

      var objectDualViewAttributeListable = new ObjectDualViewAttributeListable(list, cvHAProxyServer, 'nodes');
      objectDualViewAttributeListable.setColumnOrder({
        'hostname': 0
      });
      objectDualViewAttributeListable.initInputValue();

      var customListDataViewMapper = new CustomDualListDataViewMapper(objectDualViewAttributeListable);
      customListDataViewMapper.setElementsConfig([
        [
          {'type': 'label', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []}
        ]
      ]);

      var groupSelectorWidget = new GroupSelectorWidget({
        'context': elementCloned,
        'rootNodeId': 'nodesSelected',
        'rootNodeIdSelection': 'nodesSelection',
        'rootNodeIdAdd': 'nodesButtonAdd',
        'rootNodeIdDel': 'nodesButtonDel',
        'dataViewMapper': customListDataViewMapper
      });
      groupSelectorWidget.populate();

      groupSelectorWidget.addItemCallBack = function() {
        caller.attributeListable.change(row, col, this.dataViewMapper.attributeListable.inputValue);
      };
      groupSelectorWidget.delItemCallBack = function() {
        caller.attributeListable.change(row, col, this.dataViewMapper.attributeListable.inputValue);
      };
      return elementCloned;
    }

    var arrayObjectAttributeListable = new ArrayObjectAttributeListable(editorController.currentObject, 'servers');
    arrayObjectAttributeListable.setObject(editorController.dock.cvHAProxyServer);
    arrayObjectAttributeListable.setColumnOrder({
      'name': 0,
      'address': 1,
      'port': 2,
      'backup': 3,
      'check': 4,
      'disabled': 5,
      'useCustomServerSettings': 6,
      'forceIPs': 7,
      'inter': 8,
      'fastInter': 9,
      'downInter': 10,
      'rise': 11,
      'fall': 12,
      'maxConn': 13,
      'minConn': 14,
      'maxQueue': 15,
      'weight': 16,
      'customOptions': 17,
      'isOpenStackManagedServer': 18,
      'category': 19,
      'network': 20,
      'secondaryNetwork': 21,
      'nodes': 22
    });
    //arrayObjectAttributeListable.addRowOrder(ColumnAttributeListableSorter);
    arrayObjectAttributeListable.initInputValue();

    var customListDataViewMapper = new CustomListDataViewMapper(arrayObjectAttributeListable);
    customListDataViewMapper.setElementsConfig([
      [
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'checkimage', 'attributes': [], 'events': []},
        {'type': 'checkimage', 'attributes': [], 'events': []},
        {'type': 'checkimage', 'attributes': [], 'events': []},
        {'type': 'checkimage', 'attributes': [], 'events': [{'event': 'click', 'func': this.setUseCustomServerSettingsEvent.bind(this), 'useCapture': true}]},
        {'type': 'menulist', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': [], 'menuitems': [
          { 'value': 'PREFERIPS', 'label': 'PREFERIPS' },
          { 'value': 'PREFERHOSTNAMES', 'label': 'PREFERHOSTNAMES' },
          { 'value': 'PREFERDEFAULT', 'label': 'PREFERDEFAULT' }
        ]},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'checkimage', 'attributes': [{'name': 'disabled', 'value': 'true'}], 'events': []},
        {'type': 'menulist', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': [], 'menuitems': menulistCat},
        {'type': 'menulist', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': [], 'menuitems': menulistNet},
        {'type': 'menulist', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': [], 'menuitems': menulistNet},
        {'type': 'template', 'attributes': [], 'events': [], 'cloneFunction': nodeTemplateCloning.bind(this)}
      ]
    ]);
    customListDataViewMapper.setNewItem(['', '', '', false, true, false, false, 'PREFERDEFAULT', '2000ms', '', '', 2, 2, 0, 0, 0, 0, '', false, 0, 0, 0, []]);

    var templateListDataViewMapper = new TemplateListDataViewMapper(customListDataViewMapper, document.getElementById('template_servers'), 'servers_0_');

    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'servers',
      'dataViewMapper': templateListDataViewMapper
    });
    addDelListWidget.populate();

    this.initUseCustomServerSettings();
    this.initIsOpenStackManagedServer();

    var arrayObjectAttributeListable = new ArrayObjectAttributeListable(editorController.currentObject, 'bindLocations');
    arrayObjectAttributeListable.setObject(editorController.dock.cvHAProxyServer);
    arrayObjectAttributeListable.setColumnOrder({
      'name': 0,
      'listenOn': 1,
      'ip': 2,
      'location': 3,
      'port': 4,
      'portEnd': 5,
      'parameters': 6,
      'disabled': 7,
      'network': 8
    });
    //arrayObjectAttributeListable.addRowOrder(ColumnAttributeListableSorter);
    arrayObjectAttributeListable.initInputValue();

    var customListDataViewMapper = new CustomListDataViewMapper(arrayObjectAttributeListable);
    customListDataViewMapper.setElementsConfig([
      [
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'menulist', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': [{'event': 'command', 'func': this.setListenOnEvent.bind(this), 'useCapture': true}], 'menuitems': [
          { 'value': 'ALLINTERFACES', 'label': 'All interfaces' },
          { 'value': 'NETWORK', 'label': 'Network' },
          { 'value': 'IPADDRESS', 'label': 'IP Address' },
          { 'value': 'SOCKET', 'label': 'Socket' }
        ]},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '92px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '91px'}, {'name': 'placeholder', 'value': 'Optional'}], 'events': [],
          'valueViewMapper': function(value) {
            return (value==0) ? '' : value;
          },
          'valueModelMapper': function(value) {
            return (value==='') ? 0 : value;
          }
        },
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': []},
        {'type': 'checkimage', 'attributes': [], 'events': []},
        {'type': 'menulist', 'attributes': [{'name': 'width', 'value': '200px'}], 'events': [], 'menuitems': menulistNet}
      ]
    ]);
    customListDataViewMapper.setNewItem(['', 'ALLINTERFACES', '0.0.0.0', '', 0, 0, '', false, 0]);

    var templateListDataViewMapper = new TemplateListDataViewMapper(customListDataViewMapper, document.getElementById('template_bindLocations'), 'entryBind_0_');

    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'bindLocations',
      'dataViewMapper': templateListDataViewMapper
    });
    addDelListWidget.populate();

    this.initListenOn();

    kenum('sharedSettings', [['TCP', 'TCP'], ['HTTP', 'HTTP'], ['DEFAULTPROXYMODE', 'DEFAULTPROXYMODE']], 'proxyMode');
    kenum('sharedSettings', [
      ['ROUNDROBIN', 'ROUNDROBIN'], ['STATICROUNDROBIN', 'STATICROUNDROBIN'], ['LEASTCONNECTIONS', 'LEASTCONNECTIONS'], ['FIRST', 'FIRST'],
      ['SOURCE', 'SOURCE'], ['URI', 'URI'], ['URLPARAM', 'URLPARAM'], ['HEADER', 'HEADER'], ['RDPCOOKIE', 'RDPCOOKIE'], ['DEFAULTBALANCEMODE','DEFAULTBALANCEMODE']
    ], 'balanceMode');
    text('sharedSettings', false, 'balanceArgument');
    text('sharedSettings', false, 'balanceArguments');
    check('sharedSettings', false, 'optionTCPLog');
    addViewConditional({
      'type': 'enum',
      'elementContext': document,
      'elementId': 'sharedSettingsproxyMode',
      'callName': 'checkSharedSettingsproxyModeTCPLog',
      'initialValue': editorController.currentObject.get('sharedSettings').proxyMode,
      'evalFunc': function(key) {
        return (key==='TCP') ? false : true;
      }
    });
    addViewConditional({
      'type': 'enum',
      'elementContext': document,
      'elementId': 'sharedSettingsbalanceMode',
      'callName': 'checkSharedSettingsbalanceModeDefault',
      'initialValue': editorController.currentObject.get('sharedSettings').balanceMode,
      'evalFunc': function(key) {
        return (key!=='DEFAULTBALANCEMODE') ? false : true;
      }
    });

    check('sharedSettings', false, 'optionRedispatch');
    text('sharedSettings', false, 'maxConnections');
    text('sharedSettings', false, 'timeoutConnect');
    text('sharedSettings', false, 'timeoutClient');
    text('sharedSettings', false, 'timeoutServer');
    check('sharedSettings', false, 'disabled');
    text('sharedSettings', false, 'retries');
    text('sharedSettings', false, 'log');

    text('sharedSettings', false, 'defaultServerInter');
    text('sharedSettings', false, 'defaultServerFastInter');
    text('sharedSettings', false, 'defaultServerDownInter');
    text('sharedSettings', false, 'defaultServerRise');
    text('sharedSettings', false, 'defaultServerFall');
    text('sharedSettings', false, 'defaultServerMaxConn');
    text('sharedSettings', false, 'defaultServerMinConn');
    text('sharedSettings', false, 'defaultServerMaxQueue');
    text('sharedSettings', false, 'defaultServerWeight');
  };

  this.setUseCustomServerSettingsEvent = function(event){
    var elementContext = event.target.parentNode.parentNode.parentNode.parentNode.parentNode;
    var hidden = event.target.checked;
    this.setViewConditional(elementContext, 'checkUseCustomServerSettings', hidden);
  };

  this.initUseCustomServerSettings = function(){
    var elements = document.getElementsByClassName('servers_0_6');
    for(var i = 0; i < elements.length; i++) {
      // FM: Crazy hack to check if checkimage is checked
      if(elements[i].firstChild && elements[i].outerHTML) {
        if(elements[i].outerHTML.match(/checked="true"/)) {
          var elementContext = elements[i].parentNode.parentNode.parentNode.parentNode;
          this.setViewConditional(elementContext, 'checkUseCustomServerSettings', false);
        }
      }
    }
  };

  this.initIsOpenStackManagedServer = function(){
    var elements = document.getElementsByClassName('servers_0_18');
    for(var i = 0; i < elements.length; i++) {
      // FM: Crazy hack to check if checkimage is checked
      if(elements[i].firstChild && elements[i].outerHTML) {
        if(elements[i].outerHTML.match(/checked="true"/)) {
          var elementContext = elements[i].parentNode.parentNode.parentNode.parentNode;
          this.setDisabledConditional(elementContext, 'checkIsOpenStackManagedServer', true);
        }
      }
    }
  };

  this.setListenOnEvent = function(event) {
    var elementContext = event.target.parentNode.parentNode.parentNode.parentNode.parentNode.parentNode;
    var value = event.target.value;
    this.setListenOn(elementContext, value);
  };

  this.setListenOn = function(elementContext, value) {
    var hidden = (value==='ALLINTERFACES' || value==='NETWORK' || value==='IPADDRESS') ? false : true;
    this.setViewConditional(elementContext, 'checkPort', hidden);
    var hidden = (value==='ALLINTERFACES') ? false : true;
    this.setViewConditional(elementContext, 'checkAllInterfaces', hidden);
    var hidden = value==='NETWORK' ? false : true;
    this.setViewConditional(elementContext, 'checkNetwork', hidden);
    var hidden = value==='IPADDRESS' ? false : true;
    this.setViewConditional(elementContext, 'checkIPAddress', hidden);
    var hidden = value==='SOCKET' ? false : true;
    this.setViewConditional(elementContext, 'checkSocket', hidden);
  };

  this.initListenOn = function(){
    var elements = document.getElementsByClassName('entryBind_0_1');
    for(var i = 0; i < elements.length; i++) {
      // FM: Crazy hack to check if checkimage is checked
      if(elements[i].firstChild && elements[i].outerHTML) {
        var elementContext = elements[i].parentNode.parentNode.parentNode.parentNode;
        if(elements[i].outerHTML.match(/value="ALLINTERFACES" label="All interfaces" selected="true"/)) {
          var value = 'ALLINTERFACES';
        } else if(elements[i].outerHTML.match(/value="NETWORK" label="Network" selected="true"/)) {
          var value = 'NETWORK';
        } else if(elements[i].outerHTML.match(/value="IPADDRESS" label="IP Address" selected="true"/)) {
          var value = 'IPADDRESS';
        } else if(elements[i].outerHTML.match(/value="SOCKET" label="Socket" selected="true"/)) {
          var value = 'SOCKET';
        }
        this.setListenOn(elementContext, value);
      }
    }
  };

  this.setDisabledConditional = function(elementContext, className, disabled){
    var elements = elementContext.getElementsByClassName(className);
    for(var i = 0; i < elements.length; i++) {
      elements[i].firstChild.setAttribute('disabled', disabled);
    }
  };

  this.setViewConditional = function(elementContext, className, hidden){
    var elements = elementContext.getElementsByClassName(className);
    for(var i = 0; i < elements.length; i++) {
      elements[i].setAttribute('hidden', hidden);
    }
  };

  this.inputCheck = function(editorController) {
    if(editorController.currentObject.get('toBeRemoved')===true) {
      return true;
    }
    if(blank(editorController.currentObject.get('name'))) {
      editorController.dock.displayError("Name cannot be empty");
      return false;
    }

    return true;
  };
}

EditorRuleHaproxyEntry.prototype = new EditorRuleBase();
EditorRuleHaproxyEntry.prototype.constructor = EditorRuleHaproxyEntry;
