/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */


function VolumeRoleBackendListingController(chromeWindow) {
  this.chromeWindow = chromeWindow;

  this.getField = function() {
    return 'volumeBackends';
  };

  this.getNewDataObject = function() {
    return '';
  };

  this.getEditorRule = function(key) {
    return new EditorRuleVolumeBackend(key);
  };

  this.getListing = function(object) {
    return object.get('volumeBackends');
  };

  this.createTreeItem = function(one) {
    var item = new Object();
    for(var attribute in one) {
      if(attribute === 'uniqueKey') {
        item['key'] = one['uniqueKey'];
      } else if(attribute === 'childType') {
        item['l_backendType'] = one[attribute].replace('OpenStackVolumeBackend', '');
      } else {
        item['l_' + attribute] = one[attribute];
      }
    }
    return item;
  };

  this.addObject = function(type) {
    switch(type) {
      case 'NFS':
        var dataobject = new this.dock.cvOpenStackVolumeBackendNFS();
        dataobject.volumeDriver = 'cinder.volume.drivers.nfs.NfsDriver';
        break;
      case 'SOLIDFIRE':
        var dataobject = new this.dock.cvOpenStackVolumeBackendSolidFire();
        dataobject.volumeDriver = 'cinder.volume.drivers.solidfire.SolidFireDriver';
        break;
      case '3PAR':
        var dataobject = new this.dock.cvOpenStackVolumeBackend3PAR();
        dataobject.volumeDriver = 'cinder.volume.drivers.san.hp.hp_3par_iscsi.HP3PARFCDriver';
        break;
      case 'GPFS':
        var dataobject = new this.dock.cvOpenStackVolumeBackendGPFS();
        dataobject.volumeDriver = 'cinder.volume.drivers.ibm.gpfs.GPFSDriver';
        break;
      case 'NETAPP':
        var dataobject = new this.dock.cvOpenStackVolumeBackendNetApp();
        dataobject.volumeDriver = 'cinder.volume.drivers.netapp.common.NetAppDriver';
        break;
      case 'DELL':
        var dataobject = new this.dock.cvOpenStackVolumeBackendDellStorageCenter();
        dataobject.volumeDriver = 'cinder.volume.drivers.dell.dell_storagecenter_fc.DellStorageCenterFCDriver';
        break;
      case 'CEPH':
      default:
        var dataobject = new this.dock.cvOpenStackVolumeBackendCeph();
        dataobject.volumeDriver = 'cinder.volume.drivers.rbd.RBDDriver';
        break;
    }
    dataobject.uniqueKey = ++this.object.cluster.nextLocalKey;
    dataobject.modified = true;
    var boxed_object = dock.cvObject(this.object.cluster, dataobject, this.subobject, 'volumeBackends');
    this.subobject.addSubObject({'field': 'volumeBackends', 'object': dataobject});
    this.refresh();
    this.clearSelected();
    this.setSelected([dataobject.uniqueKey]);
  };

  this.removeObject = function() {
    var instance = this;

    if(this.select.length === 0) {
      dock.displayError('Please select a volume backend.');
    } else {

      var args = new Object();
      if(this.select.length === 1) {
        args['question'] = 'Are you sure you want to remove this volume backend?';
      } else {
        args['question'] = 'Are you sure you want to remove these volumes backend?';
      }

      var d = new dock.cvDialog('chrome://cmgui/content/question.xul', args);
      d.onOk = function() {
        var roles = instance.object.get('roles');
        for(var j = 0; j < roles.length; j++) {
          if('OpenStackVolumeRole' === roles[j].childType) {
            for(var i = instance.select.length - 1; i >= 0; i--) {
              for(var k = roles[j].volumeBackends.length - 1; k >= 0; k--) {
                if(roles[j].volumeBackends[k].uniqueKey === parseInt(instance.select[i])) {
                  roles[j].volumeBackends[k].toBeRemoved = true;
                  roles[j].modified = true;
                  instance.object.object.modified = true;
                }
              }
            }
          }
        }
        instance.clearSelected();
        instance.refresh();
      };
      dock.dialogs.showDialog(d);
    }
  };

  this.revertFinalize = function() {
    var roles = this.object.get('roles');
    for(var i = 0; i < roles.length; i++) {
      if('OpenStackVolumeRole' === roles[i].childType) {
        this.subobject.object = roles[i];
        break;
      }
    }

    this.refresh();
  };
};

VolumeRoleBackendListingController.prototype = new ListingController();
VolumeRoleBackendListingController.prototype.constructor = VolumeRoleBackendListingController;

VolumeRoleBackendListingController.prototype.onLoad = function() {
  this.initDock();
  this.object = window.parent.current;
  this.subobject = window.parent.role || window.parent.current;
  this.originalObjectData = this.subobject.old(false, true);
  this.initNodeElementTree();
  this.populateTable();
  this.edit_objects = {};
}

VolumeRoleBackendListingController.prototype.initEditor = function(key) {
  if(this.edit_objects[key] === undefined ) {
    var backends = this.subobject.get('volumeBackends');
    for(var i = 0; i < backends.length; i++) {
      if(backends[i].uniqueKey == key) {
        this.edit_objects[key] = new dock.cvObject(this.subobject.cluster, backends[i]);      
      }
    }
  }
  if(this.edit_objects[key]) {
    this.dataBindingAndCheck(this.edit_objects[key]);
  }
};


VolumeRoleBackendListingController.prototype.setModified = function(object) {
  object.set('modified', true);
  this.object.set('modified', true);
  this.subobject.set('modified', true);
  refresh();
}

// Todo: make more generic when used second time 
VolumeRoleBackendListingController.prototype.text = function(field, object, extra_options={}) {
  var opt = { what: field, currentObject: object, keyUpIsChange: true};
  for(var k in extra_options) {
    opt[k] = extra_options[k];
  }
  text(opt);
  var instance = this;
  var elementNodeId = (opt.hasOwnProperty('elementNodeId') ? opt['elementNodeId'] : field);
  document.getElementById(elementNodeId).addEventListener("change", function() { instance.setModified(object); }, true);
  document.getElementById(elementNodeId).addEventListener("keyup",  function() { instance.setModified(object); }, true);
  document.getElementById(elementNodeId).addEventListener("input",  function() { instance.setModified(object); }, true);
}

// Todo: make more generic when used second time 
VolumeRoleBackendListingController.prototype.kenum = function(field, object, list, extra_options={}) {
  var elementNodeId = extra_options.hasOwnProperty('elementNodeId') ? extra_options['elementNodeId'] : field;
  var oncommandfunction = extra_options.hasOwnProperty('oncommand') ? extra_options['oncommand'] : null;
  var subscript = extra_options.hasOwnProperty('subscript') ? extra_options['subscript'] : '';
  var instance = this;
  kenum(field, list, subscript, elementNodeId, object, function() { 
    if(typeof oncommandfunction == 'function') {
      oncommandfunction(document.getElementById(elementNodeId).value);
    }
    instance.setModified(object); });
}

// Todo: make more generic when used second time 
VolumeRoleBackendListingController.prototype.check = function(field, object, extra_options = {}) {
  var opt = { what: field, currentObject: object };
  for(var k in extra_options) {
    opt[k] = extra_options[k];
  }
  var elementNodedId = opt.hasOwnProperty('elementNodeId') ? opt['elementNodeId'] : field;
  check(opt);
  var instance = this;
  document.getElementById(elementNodedId).addEventListener("custom-change", function() { instance.setModified(object); });
}

// Todo: make more generic when used second time 
VolumeRoleBackendListingController.prototype.bindSingleArrayAttributeListable = function(field, object) {
  var arrayAttributeListable = new SingleArrayAttributeListable(object, field);
  var instance = this;
  arrayAttributeListable.changeCallback = function() {
    instance.setModified(object); 
    return true;
  }
  return arrayAttributeListable;
}

var current;

VolumeRoleBackendListingController.prototype.dataBindingAndCheck = function(object) {
  //this.resetEvents('formContainer');
  current = object;
  var obj = document.getElementById('formContainer');
  var elClone = obj.cloneNode(true);
  obj.parentNode.replaceChild(elClone, obj);
  text({ what: 'name', format: '^', currentObject: object, keyUpIsChange: true});
  var instance = this;
  document.getElementById('name').addEventListener("change", function() { instance.setModified(object); }, true);
  document.getElementById('name').addEventListener("keyup", function() { instance.setModified(object); }, true);
  document.getElementById('name').addEventListener("input", function() { instance.setModified(object); }, true);


  if (object.get('childType') === 'OpenStackVolumeBackendCeph') {
    //CEPH
    var volumeDriver = [
      ['cinder.volume.drivers.rbd.RBDDriver', 'cinder.volume.drivers.rbd.RBDDriver']
    ];
    this.check('rbdFlattenVolumeFromSnapshot', object);
    this.text('rbdPool', object);
    this.text('rbdMaxCloneDepth', object);
    this.text('rbdUser', object);
    this.text('rbdSecretUuid', object);
    this.text('rbdCephConf',object);
    showForm('ceph');
  } else if (object.get('childType') === 'OpenStackVolumeBackendNFS') {
    //NFS
    var volumeDriver = [
      ['cinder.volume.drivers.nfs.NfsDriver', 'cinder.volume.drivers.nfs.NfsDriver']
    ];
    this.text('nfsDiskUtil', object);
    this.text('nfsMountPointBase', object);
    this.check('nfsSparsedVolumes', object);
    this.text('nfsSharesConfig', object);

    var arrayAttributeListable = this.bindSingleArrayAttributeListable('nfsShares', object);
    var textboxListDataViewMapper = new TextboxListDataViewMapper(arrayAttributeListable);
    textboxListDataViewMapper.setNewItem(['']);
    textboxListDataViewMapper.setCommonAttributes([{'name': 'width', 'value': '250px'}]);
    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'nfsShares',
      'dataViewMapper': textboxListDataViewMapper
    });
    addDelListWidget.populate();
    showForm('nfs');
  } else if (object.get('childType') === 'OpenStackVolumeBackendSolidFire') {
    //SolidFire
    var volumeDriver = [
      ['cinder.volume.drivers.solidfire.SolidFireDriver', 'cinder.volume.drivers.solidfire.SolidFireDriver']
    ];
    this.text('sanIP', object);
    this.text('sanLogin', object);
    this.text('sanPassword', object);
    this.text('accountPrefix', object);
    this.text('templateAccountName', object);
    this.check('allowTenantQoS', object);
    this.check('allowTemplateCaching', object);
    this.text('apiPort', object, { format: '^[0-9]{1,5}$'});
    this.check('emulate512',object);
    showForm('solidFire');
  } else if (object.get('childType') === 'OpenStackVolumeBackend3PAR') {
    //3PAR
    var volumeDriver = [
      ['cinder.volume.drivers.san.hp.hp_3par_iscsi.HP3PARFCDriver', 'cinder.volume.drivers.san.hp.hp_3par_iscsi.HP3PARFCDriver']
    ];
    this.kenum('driverType', object, [['iSCSI', 'iSCSI'], ['FibreChannel', 'Fibre Channel']]);
    this.text('apiUrl', object);
    this.text('username', object);
    this.text('password', object);
    this.text('sanIP', object, { elementNodeId: '3par-sanIP' });
    this.text('sanLogin', object, { elementNodeId: '3par-sanLogin' });
    this.text('sanPassword', object, { elementNodeId: '3par-sanPassword' });
    var arrayAttributeListable = this.bindSingleArrayAttributeListable('cpg', object);
    var textboxListDataViewMapper = new TextboxListDataViewMapper(arrayAttributeListable);
    textboxListDataViewMapper.setNewItem(['']);
    textboxListDataViewMapper.setCommonAttributes([{'name': 'width', 'value': '250px'}]);
    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'cpg',
      'dataViewMapper': textboxListDataViewMapper
    });
    addDelListWidget.populate();
    this.text('cpgSnap',object);
    this.text('snapshotRetention', object, { format: '^[0-9]{1,10}$'});
    this.text('snapshotExpiration', object, { format: '^[0-9]{1,10}$'});

    var arrayAttributeListable = this.bindSingleArrayAttributeListable('iscsiIps', object);
    var textboxListDataViewMapper = new TextboxListDataViewMapper(arrayAttributeListable);
    textboxListDataViewMapper.setNewItem(['']);
    textboxListDataViewMapper.setCommonAttributes([{'name': 'width', 'value': '250px'}]);
    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'iscsiIps',
      'dataViewMapper': textboxListDataViewMapper
    });
    addDelListWidget.populate();
    this.check('iscsiChapEnabled', object);

    addViewConditional({
      'type': 'enum',
      'elementContext': document,
      'elementId': 'driverType',
      'callName': 'checkDriverType',
      'initialValue': object.get('driverType'),
      'evalFunc': function(key) {
        return (key!=='FibreChannel') ? false : true;
      }
    });

    this.check('debug', object);
    showForm('3par');
  } else if (object.get('childType') === 'OpenStackVolumeBackendGPFS') {
    //GPFS
    var volumeDriver = [
      ['cinder.volume.drivers.ibm.gpfs.GPFSDriver', 'cinder.volume.drivers.ibm.gpfs.GPFSDriver']
    ];
    this.text('imagesDir',object);
    this.kenum('imagesShareMode',object, [['Copy', 'Copy'], ['CopyOnWrite', 'Copy on write']]);
    this.text('maxCloneDepth', object, { format: '^[0-9]{1,10}$'});
    this.text('mountPointBase', object);
    this.check('sparseVolumes', object);
    this.text('storagePool', object);
    showForm('gpfs');
  } else if (object.get('childType') === 'OpenStackVolumeBackendNetApp') {
    //NetApp
    var volumeDriver = [
      ['cinder.volume.drivers.netapp.common.NetAppDriver', 'cinder.volume.drivers.netapp.common.NetAppDriver']
    ];
    this.kenum('storageFamily', object, [['Cluster', 'ontap cluster'], ['7mode', 'ontap 7-mode']]);
    this.kenum('storageProtocol', object, [['iSCSI', 'iSCSI'], ['NFS', 'NFS'], ['FibreChannel', 'Fibre Channel']]);
    this.text('hostname', object, { format: '^.+$'});
    this.text('port', object, { format: '^[0-9]+$'});
    this.text('transportType', object, { format: '^.+$' });
    this.text('login', object, { format: '^.+$'});
    this.text('password', object, { 'elementNodeId': 'netapp-password', format: '^.+$'});
    this.text('vserver', object);
    this.check('allowMultiAttach', object, { 'elementNodeId': 'netapp-allowMultiAttach'});
    this.text('partnerBackend', object);
    this.text('webServicePath', object);
    var arrayAttributeListable = this.bindSingleArrayAttributeListable('controllerIps', object);
    var textboxListDataViewMapper = new TextboxListDataViewMapper(arrayAttributeListable);
    textboxListDataViewMapper.setNewItem(['']);
    textboxListDataViewMapper.setCommonAttributes([{'name': 'width', 'value': '250px'}]);
    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'controllerIps',
      'dataViewMapper': textboxListDataViewMapper
    });
    addDelListWidget.populate();
    //this.text('saPassword',object);
    //var arrayAttributeListable = this.bindSingleArrayAttributeListable(editorController.currentObject, 'storagePools');
    //var textboxListDataViewMapper = new TextboxListDataViewMapper(arrayAttributeListable);
    //textboxListDataViewMapper.setNewItem(['']);
    //textboxListDataViewMapper.setCommonAttributes([{'name': 'width', 'value': '250px'}]);
    //var addDelListWidget = new AddDelListWidget({
    //  'rootNodeId': 'storagePools',
    //  'dataViewMapper': textboxListDataViewMapper
    //});
    //addDelListWidget.populate();
    //this.text('eseriesHostType', object);
    this.text('hostType', object, { elementNodeId: 'netapp-hostType' });
    this.text('osType', object, { elementNodeId: 'netapp-osType' });
    this.check('spaceReservation', object, { elementNodeId: 'netapp-spaceReservation'});
    this.text('poolNameSearchPattern', object, { elementNodeId: 'netapp-poolNameSearchPattern' });


    //var arrayAttributeListable = this.bindSingleArrayAttributeListable(editorController.currentObject, 'volumeList');
    //var textboxListDataViewMapper = new TextboxListDataViewMapper(arrayAttributeListable);
    //textboxListDataViewMapper.setNewItem(['']);
    //textboxListDataViewMapper.setCommonAttributes([{'name': 'width', 'value': '250px'}]);
    //var addDelListWidget = new AddDelListWidget({
    //  'rootNodeId': 'volumeList',
    //  'dataViewMapper': textboxListDataViewMapper
    //});
    //addDelListWidget.populate();

    // Specific for iSCSI & FibreChannel storage family 
    addViewConditional({
      'type': 'enum',
      'elementContext': document,
      'elementId': 'storageProtocol',
      'callName': 'storageProtocol-iSCSI-FibreChannel',
      'initialValue': object.get('storageProtocol'),
      'evalFunc': function(key) {
        return (key!=='iSCSI' && key !== 'FibreChannel');
      }
     });
    this.text('sizeMultiplier', object, {filter: '^[0-9]{1,10}(\.[0-9]{1,10})?$'});

    // Specific for NFS storage family 
    addViewConditional({
      'type': 'enum',
      'elementContext': document,
      'elementId': 'storageProtocol',
      'callName': 'storageProtocol-NFS',
      'initialValue': object.get('storageProtocol'),
      'evalFunc': function(key) {
        return (key!=='NFS');
      }
    });
    this.text('nfsSharesConfig', object, {elementNodeId: 'netapp-nfsSharesConfig'});
    this.text('copyOffloadToolPath',object);
    this.text('cacheClean', object, { filter: '^[0-9]{1,10}$'} );
    this.text('cacheCleanStop', object, {filter:'^[0-9]{1,10}$'});
    this.text('expiryMinutes', object, {filter: '^[0-9]{1,10}$'});

    // Specific for any 7-mode
    //this.kenum('storageFamily', object, [['Cluster', 'ontap cluster'], ['7mode', 'ontap 7-mode']]);
    addViewConditional({
      'type': 'enum',
      'elementContext': document,
      'elementId': 'storageFamily',
      'callName': 'storageFamily-7mode',
      'initialValue': object.get('storageFamily'),
      'evalFunc': function(key) {
        return (key!=='7mode');
      }
    });
    this.text('vfiler', object);
    showForm('netApp');
  } else if (object.get('childType') === 'OpenStackVolumeBackendDellStorageCenter') {
    // Dell
    var volumeDriverType = [
      [ 'FibreChannel',    'Fibre Channel Driver' ],
      [ 'iSCSI',           'iSCSI Driver' ],
    ];
    var type2driver = {
      'FibreChannel' : 'cinder.volume.drivers.dell.dell_storagecenter_fc.DellStorageCenterFCDriver',
      'iSCSI' : 'cinder.volume.drivers.dell.dell_storagecenter_iscsi.DellStorageCenterISCSIDriver'
    }
    this.kenum('driverType',  object, volumeDriverType, { elementNodeId: 'storeVmImages-dellStorageCenter-driverType',
                                                          oncommand : function(value) {
                                                            object.set('volumeDriver', type2driver[value]);
                                                          } 
                                                        });
    this.text('sanIP',        object,{ elementNodeId: 'storeVmImages-dellStorageCenter-sanIP', format : '^.+$' } );
    this.text('sanLogin',     object,{ elementNodeId: 'storeVmImages-dellStorageCenter-sanLogin', format : '^.+$' } );
    this.text('sanPassword',  object,{ elementNodeId: 'storeVmImages-dellStorageCenter-sanPassword', format : '^.+$' } );
    this.text('ssn',          object,{ elementNodeId: 'storeVmImages-dellStorageCenter-ssn' } );
    this.text('apiPort',      object,{ elementNodeId: 'storeVmImages-dellStorageCenter-apiPort' } );
    this.text('serverFolder', object,{ elementNodeId: 'storeVmImages-dellStorageCenter-serverFolder',format : '^.+$' } );
    this.text('volumeFolder', object,{ elementNodeId: 'storeVmImages-dellStorageCenter-volumeFolder', format : '^.+$' } );

    // only if driver type is iSCSI
    addViewConditional({
      'type': 'enum',
      'elementContext': document,
      'elementId': 'storeVmImages-dellStorageCenter-driverType',
      'callName': 'storeVmImages-dellStorageCenter-driverType-iscsi',
      'initialValue': object.get('driverType'),
      'evalFunc': function(key) {
        return (key!=='iSCSI');
      }
    });

    this.text('iscsiIp',      object,{ elementNodeId: 'storeVmImages-dellStorageCenter-iscsiIp',format : '^.+$' } );
    this.text('iscsiPort',    object,{ elementNodeId: 'storeVmImages-dellStorageCenter-iscsiPort',format : '[0-9]+$' } );
    showForm('dell');
  }

  function showForm(type) {
    var knownTypes = ['ceph', 'nfs', 'solidFire', '3par', 'gpfs', 'netApp', 'dell'];
    for(var i=0; i<knownTypes.length; i++) {
      if(type===knownTypes[i]) {
        document.getElementById(knownTypes[i] + 'Attributes').removeAttribute('hidden');
      } else {
        document.getElementById(knownTypes[i] + 'Attributes').setAttribute('hidden', 'true');
      }
    }
  }
};

var listing = new VolumeRoleBackendListingController(this);

/*
 * WRAPPERS
 */
function addObject(type) {
  listing.addObject(type);
}


/*
 * ***********************************************************************************************************************
 */

function EditorRuleVolumeBackend(key) {
  this.addTitle = "Add volume backend";
  this.editTitle = "Edit volume backend";
  this.api = '';
  this.key = parseInt(key);

  this.getCurrentObject = function(editorController) {
    var cluster = editorController.object.cluster;
    for(var i = 0; i < editorController.myDialog.arguments.subobject.object.volumeBackends.length; i++) {
      if(editorController.myDialog.arguments.subobject.object.volumeBackends[i].uniqueKey === this.key) {
        var dataobject = editorController.myDialog.arguments.subobject.object.volumeBackends[i];
      }
    }
    var role = editorController.myDialog.arguments.subobject;

    return new editorController.dock.cvObject(cluster, dataobject, role, 'volumeBackends');
  };

  this.volumeDriverCustom = function(editorController) {
    var volumeDriver = document.getElementById('volumeDriver').value;
    editorController.currentObject.set('volumeDriver', volumeDriver);
  };

  this.inputCheck = function(editorController) {
    if(editorController.currentObject.get('toBeRemoved')===true) {
      return true;
    }
    if(blank(editorController.currentObject.get('name'))) {
      editorController.dock.displayError("Name cannot be empty");
      return false;
    }
    if(blank(editorController.currentObject.get('volumeDriver'))) {
      editorController.dock.displayError("Volume driver cannot be empty");
      return false;
    }
    return true;
  };
}

//EditorRuleVolumeBackend.prototype = new EditorRuleBase();
//EditorRuleVolumeBackend.prototype.constructor = EditorRuleVolumeBackend;

/*

function EditorVolumeBackendController(chromeWindow, editorRule) {
  this.dock = null;
  this.object = null;
  this.myDialog = null;
  this.currentObject = null;
  this.originalObjectData = null;
  this.editingMode = null;
  this.editable = true;
  this.chromeWindow = chromeWindow;
  this.editorRule = editorRule;
  this.okPressed = false;

  this.onLoad = function () {
    println("Detached OpenStack tab initialized: " + this.chromeWindow.name + ' - ' + document.documentURI);
    this.initDock();
    this.initMyDialog();
    this.initObject();
    this.defineEdition();

    setDialogTitle(this.editorRule.getTitle(this));

    this.initCurrentObject();
    this.makeCopyOfCurrentObjectData();

    // TODO add better support for multi editing
    this.editorRule.setEditable(this);
    this.editorRule.setFormEdition(this);
    this.editorRule.setAttributes(this);
    this.refresh();
  };

  this.initDock = function() {
    this.dock = getDock();
    dock = this.dock;
  };

  this.initMyDialog = function() {
    var s = this.chromeWindow.name.indexOf('[');
    var e = this.chromeWindow.name.indexOf(']');
    var index = this.chromeWindow.name.substr(s + 1, e - s - 1);
    this.myDialog = this.dock.dialogs.list[index];
  };

  this.initObject = function() {
    this.object = this.myDialog.arguments.object;
    object = this.object;
  };

  this.initCurrentObject = function() {
    this.currentObject = this.editorRule.getCurrentObject(this);
    current = this.currentObject;
  };

  this.makeCopyOfCurrentObjectData = function() {
    if (this.isEdit()) {
      this.originalObjectData = this.currentObject.old(false, true);
    }
  };

  this.defineEdition = function() {
    if(isArray(this.myDialog.arguments.subobject)) {
      this.editingMode = 'MULTIPLE';
    }
    else if(this.myDialog.arguments.subobject <= 0) {
      this.editingMode = 'ADD';
    } else {
      this.editingMode = 'EDIT';
    }
  };

  this.isAdd = function() {
    if (this.editingMode===null) {
      this.defineEdition();
    }
    return (this.editingMode==='ADD');
  };

  this.isEdit = function() {
    if (this.editingMode===null) {
      this.defineEdition();
    }
    return (this.editingMode==='EDIT');
  };

  this.isEditable = function() {
    return this.editable;
  };

  this.isCommitted = function() {
    return (this.currentObject.get('uniqueKey') >= this.dock.MAX_LOCAL_KEY);
  };

  this.isMultiEdit = function() {
    if (this.editingMode===null) {
      this.defineEdition();
    }
    return (this.editingMode==='MULTIPLE');
  };

  this.refresh = function() {
    this.editorRule.dataBindingAndCheck(this);
  };

  this.ok = function() {
    if(!window.closed) {
      this.okPressed = true;
      if(!this.editorRule.inputCheck(this)) {
        return;
      }
      if(this.isAdd()) {
        this.object.cluster.addObject(this.currentObject, false, false);
      }
      this.myDialog.doOk();
      close();
    }
  };

  this.cancel = function() {
    if(!this.chromeWindow.closed) {
      if(this.isEdit()) {
        this.currentObject.old(this.originalObjectData);
      }
      close();
    }
  };

  this.onUnload = function() {
    if(!this.okPressed) {
      this.cancel();
    }
  };
}

EditorVolumeBackendController.prototype = new EditorController();
EditorVolumeBackendController.prototype.constructor = EditorVolumeBackendController;

*/

