/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var object;
var current;


function EditorController(chromeWindow, editorRule) {
  this.dock = null;
  this.object = null;
  this.myDialog = null;
  this.currentObject = null;
  this.originalObjectData = null;
  this.editingMode = null;
  this.editable = true;
  this.chromeWindow = chromeWindow;
  this.editorRule = editorRule;
  this.okPressed = false;

  this.onLoad = function () {
    println("Detached OpenStack tab initialized: " + this.chromeWindow.name + ' - ' + document.documentURI);
    this.initDock();
    this.initMyDialog();
    this.initObject();
    this.defineEdition();

    setDialogTitle(this.editorRule.getTitle(this));

    this.initCurrentObject();
    this.makeCopyOfCurrentObjectData();

    // TODO add better support for multi editing
    this.editorRule.setEditable(this);
    this.editorRule.setFormEdition(this);
    this.editorRule.setAttributes(this);
    this.refresh();
  };

  this.initDock = function() {
    this.dock = getDock();
    dock = this.dock;
  };

  this.initMyDialog = function() {
    var s = this.chromeWindow.name.indexOf('[');
    var e = this.chromeWindow.name.indexOf(']');
    var index = this.chromeWindow.name.substr(s + 1, e - s - 1);
    this.myDialog = this.dock.dialogs.list[index];
  };

  this.initObject = function() {
    if (this.editorRule && this.editorRule.onInitObject) {
      this.editorRule.onInitObject();
    }
    this.object = this.myDialog.arguments.object;
    object = this.object;
  };

  this.initCurrentObject = function() {
    this.currentObject = this.editorRule.getCurrentObject(this);
    current = this.currentObject;
  };

  this.makeCopyOfCurrentObjectData = function() {
    if (this.isEdit()) {
      this.originalObjectData = this.currentObject.old(false, true);
    }
  };

  this.defineEdition = function() {
    if(isArray(this.myDialog.arguments.subobject)) {
      this.editingMode = 'MULTIPLE';
    }
    else if(this.myDialog.arguments.subobject <= 0) {
      this.editingMode = 'ADD';
    } else {
      this.editingMode = 'EDIT';
    }
  };

  this.isAdd = function() {
    if (this.editingMode===null) {
      this.defineEdition();
    }
    return (this.editingMode==='ADD');
  };

  this.isEdit = function() {
    if (this.editingMode===null) {
      this.defineEdition();
    }
    return (this.editingMode==='EDIT');
  };

  this.isEditable = function() {
    return this.editable;
  };

  this.isCommitted = function() {
    return (this.currentObject.get('uniqueKey') >= this.dock.MAX_LOCAL_KEY);
  };

  this.isMultiEdit = function() {
    if (this.editingMode===null) {
      this.defineEdition();
    }
    return (this.editingMode==='MULTIPLE');
  };

  this.refresh = function() {
    this.editorRule.dataBindingAndCheck(this);
  };

  this.ok = function() {
    if(!window.closed) {
      this.okPressed = true;
      if(!this.editorRule.inputCheck(this)) {
        return;
      }
      if(this.isAdd()) {
        this.object.cluster.addObject(this.currentObject, false, false);
      }
      this.myDialog.doOk();
      close();
    }
  };

  this.cancel = function() {
    if(!this.chromeWindow.closed) {
      if(this.isEdit()) {
        this.currentObject.old(this.originalObjectData);
      }
      close();
    }
  };

  this.onUnload = function() {
    if(!this.okPressed) {
      this.cancel();
    }
  };
}

function onLoad() {
  editor.onLoad();
}

function refresh() {
  editor.refresh();
}
