/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function EditorRuleBase() {
  this.addTitle = "Add";
  this.editTitle = "Edit";
  this.editsTitle = "Edit";
  this.api = '';
  this.objectProperties = null;

  this.getTitle = function(editorController) {
    if(editorController.isAdd()) {
      return this.addTitle;
    } else if(editorController.isEdit()) {
      return this.editTitle;
    } else {
      return this.editsTitle;
    }
  };

  this.getApi = function() {
    return this.api;
  };

  this.setEditable = function(editorController) {
    editorController.editable = true;
  };

  // TODO add better support for multi editing
  this.setFormEdition = function(editorController) {
    if(!editorController.isEditable()) {
      var elementNodes = document.querySelectorAll('[data-editable="readonly"]');
      setAttribute(elementNodes, 'readonly', 'true');
      var elementNodes = document.querySelectorAll('[data-editable="disabled"]');
      setAttribute(elementNodes, 'disabled', 'true');
    }
    if(editorController.isEdit() && editorController.isCommitted()) {
      var elementNodes = document.querySelectorAll('[setonce="true"]');
      setAttribute(elementNodes, 'readonly', 'true');
    } else if(editorController.isMultiEdit()) {
      var elementNodes = document.querySelectorAll('[editing="single"]');
      setAttribute(elementNodes, 'hidden', 'true');

      var elementNodes = document.querySelectorAll('[editing="multiple-readonly"]');
      setAttribute(elementNodes, 'readonly', 'true');
    }
    function setAttribute(elementNodes, attribute, value) {
      for(var i = 0; i < elementNodes.length; i++) {
        elementNodes[i].setAttribute(attribute, value);
      }
    }
  };

  this.getObjectProperties = function(id=null) {
    if (this.objectProperties===null) {
      this.objectProperties = [];
      if (typeof getColumnsFor == 'function') {
        this.objectProperties = getColumnsFor(this.getApi());
      }
    }
    if (id!==null) {
      for(var i=0; i < this.objectProperties.length; i++) {
        if (this.objectProperties[i].id==id) {
          return this.objectProperties[i];
        }
      }
    } else {
      return this.objectProperties;
    }
  };

  this.setAttributes = function(editorController) {
    var objectProperties = this.getObjectProperties();
    for(var i=0; i < objectProperties.length; i++) {
      var id = this.getAttributeId(objectProperties[i].id, editorController);

      this.setAttributesForReadOnly(objectProperties[i], id);

      this.setAttributesForSetOnce(objectProperties[i], id, editorController);
    }
  };

  this.getAttributeId = function(id, editorController) {
    if(editorController.isMultiEdit()) {
      if (endswith(id, 'UUID') && editorController.currentObject[0].object.hasOwnProperty(id.substr(0, id.length - 4) + 'Key')) {
        id = id.substr(0, id.length - 4) + 'Key';
      }
    } else {
      if (endswith(id, 'UUID') && editorController.currentObject.object.hasOwnProperty(id.substr(0, id.length - 4) + 'Key')) {
        id = id.substr(0, id.length - 4) + 'Key';
      }
    }
    if (endswith(id, 'UUIDs')) {
      id = id.substr(0, id.length - 5) + 'Keys';
    }
    return id;
  };

  this.setAttributesForReadOnly = function(attributeProperties, id) {
    if (attributeProperties.hasOwnProperty('readonly') && attributeProperties.readonly==true) {
      var node = document.getElementById(id);
      if(node!==undefined && node!==null) {
        if(node.tagName==='textbox') {
          node.setAttribute('readonly', attributeProperties.readonly);
        } else {
          node.setAttribute('disabled', attributeProperties.readonly);
        }
      }
    }
  };

  this.setAttributesForSetOnce = function(attributeProperties, id, editorController) {
    if (attributeProperties.hasOwnProperty('setonce') && attributeProperties.setonce==true && editorController.isEdit() && editorController.isCommitted()) {
      var node = document.getElementById(id);
      if(node!==undefined && node!==null) {
        if(node.tagName==='textbox') {
          node.setAttribute('readonly', attributeProperties.setonce);
        } else {
          node.setAttribute('disabled', attributeProperties.setonce);
        }
      }
    }
  };

  this.getCurrentObjectData = function(editorController) {
    // Needs to be implemented by the extender
  };

  this.dataBindingAndCheck = function(editorController) {
    // Needs to be implemented by the extender
  };


  this.dataBindingAndCheckCommon = function(editorController) {
    var newKey = 0;
    if(editorController.isAdd()) {
      newKey = editorController.currentObject.get('uniqueKey');
    }

    if(document.getElementById('name')) {
      document.getElementById('name').focus();
    }
    text('name', '.+', '', true, '', newKey);

    if(editorController.isEdit()) {
      text('UUID', '^');
    } else {
      document.getElementById('UUID').setAttribute('value', '<auto>');
    }

    if(editorController.isAdd()) {
      document.getElementById('subFormContainerBrightControl').setAttribute('hidden', 'true');
    }
    check('addedFromBright');
    customDateText(editorController, 'addedTimestamp', 'addedTimestamp');
    customDateText(editorController, 'lastUpdateFromOpenStackTimestamp', 'lastUpdateFromOpenStackTimestamp');
    customDateText(editorController, 'lastUpdateFromBrightTimestamp', 'lastUpdateFromBrightTimestamp');

    function customDateText(editorController, attribute, elementNodeId) {
      var elementNode = document.getElementById(elementNodeId);
      var timestamp = editorController.currentObject.get(attribute);
      if(timestamp>0) {
        var d = new Date(timestamp * 1000);
        elementNode.setAttribute('value', d.format('dd/mmm/yyyy HH:MM'));
        elementNode.classList.remove('italic');
      } else {
        elementNode.setAttribute('value', 'never');
        elementNode.classList.add('italic');
      }
    }
  };

  this.resetEvents = function(elementNodeId) {
    var obj = document.getElementById(elementNodeId);
    var elClone = obj.cloneNode(true);
    obj.parentNode.replaceChild(elClone, obj);
  };
}