/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function EditorRuleFlavor() {
  this.addTitle = "Add Flavor";
  this.editTitle = "Edit Flavor";
  this.api = 'Flavor';

  this.getCurrentObject = function(editorController) {
    if(editorController.isAdd()) {
      var cvDataOpenStackApiFlavor = new editorController.dock.cvOpenStackApiFlavor();
      cvDataOpenStackApiFlavor.ownerUniqueKey = editorController.object.get('uniqueKey');
      return new editorController.dock.cvObject(editorController.object.cluster, cvDataOpenStackApiFlavor, 0, 0, false);
    } else {
      return editorController.object.cluster.find(editorController.myDialog.arguments.subobject);
    }
  };

  this.setEditable = function(editorController) {
    if(editorController.currentObject.get('deleted')===true) {
      editorController.editable = false;
    } else {
      editorController.editable = true;
    }
  };

  this.dataBindingAndCheck = function(editorController) {
    this.dataBindingAndCheckCommon(editorController);
    displayfunction('disk', 'formatter.format.GiB(value);');
    parsefunction('disk', 'formatter.parse.GiB(value);');
    text('disk', '^[0-9]+ ?GiB$');

    displayfunction('ephemeral', 'formatter.format.GiB(value);');
    parsefunction('ephemeral', 'formatter.parse.GiB(value);');
    text('ephemeral', '^[0-9]+ ?GiB$');

    displayfunction('swap', 'formatter.format.MiB(value);');
    parsefunction('swap', 'formatter.parse.MiB(value);');
    text('swap', '^[0-9]+ ?MiB$');

    text('vcpus', '^[0-9]+$');

    displayfunction('ram', 'formatter.format.MiB(value);');
    parsefunction('ram', 'formatter.parse.MiB(value);');
    text('ram', '^[0-9]+ ?MiB$');

    text('rxTxFactor');
    check('isPublic');
    check('enabled');

    var objectAttributeListable = new ObjectAttributeListable(editorController.currentObject, 'extraSpecs');
    var customListDataViewMapper = new CustomListDataViewMapper(objectAttributeListable);
    customListDataViewMapper.setElementsConfig([
      [
        {'type': 'menulist', 'attributes': [{'name': 'editable', 'value': 'true'}], 'events': [], 'menuitems': [
          // { 'value': 'quota:read_bytes_sec',        'label': 'quota:read_bytes_sec' },
          // { 'value': 'quota:write_bytes_sec',       'label': 'quota:write_bytes_sec' },
          { 'value': 'quota:cpu_shares',            'label': 'quota:cpu_shares' },
          { 'value': 'quota:cpu_period',            'label': 'quota:cpu_period' },
          // { 'value': 'quota:cpu_limit',             'label': 'quota:cpu_limit' },
          // { 'value': 'quota:cpu_reservation',       'label': 'quota:cpu_reservation' },
          { 'value': 'quota:cpu_quota',             'label': 'quota:cpu_quota' },

          { 'value': 'quota:disk_read_bytes_sec',   'label': 'quota:disk_read_bytes_sec' },
          { 'value': 'quota:disk_read_iops_sec',    'label': 'quota:disk_read_iops_sec' },
          { 'value': 'quota:disk_write_bytes_sec',  'label': 'quota:disk_write_bytes_sec' },
          { 'value': 'quota:disk_write_iops_sec',   'label': 'quota:disk_write_iops_sec' },
          { 'value': 'quota:disk_total_bytes_sec',  'label': 'quota:disk_total_bytes_sec' },
          { 'value': 'quota:disk_total_iops_sec',   'label': 'quota:disk_total_iops_sec' },

          { 'value': 'quota:vif_inbound_ average',  'label': 'quota:vif_inbound_ average' },
          { 'value': 'quota:vif_inbound_burst',     'label': 'quota:vif_inbound_burst' },
          { 'value': 'quota:vif_inbound_peak',      'label': 'quota:vif_inbound_peak' },
          { 'value': 'quota:vif_outbound_average',  'label': 'quota:vif_outbound_average' },
          { 'value': 'quota:vif_outbound_burst',    'label': 'quota:vif_outbound_burst' },
          { 'value': 'quota:vif_outbound_peak',     'label': 'quota:vif_outbound_peak' }
        ]},
        {'type': 'textbox', 'attributes': [], 'events': []}
      ]
    ]);
    customListDataViewMapper.setNewItem(['', '']);
    customListDataViewMapper.getHeader = function() {
      return [];
    };

    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'extraSpecs',
      'dataViewMapper': customListDataViewMapper,
      'addable': !editorController.currentObject.get('deleted'),
      'deletable': !editorController.currentObject.get('deleted')
    });
    addDelListWidget.populate();

    check('deleted');

/*
 *  Disabled until it gets implemented properly in cmdaemon
    var list = editorController.currentObject.cluster.find('OpenStackApiProject');

    var objectDualViewAttributeListable = new ObjectDualViewAttributeListable(list, editorController.currentObject, 'projectKeys');
    objectDualViewAttributeListable.setColumnOrder({
      'name': 0
    });
    objectDualViewAttributeListable.initInputValue();

    var customListDataViewMapper = new CustomDualListDataViewMapper(objectDualViewAttributeListable);
    customListDataViewMapper.setElementsConfig([
      [
        {'type': 'label', 'attributes': [{'name': 'width', 'value': '100px'}], 'events': []}
      ]
    ]);

    var groupSelectorWidget = new GroupSelectorWidget({
      'context': document.getElementById('projectKeys'),
      'rootNodeId': 'projectSelected',
      'rootNodeIdSelection': 'projectSelection',
      'rootNodeIdAdd': 'projectButtonAdd',
      'rootNodeIdDel': 'projectButtonDel',
      'dataViewMapper': customListDataViewMapper
    });
    groupSelectorWidget.populate();

    groupSelectorWidget.addItemCallBack = function() {
      current.correctAllUUID();
    };
    groupSelectorWidget.delItemCallBack = function() {
      current.correctAllUUID();
    };

    setAttributeRecursive(document.getElementById('projectKeys'), 'disabled', 'true');
*/
  };

  this.inputCheck = function(editorController) {
    if(blank(editorController.currentObject.get('name'))) {
      editorController.dock.displayError("Name cannot be empty");
      return false;
    }
    return true;
  };
}

EditorRuleFlavor.prototype = new EditorRuleBase();
EditorRuleFlavor.prototype.constructor = EditorRuleFlavor;
