/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function EditorRuleHostAggregate() {
  this.addTitle = "Add Host Aggregate";
  this.editTitle = "Edit Host Aggregate";
  this.api = 'HostAggregate';

  this.getCurrentObject = function(editorController) {
    if(editorController.isAdd()) {
      var cvDataOpenStackApiHostAggregate = new editorController.dock.cvOpenStackApiHostAggregate();
      cvDataOpenStackApiHostAggregate.ownerUniqueKey = editorController.object.get('uniqueKey');
      return new editorController.dock.cvObject(editorController.object.cluster, cvDataOpenStackApiHostAggregate, 0, 0, false);
    } else {
      return editorController.object.cluster.find(editorController.myDialog.arguments.subobject);
    }
  };

  this.dataBindingAndCheck = function(editorController) {
    this.dataBindingAndCheckCommon(editorController);

    //text('hosts', '^');
    document.getElementById('availabilityZone').addEventListener("change", this.syncMetadata.bind(this, editorController), true);
    document.getElementById('availabilityZone').addEventListener("keypress", this.syncMetadata.bind(this, editorController), true);
    text('availabilityZone', '^');

    var jsonAttributeListable = new JsonAttributeListable(editorController.currentObject, 'metadata');
    jsonAttributeListable.setInputFilter(function(value) {
      return (value!=='availability_zone');
    });
    jsonAttributeListable.setOutputFilter(function(o) {
      var availabilityZone = document.getElementById('availabilityZone').value;
      if(availabilityZone.length>0) {
        o['availability_zone'] = availabilityZone;
      }
      return o;
    });
    jsonAttributeListable.initInputValue();
    var textboxListDataViewMapper = new TextboxListDataViewMapper(jsonAttributeListable);
    textboxListDataViewMapper.setNewItem(['', '']);
    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'metadata',
      'dataViewMapper': textboxListDataViewMapper
    });
    addDelListWidget.populate();

    deviceselect(); //for nodes
    check('updateFromRoles');
  };

  this.inputCheck = function(editorController) {
    if(blank(editorController.currentObject.get('name'))) {
      editorController.dock.displayError("Name cannot be empty");
      return false;
    }
    return true;
  };
  
  this.syncMetadata = function(editorController) {
    var o = JSON.parse(editorController.currentObject.object.metadata);
    var availabilityZone = document.getElementById('availabilityZone').value;
    if(availabilityZone.length>0) {
      o['availability_zone'] = availabilityZone;
    } else {
      delete o['availability_zone'];
    }
    editorController.currentObject.set('metadata', JSON.stringify(o));
  };
}

EditorRuleHostAggregate.prototype = new EditorRuleBase();
EditorRuleHostAggregate.prototype.constructor = EditorRuleHostAggregate;