/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function NetworkInterface(args) {
  var prefix = args.prefix;
  var nodesListFn = args.nodesListFn;
  var connectNetworkFn = args.connectNetworkFn;
  var configAttribute = args.configAttribute;
  println('NetworkInterface');
  println(typeof(args.defaultType));
  var defaultType = args.defaultType;
  var interfaceConfigMethodFn = args.interfaceConfigMethodFn ? args.interfaceConfigMethodFn : null;
  var networkInterface = {};
  networkInterface = new WizardScreen();

  networkInterface.cache = {};

  networkInterface.hiddenNodesIds = [];
  networkInterface.hiddenNodesIdsHash = {};
  networkInterface.unknownInterface = {};

  networkInterface.initDefaultConfig = function() {
    throw Error('Must be implemented in child.');
  };

  networkInterface.populateCache = function() {
    networkInterface.cache.nodesList = nodesListFn();
  };

  networkInterface.selectedNodes = [];

  networkInterface.select = function() {
    buildSelection();
    clearSelection();
    setSelection();
    clearSelectorSetter();

    function buildSelection() {
      var categorySelected = getKeys(prefix + 'networkInterface-selector-categories');
      var rackSelected = getKeys(prefix + 'networkInterface-selector-racks');

      var hostnameSelected = document.getElementById(prefix + 'networkInterface-selector-hostname').value;
      var regExp = RegExp(hostnameSelected, 'i');

      networkInterface.selectedNodes = networkInterface.cache.nodesList.filter(function(node) {
        if (categorySelected.length !== 0 || rackSelected.length     !== 0 || hostnameSelected.length !== 0) {
          if (
            (categorySelected.length > 0 && categorySelected.indexOf(node.get('category')) !== -1 || categorySelected.length === 0) &&
            (rackSelected.length     > 0 && rackSelected.indexOf(node.get('rack'))         !== -1 || rackSelected.length     === 0) &&
            (hostnameSelected.length > 0 && node.get('hostname').search(regExp)            !== -1 || hostnameSelected.length === 0)
          ) {
            return true;
          }
        }
        return false;
      });

      function getKeys(elementNodeId) {
        var keys = [];
        var categorySelector = document.getElementById(elementNodeId);
        var index = 0;
        do {
          var selected = categorySelector.getSelectedItem(index);
          if (selected !== null) {
            keys.push(parseInt(selected.firstChild.getAttribute('key')));
          }
          index++;
        } while (selected !== null);
        return keys;
      }
    }

    function clearSelection() {
      var elements = document.getElementsByClassName('nodeInfo');
      for(var i = 0; i < elements.length; i++) {
        elements[i].classList.remove('border-selected');
      };
    }

    function setSelection() {
      networkInterface.selectedNodes.forEach(function(node) {
        document.getElementById(prefix + 'networkInterface-' + node.get('uniqueKey')).firstChild.classList.add('border-selected');
      });
      document.getElementById(prefix + 'networkInterface-selector-button').label = 'Select (' + networkInterface.selectedNodes.length + ')';
    }

    function clearSelectorSetter() {
      document.getElementById(prefix + 'networkInterface-config-selector').selectedIndex = -1;
      document.getElementById(prefix + 'networkInterface-network-selector').selectedIndex = -1;
      document.getElementById(prefix + 'networkInterface-physical-selector').selectedIndex = -1;
      var commonNetworkInterfaces = networkInterface.getAllCommonNetworkInterfacesForSelection();
      fillListDropdown({
        popupId: prefix + 'networkInterface-physical-selector',
        list: commonNetworkInterfaces
      });
      document.getElementById(prefix + 'networkInterface-vlanId-selector').value = '';
      networkInterface.showForConfigMethod('selector', 'reset');
    }
  };

  networkInterface.toggleSelector = function() {
    wizard.common.toggleHide(prefix + 'networkInterface-selector-container');
  };

  networkInterface.clearSelection = function(elementNodeId) {
    document.getElementById(elementNodeId).clearSelection();
  };

  networkInterface.clearText = function(elementNodeId) {
    document.getElementById(elementNodeId).value = '';
  };

  networkInterface.enter = function() {
    println('enter ' + prefix + 'networkInterface.enter');
    networkInterface.populateCache();

    if (!networkInterface.started) {
      networkInterface.started = true;

      initTemplateForm();
      initSelectorForm();
    } else {
      clearNetworkInterfacesPerNode();
    }

    initNetworkInterfacesPerNode();
    init();
    networkInterface.select();


    function initSelectorForm() {
      if (networkInterface.cache.nodesList.length > 10) {
        wizard.common.unhide(prefix + 'networkInterface-selector-container');
      }

      var categories = current.find('NodeCategory');
      fillListBox({
        elementNodeId: prefix + 'networkInterface-selector-categories',
        list: categories,
        clear: false
      });

      var racks = current.find('Rack');
      fillListBox({
        elementNodeId: prefix + 'networkInterface-selector-racks',
        list: racks,
        clear: false
      });

      var internalNetworks = current.findInternalNetworks();
      fillListDropdown({
        popupId: prefix + 'networkInterface-network-selector',
        list: internalNetworks,
        selectedElement: {label: wizard.config.modules.neutron.internal_net_name, defaultIndex: 0}
      });
    }

    function initTemplateForm() {
      var internalNetworks = current.findInternalNetworks();
      var cloudNetworks = current.findCloudNetworks();
      for(var i = 0; i < cloudNetworks.length; i++) {
	internalNetworks.push(cloudNetworks[i]);
      }
      fillListDropdown({
        popupId: prefix + 'networkInterface-network',
        list: internalNetworks,
        selectedElement: {label: wizard.config.modules.neutron.internal_net_name, defaultIndex: 0}
      });
    }

    function clearNetworkInterfacesPerNode() {
      document.getElementById(prefix + 'networkInterface-container').innerHTML = '';
      networkInterface.hiddenNodesIds = [];
      networkInterface.hiddenNodesIdsHash = {};
    }

    function _evalInterfaceConfigMethodFn(elem, func) {
      while (elem.firstChild) {
        elem.removeChild(elem.firstChild);
      }
      var menupopup = document.createElement('menupopup');
      elem.appendChild(menupopup);
      var items = func();
      for(var i = 0; i < items.length; i++) {
        var menuitem = document.createElement('menuitem');
        for(var k in items[i]) {
          menuitem.setAttribute(k, items[i][k]);
        }
        menupopup.appendChild(menuitem);
      }
    }

    function initNetworkInterfacesPerNode() {
      var template = document.getElementById(prefix + 'networkInterface-template');

      networkInterface.cache.nodesList.forEach(function(node, index) {
        var uniqueKey = node.get('uniqueKey');
        var cloned = template.cloneNode(true);
        cloned.id = prefix + 'networkInterface-' + uniqueKey;

        networkInterface.hiddenNodesIds.push(node);
        var _defaultType = (typeof(defaultType) == 'function' ? defaultType() : defaultType);
        networkInterface.hiddenNodesIdsHash[uniqueKey] = {name: node.get('hostname'), type: _defaultType, network: wizard.config.modules.neutron.internal_net_name};

        var nodeName = [
          node.get('hostname'),
          ' (',
          (node.isHypervisorNode) ? 'H' : '',
          (node.isNetworkNode) ? 'N' : '',
          ')'
        ].join('');

        cloned.getElementsByClassName(prefix + 'networkInterface-nodeName')[0].textContent = nodeName;
        cloned.getElementsByClassName(prefix + 'networkInterface-config-')[0].id = prefix + 'networkInterface-config-' + uniqueKey;
        cloned.getElementsByClassName(prefix + 'networkInterface-network-')[0].id = prefix + 'networkInterface-network-' + uniqueKey;
        cloned.getElementsByClassName(prefix + 'networkInterface-physical-')[0].id = prefix + 'networkInterface-physical-' + uniqueKey;
        cloned.getElementsByClassName(prefix + 'networkInterface-vlanId-')[0].id = prefix + 'networkInterface-vlanId-' + uniqueKey;
        if(interfaceConfigMethodFn) {
          var interface_config_elem = cloned.getElementsByClassName(prefix + 'networkInterface-config-')[0];
          _evalInterfaceConfigMethodFn(interface_config_elem, interfaceConfigMethodFn);
        }

        document.getElementById(prefix + 'networkInterface-container').appendChild(cloned);

        var interfacesConfigured = node.get('networks').reduce(function(accumulatedInterface, iface) {
          if (iface.childType === 'NetworkPhysicalInterface') {
            accumulatedInterface.push(iface.name);
          }
          return accumulatedInterface;
        }, []);

        var interfacesSysInfo = (typeof node.sysInfo.nics !== 'undefined') ? node.sysInfo.nics : [];

        fillListDropdown({
          popupId: prefix + 'networkInterface-physical-' + uniqueKey,
          list: [interfacesConfigured, interfacesSysInfo],
          clear: false
        });
      });

      window.addEventListener('resize', showMoreIfNeeded);
      document.getElementById(prefix + 'networkInterface-container-scroll').addEventListener('scroll', showMore, false);
      showMoreIfNeeded();

      function showMore() {
        networkInterface.hiddenNodesIds.splice(0, 10).forEach(function(node) {
          var initConfig = networkInterface.hiddenNodesIdsHash[node.get('uniqueKey')];
          delete networkInterface.hiddenNodesIdsHash[node.get('uniqueKey')];
          wizard.common.unhide(prefix + 'networkInterface-' + node.get('uniqueKey'));
          document.getElementById(prefix + 'networkInterface-' + node.get('uniqueKey')).innerHTML = document.getElementById(prefix + 'networkInterface-' + node.get('uniqueKey')).innerHTML;
          initNode.call(initConfig, node);
        });
      }
      function showMoreIfNeeded() {
        var containerNode = document.getElementById(prefix + 'networkInterface-container-scroll');
        while (networkInterface.hiddenNodesIds.length > 0 && containerNode.scrollHeight === containerNode.clientHeight) {
          showMore();
        };
      }
    }

    function init() {
      fromSystemConfig();
      fromWizardConfig();

      function fromSystemConfig() {
        var connectNetwork = connectNetworkFn();
        if (connectNetwork !== null) {
          networkInterface.cache.nodesList.forEach(function(node) {
            var networks = node.get('networks');
            networks.some(function(network) {
              if (network.name == 'br0:vxlan') {
                return false;
              }
              else if (network.network === connectNetwork.get('uniqueKey')) {
                var config = {type: 'NEW_DEDICATED', physical: network.name, vlanId: ''};
                if (network.childType === 'NetworkVLANInterface') {
                  var vlanIdMatch = network.name.match(/^([A-Za-z0-9]+)\.([0-9]+)$/);
                  config.physical = (vlanIdMatch) ? vlanIdMatch[1] : config.physical;
                  config.vlanId = (vlanIdMatch) ? vlanIdMatch[2] : config.vlanId;
                }
                initNode.call(config, node);
                return true;
              }
              return false;
            });
          });
        }
      }

      function fromWizardConfig() {
        if (typeof wizard.config.modules.neutron[configAttribute] !== 'undefined' && Array.isArray(wizard.config.modules.neutron[configAttribute])) {
          wizard.config.modules.neutron[configAttribute].forEach(function(element) {
            var physicalNodes = current.findBy({
              baseType: 'Device',
              filter: function(node) {
                if(node.get('hostname') === element.name) {
                  return true;
                }
                return false;
              }
            });
            physicalNodes.forEach(initNode, element);
          });
        }
      }
    }

    function initNode(node) {
      var key = node.get('uniqueKey');

      if (typeof this.type !== 'undefined') {
        networkInterface.setConfigMethod(key, this.type);
        networkInterface.showForConfigMethod(key, this.type);
      }
      if (typeof this.network !== 'undefined') {
        networkInterface.setNetwork(key, this.network);
      }
      if (typeof this.physical !== 'undefined') {
        networkInterface.setPhysical(key, this.physical);
      }
      if (typeof this.vlanId !== 'undefined') {
        networkInterface.setVlanId(key, this.vlanId);
      }
    }
  };

  networkInterface.forward = function(automatic) {
    function isValidPhysicalInterfaceName(physicalInterfaceName, errorMsgElementId, errorNodeElementId) {
      if (!physicalInterfaceName.match(/^([A-Za-z0-9]+)$/)) {
        wizard.common.displayError(errorMsgElementId, automatic);
        document.getElementById(errorNodeElementId).classList.add('border-error');
        networkInterface.error = true;
        return false;
      }
      if (!networkInterface.error) {
        wizard.common.hide(errorMsgElementId);
      }
      document.getElementById(errorNodeElementId).classList.remove('border-error');
      return true;
    }

    function isValidVlanId(vlanId, errorMsgElementId, errorNodeElementId) {
      if (!vlanId.match(/^([0-9]+)?$/)) {
        wizard.common.displayError(errorMsgElementId, automatic);
        document.getElementById(errorNodeElementId).classList.add('border-error');
        networkInterface.error = true;
        return false;
      }
      if (!networkInterface.error) {
        wizard.common.hide(errorMsgElementId);
      }
      document.getElementById(errorNodeElementId).classList.remove('border-error');
      return true;
    }

    println('forward ' + prefix + 'networkInterface.forward');

    var conf = networkInterface.cache.nodesList.map(function(node) {
      var config = {};
      var key = node.get('uniqueKey');
      if (typeof networkInterface.hiddenNodesIdsHash[key] === 'undefined') {
        config.name = node.get('hostname');
        config.type = document.getElementById(prefix + 'networkInterface-config-' + key).value;
        if (config.type==='NEW_SHARED') {
          config.network = document.getElementById(prefix + 'networkInterface-network-' + key).label;
        } else if (config.type==='NEW_DEDICATED') {
          config.physical = document.getElementById(prefix + 'networkInterface-physical-' + key).value;
          isValidPhysicalInterfaceName(config.physical, prefix + 'networkInterface-error-physical-invalid', prefix + 'networkInterface-physical-' + key);

          config.vlanId = document.getElementById(prefix + 'networkInterface-vlanId-' + key).value;
          isValidVlanId(config.vlanId, prefix + 'networkInterface-error-vlanId-invalid', prefix + 'networkInterface-vlanId-' + key);
        } else if (config.type==='NOTHING') {

        }
      } else {
        config = networkInterface.hiddenNodesIdsHash[key];
      }
      return config;
    });

    if (!networkInterface.error) {
      wizard.common.setConfig(wizard.config.modules.neutron, configAttribute, conf);
    }
    println(configAttribute + ':');
    printallR(wizard.config.modules.neutron[configAttribute]);

    return networkInterface.error;
  };

  networkInterface.resetALLPhysicalErrors = function() {
    wizard.common.hide(prefix + 'networkInterface-error-interfaceName-invalid');
  };

  networkInterface.getAllCommonNetworkInterfacesForSelection = function() {
    var interfacesConfigured = {};
    var availableNodesConfigured = {};

    var interfacesSysInfo = {};
    var availableNodesSysInfo = {};
    networkInterface.selectedNodes.forEach(function(node) {
      node.get('networks').forEach(function(iface) {
        if (iface.childType === 'NetworkPhysicalInterface') {
          if (interfacesConfigured[iface.name] === undefined) {
            interfacesConfigured[iface.name] = [];
          }
          if (interfacesConfigured[iface.name].indexOf(node.get('hostname')) === -1) {
            interfacesConfigured[iface.name].push(node.get('hostname'));
            availableNodesConfigured[node.get('hostname')] = true;
          }
        }
      });

      var nics = node.sysInfo.nics;
      if (typeof nics !== 'undefined') {
        nics.forEach(function(nic) {
          if (interfacesSysInfo[nic] === undefined) {
            interfacesSysInfo[nic] = [];
          }
          if (interfacesSysInfo[nic].indexOf(node.get('hostname')) === -1) {
            interfacesSysInfo[nic].push(node.get('hostname'));
            availableNodesSysInfo[node.get('hostname')] = true;
          }
        });
      }
    });

    availableNodesConfigured = Object.keys(availableNodesConfigured);
    var commonInterfaceConfigured = Object.keys(interfacesConfigured).filter(function(interfaceName) {
      return (availableNodesConfigured.diff(interfacesConfigured[interfaceName]).length === 0);
    });

    availableNodesSysInfo = Object.keys(availableNodesSysInfo);
    var commonInterfaceSysInfo = Object.keys(interfacesSysInfo).filter(function(interfaceName) {
      return (availableNodesSysInfo.diff(interfacesSysInfo[interfaceName]).length === 0);
    });

    return [commonInterfaceConfigured, commonInterfaceSysInfo];
  };

  networkInterface.setConfigMethodSelector = function() {
    var value = document.getElementById(prefix + 'networkInterface-config-selector').value;
    networkInterface.showForConfigMethod('selector', value);

    networkInterface.selectedNodes.forEach(function(node) {
      var key = node.get('uniqueKey');
      networkInterface.setConfigMethod(key, value);
      networkInterface.showForConfigMethod(key, value);
    });
  };

  networkInterface.setNetworkSelector = function() {
    var value = document.getElementById(prefix + 'networkInterface-network-selector').value;

    networkInterface.selectedNodes.forEach(function(node) {
      var key = node.get('uniqueKey');
      networkInterface.setNetwork(key, value);
    });
  };

  networkInterface.setInterfaceSelector = function() {
    var value = document.getElementById(prefix + 'networkInterface-physical-selector').value;

    networkInterface.selectedNodes.forEach(function(node) {
      var key = node.get('uniqueKey');
      networkInterface.setPhysical(key, value);
    });
  };

  networkInterface.setVlanIdSelector = function() {
    var value = document.getElementById(prefix + 'networkInterface-vlanId-selector').value;

    networkInterface.selectedNodes.forEach(function(node) {
      var key = node.get('uniqueKey');
      networkInterface.setVlanId(key, value);
    });
  };

  networkInterface.configMethodChanged = function(elementNode) {
    var key = elementNode.id.match(/^.*\-config\-([0-9]+)/)[1];
    networkInterface.showForConfigMethod(key, elementNode.value);
  };

  networkInterface.setConfigMethod = function(key, option) {
    if (typeof networkInterface.hiddenNodesIdsHash[key] === 'undefined') {
      var elementNode = document.getElementById(prefix + 'networkInterface-config-' + key);
      elementNode.value = option;
    } else {
      networkInterface.hiddenNodesIdsHash[key].type = option;
    }
  };

  networkInterface.showForConfigMethod = function(key, label) {
    switch (label) {
      case 'reset':
      case 'NEW_SHARED':
        wizard.common.unhide(prefix + 'networkInterface-network-' + key);
        wizard.common.hide(prefix + 'networkInterface-physical-' + key);
        wizard.common.hide(prefix + 'networkInterface-vlanId-' + key);
        break;
      case 'NEW_DEDICATED':
        wizard.common.hide(prefix + 'networkInterface-network-' + key);
        wizard.common.unhide(prefix + 'networkInterface-physical-' + key);
        wizard.common.unhide(prefix + 'networkInterface-vlanId-' + key);
        break;
      case 'NOTHING':
      default:
        wizard.common.hide(prefix + 'networkInterface-network-' + key);
        wizard.common.hide(prefix + 'networkInterface-physical-' + key);
        wizard.common.hide(prefix + 'networkInterface-vlanId-' + key);
        break;
    }
  };

  networkInterface.setNetwork = function(key, option) {
    if (typeof networkInterface.hiddenNodesIdsHash[key] === 'undefined') {
      var elementNode = document.getElementById(prefix + 'networkInterface-network-' + key);
      elementNode.value = option;
    } else {
      networkInterface.hiddenNodesIdsHash[key].network = option;
    }
  };

  networkInterface.setPhysical = function(key, value) {
    if (typeof networkInterface.hiddenNodesIdsHash[key] === 'undefined') {
      var elementNode = document.getElementById(prefix + 'networkInterface-physical-' + key);
      if (typeof value !== 'undefined') {
        elementNode.value = value;
        networkInterface.checkPhysical(elementNode);
      }
    } else {
      networkInterface.hiddenNodesIdsHash[key].physical = value;
    }
  };

  networkInterface.checkPhysical = function(elementNode) {
    var key = elementNode.id.match(/^.*\-physical\-([0-9]+)/)[1];
    if (elementNode.selectedIndex > -1) {
      elementNode.classList.remove('border-warning');
      networkInterface.removeUnknownInterface(key);
    } else {
      elementNode.classList.add('border-warning');
      networkInterface.addUnknownInterface(key);
    }
  };

  networkInterface.addUnknownInterface = function(key) {
    networkInterface.unknownInterface[key] = key;
    networkInterface.processUnknownInterfaceWarningMsg();
  };

  networkInterface.removeUnknownInterface = function(key) {
    delete networkInterface.unknownInterface[key];
    networkInterface.processUnknownInterfaceWarningMsg();
  };

  networkInterface.processUnknownInterfaceWarningMsg = function() {
    if (Object.keys(networkInterface.unknownInterface).length > 0) {
      wizard.common.unhide(prefix + 'networkInterface-warning-physical-unknown');
    } else {
      wizard.common.hide(prefix + 'networkInterface-warning-physical-unknown');
    }
  };

  networkInterface.setVlanId = function(key, value) {
    if (typeof networkInterface.hiddenNodesIdsHash[key] === 'undefined') {
      var elementNode = document.getElementById(prefix + 'networkInterface-vlanId-' + key);
      if (typeof value !== 'undefined') {
        elementNode.value = value;
      }
    } else {
      networkInterface.hiddenNodesIdsHash[key].vlanId = value;
    }
  };

  return networkInterface;
}
