/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var HN = 'H';
var NW = 'N';
var CN = 'C';

wizard.nodesSelection = new WizardScreen();


wizard.nodesSelection.memory_limit_controller_node       = 4000000000;  //4*1024*1024*1024 = 4294967296
wizard.nodesSelection.memory_limit_controller_node_human = "4 GB";
wizard.nodesSelection.num_controller_nodes = 0;

wizard.nodesSelection.softwareImageOfNodeSelected = function(node) {
  var ret = { selected : false,
              name : "" };
  var lu = {};
  for(var i = 0; i < wizard.config.modules.core.software_image_paths.length; i++) {
    lu[wizard.config.modules.core.software_image_paths[i]] = true;
  }
  var cluster = node.cluster;
  if(!cluster) cluster = current.cluster;
  if(!cluster) cluster = current;

  var cat = cluster.find(node.object.category);
  var softwareImageProxy = cat.object.softwareImageProxy;
  if(softwareImageProxy) {
    var softwareImage = cluster.find(softwareImageProxy.parentSoftwareImage);
    if(softwareImage) {
      var path = softwareImage.get('path');
      ret.name = softwareImage.get('name');
      if(lu.hasOwnProperty(path)) {
        ret.selected = true;
      }
    }
  }
  return ret;
}

wizard.nodesSelection.isSuboptimalNode = function(node, is_NW, is_HN, is_CN, is_OSD) {
  if(is_CN) {
    if(!node.sysInfo ||
       !node.sysInfo.processors ||
       node.sysInfo.processors.length < 2 ||
       !node.sysInfo.memoryTotal ||
       node.sysInfo.memoryTotal < this.memory_limit_controller_node) {
      return true;
    }
    if(wizard.nodesSelection.nodesDiskless.hasOwnProperty(node.object.hostname) &&
       wizard.nodesSelection.nodesDiskless[node.object.hostname]) {
      return true;
    }
  }
  if( ( is_NW || is_CN || is_HN ) && is_OSD ) {
    return true;
  }
  if(is_HN && is_CN) {
    return true;
  }
  if(node.status.status != 'UP') {
    return true;
  }
  var si = wizard.nodesSelection.softwareImageOfNodeSelected(node);
  if(!si.selected) {
    return true;
  }
  return false;
};

wizard.nodesSelection.getSuboptimalHypervisorMessages = function(nodes) {
  var n_combined = [];
  var n_not_up   = [];
  var n_combined_osd = [];
  var n_image = {};
  var n_image_nodes = [];
  var n_no_sys_info = [];
  Object.keys(nodes).forEach(function(nn) {
    var n = nodes[nn];
    if(n.HN && n.CN) {
      n_combined.push(nn);
    }
    if(n.HN && n.OSD) {
      n_combined_osd.push(nn);
    }
    if(n.HN && n.node.status.status != 'UP') {
      n_not_up.push(nn);
    }
    if(n.HN && !n.node.sysInfo) {
      n_no_sys_info.push(nn);
    }
    var si = wizard.nodesSelection.softwareImageOfNodeSelected(n.node);
    if(!si.selected) {
      n_image[si.name] = true;
      n_image_nodes.push(nn);
    }
  });
  var msg = [];
  if(n_combined_osd.length == 1) {
    msg.push({ msg: "One hypervisor node combined with Ceph OSD.",
               nodes: n_combined_osd});
  }
  else if(n_combined_osd.length > 1) {
    msg.push({ msg: n_combined_osd.length + " hypervisor node combined with Ceph OSD.",
               nodes: n_combined_osd});
  }
  if(n_combined.length == 1) {
    msg.push({ msg: "One hypervisor node combined with Controller node",
               nodes: n_combined});
  }
  else if(n_combined.length > 1) {
    msg.push({ msg: n_combined.length + " hypervisor nodes combined with Controller nodes.",
               nodes: n_combined});
  }
  if(n_not_up.length == 1) {
    msg.push({ msg: "One hypervisor node not up.",
               nodes: n_not_up});
  }
  else if(n_not_up.length > 1) {
    msg.push({ msg: n_not_up.length + " hypervisor nodes not up.",
               nodes : n_not_up});
  }
  if(n_no_sys_info.length == 1) {
    msg.push({ msg : "One hypervisor node without sysinfo ",
               nodes : n_no_sys_info});
  }
  else if(n_no_sys_info.length > 1) {
    msg.push({ msg : n_no_sys_info.length + " hypervisor nodes without sysinfo ",
               nodes : n_no_sys_info});
  }
  var si_keys = Object.keys(n_image);
  if(si_keys.length == 1) {
      msg.push({ msg: "Software image '" + si_keys[0] + "' not selected.",
                 nodes: n_image_nodes });
  }
  else if(si_keys.length > 1) {
      msg.push({ msg: "Software images '" + si_keys.join("', '") + "' not selected.",
                 nodes: n_image_nodes });

  }
  return msg;
}

wizard.nodesSelection.getSuboptimalControllerMessages = function(nodes) {
  var n_not_up = [];
  var n_processors = [];
  var n_memory = [];
  var n_combined_osd = [];
  var n_image = {};
  var n_image_nodes = [];
  var n_diskless = [];
  var n_no_sys_info = [];
  var num = 0;
  Object.keys(nodes).forEach(function(nn) {
    var n = nodes[nn];
    if(n.CN) {
      num++;
      if(n.node.status.status != 'UP') {
        n_not_up.push(nn);
      }
      if(n.OSD) {
        n_combined_osd.push(nn);
      }
      else {
        if(!n.node.sysInfo) {
          n_no_sys_info.push(nn);
        } else {
          if(!n.node.sysInfo.processors ||
             n.node.sysInfo.processors.length < 2) {
            n_processors.push(nn);
          }
          if(n.node.sysInfo.memoryTotal < wizard.nodesSelection.memory_limit_controller_node) {
            n_memory.push(nn);
          }
        }
      }
      if(wizard.nodesSelection.nodesDiskless.hasOwnProperty(nn) &&
	       wizard.nodesSelection.nodesDiskless[nn]) {
	        n_diskless.push(nn);
      }
    }
    var si = wizard.nodesSelection.softwareImageOfNodeSelected(n.node);
    if(!si.selected) {
      n_image[si.name] = true;
      n_image_nodes.push(nn);
    }
  });
  var msg = [];
  if(n_processors.length == 1) {
    msg.push({ msg : "One controller node with less than 2 cores ",
               nodes : n_processors});
  }
  else if(n_processors.length > 1) {
    msg.push({ msg : n_processors.length + " controller nodes with less than 2 cores ",
               nodes : n_processors});
  }
  if(n_combined_osd.length == 1) {
    msg.push({ msg : "One controller node combined with Ceph OSD.",
               nodes : n_combined_osd});
  }
  else if(n_combined_osd.length > 1) {
    msg.push({ msg : n_combined_osd.length + " controller nodes combined with Ceph OSD.",
               nodes : n_combined_osd});
  }
  if(n_diskless.length == 1) {
    msg.push({ msg : "One controller node has diskless setup.",
	       nodes : n_diskless });
  }
  else if(n_diskless.length > 1) {
    msg.push({ msg : n_diskless.length + " controller nodes have diskless setup.",
	       nodes : n_diskless });
  }

  if(n_memory.length == 1) {
    msg.push({ msg : "One controller node with less than " + wizard.nodesSelection.memory_limit_controller_node_human + " RAM",
               nodes : n_memory});
  }
  else if(n_memory.length > 1) {
    msg.push({ msg : n_memory.length + " controller nodes with less than " + wizard.nodesSelection.memory_limit_controller_node_human + " RAM",
               nodes: n_memory});
  }
  if(n_not_up.length == 1) {
    msg.push({ msg : "One controller node not up.",
               nodes: n_not_up});
  }
  else if(n_not_up.length > 1) {
    msg.push({ msg : n_not_up.length + " controller nodes not up.",
               nodes: n_not_up});
  }
  if(n_no_sys_info.length == 1) {
    msg.push({ msg : "One controller node without sysinfo ",
               nodes : n_no_sys_info});
  }
  else if(n_no_sys_info.length > 1) {
    msg.push({ msg : n_no_sys_info.length + " controller nodes without sysinfo ",
               nodes : n_no_sys_info});
  }
  var si_keys = Object.keys(n_image);
  if(si_keys.length == 1) {
      msg.push({ msg: "Software image '" + si_keys[0] + "' not selected.",
                 nodes: n_image_nodes });
  }
  else if(si_keys.length > 1) {
      msg.push({ msg: "Software images '" + si_keys.join("', '") + "' not selected.",
                 nodes: n_image_nodes });

  }
  if(num == 2) {
    msg.push({ msg: "2 Controller nodes not allowed."});
  } else if(num < 3) {
    msg.push({ msg: "Less than 3 controller nodes."});
  }
  return msg;
}

wizard.nodesSelection.getSuboptimalNetworkMessages = function(nodes) {
  var n_not_up = [];
  var n_combined_osd = [];
  var n_image = {};
  var n_image_nodes = [];
  var n_no_sys_info = [];
  Object.keys(nodes).forEach(function(nn) {
    var n = nodes[nn];
    if(n.NW) {
      if(n.node.status.status != 'UP') {
        n_not_up.push(nn);
      }
      if(n.OSD) {
        n_combined_osd.push(nn);
      }
      if(!n.node.sysInfo) {
        n_no_sys_info.push(nn);
      }
    }
    var si = wizard.nodesSelection.softwareImageOfNodeSelected(n.node);
    if(!si.selected) {
      n_image[si.name] = true;
      n_image_nodes.push(nn);
    }
  });
  var msg = [];
  if(n_combined_osd.length == 1) {
    msg.push({ msg: "One network node combined with Ceph OSD.",
               nodes: n_combined_osd});
  }
  else if(n_combined_osd.length > 1) {
    msg.push({ msg: n_combined.length + " network node combined with Ceph OSD.",
               nodes: n_combined});
  }
  if(n_not_up.length == 1) {
    msg.push({ msg: "One network node not up.",
               nodes : n_not_up});
  }
  else if(n_not_up.length > 1) {
    msg.push({msg:n_not_up.length + " network nodes not up.",
              nodes : n_not_up });
  }
  if(n_no_sys_info.length == 1) {
    msg.push({ msg : "One network node without sysinfo ",
               nodes : n_no_sys_info});
  }
  else if(n_no_sys_info.length > 1) {
    msg.push({ msg : n_no_sys_info.length + " network nodes without sysinfo ",
               nodes : n_no_sys_info});
  }

  var si_keys = Object.keys(n_image);
  if(si_keys.length == 1) {
      msg.push({ msg: "Software image '" + si_keys[0] + "' not selected.",
                 nodes: n_image_nodes });
  }
  else if(si_keys.length > 1) {
      msg.push({ msg: "Software images '" + si_keys.join("', '") + "' not selected.",
                 nodes: n_image_nodes });

  }
  return msg;
}

wizard.nodesSelection.suboptimalNodeMessages = function(node, is_NW, is_HN, is_CN, is_OSD) {
  var msg = [];
  if(is_CN) {
    if(!node.sysInfo ||
       !node.sysInfo.processors ||
       node.sysInfo.processors.length < 2 ||
       node.sysInfo.memoryTotal < this.memory_limit_controller_node) {
      var this_msg = "";
      if(!node.sysInfo) {
        this_msg = "Controller node without sysinfo";
        msg.push({msg: this_msg, error : false});
      } else {
        if(node.sysInfo.processors.length < 2) {
          this_msg =  "Controller node with less than 2 cores ";
        }
        if(node.sysInfo.memoryTotal < this.memory_limit_controller_node) {
          if(this_msg === "") {
            this_msg =  "Controller node with ";
          } else {
            this_msg+= "and ";
          }
          this_msg+= "less than " + this.memory_limit_controller_node_human + " RAM";
        }
        msg.push({msg: this_msg, error : false});
      }
    }
    if(wizard.nodesSelection.nodesDiskless.hasOwnProperty(node.object.hostname) &&
       wizard.nodesSelection.nodesDiskless[node.object.hostname]) {
      msg.push({msg: "Diskless node as controller", error : false});
    }
  }
  if(is_NW && !node.sysInfo) {
    msg.push({msg: "Network node without sysinfo", error: false});
  } else if(is_HN && !node.sysInfo) {
    msg.push({msg: "Hypervisor node without sysinfo", error: false});
  }
  if( ( is_NW || is_CN || is_HN ) && is_OSD ) {
    msg.push({msg: "OpenStack combined with Ceph OSD.", error : false});
  }
  if( is_HN && is_CN) {
    msg.push({msg: "Hypervisor node combined with Controller node.", error:false});
  }
  if(node.status.status != 'UP') {
    msg.push({ msg: "Selected node is not up.", error : true});
  }
  var si = wizard.nodesSelection.softwareImageOfNodeSelected(node);
  if(!si.selected) {
      msg.push({msg : "Software image '" + si.name  + "' not selected", error : false});
  }
  return msg;
};


wizard.nodesSelection.nodesWithCephOSDRoles = {};
wizard.nodesSelection.nodesDiskless = {};

wizard.nodesSelection.initDiskSetup = function(cluster) {
  var categories = {};
  cluster.find('NodeCategory').filter(function(obj) {
    categories[obj.object.uniqueKey] = obj;
  });
  cluster.find(['SlaveNode', 'CloudNode']).filter(function(obj) {
    var disksetup = '';
    if(obj.object.disksetup != "") {
      // get disksetup from node
      disksetup = obj.object.disksetup;
    }
    else {
      // get disksetup from category
      if(categories.hasOwnProperty(obj.object.category)) {
	        disksetup = categories[obj.object.category].object.disksetup;
      }
    }
    try {
      wizard.nodesSelection.nodesDiskless[obj.object.hostname] =
	          isPathDiskless(disksetup, "/var/lib");
    }
    catch(err) {
      wizard.nodesSelection.nodesDiskless[obj.object.hostname] = false;
    }
  });
};

wizard.nodesSelection.findAllNodesWithCephOSDRoles = function(cluster) {
  var role_name = 'CephOSDRole';
  var node_keys = {};
  var category_keys = {};
  var ret = {};
  cluster.find('ConfigurationOverlay').forEach(function(overlay){
    if(overlay.get('priority') != -1) {
      for(var i = 0; i < overlay.object.roles.length; i++) {
        if(overlay.object.roles[i].childType == role_name) {
          for(var j = 0; j < overlay.object.nodes.length; j++) {
            node_keys[overlay.object.nodes[j]] = true;
          }
          for(var j = 0; j < overlay.object.categories.length; j++) {
            category_keys[overlay.object.categories[j]] = true;
          }
          break;
        }
      }
    }
  });
  cluster.find('NodeCategory').forEach(function(category) {
    for(var i = 0; i < category.object.roles.length; i++) {
      if(category.object.roles[i].childType == role_name) {
        category_keys[category.object.uniqueKey]  = true;
      }
    }
  });
  cluster.find('Node').forEach(function(node) {
    if(node.object.hasOwnProperty('category')) {
      if(category_keys.hasOwnProperty(node.object.category)) {
        node_keys[node.object.uniqueKey] = true;
      }
    }
    if(!node_keys.hasOwnProperty(node.object.uniqueKey)) {
      for(var i = 0; i < node.object.roles.length; i++) {
        if(node.object.roles[i].childType == role_name) {
          node_keys[node.object.uniqueKey] = true;
          break;
        }
      }
    }
    if(node_keys.hasOwnProperty(node.object.uniqueKey)) {
      ret[node.object.uniqueKey] = node;
    }
  });
  return ret;
};

wizard.nodesSelection.filterNodeChildType = function(node) {
    return node.object.childType != "VirtualNode";
}

wizard.nodesSelection.initDefaultConfig = function() {
  // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  // current.licenseInfo.licensedOpenStackNodes = 0;
  // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  println('init wizard.nodesSelection.initDefaultConfig');
  wizard.nodesSelection.nodesWithCephOSDRoles = wizard.nodesSelection.findAllNodesWithCephOSDRoles(current);
  wizard.nodesSelection.initDiskSetup(current);
  var nodes_with_ceph_roles = [];
  Object.keys(wizard.nodesSelection.nodesWithCephOSDRoles).forEach(function(k) {
    nodes_with_ceph_roles.push(wizard.nodesSelection.nodesWithCephOSDRoles[k].get('hostname'));
  });

  function resolveHostnames(arr) {
    var ret = [];
    for(var i = 0; i < arr.length; i++) {
      ret.push(current.resolveName(arr[i]));
    }
    return ret;
  }

  if(wizard.config.modules.core.hasOwnProperty('_pre_node_selection')) {
    wizard.common.setConfig(wizard.common.getConfigOverlay('nova', 'OpenStackHypervisors'),
                            'nodes',
                            resolveHostnames(wizard.config.modules.core._pre_node_selection.HN));
    wizard.common.setConfig(wizard.common.getConfigOverlay('neutron', 'OpenStackNetworkNodes'),
                            'nodes',
                            resolveHostnames(wizard.config.modules.core._pre_node_selection.NW));
    wizard.common.setConfig(wizard.common.getConfigOverlay('galera', 'GaleraNodes'),
                            'nodes',
                            resolveHostnames(wizard.config.modules.core._pre_node_selection.CN));
    var controller_modules = [
      'cinder',
      'core',
      'glance',
      'horizon',
      'keystone',
      'neutron',
      'nova',
      'rabbitmq',
      'heat',
      'radosgw' ];
    for(var i = 0; i < controller_modules.length; i++) {
      wizard.common.setConfig(wizard.common.getConfigOverlay(controller_modules[i], 'OpenStackControllers'),
                              'nodes',
                              resolveHostnames(wizard.config.modules.core._pre_node_selection.CN));
    }
  }
};

wizard.nodesSelection.hash = (function() {
  var obj = {};
  obj[HN] = {
    categories: {},
    category: {},
    physicalNodes: {},
    selectedNodes: {}
  };
  obj[NW] = {
    categories: {},
    category: {},
    physicalNodes: {},
    selectedNodes: {}
  };
  obj[CN] = {
    categories: {},
    category: {},
    physicalNodes: {},
    selectedNodes: {}
  };
  return obj;
})();


wizard.nodesSelection.enter = function() {
  function init(list, group) {
    list.forEach(function(hostname) {
     var physicalNodes = current.findBy({
        baseType: ['SlaveNode', 'CloudNode'],
        filter: function(node) {
          if(wizard.nodesSelection.filterNodeChildType(node) && node.get('hostname') === hostname) {
            return true;
          }
          return false;
        }
      });
      physicalNodes.forEach(function(physicalNode) {
        document.getElementById(group + '-' + physicalNode.get('uniqueKey')).checked = true;
        wizard.nodesSelection.toggleSelection(group, physicalNode.get('uniqueKey'), true);
      });
    });
  }

  println('enter wizard.nodesSelection.enter');
  if (!wizard.nodesSelection.started) {
    wizard.nodesSelection.started = true;

    //licenseInfo.licensedOpenStackNodes;

    document.getElementById('nodesSelection-licensedOpenStackNodes').value = wizard.common.getNumberOfLicensedNodes(current.licenseInfo);
    var categories = current.find('NodeCategory');

    var rootElement = document.getElementById('nodesSelection-nodesSelection');

    categories.forEach(function(category, index) {
      var physicalNodesInCategory = current.findBy({
        baseType: ['SlaveNode', 'CloudNode'],
        filter: function(node) {
          if(wizard.nodesSelection.filterNodeChildType(node) && node.get('category') === category.get('uniqueKey')) {
            return true;
          }
          return false;
        }
      });
      if(physicalNodesInCategory.length > 0) {

        var vboxElement = document.createElement('vbox');
        vboxElement.setAttribute('class', 'staticdata');
        vboxElement.setAttribute('flex', '1');
        rootElement.appendChild(vboxElement);

        var hbox1Element = document.createElement('hbox');
        hbox1Element.setAttribute('id', category.get('uniqueKey'));
        hbox1Element.setAttribute('class', 'hboxlist');
        hbox1Element.setAttribute('width', '300px');
        hbox1Element.setAttribute('align', 'center');
        hbox1Element.style['margin-left'] = '10px';
        vboxElement.appendChild(hbox1Element);

        var toolbarElement = document.createElement('toolbar');
        toolbarElement.setAttribute('class', 'toolbar-transparent-background');
        toolbarElement.setAttribute('width', '81px');
        hbox1Element.appendChild(toolbarElement);

        [{label: HN}, {label: NW}, {label: CN}].forEach(function(config) {
          wizard.nodesSelection.hash[config.label].categories[category.get('uniqueKey')] = category.get('uniqueKey');
          wizard.nodesSelection.hash[config.label].category[category.get('uniqueKey')] = [];

          var toolbarbutton1Element = document.createElement('toolbarbutton');
          toolbarbutton1Element.setAttribute('id', config.label + '-' + category.get('uniqueKey'));
          toolbarbutton1Element.setAttribute('categoryId', category.get('uniqueKey'));
          toolbarbutton1Element.setAttribute('label', config.label);
          toolbarbutton1Element.setAttribute('width', '27px');
          toolbarbutton1Element.setAttribute('type',  'checkbox');
          toolbarbutton1Element.setAttribute('class', 'checkbox-greenbutton os-node-select');
          toolbarbutton1Element.setAttribute('oncommand', 'wizard.nodesSelection.toggle(this)');
          toolbarElement.appendChild(toolbarbutton1Element);
        });


        var spacer2Element = document.createElement('spacer');
        spacer2Element.setAttribute('width', '10px');
        hbox1Element.appendChild(spacer2Element);

        var caption1Element = document.createElement('caption');
        caption1Element.setAttribute('label', 'Category: ' + category.get('name'));
        caption1Element.setAttribute('class', 'transparent-background');
        hbox1Element.appendChild(caption1Element);

        var hbox2Element = document.createElement('hbox');
        hbox2Element.setAttribute('class', 'staticdottedline');
        hbox2Element.style['margin-top'] = '2px';
        vboxElement.appendChild(hbox2Element);


	      // keep it vertical: CM-6651
        var vbox3Element = document.createElement('vbox');
        //hbox3Element.style['display'] = 'block';
        vboxElement.appendChild(vbox3Element);

        physicalNodesInCategory.forEach(function(physicalNode, index) {

          var hbox4Element = document.createElement('hbox');
          hbox4Element.setAttribute('id', physicalNode.get('uniqueKey'));
          hbox4Element.setAttribute('width', '300px');
          hbox4Element.setAttribute('class', 'hboxlist');
          hbox4Element.setAttribute('align', 'top');
          hbox4Element.style['margin-left'] = '10px';
          vbox3Element.appendChild(hbox4Element);

          var toolbarElement = document.createElement('toolbar');
          toolbarElement.setAttribute('class', 'toolbar-transparent-background');
          toolbarElement.setAttribute('width', '81px');
          hbox4Element.appendChild(toolbarElement);

          [{label: HN}, {label: NW}, {label: CN}].forEach(function(config) {
            wizard.nodesSelection.hash[config.label].physicalNodes[physicalNode.get('uniqueKey')] = physicalNode.get('uniqueKey');
            wizard.nodesSelection.hash[config.label].category[category.get('uniqueKey')].push(physicalNode.get('uniqueKey'));

            var toolbarbutton1Element = document.createElement('toolbarbutton');
            toolbarbutton1Element.setAttribute('id', config.label + '-' + physicalNode.get('uniqueKey'));
            toolbarbutton1Element.setAttribute('physicalNodeId', physicalNode.get('uniqueKey'));
            toolbarbutton1Element.setAttribute('categoryId', category.get('uniqueKey'));
            toolbarbutton1Element.setAttribute('label', config.label);
            toolbarbutton1Element.setAttribute('width', '27px');
            toolbarbutton1Element.setAttribute('type',  'checkbox');
            toolbarbutton1Element.setAttribute('class', 'checkbox-greenbutton os-node-select');
            toolbarbutton1Element.setAttribute('oncommand', 'wizard.nodesSelection.toggle(this)');
            toolbarElement.appendChild(toolbarbutton1Element);
          });

          var spacer3Element = document.createElement('spacer');
          spacer3Element.setAttribute('width', '10px');
          hbox4Element.appendChild(spacer3Element);

          var caption2Element = document.createElement('caption');
          caption2Element.setAttribute('label', physicalNode.get('hostname'));
          caption2Element.setAttribute('class', 'transparent-background');
          hbox4Element.appendChild(caption2Element);

          var error_vbox = document.createElement('vbox');
          error_vbox.setAttribute('id','nodeSelection-nodeError-' + physicalNode.get('uniqueKey'));
          error_vbox.setAttribute('flex', '1');
          error_vbox.setAttribute('pack', 'end');
          hbox4Element.appendChild(error_vbox);
        }); // for each node in category
      } // if category not empty
    }); // for each category

    init(wizard.common.getConfigOverlay('nova', 'OpenStackHypervisors').nodes, HN);
    init(wizard.common.getConfigOverlay('neutron', 'OpenStackNetworkNodes').nodes, NW);
    init(wizard.common.getConfigOverlay('cinder', 'OpenStackControllers').nodes, CN);


    wizard.nodesSelection.setLegend();
    wizard.nodesSelection.checkLicense();

    if (wizard.config.modules.core.reboot_nodes===true) {
      wizard.nodesSelection.setRebootNodesUponCompletion(1);
    } else {
      wizard.nodesSelection.setRebootNodesUponCompletion(0);
    }
    var minutes = Math.round(wizard.config.setup.max_reboot_wait_sec / 60);
    document.getElementById('nodesSelection-maxRebootWaitMin').value = minutes;
  } // started
  else {
    // refresh warning flags
    init(wizard.common.getConfigOverlay('nova', 'OpenStackHypervisors').nodes, HN);
    init(wizard.common.getConfigOverlay('neutron', 'OpenStackNetworkNodes').nodes, NW);
    init(wizard.common.getConfigOverlay('cinder', 'OpenStackControllers').nodes, CN);
  }
  wizard.nodesSelection.num_controller_nodes = wizard.common.getConfigOverlay('cinder', 'OpenStackControllers').nodes.length;
};

wizard.nodesSelection.forward = function(automatic) {
  function atLeastOneNode(list, errorElementId) {
    if (list.length < 1) {
      wizard.common.displayError(errorElementId, automatic);
      wizard.nodesSelection.error = true;
      return false;
    }
    wizard.common.hide(errorElementId);
    return true;
  }

  function notTwoNodes(list, errorElementId) {
    if (list.length == 2) {
      wizard.common.displayError(errorElementId, automatic);
      wizard.nodesSelection.error = true;
      return false;
    }
    wizard.common.hide(errorElementId);
    return true;
  }

  function updateConfig(group) {
    var list = [];
    for (var key in wizard.nodesSelection.hash[group].selectedNodes) {
      if (wizard.nodesSelection.hash[group].physicalNodes.hasOwnProperty(key)) {
        list.push(current.find(key).get('hostname'));
      }
    }
    return list;
  }

  function isValidNumber(value, errorElementId) {
    if (!value.match(/^[0-9]+$/)) {
      wizard.common.displayError(errorElementId, automatic);
      wizard.nodesSelection.error = true;
      return false;
    }
    wizard.common.hide(errorElementId);
    return true;
  }
  println('forward wizard.nodesSelection.forward');

  var hypervisorNodes = updateConfig(HN);
  var networkNodes = updateConfig(NW);
  var controllerNodes = updateConfig(CN);
  var maxRebootWaitMin = document.getElementById('nodesSelection-maxRebootWaitMin').value;
  if (atLeastOneNode(hypervisorNodes, 'nodesSelection-error-no-hypervisor') &&
      atLeastOneNode(networkNodes, 'nodesSelection-error-no-network') &&
      atLeastOneNode(controllerNodes, 'nodesSelection-error-no-controller') &&
      notTwoNodes(controllerNodes, 'nodesSelection-two-controller-nodes') &&
      isValidNumber(maxRebootWaitMin, 'nodesSelection-error-maxRebootWaitMin-invalid')
      ) {

    wizard.common.setConfig(wizard.common.getConfigOverlay('nova', 'OpenStackHypervisors'), 'nodes', hypervisorNodes.slice());
    wizard.common.setConfig(wizard.common.getConfigOverlay('neutron', 'OpenStackNetworkNodes'), 'nodes', networkNodes.slice());
    wizard.common.setConfig(wizard.common.getConfigOverlay('cinder', 'OpenStackControllers'), 'nodes', controllerNodes.slice());
    wizard.common.setConfig(wizard.common.getConfigOverlay('core', 'OpenStackControllers'), 'nodes', controllerNodes.slice());
    wizard.common.setConfig(wizard.common.getConfigOverlay('glance', 'OpenStackControllers'), 'nodes', controllerNodes.slice());
    wizard.common.setConfig(wizard.common.getConfigOverlay('horizon', 'OpenStackControllers'), 'nodes', controllerNodes.slice());
    wizard.common.setConfig(wizard.common.getConfigOverlay('keystone', 'OpenStackControllers'), 'nodes', controllerNodes.slice());
    wizard.common.setConfig(wizard.common.getConfigOverlay('neutron', 'OpenStackControllers'), 'nodes', controllerNodes.slice());
    wizard.common.setConfig(wizard.common.getConfigOverlay('nova', 'OpenStackControllers'), 'nodes', controllerNodes.slice());
    wizard.common.setConfig(wizard.common.getConfigOverlay('rabbitmq', 'OpenStackControllers'), 'nodes', controllerNodes.slice());
    wizard.common.setConfig(wizard.common.getConfigOverlay('heat', 'OpenStackControllers'), 'nodes', controllerNodes.slice());
    wizard.common.setConfig(wizard.common.getConfigOverlay('galera', 'GaleraNodes'), 'nodes', controllerNodes.slice());
    wizard.common.setConfig(wizard.common.getConfigOverlay('radosgw', 'OpenStackControllers'), 'nodes', controllerNodes.slice());
//    wizard.common.setConfig(wizard.common.getConfigOverlay('rabbitmq', 'OpenStackControllersRabbitMQ'), 'nodes',controllerNodes.slice());

    var rebootNodes = (document.getElementById('nodesSelection-rebootNodesUponCompletionYes').getAttribute('on') === 'true') ? true : false;
    wizard.common.setConfig(wizard.config.modules.core, 'reboot_nodes', rebootNodes);
    wizard.common.setConfig(wizard.config.setup, 'only_reboot_controllers', !rebootNodes);
    var maxRebootWaitSec = parseInt(maxRebootWaitMin) * 60;
    wizard.common.setConfig(wizard.config.setup, 'max_reboot_wait_sec', maxRebootWaitSec);
  }

  println('OpenStackHypervisors:');
  printall(wizard.common.getConfigOverlay('nova', 'OpenStackHypervisors'));
  println('OpenStackNetworkNodes:');
  printall(wizard.common.getConfigOverlay('neutron', 'OpenStackNetworkNodes'));
  println('OpenStackControllers:');
  printall(wizard.common.getConfigOverlay('cinder', 'OpenStackControllers'));
  printallR('reboot_nodes: ' + wizard.config.modules.core.reboot_nodes);
  printallR('only_reboot_controllers: ' + wizard.config.setup.only_reboot_controllers);
  printallR('max_reboot_wait_sec: ' + wizard.config.setup.max_reboot_wait_sec);

  return wizard.nodesSelection.error;
};

wizard.nodesSelection.toggle = function(element) {
  var physicalNodeId = element.getAttribute('physicalNodeId');
  var categoryId = element.getAttribute('categoryId');
  var group = element.label;

  if (physicalNodeId !== '') {
    document.getElementById(group + '-' + categoryId).checked = false;
    wizard.nodesSelection.toggleSelection(group, physicalNodeId, element.checked);
  } else {
    var physicalNodeIds = wizard.nodesSelection.hash[element.label].category[categoryId];
    physicalNodeIds.forEach(function(physicalNodeId) {
      document.getElementById(group + '-' + physicalNodeId).checked = element.checked;
      wizard.nodesSelection.toggleSelection(group, physicalNodeId, element.checked);
    });
  }
  wizard.nodesSelection.setLegend();
  wizard.nodesSelection.checkLicense();
};

wizard.nodesSelection.checkControllerNodes = function() {
  wizard.common.hide('nodesSelection-warning-exceeded');
   if(wizard.nodesSelection.num_controller_nodes == 2) {
     wizard.common.unhide('nodesSelection-two-controller-nodes');
   } else {
     wizard.common.hide('nodesSelection-two-controller-nodes');
   }
   if(wizard.nodesSelection.num_controller_nodes < 3) {
     wizard.common.unhide('nodesSelection-less-than-three-controller-nodes');
   } else {
     wizard.common.hide('nodesSelection-less-than-three-controller-nodes');
   }
};

wizard.nodesSelection.toggleSelectionWarning = function(physicalNodeId) {
  var error_vbox = document.getElementById('nodeSelection-nodeError-' + physicalNodeId);
  while (error_vbox.firstChild) {
    error_vbox.removeChild(error_vbox.firstChild);
  }
  var is_NW = wizard.nodesSelection.hash[NW].selectedNodes.hasOwnProperty(physicalNodeId);
  var is_CN = wizard.nodesSelection.hash[CN].selectedNodes.hasOwnProperty(physicalNodeId);
  var is_HN = wizard.nodesSelection.hash[HN].selectedNodes.hasOwnProperty(physicalNodeId);
  if(is_NW || is_CN || is_HN) {
    var is_OSD = wizard.nodesSelection.nodesWithCephOSDRoles.hasOwnProperty(physicalNodeId);
    var node = current.find(physicalNodeId);
    var msg = [];
    if(wizard.nodesSelection.isSuboptimalNode(node, is_NW, is_HN, is_CN, is_OSD)) {
      msg = wizard.nodesSelection.suboptimalNodeMessages(node, is_NW, is_HN, is_CN, is_OSD);
    }
    msg.forEach(function(item) {
      var hbox = document.createElement('hbox');
      var image = document.createElement('image');
      if(item.error) {
	       image.setAttribute('src', 'chrome://cmgui/content/icons/alert.png');
      }
      else {
	       image.setAttribute('src', 'chrome://cmgui/content/icons/warning.png');
      }
      var error_label = document.createElement('label');
      hbox.appendChild(image);
      hbox.appendChild(error_label);
      if(item.error) {
	       error_label.setAttribute('class', 'error');
      }
      else {
	       error_label.setAttribute('class', 'warning');
      }
      error_label.setAttribute('pack', 'end');
      error_label.setAttribute('hidden', 'false');
      error_label.setAttribute('value', (item.error ? "Error: " : "Warning: ")+ item.msg);
      error_label.setAttribute('flex', '1');
      error_vbox.appendChild(hbox);
    });
  }
};

wizard.nodesSelection.toggleSelection = function (group, physicalNodeId, checked) {
  if (checked) {
    wizard.nodesSelection.hash[group].selectedNodes[physicalNodeId] = true;
    wizard.nodesSelection.toggleSelectionWarning(physicalNodeId);
  } else {
    delete wizard.nodesSelection.hash[group].selectedNodes[physicalNodeId];
    wizard.nodesSelection.toggleSelectionWarning(physicalNodeId);
  }
  if(group == 'C') {
    wizard.nodesSelection.num_controller_nodes =  Object.keys(wizard.nodesSelection.hash['C'].selectedNodes).length;
    wizard.nodesSelection.checkControllerNodes();
  }
};

wizard.nodesSelection.setLegend = function() {
  function count(group) {
    return Object.keys(wizard.nodesSelection.hash[group].selectedNodes).length;
  }
  var numHN = count(HN);
  var numNW = count(NW);
  var numCN = count(CN);
  document.getElementById('nodesSelection-nodesSelectionLegend').textContent = 'H = Hypervisor node (' + numHN + '); N = Network node (' + numNW + '); C = Controller node (' + numCN + ');';
};

wizard.nodesSelection.checkLicense = function() {
  var totalNodes = {};
  [HN, NW, CN].forEach(function(group) {
    Object.keys(wizard.nodesSelection.hash[group].selectedNodes).reduce(function(totalNodes, physicalNodeId){
      totalNodes[physicalNodeId] = true;
      return totalNodes;
    }, totalNodes);
  });
  var number_of_licensed_nodes = wizard.common.getNumberOfLicensedNodes(current.licenseInfo);
  if(Object.keys(totalNodes).length > number_of_licensed_nodes) {
    wizard.common.unhide('nodesSelection-warning-exceeded');
  } else {
    wizard.common.hide('nodesSelection-warning-exceeded');
  }
};

wizard.nodesSelection.search = function() {
  var searchName = document.getElementById('nodesSelection-nodesSelectionSearch').value;
  if (searchName === '') {
    return;
  }
  var regExp = RegExp(searchName, 'i');
  var foundNodes = current.findBy({
    baseType: ['SlaveNode', 'CloudNode'],
    filter: function(node) {
      if (wizard.nodesSelection.filterNodeChildType(node) &&  node.get('hostname').search(regExp) !== -1) {
        return true;
      }
      return false;
    }
  });

  var elements = document.getElementsByClassName('hboxlist');
  for(var i = 0; i < elements.length; i++) {
    elements[i].setAttribute('selected', false);
  };

  var first = true;
  foundNodes.forEach(function(node) {
    document.getElementById(node.get('uniqueKey')).setAttribute('selected', true);
    if (first) {
      document.getElementById(node.get('uniqueKey')).scrollIntoView();
      first = false;
    }
  });
};

wizard.nodesSelection.setRebootNodesUponCompletion = function(choice) {
  setSimpleRadio(choice, ['nodesSelection-rebootNodesUponCompletionNo', 'nodesSelection-rebootNodesUponCompletionYes']);
};
