/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */
wizard.storeVmDisks = new WizardScreen();

wizard.storeVmDisks.initDefaultConfig = function() {
  println('init wizard.storeVmDisks.initDefaultConfig');

  if (current.find('Ceph').length > 0) {
    wizard.common.setConfig(wizard.config.modules.nova, 'image_backend', 'ceph');
  } else {
    wizard.common.setConfig(wizard.config.modules.nova, 'image_backend', 'cow_over_nfs_cmshared');
  }
};

wizard.storeVmDisks.enter = function() {
  function initManageUsers() {
    switch(wizard.config.modules.nova.image_backend) {
      case 'ceph':
        wizard.storeVmDisks.setStoreVmDisks(0);
        break;
      case 'cow_over_nfs_cmshared':
        wizard.storeVmDisks.setStoreVmDisks(1);
        break;
      case 'nfs':
        wizard.storeVmDisks.setStoreVmDisks(1);
        break;
      case 'cow_over_gpfs':
        wizard.storeVmDisks.setStoreVmDisks(2);
        break;
      case 'existing_remote_mount':
        wizard.storeVmDisks.setStoreVmDisks(4);
        break;
      case 'cow':
        wizard.storeVmDisks.setStoreVmDisks(5);
        break;
      case 'other':
      case 'none':
      default:
        wizard.storeVmDisks.setStoreVmDisks(6);
        break;
    }
  }
  function initCephOptions() {
    if (current.find('Ceph').length === 0) {
      document.getElementById('storeVmDisks-CephContainer').removeAttribute('onclick');
      wizard.common.disable('storeVmDisks-ceph');
    }
  }
  function initNfsOptions() {
    document.getElementById('storeVmDisks-nfs-mount').value = wizard.config.modules.nova.backend_nfs_vm_dir;
    document.getElementById('storeVmDisks-nfs-mountOptions').value = wizard.config.modules.nova.backend_nfs_mount_options;
  }
  function initGpfsOptions() {
    document.getElementById('storeVmDisks-gpfs-mount').value = wizard.config.modules.nova.backend_gpfs_source_path;
    document.getElementById('storeVmDisks-gpfs-mountOptions').value = wizard.config.modules.nova.backend_gpfs_mount_options;
  }
  function initRemoteOptions() {
    document.getElementById('storeVmDisks-remote-existingPath').value = wizard.config.modules.nova.backend_remote_existing_path;
  }
  println('enter wizard.storeVmDisks.enter');

  if (!wizard.storeVmDisks.started) {
    wizard.storeVmDisks.started = true;

    initManageUsers();
    initCephOptions();
    initNfsOptions();
    initGpfsOptions();
    initRemoteOptions();
  }
};

wizard.storeVmDisks.forward = function(automatic) {
  function isValidPath(path, errorElementId) {
    if (path.trim().length===0) {
      wizard.common.displayError(errorElementId, automatic);
      wizard.storeVmDisks.error = true;
      return false;
    }
    wizard.common.hide(errorElementId);
    return true;
  }
  println('forward wizard.storeVmDisks.forward');

  if (document.getElementById('storeVmDisks-ceph').getAttribute('on') === 'true') {
    wizard.common.setConfig(wizard.config.modules.nova, 'image_backend', 'ceph');

  } else if (document.getElementById('storeVmDisks-cmShared').getAttribute('on') === 'true') {
    wizard.common.setConfig(wizard.config.modules.nova, 'image_backend', 'cow_over_nfs_cmshared');

  } else if (document.getElementById('storeVmDisks-nfs').getAttribute('on') === 'true') {
    var nfsMount = document.getElementById('storeVmDisks-nfs-mount').value;
    var nfsMountOptions = document.getElementById('storeVmDisks-nfs-mountOptions').value;
    if (isValidPath(nfsMount, 'storeVmDisks-error-nfs-mount-invalid')) {
      wizard.common.setConfig(wizard.config.modules.nova, 'image_backend', 'nfs');
      wizard.common.setConfig(wizard.config.modules.nova, 'backend_nfs_vm_dir', nfsMount);
      wizard.common.setConfig(wizard.config.modules.nova, 'backend_nfs_mount_options', nfsMountOptions);
    }

  } else if (document.getElementById('storeVmDisks-gpfs').getAttribute('on') === 'true') {
    var gpfsMount = document.getElementById('storeVmDisks-gpfs-mount').value;
    var gpfsMountOptions = document.getElementById('storeVmDisks-gpfs-mountOptions').value;
    if (isValidPath(gpfsMount, 'storeVmDisks-error-gpfs-mount-invalid')) {
      wizard.common.setConfig(wizard.config.modules.nova, 'image_backend', 'cow_over_gpfs');
      wizard.common.setConfig(wizard.config.modules.nova, 'backend_gpfs_source_path', gpfsMount);
      wizard.common.setConfig(wizard.config.modules.nova, 'backend_gpfs_mount_options', gpfsMountOptions);
    }

  } else if (document.getElementById('storeVmDisks-existingRemoteMount').getAttribute('on') === 'true') {
    var existingPath = document.getElementById('storeVmDisks-remote-existingPath').value;
    if (isValidPath(existingPath, 'storeVmDisks-error-remote-existingPath-invalid')) {
      wizard.common.setConfig(wizard.config.modules.nova, 'image_backend', 'existing_remote_mount');
      wizard.common.setConfig(wizard.config.modules.nova, 'backend_remote_existing_path', existingPath);
    }

  } else if (document.getElementById('storeVmDisks-local').getAttribute('on') === 'true') {
    wizard.common.setConfig(wizard.config.modules.nova, 'image_backend', 'cow');

  } else if (document.getElementById('storeVmDisks-none').getAttribute('on') === 'true') {
    wizard.common.setConfig(wizard.config.modules.nova, 'image_backend', 'other');

  }


  printallR('image_backend: ' + wizard.config.modules.nova.image_backend);

  printallR('backend_nfs_vm_dir: ' + wizard.config.modules.nova.backend_nfs_vm_dir);
  printallR('backend_nfs_mount_options: ' + wizard.config.modules.nova.backend_nfs_mount_options);

  printallR('backend_gpfs_source_path: ' + wizard.config.modules.nova.backend_gpfs_source_path);
  printallR('backend_gpfs_mount_options: ' + wizard.config.modules.nova.backend_gpfs_mount_options);

  printallR('backend_remote_existing_path: ' + wizard.config.modules.nova.backend_remote_existing_path);


  return wizard.storeVmDisks.error;
};

wizard.storeVmDisks.setStoreVmDisks = function(choice) {
  setSimpleRadio(choice, ['storeVmDisks-ceph', 'storeVmDisks-cmShared', 'storeVmDisks-nfs', 'storeVmDisks-gpfs', 'storeVmDisks-existingRemoteMount', 'storeVmDisks-local', 'storeVmDisks-none']);
  if (choice === 2) {
    wizard.common.enable('storeVmDisks-nfs-mount');
    wizard.common.enable('storeVmDisks-nfs-mountOptions');
  } else {
    wizard.common.disable('storeVmDisks-nfs-mount');
    wizard.common.disable('storeVmDisks-nfs-mountOptions');
    wizard.common.hide('storeVmDisks-error-nfs-mount-invalid');
    wizard.common.hide('storeVmDisks-error-nfs-mountOptions-invalid');
  }
  if (choice === 3) {
    wizard.common.enable('storeVmDisks-gpfs-mount');
    wizard.common.enable('storeVmDisks-gpfs-mountOptions');
  } else {
    wizard.common.disable('storeVmDisks-gpfs-mount');
    wizard.common.disable('storeVmDisks-gpfs-mountOptions');
    wizard.common.hide('storeVmDisks-error-gpfs-mount-invalid');
    wizard.common.hide('storeVmDisks-error-gpfs-mountOptions-invalid');
  }
  if (choice === 4) {
    wizard.common.enable('storeVmDisks-remote-existingPath');
  } else {
    wizard.common.disable('storeVmDisks-remote-existingPath');
    wizard.common.hide('storeVmDisks-error-remote-existingPath-invalid');
  }
};

wizard.storeVmDisks.getHelp = function() {
  return document.getElementById('storeVmDisks-help').innerHTML;
};
