/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */
wizard.storeVmImages = new WizardScreen();

wizard.storeVmImages.initDefaultConfig = function() {
  println('init wizard.storeVmImages.initDefaultConfig');

  if (current.find('Ceph').length > 0) {
    wizard.common.setConfig(wizard.config.modules.glance, 'image_backend', 'ceph');
  } else {
    wizard.common.setConfig(wizard.config.modules.glance, 'image_backend', 'cm_shared');
  }
};

const dellStorageCenter_fields = {
  // Todo: move default values to config.js, when this approach is confirmed
  'driverType'    : { 'type': 'select',   
                      'target' : 'backend_dellstoragecenter_san_driver_type',
                      'list' : [ 'iSCSI', 'FibreChannel' ],
                      'default_value' : 'FibreChannel' },
  'sanIP'         : { 'type': 'ip',       
                      'target' : 'backend_dellstoragecenter_san_ip',
                      'default_value' : '0.0.0.0' },
  'sanLogin'      : { 'type': 'string',   
                      'target' : 'backend_dellstoragecenter_san_login',
                      'default_value' : '' },
  'sanPassword'   : { 'type': 'password', 
                      'target' : 'backend_dellstoragecenter_san_password',
                      'default_value' : '' },
  'serverFolder'  : { 'type': 'string',   
                      'target' : 'backend_dellstoragecenter_server_folder',
                      'default_value' : 'openstack' },
  'volumeFolder'  : { 'type': 'string',   
                      'target' : 'backend_dellstoragecenter_volume_folder',
                      'default_value' : 'openstack' },
  'ssn'           : { 'type': 'uint',     
                      'target' : 'backend_dellstoragecenter_ssn',
                      'default_value' : 64702 },
  'apiPort'       : { 'type': 'port',     
                      'target' : 'backend_dellstoragecenter_api_port',
                      'default_value' : 3033 },
  'iscsiIp'       : { 'type': 'ip',       
                      'target' : 'backend_dellstoragecenter_isci_ip',
                      'default_value' : '0.0.0.0' },
  'iscsiPort'     : { 'type': 'port',     
                      'target' : 'backend_dellstoragecenter_isci_port',
                      'default_value' : 3260 },
};


wizard.storeVmImages.enter = function() {
  function initBackendSelection() {
    switch(wizard.config.modules.glance.image_backend) {
      case 'ceph':
        wizard.storeVmImages.setStoreVmImages(0);
        break;
      case 'cm_shared':
        wizard.storeVmImages.setStoreVmImages(1);
        break;
      case 'nfs':
        wizard.storeVmImages.setStoreVmImages(2);
        break;
      case 'gpfs':
        wizard.storeVmImages.setStoreVmImages(3);
        break;
      case 'existing_remote_mount':
        wizard.storeVmImages.setStoreVmImages(4);
        break;
      case 'local':
        wizard.storeVmImages.setStoreVmImages(5);
        break;
      case 'dellStorageCenter':
        wizard.storeVmImages.setStoreVmImages(6);
        break;
      case 'other':
      case 'none':
      default:
        wizard.storeVmImages.setStoreVmImages(7);
        break;
    }
  }
  function initCephOptions() {
    if (current.find('Ceph').length === 0) {
      document.getElementById('storeVmImages-CephContainer').removeAttribute('onclick');
      wizard.common.disable('storeVmImages-ceph');
    }
  }
  function initNfsOptions() {
    document.getElementById('storeVmImages-nfs-mount').value = wizard.config.modules.glance.backend_nfs_mount;
    document.getElementById('storeVmImages-nfs-mountPoint').value = wizard.config.modules.glance.backend_nfs_mount_point;
    document.getElementById('storeVmImages-nfs-mountOptions').value = wizard.config.modules.glance.backend_nfs_mount_options;
  }
  function initGpfsOptions() {
    document.getElementById('storeVmImages-gpfs-mount').value = wizard.config.modules.glance.backend_gpfs_mount;
    document.getElementById('storeVmImages-gpfs-mountPoint').value = wizard.config.modules.glance.backend_gpfs_mount_point;
    document.getElementById('storeVmImages-gpfs-mountOptions').value = wizard.config.modules.glance.backend_gpfs_mount_options;
  }
  function initRemoteOptions() {
    document.getElementById('storeVmImages-remote-existingPath').value = wizard.config.modules.glance.backend_remote_existing_path;
  }

  function initDellStorage() {
    Object.keys(dellStorageCenter_fields).forEach(function(field) {
      if(!wizard.config.modules.glance.hasOwnProperty(dellStorageCenter_fields[field].target)) {
        // Todo: move this to config.js
        wizard.config.modules.glance[dellStorageCenter_fields[field].target] = dellStorageCenter_fields[field].default_value;
      }
      var value = wizard.config.modules.glance[dellStorageCenter_fields[field].target];
      if(dellStorageCenter_fields[field].type == 'ip') {
        var res = value.split(".");
        for(var i = 0; i < 4; i++) {
          document.getElementById('storeVmImages-dellStorageCenter-' + field + '-' + i).value = 
            (i < res.length ? res[i] : '0');
        }
      }
      else {
        document.getElementById('storeVmImages-dellStorageCenter-' + field).value = value;
      }
    });
  } 
  println('enter wizard.storeVmImages.enter');

  if (!wizard.storeVmImages.started) {
    wizard.storeVmImages.started = true;

    initBackendSelection();
    initCephOptions();
    initNfsOptions();
    initGpfsOptions();
    initRemoteOptions();
    //initDellStorage();
  }
};

wizard.storeVmImages.forward = function(automatic) {
  function isValidPath(path, errorElementId) {
    if (path.trim().length===0) {
      wizard.common.displayError(errorElementId, automatic);
      wizard.storeVmImages.error = true;
      return false;
    }
    wizard.common.hide(errorElementId);
    return true;
  }
  println('forward wizard.storeVmImages.forward');

  if (document.getElementById('storeVmImages-ceph').getAttribute('on') === 'true') {
    wizard.common.setConfig(wizard.config.modules.glance, 'image_backend', 'ceph');

  } else if (document.getElementById('storeVmImages-cmShared').getAttribute('on') === 'true') {
    wizard.common.setConfig(wizard.config.modules.glance, 'image_backend', 'cm_shared');

  } else if (document.getElementById('storeVmImages-nfs').getAttribute('on') === 'true') {
    var nfsMount = document.getElementById('storeVmImages-nfs-mount').value;
    var nfsMountPoint = document.getElementById('storeVmImages-nfs-mountPoint').value;
    var nfsMountOptions = document.getElementById('storeVmImages-nfs-mountOptions').value;
    if (isValidPath(nfsMount, 'storeVmImages-error-nfs-mount-invalid') && isValidPath(nfsMountPoint, 'storeVmImages-error-nfs-mountPoint-invalid')) {
      wizard.common.setConfig(wizard.config.modules.glance, 'image_backend', 'nfs');
      wizard.common.setConfig(wizard.config.modules.glance, 'backend_nfs_mount', nfsMount);
      wizard.common.setConfig(wizard.config.modules.glance, 'backend_nfs_mount_point', nfsMountPoint);
      wizard.common.setConfig(wizard.config.modules.glance, 'backend_nfs_mount_options', nfsMountOptions);
    }

  } else if (document.getElementById('storeVmImages-gpfs').getAttribute('on') === 'true') {
    var gpfsMount = document.getElementById('storeVmImages-gpfs-mount').value;
    var gpfsMountPoint = document.getElementById('storeVmImages-gpfs-mountPoint').value;
    var gpfsMountOptions = document.getElementById('storeVmImages-gpfs-mountOptions').value;
    if (isValidPath(gpfsMount, 'storeVmImages-error-gpfs-mount-invalid') && isValidPath(gpfsMountPoint, 'storeVmImages-error-gpfs-mountPoint-invalid')) {
      wizard.common.setConfig(wizard.config.modules.glance, 'image_backend', 'gpfs');
      wizard.common.setConfig(wizard.config.modules.glance, 'backend_gpfs_mount', gpfsMount);
      wizard.common.setConfig(wizard.config.modules.glance, 'backend_gpfs_mount_point', gpfsMountPoint);
      wizard.common.setConfig(wizard.config.modules.glance, 'backend_gpfs_mount_options', gpfsMountOptions);
    }
  } else if (document.getElementById('storeVmImages-dellStorageCenter').getAttribute('on') === 'true') {
    wizard.common.setConfig(wizard.config.modules.glance, 'image_backend', 'dellstoragecenter');
    Object.keys(dellStorageCenter_fields).forEach(function(field) {
      var value = '';
      var ok = false;
      if(dellStorageCenter_fields[field].type == 'ip') {
        value = document.getElementById('storeVmImages-dellStorageCenter-' + field + '-0').value + "." + 
          document.getElementById('storeVmImages-dellStorageCenter-' + field + '-1').value + "." + 
          document.getElementById('storeVmImages-dellStorageCenter-' + field + '-2').value + "." + 
          document.getElementById('storeVmImages-dellStorageCenter-' + field + '-3').value;
        if (wizard.validator.isValidIP(value, 'storeVmImages-dellStorageCenter-' + field + '-invalid', automatic) ) {
          ok = true;
        }
      }
      else if(dellStorageCenter_fields[field].type == 'port') {
        value = document.getElementById('storeVmImages-dellStorageCenter-' + field).value;
        value = parseInt(value);
        if (wizard.validator.isValidPort(value, 'storeVmImages-dellStorageCenter-' + field + '-invalid', automatic) ) {
          ok = true;
        }
      }
      else if(dellStorageCenter_fields[field].type == 'uint') {
        value = document.getElementById('storeVmImages-dellStorageCenter-' + field).value;
        value = parseInt(value);
        if (wizard.validator.isValidUnsignedInt(value, 'storeVmImages-dellStorageCenter-' + field + '-invalid', automatic) ) {
          ok = true;
        }
      }
      else {
        value = document.getElementById('storeVmImages-dellStorageCenter-' + field).value;
        ok = true;
      }
      if(ok) {
        if(!wizard.config.modules.glance.hasOwnProperty(dellStorageCenter_fields[field].target)) {
          wizard.config.modules.glance[dellStorageCenter_fields[field].target] = null;
        }
        wizard.common.setConfig(wizard.config.modules.glance, dellStorageCenter_fields[field].target, value);
      }
    });
  } else if (document.getElementById('storeVmImages-existingRemoteMount').getAttribute('on') === 'true') {
    var existingPath = document.getElementById('storeVmImages-remote-existingPath').value;
    if (isValidPath(existingPath, 'storeVmImages-error-remote-existingPath-invalid')) {
      wizard.common.setConfig(wizard.config.modules.glance, 'image_backend', 'existing_remote_mount');
      wizard.common.setConfig(wizard.config.modules.glance, 'backend_remote_existing_path', existingPath);
    }

  } else if (document.getElementById('storeVmImages-local').getAttribute('on') === 'true') {
    wizard.common.setConfig(wizard.config.modules.glance, 'image_backend', 'local');

  } else if (document.getElementById('storeVmImages-none').getAttribute('on') === 'true') {
    wizard.common.setConfig(wizard.config.modules.glance, 'image_backend', 'other');

  }

  printallR('image_backend: ' + wizard.config.modules.glance.image_backend);

  printallR('backend_nfs_mount: ' + wizard.config.modules.glance.backend_nfs_mount);
  printallR('backend_nfs_mount_point: ' + wizard.config.modules.glance.backend_nfs_mount_point);
  printallR('backend_nfs_mount_options: ' + wizard.config.modules.glance.backend_nfs_mount_options);

  printallR('backend_gpfs_mount: ' + wizard.config.modules.glance.backend_gpfs_mount);
  printallR('backend_gpfs_mount_point: ' + wizard.config.modules.glance.backend_gpfs_mount_point);
  printallR('backend_gpfs_mount_options: ' + wizard.config.modules.glance.backend_gpfs_mount_options);

  printallR('backend_remote_existing_path: ' + wizard.config.modules.glance.backend_remote_existing_path);

  return wizard.storeVmImages.error;
};

wizard.storeVmImages.setStoreVmImages = function(choice) {
  setSimpleRadio(choice, ['storeVmImages-ceph', 'storeVmImages-cmShared', 'storeVmImages-nfs', 'storeVmImages-gpfs', 'storeVmImages-existingRemoteMount', 'storeVmImages-local', 'storeVmImages-dellStorageCenter','storeVmImages-none']);
  if (choice === 2) {
    wizard.common.enable('storeVmImages-nfs-mount');
    wizard.common.enable('storeVmImages-nfs-mountPoint');
    wizard.common.enable('storeVmImages-nfs-mountOptions');
  } else {
    wizard.common.disable('storeVmImages-nfs-mount');
    wizard.common.disable('storeVmImages-nfs-mountPoint');
    wizard.common.disable('storeVmImages-nfs-mountOptions');
    wizard.common.hide('storeVmImages-error-nfs-mount-invalid');
    wizard.common.hide('storeVmImages-error-nfs-mountPoint-invalid');
    wizard.common.hide('storeVmImages-error-nfs-mountOptions-invalid');
  }
  if (choice === 3) {
    wizard.common.enable('storeVmImages-gpfs-mount');
    wizard.common.enable('storeVmImages-gpfs-mountPoint');
    wizard.common.enable('storeVmImages-gpfs-mountOptions');
  } else {
    wizard.common.disable('storeVmImages-gpfs-mount');
    wizard.common.disable('storeVmImages-gpfs-mountPoint');
    wizard.common.disable('storeVmImages-gpfs-mountOptions');
    wizard.common.hide('storeVmImages-error-gpfs-mount-invalid');
    wizard.common.hide('storeVmImages-error-gpfs-mountPoint-invalid');
    wizard.common.hide('storeVmImages-error-gpfs-mountOptions-invalid');
  }
  if (choice === 4) {
    wizard.common.enable('storeVmImages-remote-existingPath');
  } else {
    wizard.common.disable('storeVmImages-remote-existingPath');
    wizard.common.hide('storeVmImages-error-remote-existingPath-invalid');
  }
  if (choice === 6) {
    Object.keys(dellStorageCenter_fields).forEach(function(field) {
      if(dellStorageCenter_fields[field].type == 'ip') {
        wizard.common.enable('storeVmImages-dellStorageCenter-' + field + '-0');
        wizard.common.enable('storeVmImages-dellStorageCenter-' + field + '-1');
        wizard.common.enable('storeVmImages-dellStorageCenter-' + field + '-2');
        wizard.common.enable('storeVmImages-dellStorageCenter-' + field + '-3');
      }
      else {
        wizard.common.enable('storeVmImages-dellStorageCenter-' + field);
      }
    });
  }
  else {
    Object.keys(dellStorageCenter_fields).forEach(function(field) {
      if(dellStorageCenter_fields[field].type == 'ip') {
        wizard.common.disable('storeVmImages-dellStorageCenter-' + field + '-0');
        wizard.common.disable('storeVmImages-dellStorageCenter-' + field + '-1');
        wizard.common.disable('storeVmImages-dellStorageCenter-' + field + '-2');
        wizard.common.disable('storeVmImages-dellStorageCenter-' + field + '-3');
      }
      else if(dellStorageCenter_fields[field].type == 'select'){
        fillListDropdown({
          popupId: 'storeVmImages-dellStorageCenter-' + field,
          list: dellStorageCenter_fields[field].list,
          selectedElement: { defaultIndex: 0}
        });
      }
      else {
        wizard.common.disable('storeVmImages-dellStorageCenter-' + field);
      }
      wizard.common.hide('storeVmImages-dellStorageCenter-' + field + '-invalid');
    });
  }
};
