/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */
wizard.summary = new WizardScreen();

wizard.summary.classifyNodes = function() {
  wizard.nodesSelection.nodesWithCephOSDRoles = wizard.nodesSelection.findAllNodesWithCephOSDRoles(current);
  var reboot_all = wizard.config.modules.core.reboot_nodes;
  var nodes = { 
  };
  var node_matcher_conf = { 
    'NW' : [ 'neutron', 'OpenStackNetworkNodes', reboot_all ], 
    'HN' : [ 'nova', 'OpenStackHypervisors', reboot_all ], 
    'CN' : [ 'cinder', 'OpenStackControllers', true ] };

  Object.keys(node_matcher_conf).forEach(function(k) {
    wizard.common.getConfigOverlay(node_matcher_conf[k][0], node_matcher_conf[k][1]).nodes.forEach(function(nodeName) {
      if(!nodes.hasOwnProperty(nodeName)) {
        nodes[nodeName] = { 
          'NW' : false,
          'HN' : false,
          'CN' : false,
          'OSD' : false,
          'reboot' : node_matcher_conf[k][2],  // reboot
          'node' : current.find(['SlaveNode', 'CloudNode'], nodeName)
        }
        // for testing
        //if(nodeName == 'node001') {
        //  nodes[nodeName].OSD = true;
        //}
      }
      nodes[nodeName][k] = true;
    });
  });
  Object.keys(wizard.nodesSelection.nodesWithCephOSDRoles).forEach(function(k) {
    var nodeName = wizard.nodesSelection.nodesWithCephOSDRoles[k].get('hostname');
    if(nodes.hasOwnProperty(nodeName)) {
      nodes[nodeName]['OSD'] = true;
    }
  });
  return nodes;
}

wizard.summary.showNodeWarnings = function(summary_field, rows_id, msgs, type="warning") {
  // document.getElementById(summary_field).classList.add(type);
  var rows = document.getElementById(rows_id);
  while (rows.childNodes.length > 1) {
    rows.removeChild(rows.lastChild);
  }
  if(msgs) {
    msgs.forEach(function(msg) { 
      var row = document.createElement('row');
      var description = document.createElement('description');
      row.appendChild(description);
      var hbox = document.createElement('hbox');
      var image = document.createElement('image');
      var image_src = type == "error" ? "alert" : type;
      image.setAttribute('src', 'chrome://cmgui/content/icons/' + image_src + '.png');
      if(msg.hasOwnProperty('nodes')) {
        image.setAttribute('tooltiptext', dotFormat(msg.nodes));
      }
      hbox.appendChild(image);
      var error_label = document.createElement('label');
      error_label.setAttribute('class', type);
      error_label.setAttribute('pack', 'end');
      error_label.setAttribute('hidden', 'false');
      error_label.setAttribute('value', msg.msg);
      error_label.setAttribute('flex', '1');
      hbox.appendChild(error_label);
      row.appendChild(hbox);
      rows.appendChild(row);    
    });
  }
};

wizard.summary.enter = function() {
  println('enter wizard.summary.enter');
  if (!wizard.summary.started) {
    wizard.summary.started = true;
  }
  document.getElementById('summary-softwareImage-count').value = '(' + wizard.config.modules.core.software_image_paths.length + ')';
  document.getElementById('summary-softwareImage').textContent = wizard.config.modules.core.software_image_paths.join(', ');
  document.getElementById('summary-storeVmImages').textContent = wizard.common.getGlanceImageBackendHuman();
  document.getElementById('summary-storeVolumes').textContent = wizard.common.getCinderImageBackendHuman();
  document.getElementById('summary-storeVmDisks').textContent = wizard.common.getNovaImageBackendHuman();
  document.getElementById('summary-hypervisorNodes-count').value = '(' + wizard.common.getConfigOverlay('nova', 'OpenStackHypervisors').nodes.length + ')';
  document.getElementById('summary-hypervisorNodes').textContent = dotFormat(wizard.common.getConfigOverlay('nova', 'OpenStackHypervisors').nodes);
  document.getElementById('summary-networkNodes-count').value = '(' + wizard.common.getConfigOverlay('neutron', 'OpenStackNetworkNodes').nodes.length + ')';
  document.getElementById('summary-networkNodes').textContent = dotFormat(wizard.common.getConfigOverlay('neutron', 'OpenStackNetworkNodes').nodes);
  document.getElementById('summary-controllerNodes-count').value = '(' + wizard.common.getConfigOverlay('cinder', 'OpenStackControllers').nodes.length + ')';
  document.getElementById('summary-controllerNodes').textContent = dotFormat(wizard.common.getConfigOverlay('cinder', 'OpenStackControllers').nodes);

  var totalNodes = wizard.summary.classifyNodes();
  var reboot_nodes = [];
  var reboot_selection_has_error = [];
  var licensing_has_error = false;
  var HN_selection_is_suboptimal = wizard.nodesSelection.getSuboptimalHypervisorMessages(totalNodes);
  var CN_selection_is_suboptimal = wizard.nodesSelection.getSuboptimalControllerMessages(totalNodes);
  var NW_selection_is_suboptimal = wizard.nodesSelection.getSuboptimalNetworkMessages(totalNodes);
  Object.keys(totalNodes).forEach(function(nodeName) {
    var node_cls = totalNodes[nodeName];
    if(node_cls.reboot) {
      reboot_nodes.push(nodeName);
      if(node_cls.node.status.status != 'UP') {
        reboot_selection_has_error.push(nodeName);
      }
    }
  });

  wizard.summary.showNodeWarnings('summary-networkNodes', 'summary-networkNodes-rows', NW_selection_is_suboptimal);
  wizard.summary.showNodeWarnings('summary-hypervisorNodes', 'summary-hypervisorNodes-rows', HN_selection_is_suboptimal);
  wizard.summary.showNodeWarnings('summary-controllerNodes', 'summary-controllerNodes-rows', CN_selection_is_suboptimal);
  wizard.summary.showNodeWarnings('summary-rebootNodes', 'summary-rebootNodes-rows', 
                                  reboot_selection_has_error.length > 0  ? 
                                  [ { msg: 'Not all nodes to be rebooted are UP', nodes : reboot_selection_has_error } ] : 
                                  [], 'error');


  document.getElementById('summary-rebootNodes-count').value = '(' + reboot_nodes.length + ')';
  document.getElementById('summary-rebootNodes').value = dotFormat(reboot_nodes);

  var number_of_licensed_nodes = wizard.common.getNumberOfLicensedNodes(current.licenseInfo);
  document.getElementById('summary-licensing').textContent = Object.keys(totalNodes).length + ' / ' + number_of_licensed_nodes;
  if (Object.keys(totalNodes).length > number_of_licensed_nodes) {
    document.getElementById('summary-licensing').classList.add('error');
    document.getElementById('summary-licensing-error').removeAttribute('hidden');
    licensing_has_error = true;
  } else {
    document.getElementById('summary-licensing').classList.remove('error');
    document.getElementById('summary-licensing-error').setAttribute('hidden', true);
  }
  document.getElementById('summary-internalNetwork').textContent = [
    wizard.config.modules.neutron.internal_net_name,
    ' ( ',
    wizard.config.modules.neutron.internal_net_allocation_pool_start,
    ' - ',
    wizard.config.modules.neutron.internal_net_allocation_pool_end,
    ' )'
  ].join('');
  document.getElementById('summary-networkLayer2Agent').textContent = (wizard.config.modules.neutron.l2_agent === 'OVS') ? 'Open vSwitch' : 'Linux bridge';
  document.getElementById('summary-networkIsolation').textContent = wizard.config.modules.neutron.user_network_isolation;
  document.getElementById('summary-floatinIPs').textContent = (wizard.config.modules.neutron.external_net_allocation_pool_start === null) ?
    'No' :
    [
      'Yes ( ',
      wizard.config.modules.neutron.external_net_allocation_pool_start,
      ' - ',
      wizard.config.modules.neutron.external_net_allocation_pool_end,
      ' )'
    ].join('');

  if(reboot_selection_has_error.length > 0 || licensing_has_error) {
    wizard.common.disable('deployButton');
    var message_grid = document.getElementById('summary-has-error-message');
    message_grid.removeAttribute('collapsed');
  }
  else {
    var message_grid = document.getElementById('summary-has-error-message');
    message_grid.setAttribute('collapsed',true);
  }

};

wizard.summary.forward = function(automatic) {
  println('forward wizard.summary.forward');

  return wizard.summary.error;
};

wizard.summary.preEnter = function() {
  wizard.common.unhide('deployButton');
  if (wizard.dryMode) {
    wizard.common.unhide('summary-message-try-run-no-changes');
    wizard.common.hide('summary-message-production-options');
    wizard.common.hide('summary-message-press-deploy-button-to-start');
    wizard.common.disable('deployButton');
  } else {
    wizard.common.hide('summary-message-try-run-no-changes');
    wizard.common.unhide('summary-message-production-options');
    wizard.common.unhide('summary-message-press-deploy-button-to-start');
    wizard.common.enable('deployButton');
  }
  wizard.common.disable('nextButton');
};

wizard.summary.leave = function(automatic) {
  wizard.common.unhide('nextButton');
};

