/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */
wizard.vlanConfiguration = new WizardScreen();

wizard.vlanConfiguration.initDefaultConfig = function() {
  println('init wizard.vlanConfiguration.initDefaultConfig');

  var networkTable = {
    final: 0,
    name: wizard.config.modules.neutron.vlan_host_net_name,
    vlanRange: wizard.config.modules.neutron.vlan_host_vlan_range
  };
  var internalNetworks = current.findInternalNetworks();
  internalNetworks.reduce(function(networkTable, network) {
    if (networkTable.final<3 && network.get('openStackType') === 'VLANHOST') {
      networkTable.final = 3;
      networkTable.name = network.get('name');
      networkTable.vlanRange = network.get('openStackVlanRange');
    } else if (networkTable.final<2 && network.get('name') === 'vlanhostnet') {
      networkTable.final = 2;
      networkTable.name = network.get('name');
    } else if (networkTable.final<1 && network.get('name') !== wizard.config.modules.neutron.internal_net_name) {
      networkTable.final = 1;
      networkTable.name = network.get('name');
    }
    return networkTable;
  }, networkTable);

  wizard.common.setConfig(wizard.config.modules.neutron, 'vlan_host_vlan_range', networkTable.vlanRange);
  wizard.common.setConfig(wizard.config.modules.neutron, 'vlan_host_net_name', networkTable.name);
};

wizard.vlanConfiguration.enter = function() {
  function initVlanRange() {
    var vlanRangeStart = 0;
    var vlanRangeEnd = 0;
    var vlanHostVlanRangeMatch = wizard.config.modules.neutron.vlan_host_vlan_range.match(/([0-9]+):([0-9]+)/);
    if (vlanHostVlanRangeMatch !== null) {
      vlanRangeStart = vlanHostVlanRangeMatch[1];
      vlanRangeEnd = vlanHostVlanRangeMatch[2];
    }
    document.getElementById('vlanConfiguration-RangeStart').value = vlanRangeStart;
    document.getElementById('vlanConfiguration-RangeEnd').value = vlanRangeEnd;
  }

  function initNewNetName(name) {
    document.getElementById('vlanConfiguration-NewName').value = name;
  }

  println('enter wizard.vlanConfiguration.enter');

  if (!wizard.vlanConfiguration.started) {
    wizard.vlanConfiguration.started = true;


    var internalNetworks = current.findInternalNetworks();
    var newNetName = wizard.config.modules.neutron.vlan_host_net_name;
    var newNetNameExists = false;
    var internalNetworksForVLAN = [];
    for (var i = 0; i < internalNetworks.length; i++) {
      if (internalNetworks[i].resolveName() !== wizard.config.modules.neutron.internal_net_name) {
        // Prepend vlanhostnet type/name so that it is offered first before anything else in the dropdown
        if (internalNetworks[i].get('openStackType') === 'VLANHOST' || internalNetworks[i].resolveName() === wizard.config.modules.neutron.vlan_host_net_name) {
          internalNetworksForVLAN.unshift(internalNetworks[i]);
        } else {
          internalNetworksForVLAN.push(internalNetworks[i]);
        }
        // Just in order to avoid suggesting a name of an already existing network
        if (internalNetworks[i].resolveName() === wizard.config.modules.neutron.vlan_host_net_name) {
          newNetName = '';
          newNetNameExists = true;
        }
      }
    }

    fillListDropdown({
      popupId: 'vlanConfiguration-UseExistingList',
      list: internalNetworksForVLAN,
      selectedElement: {value: wizard.config.modules.neutron.vlan_host_net_name, defaultIndex: 0}
    });

    initNewNetName(newNetName);

    initVlanRange();

    if (internalNetworksForVLAN.length > 0 && newNetNameExists) {
      wizard.vlanConfiguration.setVlanCreationMethod(0);
    } else if (internalNetworksForVLAN.length > 0 && !newNetNameExists) {
      wizard.vlanConfiguration.setVlanCreationMethod(1);
    } else {
      wizard.vlanConfiguration.setVlanCreationMethod(1);
      wizard.common.disable('vlanConfiguration-CreationMethodUse');
      wizard.common.disable('vlanConfiguration-CreationMethodNew');
      wizard.vlanConfiguration.setVlanCreationMethod = function() {};
      document.getElementById('vlanConfiguration-UseLabel').value = "No available internal networks. The '" + wizard.config.modules.neutron.internal_net_name + "' cannot be used for VLAN host.";
      wizard.common.unhide('vlanConfiguration-UseLabel');
      wizard.common.hide('vlanConfiguration-UseExistingList');
    }
  }
};

wizard.vlanConfiguration.forward = function(automatic) {
  function isValidInterfaceName(interfaceName, errorElementId) {
    if (!interfaceName.match(/^[a-zA-Z0-9]+$/)) {
      wizard.common.displayError(errorElementId, automatic);
      wizard.vlanConfiguration.error = true;
      return false;
    }
    wizard.common.hide(errorElementId);
    return true;
  }
  function isValidNumber(password, errorElementId) {
    if (!password.match(/^[0-9]+$/)) {
      wizard.common.displayError(errorElementId, automatic);
      wizard.vlanConfiguration.error = true;
      return false;
    }
    wizard.common.hide(errorElementId);
    return true;
  }
  function isValidRange(start, end, errorElementId) {
    if (parseInt(start) >= parseInt(end)) {
      wizard.common.displayError(errorElementId, automatic);
      wizard.vlanConfiguration.error = true;
      return false;
    }
    wizard.common.hide(errorElementId);
    return true;
  }
  println('forward wizard.vlanConfiguration.forward');

  if (document.getElementById('vlanConfiguration-CreationMethodNew').getAttribute('on') === 'true') {
    var interfaceName = document.getElementById('vlanConfiguration-NewName').value;
  } else {
    var interfaceName = document.getElementById('vlanConfiguration-UseExistingList').label;
  }

  if (isValidNumber(document.getElementById('vlanConfiguration-RangeStart').value, 'vlanConfiguration-error-rangeStart-invalid')) {
    if (isValidNumber(document.getElementById('vlanConfiguration-RangeEnd').value, 'vlanConfiguration-error-rangeEnd-invalid')) {
      if (isValidRange(document.getElementById('vlanConfiguration-RangeStart').value, document.getElementById('vlanConfiguration-RangeEnd').value, 'vlanConfiguration-error-rangeEnd-lower')) {
        if (isValidInterfaceName(interfaceName, 'vlanConfiguration-error-newName-invalid')) {
          wizard.common.setConfig(wizard.config.modules.neutron, 'vlan_host_vlan_range', document.getElementById('vlanConfiguration-RangeStart').value + ':' + document.getElementById('vlanConfiguration-RangeEnd').value);
          wizard.common.setConfig(wizard.config.modules.neutron, 'vlan_host_net_name', interfaceName);
        }
      }
    }
  }

  printallR('vlan_host_vlan_range: ' + wizard.config.modules.neutron.vlan_host_vlan_range);
  printallR('vlan_host_net_name:' + wizard.config.modules.neutron.vlan_host_net_name);

  return wizard.vlanConfiguration.error;
};

wizard.vlanConfiguration.resetALLCreateNewErrors = function() {
  wizard.common.hide('vlanConfiguration-error-newName-invalid');
};

wizard.vlanConfiguration.setVlanCreationMethod = function(choice) {
  setSimpleRadio(choice, [ 'vlanConfiguration-CreationMethodUse', 'vlanConfiguration-CreationMethodNew']);
  if (choice === 1) {
    wizard.common.enable('vlanConfiguration-NewName');
    wizard.common.disable('vlanConfiguration-UseExistingList');
  } else {
    wizard.common.disable('vlanConfiguration-NewName');
    wizard.common.enable('vlanConfiguration-UseExistingList');
    wizard.vlanConfiguration.resetALLCreateNewErrors();
  }
};
