/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */
function onLoad() {
  dock = getDock();

  println("OpenStack Setup Wizard initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  current = myDialog.arguments.cluster;
  window.addEventListener('close', function (event) {
      event.preventDefault();
      wizard.common.cancel();
    },
    true
  );

  wizard.common.initDefaultConfig();
  wizard.common.go(wizard.common.getNextStage.bind(undefined, null));
}

var wizard = new (function() {
  Wizard.call(this);
  function externalNetworkOrSummary() {
    if (wizard.config.modules.neutron.external_net_name !== null) {
      return 'externalNetworkInterface';
    } else {
      return 'summary';
    }
  }

  function externalNetworkOrInboundExternalTraffic() {
    if (wizard.config.modules.neutron.external_net_name !== null) {
      return 'externalNetworkInterface';
    } else {
      return 'inboundExternalTraffic';
    }
  }

  function summaryOrDeploy() {
    if (wizard.dryMode) {
      return 'summary';
    } else {
      return 'deploy';
    }
  }
  
  // add gui specific things
  function patchDefaultConfiguration(config) {
    config.modules.neutron.new_external_host_net_interface =  null;
    config.modules.neutron.new_external_host_net_interface_default_physical =  null;
    config.modules.neutron.new_external_host_net_interface_default_shared_network = null;
    config.modules.neutron.new_external_host_net_interface_default_type = "NEW_DEDICATED";
    config.modules.neutron.new_external_host_net_interface_default_vlanId = null;
    config.modules.neutron.new_virtual_host_net_interface = null;
    config.modules.neutron.new_virtual_host_net_interface_default_network_physical =  null;
    config.modules.neutron.new_virtual_host_net_interface_default_network_shared_network = "internalnet";
    config.modules.neutron.new_virtual_host_net_interface_default_network_type = "NEW_SHARED";
    config.modules.neutron.new_virtual_host_net_interface_default_network_vlanId = "";
    config.modules.neutron.new_virtual_host_net_interface_default_compute_physical =  null;
    config.modules.neutron.new_virtual_host_net_interface_default_compute_shared_network =  "internalnet";
    config.modules.neutron.new_virtual_host_net_interface_default_compute_type =  "NEW_SHARED";
    config.modules.neutron.new_virtual_host_net_interface_default_compute_vlanId = "";
    return config;
  }


  function virtualNetworkInterfaceFn() {
    document.getElementById('virtualNetworkInterface').innerHTML = document.getElementById('network-interface').innerHTML.replace(/wizard\.networkInterface/g, 'wizard.virtualNetworkInterface').replace(/networkInterface-/g, 'virtualnetworkInterface-');
    wizard.virtualNetworkInterface = new NetworkInterface({
      prefix: 'virtual',
      nodesListFn: wizard.common.getHypervisorAndNetworkNodes,
      connectNetworkFn: wizard.common.getVirtualNetwork,
      configAttribute: 'new_virtual_host_net_interface',
      defaultType:  selectVirtualInterfaceConfigDefault,
      interfaceConfigMethodFn : selectVirtualInterfaceConfigMethods
    });
    wizard.virtualNetworkInterface.initDefaultConfig = function() {
      println('init wizard.virtualNetworkInterface.initDefaultConfig');
      //  wizard.common.setConfig(wizard.config.modules.neutron, 'new_virtual_host_net_interface', null);
    };
  }

  function externalNetworkInterfaceFn() {
    document.getElementById('externalNetworkInterface').innerHTML = document.getElementById('network-interface').innerHTML
      .replace(/wizard\.networkInterface/g, 'wizard.externalNetworkInterface')
      .replace(/networkInterface-/g, 'externalnetworkInterface-')
      .replace(/external="false"/g, 'hidden="true"');

    wizard.externalNetworkInterface = new NetworkInterface({
      prefix: 'external',
      nodesListFn: nodesListFn,
      connectNetworkFn: wizard.common.getExternalNetwork,
      configAttribute: 'new_external_host_net_interface',
      defaultType: wizard.config.modules.neutron.new_external_host_net_interface_default_type
    });
    
    wizard.externalNetworkInterface.initDefaultConfig = function() {
      println('init wizard.externalNetworkInterface.initDefaultConfig');
      
      //  wizard.common.setConfig(wizard.config.modules.neutron, 'new_virtual_host_net_interface', null);
    };

    function nodesListFn() {
      var nodelist = [];
      if (wizard.config.modules.neutron.external_net_name !== null) {
        nodelist = wizard.common.getNetworkNodes();
      }
      return nodelist;
    }
  }


  // Load config from head node
  this.common.beforeInitDefaultConfig = function(followup) {
    var self = this;
    var request = { "tool" : "cm-openstack-setup" };
    // disable button with popup does not work
    wizard.common.unhide("nextButtonDisabled");
    wizard.common.hide("nextButton");
    wizard.common.unhide('main-wait-for-config');
    printall("retrieve default config");
    wizard.common.remoteValidation("cm_setup_get_tool_config", request, function(response) {
      if(response.state == 3) {
        self.error = true;
      } else {
        printall(response.result.modules);
        wizard.config.modules = patchDefaultConfiguration(response.result).modules; 
        //@FIXME: dynmaically load software image paths
        for(var k in wizard.config.modules) {
          wizard.config.modules[k].software_image_paths = [ '/cm/images/default-image' ];
        }
        virtualNetworkInterfaceFn();
        externalNetworkInterfaceFn();
      }
      wizard.common.unhide("nextButton");
      wizard.common.hide("nextButtonDisabled");
      wizard.common.hide('main-wait-for-config');
      followup();
    });
  };
  this.dryMode = false;
  this.forceDryMode = false;
  this.advancedMode = false;
  this.lastConfigSaved = false;
  this.configuration_name = "OpenStack Configuration";
  this.deck = {
    main: {
      name: 'main',
      next: 'credentials',
      previous: null,
      automatic: true,
      index: 0
    },
    credentials: {
      name: 'credentials',
      next: 'softWareImage',
      previous: 'main',
      automatic: false,
      index: 1
    },
    softWareImage: {
      name: 'softWareImage',
      next: 'manageUsers',
      previous: 'credentials',
      automatic: true,
      index: 2
    },
    manageUsers: {
      name: 'manageUsers',
      next: 'storeVmImages',
      previous: 'softWareImage',
      automatic: true,
      index: 3
    },
    storeVmImages: {
      name: 'storeVmImages',
      next: 'storeVolumes',
      previous: 'manageUsers',
      automatic: true,
      index: 4
    },
    storeVolumes: {
      name: 'storeVolumes',
      next: 'storeVmDisks',
      previous: 'storeVmImages',
      automatic: true,
      index: 5
    },
    storeVmDisks: {
      name: 'storeVmDisks',
      next: 'nodesSelection',
      previous: 'storeVolumes',
      automatic: true,
      index: 6
    },
    nodesSelection: {
      name: 'nodesSelection',
      next: 'internalNetwork',
      previous: 'storeVmDisks',
      automatic: true,
      index: 7
    },
    internalNetwork: {
      name: 'internalNetwork',
      next: 'networkLayer2Agent',
      previous: 'nodesSelection',
      automatic: true,
      index: 8
    },
    networkLayer2Agent: {
      name: 'networkLayer2Agent',
      next: 'networkIsolation',
      previous: 'internalNetwork',
      automatic: true,
      index: 9
    },
    networkIsolation: {
      name: 'networkIsolation',
      next: 'virtualNetworkInterface',
      previous: 'networkLayer2Agent',
      automatic: true,
      index: 10
    },
    vlanConfiguration: {
      name: 'vlanConfiguration',
      next: null,
      previous: null,
      automatic: true,
      index: null
    },
    vxlanConfiguration: {
      name: 'vxlanConfiguration',
      next: null,
      previous: null,
      automatic: true,
      index: null
    },
    virtualNetworkInterface: {
      name: 'virtualNetworkInterface',
      next: 'inboundExternalTraffic',
      previous: 'networkIsolation',
      automatic: true,
      index: 11
    },
    inboundExternalTraffic: {
      name: 'inboundExternalTraffic',
      next: externalNetworkOrSummary,
      previous: 'virtualNetworkInterface',
      automatic: true,
      index: 12
    },
    externalNetworkInterface: {
      name: 'externalNetworkInterface',
      next: 'summary',
      previous: 'inboundExternalTraffic',
      automatic: true,
      index: 13
    },
    summary: {
      name: 'summary',
      next: summaryOrDeploy,
      previous: externalNetworkOrInboundExternalTraffic,
      automatic: false,
      index: 14
    },
    deploy: {
      name: 'deploy',
      next: null,
      previous: 'summary',
      automatic: false,
      index: 15
    }
  },
  this.currentStage = null;
  this.previousStage = null;
})();


wizard.common.getNumberOfLicensedNodes = function(licenseInfo) {
  if(licenseInfo.licensedOpenStackNodes == -1) {
    return 0;
  }
  else if(licenseInfo.licensedOpenStackNodes == 0) {
    return licenseInfo.licensedNodes ;
  }
  else {
    return licenseInfo.licensedOpenStackNodes;
  }
};

wizard.common.resetAllStarted = function() {
  Object.keys(wizard.deck).forEach(function(deckName){
    wizard[deckName].started = false;
  });
};

wizard.common.getConfigOverlay = function(section, name) {
  var searchedOverlay = {};
  return wizard.config.modules[section].overlays.reduce(function(searchedOverlay, overlay) {
    if (overlay.name===name) {
      searchedOverlay = overlay;
    }
    return searchedOverlay;
    }, searchedOverlay);
};

wizard.common.getGlanceImageBackendHuman = function() {
    var result = '';
    switch(wizard.config.modules.glance.image_backend) {
      case 'ceph':
        result = 'Ceph - RBD volumes';
        break;
      case 'cm_shared':
        result = 'NFS mount image directory via /cm/shared';
        break;
      case 'nfs':
        result = 'NFS mount from external NAS/NFS';
        break;
      case 'gpfs':
        result = 'GPFS mount via /etc/fstab';
        break;
      case 'existing_remote_mount':
        result = 'Remote mount - Existing remote network mount';
        break;
      case 'local':
        result = 'Store images locally on the glance-api nodes';
        break;
      case 'other':
      case 'none':
      default:
        result = 'Not configured';
        break;
    }
    return result;
};

wizard.common.getCinderImageBackendHuman = function() {
  var result = '';
  switch(wizard.config.modules.cinder.volume_backend) {
  case 'ceph':
    result = 'Ceph - RBD volumes';
    break;
  case 'nfs':
    result = 'NFS mount image directory via /cm/shared';
    break;
  case 'gpfs':
    result = 'Volumes stored on GPFS';
    break;
  case 'solidfire':
    result = 'Store in Solidfire';
    break;
  case '3par':
    result = 'Store in a 3PAR appliance';
    break;
  case 'netapp':
    result = 'Store in a NetApp appliance';
    break;
  case 'other':
  case 'none':
  default:
    result = 'Not configured';
    break;
  }
  return result;
};

wizard.common.getNovaImageBackendHuman = function() {
  var result = '';
  switch(wizard.config.modules.nova.image_backend) {
  case 'ceph':
    result = 'Ceph - Store in Ceph, /var/lib/nova/instances locally';
    break;
  case 'cow_over_nfs_cmshared':
    result = '/cm/shared - /var/lib/nova/instances over NFS from /cm/shared';
    break;
  case 'nfs':
    result = 'NFS - mount from an external NFS/NAS server';
    break;
  case 'cow_over_gpfs':
    result = 'GPFS - /var/lib/nova/instances over GPFS (via /etc/fstab entry)';
    break;
  case 'existing_remote_mount':
    result = 'Remote mount - Existing remote network mount';
    break;
  case 'cow':
    result = 'Local - On the hypervisor\'s local filesystem (fast, but no live migration)';
    break;
  case 'other':
  case 'none':
  default:
    result = 'Not configured';
    break;
  }
  return result;
};

wizard.common.getHypervisorAndNetworkNodes = function() {
  var configOpenStackHypervisors = wizard.common.getConfigOverlay('nova', 'OpenStackHypervisors').nodes;
  var configOpenStackNetworkNodes = wizard.common.getConfigOverlay('neutron', 'OpenStackNetworkNodes').nodes;
  return current.findBy({
    baseType: 'Device',
    filter: function(node) {
      node.isHypervisorNode = (configOpenStackHypervisors.indexOf(node.get('hostname')) !== -1);
      node.isNetworkNode = (configOpenStackNetworkNodes.indexOf(node.get('hostname')) !== -1);
      if (node.isHypervisorNode || node.isNetworkNode) {
        return true;
      }
      return false;
    }
  });
};

wizard.common.getHypervisorNodes = function() {
  return current.findBy({
    baseType: 'Device',
    filter: function(node) {
      if (wizard.common.getConfigOverlay('nova', 'OpenStackHypervisors').nodes.indexOf(node.get('hostname')) !== -1) {
        return true;
      }
      return false;
    }
  });
};

wizard.common.getNetworkNodes = function() {
  return current.findBy({
    baseType: 'Device',
    filter: function(node) {
      if (wizard.common.getConfigOverlay('neutron', 'OpenStackNetworkNodes').nodes.indexOf(node.get('hostname')) !== -1) {
        return true;
      }
      return false;
    }
  });
};

wizard.common.getExternalNetwork = function() {
  var externalNetwork = current.findBy({
    baseType: 'Network',
    filter: function(network) {
      if (network.get('type') === 'EXTERNAL' && network.get('name') === wizard.config.modules.neutron.external_net_name) {
        return true;
      }
      return false;
    }
  });
  return (externalNetwork.length > 0) ? externalNetwork[0] : null;
};

wizard.common.getVirtualNetwork = function() {
  var virtualNetwork = current.findBy({
    baseType: 'Network',
    filter: function(network) {
      if (wizard.config.modules.neutron.user_network_isolation === 'VXLAN' && network.get('name') === wizard.config.modules.neutron.vxlan_host_net_name) {
        return true;
      } else if (wizard.config.modules.neutron.user_network_isolation === 'VLAN' && network.get('name') === wizard.config.modules.neutron.vlan_host_net_name) {
        return true;
      }
      return false;
    }
  });
  return (virtualNetwork.length > 0) ? virtualNetwork[0] : null;
};

wizard.common.getNodesToReboot = function() {
  var nodesToReboot = [];
  if (!wizard.config.modules.core.reboot_nodes) {
    nodesToReboot = nodesToReboot.concat(wizard.common.getConfigOverlay('nova', 'OpenStackHypervisors').nodes).concat(wizard.common.getConfigOverlay('nova', 'OpenStackHypervisors').nodes).concat(wizard.common.getConfigOverlay('cinder', 'OpenStackControllers').nodes);
    nodesToReboot = nodesToReboot.sort().filter(function(item, pos, arr) {
      return !pos || item !== arr[pos - 1];
    });
  }
  return nodesToReboot;
};

wizard.common.configToYaml = function() {
  function doExpand() {
    var config = JSON.parse(JSON.stringify(wizard.config));

    ['cinder', 'galera', 'glance', 'horizon', 'keystone', 'neutron', 'nova', 'heat'].forEach(function(module) {
      ['mysql_admin_password', 'mysql_admin_username', 'mysql_host', 'mysql_port', 'region_name', 'service_project_name'].forEach(function(param) {
        if (config.modules[module][param]===null) {
          config.modules[module][param] = config.modules.core[param];
        }
      });
    });
    config.modules.radosgw['region_name'] = config.modules.core['region_name'];
    config.modules.radosgw['service_project_name'] = config.modules.core['service_project_name'];
    config.modules.radosgw['software_image_paths'] = config.modules.core['software_image_paths'];
    
    ['cinder', 'galera', 'glance', 'horizon', 'keystone', 'neutron', 'nova', 'rabbitmq', 'heat'].forEach(function(module) {
      ['software_image_paths'].forEach(function(param) {
        if (config.modules[module][param]===null) {
          config.modules[module][param] = config.modules.core[param];
        }
      });
    });
    ['galera'].forEach(function(module) {
      ['internal_net_name'].forEach(function(param) {
        if (config.modules[module][param]===null) {
          config.modules[module][param] = config.modules.neutron[param];
        }
      });
    });
    ['core'].forEach(function(module) {
      ['vnodes_internal_network'].forEach(function(param) {
        if (config.modules[module][param]===null) {
          config.modules[module][param] = config.modules.neutron['internal_net_name'];
        }
      });
    });
    return config;
  }

  return jsyaml.dump(doExpand(), {
    flowLevel: -1
  });
};

/* 
wizard.common.loadConfiguration = function() {
  var win = dock.windowMediator.getMostRecentWindow('detachedopenstackloadconfig');
  if ( win ) {
    dock.windowMediator.getMostRecentWindow('detachedopenstackloadconfig').focus();
    return false;
  }

  var args = new Object();
  args['title'] = 'OpenStack configuration';
  args['style'] = 'background-color: white !important;';
  var d = new dock.cvDialog("chrome://cmgui/content/openstack/install/loadconfig.xul", args);
  d.onOk = function (config, fileName){
    if(!config.hasOwnProperty('setup')) {
      config['setup'] = {
        "only_reboot_controllers" : false,
        "max_reboot_wait_sec" : 1200
      };
    }
    wizard.config = config;
    wizard.common.resetErrors();
    wizard.common.currentDeck().leave(false);
    wizard.common.currentDeck().forward(false);
    wizard.common.go(wizard.common.getStage.bind(undefined, 'main'));
  };
  dock.dialogs.showDialog(d);
};
*/
/*
wizard.common.showConfiguration = function() {
  wizard.common.resetErrors();
  wizard.common.currentDeck().leave(false);
  wizard.common.currentDeck().forward(false);
  if (!wizard.common.hasError(false)) {
    var args = new Object();
    args['title'] = wizard.configuration_name;
    args['style'] = 'background-color: white !important;';
    args['text'] = wizard.common.configToYaml();
    var d = new dock.cvDialog(dock.myVersion + "showtext.xul", args);
    dock.dialogs.showDialog(d);
  }
  else {
    return;
  }
};

wizard.common.saveConfiguration = function() {
  wizard.common.resetErrors();
  if (wizard.currentStage.name !== 'deploy' &&
      wizard.currentStage.name !== 'summary') {
    wizard.common.currentDeck().leave(false);
    wizard.common.currentDeck().forward(false);
  }
  if (!wizard.common.hasError(false)) {
    var file ='';
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    fp.init(window, "Save OpenStack setup as", nsIFilePicker.modeSave);
    var path = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
    path.initWithPath(dock.BASE_PATH);
    fp.displayDirectory = path;
    fp.appendFilter("OpenStack setups", "*.yaml");
    var res = fp.show();
    if ((res == nsIFilePicker.returnOK) || (res == nsIFilePicker.returnReplace)) {
      file = fp.file.path;
      if (!endswith(file, '.yaml')) {
        file += '.yaml';
      }
    } else {
      return;
    }

    savefile(file, wizard.common.configToYaml());
    wizard.common.setLastConfigSaved();
  }
  else {
    return ;
  }
};
*/
/* 

wizard.common.setLastConfigSaved = function() {
  wizard.lastConfigSaved = true;
  //wizard.common.disable('saveConfigButton');
};

wizard.common.unsetLastConfigSaved = function() {
  wizard.lastConfigSaved = false;
  //wizard.common.enable('saveConfigButton');
};

*/

wizard.common.help = function() {
  var args = new Object();
  args['message'] = wizard.common.currentDeck().getHelp();
  var d = new cvDialog("chrome://cmgui/content/openstack/install/help.xul", args);
  dock.dialogs.showDialog(d);
};

wizard.common.getOSFlavor = function() {
  var osFlavor = 'unknown';
  var masterNodes = current.find('MasterNode');
  if(masterNodes.length>0 && masterNodes[0].sysInfo.osFlavor!==undefined) {
    osFlavor = masterNodes[0].sysInfo.osFlavor;
  } else if(masterNodes.length>0 && masterNodes[0].sysInfo && masterNodes[0].sysInfo.revision!=='') {
    osFlavor = masterNodes[0].sysInfo.revision;
  }
  return osFlavor;
};

wizard.common.setAdvancedMode = function(option) {
    var choice = (option) ? 1 : 0;
    wizard.main.setAdvancedMode(choice);
    wizard.advancedMode = option;
};


wizard.validator = {
  isValidIP: function (ip, errorElementId, automatic) {
    if (!ipaddr.isValid(ip)) {
      wizard.common.displayError(errorElementId, automatic);
      wizard.common.currentDeck().error = true;
      return false;
    }
    wizard.common.hide(errorElementId);
    return true;
  },

  isValidPort: function(str, errorElementId, automatic) {
    // at the moment only decimal port numbers supported
    var reg = new RegExp('^[0-9]+$');
    if(reg.test(str)) {
      return true;
    }
    else {
      wizard.common.displayError(errorElementId, automatic);
      wizard.common.currentDeck().error = true;
      return false;
    }
  },

  isValidUnsignedInt : function(str, errorElementId, automatic) {
    var reg = new RegExp('^[0-9]+$');
    if(reg.test(str)) {
      return true;
    }
    else {
      wizard.common.displayError(errorElementId, automatic);
      wizard.common.currentDeck().error = true;
      return false;
    }
  },

  isInSelectedNetwork: function (addr, netaddr, errorElementId, automatic) {
    var range = ipaddr.parse(netaddr.network());
    if (netaddr === null || !addr.match(range, netaddr.netmaskbits)) {
      wizard.common.displayError(errorElementId, automatic);
      wizard.common.currentDeck().error = true;
      return false;
    }
    wizard.common.hide(errorElementId);
    return true;
  },

  isValidIPRange: function (addrStart, addrEnd, errorElementId, automatic) {
    if (addrStart.toInt() >= addrEnd.toInt()) {
      wizard.common.displayError(errorElementId, automatic);
      wizard.common.currentDeck().error = true;
      return false;
    }
    wizard.common.hide(errorElementId);
    return true;
  },
};
