/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var cluster;
var current;
var object;

var singular = '';
var plural = '';


function ListingController(chromeWindow) {
  this.dock = null;
  this.object = null;
  this.subobject = null;
  this.myDialog = null;
  this.chromeWindow = chromeWindow;
  this.tree = null;
  this.data = new Array();
  this.editor = {};
  this.currentEditorKey = null;
  this.originalObjectData = null;

  this.getField = function() {
    // Needs to be implemented by child class
  };

  this.getNewDataObject = function() {
    // Needs to be implemented by child class
  };

  this.getEditorRule = function(key) {
    // Needs to be implemented by child class
  };

  this.getListing = function(object) {
    // Needs to be implemented by child class
  };

  this.onLoad = function() {
    println('.Load: ' + document.title);

    this.initDock();
    this.initMyDialog();
    this.initObject();
    this.makeCopyOfCurrentObjectData();

    this.initNodeElementTree();
    this.populateTable();
  };

  this.refresh = function() {
    this.initNodeElementTree();
    this.populateTable();
  };

  this.initDock = function() {
    this.dock = getDock();
    dock = this.dock;
  };

  this.initMyDialog = function() {
    var s = this.chromeWindow.name.indexOf('[');
    var e = this.chromeWindow.name.indexOf(']');
    var index = this.chromeWindow.name.substr(s + 1, e - s - 1);
    this.myDialog = this.dock.dialogs.list[index];
  };

  this.initObject = function() {
    this.object = this.myDialog.arguments.object;
    this.subobject = this.myDialog.arguments.subobject || this.object;
    object = this.object;
  };

  this.makeCopyOfCurrentObjectData = function() {
    this.originalObjectData = this.subobject.old(false, true);
  };

  this.refreshBoardForSelection = function() {
    if(this.select.length === 1) {
      this.displayBottomBoard();
      this.initEditor(this.select[0]);
    } else {
      if(this.select.length > 1) {
        dock.displayError('Please select only one ' + this.api + ' (' + this.select.length + ')');
      }
      this.hideBottomBoard();
    }
  };

  this.initEditor = function(key) {
    //Key hack
    this.currentEditorKey = key;
    if(this.editor[key]===undefined) {
      this.editor[key] = new EditorController(this.chromeWindow, this.getEditorRule(key));
    }
    this.editor[key].onLoad();
  };

  this.initNodeElementTree = function() {
    this.tree = document.getElementById('tree');
  };

  this.populateTable = function() {
    var list = this.getListing(this.subobject);

    this.data = new Array();

    for(var i = 0; i < list.length; i++) {
      if(list[i].toBeRemoved===false) {
        this.pushToTableData(list[i]);
      }
    }

    loadTable(this.tree, this.data);
  };

  this.createTreeItem = function(one) {
    var item = new Object();
    for(var attribute in one) {
      if(attribute === 'uniqueKey') {
        item['key'] = one['uniqueKey'];
      } else {
        item['l_' + attribute] = one[attribute];
      }
    }
    return item;
  };

  this.pushToTableData = function(element) {
    this.data.push(this.createTreeItem(element));
  };

  this.displayBottomBoard = function() {
    var subtabbox = document.getElementById('subtabbox');
    if(subtabbox) {
      subtabbox.removeAttribute('hidden');
    }
  };

  this.hideBottomBoard = function() {
    var subtabbox = document.getElementById('subtabbox');
    if(subtabbox) {
      subtabbox.setAttribute('hidden', true);
    }
  };

  this.addObject = function() {
    var dataobject = this.getNewDataObject();
    dataobject.uniqueKey = ++this.object.cluster.nextLocalKey;
    this.subobject.addSubObject({'field': this.getField(), 'object': dataobject});
    this.refresh();
    this.clearSelected();
    this.setSelected([dataobject.uniqueKey]);
  };

  this.removeObject = function() {
    // Needs to be implemented by child class
  };


  this.saveList = function() {
    if(!this.editor.editorRule.inputCheck(this.editor)) {
      return;
    }
    this.object.commit();
  };

  this.revertFinalize = function() {
    // Needs to be implemented by child class
  };

  this.revertList = function() {
    this.object.refresh(this.revertFinalize.bind(this));
    this.clearSelected();
    return;
  };

  this.selectObject = function(event) {
    this.initSelected();
    this.refreshBoardForSelection();
  };

  this.initSelected = function() {
    this.select = this.getSelected();
  };

  this.clearSelected = function() {
    clearSelected('list');
    this.initSelected();
  };

  this.setSelected = function(select) {
    setSelected('list', select);
  };

  this.getSelected = function getSelected() {
    return selected('list');
  };

  this.ok = function(){
    if(!window.closed) {
      for(var i in this.editor) {
        if(!this.editor[i].editorRule.inputCheck(this.editor[i])) {
          return;
        }
      }
      this.subobject.set('modified', true);
      this.object.set('modified', true);
      if (this.onOk) {
        this.onOk();
      }
      this.myDialog.doOk();
      close();
    }
  };

  this.cancel = function() {
    if(!window.closed) {
      this.subobject.old(this.originalObjectData);
      if (this.onCancel) {
        this.onCancel();
      }
      this.myDialog.doCancel();
      close();
    }
  };

  this.editOk = function() {
    if(!this.editor[this.currentEditorKey].editorRule.inputCheck(this.editor[this.currentEditorKey])) {
      return;
    }
    this.refresh();
    this.clearSelected();
    if (this.editor[this.currentEditorKey].editorRule.onEditOk) {
      this.editor[this.currentEditorKey].editorRule.onEditOk();
    }
  };

  this.editCancel = function() {
    this.editor[this.currentEditorKey].currentObject.old(this.editor[this.currentEditorKey].originalObjectData);
    this.refresh();
    this.clearSelected();
    if (this.editor[this.currentEditorKey].editorRule.onEditCancel) {
      this.editor[this.currentEditorKey].editorRule.onEditCancel();
    }
  };

  this.sortColumn = function(element) {
    sort(element, this.tree, this.data);
  };
};

/*
 * WRAPPERS
 */
function onLoad() {
  listing.onLoad();
}

function refresh(use) {
  listing.refresh(use);
}

function addObject() {
  listing.addObject();
}

function editObject() {
  listing.editObject();
}

function cloneObject() {
  listing.cloneObject();
}

function removeObject() {
  listing.removeObject();
}

function saveList() {
  listing.saveList();
}

function revertList() {
  listing.revertList();
}

function selectObject() {
  listing.selectObject();
}

function getSelected() {
  listing.getSelected();
}

function sortColumn(element) {
  listing.sortColumn(element);
}
