/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function EditorRuleNetwork() {
  this.addTitle = "Add Network";
  this.editTitle = "Edit Network";
  this.api = 'Network';

  this.getCurrentObject = function(editorController) {
    if(editorController.isAdd()) {
      var cvDataOpenStackApiNetwork = new editorController.dock.cvOpenStackApiNetwork();
      cvDataOpenStackApiNetwork.ownerUniqueKey = editorController.object.get('uniqueKey');
      return new editorController.dock.cvObject(editorController.object.cluster, cvDataOpenStackApiNetwork, 0, 0, false);
    } else {
      return editorController.object.cluster.find(editorController.myDialog.arguments.subobject);
    }
  };

  this.dataBindingAndCheck = function(editorController) {
    this.dataBindingAndCheckCommon(editorController);

    resolve('projectKey', 'OpenStackApiProject');
    extraeval('projectKey', 'current.correctAllUUID();');
    autorefresh('projectKey');

    var list = editorController.currentObject.cluster.find('OpenStackApiSubnet');

    var objectDualViewAttributeListable = new ObjectDualViewAttributeListable(list, editorController.currentObject, 'subnetKeys');
    objectDualViewAttributeListable.setColumnOrder({
      'name': 0
    });
    objectDualViewAttributeListable.initInputValue();

    var customListDataViewMapper = new CustomDualListDataViewMapper(objectDualViewAttributeListable);
    customListDataViewMapper.setElementsConfig([
      [
        {'type': 'label', 'attributes': [{'name': 'width', 'value': '100px'}], 'events': []}
      ]
    ]);

    var groupSelectorWidget = new GroupSelectorWidget({
      'context': document.getElementById('subnetKeys'),
      'rootNodeId': 'subnetSelected',
      'rootNodeIdSelection': 'subnetSelection',
      'rootNodeIdAdd': 'subnetButtonAdd',
      'rootNodeIdDel': 'subnetButtonDel',
      'dataViewMapper': customListDataViewMapper
    });
    groupSelectorWidget.populate();

    groupSelectorWidget.addItemCallBack = function() {
      current.correctAllUUID();
    };
    groupSelectorWidget.delItemCallBack = function() {
      current.correctAllUUID();
    };

    setAttributeRecursive(document.getElementById('subnetKeys'), 'disabled', 'true');

    text('status', '^');
    if(editorController.isAdd()) {
      var allNetworks = editorController.object.cluster.find('Network');
      for (var i = 0; i < allNetworks.length; i++) {
        if (allNetworks[i].get('openStackType')==='VXLANHOST') {
          editorController.currentObject.set('providerNetworkType', 'vxlan');
          break;
        } else if (allNetworks[i].get('openStackType')==='VXLANHOST') {
          editorController.currentObject.set('providerNetworkType', 'vlan');
          break;
        }
      }
    }

    editableKenum(editorController.currentObject, 'providerNetworkType', 'providerNetworkType', [
      // ['local', 'Local'],
      ['flat', 'Flat'],
      ['vlan', 'VLAN'],
      ['vxlan', 'VXLAN']
      // ['gre', 'GRE']
    ]);
    addViewConditional({
      'type': 'enum',
      'elementContext': document,
      'elementId': 'providerNetworkType',
      'callName': 'checkProviderNetworkTypeProviderPhysicalNetwork',
      'initialValue': editorController.currentObject.get('providerNetworkType'),
      'evalFunc': function(key) {
        return (key!=='local' && key!=='vxlan' && key!=='gre') ? false : true;
      }
    });
    addViewConditional({
      'type': 'enum',
      'elementContext': document,
      'elementId': 'providerNetworkType',
      'callName': 'checkProviderNetworkTypeProviderSegmentationID',
      'initialValue': editorController.currentObject.get('providerNetworkType'),
      'evalFunc': function(key) {
        return (key!=='local' && key!=='flat') ? false : true;
      }
    });

    text('providerPhysicalNetwork', '^');
    text('providerSegmentationID', '^');
    check('routerExternal');
    check('shared');
    check('adminStateUp');
  };

  this.inputCheck = function(editorController) {
    if(blank(editorController.currentObject.get('name'))) {
      editorController.dock.displayError("Name cannot be empty");
      return false;
    }
    return true;
  };
}

EditorRuleNetwork.prototype = new EditorRuleBase();
EditorRuleNetwork.prototype.constructor = EditorRuleNetwork;
