/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var cluster;
var current;

var singular = '';
var plural = '';


function ListingController() {
  this.loaded = false;
  this.container = '';
  this.api = '';
  this.basetype = '';
  this.filter = 0;
  this.alsoRemoved = false;
  this.modified = false;
  this.select = null;
  this.subtabs = 0;
  this.parentKey = 0;
  this.use = null;
  this.upperboard = false;

  this.confirmManyServersResponse = false;
  this.confirmTerminateServersResponse = false;

  this.onLoad = function() {
    dock = getDock();
    println('.Load: ' + document.title + ' - ' + document.documentURI);

    this.readInputParams();
    this.setPageTitle();

    this.initNodeElementTree();
    this.createColumns();

    this.initButtons();

    if(this.isBottomBoard()) {
      this.hideButtons();

//      if(window.parent.window.current) {
//        refresh(window.parent.window.current);
//      } else {
//        refresh(window.parent.window.cluster);
//      }
      this.refresh(window.parent.window.current);
    }
    this.loaded = true;
    this.refreshIfRefreshPending();
  };

  this.refresh = function(use) {
    if(!this.canRefresh(use)) {
      return;
    }

    println('Refresh: ' + document.title + ' - ' + document.documentURI);
    if(!!use) {
      if(use) {
        current = use;
      }
      if(!!current.cluster) {
        cluster = current.cluster;
      } else {
        cluster = use;
      }
      current = cluster.find('OpenStack'); // TODO support multiple OpenStack
      current = current[0];
      cluster.fakeOpenStackApiSettings(current);
    }


    if(current) {
      this.parentKey = current.get('uniqueKey');
    }

    this.initSelected();
    this.resetObjectModified();
    this.populateTable();
    this.setShadowTabHighlight();

    this.initButtonSave();

    this.refreshBoardForSelection();

    return true;
  };

  this.setPageTitle = function() {
    var title = '';
    if(this.container !== '') {
      title = this.container;
    }
    if(this.api !== '') {
      title += ' - ' + this.api;
    }
    if(title !== '') {
      document.title = title;
    }
  };

  this.setShadowTabHighlight = function () {
    if(data.length==0) {
      parent.document.getElementById('shadowtab-' + this.api).setAttribute('empty', 'true');
    } else {
      parent.document.getElementById('shadowtab-' + this.api).removeAttribute('empty');
    }
  };

  this.refreshIfRefreshPending = function() {
    if (this.use) {
      println('Refresh called with use while loading. Refreshing now. ' + document.title + ' - ' + document.documentURI);
      this.refresh(this.use);
      this.use = null;
      //this.refresh(window.parent.window.current);
    }
  };

  this.canRefresh = function(use) {
    if(!!use && !this.loaded) {
      println('Refresh called with use but page is not loaded yet. ' + document.title + ' - ' + document.documentURI);
      this.use = use;
      return false;
    }
    return true;
  };

  this.readInputParams = function() {
    var params = parseUrlParams(document.documentURI);
    this.container = params['container'];
    this.api = params['api'];
    this.upperboard = (params['upperboard']==='1') ? true : false;
    if(this.container === 'nocontainer') {
      this.basetype = this.api;
    } else {
      this.basetype = 'OpenStackApi' + this.api;
    }

    //println('Openstack param container: ' + this.container + ', api: ' + this.api);

    if(params.hasOwnProperty('filter')) {
      //println('Openstack params hasOwnProperty');
      try {
        this.filter = JSON.parse(params['filter']);
        //printall(this.filter);
      } catch(e) {
        this.filter = 1;
        println('Illegal filter: ' + params['filter']);
        println(e);
      }
    }
  };

  this.isUpperBoard = function () {
    return this.upperboard;
  };

  this.isBottomBoard = function () {
    return !this.isUpperBoard();
  };

  this.refreshBoardForSelection = function() {
    if(this.isUpperBoard()) {
      if(this.select.length === 1) {
        //println('=== SELECT ' + this.select[0] + ' ===');
        this.displayBottomBoard(this.select[0]);
      } else {
        this.hideBottomBoard();
      }
      this.initButtons();
    }
  };

  this.isFromCurrentInstance = function(element) {
    if(this.container !== 'nocontainer' && this.parentKey && this.parentKey !== element.getParentKey()) {
      return false;
    } else {
      return true;
    }
  };

  this.isElementVisible = function(element) {
    var visible = true;
    if(isObject(this.filter)) {
      for(var f in this.filter) {
        var fieldValue = element.get(f);
        if(isArray(this.filter[f])) {
          visible = visible && (this.filter[f].find(fieldValue)>=0);
        } else if(isArray(fieldValue)) {
          visible = visible && (fieldValue.find(this.filter[f])>=0);
        } else if(typeof(fieldValue) === 'string') {
          visible = visible && fieldValue.search(RegExp('^' + this.filter[f] + '$')) !== -1;
        } else {
          visible = visible && (fieldValue === this.filter[f]);
        }

        //println('this.filter: ' + element.get('name') + ' => ' + f + ' : ' + fieldValue + ' == ' + this.filter[f] + ' visible:' + visible);
      }
    }
    return visible;
  };

  this.initNodeElementTree = function() {
    tree = document.getElementById('tree');

    var instance = this;
    tree.addEventListener('row-count', function (event) {
      instance.updateTotalItems();
    }, false);
  };

  this.createColumns = function() {
    var cols = tree.firstChild;
    if(cols.childNodes.length <= 2) {
      var columns = getColumnsFor(this.api);
      for(var i = 0; i < columns.length; i++) {
        var col = document.createElement('treecol');
        col.setAttribute('class', 'generictreecol');
        col.setAttribute('onclick', 'sort(this);');
        col.setAttribute('sortActive', 'true');
        col.setAttribute('sortDirection', 'ascending');
        col.setAttribute('filter', 'textbox');
        for(var j in columns[i]) {
          col.setAttribute(j, columns[i][j]);
          if(j==='type' && columns[i][j]==='checkbox') {
            col.setAttribute('filter', 'checkbox');
          }
        }
        cols.appendChild(col);
        var splitter = document.createElement('splitter');
        splitter.setAttribute('class', 'tree-splitter');
        splitter.setAttribute('resizeafter', 'farthest');
        cols.appendChild(splitter);
      }
    }
  };

  this.formatters = {
    'OpenStackApiFlavor': function(attribute, value) {
      if(attribute==='disk' || attribute==='ephemeral') {
        return formatter.format.GiB(value);
      } else if(attribute==='swap' || attribute==='ram') {
        return formatter.format.MiB(value);
      } else if(attribute==='extraSpecs') {
        return JSON.stringify(value);
      }
      return value;
    },
    'OpenStackApiHostAggregate': function(attribute, value) {
      if(attribute==='nodes') {
        return cluster.resolveList(value);
      }
      return value;
    },
    'OpenStackApiHypervisor': function(attribute, value) {
      if(attribute==='freeDisk' || attribute==='freeRam' || attribute==='localStorage' || attribute==='localStorageUsed' || attribute==='memory' || attribute==='memoryUsed') {
        return formatter.format.DynamiciB(value);
      }
      return value;
    },
    'OpenStackApiImage': function(attribute, value) {
      if(attribute==='minDisk') {
        return formatter.format.GiB(value);
      } else if(attribute==='minRam') {
        return formatter.format.MiB(value);
      } else if(attribute==='size') {
        return formatter.format.DynamiciB(value);
      }
      return value;
    },
    'OpenStackApiPort' : function(attribute, value) {
      if(attribute === 'ips') {
        var temp = [];
        for(var i = 0; i < value.length; i++) {
          temp.push(value[i]['IP']);
        }
        return temp.join(', ');
      }
      return value;
    },
    'OpenStackApiProject': function(attribute, value) {
      if(attribute==='quotaRam') {
        return formatter.format.MiB(value);
      } else if(attribute==='quotaInjectedFileContent') {
        return formatter.format.iB(value);
      } else if(attribute==='quotaTotalSizeOfVolumesAndSnapshots') {
        return formatter.format.GiB(value);
      }
      return value;
    },
    'OpenStackApiSecurityGroup' : function(attribute, value) {
      if(attribute === 'rules') {
        var temp = [];
        for(var i = 0; i < value.length; i++) {
          temp.push('{' + value[i]['direction'] + ' - ' + value[i]['etherType'] + ' - ' + value[i]['portRangeMin'] + ' - ' + value[i]['portRangeMax'] + ' - ' + value[i]['protocol'] + '}');
        }
        return temp.join(', ');
      }
      return value;
    },
    'OpenStackApiSubnet' : function(attribute, value) {
      if(attribute === 'allocationPools') {
        var temp = [];
        for(var i = 0; i < value.length; i++) {
          temp.push('{' + value[i]['start'] + ' - ' + value[i]['end'] + '}');
        }
        return temp.join(', ');
      }
      return value;
    },
    'OpenStackApiVolume': function(attribute, value) {
      if(attribute==='size') {
        return formatter.format.GiB(value);
      }
      return value;
    },
    'OpenStackApiVolumeSnapshot': function(attribute, value) {
      if(attribute==='size') {
        return formatter.format.GiB(value);
      }
      return value;
    },
    'OpenStackApiVolumeType' : function(attribute, value) {
      if(attribute==='extraSpecs') {
        return JSON.stringify(value);
      }
      return value;
    }
  };

  this.updateTotalItems = function() {
    var count = 0;
    if(tree.view && tree.view.rowCount) {
      count = tree.view.rowCount - 1;
    }
    if (count===1) {
      var msg = 'Displaying ' + count + ' item';
    } else {
      var msg = 'Displaying ' + count + ' items';
    }
    document.getElementById('tableTotalItems').setAttribute('value', msg);
  };

  this.populateTable = function() {
    var list = cluster.find(this.basetype);
    println('populateTable, basetype: ' + this.basetype + ', list lenght ' + list.length + ', parentKey: ' + this.parentKey);
    data = new Array();

    for(var i = 0; i < list.length; i++) {
      if(!this.isFromCurrentInstance(list[i])) {
        continue;
      }
      if(this.isElementVisible(list[i])) {
        this.pushToTableData(list[i]);
        if(!this.isAnyObjectModified()) {
          this.checkSetIfModified(list[i]);
        }
      }
    }

    //TODO change to loadTable(this.tree, this.data);
    loadTable();
    setSelected('list', this.select);
    this.initSelected();
  };

  this.createTreeItem = function(one) {
    var item = new Object();
    for(var attribute in one.object) {
      if(attribute === 'uniqueKey') {
        item['key'] = one.get('uniqueKey');
      } else if(attribute === 'UUID') {
        item[attribute] = one.get(attribute);
      } else if(endswith(attribute, 'UUID') && one.object.hasOwnProperty(attribute.substr(0, attribute.length - 4) + 'Key')) {
        item[attribute] = one.resolveNameForUUID(attribute);
      } else if(endswith(attribute, 'UUIDs')) {
        item[attribute] = one.resolveNameForUUIDs(attribute);
      } else if(attribute === 'addedTimestamp' || attribute === 'lastUpdateFromOpenStackTimestamp' || attribute === 'lastUpdateFromBrightTimestamp') {
        var timestamp = one.get(attribute);
        if(timestamp>0) {
          var d = new Date(timestamp * 1000);
          item[attribute] = d.format('dd/mmm/yyyy HH:MM');
        } else {
          item[attribute] = 'never';
        }
      } else {
        try {
          item[attribute] = this.formatters[this.basetype](attribute, one.get(attribute));
        } catch (err) {
          item[attribute] = one.get(attribute);
        };
      }
    }
    return item;
  };

  this.pushToTableData = function(element) {
    data.push(this.createTreeItem(element));
  };

  this.resetObjectModified = function() {
    this.modified = false;
  };

  this.isAnyObjectModified = function() {
    return this.modified;
  };

  this.checkSetIfModified = function(element) {
    if(!element.get('toBeRemoved') || this.alsoRemoved) {
      this.modified = this.modified || element.get('modified') || element.get('toBeRemoved');
    } else {
      this.modified = true;
    }
  };

  this.initButtonSave = function() {
    if(this.isAnyObjectModified()) {
      document.getElementById('save-list').removeAttribute('disabled');
    } else {
      document.getElementById('save-list').setAttribute('disabled', true);
    }
  };

  this.hideBottomBoard = function() {
    var subtabbox = document.getElementById('subtabbox');
    if(subtabbox) {
      subtabbox.setAttribute('hidden', true);
    }
  };

  this.unhideBottomBoard = function() {
    var subtabbox = document.getElementById('subtabbox');
    if(subtabbox) {
      subtabbox.setAttribute('hidden', false);
    }
  };

  this.initButtons = function() {
    if(this.isEditable()) {
      document.getElementById('editObjectButton').removeAttribute('disabled');
      document.getElementById('editObjectPopup').removeAttribute('hidden');
    } else {
      document.getElementById('editObjectButton').setAttribute('disabled', true);
      document.getElementById('editObjectPopup').setAttribute('hidden', true);
    }

    if(this.basetype==='OpenStackApiAgent') {
      setActionsToEditOnly();
      changeActionEditToView();
    } else if(this.basetype==='OpenStackApiFlavor') {
      document.getElementById('showDeletedFlavor').removeAttribute('hidden');
    } else if(this.basetype==='OpenStackApiFloatingIP') {
      setActionsToEditOnly();
      changeActionEditToView();
    } else if(this.basetype==='OpenStackApiHypervisor') {
      setActionsToEditOnly();
      changeActionEditToView();
    } else if(this.basetype==='OpenStackApiImage') {
      setActionsToEditOnly();
      changeActionEditToView();
    } else if(this.basetype==='OpenStackApiProject') {
      document.getElementById('topology').removeAttribute('hidden');
    } else if(this.basetype==='OpenStackApiNetwork') {
      document.getElementById('topology').removeAttribute('hidden');
    } else if(this.basetype==='OpenStackApiPort') {
      document.getElementById('topology').removeAttribute('hidden');
    } else if(this.basetype==='OpenStackApiRouter') {
      document.getElementById('topology').removeAttribute('hidden');
    } else if(this.basetype==='OpenStackApiServer') {
      disableActionAdd();
      disableActionClone();
      document.getElementById('powerServerButton').removeAttribute('hidden');
      document.getElementById('viewServerButton').removeAttribute('hidden');
      document.getElementById('serverPopup').removeAttribute('hidden');
      document.getElementById('topology').removeAttribute('hidden');
    } else if(this.basetype==='OpenStackApiSecurityGroup') {
      setActionsToEditOnly();
      changeActionEditToView();
    } else if(this.basetype==='OpenStackApiSubnet') {
      document.getElementById('topology').removeAttribute('hidden');
    } else if(this.basetype==='OpenStackApiVolume') {
      setActionsToEditOnly();
    } else if(this.basetype==='OpenStackApiVolumeSnapshot') {
      setActionsToEditOnly();
    }
  };

  function setActionsToEditOnly() {
    disableActionAdd();
    disableActionClone();
    disableActionRemove();
  }

  function changeActionEditToView() {
    document.getElementById('editObjectButton').setAttribute('label', 'View');
    document.getElementById('editObjectPopup').setAttribute('label', 'View');
  }

  function disableActionAdd() {
    document.getElementById('addObjectButton').setAttribute('disabled', true);
  }

  function disableActionClone() {
    document.getElementById('cloneObjectButton').setAttribute('disabled', true);
    document.getElementById('cloneObjectPopup').setAttribute('hidden', true);
  }

  function disableActionRemove() {
    document.getElementById('removeObjectButton').setAttribute('disabled', true);
    document.getElementById('removeObjectPopup').setAttribute('hidden', true);
  }

  this.isEditable = function() {
    if(this.select!==null && this.select.length > 1 && this.basetype==='OpenStackApiEndpoint') {
      return true;
    } else if(this.select!==null && this.select.length === 1) {
      return true;
    } else {
      return false;
    }
  };

  this.hideButtons = function() {
    document.getElementById('buttons').setAttribute('hidden', true);
  };

  this.createBottomEmptyTabStructure = function() {
    var subtabbox = document.createElement('tabbox');
    subtabbox.setAttribute('id', 'subtabbox');
    subtabbox.setAttribute('flex', 1);
    var arrowscrollbox = document.createElement('arrowscrollbox');
    arrowscrollbox.setAttribute('orient', 'horizontal');
    arrowscrollbox.setAttribute('class', 'subtabscroll');
    subtabbox.appendChild(arrowscrollbox);
    var tabs = document.createElement('tabs');
    tabs.setAttribute('flex', 1);
    arrowscrollbox.appendChild(tabs);
    var panels = document.createElement('tabpanels');
    subtabbox.appendChild(panels);
    panels.setAttribute('flex', 1);
    document.getElementById('mainbox').appendChild(subtabbox);
  };

  this.getBottomTabsNode = function() {
    var subtabbox = document.getElementById('subtabbox');
    return subtabbox.firstChild.firstChild;
  };

  this.getBottomTabPanelsNode = function() {
    var subtabbox = document.getElementById('subtabbox');
    return subtabbox.lastChild;
  };

  this.displayBottomBoard = function(key) {
    //println('displayBottomBoard');
    key = parseInt(key);
    if(!this.existsBottomBoard()) {
      this.createBottomEmptyTabStructure();
    } else {
      this.unhideBottomBoard();
    }

    println('api=' + this.api + ', key=' + key);
    this.subtabs = getSubTabsFor(cluster, this.api, key);
    if(window.frames.length === 0) {
      this.loadFrames();
    } else {
      this.refreshFrames();
    }
  };

  this.existsBottomBoard = function() {
    var subtabbox = document.getElementById('subtabbox');
    return (subtabbox);
  };

  this.loadFrames = function() {
    var tabs = this.getBottomTabsNode();
    var panels = this.getBottomTabPanelsNode();
    //println('Make new frames: ' + this.subtabs.length + ' ' + window.frames.length);
    for(var i = 0; i < this.subtabs.length; i++) {
      this.addShadowTab(tabs, this.subtabs[i]);
      this.addIFrame(panels, this.subtabs[i]);
    }
    var subtabbox = document.getElementById('subtabbox');
    if(this.subtabs.length) {
      subtabbox.removeAttribute('hidden');
      if(subtabbox.selectedIndex < 0) {
        subtabbox.selectedIndex = 0;
      }
    } else {
      subtabbox.setAttribute('hidden', true);
    }
  };

  this.refreshFrames = function() {
    println('Refresh all frames: ' + this.subtabs.length + ' ' + window.frames.length);
    for(var i = 0; i < this.subtabs.length; i++) {
      if(typeof(window.frames[i].window.refresh) === 'function') {
        window.frames[i].window.listing.filter = this.subtabs[i].filter;
        window.frames[i].window.refresh();
      } else {
        println('Frames[' + i + '/' + this.subtabs.length + '] has no refresh (yet?)');
      }
    }
  };

  this.addShadowTab = function(node, subtab) {
    var shadowtab = document.createElement('shadowtab');
    shadowtab.setAttribute('label', subtab.title);
    shadowtab.setAttribute('id', 'shadowtab-' + subtab.api);
    node.appendChild(shadowtab);
  };

  this.addIFrame = function(node, subtab) {
    var iframe = document.createElement('iframe');
    iframe.setAttribute('src', 'chrome://cmgui/content/openstack/openstackapi.xul?container=' + subtab.container + '&api=' + subtab.api + '&upperboard=0&filter=' + JSON.stringify(subtab.filter));
    iframe.setAttribute('id', 'iframe-' + subtab.api);
    node.appendChild(iframe);
  };

  this.gotoUpperBoard = function(select) {
    var w = parent.parent.parent.window;
    var frames = w.frames;
    //println('gotoUpperBoard, #frames = ' + frames.length);
    for(var i = 0; i < frames.length; i++) {
      if (frames[i].window.container === this.container) {
        for(var j = 0; j < frames[i].frames.length; j++) {
          if(frames[i].frames[j].window.listing.basetype === this.basetype) {
            //println('=== GOTO ' + this.container + ', '+ this.basetype + '(' + i + ', ' + j+ ') ===');
            w.document.getElementById('tabs').selectedIndex = i;
            w.frames[i].document.getElementById('sub-tabs').selectedIndex = j;
            frames[i].frames[j].window.refresh();
            frames[i].frames[j].window.clearSelected('list');
            frames[i].frames[j].window.setSelected('list', [select]);
            break;
          }
        }
        break;
      }
    }
  };

  this.openEditor = function(subobject) {
    var instance = this;
    var args = new Object();
    args['subobject'] = subobject;
    args['object'] = current;

    var d = new dock.cvDialog('chrome://cmgui/content/openstack/' + this.api.toLowerCase() + '/editApi' + ucfirst(this.api) + '.xul', args);
    d.onOk = function() {
      instance.refresh();
    };
    dock.dialogs.show(d);
  };

  this.addObject = function() {
    //println('addObject');
    this.openEditor(-1);
  };

  this.editObject = function() {
    //println('editObject');
    //var select = this.getSelected();
    if(this.select.length === 0) {
      dock.displayError('Please select an entry for editing');
    } else if(this.select.length === 1) {
      if(this.isUpperBoard()) {
        this.openEditor(this.select[0]);
      } else {
        if(this.container === 'nocontainer') {
          cluster.select(this.select[0]);
        } else {
          this.gotoUpperBoard(this.select[0]);
        }
      }
    } else if (this.isEditable()) {
      this.openEditor(this.select);
    }
  };

  this.viewConsoleServer = function() {
    this.openViewer('chrome://cmgui/content/openstack/server/viewConsoleServer.xul', 'Please select a server.');
  };

  this.viewLogServer = function() {
    this.openViewer('chrome://cmgui/content/openstack/server/viewLogServer.xul', 'Please select a server.');
  };

  this.openViewer = function(xulFile, errorMsg) {
    if(this.select.length !== 1) {
      dock.displayError(errorMsg);
    } else {
      var args = new Object();
      args['uniqueKey'] = this.select[0];
      args['cluster'] = cluster;
      var d = new dock.cvDialog(xulFile, args);
      dock.dialogs.showDialog(d);
    }
  };

  this.actionPowerServer = function(action) {
    var select = getSelected();
    if(this.select.length === 0) {
      dock.displayError('Please select at least one server');
    } else {
      if(!this.confirmManyServersResponse && this.select.length > 9) {
        this.confirmManyServers(action);
      } else if(!this.confirmTerminateServersResponse && action==='serverTerminate') {
        this.confirmTerminateServers(action);
      } else {
        this.confirmManyServersResponse = false;
        this.confirmTerminateServersResponse = false;
        this.actionPowerServerDo(action);
      }
    }
  };

  this.confirmManyServers = function(action) {
    var instance = this;

    var args = new Object();
    args['question'] = 'This action might be slow due to the number of selected servers. Do you want to continue?';

    var d = new dock.cvDialog('chrome://cmgui/content/question.xul', args);
    d.onOk = function() {
      instance.confirmManyServersResponse = true;
      instance.actionPowerServer(action);
    };
    dock.dialogs.showDialog(d);
  };

  this.confirmTerminateServers = function(action) {
    var instance = this;

    var args = new Object();
    if(this.select.length === 1) {
      args['question'] = 'Do you really want to terminate this server?';
    } else {
      args['question'] = 'Do you really want to terminate these servers?';
    }

    var d = new dock.cvDialog('chrome://cmgui/content/question.xul', args);
    d.onOk = function() {
      instance.confirmTerminateServersResponse = true;
      instance.actionPowerServer(action);
    };
    dock.dialogs.showDialog(d);
  };

  this.actionPowerServerDo = function(action) {
    var uuids = cluster.findAttribute(this.select, 'UUID');
    dock.waitIcon();
    dock.workerpool.getWorker(cluster.actualurl).call('cmopenstack', 'openStackApiEntityOperation', [[], uuids, 'server', action, []], function(code, data) {
      if(code === 0) {
        // TODO
      } else {
        dock.jsonCallError(this, code, data);
      }
      refresh();
      dock.unwaitIcon();
    }, current);
  };

  this.cloneObject = function() {
    //println('cloneObject');
    var instance = this;

    if(this.select.length === 0) {
      dock.displayError('Please select an entry for cloning');
    } else if(this.select.length > 1) {
      dock.displayError('Please select only one entry');
    } else {
      var original = cluster.find(this.select[0]);
      if(isObject(original)) {
        var args = new Object();
        var originalName = original.get('name');
        args['title'] = 'Clone ' + originalName;
        args['question'] = 'Name';
        args['default'] = this.calcCloneName(originalName);
        args['known'] = cluster.known(this.basetype, 'name');
        var d = new dock.cvDialog('chrome://cmgui/content/newobject.xul', args);
        d.onOk = function(cloneName) {
          var object = original.clone();
          object.modified = true;
          object.toBeRemoved = false;
          object.name = cloneName;
          object.UUID = '';
          cluster.addObject(object, false, false);
          if(instance.isUpperBoard()) {
            instance.refresh();
          } else {
            instance.gotoUpperBoard(object.uniqueKey);
          }
        };
        dock.dialogs.showDialog(d);
      } else {
        dock.displayError('Original not found, already removed by someone else');
      }
    }
  };

  this.calcCloneName = function(originalName){
    var nn = nameNumber(originalName);
    if(nn[1].length === 0) {
      nn[1] = '0';
    }
    var cloneName;
    var index = myParseInt(nn[1]);
    while(true) {
      index++;
      cloneName = nn[0] + sprintf('%0' + nn[1].length + 'd', index);
      if(!cluster.find(this.basetype, cloneName)) {
        break;
      }
    }

    return cloneName;
  };

  this.removeValidator = {
    'OpenStackApiFlavor': function(select) {
      for(var i = select.length - 1; i >= 0; i--) {
        var one = cluster.find(select[i]);
        if(one.object.deleted===true) {
          dock.displayError('Flavor ' + one.object.name + ' has been already removed from the OpenStack list of available flavors but it hasn\'t been permanently deleted because it is in use by a server. After deleting all the servers using this flavor, it will disappear from the list.');
          return false;
        }
      }
      return true;
    },
    'OpenStackApiProject': function(select) {
      for(var i = select.length - 1; i >= 0; i--) {
        var one = cluster.find(select[i]);
        if(one.object.UUID===current.object.settingsCredentials.CMDaemonOpenStackProjectUUID) {
          dock.displayError('Project ' + one.object.name + ' is protected from accidental deletion in credential settings by CMDaemon OpenStack project UUID.');
          return false;
        }
        if(current.object.settingsCredentials.CMDaemonOpenStackDomainUUID==='' && (one.object.name===current.object.settingsCredentials.serviceProjectName || one.object.name===current.object.settingsCredentials.CMDaemonOpenStackProjectName)) {
          dock.displayError('Project ' + one.object.name + ' is protected from accidental deletion in credential settings by service project name or CMDaemon OpenStack project name.');
          return false;
        }
        if(current.object.settingsCredentials.CMDaemonOpenStackDomainUUID!=='' && one.object.domainUUID===current.object.settingsCredentials.CMDaemonOpenStackDomainUUID && (one.object.name===current.object.settingsCredentials.serviceProjectName || one.object.name===current.object.settingsCredentials.CMDaemonOpenStackProjectName)) {
          dock.displayError('Project ' + one.object.name + ' is protected from accidental deletion in credential settings by service project name or CMDaemon OpenStack project name for the specific CMDaemon OpenStack domain UUID.');
          return false;
        }
      }
      return true;
    },
    'OpenStackApiUser': function(select) {
      for(var i = select.length - 1; i >= 0; i--) {
        var one = cluster.find(select[i]);
        if(current.object.settingsCredentials.CMDaemonOpenStackDomainUUID==='' && one.object.name===current.object.settingsCredentials.CMDaemonOpenStackUsername) {
          dock.displayError('User ' + one.object.name + ' is protected from accidental deletion in credential settings by CMDaemon OpenStack username.');
          return false;
        }
        if(current.object.settingsCredentials.CMDaemonOpenStackDomainUUID!=='' && one.object.domainUUID===current.object.settingsCredentials.CMDaemonOpenStackDomainUUID && one.object.name===current.object.settingsCredentials.CMDaemonOpenStackUsername) {
          dock.displayError('User ' + one.object.name + ' is protected from accidental deletion in credential settings by CMDaemon OpenStack username for the specific CMDaemon OpenStack domain UUID.');
          return false;
        }
      }
      return true;
    }
  };

  this.checkIfCanRemove = function() {
    var canRemove = true;
    try {
      canRemove = this.removeValidator[this.basetype](this.select);
    } catch (err) {
    };

    return canRemove;
  };

  this.removeObject = function() {
    //println('removeObject');
    var instance = this;

    if(this.select.length === 0) {
      dock.displayError('Please select an entry.');
    } else if(this.checkIfCanRemove()) {

      var args = new Object();
      if(this.select.length === 1) {
        args['question'] = 'Are you sure you want to remove this entry?';
      } else {
        args['question'] = 'Are you sure you want to remove these entries?';
      }

      var d = new dock.cvDialog('chrome://cmgui/content/question.xul', args);
      d.onOk = function() {
        for(var i = instance.select.length - 1; i >= 0; i--) {
          cluster.removeObject(instance.select[i]);
        }
        instance.clearSelected();
        instance.refresh();
      };
      dock.dialogs.showDialog(d);
    }
  };

  this.saveList = function() {
    var list = cluster.find(this.basetype);
    //println('saveList, type: ' + this.basetype + ', # ' + list.length);
    //println('saveList, parentKey: ' + this.parentKey);
    for(var i = 0; i < list.length; i++) {
      if(!this.isFromCurrentInstance(list[i])) {
        continue;
      }
      if(list[i].get('modified')) {
        list[i].commit();
      }
    }
  };

  this.revertList = function() {
    var list = cluster.find(this.basetype);
    //println('revertList, type: ' + this.basetype + ', # ' + list.length);
    //println('revertList, parentKey: ' + this.parentKey);
    for(var i = 0; i < list.length; i++) {
      if(!this.isFromCurrentInstance(list[i])) {
        continue;
      }
      if(list[i].get('modified')) {
        if(list[i].object.uniqueKey < dock.MAX_LOCAL_KEY) {
          list[i].remove();
        } else {
          list[i].refresh();
        }
      } else if(list[i].object.uniqueKey < dock.MAX_LOCAL_KEY) {
        println('list[i].object.uniqueKey < dock.MAX_LOCAL_KEY, This should not be!');
      }
    }
    refresh();
  };

  this.showDeletedFlavor = function() {
    if(this.api==='Flavor') {
      if(document.getElementById('showDeletedFlavor').checked) {
        this.filter = 0;
      } else {
        this.filter = {deleted: false};
      }
      this.refresh();
    }
  };

  this.selectObject = function(event) {
    this.initSelected();
    this.refreshBoardForSelection();
  };

  this.initSelected = function() {
    this.select = this.getSelected();
  };

  this.clearSelected = function() {
    clearSelected('list');
    this.initSelected();
  };

  this.getSelected = function getSelected() {
    return selected('list');
  };

  this.refreshList = function refreshList() {
    dock.waitIcon();
    dock.workerpool.getWorker(cluster.actualurl).call('cmopenstack', this.api + 'ManagerRefreshEntities', [this.parentKey], function(code, data) {
      if(code === 0) {
        // TODO
      } else {
        dock.jsonCallError(this, code, data);
      }
      refresh();
      dock.unwaitIcon();
    }, current);
  };
}

var listing = new ListingController();

/*
 * WRAPPERS
 */
function onLoad() {
  listing.onLoad();
}

function refresh(use) {
  listing.refresh(use);
}

function addObject() {
  listing.addObject();
}

function editObject() {
  listing.editObject();
}

function actionPowerServer(action) {
  listing.actionPowerServer(action);
}

function viewConsoleServer() {
  listing.viewConsoleServer();
}

function showDeletedFlavor() {
  listing.showDeletedFlavor();
}

function viewLogServer() {
  listing.viewLogServer();
}

function cloneObject() {
  listing.cloneObject();
}

function removeObject() {
  listing.removeObject();
}

function saveList() {
  listing.saveList();
}

function revertList() {
  listing.revertList();
}

function selectObject() {
  listing.selectObject();
}

function getSelected() {
  return listing.getSelected();
}

function refreshList() {
  return listing.refreshList();
}

function topology() {
  var instance = this;
  var args = new Object();
  args['object'] = current;

  var d = new dock.cvDialog('chrome://cmgui/content/openstack/topology/topology.xul', args);
  d.onOk = function() {
  };
  dock.dialogs.show(d);
}

//
// TODO make this class a widget in a dedicated file
//
function showTreePopup(event) {
/*
  var popup = event.target;
  while(popup.firstChild) {
    popup.removeChild(popup.firstChild);
  }

  addMenuItem(popup, 'Edit', 'editObject();');
  if(dock.hasClone(current.get('baseType'))) {
    addMenuItem(popup, 'Clone', 'cloneObject();');
  }
  addMenuItem(popup, 'Remove', 'removeObject();');

  return true;

  function addMenuItem(parent, text, command) {
    var item = document.createElement("menuitem");
    item.setAttribute('label', text);
    item.setAttribute('oncommand', command);
    parent.appendChild(item);
  }

  function addMenuSeparator(parent) {
    var item = document.createElement('menuseparator');
    parent.appendChild(item);
  }
*/
}
