/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'VirtualNode';
var singular = 'node';
var plural = 'nodes';

function postrefresh() {
  var openstacks = current.find("OpenStack");
  if (openstacks.length > 0) {
    var openstack = openstacks[0];

    var list = current.find(basetype);
    if(list.length>0) {
      displayForVirtualNodes();
    } else {
      displayForNoVirtualNodes();
    }
  }
}

function displayForNoVirtualNodes() {
  if(document.getElementById('noVirtualNode').hasAttribute('hidden')) {
    document.getElementById('listVirtualNode').setAttribute('hidden', true);
    document.getElementById('noVirtualNode').removeAttribute('hidden');
    document.getElementById('editVirtualNode').setAttribute('disabled', true);
    document.getElementById('cloneVirtualNode').setAttribute('disabled', true);
    document.getElementById('delVirtualNode').setAttribute('disabled', true);
  }
}

function displayForVirtualNodes() {
  if(document.getElementById('listVirtualNode').hasAttribute('hidden')) {
    document.getElementById('listVirtualNode').removeAttribute('hidden');
    document.getElementById('noVirtualNode').setAttribute('hidden', true);
    document.getElementById('editVirtualNode').removeAttribute('disabled');
    document.getElementById('cloneVirtualNode').removeAttribute('disabled');
    document.getElementById('delVirtualNode').removeAttribute('disabled');
    
    resetTree('tree');
  }
}

function createTreeItem(one) {
  var ip = '0.0.0.0';
  var ips = one.getIPs();
  if(ips.length >= 1) ip = ips[0][1];

  var item = new Object();
  item['key'] = one.get('uniqueKey');
  item['modified'] = one.get('modified');
  item['hostname'] = one.get('hostname');
  item['mac'] = one.get('mac');
  item['tag'] = one.get('tag');
  item['category'] = current.resolveName(one.get('category'));
  item['ip'] = ip;
  var sw = one.get('ethernetSwitch');
  if(sw && sw.ethernetSwitch) {
    item['switchPort'] = one.cluster.resolveName(sw.ethernetSwitch) + ':' + sw.prt;
  } else {
    item['switchPort'] = '';
  }
  if(one.get('powerControl').toLowerCase() == 'apc') {
    item['power'] = '';
    var pdu = one.get('powerDistributionUnits');
    for(var i = 0; i < pdu.length; i++) {
      if(item['power'].length > 0) item['power'] += ', ';
      item['power'] += one.cluster.resolveName(pdu[i].pdu) + ':' + pdu[i].prt;
    }
  } else {
    item['power'] = one.get('powerControl');
  }
  var virtualNodesettings = one.get('virtualNodeSettings');
  if(virtualNodesettings) {
    if(virtualNodesettings.childType == 'OpenStackSettings') {
      item['instanceID'] = virtualNodesettings.instanceID;
      if(virtualNodesettings.openStackFlavor) {
        var flavor = one.cluster.find(virtualNodesettings.openStackFlavor);
        if(flavor) {
          if(typeof(flavor.resolveName) == 'function') {
            item['openStackFlavor'] = flavor.resolveName();
          } else {
            println('===== BAD FLAVOR =====');
            println(virtualNodesettings.openStackFlavor);
            printall(flavor);
            println('======================');
          }
        }
      }
    }
  }

  return item;
}

function addObject() {
  var cluster = current;
  var args = new Object();
  args['title'] = 'Add new ' + singular;
  args['question'] = 'Hostname';
  args['known'] = cluster.known('Device', 'hostname');
  args['default'] = 'vnode';
  var d = new dock.cvDialog("chrome://cmgui/content/newobject.xul", args);
  d.onOk = function(name) {
    var object = new dock.cvVirtualNode();
    object.hostname = name;
    var base = cluster.find("Partition", "base");
    if(isObject(base)) {
      object.partition = base.object.uniqueKey;

      var categories = cluster.findBy({
        baseType: 'NodeCategory',
        filter: function(category) {
          if(category.get('name') === 'virtual-nodes') {
            return true;
          }
          return false;
        }
      });
      if(categories && categories[0] && categories[0].get('name') === 'virtual-nodes') {
        object.category = categories[0].get('uniqueKey') ;
      } else {
        object.category = base.object.defaultCategory;
      }
      object.powerControl = 'openstack';
      if(!object.virtualNodeSettings || (object.virtualNodeSettings.childType != 'OpenStackSettings')) {
        object.virtualNodeSettings = new dock.cvOpenStackSettings();
      }
      if(object.virtualNodeSettings.uniqueKey == 0) {
        var tmp = new dock.cvObject(cluster, object.virtualNodeSettings);
      }
      if(object.virtualNodeSettings.childType == 'OpenStackSettings') {
        var flavors = cluster.find('OpenStackApiFlavor');
        var foundFlavor = false;
        for (var i = 0; i < flavors.length; i++) {
          if(flavors[i].get('deleted')!==true && flavors[i].get('ephemeral')>=10737418240) {
            object.virtualNodeSettings.openStackFlavor = flavors[i].get('uniqueKey');
            foundFlavor = true;
            break;
          }
        }
        if(foundFlavor===false) {
          dock.displayError('No flavor with at least 10GiB of ephemeral disk found');
        }
        object.virtualNodeSettings.modified = true;
      }
    }
    var work = cluster.addObject(object, dock.openAfterCreate);
    guessIP(work);
    // Hack to prevent error while launching the instance
    // Kilo issue
    work.object.networks[0].ip = '0.0.0.0';
    work.object.networks[0].dhcp = true;
    // Hack end
    refresh();
  };
  dock.dialogs.showDialog(d);
}

function cloneObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a " + singular + " for cloning");
  } else if(select.length > 1) {
    dock.displayError("Please select only one " + singular);
  } else {
    var cluster;
    if(current.isCluster) {
      cluster = current;
    } else {
      cluster = current.cluster;
    }
    var original = cluster.find(select[0]);
    if(isObject(original)) {
      var args = new Object();
      var host = original.get('hostname');
      args['title'] = 'Clone ' + host;
      args['question'] = 'Hostname';
      var nn = nameNumber(host);
      if(nn[1].length == 0) nn[1] = '0';
      var name;
      var index = myParseInt(nn[1]);
      while(true) {
        index++;
        name = nn[0] + sprintf('%0' + nn[1].length + 'd', index);
        if(!current.find('Device', name)) break;
      }
      args['default'] = name;
      args['known'] = cluster.known('Device', 'hostname');
      var d = new dock.cvDialog("chrome://cmgui/content/newobject.xul", args);
      d.onOk = function(name) {
        var object = original.clone();
        object.modified = true;
        object.toBeRemoved = false;
        object.hostname = name;
        object.mac = '00:00:00:00:00:00';
        object.tag = '00000000a000';
        object.ethernetSwitch = new dock.cvSwitchPort();
        object.powerDistributionUnits = new Array();
        cloneNodeIP(original.object, object);
        cluster.addObject(object, dock.openAfterCreate);
        refresh();
      }
      dock.dialogs.showDialog(d);
    } else {
      dock.displayError("Original not found, already removed by someone else");
    }
  }
}

function createWizard() {
  var args = new Object();
  args['cluster'] = current;
  args['basetype'] = basetype;
  args['openstack'] = current;
  var d = new dock.cvDialog("chrome://cmgui/content/nodewizard.xul", args);
  d.onOk = function() {
    refresh();
  }
  dock.dialogs.showDialog(d);
}
