/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var openstack;
var overview;
var oldcluster = 0;
var graph;
var timer = 0;

var first = true;

function onLoad() {
  dock = getDock();
  addEventListener("resize", onResize, false);
}

function onResize() {
  var svg = document.getElementById("svg");
  svg.setAttribute("height", "25px");
  svg.setAttribute("width", svg.parentNode.boxObject.width);
  svg.setAttribute("height", svg.parentNode.boxObject.height - 128); // 128 ?? KDR
  redraw();
}

function getCurrentSelection() {
  return [[current.find('Partition', 'base')], 0, ""];
}

function stop() {
  if(graph) graph.stop();
  if(timer) window.clearTimeout(timer);
}

function refresh(use) {
  if(!!use) {
    var monconf = use.find(dock.ALL_MASTERNODES);
    if(!monconf) return;
    current = use;
    openstack = use.find('OpenStack');
    if(openstack.length == 1) {
      openstack = openstack[0];
    } else {
      openstack = 0;
    }
  }
  if(!openstack) {
    return;
  }

  if(!!current.cluster) current = current.cluster;
  if(current != oldcluster) {
    setCallbackResult(new dock.cvGuiOpenStackOverview());
    if(graph) {
      graph.cluster = current;
      graph.clearScreen();
    } else {
      graph = new cvGraph(current, document.getElementById("svg"), document.getElementById("box"));
      graph.colors = [[['#7cc300', '#7dc200', '#4d7900']]];
      graph.WM = 32;
      graph.HM = 61;
      graph.legend = -1;
    }
    oldcluster = current;
  }

  setNodeApi();

  var list = document.getElementById("metriclist");
  var oldkey = parseInt(list.getAttribute('metricKey'));

  var name = current.find(oldkey);
  if(name) {
    name = name.resolveName();
    if(!startswith(oldlabel, name)) {
      oldkey = 0;
    }
  }
  var oldobject = parseInt(list.getAttribute('objectKey'));
  if(oldobject && !current.find(oldobject)) oldobject = 0;

  var oldparam = list.getAttribute('param');
  var oldlabel = list.getAttribute('label');
  if(kisNaN(oldkey) || (oldkey == 0)) {
    if(isArray(current.mainMetric)) {
      oldkey = current.mainMetric[0];
      oldparam = current.mainMetric[1];
      oldlabel = '';
      var metric = current.find(current.mainMetric[0]);
      if(metric) {
        var text = metric.resolveName();
        if(current.mainMetric[1].length > 0) text += "[" + current.mainMetric[1].metricParam + "]";
        if(metric.get('unit')) text += " (" + metric.get('unit') + ")";
        oldlabel = text;
      }
    }
  }

  var mmc = monconf.get("metrics");
// function hidefunction(metric) {
//   if(metric.baseType == "MonMetricConf") {
//     return isOpenStackTenantId(metric.metricParam);
//   } else {
//     return isOpenStackTenantId(metric.healthcheckParam);
//   }
// }
  function hidefunction(metric) {
    return isOpenStackTenantId(metric.metricParam);
  }
  createMetricPopup(current, mmc, 'metriclist', false, true, ['CEPH','AMAZONEC2','CLOUD','HADOOP_JVM','HADOOP_DFS','HADOOP_UGI','HADOOP_MAPRED','HADOOP_RPC','HADOOP_RPCDETAILED','HADOOP_METRICSYSTEM','DELLNSS', 'LUSTRE'], hidefunction);

  var found = false;
  for(var i = 0; (i < mmc.length) && !found; i++) {
    found = ((mmc[i].metric == oldkey) && (mmc[i].metricParam == oldparam));
  }

  if(!found) {
    var objectKey = 0;
    if(mmc.length == 0) {
      var item = document.createElement("menuitem");
      item.setAttribute('metricKey', 0);
      item.setAttribute('objectKey', 0);
      item.setAttribute('param', '');
      item.setAttribute('label', '');
      list.firstChild.appendChild(item);
      oldkey = 0;
      oldparam = '';
      oldlabel = '';
    } else {
      oldkey = mmc[0].metric;
      oldparam = mmc[0].metricParam;
      // OpenStackRunningVMsTotal, OpenStackPausedVMsTotal, OpenStackSuspendedVMsTotal - "detailed" numbers
      var def = current.find('Metric', 'OpenStackRunningVMsTotal');
      if(def) {
        def = def.get('uniqueKey');
        for(var i = 0; i < mmc.length; i++) {
          if(mmc[i].metric == def) {
            oldkey = def
            oldparam = mmc[i].metricParam;
            break;
          }
        }
      }
      var metric = current.find(oldkey);
      if(metric) {
        oldlabel = metric.resolveName();
        if(oldparam.length > 0) oldlabel += "[" + oldparam + "]";
        if(metric.get('unit')) oldlabel += " (" + metric.get('unit') + ")";
      }
    }
  }
  list.setAttribute('metricKey', oldkey);
  list.setAttribute('objectKey', oldobject);
  list.setAttribute('param', oldparam);
  list.setAttribute('label', oldlabel);

  graph.clear();
  if(oldobject == 0 || isNaN(oldobject)) oldobject = current.activePassiveKeys[0];
  if(oldobject) {
    if(oldkey) {
      var dmp = new cvGraphDeviceMetric(oldobject, oldkey, oldparam);
      var r = graph.add(dmp);
      if(r < 0) {
        dock.displayError("Too many metrics, only " + -r + " are supported");
      } else if(r != 0) {
        dock.displayError("Incompatible metrics: " + r);
      }
      //if(dock.DEMO) graph.zoom[0] *= 24 * 7;
      redraw();
    } else {
      dock.displayError("No metrics defined");
    }
  } else {
    dock.displayError("Primary head node not defined");
  }
  update();
}

function update() {
  if(!current.connected || !openstack) {
    return;
  }
  dock.workerpool.getWorker(current.actualurl).call('cmgui', 'getOpenStackOverview', [current.sessionid, openstack.get('uniqueKey')], function(code, data) {
    if(code == 0) {
      setCallbackResult(data);
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);
}

function setCallbackResult(value) {
  overview = value;
  if(overview == null) return;

  // Usage
  overviewValue('instancesTotal');
  overviewValue('instancesRunning');
  overviewValue('instancesDown');
  overviewValue('instancesError');
  overviewValue('instancesPaused');

  overviewValue('numberOfTenants');
  overviewValue('numberOfDisabledTenants');

  overviewValue('numberOfUsers');
  overviewValue('numberOfDisabledUsers');

  overviewValue('numberOfImages');
  overviewValue('numberOfPublicImages');

  overviewValue('numberOfFlavors');
  overviewValue('numberOfPublicFlavors');

  overviewValue('numberOfNetworks');
  overviewValue('numberOfPublicNetworks');

  overviewValue('numberOfSubnets');

  overviewValue('numberOfRouters');

  overviewValue('numberOfVolumes');

  overviewValue('numberOfSnapshots');

  overviewValue('numberOfAvailabilityZones');

  overviewValue('numberOfHostAggregates');

  overviewValue('numberOfDomains');

  overviewValue('numberOfRoles');


  // Provisioned Resources
  overviewValue('extra1'); // wasted/misused VCPUs
  overviewValue('vcpuUsed');

  overviewValue('extra2', formatKMG(overview.extra2) + 'B'); // wasted/misused memory
  overviewValue('ramUsed', formatKMG(overview.ramUsed) + 'B');

  overviewValue('extra3', formatKMG(overview.extra3) + 'B'); // wasted/misused volumes
  overviewValue('storageUsed', formatKMG(overview.storageUsed) + 'B');

  overviewValue('extra4', formatKMG(overview.extra4) + 'B'); // wasted/misused local storage
  overviewValue('ephemeralStorageUsed', formatKMG(overview.ephemeralStorageUsed) + 'B');

  overview.numberOfFloatingIPsMisused = overview.numberOfFloatingIPs - overview.numberOfFloatingIPsUsed;
  overviewValue('numberOfFloatingIPsMisused');
  overviewValue('numberOfFloatingIPsUsed');



  // Hypervisor usage
  overviewValue('hypVcpusUsed');
  overviewValue('hypVcpus');
  addStatusBar('hypVcpusStatusBar', overview.hypVcpusUsed, overview.hypVcpus);

  overviewValue('hypMemoryUsed', formatKMG(overview.hypMemoryUsed) + 'B');
  overviewValue('hypMemory', formatKMG(overview.hypMemory) + 'B');
  addStatusBar('hypMemoryStatusBar', overview.hypMemoryUsed, overview.hypMemory);

  overviewValue('hypLocalStorageUsed', formatKMG(overview.hypLocalStorageUsed) + 'B');
  overviewValue('hypLocalStorage', formatKMG(overview.hypLocalStorage) + 'B');
  addStatusBar('hypLocalStorageStatusBar', overview.hypLocalStorageUsed, overview.hypLocalStorage);

  overviewValue('numberOfHypervisors');
  overviewValue('numberOfAllDefinedHypervisors');
  addStatusBar('numberOfHypervisorsStatusBar', overview.numberOfHypervisors, overview.numberOfAllDefinedHypervisors);

  overviewValue('extra5'); //configured compute nodes UP (in bright)
  overviewValue('extra6'); //all configure compute nodes
  addStatusBar('computeNodesStatusBar', overview.extra5, overview.extra6);

  overviewValue('extra7'); //configure network nodes up
  overviewValue('extra8'); //all configured network nodes
  addStatusBar('networkNodesStatusBar', overview.extra7, overview.extra8);


  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('update();', dock.generalSettings.statusRefresh * 1000);

  function addStatusBar(elementId, used, total) {
    if((used < 0) || (total < 0)) {
      used = 0;
      total = 1;
    }
    var elementStatusBar = createStatus2("10px", used, total);

    var elementObj = document.getElementById(elementId);
    if(elementObj.lastChild) {
      elementObj.removeChild(elementObj.lastChild);
    }
    elementObj.appendChild(elementStatusBar);
  }
}

function selectMetric(event) {
  var list = document.getElementById("metriclist");
  var metric = event.target;
  if(!!metric) {
    graph.last = 0;
    var old = graph.deviceMetric[0].device;
    var newDevice = parseInt(metric.getAttribute('objectKey'));
    if(newDevice) graph.deviceMetric[0].device = newDevice;
    graph.deviceMetric[0].metric = parseInt(metric.getAttribute('metricKey'));
    graph.deviceMetric[0].param = metric.getAttribute('param');
    graph.deviceMetric[0].unit = current.find(graph.deviceMetric[0].metric).get('unit');
    var name = metric.getAttribute('label');
    list.setAttribute('objectKey', graph.deviceMetric[0].device);
    list.setAttribute('metricKey', graph.deviceMetric[0].metric);
    list.setAttribute('param', graph.deviceMetric[0].param);
    list.setAttribute('label', name);
    if(newDevice && (old != graph.deviceMetric[0].device)) graph.handleDeviceChange(graph.deviceMetric[0]);
    redraw();
    current.openStackMetric = [graph.deviceMetric[0].metric, graph.deviceMetric[0].param];
  }
}

function redraw() {
  if(graph) {
    graph.update();
  }
}

function setNodeApi() {
  var roles = {
    'OpenStackIdentityApiRole': [],
    'OpenStackComputeApiRole': [],
    'OpenStackNetworkApiRole': [],
    'OpenStackVolumeApiRole': [],
    'OpenStackImageApiRole': [],
    'OpenStackOrchestrationApiRole': [],
    'HAProxyRole': []
  };
  var nodes = current.find('Node');
  for(var i = 0; i < nodes.length; i++) {
    var openstackRoles = nodes[i].getRolesByFilter(function(role){
      return startswith(role.childType, 'OpenStack') || startswith(role.childType, 'HAProxyRole');
    }, true);
    for(var j = 0; j < openstackRoles.length; j++) {
      if(roles[openstackRoles[j].childType]===undefined) {
        roles[openstackRoles[j].childType] = [];
      }
      roles[openstackRoles[j].childType].push(nodes[i].object.hostname);
    }
  }
  document.getElementById('identityApiNodes').setAttribute('value', dotFormat(roles['OpenStackIdentityApiRole']));
  document.getElementById('computeApiNodes').setAttribute('value', dotFormat(roles['OpenStackComputeApiRole']));
  document.getElementById('networkApiNodes').setAttribute('value', dotFormat(roles['OpenStackNetworkApiRole']));
  document.getElementById('volumeApiNodes').setAttribute('value', dotFormat(roles['OpenStackVolumeApiRole']));
  document.getElementById('imageApiNodes').setAttribute('value', dotFormat(roles['OpenStackImageApiRole']));
  document.getElementById('heatApiNodes').setAttribute('value', dotFormat(roles['OpenStackOrchestrationApiRole']));
  document.getElementById('haproxyApiNodes').setAttribute('value', dotFormat(roles['HAProxyRole']));
}