/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function EditorRulePort() {
  this.addTitle = "Add Port";
  this.editTitle = "Edit Port";
  this.api = 'Port';

  this.lastProjectKey = null;

  this.getCurrentObject = function(editorController) {
    if(editorController.isAdd()) {
      var cvDataOpenStackApiPort = new editorController.dock.cvOpenStackApiPort();
      cvDataOpenStackApiPort.ownerUniqueKey = editorController.object.get('uniqueKey');
      return new editorController.dock.cvObject(editorController.object.cluster, cvDataOpenStackApiPort, 0, 0, false);
    } else {
      return editorController.object.cluster.find(editorController.myDialog.arguments.subobject);
    }
  };

  this.dataBindingAndCheck = function(editorController) {
    this.dataBindingAndCheckCommon(editorController);

    text('macAddress', '^');
    text('status', '^');
    check('adminStateUp');
    text('deviceOwner', '^');
    text('deviceUUID', '^');
    resolve('projectKey', 'OpenStackApiProject');
    extraeval('projectKey', 'port.triggerProjectChanged();');
    
    resolve('networkKey', 'OpenStackApiNetwork', '', false, '', function(network) {
      return (network.get('projectKey')!==editorController.currentObject.get('projectKey'));
    });
    extraeval('networkKey', 'current.correctAllUUID();');
    autorefresh('networkKey'); // really need for subnet selection

    text('bindingHostUUID', '^');
    text('bindingVifType', '^');
    text('bindingVnicType', '^');

    var subnetList = [{ 'value': 0, 'label': '' }];
    var found = editorController.currentObject.cluster.find('OpenStackApiSubnet');
    for(var i=0; i < found.length; i++) {
      if(found[i].get('networkKey')===editorController.currentObject.get('networkKey')) {
        subnetList.push({ 'value': found[i].object.uniqueKey, 'label': found[i].resolveName() });
      }
    }

    var arrayObjectAttributeListable = new ArrayObjectAttributeListable(editorController.currentObject, 'ips');
    arrayObjectAttributeListable.setObject(editorController.dock.cvOsapiPortIP);
    arrayObjectAttributeListable.setColumnOrder({'uniqueKey': 0, 'IP': 1, 'subnetKey': 2});
    //arrayObjectAttributeListable.addRowOrder(ColumnAttributeListableSorter);
    arrayObjectAttributeListable.initInputValue();

    var customListDataViewMapper = new CustomListDataViewMapper(arrayObjectAttributeListable);
    customListDataViewMapper.setElementsConfig([
      [
        {'type': 'textbox', 'attributes': [{'name': 'hidden', 'value': 'true'}], 'events': []},
        {'type': 'textbox', 'attributes': [], 'events': []},
        {'type': 'menulist', 'attributes': [], 'events': [], 'menuitems': subnetList}
      ]
    ]);
    customListDataViewMapper.setNewItem(['', '']);
    customListDataViewMapper.getHeader = function() {
      return [[
        this.createLabelElement('IP'),
        this.createLabelElement('Subnet')
      ]];
    };

    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'ips',
      'dataViewMapper': customListDataViewMapper
    });
    addDelListWidget.populate();


    var listAll = editorController.currentObject.cluster.find('OpenStackApiSecurityGroup');
    var list = [];
    for(var i=0; i<listAll.length; i++) {
      if(listAll[i].get('projectKey')===editorController.currentObject.get('projectKey')) {
        list.push(listAll[i]);
      }
    }

    var objectDualViewAttributeListable = new ObjectDualViewAttributeListable(list, editorController.currentObject, 'securityGroupKeys');
    objectDualViewAttributeListable.setColumnOrder({
      'name': 0
    });
    objectDualViewAttributeListable.initInputValue();

    var customListDataViewMapper = new CustomDualListDataViewMapper(objectDualViewAttributeListable);
    customListDataViewMapper.setElementsConfig([
      [
        {'type': 'label', 'attributes': [{'name': 'width', 'value': '100px'}], 'events': []}
      ]
    ]);

    var groupSelectorWidget = new GroupSelectorWidget({
      'context': document.getElementById('securityGroupKeys'),
      'rootNodeId': 'securityGroupSelected',
      'rootNodeIdSelection': 'securityGroupSelection',
      'rootNodeIdAdd': 'securityGroupButtonAdd',
      'rootNodeIdDel': 'securityGroupButtonDel',
      'dataViewMapper': customListDataViewMapper
    });
    groupSelectorWidget.populate();

    groupSelectorWidget.addItemCallBack = function() {
      current.correctAllUUID();
    };
    groupSelectorWidget.delItemCallBack = function() {
      current.correctAllUUID();
    };
  };
  
  this.fixSubnetUUID = function (editorController) {
    // HACK to keep subnetKey and subnetUUID sync
    // TODO refactor
    for (var i = 0; i < editorController.currentObject.object.ips.length; i++) {
      var foreignObject = editorController.currentObject.cluster.find(editorController.currentObject.object.ips[i].subnetKey);
      if(!!foreignObject) {
        editorController.currentObject.object.ips[i].subnetUUID = foreignObject.object.UUID;
      } else {
        editorController.currentObject.object.ips[i].subnetUUID = '';
      }
    }
  };

  this.inputCheck = function(editorController) {
    this.fixSubnetUUID(editorController);

    if(blank(editorController.currentObject.get('name'))) {
      editorController.dock.displayError("Name cannot be empty");
      return false;
    }
    return true;
  };
  
  this.triggerProjectChanged = function() {
    var currentProjectKey = current.get('projectKey');
    if(this.lastProjectKey!==currentProjectKey) {
      current.set('networkKey', 0);
    }
    
    current.correctAllUUID();
    this.lastProjectKey = current.get('projectKey');
    editor.refresh();
  };
}

EditorRulePort.prototype = new EditorRuleBase();
EditorRulePort.prototype.constructor = EditorRulePort;