/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var cluster;
var current;

function ListingController() {
  this.searchName = '';
  this.searchUUID = '';
  this.addedTimestamp1 = '';
  this.addedTimestamp2 = '';
  this.lastUpdateFromOpenStackTimestamp1 = '';
  this.lastUpdateFromOpenStackTimestamp2 = '';
  this.lastUpdateFromBrightTimestamp1 = '';
  this.lastUpdateFromBrightTimestamp2 = '';
  this.searchStatus = '';
  this.searchEnabled = '';

  this.parentKey = 0;

  this.select = null;

  this.basetype = 'OpenStackApiEntity';

  this.onLoad = function() {
    dock = getDock();
    println('.Load: ' + document.title + ' - ' + document.documentURI);

    this.initNodeElementTree();
  };

  this.refresh = function(use) {
    println('Refresh: ' + document.title + ' - ' + document.documentURI);
    if(!!use) {
      if(use) {
        current = use;
      }
      if(!!current.cluster) {
        cluster = current.cluster;
      } else {
        cluster = use;
      }
      current = cluster.find('OpenStack'); // TODO support multiple OpenStack
      current = current[0];
    }

    if(current) {
      this.parentKey = current.get('uniqueKey');
    }
  };

  this.initNodeElementTree = function() {
    tree = document.getElementById('tree');

    var instance = this;
    tree.addEventListener('row-count', function (event) {
      instance.updateTotalItems();
    }, false);
  };

  this.updateTotalItems = function() {
    var count = 0;
    if(tree.view && tree.view.rowCount) {
      count = tree.view.rowCount - 1;
    }
    if (count===1) {
      var msg = 'Displaying ' + count + ' item';
    } else {
      var msg = 'Displaying ' + count + ' items';
    }
    document.getElementById('tableTotalItems').setAttribute('value', msg);
  };

  this.getTimestamp = function (elementNodeId) {
    var timestamp = document.getElementById(elementNodeId).value;
    if(regexMatch(timestamp, DATETIME_REGEX, 'i')) {
      return Date.parse(timestamp) / 1000;
    } else {
      document.getElementById(elementNodeId).value = '';
      return '';
    }
  };

  this.search = function() {
    this.searchName = document.getElementById('searchName').value;
    this.searchUUID = document.getElementById('searchUUID').value;
    this.addedTimestamp1 = this.getTimestamp('addedTimestamp1');
    this.addedTimestamp2 = this.getTimestamp('addedTimestamp2');
    this.lastUpdateFromOpenStackTimestamp1 = this.getTimestamp('lastUpdateFromOpenStackTimestamp1');
    this.lastUpdateFromOpenStackTimestamp2 = this.getTimestamp('lastUpdateFromOpenStackTimestamp2');
    this.lastUpdateFromBrightTimestamp1 = this.getTimestamp('lastUpdateFromBrightTimestamp1');
    this.lastUpdateFromBrightTimestamp2 = this.getTimestamp('lastUpdateFromBrightTimestamp2');
    this.searchStatus = document.getElementById('searchStatus').value;
    this.searchEnabled = document.getElementById('searchEnabled').value;

    this.populateTable();
  };

  this.populateTable = function() {

    var list = [];

    var condition = [];
    if(this.searchName!=='') {
      condition.push("(row.get('name').search(RegExp('" + this.searchName + "', 'i')) !== -1)");
    }
    if(this.searchUUID!=='') {
      condition.push("(row.get('UUID').search(RegExp('" + this.searchUUID + "', 'i')) !== -1)");
    }
    if(this.addedTimestamp1!=='') {
      condition.push("(row.get('addedTimestamp') >= " + this.addedTimestamp1 + ")");
    }
    if(this.addedTimestamp2!=='') {
      condition.push("(row.get('addedTimestamp') <= " + this.addedTimestamp2 + ")");
    }
    if(this.lastUpdateFromOpenStackTimestamp1!=='') {
      condition.push("(row.get('lastUpdateFromOpenStackTimestamp') >= " + this.lastUpdateFromOpenStackTimestamp1 + ")");
    }
    if(this.lastUpdateFromOpenStackTimestamp2!=='') {
      condition.push("(row.get('lastUpdateFromOpenStackTimestamp') <= " + this.lastUpdateFromOpenStackTimestamp2 + ")");
    }
    if(this.lastUpdateFromBrightTimestamp1!=='') {
      condition.push("(row.get('lastUpdateFromBrightTimestamp') >= " + this.lastUpdateFromBrightTimestamp1 + ")");
    }
    if(this.lastUpdateFromBrightTimestamp2!=='') {
      condition.push("(row.get('lastUpdateFromBrightTimestamp') <= " + this.lastUpdateFromBrightTimestamp2 + ")");
    }
    if(this.searchStatus!=='') {
      condition.push("(row.object.hasOwnProperty('status') && row.get('status').search(RegExp('" + this.searchStatus + "', 'i')) !== -1)");
    }
    if(this.searchEnabled!=='') {
      condition.push("(row.object.hasOwnProperty('enabled') && String(row.get('enabled')) === '" + this.searchEnabled + "')");
    }

    condition = condition.join(' && ', condition);

    list = cluster.find(this.basetype);
    println('populateTable, basetype: ' + this.basetype + ', list lenght ' + list.length + ', parentKey: ' + this.parentKey);
    if(condition!=='') {
      eval("var searchFn = function(row) { return (" + condition + "); };");
      list = list.filter(searchFn);
    }


    data = new Array();

    for(var i = 0; i < list.length; i++) {
      if(!this.isFromCurrentInstance(list[i])) {
        continue;
      }
      this.pushToTableData(list[i]);
    }

    //TODO change to loadTable(this.tree, this.data);
    loadTable();
  };

  this.createTreeItem = function(one) {
    var item = new Object();
    item.status = ' - ';
    item.enabled = ' - ';
    for(var attribute in one.object) {
      if(attribute === 'uniqueKey') {
        item['key'] = one.get('uniqueKey');
      } else if(attribute === 'UUID') {
        item[attribute] = one.get(attribute);
      } else if(attribute === 'name') {
        item[attribute] = one.get(attribute);
      } else if(attribute === 'childType') {
        item[attribute] = one.get(attribute).substr(12);
      } else if(attribute === 'addedTimestamp' || attribute === 'lastUpdateFromOpenStackTimestamp' || attribute === 'lastUpdateFromBrightTimestamp') {
        var timestamp = one.get(attribute);
        if(timestamp>0) {
          var d = new Date(timestamp * 1000);
          item[attribute] = d.format('dd/mmm/yyyy HH:MM');
        } else {
          item[attribute] = 'never';
        }
      } else if(attribute === 'status' || attribute === 'enabled') {
        item[attribute] = one.get(attribute);
      }
    }
    return item;
  };

  this.pushToTableData = function(element) {
    data.push(this.createTreeItem(element));
  };

  this.isFromCurrentInstance = function(element) {
    if(this.parentKey && this.parentKey !== element.getParentKey()) {
      return false;
    } else {
      return true;
    }
  };

  this.gotoUpperBoard = function(select) {
    var entity = cluster.find(select);

    var subTabConf = new SubTabConf();
    var basetype = entity.get('childType');
    var container = subTabConf.props[basetype.substr(12)].container;

    var w = parent.window;
    var frames = w.frames;
    //println('gotoUpperBoard, #frames = ' + frames.length + ', container = ' + container + ', basetype = ' + basetype);
    for(var i = 0; i < frames.length; i++) {
      //println(frames[i].window.container);
      if (frames[i].window.container === container) {
        for(var j = 0; j < frames[i].frames.length; j++) {
          if(frames[i].frames[j].window.listing.basetype === basetype) {
            //println('=== GOTO ' + this.container + ', '+ this.basetype + '(' + i + ', ' + j+ ') ===');
            w.document.getElementById('tabs').selectedIndex = i;
            w.frames[i].document.getElementById('sub-tabs').selectedIndex = j;
            frames[i].frames[j].window.refresh();
            frames[i].frames[j].window.clearSelected('list');
            frames[i].frames[j].window.setSelected('list', [select]);
            break;
          }
        }
        break;
      }
    }
  };

  this.editObject = function() {
    //println('editObject');
    if(this.select.length === 0) {
      dock.displayError('Please select an entry');
    } else if(this.select.length === 1) {
      this.gotoUpperBoard(this.select[0]);
    }
  };

  this.selectObject = function(event) {
    this.initSelected();
  };

  this.initSelected = function() {
    this.select = this.getSelected();
  };

  this.clearSelected = function() {
    clearSelected('list');
    this.initSelected();
  };

  this.getSelected = function getSelected() {
    return selected('list');
  };

  this.setDate = function (name) {
    var args = new Object();
    var d = new Date();
    if(this.getTimestamp(name)!=='') {
      d.setTime(this.getTimestamp(name) * 1000);
    }
    args['date'] = d;
    var d = new dock.cvDialog("datepickerwindow.xul", args);
    d.parentWindow = window;
    d.onOk = function(date) {
      document.getElementById(name).value = date.format(DATETIME_FORMAT);
    };
    dock.dialogs.showDialog(d);
  };
}

var listing = new ListingController();

/*
 * WRAPPERS
 */
function onLoad() {
  listing.onLoad();
}

function refresh(use) {
  listing.refresh(use);
}

function selectObject() {
  listing.selectObject();
}

function editObject() {
  listing.editObject();
}

function setDate(name) {
  listing.setDate(name);
}
