/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function EditorRuleSecurityGroup() {
  this.addTitle = "Add Security Group";
  this.editTitle = "Edit Security Group";
  this.api = 'SecurityGroup';

  this.getCurrentObject = function(editorController) {
    if(editorController.isAdd()) {
      var cvDataOpenStackApiSecurityGroup = new editorController.dock.cvOpenStackApiSecurityGroup();
      cvDataOpenStackApiSecurityGroup.ownerUniqueKey = editorController.object.get('uniqueKey');
      return new editorController.dock.cvObject(editorController.object.cluster, cvDataOpenStackApiSecurityGroup, 0, 0, false);
    } else {
      return editorController.object.cluster.find(editorController.myDialog.arguments.subobject);
    }
  };

  this.dataBindingAndCheck = function(editorController) {
    this.dataBindingAndCheckCommon(editorController);

    text('description', '^');
    resolve('projectKey', 'OpenStackApiProject');
    extraeval('projectKey', 'current.correctAllUUID();');
    autorefresh('projectKey');

    var arrayObjectAttributeListable = new ArrayObjectAttributeListable(editorController.currentObject, 'rules');
    arrayObjectAttributeListable.setObject(editorController.dock.cvOsapiSecurityGroupRule);
    arrayObjectAttributeListable.setColumnOrder({'direction': 0, 'etherType': 1, 'portRangeMin': 2, 'portRangeMax': 3, 'protocol': 4, 'remoteIPPrefix': 5});
    //arrayObjectAttributeListable.addRowOrder(ColumnAttributeListableSorter);
    arrayObjectAttributeListable.initInputValue();

    var customListDataViewMapper = new CustomListDataViewMapper(arrayObjectAttributeListable);
    customListDataViewMapper.setElementsConfig([
      [
        {'type': 'menulist', 'attributes': [], 'events': [], 'menuitems': [
          { 'value': 'EGRESS', 'label': 'Egress' },
          { 'value': 'INGRESS', 'label': 'Ingress' }
        ]},
        {'type': 'menulist', 'attributes': [], 'events': [], 'menuitems': [
          { 'value': 'IPV4', 'label': 'ipv4' },
          { 'value': 'IPV6', 'label': 'ipv6' }
        ]},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '80px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '80px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '80px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '80px'}], 'events': []}
      ]
    ]);
    customListDataViewMapper.setNewItem(['EGRESS', 'IPV4', 0, 0, '', '']);
    customListDataViewMapper.getHeader = function() {
      return [[
        this.createLabelElement('Direction'),
        this.createLabelElement('Ether type'),
        this.createLabelElement('Port range min'),
        this.createLabelElement('Port range max'),
        this.createLabelElement('Protocol'),
        this.createLabelElement('Remote CIDR')
      ]];
    };

    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'rules',
      'dataViewMapper': customListDataViewMapper
    });

    addDelListWidget.populate();
    // TODO
    // add remoteGroupUUID
    // add projectUUID
    //setAttributeRecursive(document.getElementById('rules'), 'disabled', 'true');
  };

  this.inputCheck = function(editorController) {
    if(blank(editorController.currentObject.get('name'))) {
      editorController.dock.displayError("Name cannot be empty");
      return false;
    }
    return true;
  };
}

EditorRuleSecurityGroup.prototype = new EditorRuleBase();
EditorRuleSecurityGroup.prototype.constructor = EditorRuleSecurityGroup;