/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function DefaultUserRoleListingController(chromeWindow) {
  this.chromeWindow = chromeWindow;

  this.getField = function() {
    return 'defaultUserRoles';
  };

  this.getNewDataObject = function() {
    return new this.dock.cvOpenStackDefaultUserRole();
  };

  this.getEditorRule = function(key) {
    return new EditorRuleDefaultUserRole(key);
  };

  this.getListing = function(object) {
    return object.get('defaultUserRoles');
  };

  this.createTreeItem = function(one) {
    var item = new Object();
    for(var attribute in one) {
      if(attribute === 'uniqueKey') {
        item['key'] = one['uniqueKey'];
      } else if(attribute === 'tenants') {
        item['l_numtenants'] = one[attribute].length;
        item['l_' + attribute] = one[attribute].join(', ');
      } else {
        item['l_' + attribute] = one[attribute];
      }
    }
    return item;
  };

  this.removeObject = function() {
    var instance = this;

    if(this.select.length === 0) {
      dock.displayError('Please select an entry.');
    } else {

      var args = new Object();
      if(this.select.length === 1) {
        args['question'] = 'Are you sure you want to remove this entry?';
      } else {
        args['question'] = 'Are you sure you want to remove these entries?';
      }

      var d = new dock.cvDialog('chrome://cmgui/content/question.xul', args);
      d.onOk = function() {
        var settingsUsers = instance.object.get('users');
        for(var i = instance.select.length - 1; i >= 0; i--) {
          for(var k = settingsUsers.defaultUserRoles.length - 1; k >= 0; k--) {
            if(settingsUsers.defaultUserRoles[k].uniqueKey === parseInt(instance.select[i])) {
              settingsUsers.defaultUserRoles[k].toBeRemoved = true;
              settingsUsers.modified = true;
              instance.object.object.modified = true;
            }
          }
        }
        instance.clearSelected();
        instance.refresh();
      };
      dock.dialogs.showDialog(d);
    }
  };
};

DefaultUserRoleListingController.prototype = new ListingController();
DefaultUserRoleListingController.prototype.constructor = DefaultUserRoleListingController;

var listing = new DefaultUserRoleListingController(this);



/*
 * *******************************************************************************************************************************************************
 */

function EditorRuleDefaultUserRole(key) {
  this.addTitle = "Add default user role";
  this.editTitle = "Edit default user role";
  this.api = 'Domain';
  this.key = parseInt(key);

  this.getCurrentObject = function(editorController) {
    var cluster = editorController.object.cluster;
    for(var i = 0; i < editorController.myDialog.arguments.subobject.object.defaultUserRoles.length; i++) {
      if(editorController.myDialog.arguments.subobject.object.defaultUserRoles[i].uniqueKey === this.key) {
        var dataobject = editorController.myDialog.arguments.subobject.object.defaultUserRoles[i];
      }
    }
    var role = editorController.myDialog.arguments.subobject;

    return new editorController.dock.cvObject(cluster, dataobject, role, 'defaultUserRoles');
  };

  this.dataBindingAndCheck = function(editorController) {
    this.resetEvents('formContainer');

    text('name', '^');

    var arrayAttributeListable = new SingleArrayAttributeListable(editorController.currentObject, 'tenants');
    var textboxListDataViewMapper = new TextboxListDataViewMapper(arrayAttributeListable);
    textboxListDataViewMapper.setNewItem(['']);
    textboxListDataViewMapper.setCommonAttributes([{'name': 'width', 'value': '170px'}]);
    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'tenants',
      'dataViewMapper': textboxListDataViewMapper
    });
    addDelListWidget.populate();
  };

  this.inputCheck = function(editorController) {
    if(editorController.currentObject.get('toBeRemoved')===true) {
      return true;
    }
    if(blank(editorController.currentObject.get('name'))) {
      editorController.dock.displayError("Name cannot be empty");
      return false;
    }

    return true;
  };
}

EditorRuleDefaultUserRole.prototype = new EditorRuleBase();
EditorRuleDefaultUserRole.prototype.constructor = EditorRuleDefaultUserRole;
