/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function EditorRuleSubnet() {
  this.addTitle = "Add Subnet";
  this.editTitle = "Edit Subnet";
  this.api = 'Subnet';

  this.getCurrentObject = function(editorController) {
    if(editorController.isAdd()) {
      var cvDataOpenStackApiSubnet = new editorController.dock.cvOpenStackApiSubnet();
      cvDataOpenStackApiSubnet.ownerUniqueKey = editorController.object.get('uniqueKey');
      return new editorController.dock.cvObject(editorController.object.cluster, cvDataOpenStackApiSubnet, 0, 0, false);
    } else {
      return editorController.object.cluster.find(editorController.myDialog.arguments.subobject);
    }
  };

  this.dataBindingAndCheck = function(editorController) {
    this.dataBindingAndCheckCommon(editorController);

    check('enableDHCP');
    resolve('networkKey', 'OpenStackApiNetwork');
    extraeval('networkKey', 'subnet.syncProjectWithNetwork();');
    //autorefresh('networkKey');

    resolve('projectKey', 'OpenStackApiProject');
    extraeval('projectKey', 'current.correctAllUUID();');
    //autorefresh('projectKey');

    var arrayAttributeListable = new SingleArrayAttributeListable(editorController.currentObject, 'DNSNameservers');
    var textboxListDataViewMapper = new TextboxListDataViewMapper(arrayAttributeListable);
    textboxListDataViewMapper.changePreCheck = function(row, col, valueModelMapper, event) {
      if(!valueModelMapper(event.target.value).match(dock.REGEX_IPv4)) {
        event.target.setAttribute("style", "color: red;");
        return false;
      } else {
        event.target.setAttribute("style", "");
        return true;
      }
    };
    textboxListDataViewMapper.setNewItem(['']);
    textboxListDataViewMapper.setCommonAttributes([{'name': 'width', 'value': '190px'}]);
    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'DNSNameservers',
      'dataViewMapper': textboxListDataViewMapper
    });
    addDelListWidget.populate();

    kenum('IPVersion', this.getObjectProperties('IPVersion').enum);
    text('CIDR', '^');

    var gatewayIPOptions = [
      ['0.0.0.0', 'Disabled']
    ];
    editableKenum(editorController.currentObject, 'gatewayIP', 'gatewayIP', gatewayIPOptions, dock.REGEX_IPv4);

    var arrayObjectAttributeListable = new ArrayObjectAttributeListable(editorController.currentObject, 'allocationPools');
    arrayObjectAttributeListable.setObject(editorController.dock.cvOsapiSubnetAllocationPool);
    arrayObjectAttributeListable.setColumnOrder({'start': 0, 'end': 1});
    //arrayObjectAttributeListable.addRowOrder(ColumnAttributeListableSorter);
    arrayObjectAttributeListable.initInputValue();

    var customListDataViewMapper = new CustomListDataViewMapper(arrayObjectAttributeListable);
    customListDataViewMapper.setElementsConfig([
      [
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '100px'}], 'events': []},
        {'type': 'textbox', 'attributes': [{'name': 'width', 'value': '100px'}], 'events': []}
      ]
    ]);
    customListDataViewMapper.setNewItem(['', '']);

    var addDelListWidget = new AddDelListWidget({
      'rootNodeId': 'allocationPools',
      'dataViewMapper': customListDataViewMapper
    });
    addDelListWidget.populate();
  };

  this.inputCheck = function(editorController) {
    if(blank(editorController.currentObject.get('name'))) {
      editorController.dock.displayError("Name cannot be empty");
      return false;
    }
    return true;
  };

  this.syncProjectWithNetwork = function() {
    var project = current.cluster.find(current.get('networkKey'));
    current.set('projectKey', project.get('projectKey'));
    current.correctAllUUID();
    refresh();
  };
}

EditorRuleSubnet.prototype = new EditorRuleBase();
EditorRuleSubnet.prototype.constructor = EditorRuleSubnet;
