/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var object;
var current;
var cluster;

function TopologyController(chromeWindow) {
  this.dock = null;
  this.object = null;
  this.myDialog = null;
  this.currentObject = null;
  this.cluster = null;
  this.chromeWindow = chromeWindow;
  this.okPressed = false;
  this.network = null;
  this.showing = false;

  this.onLoad = function () {
    println("Detached OpenStack topology tab initialized: " + document.documentURI);
    this.initDock();
    this.initMyDialog();
    this.initObject();
    this.initCurrentObject();
    this.initCluster();
    this.buildForm();

    this.refresh();
  };

  this.initDock = function() {
    this.dock = getDock();
    dock = this.dock;
  };

  this.initMyDialog = function() {
    var s = this.chromeWindow.name.indexOf('[');
    var e = this.chromeWindow.name.indexOf(']');
    var index = this.chromeWindow.name.substr(s + 1, e - s - 1);
    this.myDialog = this.dock.dialogs.list[index];
  };

  this.initObject = function() {
    this.object = this.myDialog.arguments.object;
    object = this.object;
  };

  this.initCurrentObject = function() {
    this.currentObject = this.object;
    current = this.currentObject;
  };

  this.initCluster = function() {
    this.cluster = this.currentObject.cluster;
    cluster = this.cluster;
  };

  this.refresh = function() {
    this.chromeWindow.setTimeout('topology.draw();', 1000);
  };

  this.show = function() {
    if (this.showing===true) {
      document.getElementById('show-button').setAttribute('label', 'Show');
      this.showing = !this.showing;
      return;
    }
    var rootElement = document.getElementById('show');
    var searchValue = rootElement.value.toString();
    if (searchValue === '') {
      return;
    }
    this.showing = !this.showing;

    var filter = this.getFilter();
    var pThis = this;
    var index = 0;
    var foundServers = this.cluster.findBy({
      baseType: 'OpenStackApiServer',
      filter: function(server) {
        if(filter.projects.indexOf(server.get('projectKey'))===-1) {
          return false;
        }
        if(server.get('name').search(RegExp(searchValue, 'i')) !== -1 || server.get('UUID').search(RegExp(searchValue, 'i')) !== -1) {
          return true;
        }
        return false;
      }
    });

    doFocus();

    function doFocus() {
      if(index < foundServers.length && pThis.showing===true) {

        document.getElementById('show-button').setAttribute('label', 'Stop');
        var uniqueKey = foundServers[index].get('uniqueKey');

        var node = pThis.network.findNode(uniqueKey);
        if (node.length>1) {
          for (var i=0; i<node.length; i++) {
            //println()
            if (pThis.network.isCluster(node[i].id) === true) {
              pThis.network.openCluster(node[i].id);
            }
          }
        }
        pThis.network.selectNodes([uniqueKey]);
        pThis.network.focus(uniqueKey, {scale: 1.5, animation: true});
        setTimeout(function () {
          index++;
          doFocus();
        }, 3000);
      } else {
        document.getElementById('show-button').setAttribute('label', 'Show');
        pThis.showing = false;
      }
    }
  };

  this.checkAllProjects = function(nodeElement) {
    var rootElement = document.getElementById('filter-projects');
    for(var i=0; i<rootElement.children.length; i++) {
      rootElement.children[i].setAttribute('checked', nodeElement.checked);
    }
  };

  this.checkAllHypervisores = function(nodeElement) {
    var rootElement = document.getElementById('filter-hypervisores');
    for(var i=0; i<rootElement.children.length; i++) {
      rootElement.children[i].setAttribute('checked', nodeElement.checked);
    }
  };

  this.buildForm = function() {
    var rootElement = document.getElementById('filter-projects');
    this.cluster.find('OpenStackApiProject').forEach(function(project, index) {
      var checkImage = document.createElement('checkimage');
      checkImage.setAttribute('label', project.get('name'));
      checkImage.setAttribute('id', project.get('uniqueKey'));
      checkImage.setAttribute('width', '175px');
      checkImage.style['max-width'] = '175px';
      checkImage.setAttribute('checked', 'false');
      rootElement.appendChild(checkImage);
    });
    var rootElement = document.getElementById('filter-hypervisores');
    this.cluster.find('OpenStackApiHypervisor').forEach(function(hypervisor, index) {
      var checkImage = document.createElement('checkimage');
      checkImage.setAttribute('label', hypervisor.get('name'));
      checkImage.setAttribute('id', hypervisor.get('name'));
      checkImage.setAttribute('width', '175px');
      checkImage.style['max-width'] = '175px';
      checkImage.setAttribute('checked', 'true');
      rootElement.appendChild(checkImage);
    });
  };

  this.getFilter = function() {
    var filter = {
      hypervisores: [],
      projects: [],
      networkTypes: [],
      showRouters: true,
      detached: {
        networks: false,
        servers: false,
        routers: false
      }
    };
    var rootElement = null;

    rootElement = document.getElementById('filter-projects');
    for(var i=0; i<rootElement.children.length; i++) {
      if (rootElement.children[i].checked===true) {
        filter.projects.push(parseInt(rootElement.children[i].id));
      }
    }

    rootElement = document.getElementById('filter-hypervisores');
    for(var i=0; i<rootElement.children.length; i++) {
      if (rootElement.children[i].checked===true) {
        filter.hypervisores.push(rootElement.children[i].id);
      }
    }

    rootElement = document.getElementById('filter-network-types');
    for(var i=0; i<rootElement.children.length; i++) {
      if (rootElement.children[i].checked===true) {
        filter.networkTypes.push(rootElement.children[i].id);
      }
    }

    filter.showRouters = document.getElementById('filter-show-routers').checked;

    filter.detached.networks = document.getElementById('filter-detached-networks').checked;
    filter.detached.servers = document.getElementById('filter-detached-servers').checked;
    filter.detached.routers = document.getElementById('filter-detached-routers').checked;
    return filter;
  };

  this.getProjects = function() {
    return this.cluster.find('OpenStackApiProject');
  };

  this.getNetworks = function() {
    return this.cluster.find('OpenStackApiNetwork');
  };

  this.getServers = function() {
    return this.cluster.find('OpenStackApiServer');
  };

  this.getRouters = function() {
    return this.cluster.find('OpenStackApiRouter');
  };

  this.getPorts = function() {
    return this.cluster.find('OpenStackApiPort');
  };

//  this.getColors = function() {
//    return [
//      '#97C2FC'
//    ];
//  };

  this.destroy = function() {
    if (this.network !== null) {
      this.network.destroy();
      this.network = null;
    }
  };

  //TODO refactor
  this.draw = function() {
    function getGravitationalConstant(numNodes, numEdges) {
      var gravitationalConstant = -2000;
      if (numNodes>300) {
        gravitationalConstant += -500;
      } else if (numNodes>100) {
        gravitationalConstant += -200;
      }
      if (numEdges>300) {
        gravitationalConstant += -500;
      } else if (numEdges>100) {
        gravitationalConstant += -200;
      }
      return gravitationalConstant;
    }

    function getSpringConstant(numNodes, numEdges) {
      var springConstant = 40;
      if (numNodes>300) {
        springConstant += -10;
      } else if (numNodes>100) {
        springConstant += -5;
      }
      if (numEdges>300) {
        springConstant += -10;
      } else if (numEdges>100) {
        springConstant += -5;
      }
      springConstant = springConstant / 1000;
      return springConstant;
    }

    function getSpringLength(numNodes, numEdges) {
      var springLength = 95;
      if (numNodes>300) {
        springLength += 5;
      } else if (numNodes>100) {
        springLength += 2;
      }
      if (numEdges>300) {
        springLength += 5;
      } else if (numEdges>100) {
        springLength += 2;
      }
      return springLength;
    }

    document.getElementById('loadingBar').style.opacity = 1;
    document.getElementById('loadingBar').style.display = 'block';

    this.destroy();
    var filter = this.getFilter();
//    printallR(filter);
    var mynetwork = document.getElementById('network-topology');
    //var x = - mynetwork.clientWidth / 2 + 0;
    //var y = - mynetwork.clientHeight / 2 + 50;
    // FIXME
    var x = - mynetwork.clientWidth * 2;
    //var y = - mynetwork.clientHeight / 2 + 50;
    var y = x;

    var nodeDataSet = [];
    var edgeDataSet = [];
    var networkEdgeDataSet = [];

    var groupKeys = {};
    var projects = this.getProjects();
    for(var counter = 0, i = 0; i < projects.length; i++) {
      if(filter.projects.indexOf(projects[i].get('uniqueKey'))===-1) {
        continue;
      }
      nodeDataSet.push({
        id: projects[i].get('uniqueKey'),
        label: projects[i].get('name'),
        x: x + (counter % 2) * 200,
        y: y + (counter >> 1) * 70,
        value: 1,
        fixed: true,
        physics: false,
        title: '<table>' +
               '  <tr>' +
               '    <td>Name:</td>' +
               '    <td>' + projects[i].get('name') + '</td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>UUID:</td>' +
               '    <td>' + projects[i].get('UUID') + '</td>' +
               '  </tr>' +
               '</table>',
        shape: 'dot',
        group: projects[i].get('uniqueKey'),
        type: 'legend'
      });
      groupKeys[projects[i].get('uniqueKey')] = projects[i].get('UUID');
      groupKeys[projects[i].get('UUID')] = projects[i].get('uniqueKey');
      counter++;
    }
    var numProjectsVisible = counter;
    for(var counter = 0, i = 0; i < projects.length; i++) {
      if(filter.projects.indexOf(projects[i].get('uniqueKey'))===-1) {
        continue;
      }
      groupKeys['pos-' + projects[i].get('uniqueKey') + '-x' ] = Math.cos(2 * Math.PI / numProjectsVisible * counter) * 1000;
      groupKeys['pos-' + projects[i].get('uniqueKey') + '-y' ] = Math.sin(2 * Math.PI / numProjectsVisible * counter) * 1000;
      counter++;
    }

    var networkPorts = this.cluster.findBy({
      baseType: 'OpenStackApiPort',
      filter: function(port) {
        return true;
      }
    });

    var networkKeys = {};
    var networks = this.getNetworks();
    for(var i = 0; i < networks.length; i++) {
      if(filter.projects.indexOf(networks[i].get('projectKey'))===-1) {
        continue;
      }
      var nType = networks[i].get('providerNetworkType');
      if(['flat', 'vlan', 'vxlan'].indexOf(nType)===-1) {
        nType = 'others';
      }
      if(filter.networkTypes.indexOf(nType)===-1) {
        continue;
      }

      var ports = networkPorts.filter(function(port) {
        if(port.get('networkUUID') === networks[i].get('UUID')) {
          return true;
        }
        return false;
      });
      if (filter.detached.networks===false && ports.length===0) {
        continue;
      }

      nodeDataSet.push({
        id: networks[i].get('uniqueKey'),
        label: networks[i].get('name'),
        x: groupKeys['pos-' + networks[i].get('projectKey') + '-x' ],
        y: groupKeys['pos-' + networks[i].get('projectKey') + '-y' ],
        title: '<table>' +
               '  <tr>' +
               '    <td>Name:</td>' +
               '    <td>' + networks[i].get('name') + '</td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>UUID:</td>' +
               '    <td>' + networks[i].get('UUID') + '</td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>Type:</td>' +
               '    <td>' + networks[i].get('providerNetworkType') + '</td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>Ports:</td>' +
               '    <td>' +
               '      <table>' +
               '        <tr>' +
               '          <td>UUID</td>' +
               '          <td>MAC Address</td>' +
               '          <td>IP</td>' +
               '        </tr>' +
               ports.map(function(port) {
                   return '<tr>' +
               '          <td>' + port.get('UUID') + '</td>' +
               '          <td>' + port.get('macAddress') + '</td>' +
               '          <td>' + port.get('ips').map(function(ip) { return ip.IP; }).join('<br/>') + '</td>' +
               '        </tr>';
               }).join('') +
               '      </table>' +
               '    </td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>Status:</td>' +
               '    <td>' + networks[i].get('status') + '</td>' +
               '  </tr>' +
               '</table>',
        image: 'chrome://cmgui/content/icons/Network_32.png',
        shape: 'circularImage',
        group: networks[i].get('projectKey'),
        mass: 2,
        type: 'network'
      });
      networkKeys[networks[i].get('uniqueKey')] = networks[i].get('UUID');
      networkKeys[networks[i].get('UUID')] = networks[i].get('uniqueKey');
    }

    var devicePorts = this.cluster.findBy({
      baseType: 'OpenStackApiPort',
      filter: function(port) {
        if(port.get('deviceOwner').search(RegExp('^compute:.+$')) !== -1) {
          return true;
        }
        return false;
      }
    });

    var serverKeys = {};
    var servers = this.getServers();
    for(var i = 0; i < servers.length; i++) {
      if(filter.projects.indexOf(servers[i].get('projectKey'))===-1) {
        continue;
      }
      if(filter.hypervisores.indexOf(servers[i].get('hypervisorHostname'))===-1) {
        continue;
      }

      var user = this.cluster.find(servers[i].get('userKey'));
      var ports = devicePorts.filter(function(port) {
        if(port.get('deviceUUID') === servers[i].get('UUID')) {
          return true;
        }
        return false;
      });
      if (filter.detached.servers===false && ports.length===0) {
        continue;
      }

      var imageServer = 'Node-Up.png';
      switch(servers[i].get('status')) {
        case 'ACTIVE':
          imageServer = 'Node-Up_32.png';
          break;
        case 'SHUTOFF':
          imageServer = 'Node-Down_32.png';
          break;
        case 'BUILD':
        case 'ERROR':
          imageServer = 'Node-Closed_32.png';
          break;
        default:
      }


      nodeDataSet.push({
        id: servers[i].get('uniqueKey'),
        label: servers[i].get('name'),
        x: groupKeys['pos-' + servers[i].get('projectKey') + '-x' ],
        y: groupKeys['pos-' + servers[i].get('projectKey') + '-y' ],
        title: '<table>' +
               '  <tr>' +
               '    <td>Name:</td>' +
               '    <td>' + servers[i].get('name') + '</td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>UUID:</td>' +
               '    <td>' + servers[i].get('UUID') + '</td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>Hypervisor:</td>' +
               '    <td>' + servers[i].get('hypervisorHostname') + '</td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>User:</td>' +
               '    <td>' + user.get('name') + '</td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>Ports:</td>' +
               '    <td>' +
               '      <table>' +
               '        <tr>' +
               '          <td>UUID</td>' +
               '          <td>MAC Address</td>' +
               '          <td>IP</td>' +
               '        </tr>' +
               ports.map(function(port) {
                   return '<tr>' +
               '          <td>' + port.get('UUID') + '</td>' +
               '          <td>' + port.get('macAddress') + '</td>' +
               '          <td>' + port.get('ips').map(function(ip) { return ip.IP; }).join('<br/>') + '</td>' +
               '        </tr>';
               }).join('') +
               '      </table>' +
               '    </td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>Status:</td>' +
               '    <td>' + servers[i].get('status') + '</td>' +
               '  </tr>' +
               '</table>',
        image: 'chrome://cmgui/content/icons/' + imageServer,
        shape: 'circularImage',
        group: servers[i].get('projectKey'),
        mass: 1,
        type: 'server'
      });
      serverKeys[servers[i].get('uniqueKey')] = servers[i].get('UUID');
      serverKeys[servers[i].get('UUID')] = servers[i].get('uniqueKey');
      groupKeys[servers[i].get('projectKey')] = servers[i].get('projectUUID');
    }

    var routerPorts = this.cluster.findBy({
      baseType: 'OpenStackApiPort',
      filter: function(port) {
        if(port.get('deviceOwner').search(RegExp('^network:router_.+$')) !== -1) {
          return true;
        }
        return false;
      }
    });

    var routerKeys = {};
    var routers = this.getRouters();
    for(var i = 0; i < routers.length; i++) {
      if(filter.projects.indexOf(routers[i].get('projectKey'))===-1) {
        continue;
      }
      if(filter.showRouters===false) {
        continue;
      }

      var ports = routerPorts.filter(function(port) {
        if(port.get('deviceUUID') === routers[i].get('UUID')) {
          return true;
        }
        return false;
      });
      if (filter.detached.routers===false && ports.length===0) {
        continue;
      }

      nodeDataSet.push({
        id: routers[i].get('uniqueKey'),
        label: routers[i].get('name'),
        x: groupKeys['pos-' + routers[i].get('projectKey') + '-x' ],
        y: groupKeys['pos-' + routers[i].get('projectKey') + '-y' ],
        title: '<table>' +
               '  <tr>' +
               '    <td>Name:</td>' +
               '    <td>' + routers[i].get('name') + '</td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>UUID:</td>' +
               '    <td>' + routers[i].get('UUID') + '</td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>Ports:</td>' +
               '    <td>' +
               '      <table>' +
               '        <tr>' +
               '          <td>UUID</td>' +
               '          <td>MAC Address</td>' +
               '          <td>IP</td>' +
               '        </tr>' +
               ports.map(function(port) {
                   return '<tr>' +
               '          <td>' + port.get('UUID') + '</td>' +
               '          <td>' + port.get('macAddress') + '</td>' +
               '          <td>' + port.get('ips').map(function(ip) { return ip.IP; }).join('<br/>') + '</td>' +
               '        </tr>';
               }).join('') +
               '      </table>' +
               '    </td>' +
               '  </tr>' +
               '  <tr>' +
               '    <td>Status:</td>' +
               '    <td>' + routers[i].get('status') + '</td>' +
               '  </tr>' +
               '</table>',
        image: 'chrome://cmgui/content/icons/IBSwitch_32.png',
        shape: 'circularImage',
        group: routers[i].get('projectKey'),
        mass: 3,
        type: 'router'
      });
      routerKeys[routers[i].get('uniqueKey')] = routers[i].get('UUID');
      routerKeys[routers[i].get('UUID')] = routers[i].get('uniqueKey');
      groupKeys[routers[i].get('projectKey')] = routers[i].get('projectUUID');
    }

    var ports = this.getPorts();
    for(var i = 0; i < ports.length; i++) {
      if (networkKeys[ports[i].get('networkKey')]===undefined) {
        continue;
      }
      if (ports[i].get('networkKey') && routerKeys[ports[i].get('deviceUUID')]!==undefined) {
        var routerKey = routerKeys[ports[i].get('deviceUUID')];
        edgeDataSet.push({from: ports[i].get('networkKey'), to: routerKey});
      } else if (ports[i].get('networkKey') && serverKeys[ports[i].get('deviceUUID')]!==undefined) {
        var serverKey = serverKeys[ports[i].get('deviceUUID')];
        var networkKey = ports[i].get('networkKey');
        edgeDataSet.push({from: networkKey, to: serverKey});
        if (typeof networkEdgeDataSet[networkKey] === 'undefined') {
          networkEdgeDataSet[networkKey] = 1;
        } else {
          networkEdgeDataSet[networkKey] += 1;
        }
      }
    }

    //printallR(nodeDataSet);
    //printallR(groupKeys);

    //printallR(routerKeys);
    //printallR(edgeDataSet);
    var nodes = new vis.DataSet(nodeDataSet);
    var edges = new vis.DataSet(edgeDataSet);

    // create a network
    var container = document.getElementById('network-topology');
    var data = {
      nodes: nodes,
      edges: edges
    };
    var options = {
      height: '100%',
      width: '100%',
      interaction: {
        tooltipDelay: 100,
        navigationButtons: true,
        keyboard: true,
        hideEdgesOnDrag: document.getElementById('hideEdgesOnDrag').checked
      },
      nodes: {

        scaling: {
          min: 10,
          max: 30,
          label: {
            min: 8,
            max: 30,
            drawThreshold: 8,
            maxVisible: 20
          }
        },
        font: {
          size: 12,
          face: 'Tahoma'
        },
        borderWidthSelected: 4
      },

      edges: {
        smooth: {
          enabled: true,
          type: (nodes.length>100) ? "continuous" : "dynamic"
        }
      },
      physics: {
        barnesHut: {
          gravitationalConstant: getGravitationalConstant(nodes.length, edges.length),   // default -2000
          springConstant: getSpringConstant(nodes.length, edges.length),                 // default 0.04
          springLength: getSpringLength(nodes.length, edges.length),                     // default 95
          //centralGravity: 0.3,            // default 0.3
          //avoidOverlap: 1                 // default 0
        },
        stabilization: {
          enabled: document.getElementById('stabilization').checked,
//          iterations: 1000,
//          updateInterval: 100,
//          onlyDynamicEdges: true,
        }
      }
    };

    this.network = new vis.Network(container, data, options);

    var clusterOptions = {
      joinCondition: function(nodeOptions, childNode) {
        if(childNode.amountOfConnections===1) {
          return true;
        }
        return false;
      },
      processProperties: function (clusterOptions, childNodes, childEdges) {
        clusterOptions.size = 30;
        clusterOptions.id = 'cluster-' + childNodes[0].id;
        clusterOptions.image = childNodes[0].image;
        clusterOptions.group = childNodes[0].group;
        clusterOptions.label = childNodes[0].label;
        var nServers = 0;
        var nRouters = 0;
        for(var i=1; i<childNodes.length; i++) {
          if(childNodes[i].type==='server') {
            nServers++;
          } else if(childNodes[i].type==='router') {
            nRouters++;
          }
        }
        clusterOptions.title = childNodes[0].title.replace(RegExp('</table>$'), '<tr><td>Servers:</td><td>' + nServers + '</td></tr><tr><td>Routers:</td><td>' + nRouters + '</td></tr></table>');
        return clusterOptions;
      },
      clusterNodeProperties: {
        shape: 'circularImage',
        borderWidth: 3,
        borderWidthSelected: 4
      }
    };

    var numNodesClose = parseInt(document.getElementById('num-nodes-close').value);
    if (isNaN(numNodesClose)) {
      document.getElementById('num-nodes-close').value = 3;
      numNodesClose = 3;
    }
    for(var uniqueKey in networkEdgeDataSet) {
      if(networkEdgeDataSet[uniqueKey]>numNodesClose) {
        this.network.clusterByConnection(uniqueKey, clusterOptions);
      }
    }

    this.network.on("stabilizationProgress", function(params) {
      var maxWidth = 496;
      var minWidth = 20;
      var widthFactor = params.iterations/params.total;
      var width = Math.max(minWidth,maxWidth * widthFactor);

      document.getElementById('bar').style.width = width + 'px';
      document.getElementById('text').innerHTML = Math.round(widthFactor*100) + '%';
    });

    //this.network.once("stabilizationIterationsDone", function() {
    this.network.once("afterDrawing", function() {
      document.getElementById('text').innerHTML = '100%';
      document.getElementById('bar').style.width = '496px';
      document.getElementById('loadingBar').style.opacity = 0;
      // really clean the dom element
      setTimeout(function () {
        document.getElementById('loadingBar').style.display = 'none';
      }, 500);
    });

    this.network.on("doubleClick", function(params) {
      if(params.nodes.length === 1) {
        if(this.isCluster(params.nodes[0]) === true) {
          this.openCluster(params.nodes[0]);
        } else {
          var selectedNode = nodes.get(params.nodes[0]);
          if(selectedNode.type==='network') {
            this.clusterByConnection(params.nodes[0], clusterOptions);
          }
        }
      }
    });
  };

}

var topology = new TopologyController(window);