/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

//TODO refactor single responsability

function BaseWidget(args) {
  this.rootNodeId = null;
  this.rootNodeIdElement = null;

  this.init = function(args) {
    this.rootNodeId = args.rootNodeId;
  };

  this.getRootNodeIdElement = function(){
    if (this.rootNodeIdElement === null) {
      this.rootNodeIdElement = document.getElementById(this.rootNodeId);
    }
    return this.rootNodeIdElement;
  };

  this.clearRootNodeIdElement = function(rootNodeIdElement) {
    while(rootNodeIdElement.firstChild) {
      rootNodeIdElement.removeChild(rootNodeIdElement.firstChild);
    }
  };
};

function AddDelListWidget(args) {
  this.dataViewMapper = args.dataViewMapper;
  this.addable = (typeof(args.addable)!=='undefined' && args.addable===false) ? false : true;
  this.deletable = (typeof(args.deletable)!=='undefined' && args.deletable===false) ? false : true;

  this.headerCells = [];
  this.dataCells = [];
  this.addCells = [];
  this.addCell = null;
  this.delCells = [];
  this.cells = [];


  this.setHeaderCells = function(cells) {
    this.headerCells = cells;
  };

  this.setDataCells = function(cells) {
    this.dataCells = cells;

    this.initAddCells();
    this.initDelCells();
    this.initCells();
  };

  this.initAddCells = function() {
    this.addCells = [];
    for(var i = 0; i < this.dataCells.length; i++) {
      if(i == this.dataCells.length -1) {
        this.addCells.push(this.createAddButtonElement(i));
      } else {
        this.addCells.push(this.createDummyElement());
      }
    }
  };

  this.initDelCells = function() {
    this.delCells = [];
    for(var i = 0; i < this.dataCells.length; i++) {
      this.delCells.push(this.createDelButtonElement(i));
    }
  };

  this.initCells = function() {
    this.cells = [];
    if(this.headerCells.length>0) {
      this.cells = this.headerCells;
    }
    if(this.dataCells.length == 0) {
      this.cells.push([this.addCell]);
    } else {
      for(var i = 0; i < this.dataCells.length; i++) {
        this.cells.push(this.dataCells[i].concat(this.delCells[i], this.addCells[i]));
      }
    }
  };

  this.createDelButtonElement = function(index) {
    var elementFactory = new DomElementFactory({
      'type': 'button',
      'attributes': this.getDelButtonElementAttributes(),
      'events': [{'event': 'click', 'func': this.delItem.bind(this, index), 'useCapture': true}]
    });
    return elementFactory.create();
  };

  this.getDelButtonElementAttributes = function() {
    var attributes = [{'name': 'class', 'value': 'minus-button'}];
    if(!this.deletable) {
      attributes.push({'name': 'disabled', 'value': 'true'});
    }
    return attributes;
  };

  this.createAddButtonElement = function(index) {
    var elementFactory = new DomElementFactory({
      'type': 'button',
      'attributes': this.getAddButtonElementAttributes(),
      'events': [{'event': 'click', 'func': this.addItem.bind(this, index), 'useCapture': true}]
    });
    return elementFactory.create();
  };

  this.getAddButtonElementAttributes = function() {
    var attributes = [{'name': 'class', 'value': 'plus-button'}];
    if(!this.addable) {
      attributes.push({'name': 'disabled', 'value': 'true'});
    }
    return attributes;
  };

  this.createDummyElement = function() {
    var elementFactory = new DomElementFactory({
      'type': 'spacer',
      'attributes': []
    });
    return elementFactory.create();
  };

  this.populate = function() {
    this.setHeaderCells(this.dataViewMapper.getHeader());
    this.setDataCells(this.dataViewMapper.get());
    var rootNodeIdElement = this.getRootNodeIdElement();
    this.clearRootNodeIdElement(rootNodeIdElement);
    var tableManager = new TableDomManager();
    tableManager.setCells(this.cells);
    var table = tableManager.get();

    rootNodeIdElement.appendChild(table);
  };

  this.addItem = function(index, event) {
    if(event.target.getAttribute('disabled')==="true") {
      return;
    }
    this.dataViewMapper.add(index);
    this.populate();
  };

  this.delItem = function(index, event) {
    if(event.target.getAttribute('disabled')==="true") {
      return;
    }
    this.dataViewMapper.del(index);
    this.populate();
  };

  this.addCell = this.createAddButtonElement(-1);

  this.init(args);
};

AddDelListWidget.prototype = new BaseWidget();
AddDelListWidget.prototype.constructor = AddDelListWidget;


function SelectOneWidget(args) {
  this.items = [];
  this.selectedItem = null;

  this.addItem = function(ElementSelectable) {
    this.items.push(ElementSelectable);
    if(ElementSelectable.isSelected()) {
      this.selectedItem = ElementSelectable;
    }
  };

  this.getOptions = function() {
    var menuitems = [];
    for(var i = 0; i < this.items.length; i++) {
      menuitems.push({
        'attributes': [
           {'name': 'value', 'value': this.items[i].getRootNodeId()},
           {'name': 'label', 'value': this.items[i].getLabel()},
           {'name': 'selected', 'value': this.items[i].isSelected()}
        ],
        'events': [{'event': 'command', 'func': this.selectItem.bind(this, this.items[i].getRootNodeId()), 'useCapture': true}]
      });
    }
    return menuitems;
  };

  this.populate = function() {
    var rootNodeIdElement = this.getRootNodeIdElement();
    this.clearRootNodeIdElement(rootNodeIdElement);
    var menuListManager = new MenuListDomManager();
    menuListManager.setList(this.getOptions());
    var menuList = menuListManager.get();
    rootNodeIdElement.appendChild(menuList);
    if(this.selectedItem!==null) {
      this.selectItem(this.selectedItem.getRootNodeId());
    } else if(this.items.length>0) {
      this.selectItem(this.items[0].getRootNodeId());
    }
  };

  this.selectItem = function(key) {
    for(var i = 0; i < this.items.length; i++) {
      if(this.items[i].getRootNodeId()==key) {
        document.getElementById(this.items[i].getRootNodeId()).style.display = 'block';
      } else {
        document.getElementById(this.items[i].getRootNodeId()).style.display = 'none';
        this.items[i].unset();
      }
    }
  };

  this.init(args);
};

SelectOneWidget.prototype = new BaseWidget();
SelectOneWidget.prototype.constructor = SelectOneWidget;


/*
 * **********************************************************************************************************************************************************
 */


function ListDataViewMapper(attributeListable) {
  this.commonAttributes = [];
  this.newItem = null;

  this.setCommonAttributes = function(attributes) {
    this.commonAttributes = attributes;
  };

  this.setNewItem = function(newItem) {
    this.newItem = newItem;
  };

  this.getNewItem = function(index) {
    return JSON.parse(JSON.stringify(this.newItem));
  };

  this.getHeader = function() {
    return [];
  };

  this.get = function() {
    return [];
  };

  this.changePreCheck = function(row, col, valueModelMapper, event) {
    return true;
  };

  this.change = function(row, col, valueModelMapper, event) {
    if(this.changePreCheck(row, col, valueModelMapper, event)===false) {
      return;
    }
    if(event.target.nodeName=='checkimage') {
      this.attributeListable.change(row, col, !event.target.checked);
    } else {
      this.attributeListable.change(row, col, valueModelMapper(event.target.value));
    }
    this.attributeListable.save();
  };

  this.add = function(index) {
    this.attributeListable.add(this.getNewItem(index));
    this.attributeListable.save();
  };

  this.del = function(index) {
    this.attributeListable.del(index);
    this.attributeListable.save();
  };

  this.valueModelMapper = function(value) {
    return value;
  };
}


function CustomListDataViewMapper(attributeListable) {
  this.attributeListable = attributeListable;
  this.elementsConfig = [];
  this.extraElementTypes = {};

  this.setElementsConfig = function(elementsConfig) {
    this.elementsConfig = elementsConfig;
  };

  this.getElementConfigForRow = function(row) {
    for(var ii = row; ii>=0; ii--) {
      if(this.elementsConfig[ii]!==undefined) {
        return this.elementsConfig[ii];
      }
    }
    return {};
  };

  this.getAttributeListable = function() {
    return this.attributeListable.get();
  };

  this.getHeader = function() {
    var cells = [];
    if (this.attributeListable.columnFilter.length>0) {
      cells.push([]);
      for(var ii = 0; ii<this.attributeListable.columnFilter.length; ii++) {
        if (this.getElementConfigForRow(0)[ii].type === 'hidden') {
          continue;
        }
        if (this.attributeListable.columnHeaders && this.attributeListable.columnHeaders[ii]) {
          cells[0].push(this.createLabelElement(ucfirst(this.attributeListable.columnHeaders[ii])));
        } else {
          cells[0].push(this.createLabelElement(ucfirst(this.attributeListable.columnFilter[ii])));
        }
      }
    }
    return cells;
  };

  this.get = function() {
    var cells = [];
    for(var ii = 0; ii<this.getAttributeListable().length; ii++) {
      cells.push([]);
      for(var jj = 0; jj<this.getAttributeListable()[ii].length; jj++) {
        cells[ii].push(this.createElement(ii, jj));
      }
    }
    return cells;
  };

  this.addExtraElementType = function (type, createFunction)
  {
    this.extraElementTypes[type] = createFunction;
  };

  this.createElement = function(row, col) {
    var elementConfig = this.getElementConfigForRow(row)[col];
    if(elementConfig.type in this.extraElementTypes) {
      return this.extraElementTypes[elementConfig.type].bind(this)(elementConfig, row, col);
    } else if(elementConfig.type=='template') {
      return this.createTemplateElement(elementConfig, row, col);
    } else if(elementConfig.type=='menulist') {
      return this.createMenuListElement(elementConfig, row, col);
    } else if(elementConfig.type=='checkimage') {
      return this.createCheckImageElement(elementConfig, row, col);
    } else if(elementConfig.type=='hidden') {
      return this.createHiddenElement(elementConfig, row, col);
    } else {
      return this.createDefaultElement(elementConfig, row, col);
    }
  };

  this.createEmptyElement = function() {
    var elementFactory = new DomElementFactory({
      'type': 'spacer',
      'attributes': [].concat(this.commonAttributes),
      'events': []
    });
    return elementFactory.create();
  };

  //TODO Refactor
  this.createLabelElement = function(label) {
    var elementFactory = new DomElementFactory({
      'type': 'label',
      'attributes': [{'name': 'value', 'value': label}].concat(this.commonAttributes),
      'events': []
    });
    return elementFactory.create();
  };

  this.createTemplateElement = function(elementConfig, row, col) {
    return elementConfig.cloneFunction(this, elementConfig, row, col);
  };

  this.createMenuListElement = function(elementConfig, row, col) {
    var menuListManager = new MenuListDomManager();
    for(var ii = 0; ii<elementConfig.attributes.length; ii++) {
      if(elementConfig.attributes[ii].name==='editable' && elementConfig.attributes[ii].value==='true') {
        elementConfig.attributes.push({'name': 'value', 'value': this.getAttributeListable()[row][col]});
        elementConfig.events.push({'event': 'change', 'func': this.change.bind(this, row, col, this.valueModelMapper), 'useCapture': true});
        elementConfig.events.push({'event': 'keypress', 'func': this.change.bind(this, row, col, this.valueModelMapper), 'useCapture': true});
        break;
      }
    }
    menuListManager.setMenuListElementAttributes(elementConfig.attributes);
    menuListManager.setMenuListElementEvents(elementConfig.events);
    var options = [];
    for(var ii = 0; ii<elementConfig.menuitems.length; ii++) {
      options.push({
        'attributes': [
          {'name': 'value', 'value': elementConfig.menuitems[ii].value},
          {'name': 'label', 'value': elementConfig.menuitems[ii].label},
          {'name': 'selected', 'value': (this.getAttributeListable()[row][col]==elementConfig.menuitems[ii].value)}
        ],
        'events': [{'event': 'command', 'func': this.change.bind(this, row, col, this.valueModelMapper), 'useCapture': true}]
      });
    }

    menuListManager.setList(options);
    return menuListManager.get();
  };

  this.createCheckImageElement = function(elementConfig, row, col) {
    var elementFactory = new DomElementFactory({
      'type': 'checkimage',
      'attributes': [{'name': 'checked', 'value': this.getAttributeListable()[row][col]}].concat(elementConfig.attributes).concat(this.commonAttributes),
      'events': [{'event': 'click', 'func': this.change.bind(this, row, col, this.valueModelMapper), 'useCapture': true}].concat(elementConfig.events)
    });
    return elementFactory.create();
  };

  this.createHiddenElement = function(elementConfig, row, col) {
    var elem = this.createDefaultElement(elementConfig, row, col);
    elem.style.display = 'none';
    return elem;
  };
  this.createDefaultElement = function(elementConfig, row, col) {
    var value = this.getAttributeListable()[row][col];
    if(elementConfig.valueViewMapper!==undefined) {
      value = elementConfig.valueViewMapper(value);
    }
    var valueModelMapper = (elementConfig.valueModelMapper!==undefined) ? elementConfig.valueModelMapper : this.valueModelMapper;
    var elementFactory = new DomElementFactory({
      'type': elementConfig.type,
      'attributes': [{'name': 'value', 'value': value}].concat(elementConfig.attributes).concat(this.commonAttributes),
      'events': [{'event': 'change', 'func': this.change.bind(this, row, col, valueModelMapper), 'useCapture': true},
                 {'event': 'keyup',  'func': this.change.bind(this, row, col, valueModelMapper), 'useCapture': true}].concat(elementConfig.events)
    });
    return elementFactory.create();
  };
};
CustomListDataViewMapper.prototype = new ListDataViewMapper();
CustomListDataViewMapper.prototype.constructor = CustomListDataViewMapper;


function CustomDualListDataViewMapper(attributeListable) {
  this.attributeListable = attributeListable;

  this.setNewItem = function(newItem) {
    throw 'CustomDualListDataViewMapper.setNewItem sould not be called.';
  };

  this.get = function(type) {
    if(type==='selection') {
      this.getAttributeListable = function() {
        return this.attributeListable.getSelection();
      };
    } else {
      this.getAttributeListable = function() {
        return this.attributeListable.get();
      };
    }
    return CustomDualListDataViewMapper.prototype.get.call(this);
  };

  this.change = function(row, col, valueModelMapper, event) {
    throw 'CustomDualListDataViewMapper.change sould not be called.';
  };

  this.add = function(indexes) {
    this.attributeListable.add(indexes);
    this.attributeListable.save();
  };

  this.del = function(indexes) {
    this.attributeListable.del(indexes);
    this.attributeListable.save();
  };
};
CustomDualListDataViewMapper.prototype = new CustomListDataViewMapper();
CustomDualListDataViewMapper.prototype.constructor = CustomDualListDataViewMapper;


function TextboxListDataViewMapper(attributeListable) {
  this.attributeListable = attributeListable;

  this.get = function() {
    var cells = [];
    for(var ii = 0; ii<this.attributeListable.get().length; ii++) {
      cells.push([]);
      for(var jj = 0; jj<this.attributeListable.get()[ii].length; jj++) {
        var elementFactory = new DomElementFactory({
          'type': 'textbox',
          'attributes': [{'name': 'value', 'value': this.attributeListable.get()[ii][jj]}].concat(this.commonAttributes),
          // CM-6432 need to register change *and* input events
          'events': [{'event': 'change', 'func': this.change.bind(this, ii, jj, this.valueModelMapper), 'useCapture': true},
                     {'event': 'input', 'func': this.change.bind(this, ii, jj, this.valueModelMapper), 'useCapture': true}]
        });

        cells[ii].push(elementFactory.create());
      }
    }
    return cells;
  };

};
TextboxListDataViewMapper.prototype = new ListDataViewMapper();
TextboxListDataViewMapper.prototype.constructor = TextboxListDataViewMapper;


function TemplateListDataViewMapper(listDataViewMapper, elementTemplate, colNodeIdPrefix) {
  this.listDataViewMapper = listDataViewMapper;
  this.elementTemplate = elementTemplate;
  this.colNodeIdPrefix = colNodeIdPrefix;

  this.get = function() {
    var cells = [];
    var listDataViewMapper = this.listDataViewMapper.get();
    for(var ii = 0; ii<listDataViewMapper.length; ii++) {
      cells.push([]);
      var elementCloned = this.elementTemplate.cloneNode(true);
      elementCloned.id = "template_row_" + ii;
      elementCloned.hidden = false;
      for(var jj = 0; jj<listDataViewMapper[ii].length; jj++) {
        elementCloned.getElementsByClassName(this.colNodeIdPrefix + jj)[0].appendChild(listDataViewMapper[ii][jj]);
      }

      cells[ii].push(elementCloned);
    }
    return cells;
  };

  this.change = function(row, col, event) {
    this.listDataViewMapper.change(row, col, event);
  };

  this.add = function(index) {
    this.listDataViewMapper.add(index);
   };

  this.del = function(index) {
    this.listDataViewMapper.del(index);
  };
};
TemplateListDataViewMapper.prototype = new ListDataViewMapper();
TemplateListDataViewMapper.prototype.constructor = TemplateListDataViewMapper;


/*
 * **********************************************************************************************************************************************************
 */


function MenuListElementSelectable(args) {
  this.rootNodeId = args.rootNodeId;
  this.currentObject = args.currentObject;
  this.attribute = args.attribute;
  this.name = args.name;
  this.label = args.label;
  this.unsetFunctions = [];

  this.getRootNodeId = function() {
    return this.rootNodeId;
  };

  this.getLabel = function() {
    return this.label;
  };

  this.isSelected = function() {
    return this.currentObject.get(this.attribute)>0;
  };

  this.unset = function() {
    document.getElementById(this.rootNodeId).selectedIndex = -1;
    for(var i = 0; i < this.unsetFunctions.length; i++) {
      this.unsetFunctions[i]();
    }
  };

  this.registerUnsetFunction = function(func) {
    if (typeof func == 'function') {
       this.unsetFunctions.push(func);
    } else {
      println('Argument is not a function in registerUnsetFunction.');
    }
  };
};


/*
 * **********************************************************************************************************************************************************
 */

function GroupSelectorWidget(args) {
  this.context = null;

  this.rootNodeIdSelection = null;
  this.rootNodeIdElementSelection = null;

  this.rootNodeIdAdd = null;
  this.rootNodeIdElementAdd = null;

  this.rootNodeIdDel = null;
  this.rootNodeIdElementDel = null;

  this.dataViewMapper = args.dataViewMapper;

  this.cellsSelection = [];
  this.cellsSelected = [];

  this.listBoxDomManagerSelection = null;
  this.listBoxDomManager = null;

  var thisInstance = this;

  this.init = function(args) {
    if(args.context!==undefined) {
      this.context = args.context;
    }
    this.rootNodeId = args.rootNodeId;
    this.rootNodeIdSelection = args.rootNodeIdSelection;
    this.rootNodeIdAdd = args.rootNodeIdAdd;
    this.rootNodeIdDel = args.rootNodeIdDel;
  };

  this.getRootNodeIdElement = function() {
    if(this.rootNodeIdElement === null) {
      //this.rootNodeIdElement = document.getElementById(this.rootNodeId);
      this.rootNodeIdElement = this.context.getElementsByTagName(this.rootNodeId)[0];
    }
    return this.rootNodeIdElement;
  };

  this.getRootNodeIdElementSelection = function() {
    if(this.rootNodeIdElementSelection === null) {
      //this.rootNodeIdElementSelection = (this.context || document).getElementById(this.rootNodeIdSelection);
      this.rootNodeIdElementSelection = this.context.getElementsByTagName(this.rootNodeIdSelection)[0];
    }
    return this.rootNodeIdElementSelection;
  };

  this.getRootNodeIdElementAdd = function() {
    if(this.rootNodeIdElementAdd === null) {
      //this.rootNodeIdElementAdd = (this.context || document).getElementById(this.rootNodeIdAdd);
      this.rootNodeIdElementAdd = this.context.getElementsByTagName(this.rootNodeIdAdd)[0];
    }
    return this.rootNodeIdElementAdd;
  };

  this.getRootNodeIdElementDel = function() {
    if(this.rootNodeIdElementDel === null) {
      //this.rootNodeIdElementDel = (this.context || document).getElementById(this.rootNodeIdDel);
      this.rootNodeIdElementDel = this.context.getElementsByTagName(this.rootNodeIdDel)[0];
    }
    return this.rootNodeIdElementDel;
  };

  this.setDataSelectionCells = function(cells) {
    this.cellsSelection = cells;
  };

  this.setDataSelectedCells = function(cells) {
    this.cellsSelected = cells;
  };

  this.addItemCallBack = function() {
    return true;
  };

  this.delItemCallBack = function() {
    return true;
  };

  this.createDelButtonElement = function() {
    var elementFactory = new DomElementFactory({
      'type': 'button',
      'attributes': [
        {'name': 'class', 'value': 'arrowleft-button'}
      ],
      'events': [{'event': 'click', 'func': this.delItem.bind(this), 'useCapture': true}]
    });
    return elementFactory.create();
  };

  this.createAddButtonElement = function() {
    var elementFactory = new DomElementFactory({
      'type': 'button',
      'attributes': [
        {'name': 'class', 'value': 'arrowright-button'}
      ],
      'events': [{'event': 'click', 'func': this.addItem.bind(this), 'useCapture': true}]
    });
    return elementFactory.create();
  };


  this.populate = function() {
    populateSelection();
    populateSelected();
    populateAdd();
    populateDel();
  };

  var populateSelection = function() {
    thisInstance.setDataSelectionCells(thisInstance.dataViewMapper.get('selection'));
    thisInstance.listBoxDomManagerSelection = new ListBoxDomManager();
    thisInstance.listBoxDomManagerSelection.setListBoxAttributes([
      {'name': 'rows', 'value': '6'},
      {'name': 'seltype', 'value': 'multiple'},
      {'name': 'flex', 'value': '1'},
      {'name': 'class', 'value': 'nodelist'}
    ]);
    thisInstance.listBoxDomManagerSelection.setCells([], thisInstance.cellsSelection);
    var listBox = thisInstance.listBoxDomManagerSelection.get();

    var rootNodeIdElementSelection = thisInstance.getRootNodeIdElementSelection();
    thisInstance.clearRootNodeIdElement(rootNodeIdElementSelection);

    rootNodeIdElementSelection.appendChild(listBox);
  };

  var populateSelected = function() {
    thisInstance.setDataSelectedCells(thisInstance.dataViewMapper.get());
    thisInstance.listBoxDomManager = new ListBoxDomManager();
    thisInstance.listBoxDomManager.setListBoxAttributes([
      {'name': 'rows', 'value': '6'},
      {'name': 'seltype', 'value': 'multiple'},
      {'name': 'flex', 'value': '1'},
      {'name': 'class', 'value': 'nodelist'}
    ]);
    thisInstance.listBoxDomManager.setCells([], thisInstance.cellsSelected);
    var listBox = thisInstance.listBoxDomManager.get();

    var rootNodeIdElement = thisInstance.getRootNodeIdElement();
    thisInstance.clearRootNodeIdElement(rootNodeIdElement);

    rootNodeIdElement.appendChild(listBox);
  };

  var populateAdd = function() {
    var rootNodeIdElementAdd = thisInstance.getRootNodeIdElementAdd();
    thisInstance.clearRootNodeIdElement(rootNodeIdElementAdd);
    rootNodeIdElementAdd.appendChild(thisInstance.createAddButtonElement());
  };

  var populateDel = function() {
    var rootNodeIdElementDel = thisInstance.getRootNodeIdElementDel();
    thisInstance.clearRootNodeIdElement(rootNodeIdElementDel);
    rootNodeIdElementDel.appendChild(thisInstance.createDelButtonElement());
  };

  this.addItem = function(event) {
    if(event.target.getAttribute('disabled')==="true") {
      return;
    }
    var selected = this.listBoxDomManagerSelection.getSelected();
    var indexes = [];
    for(var index=0; index<selected.length; index++) {
      if(selected[index]===true) {
        indexes.push(index);
      }
    }
    this.dataViewMapper.add(indexes);
    this.addItemCallBack();
    this.populate();
  };

  this.delItem = function(event) {
    if(event.target.getAttribute('disabled')==="true") {
      return;
    }
    var selected = this.listBoxDomManager.getSelected();
    var indexes = [];
    for(var index=0; index<selected.length; index++) {
      if(selected[index]===true) {
        indexes.push(index);
      }
    }
    this.dataViewMapper.del(indexes);
    this.delItemCallBack();
    this.populate();
  };

  this.init(args);
};

GroupSelectorWidget.prototype = new BaseWidget();
GroupSelectorWidget.prototype.constructor = GroupSelectorWidget;
