'user strict';
global.__base = __dirname + '/../../';
var test  = require('unit.js');
var Context = require('cm-context');
var Configuration = require(global.__base + 'units/configurations/configuration_with_4_nodes.js');


var configuration;
var context;
var ns;

function testSetup() {
  context = new Context(global.__base);
  configuration  = new Configuration(context);
  var base_wizard_ctx = context.load('cm-setup/wizard.js');
  var wizard_ctx = context.load('openstack/install/wizard.js');
  var config_ctx = context.load('openstack/install/config.js');
  var jsondisksetup = context.load('jsondisksetup.js')
  ns             = context.load('openstack/install/nodesSelection.js');
  context.add({
    current : configuration.getCluster()
  });
};


var network_overlays    = [ [ 'neutron', 'OpenStackNetworkNodes'] ];
var controller_overlays = [ [ 'cinder', 'OpenStackControllers' ],
                            ['core', 'OpenStackControllers'], 
                            ['glance', 'OpenStackControllers'],
                            ['horizon', 'OpenStackControllers'],
                            ['keystone', 'OpenStackControllers'],
                            ['neutron', 'OpenStackControllers'],
                            ['nova', 'OpenStackControllers'],
                            ['rabbitmq', 'OpenStackControllers'],
                            ['heat', 'OpenStackControllers'],
                            ['galera', 'GaleraNodes'] ];

var hypervisors_overlays = [ [ 'nova', 'OpenStackHypervisors' ] ];

/* @todo fix software_image_path issue 
describe('Node selection', function(){
  it('hypervisor and controller at the same time is suboptimal', function() {
    testSetup();
    test.bool(ns.wizard.nodesSelection.isSuboptimalNode(configuration.nodes[0],
							true, true,true, false)).isTrue();
  });

  it('hypervisor and network node at the same time is ok', function() {
    testSetup();
    test.bool(ns.wizard.nodesSelection.isSuboptimalNode(configuration.nodes[0],
							true, true,false, false)).isFalse();
  });

  it('controller node and ceph node at the same time is suboptimal', function() {
    testSetup();
    test.bool(ns.wizard.nodesSelection.isSuboptimalNode(configuration.nodes[0],
							false, false,true, true)).isTrue();
  });

  it('controller node diskless is suboptimal', function(){
    testSetup();
    for(var i = 0; i < configuration.nodes.length; i++) {
      configuration.nodes[i].object.category = configuration.disklessCategory.object.uniqueKey;
    }
    ns.wizard.nodesSelection.initDiskSetup(configuration.cluster);
    test.bool(ns.wizard.nodesSelection.isSuboptimalNode(configuration.nodes[0],
							false,
							false,
							true,
							false)).isTrue();
  });

  it('init default with 4 nodes', function(){
    testSetup();
    ns.wizard.nodesSelection.initDefaultConfig();
    var controller_nodes = [ 'node001' ];
    var network_nodes    = [ 'node001' ];
    var hypervisor_nodes = [ 'node002', 'node003', 'node004' ];
    hypervisors_overlays.forEach(function(pair) {
    test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            hypervisor_nodes);
    });
    controller_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            controller_nodes); 
    });
    network_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            network_nodes); 
    });
  });

  it('dont select controller nodes which are down', function(){
    testSetup();
    configuration.nodes[0].status.status = 'DOWN';
    ns.wizard.nodesSelection.initDefaultConfig();
    var controller_nodes = [ 'node002' ];
    var network_nodes    = [ 'node002' ];
    var hypervisor_nodes = [ 'node003', 'node004' ];
    hypervisors_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            hypervisor_nodes);
    });
    controller_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            controller_nodes); 
    });
    network_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            network_nodes); 
    });
  });

  it('dont select controller nodes with less than 2 cpus', function(){
    testSetup();
    configuration.nodes[0].sysInfo.processors = [];
    ns.wizard.nodesSelection.initDefaultConfig();
    var controller_nodes = [ 'node002' ];
    var network_nodes    = [ 'node001' ];
    var hypervisor_nodes = [ 'node001', 'node003', 'node004' ];
    hypervisors_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            hypervisor_nodes);
    });
    controller_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            controller_nodes); 
    });
    network_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            network_nodes); 
    });
  });

  it('dont select controller nodes with diskless setup', function(){
    testSetup();
    configuration.nodes[0].object.category = configuration.disklessCategory.object.uniqueKey;
    ns.wizard.nodesSelection.initDefaultConfig();
    var controller_nodes = [ 'node002' ];
    var network_nodes    = [ 'node001' ];
    var hypervisor_nodes = [ 'node001', 'node003', 'node004' ];
    hypervisors_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            hypervisor_nodes);
    });
    controller_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            controller_nodes); 
    });
    network_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            network_nodes); 
    });
  });

  it('disksetup of node overrides disksetup of category', function(){
    testSetup();
    configuration.nodes[0].object.category = configuration.disklessCategory.object.uniqueKey;
    configuration.nodes[0].object.disksetup = 
      '<?xml version="1.0" encoding="UTF-8"?>' + 
      '<diskSetup>' +
      '<device>' +
      '<partition id="a1">'+
      '<mountPoint>/var/lib</mountPoint>' +
      '<mountOptions>defaults,noatime,nodiratime</mountOptions>' +
      '</partition>' +
      '</device>' +
      '</diskSetup>';
    ns.wizard.nodesSelection.initDefaultConfig();
    var controller_nodes = [ 'node001' ];
    var network_nodes    = [ 'node001' ];
    var hypervisor_nodes = [ 'node002', 'node003', 'node004' ];
    hypervisors_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            hypervisor_nodes);
    });
    controller_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            controller_nodes); 
    });
    network_overlays.forEach(function(pair) {
      test.assert.deepEqual(ns.wizard.common.getConfigOverlay(pair[0],pair[1]).nodes, 
                            network_nodes); 
    });
    
  });
})

*/
