/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = ['RackSensor', 'GenericDevice'];
var singular = 'switch';
var plural = 'switches';

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh otherdevice folder overview" + ' - ' + document.documentURI);
  tree = document.getElementById('tree');
  data = new Array();
  var list = current.find("RackSensor");
  var modified = false;
  for(var i = 0; i < list.length; i++) {
    if(!list[i].get('toBeRemoved') || alsoRemoved) {
      data.push(createTreeItem(list[i]));
      modified = modified || list[i].get('modified');
    }
  }
  var list = current.find("GenericDevice");
  for(var i = 0; i < list.length; i++) {
    if(!list[i].get('toBeRemoved') || alsoRemoved) {
      data.push(createTreeItem(list[i]));
      modified = modified || list[i].get('modified') || list[i].get('toBeRemoved');
    } else {
      modified = true;
    }
  }
  for(var i = data.length; i < MINTREEITEMS; i++) data.push(blankTreeItem());
  var select = selected('list');
  loadTable();
  setSelected('list', select);

  if(modified) {
    document.getElementById('save').removeAttribute("disabled");
  } else {
    document.getElementById('save').setAttribute("disabled", true);
  }
}

function getCurrentSelection() {
  var select = getSelected();
  if(select.length > 0) {
    var list = new Array();
    for(var i = 0; i < select.length; i++) {
      var one = current.find(select[i]);
      if(!!one) list.push(one);
    }
    return [list, 0];
  } else {
    var list = current.findNotToBeRemoved("RackSensor");
    return [list.concat(current.findNotToBeRemoved("GenericDevice")), 0];
  }
}

function getSelected() {
  return selected('list');
}

function createTreeItem(one) {
  var item = new Object();
  item['key'] = one.get('uniqueKey');
  item['modified'] = one.get('modified');
  item['childType'] = one.get('childType');
  if(item['childType'] == 'GenericDevice' && (dock.TreeOtherDevices != dock.DEFAULT_BRANDING)) item['childType'] = dock.TreeOtherDevices;
  item['hostname'] = one.get('hostname');
  item['mac'] = one.get('mac');
  item['tag'] = one.get('tag');
  item['ip'] = one.get('ip');
  var sw = one.get('ethernetSwitch');
  if(sw && sw.ethernetSwitch) {
    item['switchPort'] = one.cluster.resolveName(sw.ethernetSwitch) + ':' + sw.prt;
  } else {
    item['switchPort'] = '';
  }
  item['power'] = '';
  var pdu = one.get('powerDistributionUnits');
  for(var i = 0; i < pdu.length; i++) {
    if(item['power'].length > 0) item['power'] += ', ';
    item['power'] += one.cluster.resolveName(pdu[i].pdu) + ':' + pdu[i].prt;
  }
  return item;
}

function addObject() {
  var args = new Object();
  args['title'] = 'Add new device';
  args['question'] = 'Hostname';
  args['default'] = 'mydevice';
  args['known'] = current.known('Device', 'hostname');
  args['children'] = ['Generic Device', 'Rack Sensor'];
  var d = new dock.cvDialog("newchildobject.xul", args);
  d.onOk = function(name, type) {
    var object;
    switch(type) {
      case('Generic Device'): object = new dock.cvGenericDevice(); break;
      case('Rack Sensor'): object = new dock.cvRackSensor(); break;
      default:
        dock.displayError('Ilegal object type: ' + type);
        return;
    }
    object.hostname = name;
    var base = current.getBasePartition();
    if(isObject(base)) {
      object.partition = base.object.uniqueKey;
    }
    current.addObject(object, dock.openAfterCreate);
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function cloneObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a device for cloning");
  } else if(select.length > 1) {
    dock.displayError("Please select only one device");
  } else {
    var original = current.find(select[0]);
    if(isObject(original)) {
      var args = new Object();
      var host = original.get('hostname');
      args['title'] = 'Clone ' + host;
      args['question'] = 'Hostname';
      var nn = nameNumber(host);
      args['default'] = nn[0];
      args['known'] = current.known('Device', 'hostname');
      var d = new dock.cvDialog("newobject.xul", args);
      d.onOk = function(name, type) {
        var object = original.clone();
        object.hostname = name;
        object.tag = "00000000a000";
        object.mac = "00:00:00:00:00:00";
        object.ip = "0.0.0.0";
        object.ethernetSwitch = new dock.cvSwitchPort();
        object.powerDistributionUnits = new Array();
        cloneSimpleIP(original.object, object);
        current.addObject(object, dock.openAfterCreate);
        refresh();
      }
      dock.dialogs.showDialog(d);
    } else {
      dock.displayError("Original not found, already removed by someone else");
    }
  }
}

function editObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a device");
  } else if(select.length > 1) {
    dock.displayError("Please select only one device");
  } else {
    current.select(select[0]);
  }
}

function removeObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a device");
  } else {
    var args = new Object();
    args['question'] = "Are you sure you want to remove these devices?";
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      for(var i = 0; i < select.length; i++) current.removeObject(select[i]);
      refresh();
    }
    dock.dialogs.showDialog(d);
  }
}
