/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

/**
 * Show customizations tab in the tabpanels pane inside an overlay.
 *  Hides or disables irrelevant UI controls and tabs.
 */
function manageCustomizations()
{
  var tabs = document.getElementById('tabs'),
      tabpanels = document.getElementById('tabpanels'),
      customizationstab = document.createElement('shadowtab'),
      customizationsiframe = document.createElement('iframe');

  document.title += ' - Manage Customizations';

  // For UI simplicity, remove all other tabs for now
  while(tabs.firstChild) tabs.removeChild(tabs.firstChild);
  while(tabpanels.firstChild) tabpanels.removeChild(tabpanels.firstChild);

  var boxes = document.querySelectorAll(['page > hbox button',
                                         'page > hbox textbox#priority'].join(', '));
  for (var i=0; i<boxes.length; i++) {
    boxes[i].setAttribute('disabled', true);
  }
  document.querySelector('button#addrolebutton').className = 'bluebutton';
  document.querySelector('hbox.saverevert').style.display = 'none';

  var iframesrc = 'chrome://cmgui/content/customizations.xul';
  setAttributes(customizationstab, {'subtab': 'true',
                                    'id': 'tab_customizations',
                                    'label': 'Customizations',
                                    'context':'customizations'});
  setAttributes(customizationsiframe, {'id': 'iframe_customizations',
                                       'name' : 'iframe_customizations [' + myDialogIndex + ']',
                                       'src': iframesrc});
  tabs.appendChild(customizationstab);
  tabpanels.appendChild(customizationsiframe);
  changeTabFocus(tabs.getElementsByTagName('shadowtab').length - 1);
}

/**
 * Hide customizations tab in the tabpanels pane inside an overlay.
 *  Restores or re-enables UI controls.
 */
function closeManageCustomizations()
{
  document.title = document.title.replace(' - Manage Customizations', '');
  var boxes = document.querySelectorAll(['page > hbox button',
                                         'page > hbox textbox#priority'].join(', '));
  for (var i=0; i<boxes.length; i++) {
    boxes[i].setAttribute('disabled', false);
  }
  document.querySelector('button#addrolebutton').className = 'bluebuttonAR';
  document.querySelector('hbox.saverevert').style.display = '';

  // for hadoop refresh() also removes the customization tab
  refresh();
  // for openstack we need to do it manually
  if (refreshTabs) {
      refreshTabs();
  }
}

// Make sure that that ok button will close the customizations tab
function doOk()
{
  window.parent.closeManageCustomizations();
}

// Make sure that that cancel button will close the customizations tab
function doCancel()
{
  window.parent.closeManageCustomizations();
}
