/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var current;
var cluster;
var myDialog;
var dock;

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  cluster = myDialog.arguments.cluster;
  current = myDialog.arguments.tracker;
  refresh();
}

function refresh() {
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmproc', 'getBackgroundProcess', current, function(code, data) {
    if(code == 0) {
      setCallbackResult(data);
    } else {
      dock.jsonCallError(this, code, data);
    }
    refresh();
  }, current);
}

function setCallbackResult(pjob) {
  tree = document.getElementById('tree');
  data = new Array();

  if(pjob) {
    document.getElementById("cmd").setAttribute('value', pjob.cmd);
    document.getElementById("user").setAttribute('value', pjob.user);
    var d = new Date(); d.setTime(pjob.startTime * 1000);
    document.getElementById("time").setAttribute('value', d.format('dd/mmm/yyyy HH:MM'));
    var list = pjob.nodeJobs;
    for(var i = 0; i < list.length; i++) {
      data.push(createTreeItem(list[i]));
    }
    for(var i = list.length; i < MINTREEITEMS; i++) data.push(blankTreeItem());
    var select = selected('list');
  } else {
    document.getElementById("refresh").setAttribute('disabled', true);
    document.getElementById("kill").setAttribute('disabled', true);
    document.getElementById("killall").setAttribute('disabled', true);
    document.getElementById("time").setAttribute('value', 'ended');
  }
  loadTable();
  setSelected('list', select);
}

function createTreeItem(one) {
  var item = new Object();
  item['key'] = one.nodeKey;
  var node = cluster.find(one.nodeKey);
  if(node) {
    item['node'] = node.resolveName();
  } else {
    item['node'] = "Not found";
  }
  item['pid'] = one.pid;
  item['cycles'] = one.cycles;
  return item;
}

function pjobKill() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Select a node on which to kill this command", window);
  } else {
    var args = new Object();
    args['question'] = "Are you sure you want to kill this command on these nodes?";
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      dock.workerpool.getWorker(cluster.actualurl).call('cmproc', 'stopBackgroundProcessesOnRunners', [current, select], function(code, data) {
        if(code == 0) {
          stopCallback(data);
        } else {
          dock.jsonCallError(this, code, data);
        }
        refresh();
      }, cluster);
    }
    d.parentWindow = window;
    dock.dialogs.showDialog(d);
  }
}

function pjobKillAll() {
  var args = new Object();
  args['question'] = "Are you sure you want to kill this command?";
  var d = new dock.cvDialog("question.xul", args);
  d.onOk = function() {
    dock.workerpool.getWorker(cluster.actualurl).call('cmproc', 'stopAllBackgroundProcesses', current, function(code, data) {
      if(code == 0) {
        stopCallback(data);
      } else {
        dock.jsonCallError(this, code, data);
      }
      refresh();
    }, cluster);
  }
  d.parentWindow = window;
  dock.dialogs.showDialog(d);
}

function stopCallback(data) {
  var stop = new Array();
  var error = new Array();
  for(var i = 0; i < data.length; i++) {
    switch(data[i].status) {
    case('STOPPED') :
      stop.push(data[i].nodeKey);
      break;
    default: // TODO: more cases?
      error.push(data[i].nodeKey);
    }
  }
  var message = '';
  if(stop.length > 0) message += 'Stopped on nodes: ' + this.current.resolveList(stop) + "\n";
  if(error.length > 0) message += 'Failed to stop on nodes: ' + this.current.resolveList(error) + "\n";
  dock.displayInfo(message, window);
  refresh();
}