/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'PowerDistributionUnit';
var singular = 'power distribution unit';
var plural = 'power distribution units';

function createTreeItem(one) {
  var item = new Object();
  item['key'] = one.get('uniqueKey');
  item['modified'] = one.get('modified');
  item['hostname'] = one.get('hostname');
  item['mac'] = one.get('mac');
  item['tag'] = one.get('tag');
  item['ip'] = one.get('ip');
  var sw = one.get('ethernetSwitch');
  if(sw && sw.ethernetSwitch) {
    item['switchPort'] = one.cluster.resolveName(sw.ethernetSwitch) + ':' + sw.prt;
  } else {
    item['switchPort'] = '';
  }
  return item;
}

function addObject() {
  var args = new Object();
  args['title'] = 'Add new ' + singular;
  args['question'] = 'Name';
  args['default'] = '';
  args['known'] = current.known('Device', 'hostname');
  var d = new dock.cvDialog("newobject.xul", args);
  d.onOk = function(name) {
    var object = new dock.cvPowerDistributionUnit();
    object.hostname = name;
    var base = current.getBasePartition();
    if(isObject(base)) {
      object.partition = base.object.uniqueKey;
    }
    current.addObject(object, dock.openAfterCreate);
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function cloneObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a " + signular + " for cloning");
  } else if(select.length > 1) {
    dock.displayError("Please select only one " + signular);
  } else {
    var original = current.find(select[0]);
    if(isObject(original)) {
      var args = new Object();
      var name = original.get('hostname');
      args['title'] = 'Clone ' + name;
      args['question'] = 'Name';
      var nn = nameNumber(name);
      args['default'] = nn[0];
      args['known'] = current.known('Device', 'hostname');
      var d = new dock.cvDialog("newobject.xul", args);
      d.onOk = function(name) {
        var object = original.clone();
        object.modified = true;
        object.toBeRemoved = false;
        object.hostname = name;
        object.mac = '00:00:00:00:00:00';
        object.tag = '00000000a000';
        object.ip = "0.0.0.0";
        object.ethernetSwitch = new dock.cvSwitchPort();
        object.powerDistributionUnits = new Array();
        cloneSimpleIP(original.object, object);
        current.addObject(object, dock.openAfterCreate);
        refresh();
      }
      dock.dialogs.showDialog(d);
    } else {
      dock.displayError("Original not found, already removed by someone else");
    }
  }
}
