/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function getPuppetTypes() {
  return [
      { 'label': 'String',          'value' : 'String',          'selected' : true },
      { 'label': 'MultilineString', 'value' : 'MultilineString', 'selected' : false },
      { 'label': 'Numeric',         'value' : 'Numeric',         'selected' : false },
      { 'label': 'Boolean',         'value' : 'Boolean',         'selected' : false },
      { 'label': 'Variable',        'value' : 'Variable',        'selected' : false },
      { 'label': 'Undef',           'value' : 'Undef',           'selected' : false },
      { 'label': 'Expression',      'value' : 'Expression',      'selected' : false },
      { 'label': 'Select',          'value' : 'Select',          'selected' : false },
  ];
}

function checkPuppetNumericValue(value) {
  // see also PuppetNumericType in C++
  var re = /^[-+]?[0-9]*\.?[0-9]*$/;
  if(value.match(re)) { return true;  }
  else                { return false; }
}

function checkPuppetVariableValue(value) {
  // see also PuppetVariableType in C++
  value = value.trim();
  var re1 = /^\$?[a-z0-9_][a-zA-Z0-9_]*$/;
  var re2 = /^\$?([a-z][a-z0-9_]*)?(::[a-z][a-z0-9_]*)*::[a-z0-9_][a-zA-Z0-9_]*$/;
  if(value.match(re1) || value.match(re2) ) { 
    return true;  
  }
  else { 
    return false; 
  }
}

function parsePuppetVariableValue(value) {
  if(value.substr(0,1) != '$') 
    value = '$' + value;
  return value;
}

function displayPuppetVariableValue(value) {
  value = value.trim();
  if(value.substr(0,1) != '$') 
    value = '$' + value;
  return value;
}

var inputElementCreator = {

  'String' : function(factory, parameter, id, parentElementId, inputChangedEventListener, field="value") {
    var parent = document.getElementById(parentElementId);
    var textbox = document.createElement('textbox');
    textbox.setAttribute("type", "timed");
    //textbox.setAttribute("width", "420px");
    textbox.setAttribute("flex", "1");
    textbox.setAttribute('id',    id);
    if(inputChangedEventListener) {
      textbox.addEventListener("change",   inputChangedEventListener);
      textbox.addEventListener("keypress", inputChangedEventListener);
    }
    parent.appendChild(textbox);
    parameter.last_type = 'String';
    displayfunction(id, 'value;');
    parsefunction(id, 'value;');
    text(field, false, undefined, undefined, undefined, undefined, id, parameter);
  },

  'MultilineString' : function(factory, parameter, id, parentElementId, inputChangedEventListener, field="value") {
    var parent = document.getElementById(parentElementId);
    var textbox = document.createElement('textbox');
    //textbox.setAttribute("type",       "timed");
    textbox.setAttribute("class",      "textarea");
    textbox.setAttribute("wrap",       "off");
    textbox.setAttribute("flex",       "1");
    textbox.setAttribute('id',         id);
    textbox.setAttribute('multiline',  true);
    textbox.setAttribute("rows",    "6");
    textbox.setAttribute("cols",    "60");
    if(inputChangedEventListener) {
      textbox.addEventListener("change",   inputChangedEventListener);
      textbox.addEventListener("keypress", inputChangedEventListener);
    }
    parent.appendChild(textbox);
    parameter.last_type = 'MultilineString';
    displayfunction(id, 'value;');
    parsefunction(id, 'value;');
    text(field, false, undefined, undefined, undefined, undefined, id, parameter);
  },

  'Numeric' : function(factory, parameter, id, parentElementId, inputChangedEventListener, field="value") {
    var parent = document.getElementById(parentElementId);
    var textbox = document.createElement('textbox');
    textbox.setAttribute("type", "timed");
    //textbox.setAttribute("width", "420px");
    textbox.setAttribute("flex", "1");
    textbox.setAttribute('id',    id);
    if(inputChangedEventListener) {
      textbox.addEventListener("change",   inputChangedEventListener);
      textbox.addEventListener("keypress", inputChangedEventListener);
    }
    parent.appendChild(textbox);
    parameter.last_type = 'Numeric';
    displayfunction(id, 'value;');
    parsefunction(id, 'value;');
    text(field, '@FUNCTION@checkPuppetNumericValue', undefined, undefined, undefined, undefined, id, parameter);
  },

  'Boolean' : function(factory, parameter, id, parentElementId, inputChangedEventListener, field="value") {
    var parent = document.getElementById(parentElementId);
    var checkbox = document.createElement('checkimage');
    checkbox.setAttribute('id', id);
    parent.appendChild(checkbox);
    parameter.last_type = 'Boolean';
    if(inputChangedEventListener) {
      checkbox.addEventListener("custom-change", inputChangedEventListener, true);
    }
    displayfunction(id, 'value;');
    parsefunction(id, 'value;');
    check(field, undefined, undefined, id, parameter);
  },

  'Variable' : function(factory, parameter, id, parentElementId, inputChangedEventListener, field="value") {
    var parent = document.getElementById(parentElementId);
    //var label0 = document.createElement('shadowlabel'); 
    //label0.setAttribute('value', '$');
    //label0.setAttribute('class', 'staticheader');
    //label0.setAttribute('pack', 'center');

    var textbox = document.createElement('textbox');
    textbox.setAttribute("type", "timed");
    //textbox.setAttribute("width", "420px");
    textbox.setAttribute("flex", "1");
    textbox.setAttribute('id',    id);
    if(inputChangedEventListener) {
      textbox.addEventListener("change",   inputChangedEventListener);
      textbox.addEventListener("keypress", inputChangedEventListener);
    }
    //parent.appendChild(label0);
    parent.appendChild(textbox);
    parameter.last_type = 'Variable';
    displayfunction(id, 'parsePuppetVariableValue(value);');
    parsefunction(id, 'parsePuppetVariableValue(value);');
    text(field, '@FUNCTION@checkPuppetVariableValue', undefined, undefined, undefined, undefined, id, parameter);
  },

  'Undef' : function(factory, parameter, id, parentElementId, inputChangedEventListener, field="value") {
    var parent = document.getElementById(parentElementId);
    parameter.last_type = 'Undef';
  },

  'Expression' : function(factory, parameter, id, parentElementId, inputChangedEventListener, field="value") {
    var parent = document.getElementById(parentElementId);
    var textbox = document.createElement('textbox');
    //textbox.setAttribute("type",       "timed");
    textbox.setAttribute("class",      "textarea");
    textbox.setAttribute("wrap",       "off");
    textbox.setAttribute("flex",       "1");
    textbox.setAttribute('id',         id);
    textbox.setAttribute('multiline',  true);
    textbox.setAttribute("rows",    "6");
    textbox.setAttribute("cols",    "60");
    if(inputChangedEventListener) {
      textbox.addEventListener("change",   inputChangedEventListener);
      textbox.addEventListener("keypress", inputChangedEventListener);
    }
    parent.appendChild(textbox);
    parameter.last_type = 'Expression';
    displayfunction(id, 'value;');
    parsefunction(id, 'value;');
    text(field, false, undefined, undefined, undefined, undefined, id, parameter);
  },

  'Select' : function(factory, parameter, id, parentElementId, inputChangedEventListener, field="value") {
    var parent = document.getElementById(parentElementId);
    var menulist = document.createElement('menulist');
    var menupopup = document.createElement('menupopup');
    menulist.appendChild(menupopup);
    menulist.setAttribute('id', id);    
    parent.appendChild(menulist);
    var type_list = [];
    if(factory && factory.parameterLookup.hasOwnProperty(parameter.object.key)) {
      var options = factory.parameterLookup[parameter.object.key].options;
      if(options) 
      {
        try {
          var lst = JSON.parse(options);
          for(var i = 0; i < lst.length; i++) {
            type_list.push([ lst[i], lst[i] ]);
          }
        }
        catch(e) {
          println(e);
        }
      }
    }
    if(inputChangedEventListener) {
      menulist.addEventListener("change", inputChangedEventListener);
      menulist.addEventListener("command", inputChangedEventListener);
    }
    parameter.last_type = 'Select';
    displayfunction(id, 'value;');
    parsefunction(id, 'value;');
    kenum(field, type_list, undefined, id, parameter);
  },
};


function puppetRegisterChangeTypeEvent(typeElementId, parameter, valueElementId, parentElementId, factory,
                                       inputChangedEventListener=null, field="value") {
  var cb = function(elem) {
    var myNode = document.getElementById(parentElementId);
    if(!parameter.hasOwnProperty('backup_value')) {
      parameter.backup_value = {};
    }
    if(parameter.hasOwnProperty('last_type')) {
      parameter.backup_value[parameter.last_type] = parameter.object.value;
    }
    while (myNode.firstChild) {
      myNode.removeChild(myNode.firstChild);
    }

    if(inputElementCreator.hasOwnProperty(elem.target.value)) {
      if(parameter.backup_value.hasOwnProperty(elem.target.value)) {
        parameter.object.value = parameter.backup_value[elem.target.value];
      }
      else {
        parameter.object.value = '';
      }
      inputElementCreator[elem.target.value](factory, parameter, valueElementId, parentElementId, inputChangedEventListener, field);
    }
  };
  var obj = document.getElementById(typeElementId);
  var popup = obj.firstChild;
  popup.addEventListener("command", cb, true);
}



function PuppetFactory(class_key, cluster) {

  this.puppet = null;
  this.cluster = cluster;
  this.key = class_key;

  this.classFactory = null;
  this.puppetClass  = null;
  this.parameterLookup = {};

  var puppets = cluster.find('Puppet');
  if(puppets && puppets.length > 0) {
    this.puppet = puppets[0];
  }
  if(this.puppet) {
    var scannedClasses = this.puppet.get('scannedClasses');
    for(var i = 0; i < scannedClasses.length; i++) {
      if(scannedClasses[i].uniqueKey == class_key) {
        this.puppetClass = scannedClasses[i]; 
      }
    }
  }

  this.getTypesForParameter = function(parameter) {
    return getPuppetTypes();
  }


  this.load = function(done_function, args=null) {
    var ipuppet_class_factory_key = parseInt(this.key);
    var self = this;
    var cluster = this.cluster;
    dock.workerpool.getWorker(cluster.actualurl).call('cmpuppet', 'getPuppetClassFactory', [ ipuppet_class_factory_key ], 
    function(code, data) {
      if(code == 0) {   
        if(data.success) {
          self.classFactory = data.puppetClassFactory;
          data.puppetClassFactory.parameters = data.puppetClassFactory.parameters.filter(function(p) {
            return p.installed;
          });
          data.puppetClassFactory.parameters.sort(function(a,b){
              return a.name > b.name;
          });
          for(var i = 0; i < data.puppetClassFactory.parameters.length; i++) {
            self.parameterLookup[data.puppetClassFactory.parameters[i].name] = data.puppetClassFactory.parameters[i];
          }
          if(typeof(done_function) == 'function') {
            done_function(self, args);
          }
        }
        else {
          dock.displayError("Failed to retrieve details for class / resource " + puppet_class_factory_key);
        }
      }
      else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, null);
  };

  this.shortFormatDefault = function(item) {
    if(item.hasOwnProperty('type')) {
      if(item.type == 'ASTHash') {
        return item.defaultValue;
      }
      else if(item.type == 'ASTArray') {
        return item.defaultValue;
      }
      else {
        return item.defaultValue;
      }
    }
    else {
      return item.defaultValue;
    }
  };

  this.formatType = function(item) {
    if(item.hasOwnProperty('type')) {
      if(item.type == 'ASTHash') {
        return 'Hash';
      }
      else if(item.type == 'ASTArray') {
        return 'Array';
      }
      else {
        return item.type;
      }
    }
    else {
      return 'Unknown';
    }
  }

  this.setDefaultValue = function(name, parameter) {
    if(this.parameterLookup.hasOwnProperty(name)) {
      var param_factory = this.parameterLookup[name];
      if(param_factory.type == 'ASTArray') {
        parameter.type  = "Expression";
        parameter.value = param_factory.options;
      }
      else if(param_factory.type == 'ASTHash') {
        parameter.type = 'Expression';
        parameter.value = param_factory.options;
      }
      else if(inputElementCreator.hasOwnProperty(param_factory.type )) {
        parameter.type  = param_factory.type;
        parameter.value = param_factory.defaultValue;
      }
      else {
        parameter.type  = "String";
        parameter.value = '';
      }
    }
  }
}
