/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var IS_PUPPET_TAB = 1;
var RUNID_OFFSET = 4294967296; // 2^32
var dock;
var current = 0;
var overview;

var timer = 0;

var nodeLookup = null;
var singular = 'node';
var refresh_listener = null;

var status_list = [
            "OkNoChanges",                  //0
            "OkWithChanges",                //1
            "OkFailureAndChanges",          //2
            "Failures",                     //3
            "Unknown",                      //4
            "Signal",                       //5
            "SignalTimeout",                //6
            "NoPuppetSelected",             //7
            "NoNodeSelected",               //8
            "Node down",                    //9
            "CannotCreateTempFile",         //10
            "puppetExecutableDoesNotExist", //11
            "Pending",                      //12
            "Running",                      //13
            "Discarded",                    //14
            "NodeNoLongerUp"                //15
];


function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
  //updateNodeTable();
  //timer = window.setTimeout('updateNodeTable();', dock.generalSettings.statusRefresh * 10000);
}

function getCurrentSelection() {
  return [[current], 0, ""];
}

function stop() {
  println("Stop Puppet overview");
  if(timer) window.clearTimeout(timer);
}

function redraw() {
  if(graph) {
    graph.update();
  }
}

function hidePuppet() {  
  println('hidePuppet');
  if(parent.document.getElementById('nopuppet').hasAttribute('hidden')) {
    parent.document.getElementById('tabbox').setAttribute('hidden', true);
    parent.document.getElementById('nopuppet').removeAttribute('hidden');
    parent.document.getElementById('nopuppet2').removeAttribute('hidden');
  }
}

function showPuppet() {
  println('showPuppet');
  if(parent.document.getElementById('tabbox').hasAttribute('hidden')) {
    parent.document.getElementById('tabbox').removeAttribute('hidden');
    parent.document.getElementById('nopuppet').setAttribute('hidden', true);
    parent.document.getElementById('nopuppet2').setAttribute('hidden', true);
  }
}

/// apply functions
function puppetApply(noop = false) {
  var session = current.cluster.sessionid;
  updateIncludeNoopSelector(noop);
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmpuppet', 'apply', [ current.get('uniqueKey'), session, noop],function(code, data) {
    if(code != 0) {      
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
  }, current);
}

function puppetApplySelected(noop = false)
{
  var select = selected('list');
  updateIncludeNoopSelector(noop);
  puppetApplyList(select, noop);
}

function updateIncludeNoopSelector(noop) {
  var includenoop = document.getElementById('includenoop');
  if(noop && includenoop.value == "0") {
    includenoop.value = "1";
    refresh();
  }
  else if(!noop && includenoop.value == "1") {
    includenoop.value = "0";
    refresh();
  }
}


function puppetApplyList(nodes, noop = false) {
  if(nodes.length == 0) {
    dock.displayError('Select at least one node.');
    return;
  }
  var session = current.cluster.sessionid;
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmpuppet', 'applyList', [ current.get('uniqueKey'), session, nodes, noop ],function(code, data) {
    if(code != 0) {      
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
  }, current);
}

function createRunInfo(obj, resultSession=null)
{
  var hostname = 'unknown';
  if(typeof(nodeLookup[obj.nodeKey]) != 'undefined')
    hostname = nodeLookup[obj.nodeKey].get('hostname');
  var duration = 0;
  if(obj.hasOwnProperty('endTime') && obj.hasOwnProperty('startTime')) {
    duration = obj.endTime - obj.startTime;
    if(duration < 0) duration = 0;
  }
  else if(obj.hasOwnProperty('statusChangedTime') && resultSession) {
    duration = obj.statusChangedTime - resultSession.applyTime;
  }
  var status = 'unknown';
  if(obj.hasOwnProperty('returnStatus') && obj.returnStatus < status_list.length) {
    status = status_list[obj.returnStatus];
  }
  else if(obj.hasOwnProperty('status')) {
    status = obj.status;
  }
  if(status == 'Locked') status = 'Discarded';
  if(obj.hasOwnProperty('noop') && obj.noop) {
    status+= ' (noop)';
  }
  else if(resultSession && resultSession.noop) {
    status+= ' (noop)';
  }

  var formatted_date =' ';
  if(obj.hasOwnProperty('startTime')) {
    var date = new Date();
    date.setTime(obj.startTime * 1000);
    formatted_date = date.format("dd/mmm/yyyy HH:MM:ss");
    
  }
  else if(resultSession) {
    var date = new Date();
    date.setTime(resultSession.applyTime * 1000);
    formatted_date = date.format("dd/mmm/yyyy HH:MM:ss");
  }
  return {
        key        : obj.nodeKey,
        hostname   : hostname,
        status     : status,
        time       : formatted_date,
        duration   : duration,
        nclasses   : obj.numberOfClasses,
        nresources : obj.numberOfResources
  };
}





///////////////////////////////////////////////////
// refresh
///////////////////////////////////////////////////
function refreshLatestRun(rdata) {
  if(rdata == null) {
    overviewValue('runmode',     '  ');
    overviewValue('applytime',   '  ');
    overviewValue('runtime',     '  ');
    overviewValue('nclasses',    '  ');
    overviewValue('nresources',  '  ');
    overviewValue('nochanges',   '  ');
    overviewValue('changes',     '  ');
    overviewValue('mixed',       '  ');
    overviewValue('running',     '  ');
    overviewValue('pending',     '  ');
    overviewValue('failed',      '  ');
    return;
  }
  var runmode = rdata.runMode;
  if(rdata.noop) {
    runmode+= ' (noop)';
  }
  var runtime = '-';
  if(rdata.endTime) {
    if(rdata.shortestRunTime == rdata.longestRunTime) {
      runtime = rdata.shortestRunTime + " sec";
    }
    else {
      runtime = rdata.shortestRunTime + " sec - " + rdata.longestRunTime + " sec";
    }
  }
  var nclasses = 0;
  var nresources = 0;
  var nochanges = 0;
  var changes = 0;
  var mixed = 0;
  var failed = 0;
  var pending = 0;
  var running = 0;
  for(var i =0; i < rdata.resultStatistics.length; i++) {
    switch(i)
    {
    case 0                   : nochanges  = rdata.resultStatistics[i]; break;
    case 1                   : changes    = rdata.resultStatistics[i]; break;
    case 2                   : mixed      = rdata.resultStatistics[i]; break;
    case 13                  : running    = rdata.resultStatistics[i]; break;
    case 12                  : pending    = rdata.resultStatistics[i]; break;
    case status_list.length  : nclasses   = rdata.resultStatistics[i]; break;
    case status_list.length+1: nresources = rdata.resultStatistics[i]; break;
    default: 
      if(i < status_list.length) failed+= rdata.resultStatistics[i];
      break;
    }
  }
  var date = new Date();
  date.setTime(rdata.applyTime * 1000);
  overviewValue('runmode',runmode);
  overviewValue('applytime',   date.format("dd/mmm/yyyy HH:MM:ss"));
  overviewValue('runtime',     runtime);
  overviewValue('nclasses',    nclasses);
  overviewValue('nresources',  nresources);
  overviewValue('nochanges',   nochanges);
  overviewValue('changes',     changes);
  overviewValue('mixed',       mixed);
  overviewValue('running',     running);
  overviewValue('pending',     pending);
  overviewValue('failed',      failed);
}

function refresh(use) {
  println("Refresh puppet overview" + ' - ' + document.documentURI);
  if(!!use) {
    current = use.find('Puppet');
    if(current.length == 1) {
      current = current[0];
        showPuppet();
    } else {
      current = 0;
      hidePuppet();
      return;
    }
  } else if(!current) {
    return;
  }  
  if(!refresh_listener) 
  {
    refresh_listener = function(event) {
      refresh();
    }
    current.cluster.addListener('PuppetApplyDoneEvent',refresh_listener);
    current.cluster.addListener('PuppetApplyStartedEvent',refresh_listener);
  }

  nodeLookup = new Object();
  var nodes = current.cluster.find('Node');
  if(nodes.length) {
    for(var i = 0; i < nodes.length; i++) {
      // check if this can be done in a better way
      nodeLookup[nodes[i].get('uniqueKey')]=nodes[i];
    }
  }
  var displayruns = document.getElementById('displayruns');
  var filter = displayruns.value;
  var includenoop_elem = document.getElementById('includenoop');
  var include_noop = (includenoop_elem && includenoop_elem.value == "1");
  var displayruns_elem = document.getElementById('displayruns');
  var only_latest = (displayruns_elem && displayruns_elem.value =="latest");
  var tree = document.getElementById('tree');
  refreshLatestRun(null);
  loadTable(tree,[]);
  if(only_latest) {
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmpuppet', 'showrun', [ 0, include_noop ], 
      function(code, rdata) {
        if(code == 0) {
          refreshLatestRun(rdata.resultSession);
          data = new Array();
          for(var i = 0; i < rdata.resultList.length; i++) {
            data.push(createRunInfo(rdata.resultList[i], rdata.resultSession));
          }
          loadTable(tree,data);
        }
        else {
          loadTable(tree,[]);
          dock.jsonCallError(this, code, data);
        }
        dock.unwaitIcon();
      },
      current);
  }
  else {
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmpuppet', 'listruns', [ 1, include_noop ], 
      function(code, rdata) {
        if(code == 0) {      
          if(rdata.length == 1) {
            refreshLatestRun(rdata[0]);
          }
          else {
            loadTable(tree,[]);
            refreshLatestRun(null);
          }
        } 
        else {
          loadTable(tree,[]);
          refreshLatestRun(null);
          dock.jsonCallError(this, code, rdata);
          dock.unwaitIcon();
        }
      }, current);
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmpuppet', 'getLatestRunPerNode', [include_noop], 
      function(code, ldata) {
        if(code == 0) {      
          data = new Array();
          for(var i = 0; i < ldata.length; i++) {
            data.push(createRunInfo(ldata[i]));
          }
          loadTable(tree,data);
        }
        else {
          loadTable(tree,[]);
          refreshLatestRun(null);
        }
      },current);
  }
}
  

function showMessageOnlyOneNode()
{
    dock.displayInfo('Please select only one node.');
}

function getLogFromNode(node, run, includenoop) {
  if(nodeLookup && typeof(nodeLookup[node]) != 'undefined')  {
    puppetGetLogFromNode(current,nodeLookup[node],0, run);
  }
}

////////////////////////////////////
// popup menu
////////////////////////////////////
function editSubObject()
{
    var select = selected('list');
}

function addMenuItem(parent, text, command) {
  var item = document.createElement("menuitem");
  item.setAttribute('label', text);
  item.setAttribute('oncommand', command);
  parent.appendChild(item);
}

function addMenuSeparator(parent) {
  var item = document.createElement('menuseparator');
  parent.appendChild(item);
}

function getLogFromNodeSelected()
{
  var select = selected('list');
  if(select.length == 1) 
  {
    var includenoop = document.getElementById('includenoop').value == "1";
    getLogFromNode(parseInt(select[0]),-1, includenoop);
  }
  else
  {
    showMessageOnlyOneNode();
  }
}


function changeOverviewTreePopup(event) {
    var popup = event.target;
    while(popup.firstChild) popup.removeChild(popup.firstChild);
    var select = selected('list');
    var logfrom_node_code = 'showMessageOnlyOneNode();';
    if(select.length == 1) {
      var includenoop = document.getElementById('includenoop').value == "1" ? 'true' : 'false';
      logfrom_node_code = 'getLogFromNode('+ select[0] +',0,' + includenoop + ');';
    }
    addMenuItem(popup, 'Puppet code / log',  logfrom_node_code);    
    addMenuItem(popup, 'Puppet apply', 'puppetApplyList([' + select.join(",") + ']);');
    return true;
}
