
var PuppetPreview = function() {
  this.node_tab_prefix   = "node_tab_";
  this.node_panel_prefix = "node_panel_";
  this.cluster           = null;

}

PuppetPreview.prototype.preview = function (runnoop, nodes) {
  if(!current) return;
  this.cluster = current.isCluster ? current : current.cluster;
  var puppets = this.cluster.find('Puppet');
  if(puppets.length == 0) return;
  var puppet = puppets[0];
  var puppet_key = parseInt(puppet.get('uniqueKey'));
  if(nodes.length == 0) {
    dock.displayError("Please select at least one node.");
    return;
  }
  var overlays = this.cluster.find('ConfigurationOverlay');
  var changes  = [];
  var to_be_removed = [];

  for(var i = 0; i < overlays.length; i++) {
    if(overlays[i].object.toBeRemoved) {
      for(var k = 0; k < roles.length; k++) {
        if(roles[k].childType == 'PuppetRole') {
          to_be_removed.push(parseInt(overlays.get('uniqueKey')));
        }
      }
    }
    else if(overlays[i].get('modified')) {
      var roles = overlays[i].get('roles');
      for(var k = 0; k < roles.length; k++) {
        if(roles[k].childType == 'PuppetRole') {
          var change  = new dock.cvPuppetRoleChange();
          change.targetType = 'TargetOverlay';
          change.uniqueKey  = overlays[i].get('uniqueKey');
          change.name       = overlays[i].get('name');
          change.nodes      = overlays[i].get('nodes');
          change.priority   = overlays[i].get('priority');
          change.puppetRole = roles[k];
          changes.push(change);
        }
      }
    }
  }
  var session = this.cluster.sessionid;
  var self = this;
  if(runnoop) {
    var listener_id = this.cluster.addListener('PuppetApplyDoneEvent',function(event){
      if(session == event.resultSession.clientSession) {
        self.cluster.removeListener('PuppetApplyDoneEvent', listener_id);
        self.puppetPreviewHandleApplyResult(event);

      }
    });
  }
  dock.workerpool.getWorker(this.cluster.actualurl).call('cmpuppet', 'applyCompileList', 
                                                    [ puppet_key, 
                                                      session, 
                                                      nodes, 
                                                      changes, 
                                                      to_be_removed, 
                                                      runnoop ],   
                                                    function(code, data) {
                                                      if(code == 0)  {   
                                                        self.displayPuppetPreview(data,nodes,runnoop,session);
                                                      } else {
                                                        dock.jsonCallError(this, code, data);
                                                      }
                                                      dock.unwaitIcon();
                                                    }, current);
}

PuppetPreview.prototype.removePuppetPreviewTabs = function() {
  var tabs = document.getElementById('tabs');   
  var tabpanels = document.getElementById('tabpanels');   
  var child_itr = tabs.firstChild;
  while(child_itr) {
    var next_child = child_itr.nextSibling;
    if(startswith(child_itr.getAttribute('id'),this.node_tab_prefix))
      tabs.removeChild(child_itr); 
    child_itr = next_child;
  }
  var child_itr = tabpanels.firstChild;
  while(child_itr) {
    var next_child = child_itr.nextSibling;
    if(startswith(child_itr.getAttribute('id'),this.node_panel_prefix))
      tabpanels.removeChild(child_itr);       
    child_itr = next_child;
  }
}

PuppetPreview.prototype.displayPuppetPreview=function(data, nodes, noop,session) {
  this.removePuppetPreviewTabs();
  var tabs = document.getElementById('tabs');   
  var tabpanels = document.getElementById('tabpanels');   

  var ind = tabs.children.length;
  for(var i = 0; i < data.output.length; i++) {
    if(i < nodes.length) {
      var shadowtab  = document.createElement('shadowtab');
      var textbox    = document.createElement('textbox');
      var has_puppet = false;
      textbox.setAttribute('flex', '1');
      shadowtab.setAttribute('subtab', true);
      shadowtab.setAttribute('id', this.node_tab_prefix + i);
      shadowtab.setAttribute('label', this.cluster.resolveName(nodes[i]));
      if(i >= data.success.length || !data.success[i]) {
        textbox.setAttribute("value", 
                             "Failed to aggregate puppet manifest:\n" + 
                             data.output[i]);
        textbox.setAttribute("style", "color: #c85a2f");
      }
      else if(i >= data.nodeHasPuppetRole.length || !data.nodeHasPuppetRole[i]) {
        textbox.setAttribute("value", 
                             "There is no puppet class / resource assigned to this node.");
        textbox.setAttribute("style", "color: #c85a2f");
      }

      else {
        textbox.setAttribute("value", data.output[i]);
        has_puppet = true;
      }
      textbox.setAttribute("multiline", "true");
      textbox.setAttribute("readonly",  "true");
      var waiting = null;
      if(has_puppet) {
        if(noop ) {
          waiting = this.puppetPreviewCreateWaitingIconElement(nodes[i]);
        }
      }
      var grid = this.puppetPreviewCreateResultGrid(textbox, waiting)
      grid.setAttribute('id', this.node_panel_prefix + i);
      tabs.appendChild(shadowtab);
      tabpanels.appendChild(grid);
    }
  }
  if(ind >= tabs.children.length) ind = 0;
  document.getElementById("tabbox").selectedIndex = ind;
  //refresh();
}

PuppetPreview.prototype.puppetPreviewCreateResultGrid = function (textbox, waiting)
{
  var grid = document.createElement('vbox');
  grid.setAttribute("flex","1");
  var vbox = document.createElement('vbox');
  vbox.setAttribute("flex","1");
  vbox.setAttribute('class','staticdata');
  vbox.appendChild(textbox);
  grid.appendChild(vbox);
  if(waiting) {
    grid.appendChild(waiting);
  }
  return grid;
}

PuppetPreview.prototype.puppetPreviewCreateWaitingIconElement = function(nodekey)
{
  var vbox = document.createElement('vbox');
  vbox.setAttribute("flex","1");
  vbox.setAttribute('class','staticdata');
  var hbox = document.createElement('hbox');
  hbox.setAttribute("flex","0");
  var image = document.createElement('image');
  image.setAttribute('src', 'chrome://cmgui/content/icons/throbber.gif');
  image.setAttribute('id', 'resultimage_' + nodekey);
  var label = document.createElement('shadowlabel');
  label.setAttribute('class', 'staticdatavalue');
  label.setAttribute('value', "Executing puppet apply with NOOP on the node.");
  label.setAttribute('id', 'resultmessage_' + nodekey);
  hbox.appendChild(image);
  hbox.appendChild(label);
  vbox.appendChild(hbox);
  var textbox    = document.createElement('textbox');
  textbox.setAttribute('flex', '1');
  textbox.setAttribute('readonly', true);
  textbox.setAttribute("multiline", "true");
  textbox.setAttribute('id','logtextbox_' + nodekey);  
  vbox.appendChild(textbox);
  return vbox;
}

PuppetPreview.prototype.puppetPreviewHandleApplyResult = function (event) {
  var result_session = event.resultSession.uniqueKey;
  for(var i = 0; i < event.result.length; i++) {
    var nodekey = event.result[i].nodeKey;
    var image = document.getElementById('resultimage_' + nodekey);
    var label = document.getElementById('resultmessage_' + nodekey);
    if(image && label) {

      if(event.result[i].status == 'OkNoChanges') { 
        image.setAttribute('src', 'chrome://cmgui/content/icons/ok.png');      
        label.setAttribute('value', 'Puppet apply noop has been successfully applied.');
      }
      else {
        image.setAttribute('src', 'chrome://cmgui/content/icons/alert.png');
        label.setAttribute('value', 'Failed to apply Puppet noop. Status:' + event.result[i].status);
      }
    }
    this.puppetPreviewGetLogFileContent(nodekey,result_session);
  }
}

PuppetPreview.prototype.puppetPreviewGetLogFileContent = function (node,result_session) {
  var self = this;
  dock.workerpool.getWorker(this.cluster.actualurl).call('cmpuppet', 'getPuppetOutput', [ node, result_session, 0, true, [ false, true ] ], 
         function(code, data) {
          if(code == 0) 
           {      
             self.puppetPreviewShowPuppetLog(data, node);
          } else {
              dock.jsonCallError(this, code, data);
          }
          dock.unwaitIcon();
      }, current);
}

PuppetPreview.prototype.puppetPreviewShowPuppetLog = function (data, nodekey) {
  var textbox =  document.getElementById('logtextbox_' + nodekey);
  if(data.success.length >=2 && data.success[1] && data.content.length >=2) {
    var str = data.content[1].replace(/\x1B.*?m/g,'');
    //\[.*?m
    textbox.setAttribute('value', str);
  }
  else {
    textbox.setAttribute('value', 'Could not retrieve log file from node ' + 
                         this.cluster.resolveName(nodekey));
  }
}




