/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;
var openItems = {};

var basetype = 'PuppetClass';
var subfield = 'scannedModules';
var singular = 'puppet class';
var plural = 'puppet classes';

function specialUse(use) {
  if(!!use) {
    current = use.find('Puppet');
    if(current.length == 1) {
      current = current[0];
    } else {
      current = 0;
      return false;
    }
  } else if(!current) {
    return false;
  }
  return true;
}


function refresh(use) {
  println("Refresh " + singular + ' - ' + document.documentURI);
  if(typeof(specialUse) == 'function') {
    if(!specialUse(use)) {
      println("Refresh " + singular + ": specialUse failed");
      return;
    }
  } else if(!!use) {
    current = use;
  }
  
  // Todo:
  //var assignment = getPuppetRoleAssignment();
  tree = document.getElementById('tree');
  var list = current.get(subfield);
  var classes = current.get('scannedClasses');
  var modules2classes = {};
  for(var i = 0; i < classes.length; i++) {
    if(typeof(modules2classes[classes[i].moduleName]) == 'undefined') {
      modules2classes[classes[i].moduleName] = [];
    }
    modules2classes[classes[i].moduleName].push(classes[i]);
  }
  data = new Array();
  for(var i = 0; i < list.length; i++) { 
    if(typeof(modules2classes[list[i].name]) == 'undefined') {
      modules2classes[list[i].name] = [];
    }
    data.push(createPuppetModuleTreeItem(list[i],i,modules2classes[list[i].name]));
  }
  loadTable(tree,data);
  //setSelected('list', select);
  //if(typeof(postrefresh) == 'function') {
  //return postrefresh(use);
  //}
  return true;
}

function createPuppetClassTreeItem(one,i,assignment,this_is_assigned={ key:0, assigned:{}}) {
  var firstAppeared = '';
  if(one.firstAppeared) {
    var date = new Date();
    date.setTime(one.firstAppeared * 1000);
    firstAppeared = date.format("dd/mmm/yyyy HH:MM:ss");
  }
  var type = "";
  if(one.puppetType == "BuiltinTypes") type = "Builtin";
  else type = one.puppetType;
  return {
    key:            one.uniqueKey,
    name:           one.name,
    exists:         one.exists,
    puppetType:     type,
    modified:       one.modified,
    toBeRemoved:    one.toBeRemoved,
    moduleVersion:  one.moduleVersion,
    file:           one.file,
    description:    one.description,
    firstAppeared:  firstAppeared
  };
}

function createPuppetModuleTreeItem(one, i, classes) {
  var children = [];

  for(var j = 0; j < classes.length; j++) {
    var obj = new Object();
    children.push(createPuppetClassTreeItem(classes[j], j));
  }
  return {
    key             : one.uniqueKey,
    name            : one.name + " " + one.version,
    exists          : one.installed,
    version         : one.version,
    details         : one,
    puppetType      : "Module",
    treeIsContainer : true,
    treeIsOpen      : (openItems[one.name]) ? true : false,
    treeOnOpenClose : function(open, data) { openItems[one.name]=open; },
    treeChildren    : children
  }
}


function showDetails() {
    var select = selected('list');
    if(select.length != 1) 
    {
        dock.displayError("Please select one item.");
    } 
    else 
    {
        for(var i = 0; i < data.length; i++) 
        {
            if(data[i].key == select[0])
            {
                showModuleDetails(data[i].details);
                return;
            }
            if(typeof data[i].treeChildren != "undefined") 
            {
                for(var j = 0; j < data[i].treeChildren.length; j++) 
                {
                    if(data[i].treeChildren[j].key == select[0]) 
                    {
                        showClassDetails(data[i].details, data[i].treeChildren[j]);
                    }
                }
            }
        }
    }
}

function showModuleDetails(module_obj)  
{
    var args = {
        moduleData: module_obj
    }
    var d = new dock.cvDialog("chrome://cmgui/content/puppet/puppetclassdetails.xul", args);
    dock.dialogs.showDialog(d);
}

function showClassDetails(module_obj, obj)  
{
  var factory = new PuppetFactory(obj.key, current.cluster);
  factory.load(function(_factory) { 
    var args = {
      moduleData  : module_obj,
      classData   : obj,
      factory     : _factory
    };
    var d = new dock.cvDialog("chrome://cmgui/content/puppet/puppetclassdetails.xul", args);
    dock.dialogs.showDialog(d);
  });
}

function rescan() {
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmpuppet', 'rescanPuppets', current.get('uniqueKey'), function(code, data) {
    if(code == 0) {      
      dock.displayInfo('Rescan started in the background');
    } else {
      dock.jsonCallError(this, code, data);
    }
    dock.unwaitIcon();
  }, current);
}

//////////////////////////////////////////////////////////
function changeOverviewTreePopup(event) {
    var popup = event.target;
    while(popup.firstChild) popup.removeChild(popup.firstChild);
    var item = document.createElement("menuitem");
    item.setAttribute('label',     'show details');
    item.setAttribute('oncommand', 'showDetails()');
    popup.appendChild(item);
    return true;
}



