var current;
var dock;
var search_listener_id = null;
var install_listener_id = null;
var pagination_limit = 20;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function onUnload() {
  println('.Unload: ' + document.title + ' - ' + document.documentURI);
  if(current && current.cluster) {
    current.cluster.removeListener('PuppetForgeSearchDoneEvent',       search_listener_id);
    current.cluster.removeListener('PuppetForgeInstallationDoneEvent', install_listener_id);
  }
}

function refresh(use)
{
  println("Refresh puppet forge" + ' - ' + document.documentURI);
  if(!!use) {
    var old_session = 0;
    var old_cluster;
    if(current && current.cluster) {
      old_session = current.cluster.sessionid;
      old_cluster = current.cluster;
    }
    current = use.find('Puppet');
    if(current.length == 1) {
      current = current[0];
      if(current && current.cluster && current.cluster.sessionid != old_session) {
        if(old_cluster) {
          old_cluster.removeListener('PuppetForgeSearchDoneEvent',       search_listener_id);
          old_cluster.removeListener('PuppetForgeInstallationDoneEvent', install_listener_id);
          search_listener_id = null;
          install_listener_id = null;
        }
        cleanResultSet();
      }
      if(install_listener_id === null) {
        install_listener_id = current.cluster.addListener('PuppetForgeInstallationDoneEvent',puppetForgeInstallationDoneHandler);
      }
      if(search_listener_id === null) {
        search_listener_id = current.cluster.addListener('PuppetForgeSearchDoneEvent',puppetForgeSearchDoneHandler);
      }
    } 
    else {
      current = 0;
      if(old_cluster) {
          old_cluster.removeListener('PuppetForgeSearchDoneEvent',       search_listener_id);
          old_cluster.removeListener('PuppetForgeInstallationDoneEvent', install_listener_id);
          search_listener_id = null;
          install_listener_id = null;
      }
      cleanResultSet();
      return;
    }
  } 
  else if(!current) {
    cleanResultSet();
    return;
  }  
}

function puppetForgeInstallationDoneHandler(event, id) {
  if(event.request.command == "CHANGES")
  {
    showPuppetForgeChanges(event.request);
  }
  else 
  {
    displayInstallationStatus(event.request);
  }
}

function puppetForgeSearchDoneHandler(event, id) {
  showSearchResult(event.request,event.result, event.pagination);
}

/***
  <vbox flex="0" class="staticdata">
    <grid>
      <columns>
        <column flex="1"/>
        <column flex="1"/>
        <column flex="1"/>
      </columns>
      <rows>
        <row height="35px" align="top">
          <textbox id="name" type="" width="300px"/>
          <button class="bluebutton" label="Search Puppet Forge" accesskey="S" oncommand="searchPuppetForge();"/>
          <button class="bluebutton" label="Show Installed Modules" accesskey="I" oncommand="alert('not supprted');"/>
        </row>
      </rows>
    </grid>
    <grid>
      <columns>
        <column flex="1"/>
        <column flex="1"/>
        <column flex="1"/>
      </columns>
      <rows>
        <row height="35px" align="top">
          <menulist id="operatingsystem" onselect="refresh();">
            <menupopup>
              <menuitem label="Any Operating System" value=""/>
              <menuitem label="XXX" value="XXX"/>
            </menupopup>
          </menulist>
          <menulist id="puppetversion" onselect="refresh();">
            <menupopup>
              <menuitem label="Any Puppet Version" value=""/>
              <menuitem label="XXX" value="XXX"/>
            </menupopup>
          </menulist>
          <menulist id="puppetversion" onselect="refresh();">
            <menupopup>
              <menuitem label="Approved or Supported" value=""/>
              <menuitem label="Approved" value="approved"/>
              <menuitem label="Supported" value="supported"/>
            </menupopup>
          </menulist>
        </row>
      </rows>
    </grid>
  </vbox>
***/

/** 
 *  <menulist>
 *   <menupopup>
 *     <menuitem label=".." value=".."/>
 *     ...
 *   </menupopup>
 *  </menulist> 
 */

///////////////////////////////////////////////////////////////////////////////////////////////////////
//
// installation functions
//
///////////////////////////////////////////////////////////////////////////////////////////////////////
function getPuppetInstallationFlags(module_name, possible_flags) {
  var versions_menulist = document.getElementById('versions-menulist@' + module_name);
  var ret = [];
  for(var i = 0; i < possible_flags.length; i++) {
    var check_box = document.getElementById('search-result-install-' + possible_flags[i] + '@' + module_name);
    if(check_box && check_box.checked)
    {
      println('--' + possible_flags[i]);
      ret.push('--' + possible_flags[i]);
    }
  }
  return ret;
}

function installUpgradeModule(module_name) {
  var versions_menulist = document.getElementById('versions-menulist@' + module_name);
  var version = versions_menulist.value;
  var puppetKey = parseInt(current.get('uniqueKey'));
  var sessionid = parseInt(current.cluster.sessionid);
  versions_menulist.setAttribute("disabled", true);

  var install_upgrade_button = document.getElementById('search-result-install-button@' + module_name);
  var uninstall_button = document.getElementById('search-result-uninstall-button@' + module_name);
  install_upgrade_button.setAttribute("disabled-backup", install_upgrade_button.hasAttribute("disabled"));
  uninstall_button.setAttribute("disabled-backup", uninstall_button.hasAttribute("disabled"));
  install_upgrade_button.setAttribute("disabled", true);
  uninstall_button.setAttribute("disabled", true);

  displayInstallationRequest(module_name, "Module is being installed. Please wait.", false);

  var func_name = 'puppetModuleInstall';
  var flags = [];
  if(versions_menulist.puppet_forge_releases[version].isUpgrade) 
  {
    func_name = 'puppetModuleUpgrade';
    flags = getPuppetInstallationFlags(module_name, ["force","ignore-dependencies", "ignore-changes"]);
  }
  else if(versions_menulist.puppet_forge_releases[version].isDowngrade) 
  {
      func_name = 'puppetModuleDowngrade';
      flags = getPuppetInstallationFlags(module_name, ["force","ignore-dependencies", "ignore-changes"]);
  }
  else 
  {
    flags = getPuppetInstallationFlags(module_name, ["force","ignore-dependencies","ignore-changes"]);
  }
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmpuppet', 
                                                            func_name,
                                                            [ puppetKey,sessionid ,module_name, version, flags ],
    function(code, data) {
      if(code == 0) 
      { 
        if(!data.success) {
          enable_input_fields(module_name);
          displayInstallationRequest(module_name, data.message, true);
          //current.cluster.removeListener('PuppetForgeSearchDoneEvent', listener_id);
          dock.displayError("Failed to commit search request " + data.message);                  
        }
      } 
      else {
        enable_input_fields(module_name);
        //current.cluster.removeListener('PuppetForgeSearchDoneEvent', listener_id);
        displayInstallationRequest(module_name, "failed", true);
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
}

function uninstallPuppetModule(module_name)
{
  var puppetKey = parseInt(current.get('uniqueKey'));
  var sessionid = parseInt(current.cluster.sessionid);
  var versions_menulist = document.getElementById('versions-menulist@' + module_name);
  var version = versions_menulist.value;
  versions_menulist.setAttribute("disabled", true);
  var install_upgrade_button = document.getElementById('search-result-install-button@' + module_name);
  var uninstall_button = document.getElementById('search-result-uninstall-button@' + module_name);
  install_upgrade_button.setAttribute("disabled-backup", install_upgrade_button.hasAttribute("disabled"));
  uninstall_button.setAttribute("disabled-backup", uninstall_button.hasAttribute("disabled"));
  install_upgrade_button.setAttribute("disabled", true);
  uninstall_button.setAttribute("disabled", true);

  displayInstallationRequest(module_name, "Module is being installed. Please wait.", false);

  var flags = getPuppetInstallationFlags(module_name, [ "force",
                                                        "ignore-changes" ] );

  dock.workerpool.getWorker(current.cluster.actualurl).call('cmpuppet', 
                                                            'puppetModuleUninstall', 
                                                            [ puppetKey,sessionid ,module_name, "", flags ],
    function(code, data) {
      if(code == 0) 
      { 
        if(!data.success) {
          enable_input_fields(module_name);
          displayInstallationRequest(module_name, data.message, true);
          //current.cluster.removeListener('PuppetForgeSearchDoneEvent', listener_id);
          dock.displayError("Failed to commit search request " + data.message);                  
        }
      } 
      else {
        enable_input_fields(module_name);
        //current.cluster.removeListener('PuppetForgeSearchDoneEvent', listener_id);
        displayInstallationRequest(module_name, "failed", true);
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
}

function displayInstallationRequest(module_name, message, is_error) {
  var vbox = document.getElementById('search-result@' + module_name);
  if(vbox) {
    while (vbox.childNodes.length > 1) {
      vbox.removeChild(vbox.lastChild);
    }
    var grid = document.createElement('grid');
    var columns = document.createElement('columns');
    var column0 = document.createElement('column');
    column0.setAttribute('flex','0');
    columns.appendChild(column0);
    var column1 = document.createElement('column');
    column1.setAttribute('flex','1');
    columns.appendChild(column1);
    var rows = document.createElement('rows');
    var row = document.createElement('row');
    var image = document.createElement('image');
    if(is_error) {
      image.setAttribute('src', 'chrome://cmgui/content/icons/alert.png');
    }
    else {
      image.setAttribute('src', 'chrome://cmgui/content/icons/throbber.gif');        
    }
    row.appendChild(image);
    var status_element = document.createElement("shadowlabel");
    status_element.setAttribute("value", message);    
    if(is_error) {
      image.setAttribute('class','');
    }
    row.appendChild(status_element);
    rows.appendChild(row);
    grid.appendChild(columns);    
    grid.appendChild(rows);
    vbox.appendChild(grid);
  }
}


function enable_input_fields(module_name) {
  var versions_menulist = document.getElementById('versions-menulist@' + module_name);
  if(versions_menulist) versions_menulist.removeAttribute("disabled");
  var install_upgrade_button = document.getElementById('search-result-install-button@' + module_name);
  var uninstall_button = document.getElementById('search-result-uninstall-button@' + module_name);
  if(install_upgrade_button) {
    if(install_upgrade_button.hasAttribute("disabled-backup") && 
       install_upgrade_button.getAttribute("disabled-backup")) {
      install_upgrade_button.removeAttribute("disabled");
    }
  }
  if(uninstall_button) {
    if(uninstall_button.hasAttribute("disabled-backup") && 
       uninstall_button.getAttribute("disabled-backup")) {
      uninstall_button.removeAttribute("disabled");
    }
  }
}

function displayInstallationStatus(request) {
  var vbox = document.getElementById('search-result@' + request.module);
  if(vbox) {
    while (vbox.childNodes.length > 1) {
      vbox.removeChild(vbox.lastChild);
    }
    var grid = document.createElement('grid');
    var columns = document.createElement('columns');
    var column0 = document.createElement('column');
    column0.setAttribute('flex','0');
    columns.appendChild(column0);
    var column1 = document.createElement('column');
    column1.setAttribute('flex','1');
    columns.appendChild(column1);

    var rows = document.createElement('rows');
    if(request.installState == 'FAILED') {
      var row = document.createElement('row');
      var image = document.createElement('image');
      image.setAttribute('src', 'chrome://cmgui/content/icons/alert.png');
      var errmsg = 'Installation of ' + request.module + ' ' + request.version + ' failed: ';
      var long_error_msg = request.longError;

      var error_element = document.createElement("shadowlabel");
      error_element.setAttribute("value", errmsg);
      error_element.setAttribute("class", "staticdataheader");
      vbox.appendChild(error_element);       
      
      row.appendChild(image);
      row.appendChild(error_element);
      rows.appendChild(row);
      var res = long_error_msg.split("\n");
      for(var i = 0; i < res.length; i++) {
        var row = document.createElement('row');
        var empty = document.createElement("shadowlabel");
        var long_error_element = document.createElement("shadowlabel");
        long_error_element.setAttribute("value", res[i].trim().replace(/\x1B.*?m/g,''));
        long_error_element.setAttribute("class", "staticdatavalue");
        row.appendChild(empty);        
        row.appendChild(long_error_element);
        rows.appendChild(row);
      }
      enable_input_fields(request.module);
    }
    else if(request.installState == 'DONE') {
      if(request.command == 'INSTALL' || 
         request.command == 'UPGRADE' || 
         request.command == 'UNINSTALL' ||
         request.command == 'DOWNGRADE' ) {
        var versions_menulist = document.getElementById('versions-menulist@' + request.module);
        versions_menulist.setAttribute('enabled',false);
        var row = document.createElement('row');
        var image = document.createElement('image');
        image.setAttribute('src', 'chrome://cmgui/content/icons/ok.png');      
        var msg = '';
        if(request.command == 'INSTALL') 
        {
          msg = 'Module ' + request.module + ' ' + request.version + '  successfully installed. ';
        }
        else if(request.command == 'UPGRADE') 
        {
          msg = 'Module ' + request.module + ' successfully upgraded to version ' + request.version + '.';
        }
        else if(request.command == 'DOWNGRADE') 
        {
          msg = 'Module ' + request.module + ' successfully downgraded to version ' + request.version + '.';
        }
        else if(request.command == 'UNINSTALL') 
        {
          msg = 'Module ' + request.module + ' successfully uninstalled.';
        }
        var element = document.createElement("shadowlabel");
        element.setAttribute("value", msg);
        element.setAttribute("class", "staticdataheader");
        vbox.appendChild(element);       
        row.appendChild(image);
        row.appendChild(element);
        rows.appendChild(row);
      }
      //
    }
    grid.appendChild(columns);    
    grid.appendChild(rows);
    vbox.appendChild(grid);
  }
}

///////////////////////////////////////////////////////////////////////////////////////////////////////
//
// search functions 
//
///////////////////////////////////////////////////////////////////////////////////////////////////////
function showWaitingForSearchResult(message) {
  var vbox = document.getElementById('search_result');
  if(vbox) {
    while (vbox.childNodes.length > 0) {
      vbox.removeChild(vbox.firstChild);
    }
    var grid = document.createElement('grid');
    grid.setAttribute('class', 'staticdata');
    var columns = document.createElement('columns');
    var column0 = document.createElement('column');
    column0.setAttribute('flex','0');
    columns.appendChild(column0);
    var column1 = document.createElement('column');
    column1.setAttribute('flex','1');
    columns.appendChild(column1);
    var rows = document.createElement('rows');
    var row = document.createElement('row');
    var image = document.createElement('image');
    image.setAttribute('src', 'chrome://cmgui/content/icons/throbber.gif');        
    row.appendChild(image);
    var status_element = document.createElement("shadowlabel");
    status_element.setAttribute("value", message);    
    row.appendChild(status_element);
    rows.appendChild(row);
    grid.appendChild(columns);    
    grid.appendChild(rows);
    vbox.appendChild(grid);
  }
}

function searchPuppetForge(offset=0)  {
  offset = parseInt(offset);
  printall("searchPuppetForge " + offset);
  var puppetKey = parseInt(current.get('uniqueKey'));
  var sessionid = parseInt(current.cluster.sessionid);
  showWaitingForSearchResult("Searching for Puppet modules");
  var textbox = document.getElementById('term');
  var term = textbox.value;
  var options_json = document.getElementById('onlysupported').checked ? 
      '{"supported":true}' : '';
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmpuppet', 'puppetForgeSearch', [ puppetKey,sessionid ,term, pagination_limit,offset,options_json], 
    function(code, data) {
      if(code == 0) {      
        if(!data.success) {
          dock.displayError("Failed to commit search request " + data.message);                  
        }
      } 
      else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
}

function searchPuppetForgeUpdates(offset=0) {
  offset = parseInt(offset);
  var puppetKey = parseInt(current.get('uniqueKey'));
  var sessionid = parseInt(current.cluster.sessionid);
  var textbox = document.getElementById('term');
  textbox.value = '';
  var textbox = document.getElementById('term');
  var term = textbox.value;
  showWaitingForSearchResult("Searching for updates");

  dock.workerpool.getWorker(current.cluster.actualurl).call('cmpuppet', 'puppetForgeSearchUpdates', [ puppetKey, sessionid, false, pagination_limit,offset, "" ], 
    function(code, data) {
      if(code == 0) {      
        if(!data.success) {
          dock.displayError("Failed to commit search request " + data.message);                  
        }
      } 
      else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
}

function searchResultVersionsMenulistOnchange(name, button_name) {
  var versions_menulist = document.getElementById(name);  
  var value = versions_menulist.value;
  if(typeof versions_menulist.puppet_forge_releases        != 'undefined' && 
     typeof versions_menulist.puppet_forge_releases[value] != 'undefined' )
  {
    var is_downgrade = versions_menulist.puppet_forge_releases[value].isDowngrade
    var is_upgrade   = versions_menulist.puppet_forge_releases[value].isUpgrade;
    var need_new_install = versions_menulist.puppet_forge_releases[value].needNewInstall;
    var install_upgrade_button = document.getElementById(button_name);   
    toggleInstallButtons(install_upgrade_button, need_new_install, is_upgrade, is_downgrade);
  }
}

function createSearchResultVersionsMenulist(result) {
  var availableReleases   = result.availableReleases;
  var defaultUpgrade      = result.availableUpdgrade;
  var versions_menulist   = document.createElement('menulist');
  var versions_menupopup  = document.createElement('menupopup');
  var button_name         = 'search-result-install-button@' + result.name;
  
  versions_menulist.appendChild(versions_menupopup);
  versions_menulist.setAttribute('id',       'versions-menulist@' + result.name);
  versions_menulist.setAttribute('onselect', 
                                 'searchResultVersionsMenulistOnchange(\'versions-menulist@' + 
                                 result.name + '\',\''+ button_name + '\');');
  versions_menulist.puppet_forge_releases = {}
  for(var i = 0; i < availableReleases.length; i++) {
    var menuitem = document.createElement('menuitem');
    var label = availableReleases[i].version;
    if(availableReleases[i].version == result.installedVersion)
    {
      label+=' installed';
    }
    menuitem.setAttribute('label', label);
    menuitem.setAttribute('value', availableReleases[i].version);
    versions_menulist.puppet_forge_releases[availableReleases[i].version] = {
      'version'        : availableReleases[i].version,
      'isDowngrade'    : availableReleases[i].isDowngrade,
      'isUpgrade'      : availableReleases[i].isUpgrade,
      'needNewInstall' : result.needNewInstall
    };
    versions_menupopup.appendChild(menuitem);
  }
  return versions_menulist;
}

function puppetShowAdvancedSearchOptions(module_name, show=true) {
  for(var k = 0; k < 3; k++) {
    var elem = 'search-result-install-advanced' + k + '@' + module_name;
    var row_adv = document.getElementById(elem);
    if(show) 
    {
      row_adv.removeAttribute('collapsed');
    }
    else 
    {
      row_adv.setAttribute('collapsed', true);
    }
  }
}


function createPaginationRanges(pagination, request) {
  if(pagination.total && pagination.limit) {
    var pages = (Math.floor( (pagination.total-1) / (pagination.limit)))+1;
    if(pages == 1) {
      return null;
    }
    var vbox = document.createElement(vbox);
    vbox.setAttribute('flex', "0");
    var pages_menulist   = document.createElement('menulist');
    var pages_menupopup  = document.createElement('menupopup');
    pages_menulist.setAttribute('width','200px');
    pages_menulist.setAttribute('flex','0');
    pages_menulist.appendChild(pages_menupopup);
    var offset = 0;
    var limit = pagination.limit;
    for(var i = 0; i < pages; i++) {
      var menuitem = document.createElement('menuitem');
      var last = (i+1) * limit;
      if(last > pagination.total) last = pagination.total;
      menuitem.setAttribute('label', "Showing " + (i * limit + 1) + " .. " + last);
      menuitem.setAttribute('value', limit * i); // offset
      if(limit * i == pagination.offset) {
        menuitem.setAttribute('selected', 'true'); 
      }
      pages_menupopup.appendChild(menuitem);
    }
    pages_menulist.setAttribute('oncommand', 'searchPuppetForge(this.value);');
    vbox.appendChild(pages_menulist);
    return vbox;
  }
  return null;
}


function showSearchResult(request, results, pagination) {
  var search_result_vbox = document.getElementById('search_result');
  while (search_result_vbox.firstChild) {
    search_result_vbox.removeChild(search_result_vbox.firstChild);
  }
  if(request.searchState == 'Failed') {
    var vbox = document.createElement('vbox');
    vbox.setAttribute('class','staticdata');
    var label = document.createElement('shadowlabel');
    label.setAttribute('class', 'staticdatavalue');
    label.setAttribute('value', request.message);
    vbox.appendChild(label);
    search_result_vbox.appendChild(vbox);
  }
  else if(request.searchState == 'Done' && results.length == 0) {
    var vbox = document.createElement('vbox');
    vbox.setAttribute('class','staticdata');
    var label = document.createElement('shadowlabel');
    label.setAttribute('class', 'staticdatavalue');
    if(request.searchMode == 'Updates') {
      label.setAttribute('value', "No updates found");
    }
    else {
      label.setAttribute('value', "No modules found");
    }
    vbox.appendChild(label);
    search_result_vbox.appendChild(vbox);
  }
  else {
    var pagination_select = createPaginationRanges(pagination, request);
    if(pagination_select) {
      var vbox = document.createElement('vbox');
      vbox.setAttribute('width', '100px');
      vbox.setAttribute('class', 'staticdata');
      vbox.setAttribute('flex','0');
      vbox.appendChild(pagination_select);
      search_result_vbox.appendChild(vbox);
    }
    for(var i = 0; i < results.length; i++) {
      var vbox = document.createElement('vbox');
      vbox.setAttribute('class', 'staticdata');
      vbox.setAttribute('flex','1');
      vbox.setAttribute('id', 'search-result@' + results[i].name);
      var grid = document.createElement('grid');
      var columns = document.createElement('columns');
      for(var j = 0; j < 5; j++) {
        var column = document.createElement('column');
        if(j == 0) 
          column.setAttribute('flex','1');
        else 
        {
          column.setAttribute('flex','0');
          column.setAttribute('align', 'right');
        }
        columns.appendChild(column);
      }
      grid.appendChild(columns);
      var rows = document.createElement('rows');
      var row = document.createElement('row');
      //row.setAttribute('class', 'staticdata');
      var shadowlabel = document.createElement('shadowlabel');
      shadowlabel.setAttribute('value',  results[i].name);
      shadowlabel.setAttribute('height', '26px');
      shadowlabel.setAttribute('class',  'staticdataheader');
      row.appendChild(shadowlabel);
      
      var vbox_menulist = document.createElement('vbox');
      vbox_menulist.setAttribute('flex','0');
      vbox_menulist.appendChild(createSearchResultVersionsMenulist(results[i]));        
      row.appendChild(vbox_menulist);
      createSearchResultButtons(results[i], row);
      rows.appendChild(row);      

      var rows_adv = [
        document.createElement('row'),
        document.createElement('row'),
        document.createElement('row')
      ];
      var check_box_force               = document.createElement('checkbox');
      var check_box_ignore_dependencies = document.createElement('checkbox');
      var check_box_ignore_changes      = document.createElement('checkbox');
      check_box_force.setAttribute('label','force');
      check_box_ignore_dependencies.setAttribute('label','ignore-dependencies');
      check_box_ignore_changes.setAttribute('label','ignore-changes');
      check_box_force.setAttribute('id', 'search-result-install-force@' + results[i].name);
      check_box_ignore_dependencies.setAttribute('id', 'search-result-install-ignore-dependencies@' + results[i].name);
      check_box_ignore_changes.setAttribute('id', 'search-result-install-ignore-changes@' + results[i].name);

      for(var k = 0; k < rows_adv.length; k++) 
      {
        for(var l = 0; l < 2; l++) 
        {
          rows_adv[k].appendChild(document.createElement('hbox'));
        }
        rows_adv[k].setAttribute('collapsed', true);
        rows_adv[k].setAttribute('id', 'search-result-install-advanced' + k + '@' + results[i].name);
        rows.appendChild(rows_adv[k]);
      }
      rows_adv[0].appendChild(check_box_force);
      rows_adv[1].appendChild(check_box_ignore_dependencies);
      rows_adv[2].appendChild(check_box_ignore_changes);

      grid.appendChild(rows);
      vbox.appendChild(grid);
      search_result_vbox.appendChild(vbox);
    }
  }
}

function toggleInstallButtons(install_upgrade_button, needNewInstall, isUpgrade, isDowngrade) {
  if(needNewInstall) {
    install_upgrade_button.setAttribute('label', 'Install');
    install_upgrade_button.removeAttribute("disabled");
  }
  else {
    if(isDowngrade) {
      install_upgrade_button.setAttribute('label', 'Downgrade');
      install_upgrade_button.removeAttribute("disabled");
    }
    else {
      install_upgrade_button.setAttribute('label', 'Upgrade');
      if(isUpgrade) {
        install_upgrade_button.removeAttribute("disabled");
      }
      else {
        install_upgrade_button.setAttribute("disabled","true");
      }
    }
  }
}


function createSearchResultButtons(result, row) 
{
  //var toggle_details_button = document.createElement('button');
  //toggle_details_button.setAttribute('class', 'bluebutton');
  //toggle_details_button.setAttribute('label', 'Details');
  //toggle_details_button.setAttribute('width', '80px');
  //row.appendChild(toggle_details_button);
  var hbox = document.createElement('hbox');
  var install_upgrade_button = document.createElement('button');
  install_upgrade_button.setAttribute('oncommand', 'installUpgradeModule(\'' + result.name + '\');');
  install_upgrade_button.setAttribute('class', 'bluebutton');
  install_upgrade_button.setAttribute('width', '80px');
  install_upgrade_button.setAttribute('id', 'search-result-install-button@' + result.name);
  
  var isUpgrade = (result.availableReleases.length != 0 && result.availableReleases[0].isUpgrade);
  toggleInstallButtons(install_upgrade_button, result.needNewInstall, isUpgrade, false);
  hbox.appendChild(install_upgrade_button);
  

  var uninstall_button = document.createElement('button');
  uninstall_button.setAttribute('class', 'bluebutton');
  uninstall_button.setAttribute('label', 'Uninstall');
  uninstall_button.setAttribute('id', 'search-result-uninstall-button@' + result.name);
  uninstall_button.setAttribute('oncommand', 'uninstallPuppetModule(\'' + result.name + '\');');
  uninstall_button.setAttribute('width', '80px');
  if(result.installedKey==0) {
    // not installed in any version 
    uninstall_button.setAttribute("disabled","true");
  }
  var advanced_toolbar = document.createElement('toolbar');
  advanced_toolbar.setAttribute('class', 'toolbar-transparent-background');
  var advanced_toolbar_hide = document.createElement('toolbarbutton');
  advanced_toolbar_hide.setAttribute('type', 'radio');
  advanced_toolbar_hide.setAttribute('class', 'show-button');
  advanced_toolbar_hide.setAttribute('checked', 'true');
  advanced_toolbar_hide.setAttribute('group', 'RadioGroup');
  advanced_toolbar_hide.setAttribute('oncommand','');
  advanced_toolbar_hide.setAttribute('oncommand','puppetShowAdvancedSearchOptions(\'' + result.name + '\', false);');
  
  var advanced_toolbar_show = document.createElement('toolbarbutton');
  advanced_toolbar_show.setAttribute('type', 'radio');
  advanced_toolbar_show.setAttribute('class', 'showdetails-button');
  advanced_toolbar_show.setAttribute('group', 'RadioGroup');
  advanced_toolbar_show.setAttribute('oncommand','puppetShowAdvancedSearchOptions(\'' + result.name + '\', true);');
  advanced_toolbar.appendChild(advanced_toolbar_hide);
  advanced_toolbar.appendChild(advanced_toolbar_show);

  hbox.appendChild(uninstall_button);
  hbox.appendChild(advanced_toolbar);
  row.appendChild(hbox);
}

/////////////////////////////////////////////////////////////
//
// Changes 
//
////////////////////////////////////////////////////////////
function puppetForgeChanges()
{
  var puppetKey = parseInt(current.get('uniqueKey'));
  var sessionid = parseInt(current.cluster.sessionid);
  showWaitingForSearchResult("Getting manifest changes from headnode");
  //if(!install_listener_id) {
  //install_listener_id = current.cluster.addListener('PuppetForgeInstallationDoneEvent',function(event){
  //showPuppetForgeChanges(event.request);
  //});
  //}
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmpuppet', 'puppetModuleChanges', 
                                                            [ puppetKey, sessionid, "", [] ], 
    function(code, data) 
    {
      if(code == 0) {      
        if(!data.success) {
          dock.displayError("Failed to commit search request " + data.message);                  
        }
      } 
      else {
        dock.jsonCallError(this, code, data);
      }
      dock.unwaitIcon();
    }, current);
}

function showPuppetForgeChanges(data)
{
    var search_result_vbox = document.getElementById('search_result');
    while (search_result_vbox.firstChild) {
        search_result_vbox.removeChild(search_result_vbox.firstChild);
    }
    if(data.installState != 'DONE') 
    {
        // Show error 
        var vbox = document.createElement('vbox');
        vbox.setAttribute('class','staticdata');
        var label = document.createElement('shadowlabel');
        label.setAttribute('class', 'staticdatabigheader');
        label.setAttribute('value', 'failed to retrieve changed manifests from headnode' + (data.shortError?':':'') + data.shortError);
        vbox.appendChild(label);
        search_result_vbox.appendChild(vbox);
    }
    else if(data.results.length == 0 )
    {
        // no changes 
        var vbox = document.createElement('vbox');
        vbox.setAttribute('class','staticdata');
        var label = document.createElement('shadowlabel');
        label.setAttribute('class', 'staticdatabigheader');
        label.setAttribute('value', 'There are no changed manifests');
        vbox.appendChild(label);
        search_result_vbox.appendChild(vbox);
    }
    else 
    {
        var vbox = document.createElement('vbox');
        vbox.setAttribute('class','staticdata');
        var label = document.createElement('shadowlabel');
        label.setAttribute('class', 'staticdatabigheader');
        label.setAttribute('value', 'Changed manifests');
        vbox.appendChild(label);
        var grid = document.createElement('grid');
        var columns = document.createElement('columns');
        var column1 = document.createElement('column');
        column1.setAttribute('flex', '1');
        columns.appendChild(column1);
        grid.appendChild(columns);
        var rows = document.createElement('rows');
        rows.setAttribute('flex', '1');
        var row1 = document.createElement('row');
        row1.setAttribute('flex',  '1');
        row1.setAttribute('align', 'left');
        var shadowlabel = document.createElement('shadowlabel');
        shadowlabel.setAttribute('value', 'File');
        shadowlabel.setAttribute('class', 'staticdatatableheader');
        row1.appendChild(shadowlabel);
        rows.appendChild(row1);
        var hbox = document.createElement('hbox');
        hbox.setAttribute('height', '1px');
        hbox.setAttribute('style',  'border-top:dotted gray 1px;');
        rows.appendChild(hbox);
        for(var i = 0; i < data.results.length; i++) 
        {
            var row = document.createElement('row');
            row.setAttribute("flex", "1");
            row.setAttribute("align", "left");
            row.setAttribute("height", "17px");
            var label = document.createElement('label');
            label.setAttribute("class", "staticdatavaluesmall");
            label.setAttribute("value", data.results[i]);
            row.appendChild(label);
            rows.appendChild(row);
        }
        grid.appendChild(rows);
        vbox.appendChild(grid);
        search_result_vbox.appendChild(vbox);
    }
}


function cleanResultSet() {
  var search_result = document.getElementById("search_result");
  while (search_result.firstChild) {
    search_result.removeChild(search_result.firstChild);
  }
  var term = document.getElementById("term");
  if(term) {
    term.value = '';
  }
}

