/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var MAX_LARGE = 4;
var MAX_SMALL = 1;
var MAXDEMOSHOW = 2;

function cvRackBox() {
  this.keys = 0;
  this.names = 0;
  this.relpos = 0;
  this.type = 0;
  this.data = 0;
  this.height = -1;
  this.parent = -1;
  this.box = 0; /* hbox */
}

cvRackBox.prototype.add = function(name, type, key, height, relpos, format) {
  if(this.parent > 0) {
    dock.displayError(name + ' begins inside an existing device, rackview will not work properly until this problem is resolved');
    return 0;
  } else if(this.height < 0) {
    this.type = type;
    this.keys = [key];
    this.relpos = [relpos];
    this.height = height;
    if(endswith(type, 'Node') && (type != 'MasterNode') && format.exec(name)) {
      name = nameNumber(name);
      if(name[1].length > 0) {
        this.names = [name[1]];
      } else {
        this.names = [name[0]];
      }
    } else {
      this.names = [name];
    }
    this.data = [0];
    return this.height - 1;
  } else {
    if(this.height == height) {
      if(endswith(type, 'Node') && (type != 'MasterNode') && format.exec(name)) {
        name = nameNumber(name);
        if(name[1].length > 0) {
          name = name[1];
        } else {
          name = name[0];
        }
      }
      if(((this.type != 'Chassis') && endswith(type, 'Node')) || (type == 'Chassis')) {
        this.type = type;
      }
      for(var i = 0; i < this.names.length; i++) {
        if((relpos < this.relpos[i]) || ((relpos == this.relpos[i]) && (name < this.names[i]))) {
          for(var j = this.names.length; j > i; j--) {
            this.data[j] = this.data[j - 1];
            this.names[j] = this.names[j - 1];
            this.keys[j] = this.keys[j - 1];
            this.relpos[j] = this.relpos[j - 1];
          }
          this.data[i] = 0;
          this.names[i] = name;
          this.keys[i] = key;
          this.relpos[i] = relpos;
          return this.height - 1;
        }
      }
      this.data.push(0);
      this.keys.push(key);
      this.names.push(name);
      this.relpos.push(relpos);
      return this.height - 1;
    } else {
      println(name + ' does not have the same height as a existing device on this position');
      return -1;
    }
  }
  return 0;
}

function cvRack(cluster) {
  this.cluster = cluster;
  this.racks = 0;
  this.rack0nodes = 0;
  this.cloudRacks = 0;
  this.cloudRackNames = 0;
  this.metrics = cluster.rackMetrics;
  this.refreshRate = dock.generalSettings.monitoringRefresh;
  this.zoom = 0;
  this.keys = 0;
  this.min = 0;
  this.max = 0;
  this.diff = 0;
  this.live = false;
  this.fillNoData = true;
  this.timer = 0;
  this.updating = false;
  this.height = 0;
  this.realracks = 0;
}

cvRack.prototype.setLive = function(live) {
  this.live = live;
}

cvRack.prototype.setFillNoData = function(fillNoData) {
  this.fillNoData = fillNoData;
}

cvRack.prototype.setMetrics = function(metrics) {
  this.metrics = metrics.slice(0);
  this.cluster.rackMetrics = this.metrics;
}

cvRack.prototype.deviceValues = function(key) {
  var box = this.keys[key];
  var x = new Array();
  if(box) {
    for(var i = 0; i < box.keys.length; i++) {
      if(box.keys[i] == key) {
        x = box.data[i];
        break;
      }
    }
  }
  return x;
}

function rackDiff(r1, r2) {
  if(r1.get('yCoordinate') > r2.get('yCoordinate')) {
    return 1;
  } else if(r1.get('yCoordinate') < r2.get('yCoordinate')) {
    return -1;
  } else if(r1.get('xCoordinate') > r2.get('xCoordinate')) {
    return 1;
  } else if(r1.get('xCoordinate') < r2.get('xCoordinate')) {
    return -1;
  } else if(r1.get('room') > r2.get('room')) {
    return 1;
  } else if(r1.get('room') < r2.get('room')) {
    return -1;
  } else if(r1.get('name') > r2.get('name')) {
    return 1;
  } else if(r1.get('name') < r2.get('name')) {
    return -1;
  }
  return 0;
}

cvRack.prototype.init = function() {
  if(!this.metrics) this.metrics = new Array();
  this.racks = new Array();
  this.keys = new Object();
  this.realracks = this.cluster.find('Rack');
  this.realracks.sort(rackDiff);
  this.height = 0;
  for(var i = 0; i < this.realracks.length; i++) {
    this.height = Math.max(this.height, this.realracks[i].get('height'));
  }
  if(this.height == 0) this.height = 42;
  var rackkeys = new Array();
  for(var i = 0; i < this.realracks.length; i++) {
    var rack = new Array();
    for(var j = 0; j < this.height; j++) {
      rack.push(new cvRackBox());
    }
    this.racks.push(rack);
    rackkeys.push(this.realracks[i].get('uniqueKey'));
  }
  this.rack0nodes = 0;
  this.cloudRacks= 0;

  var cloudRacks = new Object();
  var cloudRackNodes = new Object();
  
  var devices = this.cluster.find('Device');
  devices = devices.sort(function(a, b) {
    return (a.resolveName() > b.resolveName());
  });

  this.cloudRackNames = new Array();
  var chassisPos = new Object();
  for(var i = 0; i < devices.length; i++) {
    if(devices[i].get('childType') == 'Chassis') {
      var members = devices[i].get('members');
      var rackIndex = devices[i].get('rack');
      if(rackIndex > 0) {
        rackIndex = rackkeys.find(rackIndex);
      } else {
        rackIndex = -1;
      }
      var rackPosition = devices[i].get('rackPosition') - 1;
      var rackHeight = devices[i].get('rackHeight');
      for(var j = 0; j < members.length; j++) {
        chassisPos[members[j]] = [rackIndex, rackPosition, rackHeight, devices[i].resolveName()];
      }
    } else if(devices[i].get('childType') == 'CloudNode') {
      var region = devices[i].getCloudRegion();
      if(region) {
        var key = region.get('uniqueKey');
        if(!!cloudRacks[key]) {
          cloudRacks[key][1]--;
          if(cloudRacks[key][1] <= 0) {
            var rack = new Array();
            for(var j = 0; j < this.height; j++) {
              rack.push(new cvRackBox());
            }
            this.racks.push(rack);
            // TODO: option for inverse order
            this.cloudRackNames.push(region.resolveName());
            this.cloudRacks++;
            cloudRacks[key] = [this.racks.length - 1, this.height];
          }
          cloudRackNodes[devices[i].get('uniqueKey')] = cloudRacks[key].slice();
        } else {
          cloudRacks[key] = [this.racks.length, this.height];
          cloudRackNodes[devices[i].get('uniqueKey')] = cloudRacks[key].slice();
          var rack = new Array();
          for(var j = 0; j < this.height; j++) {
            rack.push(new cvRackBox());
          }
          this.racks.push(rack);
          this.cloudRackNames.push(region.resolveName());
          this.cloudRacks++;
        }
      }
    }
  }
    
  var format = "^$";
  var base = this.cluster.getBasePartition();
  if(base) {
    format = "^" + base.get('slaveName') + "\\d*$";
  }
  format = new RegExp(format);

  header('Fill rack view');
  for(var i = 0; i < devices.length; i++) {
    var name = devices[i].resolveName();
    var key = devices[i].get('uniqueKey');
    var rackIndex = devices[i].get('rack');
    var rackPosition = -1;
    var rackHeight = -1;
    var height = 0;
    var inverted = false;
    var chassisPrefix = ''
    
    if(devices[i].get('childType') == 'CloudNode') {
      rackHeight = 1;
      rackIndex = cloudRackNodes[key][0];
      rackPosition = cloudRackNodes[key][1] - 1;
    } else {
      if(rackIndex > 0) {
        rackIndex = rackkeys.find(rackIndex);
      } else {
        rackIndex = -1;
      }
      if(!!chassisPos[key]) {
        rackIndex = chassisPos[key][0];
        rackPosition = chassisPos[key][1];
        rackHeight = chassisPos[key][2];
        chassisPrefix = chassisPos[key][3];
      } else if(rackIndex < 0) {
        var mod = this.rack0nodes % this.height;
        if(mod == 0) {
          println("Add extra unmanaged rack");
          var rack = new Array();
          for(var j = 0; j < this.height; j++) {
            rack.push(new cvRackBox());
          }
          this.racks.push(rack);
        }
        rackIndex = this.realracks.length + this.cloudRacks + Math.floor(this.rack0nodes / this.height);
        rackPosition = mod;
        rackHeight = 1;
        height = this.height;
        this.rack0nodes++;
      } else {
        height = this.realracks[rackIndex].get('height');      
        inverted = this.realracks[rackIndex].get('inverted');
      }
      if(rackPosition < 0) {
        rackPosition = devices[i].get('rackPosition') - 1;
        rackHeight = devices[i].get('rackHeight');
        if(rackPosition < 0) {
          println(name + ' has no rack position');
          continue;
        } else if(rackHeight <= 0) {
          println(name + ' has no height');
          continue;
        } else if(!inverted && (rackPosition + rackHeight > height)) {
          println(name + ' has illegal rack position + height: ' + rackPosition + ' + ' + rackHeight + ' > ' + height);
          continue;
        } else if(inverted && (rackPosition - rackHeight < -1)) {
          println(name + ' has illegal rack position - height: ' + rackPosition + ' - ' + rackHeight + ' < ' + -1);
          continue;
        }
      }
    }
    if((rackIndex < this.racks.length) && (rackIndex >= 0)) {
      if((rackPosition >= 0) && (rackPosition < this.racks[rackIndex].length)) {
        this.keys[key] = this.racks[rackIndex][rackPosition];
        if(rackIndex < this.realracks.length) {
          var rackname = this.realracks[rackIndex].resolveName();
          if((chassisPrefix.length > 0) && (name.length > chassisPrefix.length) && startswith(name, chassisPrefix)) {
            name = name.substr(chassisPrefix.length);
          } else if((name.length > rackname.length) && startswith(name, rackname)) {
            name = name.substr(rackname.length);
          }
        }
        var k = this.racks[rackIndex][rackPosition].add(name, devices[i].get('childType'), key, rackHeight, devices[i].get('indexInsideContainer'), format);
        var mul = 1;
        if(inverted) mul = -1;
        for(var j = 1; j <= k; j++) {
          this.racks[rackIndex][rackPosition + mul * j].parent = rackPosition;
        }
      } else {
        println("rackPosition out of range: " + rackPosition + " " + this.racks[rackIndex].length);
      }
    } else {
      println("rackIndex out of range: " + rackIndex + " " + this.racks.length);
    }
  }
  header();
}

cvRack.prototype.updateRackWithData = function(data) {
  for(var i = 0; i < data.length; i++) {
    var box = this.keys[data[i].devId];
    if(box) {
      var k = box.keys.find(data[i].devId);
      if(k >= 0) {
        for(var j1 = 0; j1 < this.metrics.length; j1++) {
          if((data[i].metricId == this.metrics[j1][0]) && (data[i].metricParam == this.metrics[j1][1])) {
            var rate = data[i].rate;
            box.data[k][j1] = rate;
            if(kisNaN(this.min[j1])) {
              this.min[j1] = rate;
              this.max[j1] = rate;
            } else if(!kisNaN(rate)) {
              this.max[j1] = Math.max(this.max[j1], rate);
              this.min[j1] = Math.min(this.min[j1], rate);
            }
            /* No break, for duplicate entries */
          }
        }
      }
    }
  }
  this.redraw2();
  dock.unwaitIcon();
}

cvRack.prototype.refresh = function() {
  if(!this.racks) this.init();
  if(this.updating) return;
  if(!this.cluster.connected) return;
  println("Refresh rack");
  this.updating = true;

  this.max = new Array();
  this.min = new Array();
  if(dock.DEMO) {
    for(var i = 0; i < this.metrics.length; i++) {
      this.max[i] = NaN;
      this.min[i] = NaN;
    }
/*    for(var i = 0; i < this.metrics.length; i++) {
      this.max[i] = 70;
      this.min[i] = 20;
    }*/
    var extra = Math.max(0, Math.ceil(this.rack0nodes / this.height));
    for(var i = 0; i < this.realracks.length + this.cloudRacks + extra; i++) {
      for(var j = 0; j < this.height; j++) {
        var box = this.racks[i][j];
        if((box.parent < 0) && (box.height > 0)) {
          box.data = new Array();
          for(var k = 0; k < box.keys.length; k++) {
            var data = new Array();
            if(endswith(box.type, 'Node') || (box.type == 'GpuUnit')) {
              for(var l = 0; l < Math.min(MAXDEMOSHOW, this.metrics.length); l++) {
                var rate = Math.exp(-(i - 3) * (i - 3) * 0.2);
                rate = 0.7 * rate * Math.exp(-(j - 11)* (j - 11) * 0.02);
                rate = (rate + 0.3 * Math.random()) * 40 + 30;
                if(this.metrics[l][2]) {
                  if(rate > 65) {
                    rate = 2;
                  } else {
                    rate = 0;
                  }
                }
                data.push(rate);
                if(kisNaN(this.min[l])) {
                  this.min[l] = rate;
                  this.max[l] = rate;
                } else {
                  this.max[l] = Math.max(this.max[l], rate);
                  this.min[l] = Math.min(this.min[l], rate);
                }
              }
              for(var l = Math.min(MAXDEMOSHOW, this.metrics.length); l < this.metrics.length; l++) {
                data.push(NaN);
              }
            }
            box.data.push(data);
          }
        }
      }
    }
    this.redraw2();
  } else {
    if(this.metrics.length > 0) {
      var metrics = new Array();
      for(var i = 0; i < this.metrics.length; i++) {
        var mpi = new dock.cvMetricPrmId();
        mpi.metricId = parseInt(this.metrics[i][0]);
        mpi.metricParam = this.metrics[i][1];
        metrics.push(mpi);
        var metric = this.cluster.find(this.metrics[i][0]);
        if(isObject(metric) && ((metric.get('minimum') != 0) || (metric.get('maximum') != 0))) {
          this.max[i] = metric.get('maximum');
          this.min[i] = metric.get('minimum');
        } else {
          this.max[i] = NaN;
          this.min[i] = NaN;
        }
      }
      var keys = new Array();
      for(var x in this.keys) keys.push(parseInt(x));
      
      var extra = Math.max(0, Math.ceil(this.rack0nodes / this.height));
      for(var i = 0; i < this.realracks.length + this.cloudRacks + extra; i++) {
        for(var j = 0; j < this.height; j++) {
          var box = this.racks[i][j];
          if((box.parent < 0) && (box.height > 0)) {
            box.data = new Array();
            for(var k = 0; k < box.keys.length; k++) box.data.push(new Array());
          }
        }
      }
      
      dock.waitIcon();
      if(this.live) {
        dock.workerpool.getWorker(this.cluster.actualurl).call('cmmon', 'sampleOndemandOnDevices', [keys, metrics], function(code, data) {
          if(code == 0) {
            this.current.updateRackWithData(data);
          } else {
            dock.jsonCallError(this, code, data);
          }
        }, this);        
      } else {
        dock.workerpool.getWorker(this.cluster.actualurl).call('cmmon', 'getLatestPickedRates', [keys, metrics], function(code, data) {
            if(code == 0) {
              this.current.updateRackWithData(data);
            } else {
              dock.jsonCallError(this, code, data);
            }
          }, this);
      }
    } else {
      this.redraw2();
    }
  }
  this.updating = false;
  this.stop();
  this.timer = window.setTimeout(createObjectCallback(this, this.refresh), this.refreshRate * 1000);
}

cvRack.prototype.stop = function() {
  if(this.timer) {
    window.clearTimeout(this.timer);
    this.timer = 0;
    return true;
  }
  return false;
}

cvRack.prototype.redraw2 = function() {
  this.diff = new Array();
  var same = (this.max.length > 1);
  var old;
  for(var i = 0; i < this.max.length; i++) {
    var metric = this.cluster.find(this.metrics[i][0]);
    var unit = ''
    if(metric) unit = metric.get('unit');
    if(i > 0) {
      same = same && (old == unit);
    } else {
      old = unit;
    }
    if(kisNaN(this.min[i])) this.min[i] = 0;
    if(kisNaN(this.max[i])) this.max[i] = 0;
    var value = this.max[i] - this.min[i];
    if(value > 0.00001) {
      this.diff.push(value);
    } else {
      this.diff.push(1);
    }
  }
  this.redraw();
}

cvRack.prototype.redraw = function(zoom) {
  this.zoom = isundefined(zoom, this.zoom);
  if(this.zoom) {
    this.drawSmall();
  } else {
    this.drawLarge();
  }
  this.legend();
}

cvRack.prototype.metricInfo = function() {
  var info = new Array();
  for(var i = 0; i < this.metrics.length; i++) {
    var metric = this.cluster.find(this.metrics[i][0]);
    if(isObject(metric)) {
      var name = metric.get('name');
      if(this.metrics[i][1].length > 0) name += ' (' + this.metrics[i][1] + ')';
      var isCheck = (this.metrics[i].length > 2) && this.metrics[i][2];
      info.push([name, metric.get('unit'), isCheck]);
    }
  }
  return info;
}

function newLabel(value) {
  var label = document.createElement('label');
  label.setAttribute('style', 'color: #efefef; font-size: 11px;');
  label.setAttribute('value', value);
  return label;
}

function newScale(check) {
  var box = document.createElement('vbox');
  box.setAttribute('flex', 1);
  box.setAttribute('pack', 'center');
  var label = document.createElement('hbox');
  if(check) {
    label.setAttribute('class', 'checklegend');
  } else {
    label.setAttribute('class', 'legend');
  }
  box.appendChild(label);
  return box;
}

cvRack.prototype.legend = function() {
  var legend = document.getElementById('legend');
  while(legend.firstChild) legend.removeChild(legend.firstChild);
  var n = this.diff.length;
  for(var i = 0; i < n; i++) {
    var column1 = document.createElement('column');
    var column2 = document.createElement('column');
    var column3 = document.createElement('column');
    var column4 = document.createElement('column');
    var metric = this.cluster.find(this.metrics[i][0]);
    if(!metric) continue;
    var name = metric.resolveName();
    var ischeck = (this.metrics[i].length > 2) && this.metrics[i][2];
    var unit = ''
    if(!ischeck) unit = metric.get('unit');
    if(this.metrics[i][1].length > 0) name += ':' + this.metrics[i][1];
    column1.appendChild(newLabel(name));
    if(ischeck) {
      column2.appendChild(newLabel('PASS'));
      column3.appendChild(newScale(true));
      column4.appendChild(newLabel('FAIL'));
    } else {
      column2.appendChild(newLabel(formatKMGselect(this.min[i], unit)));
      column3.appendChild(newScale(false));
      column4.appendChild(newLabel(formatKMGselect(this.max[i], unit)));
    }
    i++;
    if(i < n) {
      var metric = this.cluster.find(this.metrics[i][0]);
      var name = metric.resolveName();
      var ischeck = (this.metrics[i].length > 2) && this.metrics[i][2];
      var unit = ''
      if(!ischeck) unit = metric.get('unit');
      if(this.metrics[i][1].length > 0) name += ':' + this.metrics[i][1];
      column1.appendChild(newLabel(name));
      if((this.metrics[i].length > 2) && this.metrics[i][2]) {
        column2.appendChild(newLabel('PASS'));
        column3.appendChild(newScale(true));
        column4.appendChild(newLabel('FAIL'));
      } else {
        column2.appendChild(newLabel(formatKMGselect(this.min[i], unit)));
        column3.appendChild(newScale(false));
        column4.appendChild(newLabel(formatKMGselect(this.max[i], unit)));
      }
    } else {
      column1.appendChild(document.createElement('spacer'));
      column2.appendChild(document.createElement('spacer'));
      column3.appendChild(document.createElement('spacer'));
      column4.appendChild(document.createElement('spacer'));
    }
    legend.appendChild(column1);
    legend.appendChild(column2);
    legend.appendChild(column3);
    legend.appendChild(column4);
  }
}

cvRack.prototype.color = function(index, value) {
  if(this.metrics[index][2]) {
    return colorbar(0.5 + value / 4.0);
  } else {
    index = Math.min(index, this.min.length - 1);
    value = value - this.min[index];
    value = value / this.diff[index];
    return colorbar(value);
  }
}

cvRack.prototype.addColors = function(nbox, data, MAX) {
  var display = 0;
  var list = new Array();
  for(var l = 0; l < this.metrics.length; l++) {
    var nodata = (l >= data.length) || (typeof(data[l]) == "undefined") || kisNaN(data[l]);
    if(!nodata || this.fillNoData) {
      list.push(l);
      display++;
      if(display >= MAX) break;
    }
  }

  var s = Math.max(4, 16 / list.length);
  for(var i = 0; i < list.length; i++) {
    var l = list[i];
    var nodata = (l >= data.length) || (typeof(data[l]) == "undefined") || kisNaN(data[l]);
    var mbox = document.createElement('hbox');
    if(nodata) {
      mbox.setAttribute("style", "background-color: transparent; border-left: 1px solid #AAAAAA");
    } else {
      mbox.setAttribute("style", "background-color: " + this.color(l, data[l]) + "; border-left: 1px solid #AAAAAA");
    }
    mbox.setAttribute("height", "15px");
    mbox.setAttribute("width", s + "px");
    nbox.appendChild(mbox);
  }
}

cvRack.prototype.drawSmall = function() {
  var caption = document.getElementById('rackcaption');
  while(caption.firstChild) caption.removeChild(caption.firstChild);
  var label = document.createElement('label');
  label.setAttribute('class', 'rackdots');
  label.setAttribute('value', 'U');
  caption.appendChild(label);
  for(var i = 0; i < this.realracks.length; i++) {
    var label = document.createElement('label');
    label.setAttribute('class', 'rackzoom');
    label.setAttribute('value', i + 1);
    if(i == 0) label.setAttribute('width', 30);
    caption.appendChild(label);
  }
  var bar = document.createElement('box');
  bar.setAttribute('class', 'rackbarzoom');
  caption.appendChild(bar);

  var extra = this.cloudRacks;
  if(extra > 0) {
    var vbox = document.createElement('vbox');
    vbox.setAttribute('width', '16px');
    caption.appendChild(vbox);
    for(var i = 0; i < extra; i++) {
      var label = document.createElement('label');
      label.setAttribute('class', 'rackzoomunmanaged');
      label.setAttribute('value', 'C');
      caption.appendChild(label);
    }
    var bar = document.createElement('box');
    bar.setAttribute('class', 'rackbarzoomunmanaged');
    caption.appendChild(bar);
  }
  
  var extra = Math.max(0, Math.ceil(this.rack0nodes / this.height));
  if(extra > 0) {
    var vbox = document.createElement('vbox');
    vbox.setAttribute('width', '16px');
    caption.appendChild(vbox);
    for(var i = 0; i < extra; i++) {
      var label = document.createElement('label');
      label.setAttribute('class', 'rackzoomunmanaged');
      label.setAttribute('value', 'X');
      caption.appendChild(label);
    }
    var bar = document.createElement('box');
    bar.setAttribute('class', 'rackbarzoomunmanaged');
    caption.appendChild(bar);
  }

  var position = document.getElementById('rackposition');
  while(position.firstChild) position.removeChild(position.firstChild);

  var plus = 1;
  var beg = 0;
  if(this.realracks[0].get('inverted')) {
    beg = this.height - 1;
    plus = -1;
  }

  for(var j = beg; (j >= 0) && (j < this.height); j += plus) {
    var label = document.createElement('label');
    label.setAttribute('class', 'rackpositionzoom');
    var x = j + 1; if(j < 9) x = '0' + x;
    label.setAttribute('value', x);
    position.appendChild(label);
  }

  var data = document.getElementById('rackdata');
  while(data.firstChild) data.removeChild(data.firstChild);
  for(var i = 0; i < this.realracks.length + this.cloudRacks + extra; i++) {
    if((i == this.realracks.length) || (i == this.cloudRacks + this.realracks.length)) {
      var vbox = document.createElement('vbox');
      vbox.setAttribute('width', '16px');
      data.appendChild(vbox);
      var bar = document.createElement('box');
      bar.setAttribute('class', 'rackbarzoomunmanaged');
      data.appendChild(bar);
    }
    var vbox = document.createElement('vbox');
    for(var j = beg; (j >= 0) && (j < this.height); j += plus) {
      var hbox = document.createElement('vbox'); // VBOX
      var box = this.racks[i][j];
      this.racks[i][j].hbox = hbox
      if(box.height < 0) {
        hbox.setAttribute('key', 0);
        if(i < this.realracks.length) {
          hbox.setAttribute('class', 'rackemptyzoom1');
        }
      } else {
        hbox.setAttribute('class', 'racknodezoom');
        hbox.setAttribute('height', 12 * box.height);
        var twin = 0;
        var twinp = 0;
        var single = 0;
        if(box.keys.length > box.height) {
          twin = box.keys.length;
          twinp = Math.ceil(box.keys.length / box.height);
        } else {
          single = box.keys.length;
        }
        var node = 0;
        for(var k = 0; k < twin; k += twinp) {
          var tbox = document.createElement('hbox');
          tbox.setAttribute('class', 'racknodetwinzoom');
          for(var t = 0; t < twinp; t++) {
            var nbox = document.createElement('hbox');
            nbox.setAttribute('flex', '1');
            if(node < box.keys.length) {
              nbox.setAttribute('key', box.keys[node]);
              nbox.setAttribute('ondblclick', 'opendevice(' + box.keys[node] + ');');
              if(node < box.keys.length) nbox.setAttribute('locate', (dock.rackBlinkKey.find(box.keys[node]) >= 0));
              nbox.setAttribute('tooltip', 'devicetip');
              this.addColors(nbox, box.data[node], MAX_SMALL);
            } else {
              nbox.setAttribute('style', 'background-color: transparent;');
            }
            tbox.appendChild(nbox);
            if(t != twinp - 1) {
              var nbox = document.createElement('box');
              nbox.setAttribute('width', '1px');
              nbox.setAttribute('height', '11px');
              if(node < box.keys.length) {
                nbox.setAttribute('style', 'background-color: #7e7e7e; max-height: 11px;');
              } else {
                nbox.setAttribute('style', 'background-color: transparent; max-height: 11px;');
              }
              tbox.appendChild(nbox);
            }
            node++;
          }
          hbox.appendChild(tbox);
          if(box.keys.length != twinp * box.height) {
            var tbox = document.createElement('box');
            tbox.setAttribute('height', '1px');
            tbox.setAttribute('style', 'background-color: #7e7e7e;');
            hbox.appendChild(tbox);
          }
        }
        for(var k = 0; k < single; k++) {
          var nbox = document.createElement('hbox');
          nbox.setAttribute('key', box.keys[node]);
          nbox.setAttribute('ondblclick', 'opendevice(' + box.keys[node] + ');');
          if(node < box.keys.length) nbox.setAttribute('locate', (dock.rackBlinkKey.find(box.keys[node]) >= 0));
          nbox.setAttribute('tooltip', 'devicetip');
          nbox.setAttribute('flex', '1');
          this.addColors(nbox, box.data[node], MAX_SMALL);
          hbox.appendChild(nbox);
          node++;
          if(node < box.keys.length) {
            var tbox = document.createElement('hbox');
            tbox.setAttribute("height", "1px");
            tbox.setAttribute("style", "background-color: #7e7e7e;");
            hbox.appendChild(tbox);
          }
        }
        j += plus * (box.height - 1);
      }
      vbox.appendChild(hbox);
    }
    data.appendChild(vbox);
    var bar = document.createElement('box');
    if(i < this.realracks.length) {
      bar.setAttribute('class', 'rackbarzoom');
    } else {
      bar.setAttribute('class', 'rackbarzoomunmanaged');
    }
    data.appendChild(bar);
  }
}

cvRack.prototype.drawLarge = function() { // Take care of scroll
  var caption = document.getElementById('rackcaption');
  while(caption.firstChild) caption.removeChild(caption.firstChild);
  var label = document.createElement('label');
  label.setAttribute('class', 'rackdots');
  label.setAttribute('value', 'U');
  caption.appendChild(label);
  for(var i = 0; i < this.realracks.length; i++) {
    var label = document.createElement('label');
    label.setAttribute('class', 'rack');
    label.setAttribute('value', this.realracks[i].get('name'));
    caption.appendChild(label);
  }
  var bar = document.createElement('box');
  bar.setAttribute('class', 'rackbar');
  caption.appendChild(bar);

  var extra = this.cloudRacks;
  if(extra > 0) {
    var vbox = document.createElement('vbox');
    vbox.setAttribute('width', '16px');
    caption.appendChild(vbox);
    for(var i = 0; i < extra; i++) {
      var label = document.createElement('label');
      label.setAttribute('class', 'rackunmanaged');
      label.setAttribute('value', this.cloudRackNames[i]);
      caption.appendChild(label);
    }
    var bar = document.createElement('box');
    bar.setAttribute('class', 'rackbarunmanaged');
    caption.appendChild(bar);
  }

  var extra = Math.max(0, Math.ceil(this.rack0nodes / this.height));
  if(extra > 0) {
    var vbox = document.createElement('vbox');
    vbox.setAttribute('width', '16px');
    caption.appendChild(vbox);
    for(var i = 0; i < extra; i++) {
      var label = document.createElement('label');
      label.setAttribute('class', 'rackunmanaged');
      label.setAttribute('value', 'No rack');
      caption.appendChild(label);
    }
    var bar = document.createElement('box');
    bar.setAttribute('class', 'rackbarunmanaged');
    caption.appendChild(bar);
  }

  var position = document.getElementById('rackposition');
  while(position.firstChild) position.removeChild(position.firstChild);

  var plus = 1;
  var beg = 0;
  if((this.realracks.length > 0) && this.realracks[0].get('inverted')) {
    beg = this.height - 1;
    plus = -1;
  }

  for(var j = beg; (j >= 0) && (j < this.height); j += plus) {
    var label = document.createElement('label');
    label.setAttribute('class', 'rackposition');
    var x = j + 1; if(j < 9) x = '0' + x;
    label.setAttribute('value', x);
    position.appendChild(label);
  }

  var data = document.getElementById('rackdata');
  while(data.firstChild) data.removeChild(data.firstChild);
  for(var i = 0; i < this.realracks.length + this.cloudRacks + extra; i++) {
    if((i == this.realracks.length) || (i == this.cloudRacks + this.realracks.length)) {
      var vbox = document.createElement('vbox');
      vbox.setAttribute('width', '16px');
      data.appendChild(vbox);
      var bar = document.createElement('box');
      bar.setAttribute('class', 'rackbarunmanaged');
      data.appendChild(bar);
    }
    var vbox = document.createElement('vbox');
    for(var j = beg; (j >= 0) && (j < this.height); j += plus) {
      var hbox = document.createElement('hbox');
      var box = this.racks[i][j];
      this.racks[i][j].hbox = hbox
      if(box.height < 0) {
        hbox.setAttribute('key', 0);
        if(i < this.realracks.length) {
          if((j + plus >= 0) && (j + plus < this.height) && (this.racks[i][j + plus].height < 0)) {
            hbox.setAttribute('class', 'rackempty2');
            j += plus;
          } else {
            hbox.setAttribute('class', 'rackempty1');
          }
        }
      } else {
        hbox.setAttribute('class', 'racknode');
        hbox.setAttribute('height', 20 * box.height);
        var ibox = document.createElement('vbox');
        ibox.setAttribute('style', 'margin: 2px');
        var image = document.createElement('image');
        image.setAttribute('src', 'chrome://cmgui/content/icons/' + box.type + '_16.png');
        ibox.appendChild(image);
        hbox.appendChild(ibox);
        var twin = 0;
        var single = 0;
        var twinp = 0;
        if(box.keys.length > box.height) {
          twinp = Math.ceil(box.keys.length / box.height);
          twin = box.keys.length;
        } else {
          single = box.keys.length;
        }
        var node = 0;
        var xbox = document.createElement('vbox');
        xbox.setAttribute('flex', '1');
        for(var k = 0; k < twin; k += twinp) {
          var tbox = document.createElement('hbox');
          tbox.setAttribute('flex', '1');
          for(var t = 0; t < twinp; t++) {
            var nbox = document.createElement('hbox');
            nbox.setAttribute('flex', '1');
            if(node < box.keys.length) {
              nbox.setAttribute('align', 'center');
              nbox.setAttribute('class', 'rackitem');
              nbox.setAttribute('tooltip', 'devicetip');
              nbox.setAttribute('key', box.keys[node]);
              nbox.setAttribute('ondblclick', 'opendevice(' + box.keys[node] + ');');
              nbox.setAttribute('locate', (dock.rackBlinkKey.find(box.keys[node]) >= 0));
              var label = document.createElement('label');
              var name = box.names[node]
              if((twinp >= 3) && (name.length > 3)) name = name.substr(name.length - 3);
              label.setAttribute('value', name);              
              label.setAttribute('flex', '1');
              nbox.appendChild(label);
              this.addColors(nbox, box.data[node], MAX_LARGE);
            } else {
              var label = document.createElement('label');
              label.setAttribute('flex', '1');
              label.setAttribute('value', '    ');
              nbox.appendChild(label);
            }
            tbox.appendChild(nbox);
            node++;
          }
          xbox.appendChild(tbox);
          if(node < box.keys.length) {
            var tbox = document.createElement('hbox');
            tbox.setAttribute("height", "1px");
            tbox.setAttribute("style", "background-color: #9b9b9b;");
            xbox.appendChild(tbox);
          }
        }
        for(var k = 0; k < single; k++) {
          var nbox = document.createElement('hbox');
          nbox.setAttribute('key', box.keys[node]);
          nbox.setAttribute('ondblclick', 'opendevice(' + box.keys[node] + ');');
          if(node < box.keys.length) nbox.setAttribute('locate', (dock.rackBlinkKey.find(box.keys[node]) >= 0));
          nbox.setAttribute('tooltip', 'devicetip');
          nbox.setAttribute('flex', '1');
          nbox.setAttribute('class', 'rackitem');
          nbox.setAttribute('align', 'center');
          var label = document.createElement('label');
          label.setAttribute('value', box.names[node]);
          label.setAttribute('flex', '1');
          nbox.appendChild(label);
          this.addColors(nbox, box.data[node], MAX_LARGE);
          xbox.appendChild(nbox);
          node++;
          if(node < box.keys.length) {
            var tbox = document.createElement('hbox');
            tbox.setAttribute("height", "1px");
            tbox.setAttribute("style", "background-color: #9b9b9b;");
            xbox.appendChild(tbox);
          }
        }

        hbox.appendChild(xbox);
        j += plus * (box.height - 1);
      }
      vbox.appendChild(hbox);
    }
    data.appendChild(vbox);
    var bar = document.createElement('box');
    if(i < this.realracks.length) {
      bar.setAttribute('class', 'rackbar');
    } else {
      bar.setAttribute('class', 'rackbarunmanaged');
    }
    data.appendChild(bar);
  }
}
