/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var dock;
var rack;
var topview = 0;
var myDialog;
var waitCount = 0;
var rackmetrics;


function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  rack = myDialog.arguments.rack;
  if(myDialog.arguments.hasOwnProperty('topview')) topview = myDialog.arguments.topview;
  if(rack.metrics && isArray(rack.metrics)) {
    rackmetrics = rack.metrics.slice(0);
  } else {
    rackmetrics = new Array();
  }

  refresh();
}

function refresh() {
  waitIcon();

  var rates = [[60, '1 minute'], [120, '2 minutes'], [300, '5 minutes'], [600, '10 minutes'], [1800, '30 minutes'], [3600, '1 hour']];

  for(var i = 0; i < rates.length; i++) {
    if(rates[i][0] == rack.refreshRate) break;
    if(rates[i][0] > rack.refreshRate) {
      rates.splice(i, 0, [rack.refreshRate, formatDHM(rack.refreshRate)]);
      break;
    }
  }
  var popup = document.getElementById('rate').firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  for(var i = 0; i < rates.length; i++) {
    var item = document.createElement('menuitem');
    item.setAttribute('label', rates[i][1]);
    item.setAttribute('time', rates[i][0]);
    popup.appendChild(item);
    if(rates[i][0] == rack.refreshRate) document.getElementById('rate').selectedIndex = i;
  }

  document.getElementById('titlename').setAttribute('value', rack.cluster.name);

  document.getElementById('live').checked = rack.live;
  document.getElementById('fill').checked = rack.fillNoData;

  allMetricsAndChecks();
  updateList();
  unwaitIcon();
}

function allMetricsAndChecks() {
  function add_mclass(metricClass, className, subitem) {
    if(!metricClass.hasOwnProperty(className)) return false;
    mclass = metricClass[className];
    var added = false;
    if(mclass) {
      for(var k = 0; k < mclass[2]; k++) {
        if(!mclass[1].childNodes[k]) {
          println('+++ metric class child node not found: ' + className);
//           printall(metricClass);
//           printall(mclass);
//           printall(mclass[1].childNodes);
          break;
        }
        var x = mclass[1].childNodes[k].firstChild.firstChild.getAttribute('label').substr(1);
        if(text < x) {
          mclass[1].insertBefore(subitem, mclass[1].childNodes[k]);
          added = true;
          break;
        }
      }
      if(!added) mclass[1].appendChild(subitem);
      mclass[2]++;
      while(mclass[4].length && metricClass.hasOwnProperty(mclass[4])) {
        mclass = metricClass[mclass[4]];
        mclass[2]++;
      }          
    }
    return added;
  }
  
  var mmc = rack.cluster.allMetrics();
  var mmh = rack.cluster.allChecks();
  if((mmc.length > 0) || (mmh.length > 0)) {    
    var undef;
    var metricClass = buildMetricClass('rack', rack.cluster, undef, true);

    for(var j = 0; j < mmc.length; j++) {
      var metric = rack.cluster.find(mmc[j].metric);
      if(metric) {
        var text = metric.resolveName();
        if(mmc[j].translatedMetricParam.length > 0) text += "[" + mmc[j].translatedMetricParam + "]";
        else if(mmc[j].metricParam.length > 0) text += "[" + mmc[j].metricParam + "]";
        if(metric.get('unit')) text += " (" + metric.get('unit') + ")";
        var subitem = document.createElement("treeitem");
        subitem.setAttribute("metric", mmc[j].metric);
        subitem.setAttribute("param", mmc[j].metricParam);
        subitem.setAttribute("translatedParam", mmc[j].translatedMetricParam);
        subitem.setAttribute("check", 'false');
        var row = document.createElement("treerow");
        var cell = document.createElement("treecell");
        cell.setAttribute("src", "chrome://cmgui/content/icons/Monitoring_16.png");
        cell.setAttribute("label", " " + text); // space between image and label, css doesn't work
        row.appendChild(cell);
        subitem.appendChild(row);
        add_mclass(metricClass, metric.get('metricClass'), subitem);
      } else {
        println("Metric not found: " + mmc[j].metric);
      }
    }
    for(var j = 0; j < mmh.length; j++) {
      var check = rack.cluster.find(mmh[j].healthcheck);
      if(check) {
        var text = check.resolveName();
        if(mmh[j].translatedHealthcheckParam.length > 0) text += "[" + mmh[j].translatedHealthcheckParam + "]";
        else if(mmh[j].healthcheckParam.length > 0) text += "[" + mmh[j].healthcheckParam + "]";
        var subitem = document.createElement("treeitem");
        subitem.setAttribute("metric", mmh[j].healthcheck);
        subitem.setAttribute("param", mmh[j].healthcheckParam);
        subitem.setAttribute("translatedParam", mmh[j].translatedHealthcheckParam);
        subitem.setAttribute("check", 'true');
        var row = document.createElement("treerow");
        var cell = document.createElement("treecell");
        cell.setAttribute("src", "chrome://cmgui/content/icons/Health_16.png"); // TODO: Sharif new icon (KDR)
        cell.setAttribute("label", " " + text); // space between image and label, css doesn't work
        row.appendChild(cell);
        subitem.appendChild(row);
        add_mclass(metricClass, check.get('healthCheckClass'), subitem);
      } else {
        println("health check not found: " + mmh[j].healthcheck);
      }
    }
    var list = document.getElementById('metrics');
    while(list.firstChild) list.removeChild(list.firstChild);

    for(var mclass in metricClass) {
      if(metricClass[mclass][2] > 0) {
        var parent = metricClass[mclass][4];
        if(parent.length && metricClass.hasOwnProperty(parent)) {
          metricClass[parent][1].appendChild(metricClass[mclass][0]);
        } else {
          list.appendChild(metricClass[mclass][0]);
        }
      }
    }
  }
}

function addToRack() {
  var tree = document.getElementById("treecluster");
  var list = new Array();

  var start = new Object();
  var end = new Object();
  var firstProperty = 0;
  var message = '';
  for(var t = 0; t < tree.view.selection.getRangeCount(); t++){
    tree.view.selection.getRangeAt(t,start,end);
    for(var v = start.value; v <= end.value; v++) {
      var item = tree.view.getItemAtIndex(v);
      if(item.getAttribute('container') != 'true') {
        list.push([parseInt(item.getAttribute("metric")), item.getAttribute("param"), item.getAttribute("translatedParam"), eval(item.getAttribute("check"))]);
      }
    }
  }
  var currentid;
  if(list.length == 0) {
    dock.displayInfo('Select a metric or healthcheck to add to the rackview', window);
  } else {
    var added = false;
    for(var j = 0; j < list.length; j++) {
      var found = false;
      for(var i = 0; !found && (i < rackmetrics.length); i++) {
        found = (rackmetrics[i][0] == list[j][0]) && (rackmetrics[i][1] == list[j][1]);
      }
      if(!found) {
        added = true;
        rackmetrics.push(list[j]);
      }
    }
    if(added) updateList();
  }
}

function removeFromRack() {
  var list = new Array();
  var racklist = document.getElementById('racklist');
  for(var i = 0; i < racklist.childNodes.length; i++) {
    var item = racklist.childNodes[i];
    if(eval(item.getAttribute('selected'))) {
      list.push(i - 1);
    }
  }
  if(list.length == 0) {
    dock.displayInfo('Select at least one metric or healthcheck to remove from the rackview', window);
  } else {
    list = list.sort();
    for(var i = list.length - 1; i >= 0; i--) {
      rackmetrics.splice(list[i], 1);
    }
    updateList();
  }
}

function removeMetric(index) {
  rackmetrics.splice(index, 1);
  updateList();
}

function updateList() {
  var deck = document.getElementById('deck');
  if(rackmetrics.length == 0) {
    deck.selectedIndex = 0;
  } else {
    deck.selectedIndex = 1;
  }
  var racklist = document.getElementById('rackrows');
  while(racklist.firstChild) racklist.removeChild(racklist.firstChild);

  for(var i = 0; i < rackmetrics.length; ) {
    var item = document.createElement('row');
    item.setAttribute('align', 'center');
    item.setAttribute('metric', rackmetrics[i][0]);
    item.setAttribute('param', rackmetrics[i][1]);
    item.setAttribute('translatedParam', rackmetrics[i][2]);
    var image = document.createElement('image');
    var unit = false;
    if((rackmetrics[i].length > 3) && rackmetrics[i][3]) {
      image.setAttribute('src', 'chrome://cmgui/content/icons/Health_16.png');
    } else {
      image.setAttribute('src', 'chrome://cmgui/content/icons/Monitoring_16.png');
      unit = true;
    }
    var label = document.createElement('label');
    var metricOrCheck = rack.cluster.find(rackmetrics[i][0]);
    if(isObject(metricOrCheck)) {
      var name = metricOrCheck.resolveName()
      if(rackmetrics[i][2].length > 0) name += '[' + rackmetrics[i][2] + ']';
      else if(rackmetrics[i][1].length > 0) name += '[' + rackmetrics[i][1] + ']';
      if(unit && metricOrCheck.get('unit')) name += " (" + metricOrCheck.get('unit') + ")";
      label.setAttribute('value', name);
      item.appendChild(image);
      item.appendChild(label);
      var minus = document.createElement("button");
      minus.setAttribute('class', 'minus-button');
      minus.setAttribute('oncommand', "removeMetric(" + i + ");");
      item.appendChild(minus);
      racklist.appendChild(item);
      i++;
    } else {
      rackmetrics.splice(i, 1);
    }
  }
}

function ok() {
  apply();
  cancel();
}

function apply() {
  var item = document.getElementById('rate').selectedItem;
  rack.refreshRate = parseInt(item.getAttribute('time'));
  rack.live = document.getElementById('live').checked;
  rack.fillNoData = document.getElementById('fill').checked;
  rack.setMetrics(rackmetrics);
  if(topview) {
    topview.refreshRate = rack.refreshRate;
    topview.live = rack.live;
    topview.fillNoData = rack.fillNoData
    topview.setMetrics(rackmetrics);
    topview.refreshDrill();
  } else {
    rack.refresh();
    myDialog.apply();
  }
}

function cancel() {
  close();
}

function waitIcon() {
  if(waitCount == 0) {
    window.setCursor("wait");
    for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("wait");
  }
  waitCount++;
}

function unwaitIcon() {
  waitCount--;
  if(waitCount <= 0) {
    window.setCursor("auto");
    for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("auto");
    if(waitCount < 0) println("Mismatch waitCursor");
    waitCount = 0;
  }
}
