/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var scrollbarwidth = 24;
var RACK_DEBUG = 0x02 | 0x08 | 0x20; 
// 0x01
// 0x02 : positions
// 0x04
// 0x08 : top positions
// 0x10
// 0x20 : forced layout

var RACK_VIEWMODE_RACKS = 1;
var RACK_VIEWMODE_ROOMS = 2;
var RACK_VIEWMODE_CLUSTERS = 3;

function cvFakeRack(uniqueKey, name, width, height, inverted, x, y, room, one) {
  this.uniqueKey = uniqueKey;
  this.name = name;
  this.width = isundefined(width, 19);
  this.height = isundefined(height, 42);
  this.inverted = isundefined(inverted, false);
  this.xCoordinate = isundefined(x, 0);
  this.yCoordinate = isundefined(y, 0);
  this.room = isundefined(room, '');
  this.one = isundefined(one, 1);
  this.fake = true;
}

cvFakeRack.prototype.get = function(property) {
  if(this.hasOwnProperty(property)) return this[property];
  println('cvFakeRack.prototype.get, unknown property: ' + property);
  return null;
}

cvFakeRack.prototype.resolveName = function() {
  return this.name;
}

cvFakeRack.prototype.getRackRoomName = function() {
  var room = trim(this.get('room'));
  if(room == '') room = 'Untitled Room';  
  return room;
}

function RackView(cluster) {
  this.cluster = cluster;
  this.otherClusters = new Object();
  this.other = 0;
  this.topnames = 0;
  this.realracks = 0;
  this.devices = 0;
  this.cloudregions = 0;
  this.databox = new Object();
  this.backbox = new Object();
  this.data = new Object();
  this.normdata = new Object();
  this.racks = new Array();
  this.lastfill = 0;
  this.rooms = 0;
  this.positions = 0;
  this.stopped = false;
  this.fillTimer = 0;  
  this.fillCompleted = true;
  this.size = 0;
  this.padding = 0;
  this.forceRedraw = false;
  this.rackobjects = 0;

  this.svg = 0;
  this.wx = 0;
  this.wy = 0;
  
  if(this.cluster) {
    this.metrics = cluster.rackMetrics;
  } else {
    this.metrics = new Array();
  }
  this.refreshRate = dock.generalSettings.monitoringRefresh;
  this.min = 0;
  this.max = 0;
  this.fmin = 0;
  this.fmax = 0;
  this.smin = 0;
  this.smax = 0;
  this.diff = 0;
  this.live = false;
  this.fillNoData = true;
  this.timer = 0;
  this.updating = false;
  
  this.viewMode = 0; // 1: toprack, 2: room, 3: cluster
  this.showRoom = new Array();
  this.showRack = new Array();
  
  this.clusterCount = 0;
  this.roomCount = 0;
  this.rackCount = 0;
  
  this.selectedNodes = 0;
  this.selectedNodesMap = 0;
  
  this.getCustomBounds();
}

RackView.prototype.waitIcon = function() {
  if(typeof(waitIcon) == 'function') {
    waitIcon();
  } else {
    dock.waitIcon();
  }
}

RackView.prototype.unwaitIcon = function() {
  if(typeof(unwaitIcon) == 'function') {
    unwaitIcon();
  } else {
    dock.unwaitIcon();
  }
}

RackView.prototype.addOtherCluster = function(cluster) {
  if(this.otherClusters.hasOwnProperty(cluster.key)) return;
  this.otherClusters[cluster.key] = new RackView(cluster);
  this.forceRedraw = true;
}

RackView.prototype.getView = function() {
  var list = new Array()
  if(this.viewMode <= RACK_VIEWMODE_CLUSTERS) list.push('All Clusters');
  if(this.viewMode <= RACK_VIEWMODE_ROOMS) list.push(this.cluster.name);
  if(this.viewMode <= RACK_VIEWMODE_RACKS) list.push(this.showRoom.join(','));
  if(this.viewMode == 0) {
    var names = new Array();
    for(var i = 0; i < this.showRack.length; i++) {
      var rack = this.cluster.find(this.showRack[i]);
      var name;
      if(rack) {
        name = rack.resolveName();
      } else if(this.showRack[i] == 0) {
        name = 'Unassigned';
      } else {
        name = 'Unknown rack';
      } 
      names.push(name);
    }
    list.push(names.join(','));
  }
  println('view ' + this.viewMode + ' ' + list);
  return list;
}

RackView.prototype.setView = function(view) {
  if(view == 0) {
    this.forceRedraw = true;
    this.viewMode = RACK_VIEWMODE_CLUSTERS;
    this.cluster = 0;
    this.other = 0;
    this.showRoom = 0;
    this.showRack = 0;
    this.lowlevel();
    return true;
  } else if(view == 1) {
    this.forceRedraw = true;
    this.viewMode = RACK_VIEWMODE_ROOMS;
    this.showRoom = 0;
    this.showRack = 0;
    this.lowlevel();
    return true;    
  } else if(view == 2) {
    this.forceRedraw = true;
    this.viewMode = RACK_VIEWMODE_RACKS;
    this.showRack = 0;
    this.lowlevel();
    return true;
  }
  return false;
}

RackView.prototype.lowlevel = function() {
  for(var i = 1; i <= 2; i++) {
    var low = document.getElementById('lowlevel' + i);
    if(low) {
      if(this.viewMode == 0) {
        low.removeAttribute('hidden');
      } else {
        low.setAttribute('hidden', true);
      }
    }
  }
}

RackView.prototype.updateDeviceSelection = function(list) {
  var old;
  var allOld = false;
  if(this.cluster.rackDeviceSelection) {
    old = this.cluster.rackDeviceSelection.slice();
  } else {
    allOld = true;
    old = new Array();
  }
  var allNew = false;
  if(list.length) {
    this.cluster.rackDeviceSelection = list;
    old = old.concat(list);
  } else {
    this.cluster.rackDeviceSelection = 0;
    allNew = true;
  }
  this.setDeviceSelection(this.cluster.rackDeviceSelection);
  this.updateSelected(old, allOld, allNew);  
}

RackView.prototype.setDeviceSelection = function(list) {
  if(list.length) {
    this.selectedNodes = list;
  } else {
    this.selectedNodes = 0;
  }
  this.selectedNodesMap = 0;
}

RackView.prototype.isSelected = function(key) {
  if(this.selectedNodes) {
    if(!this.selectedNodesMap) {
      this.selectedNodesMap = new Object();
      for(var i = 0; i < this.selectedNodes.length; i++) {
        this.selectedNodesMap[this.selectedNodes[i]] = true;
      }
    }
    if(this.selectedNodesMap.hasOwnProperty(key)) return 1;
    return 0;
  }
  return -1;
}


RackView.prototype.backcolor = function(ss) {
  if(ss > 0) return '#abefab';
  return '#cdcdcd';
}

RackView.prototype.updateSelected = function(list, allOld, allNew) {
  var done = new Object();
  for(var i = 0; i < list.length; i++) {
    var key = list[i];
    if(!done.hasOwnProperty(key) && this.backbox.hasOwnProperty(key)) {
      var ss = this.isSelected(key);
      this.backbox[key].setAttribute("fill", this.backcolor(ss));
      if(this.databox.hasOwnProperty(key)) {
        if(ss == 0) {
          this.clearColors(this.databox[key]);
        } else if(!allOld && this.normdata.hasOwnProperty(key)) {
          this.fillColors(this.databox[key], this.normdata[key]);
        }
      }
      done[key] = true;
    }
  }
  if(allOld && !allNew) {
    for(var key in this.databox) {
      if(!done.hasOwnProperty(key)) {
        this.clearColors(this.databox[key]);
      }
    }
  } else if(!allOld && allNew) {
    this.updateValues();
  }
}

RackView.prototype.setLive = function(live) {
  this.live = live;
}

RackView.prototype.setFillNoData = function(fillNoData) {
  this.fillNoData = fillNoData;
}

RackView.prototype.deviceValues = function(key) {
  if(this.data.hasOwnProperty(key)) return this.data[key];
  return new Array();
}

RackView.prototype.setMetrics = function(metrics) {
  this.metrics = metrics.slice(0);
  if(this.cluster) this.cluster.rackMetrics = this.metrics;
  this.getCustomBounds();
}

RackView.prototype.getCustomBounds = function() {
  for(var i = 0; i < this.metrics.length; i++) {
    var metric = this.cluster.find(this.metrics[i][0]);
    if(isObject(metric)) {
      var name = metric.get('name');
      if(this.metrics[i][1].length > 0) name += ':' + this.metrics[i][1];
      var bounds = dock.customMetricBounds.getBounds(name);
      this.setBounds(bounds[0], bounds[1], i, false);
    }
  }  
}

RackView.prototype.setBounds = function(smin, smax, index, update) {
  update = isundefined(update, true);
  var changed = false;
  if(!this.smin) this.smin = new Array();
  for(var i = this.smin.length; i <= index; i++) this.smin.push(NaN);
  changed = (this.smin[index] != smin) || changed;
  this.smin[index] = smin;
  if(!this.smax) this.smax = new Array();
  for(var i = this.smax.length; i <= index; i++) this.smax.push(NaN);
  changed = (this.smax[index] != smax) || changed;
  this.smax[index] = smax;
  if(update && changed) {
    this.legend();
    this.refresh();
  }
}

RackView.prototype.metricInfo = function() {
  var info = new Array();
  for(var i = 0; i < this.metrics.length; i++) {
    var metric = this.cluster.find(this.metrics[i][0]);
    if(isObject(metric)) {
      var name = metric.get('name');
      if(this.metrics[i][2].length > 0) name += ' (' + this.metrics[i][2] + ')';
      else if(this.metrics[i][1].length > 0) name += ' (' + this.metrics[i][1] + ')';
      var isCheck = (this.metrics[i].length > 3) && this.metrics[i][3];
      var unit = '';
      if(!isCheck) unit = metric.get('unit');
      info.push([name, unit, isCheck]);
    }
  }
  return info;
}

function newLabel(value, index, low, name, unit) {
  var label = document.createElement('label');
  label.setAttribute('style', 'color: #efefef; font-size: 11px;');
  label.setAttribute('value', value);
  index = isundefined(index, -1);
  if(index >= 0) label.setAttribute('onclick', 'setMinMax(' + index + ',' + low + ',"' + name + '","' + unit + '");');
  return label;
}

function newLabelDark(value, index, low, name, unit) {
  var label = document.createElement('label');
  label.setAttribute('style', 'color: #3c3c3c; font-size: 11px;');
  label.setAttribute('value', value);
  index = isundefined(index, -1);
  if(index >= 0) label.setAttribute('onclick', 'setMinMax(' + index + ',' + low + ',"' + name + '","' + unit + '");');
  return label;
}

function setMinMax(index, low, name, unit) {
  var work = topview;
  if(!work) work = rack;
  var args = new Object();
  args['name'] = name;
  args['unit'] = unit;
  args['low'] = low;
  if(work.smin && (index < work.smin.length) && !isNaN(work.smin[index])) args['smin'] = work.smin[index];
  if(work.smax && (index < work.smax.length) && !isNaN(work.smax[index])) args['smax'] = work.smax[index];
  args['min'] = work.min[index];
  args['max'] = work.max[index];
  var d = new dock.cvDialog("metricminmax.xul", args);
  d.parentWindow = window;
  d.onOk = function(smin, smax) {
    dock.customMetricBounds.setBounds(name, smin, smax);
    work.setBounds(smin, smax, index);
  }
  dock.dialogs.show(d);  
}

function newScale(check) {
  var box = document.createElement('vbox');
  box.setAttribute('flex', 1);
  box.setAttribute('pack', 'center');
  var label = document.createElement('hbox');
  if(check) {
    label.setAttribute('class', 'checklegend');
  } else {
    label.setAttribute('class', 'legend');
  }
  box.appendChild(label);
  return box;
}

RackView.prototype.getLegendMin = function(i) {
  if(this.smin && (i < this.smin.length) && !isNaN(this.smin[i])) return this.smin[i];
  if(this.min && (i < this.min.length)) return this.min[i];
  return NaN;
}

RackView.prototype.getLegendMax = function(i) {
  if(this.smax && (i < this.smax.length) && !isNaN(this.smax[i])) return this.smax[i];
  if(this.max && (i < this.max.length)) return this.max[i];
  return NaN;
}

RackView.prototype.legend = function() {
  var legend = document.getElementById('legend');
  if(!legend || !this.cluster) return;
  while(legend.firstChild) legend.removeChild(legend.firstChild);
  var n = this.metrics.length;
  if(legend.tagName == 'rows') {
    for(var i = 0; i < n; i++) {
      var metric = this.cluster.find(this.metrics[i][0]);
      if(!metric) continue;
      var name = metric.resolveName();
      var ischeck = (this.metrics[i].length > 3) && this.metrics[i][3];
      var unit = ''
      if(!ischeck) unit = metric.get('unit');
      if(this.metrics[i][2].length > 0) name += ':' + this.metrics[i][2];
      else if(this.metrics[i][1].length > 0) name += ':' + this.metrics[i][1];
      var row = document.createElement('hbox');
      row.setAttribute('pack', 'center');
      row.appendChild(newLabelDark(name));
      legend.appendChild(row);
      var row = document.createElement('row');
      row.setAttribute('align', 'center');
      if(ischeck) {
        row.appendChild(newLabelDark('PASS'));
        row.appendChild(newScale(true));
        row.appendChild(newLabelDark('FAIL'));
      } else {
        row.appendChild(newLabelDark(formatKMGselect(this.getLegendMin(i), unit), i, 0, name, unit));
        row.appendChild(newScale(false));
        row.appendChild(newLabelDark(formatKMGselect(this.getLegendMax(i), unit), i, 1, name, unit));
      }
      legend.appendChild(row);
      if(i + 1 < n) {
        var row = document.createElement('row');
        row.appendChild(newLabelDark(''));
        legend.appendChild(row);
      }
    }    
  } else {
    for(var i = 0; i < n; i++) {
      var column1 = document.createElement('column');
      var column2 = document.createElement('column');
      var column3 = document.createElement('column');
      var column4 = document.createElement('column');
      var metric = this.cluster.find(this.metrics[i][0]);
      if(!metric) continue;
      var name = metric.resolveName();
      var ischeck = (this.metrics[i].length > 3) && this.metrics[i][3];
      var unit = ''
      if(!ischeck) unit = metric.get('unit');
      if(this.metrics[i][2].length > 0) name += ':' + this.metrics[i][2];
      else if(this.metrics[i][1].length > 0) name += ':' + this.metrics[i][1];
      column1.appendChild(newLabel(name));
      if(ischeck) {
        column2.appendChild(newLabel('PASS'));
        column3.appendChild(newScale(true));
        column4.appendChild(newLabel('FAIL'));
      } else {
        column2.appendChild(newLabel(formatKMGselect(this.getLegendMin(i), unit), i, 0, name, unit));
        column3.appendChild(newScale(false));
        column4.appendChild(newLabel(formatKMGselect(this.getLegendMax(i), unit), i, 1, name, unit));
      }
      i++;
      if(i < n) {
        var metric = this.cluster.find(this.metrics[i][0]);
        if(metric) {
          var name = metric.resolveName();
          var ischeck = (this.metrics[i].length > 3) && this.metrics[i][3];
          var unit = ''
          if(!ischeck) unit = metric.get('unit');
          if(this.metrics[i][2].length > 0) name += ':' + this.metrics[i][2];
          else if(this.metrics[i][1].length > 0) name += ':' + this.metrics[i][1];
          column1.appendChild(newLabel(name));
          if(ischeck) {
            column2.appendChild(newLabel('PASS'));
            column3.appendChild(newScale(true));
            column4.appendChild(newLabel('FAIL'));
          } else {
            column2.appendChild(newLabel(formatKMGselect(this.getLegendMin(i), unit), i, 0, name, unit));
            column3.appendChild(newScale(false));
            column4.appendChild(newLabel(formatKMGselect(this.getLegendMax(i), unit), i, 1, name, unit));
          }
        } else {
          column1.appendChild(newLabel('Metric removed'));
        }
      } else {
        column1.appendChild(document.createElement('spacer'));
        column2.appendChild(document.createElement('spacer'));
        column3.appendChild(document.createElement('spacer'));
        column4.appendChild(document.createElement('spacer'));
      }
      legend.appendChild(column1);
      legend.appendChild(column2);
      legend.appendChild(column3);
      legend.appendChild(column4);
    }
  }
}

RackView.prototype.largestRack = function() {
  if(this.viewMode) {
    return 4294967296; // 2 ^ 32
  } else {
    var mh = 0;
    for(var i = 0; i < this.racks.length; i++) {
      if(!this.racks[i].fake) mh = Math.max(mh, this.racks[i].rack.get('height'));
    }
    if(mh == 0) mh = 42;
    return mh;
  }
}

RackView.prototype.splitfake = function(height) {
  height = isundefined(height, this.largestRack());
  var n = this.racks.length;
  for(var i = 0; i < n; i++) {
    var add = this.racks[i].splitfake(height);
    this.racks.concat(add);
  }
}

RackView.prototype.rackspace = function(size, padding) {
  this.splitfake();
  size = isundefined(size, 16);
  this.rooms = new Object();
  for(var i = 0; i < this.racks.length; i++) {
    var work = this.racks[i];
    if(work.empty()) continue;
    var rack = work.rack;
    var room = rack.getRackRoomName();
    if((this.showRack && this.showRack.length && (this.showRack.find(Math.floor(rack.get('uniqueKey'))) < 0)) || (this.showRoom && this.showRoom.length && (this.showRoom.find(room) < 0))) continue;
    var space = work.requiredSpace(size);
    if(!this.rooms.hasOwnProperty(room)) this.rooms[room] = [new Object(), new Object(), new Object(), new Object()];
    var mapX = this.rooms[room][0];
    var mapY = this.rooms[room][1];
    var mapXY = this.rooms[room][2];
    var mapSX = this.rooms[room][3];
    var x = sprintf('%010d', rack.get('xCoordinate'));
    var y = sprintf('%010d', rack.get('yCoordinate'));
    var c = x + y;
    if(mapSX.hasOwnProperty(c)) {      
      mapSX[c] = mapSX[c] + space[0] + padding;
    } else {
      mapSX[c] = space[0];
    }    
    if(mapX.hasOwnProperty(x)) {
      mapX[x] = Math.max(mapX[x], mapSX[c]);
    } else {
      mapX[x] = mapSX[c];
    }
    if(mapY.hasOwnProperty(y)) {
      mapY[y] = Math.max(mapY[y], space[1]);
    } else {
      mapY[y] = space[1];
    }
    if(mapXY.hasOwnProperty(c)) {      
      mapXY[c].push(i);
    } else {
      mapXY[c] = [i];
    }
  }
}

RackView.prototype.rackpositions = function(size, padding) {
  size = isundefined(size, 16);
  padding = isundefined(padding, 0);
  if(!this.rooms) this.rackspace(size, padding);
  this.positions = new Array();
  var srx = 0;
  var sry = 0;
  this.mx = 0;
  this.my = 0;
  for(var room in this.rooms) {
    var mapX = this.rooms[room][0];
    var mapY = this.rooms[room][1];
    var mapXY = this.rooms[room][2];
    var mapSX = this.rooms[room][2];
    if(dock.generalSettings.rackGaps) {
      fillRackMapGaps(mapX);
      fillRackMapGaps(mapY);
    }
    var xx = new Object()
    var ssx = padding;
    var skx = objectSortedArray(mapX);
    for(var i = 0; i < skx.length; i++) {
      var x = skx[i];
      xx[x] = ssx;
      ssx += mapX[x] + padding;
    }
    var yy = new Object()
    var ssy = padding;
    var sky = objectSortedArray(mapY);
    for(var i = 0; i < sky.length; i++) {
      var y = sky[i];
      yy[y] = ssy;
      ssy += mapY[y] + padding;
    }
    var cy = 1e+308;
    var skxy = objectSortedArray(mapXY);
    for(var j = 0; j < skxy.length; j++) {
      var xy = skxy[j];
      var x = xy.substr(0, xy.length / 2);
      var y = xy.substr(xy.length / 2);
      var r = mapXY[xy];
      var sx = srx + xx[x];
      var sy = sry + yy[y];
      for(var i = 0; i < r.length; i++) {
        var work = this.racks[r[i]];
        var space = work.requiredSpace(size);
        this.positions[r[i]] = [sx, sy];
        sx += space[0] + padding;
        if(RACK_DEBUG & 0x02) println('Pos ' + this.racks[r[i]].rack.resolveName() + ' ' + r[i] + ' : ' + this.positions[r[i]]);
        this.mx = Math.max(this.mx, sx);
        this.my = Math.max(this.my, sy + space[1] + padding);
      }
    }
    srx += ssx + 16;
  }
}

RackView.prototype.clearScreen = function() {
  var box = document.getElementById("box");
  while(box.lastChild) box.removeChild(box.lastChild);
}

RackView.prototype.resize = function() {
  this.svg = document.getElementById("svg");
  this.wx = this.svg.parentNode.clientWidth;
  this.wy = this.svg.parentNode.clientHeight;
  
  this.svg.setAttribute('width', Math.max(this.mx, this.wx - scrollbarwidth));
  this.svg.setAttribute('height', Math.max(this.my, this.wy - scrollbarwidth));
}

RackView.prototype.draw = function(size, padding) {
  this.waitIcon();
  size = isundefined(size, 16);
  if(size <= 0) size = 16;
  if(!this.positions) this.rackpositions(size, padding);
  this.lastfill = 0;
  this.size = size;
  this.padding = padding;
  this.stopped = false;  
  
  this.resize();
  
  var box = document.getElementById("box");
  while(box.lastChild) box.removeChild(box.lastChild);

  for(var i = 0; i < this.racks.length; i++) {
    var rack = this.racks[i].rack;
    if(rack.get('height') > 0) {
      if(this.positions.hasOwnProperty(i)) {
        this.racks[i].draw(box, this.positions[i][0], this.positions[i][1], size, (this.positions[i][0] < this.wx) && (this.positions[i][1] < this.wy));
      } else if(RACK_DEBUG & 0x10) {
        println(' --- NO POSITION FOR RACK ' + i + ' ---');
        printall(this.positions);
      }
    }
  }
  this.delayedFill();
  this.legend();
  this.unwaitIcon();
}

RackView.prototype.clearColors = function(cbox) {
  while(cbox.firstChild) cbox.removeChild(cbox.firstChild);
}

RackView.prototype.rackspaceTop = function(size, padding, keepspace) {
  this.splitfake();
  size = isundefined(size, 16);
  keepspace = isundefined(keepspace, true);
  this.rooms = new Object();
  if(this.viewMode == RACK_VIEWMODE_RACKS) {
    var minx = 1e+308;
    var miny = 1e+308;
    var maxx = -1e+308;
    var maxy = -1e+308;
    for(var i = 0; i < this.racks.length; i++) {
      var work = this.racks[i];
      if(work.empty()) continue;
      var rack = work.rack;
      var room = rack.getRackRoomName();
      if((this.showRack && this.showRack.length && (this.showRack.find(rack.get('uniqueKey')) < 0)) || (this.showRoom && this.showRoom.length && (this.showRoom.find(room) < 0))) continue;
      var space = work.requiredSpaceTop(size);
      if(!this.rooms.hasOwnProperty(room)) this.rooms[room] = [new Object(), new Object(), new Object(), new Object()];
      var mapX = this.rooms[room][0];
      var mapY = this.rooms[room][1];
      var mapXY = this.rooms[room][2];
      var mapSX = this.rooms[room][3];
      var rx = rack.get('xCoordinate');
      var ry = rack.get('yCoordinate');
      minx = Math.min(minx, rx);
      maxx = Math.max(maxx, rx);
      miny = Math.min(miny, ry);
      maxy = Math.max(maxy, ry);
      var x = sprintf('%010d', rx);
      var y = sprintf('%010d', ry);
      var c = x + y;
      if(mapSX.hasOwnProperty(c)) {      
        mapSX[c] = mapSX[c] + space[0] + padding;
      } else {
        mapSX[c] = space[0];
      }    
      if(mapX.hasOwnProperty(x)) {
        mapX[x] = Math.max(mapX[x], mapSX[c]);
      } else {
        mapX[x] = mapSX[c];
      }
      if(mapY.hasOwnProperty(y)) {
        mapY[y] = Math.max(mapY[y], space[1]);
      } else {
        mapY[y] = space[1];
      }
      if(mapXY.hasOwnProperty(c)) {
        mapXY[c].push(i);
      } else {
        mapXY[c] = [i];
      }
    }
    if(keepspace) {
      var fake = new cvFakeRack(-1, 'fake');
      fake = new RackObject(fake, this);
      var space = fake.requiredSpaceTop(size);
      for(var i = minx; i <= maxx; i++) {
        var x = sprintf('%010d', i);
        if(!mapX.hasOwnProperty(x)) mapX[x] = space[0];
      }
      for(var j = miny; j <= maxy; j++) {
        var y = sprintf('%010d', j);
        if(!mapY.hasOwnProperty(y)) mapY[y] = space[1];
      }
    }    
  } else {
    this.topnames = new Array();
    if(this.viewMode == RACK_VIEWMODE_ROOMS) {
      var known = new Object();
      for(var i = 0; i < this.racks.length; i++) {
        var work = this.racks[i];
        if(work.empty()) continue;
        var rack = work.rack;
        var room = rack.getRackRoomName();
        if(!known.hasOwnProperty(room)) {
          this.topnames.push([room, room]);
          known[room] = true;
        }
      }
      this.topnames = sort_unique(this.topnames);
    } else if(this.viewMode == RACK_VIEWMODE_CLUSTERS) {
      for(var c in this.otherClusters) {
        this.topnames.push([this.otherClusters[c].cluster.name, this.otherClusters[c]]);
      }
      this.topnames.sort();
    } else {
      println('rackspaceTop, wrong mode: ' + this.viewMode);
      return;
    }
    var fake = new cvFakeRack(-1, 'fake');
    fake = new RackObject(fake, this);
    var space = fake.requiredSpaceTop(size);
    var room = '';
    if(!this.rooms.hasOwnProperty(room)) this.rooms[room] = [new Object(), new Object(), new Object(), new Object()];
    var mapX = this.rooms[room][0];
    var mapY = this.rooms[room][1];
    var mapXY = this.rooms[room][2];
    var mapSX = this.rooms[room][3];
    var rackTopColumns = dock.generalSettings.rackTopColumns;
    if(rackTopColumns == 0) rackTopColumns = Math.floor(this.wx / (space[0] + padding));
    for(var i = 0; i < this.topnames.length; i++) {
      var x = sprintf('%010d', (i % rackTopColumns));
      var y = sprintf('%010d', Math.floor(i / rackTopColumns));
      if(RACK_DEBUG & 0x08) println('Top: ' + this.topnames[i][0] + ' (' + x + ', ' + y + ')');
      var c = x + y;
      if(mapSX.hasOwnProperty(c)) {      
        mapSX[c] = mapSX[c] + space[0] + padding;
      } else {
        mapSX[c] = space[0];
      }    
      if(mapX.hasOwnProperty(x)) {
        mapX[x] = Math.max(mapX[x], mapSX[c]);
      } else {
        mapX[x] = mapSX[c];
      }
      if(mapY.hasOwnProperty(y)) {
        mapY[y] = Math.max(mapY[y], space[1]);
      } else {
        mapY[y] = space[1];
      }
      if(mapXY.hasOwnProperty(c)) {
        mapXY[c].push(i);
      } else {
        mapXY[c] = [i];
      }      
    }
  }
}

RackView.prototype.rackpositionsTop = function(size, padding) {
  size = isundefined(size, 16);
  padding = isundefined(padding, 0);
  if(!this.rooms) this.rackspaceTop(size, padding);
  this.positions = new Array();
  var srx = 0;
  var sry = 0;
  this.mx = 0;
  this.my = 0;
  var fake = new RackObject(new cvFakeRack(-1, 'fake'), this);
  for(var room in this.rooms) {
    var mapX = this.rooms[room][0];
    var mapY = this.rooms[room][1];
    var mapXY = this.rooms[room][2];
    var mapSX = this.rooms[room][2];
    var xx = new Object()
    var ssx = padding;
    var skx = objectSortedArray(mapX);
    for(var i = 0; i < skx.length; i++) {
      var x = skx[i]
      xx[x] = ssx;
      ssx += mapX[x] + padding;
    }
    var yy = new Object()
    var ssy = padding;
    var sky = objectSortedArray(mapY);
    for(var i = 0; i < sky.length; i++) {
      var y = sky[i]
      yy[y] = ssy;
      ssy += mapY[y] + padding;
    }
    var cy = 1e+308;
    var skxy = objectSortedArray(mapXY);
    for(var j = 0; j < skxy.length; j++) {
      var xy = skxy[j];
      var x = xy.substr(0, xy.length / 2);
      var y = xy.substr(xy.length / 2);
      var r = mapXY[xy];
      var sx = srx + xx[x];
      var sy = sry + yy[y];
      for(var i = 0; i < r.length; i++) {
        var work;
        if(this.viewMode == RACK_VIEWMODE_RACKS) {
          work = this.racks[r[i]];
        } else {
          work = fake;
          work.rack.name = this.topnames[r[i]][0];
        }
        var space = work.requiredSpaceTop(size);
        this.positions[r[i]] = [sx, sy];
        sx += space[0] + padding;
        if(RACK_DEBUG & 0x02) println('Pos Top ' + work.rack.resolveName() + ' ' + r[i] + ' : ' + this.positions[r[i]]);
        this.mx = Math.max(this.mx, sx);
        this.my = Math.max(this.my, sy + space[1] + padding);
      }
    }
    srx += ssx + 16;
  }
}

RackView.prototype.drawTop = function(size, padding, room, cluster) {
  this.drawRoom = isundefined(room, -1);
  this.drawCluster = isundefined(cluster, -1);

  this.waitIcon();
  size = isundefined(size, 16);
  if(size <= 0) size = 16;
  if(!this.positions) this.rackpositionsTop(size, padding);
  
  this.lastfill = 0;
  this.size = size;
  this.padding = padding;
  this.stopped = false;  
  
  this.resize();
  
  var box = document.getElementById("box");
  while(box.lastChild) box.removeChild(box.lastChild);

  if(this.viewMode == RACK_VIEWMODE_RACKS) {
    for(var i = 0; i < this.racks.length; i++) {
      var rack = this.racks[i].rack;
      if(this.positions.hasOwnProperty(i)) {
        this.racks[i].drawTop(box, this.positions[i][0], this.positions[i][1], size, (this.positions[i][0] < this.wx) && (this.positions[i][1] < this.wy));
      } else if(RACK_DEBUG & 0x10) {
        println(' --- NO TOP POSITION FOR RACK ' + i + ' ---');
        printall(this.positions);
      }
    }
  } else {
    for(var i = 0; i < this.topnames.length; i++) {
      var fake;
      var view;
      if(typeof(this.topnames[i][1]) == 'string') {
        fake = new cvFakeRack(this.cluster);
        view = this;
      } else {
        fake = new cvFakeRack(this.topnames[i][1].cluster);
        view = this.topnames[i][1];
        view.viewMode = this.viewMode;
      }
      fake.name = this.topnames[i][0];
      fake = new RackObject(fake, view);
      if(this.positions.hasOwnProperty(i)) {
        fake.drawTop(box, this.positions[i][0], this.positions[i][1], size, (this.positions[i][0] < this.wx) && (this.positions[i][1] < this.wy));
      } else { // TODO: remove
        println(' --- NO POSITION FOR RACK ' + fake.name + ' ---');
        printall(this.positions);
      }      
    }
  }
  this.delayedFill();
  this.legend();
  this.unwaitIcon();
}

RackView.prototype.fillColors = function(cbox, data) {
  this.clearColors(cbox);
  var n = 0;
  if(this.fillNoData) {
    n = this.metrics.length;
  } else {
    for(var i = 0; i < data.length; i++) {
      if(!kisNaN(data[i])) n++;
    }
  }
  if(n > 0) {
    var j = 0;
    for(var i = 0; i < data.length; i++) {
      if(!isNaN(data[i])) {
        if((this.metrics[i].length > 3) && this.metrics[i][3]) {
          svgrect(cbox, j / n, 0, 1 / n, 1, colorhash(0.5 + data[i] / 4.0), 0, '', 1);
        } else {
          svgrect(cbox, j / n, 0, 1 / n, 1, colorhash(data[i]), 0, '', 1);
        }
        j++;
      } else if(this.fillNoData) {
        j++;
      }
    }
  }
}

RackView.prototype.normalizData = function() {  
  this.diff = new Array();
  for(var i = 0; i < this.min.length; i++) {
    if(this.smin && (i < this.smin.length) && !isNaN(this.smin[i])) {
      this.min[i] = this.smin[i];
    } else if(!isNaN(this.fmin[i])) {
      this.min[i] = this.fmin[i];
    }
    if(this.smax && (i < this.smax.length) && !isNaN(this.smax[i])) {
      this.max[i] = this.smax[i];
    } else if(!isNaN(this.fmax[i])) {
      this.max[i] = this.fmax[i];
    }
    if(!isNaN(this.min[i]) && !isNaN(this.max[i])) {
      this.diff[i] = this.max[i] - this.min[i];
      if(this.diff[i] < 0.00001) this.diff[i] = 1;
    }
  }
//   this.normdata = new Object();
  for(var key in this.normdata) {
    this.normdata[key] = 0;
  }
  for(var key in this.data) {
    var data = this.data[key];
    var normdata = new Array();
    for(var i = 0; i < data.length; i++) {
      if(typeof(data[i]) == 'undefined') {
        normdata.push(NaN);
      } else if(isNaN(data[i]) || ((this.metrics[i].length > 3) && this.metrics[i][3])) {
        normdata.push(data[i]);
      } else if(data[i] <= this.min[i]) {
        normdata.push(0.0);
      } else if(data[i] >= this.max[i]) {
        normdata.push(1.0);
      } else if((this.metrics[i].length <= 3) || !this.metrics[i][3]) {
        normdata.push((data[i] - this.min[i]) / this.diff[i]);
      }
    }
    this.normdata[key] = normdata;
  }
  this.legend();
}

RackView.prototype.updateValues = function() {
  for(var key in this.normdata) {
    if(this.databox.hasOwnProperty(key) && (this.isSelected(key) != 0)) {
      var cbox = this.databox[key];
      if(!this.normdata) {
        this.clearColors(cbox[key]);
      } else {
        this.fillColors(cbox, this.normdata[key]);
      }
    }
  }
}

RackView.prototype.delayedFill = function() {
  if(this.stopped) {
    this.fillTimer = 0;
    this.fillCompleted = false;
  } else {
    if(this.cluster.treeFillBusy) {
      println("Rackview fill delayed");
      if(!this.cluster.treeFillFollowUp) this.cluster.treeFillFollowUp = new Array();
      this.cluster.treeFillFollowUp.push(createObjectCallback(this, this.delayedFill));
    } else {
      this.fillCompleted = true;
      this.fillTimer = window.setTimeout(createObjectCallback(this, this.fill), 50);
    }
  }
}

RackView.prototype.startDrill = function(redraw) {
  if(this.viewMode == 0) {
    this.start(redraw);
  } else {
    this.startTop(redraw);
  }
}

RackView.prototype.start = function(redraw) {
  redraw = isundefined(redraw, false);
  if(redraw || (this.size <= 0) || this.forceRedraw) {
    if(this.forceRedraw) this.init();
    this.forceRedraw = false;
    this.draw(this.size, this.padding);
  } else {
    this.delayedFill();
  }
}

RackView.prototype.startTop = function(redraw) {
  redraw = isundefined(redraw, false);
  if(redraw || (this.size <= 0) || this.forceRedraw) {
    if(this.forceRedraw) this.init();
    this.forceRedraw = false;
    this.drawTop(this.size, this.padding);
  } else {
    this.delayedFill();
  }
}

RackView.prototype.stop = function() {
  println('Stop rackview');
  this.stopped = true;
  if(this.fillTimer) {
    println('Clear rackview fill timer');
    window.clearTimeout(this.fillTimer);
    this.fillTimer = 0;
    this.fillCompleted = false;
  }
  this.stoptimer();
}

RackView.prototype.stoptimer = function() {
  if(this.timer) {
    println('Clear rackview timer');
    window.clearTimeout(this.timer);
    this.timer = 0;
    return true;
  }
  return false;
}

RackView.prototype.fill = function() {
  this.fillTimer = 0;
  var j = 0;
  for(; this.lastfill < this.racks.length; this.lastfill++) {
    var work = this.racks[this.lastfill];
    if(!work.fill) {
      work.fill = true;
      if(typeof(work.fillFunction) == 'function') {
        j++;
        work.fillFunction();
        if(j >= dock.RACK_FILL_COUNT) {
          this.delayedFill();
          return;
        }
      }
    }
  }
  this.fillCompleted = true;
  println('RackView delayed fill: DONE');
}

RackView.prototype.clear = function() {
  this.stop();
  this.rooms = 0;
  this.positions = 0;  
}

RackView.prototype.init = function() {
  this.waitIcon();
  
  this.databox = new Object();
  this.backbox = new Object();
  this.data = new Object();
  this.racks = new Array();
  this.rooms = 0;
  this.positions = 0;

  if(this.viewMode == RACK_VIEWMODE_CLUSTERS) {
    for(var c in this.otherClusters) {
      this.otherClusters[c].init();
    }
    this.unwaitIcon();
    return;
  }
  
  this.realracks = this.cluster.find('Rack');
  this.devices = this.cluster.typeMap('Device');
  this.cloudregions = this.cluster.find('CloudRegion');
   
  this.rackobjects = new Object();
  for(var i = 0; i < this.realracks.length; i++) {
    this.rackobjects[this.realracks[i].get('uniqueKey')] = new RackObject(this.realracks[i], this);
  }
  // no rack
  this.rackobjects[0] = new RackObject(new cvFakeRack(0, 'Unassigned', 19, 0, false, 0, 0, 'Unassigned Room'), this);
  // per cloud region
  for(var i = 0; i < this.cloudregions.length; i++) {
    var key = this.cloudregions[i].get('uniqueKey');
    this.rackobjects[key] = new RackObject(new cvFakeRack(key, this.cloudregions[i].resolveName(), 19, 0, false, 0, 0, 'Cloud'), this);
  }
  
  var rackgroups = new Object();  
  var freedevices = new Object();

  for(var key in this.devices) {
    var device = this.devices[key];
    if(device.get('childType') == 'Chassis') {
      var rack = device.getRack();
      var members = [device];
      var layout = device.get('layout');
      if(layout == '') {
        var ud1 = device.get('userdefined1');
        if(startswith(ud1, 'layout ')) {
          layout = ud1.substr(7);
          if(RACK_DEBUG & 0x20) println('UserDefined1 layout: ' + layout);
        }
      } else {
        if(RACK_DEBUG & 0x20) println("layout: '" + layout + "'");
      }
      var group = new RackGroup(device.get('rack').position, device.get('rack').height, layout, true, members);
      rackgroups[key] = group;
      var members = device.get('members');
      for(var j = 0; j < members.length; j++) {
        if(this.devices.hasOwnProperty(members[j])) {
          rackgroups[members[j]] = group;
          var found = this.devices[members[j]];
          group.members.push(found);
        } else {
          println('No such device ' + members[i] + ' for chassis: ' + device.resolveName());
        }
      }
      if(this.rackobjects.hasOwnProperty(rack)) {
        if(!this.rackobjects[rack].addGroup(group)) {
          this.rackobjects[0].addGroup(group);
        }
      } else {
        println('Chassis, unknown rack: ' + rack);
        this.rackobjects[0].addGroup(group);
      }      
    }
  }
  for(var key in this.devices) {
    if(!rackgroups.hasOwnProperty(key)) {
      var device = this.devices[key];
      var ct = device.get('childType');
      if(ct != 'Chassis') {
        var members = [device];
        var owner = device;
        var index = device.containerIndex();
        var layout = '';
        var cards = false;
        if(ct == 'MICNode') {
          var hostkey = device.get('host');
          if(hostkey) {
            host = this.cluster.find(hostkey);
            if(host) {
              owner = host;
//               layout = '-' + Math.ceil(owner.get('rack').height / 2) + ',2';
              cards = true;
            }
          }
        }
        var ud1 = device.get('userdefined1');
        if((layout.length == 0) && startswith(ud1, 'layout ')) {
          layout = ud1.substr(7) + ':';
          if(RACK_DEBUG & 0x20) println('UserDefined1 layout: ' + ud1 + " '" + layout + "'");
        }
        var rack = owner.getRack();
        var group = new RackGroup(owner.get('rack').position, owner.get('rack').height, layout + index, false, members, cards);
        rackgroups[key] = group;
        if(this.rackobjects.hasOwnProperty(rack)) {
          if(!this.rackobjects[rack].addGroup(group)) {
            this.rackobjects[0].addGroup(group);
          }
        } else {
          println('Chassis, unknown rack: ' + rack);
          this.rackobjects[0].addGroup(group);
        }
      }
    }
  }
  this.unwaitIcon();
}

RackView.prototype.updateRackWithData = function(data) {
  for(var i = 0; i < data.length; i++) {
    for(var j1 = 0; j1 < this.metrics.length; j1++) {
      if((data[i].metricId == this.metrics[j1][0]) && (data[i].metricParam == this.metrics[j1][1])) {        
        var rate = data[i].rate;
        if(typeof(rate) != 'number') continue;          
        if(!this.data.hasOwnProperty(data[i].devId)) this.data[data[i].devId] = new Array();
        this.data[data[i].devId][j1] = rate;
        if(isNaN(this.min[j1])) {
          this.min[j1] = rate;
          this.max[j1] = rate;
        } else if(!isNaN(rate)) {
          this.max[j1] = Math.max(this.max[j1], rate);
          this.min[j1] = Math.min(this.min[j1], rate);
        }
        /* No break, for duplicate entries */
      }
    }
  }
  this.normalizData();
  this.updateValues();
  this.updating = false;
}

RackView.prototype.colorTop = function(id, value) {
  if(value && ((id.indexOf('_down') > 0) || (id.indexOf('_health') > 0))) return '#ce3333';
  return '#41454C';
}

RackView.prototype.listTop = function() {
  var list;
  if(this.viewMode == RACK_VIEWMODE_RACKS) {
    list = this.rackCount;
  } else if(this.viewMode == RACK_VIEWMODE_ROOMS) {
    list = this.roomCount;
  } else if(this.viewMode == RACK_VIEWMODE_CLUSTERS) {
    list = this.clusterCount;
  } else {
    println('UpdateTop called, but viewMode = ' + this.viewMode);
    list = new Object();
  }
  return list;
}

RackView.prototype.updateTop = function() {
  var list = this.listTop();
  for(var id in list) {
    var obj = document.getElementById(id);
    if(obj) {
      var value = list[id];
      var color = this.colorTop(id, value);
      svgreplacetext(obj, value, color);
    } else {
      println('UpdateTop: unable to find: ' + id);
    }
  }
}

RackView.prototype.refreshDrill = function() {
  if(this.viewMode == 0) {
    this.refresh();
  } else {
    this.refreshTop();
  }
}

RackView.prototype.refreshTop = function() {
  println("Refresh rack top " + this.updating + ', ' + this.forceRedraw);
  
  this.clusterCount = new Object();
  this.roomCount = new Object();
  this.rackCount = new Object();
  if(!isEmpty(this.otherClusters)) {
    for(var c in this.otherClusters) {
      var work = this.otherClusters[c];
      work.viewMode = this.viewMode;
      work.refreshTop();
      for(var k in work.rackCount) this.rackCount[k] = work.rackCount[k];
      for(var k in work.roomCount) this.roomCount[k] = work.roomCount[k];
      for(var k in work.clusterCount) this.clusterCount[k] = work.clusterCount[k];
    }
  } else {
    var rackkey = this.cluster.key;
    var chassisRackInfo = new Object();
    for(var key in this.devices) {
      var device = this.devices[key];
      if(dock.baseChildTypeCheck(device.object, 'Chassis')) {
        var rack = device.getRack();
        if(rack) {
          var members = device.get('members');
          for(var i = 0; i < members.length; i++) {
            chassisRackInfo[members[i]] = rack;
          }
        }
      }
    }    
    for(var key in this.devices) {
      var device = this.devices[key];
      var rack;
      if(chassisRackInfo.hasOwnProperty(key)) {
        rack = chassisRackInfo[key];
      } else {
        rack = device.getRack();
      }
      if(this.rackobjects.hasOwnProperty(rack)) {
        var state = device.topViewState();
        var ro = this.rackobjects[rack];
        var room = ro.rack.getRackRoomName();
        if(dock.baseChildTypeCheck(device.object, 'Node')) {
          plusOne(this.clusterCount, rackkey + '_node_' + state);
          plusOne(this.roomCount, rackkey + '_' + room + '_node_' + state);
          plusOne(this.rackCount, rackkey + '_rack_' + rack + '_node_' + state);
        } else {
          plusOne(this.clusterCount, rackkey + '_device_' + state);
          plusOne(this.roomCount, rackkey + '_' + room + '_device_' + state);
          plusOne(this.rackCount, rackkey + '_rack_' + rack + '_device_' + state);        
        }
      } else {
        println('refreshTop: no such rack: ' + rack);
      }
    }
  }
  this.updateTop();
  
  function plusOne(obj, name) {
    if(!obj.hasOwnProperty(name)) obj[name] = 0;
    obj[name] += 1;
  }
}

RackView.prototype.refresh = function() {
  println("Refresh rack " + this.updating + ', ' + this.forceRedraw + ' ' + this.metrics.length + ' ' + this.fillCompleted + ' ' + this.stopped);
  if(!this.racks) {
    this.init();
  } else if(!this.fillCompleted) {
    println('Continue delayedFill');
    this.stopped = false;
    this.delayedFill();
  }
  if(this.updating) return;
  this.updating = true;

  this.max = new Array();
  this.min = new Array();
  this.fmax = new Array();
  this.fmin = new Array();
  if(this.metrics.length > 0) {
    var metrics = new Array();
    for(var i = 0; i < this.metrics.length; i++) {
      var mpi = new dock.cvMetricPrmId();
      mpi.metricId = parseInt(this.metrics[i][0]);
      mpi.metricParam = this.metrics[i][1];
      metrics.push(mpi);
      var metric = this.cluster.find(this.metrics[i][0]);
      if(isObject(metric) && !this.metrics[i][3] && ((metric.get('minimum') != 0) || (metric.get('maximum') != 0))) {
        this.fmax[i] = metric.get('maximum');
        this.fmin[i] = metric.get('minimum');
        this.max[i] = metric.get('maximum');
        this.min[i] = metric.get('minimum');
      } else {
        this.fmax[i] = NaN;
        this.fmin[i] = NaN;
        this.max[i] = NaN;
        this.min[i] = NaN;
      }
    }
    this.data = new Object();
    
    var keys = new Array();
    for(var x in this.devices) keys.push(parseInt(x));

    this.waitIcon();
    if(this.live) {
      dock.workerpool.getWorker(this.cluster.actualurl).call('cmmon', 'sampleOndemandOnDevices', [keys, metrics], function(code, data) {
          if(code == 0) {          
            this.current.updateRackWithData(data);
          } else {
            dock.jsonCallError(this, code, data);
          }
          this.current.unwaitIcon();        
        }, this);        
    } else {
      dock.workerpool.getWorker(this.cluster.actualurl).call('cmmon', 'getLatestPickedRates', [keys, metrics], function(code, data) {
          if(code == 0) {
            this.current.updateRackWithData(data);
          } else {
            dock.jsonCallError(this, code, data);
          }
          this.current.unwaitIcon();        
        }, this);
    }
  } else {
    println("Refresh rack: no metrics");
    this.updateValues();
    this.updating = false;
  }
  this.stoptimer();
  if(this.cluster.connected) this.timer = window.setTimeout(createObjectCallback(this, this.refresh), this.refreshRate * 1000);
}

function RackObject(rack, view) {
  this.rack = rack;
  this.view = view;
  view.racks.push(this);
  this.positions = new Object();
  this.maxPosition = 0;
  if(rack.hasOwnProperty('fake')) {
    this.fake = rack.fake;
  } else {
    this.fake = false;
  }  
  // draw
  this.rbox = 0;
  this.size = 0;
  this.fill = false;
  this.fillFunction = 0;
  this.posx = 0;
  this.posy = 0;
  this.x = 0;
  this.y = 0;
  this.ns = 0;
  this.rb = 0;
  this.ro = 0.9;
  this.roX = 0.35;
  this.sx = 0;
  this.sy = 0;
  this.is = 0;
  this.fs = 0;
  this.fh = 0;  
}

RackObject.prototype.topName = function() {
  var rackkey = this.view.cluster.key;
  if(this.view.viewMode == RACK_VIEWMODE_RACKS) {
    return rackkey + '_rack_' + this.rack.get('uniqueKey') + '_';
  } else if(this.view.viewMode == RACK_VIEWMODE_ROOMS) {
    return rackkey + '_' + this.rack.resolveName() + '_'; // room
  } else if(this.view.viewMode == RACK_VIEWMODE_CLUSTERS) {
    return rackkey + '_'; // cluster
  }
  return '';
}

RackObject.prototype.empty = function() {
  if(!this.fake) return false;
  return (this.maxPosition == 0);
}

RackObject.prototype.splitfake = function(height) {
  var r = new Array();
  if(this.fake && this.maxPosition > 0) {
    if(RACK_DEBUG & 0x01) println("Split fake, N = " + this.maxPosition);
    this.rack.height = height;
    var fr = true;
    var fir = true;
    var i = this.rack.one;
    var ah = 1;
    var cr = this;    
    var rcc = 0;
    var rdf = 1000;
    if(this.maxPosition >= 1000) rdf = 1000000;
    var nr = 0;
    while(i < this.maxPosition) {
      if(!this.positions.hasOwnProperty(i)) {
        println("FAKE NO : " + i);
        i++;
        continue;
      }
      var ch = this.positions[i].height;
      var nh = ah + ch;
      if(nh - 1 > height) {
        if(fir) {
          cr.height = this.maxPosition[i].height;
          for(var j = ah; j < nh; j++) {
            cr.positions[j] + this.positions[i];
            cr.positions[j].rackObject = cr;
            cr.positions[j].rackObjectPos = ah;
            cr.positions[j].pos = ah;
          }
          if(!fr) delete this.positions[i];
          i += ch;
        }
        rcc++;
        cr = new RackObject(new cvFakeRack(this.rack.get('uniqueKey') + rcc / rdf, this.rack.resolveName(), this.rack.width, height, this.rack.get('inverted'), 0, 0, this.rack.room), this.view);
        nr++;
        cr.maxPosition = height;
        fr = false;
        fir = true;
        ah = this.rack.one;
      } else {
        fir = false;
        for(var j = ah; j < nh; j++) {
          cr.positions[j] = this.positions[i];
          cr.positions[j].rackObject = cr;
          cr.positions[j].rackObjectPos = ah;
          cr.positions[j].pos = ah;
        }
        if(!fr) delete this.positions[i];
        i += ch;
        ah = nh;
      }
    }
    if(RACK_DEBUG & 0x01) println("Split fake, new racks: " + nr);
    this.maxPosition = height;
  }
  return r;
}

RackObject.prototype.addGroup = function(group) {
  if(this.fake) {
    if(this.maxPosition < this.rack.one) this.maxPosition = this.rack.one;
    if(RACK_DEBUG & 0x01) println('Add group to ' + this.rack.resolveName() + ' (fake) at ' + this.maxPosition);
    this.positions[this.maxPosition] = group;
    group.rackObject = this;
    group.rackObjectPos = this.maxPosition;
    this.maxPosition += group.height;
    return true;
  } else if(!this.inRange(group.pos, group.height)) {
    println('Failed to add group at (' + group.pos + ',' + group.height + ') not inside the rack: ' + this.rack.resolveName());
    return false;
  } else {
    var found = this.occupied(group.pos, group.height);
    if(found) {
      if(found.chassis) {
        println('Failed to add group at (' + group.pos + ',' + group.height + ') found chassis: ' + found.members[0].get('hostname'));
        return false;
      } else if((found.pos != group.pos) || (found.height != group.height)) {
        println('Failed to add group at (' + group.pos + ',' + group.height + ') found (' + found.pos + ',' + found.height + ')');
        return false;
      } else {
        return found.mergeGroup(group);
      }
    } else {
      var p = 1;
      if(dock.generalSettings.invertedRack && this.rack.get('inverted')) p = -1;
      if(RACK_DEBUG & 0x01) println('Add group at (' + group.pos + ',' + group.height + ') in ' + this.rack.resolveName());
      for(var i = 0; i < group.height; i++) {
        var pos = group.pos + p * i;
        this.positions[group.pos + p * i] = group;
      }
      group.rackObject = this;
      for(var i = 0; i < group.members.length; i++) {
        if(RACK_DEBUG & 0x01) println('  contains: ' + group.members[i].get('hostname'));
      }    
      return true;      
    }
  }
}

RackObject.prototype.inRange = function(pos, height) {
  if((pos <= 0) || (pos > this.rack.get('height'))) return false;
  var p = 1;
  if(dock.generalSettings.invertedRack && this.rack.get('inverted')) p = -1;
  var bottom = pos + p * (height - 1);
  if((bottom <= 0) || (bottom > this.rack.get('height'))) return false;    
  return true;
}

RackObject.prototype.occupied = function(pos, height) {
  var p = 1;
  if(dock.generalSettings.invertedRack && this.rack.get('inverted')) p = -1;
  for(var i = 0; i < height; i++) {
    if(this.positions.hasOwnProperty(pos + p * i)) return this.positions[pos + p * i];
  }
  return false;
}

RackObject.prototype.draw = function(box, x, y, size, fill) {
  this.svgbox = box;
  this.posx = x;
  this.posy = y;
  this.fill = isundefined(fill, true);
  this.size = isundefined(size, 16);

  this.rbox = 0;
  switch(size) {
    case(4):
    case(8): this.draw8(); break
    case(16):
    case(32): this.draw16(); break    
    default: println('Unknown size: ' + size);
  }
}

RackObject.prototype.drawTop = function(box, x, y, size, fill) {
  this.svgbox = box;
  this.posx = x;
  this.posy = y;
  this.fill = isundefined(fill, true);
  this.size = isundefined(size, 16);

  this.rbox = 0;
  switch(size) {
    case(4):
    case(8): this.draw8Top(); break
    case(16):
    case(32): this.draw16Top(); break    
    default: println('Unknown size: ' + size);
  }
}

RackObject.prototype.requiredSpace = function(size) {
  var one = 1; // rack starting at 0 ?
  var height = this.rack.get('height');
  var width = this.rack.get('width');
  
  var w = 0;
  var h = 0;
  switch(size) {
    case(4):
    case(8): w = Math.floor(((5 * size / 4) / 1.75) * width + 0.5) + 2;
             h = (5 * size / 4) * ((height - one + 1) + 1) + 2;
             break;
    case(16):
    case(32): w = Math.floor(((20 * size / 16) / 1.75) * width + 0.5) + 2;
              h = (20 * size / 16) * ((height - one + 1) + 1) + 2;
              break;
  }
  return [w, h];
}

RackObject.prototype.requiredSpaceTop = function(size) {
  var height = this.rack.get('width'); // NO: this.rack.get('height');
  var width = this.rack.get('width'); // this.rack.get('width');
  var ss = 2.75; // 1.75
  
  var w = 0;
  var h = 0;
  switch(size) {
    case(4):
    case(8): w = Math.floor(((5 * size / 4) / ss) * width + 0.5) + 2;
             h = Math.floor(((5 * size / 4) / ss) * height + 0.5) + 2;
             break;
    case(16):
    case(32): w = Math.floor(((20 * size / 16) / ss) * width + 0.5) + 2;
              h = Math.floor(((20 * size / 16) / ss) * height + 0.5) + 2;
              break;
  }
  return [w, h];
}

RackObject.prototype.draw8 = function() {
  var one = 1; // rack starting at 0 ?
  var height = this.rack.get('height');
  var width = this.rack.get('width');

  this.ns = this.size / 4;
  this.rb = 1;
  this.sx = 5 * this.ns / 1.75; 
  this.sy = 5 * this.ns;
  this.is = 4 * this.ns;
  this.fs = 2.5 * this.ns;
  this.fh = 3.25 * this.ns;
  
  this.h = this.sy * (height - one + 1);
  this.w = Math.floor(this.sx * width + 0.5);
  
  this.x = this.posx + this.rb;
  this.y = this.posy + this.rb + this.sy;
  
  this.rbox = document.createElementNS(XUL_SVG, "svg:g");
  svgrect(this.rbox, this.x - this.rb, this.y - this.rb - this.sy, this.w + 2 * this.rb, this.sy, 'url(#top)', this.rb + 1, '#41454C', this.ro);
  svgtext(this.rbox, this.x + this.w / 2, this.y - this.sy + this.fh, this.rack.resolveName(), this.fh, 0, '#eeeeee', 'middle');
  
  svgrect(this.rbox, this.x - this.rb, this.y - this.rb, this.w + 2 * this.rb, this.h + 2 * this.rb, 'none', this.rb + 1, '#41454C', this.ro);
  
  if(this.fill) {
    this.fill8();
  } else {
    this.fillFunction = this.fill8;
  }
  this.svgbox.appendChild(this.rbox);
}

RackObject.prototype.fill8 = function() {
  var one = 1; // rack starting at 0 ?
  var height = this.rack.get('height');
  var width = this.rack.get('width');
  
  var fill = new Object();
  for(var i = one; i <= height; i++) {
    fill[i] = 0;
  }
  fill[one - 1] = -1;
  fill[height + 1] = -1;

  for(var p in this.positions) {
    if(this.positions[p].visibleMembers() > 0) {
      if(this.positions[p].pos == p) {
        fill[p] = 1;
      } else {
        fill[p] = 2;
      }
    }
  }

  var u = height - one + 1;
  var o = one;
  var p = 1;
  if(this.rack.get('inverted')) {
    p = -1;
    o = height;
  }
  // offset (start from 0) ?
 
  for(var i = one; i <= height; i++) {
    svgrect(this.rbox, this.x, this.y + p * (i - o) * this.sy, this.sy, this.sy, '#cdcdcd', 0, '', this.ro);
    svgtext(this.rbox, this.x + this.sy / 2, this.y + p * (i - o) * this.sy + this.fh, i, this.fs, 0, '#333333', 'middle');
  }
  
  var n = 0;
  for(var i = one; i <= height + 1; i++) {
    if(fill[i]) {
      if(n > 0) {
        var d0 = 1;
        var d1 = 1;
        if(p < 0) {
          if(i >= height) {
            if(i - n == one) {
              d0 = 0; d1 = -1;
            } else {
              d0 = 0; d1 = 0;
            }
          } else if(i - n == one) {
            d1 = 0; 
          }
          svgrect(this.rbox, this.x + this.sy + 1, this.y + (height - (i - 1)) * this.sy + d0, this.w - this.sy - 1, this.sy * n - 1 - d1, 'url(#plate)', 0, '', this.ro);
        } else {
          if(i - n == one) {
            if(i >= height) {
              d0 = 0; d1 = -1;
            } else {
              d0 = 0; d1 = 0;
            }
          } else if(i >= height) {
            d1 = 0; 
          }
          svgrect(this.rbox, this.x + this.sy + 1, this.y + (i - n - 1) * this.sy + d0, this.w - this.sy - 1, this.sy * n - 1 - d1, 'url(#plate)', 0, '', this.ro);
        }
        n = 0;
      }
    } else {
      n++;
    }
  }
  
  for(var pp in this.positions) {
    var w = this.positions[pp];
    if(w.pos == pp) {
      var x0 = this.x + this.sy;
      var x1 = x0 + this.w - this.sy;
      var y0;
      var y1;
      if(this.rack.get('inverted')) {
        y0 = this.y + (height - pp) * this.sy;
      } else {
        y0 = this.y + (pp - one) * this.sy;
      }
      if(!dock.generalSettings.invertedRack && this.rack.get('inverted')) {
        y1 = y0 + this.sy;
        y0 = y0 - this.sy * (w.height - 1);
      } else {
        y1 = y0 + this.sy * w.height;
      }
      w.draw8(x0, y0, x1, y1);
    }
  }

  var pp = p;
  var o = 0;
  if((pp < 0) && !dock.generalSettings.invertedRack) {
    o = 1;
    pp = 1;
  }
  var d = (1 - pp) / 2;
  for(var i = one + d; i < height + d; i++) {
    var yy;
    if(p < 0) {
      yy = this.y + (height - i + 1 - o) * this.sy;
    } else {
      yy = this.y + (i - o) * this.sy;
    }
    if((fill[i + pp] == 1) || ((fill[i] > 0) && (fill[i + pp] == 0))) {
      svgline(this.rbox, this.x, yy, this.x + this.w, yy, '#555555', 0.5 * this.ro);
    } else {
      svgline(this.rbox, this.x, yy, this.x + this.sy, yy, '#555555', 0.25 * this.ro);
    }
  }
  svgline(this.rbox, this.x + this.sy, this.y, this.x + this.sy, this.y + u * this.sy, '#555555', 0.75 * this.ro);
}

// always call draw above
RackObject.prototype.draw16 = function() {
  var one = 1; // rack starting at 0 ?
  var height = this.rack.get('height');
  var width = this.rack.get('width');
  
  this.ns = this.size / 16;
  this.rb = 1;
  this.sx = 20 * this.ns / 1.75; 
  this.sy = 20 * this.ns;
  this.is = 16 * this.ns;
  this.fs = 10 * this.ns;
  this.fh = 13 * this.ns;
  
  this.h = this.sy * (height - one + 1);
  this.w = Math.floor(this.sx * width + 0.5);
  
  this.x = this.posx + this.rb;
  this.y = this.posy + this.rb + this.sy;
  
  this.rbox = document.createElementNS(XUL_SVG, "svg:g");
  svgrect(this.rbox, this.x - this.rb, this.y - this.rb - this.sy, this.w + 2 * this.rb, this.sy, 'url(#top)', this.rb + 1, '#41454C', this.ro);
  svgtext(this.rbox, this.x + this.w / 2, this.y - this.sy + this.fh, this.rack.resolveName(), this.fh, 0, '#eeeeee', 'middle');
  
  svgrect(this.rbox, this.x - this.rb, this.y - this.rb, this.w + 2 * this.rb, this.h + 2 * this.rb, 'none', this.rb + 1, '#41454C', this.ro);
  
  if(this.fill) {
    this.fill16();
  } else {
    this.fillFunction = this.fill16;
  }
  this.svgbox.appendChild(this.rbox);
}

RackObject.prototype.fill16 = function() {
  var one = 1; // rack starting at 0 ?
  var height = this.rack.get('height');
  var width = this.rack.get('width');
  
  var fill = new Object();
  for(var i = one; i <= height; i++) {
    fill[i] = 0;
  }
  fill[one - 1] = -1;
  fill[height + 1] = -1;

  for(var p in this.positions) {
    if(this.positions[p].visibleMembers() > 0) {
      if(this.positions[p].pos == p) {
        fill[p] = 1;
      } else {
        fill[p] = 2;
      }
    }
  }

  var u = height - one + 1;
  var o = one;
  var p = 1;
  if(this.rack.get('inverted')) {
    p = -1;
    o = height;
  }
  // offset (start from 0) ?
 
  for(var i = one; i <= height; i++) {
    svgrect(this.rbox, this.x, this.y + p * (i - o) * this.sy, this.sy, this.sy, '#cdcdcd', 0, '', this.ro);
    svgtext(this.rbox, this.x + this.sy / 2, this.y + p * (i - o) * this.sy + this.fh, i, this.fs, 0, '#333333', 'middle');
  }
    
  var n = 0;
  for(var i = one; i <= height + 1; i++) {
    if(fill[i]) {
      if(n > 0) {
        var d0 = 1;
        var d1 = 1;
        if(p < 0) {
          if(i >= height) {
            if(i - n == one) {
              d0 = 0; d1 = -1;
            } else {
              d0 = 0; d1 = 0;
            }
          } else if(i - n == one) {
            d1 = 0; 
          }
          svgrect(this.rbox, this.x + this.sy + 1, this.y + (height - (i - 1)) * this.sy + d0, this.w - this.sy - 1, this.sy * n - 1 - d1, 'url(#plate)', 0, '', this.ro);
        } else {
          if(i - n == one) {
            if(i >= height) {
              d0 = 0; d1 = -1;
            } else {
              d0 = 0; d1 = 0;
            }
          } else if(i >= height) {
            d1 = 0; 
          }
          svgrect(this.rbox, this.x + this.sy + 1, this.y + (i - n - 1) * this.sy + d0, this.w - this.sy - 1, this.sy * n - 1 - d1, 'url(#plate)', 0, '', this.ro);
        }
        n = 0;
      }
    } else {
      n++;
    }
  }
  
  for(var pp in this.positions) {
    var w = this.positions[pp];
    if(w.pos == pp) {
      var x0 = this.x + this.sy;
      var x1 = x0 + this.w - this.sy;
      var y0;
      var y1;
      if(this.rack.get('inverted')) {
        y0 = this.y + (height - pp) * this.sy;
      } else {
        y0 = this.y + (pp - one) * this.sy;
      }
      if(!dock.generalSettings.invertedRack && this.rack.get('inverted')) {
        y1 = y0 + this.sy;
        y0 = y0 - this.sy * (w.height - 1);
      } else {
        y1 = y0 + this.sy * w.height;
      }
      w.draw16(x0, y0, x1, y1);
    }
  }

  var pp = p;
  var o = 0;
  if((pp < 0) && !dock.generalSettings.invertedRack) {
    o = 1;
    pp = 1;
  }
  var d = (1 - pp) / 2;
  for(var i = one + d; i < height + d; i++) {
    var yy;
    if(p < 0) {
      yy = this.y + (height - i + 1 - o) * this.sy;
    } else {
      yy = this.y + (i - o) * this.sy;
    }
    if((fill[i + pp] == 1) || ((fill[i] > 0) && (fill[i + pp] == 0))) {
      svgline(this.rbox, this.x, yy, this.x + this.w, yy, '#555555', 0.5 * this.ro);
    } else {
      svgline(this.rbox, this.x, yy, this.x + this.sy, yy, '#555555', 0.25 * this.ro);
    }
  }
  svgline(this.rbox, this.x + this.sy, this.y, this.x + this.sy, this.y + u * this.sy, '#555555', 0.75 * this.ro);
}

RackObject.prototype.draw16Top = function() {
  var height = this.rack.get('width'); // this.rack.get('height');
  var width = this.rack.get('width'); // this.rack.get('width');
  
  this.ns = this.size / 16;
  this.sy = 20 * this.ns;
  this.is = this.size;
  this.fs = 10 * this.ns;
  this.fh = 13 * this.ns;
  this.ox = this.is / 4;
  this.oy = this.is / 4;

  [this.w, this.h] = this.requiredSpaceTop(this.size);
  this.x = this.posx;
  this.y = this.posy;

//   this.sx = (this.w - this.ox) / 3;
  this.sx = (this.w - this.ox) / 2;
  
  this.rbox = document.createElementNS(XUL_SVG, "svg:g");
  this.rbox.setAttribute('onclick', 'mouseclicktop(evt);');
  this.rbox.setAttribute('cmtop', this.topName());
  svgrect(this.rbox, this.x, this.y, this.w, this.sy, 'url(#top)', this.rb + 1, '#41454C', this.ro);
  svgtext(this.rbox, this.x + this.w / 2, this.y + this.fh, this.rack.resolveName(), this.fh, 0, '#eeeeee', 'middle');
  
  svgrect(this.rbox, this.x, this.y + this.sy, this.w, this.h - this.sy, '#cdcdcd', this.rb + 1, '#41454C', this.ro);
  
  if(this.fill) {
    this.fill16Top();
  } else {
    this.fillFunction = this.fill16Top;
  }
  this.svgbox.appendChild(this.rbox);
}

RackObject.prototype.fill16TopIconLabel = function(x, y, icon, id, list) {
  svguse(this.rbox, x, y, this.is, this.is, icon);
  var text = 0;
  if(list.hasOwnProperty(id)) text = list[id];
  var color = this.view.colorTop(id, text);
  var obj = svgtext(this.rbox, x + this.is + this.ox, y + this.fh, text, this.fh, 0, color);
  obj.setAttribute('id', id);
}

RackObject.prototype.fill16Top = function() {
  svgtext(this.rbox, this.x + this.ox, this.y + this.sy + this.fh + this.ox, 'Nodes', this.fh, 0, '#41454C');
  svgline(this.rbox, this.x + this.ox, this.y + 2 * this.sy, this.x + this.w - this.ox, this.y + 2 * this.sy, '#333333', 0.5);

  var list = this.view.listTop();
  var name = this.topName();
  this.fill16TopIconLabel(this.x + this.ox + 0 * this.sx, this.y + 2 * this.sy + this.oy, '#image_Node_Up_' + this.is, name + 'node_up', 3, list);
  this.fill16TopIconLabel(this.x + this.ox + 1 * this.sx, this.y + 2 * this.sy + this.oy, '#image_Node_Up_Health_' + this.is, name + 'node_up_health', list);

  this.fill16TopIconLabel(this.x + this.ox + 0 * this.sx, this.y + 3 * this.sy + this.oy / 2, '#image_Node_Down_' + this.is, name + 'node_down', list);
  this.fill16TopIconLabel(this.x + this.ox + 1 * this.sx, this.y + 3 * this.sy + this.oy / 2, '#image_Node_Closed_' + this.is, name + 'node_closed', list);
  
  svgtext(this.rbox, this.x + this.ox, this.y + 4 * this.sy + this.fh, 'Devices', this.fh, 0, '#41454C');
  svgline(this.rbox, this.x + this.ox, this.y + 5 * this.sy - this.oy, this.x + this.w - this.ox, this.y + 5 * this.sy - this.oy, '#333333', 0.5);
  
  this.fill16TopIconLabel(this.x + this.ox + 0 * this.sx, this.y + 5 * this.sy, '#image_GenericDevice_Up_' + this.is, name + 'device_up', 3, list);
  this.fill16TopIconLabel(this.x + this.ox + 1 * this.sx, this.y + 5 * this.sy, '#image_GenericDevice_Up_Health_' + this.is, name + 'device_up_health', list);

  this.fill16TopIconLabel(this.x + this.ox + 0 * this.sx, this.y + 6 * this.sy - this.oy / 2, '#image_GenericDevice_Down_' + this.is, name + 'device_down', list);
  this.fill16TopIconLabel(this.x + this.ox + 1 * this.sx, this.y + 6 * this.sy - this.oy / 2, '#image_GenericDevice_Closed_' + this.is, name + 'device_closed', list);
}

function RackGroup(pos, height, layout, chassis, members, cards) {
  this.rackObject = 0;
  this.rackObjectPos = 0;
  this.pos = pos;
  this.height = Math.max(1, height);
  this.layout = '';
  if((layout.substr(0, 1) == '|') || (layout.substr(0, 1) == '-')) {
    var semi = layout.indexOf(':');
    if(semi >= 0) {
      this.layout = layout.substr(0, semi);
    } else {
      this.layout = layout;
    }
    if(RACK_DEBUG & 0x20) println('Set layout: ' + layout);
  }
  this.layoutOrientation = 0; // 0 = horizontal
  this.layoutW = 0;
  this.layoutH = 0;
  this.chassis = chassis;
  this.members = isundefined(members, new Array());
  this.cards = isundefined(cards, false);
  this.visible = -1;
  this.layoutIndex = 0;
  if(chassis) {
    this.icon = 'Chassis';
  } else if(members.length) {
    var work = members[0].object;
    if(dock.baseChildTypeCheck(work, 'Node')) {
      this.icon = 'Node';
    } else if(dock.baseChildTypeCheck(work, 'Switch')) {
      this.icon = 'Switch';
    } else if(dock.baseChildTypeCheck(work, 'PowerDistributionUnit')) {
      this.icon = 'PowerDistributionUnit';
    } else {
      this.icon = 'GenericDevice';
    }
  }
}

RackGroup.prototype.mergeGroup = function(group) {
  if((this.layout.length == 0) || ((this.layout.substr(0, 1) != '|') && (this.layout.substr(0, 1) != '-'))) {
    if((group.layout.length > 0) && ((group.layout.substr(0, 1) == '|') || (group.layout.substr(0, 1) == '-'))) {
      var semi = group.layout.indexOf(':');
      if(semi >= 0) {
        this.layout = group.layout.substr(0, semi);
      } else {
        this.layout = group.layout;
      }
    }    
  }
  for(var i = 0; i < group.members.length; i++) {
    if(RACK_DEBUG & 0x01) println('Merging group at (' + group.pos + ',' + group.height + ') add: ' + group.members[i].get('hostname'));        
    this.members.push(group.members[i]);
  }
  this.cards = this.cards || group.cards;
  this.visible = -1;
  return true;
}

RackGroup.prototype.layoutSize = function() {
  var extra = 0;
  if(this.layoutOrientation < 0) {
    this.layoutOrientation = 0;
    var n = this.visibleMembers();
    if(this.cards && this.height > 1) { // top row for host, bottom n - 1 for (MIC) cards
      this.layoutW = 1;
      this.layoutH = this.height;
      extra = (this.layoutH - 1) * 7; // 8 cards per row?
    } else if(n <= this.height) {
      this.layoutW = 1;
      this.layoutH = this.height; // n ?
    } else {
      this.layoutW = Math.ceil(n / this.height);
      this.layoutH = this.height;
    }
    if(RACK_DEBUG & 0x04) println('Layout of ' + n + ' = (' + this.layoutW + ' * ' + this.layoutH + ')');
  }
  return this.layoutW * this.layoutH + extra;
}

RackGroup.prototype.visibleMembers = function() {
  if(this.visible < 0) {
    this.visible = 0;
    var n = this.layoutSize();
    for(var i = 0; i < this.members.length; i++) {
      var index = this.members[i].containerIndex();
      if(index <= n) this.visible++;
    }
  }
  return this.visible;
}

RackGroup.prototype.determineLayout = function() {
  this.layoutOrientation = -1;
  if(this.layout && this.layout.length > 0) {
    var wh = this.layout.substr(1).split(',');
    if((wh.length == 2) && isNumeric(wh[0]) && isNumeric(wh[1])) {
      if(this.layout.substr(0, 1) == '|') {
        this.layoutOrientation = 1;
      } else if(this.layout.substr(0, 1) == '-') {
        this.layoutOrientation = 0;
      }
      this.layoutW = parseInt(wh[0]);
      this.layoutH = parseInt(wh[1]);
      if(RACK_DEBUG & 0x20) println('Forced layout: ' + this.layoutOrientation + ', ' + this.layoutW + ', ' + this.layoutH);
    }
  } 
  var n = this.layoutSize();
  this.layoutIndex = new Array();
  for(var i = 0; i <= n; i++) this.layoutIndex.push(-1);  
  var free = new Array();
  for(var i = 0; i < this.members.length; i++) {
    var index = this.members[i].containerIndex();
    if(index > 0 && index <= n) {
      if(this.layoutIndex[index] < 0) {
        this.layoutIndex[index] = i;
        continue;
      }
    } else if(index == 0) {
      free.push([this.members[i].get('hostname'), i]);
    }
  }
  free = free.sort();
  var j = 1;
  for(var i = 0; i < free.length; i++) {
    while((j < this.layoutIndex.length) && (this.layoutIndex[j] >= 0)) {
      j++;
    }
    if(j >= this.layoutIndex.length) break;
    this.layoutIndex[j] = free[i][1];
  }
  if(RACK_DEBUG & 0x04) {
    println('-- Layout order --');
    for(var i = 1; i < this.layoutIndex.length; i++) {
      print(i + ': ');
      if(this.layoutIndex[i] < 0) {
        println('none');
      } else {
        var device = this.members[this.layoutIndex[i]];
        println(device.get('hostname') + ', pos: ' + device.containerIndex() + ', index: ' + device.get('indexInsideContainer'));
      }
    }
  }
}

RackGroup.prototype.icon = function() {
  return this.icon + '_' + this.rackObject.is + '.png';
}

RackGroup.prototype.iconref = function() {
  return '#image_' + this.icon + '_' + this.rackObject.is;
}

RackGroup.prototype.draw8 = function(x0, y0, x1, y1) {
  var r = this.rackObject;
  var gbox = document.createElementNS(XUL_SVG, "svg:g");
  
  this.determineLayout();

  var use = new Object();
  for(var i = 1; i < this.layoutIndex.length; i++) {
    if(this.layoutIndex[i] >= 0) {
      var ix = (i - 1) % this.layoutW;
      var iy = Math.floor((i - 1) / this.layoutW);
      use[ix + ',' + iy] = 1;
    }
  }
//   for(var i = 1; i < this.layoutW; i++) {
//     var j0 = 0;
//     for(var j = 0; j < this.layoutH; j++) {
//       if(!use.hasOwnProperty(i + ',' + j)) {
//         if(j0 < j) {
//           var a0 = Math.floor((x0 * (this.layoutW - i) + x1 * i) / this.layoutW);
//           var b0 = Math.floor((y0 * (this.layoutH - j0) + y1 * j0) / this.layoutH) + 1;
//           var b1 = Math.floor((y0 * (this.layoutH - j) + y1 * j) / this.layoutH) - 2;
//           svgline(gbox, a0, b0, a0, b1, '#555555', 0.25 * r.ro);
//         }
//         j0 = j + 1;
//       }
//     }
//   }
//   for(var j = 1; j < this.layoutH; j++) {
//     var i0 = 0;
//     for(var i = 0; i <= this.layoutW; i++) {
//       if(!use.hasOwnProperty(i + ',' + j)) {
//         if(i0 < i) {
//           var a0 = Math.floor((x0 * (this.layoutW - i0) + x1 * i0) / this.layoutW);
//           var a1 = Math.floor((x0 * (this.layoutW - i) + x1 * i) / this.layoutW);
//           var b0 = Math.floor((y0 * (this.layoutH - j) + y1 * j) / this.layoutH);
//           svgline(gbox, a0, b0, a1, b0, '#555555', 0.25 * r.ro);
//         }
//         i0 = i + 1;
//       }
//     }
//   }

  var rx = 0.5;
  var ry = 0;
  var view = this.rackObject.view;
  for(var i = 1; i < this.layoutIndex.length; i++) {
    if(this.layoutIndex[i] >= 0) {
      var work = this.members[this.layoutIndex[i]];
      var key = work.get('uniqueKey');
      var ix = (i - 1) % this.layoutW;
      var iy = Math.floor((i - 1) / this.layoutW);
      
      var a0 = Math.floor((x0 * (this.layoutW - ix) + x1 * ix) / this.layoutW) + rx;
      var a1 = Math.floor((x0 * (this.layoutW - ix - 1) + x1 * (ix + 1)) / this.layoutW) - 2 * rx;
      var b0 = Math.floor((y0 * (this.layoutH - iy) + y1 * iy) / this.layoutH) + ry;
      var b1 = Math.floor((y0 * (this.layoutH - iy - 1) + y1 * (iy + 1)) / this.layoutH) - 2 * ry;

//       svgrect(this.rackObject.rbox, a0, b0, (a1 - a0), (b1 - b0), '#00ff00', 0, '', 0.5);
      var ss = view.isSelected(key)
      
      var nbox = document.createElementNS(XUL_SVG, "svg:g");
      nbox.setAttribute('onmouseover', 'mouseOver(evt);');
      nbox.setAttribute('onmouseout', 'mouseOut(evt);');
      nbox.setAttribute('onclick', 'mouseclick(evt);');

      nbox.setAttribute('cmnode', key);
      
      view.backbox[key] = svgrect(nbox, a0 + 1, b0 + 1, (a1 - a0) - 1, (b1 - b0) - 1, view.backcolor(ss), 0, '', r.ro * 0.5);
//       if((rx > 0) && (ry > 0)) svgrect(nbox, a0, b0, (a1 - a0), (b1 - b0), '#cdcdcd', 0.5, '#ffffff', r.ro);
//       svgtext(nbox, a0 + 3 * r.ns, b0 + r.fh, work.get('hostname'), r.fs, 0, '#333333');

      var cbox = document.createElementNS(XUL_SVG, "svg:g");
      var ha = (a1 - a0); // - rx * 2;
      var hb = (b1 - b0) - ry * 2;
      cbox.setAttribute('transform', 'translate(' + (a0 + rx) + ',' + (b0 + ry) + ') scale(' + ha + ',' + hb + ')');
//       svgline(nbox, a0 + rx, b0 + ry, a0 + 2 * rx + ha, b0 + 2 * ry + hb, '#ffffff', r.ro * 0.25);
      if(view.normdata.hasOwnProperty(key) && (ss != 0)) view.fillColors(cbox, view.normdata[key]);
      view.databox[key] = cbox;
      nbox.appendChild(cbox);
      gbox.appendChild(nbox);
    }
  }  
  r.rbox.appendChild(gbox);
}

RackGroup.prototype.draw16 = function(x0, y0, x1, y1) {
  var r = this.rackObject;
  var view = r.view;
  var gbox = document.createElementNS(XUL_SVG, "svg:g");
  
  svgrect(gbox, x0, y0, (x1 - x0), (y1 - y0), '#d7d7d7', 0, '', r.ro);
  svguse(gbox, x0 + 1 + r.ns * 2, y0 + r.ns * 3 - 1, r.is, r.is, this.iconref());

  this.determineLayout();
  x0 += r.sy;

  var use = new Object();
  for(var i = 1; i < this.layoutIndex.length; i++) {
    if(this.layoutIndex[i] >= 0) {
      var ix = (i - 1) % this.layoutW;
      var iy = Math.floor((i - 1) / this.layoutW);
      use[ix + ',' + iy] = 1;
    }
  }
  for(var i = 1; i < this.layoutW; i++) {
    var j0 = 0;
    for(var j = 0; j < this.layoutH; j++) {
      if(!use.hasOwnProperty(i + ',' + j)) {
        if(j0 < j) {
          var a0 = Math.floor((x0 * (this.layoutW - i) + x1 * i) / this.layoutW);
          var b0 = Math.floor((y0 * (this.layoutH - j0) + y1 * j0) / this.layoutH) + 1;
          var b1 = Math.floor((y0 * (this.layoutH - j) + y1 * j) / this.layoutH) - 2;
          svgline(gbox, a0, b0, a0, b1, '#555555', 0.25 * r.ro);
        }
        j0 = j + 1;
      }
    }
  }
  for(var j = 1; j < this.layoutH; j++) {
    var i0 = 0;
    for(var i = 0; i <= this.layoutW; i++) {
      if(!use.hasOwnProperty(i + ',' + j)) {
        if(i0 < i) {
          var a0 = Math.floor((x0 * (this.layoutW - i0) + x1 * i0) / this.layoutW);
          var a1 = Math.floor((x0 * (this.layoutW - i) + x1 * i) / this.layoutW);
          var b0 = Math.floor((y0 * (this.layoutH - j) + y1 * j) / this.layoutH);
          svgline(gbox, a0, b0, a1, b0, '#555555', 0.25 * r.ro);
        }
        i0 = i + 1;
      }
    }
  }
  
  for(var i = 1; i < this.layoutIndex.length; i++) {
    if(this.layoutIndex[i] >= 0) {
      var work = this.members[this.layoutIndex[i]];
      var key = work.get('uniqueKey');
      var ix;
      var iy;
      var lW = this.layoutW;
      var lH = this.layoutH;
      var prefix = false;
      if(this.cards && (this.height > 1)) {
        if(i == 1) { // host
          ix = 0;
          iy = 0;
        } else {
          var n = this.visibleMembers() - 1;
          lW = Math.ceil(n / (this.height - 1));
          ix = (i - 2) % lW;
          iy = 1 + Math.floor((i - 2) / lW);
          prefix = (n / (this.height - 1)) <= 4;
        }
      } else {
        ix = (i - 1) % this.layoutW;
        iy = Math.floor((i - 1) / this.layoutW);
      }
      
      var a0 = Math.floor((x0 * (lW - ix) + x1 * ix) / lW) + 1;
      var a1 = Math.floor((x0 * (lW - ix - 1) + x1 * (ix + 1)) / lW) - 2;
      var b0 = Math.floor((y0 * (lH - iy) + y1 * iy) / lH) + 1;
      var b1 = Math.floor((y0 * (lH - iy - 1) + y1 * (iy + 1)) / lH) - 2;

//       svgrect(this.rackObject.rbox, a0, b0, (a1 - a0), (b1 - b0), '#00ff00', 0, '', 0.5);
      var ss = view.isSelected(key)
      
      var nbox = document.createElementNS(XUL_SVG, "svg:g");
      nbox.setAttribute('onmouseover', 'mouseOver(evt);');
      nbox.setAttribute('onmouseout', 'mouseOut(evt);');
      nbox.setAttribute('onclick', 'mouseclick(evt);');
      
      nbox.setAttribute('cmnode', key);
      
      var cbox = document.createElementNS(XUL_SVG, "svg:g");      
      view.backbox[key] = svgrect(nbox, a0, b0, (a1 - a0), (b1 - b0), view.backcolor(ss), 0.5, '#ffffff', r.ro);
      
      var hostname = work.shortHostname(prefix);
      if(this.layoutOrientation == 0) {
        svgtext(nbox, a0 + 3 * r.ns, b0 + r.fh, hostname, r.fs, 0, '#333333');
        
        var hb = (b1 - b0) - 4;
        var ha = Math.min(hb, r.sy);
        var ll = a1 - ha - 3;        
        cbox.setAttribute('transform', 'translate(' + (ll + 1) + ',' + (b0 + 2) + ') scale(' + ha + ',' + hb + ')');
        svgline(nbox, ll, b0 + 2, ll, b0 + 2 + hb, '#ffffff', r.ro * 0.25);
        if(view.normdata.hasOwnProperty(key) && (ss != 0)) view.fillColors(cbox, view.normdata[key]);
      } else {
        var a01 = (a1 - a0) / 2;
        if(a01 <= r.fs) {
          svgtext(nbox, a0 + 3 * r.ns, b0 + r.fh, hostname, r.fs, '90 ' + (a0 + a1) / 2 + ' ' + (b0 + a01), '#333333');
        } else {
          var os = (a01 - r.fs) / 2;
          svgtext(nbox, a0 + 3 * r.ns, b0 + r.fh, hostname, r.fs, '90 ' + ((a0 + a1) / 2 - os) + ' ' + (b0 + a01 - os), '#333333');
        }
        
        var hb = (a1 - a0) - 4;
        var ha = Math.min(hb, r.sy);
        var ll = b1 - ha - 3;
        cbox.setAttribute('transform', 'translate(' + (a0 + 2) + ',' + (ll + 1) + ') scale(' + hb + ',' + ha + ')');
        svgline(nbox, a0 + 2, ll, a0 + 2 + hb, ll, '#ffffff', r.ro * 0.25);
        if(view.normdata.hasOwnProperty(key) && (ss != 0)) view.fillColors(cbox, view.normdata[key]);
      }
      view.databox[key] = cbox;
      nbox.appendChild(cbox);
      gbox.appendChild(nbox);
    }
  }  
  r.rbox.appendChild(gbox);
}

var tooltip = 0;
var tooltiptimer = 0;
var tooltipcurrent = 0;
var tooltipevent = 0;

function mouseOver(event) {
  var current = parseInt(event.currentTarget.getAttribute('cmnode'));
  tooltipevent = event;
  if(tooltipcurrent != current) {
    tooltipcurrent = current;
    if(!tooltiptimer) {
      tooltiptimer = window.setTimeout('displayTooltip();', 500);
    }
  }
}

function mouseOut(event) {
  tooltipcurrent = 0;
  if(tooltiptimer) window.clearTimeout(tooltiptimer);
  tooltiptimer = 0;
  tooltipevent = 0;
  if(tooltip) {
    var box = document.getElementById("box");
    box.removeChild(tooltip);
    tooltip = 0;
  }
}

function svgtooltip(key, x, y, sx, sy) {
  var box = document.getElementById("box");
  var svg = box.parentNode.parentNode;
  var w = svg.clientWidth;
  var h = svg.clientHeight;

  var work;
  if(typeof(rack) != 'undefined') {
    work = rack;
  } else {
    work = topview;
  }
  var metrics = work.metricInfo();
  var data = work.deviceValues(key);
  var device = work.cluster.find(key);
  if(!device) {
    println("Tooltip, device no longer defined: " + key);
    return;
  }
  var hostname = device.resolveName();

  var bo = 8;
  var is = 16;
  var io = 4;
  var to = 4;
  var ttfs = 14;
  var ttw = 120;
  var tth = Math.max((metrics.length + 1) * ttfs + 2 * bo + 2, 64);
  
  if(tooltip) {
    var box = document.getElementById("box");
    box.removeChild(tooltip);
    tooltip = 0;
  }
  
  tooltip = document.createElementNS(XUL_SVG, "svg:g");
  
  var rect = document.createElementNS(XUL_SVG, "svg:rect");
  rect.setAttribute("fill", '#ffffcc');
  rect.setAttribute("style", "stroke:#333333; stroke-width:1; fill-opacity:0.7");
  rect.setAttribute("x", 0);
  rect.setAttribute("y", 0);
  rect.setAttribute("height", tth);
  rect.setAttribute("rx", 10);
  rect.setAttribute("ry", 10);
  tooltip.appendChild(rect);
  box.appendChild(tooltip);  

  var txt = svgtext(tooltip, bo + is + io + to, ttfs + bo - 4, hostname, ttfs, 0, '#333333');
  ttw = Math.max(ttw, txt.getComputedTextLength());
  
  for(var l = 0; l < metrics.length; l++) {
    var nodata = (l >= data.length) || (typeof(data[l]) == "undefined") || kisNaN(data[l]);
    var lx = bo + is + io + to;
    var ly = bo + (l + 2) * ttfs + io;
    var txt = '';
    if(nodata) {
      txt = 'no data';
    } else if((metrics[l].length > 2) && metrics[l][2]) {
      if(data[l] < 0.5) {
        txt = 'PASS';
      } else if(data[l] > 1.5) {
        txt = 'FAIL';
      } else {
        txt = 'UNKNOWN';
      }
    } else {
      txt = formatKMG(data[l]) + metrics[l][1];
    }
    var txt = svgtext(tooltip, lx, ly, metrics[l][0] + ': ' + txt, ttfs, 0, '#333333');
    ttw = Math.max(ttw, txt.getComputedTextLength());
  }
  
  ttw += 2 * bo + is + io;
  var im = device.treeitem.getAttribute("src").replace('chrome://cmgui/content/', '');
  svgimage(tooltip, 8, 8, is, is, im);
  svgline(tooltip, bo + is + io, 0, bo + is + io, tth, '#333333');
  svgline(tooltip, bo + is + io, ttfs + bo + 2, ttw, ttfs + bo + 2, '#333333');
  rect.setAttribute("width", ttw);
  
  var ttx = x;
  var tty = y;
  if(sx + ttw > w) ttx = ttx - ttw;
  if(sy + tth > h) tty = tty - tth;
  tooltip.setAttribute('transform', 'translate(' + ttx + ',' + tty + ')');
}

function displayTooltip() {
  if(tooltiptimer) window.clearTimeout(tooltiptimer);
  svgtooltip(tooltipcurrent, tooltipevent.layerX, tooltipevent.layerY, tooltipevent.clientX, tooltipevent.clientY);
  tooltiptimer = 0;
}

function mouseclick(event) {
  if(topview) {
    opendevice(topview.cluster, event.currentTarget.getAttribute('cmnode'));
  } else {
    opendevice(event.currentTarget.getAttribute('cmnode'));
  }
}

function mouseclicktop(event) {
//   if(event.detail == 1) {
    var what = event.currentTarget.getAttribute('cmtop');
    if(topview.viewMode == RACK_VIEWMODE_CLUSTERS) {
      what = what.replace(/_$/, '');
      if(topview.otherClusters.hasOwnProperty(what)) {        
        topview.other = topview.otherClusters[what];
        topview.cluster = topview.otherClusters[what].cluster;
        topview.metrics = topview.cluster.rackMetrics;
        topview.viewMode = RACK_VIEWMODE_ROOMS;
        topview.forceRedraw = true;
        topview.showRoom = 0;
        topview.showRack = 0;
        topview.getCustomBounds();
        println('Set top ' + topview.cluster + ' ' + topview.viewMode);
        refresh();
      } else {
        dock.displayInfo('Unable to find cluster: ' + what, window);
      }
    } else if(topview.viewMode == RACK_VIEWMODE_ROOMS) {
      var idx = what.indexOf('_');
      if(idx > 0) what = what.substr(idx + 1);
      what = what.replace(/_$/, '');
      topview.showRoom = [what];
      topview.showRack = 0;
      topview.viewMode = RACK_VIEWMODE_RACKS;
      topview.forceRedraw = true;
      println('Set top ' + topview.cluster + ' ' + topview.viewMode);
      refresh();      
    } else if(topview.viewMode == RACK_VIEWMODE_RACKS) {
      var idx = what.indexOf('_rack_');
      if(idx > 0) what = what.substr(idx + 6);
      what = what.replace(/_$/, '');
      topview.showRack = [parseInt(what)];
      topview.viewMode = 0;
      topview.forceRedraw = true;
      println('Set top ' + topview.cluster + ' ' + topview.viewMode);
      refresh();            
    } else {
      println('mouseclicktop, not in top mode: ' + topview.viewMode);
    }
    topview.lowlevel();
//   }
}
