/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var KEY_ENTER = 13;
var KEY_SPACE = 32;
var KEY_ESCAPE = 27;
var BLANK_CATEGORY = '*$!*';

function viewValueToObjectValue(func, value) {
  if(func!==null && func!='') {
    value = window[func](value, false);
  }
  return value;
}

function objectValueToViewValue(func, value) {
  if(func!==null && func!='') {
    value = window[func](value, true);
  }
  return value;
}

function role_text(what, role, category, format, subscript, nodups, dupnull, newKey) {
  format = isundefined(format, false);
  subscript = isundefined(subscript, '');
  nodups = isundefined(nodups, false);
  dupnull = isundefined(dupnull, '');
  newKey = isundefined(newKey, 0);
  var i = what.indexOf('_');
  var index = -1;
  var name;
  if(i > 0) {
    index = parseInt(what.substr(i + 1));
    name = what.substr(0, i);
  } else {
    name = what;
  }
  var fname = role.get('childType') + '_' + what + subscript;
  var obj = document.getElementById(fname);
  if(!obj) {
    println('+++ unable to find: ' + fname);
    return;
  }
  var value = role.get(name);
  if(isArray(value)) value = value[index];
  if(subscript) value = value[subscript];
  var valueViewMapper = obj.getAttribute('valueViewMapper');
  value = objectValueToViewValue(valueViewMapper, value)
  obj.value = value;
  obj.setAttribute('what', what);
  obj.setAttribute('role', role.get('childType'));
  obj.setAttribute('subscript', subscript);
  obj.setAttribute('nodups', nodups);
  obj.setAttribute('dupnull', dupnull);
  obj.setAttribute('newKey', newKey);
  obj.setAttribute('value', value);
  if(format) {
    obj.setAttribute('type', 'timed');
    obj.setAttribute('format', format);
    obj.setAttribute('timeout', '100');
    obj.addEventListener("command", role_textbox_command, true);
  }
  if(category) {
    obj.setAttribute('category', value);
    obj.setAttribute('disabled', true);
  } else {
    obj.setAttribute('category', BLANK_CATEGORY);
    obj.removeAttribute('disabled');
  }
  obj.addEventListener("change", role_textbox_change, true);
  obj.addEventListener("keypress", role_textbox_keypress, true);
}

function role_textbox_keypress(event) {
  var text = event.target;
  var value = text.value;
  var what = text.getAttribute('what');
  var category = text.getAttribute('category');

  var role = text.getAttribute('role');
  var roles = current.get('roles');
  var one = 0;
  for(var i = 0; i < roles.length; i++) {
    if(roles[i].childType == role) {
      one = new dock.cvObject(current.cluster, roles[i]);
      break;
    }
  }
  if(one) {
    var valueViewMapper = text.getAttribute('valueViewMapper');
    value = viewValueToObjectValue(valueViewMapper, value);
    var format = text.getAttribute('format');
    var subscript = text.getAttribute('subscript');
    var nodups = eval(text.getAttribute('nodups'));
    var dupnull = text.getAttribute('dupnull');
    var newKey = text.getAttribute('newKey');
    var i = what.indexOf('_');
    switch(event.keyCode) {
      case KEY_ENTER:
        if(text.getAttribute('class') == "textarea") break;
        if(regexMatch(value, format)) {
          if(i > 0) {
            var index = parseInt(what.substr(i + 1));
            var name = what.substr(0, i);
            var list = one.get(name);
            if(index > list.length) {
              println("Attemp to write past the last element of: " + name);
              println("  length = " + list.length + ", index = " + index);
            } else {
              if(subscript) {
                list[index][subscript] = value;
              } else {
                list[index] = value;
              }
              one.set(name, list);
              current.set('roles', roles);
            }
          } else {
            if(subscript) {
              var one = current.get(what);
              one[subscript] = value;
              one.set(what, one);
              current.set('roles', roles);
            } else {
              var dupkey = 0;
              if(nodups) dupkey = current.cluster.getKeyFor(current.get('baseType'), value, what, newKey);
              if(nodups && (value != dupnull) && (dupkey != current.get('uniqueKey') && (dupkey != 0))) {
                text.value =  objectValueToViewValue(valueViewMapper, one.get(what));
                dock.displayError("A " + current.get('baseType') + " with that " + what + " already exists");
              } else {
                one.set(what, value);
                current.set('roles', roles);
              }
            }
          }
        } else {
          if(i > 0) {
            var index = parseInt(what.substr(i + 1));
            var name = what.substr(0, i);
            var list = one.get(name);
            if(index > list.length) {
              println("Attemp to write past the last element of: " + name);
              println("  length = " + list.length + ", index = " + index);
            } else {
              if(subscript) {
                text.value = objectValueToViewValue(valueViewMapper, list[index][subscript]);
              } else {
                text.value = objectValueToViewValue(valueViewMapper, list[index]);
              }
            }
          } else {
            if(subscript) {
              var one = one.get(what);
              text.value = objectValueToViewValue(valueViewMapper, one[subscript]);
            } else {
              text.value = objectValueToViewValue(valueViewMapper, current.get(what));
            }
          }
        }
        document.commandDispatcher.advanceFocus();
        break;
      case KEY_ESCAPE:
        text.setAttribute("style", "");
        if(i > 0) {
          var index = parseInt(what.substr(i + 1));
          var name = what.substr(0, i);
          var list = one.get(name);
          if(index > list.length) {
            println("Attemp to write past the last element of: " + name);
            println("  length = " + list.length + ", index = " + index);
          } else {
            if(subscript) {
              text.value = objectValueToViewValue(valueViewMapper, list[index][subscript]);
            } else {
              text.value = objectValueToViewValue(valueViewMapper, list[index]);
            }
          }
        } else {
          if(subscript) {
            var one = one.get(what);
            text.value = objectValueToViewValue(valueViewMapper, one[subscript]);
          } else {
            text.value = objectValueToViewValue(valueViewMapper, one.get(what));
          }
        }
    }
  } else {
    println("Error: role not found " + what);
  }
}

function role_textbox_change(event) {
  var text = event.target;
  var value = text.value;
  var what = text.getAttribute('what');
  var category = text.getAttribute('category');
  var role = text.getAttribute('role');
  var roles = current.get('roles');
  var one = 0;
  for(var i = 0; i < roles.length; i++) {
    if(roles[i].childType == role) {
      one = new dock.cvObject(current.cluster, roles[i]);
      break;
    }
  }
  if(one) {
    var valueViewMapper = text.getAttribute('valueViewMapper');
    value = viewValueToObjectValue(valueViewMapper, value);
    var format = text.getAttribute('format');
    var subscript = text.getAttribute('subscript');
    var nodups = eval(text.getAttribute('nodups'));
    var dupnull = text.getAttribute('dupnull');
    var newKey = text.getAttribute('newKey');
    var i = what.indexOf('_');
    if(i > 0) {
      var index = parseInt(what.substr(i + 1));
      var name = what.substr(0, i);
      var list = one.get(name);
      if(index > list.length) {
        println("Attemp to write past the last element of: " + name);
        println("  length = " + list.length + ", index = " + index);
      } else {
        if(regexMatch(value, format)) {
          if(subscript) {
            list[index][subscript] = value;
          } else {
            list[index] = value;
          }
          one.set(name, list);
          current.set('roles', roles);
        } else {
          if(subscript) {
            text.value = objectValueToViewValue(valueViewMapper, list[index][subscript]);
          } else {
            text.value = objectValueToViewValue(valueViewMapper, list[index]);
          }
        }
      }
    } else {
      if(regexMatch(value, format)) {
        if(subscript) {
          var one = current.get(what);
          one[subscript] = value;
          one.set(what, one);
          current.set('roles', roles);
        } else {
          var dupkey = 0;
          if(nodups) dupkey = current.cluster.getKeyFor(current.get('baseType'), value, what, newKey);
          if(nodups && (value != dupnull) && (dupkey != current.get('uniqueKey') && (dupkey != 0))) {
            text.value = objectValueToViewValue(valueViewMapper, one.get(what));
            dock.displayError("A " + current.get('baseType') + " with that " + what + " already exists");
          } else {
            one.set(what, value);
            current.set('roles', roles);
          }
        }
      } else {
        if(subscript) {
          var one = one.get(what);
          text.value = objectValueToViewValue(valueViewMapper, one[subscript]);
        } else {
          text.value = objectValueToViewValue(valueViewMapper, one.get(what));
        }
      }
    }
  } else {
    println("Error: role not found " + what);
  }
  text.setAttribute("style", "");
}

function role_textbox_command(event) {
  var text = event.target;
  var value = text.value;
  var what = text.getAttribute('what');
  var category = text.getAttribute('category');
  var format = text.getAttribute('format');
  var nodups = eval(text.getAttribute('nodups'));
  var dupnull = text.getAttribute('dupnull');
  var newKey = text.getAttribute('newKey');
  var category = text.getAttribute('category');

  if(regexMatch(value, format)) {
    var dupkey = 0;
    if(nodups) dupkey = current.cluster.getKeyFor(current.get('baseType'), value, what, newKey);
    if(nodups && (value != dupnull) && (dupkey != current.get('uniqueKey') && (dupkey != 0))) {
      text.setAttribute("style", "color: red;");
    } else {
      text.setAttribute("style", "");
    }
  } else {
    text.setAttribute("style", "color: red;");
  }
}

function role_number(what, role, category, min, max, unit, blank) {
  unit = isundefined(unit, '');
  blank = isundefined(blank, (min - 1));
  var i = what.indexOf('_');
  var index = -1;
  var name;
  if(i > 0) {
    index = parseInt(what.substr(i + 1));
    name = what.substr(0, i);
  } else {
    name = what;
  }
  var obj = document.getElementById(role.get('childType') + '_' + what);
  var popup = obj.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  for(var i = min; i <= max; i++) {
    var item = document.createElement("menuitem");
    if(i == blank) {
      item.setAttribute('label', '');
    } else {
      item.setAttribute('label', i + unit);
    }
    item.setAttribute('number', i);
    popup.appendChild(item);
  }
  var found = role.get(name);
  if(isArray(found)) found = found[index];
  found = Math.max(min, Math.min(max, parseInt(found)));
  obj.selectedIndex = (found - min);
  obj.setAttribute('what', what);
  obj.setAttribute('role', role.get('childType'));
  if(category) {
    obj.setAttribute('category', (found - min));
    obj.setAttribute('disabled', true);
  } else {
    obj.setAttribute('category', '');
    obj.removeAttribute('disabled');
  }
  popup.addEventListener("command", role_number_command, true);
}

function role_number_command(event) {
  var item = event.target;
  var value = parseInt(item.getAttribute('number'));
  var what = item.parentNode.parentNode.getAttribute('what');
  var role = item.parentNode.parentNode.getAttribute('role');
  var category = item.parentNode.parentNode.getAttribute('category');
  if(category.length > 0) {
    item.parentNode.parentNode.selectedIndex = parseInt(category);
  } else {
    var roles = current.get('roles');
    var one = 0;
    for(var i = 0; i < roles.length; i++) {
      if(roles[i].childType == role) {
        one = new dock.cvObject(current.cluster, roles[i]);
        break;
      }
    }
    if(one) {
      var i = what.indexOf('_');
      if(i > 0) {
        var index = parseInt(what.substr(i + 1));
        var name = what.substr(0, i);
        var list = one.get(name);
        if(index > list.length) {
          println("Attemp to write past the last element of: " + name);
          println("  length = " + list.length + ", index = " + index);
        } else {
          list[index] = value;
          one.set(name, list);
          current.set('roles', roles);
        }
      } else {
        one.set(what, value);
        current.set('roles', roles);
      }
    } else {
      println("Error: role not found " + what);
    }
  }
}

function role_checkbox(what, role, category) {
  var obj = document.getElementById(role.get('childType') + '_' + what);
  var value = role.get(what);

  obj.setAttribute('checked', value);
  obj.setAttribute('role', role.get('childType'));
  obj.setAttribute('what', what);
  if(category) {
    obj.setAttribute('category', category);
    obj.setAttribute('disabled', true);
  } else {
    obj.setAttribute('category', '');
    obj.removeAttribute('disabled');
  }
  obj.setAttribute('oncommand', 'role_checkbox_command(event);');
}

function role_checkbox_command(event) {
  var item = event.target;
  var value = item.checked;
  var role = item.getAttribute('role');
  var what = item.getAttribute('what');
  var category = item.getAttribute('category');

  var one = 0;
  var roles = current.get('roles');
  for(var i = 0; i < roles.length; i++) {
    if(roles[i].childType == role) {
      one = new dock.cvObject(current.cluster, roles[i]);
      break;
    }
  }
  if(one) {
    if(category.length > 0) {
      item.setAttribute('checked', true);
    } else {
      one.set(what, value);
      current.set('modified', true);
    }
  } else {
    if(category.length > 0) {
      item.setAttribute('checked', true);
    } else {
      println("Error: role not found " + role + ":" + what);
    }
  }
}

function role_checkboxlist(what, role, category, list, all, allText, allYes, allNo, allOther, extraFunction) {
  extraFunction = isundefined(extraFunction, '');
  var vbox = document.getElementById(role.get('childType') + '_' + what);
  while(vbox.firstChild) vbox.removeChild(vbox.firstChild);
  var found = role.get(what);
  var hasAllYes = (typeof allYes != 'undefined');
  allYes = isundefined(allYes, true);
  allNo = isundefined(allNo, false);
  allOther = isundefined(allOther, new Array());
  if(typeof(all) == 'undefined') {
    all = allNo;
  } else {
    var allcheck = document.createElement("checkimage");
    allcheck.setAttribute('what', what);
    allcheck.setAttribute('role', role.get('childType'));
    allcheck.setAttribute('checked', role.get(all) == allYes);
    allcheck.setAttribute('all', all);
    if(hasAllYes) {
      allcheck.setAttribute('allYes', allYes);
      allcheck.setAttribute('allNo', allNo);
    }
    allcheck.setAttribute('key', 0);
    allcheck.setAttribute('label', allText);
    allcheck.setAttribute('extraFunction', extraFunction);
    all = role.get(all);
    if(category) {
      allcheck.setAttribute('category', allcheck.getAttribute('checked'));
      allcheck.setAttribute('disabled', true);
    } else {
      allcheck.setAttribute('category', '');
      allcheck.removeAttribute('disabled');
    }
    allcheck.setAttribute('oncommand', 'role_checkboxlist_command(event);');
    vbox.appendChild(allcheck);
  }
//   println('ALL: ' + all + ' ' + allYes + ' ' + (all == allYes));
  for(var i = 0; i < list.length; i++) {
    var check = document.createElement("checkimage");
    check.setAttribute('what', what);
    check.setAttribute('role', role.get('childType'));
    if(isArray(list[i])) {
      check.setAttribute('checked', (found.find(list[i][0]) >= 0));
      check.setAttribute('key', list[i][0]);
      check.setAttribute('label', list[i][1]);
    } else if(isObject(list[i])) {
      check.setAttribute('checked', (found.find(list[i].get('uniqueKey')) >= 0));
      check.setAttribute('label', list[i].resolveName());
      check.setAttribute('key', list[i].get('uniqueKey'));
    } else {
      check.setAttribute('checked', (found.find(list[i]) >= 0));
      check.setAttribute('label', list[i]);
      check.setAttribute('key', list[i]);
    }
    if(category) {
      check.setAttribute('category', check.getAttribute('checked'));
      check.setAttribute('disabled', true);
    } else {
      check.setAttribute('category', '');
      check.removeAttribute('disabled');
    }
    if(all == allYes) {
      check.setAttribute('parentChecked', true);
    } else if(allOther.find(all) >= 0) {
      check.setAttribute('checked', false);
      check.setAttribute('disabled', true);
    } else {
      check.setAttribute('oncommand', 'role_checkboxlist_command(event);');
    }
    check.setAttribute('extraFunction', extraFunction);
    vbox.appendChild(check);
  }
}

function role_checkboxlist_command(event) {
  var item = event.target;
  var value = item.checked;
  var name = item.getAttribute('label');
  var key = item.getAttribute('key');
  var all = item.getAttribute('all');
  var extraFunction = item.getAttribute('extraFunction');
  if(extraFunction.length) eval('extraFunction = ' + extraFunction);
  var allYes = true;
  var allNo = false;
  if(item.hasAttribute('allYes')) {
    allYes = item.getAttribute('allYes');
    allNo = item.getAttribute('allNo');
  }
  var role = item.getAttribute('role');
  var what = item.getAttribute('what');
  var category = item.getAttribute('category');

  var one = 0;
  var roles = current.get('roles');
  for(var i = 0; i < roles.length; i++) {
    if(roles[i].childType == role) {
      one = new dock.cvObject(current.cluster, roles[i]);
      break;
    }
  }
  if(one) {
    if(all.length > 0) {
      if(category.length > 0) {
        item.setAttribute('checked', category);
      } else {
        var list = document.getElementsByAttribute('what', what);
        for(var i = 0; i < list.length; i++) {
          if((list[i].getAttribute('all').length == 0) && (list[i].getAttribute('role') == role)) {
            list[i].setAttribute('parentChecked', value);
          }
        }
        if(value) {
          one.set(all, allYes);
        } else {
          one.set(all, allNo);
        }
        current.set('modified', true);
        if(extraFunction) extraFunction(current, one, all);
      }
    } else {
      if(category.length > 0) {
        item.setAttribute('checked', category);
      } else {
        var list = one.get(what);
        if(value) {
          if(list.find(key) < 0) {
            list.push(key);
            one.set(what, list);
            current.set('roles', roles);
          }
        } else {
          var index = list.find(key);
          if(index >= 0) {
            list.splice(index, 1);
            one.set(what, list);
            current.set('roles', roles);
          } else {
            println("Not found: " + key);
          }
        }
        if(extraFunction) extraFunction(current, one, what);
      }
    }
  } else {
    if(category.length > 0) {
      item.setAttribute('checked', category);
    } else {
      println("Error: role not found " + role + ":" + what);
    }
  }
}

function role_enum(what, role, list, category, subscript) {
  subscript = isundefined(subscript, '');
  var obj = document.getElementById(role.get('childType') + '_' + what + subscript);
  println(">>>>>>>>>>>>>>> " + role.get('childType') + '_' + what + subscript);
  var popup = obj.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  var i = what.indexOf('_');
  var index = -1;
  var name;
  if(i > 0) {
    index = parseInt(what.substr(i + 1));
    name = what.substr(0, i);
  } else {
    name = what;
  }
  var find = role.get(name);
  if(isArray(find)) find = find[index];
  if(subscript) find = find[subscript];
  var found = -1;
  var val = '';
  for(var i = 0; i < list.length; i++) {
    var item = document.createElement("menuitem");
    if(isArray(list[i])) {
      item.setAttribute('key', list[i][0]);
      item.setAttribute('label', list[i][1]);
      if(typeof(find) == 'string') {
        if(string_case_compare(find, list[i][0])) found = i;
      } else {
        if(find == list[i][0]) found = i;
      }
    } else {
      item.setAttribute('key', list[i]);
      item.setAttribute('label', list[i]);
      if(typeof(find) == 'string') {
        if(string_case_compare(find, list[i])) found = i;
      } else {
        if(find == list[i]) found = i;
      }
    }
    popup.appendChild(item);
  }
  if(found >= 0) {
    obj.selectedIndex = found;
  } else {
    obj.setAttribute('label', val);
  }
  if(category) {
    obj.setAttribute('category', find);
    obj.setAttribute('disabled', true);
  } else {
    obj.setAttribute('category', BLANK_CATEGORY);
    obj.removeAttribute('disabled');
  }
  obj.setAttribute('role', role.get('childType'));
  obj.setAttribute('what', what);
  obj.setAttribute('subscript', subscript);
  popup.addEventListener("command", role_enum_command, true);
}

function role_enum_command(event) {
  var item = event.target;
  var value = item.getAttribute('key');
  var what = item.parentNode.parentNode.getAttribute('what');
  var subscript = item.parentNode.parentNode.getAttribute('subscript');
  var category = item.parentNode.parentNode.getAttribute('category');

  var role = item.parentNode.parentNode.getAttribute('role');
  var roles = current.get('roles');
  var one = 0;
  for(var i = 0; i < roles.length; i++) {
    if(roles[i].childType == role) {
      one = new dock.cvObject(current.cluster, roles[i]);
      break;
    }
  }
  if(one) {
    var i = what.indexOf('_');
    if(i > 0) {
      var index = parseInt(what.substr(i + 1));
      var name = what.substr(0, i);
      var list = one.get(name);
      if(index > list.length) {
        println("Attemp to write past the last element of: " + name);
        println("  length = " + list.length + ", index = " + index);
      } else {
        if(subscript) {
          if(isNumeric(value)) {
            list[index][subscript] = parseInt(value);
          } else {
            list[index][subscript] = value;
          }
          list[index].modified = true;
        } else {
          if(isNumeric(value)) {
            list[index] = parseInt(value);
          } else {
            list[index] = value;
          }
        }
        one.set(name, list);
      }
    } else {
      if(subscript) {
        var work = one.get(what);
        work.modified = true;
        if(isNumeric(value)) {
          work[subscript] = parseInt(value);
        } else {
          work[subscript] = value;
        }
        one.set('modified', true);
      } else {
        if(isNumeric(value)) {
          one.set(what, parseInt(value));
        } else {
          one.set(what, value);
        }
      }
    }
    current.set('modified', true);
  } else {
    println("One not set!");
  }
}

function role_resolve(what, role, category, type, subscript, blank, specialChar, excludeFunction, displayFunction) {
  subscript = isundefined(subscript, '');
  specialChar = isundefined(specialChar, '');
  excludeFunction = isundefined(excludeFunction, false);
  displayFunction = isundefined(displayFunction, false);
  var i = what.indexOf('_');
  var index = -1;
  var name;
  if(i > 0) {
    index = parseInt(what.substr(i + 1));
    name = what.substr(0, i);
  } else {
    name = what;
  }
  var obj = document.getElementById(role.get('childType') + '_' + what + subscript);
  var value = role.get(name);
  if(isArray(value)) value = value[index];
  if(subscript) value = value[subscript];
//   if(specialChar.length > 0) {
//     value = parseInt(value.substr(specialChar.length, value.length - specialChar.length * 2));
//   } else {
//     if(isArray(value)) value = value[index];
//     if(subscript) {
//       value = value[subscript];
//     }
//   }
  if(!current.cluster.find(value)) value = 0;

  var popup = obj.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);
  if(blank || (value == 0)) {
    var item = document.createElement("menuitem");
    item.setAttribute('key', 0);
    if(typeof(blank) == 'string') {
      item.setAttribute('label', blank);
    } else {
      item.setAttribute('label', '');
    }
    popup.appendChild(item);
    if(value == 0) obj.selectedIndex = 0;
  }
  if(excludeFunction && (typeof(excludeFunction) == 'string')) eval('excludeFunction = ' + excludeFunction);
  if(displayFunction && (typeof(displayFunction) == 'string')) eval('displayFunction = ' + displayFunction);
  var found = current.cluster.find(type);
  for(var i = 0; i < found.length; i++) {
    if(excludeFunction && excludeFunction(found[i])) continue;
    var item = document.createElement("menuitem");
    var key = found[i].get('uniqueKey');
    item.setAttribute('key', key);
    if(displayFunction) {
      item.setAttribute('label', displayFunction(found[i]));
    } else {
      item.setAttribute('label', found[i].resolveName());
    }
    popup.appendChild(item);
    if(value == key) {
      obj.selectedIndex = popup.childNodes.length - 1;
      obj.setAttribute('label', item.getAttribute('label'));
    }
  }
  obj.setAttribute('what', what);
  obj.setAttribute('role', role.get('childType'));
  obj.setAttribute('subscript', subscript);
  obj.setAttribute('specialChar', specialChar);
  popup.addEventListener("command", role_resolve_command, true);
}

function role_resolve_command(event) {
  var item = event.target;
  var value = parseInt(item.getAttribute('key'));
  var what = item.parentNode.parentNode.getAttribute('what');
  var subscript = item.parentNode.parentNode.getAttribute('subscript');
  var specialChar = item.parentNode.parentNode.getAttribute('specialChar');
  var autorefresh = eval(item.parentNode.parentNode.getAttribute('autorefresh'));
  var extraeval = item.parentNode.parentNode.getAttribute('extraeval');

  var category = item.parentNode.parentNode.getAttribute('category');

  var role = item.parentNode.parentNode.getAttribute('role');
  var roles = current.get('roles');
  var one = 0;
  for(var i = 0; i < roles.length; i++) {
    if(roles[i].childType == role) {
      one = new dock.cvObject(current.cluster, roles[i]);
      break;
    }
  }
  if(one) {
    var old;
    var i = what.indexOf('_');
    if(i > 0) {
      var index = parseInt(what.substr(i + 1));
      var name = what.substr(0, i);
      var list = one.get(name);
      if(index > list.length) {
        println("Attemp to write past the last element of: " + name);
        println("  length = " + list.length + ", index = " + index);
      } else {
        if(subscript) {
          if(extraeval.length > 0) old = list[index][subscript];
          list[index].modified = true;
          list[index][subscript] = value;
        } else {
          if(extraeval.length > 0) old = list[index];
          list[index] = value;
        }
        one.set(name, list);
        current.set('modified', true);
        if(autorefresh) parent.refresh();
      }
    } else {
      if(specialChar) value = specialChar + value + specialChar;
      if(subscript) {
        var work = one.get(what);
        if(extraeval.length > 0) old = work[subscript];
        work.modified = true;
        work[subscript] = value;
        one.set('modified', true);
      } else {
        if(extraeval.length > 0) old = one.get(what);
        one.set(what, value);
      }
      current.set('modified', true);
      if(autorefresh) parent.refresh();
    }

    if(extraeval.length > 0) eval(extraeval);
  }
}

function menuListEditable(args) {
  var wrapperElementId = null;
  var wrapperElement = null;
  
  var menuListElementId = null;
  var menuListElement = null;
  
  var textboxElementId = null;
  var textboxElement = null;
  
  
  this.init = function(args) {
    this.initWrapperElement(args);
    this.initMenuListElement(args);
    this.initTextboxElement(args);
    
    var menuListElementOptions = this.menuListElement.firstChild.cloneNode(true);
    this.wrapperElement.replaceChild(menuListElementOptions, this.wrapperElement.firstChild);

    this.wrapperElement.addEventListener('command', this.onCommand.bind(this), true);
    this.wrapperElement.addEventListener('change', this.onChange.bind(this), true);
    this.wrapperElement.addEventListener('keypress', this.onKeypress.bind(this), true);
    
    if(this.textboxElement.value!=='') {
      this.wrapperElement.selectedIndex = -1;
      this.wrapperElement.value = this.textboxElement.value;
    } else {
      this.wrapperElement.selectedIndex = this.menuListElement.selectedIndex;
    }
  };
  
  this.initWrapperElement = function (args) {
    this.wrapperElementId = args.wrapperElementId;
    this.wrapperElement = document.getElementById(this.wrapperElementId);
    if(this.wrapperElement===null) {
      println('Error: wrapper elementid not found using id ' + this.wrapperElementId);
    }
  };

  this.initMenuListElement = function (args) {
    this.menuListElementId = args.menuListElementId;
    this.menuListElement = document.getElementById(this.menuListElementId);
    if(this.menuListElement===null) {
      println('Error: menulist elementid not found using id ' + this.menuListElementId);
    }
    this.menuListElement.setAttribute('hidden', true);
  };
  
  this.initTextboxElement = function (args) {
    this.textboxElementId = args.textboxElementId;
    this.textboxElement = document.getElementById(this.textboxElementId);
    if(this.textboxElement===null) {
      println('Error: textbox elementid not found using id ' + this.textboxElementId);
    }
    this.textboxElement.setAttribute('hidden', true);
  };
    
  this.onCommand = function (event) {
    this.onChange(event);
  };
  
  this.onChange = function (event) {
    var selectedIndex = this.wrapperElement.selectedIndex;
    var value = this.wrapperElement.value;
    if(selectedIndex>=0) {
      this.menuListElement.firstChild.childNodes[selectedIndex].doCommand();
      this.setMenuListButtonTextbox('');
    } else {
      this.setMenuListButtonTextbox(value);
    }
  };
  
  this.onKeypress = function (event) {
  };

  this.setMenuListButtonTextbox = function (value) {
    this.textboxElement.value = value;
    var evt = document.createEvent("HTMLEvents");
    evt.initEvent("change", false, true);
    this.textboxElement.dispatchEvent(evt); 
  };
  
  this.init(args);
}
