/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

//from remotecm/servicecodes.h
var SERVICE_OPERATION_FAILED = 0;
var SERVICE_OPERATION_SUCCEEDED = 1;
var SERVICE_ALREADY_RUNNING = 2;
var SERVICE_ALREADY_STOPPED = 3;
var SERVICE_NOT_RUNNING = 9;

var SERVICE_NOT_ALLOWED_TO_START = -2000;
var SERVICE_NOT_ALLOWED_TO_RESTART = -2001;
var SERVICE_WRONG_HA_MODE = -2010;
var SERVICE_INITD_SCRIPT_NOT_FOUND = -2020;
var SERVICE_INITD_SCRIPT_TIMEOUT = -2030;
var SERVICE_FAILING = -2040;
var SERVICE_REMOVED = -2050;

var SERVICE_EXITCODE_OFFSET = 50000;
var SERVICE_SIGNAL_OFFSET = 51000;
var SERVICE_SOAP_ERROR_OFFSET = 60000;

var SERVICE_INTERNAL_ERROR = 100000;

var SERVICE_NODE_DOWN = -1000001;
var SERVICE_NODE_NOT_FOUND = -1000002;
var SERVICE_NODE_NO_RCM = -1000003;
var SERVICE_NODE_SOAP_ERROR = -1000004;

function oneServiceCall(current, select, nodes, call, successWord, failWord) {
  var work = {n: 0, m: select.length, text: '', cluster: current.cluster};
  for(var i = 0; i < select.length; i++) {
    var name = select[i].split('|')[1];
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmserv', call, [name, [], nodes], function(code, data) {
        if(code == 0) {
          var name = this.input[0];
          var nodes = this.input[2];
          var c = new Object();
          for(var j = 0; j < Math.min(nodes.length, data.length); j++) {
            if(data[j] != SERVICE_OPERATION_SUCCEEDED) data[j] = SERVICE_OPERATION_FAILED; // TODO: disable if we want to show all types of messages
            if(!c.hasOwnProperty(data[j])) c[data[j]] = new Array();
            c[data[j]].push(nodes[j]);
          }
          this.current.n += 1;
          if(c.hasOwnProperty(SERVICE_OPERATION_SUCCEEDED)) this.current.text += "Succesfully " + successWord + " " + name + " on: " + this.current.cluster.resolveList(c[SERVICE_OPERATION_SUCCEEDED]) + "\n";
          if(c.hasOwnProperty(SERVICE_OPERATION_FAILED)) this.current.text += "Failed to " + failWord + " " + name + " on: " + this.current.cluster.resolveList(c[SERVICE_OPERATION_FAILED]) + "\n";
          if(this.current.n >= this.current.m) {
            dock.displayInfo(this.current.text);
            refresh();
          }
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, work);
  }
}

function startService() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a service");
  } else if(select.length >= 1) {
    var args = new Object();
    args['question'] = "Are you sure you want to start these services";
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      var nodes = allNodes();
      oneServiceCall(current, select, nodes, 'pstartOSService', 'started', 'start');
    }
    dock.dialogs.showDialog(d);
  }
}

function stopService() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a service");
  } else if(select.length >= 1) {
    var args = new Object();
    args['question'] = "Are you sure you want to stop these services";
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      var nodes = allNodes();
      oneServiceCall(current, select, nodes, 'pstopOSService', 'stopped', 'stop');
    }
    dock.dialogs.showDialog(d);
  }
}

function restartService() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a service");
  } else if(select.length >= 1) {
    var args = new Object();
    args['question'] = "Are you sure you want to restart these services";
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      var nodes = allNodes();
      oneServiceCall(current, select, nodes, 'prestartOSService', 'restarted', 'restart');
    }
    dock.dialogs.showDialog(d);
  }
}

function reloadService() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a service");
  } else if(select.length >= 1) {
    var args = new Object();
    args['question'] = "Are you sure you want to reload these services";
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      var nodes = allNodes();
      oneServiceCall(current, select, nodes, 'preloadOSService', 'reloaded', 'reload');
    }
    dock.dialogs.showDialog(d);
  }
}

function resetService() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a service");
  } else if(select.length >= 1) {
    var args = new Object();
    args['question'] = "Are you sure you want to reset these services";
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      var nodes = allNodes();
      oneServiceCall(current, select, nodes, 'presetOSService', 'reset', 'reset');
    }
    dock.dialogs.showDialog(d);
  }
}
