/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var current;
var myDialog;
var dock;

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  current = myDialog.arguments.cluster;
  setDialogTitle("Parallel Shell commands on " + current.name)
  refresh();
}

function refresh() {
  dock.workerpool.getWorker(current.actualurl).call('cmproc', 'listCommands', false, function(code, data) {
    if(code == 0) {
      setCallbackResult(data);
    } else {
      dock.jsonCallError(this, code, data);
    }
    refresh();
  }, current);
}

function setCallbackResult(pjobs) {
  tree = document.getElementById('tree');
  data = new Array();

  for(var i = 0; i < pjobs.length; i++) {
    data.push(createTreeItem(pjobs[i]));
  }
  for(var i = pjobs.length; i < MINTREEITEMS; i++) data.push(blankTreeItem());
  var select = selected('list');
  loadTable();
  setSelected('list', select);
}

function createTreeItem(one) {
  var item = new Object();
  item['key'] = one.input.tracker;
  item['tracker'] = one.input.tracker;
  var d = new Date(); d.setTime(one.startTime * 1000);
  item['time'] = d.format('dd/mmm/yyyy HH:MM');
  item['command'] = one.input.cmd;
  item['user'] = one.input.user;
  var nodes = one.nodes;
  item['nodes'] = current.resolveList(nodes);
  return item;
}

function pjobDetails() {
//   var select = selected('list');
//   if(select.length == 0) {
//     dock.displayError("No parallel command selected", window);
//   } else if(select.length == 1) {
//     var args = new Object();
//     args['cluster'] = current;
//     args['tracker'] = select[0];
//     var d = new dock.cvDialog("pjobdetails.xul", args);
//     d.parentWindow = window;
//     dock.dialogs.showDialog(d);
//   } else {
//     dock.displayError("Select only one parallel command", window);
//   }
}

function pjobKill() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("No parallel command selected", window);
  } else {
    var args = new Object();
    if(select.length == 1) {
      args['question'] = "Are you sure you want to kill this command on all nodes executing it?";
    } else {
      args['question'] = "Are you sure you want to kill these commands on all nodes executing it?";
    }
    for(var i = 0; i < select.length; i++) {
      select[i] = parseInt(select[i]);
    }
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      var nodes = new Array();
      dock.workerpool.getWorker(current.actualurl).call('cmproc', 'killCommand', [nodes, select], function(code, data) {
        if(code == 0) {
          var lines = new Array();
          for(var i = 0; i < data.length; i++) {
            var hostname = this.current.resolveName(data[i].node);
            var trackers = data[i].trackers;
            var results = data[i].results;
            if(trackers.length != results.length) {
              lines.push(hostname + ': Internal error');
            } else if(trackers.length == 0) {
              lines.push(hostname + ': Nothing to kill');
            } else if(data[i].node == 0) {
              switch(results[0]) {
                case(dock.PRE_NOT_STARTED_YET) :
                  lines.push('Tracker: ' + trackes[0] + ' was not started yet');
                  break;
                // TODO: cases
                default:
                  lines.push('Tracker: ' + trackes[0] + '. Error ' + results[0]);
              }
            } else {
              for(var j = 0; j < results.length; j++) {
                switch(results[j]) {
                  case(dock.PRE_KILLED) :
                    lines.push(hostname + ' killed: ' + trackers[j]);
                    break;
                  case(dock.PRE_FINISHED) :
                    lines.push(hostname + ' already finished: ' + trackers[j]);
                    break;
                  case(dock.PRE_QUEUED) :
                    lines.push(hostname + ' had not started: ' + trackers[j]);
                    break;
                  case(dock.PRE_NO_RCM) :
                    lines.push(hostname + ' failed to send data to master for: ' + trackers[j]);
                    break;
                  case(dock.PRE_STILL_RUNNING) :
                    lines.push(hostname + ' failed to kill: ' + trackers[j]);
                    break;
                  case(dock.PRE_FAILED_TO_STOP) :
                    lines.push(hostname + ' failed to stop: ' + trackers[j]);
                    break;
                  default:
                    lines.push(hostname + ' unknown kill code: ' + results[j] + ' for tracker ' + trackers[j]);
                }
              }
            }
          }
          dock.displayInfo(lines.join("\n"), window);
        } else {
          dock.jsonCallError(this, code, data);
        }
        refresh();
      }, current);
    }
    d.parentWindow = window;
    dock.dialogs.showDialog(d);
  }
}
