/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function syncNode() {
  if(current.get('modified')) {
    dock.displayInfo("Some settings have been changed, commit or revert them first.");
  } else {
    var args = new Object();
    args['question'] = "Are you sure you want update this node";
    args['check'] = "Dry run";
    args['checked'] = true;
    var d = new dock.cvDialog("question.xul", args);
    var nodes = [current.get('uniqueKey')];
    d.onOk = function(dryrun) {
      dock.workerpool.getWorker(current.cluster.actualurl).call('cmprov', 'imageUpdate', [nodes, dryrun, current.cluster.sessionid], function(code, data) {
        if(code == 0) {
          if(data) {
            dock.displayInfo("Image synchronization started");
          } else {
            dock.displayError("Image synchronization failed");
          }
        } else {
          dock.jsonCallError(this, code, data);
        }
        refresh();
      }, current);
    }
    dock.dialogs.showDialog(d);
  }
}

function reinstallNode() {
  if(current.get('modified')) {
    dock.displayInfo("Some settings have been changed, commit or revert them first.");
  } else {
    var args = new Object();
    args['question'] = "Are you sure you want reinstall this node";
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function() {
      current.set("nextBootInstallMode", "FULL");
      if(current.status.status == 'UP') {
        current.commit(1);
      } else {
        current.commit(2);
      }
    }
    dock.dialogs.showDialog(d);
  }
}

function syncImage() {
  if(current.get('modified')) {
    dock.displayInfo("Some settings have been changed, commit or revert them first.");
  } else {
    var args = new Object();
    args['question'] = "Are you sure you want synchronize the image from this node";
    args['check'] = "Dry run";
    args['checked'] = true;
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function(dryrun) {
      var category = current.cluster.find(current.get('category'));
      if(category) {
        var image = category.getSoftwareImage();
        if(image) {
          dock.workerpool.getWorker(current.cluster.actualurl).call('cmprov', 'grabImage', [current.get('uniqueKey'), 0, dryrun, current.cluster.sessionid], function(code, data) {
            if(code == 0) {
              if(data) {
                dock.displayInfo("Image grab started");
              } else {
                dock.displayError("Image grab failed");
              }
            } else {
              dock.jsonCallError(this, code, data);
            }
            refresh();
          }, current);
        } else {
          dock.displayError('The category of this node does not have a software image');
        }
      } else {
        dock.displayError('This node does not have a category');
      }
    }
    dock.dialogs.showDialog(d);
  }
}

function newImage() {
  if(current.get('modified')) {
    dock.displayInfo("Some settings have been changed, commit or revert them first.");
  } else {
    var args = new Object();
    args['title'] = 'Grab image to:';
    args['type'] = 'SoftwareImage';
    args['cluster'] = current.cluster;
    args['check'] = "Dry run";
    args['checked'] = true;
    args['no'] = false;
    args['cancel'] = true;
    var d = new dock.cvDialog("selectobject.xul", args);
    d.onOk = function(image, dryrun) {
      if(typeof(image) == 'string') image = parseInt(image);
      dock.workerpool.getWorker(current.cluster.actualurl).call('cmprov', 'grabImage', [current.get('uniqueKey'), image, dryrun, current.cluster.sessionid], function(code, data) {
        if(code == 0) {
          if(data) {
            dock.displayInfo("Image grab started");
          } else {
            dock.displayError("Image grab failed");
          }
        } else {
          dock.jsonCallError(this, code, data);
        }
        refresh();
      }, current);
    }
    dock.dialogs.showDialog(d);
  }
}

function provisioningLog() {
  var args = new Object();
  args['nodes'] = [current.get('uniqueKey')];
  args['cluster'] = current.cluster;
  var d = new dock.cvDialog("synclog.xul", args);
  dock.dialogs.showDialog(d);
}
