/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'SoftwareImage';
var singular = 'software image';
var plural = 'software images';

function createTreeItem(one) {
  if(one.get('parentSoftwareImage') > 0) return null;
  
  var item = new Object();
  item['key'] = one.get('uniqueKey');
  item['modified'] = one.get('modified');
  item['name'] = one.get('name');
  item['path'] = one.get('path');
  item['kernelversion'] = one.get('kernelVersion');  
  var revisionsDescriptionList = one.get('revisionsDescriptionList');
  var revisionsList = one.get('revisionsList');
  item['revisions'] = revisionsList.length + ' active, ' + revisionsDescriptionList.length + ' created';
  return item;
}

function addObject() {
  var base = current.getBasePartition();
  var original = current.find(base.get('defaultCategory'));
  if(isObject(original)) {
    var original = current.find(original.getSoftwareImage());
    if(isObject(original)) {
      var args = new Object();
      args['title'] = 'Add new ' + singular;
      args['question'] = 'Name';
      args['default'] = '';
      args['known'] = current.known(basetype, 'name');
      var d = new dock.cvDialog("newobject.xul", args);
      d.onOk = function(name) {
          var object = original.clone();
          object.modified = true;
          object.toBeRemoved = false;
          object.name = name;
          var slash = -1;
          var opath = original.get('path');
          if(endswith(opath, '/') && (opath.length >= 2)) {
            slash = opath.lastIndexOf('/', opath.length - 2);
          } else {
            slash = opath.lastIndexOf('/');
          }
          if(slash <= 0) {
            object.path = '/cm/images/' + name;
          } else {
            object.path = opath.substr(0, slash) + '/' + name;
          }
          current.addObject(object, dock.openAfterCreate);
          refresh();      
        }
      dock.dialogs.showDialog(d);
    } else {
      dock.displayError("Software image not set for default category");
    }
  } else {
    dock.displayError("Default category not set in base partition");
  }
}

function cloneObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a " + singular + " for cloning");
  } else if(select.length > 1) {
    dock.displayError("Please select only one " + singular);
  } else {
    var original = current.find(select[0]);
    if(isObject(original)) {
      var args = new Object();
      var name = original.get('name');
      args['title'] = 'Clone ' + name;
      args['question'] = 'Name';
      var nn = nameNumber(name);
      if(nn[1].length == 0) nn[1] = '0';
      var index = myParseInt(nn[1]);
      while(true) {
        index++;
        name = nn[0] + sprintf('%0' + nn[1].length + 'd', index);
        if(!current.find('SoftwareImage', name)) break;
      }
      args['default'] = name;
      args['known'] = current.known('SoftwareImage', 'name');
      var d = new dock.cvDialog("newobject.xul", args);
      d.onOk = function(name) {
        var object = original.clone();
        object.modified = true;
        object.toBeRemoved = false;
        object.name = name;
        var slash = -1;
        var opath = original.get('path');
        if(endswith(opath, '/') && (opath.length >= 2)) {
          slash = opath.lastIndexOf('/', opath.length - 2);
        } else {
          slash = opath.lastIndexOf('/');
        }
        if(slash <= 0) {
          object.path = '/cm/images/' + name;
          object.originalImage = 0;
        } else {
          object.path = opath.substr(0, slash) + '/' + name;
          object.originalImage = original.get('uniqueKey');
        }
        current.addObject(object, dock.openAfterCreate);
        refresh();
      }
      dock.dialogs.showDialog(d);
    } else {
      dock.displayError("Original not found, already removed by someone else");
    }
  }
}

function removeObject() {
  var select = getSelected();
  if(select.length == 0) {
    dock.displayError('Please select a ' + singular);
  } else {
    var cluster = current;
    if(!!current.cluster) cluster = current.cluster;
    var args = new Object();
    if(select.length == 1) {
      args['question'] = 'Are you sure you want to remove this ' + singular + '?';
    } else {
      args['question'] = 'Are you sure you want to remove these ' + plural + '?';
    }
    args['check'] = 'Also remove data directory from the disk';
    args['checked'] = false;
    var d = new dock.cvDialog('question.xul', args);
    d.onOk = function(data) {
      for(var i = 0; i < select.length; i++) cluster.removeObject(select[i], data, false);
      refresh();
    }
    dock.dialogs.showDialog(d);
  }
}
