/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var timer = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function stop() {
  if(timer) window.clearTimeout(timer);
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh softwareimagefolder provisioning" + ' - ' + document.documentURI);

  dock.workerpool.getWorker(current.actualurl).call('cmprov', 'getProvisioningStatus', [[], []], function(code, data) {
    if(code == 0) {
      setCallbackResult(data);
    } else {
      dock.jsonCallError(this, code, data);
    }
  }, current);
}

function setCallbackResult(status) {
  if(dock.DEMO) {
    // TODO
  }

  if(status) {
    var request = status.provisioningRequestStatusList;
    var nodes = status.provisioningNodeStatusList;

    var used = new Object();
    used[0] = new Array();
    data = new Array();
    for(var i = 0; i < request.length; i++) {
      var skey = request[i].sourceNodeKey;
      var dkey = request[i].destinationNodeKey;
      var name = current.resolveName(dkey);
      if(name.length) dkey = name;
      if(typeof(used[skey]) == 'undefined') {
        used[skey] = [dkey];
      } else {
        used[skey].push(dkey);
      }
      data.push(createReqTreeItem(request[i]));
    }
    tree = document.getElementById('treerequests');
    loadTable();
    
    data = new Array();
    data.push(createProvTreeItem(false, used[0]));
    for(var i = 0; i < nodes.length; i++) {    
      if(used.hasOwnProperty(nodes[i].nodeKey)) {
        data.push(createProvTreeItem(nodes[i], used[nodes[i].nodeKey]));
      }
    }
    tree = document.getElementById('treenodes');
    loadTable();
  }

  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('refresh();', dock.generalSettings.statusRefresh * 1000);
}

function createReqTreeItem(one) {
  var item = new Object();
  var ids = one.requestIDs;
  item['key'] = ids[0];
  item['requestid'] = ids.join(',');
  if((one.syncMode == 3) || (one.syncMode == 4)) { // GRAB, see remotecm/syncmode.h
    item['srcnode'] = current.resolveName(one.destinationNodeKey);
    item['srcpath'] = one.destinationPath;
    item['destnode'] = current.resolveName(one.sourceNodeKey);
    item['destpath'] = one.sourcePath;
  } else {
    item['srcnode'] = current.resolveName(one.sourceNodeKey);
    item['srcpath'] = one.sourcePath;
    item['destnode'] = current.resolveName(one.destinationNodeKey);
    item['destpath'] = one.destinationPath;
  }
  return item;
}

function createProvTreeItem(one, nodes) {
  var item = new Object();
  if(one) {
    item['key'] = one.nodeKey;
    item['hostname'] = current.resolveName(one.nodeKey);
    item['capacity'] = one.slotsUsed + ' / ' + one.slotsCapacity;
    var imU = new Array();
    var imO = new Array();
    var ims = one.imageKeys;
    var u2d = one.upToDate;
    for(var i = 0; i < Math.min(ims.length, u2d.length); i++) {
      var name = current.resolveName(ims[i]);
      if(name.length) {
        if(u2d[i]) {
          imU.push(name);
        } else {
          imO.push(name);
        }
      }
    }
    item['upImages'] = imU.join(',');
    item['outImages'] = imO.join(',');
    imO = new Array();
    var grps = one.nodeGroupKeys;
    for(var i = 0; i < grps.length; i++) {
      var name = current.resolveName(grps[i]);
      if(name.length) {
        imO.push(name);
      }
    }
    item['nodegroups'] = imO.join(',');
  }
  if(nodes) {
    if(!one) {
      item['hostname'] = '< unassigned >';
      item['capacity'] = nodes.length + ' pending';
    }
    item['busynodes'] = dotFormat(nodes);
  }
  return item;
}

function updateProvisioners() {
  var args = new Object();
  args['question'] = "Are you sure you want to update all software images on provisioning nodes";
  var d = new dock.cvDialog("question.xul", args);
  d.onOk = function() {
    dock.workerpool.getWorker(current.actualurl).call('cmprov', 'updateProvisioners', [[], [], current.sessionid], function(code, data) {
      if(code == 0) {
        if(data) {
          dock.displayInfo("Update started in the background, events will be sent");
        } else {
          dock.displayError("Unable to start update of software images");
        }
      } else {
        dock.jsonCallError(this, code, data);
      }
      refresh();
    }, current);
  }
  dock.dialogs.showDialog(d);
}

function requestDetails() {
  var select = selected('listrequests');
  if(select.length == 0) {
    dock.displayError("Please select a provisioning request");
  } else if(select.length > 1) {
    dock.displayError("Please select only one provisioning request");
  } else {
    var args = new Object();
    args['cluster'] = current;
    args['requestId'] = select[0];
    var d = new dock.cvDialog("provisioningrequest.xul", args);
    dock.dialogs.showDialog(d);
  }
}

function cancelRequest() {
  var select = selected('listrequests');
  if(select.length == 0) {
    dock.displayError("Please select one or more provisioning requests");
  } else {
    var requestIDs = new Array();
    for(var i = 0; i < select.length; i++) {
      requestIDs.push(parseInt(select[i]));
    }
    dock.workerpool.getWorker(current.actualurl).call('cmprov', 'cancelProvisioningRequests', [requestIDs, [], false], function(code, data) {
      if(code == 0) {
        if(data) {
          dock.displayInfo("Requests cancelled");
        } else {
          dock.displayError("Unable to cancel requests");
        }
        refresh();
      } else {
        dock.jsonCallError(this, code, data);
      }
      refresh();
    }, current);
  }
}

function nodeDetails() {
}
