/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'SoftwareImage';
var singular = 'software image';
var plural = 'software images';

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function getSelected() {
  return selected('list');
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh software image revisions" + ' - ' + document.documentURI);

  tree = document.getElementById('tree');
  data = new Array();

  var baseName = current.get('name');    
  var revisionsCreationTimeList = current.get('revisionsCreationTimeList');
  var revisionsDescriptionList = current.get('revisionsDescriptionList');
  var revisionsList = current.get('revisionsList');
  
  var removed = document.getElementById('showremoved').checked;
  var j = -1;
  var rk = -1;
  var image = false;
  for(var i = 0; i < Math.min(revisionsCreationTimeList.length, revisionsDescriptionList.length); i++) {
    var i1 = i + 1;
    if(rk < i1) {
      j++;
      image = false;
      while(j < revisionsList.length) {
        image = current.cluster.find(revisionsList[j]);
        if(image) {
          rk = image.get('revisionID');
          break;
        } else {
          j++;
        }
      }    
    }
    if(rk == i1) {
      data.push(createTreeItem(i1, image, revisionsCreationTimeList[i], revisionsDescriptionList[i]));
    } else if(removed) {
      data.push(createTreeItem(i1, false, revisionsCreationTimeList[i], revisionsDescriptionList[i], baseName + '@' + i1));
    }
  }
  
  var select = getSelected();
  loadTable();
  setSelected('list', select);
  if(typeof(postrefresh) == 'function') {
    if(postrefresh(use)) return;
  }
}

function createTreeItem(id, one, time, description, name) {
  var item = new Object();
  item['revisionID'] = id;
  if(one) {
    item['key'] = one.get('uniqueKey');
    item['name'] = one.get('name');
    item['path'] = one.get('path');
  } else {
    item['key'] = -1;
    item['name'] = name;
    item['path'] = '';
  }
  item['description'] = description;
  var dv = new Date();
  dv.setTime(time * 1000);
  item['date'] = dv.format(dock.DATETIME_FORMAT);
  return item;
}

function editRevision() {
  var select = getSelected();
  if(select.length != 1) {
    dock.displayError('Please select a ' + singular);
  } else if(select[0] <= 0) {
    dock.displayError('No data exists for this revision');
  } else {
    var args = new Object();
    args['current'] = current.cluster.find(select[0]);
    var d = new dock.cvDialog("detachedsoftwareimagetab.xul", args);
    d.parentWindow = window;
    dock.dialogs.show(d);
  }  
}

function removeRevision() {
  var select = getSelected();
  if(select.length == 0) {
    dock.displayError('Please select a ' + singular);
  } else {
    var args = new Object();
    if(select.length == 1) {
      args['question'] = 'Are you sure you want to remove this ' + singular + '?';
    } else {
      args['question'] = 'Are you sure you want to remove these ' + plural + '?';
    }
    args['check'] = 'Also remove data directory from the disk';
    args['checked'] = false;
    var d = new dock.cvDialog('question.xul', args);
    d.onOk = function(data) {
      for(var i = 0; i < select.length; i++) {
        if(select[i] > 0) current.cluster.removeObject(select[i], data, false);
      }
      refresh();
    }
    dock.dialogs.showDialog(d);
  }
}

function setRevisionAsParent() {
  var select = getSelected();
  if(select.length != 1) {
    dock.displayError('Please select a ' + singular);
  } else if(select[0] <= 0) {
    dock.displayError('No data exists for this revision');
  } else {
    var args = new Object();
    args['question'] = 'Are you sure you want to replace the working copy with this revision ?';
    var d = new dock.cvDialog('question.xul', args);
    d.onOk = function(data) {
      dock.workerpool.getWorker(current.cluster.actualurl).call('cmpart', 'setRevisionToParent', [current.get('uniqueKey'), parseInt(select[0])], function(code, data) {
          if(code == 0) {
            var msg = false;
            if(data.status == -1) {
              msg = 'Software images have not been found';
            } else if (data.status == -2) {
              msg = 'Software image revision is not a revision of ' + current.resolveName();;
            } else if (data.status == -3) {
              msg = 'A file operation on one of the images is in progress';
            }
            if(msg) dock.displayInfo(msg);
          } else {
            if(code) dock.jsonCallError(this, code, data);
          }
        }, current);
    }
    dock.dialogs.showDialog(d);
  }
}

function newRevision() {
  var args = new Object();
  args['title'] = 'New revision description';
  args['question'] = 'Description';
  args['default'] = '';
  args['known'] = [];
  var d = new dock.cvDialog("newobject.xul", args);
  d.onOk = function(description) {
    var object = current.clone();
    object.modified = true;
    object.toBeRemoved = false;
    object.name = '+NewRevision+';
    object.revisionID = -2;
    object.revisionDescription = description;
    object.path += '-newRevision'; // will not be used
    object.originalImage = current.get('uniqueKey');
    if(current.get('parentSoftwareImage') != 0) {
      object.parentSoftwareImage = current.get('parentSoftwareImage');
    } else {
      object.parentSoftwareImage = current.get('uniqueKey');
    }
    object = current.cluster.addObject(object, false);
    object.commit();
    refresh();
  }
  dock.dialogs.showDialog(d);
}

