/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var SVG_FONT_SIZE = 11;
var FONT_COLOR = "#2f5a8c";
var EPS = 1e-6;
var XUL_SVG = "http://www.w3.org/2000/svg";
var XUL_XLINK = "http://www.w3.org/1999/xlink";
var ACC_AS_WIDTH = false;
var ACC_AS_FILL = true;
var OUTLIER_R = 2.5;

function svgheatmap(box, x0, y0, x1, y1, xrange, yrange, data) {  
  var w = x1 - x0;
  var h = y1 - y0;
  
  var nx = data.xBins.length - 1;
  var ny = data.yBins.length - 1;
  
  for(var i = 0; i < nx; i++) {
    for(var j = 0; j < ny; j++) {
      var x = i / nx;
      var y = j / ny;
      var c = data.dataPoints[i * ny + j];
      svgrect(box, x0 + w * x, y0 + h * y, w / nx, h / ny, colorbar(c), 0.5, '#666666', 0.8);
    }
  }
}

function svgpiechart(box, x0, y0, x1, y1, data, names, unit, maxCount, minPercentage, basePercentage) {
  unit = isundefined(unit, "");
  maxCount = isundefined(maxCount, 65536);
  minPercentage = isundefined(minPercentage, -1);
  basePercentage = isundefined(basePercentage, 50);
  
  var w = x1 - x0;
  var h = y1 - y0;
  
  var ss = 0;
  var dd = new Array();
  var dn = new Array();
  var n = Math.min(data.length, names.length);
  for(var i = 0; i < n; i++) { // TODO correct format of data
    if(!kisNaN(data[i])) {
      if(data[i] > 0) {
        dd.push([data[i], names[i], i]);
        ss += data[i];
      }
      dn.push([names[i], data[i], i]);
    }
  }
  dd.sort(function (a, b) {
      if(a[0] > b[0]) return -1;
      else if(a[0] < b[0]) return 1;
      if(a[1] >= b[1]) return 1;
      return -1;
    }
  );
  dn.sort();

  var r = 0;
  var cx = 0;
  var cy = 0;
  var lo = 0;
  if(w >= 2 * h) {
    cy = (y1 + y0) / 2;
    cx = x0 + h / 2;
    r = h * 0.40;
    lo = x0 + h;
  } else {
    cy = (y1 + y0) / 2;
    cx = x0 + w / 4;
    r = w * 0.20;
    lo = x0 + w / 2;
  }

  var info = [cx, cy, r, [], [], []];
  if(dd.length > 1) {
    svgcircle(box, cx, cy, r, "none", '#333333', 3);
    var n = Math.min(maxCount, dd.length);
    
    var a0 = 0;
    var s0 = 0;
    var px0 = cx + 0;
    var py0 = cy - r;
    var last = false;
    for(var i = 0; !last && (i < dd.length); i++) {
      var a1;
      var v1;
      if((i >= maxCount) || ((100 * dd[i][0] / ss < minPercentage) && (100 * s0 / s0 >= basePercentage))) {
        last = true;
        v1 = ss - s0;
        a1 = 2 * Math.PI;
      } else {
        v1 = dd[i][0];
        a1 = 2 * Math.PI * (s0 + v1) / ss;
      }
      var px1 = cx + r * Math.sin(a1);
      var py1 = cy - r * Math.cos(a1);    
      var big = 0;
      if(v1 * 2 > ss) big = 1;    
      var p = 'M' + cx + ',' + cy + ' L' + px0 + ',' + py0 + ' A' + r + ',' + r + ' 0 ' + big + ',1 ' + px1 + ' ' + py1 + 'Z';
    
      var path = document.createElementNS(XUL_SVG, "svg:path");
      path.setAttribute("d", p);
      var c = 1 - i / n;
      path.setAttribute("fill", colorbar(c));
      box.appendChild(path);
      s0 += v1;
      px0 = px1;
      py0 = py1;
      info[3].push([a0, a1]);
      info[4].push(dd[i][2]);
      a0 = a1;
    }
    var s0 = 0;
    var last = false;
    for(var i = 0; !last && (i < dd.length); i++) {
      var a1;
      var v1;
      if((i >= maxCount) || ((100 * dd[i][0] / ss < minPercentage) && (100 * s0 / s0 >= basePercentage))) {
        last = true;
        v1 = ss - s0;
        a1 = 2 * Math.PI;
      } else {
        v1 = dd[i][0];
        a1 = 2 * Math.PI * (s0 + v1) / ss;
      }
      var px1 = cx + r * Math.sin(a1);
      var py1 = cy - r * Math.cos(a1);
      svgline(box, cx, cy, px1, py1, '#333333', 0.5);
      s0 += dd[i][0];
    }
    
  } else if(dd.length == 1) {
    svgcircle(box, cx, cy, r, colorbar(1), '#333333', 3);
    info[3].push([0, 2 * Math.pi]);
    info[4].push(dd[0][2]);
  } else {
    svgcircle(box, cx, cy, r, "none", '#333333', 3);
    return info;
  }
  
  var b = 2;
  var b1 = 4;
  var tx0 = lo;
  var ls = SVG_FONT_SIZE * 1.5;
  var ty0 = y0 + ls;
  var mm = 0;
  var s0 = 0;
  var last = false;
  for(var i = 0; !last && (i < dd.length); i++) {
    if(ty0 + 2 * ls > y1) { // new line
      tx0 = tx0 + mm + SVG_FONT_SIZE;
      ty0 = y0 + ls;
      if(tx0 + mm * 0.5 > x1) break; // doesn't fit
    }
    var name;
    var v1
    if((i >= maxCount) || ((100 * dd[i][0] / ss < minPercentage) && (100 * s0 / s0 >= basePercentage))) {
      last = true;
      v1 = ss - s0;
      name = 'other';
    } else {
      v1 = dd[i][0];
      name = dd[i][1];
    }
    var c = 1 - i / n;
    svgrect(box, tx0 + b, ty0 + b, ls - 2 * b, ls - 2 * b, colorbar(c), 0.5, '#333333');
    var text = svgtext(box, tx0 + ls + b1, ty0 + SVG_FONT_SIZE, name + ': ' + formatKMGselect(v1, unit) + ' ' + sprintf('(%.1f\%)', 100 * v1 / ss), SVG_FONT_SIZE, 0, FONT_COLOR, "start");
    var len = text.getComputedTextLength() + ls + b1;
    if(len > mm) mm = len;
    ty0 += ls;
    s0 += dd[i][0];
    info[5].push(text);
  }
  return info;
}

function svggraphboxwhisker(box, x0, y0, x1, y1, xrange, yrange, data, linecolor, linewidth, fillcolor, fill, outliercolor) {
  linecolor = isundefined(linecolor, '#000000');
  linewidth = isundefined(linewidth, 1);
  fillcolor = isundefined(fillcolor, '#ff0000');
  fill = isundefined(fill, true);
  outliercolor = isundefined(outliercolor, '#ffff00');

  var n = data.length;
  if(n == 0) return;
  var w = (3 * (x1 - x0)) / (8 * n);
  var nol = Math.floor((2 * w - 1) / (2 * OUTLIER_R));
  var cnol = Math.floor((nol + 1) / 2);
  var noli = new Array();
  for(var i = 0; i < nol; i++) {
    var j = i % 2;
    var k = Math.floor((i + 1) / 2);
    noli.push(j * k - (1 - j) * k);
  }

  for(var i = 0; i < n; i++) {
    var val = data[i];
    if(kisNaN(val[1]) || kisNaN(val[5])) continue;
   
    var px;
    px = (val[0] - xrange[0]) / (xrange[1] - xrange[0]);
    px = x0 * (1 - px) + x1 * px;
    if(px > x1) break;
   
    var py0 = (val[1] - yrange[0]) / (yrange[1] - yrange[0]);
    py0 = py0 * (y1 - y0) + y0;
    var py25 = (val[2] - yrange[0]) / (yrange[1] - yrange[0]);
    py25 = py25 * (y1 - y0) + y0;
    var py50 = (val[3] - yrange[0]) / (yrange[1] - yrange[0]);
    py50 = py50 * (y1 - y0) + y0;
    var py75 = (val[4] - yrange[0]) / (yrange[1] - yrange[0]);
    py75 = py75 * (y1 - y0) + y0;
    var py100 = (val[5] - yrange[0]) / (yrange[1] - yrange[0]);
    py100 = py100 * (y1 - y0) + y0;
   
    var acc = w;
    if(ACC_AS_WIDTH) acc = 1 + (acc * (1 - (val[6] / 100)));
    var pxmw = Math.max(x0, px - acc);
    var pxmw2 = Math.max(x0, px - 3 * w / 7);
    var pxpw = Math.min(x1, px + acc);
    var pxpw2 = Math.min(x1, px + 3 * w / 7);
   
    if(((py25 >= y1) && (py25 <= y0)) || ((py75 >= y1) && (py75 <= y0)) || ((py25 > y1) && (py75 < y0))) {
      var t = Math.max(y1, Math.min(py75, y0));
      var b = Math.max(y1, Math.min(py25, y0));
      if(ACC_AS_FILL && fill) {
        svgrect(box, pxmw, b, pxpw - pxmw, t - b, fillcolor, 0, 0, 1 - (val[6] / 100));
      }
      
      svgline(box, pxmw, t, pxmw, b, linecolor, linewidth);
      svgline(box, pxpw, t, pxpw, b, linecolor, linewidth);
    }
   
    if((py0 >= y1) && (py0 <= y0)) svgline(box, pxmw2, py0, pxpw2, py0, linecolor, linewidth);
    if((py25 >= y1) && (py25 <= y0)) svgline(box, pxmw, py25, pxpw, py25, linecolor, linewidth);
    if((py50 >= y1) && (py50 <= y0)) svgline(box, pxmw, py50, pxpw, py50, linecolor, linewidth);
    if((py75 >= y1) && (py75 <= y0)) svgline(box, pxmw, py75, pxpw, py75, linecolor, linewidth);
    if((py100 >= y1) && (py100 <= y0)) svgline(box, pxmw2, py100, pxpw2, py100, linecolor, linewidth);
    
    if(((py0 >= y1) && (py0 <= y0)) || ((py25 >= y1) && (py25 <= y0))) {
      svgline(box, px, Math.max(y1, Math.min(py0, y0)), px, Math.max(y1, Math.min(py25, y0)), linecolor, linewidth);
    }
    if(((py75 >= y1) && (py75 <= y0)) || ((py100 >= y1) && (py100 <= y0))) {
      svgline(box, px, Math.max(y1, Math.min(py100, y0)), px, Math.max(y1, Math.min(py75, y0)), linecolor, linewidth);
    }
    
    if(val.length > 7) {
      var oy = new Array();
      for(var k = 0; k < nol; k++) {
        oy.push(new Array());
      }
      for(var j = 7; j < val.length; j++) {
        var py = (val[j][1] - yrange[0]) / (yrange[1] - yrange[0]);
        py = py * (y1 - y0) + y0;
        if((py >= y1) && (py <= y0)) {
          for(var k = 0; k < nol; k++) {
            var good = true;
            for(var l = 0; good && (l < oy[k].length); l++) {
              if(Math.abs(oy[k][l][1] - py) < (2 * OUTLIER_R + 1)) good = false;
            }
            if(good) {
              oy[k].push([j, py]);
              break;
            }
          }
        }
      }
      var ox = 0;
      var no = 0;
      for(var k = 0; k < nol; k++) {
        ox += noli[k] * oy[k].length;
        no += oy[k].length;
      }
      ox = 2 * (w - OUTLIER_R) * ox / (no * nol);
      for(var k = 0; k < nol; k++) {
        for(var l = 0; l < oy[k].length; l++) {
          var xx = px - ox + noli[k] * 2 * (w - OUTLIER_R) / nol;
          var index = oy[k][l][0];
          var yy = oy[k][l][1];
          svgcircle(box, xx, yy, OUTLIER_R, outliercolor);
          val[index].push(xx);
          val[index].push(yy);
        }
      }
    }
  }
}

function svggraphaverage(box, x0, y0, x1, y1, xrange, yrange, data, gradient, linecolor) {
  var n = 1;
  if(data.length > 0) n = data[0].length;
  for(var i = 1; i < n - 1; i++) {
    svggraphoneaverage(box, x0, y0, x1, y1, xrange, yrange, data, gradient, i);
  }
}

function svggraphoneaverage(box, x0, y0, x1, y1, xrange, yrange, data, gradient, index, s) {
  var n = data.length;
  var px0 = x0;
  s = isundefined(s, 0);
  var pval;
  while(s < n) {
    pval = data[s];
    pval = pval[index];
    if(!kisNaN(pval)) break;
    s++;
  }
  if(s >= n) return;
/*    while(s < n) {
      if(data[s][0] >= xrange[0]) break;
      s++;
    }
    if(s >= n) return;*/
  px0 = (data[s][0] - xrange[0]) / (xrange[1] - xrange[0]);
  px0 = x0 * (1 - px0) + x1 * px0;
  if(s >= n) return;

  var py0 = Math.max(yrange[0], Math.min(yrange[1], pval));
  var pi0 = (py0 == pval);
  py0 = (py0 - yrange[0]) / (yrange[1] - yrange[0]);
  py0 = py0 * (y1 - y0) + y0;

  var z0;
  if((yrange[0] < 0) && (yrange[1] > 0)) {
    z0 = 0;
  } else if(yrange[0] >= 0) {
    z0 = yrange[0];
  } else {
    z0 = yrange[1];
  }
  z0 = (z0 - yrange[0]) / (yrange[1] - yrange[0]);
  z0 = z0 * (y1 - y0) + y0;

  var ox = px0;
  var oy = py0;
  p = "M" + px0 + " " + py0 + " ";

  var last = n;
  var pxK = x1;
  for(var i = s; i < n; i++) {
    var val = data[i];
    var px1;
    px1 = (val[0] - xrange[0]) / (xrange[1] - xrange[0]);
    px1 = x0 * (1 - px1) + x1 * px1;
    val = val[index];
    if(px1 > x1) break;
    if(kisNaN(val)) {
      last = i;
      break;
    }
    pxK = px1;
    var py1 = Math.max(yrange[0], Math.min(yrange[1], val));
    var pi1 = (py1 == val);
    py1 = (py1 - yrange[0]) / (yrange[1] - yrange[0]);
    py1 = py1 * (y1 - y0) + y0;

    if(px1 < px0) {
      println("Timestamp: " + i + " is before previous");
      printall(data);
    }

    if(pi0 && pi1) {
      p += "L" + px1 + " " + py1 + " ";
    } else if(!pi0 && pi1) {
      if(pval > yrange[1]) {
        var q = (yrange[1] - val) / (pval - val);
        q = (px0 * q) + px1 * (1 - q);
        p += "L" + q + " " + py0 + " L" + px1 + " " + py1 + " ";
      } else {
        var q = (yrange[0] - val) / (pval - val);
        q = (px0 * q) + px1 * (1 - q);
        p += "L" + q + " " + py0 + " L" + px1 + " " + py1 + " ";
      }
    } else if(pi0 && !pi1) {
      if(val > yrange[1]) {
        var q = (yrange[1] - val) / (pval - val);
        q = (px1 * q) + px0 * (1 - q);
        p += "L" + q + " " + py1 + " L" + px1 + " " + py1 + " ";
      } else {
        var q = (yrange[0] - val) / (pval - val);
        q = (px1 * q) + px0 * (1 - q);
        p += "L" + q + " " + py1 + " L" + px1 + " " + py1 + " ";
      }
    } else {
      p += "L" + px1 + " " + py1 + " ";
    }
    pval = val;
    px0 = px1;
    py0 = py1;
    pi0 = pi1;
  }

  for(var i = last - 1; i >= s; i--) {
    var val = data[i];
    var px1;
    px1 = (val[0] - xrange[0]) / (xrange[1] - xrange[0]);
    px1 = x0 * (1 - px1) + x1 * px1;
    val = val[index + 1];
    if(px1 > x1) break;
    pxK = px1;
    var py1 = Math.max(yrange[0], Math.min(yrange[1], val));
    var pi1 = (py1 == val);
    py1 = (py1 - yrange[0]) / (yrange[1] - yrange[0]);
    py1 = py1 * (y1 - y0) + y0;

    if(pi0 && pi1) {
      p += "L" + px1 + " " + py1 + " ";
    } else if(!pi0 && pi1) {
      if(pval > yrange[1]) {
        var q = (yrange[1] - val) / (pval - val);
        q = (px0 * q) + px1 * (1 - q);
        p += "L" + q + " " + py0 + " L" + px1 + " " + py1 + " ";
      } else {
        var q = (yrange[0] - val) / (pval - val);
        q = (px0 * q) + px1 * (1 - q);
        p += "L" + q + " " + py0 + " L" + px1 + " " + py1 + " ";
      }
    } else if(pi0 && !pi1) {
      if(val > yrange[1]) {
        var q = (yrange[1] - val) / (pval - val);
        q = (px1 * q) + px0 * (1 - q);
        p += "L" + q + " " + py1 + " L" + px1 + " " + py1 + " ";
      } else {
        var q = (yrange[0] - val) / (pval - val);
        q = (px1 * q) + px0 * (1 - q);
        p += "L" + q + " " + py1 + " L" + px1 + " " + py1 + " ";
      }
    } else {
      p += "L" + px1 + " " + py1 + " ";
    }
    pval = val;
    px0 = px1;
    py0 = py1;
    pi0 = pi1;
  }

  p += "L " + ox  + " " + oy + " Z";

  var path = document.createElementNS(XUL_SVG, "svg:path");
  path.setAttribute("fill", 'url(#fill' + (gradient + index) + ')');
  path.setAttribute("d", p);
  box.appendChild(path);

  if(last < n) {
    svggraphoneaverage(box, x0, y0, x1, y1, xrange, yrange, data, gradient, index, last);
  }
}

function svggraphline(box, x0, y0, x1, y1, xrange, yrange, data, linecolor, fillcolor, linewidth, s) {
  linecolor = isundefined(linecolor, "#ffffff");
  fillcolor = isundefined(fillcolor, "none");

  var n = data.length;
  var px0 = x0;
  s = isundefined(s, 0);
  var pval;
  while(s < n) {
    pval = data[s];
    if(isArray(pval)) pval = pval[1];
    if(!kisNaN(pval)) break;
    s++;
  }
  if(s >= n) return;
  if(isArray(data[s])) {
/*    while(s < n) {
      if(data[s][0] >= xrange[0]) break;
      s++;
    }
    if(s >= n) return;*/
    px0 = (data[s][0] - xrange[0]) / (xrange[1] - xrange[0]);
    px0 = x0 * (1 - px0) + x1 * px0;
  } else {
    px0 = (x0 * (n - 1 - s) + x1 * s) / (n - 1);
  }
//   if(s >= n) return;
  var py0 = Math.max(yrange[0], Math.min(yrange[1], pval));
  var pi0 = (py0 == pval);
  py0 = (py0 - yrange[0]) / (yrange[1] - yrange[0]);
  py0 = py0 * (y1 - y0) + y0;

  var z0;
  if((yrange[0] < 0) && (yrange[1] > 0)) {
    z0 = 0;
  } else if(yrange[0] >= 0) {
    z0 = yrange[0];
  } else {
    z0 = yrange[1];
  }
  z0 = (z0 - yrange[0]) / (yrange[1] - yrange[0]);
  z0 = z0 * (y1 - y0) + y0;

  var p;
  if(fillcolor == "none") {
    p = "M" + px0 + " " + py0 + " ";
  } else {
    p = "M" + px0 + " " + z0 + " L" + px0 + " " + py0 + " ";
  }

  var last = n;
  var pxK = x1;
  for(var i = s; i < n; i++) {
    var val = data[i];
    var px1;
    if(isArray(val)) {
      px1 = (val[0] - xrange[0]) / (xrange[1] - xrange[0]);
      px1 = x0 * (1 - px1) + x1 * px1;
      val = val[1];
    } else {
      px1 = (x0 * (n - 1 - i) + x1 * i) / (n - 1);
    }
    if(px1 > x1) break;
    if(kisNaN(val)) {
      last = i;
      break;
    }
    pxK = px1;
    var py1 = Math.max(yrange[0], Math.min(yrange[1], val));
    var pi1 = (py1 == val);
    py1 = (py1 - yrange[0]) / (yrange[1] - yrange[0]);
    py1 = py1 * (y1 - y0) + y0;

    if(px1 < px0) {
      println("Timestamp: " + i + " is before previous");
      printall(data);
    }

    if(pi0 && pi1) {
      p += "L" + px1 + " " + py1 + " ";
    } else if(!pi0 && pi1) {
      if(pval > yrange[1]) {
        var q = (yrange[1] - val) / (pval - val);
        q = (px0 * q) + px1 * (1 - q);
        p += "L" + q + " " + py0 + " L" + px1 + " " + py1 + " ";
      } else {
        var q = (yrange[0] - val) / (pval - val);
        q = (px0 * q) + px1 * (1 - q);
        p += "L" + q + " " + py0 + " L" + px1 + " " + py1 + " ";
      }
    } else if(pi0 && !pi1) {
      if(val > yrange[1]) {
        var q = (yrange[1] - val) / (pval - val);
        q = (px1 * q) + px0 * (1 - q);
        p += "L" + q + " " + py1 + " L" + px1 + " " + py1 + " ";
      } else {
        var q = (yrange[0] - val) / (pval - val);
        q = (px1 * q) + px0 * (1 - q);
        p += "L" + q + " " + py1 + " L" + px1 + " " + py1 + " ";
      }
    } else {
      p += "L" + px1 + " " + py1 + " ";
    }
    pval = val;
    px0 = px1;
    py0 = py1;
    pi0 = pi1;
  }

  if(fillcolor != "none") p += "L " + pxK  + " " + z0 + " Z";

  var path = document.createElementNS(XUL_SVG, "svg:path");
  path.setAttribute("fill", fillcolor);
  if(linecolor != "none") path.setAttribute("style", "stroke:" + linecolor + ";stroke-width:" + linewidth + ";");
  path.setAttribute("d", p);
  box.appendChild(path);

  if(last < n) {
    svggraphline(box, x0, y0, x1, y1, xrange, yrange, data, linecolor, fillcolor, linewidth, last);
  }
}

function svggraphbar(box, x0, y0, x1, y1, xrange, yrange, data, linecolor, fillcolor, linewidth, bar, bars, sum) {
  linecolor = isundefined(linecolor, "#ffffff");
  fillcolor = isundefined(fillcolor, "none");

  var n = Math.max(data.length, sum.length);
  var bw = (x1 - x0) / n;
  var bw0 = bw * 0.80;
  var bw2 = (bw0 / bars);
  var bw1 = (bw - bw0) / 2 + bw2 * (Math.min(bar, bars) - 1);
  var h = y1 - y0;
  var sc = yrange[1] - yrange[0]; // nagative doesn't work
 
  for(var i = 0; i < n; i++) {
    var pval = data[i];
    if(isArray(pval)) pval = pval[1];
    if(kisNaN(pval)) continue;
    
    var x = x0 + i * bw + bw1;
    var y = 0;
    var yt = pval;
    if(i < sum.length) {
      y += sum[i];
      yt += sum[i];
    }
    var d = y - yrange[0];
    var dt = yt - yrange[0];
    if((d <= sc) && (dt >= 0)) {
      d = Math.max(d, 0);
      dt = Math.min(dt, sc);
      y = y0 + d * h / sc;
      yt = y0 + dt * h / sc;
      svgrect(box, x, y, bw2, yt - y, fillcolor, linewidth, linecolor);
    }    
    if(i < sum.length) sum[i] += pval;
  }
}

function svggraphblock(box, x0, y0, x1, y1, xrange, yrange, data, c, count, colormax, opacity, greenred) {
  colormax = isundefined(colormax, 0);
  opacity = isundefined(opacity, '0.30');
  greenred = isundefined(greenred, false);

  var n = data.length;
  var px0 = x0;
  var pval;
  var s = 0;
  while(s < n) {
    pval = data[s];
    if(isArray(pval)) pval = pval[1];
    if(!kisNaN(pval)) break;
    s++;
  }
  if(s >= n) return;

  if(isArray(data[s])) {
    px0 = (data[s][0] - xrange[0]) / (xrange[1] - xrange[0]);
    px0 = x0 * (1 - px0) + x1 * px0;
  } else {
    px0 = (x0 * (n - 1 - s) + x1 * s) / (n - 1);
  }
  if(s >= n) return;

  var py0 = c / count;
  py0 = py0 * (y1 - y0) + y0;
  var h = (y1 - y0) / count;

  var px2 = px0;
  for(var i = 1; i < n; i++) {
    var val = data[i];
    var px1;
    if(isArray(val)) {
      px1 = (val[0] - xrange[0]) / (xrange[1] - xrange[0]);
      px1 = x0 * (1 - px1) + x1 * px1;
      val = val[1];
    } else {
      px1 = (x0 * (n - 1 - i) + x1 * i) / (n - 1);
    }

    if(kisNaN(val)) {
      if(kisNaN(pval)) {
        px0 = px1;
      } else {
        var color;
        if(greenred) {
          color = colorbar(0.5 + pval / (2.0 * colormax));
        } else if(colormax) {
          if(isArray(colormax)) {
            color = colormax[Math.floor(pval + 0.5)];
          } else {
            color = colorbar(pval / colormax);
          }
        } else {
          color = colorbar(0.5 + pval / 4.0);
        }
        svgrect(box, px0, py0, px2 - px0, h, color, 0, 0, opacity);
        px0 = px2;
      }
    } else if(kisNaN(pval)) {
      px0 = px1;
      px2 = px1;
    } else {
      var pa = (px2 + px1) / 2;
      var color;
      if(greenred) {
        color = colorbar(0.5 + pval / (2.0 * colormax));
      } else if(colormax) {
        if(isArray(colormax)) {
          color = colormax[Math.floor(pval + 0.5)];
        } else {
          color = colorbar(pval / colormax);
        }
      } else {
        color = colorbar(0.5 + pval / 4.0);
      }
      svgrect(box, px0, py0, pa - px0, h, color, 0, 0, opacity);
      px2 = px1;
      px0 = pa;
    }
    pval = val;
  }

  if(!kisNaN(pval)) {
    var pa = px1;
    var color;
    if(greenred) {
      color = colorbar(0.5 + pval / (2.0 * colormax));
    } else if(colormax) {
      if(isArray(colormax)) {
        color = colormax[Math.floor(pval + 0.5)];
      } else {
        color = colorbar(pval / colormax);
      }
    } else {
      color = colorbar(0.5 + pval / 4.0);
    }
    svgrect(box, px0, py0, pa - px0, h, color, 0, 0, opacity);
  }
}

function autoIntAxis(range, unit) {
  var r = new Array();
  if(unit == "time") {
    var low = range[0];
    var high = range[1];
    var diff = high - low;
    if(diff > 2 * 365 * 86400) { // Years
    } else {
                // second          minutes                        hour                               // Day
      var ds = [1,2,5,10,20,30, 1*60,2*60,5*60,10*60,20*60,30*60, 3600,2*3600,4*3600,6*3600,12*3600, 86400,2*86400,5*86400,10*86400,30*86400,60*86400,120*86400];
      var d;
      for(var i = 0; i < ds.length; i++) {
        if((diff / ds[i]) < 15) {
          d = ds[i];
          break;
        }
      }
      for(var i = Math.ceil(low / d) * d; i <= high; i += d) r.push([i]);
    }

    if(r.length >= 2) {
      var dd = new Date();
      dd.setTime(r[0][0] * 1000);
      r[0][1] = dd.format("dd/mmm/yyyy HH:MM:ss");

      var dd = new Date();
      dd.setTime(r[r.length - 1][0] * 1000);
      r[r.length - 1][1] = dd.format("dd/mmm/yyyy HH:MM:ss");
    } else {
      r[0] = [0, 'No data'];
      r[1] = [1];
    }
  } else {
    var mul = 1;
    var low = range[0];
    var high = range[1];
    if(unit == 'ms' && ((Math.abs(high) > 2.1 * Math.pow(10, 3)) || (Math.abs(low) > 2.1 * Math.pow(10, 3)))) {
      low /= 1000;
      high /= 1000;
      unit = 's';
    }
    if((unit == 'B') || (unit == 'B/s')) {
      if((Math.abs(high) > 2.1 * Math.pow(10, 15)) || (Math.abs(low) > 2.1 * Math.pow(10, 15))) {
        mul = Math.pow(2, 50);
        unit = 'Pi' + unit;
      } else if((Math.abs(high) > 2.1 * Math.pow(10, 12)) || (Math.abs(low) > 2.1 * Math.pow(10, 12))) {
        mul = Math.pow(2, 40);
        unit = 'Ti' + unit;
      } else if((Math.abs(high) > 2.1 * Math.pow(10, 9)) || (Math.abs(low) > 2.1 * Math.pow(10, 9))) {
        mul = Math.pow(2, 30);
        unit = 'Gi' + unit;
      } else if((Math.abs(high) > 2.1 * Math.pow(10, 6)) || (Math.abs(low) > 2.1 * Math.pow(10, 6))) {
        mul = Math.pow(2, 20);
        unit = 'Mi' + unit;
      } else if((Math.abs(high) > 2.1 * Math.pow(10, 3)) || (Math.abs(low) > 2.1 * Math.pow(10, 3))) {
        mul = Math.pow(2, 10);
        unit = 'Ki' + unit;
      }
    } else {
      if((Math.abs(high) > 2.1 * Math.pow(10, 15)) || (Math.abs(low) > 2.1 * Math.pow(10, 15))) {
        mul = Math.pow(10, 15);
        unit = 'P' + unit;
      } else if((Math.abs(high) > 2.1 * Math.pow(10, 12)) || (Math.abs(low) > 2.1 * Math.pow(10, 12))) {
        mul = Math.pow(10, 12);
        unit = 'T' + unit;
      } else if((Math.abs(high) > 2.1 * Math.pow(10, 9)) || (Math.abs(low) > 2.1 * Math.pow(10, 9))) {
        mul = Math.pow(10, 9);
        unit = 'G' + unit;
      } else if((Math.abs(high) > 2.1 * Math.pow(10, 6)) || (Math.abs(low) > 2.1 * Math.pow(10, 6))) {
        mul = Math.pow(10, 6);
        unit = 'M' + unit;
      } else if((Math.abs(high) > 2.1 * Math.pow(10, 3)) || (Math.abs(low) > 2.1 * Math.pow(10, 3))) {
        mul = Math.pow(10, 3);
        unit = 'K' + unit;
      }
    }
    low /= mul;
    high /= mul;
    
    var d = high - low;
    var s = Math.floor(Math.log(d) / Math.log(10));
    var sp = Math.pow(10, s);
    if(Math.abs(sp - d) < EPS) {
      s--;
      sp/=10;
    }
    var ac = Math.floor(((d / sp) - 1) / 5) + 1;
    ac *= sp;
    var n = Math.ceil(d / ac - EPS);
    var k = Math.floor(10 / n);
    var lv = low;

    if(high < lv + 2 * ac) {
      ac /= 2;
      n *= 2;
      k = Math.floor(10 / n);
    }

    if(low > 0) {
      var v = low % ac;
      if(v > EPS) {
        lv = low - v;
        if(v > ac / k) n++;
      }
    } else if(low < 0) {
      var v = (-low) % ac;
      if(v > EPS) {
        lv = low - ac + v;
        if(v > ac / k) n++;
      }
    }

    for(var i = 0; i < n; i++) {
      var v = lv + i * ac;
      if(Math.abs(v * mul) < EPS) {
        r.push([0, "0" + unit]);
      } else {
        var s = Math.floor(Math.log(Math.abs(v)) / Math.log(10));
        r.push([mul * v, sprintf("%." + Math.max(0, -s + 2) + "f", v) + unit]);
      }
      for(var j = 1; j < k; j++) {
        var w = v + (j * ac / k);
        r.push([mul * w]);
      }
    }
    if(high - (lv + n * ac) > -EPS) {
      var v = lv + n * ac;
      if(Math.abs(v * mul) < EPS) {
        r.push([0, "0" + unit]);
      } else {
        var s = Math.floor(Math.log(Math.abs(v)) / Math.log(10));
        r.push([mul * v, sprintf("%." + Math.max(0, -s + 2) + "f", v) + unit]);
      }
    }
  }
  return r;
}

function svgxaxis(box, x0, y0, x1, y1, s, xrange, x, x2) {
  x2 = isundefined(x2, x1);
  for(var i = 0; i < x.length; i++) {
    if((x[i][0] >= xrange[0]) && (x[i][0] <= xrange[1])) {
      var xx = (x[i][0] - xrange[0]) / (xrange[1] - xrange[0]);
      xx = xx * (x1 - x0) + x0;
      if(x[i].length == 1) {
        svgline(box, xx, y0, xx, y1, "#d9d9d9", 0.5);
      } else {
        svgline(box, xx, y0, xx, y1, "#d0d0d0", 1);
        var text = svgtext(box, xx, y0 + 3 * SVG_FONT_SIZE / 2, x[i][1], SVG_FONT_SIZE, 0, FONT_COLOR, "middle");
        var len = text.getComputedTextLength() / 2;
        if((xx - len) < 0) {
          box.removeChild(text);
          text = svgtext(box, 0, y0 + 3 * SVG_FONT_SIZE / 2, x[i][1], SVG_FONT_SIZE, 0, FONT_COLOR, "start");
        } else if((xx + len) > x2) {
          box.removeChild(text);
          text = svgtext(box, x2, y0 + 3 * SVG_FONT_SIZE / 2, x[i][1], SVG_FONT_SIZE, 0, FONT_COLOR, "end");
        } else {
          box.appendChild(text);
        }
      }
    }
  }
}

function svgyaxis(box, x0, y0, x1, y1, s, yrange, y) {
  for(var i = 0; i < y.length; i++) {
    if((y[i][0] >= yrange[0]) && (y[i][0] <= yrange[1])) {
      var yy = (y[i][0] - yrange[0]) / (yrange[1] - yrange[0]);
      yy = yy * (y1 - y0) + y0;
      if(y[i].length == 1) {
        svgline(box, x0, yy, x1, yy, "#d9d9d9", 0.5);
      } else {
        svgline(box, x0, yy, x1, yy, "#d0d0d0", 1);
        var text = svgtext(box, x0 - 3 , yy + (SVG_FONT_SIZE / 2), y[i][1], SVG_FONT_SIZE, 0, FONT_COLOR, "end");
//         var len = text.getComputedTextLength();
//         if(len > 36) {
//           box.removeChild(text);
//           svgtext(box, x0 - 36, yy + (SVG_FONT_SIZE / 2), y[i][1], SVG_FONT_SIZE, 0, FONT_COLOR, "start");
//         }
      }
    }
  }
}

function svgimage(box, x, y, w, h, src) {
  var rect = document.createElementNS(XUL_SVG, "svg:image");
  rect.setAttribute("x", x);
  rect.setAttribute("y", y);
  rect.setAttribute("width", w);
  rect.setAttribute("height", h);
  rect.setAttributeNS(XUL_XLINK, "xlink:href", src);
  box.appendChild(rect);
  return rect;
}

function svguse(box, x, y, w, h, src) {
  var rect = document.createElementNS(XUL_SVG, "svg:use");
  rect.setAttribute("x", x);
  rect.setAttribute("y", y);
  rect.setAttribute("width", w);
  rect.setAttribute("height", h);
  rect.setAttributeNS(XUL_XLINK, "xlink:href", src);
  box.appendChild(rect);
  return rect;
}

function svgrect(box, x, y, w, h, color, border, bordercolor, opacity) {
  color = isundefined(color, "none");
  border = isundefined(border, 0);
  opacity = isundefined(opacity, 0);
  bordercolor = isundefined(bordercolor, "#f4f4f4");

  var rect = document.createElementNS(XUL_SVG, "svg:rect");
  rect.setAttribute("fill", color);
  if(border > 0) {
    if(opacity) {
      rect.setAttribute("style", "stroke:" + bordercolor + "; stroke-width:" + border + "; fill-opacity:" + opacity);
    } else {
      rect.setAttribute("style", "stroke:" + bordercolor + "; stroke-width:" + border + ";");
    }
  } else if(opacity) {
    rect.setAttribute("style", "fill-opacity:" + opacity);
  }
  if(w < 0) {
    rect.setAttribute("x", x + w);
    rect.setAttribute("width", -w);
  } else {
    rect.setAttribute("x", x);
    rect.setAttribute("width", w);
  }
  if(h < 0) {
    rect.setAttribute("y", y + h);
    rect.setAttribute("height", -h);
  } else {
    rect.setAttribute("y", y);
    rect.setAttribute("height", h);
  }
  box.appendChild(rect);
  return rect;
}

function svgtext(box, x, y, txt, fontsize, angle, color, placement) {
  fontsize = isundefined(fontsize, "12");
  angle = isundefined(angle, 0);
  color = isundefined(color, "black");
  placement = isundefined(placement, "start");

  var text = document.createElementNS(XUL_SVG, "svg:text");
  text.setAttribute("font-size", fontsize);
  text.setAttribute("text-anchor", placement);
  text.setAttribute("fill", color);
  if(angle) text.setAttribute("transform", "rotate(" + angle + ")");
  text.setAttribute("x", x);
  text.setAttribute("y", y);
  text.appendChild(document.createTextNode(txt));
  box.appendChild(text);
  return text;
}

function svgreplacetext(obj, txt, color) {
  obj.removeChild(obj.firstChild);
  obj.appendChild(document.createTextNode(txt));
  if(typeof(color) != 'undefined') obj.setAttribute("fill", color);
}

function svgline(box, x0, y0, x1, y1, color, thickness) {
  color = isundefined(color, "black");
  thickness = isundefined(thickness, "1");
  var line = document.createElementNS(XUL_SVG, "svg:line");
  line.setAttribute("style", "stroke:" + color + ";stroke-width:" + thickness + ";");
  line.setAttribute("x1", x0);
  line.setAttribute("y1", y0);
  line.setAttribute("x2", x1);
  line.setAttribute("y2", y1);
  box.appendChild(line);
  return line;
}

function svgcircle(box, cx, cy, r, color, border, thickness) {
  color = isundefined(color, '#000000');  
  border = isundefined(border, 'none');
  thickness = isundefined(thickness, 1);
  var circle = document.createElementNS(XUL_SVG, "svg:circle");
  circle.setAttribute("cx", cx);
  circle.setAttribute("cy", cy);
  circle.setAttribute("r", r);
  circle.setAttribute("fill", color);
  circle.setAttribute("style", "stroke:" + border + "; stroke-width:" + thickness + ";");
  box.appendChild(circle);
  return circle;
}
