/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'Switch';
var singular = 'switch';
var plural = 'switches';

function createTreeItem(one) {
  var item = new Object();
  item['key'] = one.get('uniqueKey');
  item['modified'] = one.get('modified');
  item['childType'] = one.get('childType');
  item['hostname'] = one.get('hostname');
  item['mac'] = one.get('mac');
  item['tag'] = one.get('tag');
  item['ip'] = one.get('ip');
  item['power'] = '';
  var pdu = one.get('powerDistributionUnits');
  for(var i = 0; i < pdu.length; i++) {
    if(item['power'].length > 0) item['power'] += ', ';
    item['power'] += one.cluster.resolveName(pdu[i].pdu) + ':' + pdu[i].prt;
  }
  item['members'] = current.resolveList(one.get('members'));
  return item;
}

function addObject() {
  var args = new Object();
  args['title'] = 'Add new ' + singular;
  args['question'] = 'Hostname';
  args['default'] = 'switch';
  args['known'] = current.known('Device', 'hostname');
  args['children'] = ['Ethernet Switch', 'IB Switch', 'Myrinet Switch'];
  var d = new dock.cvDialog("newchildobject.xul", args);
  d.onOk = function(name, type) {
    var object;
    switch(type) {
      case('Ethernet Switch'): object = new dock.cvEthernetSwitch(); break;
      case('IB Switch'): object = new dock.cvIBSwitch(); break;
      case('Myrinet Switch'): object = new dock.cvMyrinetSwitch(); break;
      default:
        dock.displayError('Ilegal object type: ' + type);
        return;
    }
    object.hostname = name;
    var base = current.getBasePartition();
    if(isObject(base)) {
      object.partition = base.object.uniqueKey;
    }
    current.addObject(object, dock.openAfterCreate);
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function cloneObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a " + singular + " for cloning");
  } else if(select.length > 1) {
    dock.displayError("Please select only one " + singular);
  } else {
    var original = current.find(select[0]);
    if(isObject(original)) {
      var args = new Object();
      var host = original.get('hostname');
      args['title'] = 'Clone ' + host;
      args['question'] = 'Hostname';
      var nn = nameNumber(host);
      args['default'] = nn[0];
      args['known'] = current.known('Device', 'hostname');
      var d = new dock.cvDialog("newobject.xul", args);
      d.onOk = function(name, type) {
        var object = original.clone();
        object.modified = true;
        object.toBeRemoved = false;
        object.hostname = name;
        object.tag = "00000000a000";
        object.mac = "00:00:00:00:00:00";
        object.ip = "0.0.0.0";
        object.ethernetSwitch = new dock.cvSwitchPort();
        object.powerDistributionUnits = new Array();
        cloneSimpleIP(original.object, object);
        current.addObject(object, dock.openAfterCreate);
        refresh();
      }
      dock.dialogs.showDialog(d);
    } else {
      dock.displayError("Original not found, already removed by someone else");
    }
  }
}
