/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var myDialog;
var cluster;
var current;
var devices;

var graphs = new Array();
var timer;
var zoombox = 0;
var saveFile = '';
var waitCount = 0;
var broadcast;
var sizes;
var nodeTab;
var hasNode = new Object();
var defaultNodes = new Array();

var ENABLE_TOOLTIP = false;
var MAX_HEIGHT = 64;

var flags_type = 31;
var flags_fill = 32;

function onLoad() {
  dock = getDock();
  println("Monitoring initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];
  cluster = myDialog.arguments.cluster;
  current = myDialog.arguments.current;
  devices = myDialog.arguments.devices;  
  
  waitIcon();
  document.getElementById('titlename').setAttribute('value', cluster.name);

  sizes = new Array();
  graphs = new Array();
  broadcast = new Array();
  
  addEventListener("resize", redraw, false);
  
  if(current.get('baseType') == 'HadoopHDFS') {
    hadoopOverview();
  } else {
    println("Unknown tabbed graph: " + current.get('baseType'));
  }
  
  unwaitIcon();
  if(myDialog.onOpened) myDialog.onOpened();
}

function newPage(w, h, name) {
  var page = graphs.length;
  sizes.push([w, h]);
  broadcast.push(-1);

  var tabs = document.getElementById('tabs');
  var panels = document.getElementById('tabpanels');
  var tab = document.createElement('shadowtab');
  tab.setAttribute('id', 'tab' + page);
  tab.setAttribute('label', name);
  tabs.appendChild(tab);
  var panel = document.createElement('vbox');
  panel.setAttribute('id', 'graphbox' + page);
  panel.setAttribute('equalsize', 'always');
  panel.setAttribute('flex', '1');
  panels.appendChild(panel);      
  
  graphs.push(new Array());
  for(var i = 0; i < sizes[page][0] * sizes[page][1]; i++) {
    var one = new cvGraph(cluster);
    one.ownerWindow = this;
    graphs[page].push(one);
  }
  setSize(page, sizes[page][0], sizes[page][1]);  
  buildbox(page);
  
  if(tabs.selectedIndex < 0) tabs.selectedIndex = 0;
  return page;
}

function createMetricPage(page, metrics, period, deviceKey) {
  period = isundefined(period, dock.generalSettings.timespan);
  deviceKey = isundefined(deviceKey, 0);
  var key;
  var graphDevices;
  if(deviceKey) {
    key = deviceKey
  } else {
    key = current.get('uniqueKey');
    graphDevices = devices;
  }
  for(var i = 0; i < metrics.length; i++) {
    for(var j = 1; j < metrics[i].length; j++) {
      var name = metrics[i][j][0];
      var idx = name.indexOf(':');
      var param = '';
      if(idx >= 0) {
        param = name.substr(idx + 1);
        name = name.substr(0, idx);
      }
      var metric = cluster.find('Metric', name);
      if(!metric) { // TODO: bad
        metric = cluster.find('Metric', 'rnd');
        param = '' + j;
      }
      if(metric) {
        var dmp = new cvGraphDeviceMetric(key, metric.get('uniqueKey'), param, graphDevices);
        var flags = metrics[i][j][1];
        dmp.groupType = flags & flags_type;
        dmp.fill = ((flags & flags_fill) == flags_fill);
        var r = graphs[page][i].add(dmp);
      }
    }
    graphs[page][i].xzoom = [[period, 0]];
    graphs[page][i].title = metrics[i][0];
    graphs[page][i].allowRedraw(true);      
    replaceOne(page, i);
  }
}

function createPage(w, h, name, metrics, period, deviceKey) {
  var page = newPage(w, h, name);
  createMetricPage(page, metrics, period, deviceKey);
  return page;
}

function stopAll() {
  for(var j = 0; j < graphs.length; j++) {
    for(var i = 0; i < graphs[j].length; i++) {
      if(graphs[j][i].deviceMetric.length > 0) {
        graphs[j][i].stop();
        document.getElementById('play' + j + '_' + i).setAttribute('state', 'play');
      }
    }
  }
}
function startAll() {
  for(var j = 0; j < graphs.length; j++) {
    for(var i = 0; i < graphs[j].length; i++) {
      if(graphs[j][i].deviceMetric.length > 0) {
        graphs[j][i].start();
        document.getElementById('play' + j + '_' + i).setAttribute('state', 'pause');
      }
    }
  }
}

function setSize(page, x, y) {
  if((x != sizes[page][0]) || (y != sizes[page][1])) {
    broadcast = -1;
    var n = 0;
    for(var i = 0; i < graphs[page].length; i++) {
      graphs[page][i].broadcast = false;
      if(graphs[page][i].deviceMetric.length > 0) n++;
    }
    if(n > x * y) {
      dock.displayInfo("The number of active monitoring panels is larger than the desired size. Close some monitoring panels before resizing.", window);
      return false;
    } else {
      if(sizes[page][0] * sizes[page][1] > x * y) {
        var j = x * y - 1;
        var i = sizes[page][0] * sizes[page][1] - 1;
        while((i > j) && (i >= 0) && (j >= 0)) {
          if(graphs[page][j].deviceMetric && (graphs[page][j].deviceMetric.length > 0)) {
            j--;
          } else if(!graphs[page][i].deviceMetric || (graphs[page][i].deviceMetric.length == 0)) {
            i--;
          } else {
            println("SWAP " + j + " --> " + i);
            var tmp = graphs[j];
            graphs[j] = graphs[i];
            graphs[i] = tmp;
            i--;
            j--;
          }
        }
        for(var i = x * y; i < sizes[page][0] * sizes[page][1]; i++) {
          graphs[page][i].stop();
        }
        graphs[page].splice(x * y, sizes[page][0] * sizes[page][1] - x * y);
        println(x * y + " =?= " + graphs.length);
      } else {
        for(var i = sizes[page][0] * sizes[page][1]; i < x * y; i++) {
          var one = new cvGraph(cluster);
          one.ownerWindow = this;
          graphs[page].push(one);
        }
      }
      sizes[page][0] = x;
      sizes[page][1] = y;
      buildbox(page);
      redraw();
    }
  }
  return true;
}

function redraw(page, index) {
  if(typeof(index) == "number") {
    graphs[page][index].update();
  } else {
    for(var j = 0; j < graphs.length; j++) {
      for(var i = 0; i < graphs[j].length; i++) {
        if(isArray(graphs[j][i].deviceMetric) && (graphs[j][i].deviceMetric.length > 0)) {
          graphs[j][i].update();
        }
      }
    }
  }
}

function replaceOne(page, n) {
  var row = Math.floor(n / sizes[page][0]);
  var index = n - row * sizes[page][0];
  var row = document.getElementById('row' + page + '_' + row);
  row.replaceChild(addOne(page, n), row.childNodes[index]);
  redraw(page, n);
}

function playPause(page, n) {
  if(graphs[n].deviceMetric.length > 0) {
    if(graphs[n].playPause) {
      graphs[n].stop();
    } else {
      graphs[n].start();
    }
    if(graphs[n].playPause) {
      graphs[n].update();
      document.getElementById('play' + page + '_' + n).setAttribute('state', 'pause');
    } else {
      document.getElementById('play' + page + '_' + n).setAttribute('state', 'play');
    }
  } else {
    dock.displayInfo("No metrics/checks defined", window);
  }
}

function zoomIn(page, n) {
  if(graphs[page][n].deviceMetric.length > 0) {
    graphs[page][n].zoomIn();
    graphs[page][n].update();
  } else {
    dock.displayInfo("No metrics/checks defined", window);
  }
}

function zoomOut(page, n) {
  if(graphs[page][n].deviceMetric.length > 0) {
    graphs[page][n].zoomOut();
    graphs[page][n].update();
  } else {
    dock.displayInfo("No metrics/checks defined", window);
  }
}

function showProperties(page, n) {
  if(graphs[page][n].deviceMetric.length > 0) {
    graphs[page][n].showProperties();
  } else {
    dock.displayInfo("No metrics/checks defined", window);
  }
}

function setBroadcast(page, n) {
  if(graphs[page][n].deviceMetric.length > 0) {
    if(broadcast >= 0) {
      var button = document.getElementById('broadcast' + broadcast);
      if(button) button.removeAttribute('on');
      var graph = document.getElementById('graph' + broadcast);
      if(graph) graph.removeAttribute('broadcast');
      var bottom = document.getElementById('bottom' + broadcast);
      if(bottom) bottom.removeAttribute('broadcast');
      graphs[n].broadcast = false;
    }
    if(n == broadcast[page]) {
      broadcast[page] = -1;
    } else {
      broadcast[page] = n;
      var button = document.getElementById('broadcast' + broadcast);
      if(button) button.setAttribute('on', true);
      var graph = document.getElementById('graph' + broadcast);
      if(graph) graph.setAttribute('broadcast', true);
      var bottom = document.getElementById('bottom' + broadcast);
      if(bottom) bottom.setAttribute('broadcast', true);
      graphs[page][n].broadcast = true;
    }
  } else {
    dock.displayInfo("No metrics/checks defined", window);
  }
}

function setZoomBroadcast(page, index) {
  var xzoom = graphs[page][index].xzoom[graphs[page][index].xzoom.length - 1];
  var yzoom = graphs[page][index].yzoom[graphs[page][index].yzoom.length - 1];
  for(var i = 0; i < graphs[page].length; i++) {
    if((i != index) && (graphs[page][i].deviceMetric.length > 0)) {
      graphs[page][i].xzoomBroadcast = xzoom;
      graphs[page][i].yzoomBroadcast = yzoom;
      graphs[page][i].last = 0;
      graphs[page][i].update();
    }
  }
}

function addOne(page, n) {
  var box = document.createElement('vbox');
  box.setAttribute('flex', '1');
  box.setAttribute("graphindex", n);
  graphs[page][n].index = [page, n];
//   graphs[page][n].index = -1;
  if(isArray(graphs[page][n].deviceMetric) && (graphs[page][n].deviceMetric.length > 0)) {
    var box3 = document.createElement('vbox');
    box3.setAttribute('flex', '1');
    box3.setAttribute('class', 'monitoringtop');
    box3.setAttribute('id', 'graph' + page + '_' + n);
    if(broadcast == n) box3.setAttribute('broadcast', true);
    var box2 = document.createElement('hbox');
    box2.setAttribute('overflow', 'auto');
    box2.setAttribute('pack', 'end');
    var spacer = document.createElement('spacer');
    spacer.setAttribute('width', '20px'); // 37px to align
    box2.appendChild(spacer);
    var label = document.createElement('shadowlabel');
    label.setAttribute('class', 'staticdataheader');
    label.setAttribute('id', 'title' + page + '_' + n);
    label.setAttribute('flex', '1');
    label.setAttribute('style', 'overflow: hidden;');
    box2.appendChild(label);
    box3.appendChild(box2);

    var svg = document.createElementNS(XUL_SVG, "svg:svg");
    svg.setAttribute('class', 'graphbox');
    var svgg = document.createElementNS(XUL_SVG, "svg:g");
    svgg.setAttribute('style', 'fill-opacity:0.50;');

    svg.appendChild(svgg);
    box3.appendChild(svg);
    graphs[page][n].svg = svg;
    graphs[page][n].box = svgg;
    graphs[page][n].tooltip = 0;
    graphs[page][n].tooltiptext = '';
    graphs[page][n].zoombox = 0;
    zoomBox(page, n);

    var box2 = document.createElement('hbox');
    box2.setAttribute('align', 'center');
    box2.setAttribute('class', 'monitoringbottom');
    box2.setAttribute('id', 'bottom' + page + '_' + n);
    if(broadcast[page] == n) box2.setAttribute('broadcast', true);

    var label = document.createElement('label');
    label.setAttribute('id', 'info' + page + '_' + n);
    box2.appendChild(label);
    var spacer = document.createElement('spacer');
    spacer.setAttribute('flex', '1');
    box2.appendChild(spacer);

    var button = document.createElement('button');
    button.setAttribute('id', 'play' + page + '_' + n);
    if(graphs[page][n].playPause) {
      button.setAttribute('state', 'pause');
    } else {
      button.setAttribute('state', 'play');
    }
    button.setAttribute('class', 'pauseplay-button');
    button.setAttribute('tooltiptext', 'Play/Pause');
    button.setAttribute('oncommand', 'playPause(' + page + ',' + n + ');');
    box2.appendChild(button);

    var button = document.createElement('button');
    button.setAttribute('class', 'zoomin-button');
    button.setAttribute('tooltiptext', 'Zoom in');
    button.setAttribute('oncommand', 'zoomIn(' + page + ',' + n + ');');
    box2.appendChild(button);
    var button = document.createElement('button');
    button.setAttribute('class', 'zoomout-button');
    button.setAttribute('tooltiptext', 'Zoom out');
    button.setAttribute('oncommand', 'zoomOut(' + page + ',' + n + ');');
    box2.appendChild(button);

//     var button = document.createElement('button');
//     button.setAttribute('class', 'broadcast-button');
//     button.setAttribute('tooltiptext', 'broadcast');
//     button.setAttribute('id', 'broadcast' + n);
//     if(broadcast[page] == n) button.setAttribute('on', true);
//     button.setAttribute('oncommand', 'setBroadcast(' + page + ',' + n + ');');
//     box2.appendChild(button);

    var button = document.createElement('button');
    button.setAttribute('class', 'properties-button');
    button.setAttribute('tooltiptext', 'Properties');
    button.setAttribute('oncommand', 'showProperties(' + page + ',' + n + ');');
    box2.appendChild(button);

    if(startswith(getOS(), "WIN")) {
      var button = document.createElement('spacer');
      button.setAttribute('width', '4px');
      box2.appendChild(button);
    }

    box.appendChild(box3);
    box.appendChild(box2);
  } else {
    box.setAttribute('class', 'staticdata');
    box.setAttribute('align', 'center');
    var box2 = document.createElement('hbox');
    box2.setAttribute('flex', '1');
    box2.setAttribute('align', 'center');
    var image = document.createElement('image');
    image.setAttribute('src', './skin/gradient/drophere.png');
    box2.appendChild(image);
    box.appendChild(box2);
  }
  return box;
}

function buildbox(page) {
  var cols = document.getElementById('graphbox' + page);
  while(cols.firstChild) cols.removeChild(cols.firstChild);
  var n = 0;
  for(var j = 0; j < sizes[page][1]; j++) {
    var row = document.createElement('hbox');
    row.setAttribute('id', 'row' + page + '_' + j);
    row.setAttribute('equalsize', 'always');
    row.setAttribute('flex', '1');
    for(var i = 0; i < sizes[page][0]; i++) {
      row.appendChild(addOne(page, n));
      n++;
    }
    cols.appendChild(row);
  }
}

function zoomBox(page, n) {
  var pageIndex = page;
  var index = n;
  var startPos = 0;
  var mouseDown = function(event) {
    if (event.button == 0) {
      var element = event.target;
      while(!isObject(element.boxObject)) element = element.parentNode;
      var scroll = element.parentNode.parentNode.parentNode;
      var elementX = event.clientX - element.boxObject.x + PX + scroll.scrollLeft;
      var elementY = event.clientY - element.boxObject.y + PY + scroll.scrollTop;
      startPos = [elementX, elementY];
    } else {
      startPos = 0;
    }
    return false;
  }

  var mouseMove = function(event) {
    var element = event.target;
    while(!isObject(element.boxObject)) element = element.parentNode;
    var scroll = element.parentNode.parentNode.parentNode;
    var elementX = event.clientX - element.boxObject.x + PX + scroll.scrollLeft;
    var elementY = event.clientY - element.boxObject.y + PY + scroll.scrollTop;
    var info = graphs[pageIndex][index].info(elementX, elementY);
    document.getElementById('info' + pageIndex + '_' + index).setAttribute('value', info[0]);
    if(ENABLE_TOOLTIP) {
      if(info[1].length) {
        if((graphs[pageIndex][index].tooltiptext != info[1])) {
          if(graphs[pageIndex][index].tooltip) graphs[pageIndex][index].box.removeChild(graphs[pageIndex][index].tooltip);
          var svgt = document.createElementNS(XUL_SVG, "svg:g");
          svgt.setAttribute('opacity', '0.7');
    //       svgt.setAttribute('visibility', 'hidden');
    //       svgt.setAttribute('pointer-events', 'none');
          var svgr = document.createElementNS(XUL_SVG, "svg:rect");
          svgr.setAttribute('x', '0');
          svgr.setAttribute('y', '5');
          svgr.setAttribute('width', '200');
          svgr.setAttribute('height', '20');
          svgr.setAttribute('rx', '2');
          svgr.setAttribute('ry', '2');
          svgr.setAttribute('stroke', 'white');
          svgr.setAttribute('fill', 'yellow');
  /*        var svg1 = document.createElementNS(XUL_SVG, "svg:text");
          svg1.setAttribute('x', '5');
          svg1.setAttribute('y', '20');
          svg1.setAttribute("font-size", 11);
          svg1.setAttribute('fill','black');
          println(info[1]);
          svg1.appendChild(document.createTextNode(info[1]));
          svgr.appendChild(svg1)*/
          svgt.appendChild(svgr);
          graphs[pageIndex][index].box.appendChild(svgt);
          svgtext(svgt, 5, 18, info[1]);

          //return rectangle around text as SVGRect object
          var outline = svgr.getBBox();
          svgt.setAttribute('width', Number(outline.width) + 10);
          svgt.setAttribute('height', Number(outline.height) + 10);
          svgt.setAttribute('transform', 'translate(' + elementX + ',' + elementY + ')');

          graphs[pageIndex][index].tooltip = svgt;
          graphs[pageIndex][index].tooltiptext = info[1];
  //         graphs[index].tooltip = svgtext(graphs[index].box, elementX, elementY, info[1]);
        }
      } else if(graphs[pageIndex][index].tooltip) {
        graphs[pageIndex][index].box.removeChild(graphs[index].tooltip);
        graphs[pageIndex][index].tooltip = 0;
        graphs[pageIndex][index].tooltiptext = '';
      }
    }

    if(startPos != 0) {
      var zoom = graphs[pageIndex][index].zoombox;
      if(zoom) {
        while(zoom.firstChild) zoom.removeChild(zoom.firstChild);
      } else {
        zoom = document.createElementNS(XUL_SVG, "svg:g");
        graphs[pageIndex][index].zoombox = zoom;
        graphs[pageIndex][index].box.appendChild(zoom);
      }
      if(Math.abs(elementX - startPos[0]) <= MIN_ZOOM_BOX) {
        if(Math.abs(elementY - startPos[1]) <= MIN_ZOOM_BOX) {
          svgline(zoom, startPos[0] - MIN_ZOOM_BOX, startPos[1], startPos[0] + MIN_ZOOM_BOX, startPos[1], ZOOM_COLOR);
          svgline(zoom, startPos[0], startPos[1] - MIN_ZOOM_BOX, startPos[0], startPos[1] + MIN_ZOOM_BOX, ZOOM_COLOR);
        } else {
          svgline(zoom, startPos[0] - MIN_ZOOM_BOX, startPos[1], startPos[0] + MIN_ZOOM_BOX, startPos[1], ZOOM_COLOR);
          svgline(zoom, startPos[0], startPos[1], startPos[0], elementY, ZOOM_COLOR);
          svgline(zoom, startPos[0] - MIN_ZOOM_BOX, elementY, startPos[0] + MIN_ZOOM_BOX, elementY, ZOOM_COLOR);
        }
      } else if(Math.abs(elementY - startPos[1]) < MIN_ZOOM_BOX) {
        svgline(zoom, startPos[0], startPos[1] - MIN_ZOOM_BOX, startPos[0], startPos[1] + MIN_ZOOM_BOX, ZOOM_COLOR);
        svgline(zoom, startPos[0], startPos[1], elementX, startPos[1], ZOOM_COLOR);
        svgline(zoom, elementX, startPos[1] - MIN_ZOOM_BOX, elementX, startPos[1] + MIN_ZOOM_BOX, ZOOM_COLOR);
      } else {
        svgrect(zoom, startPos[0], startPos[1], elementX - startPos[0], elementY - startPos[1], '#555555', 1, ZOOM_COLOR);
      }
    }
  }

  var mouseUp = function(event) {
    if(startPos) {
      var zoom = graphs[pageIndex][index].zoombox;
      if(zoom) {
        var element = event.target;
        while(element && !isObject(element.boxObject)) element = element.parentNode;
        if(element) {
          var scroll = element.parentNode.parentNode.parentNode;
          var elementX = event.clientX - element.boxObject.x + PX + scroll.scrollLeft;
          var elementY = event.clientY - element.boxObject.y + PY + scroll.scrollTop;
          if((Math.abs(elementX - startPos[0]) > MIN_ZOOM_BOX) || (Math.abs(elementY - startPos[1]) > MIN_ZOOM_BOX)) {
            graphs[pageIndex][index].zoomIn(startPos[0], startPos[1], elementX, elementY);
            try {
              graphs[pageIndex][index].box.removeChild(zoom);
            } catch(e) {}
            graphs[pageIndex][index].zoombox = 0;
            graphs[pageIndex][index].update();
          } else {
            try {
              graphs[pageIndex][index].box.removeChild(zoom);
            } catch(e) {}
            graphs[pageIndex][index].zoombox = 0;
          }
        }
      }
      startPos=0;
    }
  }

  if(ENABLE_TOOLTIP) {
    var mouseOut = function(event) {
      if(graphs[pageIndex][index].tooltip) {
        graphs[pageIndex][index].box.removeChild(graphs[pageIndex][index].tooltip);
        graphs[pageIndex][index].tooltip = 0;
        graphs[pageIndex][index].tooltiptext = '';
      }
    }
  }

  graphs[pageIndex][index].box.addEventListener("mousedown",mouseDown, false);
  graphs[pageIndex][index].box.addEventListener("mousemove",mouseMove, false);
  if(ENABLE_TOOLTIP) {
    graphs[pageIndex][index].box.addEventListener("mouseout",mouseOut, false);
  }
  window.addEventListener("mouseup",mouseUp, false);
}

function waitIcon() {
  if(waitCount == 0) {
    window.setCursor("wait");
    for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("wait");
  }
  waitCount++;
}

function unwaitIcon() {
  waitCount--;
  if(waitCount <= 0) {
    window.setCursor("auto");
    for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("auto");
    if(waitCount < 0) println("Mismatch waitCursor");
    waitCount = 0;
  }
}

function parseFile(file, replace) {
  var lines = readListFromFile(file);
  var tabs = new Array();
  var tab = 0;
  var graph = 0;
  for(var i = 0; i < lines.length; i++) {
    var line = lines[i];
    var idx = line.indexOf('#');
    if(idx >= 0) line = line.substr(0, idx);
    line = trim(line);
    if(line.length == 0) continue;
    var lline = line.toLowerCase();
    if(startswith(lline, 'tab:')) {
      tab = [0, 0, trim(line.substr(line.indexOf(':') + 1)), new Array(), 3600];
      tabs.push(tab);
      nodeTab = 0;
    } else if(startswith(lline, 'nodetab:')) {
      nodeTab = [0, 0, trim(line.substr(line.indexOf(':') + 1)), new Array(), 3600];
      tab = 0;
    } else if(startswith(lline, 'node:')) {
      defaultNodes.push(trim(line.substr(line.indexOf(':') + 1)));
    } else if(startswith(lline, 'size:')) {
      var xy = line.substr(line.indexOf(':') + 1).split(/\s*x\s*/);
      if(xy.length == 2) {
        if(tab) {
          tab[0] = parseInt(xy[0]);
          tab[1] = parseInt(xy[1]);
        } else if(nodeTab) {
          nodeTab[0] = parseInt(xy[0]);
          nodeTab[1] = parseInt(xy[1]);
        }
      }
    } else if(startswith(lline, 'period:')) {
      line = line.substr(line.indexOf(':') + 1);
      var num = myParseInt(line);
      if(lline.indexOf('hour') > 0) {
        num *= 3600;
      } else if(lline.indexOf('day') > 0) {
        num *= 86400;
      } else if(lline.indexOf('week') > 0) {
        num *= 608400;
      } else if(lline.indexOf('month') > 0) {
        num *= 2678400;
      } else if(lline.indexOf('year') > 0) {
        num *= 31536000;
      } else {
        num = 0;
        println('Unknow period: (' + file + ') : ' + lines[i]);
      }
      if(num > 0) {
        if(tab) {
          tab[4] = num;
        } else if(nodeTab) {
          nodeTab[4] = num;
        }
      }
    } else if(startswith(lline, 'graph:')) {
      graph = [trim(line.substr(line.indexOf(':') + 1))];
      if(tab) {
        tab[3].push(graph);
      } else if(nodeTab) {
        nodeTab[3].push(graph);        
      }
    } else if(graph) {
      var def = line.split(/\s*,\s*/);
      if(def.length > 0 && def[0].length > 0) {
        var metric = def[0];
        for(var w in replace) {
          metric = metric.replace(w, replace[w]);
        }
        def.shift();
        var flags = 0;
        if(def.find('whisker') >= 0) {
          flags = dock.GRAPH_GROUP_WHISKER + flags_fill;
        } else if(def.find('heatmap') >= 0) {
          flags = dock.GRAPH_GROUP_HEATMAP;
        } else if(def.find('sum') >= 0) {
          flags = dock.GRAPH_GROUP_SUM;
          if(def.find('line') < 0) flags += flags_fill;
        } else if(def.find('max') >= 0) {
          flags = dock.GRAPH_GROUP_MAX;
          if(def.find('line') < 0) flags += flags_fill;
        } else if(def.find('min') >= 0) {
          flags = dock.GRAPH_GROUP_MIN;
          if(def.find('line') < 0) flags += flags_fill;
        } else if(def.find('avg') >= 0) {
          flags = dock.GRAPH_GROUP_AVG;
          if(def.find('line') < 0) flags += flags_fill;
        } else if(def.find('line') < 0) {
          flags = flags_fill;
        }
        graph.push([metric, flags]);
      }
    } else {
      println('Unknow line: (' + file + ') : ' + lines[i]);
    }
  }
  for(var i = 0; i < tabs.length; i++) {
    if(tabs[i][0] == 0 && tabs[i][1] == 0) {
      tabs[i][0] = Math.min(Math.max(1, Math.ceil(Math.sqrt(tabs[3].length))), MAX_HEIGHT);
      tabs[i][1] = Math.max(1, Math.ceil(tabs[3].length / h));
    } else if(tabs[i][0] == 0) {
      tabs[i][0] = Math.max(1, Math.ceil(tabs[3].length / tabs[i][1]));
    } else if(tabs[i][1] == 0) {
      tabs[i][1] = Math.max(1, Math.ceil(tabs[3].length / tabs[i][1]));
    }
  }
  if(nodeTab) {
    if(nodeTab[0] == 0 && nodeTab[1] == 0) {
      nodeTab[0] = Math.min(Math.max(1, Math.ceil(Math.sqrt(tabs[3].length))), MAX_HEIGHT);
      nodeTab[1] = Math.max(1, Math.ceil(tabs[3].length / h));
    } else if(nodeTab[0] == 0) {
      nodeTab[0] = Math.max(1, Math.ceil(tabs[3].length / nodeTab[1]));
    } else if(nodeTab[1] == 0) {
      nodeTab[1] = Math.max(1, Math.ceil(tabs[3].length / nodeTab[1]));
    }
  }
  return tabs;
}

function hadoopOverview() {
  var name = current.resolveName();

  setDialogTitle('Bright Cluster Manager (Hadoop: ' + name + ')');
  
  var file = dock.BASE_PATH + 'HadoopOverview.mine';
  if(!fileExists(file)) {
    initHadoopOverview(file);
  }
  var replace = {'$name': name};
  var tabs = parseFile(file, replace);
  if(tabs.length) {
    for(var i = 0; i < tabs.length; i++) {
      createPage(tabs[i][0], tabs[i][1], tabs[i][2], tabs[i][3], tabs[i][4]);
    }
    if(nodeTab) {
      document.getElementById('nodeTab').removeAttribute('hidden');
      for(var i = 0; i < defaultNodes.length; i++) {
        var node = cluster.find('Node', defaultNodes[i]);
        if(node && !hasNode.hasOwnProperty(node.get('uniqueKey'))) {
          hasNode[node.get('uniqueKey')] = createPage(nodeTab[0], nodeTab[1], node.resolveName(), nodeTab[3], nodeTab[4], node.get('uniqueKey'));
        }
      }
    }
  } else {
    dock.displayInfo("Unable to load " + fn);
  }
}

function createNodeTab() {
  var args = new Object();
  args['cluster'] = cluster;
  args['subtype'] = 'Node';
  var d = new dock.cvDialog("deviceselection.xul", args);
  d.apply = function(nodes) {
    for(var i = 0; i < nodes.length; i++) {
      if(!hasNode.hasOwnProperty(nodes[i])) {
        var node = cluster.find(nodes[i]);
        if(node) {
          hasNode[nodes[i]] = createPage(nodeTab[0], nodeTab[1], node.resolveName(), nodeTab[3], nodeTab[4], nodes[i]);
        }
      }
    }
  }
  dock.dialogs.show(d);
}

// Tab: DFS datanodes
// Size: 2x2
// Graph: DFS blocks
// $name_datanode_BlocksRead, sum, line
// $name_datanode_BlocksRemoved, sum, line
// $name_datanode_BlocksReplicated, sum, line
// $name_datanode_BlocksVerified, sum, line
// $name_datanode_BlocksWritten, sum, line
// 
// Graph: DFS I/O
// $name_datanode_BytesRead, sum
// $name_datanode_BytesWritten, sum
// 
// Graph: Random #1
// rnd, whisker
// 
// Graph: Random #2
// rnd, heatmap
// 
// Tab: DFS datanodes
// Size: 3x2
// ..
// 
// NodeTab: $name
// Size: 2x2
// ...


// FROM Michele:
// ===========================
// 01) Application master (fine-grained metrics on single jobs, maps, reduces)
//     1-2) Jobs
//     3) Reduces
//     4) Maps
// 02) DFS datanodes
//     1) DFS blocks
// $INSTANCE_datanode_BlocksRead
// $INSTANCE_datanode_BlocksRemoved
// $INSTANCE_datanode_BlocksReplicated
// $INSTANCE_datanode_BlocksVerified
// $INSTANCE_datanode_BlocksWritten
//     2) DFS I/O
// $INSTANCE_datanode_BytesRead
// $INSTANCE_datanode_BytesWritten
//     3) DFS timings #2
// $INSTANCE_datanode_ReadBlockOpAvgTime
// $INSTANCE_datanode_WriteBlockOpAvgTime
// + 1 regarding replace block
//     4) DFS timings #1
// $INSTANCE_datanode_HeartbeatsAvgTime
// + 2 regarding copy, reports, checksum
// 03) DFS file system
//     1) FS blocks
// $INSTANCE_namenode_MissingBlocks
// $INSTANCE_namenode_ExcessBlocks
// $INSTANCE_namenode_CorruptBlocks
// $INSTANCE_namenode_BlocksTotal
//     2) FS files
// $INSTANCE_namenode_FilesTotal
//     3) FS replication
// $INSTANCE_namenode_PendingDeletionBlocks
// $INSTANCE_namenode_UnderReplicatedBlocks
// $INSTANCE_namenode_ScheduledReplicationBlocks
// $INSTANCE_namenode_PendingReplicationBlocks
//     4) FS capacity
// NOTHING
// 04) Disk stat (non-Hadoop metrics)
//     1) Percentage
//     2) Time
//     3) Operations
//     4) Speed
// 05) Hadoop summary
//     1) Logs
// SUMS OVER
// $INSTANCE_$SERVICE_LogError
// $INSTANCE_$SERVICE_LogFatal
// $INSTANCE_$SERVICE_LogWarn
//     2) JVM memory
// SUMS OVER
// $INSTANCE_$SERVICE_MemNonHeapUsed
// $INSTANCE_$SERVICE_MemHeapUsed
//     3) DFS I/O
// $INSTANCE_datanode_BlocksRead
// $INSTANCE_datanode_BlocksWritten
//     4) Jobs
// $INSTANCE_resourcemanager_AppsSubmitted
// $INSTANCE_resourcemanager_AppsRunning
// + "Active Users"
// 06) JVM
//     1) Threads
// SUMS OVER
// $INSTANCE_$SERVICE_ThreadsBlocked
// $INSTANCE_$SERVICE_ThreadsNew
// $INSTANCE_$SERVICE_ThreadsRunnable
// $INSTANCE_$SERVICE_ThreadsTerminated
// $INSTANCE_$SERVICE_ThreadsTimedWaiting
// $INSTANCE_$SERVICE_ThreadsWaiting
//     2) Memory
// SUMS OVER
// $INSTANCE_$SERVICE_MemHeapCommitted
// $INSTANCE_$SERVICE_MemHeapUsed
// $INSTANCE_$SERVICE_MemNonHeapCommitted
// $INSTANCE_$SERVICE_MemNonHeapUsed
//     3) Garbage collection
// SUMS OVER
// $INSTANCE_$SERVICE_GcCount
//     4) Logging
// SUMS OVER
// $INSTANCE_$SERVICE_LogError
// $INSTANCE_$SERVICE_LogFatal
// $INSTANCE_$SERVICE_LogInfo
// $INSTANCE_$SERVICE_LogWarn
// 07) Network (non-Hadoop metrics)
//     1) TCP Syn
//     2) TCP establishment
//     3) TCP Active Close
//     4) TCP Passive Close
// 08) RCP operations
//     1) Blocks num ops
// $INSTANCE_namenode_BlockReportNumOps
// $INSTANCE_namenode_BlockReceivedAndDeletedNumOps
// $INSTANCE_namenode_AddBlockNumOps
//     2) Heartbeat ops
// $INSTANCE_datanode_HeartbeatsNumOps
// $INSTANCE_nodemanager_HeartbeatNumOps
// + RPC Ping
//     3) Job num ops
// $INSTANCE_resourcemanager_SubmitApplicationNumOps
// $INSTANCE_resourcemanager_RegisterApplicationMasterNumOps
// + 2 regarding jobs
//     4) FS num ops
// $INSTANCE_namenode_MkdirsNumOps
// $INSTANCE_namenode_RenameNumOps
// $INSTANCE_namenode_DeleteNumOps
// $INSTANCE_namenode_CreateNumOps
// 09) RPC time
//     1) Blocks average time
// $INSTANCE_namenode_BlockReceivedAndDeletedAvgTime
// $INSTANCE_namenode_AddBlockAvgTime
// $INSTANCE_namenode_BlockReportAvgTime
//     2) Heartbeat average time
// WEIGHTED AVG ON FOLLOWING?
// $INSTANCE_datanode_HeartbeatsAvgTime
// $INSTANCE_nodemanager_HeartbeatAvgTime
// $INSTANCE_resourcemanager_NodeHeartbeatAvgTime
//     3) Job average times
// $INSTANCE_resourcemanager_RegisterApplicationMasterAvgTime
// $INSTANCE_resourcemanager_SubmitApplicationAvgTime
// + 2 regarding jobs
//     4) FS ops average times
// $INSTANCE_namenode_MkdirsAvgTime
// $INSTANCE_namenode_DeleteAvgTime
// $INSTANCE_namenode_CreateAvgTime
// $INSTANCE_namenode_RenameAvgTime
// 10) Yarn queue
//     1) Apps
// $INSTANCE_resourcemanager_ActiveApplications
// $INSTANCE_resourcemanager_AppsCompleted
// $INSTANCE_resourcemanager_AppsRunning
// $INSTANCE_resourcemanager_AppsPending
// $INSTANCE_resourcemanager_AppsSubmitted
//     2) Containers
// $INSTANCE_nodemanager_ContainersRunning
// $INSTANCE_nodemanager_ContainersIniting
// $INSTANCE_nodemanager_ContainersKilled
// $INSTANCE_nodemanager_ContainersCompleted
// $INSTANCE_nodemanager_ContainersLaunched
// $INSTANCE_nodemanager_AllocatedContainers
//     3) Apps by duration
// $INSTANCE_resourcemanager_running_0
// $INSTANCE_resourcemanager_running_1440
// $INSTANCE_resourcemanager_running_300
// $INSTANCE_resourcemanager_running_60
//     4) Queue Containers
// $INSTANCE_resourcemanager_AggregateContainersAllocated
// $INSTANCE_resourcemanager_PendingContainers
// MAYBE $INSTANCE_nodemanager_AllocatedContainers
// $INSTANCE_resourcemanager_AggregateContainersReleased
// 11) System summary (non-Hadoop metrics)
//     1) Memory
//     2) CPU
//     3) Network
//     4) Disk
//     
//     
//     
// Generic overview
// Cluster CPU / Cluster Disk IO / Cluster Network IO
// 
// HDFS overview (latest 30m)
// 
// (ROW 1) HDFS Capacity / Total: Bytes Read / Total: Bytes Written
// ??? /$INSTANCE_datanode_BytesRead / $INSTANCE_datanode_BytesWritten
// 
// (ROW 2) Total: Blocks Read / Total: Blocks Written / Total DataNode Transceivers
// $INSTANCE_datanode_BlocksRead / $INSTANCE_datanode_BlocksWritten / $INSTANCE_namenode_TotalLoad
// 
// (ROW 3) Min, Max, Average: Transceiver Count / Min, Max, Average: Packet Ack Round Trip Average Time / Min, Max, Average: Send Data Packet Transfer Average Time
// ??? / $INSTANCE_datanode_PacketAckRoundTripTimeNanosAvgTime / $INSTANCE_datanode_SendDataPacketTransferNanosAvgTime
// 
// (ROW 4) Min, Max, Average: Send Data Packet Blocked On Network Average Time / Min, Max, Average: Disk Flushes / Min, Max, Average: Average Disk Flush Time
// $INSTANCE_datanode_SendDataPacketBlockedOnNetworkNanosAvgTime / $INSTANCE_datanode_FlushNanosNumOps / $INSTANCE_datanode_FlushNanosAvgTime
// 
// (ROW 5) Service Important Event and Alert Rate / DataNode Alert Rate / DataNode Critical Event Rate
