/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var waitCount = 0;
var treeClosed = 0;
var branches = 0;
var clusterCi = 0;
var allTasks = 0;
var taskIndex = 0;
var timer = 0;
var parents = 0;
var showChecked = {'Pending': 1, 'Running': 1, 'Failed': 1, 'Done': 1, 'Cancelled': 1};

function onLoad() {
  dock = getDock();
  println("Window initialized: " + this.name);

  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  var myDialog = dock.dialogs.list[index];

  refresh();
}

function refresh(force) {
  force = isundefined(force, false);
  if(force) {
    if(timer) window.clearTimeout(timer);
    timer = 0;
    waitIcon();
    var count = dock.connectedClustersCount();
    for(var j = 0; j < dock.clusterlist.length; j++) {
      if(dock.clusterlist[j].connected) {
        dock.workerpool.getWorker(dock.clusterlist[j].actualurl).call('cmmain', 'getBackgroundTasks', null, function(code, data) {
          if(code == 0) {
            for(var i = 0; i < data.length; i++) {
              this.current.objects[data[i].uniqueKey] = new dock.cvObject(this.current, data[i]);
            }
          } else {
            dock.jsonCallError(this, code, data);
          }
          count--;
          if(count == 0) update();
          unwaitIcon();
        }, dock.clusterlist[j]);
      }
    }
  } else {
    update();
  }
}

function update(forced) {
  forced = isundefined(forced, true);
  if(forced) {
    if(timer) window.clearTimeout(timer);
    timer = 0;
    taskIndex = 0;
    allTasks = new Array();
  }
  println("+++ TaskManager: update " + taskIndex + ", forced: " + forced);
  tree = document.getElementById('tree');
  data = new Array();
  var count = dock.connectedClustersCount();
  var list = document.getElementById('list');
  if(taskIndex == 0) {
    var column = tree.columns.getNamedColumn('key');
    var select = selected('list', column);
    if(branches) {
      treeClosed = new Object();
      for(var i = 0; i < branches.length; i++) {
        if(branches[i].parentNode.getAttribute('open') == 'false') {
          treeClosed[branches[i].parentNode.getAttribute('id')] = 1;
        } else if(branches[i].parentNode.hasAttribute('closedOnce')) {
          treeClosed[branches[i].parentNode.getAttribute('id')] = 0;
        }
      }
    }
    parents = new Object();
    branches = new Array();
    clusterCi = new Array();
    while(list.firstChild) list.removeChild(list.firstChild);
    for(var j = 0; j < dock.clusterlist.length;  j++) {
      if(dock.clusterlist[j].connected) {
        var tasks = dock.clusterlist[j].find('CMDaemonBackgroundTask', false, false);
        tasks.reverse(); // TODO: fix if needed
        tasks.sort(function(a, b) { return b.object.uniqueKey - a.object.uniqueKey; });
        allTasks = allTasks.concat(tasks);
        if(count > 1) {
          var ci = createClusterItem(list, dock.clusterlist[j]);
          branches.push(ci);
          clusterCi.push(allTasks.length);
        }
      }
    }
  }
  var done = 0;
  var k = 0;
  while(taskIndex < allTasks.length) {
    var st = allTasks[taskIndex].get('runningStatus');
    if(!showChecked.hasOwnProperty(st)) {
      taskIndex++;
      continue;
    }
    var key = allTasks[taskIndex].get('uniqueKey');
    var pt = allTasks[taskIndex].get('parentTask');
    var parent = 0;
    var ci = 0;
    for(var i = 0; i < clusterCi.length; i++) {
      ci = branches[i];
      if(taskIndex >= clusterCi[i]) break;
    }
    if(pt == 0) {
      if(ci) {
        parent = ci;
      } else {
        parent = list;
      }
    } else {
      if(parents.hasOwnProperty(pt)) {
        parent = parents[pt];
      } else {
        taskIndex++;
        println(" ++ TaskManager: parents.hasOwnProperty not found " + key + " : " + pt);
        continue;
      }
    }
    if(parent) {
      parents[key] = createTreeItem(parent, allTasks[taskIndex]);
      if(parents[key]) branches.push(parents[key]);
    } else {
      parents[key] = 0;
      println(' ++ TaskManager: no parent for ' + key + ' : ' + parents[key]);
    }
    done++;
    taskIndex++;
    if(done > dock.FILL_COUNT) {
      timer = window.setTimeout('update(false);', dock.FILL_TIMEOUT);
      return;
    }
  }
  setSelected('list', select, column);
  taskIndex = 0;
  if(timer) window.clearTimeout(timer);
  timer = 0;
}

function createClusterItem(parent, cluster) { 
  var item = document.createElement('treeitem');
  item.setAttribute('id', globalID);
  item.setAttribute('key', globalID);
  item.setAttribute('container', 'true');
  if(treeClosed.hasOwnProperty(cluster.key)) {
    if(treeClosed[cluster.key] == 1) {
      item.setAttribute('open', 'false');
    } else {
      item.setAttribute('open', 'true');
    }
  } else {
    item.setAttribute('open', 'true');
  }
  
  var row = document.createElement('treerow');
  var cell = document.createElement('treecell');
  cell.setAttribute('src', 'chrome://cmgui/content/icons/Cluster_16.png');
  cell.setAttribute('label', ' ' + cluster.name); // space between image and label, css doesn't work
  row.appendChild(cell);
  item.appendChild(row);

  var child = document.createElement('treechildren');
  child = document.createElement('treechildren');
//   child.setAttribute('id', text);
  item.appendChild(child);
  parent.appendChild(item);
  return child;
}

function createTreeItem(parent, task) { 
  var cluster = task.cluster;
  var item = document.createElement('treeitem');
  var key = cluster.key + '|' + task.get('uniqueKey');
  var text = task.get('name');
  var state = task.get('runningStatus');
  item.setAttribute('id', cluster.key + '|' + task.get('uniqueKey'));
  item.setAttribute('title', text);
  var st = task.get('subtasks');
  if(st.length) {
    item.setAttribute('container', 'true');
    if(treeClosed.hasOwnProperty(key)) {
      if(treeClosed[key] == 1) {
        item.setAttribute('open', 'false');
      } else {
        item.setAttribute('open', 'true');
        item.setAttribute('closedOnce', 'true');
      }
//     } else if(state == 'Done') {
//       item.setAttribute('open', 'false');
//       item.setAttribute('closedOnce', 'true');
    } else {
      item.setAttribute('open', 'true');
    }
  }
  var row = document.createElement('treerow');
  var cell = document.createElement('treecell');
  // http://files.softicons.com/download/application-icons/or-applications-icons-by-iconleak/png/16x16/10.png
  switch(state) { // TODO good icons!
    case('Done') :   cell.setAttribute('src', 'chrome://cmgui/content/icons/' + 'Task-done_16.png'); break;
    default:         cell.setAttribute('src', 'chrome://cmgui/content/icons/' + 'Task_16.png');
  }
  cell.setAttribute('label', ' ' + text);
  row.appendChild(cell);

  var cell = document.createElement('treecell');
  cell.setAttribute('label', key);
  row.appendChild(cell);
  
  var cell = document.createElement('treecell');
  cell.setAttribute('label', dotFormatKeys(cluster, task.get('subjects')));
  row.appendChild(cell);
  
  var cell = document.createElement('treecell');
  cell.setAttribute('label', state);
  row.appendChild(cell);
  
  var cell = document.createElement('treecell');
  var updates = task.get('updates');
  if(updates.length) {
    cell.setAttribute('label', updates[updates.length - 1]);
  } else {
    cell.setAttribute('label', '');
  }
  row.appendChild(cell);
  
  var cell = document.createElement('treecell');
  if(task.get('startTime')) {
    cell.setAttribute('label', formatTime(task.get('startTime'), true));
  } else {
    cell.setAttribute('label', '');
  }
  row.appendChild(cell);
  var cell = document.createElement('treecell');
  if(task.get('endTime')) {
    cell.setAttribute('label', formatTime(task.get('endTime'), true));
  } else {
    cell.setAttribute('label', '');
  }
  row.appendChild(cell);
  
  item.appendChild(row);
  
  var child = 0;
  if(st.length) {
    child = document.createElement('treechildren');
    item.appendChild(child);
  }
  var lt = parent.lastChild;
  if(lt && sort) {
    if(lt.getAttribute('title') > text) { // add during startup, will most likely sorted
      var i0 = 0;
      var i2 = parent.childNodes.length - 1;
      while(i0 < i2) {
        var i1 = Math.floor((i0 + i2) / 2);
        var e = parent.childNodes[i1].getAttribute('title');
        if(e > text) {
          i2 = i1;
        } else if(e < text) {
          i0 = i1 + 1;
        } else {
          i2 = i1;
          break;
        }          
      }
      if(i2 < parent.childNodes.length) {
        parent.insertBefore(item, parent.childNodes[i2]);
        item = 0;
      }
    }
  }
  if(item) parent.appendChild(item);
  return child;
}

function show(what) {
  var changed = false;
  var obj = document.getElementById('show' + what);
  if(obj) {
    if(obj.checked) {
      if(!showChecked.hasOwnProperty(what)) {
        changed = true;
        showChecked[what] = 1;
      }
    } else {
      if(showChecked.hasOwnProperty(what)) {
        changed = true;
        delete showChecked[what];
      }
    }
  }
  if(changed) update();
}

function cancel() {
  // TODO
}

function log() {
  var column = tree.columns.getNamedColumn('key');
  var select = selected('list', column);
  if(select.length == 0) {
    dock.displayError('Please select a task');
  } else {
    for(var i = 0; i < select.length; i++) {
      var work = select[i].split('|');
      var cluster = dock.getCluster(parseInt(work[0]));
      var task = cluster.find(parseInt(work[1]));
      if(task) {
        var args = new Object();
        args['title'] = "Log for task: " + task.get('name') + " for " + dotFormatKeys(cluster, task.get('subjects'));
        args['text'] = task.get('updates').join("\n");
        var d = new dock.cvDialog('showtext.xul', args);
        dock.dialogs.showDialog(d);
      }
    }
  }
}

function waitIcon() {
  if(waitCount == 0) {
    window.setCursor("wait");
    for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("wait");
  }
  waitCount++;
}

function unwaitIcon() {
  waitCount--;
  if(waitCount <= 0) {
    window.setCursor("auto");
    for(var i = 0; i < window.frames.length; i++) window.frames[i].window.setCursor("auto");
    if(waitCount < 0) println("Mismatch waitCursor");
    waitCount = 0;
  }
}
