/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var myDialog;
var dock
var cluster;
var monconf;
var metric;
var check;
var params;
var upper = true;

function onLoad() {
  dock = getDock();
  println("Threshold Wizard initialized: " + this.name);
  var s = this.name.indexOf('[');
  var e = this.name.indexOf(']');
  var index = this.name.substr(s + 1, e - s - 1);
  myDialog = dock.dialogs.list[index];

  cluster = myDialog.arguments.cluster;
  var cat = document.getElementById('category');
  var monconfs = cluster.find('MonConf');
  for(var i = 0; i < monconfs.length; i++) {
    var item = document.createElement("listitem");
    item.setAttribute('key', monconfs[i].get('uniqueKey'));
    var image = document.createElement('image');
    image.setAttribute('src', cluster.getMonConfImage(monconfs[i].get('uniqueKey')));
    item.appendChild(image);
    var label = document.createElement("label");
    label.setAttribute('value', cluster.getMonConfName(monconfs[i].get('uniqueKey')));
    item.appendChild(label);
    cat.appendChild(item);
  }

  var actions = cluster.find('ThreshAction');
  actions.sort(function(a, b) { return b.get('name') < a.get('name'); });
  for(var i = 1; i <= 3; i++) {
    var list = document.getElementById('action' + i);
    var popup = list.firstChild;
    for(var j = 0; j < actions.length; j++) {
      var item = document.createElement("menuitem");
      item.setAttribute('key', actions[j].get('uniqueKey'));
      item.setAttribute('label', actions[j].get('name'));
      popup.appendChild(item);
    }
    var item = document.createElement("menuitem");
    item.setAttribute('key', 0);
    item.setAttribute('label', '');
    popup.appendChild(item);
  }

}

function next() {
  var index = parseInt(document.getElementById('deck').selectedIndex);
  switch(index) {
  case(0) :
    if(document.getElementById('category').selectedCount == 0) {
      dock.displayInfo("Select a category");
      return;
    }
    monconf = document.getElementById('category').selectedItem;
    monconf = parseInt(monconf.getAttribute('key'));
    monconf = cluster.find(monconf);
    break;
  case(1) :
    if(document.getElementById('metric').selectedCount == 0) {
      dock.displayInfo("Select a metric(parameter)");
      return;
    }
    var item = document.getElementById('metric').getSelectedItem(0);
    metric = parseInt(item.getAttribute('key'));
    check = eval(item.getAttribute('check'));
    params = new Array();
    if(item.getAttribute('param').length > 0) params.push(item.getAttribute('param'));
    for(var i = 1; i < document.getElementById('metric').selectedCount; i++) {
      var item = document.getElementById('metric').getSelectedItem(i);
      var metric2 = parseInt(item.getAttribute('key'));
      if(metric != metric2) {
        dock.displayInfo("When selecting multiple metric(parameter) or health check(parameter) pairs\nthe metric/health check should be identical");
        return;
      }
      if(item.getAttribute('param').length > 0) params.push(item.getAttribute('param'));
    }
    if(check) index++;
    break;
  case(2) :
    var bound = document.getElementById('bound').value;
    if(!isNumericKMG(bound)) {
      dock.displayInfo("Bound should be a number (1/1K/1M/1G)");
      return;
    }
    bound = fromKMG(bound);
  }
  index++;
  switch(index) {
  case(1) :
    var metricList = document.getElementById('metric');
    while(metricList.childNodes > 1) metricList.removeChild(metricList.lastChild);
    var list = monconf.get('metrics');
    for(var i = 0; i < list.length; i++) {
      var one = cluster.find(list[i].metric);
      if(one.get('metricClass') != 'PROTOTYPE') {
        var item = document.createElement("listitem");
        item.setAttribute('key', list[i].metric);
        item.setAttribute('param', list[i].metricParam);
        var image = document.createElement('image');
        image.setAttribute('src', 'chrome://cmgui/content/icons/Monitoring_16.png');
        item.appendChild(image);
        var label = document.createElement("label");
        var name = one.resolveName();
        if(list[i].metricParam) name += ('(' + list[i].metricParam + ')');
        label.setAttribute('value', name);
        item.appendChild(label);
        metricList.appendChild(item);
      }
    }
    var list = monconf.get('checks');
    for(var i = 0; i < list.length; i++) {
      var item = document.createElement("listitem");
      item.setAttribute('key', list[i].healthcheck);
      item.setAttribute('check', 'true');
      item.setAttribute('param', list[i].healthcheckParam);
      var image = document.createElement('image');
      image.setAttribute('src', 'chrome://cmgui/content/icons/Health_16.png');
      item.appendChild(image);
      var label = document.createElement("label");
      var name = cluster.resolveName(list[i].healthcheck);
      if(list[i].healthcheckParam) name += ('(' + list[i].healthcheckParam + ')');
      label.setAttribute('value', name);
      item.appendChild(label);
      metricList.appendChild(item);
    }
    break;
  case(2):
    var name = cluster.resolveName(metric);
    if(params.length > 0) name += ('(' + params.join(',') + ')');
    document.getElementById('list').setAttribute('value', name);
    var unit = cluster.find(metric);
    unit = unit.get('unit');
    document.getElementById('unit').setAttribute('value', unit);
    break;
  case(3):
  }
  document.getElementById('deck').selectedIndex = index;
  document.getElementById('prev').setAttribute('disabled', index == 0);
  document.getElementById('next').setAttribute('hidden', index == 3);
  document.getElementById('finish').setAttribute('hidden', index < 3);
}

function prev() {
  var index = parseInt(document.getElementById('deck').selectedIndex);
  index--;
  document.getElementById('deck').selectedIndex = index;
  document.getElementById('prev').setAttribute('disabled', index == 0);
  document.getElementById('next').setAttribute('hidden', index == 3);
  document.getElementById('finish').setAttribute('hidden', index < 3);
}

function finish() {
  var one = false;
  for(var j = 1; j <= 3; j++) {
    var item = document.getElementById('action' + j).selectedItem;
    if(item) one = true;
  }
  if(!one) {
    dock.displayError("Specify at least one action");
    return;
  }
  var bound = document.getElementById('bound').value;
  bound = fromKMG(bound, '');
  if(check) {
    var name = cluster.resolveName(metric);
    name += " failing ";
    var list = monconf.get('checks');
    for(var i = 0; i < list.length; i++) {
      if((list[i].healthcheck == metric) && ((params.find(list[i].healthcheckParam) >= 0) || ((params.length == 0) && (list[i].healthcheckParam.length == 0)))) {
        var a = list[i].failActionConfs;
        for(var j = 1; j <= 3; j++) {
          var item = document.getElementById('action' + j).selectedItem;
          if(item) {
            var key = parseInt(item.getAttribute('key'));
            if(key) {
              var act = new dock.cvThreshActionConf();
              var act = new dock.cvObject(cluster, act);
              act.set('threshAction', key);
              act.set('actionParam', document.getElementById('actionparam' + j).value);
              a.push(act.object);
            }
          }
        }
        list[i].failActionConfs = a;
      }
    }
  } else {
    var name = cluster.resolveName(metric);
    if(upper) {
      name += " greater than ";
    } else {
      name += " less than ";
    }
    var unit = cluster.find(metric);
    unit = unit.get('unit');
    if(unit.length > 0) {
      name += formatKMGselect(bound, unit);
    } else {
      name += bound;
    }
    var list = monconf.get('metrics');
    for(var i = 0; i < list.length; i++) {
      if((list[i].metric == metric) && ((params.find(list[i].metricParam) >= 0) || ((params.length == 0) && (list[i].metricParam.length == 0)))) {
        var nt, add = true;
        var t = list[i].thresholds;
        for(var j = 0; j < t.length; j++) {
          if((t[j].bound == bound) && (t[j].isUpper == upper)) {
            nt = t[j];
            add = false;
            break;
          }
        }
        if(add) {
          nt = new dock.cvThreshold();
          var ntx = new dock.cvObject(cluster, nt);
          ntx.set('name', name);
          ntx.set('bound', bound);
          ntx.set('isUpper', upper);
        }
        var a = nt.actionConfs;
        for(var j = 1; j <= 3; j++) {
          var item = document.getElementById('action' + j).selectedItem;
          if(item) {
            var key = parseInt(item.getAttribute('key'));
            if(key) {
              var act = new dock.cvThreshActionConf();
              var act = new dock.cvObject(cluster, act);
              act.set('threshAction', key);
              act.set('actionParam', document.getElementById('actionparam' + j).value);
              a.push(act.object);
            }
          }
        }
        nt.actionConfs = a
        if(add) {
          t.push(nt);
          list[i].thresholds = t;
        }
      }
    }
  }
  monconf.set('modified', true);

  myDialog.onOk();
  close();
}

function cancel() {
  close();
}

function setUpper(value) {
  upper = value;
}
