/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var MINTREEITEMS = 0;

var data = null;
var tree;
var groupby = null;

var treeFilter = {
  enabled: false,
  displayLine: 0,
  filter: {},
  row: {},
  col: {},
  child: {}
}

function loadTable(realTree, realData, _groupby) {
  groupby =  isundefined(_groupby,groupby);
  realTree = isundefined(realTree, tree);
  realData = isundefined(realData, data);

  var topVisibleRow = null;
  if(realData) topVisibleRow = getTopVisibleRow(realTree);
  if(!realTree.getAttribute("sortOrder")) {
    var order = new Array();
    var col = realTree.firstChild.firstChild;
    while(col) {
      if(col.getAttribute("sortDirection")) {
        if(col.getAttribute("sortIndex")) {
          order.splice(col.getAttribute("sortIndex"), 0, col.id);
        } else {
          order.push(col.id);
        }
      }
      col = col.nextSibling;
    }
    var index = order.find('modified');
    if(index == 0) {
      order[0] = order[1];
      order[1] = 'modified';
    }
    realTree.setAttribute("sortOrder", order.join(','));
  }

  if(realTree.getAttribute('filter')==='true' && treeFilter.enabled===false) {
    treeFilter.enabled = true;
    var col = realTree.firstChild.firstChild;
    while(col) {
      if(col.getAttribute('filter')) {
        treeFilter.filter[col.id] = '';
      }
      col = col.nextSibling;
    }
  }

  sort(false, realTree, realData);
  if(topVisibleRow) setTopVisibleRow(topVisibleRow, realTree);
}

function treeViewExpandData(realData)
{
  var ret = new Array();
  for(var i = 0; i < realData.length; i++) {
    var parent = ret.length;
    ret.push(realData[i]);
    if(realData[i].treeIsContainer && realData[i].treeIsOpen) {
      for(var j = 0; j < realData[i].treeChildren.length; j++){
        ret.push(realData[i].treeChildren[j]);
      }
    }
  }
  return ret;
}

function treeView(realData, groupby, treeFilter) {
  var isTree = (typeof(groupby) != 'undefined');

  this.treeFilter = treeFilter;
  this.isFilter = function() {
    return (typeof(this.treeFilter)!=='undefined' && this.treeFilter.enabled);
  }
  if(this.isFilter()) {
    var filter = {};
    for(var f in this.treeFilter.filter) {
      filter[f] = this.treeFilter.filter[f];
      filter[f + '.src'] = 'chrome://cmgui/content/skin/button/Search_20x16.png';
    }
    realData.splice(this.treeFilter.displayLine, 0, filter);
  }

  if(isTree) {
    this.table = treeViewExpandData(realData);
  } else {
    this.table = realData;
  }
  this.rowCount = this.table.length;
  this.getCellText = function(row, col) {
    if(col) {
      var value = this.table[row][col.id];
      if(value != null) {
        if((typeof(value) != 'undefined') && (typeof(value.format) == 'function')) {
          value = value.format('dd/mmm/yyyy HH:MM:ss');
        }
//       } else {
//         println("treeView null for: " + row + ", " + col.id);
      }
      return value;
    } else if(this.table[row]['key']) {
      return this.table[row]['key'];
    }
    return null;
  };
  this.getCellValue = function(row, col) {
    if(this.table[row][col.id]) {
      return this.table[row][col.id];
    }
    return null;
  };
  this.setCellText = function(row, col, value) {
    if(this.table[row][col.id]) {
      return this.table[row][col.id] = value;
    }
    return null;
  };
  this.setCellValue = function(row, col, value) {
    if(this.table[row][col.id]) {
      return this.table[row][col.id] = value;
    }
    return null;
  };
  this.setTree = function(treebox) {
    this.treebox = treebox;
  };
  this.isEditable = function(row, col) {
    return col.editable;
  };


  this.isContainer = function(row){
    return ( isTree && typeof(this.table[row]['treeIsContainer']) !='undefined' && this.table[row]['treeIsContainer'] && Array.isArray(this.table[row]['treeChildren']) && this.table[row]['treeChildren'].length > 0);
  };
  this.isContainerOpen = function(row) {
    return (isTree && typeof(this.table[row]['treeIsContainer']) !='undefined' && typeof(this.table[row]['treeIsOpen']) !='undefined' && this.table[row]['treeIsContainer'] && this.table[row]['treeIsOpen']);
  };
  this.getParentIndex = function(row) {
    if(!isTree || (typeof(this.table[row]['treeIsContainer']) !='undefined' &&this.table[row]['treeIsContainer'])) {
      return -1;
    } else {
      for (var t = row - 1; t >= 0 ; t--) {
        if (this.isContainer(t)) {
          return t;
        }
      }
    }
    return -1;
  };
  this.toggleOpenState = function(row) {
    if(!isTree) {
      return true;
    }
    if(typeof(this.table[row]['treeIsContainer']) =='undefined' || !this.table[row]['treeIsContainer']) {
      return true;
    }
    if(typeof(this.table[row]['treeIsOpen']) != 'undefined' && this.table[row]['treeIsOpen']) {
      // collapse
      this.table[row]['treeIsOpen']=false;
      if(typeof(this.table[row]['treeOnOpenClose']) == 'function') {
        this.table[row]['treeOnOpenClose'](false,this.table[row]);
      }

      var deletecount = 0;
      for (var t = row + 1; t < this.table.length; t++) {
        if( this.getLevel(t) == 1) {
          deletecount++;
        } else {
          break;
        }
      }
      if (deletecount) {
        this.table.splice(row + 1, deletecount);
        this.rowCount = this.table.length;
        this.treebox.rowCountChanged(row + 1, -deletecount);
        this.treebox.invalidateRow(row);
      }
      if(tree) {
        var event = new Event('row-count');
        tree.dispatchEvent(event);
      }
    } else {
      // open
      this.table[row]['treeIsOpen']=true;
      if(typeof(this.table[row]['treeOnOpenClose']) == 'function') {
        this.table[row]['treeOnOpenClose'](true,this.table[row]);
      }
      var toinsert = this.table[row].treeChildren;
      for (var i = 0; i < toinsert.length; i++) {
        this.table.splice(row + i + 1, 0, toinsert[i]);
      }
      this.rowCount = this.table.length;
      this.treebox.rowCountChanged(row + 1, toinsert.length);
      this.treebox.invalidateRow(row);
      if(tree) {
        var event = new Event('row-count');
        tree.dispatchEvent(event);
      }
    }
  };
  this.isSeparator = function(row){ return false; };
  this.isSorted = function(){ return false; };
  this.getLevel = function(row){
    if(!isTree || (typeof(this.table[row]['treeIsContainer']) !='undefined' && this.table[row]['treeIsContainer'])) {
      return 0;
    } else  {
      return 1;
    }
  };
  this.getImageSrc = function(row,col) {
    if(this.table[row][col.id + '.src']) return this.table[row][col.id + '.src'];
    return null;
  };
  this.getRowProperties = function(row,props){
    if(this.isFilter() && this.treeFilter.displayLine===row) {
      return 'filter';
    }
  };
  this.getCellProperties = function(row,col,props){
    if(this.isFilter() && this.treeFilter.displayLine===row) {
      if(typeof(this.treeFilter.filter[col.id])==='boolean' && this.treeFilter.filter[col.id]) {
        return 'filter checked';
      } else if(typeof(this.treeFilter.filter[col.id])==='boolean' && !this.treeFilter.filter[col.id]) {
        return 'filter unchecked';
      }
      return 'filter';
    }
  };
  this.getColumnProperties = function(colid,col,props){};
  this.cycleHeader = function(col, elem) {};
}

function sort(column, realTree, realData) {
  realTree = isundefined(realTree, tree);
  realData = isundefined(realData, data);
  realTree.view = null;
  var order = realTree.getAttribute("sortOrder").split(',');
  if(column) {
    var index = order.find(column.id);
    if(index < 0) {
      println("ERROR: " + column.id + " not sorted");
      return;
    } else if(index == 0) {
      if(column.getAttribute("sortDirection") == "ascending") {
        column.setAttribute("sortDirection", "descending");
      } else {
        column.setAttribute("sortDirection", "ascending");
      }
    } else {
      order.splice(index, 1);
      order.unshift(column.id);
      realTree.setAttribute("sortOrder", order.join(','));
    }
    // Hack, for some reason right click does not redraw, force here
    var name = column.getAttribute("label");
    column.setAttribute("label", 'DUDE');
    column.setAttribute("label", name);
  }
  var newOrder = new Array()
  for(var i = 0; i < order.length; i++) {
    var col = document.getElementById(order[i]);
    if(col) {
      var dir = (col.getAttribute('sortDirection') == "ascending") ? 1 : -1;
      newOrder.push([order[i], dir]);
    }
  }
  order = newOrder;

  function columnSort(a, b) {
    for(var i = 0; i < order.length; i++) {
      var id = order[i][0];
      if(typeof(a[id]) == "undefined") return 1;
      if(typeof(b[id]) == "undefined") return -1;
      var aa = lc(a[id]);
      var bb = lc(b[id]);
      var ia = isNumeric(aa, true, true, true, true, true);
      var ib = isNumeric(bb, true, true, true, true, true);
      if(ia && ib) {
        if(ia == 1) {
          ia = parseFloat(aa);
        } else {
          ia = parseKMG(aa);
        }
        if(ib == 1) {
          ib = parseFloat(bb);
        } else {
          ib = parseKMG(bb);
        }
        if(ia > ib) {
          return order[i][1];
        } else if(ia < ib) {
          return -order[i][1];
        }
      } else if(aa > bb) {
        return order[i][1];
      } else if(aa < bb) {
        return -order[i][1];
      }
    }
    return 0;
  }
  realData.sort(columnSort);
  if(typeof(groupby) == 'function') {
      realData = groupby(realData);
  }

  filterTree(realTree, realData, groupby);

  if(typeof(saveOrder) == 'function') saveOrder();
}


function filterSubTree(row, field, filter) {
  var visible = false;
  if(typeof(row.treeChildren) ==='object') {
    for(var i in row.treeChildren) {
      var child = row.treeChildren[i];
      if(typeof(child) ==='object') {
        if(typeof(child[field])==='boolean') {
          visible = visible || ( (filter[field]===child[field]) );
        } else if(typeof(child[field])==='string' && typeof(filter[field])==='string' && filter[field]!='') {
          visible = visible || ( (child[field].search(RegExp(filter[field], 'i')) !== -1) );
        } else if(typeof(child[field])==='object' && typeof(filter[field])==='string' && filter[field]!='') {
          visible = visible || ( (child[field].toString().search(RegExp(filter[field], 'i')) !== -1) );
        } else if(typeof(child[field])==='number' && typeof(filter[field])==='string' && filter[field]!='') {
          visible = visible || ( (child[field].toString().search(RegExp(filter[field], 'i')) !== -1) );
        }
      }
    }
  }
  if(visible && !row.treeIsOpen) {
      row.treeIsOpen = true;
      if(typeof(row.treeOnOpenClose) === 'function') {
          row.treeOnOpenClose(row.treeIsOpen, row);
      }
  }
  return visible;
}

function filterTree(realTree, realData, groupby) {
  if(treeFilter.enabled===true) {
    this.filter = function(row) {
      var visible = true;
      for(var f in treeFilter.filter) {
        // note that filterSubTree needs to be called regardless if the other filter applies already
        var sub_tree_filter = filterSubTree(row,f, treeFilter.filter);
        if(typeof(row[f])==='boolean') {
          visible = visible && ( (treeFilter.filter[f]==='' || treeFilter.filter[f]===row[f]) ||  sub_tree_filter);
        } else if(typeof(row[f])==='string') {
          visible = visible && ( (row[f].search(RegExp(treeFilter.filter[f], 'i')) !== -1)    || sub_tree_filter );
        } else if(typeof(row[f])==='object') {
          visible = visible && ( (row[f].toString().search(RegExp(treeFilter.filter[f], 'i')) !== -1) || sub_tree_filter );
        } else if(typeof(row[f])==='number') {
          visible = visible && ( (row[f].toString().search(RegExp(treeFilter.filter[f], 'i')) !== -1) || sub_tree_filter );
        } else if(typeof(row[f])==='undefined' && treeFilter.filter[f]!=='') {
          visible = visible && sub_tree_filter;
        }
      }
      return visible;
    };

    realData = realData.filter(this.filter);
  }
  realTree.view = new treeView(realData, groupby, treeFilter);
  var event = new Event('row-count');
  realTree.dispatchEvent(event);
}

function lc(o) {
  if(typeof o == "string") return o.toLowerCase();
  return o;
}

function getTopVisibleRow(realTree) {
  realTree = isundefined(realTree, tree);
  if(realTree.treeBoxObject) return realTree.treeBoxObject.getFirstVisibleRow();
  return null;
}

function setTopVisibleRow(topVisibleRow, realTree) {
  realTree = isundefined(realTree, tree);
  if(realTree.treeBoxObject) return realTree.treeBoxObject.scrollToRow(topVisibleRow);
  return 0;
}

function blankTreeItem() {
  var item = new Object();
  return item;
}

function onTreeClicked(event) {
  if(treeFilter.enabled===false) {
    return;
  }

  var tree = event.target.parentNode;
  var tbo = tree.treeBoxObject;

  tbo.getCellAt(event.clientX, event.clientY, treeFilter.row, treeFilter.col, treeFilter.child);

  if(treeFilter.displayLine!==treeFilter.row.value || !treeFilter.col.value.element.getAttribute('filter')) {
    return;
  }
  var cellText = tree.view.getCellText(treeFilter.row.value, treeFilter.col.value);
  if(treeFilter.col.value.element.getAttribute('filter')==='checkbox') {
    var state;
    switch(cellText) {
      case 'true':
        state = 'checked';
        break;
      case 'false':
        state = 'unchecked';
        break;
      default:
        state = 'indeterminate';
    }
    document.getElementById('filterCheckimage').setAttribute('state', state);
    document.getElementById('filterCheckimage').checked = cellText;
    document.getElementById('filterCheckimage').removeAttribute('hidden');
    document.getElementById('filterTextbox').setAttribute('hidden', 'true');
  } else {
    document.getElementById('filterTextbox').value = cellText;
    document.getElementById('filterTextbox').removeAttribute('hidden');
    document.getElementById('filterCheckimage').setAttribute('hidden', 'true');
  }

  if(treeFilter.col.value.element.getAttribute('label')) {
    document.getElementById('filterDescrition').value = treeFilter.col.value.element.getAttribute('label') + ':';
  } else {
    document.getElementById('filterDescrition').value = treeFilter.col.id + ':';
  }

  var popup = document.getElementById('filterPanel');
  popup.openPopup(null, 'before_start', event.clientX, event.clientY);
}

function setTreeFilter(elementObject){
  var value;
  if(document.getElementById('filterCheckimage').hidden===false) {
    switch(document.getElementById('filterCheckimage').getAttribute('state')) {
      case 'unchecked':
        value = false;
        break;
      case 'checked':
        value = true;
        break;
      case 'indeterminate':
        value = '';
        break;
      default:
        value = '';
    }
  } else {
    value = document.getElementById('filterTextbox').value;
  }

  var popup = document.getElementById('filterPanel');
  popup.hidePopup();

  treeFilter.filter[treeFilter.col.value.id] = value;
  sort();
}

function clearTreeFilter(elementObject) {
  document.getElementById('filterCheckimage').setAttribute('state', 'indeterminate');
  document.getElementById('filterCheckimage').checked = false;
  document.getElementById('filterTextbox').value = '';
}

function resetTree(elementNodeId) {
  // HACK to reset XUL element properties
  var originalTree = document.getElementById(elementNodeId);
  var clonedTree = originalTree.cloneNode(true);
  originalTree.parentNode.replaceChild(clonedTree, originalTree);
    
  refresh();
}
