/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;

var waiting = 0;
var force = false;

var perPage = 25;
var page = 1;
var pages = 1;
var rows = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function stop() {
  force = false;
}

function refresh(use) {
  if(typeof(use) == 'boolean') {
    force = use;
  } else if(!!use) {
    force = false;
    current = use;
    var logs = current.ucsLog;
    if(!logs) logs = new dock.cvUCSLogs();
    setCallbackResult(logs);
  }
  println("Refresh UCS fault log " + force + ' - ' + document.documentURI);
  if(force) {
    dock.waitIcon();
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'getLogUCS', [current.get('uniqueKey'), 0], function(code, data) {
        if(code == 0) {
          setCallbackResult(data.logs);
        } else {
          dock.jsonCallError(this, code, data);
        }
      }, current);
      waiting += 1;
  }
}

function sortBy(event) {
  var index = 1;
  if(typeof(event) == 'number') {
    index = event;
  } else {
    var item = event.target;
    index = parseInt(item.getAttribute('sort'));
  }
  starSort(index);
  dock.generalSettings.ucsfaultsort = index;
  waiting += 1;
  dock.waitIcon();
  if(workresult) setCallbackResult(worknodes, workresult);
}


function setCallbackResult(newLogs) {
  current.ucsLog = newLogs;
  faultLogs(current.ucsLog.faultInst);
}

function faultLogs(faults) {
  var timecol = document.getElementById('sort1');

  var sortedlist = new Array();
  for(var i = 0; i < faults.length; i++) {
    var one = [0, faults[i].lastTransition, faults[i].severity, faults[i].code, faults[i].affectedDN, faults[i].cause, faults[i].descr];
    one[0] = one[dock.generalSettings.ucsfaultsort];
    if(typeof(one[0]) == 'string') one[0] = one[0].toLowerCase();
    sortedlist.push(one);
  }
  sortedlist.sort();
  
  rows = new Array();
  for(var i0 = 0; i0 < sortedlist.length; i0++) {
    var time = sortedlist[i0][1];
    var severity = sortedlist[i0][2];
    var code = sortedlist[i0][3];
    var dn = sortedlist[i0][4];
    var cause = sortedlist[i0][5];
    var description = sortedlist[i0][6];
   
    var row = document.createElement("row");
    row.setAttribute('align', 'center');
    row.setAttribute("height", "19px");
    if(i0 % 2 == 0) row.setAttribute("style", "background-color: #dcdcdc;");

    var label = document.createElement("label");
    label.setAttribute('value', time);
    row.appendChild(label);
    var label = document.createElement("label");
    label.setAttribute('value', severity);
    row.appendChild(label);
    var label = document.createElement("label");    
    label.setAttribute('value', code);      
    row.appendChild(label);    
    var label = document.createElement("label");    
    label.setAttribute('value', dn);      
    row.appendChild(label);    
    var label = document.createElement("label");
    label.setAttribute('value', cause);
    row.appendChild(label);    
    var label = document.createElement("label");
    label.setAttribute('value', description);
    row.appendChild(label);    
/*
    var label = document.createElement('button');
    label.setAttribute("label", 'Show');
    label.setAttribute("class", 'graybutton');
    label.setAttribute("oncommand", 'burnLog(' + nodes[i] + ');');
    row.appendChild(label); */
    rows.push(row);
  }
  if(waiting > 0) {
    dock.unwaitIcon();
    waiting -= 1;
  }
  var n = Math.max(1, Math.ceil(rows.length / perPage));
  createPageMenu(n);
  displayPage();
}

function displayPage(index) {
  page = isundefined(index, page);
  var list = document.getElementById('list');
  while(list.childNodes.length > 2) list.removeChild(list.lastChild);
  
  var b = Math.max(perPage * (page - 1), 0);
  var e = Math.min(perPage * page - 1, rows.length);
//   println('displayPage ' + page + ' [' + b + ',' + e + ')');
  for(var i = b; i < e; i++) {
    list.appendChild(rows[i]);
  }
  updatePageMenu(page);
}

function createPageMenu(n) {
  var pagespopup = document.getElementById("pages");
  var m = pagespopup.childNodes.length;
  for(var i = n; i < m; i++) pagespopup.removeChild(pagespopup.lastChild);
  for(var i = m + 1; i <= n; i++) {
    var item = document.createElement('menuitem');
    item.setAttribute('page', i);
    item.setAttribute('label', i);
    pagespopup.appendChild(item);
  }
  pages = n;
}

function updatePageMenu(page) {
  var pagespopup = document.getElementById("pages");
  pagespopup.parentNode.selectedIndex = page - 1;
}

function setPage(index) {
//   println("setPage " + index + " [1," + pages + "]");
  if((index >= 1) && (index <= pages)) {
    displayPage(index);
  }
}

function perpagemenu(event) {
  var perPageX = parseInt(event.target.getAttribute('page'));
  if(perPageX != perPage) {
    var old = perPage * (page - 1);
    perPage = perPageX
    var n = Math.max(1, Math.ceil(rows.length / perPageX));    
    createPageMenu(n);
    displayPage(Math.min(1 + Math.floor(old / perPageX), n));
  }
}

function pagemenu(event) {
  setPage(parseInt(event.target.getAttribute('page')));
}

function pagetop() {
  setPage(1);
}

function pagebottom() {
  setPage(pages);
}

function pageup() {
  setPage(page - 1)
}

function pagedown() {
  setPage(page + 1);
}

function starSort(index) {
  var i = 1;
  while(true) {
    var obj = document.getElementById('sort' + i);
    if(!obj) break;
    var value = obj.getAttribute('value');
    var star = value.indexOf('*');
    if((index == i) && (star < 0)) {
      obj.setAttribute('value', value + ' *');
    } else if((index != i) && (star >= 0)) {
      star = value.lastIndexOf(' ', star);
      obj.setAttribute('value', value.substr(0, star));
    }
    i++;
  }
}
