var Configuration = require('./configuration.js');

function makeBox(obj) {
  return {
    object : obj
  }
}

function makeNodes(cluster, ctx, cat) {
  var nodes = [
    new ctx.cvObject(cluster, new ctx.cvPhysicalNode()),
    new ctx.cvObject(cluster, new ctx.cvPhysicalNode()),
    new ctx.cvObject(cluster, new ctx.cvPhysicalNode()),
    new ctx.cvObject(cluster, new ctx.cvPhysicalNode())
  ];
  for(var i = 0; i < 4; i++) {
    nodes[i].object.hostname           = 'node00' + (i+1);
    nodes[i].status                    = { status : 'UP' };
    nodes[i].sysInfo                   = { processors : 4, 
                                           memoryTotal : 4294967296 },
    nodes[i].object.category           = cat.object.uniqueKey;
    nodes[i].object.softwareImageProxy = cat.object.softwareImageProxy.uniqueKey;
  }
  return nodes;
}

function makeSoftwareImage(cluster, ctx, name, cat) {
  var software_image = new ctx.cvObject(cluster, new ctx.cvSoftwareImage());
  software_image.object.name = 'default-image';
  software_image.object.path = '/cm/images/default-image';
  return software_image;
}

function makeNodeCategory(cluster, ctx, name, software_image) {
  var cat = new ctx.cvObject(cluster, new ctx.cvNodeCategory());
  var software_image_proxy = new ctx.cvObject(cluster, new ctx.cvSoftwareImageProxy());
  cat.object.softwareImageProxy.parentSoftwareImage = software_image.object.uniqueKey;
  cat.object.disksetup = '<diskSetup xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="schema.xsd">' +
    '  <device>' +
    '      <blockdev>/dev/sda</blockdev>' + 
    '      <partition id="a1">' + 
    '        <size>max</size>' + 
    '        <type>linux</type>' + 
    '        <filesystem>ext3</filesystem>' + 
    '        <mountPoint>/</mountPoint>' +
    '        <mountOptions>defaults,noatime,nodiratime</mountOptions>' + 
    '      </partition>' + 
    '  </device>' +
    '</diskSetup>';
  return cat;
}

function makeNodeCategoryDiskless(cluster, ctx, name, software_image) {
  var cat = new ctx.cvObject(cluster, new ctx.cvNodeCategory());
  var software_image_proxy = new ctx.cvObject(cluster, 
					      new ctx.cvSoftwareImageProxy());
  cat.object.softwareImageProxy.parentSoftwareImage = software_image.object.uniqueKey;
  cat.object.disksetup = 
      '<?xml version="1.0" encoding="UTF-8"?>' + 
      '<diskSetup>' +
      '  <diskless maxMemSize="90%"/>' +
      '</diskSetup>';
  return cat;
}

function ConfigurationWith4Nodes(context) {
  Configuration.call(this, context);
  var software_image = makeSoftwareImage(this.cluster, this.context.context, 'default-image');
  this.cluster.objects['SoftwareImage'] = [ software_image ];

  this.defaultCategory = makeNodeCategory(this.cluster, 
					  this.context.context, 
					  'default', 
					  software_image);

  this.disklessCategory = makeNodeCategoryDiskless(this.cluster, 
						   this.context.context, 
						   'diskless', software_image);


  this.cluster.objects['NodeCategory']  = [ this.defaultCategory,
					    this.disklessCategory ];
  this.nodes = makeNodes(this.cluster, 
			 this.context.context,
			 this.defaultCategory);
  this.cluster.objects['PhysicalNode']  = this.nodes;
  this.cluster.objects['Node']          = this.nodes;
  this.cluster.objects['SlaveNode']     = this.nodes;
  this.initClusterObjects();
}

ConfigurationWith4Nodes.prototype = Object.create(Configuration.prototype,{
});

module.exports = ConfigurationWith4Nodes;
