'user strict';
global.__base = __dirname + '/../';
var test  = require('unit.js');
var Context = require('cm-context');
var context = new Context(global.__base);

var diskless_setup = 
    '<?xml version="1.0" encoding="UTF-8"?>' + 
    '<diskSetup>' +
    '  <diskless maxMemSize="90%"/>' +
    '</diskSetup>';

var semi_diskless_setup = 
  '<diskSetup xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="schema.xsd">' +
  '  <diskless maxMemSize="0"></diskless>' +
  '  <device>' +
  '      <blockdev>/dev/sda</blockdev>' + 
  '      <partition id="a1">' + 
  '      <size>max</size>' + 
  '      <type>linux</type>' + 
  '      <filesystem>ext3</filesystem>' + 
  '      <mountPoint>/localmount</mountPoint>' +
  '      <mountOptions>defaults,noatime,nodiratime</mountOptions>' + 
  '      </partition>' + 
  '  </device>' +
  '</diskSetup>';

var semi_diskless_setup_root_in_ram = 
  '<diskSetup xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="schema.xsd">' +
  '  <diskless maxMemSize="0"></diskless>' +
  '  <device>' +
  '      <blockdev>/dev/sda</blockdev>' + 
  '      <partition id="a1">' + 
  '        <size>max</size>' + 
  '        <type>linux</type>' + 
  '        <filesystem>ext3</filesystem>' + 
  '        <mountPoint>/</mountPoint>' +
  '        <mountOptions>defaults,noatime,nodiratime</mountOptions>' + 
  '      </partition>' + 
  '  </device>' +
  '</diskSetup>';


var jsondisksetup = context.load('jsondisksetup.js')
describe('isRootFSDiskless', function(){
  it('diskless implies semi-diskless', function(){
    test.bool(jsondisksetup.isFSSemiDiskless(diskless_setup)).isTrue();
  });
  it('diskless layout xml / is disklesss', function(){
    test.bool(jsondisksetup.isRootFSDiskless(diskless_setup)).isTrue();
  });
  it('diskless layout xml: /var/lib is diskless', function(){
    test.bool(jsondisksetup.isPathDiskless(diskless_setup, "/var/lib")).isTrue();
  });
  it('semi-diskless with root FS diskless return true', function(){ 
    test.bool(jsondisksetup.isRootFSDiskless(semi_diskless_setup)).isTrue();
  });
  it('semi-diskless: / is diskless', function(){ 
    test.bool(jsondisksetup.isPathDiskless(semi_diskless_setup, "/")).isTrue();
  });
  it('semi-diskless: /var/lib is diskless', function(){ 
    test.bool(jsondisksetup.isPathDiskless(semi_diskless_setup, "/var/lib")).isTrue();
  });

  it('semi-diskless: /localmount/ is not diskless', function(){ 
    test.bool(jsondisksetup.isPathDiskless(semi_diskless_setup, "/localmount")).isFalse();
  });
  it('semi-diskless: /localmount/subdir is not diskless', function(){ 
    test.bool(jsondisksetup.isPathDiskless(semi_diskless_setup, "/localmount/subdir")).isFalse();
  });
  it('diskless layout xml should return true', function(){
    test.bool(jsondisksetup.isPathDiskless(diskless_setup, "/var/lib")).isTrue();
  });
  it('semi_diskless with root FS not diskless return false', function(){
    test.bool(jsondisksetup.isRootFSDiskless(semi_diskless_setup_root_in_ram)).isFalse();
  });
})
