/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'User';
var singular = 'user';
var plural = 'users';
var editfile = 'edituser.xul';

function createTreeItem(one) {
  var item = new Object();
  item['key'] = one.get('uniqueKey');
  item['modified'] = one.get('modified');
  item['name'] = one.get('userID');
  item['id'] = one.get('userIDNumber');
  item['groupid'] = one.get('groupIDNumber');
  item['fullname'] = one.get('commonName');
  item['home'] = one.get('homeDirectory');
  item['profile'] = one.get('profile');
  return item;
}

function addObject() {
  var args = new Object();
  args['cluster'] = current;
  args['object'] = 0;
  var d = new dock.cvDialog(editfile, args);
  d.onOk = function() {
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function editObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a " + singular);
  } else if(select.length > 1) {
    dock.displayError("Please select only one " + singular);
  } else {
    var args = new Object();
    args['cluster'] = current;
    args['object'] = select[0];
    var d = new dock.cvDialog(editfile, args);
    d.onOk = function() {
      refresh();
    }
    dock.dialogs.showDialog(d);
  }
}

function removeUserObject() {
  var select = getSelected();
  if(select.length == 0) {
    dock.displayError('Please select a ' + singular);
  } else {
    var cluster = current;
    if(!!current.cluster) cluster = current.cluster;
    var args = new Object();
    if(select.length == 1) {
      args['question'] = 'Are you sure you want to remove this ' + singular + '?';
    } else {
      args['question'] = 'Are you sure you want to remove these ' + plural + '?';
    }
    args['check'] = 'Also remove home directory ';
    var d = new dock.cvDialog('question.xul', args);
    d.onOk = function(alsoHome) {
      for(var i = 0; i < select.length; i++) cluster.removeObject(select[i], alsoHome);
    }
    dock.dialogs.showDialog(d);
  }
}

function saveUserList() {
  var list = current.find(basetype);
  for(var i = 0; i < list.length; i++) {
    if(list[i].get('modified')) {
      list[i].commit();
    }
  }
  refresh();
}

function removeObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a " + singular);
  } else {
    var args = new Object();
    if(select.length == 1) {
      args['question'] = "Are you sure you want to remove this " + singular + "?";
      args['check'] = "Remove home directory";
    } else {
      args['question'] = "Are you sure you want to remove these " + plural + "?";
      args['check'] = "Remove home directories";
    }
    args['checked'] = true;
    var d = new dock.cvDialog("question.xul", args);
    d.onOk = function(delHome) {
      for(var i = 0; i < select.length; i++) {
        current.removeObject(select[i], delHome);
      }
      parent.refresh(); // also group
    }
    dock.dialogs.showDialog(d);
  }
}

function loadUserList() {
  var list = current.find(basetype);
  for(var i = 0; i < list.length; i++) {
    if(list[i].get('modified')) {
      if(list[i].get('uniqueKey') < dock.MAX_LOCAL_KEY) {
        list[i].remove();
      }
    } else if(list[i].get('uniqueKey') < dock.MAX_LOCAL_KEY) {
      println("This should not be!");
    }
  }
  current.refresh('getUsers');
  refresh();
}

function postrefresh() {
  // checks LDAP connectivity
  var list = current.find(basetype);
  if (list.length == 0) {
    if (current.LDAPstatus.checked == undefined) {
      println('found 0 users, checking LDAP');
      current.checkLDAP();
    } else {
      println('found 0 users, LDAP already checked with status: ' + current.LDAPstatus.errorstring);
    }
  }
  return true;
}
