/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var RunTime = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime);
var dirServiceProvider = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIDirectoryServiceProvider);

var daysInMonth = new Array(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
var myVersion = '';
var cacheDir;
var workD = 0;
var homeD = 0;
var tmpD = 0;

var oct0600 = parseInt('0600', 8); // octal literals and octal escape sequences are deprecated

// ===== Hadoop-related roles =====
// --------------------------------
// mapping between role's baseType and Hadoop service name; 18 (Hadoop-)roles as of 25.03.15
var hadoopTypesToRoles = { "HDFS-movable": { "HadoopDataNodeRole":true, "HadoopNFSGatewayRole":true },
                           "HDFS-non-movable": { "HadoopNameNodeRole":true, "HadoopSecondaryNameNodeRole":true },
                           "HDFS-HA-movable": { "HadoopDataNodeRole":true, "HadoopJournalRole":true, "HadoopNFSGatewayRole":true },
                           "HDFS-HA-non-movable": { "HadoopNameNodeRole":true },
                           "MRv1-movable": { "HadoopTaskTrackerRole":true },
                           "MRv1-non-movable": { "HadoopJobTrackerRole":true },
                           "YARN-movable": { "HadoopYARNClientRole":true },
                           "YARN-non-movable": { "HadoopYARNServerRole":true },
                           "HBase-movable": { "HadoopHBaseClientRole":true },
                           "HBase-non-movable": { "HadoopHBaseServerRole":true },
                           "Spark-movable": { "HadoopSparkWorkerRole":true },
                           "Spark-non-movable": { "HadoopSparkMasterRole":true },
                           "Spark-YARN": { "HadoopSparkYARNRole":true },
                           "HDFS": { "HadoopDataNodeRole":true, "HadoopNameNodeRole":true, "HadoopSecondaryNameNodeRole":true, "HadoopJournalRole":true, "HadoopNFSGatewayRole":true },
                           "MRv1": { "HadoopTaskTrackerRole":true, "HadoopJobTrackerRole":true },
                           "YARN": { "HadoopYARNClientRole":true, "HadoopYARNServerRole":true },
                           "HBase": { "HadoopHBaseClientRole":true, "HadoopHBaseServerRole":true },
                           "Spark": { "HadoopSparkWorkerRole":true, "HadoopSparkMasterRole":true, "HadoopSparkYARNRole":true },
                           "ZooKeeper": { "HadoopZooKeeperRole":true },
                           "Hive": { "HadoopHiveRole":true },
                           "Sqoop": { "HadoopSqoopRole":true },
                           "AccumuloMaster": { "HadoopAccumuloMasterRole":true },
                           "AccumuloTablet": { "HadoopAccumuloTabletRole":true },
                           "Drill": { "HadoopDrillRole":true },
                           "Pig": { "HadoopPigRole":true },
                           "FlinkJobManager": { "HadoopFlinkJobManagerRole":true },
                           "FlinkTaskManager": { "HadoopFlinkTaskManagerRole":true },
                           "StormNimbus": { "HadoopStormNimbusRole":true },
                           "StormSupervisor": { "HadoopStormSupervisorRole":true },
                           "KafkaServer": { "HadoopKafkaServerRole":true },
                           "KMS": { "HadoopKMServerRole":true },
                           "Zeppelin": { "HadoopZeppelinRole":true },
                           "Cassandra": { "HadoopCassandraRole":true },
                           "AlluxioMaster": { "HadoopAlluxioMasterRole":true },
                           "AlluxioWorker": { "HadoopAlluxioWorkerRole":true }
                         };

// links Hadoop/Spark role types as they are called in cmdaemon to corresponding service names
// used by onvertRoleType2ServiceName()
// add more role types, not necessarily Hadoop/Spark ones, if needed 
var hadoopservicenames = { HadoopDataNodeRole:"HDFS DataNode",
                           HadoopNameNodeRole:"HDFS NameNode",
                           HadoopSecondaryNameNodeRole:"HDFS SecondaryNameNode",
                           HadoopJournalRole:"HDFS JournalNode",
                           HadoopNFSGatewayRole:"HDFS NFS Gateway",
                           HadoopJobTrackerRole:"MRv1 JobTracker",
                           HadoopTaskTrackerRole:"MRv1 TaskTracker",
                           HadoopYARNServerRole:"YARN ResourceManager",
                           HadoopYARNClientRole:"YARN NodeManager",
                           HadoopHBaseServerRole:"HBase MasterServer",
                           HadoopHBaseClientRole:"HBase RegionServer",
                           HadoopZooKeeperRole:"ZooKeeper",
                           HadoopKMServerRole:"Hadoop KMS",
                           HadoopSparkMasterRole:"Spark Master",
                           HadoopSparkWorkerRole:"Spark Worker",
                           HadoopSparkYARNRole:"Spark YARN",
                           HadoopHiveRole:"Hive",
                           HadoopAccumuloMasterRole:"Accumulo Master",
                           HadoopAccumuloTabletRole:"Accumulo Tablet",
                           HadoopDrillRole:"Drill",
                           HadoopPigRole:"Pig",
                           HadoopFlinkJobManagerRole:"Flink Job Manager",
                           HadoopFlinkTaskManagerRole:"Flink Task Manager",
                           HadoopStormNimbusRole:"Storm Nimbus",
                           HadoopStormSupervisorRole:"Storm Supervisor",
                           HadoopKafkaServerRole:"Kafka Server",
                           HadoopSqoopRole:"Sqoop",
                           HadoopZeppelinRole:"Zeppelin",
                           HadoopCassandraRole:"Cassandra",
                           HadoopAlluxioMasterRole:"AlluxioMaster",
                           HadoopAlluxioWorkerRole:"AlluxioWorker"
                         };

// wrapper for hadoopservicenames listing; 
function convertRoleType2ServiceName(roleType) {
  return (typeof hadoopservicenames[roleType] === "undefined")  ? roleType : hadoopservicenames[roleType]; 
}



// -------------END----------------
// ===== Hadoop-related roles =====


/* Special javascript function */
function $() {
  var elements = new Array();
  for (var i = 0; i < arguments.length; i++) {
    var element = arguments[i];
    if (typeof element == 'string')
      element = document.getElementById(element);
    if (arguments.length == 1)
      return element;
    elements.push(element);
  }
  return elements;
}

/* Parse global identifier
 * object:IP:port:major:minor:...
 */
function parseGlobalID(ID) {
  var x = ID.split(":");
  return x;
}

function string_case_equal(s1, s2) {
  return s1.toLowerCase() == s2.toLowerCase();
}

function ucfirst(s) {
  if(!s || (s.length == 0)) return s;
  return s.charAt(0).toUpperCase() + s.substr(1);
}

function endswith(s1, s2) {
  try{
    if(s1.length >= s2.length) return (s1.substr(s1.length - s2.length) == s2);
  } catch(e) {}
  return false;
}

function startswith(s1, s2) {
  try {
    if(s1.length >= s2.length) return (s1.substr(0, s2.length) == s2);
  } catch(e) {}
  return false;
}

/* print 'x' in 'n' or more charcters with zeros in front */
function zeros(x, n) {
  var t = Math.max(1, Math.floor(Math.log(x) / Math.log(10)) + 1);
  var s = "";
  for(var i = t; i < n; i++) {
    s += "0";
  }
  return (s + x);
}

/* returns rainbow color scheme
 * input [0..1]
 */
function colorbar(x) {
  x = (1 - Math.max(0, Math.min(1, x))) * 240;
  var rgb = hsv2rgb(x, 0.7, 1);
  return "rgb(" + rgb[0] + ", " + rgb[1] + ", " + rgb[2] + ")"
}

function convert2hex(c) {
  var h = c.toString(16);
  if(c < 16) h = '0' + h;
  return h;
}

function colorhash(x) {
  x = (1 - Math.max(0, Math.min(1, x))) * 240;
  var rgb = hsv2rgb(x, 0.7, 1);
  return "#" + convert2hex(rgb[0]) + convert2hex(rgb[1]) + convert2hex(rgb[2]);
}

/* convert rgb to hsv */
function rgb2hsv(r, g, b) {
  var i = Math.min(r, g, b);
  var v = Math.max(r, g, b);
  var s, h;
  if(v == 0) {
    s = 0;
  } else {
    s = 1 - i / v;
  }
  if(i == v) {
     h = 0;
  } else {
    if(v == r) {
      h = 60 * (G - B) / (v - i);
      if(g < b) { h += 360; }
    } else if(v == g) {
      h = 60 * (B - R) / (v - i) + 120;
    } else {
      h = 60 * (R - G) / (v - i) + 240;
    }
  }
  return [h, s, v];
}

/* convert hsv to rgb */
function hsv2rgb(h, s ,v) {
  var i = Math.floor(h / 60) % 6;
  var f = (h / 60) - i;
  var p = v * (1 - s);
  var q = v * (1 - f * s);
  var t = v * (1 - (1 - f) * s);
  var r, g, b;
  if(i == 0) {
    r = v; g = t; b = p;
  } else if(i == 1) {
    r = q; g = v; b = p;
  } else if(i == 2) {
    r = p; g = v; b = t;
  } else if(i == 3) {
    r = p; g = q; b = v;
  } else if(i == 4) {
    r = t; g = p; b = v;
  } else if(i == 5) {
    r = v; g = p; b = q;
  }
  return [Math.floor(r * 255), Math.floor(g * 255), Math.floor(b * 255)];
}

function udef(obj) {
  if((typeof obj) == "undefined") return true;
  if(obj == "undefined") return true;
  return false;
}

/* returns array index of element
 * or -1 otherwise
 */
Array.prototype.find = function(element) {
  for (var keys in this) {
    if (this[keys] == element) {
      return keys;
    }
  }
  return -1;
};

Array.prototype.remove = function(element) {
  for(var i = 0; i < this.length; i++) {
    if(this[i] == element) {
      this.splice(i, 1);
      return true;
    }
  }
  return false;
};

Array.prototype.removeall = function(element) {
  for(var i = 0; i < this.length; i++) {
    if(this[i] == element) {
      this.splice(i, 1);
    }
  }
  return this;
}

Array.prototype.diff = function(a) {
  return this.filter(function(i) {return !(a.indexOf(i) > -1);});
};


String.prototype.trim = function() {
  return this.replace(/^\s+|\s+$/g,"");
}
String.prototype.ltrim = function() {
  return this.replace(/^\s+/,"");
}
String.prototype.rtrim = function() {
  return this.replace(/\s+$/,"");
}

/* returns next permutation of list */
function permutList(list, permut) {
  var n = list.length;
  if(!permut) {
    var r = new Array();
    for(var i = 0; i < n; i++) r.push(0);
    return r;
  } else {
    var c = n - 1;
    do {
      if((++permut[c]) == list[c].length) {
        permut[c--] = 0;
      } else {
        return permut;
      }
    } while(c >= 0)
    return null;
  }
}

function string_case_compare(a, b) {
  return a.toLowerCase() == b.toLowerCase();
}

/* converts first character of string to uppercase */
function firstUpper(x) {
  if((typeof(x) != 'string') || (x.length == 0)) {
    return x;
  } else {
    return x.substr(0,1).toUpperCase() + x.substr(1);
  }
}

function firstLower(x) {
  if((typeof(x) != 'string') || (x.length == 0)) {
    return x;
  } else {
    return x.substr(0,1).toLowerCase() + x.substr(1);
  }
}

/* returns the number of settings for an object */
function countparam(obj) {
  var i=0;
  for(var j in obj) i++;
  return i;
}

function getYesNoCancel(titlebar, question, okFunc, noFunc, cancelFunc) {
  var args = {title: titlebar, question: question, ok: okFunc, no: noFunc, cancel: cancelFunc};
  window.open("yesno.xul", "yes-no-cancel-dialog", "dialog,chrome,centerscreen,modal", args);
}

function isundefined(x, def) {
  return (typeof x == 'undefined') ? def : x;
}

function rebranding(x, def) {
  if(x == dock.DEFAULT_BRANDING) {
    return def;
  } else {
    return x;
  }
}

function formatTime(time, seconds) {
  var d = new Date();
  d.setTime(time * 1000);
  seconds = isundefined(seconds, false);
  if(seconds) {
    return d.format("HH:MM:ss");
  } else {
    return d.format("HH:MM");
  }
}

function formatDateShort(time) {
  var d = new Date();
  d.setTime(time * 1000);
  return d.format("mmm dd");
}

function formatDate(time) {
  var d = new Date();
  d.setTime(time * 1000);
  return d.format("dd/mmm/yyyy");
}

function formatDateTime(time) {
  var d = new Date();
  d.setTime(time * 1000);
  return d.format("dd/mmm/yyyy HH:MM:ss");
}

// This code is in the public domain. Feel free to link back to http://jan.moesen.nu/
function sprintf() {
  if (!arguments || arguments.length < 1 || !RegExp) return "";
  var str = arguments[0];
  var re = /([^%]*)%('.|0|\x20)?(-)?(\d+)?(\.\d+)?(%|b|c|d|u|f|o|s|x|X)(.*)/;
  var a = [], b = [], numSubstitutions = 0, numMatches = 0;
  while((a = re.exec(str)) != null) {
    var leftpart = a[1], pPad = a[2], pJustify = a[3], pMinLength = a[4];
    var pPrecision = a[5], pType = a[6], rightPart = a[7];
//    printall(a);
    numMatches++;
    if (pType == '%') {
      subst = '%';
    } else {
      numSubstitutions++;
      if(numSubstitutions >= arguments.length) {
        alert('Error! Not enough function arguments (' + (arguments.length - 1) + ', excluding the string)\nfor the number of substitution parameters in string (' + numSubstitutions + ' so far).');
      }
      var param = arguments[numSubstitutions];
      var pad = '';
      if (pPad && pPad.substr(0,1) == "'") pad = leftpart.substr(1,1);
      else if (pPad) pad = pPad;
      var justifyRight = true;
      if (pJustify && pJustify == "-") justifyRight = false;
      var minLength = -1;
      if (pMinLength) minLength = parseInt(pMinLength);
      var precision = -1;
      if (pPrecision && pType == 'f') precision = parseInt(pPrecision.substring(1));
      var subst = param;
      if (pType == 'b') subst = parseInt(param).toString(2);
      else if (pType == 'c') subst = String.fromCharCode(parseInt(param));
      else if (pType == 'd') subst = parseInt(param) ? parseInt(param) : 0;
      else if (pType == 'u') subst = Math.abs(param);
      else if (pType == 'f') subst = (precision > -1) ? Math.round(parseFloat(param) * Math.pow(10, precision)) / Math.pow(10, precision): parseFloat(param);
      else if (pType == 'o') subst = parseInt(param).toString(8);
      else if (pType == 's') subst = param;
      else if (pType == 'x') subst = ('' + parseInt(param).toString(16)).toLowerCase();
      else if (pType == 'X') subst = ('' + parseInt(param).toString(16)).toUpperCase();

      subst = '' + subst;
      if(pMinLength > subst.length) {
        var n = pMinLength - subst.length;
        if(typeof(pPad) == 'undefined') {
          for(var i = 0; i < n; i++) subst = ' ' + subst;
        } else {
          for(var i = 0; i < n; i++) subst = pPad + subst;
          if(subst[n] == '-') {
            subst[n] = pPad;
            subst[0] = '-';
          }
        }
      }
    }
    str = leftpart + subst + rightPart;
  }
  return str;
}

function clone(what) {
  var p = function(){};
  p.prototype = what;
  var obj = new p();
  for(var e in obj){
    if(typeof(obj[e]) == "object") {
      obj[e] = clone(what[e]);
    } else if(isArray(obj[e])) {
      obj[e] = what[e].slice();
    } else {
      obj[e] = what[e];
    }
  }
  return obj;
}

function loadScript(filename) {
  var jsLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
      .getService(Components.interfaces.mozIJSSubScriptLoader);
  try {
    if(!startswith(filename, "chrome:")) {
      filename = "chrome://cmgui/content/" + filename;
    }
    println("Load script: " + filename);
    jsLoader.loadSubScript(filename);
    println("Loaded");
    return true;
  } catch(e) {
    println(e);
  }
  return false;
}

/* parse url
 * master.cluster                      ->   ['https', 'master.cluster', '8081']
 * master.cluster:8080                 ->   ['http', 'master.cluster', '8080']
 * master.cluster:8081                 ->   ['https', 'master.cluster', '8081']
 * master.cluster:XXXX                 ->   ['https', 'master.cluster', 'XXXX']
 * http://master.cluster               ->   ['http', 'master.cluster', '8080']
 * https://master.cluster:8081         ->   ['https', 'master.cluster', '8081']
 * master.cluster/a/b/c                ->   ['https', 'master.cluster', '8081', 'a', 'b', 'c']
 * master.cluster:8080/a/b/c           ->   ['http', 'master.cluster', '8080', 'a', 'b', 'c']
 * master.cluster:8081/a/b/c           ->   ['https', 'master.cluster', '8081', 'a', 'b', 'c']
 * master.cluster:XXXX/a/b/c           ->   ['https', 'master.cluster', 'XXXX', 'a', 'b', 'c']
 * http://master.cluster/a/b/c         ->   ['http', 'master.cluster', '8080', 'a', 'b', 'c']
 * https://master.cluster:8081/a/b/c   ->   ['https', 'master.cluster', '8081', 'a', 'b', 'c']
 */
function parseUrl(url) {
  var x = url.split(':');
  if(startswith(x[x.length - 1], '//')) {
    var y = x[x.length - 1].substr(2).split('/');
  } else {
    var y = x[x.length - 1].split('/');
  }

  if(x.length == 1) {
    var y0 = y[0];
    y.shift();
    y.unshift(8081);
    y.unshift(y0);
    y.unshift('https');
    return y;
  } else if(x.length == 2) {
    if((x[0] == 'https') && startswith(x[1], '//')) {
      var y0 = y[0];
      y.shift();
      y.unshift(8081);
      y.unshift(y0);
      y.unshift('https');
      return y;
    } else if((x[0] == 'http') && startswith(x[1], '//')) {
      var y0 = y[0];
      y.shift();
      y.unshift(8080);
      y.unshift(y0);
      y.unshift('http');
      return y;
    } else if((x[0] == 'key') && startswith(x[1], '//')) {
      var y0 = y[0];
      y.shift();
      y.unshift(-1);
      y.unshift(y0);
      y.unshift('key');
      return y;
    } else if(isNumeric(y[0], false)) {
      if(y[0] == '8080') {
        y.unshift(x[0]);
        y.unshift('http');
      } else {
        y.unshift(x[0]);
        y.unshift('https');
      }
      return y;
    } else {
      return [];
    }
  } else if(x.length == 3) {
    if(((x[0] == 'http') || (x[0] == 'https')) && startswith(x[1], '//') && isNumeric(y[0], false)) {
      y.unshift(x[1].substr(2));
      y.unshift(x[0]);
      return y;
    }
  }
  return [];
}

function isNumeric(txt, neg, percent, firstpercent, comma, bytes) {
  var type = typeof(txt);
  if(type == "undefined") return false;
  if(type == "number") return true;
  if(type == "boolean") return false;
  if(type == "object") return false;
  if(isArray(txt)) return false;
  txt = trim(txt);
  var tl = txt.length;
  if(tl == 0) return false;

  neg = isundefined(neg, true);
  percent = isundefined(percent, false);
  firstpercent = isundefined(firstpercent, false);
  comma = isundefined(comma, false);
  bytes = isundefined(bytes, false);
  var num = "0123456789.";
  var b = 0;
  if(startswith(txt, '-') && neg) {
    b = 1;
  }

  for(var i = b; i < tl; i++) {
    if(num.indexOf(txt[i]) < 0) {
      while(i < tl) {
        if(txt[i] != ' ') break;
        i++;
      }
      if(percent && ((txt[i] == '%') && ((firstpercent && (i > 0)) || (!firstpercent && (i == tl - 1))))) {
        return 1;
      } else if(comma && (txt[i] == ',')) {
        return 1;
      } else if(bytes && (txt[tl - 1] == 'b')) {
        return 2;
      } else {
        return 0;
      }
    }
  }
  return 1;
}

function isNumericKMG(txt, neg) {
  if(typeof(txt) == "undefined") return false;
  if(typeof(txt) == "boolean") return false;
  if(txt.length == 0) return false;

  neg = isundefined(neg, true);
  txt = trim(txt);
  var num = "0123456789.";
  var b = 0;
  if(startswith(txt, '-') && neg) {
    b = 1;
  }

  for(var i = b; i < txt.length; i++) {
    if(num.indexOf(txt[i]) < 0) {
      if(i == txt.length - 1) {
        var c = txt[i].toLowerCase();
        if((c == 'k') || (c == 'm') || (c == 'g')) return true;
      }
      return false;
    }
  }
  return true;
}

function fromKMG(txt) {
  txt = trim(txt);
  var value;
  var c = txt[txt.length - 1].toLowerCase();
  if(c == 'k') {
    value = myParseFloat(txt.substr(0, txt.length - 1));
    value *= 1024;
  } else if(c == 'm') {
    value = myParseFloat(txt.substr(0, txt.length - 1));
    value *= 1024 * 1024;
  } else if(c == 'g') {
    value = myParseFloat(txt.substr(0, txt.length - 1));
    value *= 1024 * 1024 * 1024;
  } else {
    value = myParseFloat(txt);
  }
  return value;
}

function trim(str) {
  if(typeof(str) == "string") return str.replace(/^\s+/g, '').replace(/\s+$/g, '');
  return str;
}

function string2byte(x) {
  var y = new Array(x.length);
  for (var i=0; i < x.length; i++) y[i] = x.charCodeAt(i);
  return y;
}

function getCommandLine() {
  println(cmdLine.findFlag("jsconsole", false));
}

/* Get the OS name */
function getOS() {
  return RunTime.OS;
}

function setVersion(version) {
  var slash = getDirectorySeparator();
  var filename = getHomeDirectory() + slash;
  filename += ".cm" + slash + "cmgui" + slash;
  filename += "version.dat";
  savefile(filename, version);
}

/* Get home directory */
function getHomeDirectory() {
  if(homeD == 0) {
    homeD = dirServiceProvider.getFile("Home", new Object()).path + getDirectorySeparator();
  }
  return homeD;
}

/* Get temp directory */
function getTempDirectory() {
  if(tmpD == 0) {
    tmpD = dirServiceProvider.getFile("TmpD", new Object()).path + getDirectorySeparator();
  }
  return tmpD;
}

/* Get working directory */
function getWorkDirectory() {
  if(workD == 0) {
    var os = getOS();
    var tmp = dirServiceProvider.getFile("CurWorkD", new Object()).path;
    if(startswith(os, "WIN")) {
      if(endswith(tmp, "\\bin")) tmp = tmp.substr(0, tmp.length - 4);
    }
    workD = tmp + getDirectorySeparator();
  }
  return workD;
}

/* Get working directory */
function getProcessDirectory() {
  if(workD == 0) {
    var os = getOS();
    var tmp = dirServiceProvider.getFile("CurProcD", new Object()).path;
    if(startswith(os, "WIN")) {
      if(endswith(tmp, "\\bin")) tmp = tmp.substr(0, tmp.length - 4);
    }
    workD = tmp + getDirectorySeparator();
  }
  return workD;
}

function createCacheDirectory() {
  var slash = getDirectorySeparator();
  cacheDir = getHomeDirectory() + slash + ".cm" + slash + "cmgui" + slash + "cache" + slash;
  var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
  file.initWithPath(cacheDir);
  if(!file.exists() || !file.isDirectory()) {   // if it doesn't exist, create
    file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, parseInt("0700", 8));
  }
  return cacheDir;
}

function getCacheDirectory() {
  return cacheDir;
}

function cacheFile(name) {
  var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
  file.initWithPath(getCacheDirectory() + name);
  file.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, parseInt("0600", 8));
  return file;
}


/* Get the directory separator
 *  Windows:    \
 *  Unix, linux: /
 *  Others:      /  ??
 */
function getDirectorySeparator() {
  return (startswith(getOS(), "WIN") ? "\\" : "/");
}

/* determine if a url exists */
function urlExists(url) {
  try {
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open('GET', url, false);
    xmlHttp.send(null);
    if((xmlHttp.readyState==4) || (xmlHttp.readyState=="complete")) {
      return xmlHttp.responseText;
    }
    return false;
  } catch(e) {
    println(e);
    return false;
  }
}

/* Send a mail
 * does not work ik xulrunner 1.8.0.9 (?)
 */
function sendMail(recipient, subject, body) {
  // Taken from browser.js's MailIntegration class
  var mailtoUrl = "mailto:" + recipient + "?body=" + body + "&subject=" + subject;
  var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var uri = ioService.newURI(encodeURI(mailtoUrl), null, null);
  // now pass this url to the operating system
  // a generic method which can be used to pass arbitrary urls to the operating system.
  // aURL --> a nsIURI which represents the url to launch
  var extProtocolSvc = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"].getService(Components.interfaces.nsIExternalProtocolService);
  if (extProtocolSvc)  {
    extProtocolSvc.loadUrl(uri);
  }
}

function postData(url, data) {
  try {
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open('POST', url, false);
    xmlHttp.setRequestHeader('Content-Type','application/x-www-form-urlencoded');
    if(isObject(data)) {
      var text = '';
      for(var x in data) {
        if(text.length > 0) text += "&";
        text += x + "=" + data[x];
      }
      xmlHttp.send(text);
    } else {
      xmlHttp.send(data);
    }
    if((xmlHttp.readyState==4) || (xmlHttp.readyState=="complete")) {
      return xmlHttp.responseText;
    }
    return false;
  } catch(e) {
    println(e);
    return false;
  }
}

function isIPv6(value) {
  var v6 = value.split(':');
  var v4 = v6[v6.length - 1].split('.');

  var done = 0;
  if(v4.length == 4) {
    for(var i = 0; i < v4.length; i++) {
      if((v4[i].length > 3) || (v4[i].length == 0)) return false;
      var v = myParseInt(v4[i]);
      if((v < 0) || (v > 255)) return false;
    }
    done = 1;
  } else if(v4.length != 1) {
    return false;
  }
  if((v6.length > (8 - 2 * done)) || (!done && (v6.length < 3 - done))) {
    return false;
  }
  if(v6.length > done) {
    var pb = 0;
    var dd = false;
    for(var i = 0; i < v6.length - done; i++) {
      if(v6[i].length == 0) {
        pb += 1;
        if(((pb > 1) && (i > 1)) || ((pb > 2) && (i == 2))) return false;
        dd = true;
      } else {
        if((pb == 1) && (i == 1)) return false;
        var v = parseInt(v6[i], 16);
        if(kisNaN(v) || (v > 0xffff) || (v < 0) || (Math.floor(v) != v)) return false;
        pb = 0;
      }
    }
    if(!dd && (v6.length != (8 - 2 * done))) return false;
  }
  return true;
}

function functionRegexMatch(value, format, mod) {
  if(format.length > 0) {
    if(startswith(format, "@FUNCTION@")) {
      var call = format.substr(10) + "('" + value + "'";
      if(typeof(mod) != 'undefined') call += ", '" + mod + "'";
      call += ")"
      return eval(call);
    } else {
      return regexMatch(value, format, mod);
    }
  } else {
    return true;
  }
}

function regexMatch(value, format, mod) {
  if(format.length > 0) {
    try {
      mod = isundefined(mod, "");
      if(mod.length > 0) {
        return value.match(new RegExp(format, mod));
      } else {
        return value.match(new RegExp(format));
      }
    } catch(e) {
      return false;
    }
  } else {
    return true;
  }
}

function createObjectCallback(obj, fn, a, b) {
  return function() {
    fn.apply(obj, [a, b]);
  };
}

function fileWithPath(name) {
  var path;
  if(startswith(getOS(), "WIN")) {
    if(name.length == 0 || ((name.length > 2) && (name[2] == ':'))) return name;
    path = dock.environment.get('PATH').split(';');
  } else {
    if(name.length == 0 || name[0] == '/') return name;
    path = dock.environment.get('PATH').split(':');
  }
  var sep = getDirectorySeparator();
  for(var i = 0; i < path.length; i++) {
    var full = path[i] + sep + name;
    try {
      var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
      file.initWithPath(full);
      if(file.exists() && !file.isDirectory()) return full;
    } catch (e) {
      // bad path
    }
  }
  return name;
}

function runCommand(cmd, args, env) {
  env = isundefined(env, {});
  cmd = fileWithPath(cmd);

  var FileFactory = new Components.Constructor("@mozilla.org/file/local;1","nsILocalFile","initWithPath");
  try {
    var obj_Program = new FileFactory(cmd);
    var obj_Process = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);
    obj_Process.init(obj_Program);
    if(!isArray(args)) {
      if(args == null) {
        args = new Array();
      } else {
        args = [args];
      }
    }
    println('run: ' + cmd + ' ' + args);
    obj_Process.run(false, args, args.length, {});
  } catch (e) {
    dock.displayError(cmd + " not found");
  }
}

function startProcess(args, title) {
  var FileFactory = new Components.Constructor("@mozilla.org/file/local;1","nsILocalFile","initWithPath");
  var os = getOS();

  if(startswith(os, "WIN")) {
    if(!isArray(args)) args = new Array();
    var work = getWorkDirectory();
    var file = getTempDirectory() + "\\rshell.bat";
    var cmd = "@echo off\n";
    cmd += "\"" + work;
    if(args.length > 0) {
      var x = args[0].replace(/\//g, "\\");
      cmd += x;
    }
    cmd += "\"";

    for(var i = 1; i < args.length; i++) {
      var x = args[i] + '';
      x = x.replace(/\//g, "\\");
      cmd += " \"" + x + "\"";
    }
    cmd += "\nexit";
    if(savefile(file, cmd)) {
      var console2 = dock.generalSettings.console2;
      if(console2.length > 2) {
        if(console2[1] != ':') console2 = work + "\\" + console2;
        var filename = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
        filename.initWithPath(console2);
        if(filename.exists()) {
          var local = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
          var process = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);
          args = ["-r", "cmd.exe /k " + file];
          local.initWithPath(console2);
          process.init(local);
          process.run(false, args, args.length);
        } else {
          dock.displayError("Console2 executable not found: " + console2);
        }
      } else {
        var bat = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
        bat.initWithPath(file);
        bat.launch();
      }
    }
  } else if(startswith(os, "Darwin")) { // macosx
    var str_LocalProgram = "/usr/X11/bin/xterm";
    try {
      var obj_Program = new FileFactory(str_LocalProgram);
      var obj_Process = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);
      obj_Process.init(obj_Program);
      if(!isArray(args)) {
        args = new Array();
      } else {
        args.unshift("-e");
      }
      obj_Process.run(false, args, args.length, {});
    } catch (e) {
      dock.displayError(str_LocalProgram + " not found");
    }
  } else {
    var str_LocalProgram = "/usr/bin/xterm";
    try {
      var obj_Program = new FileFactory(str_LocalProgram);
      var obj_Process = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);
      obj_Process.init(obj_Program);
      if(!isArray(args)) {
        args = new Array();
      } else {
        args.unshift("-e");
      }
      if(typeof(title) != 'undefined') {
        args.unshift(title);
        args.unshift('-T');
      }
      obj_Process.run(false, args, args.length, {});
    } catch (e) {
      dock.displayError(str_LocalProgram + " not found");
    }
  }
}

function getDock() {
  var old = 0;
  var dock = window;
  while(dock != old) {
    old = dock;
    if(typeof(dock.dialogs) != 'undefined') {
      break;
    } else if(!!dock.opener) { // Separate window
      println(dock.opener.document.title + " == " + dock.document.title);
      if(!dock.opener.document || (dock.opener.document.title == dock.document.title)) break;
      dock = dock.opener;
    } else if(!!dock.parent) { // docked inside tab
      dock = dock.parent;
    } else {
      break;
    }
  }
  if(typeof(dock.dialogs) == 'undefined') {
    println("Bad dock: " + window.document.title);
  }
  return dock;
}


function moveDialog() {
  var startPos=0;
  var mouseDown = function(event) {
    if (event.button == 0) {
      startPos = [ event.clientX, event.clientY];
    } else {
      startPos=0;
    }
    //cancel default action
    return false;
  }

  var mouseMove = function(event) {
    if (startPos != 0) {
      var newX = event.screenX-startPos[0];
      var newY = event.screenY-startPos[1];
      window.moveTo(newX,newY);
    }
  }

  var mouseUp = function(event) {
    startPos=0;
  }

  document.getElementById('dialogheader').addEventListener("mousedown",mouseDown, false);
  window.addEventListener("mouseup",mouseUp, false);
  window.addEventListener("mousemove",mouseMove, false);
}

function winmin() {
  window.minimize();
}

function winmax() {
  if(window.windowState == STATE_NORMAL) {
    window.maximize();
    var button = document.getElementById('maxrestore');
    if(button) button.setAttribute('class', 'restore-button');
  } else {
    window.restore();
    var button = document.getElementById('maxrestore');
    if(button) button.setAttribute('class', 'maximize-button');
  }
}

function nameNumber(name) {
  var num = "0123456789";
  for(var i = name.length - 1; i >= 0; i--) {
    if(num.indexOf(name[i]) < 0) {
//     var c = name[i];
//     if(c < '0' || c > '9') {
      return [name.substr(0, i + 1), name.substr(i + 1)]; // DO NOT USE PARSEINT!!! parseInt('016') == 14
    }
  }
  return ["", name];
}

// Fixes problem with 0 in front
function myParseInt(text) {
  for(var i = 0; i < text.length; i++) {
    if(text[i] != '0') {
      text = text.substr(i);
      break;
    }
  }
  if(text.length == 0) return 0;
  return parseInt(text);
}

function myParseFloat(text) {
  var dot = text.indexOf('.');
  var a = 0;
  var b = '';
  if(dot > 0) {
    a = myParseInt(text.substr(0, dot));
    b = text.substr(dot + 1);
  } else if(dot == 0) {
    b = text.substr(1);
  } else {
    a = myParseInt(text);
  }
  var n = 0;
  var c = 0;
  var num = "0123456789";
  for(var i = 0; i < b.length; i++) {
    if(num.indexOf(b[i]) >= 0) {
      n++;
      c = 10 * c + parseInt(b[i]);
    } else {
      break;
    }
  }

  return a + c / Math.pow(10, n);
}

function commaListToDotFormat(list, postfix) {
  if(typeof(postfix) != 'undefined') {
    for(var i = 0; i < list.length; i++) {
      list[i] = list[i].replace(postfix, '');
    }
  }
  return dotFormat(list);
}

function dotFormatKeys(cluster, keys) {
  if (!keys) return;
  var list = new Array();
  for(var i = 0; i < keys.length; i++) {
    var obj = cluster.find(keys[i]);
    if(obj) list.push(obj.resolveName());
  }
  return dotFormat(list);
}

function dotFormatObject(objs) {
  var list = new Array();
  for(var i = 0; i < objs.length; i++) {
    list.push(objs[i].resolveName());
  }
  return dotFormat(list);
}

function dotFormat(list) {
  list = list.sort();
  var r = '';
  var count = 0;
  var prev;
  for(var i = 0; i < list.length; i++) {
    var current = nameNumber(list[i]);
    if(count) {
      var b = myParseInt(prev[1]);
      var e = myParseInt(current[1]);
      if((prev[0] != current[0]) || (prev[1].length != current[1].length) || ((e - b) != count)) {
        if(r.length > 0) r += ", ";
        if(count == 1) {
          r += list[i - 1];
        } else if(count == 2) {
          r += list[i - 2] + ", " + list[i - 1];
        } else {
          r += list[i - count] + ".." + list[i - 1];
        }
        prev = current;
        count = 1;
      } else {
        count++;
      }
    } else {
      prev = current;
      count = 1;
    }
  }
  if(count > 0) {
    if(r.length > 0) r += ", ";
    if(count == 1) {
      r += list[i - 1];
    } else if(count == 2) {
      r += list[i - 2] + ", " + list[i - 1];
    } else {
      r += list[i - count] + ".." + list[i - 1];
    }
  }
  return r;
}

function undotFormat(text) {
  var result = [false, new Array()];
  var list = text.split(',');
  for(var i = 0; i < list.length; i++) {
    var idx = list[i].indexOf('..');
    if(idx >= 0) {
      var start = nameNumber(trim(list[i].substr(0, idx)));
      var end = nameNumber(trim(list[i].substr(idx + 2)));
      if(start[0] != end[0]) {
        dock.displayError("Node lists should start with the same name");
        return result;
      } else if((start[0].length > 0) && (start[1].length != end[1].length)) {
        dock.displayError("Node lists should have the same number of digits");
        return result;
      } else if(start[1].length == 0) {
        dock.displayError("Node has no digits");
        return result;
      } else {
        var n = start[1].length;
        start[1] = myParseInt(start[1]);
        end[1] = myParseInt(end[1]);
        for(var j = start[1]; j <= end[1]; j++) {
          var name = '' + j;
          while(name.length < n) name = '0' + name;
          result[1].push(start[0] + name);
        }
      }
    } else {
      result[1].push(trim(list[i]));
    }
  }
  result[0] = true;
  return result;
}

function blank(text) {
  for(var i = 0; i < text.length; i++) {
    if(text[i] != ' ') return false;
  }
  return true;
}

function formatTimespan(value) {
  if(value >= 3600) {
    return sprintf("%f", value / 3600) + ' hours';
  } else if(value >= 60) {
    return sprintf("%f", value / 60) + ' minutes';
  }
  return value;
}

function parseTimespan(text) {
  var i = 0;
  for(; i < text.length; i++) {
    var c = text[i];
    if(((c < '0') || (c > '9')) && (c != '.')) break;
  }
  var n = parseFloat(text.substr(0, i));
  for(; i < text.length; i++) {
    var c = text[i]
    if(c != ' ') break;
  }
  if(i < text.length) {
    var c = text[i].toLowerCase();
    if(c == 'm') {
      n *= 60;
    } else if(c == 'h') {
      n *= 3600;
    }
  }
  return Math.floor(n);
}

function parseKMG(text, neg) {
  neg = isundefined(neg, false);
  var i = 0;
  if(neg && text.length && (text[i] == '-')) i++;
  for(; i < text.length; i++) {
    var c = text[i];
    if(((c < '0') || (c > '9')) && (c != '.')) break;
  }
  var n = parseFloat(text.substr(0, i));
  for(; i < text.length; i++) {
    var c = text[i]
    if(c != ' ') break;
  }
  if(i < text.length) {
    var c = text[i];
    var ci = ((i + 1 < text.length) && (text[i + 1] == 'i'));
    if(c == 'k' || c == 'K') {
      if(ci) {
        n = n * 1024;
      } else {
        n = n * 1000;
      }
    } else if(c == 'm' || c == 'M') {
      if(ci) {
        n = n * 1024 * 1024;
      } else {
        n = n * 1000 * 1000;
      }
    } else if(c == 'g' || c == 'G') {
      if(ci) {
        n = n * 1024 * 1024 * 1024;
      } else {
        n = n * 1000 * 1000 * 1000;
      }
    } else if(c == 't' || c == 'T') {
      if(ci) {
        n = n * 1024 * 1024 * 1024 * 1024;
      } else {
        n = n * 1000 * 1000 * 1000 * 1000;
      }
    } else if(c == 'p' || c == 'P') {
      if(ci) {
        n = n * 1024 * 1024 * 1024 * 1024 * 1024;
      } else {
        n = n * 1000 * 1000 * 1000 * 1000 * 1000;
      }
    }
  }
  return n;
}

function formatKMGselect(size, unit) {
  unit = isundefined(unit, '');
  if(kisNaN(size)) {
    return "N/A";
  } else if(Math.abs(size) < 1e-15) {
    return '0' + unit;
  } else if(size < 1) {
    var s = Math.floor(Math.log(Math.abs(size)) / Math.log(10));
    return sprintf("%." + Math.max(0, -s + 2) + "f", size) + unit;
  } else if((unit == '')) { // optionally add non KMG units here
    if(Math.round(size) == size) {
      return size;
    } else {
      return sprintf("%.2f ", size);
    }
  } else if((unit == 'B') || (unit == 'B/s')) {
    return formatKMG(size) + unit;
  } else if(unit == '-') {
    return formatKMG10(size);
  } else {
    return formatKMG10(size) + unit;
  }
}

function formatKMG(size) {
  if(kisNaN(size)) {
    return "N/A";
  } else if(size >= ((1 << 20) * (1 << 30))) {
    return sprintf("%.2f Pi", size / ((1 << 20) * (1 << 30)));
  } else if(size >= ((1 << 20) * (1 << 20))) {
    return sprintf("%.2f Ti", size / ((1 << 20) * (1 << 20)));
  } else if(size >= (1 << 30)) {
    return sprintf("%.2f Gi", size / (1 << 30));
  } else if(size >> 20) {
    return sprintf("%.2f Mi", size / (1 << 20));
  } else if(size >> 10) {
    return sprintf("%.2f Ki", size / (1 << 10));
  }
  return sprintf("%.2f ", size);
}

function formatKMG10(size) {
  if(kisNaN(size)) {
    return "N/A";
  } else if(size >= 1000000000000000) {
    return sprintf("%.2f P", size / 1000000000000000);
  } else if(size >= 1000000000000) {
    return sprintf("%.2f T", size / 1000000000000);
  } else if(size >= 1000000000) {
    return sprintf("%.2f G", size / 1000000000);
  } else if(size >= 1000000) {
    return sprintf("%.2f M", size / 1000000);
  } else if(size >= 1000) {
    return sprintf("%.2f K", size / 1000);
  }
  return sprintf("%.2f ", size);
}

function conditionalPlural(n, word) {
  if(n == 1) return word;
  return word + 's';
}

function formatDHM(up) {
  var days = Math.floor(up / 86400); up -= days * 86400;
  var hours = Math.floor(up / 3600); up -= hours * 3600;
  var mins = Math.floor(up / 60);
  if(days) {
    if(hours) return days + " " + conditionalPlural(days, 'day') + ", " + hours + " " + conditionalPlural(hours, 'hour');
    return days + " " + conditionalPlural(hours, 'day')
  } else if(hours) {
    if(mins) return hours + " " + conditionalPlural(hours, 'hour') + ", " + mins + " " + conditionalPlural(mins, 'minute');
    return hours + " " + conditionalPlural(hours, 'hour');
  }
  return mins + " " + conditionalPlural(mins, 'minute');
}

function formatDHMS(up) {
  var days = Math.floor(up / 86400); up -= days * 86400;
  var hours = Math.floor(up / 3600); up -= hours * 3600;
  var mins = Math.floor(up / 60); up -= mins * 60;
  if(days) {
    if(hours) return days + " " + conditionalPlural(days, 'day') + ", " + hours + " " + conditionalPlural(hours, 'hour');
    return days + " days";
  } else if(hours) {
    if(mins) return hours + " " + conditionalPlural(hours, 'hour') + ", " + mins + " " + conditionalPlural(mins, 'minute');
    return hours + " " + conditionalPlural(hours, 'hour');
  } else if(mins) {
    return mins + " " + conditionalPlural(mins, 'minute');
  }
  return up + " " + conditionalPlural(up, 'second');
}

function formatDays(up) {
  var days = Math.floor(up / 86400); up -= days * 86400;
  var hours = Math.floor(up / 3600); up -= hours * 3600;
  var mins = Math.floor(up / 60);
  if(mins < 10) mins = "0" + mins;
  if(days) {
    return days + " days " + hours + ":" + mins;
  } else {
    return hours + ":" + mins;
  }
}

function thresActionString(cluster, object, field, char) {
  char = isundefined(char, '; ');
  var values = object[field];
  var text = '';
  for(var i = 0; i < values.length; i++) {
    if(text) text += char;
    var resolve = cluster.find(values[i].threshAction);
    if(!!resolve) {
      resolve = resolve.resolveName();
      if(values[i].actionParam.length) {
        text += (resolve + '(' + values[i].actionParam + ')');
      } else {
        text += resolve;
      }
      if(values[i].doWhen == 'DURING') {
        text += ' [during]';
      } else if(values[i].doWhen == 'LEAVE') {
        text = ' [leave]';
      }
    }
  }
  return text;
}

/*
 * Date Format 1.2.2
 * (c) 2004-2014 Steven Levithan <stevenlevithan.com>
 * MIT license
 * Includes enhancements by Scott Trenda <scott.trenda.net> and Kris Kowal <cixar.com/~kris.kowal/>
 *
 * Accepts a date, a mask, or a date and a mask.
 * Returns a formatted version of the given date.
 * The date defaults to the current date/time.
 * The mask defaults to dateFormat.masks.default.
 */
var dateFormat = function () {
  var token = /d{1,4}|m{1,4}|yy(?:yy)?|([HhMsTt])\1?|[LloSZ]|"[^"]*"|'[^']*'/g,
    timezone = /\b(?:[PMCEA][SDP]T|(?:Pacific|Mountain|Central|Eastern|Atlantic) (?:Standard|Daylight|Prevailing) Time|(?:GMT|UTC)(?:[-+]\d{4})?)\b/g,
    timezoneClip = /[^-+\dA-Z]/g,
    pad = function (val, len) {
      val = String(val);
      len = len || 2;
      while (val.length < len) val = "0" + val;
      return val;
    };

  // Regexes and supporting functions are cached through closure
  return function (date, mask, utc) {
    var dF = dateFormat;

    // You can't provide utc if you skip other args (use the "UTC:" mask prefix)
    if (arguments.length == 1 && (typeof date == "string" || date instanceof String) && !/\d/.test(date)) {
      mask = date;
      date = undefined;
    }

    // Passing date through Date applies Date.parse, if necessary
    date = date ? new Date(date) : new Date();
    if(kisNaN(date)) throw new SyntaxError("invalid date:" + date);

    mask = String(dF.masks[mask] || mask || dF.masks["default"]);

    // Allow setting the utc argument via the mask
    if (mask.slice(0, 4) == "UTC:") {
      mask = mask.slice(4);
      utc = true;
    }

    var _ = utc ? "getUTC" : "get",
      d = date[_ + "Date"](),
      D = date[_ + "Day"](),
      m = date[_ + "Month"](),
      y = date[_ + "FullYear"](),
      H = date[_ + "Hours"](),
      M = date[_ + "Minutes"](),
      s = date[_ + "Seconds"](),
      L = date[_ + "Milliseconds"](),
      o = utc ? 0 : date.getTimezoneOffset(),
      flags = {
        d:    d,
        dd:   pad(d),
        ddd:  dF.i18n.dayNames[D],
        dddd: dF.i18n.dayNames[D + 7],
        m:    m + 1,
        mm:   pad(m + 1),
        mmm:  dF.i18n.monthNames[m],
        mmmm: dF.i18n.monthNames[m + 12],
        yy:   String(y).slice(2),
        yyyy: y,
        h:    H % 12 || 12,
        hh:   pad(H % 12 || 12),
        H:    H,
        HH:   pad(H),
        M:    M,
        MM:   pad(M),
        s:    s,
        ss:   pad(s),
        l:    pad(L, 3),
        L:    pad(L > 99 ? Math.round(L / 10) : L),
        t:    H < 12 ? "a"  : "p",
        tt:   H < 12 ? "am" : "pm",
        T:    H < 12 ? "A"  : "P",
        TT:   H < 12 ? "AM" : "PM",
        Z:    utc ? "UTC" : (String(date).match(timezone) || [""]).pop().replace(timezoneClip, ""),
        o:    (o > 0 ? "-" : "+") + pad(Math.floor(Math.abs(o) / 60) * 100 + Math.abs(o) % 60, 4),
        S:    ["th", "st", "nd", "rd"][d % 10 > 3 ? 0 : (d % 100 - d % 10 != 10) * d % 10]
      };

    return mask.replace(token, function ($0) {
      return $0 in flags ? flags[$0] : $0.slice(1, $0.length - 1);
    });
  };
}();

// Some common format strings
dateFormat.masks = {
  "default":      "ddd mmm dd yyyy HH:MM:ss",
  shortDate:      "m/d/yy",
  mediumDate:     "mmm d, yyyy",
  longDate:       "mmmm d, yyyy",
  fullDate:       "dddd, mmmm d, yyyy",
  shortTime:      "h:MM TT",
  mediumTime:     "h:MM:ss TT",
  longTime:       "h:MM:ss TT Z",
  isoDate:        "yyyy-mm-dd",
  isoTime:        "HH:MM:ss",
  isoDateTime:    "yyyy-mm-dd'T'HH:MM:ss",
  isoUtcDateTime: "UTC:yyyy-mm-dd'T'HH:MM:ss'Z'"
};

// Internationalization strings
dateFormat.i18n = {
  dayNames: [
    "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat",
    "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"
  ],
  monthNames: [
    "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
    "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"
  ]
};

// For convenience...
Date.prototype.format = function (mask, utc) {
  return dateFormat(this, mask, utc);
}

function parseDate(text) {
  if(regexMatch(text, DATE_REGEX, 'i')) {
    text = text.split('/');
    text[0] = myParseInt(text[0]);
    text[2] = myParseInt(text[2]);
    if(isNumeric(text[1])) {
      text[1] = myParseInt(text[1]) - 1;
    } else {
      for(var i = 0; i < 12; i++) {
        if(string_case_compare(dateFormat.i18n['monthNames'][i], text[1])) {
          text[1] = i;
          break;
        }
      }
    }
    return (new Date(text[2], text[1], text[0]));
  } else if(regexMatch(text, DATETIME_REGEX, 'i')) {
    text = text.split(/(:|\s|\/)/);
    text[0] = myParseInt(text[0]);
    text[4] = myParseInt(text[4]);
    text[6] = myParseInt(text[6]);
    text[8] = myParseInt(text[8]);
    if(isNumeric(text[1])) {
      text[1] = myParseInt(text[1]) - 1;
    } else {
      for(var i = 0; i < 12; i++) {
        if(string_case_compare(dateFormat.i18n['monthNames'][i], text[2])) {
          text[2] = i;
          break;
        }
      }
    }
    return (new Date(text[4], text[2], text[0], text[6], text[8], 0));
  }
  return 0;
}

function showPresult(nodes, done, call) {
  var text = '';
  if(nodes.length != done.length) {
    text = "Length of result is not equal to number of nodes";
  }
  var fail = new Array();
  for(var i = 0; i < Math.min(nodes.length, done.length); i++) {
    if(!done[i]) fail.push(nodes[i]);
  }
  var cluster = current;
  if(cluster.cluster) cluster = cluster.cluster;
  if(fail.length == 1) {
    dock.displayInfo(call + " failed on node: " + cluster.resolveName(fail[0]));
  } else if(fail.length > 1) {
    dock.displayInfo(call + " failed on nodes: " + cluster.resolveList(fail));
  } else {
    // No need bug 585
//    dock.displayInfo(call + " successful");
  }
}

function allUpper(text) {
  for(var i = 0; i < text.length; i++) {
    var c = text[i];
    if((c != ' ') && (c != '_') && ((c < 'A') || (c > 'Z'))) return false;
  }
  return true;
}

function createDellNSSMetricPopup(cluster, name, empty, subitems) {
  // TODO: non HA NSS
  var nodesInGroup = new Array();
  var obj = document.getElementById(name);
  var popup = obj.firstChild;
  var storagePairs = cluster.find('NodeGroup');
  for(var i = 0; i < storagePairs.length; i++) {
    var nodes = storagePairs[i].get('nodes');
    if(storagePairs[i].get('type') == 'STORAGEPAIR' && nodes.length > 0) {
      nodesInGroup = nodesInGroup.concat(nodes);
      var mc1 = cluster.getMonConfForDevice(nodes[0]);
      var mc2 = cluster.getMonConfForDevice(nodes[1]);
      if(mc1 == mc2) { // != should not happen ...
        var item = document.createElement("menu");
        item.setAttribute("label", 'Dell NSS: ' + storagePairs[i].get('name'));
        item.setAttribute("class", 'submenu');
        var child = document.createElement("menupopup");
        item.appendChild(child);
        var metrics = mc1.get('metrics');
        var n = 0;
        var objectKey = storagePairs[i].get('uniqueKey');
        for(var j = 0; j < metrics.length; j++) {
          var metric = cluster.find(metrics[j].metric);
          if(metric && metric.get('metricClass') == 'DELLNSS') { // others?
            var text = metric.resolveName();
            if(text == 'StoragePairState') continue;
            if(metrics[j].metricParam.length > 0) text += "[" + metrics[j].metricParam + "]";
            var subitem = document.createElement("menuitem");
            var unit = metric.get('unit');
            if(metric.get('unit')) text += " (" + unit + ")";
            subitem.setAttribute("metricKey", metrics[j].metric);
            subitem.setAttribute("objectKey", objectKey);
            subitem.setAttribute("param", metrics[j].metricParam);
            subitem.setAttribute("label", text);
            child.appendChild(subitem);
            n++;
          }
        }
        println('createDellNSSMetricPopup: ' + n);
        if(n) popup.appendChild(item);
      } else {
        println('StoragePair nodes do not have the same MonConf');
      }
    }
  }
//   var nonHAnss = cluster.getDellNSSNodes();
}

function createMetricPopupShowClass(cluster, metrics, name, empty, subitems, showclass) {
  empty = isundefined(empty, false);
  subitems = isundefined(subitems, true);
  showclass = isundefined(showclass, 0);

  var obj = document.getElementById(name);
  var oldKey = parseInt(obj.getAttribute('metricKey'));
  var oldParam = obj.getAttribute('param');
  var oldLabel = obj.getAttribute('label');
  var popup = obj.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);

  var names = new Object();
  for(var j = 0; j < metrics.length; j++) {
    var key = metrics[j].metric + ":" + metrics[j].metricParam;
    if(names.hasOwnProperty(key)) continue;
    names[key] = 1;
    
    var metric = cluster.find(metrics[j].metric);
    var metric_class = metric.get('metricClass');
    if(showclass && showclass.find(metric_class) == -1) continue;

    if(metric) {
      var text = metric.resolveName();
      if(metrics[j].translatedMetricParam.length > 0) {
        text += "[" + metrics[j].translatedMetricParam + "]";
      } else if(metrics[j].metricParam.length > 0) {
        text += "[" + metrics[j].metricParam + "]";
      }
      var subitem = document.createElement("menuitem");
      subitem.setAttribute("metricKey", metrics[j].metric);
      subitem.setAttribute("objectKey", 0);
      subitem.setAttribute("param", metrics[j].metricParam);
      subitem.setAttribute("label", text);

      obj.firstChild.appendChild(subitem);
    } else {
      println("Metric not found: " + metrics[j].metric);
    }
  }
  if(empty) {
    var item = document.createElement("menuitem");
    item.setAttribute("label", '');
    item.setAttribute("metricKey", 0);
    item.setAttribute("objectKey", 0);
    item.setAttribute("param", '');
    popup.appendChild(item);
  }
}

function createMetricPopup(cluster, metrics, name, empty, subitems, hideclass, hidefunction, showfunction) {
  var MAX_SORTED_CHILDREN = 512;

  empty = isundefined(empty, false);
  subitems = isundefined(subitems, true);
  hideclass = isundefined(hideclass, 0);
  hidefunction = isundefined(hidefunction, 0);
  showfunction = isundefined(showfunction, 0);

  var obj = document.getElementById(name);
  var oldKey = parseInt(obj.getAttribute('metricKey'));
  var oldParam = obj.getAttribute('param');
  var oldLabel = obj.getAttribute('label');
  var popup = obj.firstChild;
  while(popup.firstChild) popup.removeChild(popup.firstChild);

  var undef; // for undefined
  var metricClass = buildMetricClass('menu', cluster, undef, subitems);

  var names = new Object();
  for(var j = 0; j < metrics.length; j++) {
    var key = metrics[j].metric + ":" + metrics[j].metricParam;
    if(names.hasOwnProperty(key)) continue;
    names[key] = 1;
    
    var metric = cluster.find(metrics[j].metric);
    if(metric) {
      var text = metric.resolveName();
      if(metrics[j].translatedMetricParam.length > 0) {
        text += "[" + metrics[j].translatedMetricParam + "]";
      } else if(metrics[j].metricParam.length > 0) {
        text += "[" + metrics[j].metricParam + "]";
      }
      var subitem = document.createElement("menuitem");
      var unit = metric.get('unit');
      if(metric.get('unit')) text += " (" + unit + ")";
      subitem.setAttribute("metricKey", metrics[j].metric);
      subitem.setAttribute("objectKey", 0);
      subitem.setAttribute("param", metrics[j].metricParam);
      subitem.setAttribute("label", text);
      var mclass = metric.get('metricClass');

      metrics[j]._mtext = text;
      metrics[j]._mclass = mclass;

      if(showfunction && !showfunction(metrics[j])) continue;
      if(hideclass && hideclass.find(mclass) >= 0) continue;
      if(hidefunction && hidefunction(metrics[j])) continue;
      mclass = metricClass[mclass];
      if(mclass) {
        var it = mclass[1].firstChild;
        while(it && (metrics.length < MAX_SORTED_CHILDREN)) {
          if(it.getAttribute('label') > text) {
            mclass[1].insertBefore(subitem, it);
            subitem = 0;
            break;
          }
          it = it.nextSibling;
        }
        if(subitem) mclass[1].appendChild(subitem);
        mclass[2]++;
        while(mclass[3].length && metricClass.hasOwnProperty(mclass[3])) {
          mclass = metricClass[mclass[3]];
          mclass[2]++;
        }
      }
    } else {
      println("Metric not found: " + metrics[j].metric);
    }
  }
  for(var mclass in metricClass) {
    if(metricClass[mclass][2] > 0) {
      var parent = metricClass[mclass][3];
      if(parent.length && metricClass.hasOwnProperty(parent)) {
        metricClass[parent][1].appendChild(metricClass[mclass][0]);
      } else {
        popup.appendChild(metricClass[mclass][0]);
      }
    }
  }
  if(empty) {
    var item = document.createElement("menuitem");
    item.setAttribute("label", '');
    item.setAttribute("metricKey", 0);
    item.setAttribute("objectKey", 0);
    item.setAttribute("param", '');
    popup.appendChild(item);
  }
}

function sortByName(list, cluster) {
  var entities = new Array();
  if(typeof(cluster) != 'undefined') {
    for(var i = 0; i < list.length; i++) {
      if(typeof(list[i]) == 'number') {
        entities.push(cluster.find(list[i]));
      } else {
        entities.push(list[i]);
      }
    }
  }
  return entities.sort(function(a, b) {
    return (a.resolveName() > b.resolveName());
  });
}

function sortKeysByName(list, cluster) {
  var entities = new Array();
  if(typeof(cluster) != 'undefined') {
    for(var i = 0; i < list.length; i++) {
      if(typeof(list[i]) == 'number') {
        entities.push(cluster.find(list[i]));
      } else {
        entities.push(list[i]);
      }
    }
  }
  entities = entities.sort(function(a, b) {
      return (a.resolveName() > b.resolveName());
    });
  for(var i = 0; i < entities.length; i++) {
    entities[i] = entities[i].get('uniqueKey');
  }
  return entities;
}

function setDialogTitle(value) {
  var title = document.getElementById("dialogtitle");
  if(title) {
    title.setAttribute('value', value);
  } else {
    window.document.title = value;
  }
}

function getDialogTitle(value) {
  var title = document.getElementById("dialogtitle");
  if(title) {
    return title.getAttribute('value');
  } else {
    return window.document.title;
  }
}

var $utf8 = {
  encode : function (string) {
    var utftext = "";

    for (var n=0, k=string.length; n < k; n++) {
      var c = string.charCodeAt(n);

      if (c < 128) {
        utftext += String.fromCharCode(c);
      } else if((c > 127) && (c < 2048)) {
        utftext += String.fromCharCode((c >> 6) | 192);
        utftext += String.fromCharCode((c & 63) | 128);
      } else {
        utftext += String.fromCharCode((c >> 12) | 224);
        utftext += String.fromCharCode(((c >> 6) & 63) | 128);
        utftext += String.fromCharCode((c & 63) | 128);
      }
    }
    return utftext;
  },
  decode : function (utftext) {
    var string = "";
    var i = 0;
    var c = 0;
    var c1 = 0
    var c2 = 0;

    while ( i < utftext.length ) {
      c = utftext.charCodeAt(i);
      if (c < 128) {
        string += String.fromCharCode(c);
        i++;
      } else if((c > 191) && (c < 224)) {
        c2 = utftext.charCodeAt(i+1);
        string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
        i += 2;
      } else {
        c2 = utftext.charCodeAt(i+1);
        c3 = utftext.charCodeAt(i+2);
        string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
        i += 3;
      }
    }
    return string;
  }
}

function singularToPlural(text) {
  if(endswith(text, 'y')) {
    return text.substr(0, text.length - 1) + 'ies'
  } else if(endswith(text, 'ch')) {
    return text + 'es';
  } else if(!endswith(text, 's')) {
    return text + 's';
  }
  return test;
}

function pluralToSingular(text) {
  // optionally add special cases here;
  if(endswith(text, 'ies')) {
    return text.substr(0, text.length - 3) + 'y';
  } else if(endswith(text, 'ches')) {
    return text.substr(0, text.length - 2);
  } else if(endswith(text, 's')) {
    return text.substr(0, text.length - 1);
  }
  return text;
}

function firstLine(text, len) {
  len = isundefined(len, text.length);
  var idx = text.indexOf('\n');
  if((idx >= 0) && (idx <= len)) {
    return text.substr(0, len);
  } else if(text.length < len) {
    return text;
  } else {
    idx = text.indexOf(' ', len);
    if((idx > 0) && (idx <= len * 1.1)) {
      return text.substr(0, idx);
    } else {
      return text.substr(0, len);
    }
  }
}

function daysSince1970(days) {
  var d = new Date();
  d.setTime(days * 86400 * 1000);
  return d.format("dd/mmm/yyyy");
}

function severity2human(value) {
  if(value <= -2000) {
    return "debug";
  } else if(value <= -1000) {
    return "info";
  } else if(value <= 0) {
    return "notice";
  } else if(value <= 1000) {
    return "warning";
  } else if(value <= 2000) {
    return "error";
  } else if(value <= 3000) {
    return "alert";
  } else {
    return "emergency";
  }
}

function severity2icon(value) {
  if(value <= -2000) {
    return "chrome://cmgui/content/icons/info.png"; // TODO: debu
  } else if(value <= -1000) {
    return "chrome://cmgui/content/icons/info.png";
  } else if(value <= 0) {
    return "chrome://cmgui/content/icons/notice.png";
  } else if(value <= 1000) {
    return "chrome://cmgui/content/icons/warning.png";
  } else if(value <= 2000) {
    return "chrome://cmgui/content/icons/error.png";
  } else if(value <= 3000) {
    return "chrome://cmgui/content/icons/alert.png";
  } else {
    return "chrome://cmgui/content/icons/alert.png"; // TODO: emenrgency
  }
}

function refreshTypesCheck(id, type) {
  println("refreshTypesCheck " + id + " " + type);
  return (((id == 'clustertab') && ((type == 'Partition') || (type == 'Device') || (type == 'Rack'))) ||
          ((id == 'monitoringsetuptab') && ((type == 'Metric') || (type == 'HealthCheck') || (type == 'MonConf') || (type == 'ThreshAction'))) ||
          ((id == 'usergroupsetuptab') && ((type == 'User') || (type == 'Group'))) ||
          ((id == 'workloadsetuptab') && (type == 'JobQueue')) ||
          ((id == 'softwareimagefoldertab') && (type == 'SoftwareImage')) ||
          ((id == 'categoryfoldertab') && (type == 'Category')) ||
          ((id == 'cloudnodefoldertab') && ((type == 'CloudProvider') || (type == 'Device'))) ||
          ((id == 'cloudprovidertab') && ((type == 'CloudProvider') || (type == 'Device') || (type == 'CloudStaticIP'))) ||
          ((id == 'cloudproviderfoldertab') && ((type == 'CloudProvider') || (type == 'Device'))) ||
          ((id == 'rackfoldertab') && (type == 'Rack')) ||
          ((id == 'networkfoldertab') && (type == 'Network')) ||
          ((id == 'hadoophfdsfoldertab') && (type == 'HadoopHDFS')) ||
          ((id == 'hadoophfdstab') && (type == 'HadoopHDFS')) ||
          ((id == 'hadoophdfstab') && (type == 'ConfigurationOverlay')) ||
	  ((id == 'puppettab')     && (type == 'ConfigurationOverlay')) ||
          ((id == 'cephtab') && (type == 'Ceph')) ||
          ((id == 'openstackfoldertab') && ((type == 'OpenStack') || (type == 'OpenStackApiEntity'))) ||
          ((id == 'authorisationsetuptab') && (type == 'Profile')) ||
          (((id == 'nodefoldertab') || (id == 'headnodefoldertab') || (id == 'switchfoldertab') || (id == 'gpuunitfoldertab') || (id == 'micnodefoldertab') || (id == 'openstackfoldertab') ||
            (id == 'powerdistributionunitfoldertab') || (id == 'otherdevicefoldertab') || (id == 'virtualsmpnodefoldertab') || (id == 'virtualnodefoldertab')) && (type == 'Device')) ||
          ((id == 'nodegroupfoldertab') && ((type == 'NodeGroup') || (type == 'Device'))));
}

function isCloudProviderKey(key) {
  return ((Math.floor(key / 4294967296) == 22) && ((key & 0xffffffff) >> 13 == 0));
}

function isCloudImageKey(key) {
  return ((Math.floor(key / 4294967296) == 22) && ((key & 0xffffffff) >> 13 == 1));
}

function isCloudRegionKey(key) {
  return ((Math.floor(key / 4294967296) == 22) && ((key & 0xffffffff) >> 13 == 2));
}

function isCloudTypeKey(key) {
  return ((Math.floor(key / 4294967296) == 22) && ((key & 0xffffffff) >> 13 == 3));
}

function isCloudStaticIPKey(key) {
  return ((Math.floor(key / 4294967296) == 22) && ((key & 0xffffffff) >> 13 == 4));
}

function isHadoopHDFSKey(key) {
  return (Math.floor(key / 4294967296) == 24);
}

function strip_beginning_trailing_newlines(txt) {
  txt = txt.replace(/[\s\r\n]+$/, '');
  txt = txt.replace(/^\n+/, '');
  return txt;
}

function setDisabled(name, disabled) {
  var obj = document.getElementById(name)
  if(obj) {
    if(disabled) {
      obj.setAttribute('disabled', true);
    } else {
      obj.removeAttribute('disabled');
    }
  }
}

function setHidden(name, hidden) {
  var obj = document.getElementById(name)
  if(obj) {
    if(hidden) {
      obj.setAttribute('hidden', true);
    } else {
      obj.removeAttribute('hidden');
    }
  }
}

function hasNetMapIP(node) {
  var interfaces = node.get('networks');
  for(var i = 0; i < interfaces.length; i++) {
    if(interfaces[i].childType == 'NetworkNetMapInterface') return interfaces[i].ip;
  }
  return false;
}

function getManagementIP(node) {
  var man = 0;
  if(dock.baseChildTypeCheck(node.object, 'SlaveNode')) {
    man = node.get('managementNetwork');
    if(!man) {
      var x = node.get('category');
      if(x) {
        x = node.cluster.find(x);
        if(x) man = x.get('managementNetwork');
      }
    }
  }
  if(!man) {
    x = node.get('partition');
    if(x) {
      x = node.cluster.find(x);
      if(x) man = x.get('managementNetwork');
    }
  }
  var ip = [0, '0.0.0.0'];
  var n = node.get('networks');
  for(var i = 0; i < n.length; i++) {
    if((n[i].network == man) && (n[i].startIf == 'ALWAYS')) {
      ip = [node.cluster.find(man), n[i].ip];
      break;
    }
  }
  return ip;
}



function buildMetricClass(type, cluster, list, subitems) {
  type = isundefined(type, 'menu');
  subitems = isundefined(subitems, false);
  var func;
  if(type == 'menu') {
    func = function createMetricType(name, parent) {
      parent = isundefined(parent, '');
      var item = document.createElement("menu");
      item.setAttribute("label", name);
      item.setAttribute("class", 'submenu');
      var child = document.createElement("menupopup");
      item.appendChild(child);
      return [item, child, 0, parent];
    }
  } else if(type == 'rack') {
    func = function createMetricTypeRack(name, parent) {
      parent = isundefined(parent, '');
      var item = document.createElement("treeitem");
    //   item.setAttribute("id", key + "_" + name);
      var row = document.createElement("treerow");
      var cell = document.createElement("treecell");
      if(name == 'OpenStack') {
        cell.setAttribute("src", "chrome://cmgui/content/icons/OpenStack_16.png");
      } else {
        cell.setAttribute("src", "chrome://cmgui/content/icons/Category_16.png");
      }
      cell.setAttribute("label", " " + name); // space between image and label, css doesn't work
      row.appendChild(cell);
      item.appendChild(row);
      item.setAttribute("container", "true");
      item.setAttribute("open", "false");
      var child = document.createElement("treechildren");
      item.appendChild(child);
      return [item, child, 0, new Object(), parent];
    }
  } else {
    dock.displayError('buildMetricClass type not known: ' + type);
    return new Object();
  }

  var metricClass = new Object();
  list = isundefined(list, cluster.metricEnumName(true, false));
  if(subitems) {
    var tmp = 0;
    var done = new Object();
    for(var i = 0; i < list.length; i++) {
      var word = list[i][1];
      var idx = 0;
      var parent = false;
      while(idx < word.length) {
        var jdx = word.indexOf('/', idx);
        var leaf = (jdx < 0);
        if(leaf) jdx = word.length;
        var full = word.substr(0, jdx);
        var name = word.substr(idx, jdx - idx);
        if(leaf) {
//           println(' +++ ' + full + ' : ' + name + ' : ' + list[i][0]);
          metricClass[list[i][0]] = func(name, parent);
          done[full] = list[i][0];
        } else {
          if(done.hasOwnProperty(full)) {
            parent = done[full];
          } else {
            var old = parent;
            tmp += 1;
            parent = '@PLACEHOLDER@' + tmp;
            metricClass[parent] = func(name, old);
            done[full] = parent;
//             println(' *** ' + full + ' : ' + name + ' : ' + parent);
          }
        }
        idx = jdx + 1;
      }
    }
  } else {
    for(var i = 0; i < list.length; i++) {
      metricClass[list[i][0]] = func(list[i][1]);
    }
  }
  return metricClass;
}

function endswithslash(url) {
  if((url.length == 0) || (url.substr(url.length - 1, 1) != '/')) url += '/';
  return url;
}

function md5sum(file) {
  var ch = Components.classes["@mozilla.org/security/hash;1"].createInstance(Components.interfaces.nsICryptoHash);
  ch.init(ch.MD5);
  var f = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
  f.initWithPath(file);
  var istream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
  istream.init(f, 0x01, parseInt("0444", 8), 0);
  var PR_UINT32_MAX = 0xffffffff;
  try {
    ch.updateFromStream(istream, PR_UINT32_MAX);
  } catch(e) {
  }
  var hash = ch.finish(false);
  function toHexString(charCode) {
    return ("0" + charCode.toString(16)).slice(-2);
  }
  var v = new Array();
  for(var i in hash) {
    v.push(hash.charCodeAt(i));
  }
  return v.join("");
//   return [toHexString(hash.charCodeAt(i)) for (i in hash)].join("");
}

function kisNaN(x) {
  return isNaN(x) || (x == null);
}

function metricClass2String(classname) {
  switch(classname) {
    case 'CPU':                         return 'CPU';
    case 'GPU':                         return 'GPU';
    case 'DISK':                        return 'Disk';
    case 'MEMORY':                      return 'Memory';
    case 'NETWORK':                     return 'Network';
    case 'ENVIRONMENTAL':               return 'Environmental';
    case 'OPERATINGSYSTEM':             return 'Operating System';
    case 'INTERNAL':                    return 'Internal';
    case 'WORKLOAD':                    return 'Workload';
    case 'MISC':                        return 'Misc';
    case 'CLUSTER':                     return 'Cluster';
    case 'PROTOTYPE':                   return 'Prototype';
    case 'OPENSTACK_API_IMAGE':         return 'OpenStack/API/Image';
    case 'OPENSTACK_API_IDENTITY':      return 'OpenStack/API/Identity';
    case 'OPENSTACK_API_COMPUTE':       return 'OpenStack/API/Compute';
    case 'OPENSTACK_API_NETWORK':       return 'OpenStack/API/Network';
    case 'OPENSTACK_API_VOLUME':        return 'OpenStack/API/Volume';
    case 'OPENSTACK_API_ALLSERVICES':   return 'OpenStack/API/AllServices';
    case 'OPENSTACK_API_ORCHESTRATION': return 'OpenStack/API/Orchestration';
    case 'OPENSTACK_VM_MEMORY':         return 'OpenStack/VM/Memory';
    case 'OPENSTACK_VM_BLOCKDEVICE':    return 'OpenStack/VM/BlockDevice';
    case 'OPENSTACK_VM_NETWORK':        return 'OpenStack/VM/Network';
    case 'OPENSTACK_VM_CPU':            return 'OpenStack/VM/CPU';
    case 'OPENSTACK_VM_OTHER':          return 'OpenStack/VM/Other';
    case 'OPENSTACK_HYPERVISORS':       return 'OpenStack/Hypervisors';
    case 'OPENSTACK_OTHER':             return 'OpenStack/Other';
    case 'OPENSTACK_PROJECTS':          return 'OpenStack/Projects';
    case 'OPENSTACK_OBJECTS':           return 'OpenStack/Objects';
    case 'DELLNSS':                     return 'Dell NSS';
    case 'CEPH':                        return 'Ceph';
    case 'LUSTRE':                      return 'Lustre';
    case 'AMAZONEC2':                   return 'Amazon EC2';
    case 'CLOUD':                       return 'Cloud';
    case 'HADOOP_JVM':                  return 'Hadoop/JVM';
    case 'HADOOP_DFS':                  return 'Hadoop/DFS';
    case 'HADOOP_UGI':                  return 'Hadoop/UGI';
    case 'HADOOP_MAPRED':               return 'Hadoop/map reduce';
    case 'HADOOP_RPC':                  return 'Hadoop/RPC';
    case 'HADOOP_RPCDETAILED':          return 'Hadoop/RPC/detailed';
    case 'HADOOP_METRICSYSTEM':         return 'Hadoop/metric system';
  }
  return 'Unknown';
}

function fillRackMapGaps(mapX) {
  var i0 = 0;
  var s0 = 0;
  var keys = Object.keys(mapX);
  for(var j = 0; j < keys.length; j++) {
    var k = keys[j];
    var i1 = parseInt(k, 10);
    var s1 = mapX[k];
    var s = Math.max(s1, s0);
    for(var i = i0 + 1; i < i1; i++) {
      mapX[sprintf('%010d', i)] = s;
    }
    i0 = i1;
    s0 = s1;
  }
}

function objectSortedArray(o) {
  var a = new Array();
  for(var k in o) a.push(k);
  a.sort();
  return a;
}

var bind = function(func, thisValue) {
  return function() {
    return func.apply(thisValue, arguments);
  }
}

function parseJobTime(ts) {
// slurm
// cmd:    if(parseOneTime(s, "%Y-%m-%dT%H:%M:%S", ts)) return true;
//   var d = Date.parse('2011-10-03T11:03:02');
//   printall(d);

// pbs
// cmd:  if(strptime(starttime.c_str(), "%b %d %H:%M:%S %Y", &tm) != NULL) {
//   var d = Date.parse('Oct 03 11:03:02 2011');
//   printall(d);
// cmd:  } else if(strptime(starttime.c_str(), "%a %b %d %H:%M:%S %Y", &tm) != NULL) {
//   var d = Date.parse('Wed Oct 03 11:03:02 2011');
//   printall(d);

// sge
// cmd:    if(strptime(starttime.c_str(), "%m/%d/%Y %H:%M:%S", &tm) != NULL) {
//   var d = Date.parse('10/03/2011 11:03:02');
//   printall(d);

// lsf/openlava
// cmd:  if(parseOneTime(s, "%a %b %d %H:%M:%S", ts)) {
//   var d = Date.parse('Wed Oct  3 11:03:02 2011');
//   printall(d);

  if(typeof(ts) == 'number') {
    return ts;
  } else if(typeof(ts) == 'string') {
    if(isNumeric(ts)) return parseInt(ts);
    var d = new Date(Date.parse(ts));
    if(isNaN(d)) {
      var now = new Date();
      var yy = now.getFullYear();
      var d = new Date(Date.parse(ts + ' ' + yy));
      if(!isNaN(d)) {
        if(d < now) return Math.floor(d.getTime() / 1000);
        var d = new Date(Date.parse(ts + ' ' + (yy - 1)));
        if(!isNaN(d)) return Math.floor(d.getTime() / 1000);
      }
    } else {
      return Math.floor(d.getTime() / 1000);
    }
  }
  return 0;
}

function isEmpty(obj) {
  for(var i in obj) { return false; }
  return true;
}

function sort_unique(arr) {
  arr = arr.sort();
  var ret = [arr[0]];
  for (var i = 1; i < arr.length; i++) { // start loop at 1 as element 0 can never be a duplicate
    if (arr[i-1] != arr[i]) {
      ret.push(arr[i]);
    }
  }
  return ret;
}

// when using a read-only profile, cloud provider contains magic values
function isReadOnlyCloudProvider(provider) {
  if (((provider.get('username') == 'CMD_ACCESS_RESTRICTED') &&
       (provider.get('accountId') == 'CMD_ACCESS_RESTRICTED') &&
       (provider.get('accessKeyId') == 'CMD_ACCESS_RESTRICTED') &&
       (provider.get('accessKeySecret') == 'CMD_ACCESS_RESTRICTED')) ||
      (provider.get('certificate') == 'CMD_ACCESS_RESTRICTED')) {
    return true;
  } else {
    return false;
  }
}

function sleep(delay) {
  delay = isundefined(delay, 0);
  var end = Date.now() + delay;
  while(true) {
    dock.mainthread.processNextEvent(true);
    if(Date.now() >= end) break;
  }
}

function startKVM() {
  println('startKVM');
  dock.waitIcon();
  var type = 1;
  if ( (dock.waitforkvmstart != null) && (typeof (dock.waitforkvmstart) != undefined) ) {
    dock.waitforkvmstart.setAttribute('hidden', false);
  }
  dock.workerpool.getWorker(current.cluster.actualurl).call('cmdevice', 'startKvmUCS', [current.get('uniqueKey'), type], function(code, data) {
      if(code == 0) {
        getjnlp(data.success, data.data, type);
      } else {
        dock.unwaitIcon();
        dock.jsonCallError(this, code, data);
      }
    }, current);
}

function getjnlp(success, data, type) {
  if(!success) {
    dock.unwaitIcon();
    dock.displayInfo('Unable to start UCS KVM, is the node configured correctly?');
  } else if(data.length == 0) {
    dock.unwaitIcon();
    dock.displayInfo('Unable to start UCS KVM, no URL was returned?');
  } else {
    if(type == 1) {
      savejnlp(data);
    } else {
      dock.workerpool.getWorker(current.cluster.actualurl).wget(data, function(code, data) {
          if(code == 0) {
            savejnlp(data);
          } else {
            dock.unwaitIcon();
            dock.jsonCallError(this, code, data);
          }
        }, current);
    }
  }
}

function savejnlp(data) {
  dock.unwaitIcon();

  var file = dock.cacheFile('ucs-kvm.jnlp');
  var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
  outputStream.init(file, 0x02 | 0x08 | 0x20, oct0600, 0);
  outputStream.write(data, data.length);
  outputStream.close();
  println('savejnlp: ' + file.path);
  var javaws;
  if(dock.generalSettings.javaws.length) {
    javaws = dock.generalSettings.javaws;
  } else {
    if(startswith(getOS(), "WIN")) {
      javaws = 'javaws.exe';
    } else {
      javaws = 'javaws';
    }
  }
  runCommand(javaws, file.path);
  if ( (dock.waitforkvmstart != null) && (typeof (dock.waitforkvmstart) != undefined) ) {
    dock.waitforkvmstart.setAttribute('hidden', true);
  }
}

function isOpenStackTenantId(txt) {
  if(txt.length != 32) return false;
  for(var i = 0; i < txt.length; i++) {
    if((txt[i] < '0') || (txt[i] > 'f')) return false;
    if((txt[i] > '9') && (txt[i] < 'a')) return false;
  }
  return true;
}

function longLineSplit(msg, chars, newLine) {
  chars = isundefined(chars, 80);
  newLine = isundefined(newLine, '\n');
  var lines;
  if(newLine != false) {
    lines = msg.split(newLine);
  } else {
    lines = [msg];
  }
  if(chars > 0) {
    var i = 0;
    while(i < lines.length) {
      if(lines[i].length > chars) {
        var j;
        var k = -1;
        for(j = chars; j > 0; j--) {
          if(lines[i][j] == ',' || lines[i][j] == '.' || lines[i][j] == ';') {
            j += 1;
            k = j;
            if(k < lines[i].length && lines[i][k] == ' ') k++;
            break;
          } else if(lines[i][j] == ' ') {
            k = j + 1;
            break;
          }
        }
        if(k < 0) {
          j = chars;
          k = chars;
        }
        lines.splice(i + 1, 0, lines[i].substr(k));
        lines[i] = lines[i].substr(0, j);
      }
      ++i;
      if(i > 10) break;
    }
  }
  return lines;
}

var PREFS_CID   = "@mozilla.org/preferences;1";
var PREF_STRING = "browser.dom.window.dump.enabled";

function enableDump(bool) {
  try {
    var prefs = Components.classes[PREFS_CID].getService(Components.interfaces.nsIPrefService).getBranch("");
    prefs.setBoolPref(PREF_STRING, bool);
  } catch(e) {
    println(e);
  }
}

function checkDump() {
  try {
    var prefs = Components.classes[PREFS_CID].getService(Components.interfaces.nsIPrefService).getBranch("");
    return prefs.getBoolPref(PREF_STRING);
  } catch(e) {
    println(e);
  }
  return false;
}

function parseUrlParams(url) {
  var args = url.split('?')[1];
  var match;
  var pl = /\+/g;
  var search = /([^&=]+)=?([^&]*)/g;
  var decode = function (s) { return decodeURIComponent(s.replace(pl, " ")); };

  var urlParams = {};
  var match = search.exec(args);
  while (match) {
    urlParams[decode(match[1])] = decode(match[2]);
    match = search.exec(args);
  }
  return urlParams;
}

function setAttributeRecursive(element, attribute, value) {
  element.setAttribute(attribute, value);
  for(var i=0; i<element.children.length; i++) {
    setAttributeRecursive(element.children[i], attribute, value);
  }
}

function removeAttributeRecursive(element, attribute) {
  element.removeAttribute(attribute);
  for(var i=0; i<element.children.length; i++) {
    removeAttributeRecursive(element.children[i], attribute);
  }
}

var formatter = {};
formatter.format = {};
formatter.format.DynamiciB = function(value) {
  return formatKMGselect(value, "B");
};
formatter.format.TiB = function(value) {
  return sprintf("%.2f TiB", value / ((1 << 20) * (1 << 20)));
};
formatter.format.GiB = function(value) {
  return sprintf("%.2f GiB", value / (1 << 30));
};
formatter.format.MiB = function(value) {
  return sprintf("%.2f MiB", value / (1 << 20));
};
formatter.format.KiB = function(value) {
  return sprintf("%.2f KiB", value / (1 << 10));
};
formatter.format.iB = function(value) {
  return sprintf("%.2f Bytes", value);
};
formatter.parse = {};
formatter.parse.getNumericPart = function(text) {
  var i = 0;
  for(; i < text.length; i++) {
    var c = text[i];
    if(((c < '0') || (c > '9')) && (c !== '.')) {
      break;
    }
  }
  var n = parseFloat(text.substr(0, i));
  return n;
};
formatter.parse.DynamiciB = function(text) {
  return parseKMG(text);
};
formatter.parse.TiB = function(text) {
  var n = formatter.parse.getNumericPart(text);
  return n * 1024 * 1024 * 1024 * 1024;
};
formatter.parse.GiB = function(text) {
  var n = formatter.parse.getNumericPart(text);
  return n * 1024 * 1024 * 1024;
};
formatter.parse.MiB = function(text) {
  var n = formatter.parse.getNumericPart(text);
  return n * 1024 * 1024;
};
formatter.parse.KiB = function(text) {
  var n = formatter.parse.getNumericPart(text);
  return n * 1024;
};
formatter.parse.iB = function(text) {
  var n = formatter.parse.getNumericPart(text);
  return n;
};

function HostnameEntity() {
  this.hostname = null;
  this.domain = null;
  this.fqdn = null;

  this.setHostname = function(hostname) {
    this.hostname = hostname;
    this.fqdn = this.hostname + '.' + this.domain;
  };

  this.setDomain = function(domain) {
    this.domain = domain;
    this.fqdn = this.hostname + '.' + this.domain;
  };

  this.setFQDN = function(fqdn) {
    this.fqdn = fqdn;
    this.hostname = this.fqdn.split('.')[0];
    this.domain = this.fqdn.substring(this.fqdn.indexOf('.'));
  };

  this.getHostname = function() {
    return this.hostname;
  };

  this.getDomain = function() {
    return this.domain;
  };

  this.getFQDN = function() {
    return this.fqdn;
  };
};

function checkboxTriState(elementNode) {
  var state;
  var checked;

  switch(elementNode.getAttribute('state')) {
    case 'unchecked':
      state = 'checked';
      checked = true;
      break;
    case 'checked':
      state = 'indeterminate';
      checked = false;
      break;
    default:
      state = 'unchecked';
      checked = false;
  }
  elementNode.setAttribute('state', state);
  elementNode.checked = checked;
}

function viewTaskManager() {
  var taskmanager = dock.dialogs.find('taskmanager.xul');
  if(taskmanager) {
    if(taskmanager.closed) taskmanager.focus();
    taskmanager.refresh();
  } else {
    var d = new dock.cvDialog('taskmanager.xul', []);
    dock.dialogs.showDialog(d);
  }
}

function checkConfOverlayPriority(value) {
  if ( (+value===parseInt(value)) && value >= -1 && value <= 1000 && value != 250 && value != 750 )   return true;
  else    return false;
}  

function metricInCGroup(mm) {
  return (mm.metricParam.length >= this.length) &&
          ((mm.metricParam == this) || startswith(mm.metricParam, this + ':'));
}
function checkInCGroup(mm) {
  return (mm.healthcheckParam.length >= this.length) &&
          ((mm.healthcheckParam == this) || startswith(mm.healthcheckParam, this + ':'));
}
function metricNotInList(mm) {
  return !this.hasOwnProperty(mm.metric);
}
function checkNotInList(mm) {
  return !this.hasOwnProperty(mm.healthcheck);
}

function metricParamText(mclass, param, translated, cgroup) {
  cgroup = isundefined(cgroup, 1);
  if((param.length > 0) && 
      (translated.length == 0) && 
      (mclass != 'OPENSTACK_HYPERVISORS') && 
      !isOpenStackTenantId(param)) {
    if(startswith(mclass, 'CGROUPS_') && cgroup < 2) {
      var idx = param.indexOf(':');
      if(idx > 0) return "[" + param.substr(idx + 1) + "]";
    } else {
      return "[" + param + "]";
    }
  } else if(translated.length > 0) {
    return "[" + translated + "]";
  }
  return "";
}
function osdText(mclass, param, selected) {
  if((param.length > 0) && (mclass == 'CEPH')) {
    var osd = cluster.getCephOSDid(selected, param);
    if(osd >= 0) return ' (osd.' + osd + ')';
  }
  return "";
}

function setSoftwareImageParent(current, old) {
  println('setSoftwareImageParent : ' + current);
  var proxy = current.get('softwareImageProxy');
  var list = [[-1, '']];
  if(proxy) {
    var parentImage = current.cluster.find(proxy.parentSoftwareImage);
    var revisionsList = parentImage.get('revisionsList');
    if(revisionsList.length) {
      list.push([0, 'latest']);
      for(var i = 0; i < revisionsList.length; i++) {
        var image = current.cluster.find(revisionsList[i]);
        if(image) list.push([image.get('revisionID'), image.get('revisionID')]);
      }
    }
  }
  kenum('softwareImageProxy', list, 'revisionID');
}
