/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

/**
 * Helper to manage rendering the progression of steps, like deployment steps or verification steps in progress.
 *
 * @param rowsId - id of rows form element containing the row (template) elements
 *
 * Example for XUL code for given row:
 *
 * <rows id="rowsId">
 *   <row align="center">
 *     <shadowlabel class="staticdataheader"/>
 *     <shadowlabel class="staticdataheader"/>
 *   </row>
 *   <row class="staticdottedline"/>
 * </rows>
 */
var ProgressStepDisplayer = function (rowsId) {
  return {
    'rowsElement' : document.getElementById(rowsId),

    // public functions
    'clearAll' : function () {
      while (this.rowsElement.childNodes.length > 2) {
        this.rowsElement.removeChild(this.rowsElement.lastChild);
      }
    },

    'add' : function (state, labelText) {
      var row      = document.createElement('row'),
          image    = document.createElement('image'),
          stepDesc = document.createElement('shadowlabel');


      switch (state) {
        case 'STARTED':
          image.setAttribute('src', 'chrome://cmgui/content/icons/throbber.gif');
          break;
        case 'FINISHED':
          image.setAttribute('src', 'chrome://cmgui/content/icons/ok.png');
          break;
        case 'FAILED':
          image.setAttribute('src', 'chrome://cmgui/content/icons/alert.png');
          break;
        case 'INFO':
          image.setAttribute('src', 'chrome://cmgui/content/icons/info.png');
          break;
      }

      row.setAttribute('align', 'center');
      stepDesc.setAttribute('value', labelText);

      row.appendChild(image);
      row.appendChild(stepDesc);
      this.rowsElement.appendChild(row);
    }
  };
};

/**
 * Helper for managing the ProgressStepDisplayer instance for multiple checkers that use it in parallel.
 */
function MultipleCheckersProgressStepDisplayer(rowsId)
{
  var stepDisplayer = new ProgressStepDisplayer(rowsId),
      checkers = [],
      buffer = {};

  function initChecker(checker) {
    if (buffer[checker]) return;
    
    checkers.push(checker);
    buffer[checker] = [];
  }

  return {
    'clearProgressLines' : function (checker) {
      initChecker(checker);
      buffer[checker] = [];
    },
    'addProgressLine' : function (checker, status, message) {
      initChecker(checker);

      buffer[checker].push([status, message]);
    },
    'addReturnLine' : function (checker_name, return_line) {
      println(checker_name + " " + return_line);
    },
    'display' : function () {
      var i, j, step;
      stepDisplayer.clearAll();
      for (i=0; i<checkers.length; i++) {
        for (j=0; j<buffer[checkers[i]].length; j++) {
          step = buffer[checkers[i]][j];
          stepDisplayer.add(step[0], step[1]);
        }
      }
    }
  };
}

function GetReturnValuesFromPreConditionChecker(callback)
{
  var buffer = [],
    checker_ = null,
    callback_ = callback;

  return {
    'setChecker' : function (checker) {
    },
    'clearProgressLines' : function (checker_name) {
    },
    'addProgressLine' : function (checker_name, status, message) {
    },
    'addReturnLine' : function (checker_name, return_line) {
      buffer.push_back(return_line);
    },
    'display' : function (isCompleted, results, error) {
      error = error || false;
      callback_(isCompleted, results, error);
    }
  };
}

function scrollUpVboxById(id) {
  var vbox = document.getElementById(id);
  if (vbox != undefined) {
    vbox.scrollTop = 0;
  }
}
