/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var usedDrives = 0;

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function updateTypeDescription(current) {
  var settings = current.get('virtualNodeSettings');
  if(settings.childType === 'OpenStackSettings') {
    var flavor;
    if(!settings.openStackFlavor) {
      flavor = getDefaultFlavor(current);
    }
    else {
      flavor = current.cluster.find(settings.openStackFlavor);
    }
    if(flavor) {
      var info = new Array();
      info.push('vCPUs: ' + flavor.get('vcpus'));
      info.push('Memory: ' + formatKMGselect(flavor.get('ram'), 'B'));
      info.push('Root disk: ' + formatKMGselect(flavor.get('disk'), 'B'));
      info.push('Ephemeral disk: ' + formatKMGselect(flavor.get('ephemeral'), 'B'));
      info.push('Swap disk: ' + formatKMGselect(flavor.get('swap'), 'B'));
      document.getElementById('cloudtypedescription').value = info.join('\n');
      addOpenStackStorage(settings.storage, false);
    } else {
      document.getElementById('cloudtypedescription').value = "Not found: " + settings.openStackFlavor;
    }
  }
}

function getDefaultFlavor(current) {
  var openstack = current.cluster.find('OpenStack');
  if(openstack.length > 0) {
    var default_flavor = openstack[0].get('defaultFlavor');
    var flavors = current.cluster.findBy({'baseType': 'OpenStackApiFlavor', 
                                          'filter' : function(item) { 
                                            return item.resolveName() == default_flavor
                                          } });
    if(flavors.length > 0) {
      return flavors[0];
    }
  }
  return null;
}

function refresh(use) {
  if(!!use) current = use;
  println("Refresh node cloud settings" + ' - ' + document.documentURI);
  document.getElementById('revert').setAttribute('disabled', current.object.uniqueKey < dock.MAX_LOCAL_KEY);
  var settings = current.get('virtualNodeSettings');
  if(settings.childType === 'OpenStackSettings') {
    var default_flavor = getDefaultFlavor(current);
    resolve('virtualNodeSettings', 'OpenStackApiFlavor', 'openStackFlavor', 
            'default' + (default_flavor ? ( ': ' + default_flavor.get('name'))  : ''));

    text('virtualNodeSettings', '', 'instanceID');
    text('virtualNodeSettings', '', 'name');

    text('virtualNodeSettings', '', 'region');
    text('virtualNodeSettings', '', 'availabilityZone');
    text('virtualNodeSettings', '', 'tenant');
    resolve('virtualNodeSettings', 'SlaveNode', 'defaultHost', true, '', function(slaveNode){
      var openStackComputeRole = slaveNode.getRolesByFilter(function(role) {
        return (role.childType==='OpenStackComputeRole');
      }, true);
      return (openStackComputeRole.length===0);
    });

    text('virtualNodeSettings', '', 'options');
    text('virtualNodeSettings', '', 'openStackImage');

    if(!started) extraeval('virtualNodeSettingsopenStackFlavor', 'updateTypeDescription(current);');

    updateTypeDescription(current);

    var started = (settings.instanceID !== '');
    setDisabled('virtualNodeSettingsopenStackFlavor', started);
    setDisabled('virtualNodeSettingsregion', started);
    setDisabled('virtualNodeSettingsavailabilityZone', started);
    setDisabled('virtualNodeSettingstenant', started);
    setDisabled('virtualNodeSettingsdefaultHost', started);
    setDisabled('virtualNodeSettingsoptions', started);
    setDisabled('virtualNodeSettingsopenStackImage', started);

    addOpenStackStorage(settings.storage, started);
  } else {
    println('--------------- NOT OpenStack SETTINGS -----------------');
    printall(settings);
  }
}

function getFlavor(settings) {
  var flavor = current.cluster.find(settings.openStackFlavor);
  if(!flavor) {
    var openstacks = current.cluster.find('OpenStack');
    if(openstacks.length > 0) {
      var openstack = openstacks[0];
      var flavors = current.cluster.find('OpenStackApiFlavor');
      for(var i = 0; i < flavors.length; i++) {
        if(flavors[i].name == openstack.defaultFlavor) {
          flavor = flavors[i];
          break;
        }
      }
    }
  }
  return flavor;
}

function addOpenStackStorage(realStorage, started) {
  println('addOpenStackStorage');

  function addFirstOpenStackStorage(storage, rows) {
    var count = 0;
    for(var i = 0; i < storage.length; i++) {
      if(storage[i].toBeRemoved) {
        continue;
      }
      count++;
    }
    if(count === 0) {
      var row = document.createElement("row");
      row.setAttribute('align', 'center');
      rows.appendChild(row);

      var hbox = document.createElement("hbox");
      hbox.setAttribute('pack', 'end');
      var label = document.createElement("shadowlabel");
      label.setAttribute('value', 'Storage:');
      label.setAttribute('class', 'staticdataheader');
      hbox.appendChild(label);
      row.appendChild(hbox);
      
      var plus = document.createElement("button");
      plus.setAttribute('class', 'plus-button');
      plus.setAttribute('oncommand', "addStorageItem();");
      row.appendChild(plus);
    }
  }

  var settings = current.get('virtualNodeSettings');
  if(settings.childType !== 'OpenStackSettings') return;

  var storage = realStorage.slice(0);

  var flavor = getFlavor(settings);
  if(flavor) {
    var rootVolumeDefined = false;
    for(var i = 0; i < storage.length; i++) { // order ??
      if(storage[i].toBeRemoved) continue;
      if(storage[i].drive === '/dev/vda') {
        rootVolumeDefined = true;
        break;
      }
    }
    if(!usedDrives) usedDrives = new Array();
    if(!rootVolumeDefined) {
      var one = new dock.cvOpenStackBlockStorage();
      one.uniqueKey = -1; // no delete!
      one.name = "[flavor] root";
      one.drive = "/dev/vda";
      one.size = flavor.get('disk');
      storage.unshift(one);
      if(usedDrives.find(one.drive) < 0) usedDrives.push(one.drive);
    }
    if(flavor.ephemeralDisk) {
      var one = new dock.cvOpenStackBlockStorage();
      one.uniqueKey = -1; // no delete!
      one.name = "[flavor] ephemeral";
      one.drive = "/dev/vdb";
      one.size = flavor.get('ephemeral');
      storage.unshift(one);
      if(usedDrives.find(one.drive) < 0) usedDrives.push(one.drive);
    }
    if(flavor.swapDisk) {
      var one = new dock.cvOpenStackBlockStorage();
      one.uniqueKey = -1; // no delete!
      one.name = "[flavor] swap";
      if(flavor.ephemeralDisk) {
        one.drive = "/dev/vdc";
      } else {
        one.drive = "/dev/vdb";
      }
      one.size = flavor.get('swap');
      storage.unshift(one);
      if(usedDrives.find(one.drive) < 0) usedDrives.push(one.drive);
    }
  } else {
    println('** No flavor for storage **');
  }

  var rows = document.getElementById('storage');
  while(rows.firstChild) rows.removeChild(rows.firstChild);

  if(!started) {
    addFirstOpenStackStorage(storage, rows);
  }

  var j = 0;
  var last_index = -1;
  for(var i = 0; i < storage.length; i++) {
    if(!storage[i].toBeRemoved) {
      last_index = i;
    }
  }
  for(var i = 0; i < storage.length; i++) {
    if(storage[i].toBeRemoved) {
      if(storage[i].uniqueKey >= 0) j++;
      continue;
    }

    var row = document.createElement("row");
    row.setAttribute('align', 'center');
    rows.appendChild(row);

    var hbox = document.createElement("hbox");
    hbox.setAttribute('pack', 'end');
    var label = document.createElement("shadowlabel");
    label.setAttribute('value', 'Storage:');
    label.setAttribute('class', 'staticdataheader');
    hbox.appendChild(label);
    row.appendChild(hbox);

    var textbox = document.createElement("textbox");
    if(started || (storage[i].uniqueKey < 0)) textbox.setAttribute('disabled', true);
    if(storage[i].uniqueKey < 0) {
      textbox.setAttribute('value', storage[i].name);
    } else {
      textbox.setAttribute('id', 'virtualNodeSettingsstorage_' + j + 'name');
    }
    row.appendChild(textbox);

    var textbox = document.createElement("textbox");
    if(started || (storage[i].uniqueKey < 0)) textbox.setAttribute('disabled', true);
    if(storage[i].uniqueKey < 0) {
      textbox.setAttribute('value', storage[i].drive);
    } else {
      textbox.setAttribute('id', 'virtualNodeSettingsstorage_' + j + 'drive');
    }
    row.appendChild(textbox);

    var textbox = document.createElement("textbox");
    if(started || (storage[i].uniqueKey < 0)) textbox.setAttribute('disabled', true);
    if(storage[i].uniqueKey < 0) {
      textbox.setAttribute('value', formatKMGselect(storage[i].size, 'B'));
    } else {
      textbox.setAttribute('id', 'virtualNodeSettingsstorage_' + j + 'size');
    }
    row.appendChild(textbox);

    if(!started) {
      if(storage[i].uniqueKey >= 0) {
        var minus = document.createElement("button");
        minus.setAttribute('class', 'minus-button');
        minus.setAttribute('oncommand', "removeStorageItem(" + j + ");");
        row.appendChild(minus);
      }
      if(i === last_index) {
        var plus = document.createElement("button");
        plus.setAttribute('class', 'plus-button');
        plus.setAttribute('oncommand', "addStorageItem();");
        row.appendChild(plus);
      } else {
        var hbox = document.createElement("hbox");
        hbox.setAttribute('height', '4px');
        rows.appendChild(hbox);
      }
    }
    if(storage[i].uniqueKey >= 0) j++; 
  }
  var j = 0;
  for(var i = 0; i < storage.length; i++) {
    if(storage[i].uniqueKey < 0) continue;
    if(!storage[i].toBeRemoved) {
      displayfunction('virtualNodeSettingsstorage_' + j + 'size', 'formatKMGselect(value, "B");');
      parsefunction('virtualNodeSettingsstorage_' + j + 'size', 'parseKMG(value);');
      text('virtualNodeSettingsstorage_' + j, '^', 'storage|name');
      text('virtualNodeSettingsstorage_' + j, '^(/dev/)?(h|xv|v|s)d[a-z]$', 'storage|drive');
      text('virtualNodeSettingsstorage_' + j, '^[0-9]+(\.[0-9]*)?\s*[kmgtKMGT]?i?[Bb]?$', 'storage|size');
    }
    j++;
  }
}

function addStorageItem() {
  var a = 97;

  var settings = current.get('virtualNodeSettings');
  if(settings.childType === 'OpenStackSettings') {
    var one = new dock.cvOpenStackBlockStorage();
    one = new dock.cvObject(current.cluster, one);
    var drive = 0;
    var ud = [];
    if(usedDrives) {
      ud = usedDrives.slice(0);  
    }
    for(var c = 0; c < 26; c++) {
      if(ud.find('/dev/vd' + String.fromCharCode(c + a)) < 0) {
        drive = c;
        break;
      }
    }
    var index = 1;
    for(var i = 0; i < settings.storage.length; i++) {
      if((settings.storage[i].name.length > 6) && startswith(settings.storage[i].name, 'volume')) {
        index = Math.max(index, myParseInt(settings.storage[i].name.substr(6)) + 1);
      }
      if((settings.storage[i].drive.length === 8) && startswith(settings.storage[i].drive, '/dev/vd')) {
        ud.push(settings.storage[i].drive);
        if(settings.storage[i].drive[7] === String.fromCharCode(drive + a)) {
          for(var c = drive; c < 26; c++) {
            if(ud.find('/dev/vd' + String.fromCharCode(c + a)) < 0) {
              drive = c;
              break;
            }
          }
        }
      }
    }
    one.set('name', 'volume' + index);
    one.set('drive', '/dev/vd' + String.fromCharCode(c + a));
    one.set('size', 40 * 1024 * 1024 * 1024);
    one.set('toBeRemoved', false);
    one.set('modified', true);
    settings.storage.push(one.object);
    current.set('modified', true);
    addOpenStackStorage(settings.storage, false);
    settings.modified = true;
    //var boxed_storage = new dock.cvObject(current.cluster, settings.storage);
    current.set('virtualNodeSettings',settings);
  }
}

function removeStorageItem(index) {
  var settings = current.get('virtualNodeSettings');
  if(settings.childType === 'OpenStackSettings') {
    var storage = settings.storage;
    if(storage.length > index) {
      storage[index].toBeRemoved = true;
      settings.modified = true;
      addOpenStackStorage(storage, false);
      current.set('modified', true);
    }
  }
}
