/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var MAX_THREADS_PER_CLUSTER = 30;
var MAX_THREADS = 0;

var cvWorker = function(file, url) {
  this.file = file;
  this.url = endswithslash(url);
  this.worker = 0;
  this.busy = false;
  this.callback = 0;
  this.input = null;
  this.output = null;
  this.current = null;
  this.rpc = '';
}

cvWorker.prototype.call = function(service, rpc, input, callback, current, debug) {
  this.current = isundefined(current, null);
  this.createWorker(debug);
  this.callback = callback;
  this.busy = true;
  this.input = input;
  this.rpc = service + ':' + rpc;
  
  var data = new Object();
  data['service'] = service;
  data['call'] = rpc;
  if(isArray(input)) {
    data['args'] = input;
  } else if(input != null) {
    data['arg'] = input;
  }
  this.worker.postMessage([JSON.stringify(data), rpc]);
  while((callback == 0) && this.busy) {
    mainthread.processNextEvent(true);
  }
  if(callback == 0) return this.output;
  return true;
}

cvWorker.prototype.wget = function(url, callback, current, debug) {
  this.current = isundefined(current, null);
  this.createWorker(debug);
  this.callback = callback;
  this.busy = true;
  this.rpc = url;
  
  this.worker.postMessage(['WGET', url]);
  while((callback == 0) && this.busy) {
    mainthread.processNextEvent(true);
  }
  if(callback == 0) return this.output;
  return true;
}

cvWorker.prototype.createWorker = function(debug) {
  if(!this.worker) {
    this.worker = new Worker(this.file);
    this.worker.parent = this;
    this.worker.onmessage = function(event) {
      if(this.parent.callback == 0) {
        this.parent.output = event.data;
      } else {        
        var code = event.data[0];
        if(event.data[1] == null) {
          println("Worker: got null for: " + this.parent.rpc);
          this.parent.callback(code, null);
        } else {
          this.parent.callback(code, event.data[1]);
        }
        this.parent.output = null;
      }
      this.parent.rpc = '';
      this.parent.busy = false;
    }
    this.worker.postMessage(['URL', this.url]);
  }
  if(typeof(debug) != 'undefined') {
    if(debug) {
      this.worker.postMessage(['DEBUG', 'ON']);
    } else {
      this.worker.postMessage(['DEBUG', 'OFF']);
    }
  }
}

cvWorker.prototype.stop = function() {
  if(this.worker) {
    this.worker.postMessage(['ABORT']);
    if(this.parent) this.parent.callback(-3, {'errormessage': 'aborted'});
    this.worker.terminate();
    this.worker = 0;
  }
}

cvWorker.prototype.postMessage = function(data) {
  if(this.worker) this.worker.postMessage(data);
}

var cvWorkerPool = function(file) {
  this.file = file;
  this.workers = new Array();
}

cvWorkerPool.prototype.getWorker = function(url, file) {
  file = isundefined(file, this.file);
  url = endswithslash(url);
  var work = 0;
  var first = 0;
  var last = 0;
  while(!work) {
    for(var i = 0; i < this.workers.length; i++) {
      if(!this.workers[i].busy && (url == this.workers[i].url) && (file == this.workers[i].file)) {
        work = this.workers[i];
        break;
      }
    }
    if(!work) {    
      if((this.workers.length < MAX_THREADS) || ((last - first) > 500)) {
        work = new cvWorker(file, url);
        this.workers.push(work);
        println("New JSON worker, count = " + this.workers.length + " / " + MAX_THREADS);
      } else {
        if(first == 0) {
          first = new Date();
          first = first.getTime();
        }
        dock.mainthread.processNextEvent(true);
        if(last == 0) {
          last = first;
        } else {
          last = new Date();
          last = last.getTime();
        }        
      }
    }
  }
  return work;
}

cvWorkerPool.prototype.stop = function(url) {
  url = endswithslash(url);
  var i = 0;
  while(i < this.workers.length) {
    if(url == this.workers[i].url) {
      this.workers[i].stop();
      this.workers.splice(i, 1);
    } else {
      i++;
    }
  }
  println("Workers left: " + this.workers.length);
}
