/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var ACTION_DRAIN = 2;
var ACTION_DECOMMISSION = 1;

var dock;
var cluster;
var current;
var timer = 0;
var drainAction = ACTION_DRAIN;
var nodedata;
var workload = '';

function onLoad() {
  dock = getDock();
  println('.Load: ' + document.title + ' - ' + document.documentURI);
}

function refresh(use) {
  println("Refresh job folder overview" + ' - ' + document.documentURI);
  if(!!use) {
    workload = '';
    current = use;
    if(current.isCluster) {
      if(parent.frames[0].IS_CEPH_TAB) {
        current = use.find('Ceph');
        if(current.length == 1) {
          current = current[0];
        } else {
          current = 0;
          return;
        }        
      }
    }    
    if(current.isCluster) {
      cluster = current;
    } else {
      cluster = current.cluster;
      if(current.get('baseType') == 'HadoopHDFS') {
        workload = 'hadoop-' + current.resolveName() + '-jobscheduler';
      } else if(current.get('baseType') == 'Ceph') {
        workload = 'ceph-' + current.resolveName() + '-jobscheduler';
      }
      drainAction = ACTION_DECOMMISSION;
      document.getElementById('drain').setAttribute('label', 'Decommission');
      document.getElementById('undrain').setAttribute('label', 'Recommission');
      document.getElementById('undrain').setAttribute('accesskey', 'e');
      document.getElementById('showinvalid').setAttribute('hidden', true);
      document.getElementById('actions').setAttribute('hidden', true);
    }
  }
  
  var nodes = cluster.find('Node');
  if(drainAction == ACTION_DECOMMISSION) {
    if(current.get('baseType') == 'HadoopHDFS') {
      var allHadoopNodeKeys = new Array();
      var hdfsKey = current.get('uniqueKey');
      for(var i = 0; i < nodes.length; i++) {
        var hadoopRoles = nodes[i].getHadoopRoles(hdfsKey);
        if(hadoopRoles.length) allHadoopNodeKeys.push(nodes[i].get('uniqueKey'));
      }
      nodes = allHadoopNodeKeys;
    } else if(current.get('baseType') == 'Ceph') {
      var allCephNodeKeys = new Array();
      for(var i = 0; i < nodes.length; i++) {
        var cephRoles = nodes[i].getCephRoles(hdfsKey);
        if(cephRoles.length) allCephNodeKeys.push(nodes[i].get('uniqueKey'));
      }
      nodes = allCephNodeKeys;
    }
  } else {
    for(var i = 0; i < nodes.length; i++) {
      nodes[i] = nodes[i].get('uniqueKey');
    }
  }

  println("Drain overview: " + workload);
  
  dock.workerpool.getWorker(cluster.actualurl).call('cmjob', 'drainOverview', [workload, nodes], function(code, data) {
      if(code == 0) {
        setCallbackResult(data);
      } else {
        dock.jsonCallError(this, code, data);
      }
    }, cluster);

  if(timer) window.clearTimeout(timer);
  timer = window.setTimeout('refresh();', dock.generalSettings.statusRefresh * 10000);
}

function setCallbackResult(value) {
  var list = value;

  tree = document.getElementById('tree');
  data = new Array();
  nodedata = new Object();
  for(var i = 0; i < list.length; i++) {
    if(list[i].scheduler.length > 0) {
      var nodes = list[i].node;
      var queues = list[i].queue;
      var results = list[i].result;
      for(var j = 0; j < nodes.length; j++) {
        work = nodedata[nodes[j]];
        if(typeof(work) == 'undefined') {
          work = new Object();
          work['queues'] = new Array();
          work['states'] = new Array();
          work['actions'] = new Array();
          nodedata[nodes[j]] = work;
        }
        var qn = cluster.resolveName(queues[j]);
        if(qn.length) {
          work['queues'].push([list[i].scheduler, cluster.resolveName(queues[j])]);
          work['states'].push(results[j]);
        }
      }
    }
  }
  var i = list.length - 1;
  if((list.length > 0) && (list[i].scheduler.length == 0)) {
    var nodes = list[i].node;
    var actions = list[i].actions;
    var results = list[i].result;
    var k = 0;
    for(var j = 0; j < nodes.length; j++) {
      work = nodedata[nodes[j]];
      for(var kk = 0; (kk < results[j]) && (k < actions.length); kk++) {
        var tmp = cluster.resolveName(actions[k].threshAction);
        if(tmp) {
          if(actions[k].actionParam.length) tmp += ':' + actions[k].actionParam;
          work['actions'].push(tmp);
        }
        k++;
      }
    }
  }
  for(var key in nodedata) {
    var work = nodedata[key];
    var item = new Object();
    var status = -1;
    for(var i = 0; i < work['states'].length; i++) {
      if(status < 0) {
        status = work['states'][i];
      } else if(status != work['states'][i]) {
        status == -2;
        break;
      }
    }
    
    switch(status) {
      case(2): item['status'] = ''; break;
      case(1): item['status'] = 'Drained'; break;
//       case('ERROR'): item['status'] = 'Error'; break;
      case(-1):
      case(3): if(!document.getElementById('showinvalid').checked) continue;
               item['status'] = 'Unknown by workload managers';
               break;
      case(4): item['status'] = 'Failed'; break;
      case(6): item['status'] = 'Draining'; break;
      case(7): item['status'] = 'Decommissioned'; break;
      case(8): item['status'] = 'Decommissioning'; break;
      case(9): item['status'] = 'Partially decommissioned'; break;
      case(0): item['status'] = 'Not a node'; break;
      case(-2): item['status'] = 'Various (see details)'; break;
      default:  item['status'] = 'Unknown: ' + status;
    }
    
    item['key'] = key;
    item['node'] = cluster.resolveName(key);
    var queue = new Array();
    for(var i = 0; i < work['queues'].length; i++) {
      queue.push(work['queues'][i][0] + ':' + work['queues'][i][1]);
    }
    item['queues'] = queue.join(',');
    item['actions'] = work['actions'].join(',');
    data.push(item);
  }

  for(var i = list.length + list.length; i < MINTREEITEMS; i++) data.push(blankTreeItem());
  var select = selected('list');
  loadTable();
  setSelected('list', select);
}

function stop() {
  if(timer) window.clearTimeout(timer);
}

function drain() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select at least one node");
  } else {
    var nodes = new Array();
    for(var i = 0; i < select.length; i++) {
      nodes.push(parseInt(select[i]));
    }
    if(drainAction == ACTION_DECOMMISSION) { // KDR: no actions, doesn't make much sense
      dock.workerpool.getWorker(cluster.actualurl).call('cmjob', 'drainNodes', [workload, '', nodes, drainAction, [], [], false], function(code, data) {
          if(code != 0) {
            dock.jsonCallError(this, code, data);
          }
          refresh();
        }, cluster);
    } else {
      var args = new Object();
      args['cluster'] = cluster;
      args['nodes'] = nodes;
      var d = new dock.cvDialog("drainaction.xul", args);
      d.onOk = function(addActions, removeActions, clearActions) {
        dock.workerpool.getWorker(cluster.actualurl).call('cmjob', 'drainNodes', [workload, '', nodes, drainAction, addActions, removeActions, clearActions], function(code, data) {
            if(code != 0) {
              dock.jsonCallError(this, code, data);
            }
            refresh();
          }, cluster);
      }
      dock.dialogs.showDialog(d);
    }
  }
}

function undrain() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select at least one node");
  } else {
    var nodes = new Array();
    for(var i = 0; i < select.length; i++) {
      nodes.push(parseInt(select[i]));
    }
    var addActions = new Array();
    var removeActions = new Array();
    var clearActions = true;
    dock.workerpool.getWorker(cluster.actualurl).call('cmjob', 'drainNodes', [workload, '', nodes, 0, addActions, removeActions, clearActions], function(code, data) {
        if(code != 0) {
          dock.jsonCallError(this, code, data);
        }
        refresh();
      }, cluster);
  }
}

function details() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError("Please select a node");
  } else if(select.length > 1) {
    dock.displayError("Please select only one node");
  } else {
    var args = new Object();
    args['nodedata'] = nodedata[parseInt(select[0])];
    var d = new dock.cvDialog("draindetails.xul", args);
    dock.dialogs.showDialog(d);
  }
}
