/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

var dock;
var current;
var alsoRemoved = false;

var basetype = 'JobQueue';
var singular = 'queue';
var plural = 'queues';


function createTreeItem(one) {
  var item = new Object();
  item['key'] = one.get('uniqueKey');
  item['modified'] = one.get('modified');
  item['name'] = one.get('name');
  item['scheduler'] = one.get('childType').replace('JobQueue','').toLowerCase();
  item['nodes'] = commaListToDotFormat(one.get('nodes'), new RegExp("\\..*$"));
  return item;
}

function addObject() {
  var args = new Object();
  args['title'] = 'Add new ' + singular;
  args['question'] = 'Name';
  args['default'] = '';
  args['known'] = current.known('JobQueue', 'name');
  args['children'] = new Array();
  var known = current.find('JobQueue');
  for(var i = 0; i < known.length; i++) {
    var s = known[i];
    s = s.get('childType');
    s = s.replace('JobQueue', '');
    s = s.toLowerCase();
    if(args['children'].find(s) < 0) args['children'].push(s);
  }
  
  var other = ['torque', 'pbspro', 'slurm', 'sge', 'uge', 'openlava', 'lsf'];
  for(var i = 0; i < other.length; i++) {
    var s = other[i];
    if(args['children'].find(s) < 0) args['children'].push(s);
  }
  var d = new dock.cvDialog("newchildobject.xul", args);
  d.onOk = function(name, type) {
    var object;
    switch(type) {
      case('uge'): object = new dock.cvUGEJobQueue(); break;
      case('sge'): object = new dock.cvSGEJobQueue(); break;
      case('torque'): object = new dock.cvTorqueJobQueue(); break;
      case('pbspro'): object = new dock.cvPbsProJobQueue(); break;
      case('slurm'): object = new dock.cvSlurmJobQueue(); break;
      case('lsf'): object = new dock.cvLSFJobQueue(); break;
      case('openlava'): object = new dock.cvOpenLavaJobQueue(); break;
      case('fake'): object = new dock.cvFakeJobQueue(); break;
      default:
        dock.displayError('Ilegal object type: ' + type);
        return;
    }
    object.name = name;
    current.addObject(object);
    refresh();
  }
  dock.dialogs.showDialog(d);
}

function editObject() {
  var select = selected('list');
  if(select.length == 0) {
    dock.displayError('Please select a jobqueue');
  } else if(select.length > 1) {
    dock.displayError('Please select only one jobqueue');
  } else {
    var jq = current.find(select[0]);
    if(jq) {
      var args = new Object();
      args['jobqueue'] = jq;
      var d = new dock.cvDialog('edit' + jq.get('childType').toLowerCase() + '.xul', args);
      d.onOk = function() {
        refresh();
      }
      dock.dialogs.showDialog(d);
    } else {
      dock.displayInfo('Jobqueue no longer found');
      refresh();
    }
  }
}
